unit bfaBaseObjects;

interface

uses
  Classes, SysUtils, Generics.Collections,
  bfaCommonUtils{, ievect, bfaImageEnVect, bfaImageEnMView }
  ;

const
  REQTYPELIST: array [0..36] of string = (
                   'reqSendPreview',  'reqSendIev', 'reqSendOri', 'respSendReady',
                   'respSendError', 'respSendComplete', 'shutdownServer',
                   'reqAuthenticate', 'respAuth', 'respAuthError', 'reqDoc',
                   'reqDocTH', 'reqOri', 'reqOriTH', 'reqIev', 'reqIevCheck',
                   'reqIevTH', 'reqPrevies', 'reqPreviewTH', 'reqQueuecount',
                   'reqDuplicateDoc', 'reqResetOriginal', 'reqMoveDoc', 'respDoc',
                   'respOK', 'respError', 'respDocError', 'respMySQLError',
                   'respIevCheck',  'reqIevDelete', 'reqAdminUpdate', 'respMoveError',
                   'reqSetLogLevel', 'reqStartSocketLog', 'reqStopSocketLog',
                   'reqPreview', 'respIsLates');


type
  TbfaOptionStoreType = (ostRegistry, ostIniFile, ostDatabase);
  TbfaOptionKind = (okBoolean, okInteger, okString, okDouble);
  { TODO -ob.takacs: in V4 berprfen, welchen Treeviewkind wir bentigen, auch Schreibweise prfen (tvk oder tv) }
  TbfaViewKind = (tvkViewer, tvkImporter, tvkAdministrator, tvNormal,
                  tvRecord, tvkWorkplace, tvkResub, tvkFavorites, tvkNone);
  TbfaBracketKind = (brOpen, brClose);
  TbfaJunctionKind = (jnAND, jnOR, jnNOT, jnEmpty);
  TbfaColumnType = (cltEdit, cltDate, cltNumber, cltCurrency, cltList, cltDBList, cltAutoIncrement, cltFloat);
  TbfaValueType = (vltString, vltInteger, vltFloat, vltDate, vltCurrency);
  TbfaSummaryKind = (smtNone, smtSum, smtMin, smtMax, smtCount, smtAverage);
  TbfaListType = (lstNone, lstItems, lstAddfieldItems, lstAddfieldValues, lstQuery);



  TDocLink = record
    RefID: Int64;
    RefName: WideString;
    Count: Integer;
    GDocID: Int64;
  end;

  TbfaInt64 = class(TObject)
  private
    FValue: Int64;
    FArcType: Integer;  //25.11.2020 Ticket 25242
  public
    constructor Create; overload;
    constructor Create(AValue: Int64); overload;
    procedure Clone(Source: TBfaInt64);
    property Value: Int64 read FValue write FValue;
    property ArcType: Integer read FArcType write FArcType; //25.11.2020 Ticket 25242
  end;

  PbfaInt64 = ^TbfaInt64;

  TbfaInt = class(TObject)
  private
    FValue: Integer;
  public
    constructor Create; overload;
    constructor Create(AValue: Integer); overload;

    procedure Clone(Source: TBfaInt);
    property Value: Integer read FValue write FValue;
  end;

  PbfaInt = ^TbfaInt;

  TbfaString = class(TObject)
  private
    FValue: WideString;
  public
    constructor Create; overload;
    constructor Create(AValue: WideString); overload;

    Procedure Clone(Source: TbfaString);
    property Value: WideString read FValue write FValue;
  end;

  /// <summary>
  ///   Ist einer Oberklasse. Kann nachher Dokumente oder zB Request
  ///   darstellen. Wird weiter vererbt und gibt nur das Basis-Konstrukt vor.
  /// </summary>
  TbfaBaseObject = class(TObject)
  private
    procedure SetLoaded(const Value: Boolean);
  protected
    FID: Int64;
    FTempID: TGUID;
    FObjType: Integer;
    FName: WideString;
    FDescription: WideString;
    FStatus: Integer;
    FErrorCode: Integer;
    FInitialized: Boolean;
    FLoaded: Boolean;
    FModified: Boolean;
    FGeneralPropertiesModified: Boolean;
    FAddfields_Arcid: Integer;
    FSortingPos: Integer;

    /// <summary>
    ///   Setzt Werte zurck
    /// </summary>
    procedure Reset; virtual;

    procedure SetName(Value: WideString);
    procedure SetDescription(Value: WideString);
    procedure SetStatus(Value: Integer);

  public
    /// <summary>
    ///   Setzt die Standardwerte und erstellt die Objekte
    /// </summary>
    constructor Create; virtual;

    destructor Destroy; override;

    /// <summary>
    ///   Erzeugt neues Objekt und setzt Werte von Source
    /// </summary>
    procedure Clone(Source: TbfaBaseObject); virtual;

    /// <summary>
    ///   Object ID
    /// </summary>
    property ID: Int64 read FID write FID;

    /// <summary>
    ///   Temporary ID fr noch nicht archivierte Objekte
    /// </summary>
    property TempID: TGUID read FTempID write FTempID;

    /// <summary>
    ///   Objektname
    /// </summary>
    property Name: WideString read FName write SetName;

    /// <summary>
    ///   Beschreibung
    /// </summary>
    property Description: WideString read FDescription write SetDescription;

    /// <summary>
    ///   Objekt Typ
    /// </summary>
    property Typ: Integer read FObjType write FObjType;

    /// <summary>
    ///   Fehlercode - aktuell nicht verwendet
    /// </summary>
    /// <value>
    ///   <list type="table">
    ///     <listheader>
    ///       <term>Term</term>
    ///       <description>Description</description>
    ///     </listheader>
    ///     <item>
    ///       <term>-1</term>
    ///       <description>nicht initialisiert</description>
    ///     </item>
    ///     <item>
    ///       <term>0</term>
    ///       <description>alles in Ordnung</description>
    ///     </item>
    ///     <item>
    ///       <term>&gt;0</term>
    ///       <description>Fehler (siehe bfaCommonUtils "Document Class
    ///         Error Codes")</description>
    ///     </item>
    ///   </list>
    /// </value>
    property ErrorCode: Integer read FErrorcode;

    /// <summary>
    ///   Wenn das Objekt zustande gekommen ist, dann true, sonst false.
    /// </summary>
    property Initialized: Boolean read FInitialized;

    property Status: Integer read FStatus write SetStatus;

    /// <summary>
    ///   Wenn Objekt geladen True, sonst False
    /// </summary>
    property Loaded: Boolean read FLoaded write SetLoaded;

    property Modified: Boolean read FModified write FModified;

    property GeneralPropertiesModified: Boolean read FGeneralPropertiesModified write FGeneralPropertiesModified;

    /// <summary>
    ///   Um ggf. Berechtigungen von anderen Archiven zu verwenden, als die vom aktuellen
    /// </summary>
    property Addfields_Arcid: Integer read FAddfields_Arcid write FAddfields_Arcid;

    property SortingPos: Integer read FSortingPos write FSortingPos;
  end;

  PbfaBaseObject = ^TbfaBaseObject;

  TbfaGroup = class(TbfaBaseObject)
    public
    /// <summary>
    ///   kommagetrennte Liste an UserIDs
    /// </summary>
    Members: TStringList;

    ParentID: Int64;

    /// <summary>
    ///   ein Satz an Standardrechten als Bitmask
    /// </summary>
    Role: Int64;

    Flags: Integer;
    ADPath: WideString;
    Email: WideString;
    constructor Create; override;
    destructor Destroy; override;

    procedure Clone(Source: TbfaGroup); overload;
  end;

  TbfaGroupEx = class(TbfaGroup)
    public
      NewMembers: TStringList;
      MembersModified: Boolean;

      constructor Create; override;
      destructor Destroy; override;

      procedure Clone(Source: TbfaGroup); overload;
  end;

  TbfaUser = class(TbfaBaseObject)
    public
      ID: Int64;
      Rights: Integer;

      /// <summary>
      ///   ein Satz an Standardrechten als Bitmask
      /// </summary>
      Role: Int64;

      DisplayName: WideString;
      Src_Userid: Int64;
      Email: WideString;
      Flags: Integer;
      OldFlags: Integer;
      Password: WideString;
      GroupList: TStringList;
      EmployeesList: TStringList;
      isAdmin: Boolean;
      DMSAdmin: Boolean;
      SuAnnoList: TStringList;
      HolidayList: TStringList;
      constructor Create; override;
      destructor Destroy; override;

      /// <summary>
      ///   Erzeugt neues Objekt und setzt Werte von Source
      /// </summary>
      procedure Clone(Source: TbfaUser); overload;
    End;

  /// <summary>
  ///   Ein Feldberechtigungssatz fr ein <see cref="bfaObjects36.TbfaAddField">
  ///   TbfaAddField</see> ODER ein <see cref="bfaObjects36.TbfaStateField">
  ///   TbfaStateField</see>. <br />Es kann entweder eine Userid ODER eine
  ///   GroupID hinterlegt werden. <br /><br />Die Rechte werden als Bitmaske
  ///   gespeichert
  /// </summary>
  /// <seealso cref="bfaObjects36.TbfaAddField" />
  /// <seealso cref="bfaObjects36.TbfaStateField" />
  TbfaFieldRight = class(TObject)
  private
    fRights: Integer;
    procedure SetRights(Value: Integer);
    function GetRights: Integer;
  public
    ArcID: Integer;
    FieldID: Integer;
    Userid: Integer;
    Groupid: Integer;
    Fieldpos: Integer;
    Modified: Boolean;
    Deleted: Boolean;
    ArcTempID: TGUID;
    constructor Create; overload;
    destructor Destroy; override;

    /// <summary>
    ///   Erzeugt neues Objekt und setzt Werte von Source
    /// </summary>
    procedure Clone(Source: TbfaFieldRight);

    /// <remarks>
    ///   <list type="bullet">
    ///     <item>
    ///       rFieldRead = 1;
    ///     </item>
    ///     <item>
    ///       rFieldModify = 4;
    ///     </item>
    ///   </list>
    /// </remarks>
    property Rights: Integer read GetRights write SetRights;
  end;

  TbfaBaseField = class(TbfaBaseObject)
  private
    FFlags: Integer;
    FAssigned: Boolean;
    FFieldpos: Integer;
    FFieldRights: TList<TbfaFieldRight>;
  protected
    FDisplayName: WideString;
    FDefaultValue: WideString;
    FIndexTableID: Integer;
    FValue: Variant;
    FValueHigh: Variant;
    FOldValue: Variant;
    FValueType: Integer;
    FControlType: Integer;
    FEnabled: Boolean;
    FControlItems: WideString;
    FSize: Integer;
    FMode: Integer;
    FOptions: Integer;
    FModifiedSearch: Boolean;
    FNegate: Boolean;
    FOr_Junction: Boolean;

  public
    /// <summary>
    ///   Setzt die Standardwerte und erstellt die Objekte
    /// </summary>
    constructor Create; override;

    destructor Destroy; override;

    /// <summary>
    ///   Erzeugt neues Objekt und setzt Werte von Source
    /// </summary>
    procedure Clone(Source: TbfaBaseField); overload;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property Typ: Integer read FObjType write FObjType;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property DefaultValue: WideString read FDefaultValue write FDefaultValue;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property DisplayName: WideString read FDisplayName write FDisplayName;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property IndexTableID: Integer read FIndexTableID write FIndexTableID;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property ControlType: Integer read FControlType write FControlType;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property Enabled: Boolean read FEnabled write FEnabled;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property ControlItems: WideString read FControlItems write FControlItems;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property Size: Integer read FSize write FSize;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property Mode: Integer read FMode write FMode;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property Options: Integer read FOptions write FOptions;

    /// <summary>
    ///   Wert eines Zusaztzfeldes, bzw unterster Wert bei von - bis
    /// </summary>
    property Value: Variant read FValue write FValue;

    /// <summary>
    ///   oberster Wert bei von - bis
    /// </summary>
    property ValueHigh: Variant read FValueHigh write FValueHigh;

    /// <summary>
    ///   Initialwert, wenn das Feld gefllt wird, wichtig fr Abbrechen etc.
    /// </summary>
    property OldValue: Variant read FOldValue write FOldValue;

    /// <summary>
    ///   das Feld wurde fr die Suche gendert
    /// </summary>
    property ModifiedSearch: Boolean read FModifiedSearch write FModifiedSearch;

    /// <summary>
    ///   die Suche soll negiert durchgefhrt werden
    /// </summary>
    property Negate: Boolean read FNegate write FNegate;

    /// <summary>
    ///   noch nicht implementiert
    /// </summary>
    property Or_Junction: Boolean read FOr_Junction write FOr_Junction;

    /// <summary>
    ///   Konfiguration. Unterschiedlich bei <see cref="TbfaUser" /> / <see cref="bfaObjects36.TbfaAddField" /> /
    ///   <see cref="bfaObjects36.TbfaStateField" />
    /// </summary>
    property Flags: Integer read FFlags write FFlags;
    property Assigned: Boolean read FAssigned write FAssigned;
    property Fieldpos: Integer read FFieldpos write FFieldpos;
    property FieldRights: TList<TbfaFieldRight> read FFieldRights write FFieldRights;
  end;

  PbfaBaseField = ^TbfaBaseField;

  //TbfaAnnoObj = record
    //ID: Integer;
    //ObjKind: TIEVObjectKind;
    //ObjText: WideString;
    //Modified: Boolean;
    //NewText: WideString;
    //Name: WideString;
    //ModifiedType: Integer;
  //end;

  {$IFDEF ENTERPRISE}
  TPlugin = record
    Name, Parameters, Command: string;
    ShowButton: Boolean;
    Index: Integer;
  end;

  TPluginObj = class(TObject)
  public
    Name, FileName, Parameters, Command: string;
    Index: Integer;
    constructor Create;
  end;
  {$ENDIF}

  TbfaHistory = class(TbfaBaseObject)
  public
    Taskid: Integer;
    Userid: Int64;
    Gdocid: Int64;
    Date: TDateTime;
    Msg: WideString;
    Value1: WideString;
    Value2: WideString;
    Value3: WideString;
    UserName: WideString;

    constructor Create;
    procedure Clone(Source: TbfaHistory); overload;
  end;

  //TbfaAnnotation = class(TbfaBaseObject)
  //private
    //FAnnoStream: TMemoryStream;
    //FPage: Integer;
    //FLayer: Integer;
    //FPriv: Boolean;
    //FFilePath: WideString;
    //FText: WideString;
    //FRights: Integer;
    //FHistory: TList <TbfaHistory>;
    //FDeleted: Boolean;
    //procedure SetDeleted(Value: Boolean);
    //function GetDeleted: Boolean;
  //public
    //ObjList: array of TbfaAnnoObj;
//
    //constructor Create; override;
    //destructor Destroy; override;
//
    ///// <summary>
    /////   Erzeugt neues Objekt und setzt Werte von Source
    ///// </summary>
    //procedure Clone(Source: TbfaAnnotation);
//
    //procedure ClearHistoryList;
//
    //property AnnoStream: TMemoryStream read FAnnoStream write FAnnoStream;
    //property Page: Integer read FPage write FPage;
    //property Priv: Boolean read FPriv write FPriv;
    //property Layer: Integer read FLayer write FLayer;
    //property FilePath: WideString read FFilePath write FFilePath;
    //property Text: WideString read FText write FText;
    //property Rights: Integer read FRights write FRights;
    //property History: TList<TbfaHistory> read FHistory write FHistory;
    //property Deleted: Boolean read GetDeleted write SetDeleted;
  //end;
//
//
  //TbfaStationery = class(TObject)
  //private
    //FStationeryStream: TMemoryStream;
    //FPages: Integer;
  //public
    //Layers: array of Integer;
//
    //constructor Create;
    //destructor Destroy; override;
    //procedure Clone(Source: TbfaStationery);
    //procedure Load(var img: TbfaImageEnVect;
      //DocPageCount: Integer; FilePath: WideString = ''; Str: TMemoryStream = nil);
//
    //property StationeryStream: TMemoryStream read FStationeryStream;
    //property Pages: Integer read FPages;
  //end;

  TbfaBaseDocument = class(TbfaBaseObject)
  private
    FTimID, FResubUserID: Int64;
    FFilter, FOcrType, FInEditIDs: string;
    FModifiedDate, FTargetDate, FExpired: Variant;
    FAboUserid: TStringList;
    FUserName, FEditor, FInfo: WideString;
    FAlarm, FTimerOptions, FExpired_Setby, FExpired_Userid: Integer;
    FExpired_Modified: Boolean;
  protected
    FPageCount: Integer;
    FArcID: Int64;
    FDocLoc: WideString;
    FExtLoc: WideString;
    FOriLoc: WideString;
    FSrcLoc: WideString;
    FDocTXT: WideString;
    FDocTags: WideString;
		FStreamTIF: TMemoryStream;
//24.05.2023		FStreamORI: TMemoryStream;
		FStreamORI: TStream;
		FStreamIEV: TMemoryStream;
    FStreamLetter: TMemoryStream;
		FCheckOutUserID: Int64;
    FDocType: Integer;
    FFileType: WideString;
    FFileName: WideString;
    FHasPreview: Boolean;
    FUserID: Int64;
    FDocDate: Variant;
    FRevSave: Boolean;
    FModifiedUserID: Int64;
    FEditModeUserID: Int64;
    FDocumentName: WideString;
    FPreviewChanging: Boolean;
    FSelectedPages: TStringList;
    FInProcess: Boolean;
    FInSaveProcess: Boolean;
    FInLoadProcess: Boolean;
    FInShowProcess: Boolean;
    FInRefreshProcess: Boolean;
    FInCloneProcess: Boolean;
    FInDeleteProcess: Boolean;
    FInMoveProcess: Boolean;
    //FAnnotations: TList<TbfaAnnotation>;
    FInEditMode: Boolean;
    FRights: Int64;
    FModified_Preview: Boolean;
    FModified_Annos: Boolean;
    FModified_AnnosDelete: Boolean;
    FSymbolicArcID: Int64;
    FHasSymlinks: string;
    FImporterDocNameModified: Boolean;
    //FScanPoolView: TbfaImageEnMView;
    //FThumbnailView: TbfaImageEnMView;
    //FScanPoolViewChanged: Boolean;
    //FScanPoolViewLoaded: Boolean;

    FHasComment: Boolean;
    procedure SetPageCount(NewPageCount: Integer);
    function GetInEditMode: Boolean;

	public
    DocLinkList: array of TDocLink;

    /// <summary>
    ///   TIFF von der Voransicht. Kann revisionssicher oder nicht revisionssicher sein
    /// </summary>
    property DocLoc: WideString read FDocLoc write FDocLoc;

    /// <summary>
    ///   Volltext
    /// </summary>
    property FullTXT: WideString read FDocTXT write FDocTXT;

    /// <summary>
    ///   Dateipfad TIFF - bei nderungen oder Notizen wird hier gespeichert
    /// </summary>
    property ExtLoc: WideString read FExtLoc write FExtLoc;

    /// <summary>
    ///   Dateipfad Originaldatei
    /// </summary>
    property OriLoc: WideString read FOriLoc write FOriLoc;

    /// <summary>
    ///   Dateipfad Source Datei - ndert sich nie
    /// </summary>
    property SrcLoc: WideString read FSrcLoc write FSrcLoc;

    /// <summary>
    ///   Schlagworte. Komma separated
    /// </summary>
    property DocTags: WideString read FDocTags write FDocTags;

    /// <summary>
    ///   Archiv-ID
    /// </summary>
    property ArcID: Int64 read FArcID write FArcID;

    /// <summary>
    ///   Stream fr DocLoc
    /// </summary>
    property DocStream: TMemoryStream read FStreamTIF write FStreamTIF;

    /// <summary>
    ///   Stream fr Annotationen
    /// </summary>
    property AnnoStream: TMemoryStream read FStreamIEV write FStreamIEV;

    /// <summary>
    ///   Stream fr OriLoc
    /// </summary>
//24.05.2023    property OriStream: TMemoryStream read FStreamORI write FStreamORI;
    property OriStream: TStream read FStreamORI write FStreamORI;

    /// <summary>
    ///   Stream fr Briefpapier
    /// </summary>
    property LetterStream: TMemoryStream read FStreamLetter write FStreamLetter;

    /// <summary>
    ///   Siehe "document Types" in bfaCommonUtils
    /// </summary>
    property DocType: Integer read FDocType write FDocType;

    /// <summary>
    ///   Benutzer ID die das Dokument ausgecheckt hat
    /// </summary>
    property CheckOutUserID: Int64 read FCheckOutUserID write FCheckOutUserID;

    /// <summary>
    ///   Dateityp mit fhrendem Punk (zB .tif)
    /// </summary>
    property FileType: WideString read FFileType write FFileType;

    /// <summary>
    ///   Dateiname wenn vorhanden
    /// </summary>
    property FileName: WideString read FFileName write FFileName;

    /// <summary>
    ///   Dokument mit Voransicht?
    /// </summary>
    property HasPreview: Boolean read FHasPreview write FHasPreview;

    /// <summary>
    ///   Benutzer ID die das Dokument archiviert hat
    /// </summary>
    property UserID: Int64 read FUserID write FUserID;

    /// <summary>
    ///   Datum wann das Dokument archiviert wurde
    /// </summary>
    property DocDate: Variant read FDocDate write FDocDate;

    /// <summary>
    ///   letzte nderung am Dokument. Auch bei Zusatzfeldnderungen, Annotationen und neuen Dokumenten-Versionen
    /// </summary>
    property ModifiedDate: Variant read FModifiedDate write FModifiedDate;

    /// <summary>
    ///   Zeitstempel wann das Dokument durch das DLM gelscht wird
    /// </summary>
    property Expired: Variant read FExpired write FExpired;

    /// <summary>
    ///   <list type="table">
    ///     <listheader>
    ///       <term>Value</term>
    ///       <description>Description</description>
    ///     </listheader>
    ///     <item>
    ///       <term>0</term>
    ///       <description>Archivierung</description>
    ///     </item>
    ///     <item>
    ///       <term>1</term>
    ///       <description>Benutzer</description>
    ///     </item>
    ///     <item>
    ///       <term>2</term>
    ///       <description>Admin</description>
    ///     </item>
    ///   </list>
    /// </summary>
    property Expired_Setby: Integer read FExpired_Setby write FExpired_Setby;

    /// <summary>
    ///   Benutzer ID die das Lschdatum gesetzt hat
    /// </summary>
    property Expired_Userid: Integer read FExpired_Userid write FExpired_Userid;

    /// <summary>
    ///   Beim setzen eines neuen Lschdatums
    /// </summary>
    property Expired_Modified: boolean read FExpired_Modified write FExpired_Modified;

    /// <summary>
    ///   Wenn true ist das Dokument revisionssicher
    /// </summary>
    property RevSave: Boolean read FRevSave write FRevSave;

    /// <summary>
    ///   Benutzer ID die das Dokument zuletzt gendert hat
    /// </summary>
    property ModifiedUserID: Int64 read FModifiedUserID write FModifiedUserID;

    /// <summary>
    ///   Benutzer ID die das Dokument in bearbeitung hat
    /// </summary>
    property EditmodeUserID: Int64 read FEditmodeUserID write FEditmodeUserID;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property PreviewChanging: Boolean read FPreviewChanging write FPreviewChanging;

    /// <summary>
    ///   Dokumententitel
    /// </summary>
    property DocName: WideString read FDocumentName write FDocumentName;

    property UserName: WideString read FUserName write FUserName;

    /// <summary>
    ///   Seitenzahl von der Voransicht
    /// </summary>
    property PageCount: Integer read FPageCount write SetPageCount;

    /// <summary>
    ///   True wenn Dokument in bearbeitung ist. Nicht archivierte Dokumente sind immer in bearbeitung
    /// </summary>
    property InEditMode: Boolean read GetInEditMode write FInEditMode;

    /// <summary>
    ///   True wenn Kommentare vorliegen
    /// </summary>
    /// <returns>Boolean</returns>
    property HasComment: Boolean read FHasComment write FHasComment;

    property SelectedPages: TStringList read FSelectedPages write FSelectedPages;
    property AboUserid: TStringList read FAboUserid write FAboUserid;
    property InEditIDs: string read FInEditIDs write FInEditIDs;
    property InProcess: Boolean read FInProcess;
    property InSaveProcess: Boolean read FInSaveProcess;
    property InLoadProcess: Boolean read FInLoadProcess;
    property InShowProcess: Boolean read FInShowProcess;
    property InRefreshProcess: Boolean read FInRefreshProcess;
    property InCloneProcess: Boolean read FInCloneProcess;
    property InDeleteProcess: Boolean read FInDeleteProcess;
    property InMoveProcess: Boolean read FInMoveProcess;
    //property Annotations: TList<TbfaAnnotation> read FAnnotations write FAnnotations;
    property Rights: Int64 read FRights write FRights;
    property Modified_Preview: Boolean read FModified_Preview write FModified_Preview;
    property Modified_Annos: Boolean read FModified_Annos write FModified_Annos;
    property Modified_AnnosDelete: Boolean read FModified_AnnosDelete write FModified_AnnosDelete;

    /// <summary>
    ///   das Archiv, in dem das Dokument liegt, wenn es ein SymbolicLink ist.
    /// </summary>
    property SymbolicArcID: Int64 read FSymbolicArcID write FSymbolicArcID;

    /// <summary>
    ///   die Archive, als semikolon getrennter String, wo das Dokument als SymbolicLink zustzlich liegt
    /// </summary>
    property HasSymlinks: string read FHasSymlinks write FHasSymlinks;
    property TimID: Int64 read FTimID write FTimID;
    property Alarm: Integer read FAlarm write FAlarm;
    property TimerOptions: Integer read FTimerOptions write FTimerOptions;
    property Editor: WideString read FEditor write FEditor;
    property Info: WideString read FInfo write FInfo;
    property TargetDate: Variant read FTargetDate write FTargetDate;
    property Filter: string read FFilter write FFilter;
    property OcrType: string read FOcrType write FOcrType;
    property ResubUserID: Int64 read FResubUserID write FResubUserID;
    property ImporterDocNameModified: Boolean read FImporterDocNameModified write FImporterDocNameModified;

    /// <summary>
    ///   Setzt die Standardwerte und erstellt die Objekte
    /// </summary>
		constructor Create; override;

    /// <summary>
    ///   Lscht Listen und Streams
    /// </summary>
    destructor Destroy; override;

    /// <summary>
    ///   Leert Listen und Streams und setzt Werte auf default
    /// </summary>
    procedure Reset; override;

    /// <summary>
    ///   Leert Listen und Streams und setzt Werte auf default
    /// </summary>
    procedure Clone(Source: TbfaBaseDocument; WithTempID: Boolean = False;
      WithOriStream: Boolean = True; WithProcesses: boolean = True;
      WithThumbnailView: Boolean = True;
      OnlyDoc: Boolean = False; //13.03.2019 Ticket 18518 wenn nur gespeichert wir und bleibt stehen, dann muss nicht alles neugeladen werden
      WithDocStream: Boolean = True); overload;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure StartProcess;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure EndProcess;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure StartLoadProcess;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure EndLoadProcess;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure StartShowProcess;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure EndShowProcess;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure StartRefreshProcess;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure EndRefreshProcess;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure StartSaveProcess;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure EndSaveProcess;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure StartCloneProcess;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure EndCloneProcess;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure StartDeleteProcess;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure EndDeleteProcess;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure StartMoveProcess;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure EndMoveProcess;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function GetInProcesses: Boolean;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure SetProcesses(Value: Boolean);

    ///// <Beschreibung></Beschreibung>
    ///// <Description></Description>
    //function GetAnnotation(PageIdx: Integer; LayerIdx: Int64;
      //var ContainerIdx: Integer): TbfaAnnotation;
//
    ///// <summary>
    /////   True wenn Annos vorliegen
    ///// </summary>
    //function HasAnnotation: Boolean;

    ///// <Beschreibung></Beschreibung>
    ///// <Description></Description>
    //procedure SetAnnoPageCount(OldPage, NewPage: Integer);
//
    ///// <Beschreibung></Beschreibung>
    ///// <Description></Description>
    //procedure DeleteAnno(Page: Integer; MovePage: Boolean = False);

    ///// <Beschreibung></Beschreibung>
    ///// <Description></Description>
    //property ScanPoolView: TbfaImageEnMView read FScanPoolView write FScanPoolView;
//
    ///// <Beschreibung></Beschreibung>
    ///// <Description></Description>
    //property ScanPoolViewchanged: Boolean read FScanPoolViewChanged write FScanPoolViewChanged;
//
    ///// <Beschreibung></Beschreibung>
    ///// <Description></Description>
    //property ScanPoolViewLoaded: Boolean read FScanPoolViewLoaded write FScanPoolViewLoaded;
//
    ///// <Beschreibung></Beschreibung>
    ///// <Description></Description>
    //property ThumbnailView: TbfaImageEnMView read FThumbnailView write FThumbnailView;
  end;

  PbfaBaseDocument = ^TbfaBaseDocument;


  /// <Beschreibung>bitfarm Applikationsrechte Klasse</Beschreibung>
  /// <Description>bitfarm Application rights class</Description>
  TbfaBaseApplicationRights = class(TObject)
  protected
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FErrorCode: Integer;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FAppAdmin: Boolean;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FAppImport: Boolean;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FAppView: Boolean;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FArchiveManagement: Boolean;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FUserManagement: Boolean;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FGroupManagement: Boolean;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FFieldManagement: Boolean;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FModifyUserPwd: Boolean;

    /// <Beschreibung>Passwort des Benutzers zurcksetzen</Beschreibung>
    /// <Description>Reset user's password</Description>
    FResetUserPwd: Boolean;

    /// <Beschreibung>Benutzer sperren</Beschreibung>
    /// <Description>Lock user</Description>
    FLockUser: Boolean;

    /// <Beschreibung>Workflow anlegen</Beschreibung>
    /// <Description>Create workflow</Description>
    FCreateWorkflow: Boolean;

    /// <Beschreibung>Workflow ndern</Beschreibung>
    /// <Description>Modify Workflow</Description>
    FModifyWorkflow: Boolean;

    /// <Beschreibung>Workflow lschen</Beschreibung>
    /// <Description>Delete workflow</Description>
    FDeleteWorkflow: Boolean;

    /// <Beschreibung>Workflow abbrechnen</Beschreibung>
    /// <Description>Cancel workflow</Description>
    FCancelWorkflow: Boolean;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure SetRights(HasRight: Boolean);


  public
    property AppAdmin: Boolean read FAppAdmin;
    property AppImport: Boolean read FAppImport;
    property AppView: Boolean read FAppView;
    property ArchiveManagement: Boolean read FArchiveManagement;
    property UserManagement: Boolean read FUserManagement;
    property GroupManagement: Boolean read FGroupManagement;
    property FieldManagement: Boolean read FFieldManagement;
    property ModifyUserPwd: Boolean read FModifyUserPwd;
    property ResetUserPwd: Boolean read FResetUserPwd;
    property LockUser: Boolean read FLockUser;
    property CreateWorkflow: Boolean read FCreateWorkflow;
    property ModifyWorkflow: Boolean read FModifyWorkflow;
    property DeleteWorkflow: Boolean read FDeleteWorkflow;
    property CancelWorkflow: Boolean read FCancelWorkflow;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    constructor Create;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure ClearRights;

  end;

  /// <Beschreibung>bitfarm Archivrechte Klasse</Beschreibung>
  /// <Description>bitfarm Archive rights class</Description>
  TbfaBaseArchiveRights = class(TObject)
  protected
    FErrorCode: Integer;
    FArchiveSearch: Boolean;
    FArchiveShow: Boolean;
    FArchiveModify: Boolean;
    FArchiveWrite: Boolean;
    FArchiveEmail: Boolean;
    FArchiveMove: Boolean;
    FArchiveDelete: Boolean;
    FArchiveExport: Boolean;
    FArchivePrint: Boolean;
    FArchiveLink: Boolean;
    FArchiveImport: Boolean;
    FArchiveHistory: Boolean;
    FArchiveListProtokoll: Boolean;
    FArchiveRevSave: Boolean;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure SetRights(HasRight: Boolean);

  public
    property ASearch: Boolean read FArchiveSearch;
    property AShow: Boolean read FArchiveShow;
    property AModify: Boolean read FArchiveModify;
    property AWrite: Boolean read FArchiveWrite;
    property AEmail: Boolean read FArchiveEmail;
    property AMove: Boolean read FArchiveMove;
    property ADelete: Boolean read FArchiveDelete;
    property AExport: Boolean read FArchiveExport;
    property APrint: Boolean read FArchivePrint;
    property ALink: Boolean read FArchiveLink;
    property AImport: Boolean read FArchiveImport;
    property AHistory: Boolean read FArchiveHistory;
    property AListProtokoll: Boolean read FArchiveListProtokoll;
    property ARevSave: Boolean read FArchiveRevSave;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    constructor Create;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure ClearRights;
  end;
(*
  /// <Beschreibung>bitfarm Dokumentrechte Klasse</Beschreibung>
  /// <Description>bitfarm Document rights class</Description>
  TbfaDocumentRights = class(TObject)
  end;
*)

  TbfaPermission = class(TObject)
  private
    FObj: TbfaBaseObject;
    FMandantName: WideString;
    FRead,
    FWrite,
    FModify: Boolean;
  public
    constructor Create(PObj: TbfaBaseObject; MandantName: WideString; ReadRight, WriteRight, ModifyRight: Boolean);

    property Obj: TbfaBaseObject read FObj;
    property MandantName: WideString read FMandantName write FMandantName;
    property ReadRight: Boolean read FRead write FRead;
    property WriteRight: Boolean read FWrite write FWrite;
    property ModifyRight: Boolean read FModify write FModify;
  end;

  TbfaRole = class(TObject)
  private
    FRoleName: WideString;
    FRights: Int64;
    FCfgID: Int64;

  public
    constructor Create(RName: WideString; RRights, RID: Int64);

    property RoleName: WideString read FRoleName;
    property Rights: Int64 read FRights;
    property CfgID: Int64 read FCfgID;

  end;

  TbfaLayout = class(TObject)
  private
    FCaption: WideString;
    FID: Integer;

  public
    constructor Create(Caption: WideString; ID: Integer);

    property Caption: WideString read FCaption;
    property ID: Integer read FID;

  end;

  TbfaTreeNodeData = class(TObject)
  private
    FObj: TbfaBaseObject;

    FLayoutID,
    FParentID: Int64;

    FIconIndex: Integer;

  public
    constructor Create;

    property Obj: TbfaBaseObject read FObj write FObj;
    property LayoutID: Int64 read FLayoutID write FLayoutID;
    property ParentID: Int64 read FParentID write FParentID;
    property IconIndex: Integer read FIconIndex write FIconIndex;


  end;

  PbfaTreeNodeData = ^TbfaTreeNodeData;

//  TbfaBaseSearch = class(TObject)
  TbfaBaseSearch = class(TbfaBaseObject)
  private
//    /// <Beschreibung>Name der Suche/Lesezeichen</Beschreibung>
//    /// <Description></Description>
//    FName: WideString;

//    /// <Beschreibung>ID der Suche/Lesezeichen</Beschreibung>
//    /// <Description></Description>
//    FID: Int64;

    /// <Beschreibung>Datum der Suche/Lesezeichen</Beschreibung>
    /// <Description></Description>
    FDate: Variant;//TDateTime;

    /// <Beschreibung>Typ der Suche, 0 = normale Suche, 1 = lokales Lesezeichen, 2 = globales Lesezeichen, 3 = Verlaufs-Lesezeichen, 4= WVL, 5 = Zusatzfeldsuche (Verschlagwortungshilfe)</Beschreibung>
    /// <Description></Description>
    FSearchType: Integer;

    /// <Beschreibung>eine Liste von UserIDs die berechtigt sind</Beschreibung>
    /// <Description></Description>
    FUserID: Integer;

    /// <Beschreibung>eine Liste von GroupIDs die berechtigt sind</Beschreibung>
    /// <Description></Description>
    FGroupID: Integer;

    /// <Beschreibung>eine Liste berechtigter Usernamen, deprecated, zur abwrtskompatibilitt mit 3.5, die tatschlich vorhandenen User stehen in userid</Beschreibung>
    /// <Description></Description>
//    FURights: WideString;
    FURights: Variant;

    /// <Beschreibung>eine Liste berechtigter Gruppennamen, deprecated, zur abwrtskompatibilitt mit 3.5, die tatschlich vorhandenen Gruppen stehen in groupid</Beschreibung>
    /// <Description></Description>
//    FGRights: WideString;
    FGRights: Variant;

    /// <Beschreibung>die fr die Suche selektierten Spalten der Tabelle, wird nicht angegeben wird * gesetzt, ansonsten Liste von Bezeichnern</Beschreibung>
    /// <Description></Description>
//    FSelectedCols: WideString;
    FSelectedCols: Variant;

    /// <Beschreibung>ein sql-where Statement, welches direkt ausgefhrt werden soll</Beschreibung>
    /// <Description></Description>
//    FSQLWhere: WideString;
    FSQLWhere: Variant;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FSQLWhereOperator: String;

    /// <Beschreibung>es soll nur das Where-Statement ausgefhrt werden, alle anderen ggf vorhandenen Suchparametern werden ignoriert, entspricht gbadvusrsrc, fhrt zwangslufig zu einer nicht-aufbauenden Suche</Beschreibung>
    /// <Description></Description>
    FSQLWhereOnly: Boolean;

    /// <Beschreibung>kann gesetzt werden, um zu verhindern, dass aufbauend gesucht werden kann, wird sqlwhere_only gesetzt, dann ist auch nonconstructive gesetzt</Beschreibung>
    /// <Description></Description>
    FConstructive: Boolean;

    /// <Beschreibung>die maximalen Treffer der Suche, none = alle</Beschreibung>
    /// <Description></Description>
    FMaxValues: Integer;

    /// <Beschreibung>Suchparameter Verknpfungsname</Beschreibung>
    /// <Description></Description>
//    FRefSearch: WideString;
    FRefSearch: Variant;

    /// <Beschreibung>Suchparameter Verknpfungsname negiert</Beschreibung>
    /// <Description></Description>
    FRefSearchNegated: Boolean;

    /// <Beschreibung>Suchparameter UserID</Beschreibung>
    /// <Description></Description>
    FSearchUserID: Int64;

    /// <Beschreibung>Suchparameter Username</Beschreibung>
    /// <Description></Description>
//    FSearchUser: WideString;
    FSearchUser: Variant;

    /// <Beschreibung>Suchparameter Username negiert</Beschreibung>
    /// <Description></Description>
    FSearchUserNegated: Boolean;

    /// <Beschreibung>Suchparameter Gruppe ID</Beschreibung>
    /// <Description></Description>
    FSearchGroupID: Int64;

    /// <Beschreibung>Suchparameter Gruppe</Beschreibung>
    /// <Description></Description>
//    FSearchGroup: WideString;
    FSearchGroup: Variant;

    /// <Beschreibung>Suchparameter Gruppe negiert</Beschreibung>
    /// <Description></Description>
    FSearchGroupNegated: Boolean;

    /// <Beschreibung>Suchparameter ArchivID</Beschreibung>
    /// <Description></Description>
    FArcID: TStringList;

    /// <Beschreibung>Suchparameter gdocid</Beschreibung>
    /// <Description></Description>
    FGdocID: TStringList;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FGdocIDWithSymLink: Boolean;

    /// <Beschreibung>Suchparameter timref</Beschreibung>
    /// <Description></Description>
    FTimRef: TStringList;

    /// <Beschreibung>Suchparameter Ausgecheckt</Beschreibung>
    /// <Description></Description>
    FCheckOut: Integer;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FInResub: Variant;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FInResubNegated: Boolean;

    /// <Beschreibung>Suchparameter Status</Beschreibung>
    /// <Description></Description>
    FState: Integer;

    /// <Beschreibung>Suchparameter negiert Status</Beschreibung>
    /// <Description></Description>
    FNotState: Integer;

    /// <Beschreibung>Suchparameter Datum</Beschreibung>
    /// <Description></Description>
    FSearchDate: Variant;

    /// <Beschreibung>Suchparameter Datum</Beschreibung>
    /// <Description></Description>
    FSearchDateHigh: Variant;

    /// <Beschreibung>Suchparameter Datum negiert</Beschreibung>
    /// <Description></Description>
    FSearchDateNegated: Boolean;

    /// <Beschreibung>Bitmaske - Erste Bit Volltext, zweite Bit Schlagwrte</Beschreibung>
    /// <Description></Description>
    FFtxFields: Integer;

    /// <Beschreibung>Suchparameter Volltext/Schlagwrte (FtxFields Bitmaske wird auswerten), die gesamte Suchphrase wird hier eingegeben</Beschreibung>
    /// <Description></Description>
//    FSearchString: WideString;
    FSearchString: Variant;

    /// <Beschreibung>Suchparameter negiert Volltext/Schlagworte (FtxFields Bitmaske wird auswerten)</Beschreibung>
    /// <Description></Description>
    FSearchStringNegated: Boolean;

    /// <Beschreibung>richtige Volltextsuche verwenden (MySQL)</Beschreibung>
    /// <Description></Description>
    FFtxMatchAgainst: Boolean;

    /// <Beschreibung>richtige Volltextsuche verwenden (MySQL) - Natural Mode</Beschreibung>
    /// <Description></Description>
    FFtxNaturalMode: Boolean;

    /// <Beschreibung>richtige Volltextsuche verwenden (MySQL) - Boolean Mode</Beschreibung>
    /// <Description></Description>
    FFtxBooleanMode: Boolean;

    /// <Beschreibung>richtige Volltextsuche verwenden (MySQL) - With Query Expansion</Beschreibung>
    /// <Description></Description>
    FFtxWithQueryExpansion: Boolean;

    /// <Beschreibung>statt einem Suchergebnis wird nur die Anzahl als int zurckgegeben</Beschreibung>
    /// <Description></Description>
    FCount: Boolean;

    /// <Beschreibung>'' - Dataset, 'temp' - MySql temporary table, 'tablename' - MySql table</Beschreibung>
    /// <Description></Description>
//    FResultType: WideString;
    FResultType: Variant;

    FGroupBy: Variant;

    /// <Beschreibung>die Spalte nach der das Ergebnis sortiert sein soll  TODO Liste mit Spalten</Beschreibung>
    /// <Description></Description>
//    FOrder: WideString;
    FOrder: Variant;

    /// <Beschreibung>die Sortrierreihenfolge asc oder desc</Beschreibung>
    /// <Description></Description>
//    FDirection: WideString;
    FDirection: Variant;

    /// <Beschreibung>Benutzer ID der die Suche ausgefrt hat</Beschreibung>
    /// <Description></Description>
    FUserContext: Int64;

    /// <Beschreibung>Suchparameter WVL Termin</Beschreibung>
    /// <Description></Description>
    FResubDate: Variant;

    /// <Beschreibung>Suchparameter WVL Termin</Beschreibung>
    /// <Description></Description>
    FResubDateHigh: Variant;

    /// <Beschreibung>Suchparameter WVL Erinnerung</Beschreibung>
    /// <Description></Description>
    FResubAlarm: Variant;

    /// <Beschreibung>Suchparameter WVL Erinnerung</Beschreibung>
    /// <Description></Description>
    FResubAlarmHigh: Variant;

    /// <Beschreibung>Suchparameter WVL Benutzer  ID</Beschreibung>
    /// <Description></Description>
    FResubUserID: Int64;

    /// <Beschreibung>Suchparameter negiert WVL Benutzer  ID</Beschreibung>
    /// <Description></Description>
    FResubUserIDNegated: Boolean;

    /// <Beschreibung>Suchparameter WVL Kontroller  ID</Beschreibung>
    /// <Description></Description>
    FResubControllerID: Int64;

    /// <Beschreibung>Suchparameter negiert WVL Kontroller  ID</Beschreibung>
    /// <Description></Description>
    FResubControllerIDNegated: Boolean;

    /// <Beschreibung>Suchparameter WVL Status</Beschreibung>
    /// <Description></Description>
    FResubState: Integer;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FResubDateNegated: Boolean;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FResubAlarmNegated: Boolean;

    /// <Beschreibung>Suchparameter Dokument Name</Beschreibung>
    /// <Description>Document name</Description>
//    FDocName: WideString;
    FDocName: Variant;

    /// <Beschreibung>Suchparameter negiert Dokument Name</Beschreibung>
    /// <Description></Description>
    FDocNameNegated: Boolean;

    /// <Beschreibung>Suchparameter GroupIDs, wo der Benutzer Mitglied ist</Beschreibung>
    /// <Description></Description>
    FGroupList: TStringList;

    /// <Beschreibung>Suchparameter Bearbeitung</Beschreibung>
    /// <Description></Description>
    FUseModifiedDate: Boolean;

    /// <Beschreibung>Suchparameter Dateityp</Beschreibung>
    /// <Description>Document filetype</Description>
    FFileType: Variant;

    /// <Beschreibung>Suchparameter negiert Dateityp</Beschreibung>
    /// <Description></Description>
    FFileTypeNegated: Boolean;

    /// <Beschreibung>bei Aufbauende Suche fr GUI notwendig beim mehreren welche Archive war ausgewhlt</Beschreibung>
    /// <Description></Description>
    FSelectedArcID: Int64;

    FVerify: Boolean;
    FResubStateNegated: Boolean;
    FResubUserIDList: Widestring;
    FDebug: Boolean;
    FAboUserid: Integer;

    /// <Beschreibung>Suchen wie in Version 3.5</Beschreibung>
    /// <Description></Description>
    FUseSearch35: Boolean;

    FResubNotState: integer;
    FUserList: TStringList;
    FGUID: string;
    FSearchDateType: Integer;
    FRightsMask: Integer;
    FDontInheritate: Boolean;
    FDontRefreshDataSet: Boolean;

    procedure SetSQLWhereOnly(Value: Boolean);

  protected
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
//    FAddFieldXML: WideString;
    FAddFieldXML: Variant;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
//    FStsFieldXML: WideString;
    FStsFieldXML: Variant;

  public

    constructor Create; override;
    destructor Destroy; override;

    procedure Reset; virtual;
    procedure Clone(Source: TbfaBaseSearch); virtual;
    property GeneralPropertiesModified;

//    property Name: WideString read FName write FName;
//    property ID: Int64 read FID write FID;
    property Date: Variant read FDate write FDate;
    property SelectedArcID: Int64 read FSelectedArcID write FSelectedArcID;

    /// <summary><code> 0 = normale Suche
    /// 1 = lokale Lesezeichen
    /// 2 = globale Lesezeichen
    /// 3 = Verlaufslesezeichen
    /// 4 = Wiedervorlage
    /// 5 = Verschlagwortungshilfe
    /// </code></summary>
    property SearchType: Integer read FSearchType write FSearchType;

    property UserID: Integer read FUserID write FUserID;
    property GroupID: Integer read FGroupID write FGroupID;
//    property URights: WideString read FURights write FURights;
    property URights: Variant read FURights write FURights;
//    property GRights: WideString read FGRights write FGRights;
    property GRights: Variant read FGRights write FGRights;
//    property SelectedCols: Widestring read FSelectedCols write FSelectedCols;
    property SelectedCols: Variant read FSelectedCols write FSelectedCols;
//    property SQLWhere: WideString read FSQLWhere write FSQLWhere;
    property SQLWhere: Variant read FSQLWhere write FSQLWhere;
    property SQLWhereOperator: String read FSQLWhereOperator write FSQLWhereOperator;
    property SQLWhereOnly: Boolean read FSQLWhereOnly write SetSQLWhereOnly;
    property Constructive: Boolean read FConstructive write FConstructive;
    property MaxValues: Integer read FMaxValues write FMaxValues;
//    property RefSearch: WideString read FRefSearch write FRefSearch;
    property RefSearch: Variant read FRefSearch write FRefSearch;
    property RefSearchNegated: Boolean read FRefSearchNegated write FRefSearchNegated;
    property SearchUserID: Int64 read FSearchUserID write FSearchUserID;
//    property SearchUser: WideString read FSearchUser write FSearchUser;
    property SearchUser: Variant read FSearchUser write FSearchUser;
    property SearchUserNegated: Boolean read FSearchUserNegated write FSearchUserNegated;
    property SearchGroupID: Int64 read FSearchGroupID write FSearchGroupID;
//    property SearchGroup: WideString read FSearchGroup write FSearchGroup;
    property SearchGroup: Variant read FSearchGroup write FSearchGroup;
    property SearchGroupNegated: Boolean read FSearchGroupNegated write FSearchGroupNegated;
    property ArcID: TStringList read FArcID write FArcID;
    property GdocID: TStringList read FGdocID write FGdocID;
    property GdocIDWithSymLink: Boolean read FGdocIDWithSymLink write FGdocIDWithSymLink;
    property TimRef: TStringList read FTimRef write FTimRef;
    property CheckOut: Integer read FCheckOut write FCheckOut;
    property InResub: Variant read FInResub write FInResub;
    property InResubNegated: Boolean read FInResubNegated write FInResubNegated;
    property State: Integer read FState write FState;
    property NotState: Integer read FNotState write FNotState;
    property SearchDate: Variant read FSearchDate write FSearchDate;
    property SearchDateHigh: Variant read FSearchDateHigh write FSearchDateHigh;
    property SearchDateNegated: Boolean read FSearchDateNegated write FSearchDateNegated;
    property SearchDateType: Integer read FSearchDateType write FSearchDateType;
    property FtxFields: Integer read FFtxFields write FFtxFields;
    property SearchString: Variant read FSearchString write FSearchString;
    property SearchStringNegated: Boolean read FSearchStringNegated write FSearchStringNegated;
    property FtxMatchAgainst: Boolean read FFtxMatchAgainst write FFtxMatchAgainst;
    property FtxNaturalMode: Boolean read FFtxNaturalMode write FFtxNaturalMode;
    property FtxBooleanMode: Boolean read FFtxBooleanMode write FFtxBooleanMode;
    property FtxWithQueryExpansion: Boolean read FFtxWithQueryExpansion write FFtxWithQueryExpansion;
    property Count: Boolean read FCount write FCount;
    property ResultType: Variant read FResultType write FResultType;
    property GroupBy: Variant read FGroupBy write FGroupBy;
    property Order: Variant read FOrder write FOrder;
    property Direction: Variant read FDirection write FDirection;
    property AddFieldXML: Variant read FAddFieldXML write FAddFieldXML;
    property StsFieldXML: Variant read FStsFieldXML write FStsFieldXML;
    property UserContext: Int64 read FUserContext write FUserContext;
    property ResubDate: Variant read FResubDate write FResubDate;
    property ResubDateHigh: Variant read FResubDateHigh write FResubDateHigh;
    property ResubAlarm: Variant read FResubAlarm write FResubAlarm;
    property ResubAlarmHigh: Variant read FResubAlarmHigh write FResubAlarmHigh;
    property ResubUserID: Int64 read FResubUserID write FResubUserID;
    property ResubUserIDNegated: Boolean read FResubUserIDNegated write FResubUserIDNegated;
    property ResubUserIDList: Widestring read FResubUserIDList write FResubUserIDList;
    property ResubControllerID: Int64 read FResubControllerID write FResubControllerID;
    property ResubControllerIDNegated: Boolean read FResubControllerIDNegated write FResubControllerIDNegated;
    property ResubState: Integer read FResubState write FResubState;
    property ResubNotState: integer read FResubNotState write FResubNotState;
    property ResubStateNegated: Boolean read FResubStateNegated write FResubStateNegated;
    property ResubDateNegated: Boolean read FResubDateNegated write FResubDateNegated;
    property ResubAlarmNegated: Boolean read FResubAlarmNegated write FResubAlarmNegated;
    property DocName: Variant read FDocName write FDocName;
    property DocNameNegated: Boolean read FDocNameNegated write FDocNameNegated;
    property UserList: TStringList read FUserList write FUserList;
    property GroupList: TStringList read FGroupList write FGroupList;
    property UseModifiedDate: Boolean read FUseModifiedDate write FUseModifiedDate;
    property FileType: Variant read FFileType write FFileType;
    property FileTypeNegated: Boolean read FFileTypeNegated write FFileTypeNegated;
    property Verify: Boolean read FVerify write FVerify;
    property Debug: Boolean read FDebug write FDebug;
    property AboUserid: Integer read FAboUserid write FAboUserid;
    property UseSearch35: Boolean read FUseSearch35 write FUseSearch35;
    property GUID: string read FGUID write FGUID;
    property RightsMask: Integer read FRightsMask write FRightsMask;
    property DontRefreshDataSet: Boolean read FDontRefreshDataSet write FDontRefreshDataSet;
    property DontInheritate: Boolean read FDontInheritate write FDontInheritate;
  end;

  TbfaBaseSearchProcessing = class(TObject)
  protected
    /// <Beschreibung>Versionsnummer 36 oder 40</Beschreibung>
    /// <Description></Description>
    FVersion: Integer;

  public
    property Version: Integer read FVersion write FVersion;

    /// <Beschreibung>Funktion um eine Lesezeichendatei einzulesen. Gibt eine Liste an SearchObjekten zurck. Optional kann man den Namen oder die ID eines bestimmten Lesezeichen angeben, welches dann zurckgegeben wird. Wird eine UserId/GroupID bzw eine Liste von IDs angegeben, so werden nur die Lesezeichen zurckgegeben, auf die mindestens eine (oder mehrere) ID berechtigt ist. Die Eingrenzung auf UserId hat Vorrang vor GroupID</Beschreibung>
    /// <Description></Description>
//    function ReadXMLFile(Path: WideString; Name: WideString = ''; ID: Int64 = -1; UserID: TStringList = nil; GroupID: TStringList = nil): TList<TbfaBaseSearch>;
    /// <Beschreibung>Funktion um ein SearchObject direkt mit einem XML-Knoten (fav-Node) zu fllen, bei Erfolg wird ein SearchObject zurckgegeben, andernfalls None. Der Knoten muss ein ElementTree.Element Objekt sein</Beschreibung>
    /// <Description></Description>
//    function FillSearchWithNode(Fav: IXMLNode): TbfaBaseSearch; virtual;
    /// <Beschreibung>Funktion um SearchObjekte als XML-File zu schreiben. Das Format der XML entspricht der Version 3.6. Es kann ein Objekte oder eine Liste an Objekten bergeben werden. Funktion liefert True bei Erfolg</Beschreibung>
    /// <Description></Description>
//    function WriteXMLFile(Path: WideString; Items: TList<TbfaBaseSearch>): Boolean;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
//    function GetSearchTypes(idx: Integer): String;

  end;

  TbfaRequest = class(TbfaBaseObject)
  private
    FReqType: string;
    FAnno: TObject;
    FData: TList<TbfaBaseObject>;
    FFields: TObject;
    FTasks: TObject;

    /// <Beschreibung>Wiedervorlage-Benutzer (Userid) der den Request stellt, falls vom Benutzer abweichend.</Beschreibung>
    /// <Description></Description>
    FResubUserID: Int64;

    /// <Beschreibung>Filename einer zu bermittelnden Datei.</Beschreibung>
    /// <Description></Description>
    FFileName: WideString;

    /// <Beschreibung>Lnge in Bytes einer Datenbertragung.</Beschreibung>
    /// <Description></Description>
//01.06.2023    FDataLen: Integer;
    FDataLen: Int64;

    /// <Beschreibung>
    ///    Hashwert einer zu bermittelnden Datei. Wird in einem Request ein filehash bergeben,
    ///    so wird das Dokument nur geladen, wenn der aktuelle filehash der Datei unterschiedlich ist,
    ///    also die Datei gendert wurde.
    /// </Beschreibung>
    /// <Description></Description>
    FFileHash: WideString;

    /// <Beschreibung>Dateidatum einer zu bermittelnden Datei. Wird in einem Request ein filedate bergeben,
    /// so wird das Dokument nur geladen, wenn der Modified-Zeitstempel jnger ist (os.stat.st_mtime)</Beschreibung>
    /// <Description></Description>
    FFileDate: TDateTime;

    /// <Beschreibung>Die Seite des betreffenden Dokuments.</Beschreibung>
    /// <Description></Description>
    FPage: Integer;

    /// <Beschreibung>Die Revision des betreffenden Dokuments.</Beschreibung>
    /// <Description></Description>
    FRevision: Integer;

    /// <Beschreibung>der Typ einer angefragten oder zu bertragenden Annotation 0 = public 1 = private.</Beschreibung>
    /// <Description></Description>
    FAnnoType: Integer;

    /// <Beschreibung>der Benutzer (Userid) der Annotation, falls vom Benutzer abweichend.</Beschreibung>
    /// <Description></Description>
    FAnnoUserID: Int64;

    /// <Beschreibung>der Status des betreffenden Dokuments.</Beschreibung>
    /// <Description></Description>
    FStatus: Integer;

    /// <Beschreibung>der Socket, ber den der Request kam, fr Rckantworten.</Beschreibung>
    /// <Description></Description>
    FSock: TObject;

    /// <Beschreibung>optionaler Parameter, um Request weiter zu definieren</Beschreibung>
    /// <Description></Description>
    FParameter: WideString;
    FParameterList: TStringList;
    FPrio: Integer;
    FGdocid: Int64;
    FVersion: Integer;
    FUserID: Int64;
    FDataString: WideString;
    FProfile: WideString;
    FLang: string;
    FArcid: Int64;
    FSymbolicArcID: Int64;
    FTimID: Int64;
    FDataType: WideString;
    FSessionid: string;
  public
    constructor Create; override;
    destructor Destroy; override;

    property ReqType: string read FReqType write FReqType;
    property Anno: TObject read FAnno write FAnno;
    property Data: TList<TbfaBaseObject> read FData write FData;
    property Fields: TObject read FFields write FFields;
    property Tasks: TObject read FTasks write FTasks;
    property UserID: Int64 read FUserID write FUserID;
    property ResubUserID: Int64 read FResubUserID write FResubUserID;
    property FileName: WideString read FFileName write FFileName;
//01.06.2023    property DataLen: Integer read FDataLen write FDataLen;
    property DataLen: Int64 read FDataLen write FDataLen;
    property FileHash: WideString read FFileHash write FFileHash;
    property FileDate: TDateTime read FFileDate write FFileDate;
    property Page: Integer read FPage write FPage;
    property Revision: Integer read FRevision write FRevision;
    property AnnoType: Integer read FAnnoType write FAnnoType;
    property AnnoUserID: Int64 read FAnnoUserID write FAnnoUserID;
    property Status: Integer read FStatus write FStatus;
    property Sock: TObject read FSock write FSock;
    property Parameter: WideString read FParameter write FParameter;
    property ParameterList: TStringList read FParameterList write FParameterList;
    property Prio: Integer read FPrio write FPrio;
    property Gdocid: Int64 read FGdocid write FGdocid;
    property Arcid: Int64 read FArcid write FArcid;
    property Version: Integer read FVersion write FVersion;
    property DataString: WideString read FDataString write FDataString;
    property Profile: WideString read FProfile write FProfile;
    property Lang: String read FLang write FLang;
    property SymbolicArcID: Int64 read FSymbolicArcID write FSymbolicArcID;
    property TimID: Int64 read FTimID write FTimID;
    property DataType: WideString read FDataType write FDataType;
    property Sessionid: string read FSessionid write FSessionid;
  end;

  TbfaPredictionResult = class(TbfaBaseObject)
  public
    GUID: string;
    Page: Integer;
    ClassID: Integer;

    constructor Create; override;
    destructor Destroy; override;
    procedure Clone(Source: TbfaPredictionResult);
  end;

  TbfaColumnConfig = class(TbfaBaseObject)
    Name: string;
    Fieldname: string;   // der Feldname referenziert die Datenbankspalte (vom Typ zfld14_GridColumn)
    ColumnType: TbfaColumnType;
    ValueType: TbfaValueType;
    SummaryType: TbfaSummaryKind;
    ReadOnly: Boolean;
    DisplayFormat: string;
    EditFormat: string;          // derzeit wird nur Displayformat verwendet
    Expression: string;
    DefaultValue: string;        // der default Wert des Feldes, falls gewnscht
    ListType: TbfaListType;
    ListItems: TStringList;
    ListAddfield: Integer;     // falls Werte eines Zusatzfeldes
    ListQuery: string;         // oder mittels einer Query ins Auswahlfeld sollen
    ColWidth: Integer;
    constructor Create;
    procedure Clone(Source: TbfaColumnConfig);
  end;

  TbfaGridConfig = class(TbfaBaseObject)
    Displayname: string;
    ShowHeader: Boolean;
    ShowSummary: Boolean;
    ShowGroupbybox: Boolean;
    ShowIndicator: Boolean;
    UsePopups: Boolean;
    UseColumnNames: Boolean;
    ExportAddfields: Boolean;
    ExportUnicode: Boolean;
    ManualSorting: Boolean;
    AlternateRows: Boolean;
    MultiSelect: Boolean;
    Stamp_Transparent: Boolean;
    Stamp_Size: Integer;
    Stamp_Color: string;
    SearchField: string;
    Columns: TList<TbfaColumnConfig>;
    constructor Create;
    procedure Reset;
    procedure Clone(Source: TbfaGridConfig);
  end;


  TArchiveRec = record
    LayoutID: Int64;
    ArchiveID: Int64;
    ParentID: Int64;
    ArcName: WideString;
    BgrPicture: WideString;
    ArcType: Integer;
    DocName: WideString;
    InheritPermissions,
    InheritPermissionsModified,
    InheritFields,
    InheritFieldsModified: Boolean;
    TextColor: Integer;
    Description: WideString;
    ArchiveLayout: TMemoryStream;
    Kind: TbfaViewKind;
    Icon: Integer;
    UserContainerID,
    FieldContainerID: Int64;
  end;

  TLayoutRec = record
    LayoutID: Int64;
    ParentID: Int64;
    ObjID: Int64;
    ObjType: Integer;
    Level: Integer;
    IdxL: Int64;
    IdxR: Int64;
    Rights: Int64;
    RoleID: Int64;
  end;

  TBaseFldRec = record
    FldID: Int64;
    DisplayName: WideString;
    FieldType: Integer;
    ValueType: Integer;
    ControlType: Integer;
    Mode: Integer;
    Options: Integer;
    DefaultValue: WideString;
    IndexTableID: Integer;
    Description: WideString;
    Enabled: Boolean;
    Size: Integer;
    ControlItems: WideString;
    Status: Integer;
  end;

  TFldRec = record
    FldID: Int64;
    ArchiveID: Int64;
    DisplayName: WideString;
    FieldType: Integer;
    ValueType: Integer;
    ControlType: Integer;
    Mode: Integer;
    Options: Integer;
    DefaultValue: WideString;
    IndexTableID: Integer;
    Description: WideString;
    Enabled: Boolean;
    Size: Integer;
    ControlItems: WideString;
    Status: Integer;
    UserRead,
    UserWrite,
    UserModify,
    HasRights: Boolean;
  end;

  TDocumentRec = record
    ArcID: Int64;
    Documentname: WideString;
    DocLoc: WideString;
    ExtLoc: WideString;
    OriLoc: WideString;
    DocTXT: WideString;
    Vocabulary: WideString;
    CheckedOut: Boolean;
		DocStream: TMemoryStream;
		ORIStream: TMemoryStream;
		IEVStream: TMemoryStream;
    LetterStream: TMemoryStream;
		CheckOutUserID: Int64;
    PageCount: Integer;
    DocType: Integer;
    FileType: WideString;
    FilePath: WideString;
    Image: Boolean;
    UserID: Int64;
    DocDate: string;
    RevSave: Boolean;
    Modified: Boolean;
    ModifiedUserID: Int64;
  end;


implementation

uses
  Variants, uni, ActiveX, bfaCommonMethods,
  Graphics, Controls, System.IOUtils;

constructor TbfaBaseObject.Create;
begin
  FID := -1;
  FName := '';
  FDescription := '';
  FObjType := -1;
  FErrorCode := -1;
  FStatus := -1;
  FInitialized := False;
  FLoaded := False;
  FModified := False;
  FGeneralPropertiesModified := False;
  FSortingPos := -1;
end;

destructor TbfaBaseObject.Destroy;
begin
  inherited Destroy;
end;

procedure TbfaBaseObject.Clone(Source: TbfaBaseObject);
begin
  FID := Source.ID;
  FName := Source.Name;
  FDescription := Source.Description;
  FObjType := Source.Typ;
  FErrorCode := Source.ErrorCode;
  FStatus := Source.Status;
  FInitialized := Source.Initialized;
  FLoaded := Source.Loaded;
  FModified := Source.Modified;
  FGeneralPropertiesModified := Source.GeneralPropertiesModified;
  FAddfields_Arcid := Source.Addfields_Arcid;
  FSortingPos := Source.SortingPos;
end;

procedure TbfaBaseObject.Reset;
begin
  FID := -1;
  FName := '';
  FDescription := '';
  FObjType := -1;
  FErrorCode := -1;
  FStatus := -1;
  FInitialized := False;
  FLoaded := False;
  FModified := False;
  FGeneralPropertiesModified := False;
  FSortingPos := -1;
end;

procedure TbfaBaseObject.SetName(Value: WideString);
begin
  FName := Value;
  FGeneralPropertiesModified := True;
//  FModified := True;
end;

procedure TbfaBaseObject.SetStatus(Value: Integer);
begin
  FStatus := Value;
  FGeneralPropertiesModified := True;
  //FModified := True;
end;

procedure TbfaBaseObject.SetDescription(Value: WideString);
begin
  FDescription := Value;
  FGeneralPropertiesModified := True;
  //FModified := True;
end;

procedure TbfaBaseObject.SetLoaded(const Value: Boolean);
begin
  FLoaded := Value;
end;

constructor TbfaRole.Create(RName: WideString; RRights, RID: Int64);
begin
  FRoleName := RName;
  FRights := RRights;
  FCfgID := RID;
end;

constructor TbfaLayout.Create(Caption: WideString; ID: Integer);
begin
  FCaption := Caption;
  FID := ID;
end;

constructor TbfaPermission.Create(PObj: TbfaBaseObject; MandantName: WideString; ReadRight, WriteRight, ModifyRight: Boolean);
begin
  FObj := PObj;
  FMandantName := MandantName;
  FRead := ReadRight;
  FWrite := WriteRight;
  FModify := ModifyRight;
end;

constructor TbfaBaseDocument.Create;
begin
  try
    inherited Create;

    CreateGUID(FTempID);
    FStreamTIF := TMemoryStream.Create;
//26.05.2023    FStreamORI := TMemoryStream.Create;
    FStreamORI := nil;
    FStreamIEV := TMemoryStream.Create;
    FStreamLetter := TMemoryStream.Create;
    //FAnnotations := TList<TbfaAnnotation>.Create;
    FSelectedPages := TStringList.Create;
    FSelectedPages.Delimiter := ';';
    FSelectedPages.StrictDelimiter := True;
    FAboUserid := TStringList.Create;
    FAboUserid.Delimiter := ';';
    FAboUserid.StrictDelimiter := True;

    //FScanPoolView := nil;
    //FThumbnailView := nil;
    Reset;
  except

  end;
end;

procedure TbfaBaseDocument.Reset;
var
  i: Integer;
begin
  inherited Reset;
  FPageCount := 0;
  FArcID := -1;
  FDocLoc := '';
  FExtLoc := '';
  FOriLoc := '';
  FSrcLoc := '';
  FDocTXT := '';
  FDocTags := '';
  FDocumentName := '';
  FCheckOutUserID := -1;
  FDocType := -1;
  FFileType := '';
  FHasPreview := False;
  FUserID := -1;
  FDocDate := NULL;
  FRevSave := False;
  FModified := False;
  FModifiedUserID := -1;
  FPreviewChanging := False;
  FSelectedPages.Clear;
  FAboUserid.Clear;
  FInitialized := False;
  FInProcess := False;
  FInLoadProcess := False;
  FInCloneProcess := False;
  FInSaveProcess := False;
  FInShowProcess := False;
  FInRefreshProcess := False;
  FInDeleteProcess := False;
  FInMoveProcess := False;
  FStreamTIF.Clear;
//24.05.2023  FStreamORI.Clear;
  if FStreamORI <> nil then
    FStreamOri.Free;
  FStreamIEV.Clear;
  FStreamLetter.Clear;
  //for i := FAnnotations.Count - 1 downto 0 do
    //FAnnotations[i].Free;
  //FAnnotations.Clear;

  FDocType := dtEmpty;
  FHasPreview := False;
  FDocDate := Now();
  FRevSave := False;
  FModifiedUserID := -1;
  FPreviewChanging := False;
  FInEditMode := False;
  FModified_Preview := False;
  FModified_Annos := False;
  FModified_AnnosDelete := False;
  FModified := False;
  FSelectedPages.Clear;
  SetLength(DocLinkList, 0);
  FSymbolicArcID := -1;
  FHasSymlinks := '';
  FTimID := -1;
  FFilter := '';
  FInitialized := True;
  FAlarm := 0;
  FTimerOptions := 0;
  FEditor := '';
  FInfo := '';
  FTargetDate := NULL;
  FModifiedDate := NULL;
  FImporterDocNameModified := False;
  //FScanPoolViewChanged := False;
  //FScanPoolViewLoaded := False;
  FHasComment := False;
end;

//function TbfaBaseDocument.HasAnnotation: Boolean;
//begin
  //try
    //Result := FAnnotations.Count > 0;
  //except
    //Result := False;
  //end;
//end;
//
//function TbfaBaseDocument.GetAnnotation(PageIdx: Integer; LayerIdx: Int64;
  //var ContainerIdx: Integer): TbfaAnnotation;
//var
  //found: Boolean;
  //i: Integer;
//begin
  //Result := nil;
  //try
    //if FAnnotations = nil then
      //Exit;
//
    //try
      //ContainerIdx := -1;
      //if FAnnotations.Count > 0 then
      //begin
        //i := 0;
        //found := False;
        //while ((not found) and (i <= FAnnotations.Count - 1)) do
        //begin
          //if (FAnnotations[i].Page = PageIdx) and (LayerIdx = -1) then
            //found := True
          //else if (FAnnotations[i].Page = PageIdx) and (FAnnotations[i].Layer = LayerIdx) then
            //found := True
          //else
            //inc(i);
        //end;
        //if found then
        //begin
          //ContainerIdx := i;
          //Result := FAnnotations[i];
        //end;
      //end;
    //except
      //ContainerIdx := -1;
      //Result := nil;
    //end;
  //except
    //Result := nil;
  //end;
//end;

destructor TbfaBaseDocument.Destroy;
var
  i: Integer;
begin
  try
    try
      FStreamTIF.Clear;
      FreeAndNil(FStreamTIF);
    except
      //17.01.2019
      FStreamTIF := nil;
    end;
    try
//24.05.2023      FStreamORI.Clear;
      if FStreamORI <> nil then
        FStreamORI.Free;
      FStreamORI := nil;
//24.05.2023      FreeAndNil(FStreamORI);
    except
      //17.01.2019
      FStreamORI := nil;
    end;
    try
      FStreamIEV.Clear;
      FreeAndNil(FStreamIEV);
    except
      //17.01.2019
      FStreamIEV := nil;
    end;    
    try
      FStreamLetter.Clear;
      FreeAndNil(FStreamLetter);
    except
      //17.01.2019
      FStreamLetter := nil;
    end;
    try
      FSelectedPages.Clear;
      FreeAndNil(FSelectedPages);
    except
      //17.01.2019
      FSelectedPages := nil;
    end;

    try
      FAboUserid.Clear;
      FreeAndNil(FAboUserid);
    except
      //17.01.2019
      FAboUserid := nil;
    end;

    //try
      //for i := FAnnotations.Count - 1 downto 0 do
      //begin
        //try
          //FAnnotations[i].Free;
        //finally
          //FAnnotations[i] := nil;
        //end;
      //end;
      //FAnnotations.Clear;
      //FreeAndNil(FAnnotations);
    //except
      ////17.01.2019
      //FAnnotations := nil;
    //end;

    try
      SetLength(DocLinkList, 0);
    except
      //17.01.2019
    end;

    //try
      //if FScanPoolView <> nil then
      //begin
        //try
////17.01.2019          FScanPoolView.visible := False;
          //FScanPoolView.Clear;
////17.01.2019          FScanPoolView.Parent := nil;
        //finally
          //FScanPoolView := nil;
        //end;
      //end;
//
      //if FThumbnailView <> nil then
      //begin
        //try
////17.01.2019          FThumbnailView.Visible := False;
          //FThumbnailView.Clear;
////17.01.2019          FThumbnailView.Parent := nil;
        //finally
          //FThumbnailView := nil;
        //end;
      //end;
    //except
      ////09.01.2019
    //end;
    
(*17.01.2019  

//24.05.2018 Ticket 14576 -->
    try
      if FScanPoolView <> nil then
      begin
        try
          FScanPoolView.visible := False;
          FScanPoolView.Clear;
          FScanPoolView.Parent := nil;
  //04.01.2019 Sonst CrashLog nach archivieren beim scannen         FScanPoolView.Free;
        finally
          FScanPoolView := nil;
        end;
      end;

      if FThumbnailView <> nil then
      begin
        try
          FThumbnailView.Visible := False;
          FThumbnailView.Clear;
          FThumbnailView.Parent := nil;
  //04.01.2019 Sonst CrashLog nach archivieren beim scannen         FThumbnailView.Free;
        finally
          FThumbnailView := nil;
        end;
      end;
    except
      //09.01.2019
    end;  *)
//24.05.2018 Ticket 14576 <--
  finally
    inherited Destroy;
  end;
end;

procedure TbfaBaseDocument.Clone(Source: TbfaBaseDocument;
  WithTempID: Boolean = False; WithOriStream: Boolean = True;
  WithProcesses: boolean = True; WithThumbnailView: Boolean = True;
  OnlyDoc: Boolean = False; //13.03.2019 Ticket 18518 wenn nur gespeichert wir und bleibt stehen, dann muss nicht alles neugeladen werden
  WithDocStream: Boolean = True);
var
  i: Integer;
  //AnnoObj: TbfaAnnotation;
  BytesCopied, AllBytesCopied: Int64;
  ChunkSize: Integer;
begin
  FInCloneProcess := True;
  try
    Source.StartCloneProcess;
    FErrorCode := 0;
    FID := Source.ID;
    FSortingPos := Source.SortingPos;
    if WithTempID then
    begin
      if GUIDToString(Source.TempID) <> cEmptyGUID then
        FTempID := Source.TempID
      else
        CreateGUID(FTempID);
    end;
    FArcID := Source.ArcID;
    FDocDate := Source.DocDate;
    FDocLoc := Source.DocLoc;
    FExtLoc := Source.ExtLoc;
    FOriLoc := Source.OriLoc;
    FDocumentName := Source.DocName;
    FDocTXT := Source.FullTXT;
    FDocTags := Source.DocTags;
    FLoaded := Source.Loaded;
    if not OnlyDoc then
    begin
      if Source.DocStream <> nil then
        Source.DocStream.Position := 0;
      FStreamTIF.Clear;
      if WithDocStream then
      begin
        FStreamTIF.Position := 0;
        try
          if (Source.DocStream <> nil) and (Source.DocStream.Size > 0) then
            FStreamTIF.CopyFrom(Source.DocStream, Source.DocStream.Size);
        except
          FErrorCode := ecDocumentFailureInClone;
        end;
        FStreamTIF.Position := 0;
      end;

      if WithOriStream then
      begin
        if Source.OriStream <> nil then //26.05.2023 Umbau TFileStream
        begin
//06.06.2023 -->
//          Source.OriStream.Position := 0;
//          FStreamORI.Position := 0;
//          try
//            FStreamORI.CopyFrom(Source.OriStream, Source.OriStream.Size);
//          except
//            FErrorCode := ecDocumentFailureInClone;
//          end;
//          FStreamORI.Position := 0;
            Source.OriStream.Position := 0;
            try
              if FStreamORI = nil then
                FStreamORI := TFileStream.Create(TFileStream(Source.OriStream).FileName, fmOpenReadWrite or fmShareDenyNone);
            finally
              FStreamORI.Position := 0;
            end;
//06.06.2023 <--
        end;
      end;

      Source.AnnoStream.Position := 0;
      FStreamIEV.Position := 0;
      try
        FStreamIEV.CopyFrom(Source.AnnoStream, Source.AnnoStream.Size);
      except
        FErrorCode := ecDocumentFailureInClone;
      end;
      FStreamIEV.Position := 0;
      Source.LetterStream.Position := 0;
      FStreamLetter.Position := 0;
      try
        FStreamLetter.CopyFrom(Source.LetterStream, Source.LetterStream.Size);
      except
        FErrorCode := ecDocumentFailureInClone;
      end;
      FStreamLetter.Position := 0;

      //FAnnotations.Clear;
      //for i := 0 to Source.Annotations.Count - 1 do
      //begin
        //AnnoObj := TbfaAnnotation.Create;
        //AnnoObj.Clone(Source.Annotations[i]);
        //FAnnotations.Add(AnnoObj);
      //end;

      FSelectedPages.CommaText := Source.SelectedPages.CommaText;
      FSelectedPages.Clear;
      for i := 0 to Source.SelectedPages.Count - 1 do
        FSelectedPages.Add(Source.SelectedPages[i]);
      FHasPreview := Source.HasPreview;
    end else
      FHasPreview := False;   // wenn wir OnlyDoc clonen, kann der Clon keine Preview haben!

    SetLength(DocLinkList, 0);
    if Length(Source.DocLinkList) > 0 then
    begin
      SetLength(DocLinkList, Length(Source.DocLinkList));
      for i := Low(Source.DocLinkList) to High(Source.DocLinkList) do
      begin
        DocLinkList[i] := Source.DocLinkList[i];
      end;
    end;

    FCheckOutUserID := Source.CheckOutUserID;
    FDocType := Source.DocType;
    FFileType := Source.FileType;
    FUserID := Source.UserID;
    FDocDate := Source.DocDate;
    FRevSave := Source.RevSave;
    FModified := Source.Modified;
    FModifiedUserID := Source.ModifiedUserID;
    FEditmodeUserID := Source.EditmodeUserID;
    FPreviewChanging := Source.PreviewChanging;
    FRights := Source.Rights;
    FInitialized := Source.Initialized;
    FInEditMode := Source.InEditMode;
    FModified_Preview := Source.Modified_Preview;
    if Source.SymbolicArcID > 0 then
      FSymbolicArcID := Source.SymbolicArcID;
    FHasSymlinks := Source.HasSymlinks;
    FTimID := Source.TimID;
    FFilter := Source.Filter;
    FAboUserid.CommaText := Source.AboUserid.CommaText;
    FAlarm := Source.Alarm;
    FTimerOptions := Source.TimerOptions;
    FEditor := Source.Editor;
    FInfo := Source.Info;
    FTargetDate := Source.TargetDate;
    FResubuserID := Source.ResubUserID;
    FModifiedDate := Source.ModifiedDate;
    FExpired := Source.Expired;
    FExpired_Setby := Source.Expired_Setby;
    FExpired_Userid := Source.Expired_Userid;
    FExpired_Modified := Source.Expired_Modified;
    FInEditIDs := Source.InEditIDs;
    FImporterDocNameModified := Source.ImporterDocNameModified;
    //FScanPoolViewLoaded := Source.ScanPoolViewLoaded;
    //FScanPoolViewChanged := Source.ScanPoolViewchanged;
    FHasComment := Source.HasComment;
    if WithProcesses then
    begin
      FInProcess := Source.InProcess;
      FInLoadProcess := Source.InLoadProcess;
      FInCloneProcess := Source.InCloneProcess;
      FInSaveProcess := Source.InSaveProcess;
      FInShowProcess := Source.InShowProcess;
      FInDeleteProcess := Source.InDeleteProcess;
      FInMoveProcess := Source.InMoveProcess;
    end;
  finally
    FInCloneProcess := False;
    Source.EndCloneProcess;
  end;
end;
//
procedure TbfaBaseDocument.SetPageCount(NewPageCount: Integer);
begin
  FErrorCode := 0;
  try
    FPageCount := NewPageCount;
  except
    FErrorCode := ecDocumentFailureinSetPageCount;
  end;
end;

//procedure TbfaBaseDocument.SetAnnoPageCount(OldPage, NewPage: Integer);
//var
  //i: Integer;
  //found: Boolean;
//begin
  //FErrorCode := 0;
  //try
    //if ((FID = -1)) then// and ((FDocType = dtEmpty) or (FDocType = dtScannedPage))) then
    //begin
      //i := 0;
      //found := False;
      //while ((not found) and (i <= FAnnotations.Count - 1)) do
      //begin
        //if FAnnotations[i].Page = OldPage then
        //begin
          //found := True;
          //FAnnotations[i].Page := NewPage;
        //end;
//
        //inc(i);
      //end;
    //end
    //else
      //FErrorCode := ecDocumentPageCountCannotbeSetbyArchivedDocument;
//
  //except
    //FErrorCode := ecDocumentFailureinSetPageCount;
  //end;
//end;
//
//procedure TbfaBaseDocument.DeleteAnno(Page: Integer; MovePage: Boolean = False);
//var
  //i: Integer;
  //found: Boolean;
//begin
  //try
    //if (FID = -1)  then //or ((FID <> -1) and (FInEditMode)) then // and ((FDocType = dtEmpty) or (FDocType = dtScannedPage))) then
    //begin
      //i := 0;
      //found := False;
      //while ((not found) and (i <= FAnnotations.Count - 1)) do
      //begin
        //if FAnnotations[i].Page = Page then
          //found := True
        //else
          //inc(i);
      //end;
//
      //if found then
      //begin
        //try
          //FAnnotations[i].AnnoStream.Clear;
          //FAnnotations[i].AnnoStream.Free;
          //FAnnotations[i].AnnoStream := nil;
          //FAnnotations[i].Free;
          //FAnnotations[i] := nil;
        //finally                   //17.01.2019
          //FAnnotations.Delete(i);
        //end;
      //end;
//
      //if MovePage then
      //begin
        //for i := 0 to FAnnotations.Count - 1 do
        //begin
          //if FAnnotations[i].Page > Page then
            //FAnnotations[i].Page := FAnnotations[i].Page - 1;
        //end;
      //end;
    //end;
  //except
    //FErrorCode := ecDocumentFailureinSetPageCount;
  //end;
//end;

function TbfaBaseDocument.GetInEditmode: Boolean;
begin
  Result := ((FInEditMode) or (FID = -1));
end;

function TbfaBaseDocument.GetInProcesses;
begin
  Result := FInProcess or FInSaveProcess or FInLoadProcess or FInCloneProcess or
    FInShowProcess or FInRefreshProcess or FInDeleteProcess or FInMoveProcess;
end;

procedure TbfaBaseDocument.SetProcesses(Value: Boolean);
begin
  FInProcess := Value; 
  FInSaveProcess := Value;
  FInLoadProcess := Value;
  FInCloneProcess := Value;
  FInShowProcess := Value;
  FInRefreshProcess := Value;
  FInDeleteProcess := Value;
  FInMoveProcess := Value;
end;

procedure TbfaBaseDocument.StartProcess;
begin
  FInProcess := True;
end;

procedure TbfaBaseDocument.EndProcess;
begin
  FInProcess := False;
end;

procedure TbfaBaseDocument.StartLoadProcess;
begin
  FInLoadProcess := True;
end;

procedure TbfaBaseDocument.EndLoadProcess;
begin
  FInLoadProcess := False;
end;

procedure TbfaBaseDocument.StartShowProcess;
begin
  FInShowProcess := True;
end;

procedure TbfaBaseDocument.EndShowProcess;
begin
  FInShowProcess := False;
end;

procedure TbfaBaseDocument.StartRefreshProcess;
begin
  FInRefreshProcess := True;
end;

procedure TbfaBaseDocument.EndRefreshProcess;
begin
  FInRefreshProcess := False;
end;

procedure TbfaBaseDocument.StartSaveProcess;
begin
  FInSaveProcess := True;
end;

procedure TbfaBaseDocument.EndSaveProcess;
begin
  FInSaveProcess := False;
end;

procedure TbfaBaseDocument.StartCloneProcess;
begin
  FInCloneProcess := True;
end;

procedure TbfaBaseDocument.EndCloneProcess;
begin
  FInCloneProcess := False;
end;

procedure TbfaBaseDocument.StartDeleteProcess;
begin
  FInDeleteProcess := True;
end;

procedure TbfaBaseDocument.EndDeleteProcess;
begin
  FInDeleteProcess := False;
end;

procedure TbfaBaseDocument.StartMoveProcess;
begin
  FInMoveProcess := True;
end;

procedure TbfaBaseDocument.EndMoveProcess;
begin
  FInMoveProcess := False;
end;

constructor TbfaBaseField.Create;
begin
  try
    inherited Create;

    FDisplayName := '';
    FDefaultValue := '';
    FIndexTableID := 0;
    FValueType := -1;
    FControlType := -1;
    FControlItems := '';
    FEnabled := False;
    FSize := -1;
    FMode := 0;
    FOptions := 0;
    FValue := NULL;
    FValueHigh := NULL;
    FOldValue := NULL;
    FModifiedSearch := False;
    FNegate := False;
    FFieldRights := TList<TbfaFieldRight>.Create;
  except

  end;
end;

destructor TbfaBaseField.Destroy;
//04.05.2020var
//  i: Integer;
begin
  FFieldRights.Clear;
  inherited Destroy;
end;

procedure TbfaBaseField.Clone(Source: TbfaBaseField);
var
  i: Integer;
  fieldright: TbfaFieldRight;
begin
  inherited Clone(Source);
  FDisplayName := Source.DisplayName;
  FDefaultValue := Source.DefaultValue;
  FIndexTableID := Source.IndexTableID;

  FAssigned := Source.Assigned;
  FControlType := Source.ControlType;
  FControlItems := Source.ControlItems;
  FEnabled := Source.Enabled;
  FSize := Source.Size;
  FMode := Source.Mode;
  FOptions := Source.Options;
  FValue := Source.Value;
  FValueHigh := Source.ValueHigh;
  FOldValue := Source.OldValue;
  FModifiedSearch := Source.ModifiedSearch;
  FNegate := Source.Negate;
  FOr_Junction := Source.Or_Junction;
  FFlags := Source.FFlags;
  FFieldpos := Source.Fieldpos;
  for i := 0 to Source.FieldRights.Count -1 do
  begin
    fieldright := TbfaFieldRight.Create;
    fieldright.Clone(Source.FieldRights[i]);
    FFieldRights.Add(fieldright);
  end;
end;

//
constructor TbfaBaseApplicationRights.Create;
begin
  inherited Create;
  FErrorCode := 0;
  FAppAdmin := False;
  FAppImport := False;
  FAppView := False;
  ClearRights;
end;

//
procedure TbfaBaseApplicationRights.ClearRights;
begin
  SetRights(False);
end;

//
procedure TbfaBaseApplicationRights.SetRights(HasRight: Boolean);
begin
  FArchiveManagement := HasRight;
  FUserManagement := HasRight;
  FGroupManagement := HasRight;
  FFieldManagement := HasRight;
  FModifyUserPwd := HasRight;
  FResetUserPwd := HasRight;
  FLockUser := HasRight;
  FCreateWorkflow := HasRight;
  FModifyWorkflow := HasRight;
  FDeleteWorkflow := HasRight;
  FCancelWorkflow := HasRight;
end;

//
constructor TbfaBaseArchiveRights.Create;
begin
  inherited Create;
  FErrorCode := 0;
  ClearRights;
end;

//
procedure TbfaBaseArchiveRights.ClearRights;
begin
  SetRights(False);
end;

//
procedure TbfaBaseArchiveRights.SetRights(HasRight: Boolean);
begin
  FArchiveSearch := HasRight;
  FArchiveShow := HasRight;
  FArchiveModify := HasRight;
  FArchiveWrite := HasRight;
  FArchiveEmail := HasRight;
  FArchiveMove := HasRight;
  FArchiveDelete := HasRight;
  FArchiveExport := HasRight;
  FArchivePrint := HasRight;
  FArchiveLink := HasRight;
  FArchiveImport := HasRight;
  FArchiveHistory := HasRight;
  FArchiveListProtokoll := HasRight;
  FArchiveRevSave := HasRight;
end;

constructor TbfaTreeNodeData.Create;
begin
  FObj := TbfaBaseObject.Create;
end;

constructor TbfaBaseSearch.Create;
begin
  Reset;
end;

destructor TbfaBaseSearch.Destroy;
begin
  FreeAndNil(FArcID);
  FreeAndNil(FGDocID);
  FreeAndNil(FTimRef);
  FreeAndNil(FGroupList);
  FreeAndNil(FUserList);
  inherited Destroy;
end;

procedure TbfaBaseSearch.Reset;
begin
  FName := '';
  FID := -1;
  FDate := NULL;
  FSearchType := -1;
  FUserID := -1;
  FGroupID := -1;

  FURights := NULL;
  FGRights := NULL;
  FSelectedCols := NULL;
  FSQLWhere := NULL;
  FSQLWhereOperator := 'and';
  FSQLWhereOnly := False;
  FConstructive := False;
  FMaxValues := -1;
  FRefSearch := NULL;
  FRefSearchNegated := False;
  FSearchUserID := -1;
  FSearchUser := NULL;
  FSearchUserNegated := False;
  FSearchGroupID := -1;
  FSearchGroup := NULL;
  FSearchGroupNegated := False;
  if FArcID <> nil then
    FreeAndNil(FArcID);
  FArcID := TStringList.Create;
  FArcID.Duplicates := dupIgnore;
  if FGdocID <> nil then
    FreeAndNil(FGdocID);
  FGdocID := TStringList.Create;
  FGdocID.Duplicates := dupIgnore;
  if FTimRef <> nil then
    FreeAndNil(FTimRef);
  FTimRef := TStringList.Create;
  FTimRef.Duplicates := dupIgnore;
  FCheckOut := -1;
  FInResub := NULL;
  FInResubNegated := False;
  FState := 0;
  FNotState := 0;
  FSearchString := NULL;
  FSearchStringNegated := False;
  FSearchDate := NULL;
  FSearchDateHigh := NULL;
  FSearchDateNegated := False;
  FSearchDateType := 0;
  FFtxMatchAgainst := False;
  FCount := False;
  FFtxNaturalMode := False;
  FFtxBooleanMode := False;
  FFtxWithQueryExpansion := False;
  FResultType := NULL;
  FGroupBy := NULL;
  FOrder := NULL;
  FDirection := NULL;
  FAddFieldXML := NULL;
  FStsFieldXML := NULL;
  FFtxFields := -1;
  FUserContext := -1;
  FResubDate := NULL;
  FResubDateHigh := NULL;
  FResubDateNegated := False;
  FResubAlarm := NULL;
  FResubAlarmHigh := NULL;
  FResubAlarmNegated := False;
  FResubUserID := -1;
  FResubUserIDNegated := False;
  FResubUserIDList := '';
  FResubControllerID := -1;
  FResubControllerIDNegated := False;
  FResubState := -1;
  FResubNotState := -1;
  FResubStateNegated := False;
  FDocName := NULL;
  FDocNameNegated := False;
  if FGroupList <> nil then
    FreeAndNil(FGroupList);
  if FUserList <> nil then
    FreeAndNil(FUserList);
  FGroupList := TStringList.Create;
  FGroupList.Duplicates := dupIgnore;
  FUserList := TStringList.Create;
  FUserList.Duplicates := dupIgnore;
  FUseModifiedDate := False;
  FFileType := NULL;
  FFileTypeNegated := False;
  FSelectedArcID := -1;
  FVerify := False;
  FGdocIDWithSymLink := False;
  FDebug := False;
  FAboUserid := -1;
  FUseSearch35 := False;
  FRightsMask := -1;
  FDontRefreshDataSet := False;
  FDontInheritate := False;
end;

procedure TbfaBaseSearch.Clone(Source: TbfaBaseSearch);
begin
  inherited Clone(Source);
  FName := Source.Name;
  FID := Source.ID;
  FDate := Source.Date;
  FSearchType := Source.SearchType;
  FUserID := Source.UserID;
  FGroupID := Source.GroupID;
  FURights := Source.URights;
  FGRights := Source.GRights;
  FSelectedCols := Source.SelectedCols;
  FSQLWhere := Source.SQLWhere;
  FSQLWhereOperator := Source.SQLWhereOperator;
  FSQLWhereOnly := Source.SQLWhereOnly;
  FConstructive := Source.Constructive;
  FMaxValues := Source.MaxValues;
  FRefSearch := Source.RefSearch;
  FRefSearchNegated := Source.RefSearchNegated;
  FSearchUserID := Source.SearchUserID;
  FSearchUser := Source.SearchUser;
  FSearchUserNegated := Source.SearchUserNegated;
  FSearchGroupID := Source.SearchGroupID;
  FSearchGroup := Source.SearchGroup;
  FSearchGroupNegated := Source.SearchGroupNegated;
  if FArcID <> nil then
    FreeAndNil(FArcID);
  FArcID := TStringList.Create;
//  FArcID.Sorted := True;
  FArcID.Duplicates := dupIgnore;
  FArcID.CommaText := Source.ArcID.CommaText;
  if FGdocID <> nil then
    FreeAndNil(FGdocID);
  FGdocID := TStringList.Create;
//  FGdocID.Sorted := True;
  FGdocID.Duplicates := dupIgnore;
  FGdocID.CommaText := Source.GdocID.CommaText;
  if FTimRef <> nil then
    FreeAndNil(FTimRef);
  FTimRef := TStringList.Create;
//  FTimRef.Sorted := True;
  FTimRef.Duplicates := dupIgnore;
  FTimRef.CommaText := Source.TimRef.CommaText;
  FCheckOut := Source.CheckOut;
  FInResub := Source.InResub;
  FInResubNegated := Source.InResubNegated;
  FState := Source.State;
  FNotState := Source.NotState;
  FSearchString := Source.SearchString;
  FSearchStringNegated := Source.SearchStringNegated;
  FSearchDate := Source.SearchDate;
  FSearchDateHigh := Source.SearchDateHigh;
  FSearchDateNegated := Source.SearchDateNegated;
  FSearchDateType := Source.SearchDateType;
  FFtxMatchAgainst := Source.FtxMatchAgainst;
  FCount := Source.Count;
  FFtxNaturalMode := Source.FtxNaturalMode;
  FFtxBooleanMode := Source.FtxBooleanMode;
  FFtxWithQueryExpansion := Source.FtxWithQueryExpansion;
  FResultType := Source.ResultType;
  FGroupBy := Source.GroupBY;
  FOrder := Source.Order;
  FDirection := Source.Direction;
  FAddFieldXML := Source.AddFieldXML;
  FStsFieldXML := Source.FStsFieldXML;
  FFtxFields := Source.FtxFields;
  FUserContext := Source.UserContext;
  FResubDate := Source.ResubDate;
  FResubDateHigh := Source.ResubDateHigh;
  FResubDateNegated := Source.ResubDateNegated;
  FResubAlarm := Source.ResubAlarm;
  FResubAlarmHigh := Source.ResubAlarmHigh;
  FResubAlarmNegated := Source.ResubAlarmNegated;
  FResubUserID := Source.ResubUserID;
  FResubUserIDNegated := Source.ResubUserIDNegated;
  FResubUserIDList := Source.ResubUserIDList;
  FResubControllerID := Source.ResubControllerID;
  FResubControllerIDNegated := Source.ResubControllerIDNegated;
  FResubState := Source.ResubState;
  FResubNotState := Source.ResubNotState;
  FResubStateNegated := Source.ResubStateNegated;
  FDocName := Source.DocName;
  FDocNameNegated := Source.DocNameNegated;
  FGroupList := TStringList.Create;
  FGroupList.Duplicates := dupIgnore;
  FGroupList.CommaText := Source.GroupList.CommaText;
  FUserList := TStringList.Create;
  FUserList.Duplicates := dupIgnore;
  FUserList.CommaText := Source.FUserList.CommaText;
  FUseModifiedDate := Source.UseModifiedDate;
  FFileType := Source.FileType;
  FFileTypeNegated := Source.FileTypeNegated;
  FSelectedArcID := Source.SelectedArcID;
  FVerify := Source.Verify;
  FGdocIDWithSymLink := Source.GdocIDWithSymLink;
  FAboUserid := Source.AboUserid;
  FUseSearch35 := Source.UseSearch35;
  FRightsMask := Source.RightsMask;
  FDontRefreshDataSet := Source.DontRefreshDataSet;
  FDontInheritate := Source.DontInheritate;
end;

procedure TbfaBaseSearch.SetSQLWhereOnly(Value: Boolean);
begin
  FSQLWhereOnly := Value;
  if Value Then
    FConstructive := Value;
end;

constructor TbfaRequest.Create;
begin
  inherited Create;
  FReqType := '';
  FAnno := nil;
  FData := nil;
  FFields := nil;
  FTasks := nil;
  FResubUserID := -1;
  FFileName := '';
  FDataLen := -1;
  FFileHash := '';
  FFileDate := cUnassignedDate;
  FPage := -1;
  FRevision := -1;
  FAnnoType := -1;
  FAnnoUserID := -1;
  FStatus := -1;
  FSock := nil;
  FParameter := '';
  FSymbolicArcID := -1;
  FTimID := -1;
  FParameterList := nil;
end;

destructor TbfaRequest.Destroy;
begin
  FreeAndNil(FAnno);
  FreeAndNil(FData);
  FreeAndNil(FFields);
  FreeAndNil(FTasks);
  FreeAndNil(FSock);
  FreeAndNil(ParameterList);
  inherited Destroy;
end;

//procedure TbfaStationery.Clone(Source: TbfaStationery);
//begin
  //FPages := Source.Pages;
//
  //if Length(Source.Layers) > 0 then
    //Layers[0] := Source.Layers[0];
  //if Length(Source.Layers) > 1 then
    //Layers[1] := Source.Layers[1];
  //if Length(Source.Layers) > 2 then
    //Layers[2] := Source.Layers[2];
  //SetLength(Layers, length(Source.Layers));
  //FStationeryStream.Clear;
  //Source.FStationeryStream.Position := 0;
  //FStationeryStream.CopyFrom(Source.FStationeryStream, Source.FStationeryStream.Size);
//
//end;
//
//constructor TbfaStationery.Create;
//begin
  //FStationeryStream := TMemoryStream.Create;
  //FPages := -1;
  //SetLength(Layers, 3);
  //Layers[0] := 0;
  //Layers[1] := 0;
  //Layers[2] := 0;
//end;
//
//destructor TbfaStationery.Destroy;
//begin
  //FStationeryStream.Clear;
  //FreeAndNil(FStationeryStream);
//
  //SetLength(Layers, 0);
  //Layers := nil;
//
  //inherited Destroy;
//end;
//
//procedure TbfaStationery.Load(var img: TbfaImageEnVect;
  //DocPageCount: Integer; FilePath: WideString = ''; Str: TMemoryStream = nil);
//
//var
  //Lyr: Integer;
//
  //procedure SetLayer;
  //begin
    //img.Layers[lyr].PosX := 0;
    //img.Layers[lyr].Posy := 0;
    //img.Layers[lyr].Width := img.Layers[0].Bitmap.Width;
    //img.Layers[lyr].Height := img.Layers[0].Bitmap.Height;
    //img.Layers[lyr].Operation := ielAND;
  //end;
//
//begin
  //try
    //FStationeryStream.Clear;
    //if (FilePath <> '') then
      //FStationeryStream.LoadFromFile(FilePath)
    //else if (Str <> nil) then
    //begin
      //Str.Position := 0;
      //FStationeryStream.CopyFrom(Str, Str.Size);
    //end;
    //FStationeryStream.Position := 0;
//
    //if ((img <> nil) and (FStationeryStream.Size > 0)) then
    //begin
//
      //img.CancelInteracts;
//
      //Lyr := img.LayersAdd;
      //img.IO.Params.TIFF_ImageIndex := 0;
      //img.LayersCurrent := Lyr;
      //FStationeryStream.Position := 0;
      //img.IO.LoadFromStream(FStationeryStream);
//
      //SetLength(Layers, img.IO.Params.ImageCount);
      //Layers[0] := Lyr;
//
      //FPages := img.IO.Params.ImageCount;
//
      //SetLayer;
//
      //if DocPageCount > 1 then
      //begin
        //if (FPages > 1) then
        //begin
          //Lyr := img.LayersAdd;
          //img.IO.Params.TIFF_ImageIndex := 1;
          //FStationeryStream.Position := 0;
          //img.IO.LoadFromStream(FStationeryStream);
//
          //Layers[1] := Lyr;
//
          //img.Layers[lyr].Visible := False;
          //SetLayer;
//
          //if (FPages > 2) then
          //begin
            //Lyr := img.LayersAdd;
            //img.IO.Params.TIFF_ImageIndex := 2;
            //FStationeryStream.Position := 0;
            //img.IO.LoadFromStream(FStationeryStream);
//
            //Layers[2] := Lyr;
//
            //img.Layers[lyr].Visible := False;
            //SetLayer;
          //end;
        //end;
      //end;
      //img.Update;
      //img.LayersCurrent := 0;
    //end;
  //except
//
  //end;
//end;

//constructor TbfaAnnotation.Create;
//begin
  //inherited Create;
  //FAnnoStream := TMemoryStream.Create;
  //FPage := 0;
  //FPriv := False;
  //FLayer := 0;
  //SetLength(ObjList, 0);
  //FHistory := TList<TbfaHistory>.Create;
  //Deleted := False;
//end;
//
//procedure TbfaAnnotation.ClearHistoryList;
//var
  //i: Integer;
//begin
  //try
    //for i := FHistory.Count - 1 downto 0 do
    //begin
      //FHistory[i].Free;
      //FHistory[i] := nil;
    //end;
    //FHistory.Clear;
  //except
//
  //end;
//end;
//
//procedure TbfaAnnotation.Clone(Source: TbfaAnnotation);
//var
  //i: Integer;
  //hst: TbfaHistory;
//begin
  //try
    //FPage := Source.Page;
    //FPriv := Source.Priv;
    //FLayer := Source.Layer;
    //FRights := Source.Rights;
    //FModified := Source.Modified;
    //FAnnoStream.Position := 0;
    //Source.AnnoStream.Position := 0;
    //FAnnoStream.CopyFrom(Source.AnnoStream, Source.AnnoStream.Size);
    //FAnnoStream.Position := 0;
    //SetLength(ObjList, Length(Source.ObjList));
    //for i := Low(Source.ObjList) to High(Source.ObjList) do
      //ObjList[i] := Source.ObjList[i];
    //ClearHistoryList;
    //for i := 0 to Source.History.Count - 1 do
    //begin
      //hst := TbfaHistory.Create;
      //hst.Clone(Source.History[i]);
      //FHistory.Add(hst);
    //end;
  //except
//
  //end;
//end;
//
//destructor TbfaAnnotation.Destroy;
//begin
  //FreeAndNil(FAnnoStream);
//
  //SetLength(ObjList, 0);
//
  //inherited Destroy;
//end;
//
//function TbfaAnnotation.GetDeleted: Boolean;
//begin
  //// aktuell nur als Haltepunkt da...
  //Result := FDeleted;
//end;
//
//procedure TbfaAnnotation.SetDeleted(Value: Boolean);
//begin
  //// aktuell nur als Haltepunkt da...
  //FDeleted := Value;
//end;

{ TbfaHistory }

procedure TbfaHistory.Clone(Source: TbfaHistory);
begin
  inherited Clone(Source);
  Taskid := Source.Taskid;
  Userid := Source.Userid;
  Gdocid := Source.Gdocid;
  Msg := Source.Msg;
  Value1 := Source.Value1;
  Value2 := Source.Value2;
  Value3 := Source.Value3;
  Date := Source.Date;
  UserName := Source.UserName;
end;

constructor TbfaHistory.Create;
begin
  Taskid := 0;
  Userid := 0;
  Gdocid := 0;
  Msg := '';
  Value1 := '';
  Value2 := '';
  Value3 := '';
  UserName := '';
end;

procedure TbfaInt64.Clone(Source: TBfaInt64);
begin
  FValue := Source.Value;
  FArcType := Source.ArcType;
end;

constructor TbfaInt64.Create;
begin
  FArcType := -1;
  FValue := -1;
end;

{ TbfaInt }

procedure TbfaInt.Clone(Source: TBfaInt);
begin
  FValue := Source.Value;
end;

constructor TbfaInt.Create;
begin
  FValue := -1;
end;

constructor TbfaInt.Create(AValue: Integer);
begin
  FValue := AValue;
end;

constructor TbfaInt64.Create(AValue: Int64);
begin
  FValue := AValue;
  FArcType := -1;
end;

{ TbfaUser }

procedure TbfaUser.Clone(Source: TbfaUser);
begin
  inherited Clone(Source);
  Rights := Source.Rights;
  DisplayName := Source.DisplayName;
//  Name := Source.Name;
//  Modified := Source.Modified;
  ID := Source.ID;
  Src_Userid := Source.Src_Userid;
  Email := Source.Email;
  Flags := Source.Flags;
  OldFlags := Source.OldFlags;
  Password := Source.Password;
  GroupList.CommaText := Source.GroupList.CommaText;
  EmployeesList.CommaText := Source.EmployeesList.CommaText;
  SuAnnoList.CommaText := Source.SuAnnoList.CommaText;
  HolidayList.CommaText := Source.HolidayList.CommaText;
  isAdmin := Source.isAdmin;
  DMSAdmin := Source.DMSAdmin;
  Role := Source.Role;
end;

constructor TbfaUser.Create;
begin
//  Modified := False;
  ID := -1;
  Rights := 0;
//  Name := '';
  DisplayName := '';
  Src_Userid := 0;
  Email := '';
  Flags := 0;
  OldFlags := 0;
  Password := '';
  GroupList := TStringList.Create;
  GroupList.Delimiter := ';';
  GroupList.StrictDelimiter := True;
  GroupList.Sorted := True;
  EmployeesList := TStringList.Create;
  EmployeesList.Delimiter := ';';
  EmployeesList.StrictDelimiter := True;
  EmployeesList.Sorted := True;
  SuAnnoList := TStringList.Create;
  SuAnnoList.Delimiter := ';';
  SuAnnoList.StrictDelimiter := True;
  SuAnnoList.Sorted := True;
  HolidayList := TStringList.Create;
  HolidayList.Delimiter := ';';
  HolidayList.StrictDelimiter := True;
  HolidayList.Sorted := True;
  isAdmin := False;
  DMSAdmin := False;
  Role := 0;
end;

destructor TbfaUser.Destroy;
begin
  FreeAndNil(GroupList);
  FreeAndNil(EmployeesList);
  FreeAndNil(SuAnnoList);
  FreeAndNil(HolidayList);
  inherited Destroy;
end;

{ TbfaGroup }

procedure TbfaGroup.Clone(Source: TbfaGroup);
var
  i: Integer;
begin
  inherited Clone(Source);
  ID := Source.ID;
  Name := Source.Name;
  for i := 0 to Source.Members.Count -1 do
   if Source.Members.Objects[i] <> nil then
     Members.AddObject(Source.Members[i], TbfaString.Create(TbfaString(Source.Members.Objects[i]).Value))
   else
     Members.Add(Source.Members[i]);
  Description := Source.Description;
  ParentID := Source.ParentID;
  Role := Source.Role;
  Flags := Source.Flags;
  ADPath := Source.ADPath;
  Email := Source.Email;
end;

constructor TbfaGroup.Create;
begin
  inherited Create;
//  ID := -1;
//  Name := '';
  Members := TStringList.Create;
  Members.Delimiter := ';';
  Members.StrictDelimiter := True;
//  Description := '';
  ParentID := -1;
  Role := 0;
  Flags := 0;
  ADPath := '';
  Email := '';
end;

destructor TbfaGroup.Destroy;
begin
  Members.Clear;
  FreeAndNil(Members);
  inherited;
end;


{ TbfaString }

procedure TbfaString.Clone(Source: TbfaString);
begin
  FValue := Source.Value;
end;

constructor TbfaString.Create;
begin
  FValue := '';
end;

constructor TbfaString.Create(AValue: WideString);
begin
  FValue := AValue;
end;

{ TbfaGroupEx }

procedure TbfaGroupEx.Clone(Source: TbfaGroup);
var
  i: Integer;
begin
  inherited Clone(Source);

  if Source is TbfaGroupEx then
  begin
    if Assigned(TbfaGroupEx(Source).NewMembers) then
      for i := 0 to TbfaGroupEx(Source).NewMembers.Count -1 do
        if TbfaGroupEx(Source).NewMembers.Objects[i] <> nil then
          NewMembers.AddObject(TbfaGroupEx(Source).NewMembers[i], TbfaString.Create(TbfaString(TbfaGroupEx(Source).NewMembers.Objects[i]).Value))
        else
          NewMembers.Add(TbfaGroupEx(Source).NewMembers[i]);

    MembersModified := TbfaGroupEx(Source).MembersModified;
  end;
end;

constructor TbfaGroupEx.Create;
begin
  inherited;
  NewMembers := TStringList.Create;
  MembersModified := False;
end;

destructor TbfaGroupEx.Destroy;
begin
  FreeAndNil(NewMembers);
  inherited;
end;


{ TbfaFieldRight }
procedure TbfaFieldRight.Clone(Source: TbfaFieldRight);
begin
  ArcID := Source.Arcid;
  ArcTempID := Source.ArcTempID;
  FieldID := Source.FieldID;
  Userid := Source.Userid;
  Groupid := Source.Groupid;
  Rights := Source.Rights;
  Fieldpos := Source.Fieldpos;
  Modified := Source.Modified;
  Deleted := Source.Deleted;
end;

constructor TbfaFieldRight.Create;
begin
  inherited;
  Arcid := -1;
  FieldID := -1;
  Userid := -1;
  Groupid := -1;
  Rights := -1;
  Fieldpos := -1;
  Modified := False;
  Deleted := False;
  ArcTempID := TGUID.Empty;
end;

{ TPluginObj }
{$IFDEF ENTERPRISE}
constructor TPluginObj.Create;
begin
  Name := '';
  FileName := '';
  Parameters := '';
  Command := '';
  Index := -1;
end;
{$ENDIF}

destructor TbfaFieldRight.Destroy;
begin
  inherited;
end;

function TbfaFieldRight.GetRights: Integer;
begin
  // aktuell nur als Haltepunkt da...
  Result := fRights;
end;

procedure TbfaFieldRight.SetRights(Value: Integer);
begin
  // aktuell nur als Haltepunkt da...
  fRights := Value;
end;

{ TbfaPredictionResult }

procedure TbfaPredictionResult.Clone(Source: TbfaPredictionResult);
begin
  Self.GUID := Source.GUID;
  Self.Page := Source.Page;
  Self.ClassID := Source.ClassID
end;

constructor TbfaPredictionResult.Create;
begin
  inherited;
  Self.GUID := '';
  Self.Page := -1;
  Self.ClassID := -1;
end;

destructor TbfaPredictionResult.Destroy;
begin
  inherited;
end;

{ TbfaColumnConfig }

constructor TbfaColumnConfig.Create;
begin
  Name := '';
  Fieldname := '';
  ColumnType := cltEdit;
  ValueType := vltString;
  SummaryType := smtNone;
  ReadOnly := False;
  DisplayFormat:= '';
  EditFormat := '';
  Expression := '';
  ListType := lstNone;
  ListItems := TStringList.Create;
  ListItems.StrictDelimiter := True;
  ListItems.Delimiter := ';';
  DefaultValue := '';
  ListAddfield := -1;
  ListQuery := '';
  ColWidth := -1;
end;

procedure TbfaColumnConfig.Clone(Source: TbfaColumnConfig);
begin
  Name := Source.Name;
  Fieldname := Source.Fieldname;
  ColumnType := Source.ColumnType;
  ValueType := Source.ValueType;
  SummaryType := Source.SummaryType;
  ReadOnly := Source.ReadOnly;
  DisplayFormat:= Source.DisplayFormat;
  EditFormat := Source.EditFormat;
  Expression := Source.Expression;
  DefaultValue := Source.DefaultValue;
  ListType := Source.ListType;
  ListItems.CommaText := Source.ListItems.CommaText;
  ListAddfield := Source.ListAddfield;
  ListQuery:= Source.ListQuery;
  ColWidth := Source.ColWidth;
end;

{ TbfaGridConfig }

procedure TbfaGridConfig.Clone(Source: TbfaGridConfig);
var
  i: Integer;
  Column: TbfaColumnConfig;
begin
  Displayname := Source.Displayname;
  ShowSummary := Source.ShowSummary;
  ShowHeader := Source.ShowHeader;
  ShowGroupbybox := Source.ShowGroupbybox;
  ShowIndicator := Source.ShowIndicator;
  UsePopups := Source.UsePopups;
  UseColumnNames := Source.UseColumnNames;
  ExportAddfields := Source.ExportAddfields;
  ExportUnicode := Source.ExportUnicode;
  ManualSorting := Source.ManualSorting;
  AlternateRows := Source.AlternateRows;
  MultiSelect := Source.MultiSelect;
  Stamp_Transparent := Source.Stamp_Transparent;
  Stamp_Size := Source.Stamp_Size;
  Stamp_Color := Source.Stamp_Color;
  SearchField := Source.SearchField;
  if Columns = nil then
    Columns := TList<TbfaColumnConfig>.Create;
  for i := 0 to Source.Columns.Count -1 do
  begin
    Column := TbfaColumnConfig.Create;
    Column.Clone(Source.Columns[i]);
    Columns.Add(Column);
  end;

end;

constructor TbfaGridConfig.Create;
begin
  Displayname := '';
  ShowSummary := False;
  ShowHeader := True;
  ShowGroupbybox := False;
  ShowIndicator := False;
  UsePopups := False;
  UseColumnNames := False;
  ExportAddfields := False;
  ExportUnicode := False;
  ManualSorting := False;
  Stamp_Transparent := False;
  Stamp_Size := 100;
  Stamp_Color := '#000000';
  SearchField := '';
  Columns := TList<TbfaColumnConfig>.Create;
end;


procedure TbfaGridConfig.Reset;
begin
  Displayname := '';
  ShowSummary := False;
  ShowHeader:= True;
  ShowGroupbybox := False;
  ShowIndicator := False;
  UsePopups := False;
  UseColumnNames := False;
  ExportAddfields := False;
  ExportUnicode := False;
  ManualSorting := False;
  Stamp_Transparent := False;
  Stamp_Size := 100;
  Stamp_Color := '#000000';
  SearchField := '';
  Columns.Clear;
end;

end.


