unit bfaBaseSocketConnection;

interface

uses IdTCPClient, IdSSLOpenSSL, Classes, XMLIntf, bfaCommonUtils, {bfaObjects36,}
     Windows, SysUtils, bfaBaseObjects, Generics.Collections, bfaLog;

type
  /// <Beschreibung>bitfarm SocketConnection Klasse</Beschreibung>
  /// <Description>bitfarm SocketConnection class</Description>
	TBaseSocketConnection = class(TObject)

  protected
    FReadSocket_CS: TRTLCriticalSection;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
  	FConnectionType: Byte;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
		FServerHost: string;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FServerPort: Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FServerReadTimeout: Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FDir: string;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FCertFile: string;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FConnected: Boolean;
    FReceivedData, FDataLength: Int64;  // SJ Abschlussprojekt
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    TCPComp: TIdTCPClient;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    IoHandlerComp: TIdSSLIOHandlerSocketOpenSSL;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    Bfr: TMemoryStream;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FSid : string;
    FAutoreconnect: Boolean;
    FUsed: Boolean;

		function CreateNoSSLConnection: boolean;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
	  function CreateSSLConnection: boolean;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure WriteStringsViaSocket(Strs: string; Size: integer);
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function StreamToString(aStream: TStream): string;
    procedure CheckAutoreconnect;

  published
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
  	property ConnectionType: Byte read FConnectionType write FConnectionType;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
  	property ServerHost: string read FServerHost;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property ServerPort: Integer read FServerPort;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property ServerReadTimeout: Integer read FServerReadTimeout write FServerReadTimeout;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property Dir: string read FDir;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property CertFile: string read FCertFile;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property Connected: Boolean read FConnected;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property Autoreconnect: Boolean read FAutoreconnect write FAutoreconnect;
    function ReceiveXML(AThread : TThread = nil): string;
    function ReceiveResponse(AThread : TThread = nil): TbfaRequest; virtual;
    procedure SendRequestToServer(req: TbfaRequest); virtual;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>

    function SendDataToServer(req: TbfaRequest; Stream: TStream; AThread : TThread = nil): TbfaRequest; virtual;
//    function ReceiveData(Doc: TbfaDocument; AThread : TThread = nil): TbfaRequest;
//    function ReceiveDataToFile(Doc: TbfaDocument; filename: string; AThread : TThread = nil): TbfaRequest;
    function ReceiveDataToFile(Doc: TbfaBaseDocument; filename: string; AThread : TThread = nil): TbfaRequest; virtual; // SJ


    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
//    function ReceiveDoc(Doc: TbfaDocument; AThread : TThread = nil): TbfaRequest;
  public
//    bitfarmLog : TbfaLog;
    Name: string;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
  	constructor Create(ConnectionType: byte; SHost: string; SPort: integer;
    								   CertificateFile: string; Autoreconnect: Boolean = False); overload;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    destructor Destroy;  override;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure CreateComponents;

    procedure DestroyComponents(ANotifyPeer: Boolean = False);
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function Connect: boolean;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function Disconnect: Boolean;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
		procedure SendXMLToServer(strXML: string);

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure ReceiveDataToFile_old(var xmlDocFromServer: IXMLDocument;
                                        var filename: string);

//    procedure GetPass(var Password: string);
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure WriteStreamViaSocket(Stream: TStream);
    procedure SetSid(value: string);
    property ReceivedData: Int64 read FReceivedData; // SJ Abschlussprojekt
    property DataLength: Int64 read FDataLength; // SJ Abschlussprojekt
  end;

implementation

uses
  gnugettext, bfaBaseParser, System.StrUtils;

constructor TBaseSocketConnection.Create(ConnectionType: byte; SHost: string;
																		 SPort: integer; CertificateFile: string; Autoreconnect: Boolean = False);
begin
	inherited Create;

  FConnectionType := ConnectionType;
  fServerHost := SHost;
  fServerPort := SPort;
  fCertFile := CertificateFile;
  FServerReadTimeout := 60000; // 60 sec Timeout
  fConnected := False;
  InitializeCriticalSection(FReadSocket_CS);
  FAutoreconnect := Autoreconnect;
  FUsed := False;
end;


destructor TBaseSocketConnection.Destroy;
begin
  DestroyComponents(True);

  DeleteCriticalSection(FReadSocket_CS);

  inherited Destroy;
end;

procedure TBaseSocketConnection.DestroyComponents(ANotifyPeer: Boolean = False);
begin
  TCPComp.Disconnect(ANotifyPeer);
  TCPComp.Free;

  if IoHandlerComp <> nil then
    IoHandlerComp.Free;

  Bfr.Free;
  FUsed := False;
end;

procedure TBaseSocketConnection.CreateComponents;
begin
  TCPComp := TIdTCPClient.Create(nil);

  if fConnectionType = sctpSSL then
    IoHandlerComp := TIdSSLIOHandlerSocketOpenSSL.Create;

  Bfr := TMemoryStream.Create;
end;

function TBaseSocketConnection.StreamToString(aStream: TStream): string;
var
  SS: TStringStream;
begin
  if aStream <> nil then
  begin
    SS := TStringStream.Create('');
    try
      aStream.Position := 0;
      SS.CopyFrom(aStream, aStream.Size);
      Result := SS.DataString;
    finally
      SS.Free;
    end;
  end
  else
    Result := '';
end;

procedure TBaseSocketConnection.CheckAutoreconnect;
begin
  if FAutoreconnect and FUsed then
  begin
    Self.DestroyComponents(False);
    Self.CreateComponents;
    Self.Connect;
  end;
end;

function TBaseSocketConnection.Connect: boolean;
begin
  try
    if Assigned(bitfarmLog) then
      bitfarmLog.Debug(llDebug, 'SocketConnection - Connect', Classname);

    if TCPComp = nil then
	    CreateComponents;
    if TCPComp.Connected then
      TCPComp.Disconnect;

    if not TCPComp.Connected then
    begin
      if fConnectionType = sctpNoSSL then
      begin
        if CreateNoSSLConnection then
          Result := True
        else
          Result := False;
      end else
      begin
        if CreateSSLConnection then
          Result := True
        else
          Result := False;
      end;
    end
    else
      Result := True;

    fConnected := Result;
  except
    Result := False;
    if Assigned(bitfarmLog) then
      bitfarmLog.Write(llNormal, ltException, 'SocketConnection - Connect', ClassName);
  end;
end;

function TBaseSocketConnection.Disconnect: Boolean;
begin
  if Assigned(bitfarmLog) then
    bitfarmLog.Debug(llDebug, 'SocketConnection - DisConnect', Classname);

  Result := True;
  if TCPComp = nil then
   	Exit;

  try
    if TCPComp.Connected then
      TCPComp.Disconnect;
  except
    Result := False;
  end;

  fConnected := not Result;
end;

function TBaseSocketConnection.CreateSSLConnection: boolean;
var
	libeay32, ssleay32: string;
begin
  if Assigned(bitfarmLog) then
    bitfarmLog.Debug(llDebug, 'SocketConnection - CreateSSLConnection', Classname);

	try
		Result := False;
    IoHandlerComp.Host := ServerHost;
    IoHandlerComp.Port := ServerPort;
    TCPComp.ReadTimeout := ServerReadTimeout;

//    IoHandlerComp.SSLOptions.Method := TIdSSLVersion(sslvSSLv23);
    IoHandlerComp.SSLOptions.Method := TIdSSLVersion(sslvTLSv1_2);
//    IoHandlerComp.SSLOptions.CertFile:= 'C:\Development\Delphi-3.6.2\Viewer36\DEBUG\Local2.crt';
//    IoHandlerComp.SSLOptions.KeyFile:= 'C:\Development\Delphi-3.6.2\Viewer36\DEBUG\Local2.crt';
//    IoHandlerComp.SSLOptions.CertFile:= 'C:\OpenSSL\bin\ca.cer';
//    IoHandlerComp.SSLOptions.KeyFile:= 'C:\OpenSSL\bin\client.key';
//    IoHandlerComp.OnGetPassword := GetPass;
    IoHandlerComp.PassThrough := False;
    //    IoHandlerComp.SSLOptions.CertFile:= CertFile;
    //    IoHandlerComp.SSLOptions.KeyFile:= CertFile;

    //libeay32 := BD.ExeDir + 'libeay32.dll';   //f.schneider v3.5.0.153 es reicht das die DLL im Exepfad liegen
    //ssleay32 := BD.ExeDir + 'ssleay32.dll';
    //LoadLibrary(pChar(libeay32));
    //LoadLibrary(pChar(ssleay32));

    IoHandlerComp.Open;

    if IoHandlerComp.Connected then
    begin
      Result := True;
      TCPComp.IOHandler := IoHandlerComp;
      TCPComp.IOHandler.MaxLineLength := MaxInt;
    end;

  except
    on E: Exception do
    begin
      Result := False;
      if Assigned(bitfarmLog) then
      begin
        bitfarmLog.Write(llDebug, ltInfo, 'ServerHost: ' + ServerHost + '; ServerPort: ' + IntToStr(ServerPort), ClassName);
        bitfarmLog.Write(llNormal, ltException, 'CreateSSLConnection: ' + E.Message, ClassName);
        if E is EIdOSSLCouldNotLoadSSLLibrary then
          bitfarmLog.Write(llNormal, ltException, 'SSL Bibiothek konnte nicht geladen werden', ClassName);
      end;
    end;
  end;
end;

function TBaseSocketConnection.CreateNoSSLConnection: boolean;
begin
  if Assigned(bitfarmLog) then
    bitfarmLog.Debug(llDebug, 'SocketConnection - CreateNoSSLConnection', Classname);

  try
    TCPComp.Host := ServerHost;
    TCPComp.Port := ServerPort;
    TCPComp.ReadTimeout := ServerReadTimeout;

    TCPComp.IOHandler := nil;

    TCPComp.Connect;

    TCPComp.IOHandler.MaxLineLength := MaxInt;
    Result := TCPComp.Connected;
  except
    on E: Exception do
    begin
      Result := False;
      if Assigned(bitfarmLog) then
      begin
        bitfarmLog.Write(llDebug, ltInfo, 'ServerHost: ' + ServerHost + '; ServerPort: ' + IntToStr(ServerPort), ClassName);
        bitfarmLog.Write(llDebug, ltInfo, 'Error: ' + E.Message, ClassName);
        bitfarmLog.Write(llNormal, ltException, 'CreateNoSSLConnection', ClassName);
      end;
    end;
  end;
end;

procedure TBaseSocketConnection.SendXMLToServer(strXML: string);
begin
  if Assigned(bitfarmLog) then
    bitfarmLog.Debug(llDebug, 'SocketConnection - SendXMLToServer', Classname);
	try
    CheckAutoreconnect;

    WriteStringsViaSocket(strXML, Length(strXML));
    if Assigned(bitfarmLog) then
      bitfarmLog.Write(llDebug, ltInfo, 'SendXMLToServer: ' + strXML, ClassName);
  except
    if Assigned(bitfarmLog) then
      bitfarmLog.Write(llNormal, ltException, 'SendXMLToServer', ClassName);
  end;
end;


procedure TBaseSocketConnection.SetSid(value: string);
begin
  FSid := Value;
end;

procedure TBaseSocketConnection.SendRequestToServer(req: TbfaRequest);
var
  xmlstr: string;
  lst: TList<TbfaBaseObject>;
begin
  lst := TList<TbfaBaseObject>.Create;
  try
    CheckAutoreconnect;

    req.Sessionid := FSid;
    lst.Add(req);
    xmlstr := ObjectToXML(lst);

    try
      if Assigned(bitfarmLog) and (req.reqType <> 'reqAuthenticate') and
         (req.ReqType <> 'reqADUser') and (req.ReqType <> 'reqADTree')
      then
        bitfarmLog.Write(llDebug, ltInfo, 'SendRequestToServer: ' + xmlstr, ClassName);
      WriteStringsViaSocket(xmlstr, Length(xmlstr));
    except
      if Assigned(bitfarmLog) then
        bitfarmLog.Write(llNormal, ltException, 'SendRequestToServer', ClassName);
    end;
  finally
    FreeAndNil(lst);
  end;
end;

function TBaseSocketConnection.SendDataToServer(req: TbfaRequest; Stream: TStream;
  AThread : TThread = nil): TbfaRequest;
var
  xml: string;
  lst: TList<TbfaBaseObject>;
  i, idx: Integer;
begin
  Result := nil;
  lst := TList<TbfaBaseObject>.Create;
  idx := 0;
  try
    CheckAutoreconnect;

    req.Sessionid := FSid;
    lst.Add(req);
    xml := ObjectToXML(lst);
    try
      if Assigned(bitfarmLog) then
        bitfarmLog.Write(llDebug, ltInfo, 'SendDataToServer: ' + xml, ClassName);

      WriteStringsViaSocket(xml, Length(xml));

      xml := ReceiveXML(AThread);
      if xml <> '' then
      begin
        lst := XMLStringToObject(xml);
        if TbfaRequest(lst[0]).ReqType = 'respSendFile' then
        begin
          WriteStreamViaSocket(Stream);
          xml := ReceiveXML(AThread);
          lst.Clear;
          lst := XMLStringToObject(xml);
        end;
        if lst.Count = 1 then
          Result := TbfaRequest(lst[0])
        else begin
          for i := 0 to lst.Count -1 do
            if lst[i] is TbfaRequest then
              idx := i;
          Result := TbfaRequest(lst[idx]);
          Result.Data := TList<TbfaBaseObject>.Create;
          for i := 0 to lst.Count -1 do
          begin
            if i = idx then
              Continue;
            Result.Data.Add(lst[i])
          end;
        end;

        if Assigned(bitfarmLog) then
          bitfarmLog.Write(llDebug, ltInfo, 'SendDataToServer: ' + xml, ClassName);
      end;
    except
      //
    end;
  finally
    FreeAndNil(lst);
  end;
end;

procedure TBaseSocketConnection.WriteStringsViaSocket(Strs: string; Size: integer);
var
	s: string;
begin
  if Assigned(bitfarmLog) then
  begin
    bitfarmLog.Debug(llDebug, 'SocketConnection - WriteStringsViaSocket', Classname);
    bitfarmLog.Debug(llDebug, 'TCPComp.Socket.Opened: ' + BoolToStr(TCPComp.Socket.Opened, True), Classname);
    bitfarmLog.Debug(llDebug, 'TCPComp.Socket.ClosedGracefully: ' + BoolToStr(TCPComp.Socket.ClosedGracefully, True), Classname);
  end;

	s := Strs;
  s := StringReplace(s, chr(13), '', [rfReplaceAll]);
  s := StringReplace(s, chr(10), '', [rfReplaceAll]);
  Size := Length(s);

  try
    if fConnectionType = sctpSSL	then    // if SSL Connection then use IoHandler
    begin
      TCPComp.Socket.SendBufferSize := Size;
      TCPComp.Socket.Write(s);
    end
    else if fConnectionType = sctpNoSSL then     // if NoSSL Connection  don't use IoHandler
    begin
        TCPComp.Socket.Write(s);
    end;
  except
    begin
      if Assigned(bitfarmLog) then
        bitfarmLog.Exception(llDebug, 'SocketConnection - WriteStringsViaSocket, try to reconnect...', Classname);

      Self.DestroyComponents;
      Self.CreateComponents;
      Self.Connect;

      if fConnectionType = sctpSSL	then    // if SSL Connection then use IoHandler
      begin
        TCPComp.Socket.SendBufferSize := Size;
        TCPComp.Socket.Write(s);
      end
      else if fConnectionType = sctpNoSSL then     // if NoSSL Connection  don't use IoHandler
      begin
          TCPComp.Socket.Write(s);
      end;
    end;
  end;
  FUsed := True;
end;

procedure TBaseSocketConnection.WriteStreamViaSocket(Stream: TStream);
begin
  if Assigned(bitfarmLog) then
    bitfarmLog.Debug(llDebug, 'SocketConnection - WriteStreamViaSocket', Classname);
	if fConnectionType = sctpSSL	then    // if SSL Connection then use IoHandler
  begin
    TCPComp.Socket.SendBufferSize := 65536; //25.05.2023
    TCPComp.Socket.Write(Stream);
  end
  else if fConnectionType = sctpNoSSL then     // if NoSSL Connection  don't use IoHandler
  begin
    TCPComp.Socket.SendBufferSize := 65536; //25.05.2023
    TCPComp.Socket.Write(Stream);
  end;
  FUsed := True;
end;

//function TSocketConnection.ReceiveData(Doc: TbfaDocument; AThread : TThread = nil): TbfaRequest;
//var
//  i, j: integer;
//  s, xml:string;
//  req : TbfaRequest;
//  dataLen: Int64;
//  lst : TList<TbfaBaseObject>;
//  UserAction: PUserAction;
//  TmpStream : TMemoryStream;
//begin
//   bitfarmLog.Debug(llDebug, 'SocketConnection - ReceiveData', Classname);
//   Result := nil;
//   dataLen:= 0;
//   Bfr.Clear;
//   if Doc.DocStream <> nil then
//     Doc.DocStream.Clear;
//
//   try
//      xml := ReceiveXML(AThread);
//   finally
//      if xml <> '' then
//      begin
//        try
//          lst := XMLStringToObject(xml);
//          if lst <> nil then
//          begin
//            if lst.Count > 0 then
//            begin
//              if lst.Count > 1 then
//              begin
//                req := TbfaRequest(lst[1]);
//                Result := req;
//                if Doc = nil then
//                  Result := nil
//                else
//                  Doc.Clone(TbfaDocument(lst[0]), False, True, False);
//                TbfaDocument(lst[0]).Free;
//                lst[0] := nil; //23.01.2018 wegen CrashLog
//                dataLen := req.DataLen;
//              end else
//              begin
//                Req := TbfaRequest(lst[0]);
//                Result := req;
//                dataLen := req.DataLen;
//              end;
//              lst.Clear;
//            end
//            else
//              Result := nil;
//            FreeAndNil(lst);
//          end else begin
//            Result := nil;
//            bitfarmLog.Error(llDebug, 'Fehler beim Parsen der XML (1)', Classname);
//          end;
//        except
//          Result := nil;
//          bitfarmLog.Error(llDebug, 'Fehler beim Parsen der XML (2)', Classname);
//        end;
//      end else
//      begin
//        bitfarmLog.Error(llDebug, 'SocketConnection - ReceiveData keine Antwort vom Server erhalten', Classname);
//        BfShowMessage(_('Es hat eine Fehler bei der Verbindung mit dem Server gegeben. Bitte melden Sie sich erneut an.'), AThread);
//
////        UserAction := bitfarmApp.CreateUserAction;
////        UserAction^.ActionName := 'actRelogin';
////        bitfarmApp.MainThread.Add(UserAction);
//      end;
//   end;
//
//
//  if Result = nil then
//  begin
//    BfShowMessage(_('Es hat eine Fehler bei der Verbindung mit dem Server gegeben.'), AThread);
//    Exit;
//  end;
//
//  if (Result.ReqType = 'respDenied') then
//  begin
//    BfShowMessage(_('Keine Berechtigung auf das Dokument.') + #13#10 + Result.DataString, AThread);
//    Exit;
//  end;
//
//  if (Result.ReqType = 'respDocError') or (Result.ReqType = 'respError')
//      or ((Result.ReqType <> 'respOK') and ((Result.ReqType <> 'respDoc'))) then
//  begin
//    if not bitfarmApp.DoDropping then
//    begin
//      BfShowMessage(_('Das Dokument konnte nicht geladen werden.') + #13#10 + Result.DataString, AThread);
//      if (Result.ReqType = 'respAuthError') then
//        BfShowMessage(_('Bitte melden Sie sich neu an (Datei - Neu anmelden).'), AThread);
//    end;
//
//    Exit;
//  end;
//
//  EnterCriticalSection(FReadSocket_CS);
//  TmpStream := TMemoryStream.Create; //02.08.2017 Ticket 11319
//  try
//    try
////02.08.2017 Ticket 11319      TCPComp.Socket.ReadStream(Doc.DocStream, dataLen, false);
////02.08.2017 Ticket 11319 --> Erst in einem TmpStream laden und wenn das DocObjekt noch vorhanden ist, dann ins DocObjekt laden
//      TmpStream.Clear;
//      TCPComp.Socket.ReadStream(TmpStream, dataLen, false);
//      TmpStream.Position := 0;
//
//      if Doc <> nil then
//      begin
//        Doc.DocStream.Clear;
//        Doc.DocStream.CopyFrom(TmpStream, TmpStream.Size);
//      end;
////02.08.2017 Ticket 11319 <--
//    except
//      bitfarmLog.Error(llDebug, 'SocketConnection - Fehler beim Streamlesen', Classname);
//      BfShowMessage(_('Das Dokument konnte nicht geladen werden.'), AThread);
//    end;
//  finally
//    FreeAndNil(TmpStream);
//    LeaveCriticalSection(FReadSocket_CS);
//  end;
//end;

//function TSocketConnection.ReceiveDataToFile(Doc: TbfaDocument; filename: string; AThread : TThread = nil): TbfaRequest;
//var
//  i, j, blocksize: integer;
//  dataLen, recv: Int64;
//  s, xml:string;
//  req : TbfaRequest;
//  lst : TList<TbfaBaseObject>;
//  UserAction: PUserAction;
//  myFile : TFileStream;
//  buffer : PByte;
//begin
//   bitfarmLog.Debug(llDebug, 'SocketConnection - ReceiveData', Classname);
//   Result := nil;
//   blocksize := 524288;
//   dataLen:= 0;
//   recv := 0;
//   GetMem(buffer, blocksize);
//   Bfr.Clear;
//
//   try
//      xml := ReceiveXML(AThread);
//   finally
//      if xml <> '' then
//      begin
//        try
//          lst := XMLStringToObject(xml);
//          if lst <> nil then
//          begin
//            if lst.Count > 0 then
//            begin
//              if lst.Count > 1 then
//              begin
//                req := TbfaRequest(lst[1]);
//                Result := req;
//                if Doc = nil then
//                  Result := nil
//                else
//                  Doc.Clone(TbfaDocument(lst[0]));
//                TbfaDocument(lst[0]).Free;
//                lst[0] := nil; //23.01.2018 wegen CrashLog
//                dataLen := req.DataLen;
//              end else
//              begin
//                Req := TbfaRequest(lst[0]);
//                Result := req;
//                dataLen := req.DataLen;
//              end;
//              lst.Clear;
//            end
//            else
//              Result := nil;
//            FreeAndNil(lst);
//          end else begin
//            Result := nil;
//            bitfarmLog.Error(llDebug, 'Fehler beim Parsen der XML (1)', Classname);
//          end;
//        except
//          Result := nil;
//          bitfarmLog.Error(llDebug, 'Fehler beim Parsen der XML (2)', Classname);
//        end;
//      end else
//      begin
//        bitfarmLog.Error(llDebug, 'SocketConnection - ReceiveData keine Antwort vom Server erhalten', Classname);
//        BfShowMessage(_('Es hat eine Fehler bei der Verbindung mit dem Server gegeben. Bitte melden Sie sich erneut an.'), AThread);
//
////        UserAction := bitfarmApp.CreateUserAction;
////        UserAction^.ActionName := 'actRelogin';
////        bitfarmApp.MainThread.Add(UserAction);
//      end;
//   end;
//
//
//  if Result = nil then
//  begin
//    BfShowMessage(_('Es hat eine Fehler bei der Verbindung mit dem Server gegeben.'), AThread);
//    Exit;
//  end;
//
//  if (Result.ReqType = 'respDenied') then
//  begin
//    BfShowMessage(_('Keine Berechtigung auf die Datei.') + #13#10 + Result.DataString, AThread);
//    Exit;
//  end;
//
//  if (Result.ReqType = 'respDocError') or (Result.ReqType = 'respError')
//      or ((Result.ReqType <> 'respOK') and ((Result.ReqType <> 'respDoc'))) then
//  begin
//    if not bitfarmApp.DoDropping then
//    begin
//      BfShowMessage(_('Das Dokument konnte nicht geladen werden.') + #13#10 + Result.DataString, AThread);
//      if (Result.ReqType = 'respAuthError') then
//        BfShowMessage(_('Bitte melden Sie sich neu an (Datei - Neu anmelden).'), AThread);
//    end;
//
//    Exit;
//  end;
//  try
//    myFile :=  TFileStream.Create(filename, fmOpenWrite or fmCreate);
//  except
////    on e:Exception do
////        outputdebugstring(PChar('Fehler bei ReceiveDataToFile: (TFileStream.Create)' + e.Message));
//  end;
//
//
//  Bfr.Position:= 0;
//  //  TCPComp.Socket.ReadStream(Bfr,dataLen-FStreamTIF.Size,false);
//  try
//    try
//       while recv < dataLen do
//       begin
//          if dataLen - recv < blocksize then
//            blocksize := dataLen - recv;
//
//          TCPComp.Socket.ReadStream(Bfr,blocksize,False);
//          Bfr.Position := 0;
//
//          Bfr.ReadBuffer(buffer^,blocksize);
//          myFile.WriteBuffer(buffer^ , blocksize);
//
//          recv := recv + blocksize;
//          Bfr.Clear;
//       end;
//    except
////      on e: Exception do
////        OutputDebugString(PChar('exception:'+ e.Message));
//    end;
//  finally
//    FreeMem(buffer);
//   myFile.Free;
//  end;
//end;


//function TSocketConnection.ReceiveDoc(Doc: TbfaDocument; AThread : TThread = nil): TbfaRequest;
//var
//  i, j: integer;
//  s, xml:string;
//  req : TbfaRequest;
//  dataLen: Int64;
//  lst : TList<TbfaBaseObject>;
//  UserAction: PUserAction;
//begin
//   bitfarmLog.Debug(llDebug, 'SocketConnection - ReceiveData', Classname);
//   Result := nil;
//
//   try
//      xml := ReceiveXML(AThread);
//   finally
//      if xml <> '' then
//      begin
//   bitfarmLog.Debug(llDebug, 'BEFORE - lst := XMLStringToObject(xml);', Classname);
//        lst := XMLStringToObject(xml);
//   bitfarmLog.Debug(llDebug, 'AFTER - lst := XMLStringToObject(xml);', Classname);
//
//        if lst.Count > 1 then
//        begin
//          Result := TbfaRequest(lst[1]);
//          Doc.Clone(TbfaDocument(lst[0]));
//        end else
//        begin
//          Result := TbfaRequest(lst[0]);
//        end;
//      end else
//      begin
//        bitfarmLog.Error(llDebug, 'SocketConnection - ReceiveDoc keine Antwort vom Server erhalten', Classname);
//        BfShowMessage(_('Es hat eine Fehler bei der Verbindung mit dem Server gegeben. Bitte melden Sie sich erneut an.'), AThread);
////        UserAction := bitfarmApp.CreateUserAction;
////        UserAction^.ActionName := 'actRelogin';
////        bitfarmApp.MainThread.Add(UserAction);
//      end;
//   end;
//end;


function TBaseSocketConnection.ReceiveDataToFile(Doc: TbfaBaseDocument; filename: string; AThread : TThread = nil): TbfaRequest;
var
  i, j, blocksize: integer;
  dataLen, recv: Int64;
  s, xml: string;
  req: TbfaRequest;
  lst: TList<TbfaBaseObject>;
  myFile: TFileStream;
  buffer: PByte;
begin
  bitfarmLog.Debug(llDebug, 'SocketConnection - ReceiveDataToFile', Classname);
  Result := nil;
  blocksize := 524288;
  dataLen:= 0;
  recv := 0;
  GetMem(buffer, blocksize);
  Bfr.Clear;

  try
    xml := ReceiveXML(AThread);
  finally
    if xml <> '' then
    begin
      try
        lst := XMLStringToObject(xml);
        if lst <> nil then
        begin
          if lst.Count > 0 then
          begin
            if lst.Count > 1 then
            begin
              req := TbfaRequest(lst[1]);
              Result := req;
              if Doc = nil then
                Result := nil
              else
                Doc.Clone(lst[0]);
              lst[0].Free;
              lst[0] := nil; //23.01.2018 wegen CrashLog
              dataLen := req.DataLen;
              FDataLength := req.DataLen; // SJ Abschlussprojekt
            end else
            begin
              Req := TbfaRequest(lst[0]);
              Result := req;
              dataLen := req.DataLen;
              FDataLength := req.DataLen; // SJ Abschlussprojekt
            end;
            lst.Clear;
          end
          else
            Result := nil;
          FreeAndNil(lst);
        end else
        begin
          Result := nil;
          bitfarmLog.Error(llDebug, 'Fehler beim Parsen der XML (1)', Classname);
        end;
      except
        Result := nil;
        bitfarmLog.Error(llDebug, 'Fehler beim Parsen der XML (2)', Classname);
      end;
    end else
    begin
      bitfarmLog.Error(llDebug, 'SocketConnection - ReceiveDataToFile keine Antwort vom Server erhalten', Classname);
    end;
   end;


  if Result = nil then
  begin
    bitfarmLog.Error(llDebug, 'Es hat einen Fehler bei der Verbindung mit dem Server gegeben.', Classname);
    Exit;
  end;

  if (Result.ReqType = 'respDenied') then
  begin
    bitfarmLog.Error(llDebug, 'Keine Berechtigung auf die Datei: ' + Result.DataString, Classname);
    Exit;
  end;

  try
    myFile :=  TFileStream.Create(filename, fmOpenWrite or fmCreate);
  except
//    on e:Exception do
//        outputdebugstring(PChar('Fehler bei ReceiveDataToFile: (TFileStream.Create)' + e.Message));
  end;


  Bfr.Position:= 0;
  //  TCPComp.Socket.ReadStream(Bfr,dataLen-FStreamTIF.Size,false);
  try
    try
      while recv < dataLen do
      begin
        if dataLen - recv < blocksize then
          blocksize := dataLen - recv;

        TCPComp.Socket.ReadStream(Bfr, blocksize, False);
        Bfr.Position := 0;

        Bfr.ReadBuffer(buffer^, blocksize);
        myFile.WriteBuffer(buffer^, blocksize);

        recv := recv + blocksize;
        FReceivedData := recv;
        Bfr.Clear;
      end;
    except
//      on e: Exception do
//        OutputDebugString(PChar('exception:'+ e.Message));
    end;
  finally
    FreeMem(buffer);
   myFile.Free;
  end;
end;

procedure TBaseSocketConnection.ReceiveDataToFile_old(var xmlDocFromServer: IXMLDocument;
                                        var filename: string);
var
  i, j, blocksize: integer;
  dataLen, recv: Int64;
  s, xml:string;
  myFile : TFileStream;
  buffer : PByte;
begin
  if Assigned(bitfarmLog) then
    bitfarmLog.Debug(llDebug, 'SocketConnection - ReceiveDataToFile_old', Classname);

  blocksize := 524288;
  datalen:= 0;
  recv := 0;
  Bfr.Clear;
  GetMem(buffer, blocksize);
  xml := ReceiveXML;

  if (Pos('respAuthError', xml) > 0) or (Pos('respDocError', xml) > 0) then
  begin
    FreeMem(buffer);
    Exit;
  end;

  if (Pos('<dataLen>', xml) = 0) or (xml = '') then
	  Exit;

  dataLen := StrToInt64(Copy(xml,Pos('<dataLen>',xml) + 9,Pos('</dataLen>',xml)-Pos('<dataLen>',xml) -9));

  try
    xmlDocFromServer.XML.Clear;
    xmlDocFromServer.XML.Add(xml);
    xmlDocFromServer.Active:= true;
  except
//       on e:Exception do
//       	outputdebugstring(PChar('Fehler bei ReceiveDataToFile (xmlDocFromServer.Active:= true):' + e.Message));
  end;

  try
	  myFile :=  TFileStream.Create(filename, fmOpenWrite or fmCreate);
  except
//   	on e:Exception do
//       	outputdebugstring(PChar('Fehler bei ReceiveDataToFile: (TFileStream.Create)' + e.Message));
  end;


  Bfr.Position:= 0;
  //  TCPComp.Socket.ReadStream(Bfr,dataLen-FStreamTIF.Size,false);
  try
    try
      while recv < dataLen do
      begin
        if dataLen - recv < blocksize then
         	blocksize := dataLen - recv;

        TCPComp.Socket.ReadStream(Bfr, blocksize, False);
        Bfr.Position := 0;

        Bfr.ReadBuffer(buffer^, blocksize);
        myFile.WriteBuffer(buffer^, blocksize);

        recv := recv + blocksize;
        Bfr.Clear;
      end;
		except
//      	on e: Exception do
//         	OutputDebugString(PChar('exception:'+ e.Message));
    end;
	finally
    FreeMem(buffer);
	  myFile.Free;
  end;
end;

function TBaseSocketConnection.ReceiveXML(AThread: TThread = nil): string;
var
  xml : String;
  xmllog: string;
begin
  if Assigned(bitfarmLog) then
    bitfarmLog.Debug(llDebug, 'SocketConnection - ReceiveXML', Classname);
  xml := '';
  xmllog := '';
  try
   	try
      xml := TCPComp.Socket.ReadLn('</bfaReq>');
      if Assigned(bitfarmLog) then
      begin
        xmllog := xml;
        if (Pos('<fulltxt', xmllog) > 0) and (Pos('</fulltxt>', xmllog) > 0) then
          xmllog := LeftStr(xmllog, Pos('<fulltxt', xmllog)) + RightStr(xmllog, Length(xmllog) - Pos('</fulltxt>', xmllog) -10);

        if (Pos('<doctags', xmllog) > 0) and (Pos('</doctags>', xmllog) > 0) then
          xmllog := LeftStr(xmllog, Pos('<doctags', xmllog)) + RightStr(xmllog, Length(xmllog) - Pos('</doctags>', xmllog) -10);

        bitfarmLog.Write(llDebug, ltInfo, 'ReceiveXML: ' + xmllog, ClassName);
      end;

    except
      if Assigned(bitfarmLog) then
        bitfarmLog.Exception(llDebug,_('Timeout beim Warten auf eine Serverantwort'),ClassName);
//      BfShowMessage(_('Timeout beim Warten auf eine Serverantwort'), AThread);
      xml := ''
    end;
  finally
    if xml = '' then
    begin
      Result := '';
      if Assigned(bitfarmLog) then
        bitfarmLog.Error(llDebug, 'SocketConnection - ReceiveXML leeren String erhalten', Classname);
    end
    else
      Result := xml + '</bfaReq>';
	end;
end;

function TBaseSocketConnection.ReceiveResponse(AThread : TThread = nil): TbfaRequest;
var
  xml : String;
  lst : TList<TbfaBaseObject>;
  i : Integer;
begin
  if Assigned(bitfarmLog) then
    bitfarmLog.Debug(llDebug, 'SocketConnection - ReceiveResponse', Classname);
  Result := nil;
  try
    xml := ReceiveXML(AThread);
  finally
    if xml = '' then
      Result := nil
    else
    begin
      lst := XMLStringToObject(xml);
      try
        for i := 0 to lst.Count - 1 do
        begin
          if lst[i] is TbfaRequest then
          begin
            Result := TbfaRequest(lst[i]);
            Result.Data := TList<TbfaBaseObject>.Create;
          end;

        end;

        if Result <> nil then
        begin
          for i := 0 to lst.Count - 1 do
          begin
//            if lst[i] is TbfaAddfield then
//              Result.Data.Add(TbfaAddfield(lst[i]));
//            if lst[i] is TbfaStateField then
//              Result.Data.Add(TbfaStateField(lst[i]));
//            if lst[i] is TbfaWorkflow then
//              Result.Data.Add(TbfaWorkflow(lst[i]));
//            if lst[i] is TbfaSearch then
//              Result.Data.Add(TbfaSearch(lst[i]));
            if lst[i] is TbfaBaseField then
              Result.Data.Add(TbfaBaseField(lst[i]));
            if lst[i] is TbfaBaseField then
              Result.Data.Add(TbfaBaseField(lst[i]));
            if lst[i] is TbfaBaseObject then
              Result.Data.Add(TbfaBaseObject(lst[i]));
            if lst[i] is TbfaBaseSearch then
              Result.Data.Add(TbfaBaseSearch(lst[i]));
          end;
        end;
      finally
        FreeAndNil(lst);
      end;
    end;
	end;
end;

end.
