unit bfaCommonDialogs;

interface

uses
  Windows, Messages, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, SpTBXControls, SpTBXEditors, SpTBXItem, gnugettext, bfaOptions;

type
  Tfrm_Dialog = class(TForm)
    pnl1: TSpTBXPanel;
    edt_Dialog: TSpTBXEdit;
    lbl_Dialog: TSpTBXLabel;
    cbx_Dialog: TSpTBXComboBox;
    btn_Dialog: TSpTBXButton;
    btnCancel: TSpTBXButton;

    procedure edt_DialogChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure edt_DialogKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormCreate(Sender: TObject);
    procedure edt_DialogKeyPress(Sender: TObject; var Key: Char);
    procedure cbx_DialogKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure cbx_DialogKeyPress(Sender: TObject; var Key: Char);

  private
    { Private-Deklarationen }
    isCombobox: Boolean;
    not_allowed: string;
  public
    { Public-Deklarationen }
  end;
function bfaInputQuery(const ACaption: string; const APrompt: string; var Value: string; not_allowed_chars: string = ''): boolean; overload;
function bfaInputQuery(const ACaption: string; const APrompt: string; var Value: string; Items: TStringList; fixed: Boolean = False; not_allowed_chars: string = ''): boolean; overload;

function BfMessageDialogWithCheckBox(Const Title, Msg: WideString; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; Captions: array of WideString;
  ShowCheckBox: Boolean; ViewerOptions: TbfaViewerOptions;
  DlgSectionID: string; AcceptedButton: Integer = -1): Integer;
function S(const S: string):string;
function ParseMsg(const msg: string): string;
function ParseMsgCRLF(const msg: string): string;
procedure BfShowMessage(const Msg: string; AThread : TThread = nil);
function BfMessageDlg(Msg: WideString; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; Default_NO: boolean = False; scustom: WideString = ''): Integer;
function BfMessageBox(Msg, Caption: string; Flags: Word; Handle: THandle = 0): Integer;
function BfaGetPath(sIniFile: string; sSection, sKey, sTitel: WideString;
  Updater: Boolean = false): WideString;

var
  frm_Dialog: Tfrm_Dialog;

implementation

uses
  bfaCommonUtils, bfaCommonStyles, System.IniFiles, System.SysUtils;
{$R *.dfm}

function BfMessageDialogWithCheckBox(Const Title, Msg: WideString; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; Captions: array of WideString;
  ShowCheckBox: Boolean; ViewerOptions: TbfaViewerOptions;
  DlgSectionID: string; AcceptedButton: Integer = -1): Integer;
var
  AMsgDialog: TForm;
  ACheckBox: TCheckBox;
  dlgButton: TButton;
  i, CaptionIndex: Integer;
  ini: TIniFile;
  ShowDlg, found, CheckBoxValue: Boolean;
  DlgValue: TModalResult;
  syes, sno, scancel, swarning, sconfirm, sinformation, serror, sMsgCaption: WideString;
  SavingButton: String;
begin
  syes := _('Ja');
  sno := _('Nein');
  scancel := _('Abbrechen');
//14.02.2023 bfacrm:ticket__show__33967 swarning := _('Warnung');
  swarning := dgettext('Shared36', 'Warnung');
//14.02.2023 bfacrm:ticket__show__33967  sconfirm := _('Besttigung');
  sconfirm := dgettext('Shared36', 'Besttigung');
  sinformation := _('Informationen');
  serror := _('Fehler');
//14.02.2023 bfacrm:ticket__show__33967 sMsgCaption := _('Diese Meldung nicht mehr anzeigen');
  sMsgCaption := dgettext('Shared36', 'Diese Meldung nicht mehr anzeigen');

  ShowDlg := True;
  SavingButton := '';
  try
    for i := Low(ViewerOptions.CheckBoxDialogs) to High(ViewerOptions.CheckBoxDialogs) do
    begin
      if ViewerOptions.CheckBoxDialogs[i].DialogName = DlgSectionID then
      begin
        ShowDlg := ViewerOptions.CheckBoxDialogs[i].ShowDialog;
        DlgValue := ViewerOptions.CheckBoxDialogs[i].DialogResultType;
        //bei manche Dialogs darf man nur ein bestimmte Knopf erlauben z.B bei FormClose!
        if not ShowDlg and (AcceptedButton > -1) then
        begin
          ShowDlg := DlgValue <> AcceptedButton;
          if ShowDlg then
            DlgValue := mrNone;
        end;
        Break;
      end;
    end;
  except
    ShowDlg := True;
  end;

  if not ShowDlg then
  begin
    Result := DlgValue;
    Exit;
  end
  else
  begin
    AMsgDialog := CreateMessageDialog(S(Msg), DlgType, Buttons);
    ACheckBox := TCheckBox.Create(AMsgDialog);
    with AMsgDialog do
    try
      if Title <> '' then
        Caption := Title
      else
      begin
        if DlgType = mtConfirmation then
          Caption := sconfirm
        else if DlgType = mtWarning then
          Caption := swarning
        else if DlgType = mtInformation then
          Caption := sinformation
        else if DlgType = mtError then
          Caption := serror;
      end;
      CaptionIndex := 0;
      for i := 0 to ComponentCount - 1 do
      begin
        if Components[i] is TButton then
        begin
          dlgButton := TButton(Components[i]);
          if CaptionIndex > High(Captions) then
            Break;
          dlgButton.Font.Color := stlControl.TextColor;
          dlgButton.Caption := Captions[CaptionIndex];
          if (AcceptedButton > -1) and (dlgButton.ModalResult = AcceptedButton) then
             SavingButton := Captions[CaptionIndex];
          Inc(CaptionIndex);
        end;
      end;
      RePaint;
      if ShowCheckBox then
      begin
        with ACheckBox do
        begin
          Parent := AMsgDialog;
          Caption := sMsgCaption;
          if SavingButton <> '' then
          begin
//14.02.2023 bfacrm:ticket__show__33967            Caption := sMsgCaption + ' ' + Format(_('(bei %s)'), [SavingButton]);
            Caption := sMsgCaption + ' ' + Format(dgettext('Shared36', '(bei %s)'), [SavingButton]);
            AMsgDialog.Width := AMsgDialog.Width + 10; //14.02.2023
          end;
          width := Canvas.TextWidth(Caption) + 50;
          Top := AMsgDialog.Height - ACheckBox.Height - 20;
          Left := 8;
        end;

        if ShowCheckBox then
        {$IF CompilerVersion >= 35}
          AMsgDialog.Height := AMsgDialog.Height + ACheckBox.Height + 10; //05.10.2022 im Delphi11 wird anders dargestellt
        {$ELSE}
          AMsgDialog.Height := AMsgDialog.Height + ACheckBox.Height;
        {$ENDIF}
      end;
      Result := ShowModal;

      if ShowCheckBox then
      begin
        CheckBoxValue := ACheckBox.Checked;
        if (CheckBoxValue) and (Result <> mrNone) then
        begin
          found := False;
          i := 0;
          while (i <= High(ViewerOptions.CheckBoxDialogs)) and not found do
          begin
            if ViewerOptions.CheckBoxDialogs[i].DialogName = DlgSectionID then
              found := True
            else
              inc(i);
          end;

          if found then
          begin
            ViewerOptions.CheckBoxDialogs[i].ShowDialog := not CheckBoxValue;
            ViewerOptions.CheckBoxDialogs[i].DialogResultType := Result;
          end else
          begin
            i := Length(ViewerOptions.CheckBoxDialogs);
            SetLength(ViewerOptions.CheckBoxDialogs, i + 1);
            ViewerOptions.CheckBoxDialogs[i].DialogName := DlgSectionID;
            ViewerOptions.CheckBoxDialogs[i].ShowDialog := not CheckBoxValue;
            ViewerOptions.CheckBoxDialogs[i].DialogResultType := Result;
          end;
          ViewerOptions.SaveCheckboxDialogsOptions;
        end;
      end;
    finally
      Free;
    end;
  end;
end;

function S(const S: string):string;
begin
  Result := StringReplace(S     , '\n', #13, [rfReplaceAll]);
  Result := StringReplace(Result, '<nl>', #13, [rfReplaceAll]);
  Result := StringReplace(Result, '&', '&&', [rfReplaceAll]); // SJ 15.08.2019
end;

function ParseMsg(const msg: string): string;
begin
  Result := S(msg);
end;

function ParseMsgCRLF(const msg: string): string;
begin
  Result := StringReplace(msg     , '\n', #13#10, [rfReplaceAll]);
  Result := StringReplace(Result, '<nl>', #13#10, [rfReplaceAll]);
  Result := StringReplace(Result, '&', '&&', [rfReplaceAll]); // SJ 15.08.2019
end;

procedure BfShowMessage(const Msg: string; AThread : TThread = nil);
begin
  if AThread <> nil then
  begin
    AThread.Synchronize(nil,
      procedure
      begin
        BfMessageBox(Msg, sSysBitfarm,0);
      end);
  end else
    BfMessageBox(Msg, sSysBitfarm,0);
end;

function BfMessageDlg(Msg: WideString; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; Default_NO: boolean = False; scustom: WideString = ''): Integer;
var
  syes, sno, scancel, swarning, sconfirm, sinformation, serror : WideString;
  Dialog: TForm;
begin
   Msg := S(Msg);

   syes := _('Ja');
   sno := _('Nein');
   scancel := _('Abbrechen');
   swarning := _('Warnung');
   sconfirm := _('Besttigung');
   sinformation := _('Informationen');
   serror := _('Fehler');
   Dialog := CreateMessageDialog(Msg, DlgType, Buttons);
   with Dialog do
      try
         if Default_NO then
         	if (DlgType = mtWarning) then
          begin
            ActiveControl := FindComponent('CANCEL') as TWinControl;
            if ActiveControl = nil then
	            ActiveControl := FindComponent('NO') as TWinControl;
          end
          else
	          ActiveControl := FindComponent('NO') as TWinControl;
         (FindComponent('NO') as TWinControl).SetTextBuf(PChar(sno));
         (FindComponent('YES') as TWinControl).SetTextBuf(PChar(syes));
         (FindComponent('CANCEL') as TWinControl).SetTextBuf(PChar(scancel));
         if DlgType = mtConfirmation then
	         Caption := sconfirm;
         if DlgType = mtWarning then
	         Caption := swarning;
         if DlgType = mtInformation then
         	 Caption := sinformation;
	       if DlgType = mtError then
         	 Caption := serror;
	       if (DlgType = mtCustom) or (scustom <> '') then
         	 Caption := scustom;

         Dialog.Position := poMainFormCenter;
         Result := Dialog.ShowModal;
      finally
         Close;
         Free;
      end
end;


function BfMessageBox(Msg, Caption: string; Flags: Word; Handle: THandle = 0): Integer;
begin
  try
    Msg := S(Msg);
    Caption := S(Caption);
    Result := BfMessageDlg(Msg, mtInformation, [mbOK], False, Caption);
  except

  end;
end;
function BfaGetPath(sIniFile: string; sSection, sKey, sTitel: WideString; Updater: Boolean = false): WideString;
var
  ini: TIniFile;
begin
  ini:= TIniFile.Create(sIniFile);
  try
    Result:= ini.ReadString(sSection, sKey, '');

    if (Result = '' ) and (Updater) then
    begin
      with TFileOpenDialog.Create(nil) do
      begin
        Title:= sTitel;
        Options:= [fdoPickFolders];
        if Execute then
        begin
          Result:= FileName;
          ini.WriteString(sSection, sKey, Result);
        end;
      end;
    end;
  finally
    ini.Free;
  end;
end;



function bfaInputQuery(const ACaption: string; const APrompt: string; var Value: string; not_allowed_chars: string = ''): boolean; overload;
begin
  frm_Dialog := Tfrm_Dialog.Create(Application);
  frm_Dialog.cbx_Dialog.Visible := False;
  frm_Dialog.lbl_Dialog.Caption := APrompt;
  frm_Dialog.Caption := ACaption;
  frm_Dialog.isComboBox := False;
  frm_Dialog.not_allowed := not_allowed_chars;
  if Value <> '' then
  begin
    frm_Dialog.edt_Dialog.Text := Value;

  end;
  //frm_Dialog.edt_Dialog.Font.Color := clInactiveCaption;

  frm_Dialog.ShowModal;
  if frm_Dialog.ModalResult = mrOk then
    Value := frm_Dialog.edt_Dialog.Text
  else
    Value := '';

  frm_Dialog.Release;

  Result := frm_Dialog.ModalResult = mrOk;
end;


function bfaInputQuery(const ACaption: string; const APrompt: string; var Value: string; Items: TStringList; fixed: Boolean = False; not_allowed_chars: string = ''): boolean;
begin
  frm_Dialog := Tfrm_Dialog.Create(Application);
  frm_Dialog.edt_Dialog.Visible := False;

  frm_Dialog.lbl_Dialog.Caption := APrompt;
  frm_Dialog.Caption := ACaption;
  frm_Dialog.isComboBox := True;
  frm_Dialog.not_allowed := not_allowed_chars;
  if (Items <> nil) and (Items.Count <> 0) then
  begin
    frm_Dialog.cbx_Dialog.Items := Items;
    frm_Dialog.cbx_Dialog.ItemIndex := 0;
  end;
  if fixed then
    frm_Dialog.cbx_Dialog.Style := csDropDownList
  else
    frm_Dialog.cbx_Dialog.Style := csDropDown;

//  frm_Dialog.edt_Dialog.Font.Color := clInactiveCaption;

  frm_Dialog.ShowModal;
  if frm_Dialog.ModalResult = mrOk then
    Value := frm_Dialog.cbx_Dialog.Text
  else
    Value := '';

  frm_Dialog.Release;

  Result := frm_Dialog.ModalResult = mrOk;
end;

procedure Tfrm_Dialog.cbx_DialogKeyPress(Sender: TObject; var Key: Char);
var
  FBalloonHint: TBalloonHint;
begin
  if (not_allowed <> '') and (Pos(Key, not_allowed) <> 0) then
  begin
    try
      if FBalloonHint = nil then
        FBalloonHint := TBalloonHint.Create(Self);
      FBalloonHint.Delay := 0;
      FBalloonHint.HideAfter := 800;
      FBalloonHint.Description := _(sMsgNotAllowedCharachters) + not_allowed;
      FBalloonHint.ShowHint(TSpTBXComboBox(Sender));
      Key := #0;
      Beep;
      Abort;
    except
    //
    end;
  end;
end;

procedure Tfrm_Dialog.cbx_DialogKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if Key = VK_ESCAPE then  //Escape
  begin
    frm_Dialog.ModalResult := mrCancel;
    frm_Dialog.Close;
  end;

  if Key = VK_RETURN then  //Enter
  begin
    frm_Dialog.btn_Dialog.Click;
  end;
end;

procedure Tfrm_Dialog.edt_DialogChange(Sender: TObject);
begin
//  if frm_Dialog.edt_Dialog.Font.Color = clInactiveCaption then
//  begin
//    edt_Dialog.Font.Color := clWindowText;
//  end;
end;

procedure Tfrm_Dialog.FormShow(Sender: TObject);
begin
  if not isCombobox then
    frm_Dialog.edt_Dialog.SetFocus
  else
    frm_Dialog.cbx_Dialog.SetFocus;
end;

procedure Tfrm_Dialog.FormCreate(Sender: TObject);
begin

  pnl1.Color := stlControl.Color;
  lbl_Dialog.Font.Color := stlControl.Font.Color;

  btn_Dialog.Font.Color := stlControl.Font.Color;
  btn_Dialog.Color := stlControl.Color;
  btnCancel.Font.Color := stlControl.Font.Color;
  btnCancel.Color := stlControl.Color;
  cbx_Dialog.Font.Color := stlControl.Font.Color;
  cbx_Dialog.Color := stlControl.Color;
  edt_Dialog.Font.Color := stlControl.Font.Color;
  edt_Dialog.Color := stlControl.Color;
end;

procedure Tfrm_Dialog.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_ESCAPE then  //Escape
  begin
    frm_Dialog.ModalResult := mrCancel;
    frm_Dialog.Close;
  end;
end;

procedure Tfrm_Dialog.edt_DialogKeyPress(Sender: TObject; var Key: Char);
var
  FBalloonHint: TBalloonHint;
begin
  if (not_allowed <> '') and (Pos(Key, not_allowed) <> 0) then
  begin
    try
      if FBalloonHint = nil then
        FBalloonHint := TBalloonHint.Create(Self);
      FBalloonHint.Delay := 0;
      FBalloonHint.HideAfter := 800;
      FBalloonHint.Description := _(sMsgNotAllowedCharachters) + not_allowed;
      FBalloonHint.ShowHint(TSpTBXEdit(Sender));
      Key := #0;
      Beep;
      Abort;
    except
    //
    end;
  end;
end;

procedure Tfrm_Dialog.edt_DialogKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_ESCAPE then  //Escape
  begin
    frm_Dialog.ModalResult := mrCancel;
    frm_Dialog.Close;
  end;

  if Key = VK_RETURN then  //Enter
  begin
    frm_Dialog.btn_Dialog.Click;
  end;

end;

end.
