unit bfaCommonFiles;

interface

uses
    Windows, Classes;

type
  THandleArray = array of THandle;

  PROCESS_BASIC_INFORMATION = packed record
    ExitStatus: DWORD;
    PebBaseAddress: Pointer;
    AffinityMask: DWORD;
    BasePriority: DWORD;
    UniqueProcessId: DWORD;
    InheritedUniquePID:DWORD;
  end;

  _UNICODE_STRING = record
    Length: Word;
    MaximumLength: Word;
    Buffer: LPWSTR;
  end;
  UNICODE_STRING = _UNICODE_STRING;

procedure FindAllFiles(var FileList: TStrings; RootFolder: string;
   Mask: string = '*.*'; Recurse: boolean = True; CheckExt : Boolean = True);
procedure KillFile(FileName: string);
procedure CopyDirectoryToClient(SrvrDir, ClntDir: WideString);
function CopyBfaFile(ServerDir, ClientDir, FName: WideString) : Boolean;
function BfDeleteFile(const AFile: string): boolean;
function IsDirectoryEmpty(const directory : string) : boolean; //07.06.2011 b.takacs 3.5.1.19 [3069]
function EmptyDirectory(const ADir: string): boolean;
function BfCopyDir(const fromDir, toDir: string; ARename: boolean = false): Boolean;
function BfMoveDir(const fromDir, toDir: string; subdir: Boolean = False): Boolean;
function BfDelDir(dir: string): Boolean;
function ReadDir(Path, Mask: string; ShowPath: boolean): TStringlist;
procedure StringToFile(const Dateiname:Ansistring; const Data:Ansistring); overload;
procedure StringToFile(const Dateiname:WideString; const Data:WideString); overload;
procedure StringToFileUTF8(const Dateiname:WideString; const Data:WideString);
procedure SaveToFile(const Dateiname:Ansistring; const Buffer; Count: Longint);
function BfaGetVersion(sFileName:string): string;
procedure DeleteDirectory(const DirName: string);
function ShellExecute_AndWait(Operation, FileName, Parameter, Directory: string;
   Show: word; bWait: boolean): longint;
function bfaExecFile(filepath:string; alternatestart: Boolean = False): Boolean;
function BfSelectDirectory(const Caption: string; const Root: WideString;
   var Directory: string; EditBox: boolean = False; ShowFiles: boolean = False;
   AllowCreateDirs: boolean = True): boolean;
procedure CopyFilesToClipboard(FileList : string);
procedure CopyLinkToClipboard(const str, htmlStr : AnsiString);
function GetFileModDate(filename : string) : TDateTime;
function processExists(exeFileName: string): Boolean;
function GetCommandLineFromPid(PID: THandle): string;
function  GetCommandLineFromPidWMI(ProcessId:DWORD): string;
function getProcessHandle(exeFileName: string): THandle;
function getProcessHandles(exeFileName: string): THandleArray;
function  NtQueryInformationProcess(ProcessHandle : THandle; ProcessInformationClass : DWORD; ProcessInformation : Pointer; ProcessInformationLength : ULONG; ReturnLength : PULONG ): LongInt; stdcall; external 'ntdll.dll';


implementation

uses
    Forms, ShellAPI, bfaCommonUtils, SysUtils, ShlObj, ActiveX,
    Clipbrd, TlHelp32, System.IOUtils, Variants, ComObj;

procedure CopyDirectoryToClient(SrvrDir, ClntDir: WideString);
var
  sts: Integer ;
  SR: TSearchRec;
begin
  sts:= FindFirst(SrvrDir + '*.*' , faAnyFile , SR );
  if sts = 0 then
  begin
    while FindNext( SR ) = 0 do
    begin
      if (SR.Name <> '.' ) and ( SR.Name <> '..' ) then
      begin
        if Pos('.', SR.Name) = 0 then
        begin
          ForceDirectories( ClntDir + SR.Name );
          CopyDirectoryToClient(SrvrDir + SR.Name + '\', ClntDir + SR.Name
                              + '\' ) ;
        end
        else
          copyfile( pchar(SrvrDir + SR.Name), pchar(ClntDir + sr.name), true );
      end;
    end;
    FindClose( SR ) ;
  end;
end;

function CopyBfaFile(ServerDir, ClientDir, FName: WideString) : Boolean;
var
  ServerFn, ClientFn: WideString;
  ServerVer, ClientVer: string;
begin
  Result := False;
  try
    ServerFn:= ServerDir + '\' + FName;
    if FileExists(ServerFn) then
      ServerVer:= BfaGetVersion(ServerFn)
    else
      ServerVer:= '';

    ClientFn:= ClientDir + '\' + FName;
    if FileExists(ClientFn) then
      ClientVer:= BfaGetVersion(ClientFn)
    else
      ClientVer:= '';

    if (ServerVer <> '') then
    begin
      if (ServerVer > ClientVer) or (ClientVer = '') then
        Result := CopyFile(PWideChar(ServerFn), pWideChar(ClientFn), False);
    end;
  except
    Result := False;
  end;
end;

procedure FindAllFiles(var FileList: TStrings; RootFolder: string;
   Mask: string = '*.*'; Recurse: boolean = True; CheckExt : Boolean = True);
var
   SR: TSearchRec;
begin
   IncludeTrailingPathDelimiter(RootFolder);

   if Recurse then
      if FindFirst(RootFolder + '*.*', faAnyFile, SR) = 0 then
         try
            repeat
               if SR.Attr and faDirectory = faDirectory then
                  // --> ein Verzeichnis wurde gefunden
                  //   der Verzeichnisname steht in SR.Name
                  //   der vollstndige Verzeichnisname (inkl. darberliegender Pfade) ist
                  //       RootFolder + SR.Name
                  if (SR.Name <> '.') and (SR.Name <> '..') then
                     FindAllFiles(FileList, RootFolder + SR.Name, Mask, Recurse);
            until FindNext(SR) <> 0;
         finally
            FindClose(SR);
         end;
   if FindFirst(RootFolder + Mask, faAnyFile, SR) = 0 then
      try
         repeat
            if SR.Attr and faDirectory <> faDirectory then
            begin
               // --> eine Datei wurde gefunden
               //   der Dateiname steht in SR.Name
               //   der vollstndige Dateiname (inkl. Pfadangabe) ist
               //       RootFolder + SR.Name
               // sonst findet *.con auch .conf
               if CheckExt then
               begin
                  if (lowercase(ExtractFileExt(SR.Name)) = lowercase(ExtractFileExt(mask))) then
    	               FileList.Add(RootFolder + SR.Name);
               end else
                  FileList.Add(RootFolder + SR.Name);
            end;
         until FindNext(SR) <> 0;
      finally
         FindClose(SR);
      end;
end;

procedure KillFile(FileName: string);
var
   f: file;
begin
//   GgfLogEvnt('KillFile:' + FileName);

   if FileExists(FileName) then
   begin
      assignfile(f, FileName);
{$I-}
      Erase(f);
{$I+}
   end
   else
   begin
      exit;
   end;
end;


function IsDirectoryEmpty(const directory : string) : boolean;
var
   searchRec :TSearchRec;
begin
  try
    result := (FindFirst(directory + '\*.*', faAnyFile, SearchRec) = 0) AND
              (FindNext(searchRec) = 0) AND
              (FindNext(searchRec) <> 0) ;
  finally
     FindClose(searchRec) ;
  end;
end;


function EmptyDirectory(const ADir: string): boolean;
var
   searchRec :TSearchRec;
begin
   Result := True;
   try
      if (FindFirst(ADir + '\*.*', faAnyfile - faDirectory, SearchRec) = 0) then
      begin
         repeat
            DeleteFile(ADir + '\' + SearchRec.Name);
         until (FindNext(SearchRec) <> 0);
         FindClose(SearchRec);
      end;
   except
       Result := False;
   end;
end;

function BFDeleteFile(const AFile: string): boolean;
var
   sh: SHFileOpStruct;
begin
  try
    ZeroMemory(@sh, sizeof(sh));
    with sh do
    begin
      Wnd := Application.Handle;
      wFunc := fo_Delete;
      pFrom := PChar(AFile + #0);
      fFlags := fof_Silent or fof_NoConfirmation;
    end;
    Result := SHFileOperation(sh) = 0;
  except
  	result:= false; //30.05.2011 b.takacs 3.5.1.16
  end;
end;

function BfCopyDir(const fromDir, toDir: string; ARename: boolean = false): Boolean;
var
  fos: TSHFileOpStruct;
begin
  ZeroMemory(@fos, SizeOf(fos));
  with fos do
  begin
    wFunc  := FO_COPY;
    fFlags := FOF_FILESONLY;
    pFrom  := PChar(fromDir + #0);
    pTo    := PChar(toDir);
    if ARename then
    	 fFlags := fFlags or fof_RenameOnCollision;
  end;
  Result := (0 = ShFileOperation(fos));
end;

function BfMoveDir(const fromDir, toDir: string; subdir: Boolean = False): Boolean;
var
  fos: TSHFileOpStruct;
begin
  ZeroMemory(@fos, SizeOf(fos));
  with fos do
  begin
    if (subdir = True) then
    begin
      wFunc  := FO_MOVE;
      fFlags := FOF_SILENT + FOF_NOCONFIRMATION;
      pFrom  := PChar(fromDir + '*.*' + #0);
      pTo    := PChar(toDir);
    end else
    begin
      wFunc  := FO_MOVE;
      fFlags := FOF_FILESONLY;
      pFrom  := PChar(fromDir + #0);
      pTo    := PChar(toDir);
    end;
  end;
  Result := (0 = ShFileOperation(fos));
end;

function BfDelDir(dir: string): Boolean;
var
  fos: TSHFileOpStruct;
begin
  ZeroMemory(@fos, SizeOf(fos));
  with fos do
  begin
    wFunc  := FO_DELETE;
    fFlags := FOF_SILENT or FOF_NOCONFIRMATION;
    pFrom  := PChar(dir + #0);
  end;
  Result := (0 = ShFileOperation(fos));
end;

function ReadDir(Path, Mask: string; ShowPath: boolean): TStringlist;
var SRec: TSearchRec;
begin
   //Path := IncludeTrailingBackslash(Path);
   Path := IncludeTrailingPathDelimiter(Path);
   Result := TStringList.Create;
   try
      if FindFirst(Path + Mask, not faDirectory, SRec) <> 0 then
         Exit;
      repeat
         case ShowPath of
            True:
               result.Add(Path + SRec.Name);
            False:
               result.Add(SRec.Name);
         end;
      until FindNext(SRec) <> 0;
   finally
      FindClose(SRec);
   end;
end;

procedure StringToFile(const Dateiname:Ansistring; const Data:Ansistring); overload;
begin
	if Data <> '' then    //18.07.2011 b.takacs 3.5.1.30 [3117]
	   SaveToFile(Dateiname,Data[1],length(Data))
  else
	   SaveToFile(Dateiname,Data,length(Data)); //18.07.2011 b.takacs 3.5.1.30 [3117]
end;
{>3.6}

procedure StringToFile(const Dateiname:WideString; const Data:WideString); overload;
begin
//	if Data <> '' then
//	   SaveToFile(Dateiname,Data[1],length(Data))
//  else
//	   SaveToFile(Dateiname,Data,length(Data));
  TFile.WriteAllText(Dateiname, Data, TEncoding.ANSI)
end;

procedure StringToFileUTF8(const Dateiname:WideString; const Data:WideString);
begin
  TFile.WriteAllText(Dateiname, Data, TEncoding.UTF8)

end;

//-----------------------------------------------------------------------------
procedure SaveToFile(const Dateiname:Ansistring; const Buffer; Count: Longint);
var FS:TFileStream; // w:integer; f:file;
begin
   FS:=TFileStream.Create(Dateiname, fmCreate);
   try
      FS.Write(Buffer, Count);
   finally
      FS.Free;
   end;
end;

function BfaGetVersion(sFileName:string): string;
var
  VerInfoSize : DWORD;
  VerInfo : Pointer;
  VerValueSize : DWORD;
  VerValue : PVSFixedFileInfo;
  Dummy : DWORD;
begin
  try
    VerInfo := nil;
    VerInfoSize := GetFileVersionInfoSize(PChar(sFileName), Dummy);
    if VerInfoSize > 0 then
    begin
      GetMem(VerInfo, VerInfoSize);
      GetFileVersionInfo(PChar(sFileName), 0, VerInfoSize, VerInfo);
      VerQueryValue(VerInfo, '\', Pointer(VerValue), VerValueSize);
      with VerValue^ do
      begin
        Result := IntToStr(dwFileVersionMS shr 16);
        Result := Result + '.' + IntToStr(dwFileVersionMS and $FFFF);
        Result := Result + '.' + IntToStr(dwFileVersionLS shr 16);
        Result := Result + '.' + IntToStr(dwFileVersionLS and $FFFF);
      end;
    end
    else
    begin
      Result:= '0.0.0.0';
    end;
    FreeMem(VerInfo, VerInfoSize);
  except
    Result:= '0.0.0.0';
  end;
end;

procedure DeleteDirectory(const DirName: string);
var
  FileOp: TSHFileOpStruct;
begin
  try
    FillChar(FileOp, SizeOf(FileOp), 0);
    FileOp.wFunc := FO_DELETE;
    FileOp.pFrom := PChar(DirName + #0);
//01.08.2023 bfacrm:ticket__show__35875
//02.08.2023    FileOp.fFlags := FOF_SILENT or FOF_NOCONFIRMATION; //ViewerV362_179_without_noerror.exe
//01.08.2023    FileOp.fFlags := FOF_SILENT or FOF_SIMPLEPROGRESS or FOF_NOCONFIRMATION; //ViewerV362_179_without_noerror_with_simpleproc.exe
    FileOp.fFlags := FOF_SILENT or FOF_NOERRORUI or FOF_NOCONFIRMATION; //orig 3.6.2.179
    SHFileOperation(FileOp);
  except
    //01.08.2023 3.6.2.179
  end;
end;

function ShellExecute_AndWait(Operation, FileName,
   Parameter, Directory: string; Show: word; bWait: boolean): longint;
var
   bOK : boolean;
   Info : TShellExecuteInfo;
   rtn : Integer;
{
  ****** Parameters ******
  Operation:

  edit  Launches an editor and opens the document for editing.
  explore Explores the folder specified by lpFile.
  find Initiates a search starting from the specified directory.
  open Opens the file, folder specified by the lpFile parameter.
  print Prints the document file specified by lpFile.
  properties Displays the file or folder's properties.

  FileName:

  Specifies the name of the file or object on which
  ShellExecuteEx will perform the action specified by the lpVerb parameter.

  Parameter:

  String that contains the application parameters.
  The parameters must be separated by spaces.

  Directory:

  specifies the name of the working directory.
  If this member is not specified, the current directory is used as the working directory.

  Show:

  Flags that specify how an application is to be shown when it is opened.
  It can be one of the SW_ values

  bWait:

  If true, the function waits for the process to terminate
}
begin
   FillChar(Info, SizeOf(Info), Chr(0));
   Info.cbSize := SizeOf(Info);
   Info.fMask := SEE_MASK_NOCLOSEPROCESS;
   Info.lpVerb := PChar(Operation);
   Info.lpFile := PChar(FileName);
   Info.lpParameters := PChar(Parameter);
   Info.lpDirectory := PChar(Directory);
   Info.nShow := Show;
   bOK := boolean(ShellExecuteEx(@Info));
   if bOK then
   begin
      if bWait then
      begin
         while WaitForSingleObject(Info.hProcess, 100) = WAIT_TIMEOUT do
         begin
//         	 Application.ProcessMessages;
           Sleep(2);
         end;

         bOK := GetExitCodeProcess(Info.hProcess, DWORD(Result));
      end
      else
         Result := 0;
   end;
   if not bOK then
      Result := -1;
end;

function bfaExecFile(filepath:string; alternatestart: Boolean = False): Boolean;
var
   si: TStartupInfo;
   pi: TProcessInformation;
   parameter, aparameter, lsprogname: string;
   rc: Integer;
begin
   if FileExists(filepath) then
   begin
     if alternatestart then
     begin
        FillChar(si, SizeOf(si), 0);
        si.cb := SizeOf(si);
        si.dwFlags := STARTF_USESHOWWINDOW;
        si.wShowWindow := SW_hide;
        FillChar(pi, SizeOf(pi), 0);

        parameter := filepath;
        AParameter := Format('cmd /c %s', [TrimRight(parameter)]);
        {errb := }CreateProcess(nil, PChar(Aparameter), nil, nil,
           longbool(False), CREATE_NEW_CONSOLE or NORMAL_PRIORITY_CLASS,
           nil, PChar(BD.ExeDir), si, pi);
     end
     else
     begin
        try
          rc := ShellExecute(Application.Handle, nil,
           PChar(filepath), nil,
           nil, SW_NORMAL);
        except
          on e: Exception do
          begin
            Result := False;
          end;
        end;

        if (rc < 33) then
        begin
           if (rc = 31) then
           begin
              lsprogname := Format('shell32.dll,OpenAs_RunDLL %s',
                 [filepath]);
              rc := ShellExecute(Application.Handle, nil,
                 'rundll32.exe', PChar(lsprogname), nil, SW_SHOW);
           end
           else
              Result := False;
        end;
     end;
   end;
end;

//   Erweiterte Funktion zur Auswahl eines Verzeichnisses
//   Die Funktion wurde um einen Button zum Erzeugen eines Verzeichnisses erweitert
//   BfSelectDirectory('Caption', 'c:\', dir, False, False, True);
//   lbl_PathtoDir.Caption := dir;
function BfSelectDirectory(const Caption: string; const Root: WideString;
   var Directory: string; EditBox: boolean = False; ShowFiles: boolean = False;
   AllowCreateDirs: boolean = True): boolean;

   function SelectDirCB(Wnd: HWND; uMsg: UINT; lParam, lpData: lParam): integer; stdcall;
   begin
      case uMsg of
         BFFM_INITIALIZED:
            SendMessage(Wnd, BFFM_SETSELECTION, Ord(True),
               integer(lpData));
      {BFFM_SELCHANGED:
      begin
        SHGetPathFromIDList(PItemIDList(lParam), @PathName);
      end;}
      end;
      Result := 0;
   end;
var
   WindowList: Pointer;
   BrowseInfo: TBrowseInfo;
   Buffer: PChar;
   RootItemIDList, ItemIDList: PItemIDList;
   ShellMalloc: IMalloc;
   IDesktopFolder: IShellFolder;
   Eaten, Flags: longword;
const
   BIF_USENEWUI = $0040;
   BIF_NOCREATEDIRS = $0200;
begin
   Result := False;
   if (Directory = '') and (sLastDir <> '') then
   	Directory := sLastDir;
   if not DirectoryExists(Directory) then
      Directory := '';
   FillChar(BrowseInfo, SizeOf(BrowseInfo), 0);
   if (ShGetMalloc(ShellMalloc) = S_OK) and (ShellMalloc <> nil) then
   begin
      Buffer := ShellMalloc.Alloc(MAX_PATH);
      try
         RootItemIDList := nil;
         if Root <> '' then
         begin
            SHGetDesktopFolder(IDesktopFolder);
            IDesktopFolder.ParseDisplayName(Application.Handle, nil,
               POleStr(Root), Eaten, RootItemIDList, Flags);
         end;
         OleInitialize(nil);
         with BrowseInfo do
         begin
            hwndOwner := Application.Handle;
            pidlRoot := RootItemIDList;
            pszDisplayName := Buffer;
            lpszTitle := PChar(Caption);
            ulFlags := BIF_RETURNONLYFSDIRS or BIF_USENEWUI or
               BIF_EDITBOX * Ord(EditBox) or BIF_BROWSEINCLUDEFILES *
               Ord(ShowFiles) or BIF_NOCREATEDIRS * Ord(not AllowCreateDirs);
            lpfn := @SelectDirCB;
            if Directory <> '' then
               lParam := integer(PChar(Directory));
         end;
         WindowList := DisableTaskWindows(0);
         try
            ItemIDList := ShBrowseForFolder(BrowseInfo);
         finally
            EnableTaskWindows(WindowList);
         end;
         Result := ItemIDList <> nil;
         if Result then
         begin
            ShGetPathFromIDList(ItemIDList, Buffer);
            ShellMalloc.Free(ItemIDList);
            Directory := Buffer;
            sLastDir := ExtractFileDir(Directory);
         end;
      finally
         ShellMalloc.Free(Buffer);
      end;
   end;
end;

procedure CopyFilesToClipboard(FileList : string);
var
  DropFiles : PDropFiles;
  hGlobal : THandle;
  iLen : Integer;
begin
  iLen := Length(FileList);
  hGlobal := GlobalAlloc(GMEM_SHARE or GMEM_MOVEABLE or GMEM_ZEROINIT,
  SizeOf(TDropFiles) + ((iLen + 2) * SizeOf(Char)));
  if (hGlobal = 0) then raise Exception.Create('Could not allocate memory.');
  try
    DropFiles := GlobalLock(hGlobal);
    if (DropFiles = nil) then raise Exception.Create('Could not access allocated memory.');
    try
      DropFiles^.pFiles := SizeOf(TDropFiles);
      DropFiles^.fWide := True;

      if FileList <> '' then
        Move(FileList[1], (PByte(DropFiles) + SizeOf(TDropFiles))^, iLen *
      SizeOf(Char));
    finally
      GlobalUnlock(hGlobal);
    end;
    Clipboard.SetAsHandle(CF_HDROP, hGlobal);
  except
    GlobalFree(hGlobal);
  end;
end;

function FormatHTMLClipboardHeader(HTMLText : AnsiString) : AnsiString;
const
  CrLf = #13#10;
begin
  Result := 'Version:0.9' + CrLf;
  Result := Result + 'StartHTML:000087' + CrLf;
  Result := Result + 'EndHTML:' + CrLf;
  Result := Result + 'StartFragment:000101' + CrLf;
  Result := Result + 'EndFragment:^^^^^^' + CrLf;
  Result := Result + HTMLText + CrLf;
  Result := StringReplace(Result, '', Format('%.6d', [Length(Result)]), [rfReplaceAll]);
  Result := StringReplace(Result, '^^^^^^', Format('%.6d', [Length(Result) - 16]), [rfReplaceAll]);
end;

procedure CopyLinkToClipboard(const str, htmlStr : AnsiString);
var
  gMem : HGLOBAL;
  lp : PChar;
  Strings : array[0..1] of AnsiString;
  Formats : array[0..1] of UINT;
  i : Integer;
begin
  {$IFNDEF USEVCLCLIPBOARD}
    Win32Check(OpenClipBoard(0));
  {$ENDIF}
  try
    //most descriptive first as per api docs

    Strings[0] := FormatHTMLClipboardHeader(htmlStr);
    Strings[1] := str;
    Formats[0] := RegisterClipboardFormat('HTML Format');
//02.12.2016 Ticket 9209   Formats[1] := CF_UNICODETEXT;
    Formats[1] := CF_TEXT;

    {$IFNDEF USEVCLCLIPBOARD}
      Win32Check(EmptyClipBoard);
    {$ENDIF}
    for i := 0 to High(Strings) do
    begin
      if Strings[i] = '' then
        Continue;
       //an extra "1" for the null terminator
      gMem := GlobalAlloc(GMEM_DDESHARE + GMEM_MOVEABLE, (Length(Strings[i]) + 1));
      {Succeeded, now read the stream contents into the memory the pointer points at}
      try
        Win32Check(gmem <> 0);
        lp := GlobalLock(gMem);
        Win32Check(lp <> nil);
        CopyMemory(lp, PChar(Strings[i]), (Length(Strings[i]) + 1));
      finally
        GlobalUnlock(gMem);
      end;
      Win32Check(gmem <> 0);
      SetClipboardData(Formats[i], gMEm);
      Win32Check(gmem <> 0);
    end;
  finally
    {$IFNDEF USEVCLCLIPBOARD}
      Win32Check(CloseClipBoard);
    {$ENDIF}
  end;
end;

function GetFileModDate(filename : string) : TDateTime;
var
   F : TSearchRec;
begin
   FindFirst(filename,faAnyFile,F);
   Result := F.TimeStamp;
   //if you really wanted an Int, change the return type and use this line:
   //Result := F.Time;
   FindClose(F);
end;

function GetCommandLineFromPid(PID: THandle): string;
const
  STATUS_SUCCESS             = $00000000;
  SE_DEBUG_NAME              = 'SeDebugPrivilege';
  OffsetProcessParametersx32 = $10;//16
  OffsetCommandLinex32       = $40;//64
var
  ProcessHandle        : THandle;
  rtlUserProcAddress   : Pointer;
  CommandLine          : UNICODE_STRING;
  CommandLineContents  : WideString;
  ProcessBasicInfo     : PROCESS_BASIC_INFORMATION;
  ReturnLength         : Cardinal;
  ReturnLength2        : NativeUInt; //01.03.2019 XE2
  TokenHandle          : THandle;
  lpLuid               : TOKEN_PRIVILEGES;
  OldlpLuid            : TOKEN_PRIVILEGES;
begin
  Result:='';
  if OpenProcessToken(GetCurrentProcess(), TOKEN_ADJUST_PRIVILEGES or TOKEN_QUERY, TokenHandle) then
  begin
    try
      if not LookupPrivilegeValue(nil, SE_DEBUG_NAME, lpLuid.Privileges[0].Luid) then
        RaiseLastOSError
      else
      begin
        lpLuid.PrivilegeCount := 1;
        lpLuid.Privileges[0].Attributes  := SE_PRIVILEGE_ENABLED;
        ReturnLength := 0;
        OldlpLuid    := lpLuid;
        //Set the SeDebugPrivilege privilege
        if not AdjustTokenPrivileges(TokenHandle, False, lpLuid, SizeOf(OldlpLuid), OldlpLuid, ReturnLength) then RaiseLastOSError;
      end;

      ReturnLength2 := ReturnLength; //01.03.2019 XE2
      ProcessHandle := OpenProcess(PROCESS_QUERY_INFORMATION or PROCESS_VM_READ, false, PID);
      if ProcessHandle=0 then RaiseLastOSError
      else
      try
        // get the PROCESS_BASIC_INFORMATION to access to the PEB Address
        if (NtQueryInformationProcess(ProcessHandle,0{=>ProcessBasicInformation},@ProcessBasicInfo, sizeof(ProcessBasicInfo), @ReturnLength)=STATUS_SUCCESS) and (ReturnLength=SizeOf(ProcessBasicInfo)) then
        begin
          //get the address of the RTL_USER_PROCESS_PARAMETERS struture
//01.03.2019 XE2          if not ReadProcessMemory(ProcessHandle, Pointer(Longint(ProcessBasicInfo.PEBBaseAddress) + OffsetProcessParametersx32), @rtlUserProcAddress, sizeof(Pointer), ReturnLength) then
          if not ReadProcessMemory(ProcessHandle, Pointer(Longint(ProcessBasicInfo.PEBBaseAddress) + OffsetProcessParametersx32), @rtlUserProcAddress, sizeof(Pointer), ReturnLength2) then
            RaiseLastOSError
          else
//01.03.2019 XE2          if ReadProcessMemory(ProcessHandle, Pointer(Longint(rtlUserProcAddress) + OffsetCommandLinex32), @CommandLine, sizeof(CommandLine), ReturnLength) then
          if ReadProcessMemory(ProcessHandle, Pointer(Longint(rtlUserProcAddress) + OffsetCommandLinex32), @CommandLine, sizeof(CommandLine), ReturnLength2) then
          begin
            SetLength(CommandLineContents, CommandLine.length);
            //get the CommandLine field
//01.03.2019 XE2            if ReadProcessMemory(ProcessHandle, CommandLine.Buffer, @CommandLineContents[1], CommandLine.Length, ReturnLength) then
            if ReadProcessMemory(ProcessHandle, CommandLine.Buffer, @CommandLineContents[1], CommandLine.Length, ReturnLength2) then
             Result := WideCharLenToString(PWideChar(CommandLineContents), CommandLine.length div 2)
            else
            RaiseLastOSError;
          end;
        end
        else
        RaiseLastOSError;
      finally
        CloseHandle(ProcessHandle);
      end;
    finally
      CloseHandle(TokenHandle);
    end;
  end
  else
    RaiseLastOSError;
end;

function GetCommandLineFromPidWMI(ProcessId:DWORD): string;
var
  FSWbemLocator : OLEVariant;
  FWMIService   : OLEVariant;
  FWbemObjectSet: OLEVariant;
begin;
  CoInitialize(nil);
  try
    Result:='';
    FSWbemLocator := CreateOleObject('WbemScripting.SWbemLocator');
    FWMIService   := FSWbemLocator.ConnectServer('localhost', 'root\CIMV2', '', '');
    //if the pid not exist a EOleException exception will be raised with the code $80041002 - Object Not Found
    FWbemObjectSet:= FWMIService.Get(Format('Win32_Process.Handle="%d"',[ProcessId]));
    Result:=FWbemObjectSet.CommandLine;
  finally
    CoUninitialize;
  end;
end;

function processExists(exeFileName: string): Boolean;
var
  ContinueLoop: BOOL;
  FSnapshotHandle: THandle;
  FProcessEntry32: TProcessEntry32;
begin
  FSnapshotHandle := CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
  FProcessEntry32.dwSize := SizeOf(FProcessEntry32);
  ContinueLoop := Process32First(FSnapshotHandle, FProcessEntry32);
  Result := False;
  while Integer(ContinueLoop) <> 0 do
  begin
    if ((UpperCase(ExtractFileName(FProcessEntry32.szExeFile)) =
      UpperCase(ExeFileName)) or (UpperCase(FProcessEntry32.szExeFile) =
      UpperCase(ExeFileName))) then
    begin
      Result := True;
    end;
    ContinueLoop := Process32Next(FSnapshotHandle, FProcessEntry32);
  end;
  CloseHandle(FSnapshotHandle);
end;

function getProcessHandle(exeFileName: string): THandle;
var
  ContinueLoop: BOOL;
  FSnapshotHandle: THandle;
  FProcessEntry32: TProcessEntry32;
begin
  FSnapshotHandle := CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
  FProcessEntry32.dwSize := SizeOf(FProcessEntry32);
  ContinueLoop := Process32First(FSnapshotHandle, FProcessEntry32);
  Result := 0;
  while Integer(ContinueLoop) <> 0 do
  begin
    if ((UpperCase(ExtractFileName(FProcessEntry32.szExeFile)) =
      UpperCase(ExeFileName)) or (UpperCase(FProcessEntry32.szExeFile) =
      UpperCase(ExeFileName))) then
    begin
      Result := FProcessEntry32.th32ProcessID;
    end;
    ContinueLoop := Process32Next(FSnapshotHandle, FProcessEntry32);
  end;
  CloseHandle(FSnapshotHandle);
end;

function getProcessHandles(exeFileName: string): THandleArray;
var
  ContinueLoop: BOOL;
  FSnapshotHandle: THandle;
  FProcessEntry32: TProcessEntry32;
begin
  FSnapshotHandle := CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
  FProcessEntry32.dwSize := SizeOf(FProcessEntry32);
  ContinueLoop := Process32First(FSnapshotHandle, FProcessEntry32);
  SetLength(result,0);
  while Integer(ContinueLoop) <> 0 do
  begin
    if ((UpperCase(ExtractFileName(FProcessEntry32.szExeFile)) =
      UpperCase(ExeFileName)) or (UpperCase(FProcessEntry32.szExeFile) =
      UpperCase(ExeFileName))) then
    begin
      SetLength(Result,Length(Result) +1);
      Result[High(Result)] := FProcessEntry32.th32ProcessID;
    end;
    ContinueLoop := Process32Next(FSnapshotHandle, FProcessEntry32);
  end;
  CloseHandle(FSnapshotHandle);
end;

end.
