unit bfaLog;

interface

uses
  Classes,
  Generics.Collections, SyncObjs, StrUtils;
  
const
  ///////// Log Levels ///////////////////////////////////////////////////////////
  llNormal = 0;  // Fehlercodes
//  llLow = 1;  // Low
//  llLowGUI = 2;  // GUI Sachen
  llMedium = 1;  // BaseObj Initialization Sachen
  llHigh = 2;  // BaseObjects
  llDebug = 3;  // DEBUG

  ////////// Log Types//  ////////////////////////////////////////////////////////
  ltInfo = 'INFO:';
  ltDebug = 'DEBUG:';
  ltError = 'ERROR:';
  ltWarning = 'WARNING:';
  ltException = 'EXCEPTION - ERROR:';

  function SortByDate(aList: TStringList; aIndex1, aIndex2: Integer): Integer;

type
  TThreadedLog = class;

  TCleanLogFilesThread = class(TThread)
  public
    Logdir: string;
    Logprefix: string;

    constructor Create(CreateSuspended: Boolean; Directory: string; Prefix: string); overload;
    procedure Execute; override;
  end;

  /// <summary>
  ///   bitfarm Log Klasse
  /// </summary>
  TbfaLog = class(TObject)
	private
    fIndent: Integer;

    /// <Beschreibung>Die Log Bibliothek</Beschreibung>
    /// <Description>The Log directory</Description>
		FLogDir: WideString;

    /// <Beschreibung>Logdatei Name</Beschreibung>
    /// <Description>Logfile name</Description>
		FLogFileName: WideString;

    /// <Beschreibung>Aktuelle Log Level</Beschreibung>
    /// <Description>Actual Log Level</Description>
		FLogLevel: Integer;

    /// <Beschreibung>Zustzliche Parameter zu ergnzen die Logdatei Name</Beschreibung>
    /// <Description>Additional parameter to complete the logfile name </Description>
		FAdditionalParam: WideString;

    /// <Beschreibung>Fehlercode</Beschreibung>
    /// <Description>Error Code</Description>
		FErrorCode: Integer;

    /// <Beschreibung>Wenn das Objekt zustande gekommen ist, dann true, sonst false.</Beschreibung>
    /// <Description>If the object is initialized, then true, else false</Description>
		FInitialized: Boolean;

    /// <Beschreibung>Vorbergehende Speicher zu Log Eintrge</Beschreibung>
    /// <Description>Temporary Container for the Log lines</Description>
    FQueue: TQueue<String>;

    /// <Beschreibung>Log Thread zum Logdatei schreiben</Beschreibung>
    /// <Description>Log Thread to write the Log file</Description>
    FWriteLogThread: TThreadedLog;

    /// <Beschreibung>Wenn der Queue ist gendert, wird das LogThread neustarten</Beschreibung>
    /// <Description>If the Queue changed set a new event to restart the LogThread</Description>
    FDevDebug: Boolean;

    /// <Beschreibung>Anzahl der Log-Dateien die nicht gelscht werden</Beschreibung>
    /// <Description>...</Description>
    FRotationCount: Integer;

    procedure QueueChanged(Sender: TObject; const Item: String;
      Action: TCollectionNotification);

    /// <Beschreibung>Schreibt den Parameter in der Logdatei aus</Beschreibung>
    /// <Description>Write out the parameter to the Logfile</Description>
    function WriteOutToLog(Line: WideString): Integer;

    function Get_QueueCount: Integer;
    
  public
    /// <Beschreibung>Setzt die Standardwerte und erstellt die Objekte</Beschreibung>
    /// <Description>Sets the default values and creates the objects</Description>
    constructor Create;

    destructor Destroy; override;

    /// <Beschreibung>Initialisiert das Log Objekt, und einstellt die Logdatei und Loglevel</Beschreibung>
    /// <Description>Initialize the Log object and set the Logdir and loglevel</Description>
    function Init(const LogDir, AdditionalParam: WideString; LogLevel: Integer; AppName: WideString = ''; devdebug: Boolean = False): WordBool;

    /// <Beschreibung>Schreibt die Zeile in der Queue</Beschreibung>
    /// <Description>Write to the LogQueue</Description>
    procedure Write(LogLevel: Integer; const LogType, Line: WideString; ClsName: WideString = '');

    /// <Beschreibung>Schreibt die Meldung im Debugmodus</Beschreibung>
    /// <Description>Write Debug Message</Description>
    procedure Debug(LogLevel: Integer; Line: WideString; ClsName: WideString = '');

    /// <Beschreibung>Schreibt die Meldung im Debugmodus, rckt nachfolgende Meldung ein</Beschreibung>
    /// <Description>Write Debug Message, increase indent</Description>
    procedure DebugBegin(LogLevel: Integer; Line: WideString; ClsName: WideString = '');

    /// <Beschreibung>Schreibt die Meldung im Debugmodus, rckt nachfolgende Meldungen aus</Beschreibung>
    /// <Description>Write Debug Message, decrease indent</Description>
    procedure DebugEnd(LogLevel: Integer; Line: WideString; ClsName: WideString = '');

    /// <Beschreibung>Schreibt die Meldung , rckt nachfolgende Meldungen ein</Beschreibung>
    /// <Description>Write Message, increase indent</Description>
    procedure WriteBegin(LogLevel: Integer; LogType, Line, ClsName: WideString);

    /// <Beschreibung>Schreibt die Meldung, rckt nachfolgende Meldungen aus</Beschreibung>
    /// <Description>Write Message, decrease indent</Description>
    procedure WriteEnd(LogLevel: Integer; LogType, Line, ClsName: WideString);

    /// <Beschreibung>Schreibt die Meldung im Infomodus</Beschreibung>
    /// <Description>Write Info Message</Description>
    procedure Info(LogLevel: Integer; Line: WideString; ClsName: WideString = '');

    /// <Beschreibung>Schreibt die Meldung im Warningmodus</Beschreibung>
    /// <Description>Write Warning Message</Description>
    procedure Warning(LogLevel: Integer; Line: WideString; ClsName: WideString = '');

    /// <Beschreibung>Schreibt die Meldung im Errormodus</Beschreibung>
    /// <Description>Write Error Message</Description>
    procedure Error(LogLevel: Integer; Line: WideString; ClsName: WideString = '');

    /// <Beschreibung>Schreibt die Meldung im Exceptionmodus</Beschreibung>
    /// <Description>Write Exception Message</Description>
    procedure Exception(LogLevel: Integer; Line: WideString; ClsName: WideString = '');

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function FlushAndClone: string;

    procedure ChangeLogLevel(Value: Integer);

    /// <Beschreibung>Die Log Bibliothek</Beschreibung>
    /// <Description>The Log directory</Description>
    property LogDir: WideString read FLogDir;

    /// <Beschreibung>Logdatei Name</Beschreibung>
    /// <Description>Logfile name</Description>
    property LogFileName: WideString read FLogFileName;

    property RotationCount: Integer read FRotationCount write FRotationCount default 10;

    /// <Beschreibung>Aktuelle Log Level</Beschreibung>
    /// <Description>Actual Log Level</Description>
    property LogLevel: Integer read FLogLevel;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property QueueCount: Integer read Get_QueueCount;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property Initialized: Boolean read FInitialized;
  end;

  PbfaLog = ^TbfaLog;

  /// <Beschreibung></Beschreibung>
  /// <Description></Description>
  TThreadedLog = class (TThread)
   private
     /// <Beschreibung></Beschreibung>
     /// <Description></Description>
     FQueue: TQueue<String>;

     /// <Beschreibung></Beschreibung>
     /// <Description></Description>
     FLog: TbfaLog;

     /// <Beschreibung></Beschreibung>
     /// <Description></Description>
     FEvent: TEvent;

     /// <Beschreibung></Beschreibung>
     /// <Description></Description>
     FRunning: boolean;
   public
      destructor Destroy; override;

      /// <Beschreibung></Beschreibung>
      /// <Description></Description>
      property Terminated;

      /// <Beschreibung>Erstellt und startet das Log Thread</Beschreibung>
      /// <Description>Creates and start the Log Thread</Description>
      function RunThread(Queue: TQueue<String>; Log: TbfaLog): TThreadedLog;

      /// <Beschreibung></Beschreibung>
      /// <Description></Description>
      property Running: boolean read FRunning;
   protected
      /// <Beschreibung>Schreibt aus dem Inhalt des Queues in der Logdatei</Beschreibung>
      /// <Description>Writes out the Queue contents into the Logfile</Description>
      procedure Execute; override;
    end;

var
  bitfarmLog: TbfaLog = nil;

implementation

uses
  Windows, SysUtils, bfaCommonUtils, ShlObj;

var
  Log_CS: TRTLCriticalSection;
  Log_CSL: TRTLCriticalSection;

//
//  Name: RunThread
//  Parameters: Queue
//              Log
//  Beschreibung: Erstellt und startet das Log Thread
//  Description: Creates and start the Log Thread
//  Return value: Log Thread
// [] 17.04.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TThreadedLog.RunThread(Queue: TQueue<String>; Log: TbfaLog): TThreadedLog;
var
  thr: TThreadedLog;
begin
  thr := TThreadedLog.Create;
  thr.FreeOnTerminate := False;
  thr.Priority := tpTimeCritical;
  thr.FQueue := Queue;
  thr.FLog := Log;
  thr.FRunning := False;
  thr.FEvent := TEvent.Create(nil, True, False, '');
  Result := thr;
end;

destructor TThreadedLog.Destroy;
begin
  FreeAndNil(FEvent);
  inherited Destroy;
end;

//
//  Name: Execute
//  Parameters: -
//  Beschreibung: Schreibt aus dem Inhalt des Queues in der Logdatei 
//  Description: Writes out the Queue contents into the Logfile 
//  Return value: -
// [] 17.04.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
procedure TThreadedLog.Execute;
begin
  try
    FRunning := True;
    while not Terminated  do
    begin
      try
        if FEvent <> nil then
        begin
          if FEvent.WaitFor(5000) = wrSignaled then
          begin
            FEvent.ResetEvent;
            try
              while (FQueue.Count > 0) and (FLog <> nil) and (not Terminated) do
              begin
                Flog.WriteOutToLog(FQueue.Dequeue);
              end;
            except
              //
            end;
          end;
        end;
      except
        //
      end;
    end;
  finally
    FRunning := False;
  end;
end;

procedure TbfaLog.Info(LogLevel: Integer; Line, ClsName: WideString);
begin
   Write(LogLevel, ltInfo, Line, ClsName);
end;

//
//  Name: Init
//  Parameters: LogDir
//              AdditionalParam
//              LogLevel
//  Beschreibung: Initialisiert das Log Objekt, und einstellt die Logdatei und Loglevel
//  Description: Initialize the Log object and set the Logdir and loglevel
//  Return value: true if everything is set, false by error
// [] 17.04.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaLog.Init(const LogDir, AdditionalParam: WideString; LogLevel: Integer;
  AppName: WideString = ''; devdebug: Boolean = False): WordBool;

  function GetUserTempFolder: string;
  var
    TmpStr: array [0..Max_Path] of Char;
  begin
    if SHGetSpecialFolderPath(0, TmpStr, CSIDL_LOCAL_APPDATA, False) then
      Result := IncludeTrailingBackslash(Trim(TmpStr)) + sSysTemp + '\'
    else
      RaiseLastOSError;
  end;

  procedure CleanOldLogFiles(dir, prefix: string);
  var
    TmpThread: TCleanLogFilesThread;
  begin
    try
       TmpThread := TCleanLogFilesThread.Create(True); // s
       TmpThread.Logdir := dir;
       TmpThread.Logprefix := prefix;

       TmpThread.Resume;
    except

    end;
  end;
var
  log: TextFile;
  LogLine: WideString;
begin
  FDevDebug := devdebug;
  if LogDir = '' then
    FLogDir := BD.EnvTemp + '\' + sSysBfaLOG + '\'
  else
  begin
    if LogDir[length(LogDir)] = '\' then
      FLogDir := LogDir
    else
      FlogDir := LogDir + '\';
  end;
  
  if (FLogDir <> '') then
  begin
    EnterCriticalSection(Log_CS);
    try
      try
        FLogLevel := LogLevel;
        FAdditionalParam := AdditionalParam;
        if not DirectoryExists(FLogDir) then
          CreateDir(FLogDir);
        if AppName = '' then
          AppName := 'ViewerV36';
        FLogFileName := FLogDir + AppName + '_' + FormatDateTime('YYYY-MM-DD', Now);
        if AdditionalParam <> '' then
          FLogFileName := FLogFileName + '_' + AdditionalParam;
        if FLogLevel = llDebug then
          FLogFileName := FLogFileName + '_' + sSysDEBUG;
        FLogFileName := FLogFileName + '.' + sSysLOG;
        AssignFile(log, FLogFileName);
        if FileExists(FLogFileName) then
          Append(log)
        else
          ReWrite(log);
        LogLine := FormatDateTime('YYYY.MM.DD hh:mm:ss.zzz', Now) + '   ' + ltInfo + ' -  +++ START';
        LogLine := LogLine + sSysLogLevel + ': ' + IntToStr(FLogLevel);
        WriteLn(log, LogLine);
        CloseFile(Log);
        FInitialized := True;

        CleanOldLogFiles(FLogDir, AppName + '_');

        Result := FInitialized;
      except
        FInitialized := False;
        Result := False;
      end;
    finally
      LeaveCriticalSection(Log_CS);
    end;
  end
  else
  begin
    FErrorCode := ecLogLogDirIsEmpty;
    FInitialized := False;
    Result := FInitialized;
  end;
end;

procedure TbfaLog.Warning(LogLevel: Integer; Line, ClsName: WideString);
begin
   Write(LogLevel, ltWarning, Line, ClsName);
end;

//
//  Name: Write
//  Parameters: LogLevel
//              LogType
//              Line - Log Zeile
//              ClsName - Klasse Name
//  Beschreibung: Schreibt die Zeile in der Queue
//  Description: Write to the LogQueue
//  Return value: -
// [] 17.04.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
procedure TbfaLog.Write(LogLevel: Integer; const LogType, Line: WideString; ClsName: WideString = '');
var
  LogLine: WideString;
  sLine: string;
begin
  FErrorCode := 0;
  if FInitialized then
  begin
    if (LogLevel <= FLoglevel) then
    begin
      EnterCriticalSection(Log_CS);
      try
        try
          sLine := DupeString(' ', fIndent) + Line;
          if Pos('<sessionid', sLine) > 0 then
            sLine := LeftStr(sLine,Pos('<sessionid', sLine) -1) + RightStr(sLine, Length(sLine) - Pos('</sessionid>', sLine) -11);

          LogLine := FormatDateTime('YYYY.MM.DD hh:mm:ss.zzz',Now) + ' ' + LogType + ' ' + ClsName + ' - ';
          LogLine := LogLine + StringOfChar(' ', 68 - Length(LogLine));
          LogLine := LogLine + sLine;
        except
          FQueue.Enqueue('TbfaLog.Write Fehler bei DupeString');
        end;
        try
          if FQueue <> nil then
            FQueue.Enqueue(LogLine);
        except
          FQueue.Enqueue('TbfaLog.Write Fehler bei Enqueue');
        end;
      finally
        LeaveCriticalSection(Log_CS);
      end;
    end;
  end else
  begin
    FErrorCode := ecLogNotInitialized;
  end;
end;

procedure  TbfaLog.Debug(LogLevel: Integer; Line: WideString; ClsName: WideString = '');
begin
  Write(LogLevel, ltDebug, Line, ClsName);
end;

procedure TbfaLog.DebugBegin(LogLevel: Integer; Line, ClsName: WideString);
begin
   Write(LogLevel, ltDebug, 'Begin ' + Line, ClsName);
   fIndent := fIndent + 2;
end;

procedure TbfaLog.DebugEnd(LogLevel: Integer; Line, ClsName: WideString);
begin
   if fIndent >= 2 then
     fIndent := fIndent -2;
   Write(LogLevel, ltDebug, 'End   ' + Line, ClsName);
end;

procedure TbfaLog.WriteBegin(LogLevel: Integer; LogType, Line, ClsName: WideString);
begin
   Write(LogLevel, LogType, 'Begin ' + Line, ClsName);
   fIndent := fIndent + 2;
end;

procedure TbfaLog.WriteEnd(LogLevel: Integer; LogType, Line, ClsName: WideString);
begin
   if fIndent >= 2 then
     fIndent := fIndent -2;
   Write(LogLevel, LogType, 'End   ' + Line, ClsName);
end;

procedure TbfaLog.Error(LogLevel: Integer; Line, ClsName: WideString);
begin
   Write(LogLevel, ltError, Line, ClsName);
end;

procedure TbfaLog.Exception(LogLevel: Integer; Line, ClsName: WideString);
begin
   Write(LogLevel, ltException, Line, ClsName);
end;

//
//  Name: Create
//  Parameters: -
//  Beschreibung: 
//  Description: 
//  Return value: -
// [] 17.04.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
procedure TbfaLog.ChangeLogLevel(Value: Integer);
begin
  if FlogLevel <> Value then
    FLoglevel := Value;
end;

constructor TbfaLog.Create;
begin
  inherited Create;
  FLogDir := '';
  FLogFileName := '';
  FLogLevel := llNormal;
  FErrorCode := -1;
  FInitialized := False;
  FQueue := TQueue<String>.Create;
  FQueue.OnNotify := QueueChanged;
  FWriteLogThread := FWriteLogThread.RunThread(FQueue, Self);
end;

destructor TbfaLog.Destroy;
var
  i: Integer;
begin
  try
    i := 0;
    while (FQueue.Count > 0) and (i < 10) do //06.09.2017 Tbfaapplication.Destroy
    begin
      inc(i);
      Sleep(10); //06.09.2017 Tbfaapplication.Destroy
    end;
    FWriteLogThread.Terminate;
  //  FWriteLogThread.Free;
    FWriteLogThread := nil;
    FreeAndNil(FQueue);
    inherited Destroy;
  except

  end;
end;

//
//  Name: QueueChanged
//  Parameters: Sender
//              Item
//              Action
//  Beschreibung: Wenn der Queue ist gendert, wird das LogThread neustarten
//  Description: If the Queue changed set a new event to restart the LogThread
//  Return value: - 
// [] 17.04.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
procedure TbfaLog.QueueChanged(Sender: TObject; const Item: String; Action: TCollectionNotification);
begin
  if Action = cnAdded then
  begin
    if not FWriteLogThread.Running then
    begin
      FWriteLogThread.Terminate;
//      FWriteLogThread.Free;
      FWriteLogThread := nil;
      FWriteLogThread := FWriteLogThread.RunThread(FQueue,Self);
    end;
    FWriteLogThread.FEvent.SetEvent;
  end;
end;

//
//  Name: WriteOutToLog
//  Parameters: Line
//  Beschreibung: Schreibt den Parameter in der Logdatei aus
//  Description: Write out the parameter to the Logfile
//  Return value: FErrorCode
// [] 17.04.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaLog.WriteOutToLog(Line: WideString): Integer;
var
  log: TextFile;
  LogLine: WideString;
begin
  try
    FErrorCode := 0;
    Result := FErrorCode;
    if FInitialized then
    begin
      EnterCriticalSection(Log_CSL);
      try
        LogLine := Line;
        AssignFile(log, FLogFileName);
        try
          try
            Append(log);
            WriteLn(log, LogLine);
          finally
            CloseFile(log);
            Result := FErrorCode;
          end;
        except
          if not FileExists(FLogFileName) then // if the file not exists create it
          begin
            FErrorCode := ecLogFileNotExists;
            ReWrite(log);
            WriteLn(log, LogLine);
            CloseFile(Log);
          end else
          begin        // if the file exists but cannot write into create a new file
            FErrorCode := ecLogFileNotExistsButException;
            Result := FErrorCode;
            Init(FLogDir, FAdditionalParam, FLogLevel);
            FErrorCode := WriteOutToLog(LogLine);
            Result := FErrorCode;
          end;
        end;
      finally
        LeaveCriticalSection(Log_CSL);
      end;
    end else
    begin
      FErrorCode := ecLogNotInitialized;
      Result := FErrorCode;
    end;
  except
    //
  end;
end;

function TbfaLog.Get_QueueCount: Integer;
begin
  Result := FQueue.Count;
end;

function TbfaLog.FlushAndClone: string;
var
  NewFileName: string;
begin
  try
//18.10.2017    NewFileName := GetEnvironmentVariable('APPDATA') + '\bitfarm-archiv\Bug Reports\' + ExtractFileName(FLogFileName);
    NewFileName := BD.EnvAppdata + '\bitfarm-archiv\Bug Reports\' +
      ExtractFileName(FLogFileName);
    CopyFile(PWideChar(FLogFileName), PWideChar(NewFileName), False);
    Result := NewFileName;
  except
    Result := '';
  end;
end;


{ TCleanLogFilesThread }

constructor TCleanLogFilesThread.Create(CreateSuspended: Boolean; Directory, Prefix: string);
begin
  inherited Create(True);

  Self.Logdir := Directory;
  Self.Logprefix := Prefix;

  Self.Suspended := CreateSuspended;
end;

function SortByDate(aList: TStringList; aIndex1, aIndex2: Integer): Integer;
begin
  Result := Integer(aList.Objects[aIndex1]) - Integer(aList.Objects[aIndex2]);
end;

procedure TCleanLogFilesThread.Execute;
var
  SR: TSearchRec;
  FileList: TStringList;
  code: Integer;
begin
  inherited;

  begin
    try
      try
        FileList := TStringList.Create;
        FileList.Sorted := False;
        FileList.BeginUpdate;
        code := FindFirst(Logdir + Logprefix + '*.log', faAnyFile, SR);
        while (code = 0) do
          begin
            FileList.AddObject(SR.Name, TObject(SR.Time));
            code := FindNext(SR);
          end;
        SysUtils.FindClose(SR);
        FileList.CustomSort(SortByDate);
        FileList.EndUpdate;

        for var I := 0 to FileList.Count - 26 do
        begin
          try
            if FileExists(Logdir + FileList[I]) then
              DeleteFile(PChar(Logdir + FileList[I]));
          except
            //
          end;
        end;
      except
        //
      end;
    finally
      FileList.Free;
    end;
  end;
end;

initialization
  InitializeCriticalSection(Log_CS);
  InitializeCriticalSection(Log_CSL);

  bitfarmLog := TbfaLog.Create;

finalization
  DeleteCriticalSection(Log_CS);
//23.10.2018  DeleteCriticalSection(Log_CSL);
//15.10.2018 mit leerem Importer kommt ntdll.dll Exception, deswegen hier auskommentiert. DeleteCriticalSection(Log_CSL);
end.
