unit bfaOfflineDB;

interface

uses
   gnugettext, Forms, Windows, ShellAPI, Classes, Uni, UniDump, SysUtils, DADump, DALoader, UniLoader;


type
	TOfflineDb = class (TObject)
   fServer, fDB, fUsername, fPassword : string;
   fPort : Integer;
   fBasePath, fPathDB, fPathMyIni, fProfil : string;
   fDBStarted, fDBConnected : Boolean;
	published
   	property Server : string read fServer write fServer;
      property DB : string read fDB write fDB;
      property Username : string read fUsername write fUsername;
      property Password : string read fPassword write fPassword;
      property Port : Integer read fPort write fPort;
      property PathDB : string read fPathDB write fPathDB;
      property Profil : string read fProfil write fProfil;
      property DBStarted : Boolean read fDBStarted;
      property DBConnected : Boolean read fDBConnected;

   public
      Off_Con : TUniConnection;
      Off_Cmd : TUniSQL;
      Off_Q1  : TUniQuery;
      Off_Q2  : TUniQuery;
		  Off_Q3  : TUniQuery;
//      Off_Ctrl : TUniServerControl;
      Off_Dump : TUniDump;
      Off_Loader : TUniLoader;

      constructor Create; overload;
      constructor Create(inipath: string = ''; dbpath: string = ''); overload;

      destructor Destroy; virtual;

      function Connect(Server: string = ''; DB : string = ''; Port: Integer = 0; Username : string = ''; Password: string = ''; AThread: TThread = nil): Boolean;
      function StartDB(Path : string = ''; MyIniFile : string =''; AThread: TThread = nil): Boolean;
		  function CreateDB(Server: string = ''; DB : string = ''; Port: Integer = 0; Username : string = ''; Password: string = ''; AThread: TThread = nil): Boolean;

      function StopDB : Boolean;
      function TableExist(Name:string):Boolean;
end;

implementation

uses
  System.StrUtils, bfaCommonMethods, bfaCommonFiles, bfaCommonDialogs, MainDataModule , bfaCommonDB, bfaCommonUtils,
  IniFiles, bitfarm_enterprise, bfaObjects36, bfaLog
  ;


constructor TOfflineDB.Create;
begin
   inherited Create;
	 Off_Con := TUniConnection.Create(nil);
   Off_Con.ProviderName := 'mysql';
   Off_Cmd := TUniSQL.Create(nil);
   Off_Cmd.Connection := Off_Con;
   Off_Q1 := TUniQuery.Create(nil);
   Off_Q1.Connection := Off_Con;
   Off_Q2 := TUniQuery.Create(nil);
   Off_Q2.Connection := Off_Con;
	 Off_Q3 := TUniQuery.Create(nil);
   Off_Q3.Connection := Off_Con;
//   Off_Ctrl := TUniServerControl.Create(nil);
//   Off_Ctrl.Connection := Off_Con;
   Off_Dump := TUniDump.Create(nil);
   Off_Dump.Connection := Off_Con;

   fPort := 53306;
   fBasePath := BD.ExeDir;
   fPathDB := BD.ExeDir + 'mysql\';
   fPathMyIni := BD.ExeDir + 'mysql\my.ini';
end;

destructor TOfflineDB.Destroy;
begin
	inherited  Destroy;
    if Off_Con.Connected then
    	Off_Con.Close;
end;


function TOfflineDB.Connect(Server: string = ''; DB : string = ''; Port: Integer = 0; Username : string = ''; Password: string = ''; AThread: TThread = nil): Boolean;
begin
  Result := False;
  if Server <> '' then
  	fServer := Server;
  if DB <> '' then
  	fDB := DB;
  if Port <> 0 then
     fPort := Port;
  if Username <> '' then
     fUsername := Username;
  if Password <> '' then
  	fPassword := Password;

  try
    Off_Con.Server := fServer;
    Off_Con.Database := fDB;
    Off_Con.Port := FPort;
    Off_Con.Username := fUsername;
    Off_Con.Password := fPassword;
    Off_Con.ProviderName := 'mysql';
    Off_Con.Connect;
  except
    on e: Exception do
    begin
      BfShowMessage(_('Es konnte keine Verbindung zur Offline-DB hergestellt werden:') + ' '  + e.Message, AThread);
      Exit;
    end;
	end;

  fDBConnected := Off_Con.Connected;
  Result := Off_Con.Connected;

end;

function TOfflineDB.StartDB(Path : string = ''; MyIniFile : string =''; AThread: TThread = nil): Boolean;
var
   rc : Integer;
   timeout : Boolean;
   cnt: Integer;
begin
	Result := False;
   fDBStarted := False;
   if Path <> '' then
      fPathDB := BD.ExeDir + Path;
   if MyIniFile <> '' then
   	  fPathMyIni := MyIniFile
   else
   begin
//      fPathMyIni := ExpandFileName(fPathDB + '\..');
      fPathMyIni := fPathDB + 'my.ini';
   end;

	if not FileExists(fPathDB + 'bin\mysqld.exe') then
   begin
      BfShowMessage(_('Die Offline-MySQL konnte nicht gefunden werden.') + ' ' + fPathDB, AThread);
      Exit;
   end;
   if not FileExists(fPathMyIni) then
   begin
      BfShowMessage(_('Die Offline-MySQL konnte nicht gefunden werden (my.ini).') + ' ' + fPathDB, AThread);
      Exit;
   end;

//   rc := ShellExecute(Application.Handle, nil,
//                         PChar(fPathDB + 'bin\mysqld.exe'), PChar('--defaults-file='+ fPathMyIni), PChar(fPathDB), SW_HIDE);


    rc := ShellExecute_AndWait('', fPathDB + 'bin\mysqld.exe', #34 + '--defaults-file='+ fPathMyIni + #34, fPathDB, SW_HIDE, False);

    cnt := 0;
    timeout := False;
    while ShellExecute_AndWait('', fPathDB + 'bin\mysqladmin.exe', 'ping -u sa -psa -P 53306', fPathDB, SW_HIDE, True) <> 0 do
    begin
      Sleep(500);
      Inc(cnt);
      if cnt > 5 then   // sollte 5sec ergeben 10 * 500ms
      begin
        timeout := True;
        BfShowMessage(_('Timeout beim Start der Offline-MySQL'), AThread);
        Break;
      end;
    end;

   fDBStarted := not timeout;
   Result := fDBStarted;
end;

function TOfflineDB.StopDB : Boolean;
var
   rc : Integer;
   mysqlpath: WideString;
begin
  try
    Result := False;
    if Off_Con <> nil then
      if Off_Con.Connected then
          Off_Con.Close();

    fDBConnected := Off_Con.Connected;

    if FileExists(BD.ExeDir + 'Viewer-Files\mysql\bin\mysqladmin.exe') then
      mysqlpath := BD.ExeDir + 'Viewer-Files\mysql\'
    else
      mysqlpath := fPathDB;

    if FileExists(mysqlpath + 'bin\mysqladmin.exe') then
    begin
      rc := ShellExecute_AndWait('', mysqlpath + 'bin\mysqladmin.exe', 'processlist -u root -pBitfarm2017 -P 53306', mysqlpath, SW_HIDE, True);

      if rc = 0 then
      begin
        rc := ShellExecute_AndWait('', mysqlpath + 'bin\mysqladmin.exe', '-u root -pBitfarm2017 -P 53306 shutdown', mysqlpath, SW_HIDE, True);
        Sleep(3000);
      end;
    end;
  finally
    fDBStarted := False;
    Result := True;
  end;
end;



constructor TOfflineDb.Create(inipath: string = ''; dbpath: string = '');
var
  inifile: TIniFile;
begin
//  inherited;
   fBasePath := '';
	 Off_Con := TUniConnection.Create(nil);
   Off_Con.ProviderName := 'mysql';
   Off_Cmd := TUniSQL.Create(nil);
   Off_Cmd.Connection := Off_Con;
//   Off_Q1 := TUniQuery.Create(nil);
//   Off_Q1.Connection := Off_Con;
   Off_Q2 := TUniQuery.Create(nil);
   Off_Q2.Connection := Off_Con;
	 Off_Q3 := TUniQuery.Create(nil);
   Off_Q3.Connection := Off_Con;
//   Off_Ctrl := TUniServerControl.Create(nil);
//   Off_Ctrl.Connection := Off_Con;
   Off_Dump := TUniDump.Create(nil);
   Off_Dump.Connection := Off_Con;

   fPort := 53306;

   if (inipath <> '') and FileExists(inipath) then
   begin
     inifile := TIniFile.Create(inipath);
     fUsername := Decrypt(inifile.ReadString(sSysConnectionSection, sSysSUUser, ' '), GlobalKey);
     fPassword := Decrypt(inifile.ReadString(sSysConnectionSection, sSysSUPass, ' '), GlobalKey);
     fDB := inifile.ReadString(sSysConnectionSection, sSysDB, '');
     fServer := inifile.ReadString(sSysConnectionSection, sSysServer, '');
     fPort := inifile.ReadInteger(sSysConnectionSection, sSysPort, 3306);
//     ProviderName := ini.ReadString(sSysConnectionSection, sSysDBProvider, 'MySQL');
     fBasePath :=ExtractFilePath(inipath);
   end else if dbpath <> '' then
     fBasePath := dbpath;

   if fBasePath = '' then
    fBasePath := BD.ExeDir;

   fBasePath := IncludeTrailingBackslash(fBasePath);

   fPathDB := fBasePath + 'mysql\';
   fPathMyIni := fBasePath + 'mysql\my.ini';
end;

function TOfflineDB.CreateDB(Server: string = ''; DB : string = ''; Port: Integer = 0; Username : string = ''; Password: string = ''; AThread: TThread = nil): Boolean;
var
  rc, i : Integer;
  sqlfile, sqlfile2 : TStringList;
  sql : string;
begin
  Result := True;
  try
    try
      Off_Cmd.SQL.Text := 'Create DATABASE IF NOT EXISTS ' + DB + ' COLLATE utf8_unicode_ci;';
      Off_Cmd.Execute;

      Off_Cmd.SQL.Text := 'USE ' + DB;
      Off_Cmd.Execute;

      sqlfile := TStringList.Create;
      sqlfile.LoadFromFile(fBasePath +'\create_sys_36.sql');

      Off_Cmd.SQL.Text := sqlfile.Text;
      Off_Cmd.Execute;

      sqlfile.Clear;
      sqlfile.LoadFromFile(fBasePath +'\SearchEngine_36.sql', TEncoding.UTF8);

      sqlfile2 := Split2(sqlfile.Text, '$$');

      for i := 0 to sqlfile2.Count -1 do
      begin
        if Trim(sqlfile2[i]) = 'DELIMITER' then
          continue;
        if Trim(RightStr(sqlfile2[i],10)) = 'DELIMITER' then
          continue;
        if Trim(sqlfile2[i]) = '' then
          continue;

        Off_Cmd.SQL.Text := Trim(sqlfile2[i]);
        try
          Off_Cmd.Execute;
        except
          on e:Exception do
          begin
            bitfarmLog.Write(llNormal, ltException, 'Insert SearchEngine:' + e.Message, ClassName);
            Result := False;
          end;
        end;
      end;

      Off_Cmd.SQL.Text := 'GRANT ALL PRIVILEGES ON `' + DB + '`.* TO ' + #39 + Username +#39+ '@' +#39+ '%' +#39+ ' IDENTIFIED BY ' +#39 + Password + #39;
      Off_Cmd.Execute;

      Off_Cmd.SQL.Text := 'GRANT  CREATE TEMPORARY TABLES, SELECT, EXECUTE ON `' + DB + '`.* TO ' + #39 + 'su' +#39+ '@' +#39+ '%' +#39+ ' IDENTIFIED BY ' +#39 + 'su' + #39;
      Off_Cmd.Execute;

      Off_Cmd.SQL.Text := 'GRANT  SELECT, INSERT ON `' + DB +'`.tmp_fullsql TO ' + #39 + 'su' +#39+ '@' +#39+ '%' +#39+ ' IDENTIFIED BY ' +#39 + 'su' + #39;
      Off_Cmd.Execute;

      Off_Cmd.SQL.Text := 'Flush Privileges;';
      Off_Cmd.Execute;

      fDB := DB;
    except
      on e:Exception do
      begin
        bitfarmLog.Write(llNormal, ltException, 'Fehler beim Anlegen der Offline-Datenbank:' + e.Message, ClassName);
        Result := False;
      end;
    end;
  finally
    FreeAndNil(sqlfile);
    FreeAndNil(sqlfile2);
  end;

end;


function TOfflineDB.TableExist(Name:string):Boolean;
var lmq: TUniQuery;
	LastSQL: string;
begin
   lmq := TUniQuery.Create(nil);
   lmq.Connection := Off_Con;
   try
      // Prfen ob die Tabelle da ist.
      LastSQL:='show tables like "' + Name + '"';
      DbExecSql(lmq,LastSQL);
      Result:=lmq.recordcount > 0;
   finally
      lmq.Free;
   end;
end;




end.

