unit bfaOptions;

interface

uses
  Classes, Generics.Collections, System.Win.Registry, bfaBaseObjects, Windows;

type
  TbfaOption = class(TObject)
  private
    FValue: Variant;
    FKind: TbfaOptionKind;
    FName: WideString;
    FDefaultValue: Variant;
    FModified: Boolean;
    FSectionName: WideString;
    procedure SetValue(Value: Variant);
  public
    procedure SetNewValue(Value: Variant);
    property Value: Variant read FValue write SetValue;
    property Kind: TbfaOptionKind read FKind write FKind;
    property Name: WideString read FName write FName;
    property DefaultValue: Variant read FDefaultValue write FDefaultValue;
    property Modified: Boolean read FModified write FModified;
    property SectionName: WideString read FSectionName write FSectionName;
  end;

  TbfaOptions = class(TObject)
  private
    FOptionsContainer: TList<TbfaOption>;
    FStoreType: TbfaOptionStoreType;
    FReg: TRegistry;
    FRegKey: WideString;
    FIniFile, FGlobalIniFile: WideString;
    FSaveOptions: Boolean;
    procedure SetRegKey(Key: WideString);
    procedure ReadOption(Opt: TbfaOption);
    procedure ReadOptionRegistry(Opt: TbfaOption);
    procedure ReadOptionIni(Opt: TbfaOption);
  public
    constructor Create(StoreTyp: TbfaOptionStoreType = ostRegistry; IniFile: WideString = ''; GlobalIniFile: WideString = ''); virtual;
    destructor Destroy; override;
    function GetOption(OptionName: WideString; Kind: TbfaOptionKind; DefaultValue: Variant; SectionName: WideString = ''; DoSearch: Boolean = True): TbfaOption;
    function Save: Integer; virtual;
    procedure WriteOption(Opt: TbfaOption; ExportFile: WideString = '');
    procedure WriteOptionReg(Opt: TbfaOption);
    procedure WriteOptionIni(Opt: TbfaOption; ExportFile: WideString = '');
    function SetOption(OptionName: WideString; Kind: TbfaOptionKind; Value: Variant; SectionName: WideString = ''): Integer;
    function ClearAll: Integer;
    property OptionsContainer: TList<TbfaOption> read FOptionsContainer;
    property StoryType: TbfaOptionStoreType read FStoreType;
    property RegKey: WideString read FRegKey write SetRegKey;
    property SaveOptions: Boolean read FSaveOptions;
  end;

  TbfaScannerOptions = class(TbfaOptions)
  public
    constructor Create(StoreTyp: TbfaOptionStoreType = ostRegistry; IniPath: WideString = '';
      GlobalIniFile: WideString = ''); override;
    destructor Destroy; override;
  end;

  TbfaScanner = class(TObject)
  private
    FProfileName, FName: WideString;
    FScannerID, FColorMode, FFormat, FDPI, FCompression, FOrientation, FTransfer,
    FFilter, FDeletePageKB, FBitDepth, FApiType: Integer;
    FFrameLeft, FFrameRight, FFrameTop, FFrameBottom,
    FBrightness,
    FBrightnessMin,
    FBrightnessMax,
    FContrast,
    FContrastMin,
    FContrastMax,
    FThreshold,
    FThresholdMin,
    FThresholdMax
    : Double;
    FAdf, FDuplex, FStapel, FUseFrame, FDeletePage, FInverse, FIgnoreTWAINError, FDefault, FGlobal, FCompatibleMode, FScanByID, FScanByName: Boolean;
  public
    constructor Create;
    property ProfileName: WideString read FProfileName write FProfileName;
    property ScannerName: WideString read FName write FName;
    property ScannerID: Integer read FScannerID write FScannerID;
    property ScanByID: Boolean read FScanByID write FScanByID;
    property ScanByName: Boolean read FScanByName write FScanByName;
    property ColorMode: Integer read FColorMode write FColormode;
    property DPI: Integer read FDPI write FDPI;
    property Compression: Integer read FCompression write FCompression;
    property Orientation: Integer read FOrientation write FOrientation;
    property Format: Integer read FFormat write FFormat;
    property ADF: Boolean read FAdf write FAdf;
    property Duplex: Boolean read FDuplex write FDuplex;
    property Transfer: Integer read FTransfer write FTransfer;
    property Filter: Integer read FFilter write FFilter;
    property Brightness: Double read FBrightness write FBrightness;
    property BrightnessMin: Double read FBrightnessMin write FBrightnessMin;
    property BrightnessMax: Double read FBrightnessMax write FBrightnessMax;
    property Contrast: Double read FContrast write FContrast;
    property ContrastMin: Double read FContrastMin write FContrastMin;
    property ContrastMax: Double read FContrastMax write FContrastMax;
    property Threshold: Double read FThreshold write FThreshold;
    property ThresholdMin: Double read FThresholdMin write FThresholdMin;
    property ThresholdMax: Double read FThresholdMax write FThresholdMax;
    property UseFrame: Boolean read FUseFrame write FUseFrame;
    property DeletePage: Boolean read FDeletePage write FDeletePage;
    property Inverse: Boolean read FInverse write FInverse;
    property Stapel: Boolean read FStapel write FStapel;
    property FrameLeft: Double read FFrameLeft write FFrameLeft;
    property FrameRight: Double read FFrameRight write FFrameRight;
    property FrameTop: Double read FFrameTop write FFrameTop;
    property FrameBottom: Double read FFrameBottom write FFrameBottom;
    property DeletePageKB: Integer read FDeletePageKB write FDeletePageKB;
    property BitDepth: Integer read FBitDepth write FBitDepth;
    property IgnoreTWAINError: Boolean read FIgnoreTWAINError write FIgnoreTWAINError;
    property DefaultProfile: Boolean read FDefault write FDefault;
    property GlobalProfile: Boolean read FGlobal write FGlobal;
    property CompatibleMode: Boolean read FCompatibleMode write FCompatibleMode;

    /// <summary>
    ///   Bei ausgeschaltetem Scanner wissen wir sonst nicht um welche API es
    ///   sich bei dem Scanner handelt
    /// </summary>
    /// <value>
    ///   -1 = unbekannt <br />0 = TWAIN <br />1 = WIA <br />
    /// </value>
    property ApiType: Integer read FApiType write FApiType;
  end;

  TbfaScannerProfiles = class(TObject)
  private
    FGlobalScannerOptions, FLocalScannerOptions : TbfaScannerOptions;
    FScannerOptionContainer: TList<TbfaScanner>;
    FDefaultScanner: Integer;
  public
    SaveOptions: Boolean;
    constructor Create(IniPath: WideString = ''); overload;
    destructor Destroy; override;
    function Save: Integer;
    procedure ExportProfiles(FileName: WideString);
    procedure ReadScannerOptions(Ini: WideString; RKey: HKEY; RegKey,
      ScanRegKey: WideString; ScannerOptions: TbfaScannerOptions; Global: Boolean);
    procedure ReadSubReg(SubReg: TRegistry; Scanner: TbfaScanner;
      ScannerOptions: TbfaScannerOptions; i: Integer; Profil: WideString);
    procedure ReadFromIni(Scanner: TbfaScanner; ScannerOptions: TbfaScannerOptions;
      i: Integer);
    property Container: TList<TbfaScanner> read FScannerOptionContainer;
    property DefaultScanner: Integer read FDefaultScanner write FDefaultScanner;
    property LocalScannerOptions: TbfaScannerOptions
      read FLocalScannerOptions write FLocalScannerOptions;
    property GlobalScannerOptions: TbfaScannerOptions
      read FGlobalScannerOptions write FGlobalScannerOptions;
  end;

  TbfaGeneralOptions = record
    DefaultSaveDirectory, TimeSyncServer, DefaultPrinterName, PluginButtons,
    ImportFilter : WideString;

    UseDefaultPrinter, NoAutoHintByAnnos, HintBallons, ShowLinkCount,
    ShowRoot, AutoPreview, NoLoginWindow, MsLogon, TimeSync, AlertWindow,
    ExportPDF, FoundMailClients, OleOutlook, MailTo, ShowDocLinks, ShowDocLinksInTab,
    AutoViewerAtArchiveChange, AutoCheckout, AutoCheckin, DisplayCheckoutAtShutdown,
    NoOCR, LateOCR, ResetFieldsAfterArchiving, CopyGdocidLink, CheckoutNewDocuments,
    PrintAfterArchiving, EnableSymlinks, SymlinkAddfields, ChangePageByEnter,
    DragDropAsWindows, SetAnnoDPI, UseFileNameAsDocName, OverwriteLocalSave,
    RowColorEvenOdd, ComboAutoDown, AlternativeScanMode, ImportColor, ImportHighRes,
    ImportBarcode, CalculateOnSelectedRecordsOnly, NegativeCurrency, CloseBookmarkDialog: Boolean;

    DefaultPrinter, DefaultEmail, DefaultCleanup, PluginTimeOut, Update, StartingArea: Integer;
  end;

  TbfaEditorOptions = record
    ShowViewerThumbnail, ShowImporterThumbnail, ShowWorkplaceThumbnail,
    InvertDirection, InUnDockedPanel, AutoEditMode, DocEditorZoom, DocNoAction,
    ImporterOpenDocInImporter, ImporterInUnDockedPanel, WorkplaceOpenDoc, WorkplaceAutoEditMode,
    WorkplaceOpenDocInWorkplace, WorkplaceInUnDockedPanel, OCRAutoClipbrd,
    OCRAdd, RememberLastTool: Boolean;
    AutoZoom, OCRKey : Integer;
  end;

  TbfaSearchOptions = record
    AutoDisplay, ShowDocLinks, DontResetOnArcSet, FtxPos, ShowAllAddfields,
    FulltextSearch, DocTagsSearch, AnnoTxtSearch, FieldTxtSearch, UseOnlyGeneralSearchField,
    SearchLastYear, SQLFullText, DocModified : Boolean;
    Limit, HistoryBookmarkLimit : Integer;
  end;

  TbfaResubOptions = record
    ResubOnStartup, ResubTimer, ReturnToSender, Emailreminder, NotifyByMail,
    ShowAllAddfields, ColumnTitle, ColumnInfo, ColumnComment, ColumnAlarm,
    ColumnModified, ColumnArchive, ColumnDocDate, ResubFavsInPreview : Boolean;
    ResubTimerInterval : Integer;
  end;

  TbfaAdminOptions = record
    DefaultArchiveGroup, DefaultAddfieldGroup, DefaultStatefieldGroup,
    DefaultResubStatefield, AdminFullscreen: Boolean;
    DefaultArchiveGroupRights : Int64;
    DefaultAddfieldGroupRights, DefaultStatefieldGroupRights : Integer;
    LogLevel: Integer; //22.03.2024
  end;

  TbfaGUIOptions = record
    StoredFloatingPosX, StoredFloatingPosY, StoredFloatingOldPosX,
    StoredFloatingOldPosY, SearchPanelFloatingPosX, SearchPanelFloatingPosY,
    SearchPanelFloatingWidth, SearchPanelFloatingHeight, SearchPanelWidth,
    StoredFloatingWidth, StoredFloatingHeight, StoredFloatingOldWidth,
    StoredFloatingOldHeight: Integer;
    StoredFloatingFullScreen, SearchPanelFloating: Boolean;
  end;

  TbfaSystemOptions = record
    MarkedDocs: string;
  end;

  TbfaPreviewDlgOptions = record
    PageSize, Copies, FileType, EmailType: Integer;
    BorderTop, BorderLeft, BorderBottom, BorderRight, PageHeight, PageWidth: Double;
    Annos, PubAnnos, PrivAnnos, Stationery, Center, Rotate, Sort, Merge: Boolean;
  end;

  TbfaCheckboxDialogOption = record
    DialogName: string;
    ShowDialog: Boolean;
    DialogResultType: Integer;
  end;

  TbfaViewerOptions = class(TbfaOptions)
  private
    FSQLFulltextDefault: Boolean;
    procedure ReadGeneralOptions(RKey: HKEY; RegKey: WideString);
    function SaveGeneralOptions: Integer;
    procedure ReadEditorOptions(RKey: HKEY; RegKey: WideString);
    function SaveEditorOptions: Integer;
    procedure ReadSearchOptions(RKey: HKEY; RegKey: WideString);
    function SaveSearchOptions: Integer;
    procedure ReadGUIOptions;
    procedure ReadPreviewDlgOptions(RKey: HKEY; RegKey: WideString);
    procedure ReadResubOptions(RKey: HKEY; RegKey: WideString);
    procedure ReadAdminOptions;
    procedure ReadCheckboxDialogOptions;
  public
    EditorOptions: TbfaEditorOptions;
    SearchOptions: TbfaSearchOptions;
    GeneralOptions: TbfaGeneralOptions;
    GUIOptions: TbfaGUIOptions;
    SystemOptions: TbfaSystemOptions;
    PreviewDlgOptions: TbfaPreviewDlgOptions;
    ResubOptions: TbfaResubOptions;
    AdminOptions: TbfaAdminOptions;
    CheckboxDialogOptions: string;
    CheckBoxDialogs: array of TbfaCheckboxDialogOption;

    constructor Create(StoreTyp: TbfaOptionStoreType = ostRegistry;
      IniPath: WideString = ''; GlobalIniFile: WideString = '';
      SQLFulltext: Boolean = False); overload;
    procedure Reset;
    procedure Reload;
    procedure ReadSystemOptions;
    function Save: Integer;
    function SaveGUIOptions(SaveTofile: Boolean = False): Integer;
    function SaveSystemOptions(SaveTofile: Boolean = False): Integer;
    function SavePreviewDlgOptions(SaveTofile: Boolean = False): Integer;
    function SaveResubOptions(SaveTofile: Boolean = False): Integer;
    function SaveAdminOptions(SaveTofile: Boolean = False): Integer;
    function SaveCheckboxDialogsOptions(SaveTofile: Boolean = False): Integer;
  end;

implementation

uses
  bfaLog, IniFiles, SysUtils, bfaCommonUtils, Printers,
  gnugettext, Types, StrUtils;

constructor TbfaOptions.Create(StoreTyp: TbfaOptionStoreType = ostRegistry;
  IniFile: WideString = ''; GlobalIniFile: WideString = '');
begin
  bitfarmLog.DebugBegin(llDebug, 'TbfaOptions.Create', ClassName);
  try
    try
      FSaveoptions := False;
      FOptionsContainer := TList<TbfaOption>.Create;
      FStoreType := StoreTyp;
      if FStoreType = ostRegistry then
      begin
        bitfarmLog.Write(llDebug, ltInfo, 'FStoreType = ostRegistry', ClassName);
        FReg := TRegistry.Create;
        FReg.RootKey := HKEY_CURRENT_USER;
        FRegKey := IniFile;
      end
      else if FStoreType = ostIniFile then
      begin
        bitfarmLog.Write(llDebug, ltInfo, 'FStoreType = ostIniFile', ClassName);
        FIniFile := IniFile;
        FGlobalIniFile := GlobalIniFile;
        bitfarmLog.Write(llDebug, ltInfo, 'FIniFile: ' + FIniFile, ClassName);
        bitfarmLog.Write(llDebug, ltInfo, 'FGlobalIniFile: ' + FGlobalIniFile, ClassName);
      end;
    except
      bitfarmLog.Write(llNormal, ltException, 'TbfaOptions.Create', ClassName);
    end;
  finally
    bitfarmLog.DebugEnd(llDebug, 'TbfaOptions.Create', ClassName);
  end;
end;

destructor TbfaOptions.Destroy;
var
  i: Integer;
begin
  bitfarmLog.Debugbegin(llDebug, 'TbfaOptions.Destroy', ClassName);
  try
    for i := 0 to FOptionsContainer.Count - 1 do
    begin
      FOptionsContainer[i].Free;
      FOptionsContainer[i] := nil;
    end;
    FOptionsContainer.Clear;
    bitfarmLog.Write(llDebug, ltInfo, 'FOptionsContainer cleared', ClassName);

    FreeAndNil(FOptionsContainer);
    bitfarmLog.Write(llDebug, ltInfo, 'FOptionsContainer destroyed', ClassName);

    FreeAndNil(FReg);
    bitfarmLog.Write(llDebug, ltInfo, 'FReg destroyed', ClassName);

    inherited Destroy;
  finally
    bitfarmLog.DebugEnd(llDebug, 'TbfaOptions.Destroy', ClassName);
  end;
end;

function TbfaOptions.GetOption(OptionName: WideString; Kind: TbfaOptionKind;
  DefaultValue: Variant; SectionName: WideString = ''; DoSearch: Boolean = True): TbfaOption;
var
  i: Integer;
  found: Boolean;
  Opt: TbfaOption;
begin
  try
    Result := nil;
    found := False;
    if (FOptionsContainer.Count > 0) and (DoSearch) then
    begin
      i := 0;
      while (not found) and (i <= FOptionsContainer.Count - 1) do
      begin
        if (((FStoreType = ostRegistry) and (FOptionsContainer.Items[i].Name = OptionName)) or
          ((FStoreType = ostIniFile) and ((FOptionsContainer.Items[i].Name = OptionName) and
          (FOptionsContainer.Items[i].SectionName = SectionName)))) then
        begin
          Result := FOptionsContainer.Items[i];
          found := True;
        end;
        inc(i);
      end;

    end;

    if (not found) then
    begin
      Opt := TbfaOption.Create;
      Opt.Name := OptionName;
      Opt.Kind := Kind;
      Opt.DefaultValue := DefaultValue;
      Opt.SectionName := SectionName;

      ReadOption(Opt);
      Result := Opt;

      if (Opt <> nil) then
        FOptionsContainer.Add(Opt);
    end;

  except
    Result := nil;
  end;
end;

procedure TbfaOption.SetValue(Value: Variant);
begin
  FValue := Value;
  FModified := False;
end;

procedure TbfaOption.SetNewValue(Value: Variant);
begin
  FValue := Value;
  FModified := True;
end;

function TbfaOptions.SetOption(OptionName: WideString; Kind: TbfaOptionKind; Value: Variant; SectionName: WideString = ''): Integer;
var
  i: Integer;
  found: Boolean;
  Opt: TbfaOption;
begin
  try
    Result := 0;
    found := False;
    if FOptionsContainer.Count > 0 then
    begin
      i := 0;
      while (not found) and (i <= FOptionsContainer.Count - 1) do
      begin
        if (((FStoreType = ostRegistry) and (FOptionsContainer.Items[i].Name = OptionName)) or ((FStoreType = ostIniFile) and ((FOptionsContainer.Items[i].Name = OptionName) and (FOptionsContainer.Items[i].SectionName = SectionName)))) then
        begin
          FOptionsContainer.Items[i].SetNewValue(Value);
          found := True;
        end;
        inc(i);
      end;

    end;

    if (not found) then
    begin
      Opt := TbfaOption.Create;
      Opt.Name := OptionName;
      Opt.Kind := Kind;
      Opt.Value := Value;
      Opt.DefaultValue := Value;
      Opt.SectionName := SectionName;

      WriteOption(Opt);
      if (Opt <> nil) then
        FOptionsContainer.Add(Opt);
    end;

  except
    Result := -1;
  end;
end;

function TbfaOptions.ClearAll: Integer;
var
  i: Integer;
  ini: TIniFile;
begin
  Result := 0;
  try
    if FOptionsContainer.Count > 0 then
    begin
      ini := TIniFile.Create(FIniFile);
      try
        for i := FOptionsContainer.Count - 1 downto 0 do
        begin
          ini.EraseSection(FOptionsContainer[i].SectionName);
          FOptionsContainer[i].Free;
          FOptionsContainer[i] := nil;
          FOptionsContainer.Delete(i);
        end;
      finally
        FreeAndNil(ini);
      end;
    end;
  except
    Result := -1;
  end;
end;

function TbfaOptions.Save: Integer;
var
  i: Integer;
begin
  bitfarmLog.DebugBegin(llDebug, 'TbfaOptions.Save - IniFile: ' + FIniFile, ClassName);
  try
    try
      Result := 0;
      for i := 0 to FOptionsContainer.Count - 1 do
      begin
        if FOptionsContainer.Items[i].Modified then
        begin
          WriteOption(FOptionsContainer.Items[i]);
          FOptionsContainer.Items[i].Modified := False;
        end;
      end;

    except
      Result := -1;
      bitfarmLog.Write(llNormal, ltException, 'TbfaOptions.Save - IniFile: ' + FIniFile +
        '; Counter i: ' + IntToStr(i), ClassName);
    end;
  finally
    bitfarmLog.DebugEnd(llDebug, 'TbfaOptions.Save - IniFile: ' + FIniFile, ClassName);
  end;
end;

procedure TbfaOptions.SetRegKey(Key: WideString);
begin
  try
    FRegKey := Key;
  except
    FRegKey := '';
  end;
end;

procedure TbfaOptions.ReadOption(Opt: TbfaOption);
begin
  if FStoreType = ostRegistry then
    ReadOptionRegistry(Opt)
  else if FStoreType = ostIniFile then
    ReadOptionIni(Opt);
end;

procedure TbfaOptions.ReadOptionRegistry(Opt: TbfaOption);
begin
  try
    with FReg do
    begin
      RootKey := HKEY_CURRENT_USER;
      if OpenKey(FRegKey, False) then
      begin
        if ValueExists(Opt.Name) then
        begin
          if (Opt.Kind = okBoolean) then
            Opt.Value := ReadBool(Opt.Name)
          else if (Opt.Kind = okInteger) then
            Opt.Value := ReadInteger(Opt.Name)
          else if (Opt.Kind = okDouble) then
            Opt.Value := ReadFloat(Opt.Name)
          else if (Opt.Kind = okString) then
            Opt.Value := ReadString(Opt.Name);
        end
        else
          Opt.Value := Opt.DefaultValue;
      end
      else
        Opt.Value := Opt.DefaultValue;
      CloseKey;
    end;
  except
    Opt := nil;
  end;
end;

procedure TbfaOptions.ReadOptionIni(Opt: TbfaOption);
var
  ini: TIniFile;
begin
  try
    if (FGlobalIniFile <> '') and (FileExists(FGlobalIniFile)) then
    begin
      ini := TIniFile.Create(FGlobalIniFile);
      if not ini.ValueExists(Opt.SectionName, Opt.Name) then
        ini := TIniFile.Create(FIniFile);
    end
    else
      ini := TIniFile.Create(FIniFile);
    try

      if (Opt.Kind = okBoolean) then
        Opt.Value := ini.ReadBool(Opt.SectionName, Opt.Name, Opt.DefaultValue)
      else if (Opt.Kind = okInteger) then
        Opt.Value := ini.ReadInteger(Opt.SectionName, Opt.Name, Opt.DefaultValue)
      else if (Opt.Kind = okDouble) then
        Opt.Value := ini.ReadFloat(Opt.SectionName, Opt.Name, Opt.DefaultValue)
      else if (Opt.Kind = okString) then
        Opt.Value := ini.ReadString(Opt.SectionName, Opt.Name, Opt.DefaultValue);
    finally
      ini.Free;
    end;

  except
    Opt := nil;
  end;
end;

procedure TbfaOptions.WriteOption(Opt: TbfaOption; ExportFile: WideString = '');
begin
  if FStoreType = ostRegistry then
    WriteOptionReg(Opt)
  else if FStoreType = ostIniFile then
    WriteOptionIni(Opt, ExportFile);
end;

procedure TbfaOptions.WriteOptionIni(Opt: TbfaOption; ExportFile: WideString = '');
var
  ini: TIniFile;
begin
  try
    if ExportFile = '' then
      ExportFile := Finifile;
    ini := TIniFile.Create(ExportFile);
    try
      if (Opt.Kind = okBoolean) then
        ini.WriteBool(Opt.SectionName, Opt.Name, Opt.Value)
      else if (Opt.Kind = okInteger) then
        ini.WriteInteger(Opt.SectionName, Opt.Name, Opt.Value)
      else if (Opt.Kind = okDouble) then
        ini.WriteFloat(Opt.SectionName, Opt.Name, Opt.Value)
      else if (Opt.Kind = okString) then
        ini.WriteString(Opt.SectionName, Opt.Name, Opt.Value);
    finally
      ini.Free;
    end;
  except

  end;
end;

procedure TbfaOptions.WriteOptionReg(Opt: TbfaOption);
begin
  try
    with FReg do
    begin
      RootKey := HKEY_CURRENT_USER;
      if OpenKey(FRegKey, True) then
      begin
        if (Opt.Kind = okBoolean) then
          WriteBool(Opt.Name, Opt.Value)
        else if (Opt.Kind = okInteger) then
          WriteInteger(Opt.Name, Opt.Value)
        else if (Opt.Kind = okDouble) then
          WriteFloat(Opt.Name, Opt.Value)
        else if (Opt.Kind = okString) then
          WriteString(Opt.Name, Opt.Value);
      end;
      CloseKey;
    end;
  except
  end;
end;

constructor TbfaScannerOptions.Create(StoreTyp: TbfaOptionStoreType = ostRegistry;
  IniPath: WideString = ''; GlobalIniFile: WideString = '');
begin
  inherited Create(ostIniFile, IniPath, GlobalIniFile);
end;

destructor TbfaScannerOptions.Destroy;
begin
  inherited Destroy;
end;

procedure TbfaScannerProfiles.ReadScannerOptions(Ini: WideString; RKey: HKEY;
  RegKey, ScanRegKey: WideString; ScannerOptions : TbfaScannerOptions; Global: Boolean);
var
  i, counter: Integer;
  Scanner: TbfaScanner;
  Reg, ScanReg: TRegistry;
  slProfiles: TStringList;
  str: string;
  iniFile: TIniFile;
begin
  if not FileExists(Ini) then
  begin
    bitfarmLog.Write(llDebug, ltInfo, 'Ini NOT Exists: ' + Ini, ClassName);
    //muss aus der Registry lesen
    SaveOptions := True;
    Reg := TRegistry.Create;
    slProfiles := TStringList.Create;
    try
      bitfarmLog.WriteBegin(llDebug, ltInfo, 'Begin Read Registry', ClassName);
      Reg.RootKey := RKey;
      Reg.OpenKeyReadOnly(RegKey);
      bitfarmLog.Write(llDebug, ltInfo, 'RegKey: ' + RegKey, ClassName);
      Reg.GetKeyNames(slProfiles);
      bitfarmLog.Write(llDebug, ltInfo, 'Profiles: ' + slProfiles.CommaText, ClassName);
      for i := 0 to slProfiles.Count - 1 do
      begin
        ScanReg := TRegistry.Create;
        Scanner := TbfaScanner.Create;
        Scanner.GlobalProfile := Global;
        if Global then
          bitfarmLog.Write(llDebug, ltInfo, 'Profile: Global', ClassName)
        else
          bitfarmLog.Write(llDebug, ltInfo, 'Profile: Local', ClassName);

        try
          with ScanReg do
          begin
            RootKey := Reg.RootKey;
            OpenKeyReadOnly(ScanRegKey + slProfiles[i]);
            bitfarmLog.Write(llDebug, ltInfo, 'OpenKey: ' + ScanRegKey + slProfiles[i], ClassName);

            ReadSubReg(ScanReg, Scanner, ScannerOptions, i, slProfiles[i]);
          end;

          FScannerOptionContainer.Add(Scanner);
          bitfarmLog.Write(llDebug, ltInfo, 'Options added to FScannerOptionContainer', ClassName);

        finally
          FreeAndNil(ScanReg);
        end;
        Reg.CloseKey;
      end;
      bitfarmLog.WriteEND(llDebug, ltInfo, 'END Read Registry', ClassName);
    finally
      FreeAndNil(Reg);
      FreeAndNil(slProfiles);
    end;
  end
  else
  begin
    iniFile := TIniFile.Create(Ini);
    try
      bitfarmLog.Write(llDebug, ltInfo, 'Ini Exists: ' + Ini, ClassName);
      str := iniFile.ReadString(sSysVersion, sSysVersion, '');
      if str = '3.2.0.0' then  // Version 3.5
      begin
        bitfarmLog.Write(llDebug, ltInfo, 'Version35: ' + str, ClassName);
        counter := iniFile.ReadInteger(sSysVersion, 'UserCount', 0);
        bitfarmLog.Write(llDebug, ltInfo, 'UserCount: ' + IntToStr(counter), ClassName);

        if counter > 0 then
        begin
          for i := 0 to counter - 1 do
          begin
            Scanner := TbfaScanner.Create;
            Scanner.GlobalProfile := False;
            bitfarmLog.Write(llDebug, ltInfo, 'ReadFromIni: ' + Ini, ClassName);
            ReadFromIni(Scanner, ScannerOptions, i);
            FScannerOptionContainer.Add(Scanner);
          end;
        end;

        counter := iniFile.ReadInteger(sSysVersion, 'GlobalCount', 0);
        bitfarmLog.Write(llDebug, ltInfo, 'GlobalCount: ' + IntToStr(counter), ClassName);

        if counter > 0 then
        begin
          for i := 0 to counter - 1 do
          begin
            Scanner := TbfaScanner.Create;
            Scanner.GlobalProfile := True;
            bitfarmLog.Write(llDebug, ltInfo, 'ReadFromIni: ' + Ini, ClassName);
            ReadFromIni(Scanner, ScannerOptions, i);
            FScannerOptionContainer.Add(Scanner);
          end;
        end;

      end
      else
      begin
        bitfarmLog.Write(llDebug, ltInfo, 'Version36: ' + str, ClassName);
        counter := iniFile.ReadInteger(sSysSystem, sSysCount, 0);

        if counter > 0 then
        begin
          for i := 0 to counter - 1 do
          begin
            try
              Scanner := TbfaScanner.Create;
              Scanner.GlobalProfile := True;
              ReadFromIni(Scanner, ScannerOptions, i);

              FScannerOptionContainer.Add(Scanner);
            except

            end;

          end;
        end;
      end;
    finally
      FreeAndNil(iniFile);
    end;
  end;
end;

procedure TbfaScannerProfiles.ReadSubReg(SubReg: TRegistry; Scanner: TbfaScanner; ScannerOptions: TbfaScannerOptions; i: Integer; Profil: WideString);
var
  TempTransfer: Integer;
begin
  with SubReg do
  begin
    Scanner.ProfileName := ScannerOptions.GetOption(sSysProfilName, okString, Profil, sSysProfil_ + IntToStr(i)).Value;

    if ValueExists('ScanID') then
      Scanner.ScannerID := ScannerOptions.GetOption(sSysScannerID, okInteger, ReadInteger('ScanID'), sSysProfil_ + IntToStr(i)).Value
    else
      Scanner.ScannerID := ScannerOptions.GetOption(sSysScannerID, okInteger, 0, sSysProfil_ + IntToStr(i)).Value;

    if ValueExists('ScanName') then
      Scanner.ScannerName := ScannerOptions.GetOption(sSysScannerName, okString, ReadString('ScanName'), sSysProfil_ + IntToStr(i)).Value
    else
      Scanner.ScannerName := ScannerOptions.GetOption(sSysScannerName, okString, '', sSysProfil_ + IntToStr(i)).Value;

    if ValueExists('ScanByID') then
      Scanner.ScanByID := ScannerOptions.GetOption(sSysScanByID, okBoolean, ReadBool('ScanByID'), sSysProfil_ + IntToStr(i)).Value
    else
      Scanner.ScanByID := ScannerOptions.GetOption(sSysScanByID, okBoolean, False, sSysProfil_ + IntToStr(i)).Value;

    if Scanner.ScanByID then
      Scanner.ScanByName := False
    else
    begin
      if ValueExists('ScanByName') then
        Scanner.ScanByName := ScannerOptions.GetOption(sSysScanByName, okBoolean, ReadBool('ScanByName'), sSysProfil_ + IntToStr(i)).Value
      else
        Scanner.ScanByName := ScannerOptions.GetOption(sSysScanByName, okBoolean, False, sSysProfil_ + IntToStr(i)).Value;
    end;

    if ValueExists('ColorMode') then
      Scanner.ColorMode := ScannerOptions.GetOption(sSysColorMode, okInteger, ReadInteger('ColorMode'), sSysProfil_ + IntToStr(i)).Value
    else
      Scanner.ColorMode := ScannerOptions.GetOption(sSysColorMode, okInteger, 0, sSysProfil_ + IntToStr(i)).Value;

    if ValueExists('BitDepth') then
      Scanner.BitDepth := ScannerOptions.GetOption(sSysBitDepth, okInteger, ReadInteger('BitDepth'), sSysProfil_ + IntToStr(i)).Value
    else
      Scanner.BitDepth := ScannerOptions.GetOption(sSysBitDepth, okInteger, 0, sSysProfil_ + IntToStr(i)).Value;

    if ValueExists('DPIX') then
      Scanner.DPI := ScannerOptions.GetOption(sSysDPI, okInteger, ReadInteger('DPIX'), sSysProfil_ + IntToStr(i)).Value
    else
      Scanner.DPI := ScannerOptions.GetOption(sSysDPI, okInteger, 300, sSysProfil_ + IntToStr(i)).Value;

    if ValueExists('Transfer') then
      Scanner.Transfer := ScannerOptions.GetOption(sSysTransfer, okInteger, ReadInteger('Transfer'), sSysProfil_ + IntToStr(i)).Value
    else
      Scanner.Transfer := ScannerOptions.GetOption(sSysTransfer, okInteger, 1, sSysProfil_ + IntToStr(i)).Value;

    if ValueExists('BrightCur') then
      Scanner.Brightness := ScannerOptions.GetOption(sSysBrightness, okInteger, ReadFloat('BrightCur'), // ReadBinaryData('BrightCur', BinaryValue, SizeOf(BinaryValue)),
        sSysProfil_ + IntToStr(i)).Value
    else
      Scanner.Brightness := ScannerOptions.GetOption(sSysBrightness, okInteger, 0, sSysProfil_ + IntToStr(i)).Value;

    if ValueExists('BrightMin') then
      Scanner.BrightnessMin := ScannerOptions.GetOption(sSysBrightnessMin, okInteger, ReadFloat('BrightMin'), // ReadBinaryData('BrightCur', BinaryValue, SizeOf(BinaryValue)),
        sSysProfil_ + IntToStr(i)).Value
    else
      Scanner.BrightnessMin := ScannerOptions.GetOption(sSysBrightnessMin, okInteger, -1000, sSysProfil_ + IntToStr(i)).Value;

    if ValueExists('BrightMax') then
      Scanner.BrightnessMax := ScannerOptions.GetOption(sSysBrightnessMax, okInteger, ReadFloat('BrightMax'), // ReadBinaryData('BrightCur', BinaryValue, SizeOf(BinaryValue)),
        sSysProfil_ + IntToStr(i)).Value
    else
      Scanner.BrightnessMax := ScannerOptions.GetOption(sSysBrightnessMax, okInteger, 1000, sSysProfil_ + IntToStr(i)).Value;

    if ValueExists('ContrastCur') then
      Scanner.Contrast := ScannerOptions.GetOption(sSysContrast, okInteger, ReadFloat('ContrastCur'), //ReadBinaryData('ContrastCur', BinaryValue, SizeOf(BinaryValue)),
        sSysProfil_ + IntToStr(i)).Value
    else
      Scanner.Contrast := ScannerOptions.GetOption(sSysContrast, okInteger, 0, sSysProfil_ + IntToStr(i)).Value;

    if ValueExists('ContrastMin') then
      Scanner.ContrastMin := ScannerOptions.GetOption(sSysContrastMin, okInteger, ReadFloat('ContrastMin'), //ReadBinaryData('ContrastMin', BinaryValue, SizeOf(BinaryValue)),
        sSysProfil_ + IntToStr(i)).Value
    else
      Scanner.ContrastMin := ScannerOptions.GetOption(sSysContrastMin, okInteger, -1000, sSysProfil_ + IntToStr(i)).Value;

    if ValueExists('ContrastMax') then
      Scanner.ContrastMax := ScannerOptions.GetOption(sSysContrastMax, okInteger, ReadFloat('ContrastMax'), //ReadBinaryData('ContrastMax', BinaryValue, SizeOf(BinaryValue)),
        sSysProfil_ + IntToStr(i)).Value
    else
      Scanner.ContrastMax := ScannerOptions.GetOption(sSysContrastMax, okInteger, 1000, sSysProfil_ + IntToStr(i)).Value;

    if ValueExists('ThresholdCur') then
      Scanner.Threshold := ScannerOptions.GetOption(sSysThreshold, okInteger, ReadFloat('ThresholdCur'), //ReadBinaryData('ThresholdCur', BinaryValue, SizeOf(BinaryValue)),
        sSysProfil_ + IntToStr(i)).Value
    else
      Scanner.Threshold := ScannerOptions.GetOption(sSysThreshold, okInteger, 0, sSysProfil_ + IntToStr(i)).Value;

    if ValueExists('ThresholdMin') then
      Scanner.ThresholdMin := ScannerOptions.GetOption(sSysThresholdMin, okInteger, ReadFloat('ThresholdMin'), //ReadBinaryData('ThresholdCur', BinaryValue, SizeOf(BinaryValue)),
        sSysProfil_ + IntToStr(i)).Value
    else
      Scanner.ThresholdMin := ScannerOptions.GetOption(sSysThresholdMin, okInteger, 0, sSysProfil_ + IntToStr(i)).Value;

    if ValueExists('ThresholdMax') then
      Scanner.ThresholdMax := ScannerOptions.GetOption(sSysThresholdMax, okInteger, ReadFloat('ThresholdMax'), //ReadBinaryData('ThresholdCur', BinaryValue, SizeOf(BinaryValue)),
        sSysProfil_ + IntToStr(i)).Value
    else
      Scanner.ThresholdMax := ScannerOptions.GetOption(sSysThresholdMax, okInteger, 255, sSysProfil_ + IntToStr(i)).Value;

    if ValueExists('PaperFormat') then
    begin
      Scanner.Orientation := ScannerOptions.GetOption(sSysOrientation, okInteger, ReadInteger('PaperFormat'), sSysProfil_ + IntToStr(i)).Value;
    end
    else
      Scanner.Orientation := ScannerOptions.GetOption(sSysOrientation, okInteger, 0, sSysProfil_ + IntToStr(i)).Value;

    if ValueExists('PaperType') then
    begin
      Scanner.Format := ScannerOptions.GetOption(sSysFormat, okInteger, ReadInteger('PaperType'), sSysProfil_ + IntToStr(i)).Value;
    end
    else
      Scanner.Format := ScannerOptions.GetOption(sSysFormat, okString, 1, sSysProfil_ + IntToStr(i)).Value;

    if ValueExists('Filter') then
    begin
      Scanner.Filter := ScannerOptions.GetOption(sSysFilter, okInteger, ReadInteger('Filter'), sSysProfil_ + IntToStr(i)).Value;

      if Scanner.Filter = -1 then
        Scanner.Filter := 0;
    end
    else
      Scanner.Filter := ScannerOptions.GetOption(sSysFilter, okInteger, 0, sSysProfil_ + IntToStr(i)).Value;

    if ValueExists('Cutted') then
      Scanner.UseFrame := ScannerOptions.GetOption(sSysUseFrame, okBoolean, ReadBool('Cutted'), sSysProfil_ + IntToStr(i)).Value
    else
      Scanner.UseFrame := ScannerOptions.GetOption(sSysUseFrame, okBoolean, False, sSysProfil_ + IntToStr(i)).Value;

    if ValueExists('Cut_Top') then
      Scanner.FrameTop := ScannerOptions.GetOption(sSysFrameTop, okInteger, (ReadFloat('Cut_Top') / Zoll), sSysProfil_ + IntToStr(i)).Value
    else
      Scanner.FrameTop := ScannerOptions.GetOption(sSysFrameTop, okInteger, 0, sSysProfil_ + IntToStr(i)).Value;

    if ValueExists('Cut_Left') then
      Scanner.FrameLeft := ScannerOptions.GetOption(sSysFrameLeft, okInteger, (ReadFloat('Cut_Left') / Zoll), sSysProfil_ + IntToStr(i)).Value
    else
      Scanner.FrameLeft := ScannerOptions.GetOption(sSysFrameLeft, okInteger, 0, sSysProfil_ + IntToStr(i)).Value;

    if ValueExists('Cut_Right') then
      Scanner.FrameRight := ScannerOptions.GetOption(sSysFrameRight, okInteger, (ReadFloat('Cut_Right') / Zoll), sSysProfil_ + IntToStr(i)).Value
    else
      Scanner.FrameRight := ScannerOptions.GetOption(sSysFrameRight, okInteger, 0, sSysProfil_ + IntToStr(i)).Value;

    if ValueExists('Cut_Bottom') then
      Scanner.FrameBottom := ScannerOptions.GetOption(sSysFrameBottom, okInteger, (ReadFloat('Cut_Bottom') / Zoll), sSysProfil_ + IntToStr(i)).Value
    else
      Scanner.FrameBottom := ScannerOptions.GetOption(sSysFrameBottom, okInteger, 0, sSysProfil_ + IntToStr(i)).Value;

    if ValueExists('Opt_ADF') then
      Scanner.ADF := ScannerOptions.GetOption(sSysADF, okBoolean, ReadBool('Opt_ADF'), sSysProfil_ + IntToStr(i)).Value
    else
      Scanner.ADF := ScannerOptions.GetOption(sSysADF, okBoolean, False, sSysProfil_ + IntToStr(i)).Value;

    if ValueExists('Opt_Duplex') then
      Scanner.Duplex := ScannerOptions.GetOption(sSysDuplex, okBoolean, ReadBool('Opt_Duplex'), sSysProfil_ + IntToStr(i)).Value
    else
      Scanner.Duplex := ScannerOptions.GetOption(sSysDuplex, okBoolean, False, sSysProfil_ + IntToStr(i)).Value;

    if ValueExists('Opt_Stacked') then
      Scanner.Stapel := ScannerOptions.GetOption(sSysStapel, okBoolean, ReadBool('Opt_Stacked'), sSysProfil_ + IntToStr(i)).Value
    else
      Scanner.Stapel := ScannerOptions.GetOption(sSysStapel, okBoolean, False, sSysProfil_ + IntToStr(i)).Value;

    if ValueExists('Opt_UseInvers') then
      Scanner.Inverse := ScannerOptions.GetOption(sSysInverse, okBoolean, ReadBool('Opt_UseInvers'), sSysProfil_ + IntToStr(i)).Value
    else
      Scanner.Inverse := ScannerOptions.GetOption(sSysInverse, okBoolean, False, sSysProfil_ + IntToStr(i)).Value;

    if ValueExists('Opt_FiltLeer') then
      Scanner.DeletePage := ScannerOptions.GetOption(sSysDeletePage, okBoolean, ReadBool('Opt_FiltLeer'), sSysProfil_ + IntToStr(i)).Value
    else
      Scanner.DeletePage := ScannerOptions.GetOption(sSysDeletePage, okBoolean, False, sSysProfil_ + IntToStr(i)).Value;

    if ValueExists('Opt_Leerkb') then
      Scanner.DeletePageKB := ScannerOptions.GetOption(sSysDeletePageKB, okInteger, ReadFloat('Opt_Leerkb'),  //ReadBinaryData('Opt_Leerkb', BinaryValue, SizeOf(BinaryValue)),
        sSysProfil_ + IntToStr(i)).Value
    else
      Scanner.DeletePageKB := ScannerOptions.GetOption(sSysDeletePageKB, okInteger, 0, sSysProfil_ + IntToStr(i)).Value;

    if ValueExists('IgnoreTWAINError') then
      Scanner.IgnoreTWAINError := ReadBool('IgnoreTWAINError')
    else
      Scanner.IgnoreTWAINError := False;

    if ValueExists('Compression') then
      Scanner.Compression := ScannerOptions.GetOption(sSysCompression, okInteger, ReadInteger('Compression'), sSysProfil_ + IntToStr(i)).Value
    else
      Scanner.Compression := ScannerOptions.GetOption(sSysCompression, okInteger, 0, sSysProfil_ + IntToStr(i)).Value;
  end;
end;

procedure TbfaScannerProfiles.ReadFromIni(Scanner: TbfaScanner; ScannerOptions: TbfaScannerOptions; i: Integer);
var
  str, Profile: string;

  procedure CheckBrightnessRange;
  var
    dValueTemp: double;
  begin
    if (Scanner.BrightnessMax < Scanner.BrightnessMin) then
    begin
      dValueTemp := Scanner.BrightnessMin;
      Scanner.BrightnessMin := Scanner.BrightnessMax;
      Scanner.BrightnessMax := dValueTemp;
    end;
  end;

begin
  str := ScannerOptions.GetOption(sSysVersion, okString, '', sSysVersion, False).Value;
  if str = '3.2.0.0' then  // Version 3.5
  begin
    bitfarmLog.Write(llDebug, ltInfo, 'ReadFromIni Version35', ClassName);

    if Scanner.GlobalProfile then
      Profile := 'GProfile'
    else
      Profile := 'UProfile';

    Scanner.ProfileName := ScannerOptions.GetOption('Name', okString, '', Profile + IntToStr(i), False).Value;
    Scanner.ScannerName := ScannerOptions.GetOption('ScanName', okString, '', Profile + IntToStr(i), False).Value;
    Scanner.ColorMode := ScannerOptions.GetOption(sSysColorMode, okInteger, 0, Profile + IntToStr(i), False).Value;
    Scanner.DPI := ScannerOptions.GetOption('DPIX', okInteger, 300, Profile + IntToStr(i), False).Value;

    case ScannerOptions.GetOption(sSysCompression, okInteger, 0, Profile + IntToStr(i), False).Value of
      0:
        Scanner.Compression := cmpLZW;
      1:
        Scanner.Compression := cmpLZW;
      2:
        Scanner.Compression := cmpCCITT4;
      3:
        Scanner.Compression := cmpJPG;
      4:
        Scanner.Compression := cmpPackBits;
      5:
        Scanner.Compression := cmpLZW;
    else
      Scanner.Compression := cmpLZW;
    end;

    Scanner.Orientation := ScannerOptions.GetOption('PaperFormat', okInteger, 0, Profile + IntToStr(i), False).Value;
    Scanner.Format := ScannerOptions.GetOption('PaperType', okInteger, 0, Profile + IntToStr(i), False).Value;
    Scanner.ADF := ScannerOptions.GetOption('Opt_ADF', okBoolean, False, Profile + IntToStr(i), False).Value;
    Scanner.Duplex := ScannerOptions.GetOption('Opt_Duplex', okBoolean, False, Profile + IntToStr(i), False).Value;
    Scanner.Transfer := ScannerOptions.GetOption(sSysTransfer, okInteger, 1, Profile + IntToStr(i), False).Value;
    Scanner.Filter := ScannerOptions.GetOption(sSysFilter, okInteger, -1, Profile + IntToStr(i), False).Value;
    Scanner.Brightness := ScannerOptions.GetOption('BrightCur', okInteger, 0, Profile + IntToStr(i), False).Value;
    Scanner.BrightnessMin := ScannerOptions.GetOption('BrightMin', okInteger, -1000, Profile + IntToStr(i), False).Value;
    Scanner.BrightnessMax := ScannerOptions.GetOption('BrightMax', okInteger, 1000, Profile + IntToStr(i), False).Value;
    Scanner.Contrast := ScannerOptions.GetOption('ContrastCur', okInteger, 0, Profile + IntToStr(i), False).Value;
    Scanner.ContrastMin := ScannerOptions.GetOption('ContrastMin', okInteger, -1000, Profile + IntToStr(i), False).Value;
    Scanner.ContrastMax := ScannerOptions.GetOption('ContrastMax', okInteger, 1000, Profile + IntToStr(i), False).Value;
    Scanner.Threshold := ScannerOptions.GetOption('ThresholdCur', okInteger, 0, Profile + IntToStr(i), False).Value;
    Scanner.ThresholdMin := ScannerOptions.GetOption('ThresholdMin', okInteger, 0, Profile + IntToStr(i), False).Value;
    Scanner.ThresholdMax := ScannerOptions.GetOption('ThresholdMax', okInteger, 255, Profile + IntToStr(i), False).Value;
    Scanner.Inverse := ScannerOptions.GetOption('Opt_UseInvers', okBoolean, False, Profile + IntToStr(i), False).Value;
    Scanner.UseFrame := ScannerOptions.GetOption('Cutted', okBoolean, False, Profile + IntToStr(i), False).Value;
    Scanner.DeletePage := ScannerOptions.GetOption('Opt_FiltLeer', okBoolean, False, Profile + IntToStr(i), False).Value;
    Scanner.Stapel := ScannerOptions.GetOption('Opt_Stacked', okBoolean, False, Profile + IntToStr(i), False).Value;
    Scanner.FrameLeft := ScannerOptions.GetOption('Cut_Left', okInteger, 0, Profile + IntToStr(i), False).Value;
    Scanner.FrameRight := ScannerOptions.GetOption('Cut_Right', okInteger, 0, Profile + IntToStr(i), False).Value;
    Scanner.FrameTop := ScannerOptions.GetOption('Cut_Top', okInteger, 0, Profile + IntToStr(i), False).Value;
    Scanner.FrameBottom := ScannerOptions.GetOption('Cut_Bottom', okInteger, 0, Profile + IntToStr(i), False).Value;
    Scanner.DeletePageKB := ScannerOptions.GetOption('Opt_Leerkb', okInteger, 0, Profile + IntToStr(i), False).Value;
  end
  else
  begin
    bitfarmLog.Write(llDebug, ltInfo, 'ReadFromIni Version36', ClassName);

    Scanner.ProfileName := ScannerOptions.GetOption(sSysProfilName, okString, '', sSysProfil_ + IntToStr(i)).Value;
    Scanner.ScannerName := ScannerOptions.GetOption(sSysScannerName, okString, '', sSysProfil_ + IntToStr(i)).Value;
    Scanner.ColorMode := ScannerOptions.GetOption(sSysColorMode, okInteger, 0, sSysProfil_ + IntToStr(i)).Value;
    Scanner.DPI := ScannerOptions.GetOption(sSysDPI, okInteger, 300, sSysProfil_ + IntToStr(i)).Value;
    Scanner.Compression := ScannerOptions.GetOption(sSysCompression, okInteger, 0, sSysProfil_ + IntToStr(i)).Value;
    Scanner.BitDepth := ScannerOptions.GetOption(sSysBitDepth, okInteger, 0, sSysProfil_ + IntToStr(i)).Value;  //13.05.2019 Ticket 19113
    Scanner.Orientation := ScannerOptions.GetOption(sSysOrientation, okInteger, 0, sSysProfil_ + IntToStr(i)).Value;
    Scanner.Format := ScannerOptions.GetOption(sSysFormat, okString, '', sSysProfil_ + IntToStr(i)).Value;
    Scanner.ADF := ScannerOptions.GetOption(sSysADF, okBoolean, False, sSysProfil_ + IntToStr(i)).Value;
    Scanner.Duplex := ScannerOptions.GetOption(sSysDuplex, okBoolean, False, sSysProfil_ + IntToStr(i)).Value;
    Scanner.Transfer := ScannerOptions.GetOption(sSysTransfer, okInteger, 1, sSysProfil_ + IntToStr(i)).Value;
    Scanner.CompatibleMode := ScannerOptions.GetOption(sSysCompatibleMode, okBoolean, False, sSysProfil_ + IntToStr(i)).Value;
    Scanner.Filter := ScannerOptions.GetOption(sSysFilter, okInteger, -1, sSysProfil_ + IntToStr(i)).Value;
    Scanner.Brightness := ScannerOptions.GetOption(sSysBrightness, okInteger, 0, sSysProfil_ + IntToStr(i)).Value;
    Scanner.BrightnessMin := ScannerOptions.GetOption(sSysBrightnessMin, okInteger, -1000, sSysProfil_ + IntToStr(i)).Value;
    Scanner.BrightnessMax := ScannerOptions.GetOption(sSysBrightnessMax, okInteger, 1000, sSysProfil_ + IntToStr(i)).Value;
    Scanner.Contrast := ScannerOptions.GetOption(sSysContrast, okInteger, 0, sSysProfil_ + IntToStr(i)).Value;
    Scanner.ContrastMin := ScannerOptions.GetOption(sSysContrastMin, okInteger, -1000, sSysProfil_ + IntToStr(i)).Value;
    Scanner.ContrastMax := ScannerOptions.GetOption(sSysContrastMax, okInteger, 1000, sSysProfil_ + IntToStr(i)).Value;
    Scanner.Threshold := ScannerOptions.GetOption(sSysThreshold, okInteger, 0, sSysProfil_ + IntToStr(i)).Value;
    Scanner.ThresholdMin := ScannerOptions.GetOption(sSysThresholdMin, okInteger, 0, sSysProfil_ + IntToStr(i)).Value;
    Scanner.ThresholdMax := ScannerOptions.GetOption(sSysThresholdMax, okInteger, 255, sSysProfil_ + IntToStr(i)).Value;
    Scanner.Inverse := ScannerOptions.GetOption(sSysInverse, okBoolean, False, sSysProfil_ + IntToStr(i)).Value;
    Scanner.UseFrame := ScannerOptions.GetOption(sSysUseFrame, okBoolean, False, sSysProfil_ + IntToStr(i)).Value;
    Scanner.DeletePage := ScannerOptions.GetOption(sSysDeletePage, okBoolean, False, sSysProfil_ + IntToStr(i)).Value;
    Scanner.Stapel := ScannerOptions.GetOption(sSysStapel, okBoolean, False, sSysProfil_ + IntToStr(i)).Value;
    Scanner.FrameLeft := ScannerOptions.GetOption(sSysFrameLeft, okInteger, 0, sSysProfil_ + IntToStr(i)).Value;
    Scanner.FrameRight := ScannerOptions.GetOption(sSysFrameRight, okInteger, 0, sSysProfil_ + IntToStr(i)).Value;
    Scanner.FrameTop := ScannerOptions.GetOption(sSysFrameTop, okInteger, 0, sSysProfil_ + IntToStr(i)).Value;
    Scanner.FrameBottom := ScannerOptions.GetOption(sSysFrameBottom, okInteger, 0, sSysProfil_ + IntToStr(i)).Value;
    Scanner.DeletePageKB := ScannerOptions.GetOption(sSysDeletePageKB, okInteger, 0, sSysProfil_ + IntToStr(i)).Value;
    Scanner.DefaultProfile := ScannerOptions.GetOption(sSysDefault, okBoolean, False, sSysProfil_ + IntToStr(i)).Value;
    Scanner.GlobalProfile := ScannerOptions.GetOption(sSysGlobal, okBoolean, False, sSysProfil_ + IntToStr(i)).Value;
    Scanner.IgnoreTWAINError := ScannerOptions.GetOption(sSysIgnoreTWAINError, okBoolean, False, sSysProfil_ + IntToStr(i)).Value;

    // default hier 0 (Twain), NICHT -1, wegen Abwrtskompatibilitt
    Scanner.ApiType := ScannerOptions.GetOption(sSysScannerApiType, okInteger, 0, sSysProfil_ + IntToStr(i)).Value;

    // Ticket 16696:
    // Durch ursprnglichen Initialisierungfehler (vertauschte Min, Max-Werte) knnen die falschen Werte auch in der ini stehen.
    CheckBrightnessRange;
  end;
end;

constructor TbfaScannerProfiles.Create(IniPath: WideString = '');
var
  reg: TRegistry;
begin
  inherited Create;

  SaveOptions := False;

  FScannerOptionContainer := TList<TbfaScanner>.Create;
  FScannerOptionContainer.Clear;

  FDefaultScanner := -1;

  FGlobalScannerOptions := TbfaScannerOptions.Create(ostIniFile, BD.EnvProgramDataPD +
    'bitfarm-archiv\' + IniPath);

  reg := TRegistry.Create;
  try
    reg.RootKey := HKEY_LOCAL_MACHINE;
    bitfarmLog.Write(llDebug, ltInfo, 'Read Global Scanprofiles', ClassName);
    if reg.KeyExists('Software\bitfarm\Archiv_V3\scan\profiles') then
    begin
      bitfarmLog.Write(llDebug, ltInfo, 'Read Global Scanprofiles - HKEY_LOCAL_MACHINE\Software\bitfarm\Archiv_V3\scan\profiles', ClassName);
      ReadScannerOptions(BD.EnvProgramDataPD + 'bitfarm-archiv\' + IniPath,
        HKEY_LOCAL_MACHINE, 'Software\bitfarm\Archiv_V3\scan\profiles',
        'Software\bitfarm\Archiv_V3\scan\profiles\', FGlobalScannerOptions, True)
    end
    else if reg.KeyExists('Software\WOW6432Node\bitfarm\Archiv_V3\scan\profiles') then
    begin
      bitfarmLog.Write(llDebug, ltInfo, 'Read Global Scanprofiles - HKEY_LOCAL_MACHINE\WOW6432Node\Software\bitfarm\Archiv_V3\scan\profiles', ClassName);
      ReadScannerOptions(BD.EnvProgramDataPD + 'bitfarm-archiv\' + IniPath,
        HKEY_LOCAL_MACHINE, 'Software\WOW6432Node\bitfarm\Archiv_V3\scan\profiles',
        'Software\WOW6432Node\bitfarm\Archiv_V3\scan\profiles\', FGlobalScannerOptions, True);
    end
    else
    begin //30.08.2017 frisch 36er ohne Registry
      bitfarmLog.Write(llDebug, ltInfo, 'Read Global Scanprofiles (ini) - ' + FGlobalScannerOptions.FIniFile, ClassName);
      ReadScannerOptions(BD.EnvProgramDataPD + 'bitfarm-archiv\' + IniPath,
        HKEY_LOCAL_MACHINE, 'Software\bitfarm\Archiv_V3\scan\profiles', 'Software\bitfarm\Archiv_V3\scan\profiles', FGlobalScannerOptions, True);
    end;

  finally
    FreeAndNil(reg);
  end;

  FLocalScannerOptions := TbfaScannerOptions.Create(ostIniFile, BD.AppdataBitfarmPd + IniPath);

  reg := TRegistry.Create;
  try
    reg.RootKey := HKEY_CURRENT_USER;
    bitfarmLog.Write(llDebug, ltInfo, 'Read Local Scanprofiles', ClassName);
    if reg.KeyExists('Software\bitfarm\Archiv_V3\scan\profiles') then
    begin
      bitfarmLog.Write(llDebug, ltInfo, 'Read Local Scanprofiles - HKEY_CURRENT_USER\Software\bitfarm\Archiv_V3\scan\profiles', ClassName);
      ReadScannerOptions(BD.AppdataBitfarmPd + IniPath, HKEY_CURRENT_USER,
        'Software\bitfarm\Archiv_V3\scan\profiles',
        'Software\bitfarm\Archiv_V3\scan\profiles\', FLocalScannerOptions, False);
    end
    else if reg.KeyExists('Software\WOW6432Node\bitfarm\Archiv_V3\scan\profiles') then
    begin
      bitfarmLog.Write(llDebug, ltInfo, 'Read Local Scanprofiles - HKEY_CURRENT_USER\WOW6432Node\Software\bitfarm\Archiv_V3\scan\profiles', ClassName);
      ReadScannerOptions(BD.AppdataBitfarmPd + IniPath, HKEY_CURRENT_USER,
        'Software\WOW6432Node\bitfarm\Archiv_V3\scan\profiles',
        'Software\WOW6432Node\bitfarm\Archiv_V3\scan\profiles\', FLocalScannerOptions, False);
    end
    else
    begin //30.08.2017 frisch 36er ohne Registry
      bitfarmLog.Write(llDebug, ltInfo, 'Read Local Scanprofiles (ini) - ' + FLocalScannerOptions.FIniFile, ClassName);
      ReadScannerOptions(BD.AppdataBitfarmPd + IniPath,
        HKEY_CURRENT_USER, 'Software\bitfarm\Archiv_V3\scan\profiles', 'Software\bitfarm\Archiv_V3\scan\profiles\', FLocalScannerOptions, False);
    end;

  finally
    FreeAndNil(reg);
  end;

  if (FDefaultScanner = -1) and (FScannerOptionContainer.Count > 0) then
    FDefaultScanner := 0;

end;

destructor TbfaScannerProfiles.Destroy;
var
  i: Integer;
begin
  bitfarmLog.DebugBegin(llDebug, 'TbfaScannerProfiles.Destroy', ClassName);
  try
    FGlobalScannerOptions.Free;
    FGlobalScannerOptions := nil;
    bitfarmLog.Write(llDebug, ltInfo, 'FGlobalScannerOptions.Free', ClassName);
    FLocalScannerOptions.Free;
    FLocalScannerOptions := nil;
    bitfarmLog.Write(llDebug, ltInfo, 'FLocalScannerOptions.Free', ClassName);

    for i := 0 to FScannerOptionContainer.Count - 1 do
    begin
      FScannerOptionContainer[i].Free;
      FScannerOptionContainer[i] := nil;
    end;
    bitfarmLog.Write(llDebug, ltInfo, 'FScannerOptionContainers Free', ClassName);

    FScannerOptionContainer.Clear;
    FreeAndNil(FScannerOptionContainer);
    bitfarmLog.Write(llDebug, ltInfo, 'FScannerOptionContainer.Free', ClassName);

    inherited Destroy;
  finally
    bitfarmLog.DebugEnd(llDebug, 'TbfaScannerProfiles.Destroy', ClassName);
  end;
end;

procedure TbfaScannerProfiles.ExportProfiles(FileName: WideString);
var
  ExportCount : Integer;
  Opt : TbfaOption;
  ExportOptions : TbfaScannerOptions;

  procedure ExportProf(OptCont: TList<TbfaOption>);
  var
    i, OldCount, OldProfilCount: Integer;
  begin
    for i := 0 to OptCont.Count - 1 do
    begin
      if OptCont.Items[i].FSectionName = sSysVersion then
         Continue;

      if ((ExportCount <> 0) and (OptCont.Items[i].Name = sSysCount)) then
      begin
        OldCount := OptCont.Items[i].Value;
        OptCont.Items[i].Value := OptCont.Items[i].Value + ExportCount;
      end
      else if (ExportCount <> 0) then
      begin
        OldProfilCount := StrToInt(StringReplace(OptCont.Items[i].FSectionName, sSysProfil_, '', [rfReplaceAll]));
        OptCont.Items[i].SectionName := sSysProfil_ + IntToStr(OldProfilCount + ExportCount);
      end;

      ExportOptions.WriteOption(OptCont.Items[i], FileName);

      if ((ExportCount <> 0) and (OptCont.Items[i].Name = sSysCount)) then
        OptCont.Items[i].Value := OldCount
      else if (ExportCount <> 0) then
        OptCont.Items[i].SectionName := sSysProfil_ + IntToStr(OldProfilCount);

    end;
  end;

begin
  Opt := TbfaOption.Create;
  ExportOptions := TbfaScannerOptions.Create(ostIniFile, FileName);
  try
    Opt.Value := '36';
    Opt.Kind := okString;
    Opt.SectionName := sSysSystem;
    Opt.Name := sSysVersion;
    Opt.Modified := True;
    ExportOptions.WriteOption(Opt, FileName);

    ExportCount := 0;
    ExportProf(FGlobalScannerOptions.OptionsContainer);
    ExportCount := FGlobalScannerOptions.GetOption(sSysCount, okInteger, 0, sSysSystem).Value;
    ExportProf(FLocalScannerOptions.OptionsContainer);
    ExportCount := FLocalScannerOptions.GetOption(sSysCount, okInteger, 0, sSysSystem).Value + ExportCount;

    Opt.Value := ExportCount;
    Opt.Kind := okInteger;
    Opt.SectionName := sSysSystem;
    Opt.Name := sSysCount;
    Opt.Modified := True;

    ExportOptions.WriteOption(Opt, FileName);
  finally
    FreeAndNil(Opt);
    FreeAndNil(ExportOptions);
  end;
end;

function TbfaScannerProfiles.Save: Integer;

  procedure SetScannerOptions(ScannerOptions: TbfaScannerOptions; i, profil: Integer);
  begin
    ScannerOptions.SetOption(sSysProfilName, okString, FScannerOptionContainer.Items[i].ProfileName, sSysProfil_ + IntToStr(profil));
    ScannerOptions.SetOption(sSysScannerName, okString, FScannerOptionContainer.Items[i].ScannerName, sSysProfil_ + IntToStr(profil));
    ScannerOptions.SetOption(sSysColorMode, okInteger, FScannerOptionContainer.Items[i].ColorMode, sSysProfil_ + IntToStr(profil));
    ScannerOptions.SetOption(sSysDPI, okInteger, FScannerOptionContainer.Items[i].DPI, sSysProfil_ + IntToStr(profil));
    ScannerOptions.SetOption(sSysBitDepth, okInteger, FScannerOptionContainer.Items[i].BitDepth, sSysProfil_ + IntToStr(profil)); //13.05.2019 Ticket 19113
    ScannerOptions.SetOption(sSysCompression, okInteger, FScannerOptionContainer.Items[i].Compression, sSysProfil_ + IntToStr(profil));
    ScannerOptions.SetOption(sSysOrientation, okInteger, FScannerOptionContainer.Items[i].Orientation, sSysProfil_ + IntToStr(profil));
    ScannerOptions.SetOption(sSysFormat, okString, FScannerOptionContainer.Items[i].Format, sSysProfil_ + IntToStr(profil));
    ScannerOptions.SetOption(sSysADF, okBoolean, FScannerOptionContainer.Items[i].ADF, sSysProfil_ + IntToStr(profil));
    ScannerOptions.SetOption(sSysDuplex, okBoolean, FScannerOptionContainer.Items[i].Duplex, sSysProfil_ + IntToStr(profil));
    ScannerOptions.SetOption(sSysTransfer, okInteger, FScannerOptionContainer.Items[i].Transfer, sSysProfil_ + IntToStr(profil));
    ScannerOptions.SetOption(sSysCompatibleMode, okBoolean, FScannerOptionContainer.Items[i].CompatibleMode, sSysProfil_ + IntToStr(profil));
    ScannerOptions.SetOption(sSysFilter, okInteger, FScannerOptionContainer.Items[i].Filter, sSysProfil_ + IntToStr(profil));
    ScannerOptions.SetOption(sSysBrightness, okInteger, FScannerOptionContainer.Items[i].Brightness, sSysProfil_ + IntToStr(profil));
    ScannerOptions.SetOption(sSysBrightnessMin, okInteger, FScannerOptionContainer.Items[i].BrightnessMin, sSysProfil_ + IntToStr(profil));
    ScannerOptions.SetOption(sSysBrightnessMax, okInteger, FScannerOptionContainer.Items[i].BrightnessMax, sSysProfil_ + IntToStr(profil));
    ScannerOptions.SetOption(sSysContrast, okInteger, FScannerOptionContainer.Items[i].Contrast, sSysProfil_ + IntToStr(profil));
    ScannerOptions.SetOption(sSysContrastMin, okInteger, FScannerOptionContainer.Items[i].ContrastMin, sSysProfil_ + IntToStr(profil));
    ScannerOptions.SetOption(sSysContrastMax, okInteger, FScannerOptionContainer.Items[i].ContrastMax, sSysProfil_ + IntToStr(profil));
    ScannerOptions.SetOption(sSysThreshold, okInteger, FScannerOptionContainer.Items[i].Threshold, sSysProfil_ + IntToStr(profil));
    ScannerOptions.SetOption(sSysThresholdMin, okInteger, FScannerOptionContainer.Items[i].ThresholdMin, sSysProfil_ + IntToStr(profil));
    ScannerOptions.SetOption(sSysThresholdMax, okInteger, FScannerOptionContainer.Items[i].ThresholdMax, sSysProfil_ + IntToStr(profil));
    ScannerOptions.SetOption(sSysInverse, okBoolean, FScannerOptionContainer.Items[i].Inverse, sSysProfil_ + IntToStr(profil));
    ScannerOptions.SetOption(sSysUseFrame, okBoolean, FScannerOptionContainer.Items[i].UseFrame, sSysProfil_ + IntToStr(profil));
    ScannerOptions.SetOption(sSysStapel, okBoolean, FScannerOptionContainer.Items[i].Stapel, sSysProfil_ + IntToStr(profil));
    ScannerOptions.SetOption(sSysDeletePage, okBoolean, FScannerOptionContainer.Items[i].DeletePage, sSysProfil_ + IntToStr(profil));
    ScannerOptions.SetOption(sSysFrameLeft, okInteger, FScannerOptionContainer.Items[i].FrameLeft, sSysProfil_ + IntToStr(profil));
    ScannerOptions.SetOption(sSysFrameTop, okInteger, FScannerOptionContainer.Items[i].FrameTop, sSysProfil_ + IntToStr(profil));
    ScannerOptions.SetOption(sSysFrameRight, okInteger, FScannerOptionContainer.Items[i].FrameRight, sSysProfil_ + IntToStr(profil));
    ScannerOptions.SetOption(sSysFrameBottom, okInteger, FScannerOptionContainer.Items[i].FrameBottom, sSysProfil_ + IntToStr(profil));
    ScannerOptions.SetOption(sSysDeletePageKB, okInteger, FScannerOptionContainer.Items[i].DeletePageKB, sSysProfil_ + IntToStr(profil));
    ScannerOptions.SetOption(sSysIgnoreTWAINError, okInteger, FScannerOptionContainer.Items[i].IgnoreTWAINError, sSysProfil_ + IntToStr(profil));
    ScannerOptions.SetOption(sSysDefault, okBoolean, FScannerOptionContainer.Items[i].DefaultProfile, sSysProfil_ + IntToStr(profil));
    ScannerOptions.SetOption(sSysGlobal, okBoolean, FScannerOptionContainer.Items[i].GlobalProfile, sSysProfil_ + IntToStr(profil));
    ScannerOptions.SetOption(sSysScannerApiType, okInteger, FScannerOptionContainer.Items[i].ApiType, sSysProfil_ + IntToStr(profil));
  end;

var
  i, GlobalCount, LocalCount: Integer;
begin
  bitfarmLog.DebugBegin(llDebug, 'TbfaScannerProfiles.Save', ClassName);
  try
    Result := 0;
    try
      FLocalScannerOptions.ClearAll;
      FGlobalScannerOptions.ClearAll;
      GlobalCount := 0;
      LocalCount := 0;
      for i := 0 to FScannerOptionContainer.Count - 1 do
      begin
        if not FScannerOptionContainer[i].GlobalProfile then
        begin
          SetScannerOptions(FLocalScannerOptions, i, LocalCount);
          Inc(LocalCount);
        end
        else
        begin
          SetScannerOptions(FGlobalScannerOptions, i, GlobalCount);
          Inc(GlobalCount);
        end;
      end;

      FLocalScannerOptions.SetOption(sSysCount, okInteger, LocalCount, sSysSystem);
      Result := FLocalScannerOptions.Save;

      if result = 0 then
      begin
        FGlobalScannerOptions.SetOption(sSysCount, okInteger, GlobalCount, sSysSystem);
        Result := FGlobalScannerOptions.Save;

        if Result <> 0 then
          Result := -3;
      end
      else
        Result := -2;
    except
      Result := -1;
      bitfarmLog.Write(llNormal, ltException, 'TbfaScannerProfiles.Save', ClassName);
    end;
  finally
    bitfarmLog.DebugEnd(llDebug, 'TbfaScannerProfiles.Save', ClassName);
  end;
end;

constructor TbfaScanner.Create;
begin
  FName := '';
  FProfileName := '';
  FScannerID := -1;
  FScanByID := False;
  FScanByName := False;
  FColorMode := -1;
  FDPI := -1;
  FCompression := -1;
  FOrientation := -1;
  FFormat := 1;
  FAdf := False;
  FDuplex := False;
  FTransfer := -1;
  FFilter := -1;
  FBrightness := 0;
  FBrightnessMin := -1000;
  FBrightnessMax := 1000;
  FContrast := 0;
  FContrastMin := -1000;
  FContrastMax := 1000;
  FThreshold := 0;
  FThresholdMin := 0;
  FThresholdMax := 255;
  FStapel := False;
  FUseFrame := False;
  FDeletePage := False;
  FInverse := False;
  FFrameLeft := 0;
  FFrameRight := 0;
  FFrameTop := 0;
  FFrameBottom := 0;
  FDeletePageKB := 0;
  FIgnoreTWAINError := False;
  FDefault := False;
  FGlobal := False;
  FCompatibleMode := False;
  FApiType := -1;
end;

constructor TbfaViewerOptions.Create(StoreTyp: TbfaOptionStoreType = ostRegistry;
  IniPath: WideString = ''; GlobalIniFile: WideString = ''; SQLFulltext: Boolean = False);
begin
  bitfarmLog.DebugBegin(llDebug, 'TbfaViewerOptions.Create', ClassName);
  try
    inherited Create(ostIniFile, IniPath, GlobalIniFile);
    FSQLFulltextDefault := SQLFulltext;
    Reload;
  finally
    bitfarmLog.DebugEnd(llDebug, 'TbfaViewerOptions.Create', ClassName);
  end;
end;

procedure TbfaViewerOptions.Reset;
var
  i: Integer;
begin
  bitfarmLog.DebugBegin(llDebug, 'TbfaViewerOptions.Reset', ClassName);
  try
    try
      for i := OptionsContainer.Count - 1 downto 0 do
      begin
        OptionsContainer[i].Free;
        OptionsContainer[i] := nil;
        OptionsContainer.Delete(i);
      end;
      OptionsContainer.Clear;
    except
      bitfarmLog.Write(llNormal, ltException, 'TbfaViewerOptions.Reset', ClassName);
    end;
  finally
    bitfarmLog.DebugEnd(llDebug, 'TbfaViewerOptions.Reset', ClassName);
  end;
end;

procedure TbfaViewerOptions.Reload;
begin
  bitfarmLog.DebugBegin(llDebug, 'TbfaViewerOptions.Reload', ClassName);
  try
    Reset;
    ReadGeneralOptions(HKEY_CURRENT_USER, 'Software\bitfarm\Archiv_V3\Viewer\Optionen');
    ReadEditorOptions(HKEY_CURRENT_USER, 'Software\bitfarm\Archiv_V3\Viewer\Optionen');
    ReadSearchOptions(HKEY_CURRENT_USER, 'Software\bitfarm\Archiv_V3\Viewer\Optionen');
    ReadResubOptions(HKEY_CURRENT_USER, 'Software\bitfarm\Archiv_V3\Viewer\Optionen');
    ReadAdminOptions;
    ReadGUIOptions;
    ReadSystemOptions;
    ReadPreviewDlgOptions(HKEY_CURRENT_USER, 'Software\bitfarm\Archiv_V3\Viewer\Optionen\Printing');
    ReadCheckboxDialogOptions;
  finally
    bitfarmLog.DebugEnd(llDebug, 'TbfaViewerOptions.Reload', ClassName);
  end;
end;

procedure TbfaViewerOptions.ReadGeneralOptions(RKey: HKEY; RegKey: WideString);
var
  Reg : TRegistry;
  found : Boolean;
  ini : TIniFile;
begin
  bitfarmLog.DebugBegin(llDebug, 'ReadGeneralOptions', ClassName);
  try
    try
      // GPL
//      GeneralOptions.AlertWindow := GetOption(sSysAlertWindow, okBoolean, False, sSysGeneralOptions).Value;
      GeneralOptions.AlertWindow := GetOption(sSysAlertWindow, okBoolean, True, sSysGeneralOptions).Value;
      GeneralOptions.ShowDocLinksInTab := GetOption(sSysShowDocLinksInTab, okBoolean, True, sSysGeneralOptions).Value;
      GeneralOptions.AutoViewerAtArchiveChange := GetOption(sSysAutoViewerAtArchiveChange, okBoolean, False, sSysGeneralOptions).Value;
      GeneralOptions.PluginTimeOut := GetOption(sSysPluginTimeOut, okInteger, 60, sSysGeneralOptions).Value;
      if GeneralOptions.PluginTimeOut < 1 then // SJ 25.10.2021 - man konnte TimeOut auf 0 setzen = immer TimeOut
        GeneralOptions.PluginTimeOut := 1;
      GeneralOptions.EnableSymlinks := GetOption(sSysEnableSymlinks, okBoolean, False, sSysGeneralOptions).Value;
      GeneralOptions.SymlinkAddfields := GetOption(sSysSymlinkAddfields, okBoolean, False, sSysGeneralOptions).Value;
      GeneralOptions.DragDropAsWindows := GetOption(sSysDragDropAsWindows, okBoolean, False, sSysGeneralOptions).Value;
      GeneralOptions.SetAnnoDPI := GetOption(sSysSetAnnoDPI, okBoolean, False, sSysGeneralOptions).Value;
      GeneralOptions.ComboAutoDown := GetOption(sSysComboAutoDown, okBoolean, True, sSysGeneralOptions).Value;
      GeneralOptions.AlternativeScanMode := GetOption(sSysAlternativeScanMode, okBoolean, False, sSysGeneralOptions).Value;
      GeneralOptions.ImportColor := GetOption(sSysImportColor, okBoolean, False, sSysGeneralOptions).Value;
      GeneralOptions.ImportHighRes := GetOption(sSysImportHighRes, okBoolean, False, sSysGeneralOptions).Value;
      GeneralOptions.ImportBarcode := GetOption(sSysImportBarcode, okBoolean, False, sSysGeneralOptions).Value;
      GeneralOptions.CalculateOnSelectedRecordsOnly := GetOption(sSysCalculateOnSelectedRecordsOnly, okBoolean, False, sSysGeneralOptions).Value;
      GeneralOptions.Update := GetOption(sSysUpdate, okInteger, 1, sSysGeneralOptions).Value;
      GeneralOptions.StartingArea := GetOption(sSysStartingArea, okInteger, 1, sSysGeneralOptions).Value;
      GeneralOptions.NegativeCurrency := GetOption(sSysNegativeCurrency, okBoolean, True, sSysGeneralOptions).Value;
      GeneralOptions.CloseBookmarkDialog := GetOption(sSysCloseBookmarkDialog, okBoolean, True, sSysGeneralOptions).Value;

      if FileExists(FIniFile) or FileExists(FGlobalIniFile) then
      begin
        bitfarmLog.Write(llDebug, ltInfo, 'FileExists(FIniFile) or FileExists(FGlobalIniFile)', ClassName);
        bitfarmLog.Write(llDebug, ltInfo, 'FIniFile: ' + FIniFile, ClassName);
        bitfarmLog.Write(llDebug, ltInfo, 'FGlobalIniFile: ' + FGlobalIniFile, ClassName);

        GeneralOptions.DefaultSaveDirectory := GetOption(sSysDefaultSaveDirectory, okString, '', sSysGeneralOptions).Value;
        GeneralOptions.DefaultPrinter := GetOption(sSysDefaultPrinter, okInteger, Printer.PrinterIndex, sSysGeneralOptions).Value;
        GeneralOptions.PluginButtons := GetOption(sSysPluginButtons, okString, '', sSysGeneralOptions).Value;
        GeneralOptions.DefaultPrinterName := GetOption(sSysDefaultPrinterName, okString, _('Windows-Standarddrucker'), sSysGeneralOptions).Value;
        GeneralOptions.UseDefaultPrinter := GetOption(sSysUseDefaultPrinter, okBoolean, True, sSysGeneralOptions).Value;
        GeneralOptions.OverwriteLocalSave := GetOption(sSysOverwriteLocalSave, okBoolean, False, sSysGeneralOptions).Value;
        GeneralOptions.RowColorEvenOdd := GetOption(sSysRowColorEvenOdd, okBoolean, True, sSysGeneralOptions).Value;
        GeneralOptions.NoAutoHintByAnnos := GetOption(sSysNoAutoHintByAnnos, okBoolean, False, sSysGeneralOptions).Value;
        GeneralOptions.HintBallons := GetOption(sSysHintBallons, okBoolean, True, sSysGeneralOptions).Value;
        GeneralOptions.ShowLinkCount := GetOption(sSysShowLinkCount, okBoolean, False, sSysGeneralOptions).Value;
        GeneralOptions.ShowRoot := GetOption(sSysShowRoot, okBoolean, False, sSysGeneralOptions).Value;
        GeneralOptions.AutoPreview := GetOption(sSysAutoPreview, okBoolean, True, sSysGeneralOptions).Value;
        GeneralOptions.NoLoginWindow := GetOption(sSysNoLoginWindow, okBoolean, False, sSysGeneralOptions).Value;
        GeneralOptions.MsLogon := GetOption(sSysMsLogon, okBoolean, False, sSysGeneralOptions).Value;
        GeneralOptions.DefaultEmail := GetOption(sSysDefaultEmail, okInteger, 2, sSysGeneralOptions).Value;
        GeneralOptions.DefaultCleanup := GetOption(sSysDefaultCleanup, okInteger, 4, sSysGeneralOptions).Value;
        GeneralOptions.TimeSyncServer := GetOption(sSysTimeSyncServer, okString, '', sSysGeneralOptions).Value;
        GeneralOptions.TimeSync := GetOption(sSysTimeSync, okBoolean, False, sSysGeneralOptions).Value;
//20.09.2024        GeneralOptions.ExportPDF := GetOption(sSysExportPDF, okBoolean, False, sSysGeneralOptions).Value;
        GeneralOptions.ExportPDF := GetOption(sSysExportPDF, okBoolean, True, sSysGeneralOptions).Value;
        GeneralOptions.OleOutlook := GetOption(sSysOleOutlook, okBoolean, False, sSysGeneralOptions).Value;
        GeneralOptions.MailTo := GetOption(sSysMailTo, okBoolean, False, sSysGeneralOptions).Value;
        GeneralOptions.ShowDocLinks := GetOption(sSysShowDocLinks, okBoolean, True, sSysGeneralOptions).Value;
        GeneralOptions.AutoCheckout := GetOption(sSysAutoCheckout, okBoolean, False, sSysGeneralOptions).Value;
        GeneralOptions.CheckoutNewDocuments := GetOption(sSysCheckoutNewDocuments, okBoolean, True, sSysGeneralOptions).Value; // bfacrm:ticket__show__28712
        GeneralOptions.AutoCheckin := GetOption(sSysAutoCheckin, okBoolean, False, sSysGeneralOptions).Value;
        GeneralOptions.DisplayCheckoutAtShutdown := GetOption(sSysDisplayCheckoutAtShutdown, okBoolean, False, sSysGeneralOptions).Value;
        // Importer
        GeneralOptions.ResetFieldsAfterArchiving := GetOption(sSysResetFieldsAfterArchiving, okBoolean, False, sSysGeneralOptions).Value;
        GeneralOptions.PrintAfterArchiving := GetOption(sSysPrintAfterArchiving, okBoolean, False, sSysGeneralOptions).Value;
        GeneralOptions.LateOCR := GetOption(sSysLateOCR, okBoolean, False, sSysGeneralOptions).Value;
        GeneralOptions.NoOCR := GetOption(sSysNoOCR, okBoolean, False, sSysGeneralOptions).Value;
        GeneralOptions.CopyGdocidLink := GetOption(sSysCopyGdocidLink, okBoolean, False, sSysGeneralOptions).Value;
        GeneralOptions.ImportFilter := GetOption(sSysImportFilter, okString, '', sSysGeneralOptions).Value;
        GeneralOptions.ChangePageByEnter := True;

//10.11.2017 wenn schon installiert war, aber das wert noch nicht vorhanden, dann mssen wir Dateiname als Titel benutzen -->
        found := False;

        if FileExists(FIniFile) then
        begin
          ini := TIniFile.Create(FIniFile);
          try
            found := ini.ValueExists(sSysGeneralOptions, sSysUseFileNameAsDocName)
          finally
            FreeAndNil(ini);
          end;
        end
        else if FileExists(FGlobalIniFile) then
        begin
          ini := TIniFile.Create(FGlobalIniFile);
          try
            found := ini.ValueExists(sSysGeneralOptions, sSysUseFileNameAsDocName)
          finally
            FreeAndNil(ini);
          end;
        end;

        GeneralOptions.UseFileNameAsDocName := GetOption(sSysUseFileNameAsDocName, okBoolean, not found, sSysGeneralOptions).Value;

//10.11.2017 wenn schon installiert war, aber das wert noch nicht vorhanden, dann mssen wir Dateiname als Titel benutzen <--
      end
      else
      begin
        //muss aus der Registry lesen
        bitfarmLog.Write(llDebug, ltInfo, 'Read from Registry', ClassName);

        GeneralOptions.UseFileNameAsDocName := GetOption(sSysUseFileNameAsDocName, okBoolean, True, sSysGeneralOptions).Value; //10.11.2017 bei Upgrade von 3.5 muss false sein!

        FSaveOptions := True;
        Reg := TRegistry.Create;
        try
          with Reg do
          begin
            RootKey := HKEY_CURRENT_USER;
            OpenKey(RegKey, True);
            if ValueExists('standardlgrsave') then
              GeneralOptions.DefaultSaveDirectory := GetOption(sSysDefaultSaveDirectory, okString, ReadString('standardlgrsave'), sSysGeneralOptions).Value
            else
              GeneralOptions.DefaultSaveDirectory := GetOption(sSysDefaultSaveDirectory, okString, '', sSysGeneralOptions).Value;

            if ValueExists('UseStdPrn') then
            begin
              if (ReadInteger('UseStdPrn') = -1) then
              begin
                GeneralOptions.DefaultPrinter := GetOption(sSysDefaultPrinter, okInteger, ReadInteger('UseStdPrn'), sSysGeneralOptions).Value;
                GeneralOptions.UseDefaultPrinter := True;
                GeneralOptions.DefaultPrinterName := GetOption(sSysDefaultPrinterName, okString, _('Windows-Standarddrucker'), sSysGeneralOptions).Value;
              end
              else
              begin
                GeneralOptions.DefaultPrinter := GetOption(sSysDefaultPrinter, okInteger, Printer.PrinterIndex, sSysGeneralOptions).Value;
                GeneralOptions.UseDefaultPrinter := False;
                GeneralOptions.DefaultPrinterName := GetOption(sSysDefaultPrinterName, okString, Printer.Printers[Printer.PrinterIndex], sSysGeneralOptions).Value;
              end;
            end
            else
            begin
              GeneralOptions.DefaultPrinter := GetOption(sSysDefaultPrinter, okInteger, Printer.PrinterIndex, sSysGeneralOptions).Value;
              GeneralOptions.UseDefaultPrinter := False;
              GeneralOptions.DefaultPrinterName := GetOption(sSysDefaultPrinterName, okString, Printer.Printers[Printer.PrinterIndex], sSysGeneralOptions).Value;
            end;

            if ValueExists('NoAutoHintByAnnos') then
              GeneralOptions.NoAutoHintByAnnos := GetOption(sSysNoAutoHintByAnnos, okBoolean, ReadBool('NoAutoHintByAnnos'), sSysGeneralOptions).Value
            else
              GeneralOptions.NoAutoHintByAnnos := GetOption(sSysNoAutoHintByAnnos, okBoolean, False, sSysGeneralOptions).Value;

            if ValueExists('BalloonHints') then
              GeneralOptions.HintBallons := GetOption(sSysHintBallons, okBoolean, ReadBool('BalloonHints'), sSysGeneralOptions).Value
            else
              GeneralOptions.HintBallons := GetOption(sSysHintBallons, okBoolean, True, sSysGeneralOptions).Value;

            if ValueExists('Showlinkcount') then
              GeneralOptions.ShowLinkCount := GetOption(sSysShowLinkCount, okBoolean, ReadBool('Showlinkcount'), sSysGeneralOptions).Value
            else
              // GPL
//              GeneralOptions.ShowLinkCount := GetOption(sSysShowLinkCount, okBoolean, False, sSysGeneralOptions).Value;
              GeneralOptions.ShowLinkCount := GetOption(sSysShowLinkCount, okBoolean, True, sSysGeneralOptions).Value;

            if ValueExists('ShowRoot') then
              GeneralOptions.ShowRoot := GetOption(sSysShowRoot, okBoolean, ReadBool('ShowRoot'), sSysGeneralOptions).Value
            else
              GeneralOptions.ShowRoot := GetOption(sSysShowRoot, okBoolean, False, sSysGeneralOptions).Value;

            if ValueExists('AutoPreView') then
              GeneralOptions.AutoPreview := GetOption(sSysAutoPreview, okBoolean, ReadBool('AutoPreView'), sSysGeneralOptions).Value
            else
              GeneralOptions.AutoPreview := GetOption(sSysAutoPreview, okBoolean, True, sSysGeneralOptions).Value;

            if ValueExists('nologinwindow') then
              GeneralOptions.NoLoginWindow := GetOption(sSysNoLoginWindow, okBoolean, ReadBool('nologinwindow'), sSysGeneralOptions).Value
            else
              GeneralOptions.NoLoginWindow := GetOption(sSysNoLoginWindow, okBoolean, False, sSysGeneralOptions).Value;

            if ValueExists('mslogon') then
              GeneralOptions.MsLogon := GetOption(sSysMsLogon, okBoolean, ReadBool('mslogon'), sSysGeneralOptions).Value
            else
              GeneralOptions.MsLogon := GetOption(sSysMsLogon, okBoolean, False, sSysGeneralOptions).Value;

            if ValueExists('StdMail') then
              GeneralOptions.DefaultEmail := GetOption(sSysDefaultEmail, okInteger, ReadInteger('StdMail'), sSysGeneralOptions).Value
            else
              GeneralOptions.DefaultEmail := GetOption(sSysDefaultEmail, okInteger, 2, sSysGeneralOptions).Value;

            if ValueExists('StdDesp') then
              GeneralOptions.DefaultCleanup := GetOption(sSysDefaultCleanup, okInteger, ReadInteger('StdDesp'), sSysGeneralOptions).Value
            else
              GeneralOptions.DefaultCleanup := GetOption(sSysDefaultCleanup, okInteger, 4, sSysGeneralOptions).Value;

            if ValueExists('bfaTimeServer') then
              GeneralOptions.TimeSyncServer := GetOption(sSysTimeSyncServer, okString, ReadString('bfaTimeServer'), sSysGeneralOptions).Value
            else
              GeneralOptions.TimeSyncServer := GetOption(sSysTimeSyncServer, okString, '', sSysGeneralOptions).Value;

            if ValueExists('SyncTime') then
              GeneralOptions.TimeSync := GetOption(sSysTimeSync, okBoolean, ReadBool('SyncTime'), sSysGeneralOptions).Value
            else
              GeneralOptions.TimeSync := GetOption(sSysTimeSync, okBoolean, False, sSysGeneralOptions).Value;

            if ValueExists('ExportPDF') then
              GeneralOptions.ExportPDF := GetOption(sSysExportPDF, okBoolean, ReadBool('ExportPDF'), sSysGeneralOptions).Value
            else
//20.09.2024              GeneralOptions.ExportPDF := GetOption(sSysExportPDF, okBoolean, False, sSysGeneralOptions).Value;
              GeneralOptions.ExportPDF := GetOption(sSysExportPDF, okBoolean, True, sSysGeneralOptions).Value;

            if ValueExists('OleOutlook') then
              GeneralOptions.OleOutlook := GetOption(sSysOleOutlook, okBoolean, ReadBool('OleOutlook'), sSysGeneralOptions).Value
            else
              GeneralOptions.OleOutlook := GetOption(sSysOleOutlook, okBoolean, False, sSysGeneralOptions).Value;

             if ValueExists('MailTo') then
              GeneralOptions.MailTo := GetOption(sSysMailTo, okBoolean, ReadBool('MailTo'), sSysGeneralOptions).Value
            else
              GeneralOptions.MailTo := GetOption(sSysMailTo, okBoolean, False, sSysGeneralOptions).Value;

            if ValueExists('ActDocLinkList') then
              GeneralOptions.ShowDocLinks := GetOption(sSysShowDocLinks, okBoolean, ReadBool('ActDocLinkList'), sSysGeneralOptions).Value
            else
              GeneralOptions.ShowDocLinks := GetOption(sSysShowDocLinks, okBoolean, True, sSysGeneralOptions).Value;

            if ValueExists('AutoCheckout') then
              GeneralOptions.AutoCheckout := GetOption(sSysAutoCheckout, okBoolean, ReadBool('AutoCheckout'), sSysGeneralOptions).Value
            else
              GeneralOptions.AutoCheckout := GetOption(sSysAutoCheckout, okBoolean, False, sSysGeneralOptions).Value;

            if ValueExists('DisplayCheckoutAtShutdown') then
              GeneralOptions.DisplayCheckoutAtShutdown := GetOption(sSysDisplayCheckoutAtShutdown, okBoolean, ReadBool('DisplayCheckoutAtShutdown'), sSysGeneralOptions).Value
            else
              GeneralOptions.DisplayCheckoutAtShutdown := GetOption(sSysDisplayCheckoutAtShutdown, okBoolean, False, sSysGeneralOptions).Value;

            if ValueExists(sSysPluginButtons) then
              GeneralOptions.PluginButtons := GetOption(sSysPluginButtons, okString, ReadString(sSysPluginButtons), sSysGeneralOptions).Value
            else
              GeneralOptions.PluginButtons := GetOption(sSysPluginButtons, okString, '', sSysGeneralOptions).Value;

            GeneralOptions.ChangePageByEnter := True;

            // diese Settings standen nie in der Registry. Also default, bis es eine ini gibt...
            GeneralOptions.CheckoutNewDocuments := True;
            GeneralOptions.AutoCheckin := False;
            GeneralOptions.RowColorEvenOdd := True;
            GeneralOptions.OverwriteLocalSave := False;

            CloseKey;

            RootKey := RKey;
            RegKey := 'Software\bitfarm\Archiv_V3\ManuScan3.1\Optionen';
            OpenKey(RegKey, True);

            if ValueExists('Resetzusatzfields') then
              GeneralOptions.ResetFieldsAfterArchiving := GetOption(sSysResetFieldsAfterArchiving, okBoolean, ReadBool('Resetzusatzfields'), sSysGeneralOptions).Value
            else
              GeneralOptions.ResetFieldsAfterArchiving := GetOption(sSysResetFieldsAfterArchiving, okBoolean, False, sSysGeneralOptions).Value;

            if ValueExists('pnachArc') then
              GeneralOptions.PrintAfterArchiving := GetOption(sSysPrintAfterArchiving, okBoolean, ReadBool('pnachArc'), sSysGeneralOptions).Value
            else
              GeneralOptions.PrintAfterArchiving := GetOption(sSysPrintAfterArchiving, okBoolean, False, sSysGeneralOptions).Value;

            if ValueExists('fastpath') then
              GeneralOptions.LateOCR := GetOption(sSysLateOCR, okBoolean, ReadBool('fastpath'), sSysGeneralOptions).Value
            else
              GeneralOptions.LateOCR := GetOption(sSysLateOCR, okBoolean, False, sSysGeneralOptions).Value;

            if ValueExists('noocr') then
              GeneralOptions.NoOCR := GetOption(sSysNoOCR, okBoolean, ReadBool('noocr'), sSysGeneralOptions).Value
            else
              GeneralOptions.NoOCR := GetOption(sSysNoOCR, okBoolean, False, sSysGeneralOptions).Value;

            if ValueExists('cpclipbrdgdocid') then
              GeneralOptions.CopyGdocidLink := GetOption(sSysCopyGdocidLink, okBoolean, ReadBool('cpclipbrdgdocid'), sSysGeneralOptions).Value
            else
              GeneralOptions.CopyGdocidLink := GetOption(sSysCopyGdocidLink, okBoolean, False, sSysGeneralOptions).Value;

            if ValueExists('JobFilter') then
              GeneralOptions.ImportFilter := GetOption(sSysImportFilter, okString, ReadString('JobFilter'), sSysGeneralOptions).Value
            else
              GeneralOptions.ImportFilter := GetOption(sSysImportFilter, okString, '', sSysGeneralOptions).Value;

            CloseKey;
          end;
        finally
          FreeAndNil(Reg);
        end;
      end;

      Reg := TRegistry.Create;
      try
        try
          Reg.RootKey := HKEY_LOCAL_MACHINE;
          Reg.OpenKey('SOFTWARE\Clients', false);
          GeneralOptions.FoundMailClients := Reg.KeyExists('Mail');
        except
          GeneralOptions.FoundMailClients := false;
        end;
      finally
        Reg.Free;
      end;
    except
      bitfarmLog.Write(llNormal, ltException, 'ReadGeneralOptions', ClassName);
    end;
  finally
    bitfarmLog.DebugBegin(llDebug, 'ReadGeneralOptions', ClassName);
  end;
end;

procedure TbfaViewerOptions.ReadAdminOptions;
var
  DefaultRights: string;
begin
  AdminOptions.DefaultArchiveGroup := GetOption(sSysDefaultArchiveGroup, okBoolean, False, sSysAdminOptions).Value;
  AdminOptions.DefaultAddfieldGroup := GetOption(sSysDefaultAddfieldGroup, okBoolean, True, sSysAdminOptions).Value;
  AdminOptions.DefaultStatefieldGroup := GetOption(sSysDefaultStatefieldGroup, okBoolean, True, sSysAdminOptions).Value;
  AdminOptions.DefaultResubStatefield := GetOption(sSysDefaultResubStatefield, okBoolean, True, sSysAdminOptions).Value;
  AdminOptions.AdminFullscreen := GetOption(sSysAdminFullscreen, okBoolean, False, sSysAdminOptions).Value;
  AdminOptions.LogLevel := GetOption(sSysLogLevel, okInteger, llNormal, sSysAdminOptions).Value;

  DefaultRights := GetOption(sSysDefaultArchiveGroupRights, okString, '1077247', sSysAdminOptions).Value;

  AdminOptions.DefaultArchiveGroupRights := StrToInt64(DefaultRights);
  AdminOptions.DefaultAddfieldGroupRights := GetOption(sSysDefaultAddfieldGroupRights, okInteger, 4, sSysAdminOptions).Value;
  AdminOptions.DefaultStatefieldGroupRights := GetOption(sSysDefaultStatefieldGroupRights, okInteger, 4, sSysAdminOptions).Value;

end;

procedure TbfaViewerOptions.ReadCheckboxDialogOptions;
var
  sList, sList2: TStringDynArray;
  i: Integer;
begin
  try
    SetLength(CheckBoxDialogs, 0);
    CheckboxDialogOptions :=
      GetOption(sSysCheckboxDialogs, okString, '', sSysSystemOptions).Value;

    sList := SplitString(CheckboxDialogOptions, ';');
    for i := Low(sList) to High(sList) - 1 do
    begin
      sList2 := SplitString(sList[i], ',');
      SetLength(CheckBoxDialogs, Length(CheckBoxDialogs) + 1);
      CheckBoxDialogs[i].DialogName := sList2[0];
      CheckBoxDialogs[i].ShowDialog := StrToBool(sList2[1]);
      CheckBoxDialogs[i].DialogResultType := StrToInt(sList2[2]);
    end;

  except

  end;
end;

procedure TbfaViewerOptions.ReadEditorOptions(RKey: HKEY; RegKey: WideString);
var
  Reg: TRegistry;
begin
  try
    EditorOptions.ShowViewerThumbnail := GetOption(sSysShowViewerThumbnail, okBoolean, True, sSysEditorOptions).Value;
    EditorOptions.ShowImporterThumbnail := GetOption(sSysShowImporterThumbnail, okBoolean, True, sSysEditorOptions).Value;
    EditorOptions.ShowWorkplaceThumbnail := GetOption(sSysShowWorkplaceThumbnail, okBoolean, True, sSysEditorOptions).Value;
    EditorOptions.InUnDockedPanel := GetOption(sSysInUnDockedPanel, okBoolean, True, sSysEditorOptions).Value;
    EditorOptions.AutoEditMode := GetOption(sSysAutoEditMode, okBoolean, False, sSysEditorOptions).Value;
    EditorOptions.DocNoAction := GetOption(sSysDocNoAction, okBoolean, False, sSysEditorOptions).Value;
    EditorOptions.ImporterInUnDockedPanel := GetOption(sSysImporterInUnDockedPanel, okBoolean, True, sSysEditorOptions).Value;
    EditorOptions.ImporterOpenDocInImporter := GetOption(sSysImporterOpenDocInImporter, okBoolean, False, sSysEditorOptions).Value;
    EditorOptions.WorkplaceAutoEditMode := GetOption(sSysWorkplaceAutoEditMode, okBoolean, False, sSysEditorOptions).Value;
    EditorOptions.WorkplaceInUnDockedPanel := GetOption(sSysWorkplaceInUnDockedPanel, okBoolean, True, sSysEditorOptions).Value;
    EditorOptions.WorkplaceOpenDocInWorkplace := GetOption(sSysWorkplaceOpenDocInWorkplace, okBoolean, False, sSysEditorOptions).Value;
//    EditorOptions.WorkplaceOpenDoc := GetOption(sSysWorkplaceOpenDoc, okBoolean, True, sSysEditorOptions).Value;
//  Weil WorkplaceAutoEditMode jetzt Checkbox ist, muss es einer der Radioboxen sein
    EditorOptions.WorkplaceOpenDoc := (not EditorOptions.WorkplaceOpenDocInWorkplace) or GetOption(sSysWorkplaceOpenDoc, okBoolean, True, sSysEditorOptions).Value;
    EditorOptions.RememberLastTool :=  GetOption(sSysRememberLastTool, okBoolean, False, sSysEditorOptions).Value;

    if FileExists(FIniFile) or FileExists(FGlobalIniFile) then
    begin
      EditorOptions.InvertDirection := GetOption(sSysInvertDirection, okBoolean, False, sSysEditorOptions).Value;
      // Achtung: es gibt .Zoom und .Autozoom
      // .Zoom ist die zuletzt gesetzte AutoZoom Einstellung
      // .AutoZoom ist die in der OptionsForm (ViewerOptions.ini) gespeicherte Einstellung, welche beim Start des Viewers gesetzt wird
      EditorOptions.AutoZoom := GetOption(sSysAutoZoom, okInteger, 2, sSysEditorOptions).Value;
      EditorOptions.OCRAutoClipbrd := GetOption(sSysOCRAutoClipbrd, okBoolean, False, sSysEditorOptions).Value;
      EditorOptions.OCRAdd := GetOption(sSysOCRAdd, okBoolean, False, sSysEditorOptions).Value;
      EditorOptions.OcrKey := GetOption(sSysOcrKey, okInteger, 16473, sSysEditorOptions).Value;
      EditorOptions.DocEditorZoom := GetOption(sSysDocEditorZoom, okBoolean, True, sSysEditorOptions).Value;
    end
    else
    begin
    //  Achtung:
    //  Dieser ganze Bereich wird nur gelesen, wenn keine INI vorliegt !!!
      // GPL
//      EditorOptions.DocEditorZoom := GetOption(sSysDocEditorZoom, okBoolean, False, sSysEditorOptions).Value;
      EditorOptions.DocEditorZoom := GetOption(sSysDocEditorZoom, okBoolean, True, sSysEditorOptions).Value;

      //muss aus der Registry lesen
      FSaveOptions := True;
      Reg := TRegistry.Create;
      try
        with Reg do
        begin
          RootKey := RKey;
          OpenKey(RegKey, True);
          if ValueExists('MouseRadDirection') then
          begin
            EditorOptions.InvertDirection := GetOption(sSysInvertDirection, okBoolean, ReadBool('MouseRadDirection'), sSysEditorOptions).Value;

            EditorOptions.InvertDirection := not EditorOptions.InvertDirection;
              // weil im 3.6 ist es im umprogramiert, genau umgekehrt, wie im 3.5
          end
          else
            EditorOptions.InvertDirection := GetOption(sSysInvertDirection, okBoolean, False, sSysEditorOptions).Value;

          if ValueExists('Azoom') then
            EditorOptions.AutoZoom:= GetOption(sSysAutoZoom, okInteger, ReadInteger('Azoom'), sSysEditorOptions).Value
          else
            EditorOptions.AutoZoom := GetOption(sSysAutoZoom, okInteger, 2, sSysEditorOptions).Value;

          if ValueExists('OCRClip') then
            EditorOptions.OCRAutoClipbrd := GetOption(sSysOCRAutoClipbrd, okBoolean, ReadBool('OCRClip'), sSysEditorOptions).Value
          else
            EditorOptions.OCRAutoClipbrd := GetOption(sSysOCRAutoClipbrd, okBoolean, False, sSysEditorOptions).Value;

          if ValueExists('OcrAdd') then
            EditorOptions.OCRAdd := GetOption(sSysOCRAdd, okBoolean, ReadBool('OcrAdd'), sSysEditorOptions).Value
          else
            EditorOptions.OCRAdd := GetOption(sSysOCRAdd, okBoolean, False, sSysEditorOptions).Value;

          if ValueExists('OcrKey') then
            EditorOptions.OcrKey := GetOption(sSysOcrKey, okInteger, ReadInteger('OcrKey'), sSysEditorOptions).Value
          else
            EditorOptions.OcrKey := GetOption(sSysOcrKey, okInteger, 16473, sSysEditorOptions).Value;

          CloseKey;
        end;
      finally
        FreeAndNil(Reg);
      end;

    end;
  except

  end;
end;

procedure TbfaViewerOptions.ReadSearchOptions(RKey: HKEY; RegKey: WideString);
var
  Reg: TRegistry;
  str: string;
begin
  try
    SearchOptions.FulltextSearch := GetOption(sSysFulltextSearch, okBoolean, True, sSysSearchOptions).Value;
    SearchOptions.DocTagsSearch := GetOption(sSysDocTagsSearch, okBoolean, True, sSysSearchOptions).Value;
    SearchOptions.AnnoTxtSearch := GetOption(sSysAnnoTxtSearch, okBoolean, True, sSysSearchOptions).Value;
    SearchOptions.FieldTxtSearch := GetOption(sSysFieldSearch, okBoolean, True, sSysSearchOptions).Value;
    SearchOptions.UseOnlyGeneralSearchField := GetOption(sSysUseOnlyGeneralSearchField, okBoolean, False, sSysSearchOptions).Value;
    SearchOptions.HistoryBookmarkLimit := GetOption(sSysHistoryBookmarkLimit, okInteger, 20, sSysSearchOptions).Value;
    SearchOptions.FtxPos := GetOption(sSysFtxPos, okBoolean, True, sSysSearchOptions).Value;
    SearchOptions.SQLFullText := GetOption(sSysSQLFullText, okBoolean, FSQLFulltextDefault, sSysSearchOptions).Value;
    SearchOptions.DocModified := GetOption(sSysDocModified, okBoolean, False, sSysSearchOptions).Value;

    with SearchOptions do
    begin
      if ((not FulltextSearch) and (not DocTagsSearch) and (not AnnoTxtSearch) and (not FieldTxtSearch)) then
        FulltextSearch := True;
    end;

    if FileExists(FIniFile) or FileExists(FGlobalIniFile) then
    begin
      SearchOptions.Limit := GetOption(sSysLimit, okInteger, 50, sSysSearchOptions).Value;
      SearchOptions.AutoDisplay := GetOption(sSysAutoDisplay, okBoolean, False, sSysSearchOptions).Value;
      SearchOptions.SearchLastYear := GetOption(sSysSearchLastYear, okBoolean, False, sSysSearchOptions).Value;
      SearchOptions.ShowDocLinks := GetOption(sSysShowDocLinks, okBoolean, True, sSysSearchOptions).Value;
      SearchOptions.DontResetOnArcSet := GetOption(sSysDontResetOnArcSet, okBoolean, False, sSysSearchOptions).Value;
      SearchOptions.ShowAllAddfields := GetOption(sSysShowAllAddfields, okBoolean, False, sSysSearchOptions).Value;
    end
    else
    begin
      //muss aus der Registry lesen
      FSaveOptions := True;
      Reg := TRegistry.Create;
      try
        with Reg do
        begin
          RootKey := RKey;
          OpenKey(RegKey, True);
          if ValueExists('Hits') then
          begin
            str := GetOption(sSysLimit, okString, ReadString('Hits'), sSysSearchOptions).Value;
            try
              SearchOptions.Limit := StrToInt(str);
            except
              SearchOptions.Limit := 50;
            end;
          end
          else
            SearchOptions.Limit := GetOption(sSysLimit, okInteger, 50, sSysSearchOptions).Value;

          if ValueExists('AutoDispNew') then
            SearchOptions.AutoDisplay := GetOption(sSysAutoDisplay, okBoolean, ReadBool('AutoDispNew'), sSysSearchOptions).Value
          else
            SearchOptions.AutoDisplay := GetOption(sSysAutoDisplay, okBoolean, False, sSysSearchOptions).Value;

          if ValueExists('SearchLastYear') then
            SearchOptions.SearchLastYear := GetOption(sSysSearchLastYear, okBoolean, ReadBool('SearchLastYear'), sSysSearchOptions).Value
          else
            SearchOptions.SearchLastYear := GetOption(sSysSearchLastYear, okBoolean, False, sSysSearchOptions).Value;

          if ValueExists('DocLinkList') then
            SearchOptions.ShowDocLinks := GetOption(sSysShowDocLinks, okBoolean, ReadBool('DocLinkList'), sSysSearchOptions).Value
          else
            SearchOptions.ShowDocLinks := GetOption(sSysShowDocLinks, okBoolean, True, sSysSearchOptions).Value;

          if ValueExists('DontResetOnArcSet') then
            SearchOptions.DontResetOnArcSet := GetOption(sSysDontResetOnArcSet, okBoolean, ReadBool('DontResetOnArcSet'), sSysSearchOptions).Value
          else
            SearchOptions.DontResetOnArcSet := GetOption(sSysDontResetOnArcSet, okBoolean, False, sSysSearchOptions).Value;

          CloseKey;
        end;
      finally
        FreeAndNil(Reg);
      end;
    end;
  except

  end;
end;

procedure TbfaViewerOptions.ReadGUIOptions;
begin
  try
    GuiOptions.StoredFloatingPosX := GetOption(sSysStoredFloatingPosX, okInteger, 1, sSysGUIOptions).Value;
    GuiOptions.StoredFloatingPosY := GetOption(sSysStoredFloatingPosY, okInteger, 1, sSysGUIOptions).Value;
    GuiOptions.StoredFloatingOldPosX := GetOption(sSysStoredFloatingOldPosX, okInteger, 1, sSysGUIOptions).Value;
    GuiOptions.StoredFloatingOldPosY := GetOption(sSysStoredFloatingOldPosY, okInteger, 1, sSysGUIOptions).Value;
    GuiOptions.SearchPanelFloatingPosX := GetOption(sSysSearchPanelFloatingPosX, okInteger, 1, sSysGUIOptions).Value;
    GuiOptions.SearchPanelFloatingPosY := GetOption(sSysSearchPanelFloatingPosY, okInteger, 1, sSysGUIOptions).Value;
    GuiOptions.SearchPanelFloatingWidth := GetOption(sSysSearchPanelFloatingWidth, okInteger, 300, sSysGUIOptions).Value;
    GuiOptions.SearchPanelFloatingHeight := GetOption(sSysSearchPanelFloatingHeight, okInteger, 300, sSysGUIOptions).Value;
    GuiOptions.StoredFloatingWidth := GetOption(sSysStoredFloatingWidth, okInteger, 300, sSysGUIOptions).Value;
    GuiOptions.StoredFloatingHeight := GetOption(sSysStoredFloatingHeight, okInteger, 300, sSysGUIOptions).Value;
    GuiOptions.StoredFloatingOldWidth := GetOption(sSysStoredFloatingOldWidth, okInteger, 300, sSysGUIOptions).Value;
    GuiOptions.StoredFloatingOldHeight := GetOption(sSysStoredFloatingOldHeight, okInteger, 300, sSysGUIOptions).Value;
    GuiOptions.StoredFloatingFullScreen := GetOption(sSysStoredFloatingFullScreen, okBoolean, False, sSysGUIOptions).Value;
    GuiOptions.SearchPanelFloating := GetOption(sSysSearchPanelFloating, okBoolean, False, sSysGUIOptions).Value;
    GuiOptions.SearchPanelWidth := GetOption(sSysSearchPanelWidth, okInteger, 337, sSysGUIOptions).Value;

  except

  end;
end;

procedure TbfaViewerOptions.ReadSystemOptions;
begin
  try
    SystemOptions.MarkedDocs :=
      GetOption(sSysMarkedDocs, okString,
      '-1,-1;-1,-1;-1,-1;-1,-1;-1,-1;', sSysSystemOptions).Value;
  except

  end;
end;

procedure TbfaViewerOptions.ReadPreviewDlgOptions(RKey: HKEY; RegKey: WideString);
var
  Reg: TRegistry;
  str: string;
begin
  try
    PreviewDlgOptions.PubAnnos := GetOption(sSysPubAnnos, okBoolean, True, sSysPreviewDlgOptions).Value;
    PreviewDlgOptions.PrivAnnos := GetOption(sSysPrivAnnos, okBoolean, False, sSysPreviewDlgOptions).Value;

    if FileExists(FIniFile) or FileExists(FGlobalIniFile) then
    begin
      PreviewDlgOptions.PageSize := GetOption(sSysPageSize, okInteger, 0, sSysPreviewDlgOptions).Value;
      PreviewDlgOptions.PageHeight := GetOption(sSysPageHeight, okDouble, 0.00, sSysPreviewDlgOptions).Value;
      PreviewDlgOptions.PageWidth := GetOption(sSysPageWidth, okDouble, 0.00, sSysPreviewDlgOptions).Value;
      PreviewDlgOptions.BorderLeft := GetOption(sSysBorderLeft, okDouble, 0.00, sSysPreviewDlgOptions).Value;
      PreviewDlgOptions.BorderTop := GetOption(sSysBorderTop, okDouble, 0.00, sSysPreviewDlgOptions).Value;
      PreviewDlgOptions.BorderBottom := GetOption(sSysBorderBottom, okDouble, 0.00, sSysPreviewDlgOptions).Value;
      PreviewDlgOptions.BorderRight := GetOption(sSysBorderRight, okDouble, 0.00, sSysPreviewDlgOptions).Value;

      PreviewDlgOptions.Annos := GetOption(sSysAnnos, okBoolean, False, sSysPreviewDlgOptions).Value;
      PreviewDlgOptions.Stationery := GetOption(sSysStationery, okBoolean, False, sSysPreviewDlgOptions).Value;
      PreviewDlgOptions.Center := GetOption(sSysCenter, okBoolean, False, sSysPreviewDlgOptions).Value;
      PreviewDlgOptions.Rotate := GetOption(sSysRotate, okBoolean, True, sSysPreviewDlgOptions).Value;
      PreviewDlgOptions.Merge := GetOption(sSysMerge, okBoolean, False, sSysPreviewDlgOptions).Value;

      PreviewDlgOptions.Copies := 1;
      PreviewDlgOptions.Sort := GetOption(sSysSort, okBoolean, False, sSysPreviewDlgOptions).Value;
      PreviewDlgOptions.FileType := GetOption(sSysFileType, okInteger, 0, sSysPreviewDlgOptions).Value;
      PreviewDlgOptions.EmailType := GetOption(sSysEmailType, okInteger, 0, sSysPreviewDlgOptions).Value;
    end
    else
    begin
      PreviewDlgOptions.Copies := 1;
      PreviewDlgOptions.EmailType := 0;
      PreviewDlgOptions.FileType := 0;

      //muss aus der Registry lesen
      FSaveOptions := True;
      Reg := TRegistry.Create;
      try
        with Reg do
        begin
          RootKey := RKey;
          OpenKey(RegKey, True);

          if ValueExists('BorTop') then
            PreviewDlgOptions.BorderTop := GetOption(sSysBorderTop, okDouble, ReadFloat('BorTop') * Zoll, sSysPreviewDlgOptions).Value
          else
            PreviewDlgOptions.BorderTop := GetOption(sSysBorderTop, okDouble, 0.00, sSysPreviewDlgOptions).Value;

          if ValueExists('BorBottom') then
            PreviewDlgOptions.BorderBottom := GetOption(sSysBorderBottom, okDouble, ReadFloat('BorBottom') * Zoll, sSysPreviewDlgOptions).Value
          else
            PreviewDlgOptions.BorderBottom := GetOption(sSysBorderBottom, okDouble, 0.00, sSysPreviewDlgOptions).Value;

          if ValueExists('BorLeft') then
            PreviewDlgOptions.BorderLeft := GetOption(sSysBorderLeft, okDouble, ReadFloat('BorLeft') * Zoll, sSysPreviewDlgOptions).Value
          else
            PreviewDlgOptions.BorderLeft := GetOption(sSysBorderLeft, okDouble, 0.00, sSysPreviewDlgOptions).Value;

          if ValueExists('BorRight') then
            PreviewDlgOptions.BorderRight := GetOption(sSysBorderRight, okDouble, ReadFloat('BorRight') * Zoll, sSysPreviewDlgOptions).Value
          else
            PreviewDlgOptions.BorderRight := GetOption(sSysBorderRight, okDouble, 0.00, sSysPreviewDlgOptions).Value;

          if ValueExists('SWidth') then
            PreviewDlgOptions.PageWidth := GetOption(sSysPageWidth, okDouble, ReadFloat('SWidth'), sSysPreviewDlgOptions).Value
          else
            PreviewDlgOptions.PageWidth := GetOption(sSysPageWidth, okDouble, 100 * Zoll, sSysPreviewDlgOptions).Value;

          if ValueExists('SHeight') then
            PreviewDlgOptions.PageHeight := GetOption(sSysPageHeight, okDouble, ReadFloat('SHeight'), sSysPreviewDlgOptions).Value
          else
            PreviewDlgOptions.PageHeight := GetOption(sSysPageHeight, okDouble, 100 * Zoll, sSysPreviewDlgOptions).Value;

          if ValueExists('SAutoRot') then
            PreviewDlgOptions.Rotate := GetOption(sSysRotate, okBoolean, ReadBool('SAutoRot'), sSysPreviewDlgOptions).Value
          else
            PreviewDlgOptions.Rotate := GetOption(sSysRotate, okBoolean, True, sSysPreviewDlgOptions).Value;

          if ValueExists('OAnno') then
            PreviewDlgOptions.Annos := GetOption(sSysAnnos, okBoolean, ReadBool('OAnno'), sSysPreviewDlgOptions).Value
          else
            PreviewDlgOptions.Annos := GetOption(sSysAnnos, okBoolean, False, sSysPreviewDlgOptions).Value;

          CloseKey;
        end;
      finally
        FreeAndNil(Reg);
      end;

    end;
  except

  end;
end;

procedure TbfaViewerOptions.ReadResubOptions(RKey: HKEY; RegKey: WideString);
var
  Reg: TRegistry;
begin
  try
    ResubOptions.ColumnTitle := GetOption(sSysColumnTitle, okBoolean, True, sSysResubOptions).Value;
    ResubOptions.ColumnInfo := GetOption(sSysColumnInfo, okBoolean, True, sSysResubOptions).Value;
    ResubOptions.ColumnComment := GetOption(sSysColumnComment, okBoolean, True, sSysResubOptions).Value;
    ResubOptions.ColumnAlarm := GetOption(sSysColumnAlarm, okBoolean, False, sSysResubOptions).Value;
    ResubOptions.ColumnModified := GetOption(sSysColumnModified, okBoolean, False, sSysResubOptions).Value;
    ResubOptions.ColumnDocDate := GetOption(sSysColumnDocDate, okBoolean, False, sSysResubOptions).Value;
    ResubOptions.ColumnArchive := GetOption(sSysColumnArchive, okBoolean, True, sSysResubOptions).Value;
    ResubOptions.ShowAllAddfields := GetOption(sSysShowAllAddfields, okBoolean, True, sSysResubOptions).Value;
    ResubOptions.ResubFavsInPreview := GetOption(sSysResubFavsInPreview, okBoolean, False, sSysResubOptions).Value;

    if FileExists(FIniFile) or FileExists(FGlobalIniFile) then
    begin
      ResubOptions.ResubOnStartup := GetOption(sSysResubOnStartup, okBoolean, True, sSysResubOptions).Value;
      ResubOptions.ResubTimer := GetOption(sSysResubTimer, okBoolean, True, sSysResubOptions).Value;
      ResubOptions.ResubTimerInterval := GetOption(sSysResubTimerInterval, okInteger, 5, sSysResubOptions).Value;
      ResubOptions.ReturnToSender := GetOption(sSysReturnToSender, okBoolean, False, sSysResubOptions).Value;
      ResubOptions.Emailreminder := GetOption(sSysEmailreminder, okBoolean, False, sSysResubOptions).Value;
      ResubOptions.NotifyByMail := GetOption(sSysNotifyByMail, okBoolean, False, sSysResubOptions).Value;
    end
    else
    begin
      //muss aus der Registry lesen
      FSaveOptions := True;
      Reg := TRegistry.Create;
      try
        with Reg do
        begin
          RootKey := RKey;
          OpenKey(RegKey, True);
          if ValueExists('ChkTasksTimer') then
            ResubOptions.ResubTimer := GetOption(sSysResubTimer, okBoolean, ReadBool('ChkTasksTimer'), sSysResubOptions).Value
          else
            ResubOptions.ResubTimer := GetOption(sSysResubTimer, okBoolean, True, sSysResubOptions).Value;

          if ValueExists('ChkTasksInterval') then
            ResubOptions.ResubTimerInterval := GetOption(sSysResubTimerInterval, okInteger, ReadInteger('ChkTasksInterval'), sSysResubOptions).Value
          else
            ResubOptions.ResubTimerInterval := GetOption(sSysResubTimerInterval, okInteger, 5, sSysResubOptions).Value;

          if ValueExists('DocTimer') then
            ResubOptions.ResubOnStartup := GetOption(sSysResubOnStartup, okBoolean, ReadBool('DocTimer'), sSysResubOptions).Value
          else
            ResubOptions.ResubOnStartup := GetOption(sSysResubOnStartup, okBoolean, True, sSysResubOptions).Value;

          if ValueExists('ReturntoSender') then
            ResubOptions.ReturnToSender := GetOption(sSysReturnToSender, okBoolean, ReadBool('ReturntoSender'), sSysResubOptions).Value
          else
            ResubOptions.ReturnToSender := GetOption(sSysReturnToSender, okBoolean, False, sSysResubOptions).Value;

          if ValueExists('Erinnerungsmail') then
            ResubOptions.Emailreminder := GetOption(sSysEmailreminder, okBoolean, ReadBool('Erinnerungsmail'), sSysResubOptions).Value
          else
            ResubOptions.Emailreminder := GetOption(sSysEmailreminder, okBoolean, False, sSysResubOptions).Value;

          if ValueExists('WKommentar') then
            ResubOptions.ColumnComment := GetOption(sSysColumnComment, okBoolean, ReadBool('WKommentar'), sSysResubOptions).Value
          else
            ResubOptions.ColumnComment := GetOption(sSysColumnComment, okBoolean, False, sSysResubOptions).Value;

          CloseKey;
        end;
      finally
        FreeAndNil(Reg);
      end;

    end;
  except

  end;

end;

function TbfaViewerOptions.Save: Integer;
begin
  bitfarmLog.DebugBegin(llDebug, 'TbfaViewerOptions.Save', ClassName);
  try
    try
      SaveGeneralOptions;
      SaveEditorOptions;
      SaveSearchOptions;
      SaveResubOptions;
      SaveAdminOptions;
      if bitfarmLog.LogLevel <> AdminOptions.LogLevel then
        bitfarmLog.ChangeLogLevel(AdminOptions.LogLevel);
      SaveCheckboxDialogsOptions;
      Result := inherited Save;
    except
      Result := -1;
      bitfarmLog.Write(llNormal, ltException, 'TbfaViewerOptions.Save', ClassName);
    end;
  finally
    bitfarmLog.DebugEnd(llDebug, 'TbfaViewerOptions.Save', ClassName);
  end;
end;

function TbfaViewerOptions.SaveAdminOptions(SaveTofile: Boolean): Integer;
var
  DefaultRights: string;
begin
  bitfarmLog.DebugBegin(llDebug, 'SaveAdminOptions', ClassName);
  try
    Result := 0;
    try
      SetOption(sSysDefaultArchiveGroup, okBoolean, AdminOptions.DefaultArchiveGroup, sSysAdminOptions);
      SetOption(sSysDefaultAddfieldGroup, okBoolean, AdminOptions.DefaultAddfieldGroup, sSysAdminOptions);
      SetOption(sSysDefaultStatefieldGroup, okBoolean, AdminOptions.DefaultStatefieldGroup, sSysAdminOptions);
      SetOption(sSysDefaultResubStatefield, okBoolean, AdminOptions.DefaultResubStatefield, sSysAdminOptions);
      SetOption(sSysAdminFullscreen, okBoolean, AdminOptions.AdminFullscreen, sSysAdminOptions);
      SetOption(sSysLogLevel, okInteger, AdminOptions.LogLevel, sSysAdminOptions);

      DefaultRights := IntToStr(AdminOptions.DefaultArchiveGroupRights);

      SetOption(sSysDefaultArchiveGroupRights, okString, DefaultRights, sSysAdminOptions);
      SetOption(sSysDefaultAddfieldGroupRights, okInteger, AdminOptions.DefaultAddfieldGroupRights, sSysAdminOptions);
      SetOption(sSysDefaultStatefieldGroupRights, okInteger, AdminOptions.DefaultStatefieldGroupRights, sSysAdminOptions);

      if SaveTofile then
        Result := inherited Save;

    except
      Result := -1;
      bitfarmLog.Write(llNormal, ltException, 'SaveAdminOptions', ClassName);
    end;
  finally
    bitfarmLog.DebugEnd(llDebug, 'SaveAdminOptions', ClassName);
  end;
end;

function TbfaViewerOptions.SaveCheckboxDialogsOptions(
  SaveTofile: Boolean): Integer;
var
  CheckBoxes: string;
  i: Integer;
begin
//    SetLength(CheckBoxDialogs, 0);
//    CheckboxDialogOptions :=
//      GetOption(sSysCheckboxDialogs, okString, '', sSysSystemOptions).Value;
//
//    sList := SplitString(CheckboxDialogOptions, ';');
//    for i := Low(sList) to High(sList) - 1 do
//    begin
//      sList2 := SplitString(sList[i], ',');
//      CheckBoxDialogs[i + 1].DialogName := sList2[0];
//      CheckBoxDialogs[i + 1].ShowDialog := StrToBool(sList2[1]);
//      CheckBoxDialogs[i + 1].DialogResultType := StrToInt(sList2[2]);
//    end;
//
//


  bitfarmLog.DebugBegin(llDebug, 'SaveCheckboxDialogsOptions', ClassName);
  Result := 0;
  try
    try

      CheckBoxes := '';
      for i := Low(CheckBoxDialogs) to High(CheckBoxDialogs) do
        CheckBoxes := CheckBoxes + CheckBoxDialogs[i].DialogName + ',' +
          BoolToStr(CheckBoxDialogs[i].ShowDialog) + ',' +
          IntToStr(CheckBoxDialogs[i].DialogResultType) + ';';

      CheckboxDialogOptions := CheckBoxes;

      SetOption(sSysCheckboxDialogs, okString, CheckboxDialogOptions, sSysSystemOptions);

      if SaveTofile then
        Result := inherited Save;

    except
      Result := -1;
      bitfarmLog.Write(llNormal, ltException, 'SaveCheckboxDialogsOptions', ClassName);
    end;
  finally
    bitfarmLog.DebugEnd(llDebug, 'SaveCheckboxDialogsOptions', ClassName);
  end;

end;

function TbfaViewerOptions.SaveGeneralOptions: Integer;
begin
  bitfarmLog.DebugBegin(llDebug, 'SaveGeneralOptions', ClassName);
  Result := 0;
  try
    try
      SetOption(sSysDefaultSaveDirectory, okString, GeneralOptions.DefaultSaveDirectory, sSysGeneralOptions);
      SetOption(sSysOverwriteLocalSave, okBoolean, GeneralOptions.OverwriteLocalSave, sSysGeneralOptions);
      SetOption(sSysRowColorEvenOdd, okBoolean, GeneralOptions.RowColorEvenOdd, sSysGeneralOptions);
      SetOption(sSysDefaultPrinter, okInteger, GeneralOptions.DefaultPrinter, sSysGeneralOptions);
      SetOption(sSysUseDefaultPrinter, okBoolean, GeneralOptions.UseDefaultPrinter, sSysGeneralOptions);
      SetOption(sSysDefaultPrinterName, okString, GeneralOptions.DefaultPrinterName, sSysGeneralOptions);
      SetOption(sSysNoAutoHintByAnnos, okBoolean, GeneralOptions.NoAutoHintByAnnos, sSysGeneralOptions);
      SetOption(sSysHintBallons, okBoolean, GeneralOptions.HintBallons, sSysGeneralOptions);
      SetOption(sSysShowLinkCount, okBoolean, GeneralOptions.ShowLinkCount, sSysGeneralOptions);
      SetOption(sSysShowRoot, okBoolean, GeneralOptions.ShowRoot, sSysGeneralOptions);
      SetOption(sSysAutoPreview, okBoolean, GeneralOptions.AutoPreview, sSysGeneralOptions);
      SetOption(sSysNoLoginWindow, okBoolean, GeneralOptions.NoLoginWindow, sSysGeneralOptions);
      SetOption(sSysMsLogon, okBoolean, GeneralOptions.MsLogon, sSysGeneralOptions);
      SetOption(sSysDefaultEmail, okInteger, GeneralOptions.DefaultEmail, sSysGeneralOptions);
      SetOption(sSysDefaultCleanup, okInteger, GeneralOptions.DefaultCleanup, sSysGeneralOptions);
      SetOption(sSysTimeSync, okBoolean, GeneralOptions.TimeSync, sSysGeneralOptions);
      SetOption(sSysTimeSyncServer, okString, GeneralOptions.TimeSyncServer, sSysGeneralOptions);
      SetOption(sSysAlertWindow, okBoolean, GeneralOptions.AlertWindow, sSysGeneralOptions);
      SetOption(sSysExportPDF, okBoolean, GeneralOptions.ExportPDF, sSysGeneralOptions);
      SetOption(sSysOleOutlook, okBoolean, GeneralOptions.OleOutlook, sSysGeneralOptions);
      SetOption(sSysMailTo, okBoolean, GeneralOptions.MailTo, sSysGeneralOptions);
      SetOption(sSysComboAutoDown, okBoolean, GeneralOptions.ComboAutoDown, sSysGeneralOptions);
      SetOption(sSysAlternativeScanMode, okBoolean, GeneralOptions.AlternativeScanMode, sSysGeneralOptions);
      SetOption(sSysShowDocLinks, okBoolean, GeneralOptions.ShowDocLinks, sSysGeneralOptions);
      SetOption(sSysShowDocLinksInTab, okBoolean, GeneralOptions.ShowDocLinksInTab, sSysGeneralOptions);
      SetOption(sSysAutoViewerAtArchiveChange, okBoolean, GeneralOptions.AutoViewerAtArchiveChange, sSysGeneralOptions);
      SetOption(sSysAutoCheckout, okBoolean, GeneralOptions.AutoCheckout, sSysGeneralOptions);
      SetOption(sSysAutoCheckin, okBoolean, GeneralOptions.AutoCheckin, sSysGeneralOptions);
      SetOption(sSysCheckoutNewDocuments, okBoolean, GeneralOptions.CheckoutNewDocuments, sSysGeneralOptions);
      SetOption(sSysDisplayCheckoutAtShutdown, okBoolean, GeneralOptions.DisplayCheckoutAtShutdown, sSysGeneralOptions);
      SetOption(sSysChangePageByEnter, okBoolean, GeneralOptions.ChangePageByEnter, sSysGeneralOptions);
      SetOption(sSysDragDropAsWindows, okBoolean, GeneralOptions.DragDropAsWindows, sSysGeneralOptions);
      SetOption(sSysSetAnnoDPI, okBoolean, GeneralOptions.SetAnnoDPI, sSysGeneralOptions);
      SetOption(sSysUseFileNameAsDocName, okBoolean, GeneralOptions.UseFileNameAsDocName, sSysGeneralOptions);

      SetOption(sSysPluginButtons, okString, GeneralOptions.PluginButtons, sSysGeneralOptions);
      SetOption(sSysPluginTimeOut, okInteger, GeneralOptions.PluginTimeOut, sSysGeneralOptions);
  
      SetOption(sSysResetFieldsAfterArchiving, okBoolean, GeneralOptions.ResetFieldsAfterArchiving, sSysGeneralOptions);
      SetOption(sSysPrintAfterArchiving, okBoolean, GeneralOptions.PrintAfterArchiving, sSysGeneralOptions);
      SetOption(sSysLateOCR, okBoolean, GeneralOptions.LateOCR, sSysGeneralOptions);
      SetOption(sSysNoOCR, okBoolean, GeneralOptions.NoOCR, sSysGeneralOptions);
      SetOption(sSysCopyGdocidLink, okBoolean, GeneralOptions.CopyGdocidLink, sSysGeneralOptions);
      SetOption(sSysImportFilter, okString, GeneralOptions.ImportFilter, sSysGeneralOptions);
      SetOption(sSysImportColor, okBoolean, GeneralOptions.ImportColor, sSysGeneralOptions);
      SetOption(sSysImportHighRes, okBoolean, GeneralOptions.ImportHighRes, sSysGeneralOptions);
      SetOption(sSysImportBarcode, okBoolean, GeneralOptions.ImportBarcode, sSysGeneralOptions);
      SetOption(sSysCalculateOnSelectedRecordsOnly, okBoolean, GeneralOptions.CalculateOnSelectedRecordsOnly, sSysGeneralOptions);
      SetOption(sSysEnableSymlinks, okBoolean, GeneralOptions.EnableSymlinks, sSysGeneralOptions);
      SetOption(sSysSymlinkAddfields, okBoolean, GeneralOptions.SymlinkAddfields, sSysGeneralOptions);

      SetOption(sSysUpdate, okInteger, GeneralOptions.Update, sSysGeneralOptions); // SJ Abschlussprojekt
      SetOption(sSysStartingArea, okInteger, GeneralOptions.StartingArea, sSysGeneralOptions);
      SetOption(sSysNegativeCurrency, okBoolean, GeneralOptions.NegativeCurrency, sSysGeneralOptions);
      SetOption(sSysCloseBookmarkDialog, okBoolean, GeneralOptions.CloseBookmarkDialog, sSysGeneralOptions);
    except
      Result := -1;
      bitfarmLog.Write(llNormal, ltException, 'SaveGeneralOptions', ClassName);
    end;
  finally
    bitfarmLog.DebugEnd(llDebug, 'SaveGeneralOptions', ClassName);
  end;
end;

function TbfaViewerOptions.SaveEditorOptions: Integer;
begin
  bitfarmLog.DebugBegin(llDebug, 'SaveGeneralOptions', ClassName);
  Result := 0;
  try
    try
      SetOption(sSysShowViewerThumbnail, okBoolean, EditorOptions.ShowViewerThumbnail, sSysEditorOptions);
      SetOption(sSysShowImporterThumbnail, okBoolean, EditorOptions.ShowImporterThumbnail, sSysEditorOptions);
      SetOption(sSysShowWorkplaceThumbnail, okBoolean, EditorOptions.ShowWorkplaceThumbnail, sSysEditorOptions);
      SetOption(sSysInvertDirection, okBoolean, EditorOptions.InvertDirection, sSysEditorOptions);
      SetOption(sSysOCRAutoClipbrd, okBoolean, EditorOptions.OCRAutoClipbrd, sSysEditorOptions);
      SetOption(sSysOCRAdd, okBoolean, EditorOptions.OCRAdd, sSysEditorOptions);
      SetOption(sSysAutoZoom, okInteger, EditorOptions.AutoZoom, sSysEditorOptions);
      SetOption(sSysOCRKey, okInteger, EditorOptions.OCRKey, sSysEditorOptions);
      SetOption(sSysInUnDockedPanel, okBoolean, EditorOptions.InUnDockedPanel, sSysEditorOptions);
      SetOption(sSysAutoEditMode, okBoolean, EditorOptions.AutoEditMode, sSysEditorOptions);
      SetOption(sSysDocNoAction, okBoolean, EditorOptions.DocNoAction, sSysEditorOptions);
      SetOption(sSysDocEditorZoom, okBoolean, EditorOptions.DocEditorZoom, sSysEditorOptions);
      SetOption(sSysImporterInUnDockedPanel, okBoolean, EditorOptions.ImporterInUnDockedPanel, sSysEditorOptions);
      SetOption(sSysImporterOpenDocInImporter, okBoolean, EditorOptions.ImporterOpenDocInImporter, sSysEditorOptions);
      SetOption(sSysWorkplaceAutoEditMode, okBoolean, EditorOptions.WorkplaceAutoEditMode, sSysEditorOptions);
      SetOption(sSysWorkplaceInUnDockedPanel, okBoolean, EditorOptions.WorkplaceInUnDockedPanel, sSysEditorOptions);
      SetOption(sSysWorkplaceOpenDocInWorkplace, okBoolean, EditorOptions.WorkplaceOpenDocInWorkplace, sSysEditorOptions);
      SetOption(sSysWorkplaceOpenDoc, okBoolean, EditorOptions.WorkplaceOpenDoc, sSysEditorOptions);
      SetOption(sSysRememberLastTool, okBoolean, EditorOptions.RememberLastTool, sSysEditorOptions);
    except
      Result := -1;
      bitfarmLog.Write(llNormal, ltException, 'SaveGeneralOptions', ClassName);
    end;
  finally
    bitfarmLog.DebugEnd(llDebug, 'SaveGeneralOptions', ClassName);
  end;
end;

function TbfaViewerOptions.SaveSearchOptions: Integer;
begin
  bitfarmLog.DebugBegin(llDebug, 'SaveSearchOptions', ClassName);
  Result := 0;
  try
    try
      SetOption(sSysLimit, okInteger, SearchOptions.Limit, sSysSearchOptions);
      SetOption(sSysAutoDisplay, okBoolean, SearchOptions.AutoDisplay, sSysSearchOptions);
      SetOption(sSysSearchLastYear, okBoolean, SearchOptions.SearchLastYear, sSysSearchOptions);
      SetOption(sSysSQLFullText, okBoolean, SearchOptions.SQLFullText, sSysSearchOptions);
      SetOption(sSysDocModified, okBoolean, SearchOptions.DocModified, sSysSearchOptions);
      SetOption(sSysShowDocLinks, okBoolean, SearchOptions.ShowDocLinks, sSysSearchOptions);
      SetOption(sSysDontResetOnArcSet, okBoolean, SearchOptions.DontResetOnArcSet, sSysSearchOptions);
      SetOption(sSysHistoryBookmarkLimit, okInteger, SearchOptions.HistoryBookmarkLimit, sSysSearchOptions);
      SetOption(sSysFtxPos, okBoolean, SearchOptions.FtxPos, sSysSearchOptions);

      SetOption(sSysFulltextSearch, okBoolean, SearchOptions.FulltextSearch, sSysSearchOptions);
      SetOption(sSysDocTagsSearch, okBoolean, SearchOptions.DocTagsSearch, sSysSearchOptions);
      SetOption(sSysAnnoTxtSearch, okBoolean, SearchOptions.AnnoTxtSearch, sSysSearchOptions);
      SetOption(sSysFieldSearch, okBoolean, SearchOptions.FieldTxtSearch, sSysSearchOptions);
      SetOption(sSysUseOnlyGeneralSearchField, okBoolean, SearchOptions.UseOnlyGeneralSearchField, sSysSearchOptions);
      SetOption(sSysShowAllAddfields, okBoolean, SearchOptions.ShowAllAddfields, sSysSearchOptions);

    except
      Result := -1;
      bitfarmLog.Write(llNormal, ltException, 'SaveSearchOptions', ClassName);
    end;
  finally
    bitfarmLog.DebugEnd(llDebug, 'SaveSearchOptions', ClassName);
  end;
end;

function TbfaViewerOptions.SaveGUIOptions(SaveTofile: Boolean = False): Integer;
begin
  bitfarmLog.DebugBegin(llDebug, 'SaveGUIOptions', ClassName);
  Result := 0;
  try
    try
      SetOption(sSysStoredFloatingPosX, okInteger, GuiOptions.StoredFloatingPosX, sSysGUIOptions);
      SetOption(sSysStoredFloatingPosY, okInteger, GuiOptions.StoredFloatingPosY, sSysGUIOptions);
      SetOption(sSysStoredFloatingOldPosX, okInteger, GuiOptions.StoredFloatingOldPosX, sSysGUIOptions);
      SetOption(sSysStoredFloatingOldPosY, okInteger, GuiOptions.StoredFloatingOldPosY, sSysGUIOptions);
      SetOption(sSysSearchPanelFloatingPosX, okInteger, GuiOptions.SearchPanelFloatingPosX, sSysGUIOptions);
      SetOption(sSysSearchPanelFloatingPosY, okInteger, GuiOptions.SearchPanelFloatingPosY, sSysGUIOptions);
      SetOption(sSysSearchPanelFloatingWidth, okInteger, GuiOptions.SearchPanelFloatingWidth, sSysGUIOptions);
      SetOption(sSysSearchPanelFloatingHeight, okInteger, GuiOptions.SearchPanelFloatingHeight, sSysGUIOptions);
      SetOption(sSysStoredFloatingWidth, okInteger, GuiOptions.StoredFloatingWidth, sSysGUIOptions);
      SetOption(sSysStoredFloatingHeight, okInteger, GuiOptions.StoredFloatingHeight, sSysGUIOptions);
      SetOption(sSysStoredFloatingOldWidth, okInteger, GuiOptions.StoredFloatingOldWidth, sSysGUIOptions);
      SetOption(sSysStoredFloatingOldHeight, okInteger, GuiOptions.StoredFloatingOldHeight, sSysGUIOptions);
      SetOption(sSysStoredFloatingFullScreen, okBoolean, GUIOptions.StoredFloatingFullScreen, sSysGUIOptions);
      SetOption(sSysSearchPanelFloating, okBoolean, GUIOptions.SearchPanelFloating, sSysGUIOptions);
      SetOption(sSysSearchPanelWidth, okInteger, GuiOptions.SearchPanelWidth, sSysGUIOptions);

      if SaveTofile then
        Result := inherited Save;

    except
      Result := -1;
      bitfarmLog.Write(llNormal, ltException, 'SaveGUIOptions', ClassName);
    end;
  finally
    bitfarmLog.DebugEnd(llDebug, 'SaveGUIOptions', ClassName);
  end;
end;

function TbfaViewerOptions.SavePreviewDlgOptions(SaveTofile: Boolean = False): Integer;
begin
  bitfarmLog.DebugBegin(llDebug, 'SavePreviewDlgOptions', ClassName);
  Result := 0;
  try
    try
      SetOption(sSysPageSize, okInteger, PreviewDlgOptions.PageSize, sSysPreviewDlgOptions);
      SetOption(sSysCopies, okInteger, PreviewDlgOptions.Copies, sSysPreviewDlgOptions);
      SetOption(sSysBorderTop, okDouble, PreviewDlgOptions.BorderTop, sSysPreviewDlgOptions);
      SetOption(sSysBorderLeft, okDouble, PreviewDlgOptions.BorderLeft, sSysPreviewDlgOptions);
      SetOption(sSysBorderRight, okDouble, PreviewDlgOptions.BorderRight, sSysPreviewDlgOptions);
      SetOption(sSysBorderBottom, okDouble, PreviewDlgOptions.BorderBottom, sSysPreviewDlgOptions);
      SetOption(sSysPageHeight, okDouble, PreviewDlgOptions.PageHeight, sSysPreviewDlgOptions);
      SetOption(sSysPageWidth, okDouble, PreviewDlgOptions.PageWidth, sSysPreviewDlgOptions);
      SetOption(sSysAnnos, okBoolean, PreviewDlgOptions.Annos, sSysPreviewDlgOptions);
      SetOption(sSysPubAnnos, okBoolean, PreviewDlgOptions.PubAnnos, sSysPreviewDlgOptions);
      SetOption(sSysPrivAnnos, okBoolean, PreviewDlgOptions.PrivAnnos, sSysPreviewDlgOptions);
      SetOption(sSysStationery, okBoolean, PreviewDlgOptions.Stationery, sSysPreviewDlgOptions);
      SetOption(sSysCenter, okBoolean, PreviewDlgOptions.Center, sSysPreviewDlgOptions);
      SetOption(sSysRotate, okBoolean, PreviewDlgOptions.Rotate, sSysPreviewDlgOptions);
      SetOption(sSysSort, okBoolean, PreviewDlgOptions.Sort, sSysPreviewDlgOptions);
      SetOption(sSysMerge, okBoolean, PreviewDlgOptions.Merge, sSysPreviewDlgOptions);

      if SaveTofile then
        Result := inherited Save;

    except
      Result := -1;
      bitfarmLog.Write(llNormal, ltException, 'SavePreviewDlgOptions', ClassName);
    end;
  finally
    bitfarmLog.DebugEnd(llDebug, 'SavePreviewDlgOptions', ClassName);
  end;
end;

function TbfaViewerOptions.SaveResubOptions(SaveTofile: Boolean = False): Integer;
begin
  bitfarmLog.DebugBegin(llDebug, 'SaveResubOptions', ClassName);
  Result := 0;
  try
    try
      SetOption(sSysColumnTitle, okBoolean, ResubOptions.ColumnTitle, sSysResubOptions);
      SetOption(sSysColumnInfo, okBoolean, ResubOptions.ColumnInfo, sSysResubOptions);
      SetOption(sSysColumnComment, okBoolean, ResubOptions.ColumnComment, sSysResubOptions);
      SetOption(sSysColumnAlarm, okBoolean, ResubOptions.ColumnAlarm, sSysResubOptions);
      SetOption(sSysColumnModified, okBoolean, ResubOptions.ColumnModified, sSysResubOptions);
      SetOption(sSysColumnDocDate, okBoolean, ResubOptions.ColumnDocDate, sSysResubOptions);
      SetOption(sSysColumnArchive, okBoolean, True, sSysResubOptions);
      SetOption(sSysResubOnStartup, okBoolean, ResubOptions.ResubOnStartup, sSysResubOptions);
      SetOption(sSysResubTimer, okBoolean, ResubOptions.ResubTimer, sSysResubOptions);
      SetOption(sSysResubTimerInterval, okInteger, ResubOptions.ResubTimerInterval, sSysResubOptions);
      SetOption(sSysReturnToSender, okBoolean, ResubOptions.ReturnToSender, sSysResubOptions);
      SetOption(sSysEmailreminder, okBoolean, ResubOptions.Emailreminder, sSysResubOptions);
      SetOption(sSysNotifyByMail, okBoolean, ResubOptions.NotifyByMail, sSysResubOptions);
      SetOption(sSysShowAllAddfields, okBoolean, ResubOptions.ShowAllAddfields, sSysResubOptions);
      SetOption(sSysResubFavsInPreview, okBoolean, ResubOptions.ResubFavsInPreview, sSysResubOptions);

      if SaveTofile then
        Result := inherited Save;

    except
      Result := -1;
      bitfarmLog.Write(llNormal, ltException, 'SaveResubOptions', ClassName);
    end;
  finally
    bitfarmLog.DebugEnd(llDebug, 'SaveResubOptions', ClassName);
  end;
end;

function TbfaViewerOptions.SaveSystemOptions(SaveTofile: Boolean = False): Integer;
begin
  bitfarmLog.DebugBegin(llDebug, 'SaveSystemOptions', ClassName);
  Result := 0;
  try
    try
//12.04.2022 wird im Viewer.ini profileabhngig gespeichert      SetOption(Self.MarkedDocsProfile, okString, SystemOptions.MarkedDocs, sSysSystemOptions);

      if SaveTofile then
        Result := inherited Save;

    except
      Result := -1;
      bitfarmLog.Write(llNormal, ltException, 'SaveSystemOptions', ClassName);
    end;
  finally
    bitfarmLog.DebugEnd(llDebug, 'SaveSystemOptions', ClassName);
  end;
end;

end.

