unit bfaProcessing;

interface

uses
  Generics.Collections, Classes, ExtCtrls, bfaLog, bfaApplication, bfaBaseObjects,
  ievect, bfaBasePreview, bfaPreview36, bfaObjects36, bfaImageEnVect,
  bfaImageEnMView
  ;

procedure Rotate(Docs: TList<TbfaDocument>; Grad: Integer; AllPages: Boolean = False;
  Pages: TStringList = nil; SingleView: Tfrm_Voransicht = nil; ThumbnailView: TbfaImageEnMView = nil;
  AThread: TThread = nil; WithAnnos: Boolean = False);

procedure LoadImage(PageIdx: Integer; DocView: Tfrm_Voransicht; Doc: TbfaDocument);

procedure ReloadLinks(Doc: TbfaDocument);

function bfaImageRendering(AThread : TThread; Docs: TList<TbfaDocument>; Stationery,
  PrivAnno, PubAnno: Boolean; Layer: Integer = -1; Copies: Integer = 1;
  Sorting: Boolean = False; imgM: TbfaImageEnMView = nil; StationeryPath: WideString = '';
  AlwaysWithAnno : Boolean = False; Duplex : Boolean = False) : TMemoryStream;

procedure ShowStationery(PageIdx: Integer; imgtmp: TbfaImageEnVect; StatObj: TbfaStationery);

implementation

uses
  SysUtils, StrUtils, Variants, Controls, gnugettext, bfaSocketConnection, DB,
  Graphics, bfaCommonUtils, bfaCommonImageEN, MainDataModule, Dialogs,
  bfaCommonMethods, hyieutils, Math, imageenio, hyiedefs, uni,
  imageenview, bfaIconsDM, SpTBXSkins, ShellAPI, bfaCommonDB, MainForm,
  SpTBXItem, Windows
  {$IFDEF IMAGEEN1140}
  , iexBitmaps, iexClasses
  {$ENDIF}
  ;

function PreProcess(DocView: TbfaImageEnVect): Integer;
begin
  Result := 0;
  DocView.LockPaint;
  try
    DocView.CancelInteracts;
    DocView.RemoveAllObjects;
    Result := DocView.IO.Params.ImageIndex;
    DocView.OnProgress := nil;
  finally
    DocView.UnLockPaint;
  end;
end;

procedure LoadImage(PageIdx: Integer; DocView: Tfrm_Voransicht; Doc: TbfaDocument);
var
  iebmp2 : TIEBitmap;
begin
  bitfarmLog.DebugBegin(llDebug, 'LoadImage', 'bfaProcessing');
  try
    try
      if Doc <> nil then
      begin
        if Doc.HasPreview then  //25.07.2017 muss im PageSortings Frame alle Dateityp angezeigt werden!
        begin
          if (Doc.ScanPoolView <> nil) and Doc.ScanPoolViewLoaded then
          begin
            DocView.img_DocView.IO.Params.ImageIndex := 0;
            iebmp2 := TIEBitmap.Create;
            try
              Doc.ScanPoolView.CopyToIEBitmap(PageIdx, iebmp2);
              {$IFDEF ImageEN1140}
              DocView.img_DocView.Bitmap.Assign(iebmp2.VclBitmap);
              {$ELSE}
              DocView.img_DocView.IO.Bitmap.Assign(iebmp2.VclBitmap);
              {$ENDIF}
            finally
              FreeandNil(iebmp2);
            end;
            DocView.img_DocView.IO.Params.Width := DocView.img_DocView.Bitmap.Width;
            DocView.img_DocView.IO.Params.Height := DocView.img_DocView.Bitmap.Height;
            DocView.img_DocView.Update; // SJ 31.03.23 - img_DocView.Fit hat nicht funktioniert weil fMaxLayerWidth und fMaxLayerHeight nicht korrekt gesetzt waren
          end else
          begin
            DocView.img_DocView.IO.Params.ImageIndex := PageIdx;
            Doc.DocStream.Position := 0;
            DocView.img_DocView.IO.LoadFromStream(Doc.DocStream);
          end;
        end
        else begin
          DocView.img_DocView.IO.Params.ImageIndex := 0;
          CreateEmptyImage(DocView.img_DocView, Doc.FileType, clWhite);
        end;
      end;
    except
      bitfarmLog.Write(llNormal, ltException, 'LoadImage', 'bfaProcessing');
    end;
  finally
    bitfarmLog.DebugEnd(llDebug, 'LoadImage', 'bfaProcessing');
  end;
end;

procedure LoadAndRotate(Page: Integer; DocView: Tfrm_Voransicht; ThumbnailView: TbfaImageEnMView;
  Grad: Integer; Doc: TbfaDocument; WithAnnos: Boolean);
var
  idx: Integer;
begin
  try
    idx := -1;
    if DocView.img_DocView.ObjectsCount > 0 then
      DocView.img_DocView.RemoveAllObjects;
    if DocView = nil then
      Doc.GetAnnotation(Page, 0, idx)
    else
    begin
      if DocView.Owner <> nil then
        Doc.GetAnnotation(Page, TbfaPreview(DocView.Owner).CurrentLayer, idx)
      else
        idx := -1;
    end;
    LoadImage(Page, DocView, Doc);
    if (idx <> -1) and WithAnnos then
    begin
      Doc.Annotations[idx].AnnoStream.Position := 0;
      DocView.img_DocView.LoadFromStreamIEV(Doc.Annotations[idx].AnnoStream);
      Doc.Annotations[idx].AnnoStream.Position := 0;
      Doc.Modified_Annos := True;
      DocView.img_DocView.Update;
    end;

    DocView.RotateImage(Grad);

    if (Doc <> nil) and
       (Doc.ScanPoolView <> nil) and (Doc.ScanPoolViewLoaded) then
    begin   
      Doc.ScanPoolView.Deselect;
      Doc.ScanPoolView.SelectedImage := Page;
      Doc.ScanPoolView.Proc.Rotate(Grad);
      Doc.ScanPoolView.Deselect;
    end;

      if idx = -1 then
      begin
        if ThumbnailView <> nil then
        begin
          ThumbnailView.SetIEBitmap(Page, DocView.img_DocView.IEBitmap);
        end;
      end
      else
      begin
        if ThumbnailView <> nil then
        begin
          RefreshOneThumbWithAnnos(DocView, Doc, Page, TbfaPreview(DocView.Owner).CurrentLayer,
            ThumbnailView, Grad);
        end;
      end;

  except
    bitfarmLog.Write(llNormal, ltException, 'LoadAndRotate', 'bfaProcessing');
  end;
end;

procedure RotateOnePage(Page: Integer; DocView: Tfrm_Voransicht; AThread: TThread;
  ThumbnailView: TbfaImageEnMView; Grad: Integer; Doc: TbfaDocument; WithAnnos: Boolean = False);
begin
  try
    bitfarmLog.Write(llNormal, ltInfo, _('Drehe Seite ') + IntToStr(Page), 'bfaProcessing');

    AThread.Synchronize(nil,
      procedure
      begin
        if Doc.ThumbnailView <> nil then
          LoadAndRotate(Page, DocView, Doc.ThumbnailView, Grad, Doc, WithAnnos)
        else
          LoadAndRotate(Page, DocView, ThumbnailView, Grad, Doc, WithAnnos);
      end);

    AThread.Synchronize(nil,
      procedure
      begin
        if (Doc.ScanPoolView <> nil) and (Doc.ScanPoolViewLoaded) then
        begin
          Doc.ScanPoolViewchanged := True;
        end else
        begin
          with Doc do
          begin
            StartSaveProcess;
            try
              DocStream.Position := 0;
              if DocView.img_DocView.IO.Params.ImageCount > 1 then
                DocView.img_DocView.IO.ReplaceStreamTIFF(DocStream)
              else
                DocView.img_DocView.IO.SaveToStreamTIFF(DocStream);
              DocStream.Position := 0;

            finally
              EndSaveProcess;
            end;
          end;
        end;
      end);

    bitfarmApp.SaveAnnotation(AThread, DocView, Doc);

  except
    bitfarmLog.Write(llNormal, ltException, 'RotateOnePage', 'bfaProcessing');
  end;
end;

procedure Rotate(Docs: TList<TbfaDocument>; Grad: Integer; AllPages: Boolean = False;
  Pages: TStringList = nil; SingleView: Tfrm_Voransicht = nil; ThumbnailView: TbfaImageEnMView = nil;
  AThread: TThread = nil; WithAnnos: Boolean = False);
var
  i, j, iCount,
//  licount,
  lstIdx, cnt: integer;
  Doc: TbfaDocument;
  DocList: TList<TbfaDocument>;
  DocView: Tfrm_Voransicht;
  Lst: TStringList;
begin
  bitfarmLog.DebugBegin(llDebug, 'Rotate(' + IntToStr(Grad) + ')', 'bfaProcessing');
  try
    try
      if ((Docs.Count = 0) or ((not AllPages) and (Pages = nil))) then
        Exit;

      Lst := TStringList.Create;
      try
        Lst.Duplicates := dupIgnore;
        Lst.Sorted := True;
        for i := 0 to Docs.Count - 1 do
        begin
          if ((not Docs[i].GetInProcesses) and (Docs[i].HasPreview)) then
          begin
            Docs[i].StartProcess;
            Lst.Add(IntToStr(i));
            RefreshDocumentInPreviews(AThread, Docs[i], nil, nil, True, True);
          end
          else
            bitfarmLog.Write(llNormal, ltWarning, 'Doc.ID: ' + IntToStr(Docs[i].ID) +
              '; Doc.TempID: ' + GUIDToString(Docs[i].TempID) +
              _(' konnte nicht gedreht werden weil noch in Prozess ist!'), 'bfaProcessing');
        end;

        DocList := TList<TbfaDocument>.Create;

        for i := 0 to Docs.Count - 1 do
        begin
          if Lst.Find(IntToStr(i), LstIdx) then
          begin
            if Docs[i].HasPreview then
            begin
//30.05.2018 Ticket 14576              Doc := TbfaDocument.Create;
//              Doc.Clone(Docs[i], True);
//              DocList.Add(Doc);
              DocList.Add(Docs[i]);
            end;
          end;
        end;
      finally
        FreeAndNil(Lst);
      end;

      if DocList.Count > 0 then
      begin
        if (SingleView <> nil) then
          DocView := SingleView
        else
          DocView := Tfrm_Voransicht.Create(nil);

        AThread.Synchronize(nil,
          procedure
          begin
            // licount :=
            PreProcess(DocView.img_DocView);
          end);

        for i := 0 to DocList.Count - 1 do
        begin
          DocList[i].StartSaveProcess;
          try
            if AllPages then
            begin
              if (DocList[i].ScanPoolView <> nil) and (DocList[i].ScanPoolViewLoaded) then
                cnt := DocList[i].ScanPoolView.ImageCount
              else
              begin
                DocList[i].DocStream.Position := 0;
                DocView.img_DocView.IO.LoadFromStream(DocList[i].DocStream);
                cnt := DocView.img_DocView.IO.Params.ImageCount;
              end;

              for j := 0 to cnt - 1 do
              begin
                RotateOnePage(j, DocView, AThread, ThumbnailView, Grad, DocList[i], WithAnnos);
              end;
            end else
            begin
              for j := 0 to Pages.Count - 1 do
              begin
                RotateOnePage(StrToInt(Pages[j]), DocView, AThread, ThumbnailView, Grad, DocList[i], WithAnnos);
              end;
            end;
          finally
            DocList[i].Modified_Preview := True;
            DocList[i].EndSaveProcess;
            DocList[i].EndProcess;

            if (DocList[i].ScanPoolView <> nil) and (DocList[i].ScanPoolViewLoaded) then
            begin
              DocList[i].ScanPoolViewchanged := True;
              if (SingleView <> nil) and (SingleView.Owner <> nil) then
                RefreshDocumentInPreviews(AThread, DocList[i], TbfaPreview(SingleView.Owner), nil, True)  //Nur Objekt Eigenschaften werden aktualisiert!
              else
                RefreshDocumentInPreviews(AThread, DocList[i], nil, nil, False);  
            end else
              RefreshDocumentInPreviews(AThread, DocList[i], nil, nil, True);  //Nur Objekt Eigenschaften werden aktualisiert!
          end;

          if SingleView <> nil then
          begin
            AThread.Synchronize(nil,
              procedure
              begin
                TbfaPreview(SingleView.Owner).SetActions;
              end);
          end
          else
          begin
            AThread.Synchronize(nil,
              procedure
              begin
                DocView.Update;
              end);
          end;
        end;
      end;
    except
      bitfarmLog.Write(llNormal, ltException, 'Rotate(' + IntToStr(Grad) + ')', 'bfaProcessing');
    end;
  finally
    if (SingleView = nil) then
      FreeAndNil(DocView);
    bitfarmLog.DebugEnd(llDebug, 'Rotate(' + IntToStr(Grad) + ')', 'bfaProcessing');
  end;
end;

procedure ReloadLinks(Doc: TbfaDocument);
var
  qry: TUniQuery;
  idx: Integer;
begin
  SetLength(Doc.DocLinkList, 0);
  qry := TUniQuery.Create(nil);
  try
    qry.Connection := dmMain.cntMain;
//17.05.2024 -->
//    qry.SQL.Text := 'SELECT sd.refid, sd.refname, sd.gdocid, ' +
//      '(SELECT COUNT(*) FROM sys_doclink sd2 WHERE sd2.refname = sd.refname) as Cnt ' +
//      ' FROM sys_doclink sd WHERE sd.gdocid = ' + IntToStr(Doc.ID) +
//      ' GROUP BY sd.refname, sd.gdocid, sd.refid ORDER BY sd.refname;';

//    qry.SQL.Text := 'SELECT sd.refid, sd.refname, sd.gdocid, ' +
//      ' ((SELECT COUNT(gdocid) FROM sys_doclink sd2 WHERE sd2.refname = sd.refname) + ' +
//      ' (SELECT Count(sl.gdocid) FROM sys_symlink sl LEFT JOIN sys_doc sd1 ON (sl.gdocid = sd1.gdocid) ' +
//      '  WHere (sl.gdocid in ( SELECT sd3.gdocid FROM sys_doclink sd3 WHERE sd3.refname = sd.refname)) AND (sl.arcid <> sd1.arcid)))as Cnt ' +
//      ' FROM sys_doclink sd WHERE sd.gdocid = ' + IntToStr(Doc.ID) +
//      ' GROUP BY sd.refname, sd.gdocid, sd.refid ORDER BY sd.refname;';

    qry.SQL.Text := 'CALL sp_get_doclinklist(' + IntToStr(Doc.ID) + ',' + IntToStr(bitfarmApp.User.ID) + ');';
//17.05.2024 <--

    qry.Open;
    if qry.RecordCount > 0 then
    begin
      SetLength(Doc.DocLinkList, qry.RecordCount);
      qry.First;
      idx := 0;
      while not qry.Eof do
      begin
        with Doc.DocLinkList[idx] do
        begin
          RefName := qry.FieldByName('refname').AsWideString;
          Count := qry.FieldByName('cnt').AsInteger;
          GDocID := qry.FieldByName('gdocid').AsLargeInt;
          RefID := qry.FieldByName('refid').AsLargeInt;
        end;
        inc(idx);
        qry.Next;
      end;
      qry.Close;

    end;
  finally
    FreeAndNil(qry);
  end;
end;

procedure ShowStationery(PageIdx: Integer; imgtmp: TbfaImageEnVect; StatObj: TbfaStationery);
begin
  bitfarmLog.DebugBegin(llDebug, 'ShowStationery', 'bfaProcessing');
  try
    try
      if Length(StatObj.Layers) > imgtmp.LayersCount then
        Exit;

      if PageIdx = 0 then
      begin
        if (StatObj.Pages > 0) then
          imgtmp.Layers[StatObj.Layers[0]].Visible := True
        else
          imgtmp.Layers[StatObj.Layers[0]].Visible := False;
      end
      else if ((PageIdx > 0) and (PageIdx < imgtmp.IO.Params.ImageCount - 1)) then
      begin
        if (StatObj.Pages > 1) then
        begin
          imgtmp.Layers[StatObj.Layers[0]].Visible := False;
          imgtmp.Layers[StatObj.Layers[1]].Visible := True;
        end
        else
          imgtmp.Layers[StatObj.Layers[0]].Visible := True;
      end
      else if ((PageIdx = imgtmp.IO.Params.ImageCount - 1)) then
      begin
        if (StatObj.Pages = 3) then
        begin
          imgtmp.Layers[StatObj.Layers[0]].Visible := False;
          imgtmp.Layers[StatObj.Layers[1]].Visible := False;
          imgtmp.Layers[StatObj.Layers[2]].Visible := True;
        end
        else if (StatObj.Pages = 2) then
        begin
          imgtmp.Layers[StatObj.Layers[0]].Visible := False;
          imgtmp.Layers[StatObj.Layers[1]].Visible := True;
        end
        else
          imgtmp.Layers[StatObj.Layers[0]].Visible := True;
      end;
    except
      bitfarmLog.Write(llNormal, ltException, 'ShowStationery', 'bfaProcessing');
    end;
  finally
    bitfarmLog.DebugEnd(llDebug, 'ShowStationery', 'bfaProcessing');
  end;
end;

procedure SetColors(imgTmp: TbfaImageEnVect);
var
  iColors : Integer;
begin
  iColors := imgtmp.Proc.CalcImageNumColors;
  if (iColors < power(imgtmp.io.Params.BitsPerSample, imgtmp.io.Params.SamplesPerPixel) + 30) then
  begin
    if (imgtmp.io.Params.BitsPerSample <> 8) or (imgtmp.io.Params.SamplesPerPixel <> 3) then
    begin
      imgtmp.io.Params.BitsPerSample := 8;
      imgtmp.io.Params.SamplesPerPixel := 3;
      imgtmp.proc.convertTo24bit;
      case imgTmp.IO.Params.BitsPerSample * imgTmp.IO.Params.SamplesPerPixel of
        1:
          imgTmp.IO.Params.TIFF_Compression := ioTIFF_G4FAX;
        4, 8:
          imgTmp.IO.Params.TIFF_Compression := ioTIFF_LZW;
        24:
          imgTmp.IO.Params.TIFF_Compression := ioTIFF_LZW;
      else
        imgTmp.IO.Params.TIFF_Compression := ioTIFF_LZW;
      end;
    end;
  end;
end;

procedure DoOneImage(AThread: TThread; PageIdx: Integer; DoInsert: Boolean;
  imgTmp: TbfaImageEnVect;
  Docs: TList<TbfaDocument>; i, Layer, idx: Integer; Stationery, PrivAnno,
  PubAnno, AlwaysWithAnno: Boolean;  img, TmpMView: TbfaImageEnMView;
  newtags, tags: TIETIFFHandler; StatObj: TbfaStationery; var ipos: Integer);
var
  PageCnt: Integer;
  strtif : TMemoryStream;
  iColors, ipostmp : Integer;
  AnnoObj : TbfaAnnotation;
  iebmp2 : TIEBitmap;
begin
  bitfarmLog.DebugBegin(llDebug, 'DoOneImage', 'bfaProcessing');
  try
    try
      iColors := 0;
      AnnoObj := Docs[i].GetAnnotation(PageIdx, Layer, idx);
      if (Stationery) or ((PubAnno or PrivAnno) and (AnnoObj <> nil)) or
         (AlwaysWithAnno) then
      begin
        imgTmp.Clear;

        if (Docs[i] <> nil) and
           (Docs[i].ScanPoolView <> nil) and
           (Docs[i].ScanPoolViewLoaded) then
        begin
          imgtmp.IO.Params.ImageIndex := 0;
          iebmp2 := TIEBitmap.Create;
          try
            Docs[i].ScanPoolView.CopyToIEBitmap(PageIdx, iebmp2);
            {$IFDEF ImageEN1140}
            imgtmp.Bitmap.Assign(iebmp2.VclBitmap);
            {$ELSE}
            imgtmp.IO.Bitmap.Assign(iebmp2.VclBitmap);
            {$ENDIF}
          finally
            FreeandNil(iebmp2);
          end;
          imgTmp.IO.Params.TIFF_Compression := Docs[i].ScanPoolView.MIO.Params[PageIdx].TIFF_Compression;
          imgTmp.IO.Params.BitsPerSample := Docs[i].ScanPoolView.MIO.Params[PageIdx].BitsPerSample;
          imgTmp.IO.Params.SamplesPerPixel := Docs[i].ScanPoolView.MIO.Params[PageIdx].SamplesPerPixel;
          imgtmp.FitToHeight;
        end
        else
        begin
          imgtmp.IO.Params.ImageIndex := PageIdx;
          Docs[i].DocStream.Position := 0;
          imgtmp.IO.LoadFromStream(Docs[i].DocStream);
          imgtmp.FitToHeight;
        end;

        if Stationery then
        begin
          PageCnt := imgtmp.IO.Params.ImageCount;
          SetColors(imgtmp);

          if Length(aStationeries) > 0 then
          begin
            if AThread <> nil then
            begin
              AThread.Synchronize(nil,
              procedure
              begin
                bitfarmApp.LoadStationeryToDocument(Docs[i], '');
                StatObj.Load(imgtmp, PageCnt, '', Docs[i].LetterStream);

                if imgtmp.LayersCount > 1 then
                begin
                  ShowStationery(PageIdx, imgTmp, StatObj);
                  if PageIdx < High(StatObj.Layers) then
                    imgTmp.LayersMerge(0, StatObj.Layers[PageIdx])
                  else
                    imgTmp.LayersMerge(0, StatObj.Layers[High(StatObj.Layers)]);
                end;
              end);
            end
            else
            begin
              bitfarmApp.LoadStationeryToDocument(Docs[i], '');
              StatObj.Load(imgtmp, PageCnt, '', Docs[i].LetterStream);

              if imgtmp.LayersCount > 1 then
              begin
                ShowStationery(PageIdx, imgTmp, StatObj);
                if PageIdx < High(StatObj.Layers) then
                  imgTmp.LayersMerge(0, StatObj.Layers[PageIdx])
                else
                  imgTmp.LayersMerge(0, StatObj.Layers[High(StatObj.Layers)]);
              end;
            end;
          end;
        end;

        if PrivAnno or PubAnno then
        begin
          if AnnoObj <> nil then
          begin
            if AThread <> nil then
            begin
              AThread.Synchronize(nil,
              procedure
              begin
                SetColors(imgTmp);
                AnnoObj.AnnoStream.Position := 0;
                try
                  imgTmp.LoadFromStreamIEV(AnnoObj.AnnoStream);
                  imgTmp.ObjGraphicRender := False;
                  imgTmp.ObjAntialias := False;

                  imgTmp.CopyObjectsToBack(False);
                except
                  imgtmp.Clear;
                end;
              end);
            end
            else
            begin
              SetColors(imgTmp);
              AnnoObj.AnnoStream.Position := 0;
              try
                imgTmp.LoadFromStreamIEV(AnnoObj.AnnoStream);
                imgTmp.ObjGraphicRender := False;
                imgTmp.ObjAntialias := False;

                imgTmp.CopyObjectsToBack(False);
              except
                imgtmp.Clear;
              end;
            end;
          end;
        end;

        if AThread <> nil then
        begin
          AThread.Synchronize(nil,
          procedure
          begin
            CompressImage(iColors, imgTmp);
          end);
        end
        else
          CompressImage(iColors, imgTmp);

        if (img <> nil) then
        begin
          if AThread <> nil then
          begin
            ipostmp := ipos;
            AThread.Synchronize(nil,
            procedure
            begin
              if DoInsert then
                img.InsertImage(ipostmp);
              img.SetImage(ipostmp, imgtmp.Bitmap);

              img.MIO.Params[ipostmp].TIFF_Compression := imgTmp.IO.Params.TIFF_Compression;
              img.MIO.Params[ipostmp].BitsPerSample := imgTmp.IO.Params.BitsPerSample;
              img.MIO.Params[ipostmp].SamplesPerPixel := imgTmp.IO.Params.SamplesPerPixel;
            end);
          end
          else
          begin
            if DoInsert then
              img.InsertImage(ipos);
            img.SetImage(ipos, imgtmp.Bitmap);

            img.MIO.Params[ipos].TIFF_Compression := imgTmp.IO.Params.TIFF_Compression;
            img.MIO.Params[ipos].BitsPerSample := imgTmp.IO.Params.BitsPerSample;
            img.MIO.Params[ipos].SamplesPerPixel := imgTmp.IO.Params.SamplesPerPixel;
          end;
        end
        else
        begin
          strtif := TMemoryStream.Create;
          try
            imgTmp.IO.SaveToStreamTIFF(strtif);
            strtif.Position := 0;
            newtags.InsertTIFFStream(strtif, ipos);
          finally
            FreeAndNil(strtif);
          end;
        end;
      end
      else
      begin
        strtif := TMemoryStream.Create;
        try
          if tags <> nil then
          begin
            tags.WriteStream(strtif, PageIdx);
            strtif.Position := 0;
            newtags.InsertTIFFStream(strtif, ipos);
          end
          else if TmpMView <> nil then
          begin
            if (TmpMView.MIO.Params[PageIdx] <> nil) and (TmpMView.MIO.Params[PageIdx].FileType = ioTIFF) then
            begin
              TmpMView.SelectedImage := PageIdx;
              TmpMView.MIO.SaveToStreamTIFF(strtif, True);
            end
            else
              TmpMView.MIO.SaveToStreamTIFF(strtif); //z.B JPEG
            strtif.Position := 0;
            newtags.InsertTIFFStream(strtif, ipos);
          end;
        finally
          FreeAndNil(strtif);
        end;
      end;
      Inc(ipos);
    except
      bitfarmLog.Write(llNormal, ltException, 'DoOneImage', 'bfaProcessing');
    end;
  finally
    bitfarmLog.DebugEnd(llDebug, 'DoOneImage', 'bfaProcessing');
  end;
end;

function bfaImageRendering(AThread: TThread; Docs: TList<TbfaDocument>; Stationery, PrivAnno,
  PubAnno: Boolean; Layer: Integer = -1; Copies: Integer = 1; Sorting: Boolean = False;
  imgM: TbfaImageEnMView = nil; StationeryPath: WideString = '';
  AlwaysWithAnno: Boolean = False; Duplex: Boolean = False): TMemoryStream;
var
  MStream, EmptyStr: TMemoryStream;
  img, TmpMView: TbfaImageEnMView;
  imgTmp, imgDuplex: TbfaImageEnVect;
  i, j, k, l, h, ipos, imgcount, idx, MViewPos,
  ActPage, imgCnt, PCount: Integer;
  StatObj: TbfaStationery;
  newtags, tags: TIETIFFHandler;
  nottif: Boolean;
  comp: Variant;
  iebmp: TIEBitmap;
  bmp: Graphics.TBitmap;

  function IsPageSelected(Doc: TbfaDocument; imgpos: Integer): Boolean;
  var
    found: Boolean;
    imgidx : Integer;
  begin
    try
      imgpos := 0;
      found := False;
      if (Doc.SelectedPages.Count > 0) then
      begin
        imgidx := imgpos;
        while ((not found) and (imgidx <= Doc.SelectedPages.Count - 1)) do
        begin
          if StrToInt(Doc.SelectedPages[imgidx]) = ActPage then
            found := True
          else
            inc(imgidx);
        end;
      end;

      Result := found;
      bitfarmLog.Write(llDebug, ltInfo, 'bfaImageRendering - IsPageSelected: ' +
        BoolToStr(Result, True) , 'bfaProcessing');
    except
      Result := False;
      bitfarmLog.Write(llNormal, ltException, 'bfaImageRendering - IsPageSelected ', 'bfaProcessing');
    end;
  end;

begin
  bitfarmLog.DebugBegin(llDebug, 'bfaImageRendering', 'bfaProcessing');
  Result := nil;
  imgtmp := nil;
  try
    try         
      if BD.SetDuplex then
        Duplex := True; //Wichtig beim testen!

      bitfarmLog.Write(llDebug, ltInfo, 'bfaImageRendering - 1', 'bfaProcessing');
      newtags := nil;
      tags := nil;
      ActPage := 1;
      img := imgM;
      try
        ipos := 0;
        bitfarmLog.Write(llDebug, ltInfo, 'bfaImageRendering - 2', 'bfaProcessing');
        imgCnt := 0;
        for i := 0 to Docs.Count - 1 do
          if Docs[i].SelectedPages.Count > 0 then
            imgCnt := imgCnt + Docs[i].SelectedPages.Count;

        bitfarmLog.Write(llDebug, ltInfo, 'bfaImageRendering - imgcnt: ' + IntToStr(imgcnt) +
          '; Docs.Count: ' + IntToStr(Docs.Count), 'bfaProcessing');
        bitfarmLog.Write(llDebug, ltInfo, 'bfaImageRendering - Stationery: ' + BoolToStr(Stationery, True) +
          '; PrivAnno: ' + BoolToStr(PrivAnno, True) +
          '; PubAnno: ' + BoolToStr(PubAnno, True) +
          '; Docs[0].HasAnnotation: ' + BoolToStr(Docs[0].HasAnnotation, True) +
          '; Sorting: ' + BoolToStr(Sorting, True) +
          '; Copies: ' + IntToStr(Copies)
          , 'bfaProcessing');
        if (imgCnt = 0) and (Docs.Count = 1) and
           (
              ((not Stationery) and (StationeryPath = '')) and
              ((not PrivAnno) or (not Docs[0].HasAnnotation)) and
              ((not PubAnno) or (not Docs[0].HasAnnotation)) and
              (Copies <= 1) and (not Sorting))
        then
        begin
          bitfarmLog.Write(llDebug, ltInfo, 'MStream := TMemoryStream.Create;', 'bfaProcessing');
          MStream := TMemoryStream.Create;

          if (Docs[0] <> nil) and
             (Docs[0].ScanPoolView <> nil) and (Docs[0].ScanPoolViewLoaded)
             and (Docs[0].ScanPoolViewchanged) then
          begin
            Docs[0].ScanPoolView.MIO.SaveToStreamTIFF(MStream);
          end else
          begin
            Docs[0].DocStream.Position := 0;
            MStream.CopyFrom(Docs[0].DocStream, Docs[0].DocStream.Size);
          end;

          MStream.Position := 0;
          bitfarmLog.Write(llDebug, ltInfo, 'MStream.Size: ' + IntToStr(MStream.size), 'bfaProcessing');

          if imgM <> nil then
          begin
            bitfarmLog.Write(llDebug, ltInfo, '(imgM <> nil)', 'bfaProcessing');
            imgM.Clear;
            bitfarmLog.Write(llDebug, ltInfo, 'imgM.MIO.LoadFromStream(MStream)', 'bfaProcessing');
            imgM.MIO.LoadFromStream(MStream); //weil Durch imgM wird nachher gedruckt
            MStream.Position := 0;
            bitfarmLog.Write(llDebug, ltInfo, '(imgM <> nil) - MStream.Size: ' + IntToStr(MStream.size), 'bfaProcessing');
          end;
          Result := MStream;
        end
        else
        begin
          bitfarmLog.Write(llDebug, ltInfo, 'ELSE', 'bfaProcessing');
          if (img <> nil) and (imgCnt > 0) then  // wenn selectedpages > 0 dann muss wegen die selectierte Seiten reinfolge erst erstellen
          begin
            for j := 1 to Copies do
              for i := 0 to imgCnt - 1 do
                img.InsertImage(i);
          end;

//29.08.2023 -->
//          if AThread <> nil then
//          begin
//            AThread.Synchronize(nil,  //09.06.2023
//              procedure
//              begin
//                newtags := TIETIFFHandler.Create;
//                TmpMView := TbfaImageEnMView.Create(nil);
//                TmpMView.EnableMultiSelect := True;
//              end);
//          end
//          else
//          begin
//            newtags := TIETIFFHandler.Create;
//            TmpMView := TbfaImageEnMView.Create(nil);
//            TmpMView.EnableMultiSelect := True;
//          end;
          {$IFDEF IMAGEEN_SYNCHRONIZED}
          if AThread <> nil then
          begin
            AThread.Synchronize(nil,  //09.06.2023
              procedure
              begin
                newtags := TIETIFFHandler.Create;
                TmpMView := TbfaImageEnMView.Create(nil);
                TmpMView.EnableMultiSelect := True;
              end);
          end
          else
          begin
            newtags := TIETIFFHandler.Create;
            TmpMView := TbfaImageEnMView.Create(nil);
            TmpMView.EnableMultiSelect := True;
          end;
          {$ELSE}
            newtags := TIETIFFHandler.Create;
            TmpMView := TbfaImageEnMView.Create(nil);
            TmpMView.EnableMultiSelect := True;
          {$ENDIF}
//29.08.2023 <--
          try
            bitfarmLog.Write(llDebug, ltInfo, 'newtags, TmpMView created', 'bfaProcessing');
            if ((Copies = 1) or (Sorting)) then
            begin
              bitfarmLog.Write(llDebug, ltInfo, ' ((Copies = 1) or (Sorting)) ', 'bfaProcessing');
              bitfarmLog.Write(llDebug, ltInfo, ' Copies = ' + IntToStr(Copies), 'bfaProcessing');
              bitfarmLog.Write(llDebug, ltInfo, ' Sorting : ' + BoolToStr(Sorting, True), 'bfaProcessing');
              for i := 0 to Docs.Count - 1 do
              begin
                nottif := True;
                if (Docs[i] <> nil) and
                   (Docs[i].ScanPoolView <> nil) and (Docs[i].ScanPoolViewLoaded) then
                begin
                  bitfarmLog.Write(llDebug, ltInfo, '(Docs[i].FileType <> .tif) and (ScanPoolView)', 'bfaProcessing');
//29.08.2023 -->
//                  if AThread <> nil then
//                  begin
//                    AThread.Synchronize(nil,  //09.06.2023
//                      procedure
//                      begin
//                        TmpMView.Clear;
//                      end);
//                  end
//                  else
//                    TmpMView.Clear;
                  {$IFDEF IMAGEEN_SYNCHRONIZED}
                  if AThread <> nil then
                  begin
                    AThread.Synchronize(nil,  //09.06.2023
                      procedure
                      begin
                        TmpMView.Clear;
                      end);
                  end
                  else
                    TmpMView.Clear;
                  {$ELSE}
                  TmpMView.Clear;
                  {$ENDIF}
//29.08.2023 <--

                  PCount := Docs[i].ScanPoolView.ImageCount;
                  MViewPos := 0;
                  for k := 1 to Copies do
                  begin
                    for l := 0 to Docs[i].ScanPoolView.ImageCount - 1 do
                    begin
                      TmpMView.LockPaint;
                      iebmp := TIEBitmap.Create;
                      bmp := Graphics.TBitmap.Create;
                      try
                        Docs[i].ScanPoolView.CopyToIEBitmap(l, iebmp);
                        TmpMView.InsertImageEx(MViewPos);
                        TmpMView.SetIEBitmap(MViewPos, iebmp);

                        TmpMView.Mio.Params[MViewPos].TIFF_Compression :=
                          Docs[i].ScanPoolView.MIO.Params[l].TIFF_Compression;
                        TmpMView.Mio.Params[MViewPos].BitsPerSample :=
                          Docs[i].ScanPoolView.MIO.Params[l].BitsPerSample;
                        TmpMView.Mio.Params[MViewPos].SamplesPerPixel :=
                          Docs[i].ScanPoolView.MIO.Params[l].SamplesPerPixel;
                        TmpMView.Mio.Params[MViewPos].FileType := ioTIFF;
                        inc(MViewPos);
                      finally
                        TmpMView.UnLockPaint;
                        TmpMView.Update;
                        FreeAndNil(iebmp);
                        FreeAndNil(bmp);
                      end;
                    end;
                  end;
                end
                else
                begin
                  if (Docs[i].FileType = '.tif') or (Docs[i].FileType = '.tiff') then
                  begin
                    bitfarmLog.Write(llDebug, ltInfo, 'Docs[i].FileType = .tif', 'bfaProcessing');
                    tags := TIETIFFHandler.Create;
                    Docs[i].DocStream.Position := 0;
                    tags.ReadStream(Docs[i].DocStream);
                    PCount := tags.GetPagesCount;
                    comp := tags.GetValue(0, tags.FindTag(0,259), 0);
                    if (PCount = 0) or (comp = ioTIFF_JPEG) or (comp = ioTIFF_OLDJPEG) then //28.09.2018 wegen oldjpeg tiff reingekommen, sonst absturz in DrawOneImage
                      FreeAndNil(tags)
                    else
                    begin
                      nottif := False;
                      for k := 2 to Copies do
                      begin
                        Docs[i].DocStream.Position := 0;
                        tags.ReadStream(Docs[i].DocStream);
                      end;
                    end;
                  end;
                  if nottif then
                  begin
                    bitfarmLog.Write(llDebug, ltInfo, 'Docs[i].FileType <> .tif', 'bfaProcessing');
                    TmpMView.Clear;
                    Docs[i].DocStream.Position := 0;
                    TmpMView.MIO.LoadFromStream(Docs[i].DocStream);
                    PCount := TmpMView.ImageCount;
                    for k := 2 to Copies do
                    begin
                      Docs[i].DocStream.Position := 0;
                      TmpMView.MIO.LoadFromStream(Docs[i].DocStream);
                    end;
                  end;
                end;
                bitfarmLog.Write(llDebug, ltInfo, 'PCount: ' + IntToStr(PCount), 'bfaProcessing');
                try
                  imgtmp := bfaCommonImageEn.CreateInvisibleVect(nil);
                  try
                    for k := 1 to Copies do
                    begin
                      StatObj := TbfaStationery.Create;
                      try
                        imgcount := PCount;

                        if (Docs[i].SelectedPages.Count <> 0) then // sind ausgewhlte Seiten
                        begin
                          for h := 0 to Docs[i].SelectedPages.Count - 1 do
                          begin
                            j := StrToInt(Docs[i].SelectedPages[h]) - 1;
                            if AThread <> nil then
                            begin
                              AThread.Synchronize(nil,
                              procedure
                              begin
                                DoOneImage(nil, j, (imgCnt = 0), imgTmp, Docs, i, Layer, idx,
                                  Stationery, PrivAnno, PubAnno, AlwaysWithAnno, img,
                                  TmpMView, newtags, tags, StatObj, ipos);
                              end);
                            end
                            else
                              DoOneImage(AThread, j, (imgCnt = 0), imgTmp, Docs, i, Layer, idx,
                                Stationery, PrivAnno, PubAnno, AlwaysWithAnno, img,
                                TmpMView, newtags, tags, StatObj, ipos);

                            inc(ActPage);
                          end;                        
                        end
                        else
                        begin
                        for j := 0 to imgCount - 1 do
                        begin
                            if ((Docs[i].SelectedPages.Count = 0) or (IsPageSelected(Docs[i], j))) then
                          begin
                            if AThread <> nil then
                            begin
                              AThread.Synchronize(nil,
                              procedure
                              begin
                                DoOneImage(nil, j, (imgCnt = 0), imgTmp, Docs, i, Layer, idx,
                                  Stationery, PrivAnno, PubAnno, AlwaysWithAnno, img,
                                  TmpMView, newtags, tags, StatObj, ipos);
                              end);
                            end
                            else
                              DoOneImage(AThread, j, (imgCnt = 0), imgTmp, Docs, i, Layer, idx,
                                Stationery, PrivAnno, PubAnno, AlwaysWithAnno, img,
                                TmpMView, newtags, tags, StatObj, ipos);
                          end;

                          inc(ActPage);
                        end;
                        end;
                      finally
                        FreeAndNil(StatObj);
                      end;

                      if Duplex and 
                         (((Docs[i].SelectedPages.Count = 0) and (PCount mod 2 <> 0)) or 
                         ((Docs[i].SelectedPages.Count <> 0) and (Docs[i].SelectedPages.Count mod 2 <> 0)))
                      then
                      begin
                        imgDuplex := TbfaImageEnVect.Create(nil);
                        EmptyStr := TMemoryStream.Create;
                        try
                          imgDuplex.Visible := False;
                          imgDuplex.IO.AutoAdjustDPI := True;
                          imgDuplex.IO.Params.OriginalWidth := 0;
                          imgDuplex.IO.Params.OriginalHeight := 0;
                          CreateEmptyImage(imgDuplex, '', clWhite);
                          imgDuplex.IO.SaveToStreamTIFF(EmptyStr);
                          EmptyStr.Position := 0;
                          newtags.InsertTIFFStream(EmptyStr, ipos);
                          inc(ipos);
                          inc(ActPage);
                        finally
                          imgDuplex.Free;
                          FreeAndNil(EmptyStr);
                        end;
                      end;
                    end;
                  finally
                    imgtmp.Free;
                    imgTmp := nil;
                  end;
                finally
                  if tags <> nil then
                    FreeAndNil(tags);
                end;
              end;
            end
            else
            begin
              bitfarmLog.Write(llDebug, ltInfo, ' NOT! ((Copies = 1) or (Sorting)) ', 'bfaProcessing');
              bitfarmLog.Write(llDebug, ltInfo, ' Copies = ' + IntToStr(Copies), 'bfaProcessing');
              bitfarmLog.Write(llDebug, ltInfo, ' Sorting : ' + BoolToStr(Sorting, True), 'bfaProcessing');
              for i := 0 to Docs.Count - 1 do
              begin
                imgtmp := bfaCommonImageEn.CreateInvisibleVect(nil);

                StatObj := TbfaStationery.Create;
                try
                  nottif := True;
                  if (Docs[i] <> nil) and
                     (Docs[i].ScanPoolView <> nil) and (Docs[i].ScanPoolViewLoaded) then
                  begin
                    bitfarmLog.Write(llDebug, ltInfo, '(ScanPoolView)', 'bfaProcessing');
                    PCount := Docs[i].ScanPoolView.ImageCount;
                    TmpMView.Clear;
                    MViewPos := 0;
                    for k := 1 to Copies do
                    begin
                      for l := 0 to Docs[i].ScanPoolView.ImageCount - 1 do
                      begin
                        TmpMView.LockPaint;
                        iebmp := TIEBitmap.Create;
                        bmp := Graphics.TBitmap.Create;
                        try
                          Docs[i].ScanPoolView.CopyToIEBitmap(l, iebmp);
                          TmpMView.InsertImageEx(MViewPos);
                          TmpMView.SetIEBitmap(MViewPos, iebmp);

                          TmpMView.Mio.Params[MViewPos].TIFF_Compression :=
                            Docs[i].ScanPoolView.MIO.Params[l].TIFF_Compression;
                          TmpMView.Mio.Params[MViewPos].BitsPerSample :=
                            Docs[i].ScanPoolView.MIO.Params[l].BitsPerSample;
                          TmpMView.Mio.Params[MViewPos].SamplesPerPixel :=
                            Docs[i].ScanPoolView.MIO.Params[l].SamplesPerPixel;
                          TmpMView.Mio.Params[MViewPos].FileType := ioTIFF;
                          inc(MViewPos);
                        finally
                          TmpMView.UnLockPaint;
                          TmpMView.Update;
                          FreeAndNil(iebmp);
                          FreeAndNil(bmp);
                        end;
                      end;
                    end;                  
                  end
                  else
                  begin
                    if (Docs[i].FileType = '.tif') or (Docs[i].FileType = '.tiff') then
                    begin
                      bitfarmLog.Write(llDebug, ltInfo, 'Docs[i].FileType = .tif', 'bfaProcessing');
                      tags := TIETIFFHandler.Create;
                      Docs[i].DocStream.Position := 0;
                      tags.ReadStream(Docs[i].DocStream);
                      PCount := tags.GetPagesCount;
                      comp := tags.GetValue(0, tags.FindTag(0,259), 0);
                      if (PCount = 0) or (comp = ioTIFF_JPEG) or (comp = ioTIFF_OLDJPEG) then //28.09.2018 wegen oldjpeg tiff reingekommen, sonst absturz in DrawOneImage
                        FreeAndNil(tags)
                      else
                        nottif := False
                    end;
                    if nottif then
                    begin
                      bitfarmLog.Write(llDebug, ltInfo, 'Docs[i].FileType <> .tif', 'bfaProcessing');
                      TmpMView.Clear;
                      Docs[i].DocStream.Position := 0;
                      TmpMView.MIO.LoadFromStream(Docs[i].DocStream);
                      PCount := TmpMView.ImageCount;
                    end;
                  end;

                  bitfarmLog.Write(llDebug, ltInfo, 'PCount: ' + IntToStr(PCount), 'bfaProcessing');
                  imgcount := PCount;
                  if Duplex then
                  begin
                    if (Docs[i].SelectedPages.Count <> 0) then // es gibt ausgewhlte Seiten
                    begin
                      h := 0; 
                      while (h <= Docs[i].SelectedPages.Count - 1) do
                      begin
                        for k := 1 to Copies do
                        begin
                          ActPage := 0;
                          if ((Docs[i].SelectedPages.Count = 0) or (IsPageSelected(Docs[i], ipos))) then
                            if AThread <> nil then
                            begin
                              AThread.Synchronize(nil,
                              procedure
                              begin
                                  j := StrToInt(Docs[i].SelectedPages[h]) - 1;
                                DoOneImage(nil, j, (imgCnt = 0), imgTmp, Docs, i, Layer, idx,
                                  Stationery, PrivAnno, PubAnno, AlwaysWithAnno, img,
                                  TmpMView, newtags, tags, StatObj, ipos);
                                  inc(h);
                                  inc(ActPage);
                                  if h < Docs[i].SelectedPages.Count - 1 then
                                  begin
                                    j := StrToInt(Docs[i].SelectedPages[h]) - 1;
                                    DoOneImage(nil, j, (imgCnt = 0), imgTmp, Docs, i, Layer, idx,
                                      Stationery, PrivAnno, PubAnno, AlwaysWithAnno, img,
                                      TmpMView, newtags, tags, StatObj, ipos);
                                    inc(h);
                                    inc(ActPage);
                                  end;
                              end);
                            end
                            else
                            begin
                              j := StrToInt(Docs[i].SelectedPages[h]) - 1;
                              DoOneImage(AThread, j, (imgCnt = 0), imgTmp, Docs, i, Layer, idx,
                              Stationery, PrivAnno, PubAnno, AlwaysWithAnno, img,
                              TmpMView, newtags, tags, StatObj, ipos);
                                inc(h);
                                inc(ActPage);
                              if h < Docs[i].SelectedPages.Count - 1 then
                              begin
                                j := StrToInt(Docs[i].SelectedPages[h]) - 1;
                                DoOneImage(AThread, j, (imgCnt = 0), imgTmp, Docs, i, Layer, idx,
                                  Stationery, PrivAnno, PubAnno, AlwaysWithAnno, img,
                                  TmpMView, newtags, tags, StatObj, ipos);
                                inc(h);
                                inc(ActPage);
                              end;
                            end;

                          if Duplex and ((Docs[i].SelectedPages.Count) mod 2 <> 0) and (h = Docs[i].SelectedPages.Count)then
                          begin
                            imgDuplex := TbfaImageEnVect.Create(nil);
                            EmptyStr := TMemoryStream.Create;
                            try
                              imgDuplex.Visible := False;
                              imgDuplex.IO.AutoAdjustDPI := True;
                              imgDuplex.IO.Params.OriginalWidth := 0;
                              imgDuplex.IO.Params.OriginalHeight := 0;
                              CreateEmptyImage(imgDuplex, '', clWhite);
                              imgDuplex.IO.SaveToStreamTIFF(EmptyStr);
                              EmptyStr.Position := 0;
                              newtags.InsertTIFFStream(EmptyStr, ipos);
                              inc(ipos);
                            finally
                              imgDuplex.Free;
                              FreeAndNil(EmptyStr);
                            end;
                          end;
                          if k < Copies then
                            dec(h, ActPage);
                        end;
                      end;                        
                    end
                    else
                    begin
                      j := 0;
                      while (j <= imgcount - 1) do
                      begin
                        for k := 1 to Copies do
                        begin
                          ActPage := 0;
                          if ((Docs[i].SelectedPages.Count = 0) or (IsPageSelected(Docs[i], j))) then
                            if AThread <> nil then
                            begin
                              AThread.Synchronize(nil,
                              procedure
                              begin
                                DoOneImage(nil, j, (imgCnt = 0), imgTmp, Docs, i, Layer, idx,
                                  Stationery, PrivAnno, PubAnno, AlwaysWithAnno, img,
                                  TmpMView, newtags, tags, StatObj, ipos);
                                inc(j);
                                inc(ActPage);
                                if j <= imgcount - 1 then
                                begin
                                  DoOneImage(nil, j, (imgCnt = 0), imgTmp, Docs, i, Layer, idx,
                                    Stationery, PrivAnno, PubAnno, AlwaysWithAnno, img,
                                    TmpMView, newtags, tags, StatObj, ipos);
                                  inc(j);
                                  inc(ActPage);
                                end;
                              end);
                            end
                            else
                            begin
                              DoOneImage(AThread, j, (imgCnt = 0), imgTmp, Docs, i, Layer, idx,
                                Stationery, PrivAnno, PubAnno, AlwaysWithAnno, img,
                                TmpMView, newtags, tags, StatObj, ipos);
                                inc(j);
                                inc(ActPage);
                              if j <= imgcount - 1 then
                              begin
                                DoOneImage(AThread, j, (imgCnt = 0), imgTmp, Docs, i, Layer, idx,
                                  Stationery, PrivAnno, PubAnno, AlwaysWithAnno, img,
                                  TmpMView, newtags, tags, StatObj, ipos);
                                inc(j);
                                inc(ActPage);
                              end;
                            end;
                          if Duplex and (PCount mod 2 <> 0) and (j = imgCount)then
                          begin
                            imgDuplex := TbfaImageEnVect.Create(nil);
                            EmptyStr := TMemoryStream.Create;
                            try
                              imgDuplex.Visible := False;
                              imgDuplex.IO.AutoAdjustDPI := True;
                              imgDuplex.IO.Params.OriginalWidth := 0;
                              imgDuplex.IO.Params.OriginalHeight := 0;
                              CreateEmptyImage(imgDuplex, '', clWhite);
                              imgDuplex.IO.SaveToStreamTIFF(EmptyStr);
                              EmptyStr.Position := 0;
                              newtags.InsertTIFFStream(EmptyStr, ipos);
                              inc(ipos);
                            finally
                              imgDuplex.Free;
                              FreeAndNil(EmptyStr);
                            end;
                          end;
                          if k < Copies then
                            dec(j, ActPage);
                        end;
                      end;
                    end;
                  end
                  else
                  begin
                    if (Docs[i].SelectedPages.Count <> 0) then // es gibt ausgewhlte Seiten
                    begin
                      for h := 0 to Docs[i].SelectedPages.Count - 1 do
                      begin
                        j := StrToInt(Docs[i].SelectedPages[h]) - 1;
                        for k := 1 to Copies do
                        begin
                          if AThread <> nil then
                          begin
                            AThread.Synchronize(nil,
                            procedure
                            begin
                              DoOneImage(nil, j, (imgCnt = 0), imgTmp, Docs, i, Layer, idx,
                                Stationery, PrivAnno, PubAnno, AlwaysWithAnno, img,
                                TmpMView, newtags, tags, StatObj, ipos);
                            end);
                          end
                          else
                            DoOneImage(AThread, j, (imgCnt = 0), imgTmp, Docs, i, Layer, idx,
                              Stationery, PrivAnno, PubAnno, AlwaysWithAnno, img,
                              TmpMView, newtags, tags, StatObj, ipos);

                          inc(ActPage);
                        end;
                      end;                        
                    end
                    else
                    begin
                      for j := 0 to imgCount - 1 do
                      begin
                        for k := 1 to Copies do
                        begin
                          if ((Docs[i].SelectedPages.Count = 0) or (IsPageSelected(Docs[i], j))) then
                            if AThread <> nil then
                            begin
                              AThread.Synchronize(nil,
                              procedure
                              begin
                                DoOneImage(nil, j, (imgCnt = 0), imgTmp, Docs, i, Layer, idx,
                                  Stationery, PrivAnno, PubAnno, AlwaysWithAnno, img,
                                  TmpMView, newtags, tags, StatObj, ipos);
                              end);
                            end
                            else
                              DoOneImage(AThread, j, (imgCnt = 0), imgTmp, Docs, i, Layer, idx,
                                Stationery, PrivAnno, PubAnno, AlwaysWithAnno, img,
                                TmpMView, newtags, tags, StatObj, ipos);
                          inc(ActPage);
                        end;
                      end;
                    end;
                  end;
                finally
                  FreeAndNil(StatObj);
                  imgtmp.Free;
                  imgtmp := nil;
                  if tags <> nil then
                    FreeAndNil(tags);
                end;
              end;
            end;
          finally
            if TmpMView <> nil then
            begin
//29.08.2023 -->
//              if AThread <> nil then
//              begin
//                AThread.Synchronize(nil, //09.06.2023
//                procedure
//                begin
//                  TmpMView.Free;
//                  TmpMView := nil;
//                end);
//              end
//              else
//              begin
//                TmpMView.Free;
//                TmpMView := nil;
//              end;
              {$IFDEF IMAGEEN_SYNCHRONIZED}
              if AThread <> nil then
              begin
                AThread.Synchronize(nil, //09.06.2023
                procedure
                begin
                  TmpMView.Free;
                  TmpMView := nil;
                end);
              end
              else
              begin
                TmpMView.Free;
                TmpMView := nil;
              end;
              {$ELSE}
              TmpMView.Free;
              TmpMView := nil;
              {$ENDIF}
//29.08.2023 <--
            end;
          end;
          if (imgM = nil) then
          begin
            bitfarmLog.Write(llDebug, ltInfo, ' (imgM = nil) ', 'bfaProcessing');

            MStream := TMemoryStream.Create;
            if (img <> nil) then
            begin
              bitfarmLog.Write(llDebug, ltInfo, 'img.MIO.SaveToStreamTIFF(MStream);', 'bfaProcessing');
              img.MIO.SaveToStreamTIFF(MStream);
            end
            else
            begin
              bitfarmLog.Write(llDebug, ltInfo, 'newtags.WriteStream(MStream);', 'bfaProcessing');
//29.08.2023 -->
//              if AThread <> nil then
//              begin
//                AThread.Synchronize(nil, //09.06.2023
//                procedure
//                begin
//                  newtags.WriteStream(MStream);
//                end);
//              end
//              else
//                newtags.WriteStream(MStream);
              {$IFDEF IMAGEEN_SYNCHRONIZED}
              if AThread <> nil then
              begin
                AThread.Synchronize(nil, //09.06.2023
                procedure
                begin
                  newtags.WriteStream(MStream);
                end);
              end
              else
                newtags.WriteStream(MStream);
              {$ELSE}
              newtags.WriteStream(MStream);
              {$ENDIF}
//29.08.2023 <--
            end;
            MStream.Position := 0;

            Result := MStream;
            bitfarmLog.Write(llDebug, ltInfo, ' Result := MStream; ', 'bfaProcessing');
          end
          else if (imgM <> nil) and (newtags <> nil) and (newtags.GetPagesCount > 0) then
          begin
            bitfarmLog.Write(llDebug, ltInfo, ' (imgM <> nil) ', 'bfaProcessing');
            MStream := TMemoryStream.Create;
            bitfarmLog.Write(llDebug, ltInfo, 'newtags.WriteStream(MStream);', 'bfaProcessing');
            newtags.WriteStream(MStream);
            MStream.Position := 0;

            if imgM <> nil then
            begin
              bitfarmLog.Write(llDebug, ltInfo, ' imgM <> nil ', 'bfaProcessing');
              imgM.Clear;
              imgM.MIO.LoadFromStream(MStream); //weil Durch imgM wird nachher gedruckt
              MStream.Position := 0;
            end;

            Result := MStream;
            bitfarmLog.Write(llDebug, ltInfo, ' Result := MStream; ', 'bfaProcessing');
          end
          else
            Result := nil;
        end;

      finally
        bitfarmLog.Write(llDebug, ltInfo, 'finally end;', 'bfaProcessing');
//29.08.2023 -->
//        if AThread <> nil then
//        begin
//          AThread.Synchronize(nil, //09.06.2023
//          procedure
//          begin
//            if (imgM = nil) and (img <> nil) then
//            begin
//              img.Free;
//              img := nil;
//            end;
//            if (imgtmp <> nil) then
//            begin
//              imgtmp.Free;
//              imgTmp := nil;
//            end;
//            if newtags <> nil then
//              FreeAndNil(newtags);
//          end);
//        end
//        else
//        begin
//          if (imgM = nil) and (img <> nil) then
//          begin
//            img.Free;
//            img := nil;
//          end;
//          if (imgtmp <> nil) then
//          begin
//            imgtmp.Free;
//            imgTmp := nil;
//          end;
//          if newtags <> nil then
//            FreeAndNil(newtags);
//        end;
        {$IFDEF IMAGEEN_SYNCHRONIZED}
        if AThread <> nil then
        begin
          AThread.Synchronize(nil, //09.06.2023
          procedure
          begin
            if (imgM = nil) and (img <> nil) then
            begin
              img.Free;
              img := nil;
            end;
            if (imgtmp <> nil) then
            begin
              imgtmp.Free;
              imgTmp := nil;
            end;
            if newtags <> nil then
              FreeAndNil(newtags);
          end);
        end
        else
        begin
          if (imgM = nil) and (img <> nil) then
          begin
            img.Free;
            img := nil;
          end;
          if (imgtmp <> nil) then
          begin
            imgtmp.Free;
            imgTmp := nil;
          end;
          if newtags <> nil then
            FreeAndNil(newtags);
        end;
        {$ELSE}
        if (imgM = nil) and (img <> nil) then
        begin
          img.Free;
          img := nil;
        end;
        if (imgtmp <> nil) then
        begin
          imgtmp.Free;
          imgTmp := nil;
        end;
        if newtags <> nil then
          FreeAndNil(newtags);
        {$ENDIF}
//29.08.2023 <--
      end;
    except
      Result := nil;
      bitfarmLog.Write(llNormal, ltException, 'bfaImageRendering', 'bfaProcessing');
    end;
  finally
    bitfarmLog.DebugEnd(llDebug, 'bfaImageRendering', 'bfaProcessing');
  end;
end;

end.




