unit bfaCalloutPopup;

interface

uses
  Classes, dxCalloutPopup, cxLabel, System.SysUtils, Vcl.ExtCtrls, Controls,
  CalloutFrame, System.Types;

type
  TbfaCalloutPopup = class(TdxCalloutPopup)
  private
    FTopPos, FMaxWidth : Integer;
    FCalloutFrame : TfraCallout;
    FTimer : TTimer;
    FLabelPublic, FLabelPrivate, FLabelShared, FLabelComment : TcxLabel;  //23.08.2024 SD -->bfacrm:ticket__show__39650
    procedure DoTimer(Sender : TObject);
    procedure DoCloseClick(Sender : TObject);
  protected
    procedure InitializeCalloutWindow; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy;
    procedure Popup(AOwnerControl: TWinControl); overload;
    procedure Popup(AOwnerControl: TWinControl; const AOwnerBounds : TRect); overload;
    procedure SetCaptions(ValuePublic, ValuePrivate, ValueShared, ValueComment : string); //23.08.2024 SD -->bfacrm:ticket__show__39650
    property CalloutFrame : TfraCallout read FCalloutFrame;
  end;

  TdxCustomCalloutPopupWindowAccess = class(TdxCustomCalloutPopupWindow);

implementation

uses
  bfaCommonUtils;

{ TbfaCalloutPopup }

constructor TbfaCalloutPopup.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FCalloutFrame := TfraCallout.Create(nil);
  FCalloutFrame.btnClose.OnClick := DoCloseClick;

  FTimer := TTimer.Create(nil);
  FTimer.Enabled := False;
  FTimer.Interval := 5000;
  FTimer.OnTimer := DoTimer;

  FLabelPublic := TcxLabel.Create(Self);
  FLabelPublic.Parent := FCalloutFrame;
  FLabelPublic.Caption := 'Label1';
  FLabelPublic.Visible := False;
  FLabelPublic.Left := 40;
  FLabelPublic.AutoSize := True;
  FLabelPublic.Transparent := True;
  FLabelPublic.Style.Font.Size := bfaFontSize;
  FLabelPrivate := TcxLabel.Create(Self);
  FLabelPrivate.Parent := FCalloutFrame;
  FLabelPrivate.Visible := False;
  FLabelPrivate.Left := 40;
  FLabelPrivate.AutoSize := True;
  FLabelPrivate.Transparent := True;
  FLabelPrivate.Style.Font.Size := bfaFontSize;
  FLabelPrivate.Caption := 'Label1';
  //23.08.2024 SD -->bfacrm:ticket__show__39650
  FLabelShared := TcxLabel.Create(Self);
  FLabelShared.Parent := FCalloutFrame;
  FLabelShared.Visible := False;
  FLabelShared.Left := 40;
  FLabelShared.AutoSize := True;
  FLabelShared.Transparent := True;
  FLabelShared.Style.Font.Size := bfaFontSize;
  FLabelShared.Caption := 'Label1';
  //23.08.2024 SD -->bfacrm:ticket__show__39650
  FLabelComment := TcxLabel.Create(Self);
  FLabelComment.Parent := FCalloutFrame;
  FLabelComment.Visible := False;
  FLabelComment.Left := 40;
  FLabelComment.AutoSize := True;
  FLabelComment.Transparent := True;
  FLabelComment.Style.Font.Size := bfaFontSize;
  FLabelComment.Caption := 'Label1';

  Self.Rounded := True;
  Self.LookAndFeel.NativeStyle := False;
  Self.PopupControl := FCalloutFrame;
  Self.Alignment := cpaTopLeft;
end;

destructor TbfaCalloutPopup.Destroy;
begin
  try
    FTimer.Enabled := False;
    FreeAndNil(FCalloutFrame);
    FreeAndNil(FTimer);
  finally
    inherited Destroy;
  end;
end;

procedure TbfaCalloutPopup.InitializeCalloutWindow;
begin
  inherited;
  TdxCustomCalloutPopupWindowAccess(Callout).FShowWithoutActivation := True;
end;

procedure TbfaCalloutPopup.Popup(AOwnerControl: TWinControl;
  const AOwnerBounds: TRect);
begin
  if FTimer.Enabled then
    DoTimer(AOwnerControl);

  inherited Popup(AOwnerControl, AOwnerBounds);
  FTimer.Enabled := True;
end;

procedure TbfaCalloutPopup.Popup(AOwnerControl: TWinControl);
begin
  if FTimer.Enabled then
    DoTimer(AOwnerControl);

  inherited Popup(AOwnerControl);
  FTimer.Enabled := True;
end;

procedure TbfaCalloutPopup.DoCloseClick(Sender: TObject);
begin
  DoTimer(Sender);
end;

procedure TbfaCalloutPopup.DoTimer(Sender: TObject);
begin
  try
    if Self.Callout <> nil then
      Self.Callout.CloseUp;

    Self.Close;
  finally
    FTimer.Enabled := False;
  end;
end;

procedure TbfaCalloutPopup.SetCaptions(ValuePublic, ValuePrivate, ValueShared, ValueComment : string);
  procedure SetLabelCaptions(var Lbl : TcxLabel; str : string);
  begin
    if str <> '' then
    begin
      Lbl.Caption := str;
      Lbl.Top := FTopPos;
      Lbl.Visible := True;
      Inc(FTopPos, 20);
      if Lbl.Width > FMaxWidth then
        FMaxWidth := Lbl.Width;
    end
    else
      Lbl.Visible := False;
  end;

begin
  try
    FTopPos := 8;
    FMaxWidth := 0;
    if (FLabelPublic <> nil) and (FLabelPublic.ActiveProperties <> nil) then
      SetLabelCaptions(FLabelPublic, ValuePublic);
    if (FLabelPrivate <> nil) and (FLabelPrivate.ActiveProperties <> nil) then
      SetLabelCaptions(FLabelPrivate, ValuePrivate);
    //23.08.2024 SD -->bfacrm:ticket__show__39650
    if (FLabelShared <> nil) and (FLabelShared.ActiveProperties <> nil) then
      SetLabelCaptions(FLabelShared, ValueShared);
    //23.08.2024 SD -->bfacrm:ticket__show__39650
    if (FLabelComment <> nil) and (FLabelComment.ActiveProperties <> nil) then
      SetLabelCaptions(FLabelComment, ValueComment);

    FCalloutFrame.Height := FTopPos + 10;
    FCalloutFrame.Width := 8 + FMaxWidth + 50;
    FCalloutFrame.Width := FMaxWidth;
    //23.08.2024 SD -->bfacrm:ticket__show__39650
    if ValueShared = '' then
    begin
      if FCalloutFrame.Width < 315 then  //beim Start wird es hier immer kleiner...
        FCalloutFrame.Width := 315
    end
    else
      if FCalloutFrame.Width < 355 then
        FCalloutFrame.Width := 355;
    //23.08.2024 SD -->bfacrm:ticket__show__39650
  except
    //
  end;
end;

end.
