unit u_bfaGrid;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, graphutil,
  Grids, DBGrids, CRGrid, CRAccess, ImgList, TB2Item, DB, DBAccess,
  StdCtrls, ExtCtrls, DateUtils;

type

  TOnChange = procedure (Sender: TObject) of object;
  TOnClear = procedure (Sender: TObject) of object;
  TOnColSizing = procedure of object;
  TOnSelectAll = procedure of object;
  TOnDeselect  = procedure of object;
  TGridKind = (gkNone, gkSearch, gkResub, gkResubDelegate);


  TbfaDBGrid = class(TCRDBGrid)
    private
      FDelay: TTimer;
      FChangeDelay: Integer;
      FOnChange: TOnChange;
      FOnClear: TOnClear;
      FOnColSizing: TOnColSizing;
      FOnSelectAll: TOnSelectAll;
      FOnDeselect: TOnDeselect;
      FBM1: TBookmark;
      FKind: TGridKind;
      FDeselectedMyself: Boolean;
      FWordWrap: Boolean;
      FSelectedAll: Boolean; //03.06.2019 Ticket 19369
      FLeftCol: Integer;
      FUserid: Integer;
      FLockSelection: Boolean;
      procedure UpdateRowHeights;
      procedure DoDelayTimer(Sender: TObject);
    public
      FExecColAjust, RowColorEvenOdd: Boolean;
      FTopLeft: TGridCoord;
      clFree, clSelected, clProgress, clProgressSelected, clDone, clDoneSelected,
      clWork, clWorkSelected, clUrgent, clUrgentSelected,
      clFreeText, clSelectedText: TColor;
      ImageList: TImageList;
      FInEditMode: TStringList;

      constructor Create(AOwner: TComponent); override;
      destructor Destroy; override;

      procedure bfaOnStateChange(Sender: TObject);
      procedure SetDelay(Value: Integer);
      procedure bfaOnKeyUp(Sender: TObject; var Key: Word;
         Shift: TShiftState);
      procedure bfaOnKeyDown(Sender: TObject; var Key: Word;
         Shift: TShiftState);
      procedure SetBookmark;
      procedure AdjustColumns;
   protected
      procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
      procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
        X, Y: integer); override;
      procedure WMMouseWheel(var Message: TWMMouseWheel); message WM_MOUSEWHEEL;
      procedure DrawColumnCell(const Rect: TRect; DataCol: integer;
        Column: TColumn; State: TGridDrawState); override;
      procedure Paint; override;
      procedure Resize; override;
      procedure bfaOnCellClick(Column: TColumn);
      procedure bfaOnColExit(Sender: TObject);
      procedure bfaOnColumnMoved(Sender: TObject; FromIndex, ToIndex: Integer);


    published
      property OnTitleClick;
      property ChangeDelay: Integer read FChangeDelay write SetDelay;
      property OnChange: TOnChange read FOnChange write FOnChange;
      property OnClear: TOnClear read FOnClear write FOnClear;
      property OnColSizing: TOnColSizing read FOnColSizing write FOnColSizing;
      property OnSelectAll: TOnSelectAll read FOnSelectAll write FOnSelectAll;
      property OnDeselect: TOnDeselect read FOnDeselect write FOnDeselect;

      property Kind: TGridKind read FKind write FKind;
      property Userid: Integer read FUserid write FUserid;
      property OnMouseEnter;
      property OnMouseLeave;
      property BM1: TBookmark read FBM1 write FBM1;
      property DeselectedMyself: Boolean read FDeselectedMyself;
      property WordWrap: Boolean read FWordWrap write FWordWrap;
      property SelectedAll: Boolean read FSelectedAll write FSelectedAll; //03.06.2019 Ticket 19369
      property SelectedRows;
      property LockSelection: Boolean read FLockSelection write FLockSelection;
   end;


implementation

uses
  bfaCommonUtils;


constructor TbfaDBGrid.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDelay := TTimer.Create(Self);
  FDelay.OnTimer := DoDelayTimer;
  FChangeDelay := 250;
  SetDelay(FChangeDelay);
  Self.ReadOnly := True;
  Self.OnKeyUp := Self.bfaOnKeyUp;
  Self.OnKeyDown := Self.bfaOnKeyDown;
  Self.OnCellClick := Self.bfaOnCellClick;
  Self.OnColExit := Self.bfaOnColExit;
  Self.OnColumnMoved := Self.bfaOnColumnMoved;
  Self.Options :=  [dgTitles, {dgIndicator,} dgColumnResize, dgColLines, dgRowLines,
    dgTabs, {dgRowSelect,} {dgAlwaysShowSelection,} dgConfirmDelete, dgCancelOnExit,
    dgMultiSelect{, {dgTitleClick, dgTitleHotTrack}];
  Self.OptionsEx := [dgeEnableSort,dgeLocalFilter,dgeLocalSorting,dgeRecordCount];
  Self.Filtered := True;
  Self.clFree := gclFree;
  Self.clSelected := gclSelected;
  Self.clProgress := gclProgress;
  Self.clProgressSelected := gclProgressSelected;
  Self.clWork := gclWork;
  Self.clWorkSelected := gclWorkSelected;
  Self.clDone := gclDone;
  Self.clDoneSelected := gclDoneSelected;
  Self.clUrgent := gclUrgent;
  Self.clUrgentSelected := gclUrgentSelected;
  Self.clFreeText := clBlack;
  Self.clSelectedText := clBlack;
  Self.FWordWrap := False;
  Self.FLeftCol := 0;
  Self.FLockSelection := False;
  FInEditMode := TStringList.Create;
end;

destructor TbfaDBGrid.Destroy;
begin
  FDelay.Enabled := False;
  FDelay.Free;
  FInEditMode.Free;
  inherited Destroy;
end;

procedure TbfaDBGrid.DoDelayTimer(Sender: TObject);
var
   State: TKeyboardState;
begin
  if DataSource = nil then
    Exit;
  if DataSource.Dataset = nil then
    Exit;
  if not DataSource.DataSet.Active then
    Exit;

  GetKeyboardState(State);
  if ((State[VK_UP] and 128) <> 0) or
  ((State[VK_DOWN] and 128) <> 0) then
    Exit;

  FDelay.Enabled := False;
  SelectedRows.CurrentRowSelected := True;
  FBM1 := DataSource.DataSet.GetBookmark;
  if Assigned(FOnChange) then
    FOnChange(Sender);
  Self.Repaint;
end;

procedure TbfaDBGrid.DrawColumnCell(const Rect: TRect; DataCol: integer;
      Column: TColumn; State: TGridDrawState);
var
  lcfree, lcdone: TColor;
  DrawRect: TRect;
begin
  try
    if Canvas.brush.color = clHighlight then
      Canvas.brush.color := clwhite;

    if FKind = gkNone then
    begin
      if SelectedRows.CurrentRowSelected then
        Canvas.brush.color := clSelected
      else
        Canvas.brush.color := clFree;
    end;

    if (DataSource <> nil) and (DataSource.DataSet <> nil) and  //07.08.2017
      (DataSource.DataSet.RecordCount > 0) then
    begin
      if (FKind = gkSearch) then
      begin
         (*if DataSource.DataSet.FieldDefList.Find('editmode') <> nil then
         begin
           if (not VarIsNull(DataSource.DataSet.FieldByName('editmode').AsVariant)) and
              (DataSource.DataSet.FieldByName('editmode').AsInteger > 0) then
           begin
              if SelectedRows.CurrentRowSelected then
                Canvas.brush.color := clProgressSelected
              else
                Canvas.brush.color := clProgress;
           end
           else
           begin
             if SelectedRows.CurrentRowSelected then
               Canvas.brush.color := clSelected
             else
               Canvas.brush.color := clFree;
           end;
         end else*) if FInEditMode.IndexOf(DataSource.DataSet.FieldByName('gdocid').AsString) > -1 then
         begin
            if SelectedRows.CurrentRowSelected then
                Canvas.brush.color := clProgressSelected
            else
              Canvas.brush.color := clProgress;
         end
         else begin
            if SelectedRows.CurrentRowSelected then
              Canvas.brush.color := clSelected
            else
              Canvas.brush.color := clFree;
         end;
      end else if (FKind in [gkResub, gkResubDelegate]) then
      begin
         if DataSource.DataSet.FieldDefList.Find('status') <> nil then
         begin
           if DataSource.DataSet.FieldByName('status').AsInteger = 0 then
           begin
              if SelectedRows.CurrentRowSelected then
                Canvas.brush.color := clSelected
              else
                Canvas.brush.color := clFree;
           end;
           if (DataSource.DataSet.FieldByName('status').AsInteger and 1) = 1 then
           begin
              if SelectedRows.CurrentRowSelected then
                Canvas.brush.color := clSelected
              else
                Canvas.brush.color := clFree;
           end;
           if (DataSource.DataSet.FieldByName('status').AsInteger and 2) = 2 then
           begin
              if SelectedRows.CurrentRowSelected then
                Canvas.brush.color := clWorkSelected
              else
                Canvas.brush.color := clWork;
           end;
           if (DataSource.DataSet.FieldByName('status').AsInteger and 4) = 4 then
           begin
              if SelectedRows.CurrentRowSelected then
                Canvas.brush.color := clDoneSelected
              else
                Canvas.brush.color := clDone;
           end;
         end;

         if DataSource.DataSet.FieldDefList.Find('alarm') <> nil then
         begin
  //14.05.2018 Ticket 14391        if (DataSource.DataSet.FieldByName('alarm').AsDateTime <= StartOfTheDay(Date)) and
           if (DataSource.DataSet.FieldByName('alarm').AsDateTime <= Now()) and
               ((DataSource.DataSet.FieldByName('status').AsInteger and 4) = 0) then
           begin
              if SelectedRows.CurrentRowSelected then
                Canvas.brush.color := clUrgentSelected
              else
                Canvas.brush.color := clUrgent;
           end;
         end;

         if (DataSource.DataSet.FieldDefList.Find('groupid') <> nil)
          and (DataSource.DataSet.FieldDefList.Find('options') <> nil)
          and Assigned(ImageList)
          and (Column.FieldName = 'typ') then
         begin
           DefaultDrawColumnCell(Rect, -1, Column, State);
           if FKind = gkResub then
           begin
             if (DataSource.DataSet.FieldByName('userid_work').AsInteger = FUserid) then
             begin
               if  (DataSource.DataSet.FieldByName('groupid').AsInteger <> 0) then
               begin
                  if  (DataSource.DataSet.FieldByName('options').AsInteger and 1) = 1 then
                    ImageList.Draw(Canvas, Rect.Left, Rect.Top, 257, True )
                  else
                    ImageList.Draw(Canvas, Rect.Left, Rect.Top, 9, True )
               end else
                  ImageList.Draw(Canvas, Rect.Left, Rect.Top, 8, True );
             end else
             begin
               if  (DataSource.DataSet.FieldByName('groupid').AsInteger <> 0) then
               begin
                  if  (DataSource.DataSet.FieldByName('options').AsInteger and 1) = 1 then
                    ImageList.Draw(Canvas, Rect.Left, Rect.Top, 258, True )
                  else
                    ImageList.Draw(Canvas, Rect.Left, Rect.Top, 228, True )
               end else
                  ImageList.Draw(Canvas, Rect.Left, Rect.Top, 227, True );
             end;
           end else
           begin
             if DataSource.DataSet.FieldByName('groupid').AsInteger <> 0 then
             begin
                if  (DataSource.DataSet.FieldByName('options').AsInteger and 1) = 1 then
                  ImageList.Draw(Canvas, Rect.Left, Rect.Top, 257, True )
                else
                  ImageList.Draw(Canvas, Rect.Left, Rect.Top, 9, True )
             end else
                ImageList.Draw(Canvas, Rect.Left, Rect.Top, 8, True );
           end;
           Exit;
         end;
      end;
    end;

////15.01.2018 Ticket 13045-->
    try
      if TCRColumn(Column).FloatFormat = ffcurrency then
        TCurrencyField(Column.Field).DisplayFormat := '###,###,#0.00';
    except

    end;
////15.01.2018 Ticket 13045<--

  //  if SelectedRows.CurrentRowSelected and (DataSource.DataSet.RecNo = Row) then
    if SelectedRows.CurrentRowSelected and (dgMultiSelect in Self.Options)
        and
        (DataSource <> nil) and (DataSource.DataSet <> nil) and //07.08.2017
        (DataSource.DataSet.CompareBookmarks(DataSource.DataSet.GetBookmark, FBM1)= 0) then
      Canvas.Font.Style := [fsBold]
    else
       Canvas.Font.Style := [];

    if Enabled then
    begin
//30.08.2022
//      if Canvas.brush.color = clUrgentSelected then
//        Canvas.Font.Color :=  gclUrgentSelectedText
//      else
//       Canvas.Font.Color := clBlack
      if Canvas.brush.color = clFree then
        Canvas.Font.Color := clFreeText
      else if Canvas.brush.color = clSelected then
        Canvas.Font.Color := clSelectedText
    end
    else
       Canvas.Font.Color := clGrayText;


//  SJ 25.09.2019 -
    if Self.RowColorEvenOdd and
        ((Self.DataLink.DataSet.RecNo mod 2) <> 0) then
    begin
      if (Canvas.brush.Color = clUrgent) then
        Canvas.brush.color := ColorAdjustLuma(colortorgb(Canvas.brush.color),-30,false)
       else if (Canvas.brush.Color = clWork) then
        Canvas.brush.color := ColorAdjustLuma(colortorgb(Canvas.brush.color),-55,false)
      else
        Canvas.brush.color := ColorAdjustLuma(colortorgb(Canvas.brush.color),-15,false);
    end;

    if FWordWrap and (Column.Field <> nil) and (length(Column.Field.DisplayText) > 0) then
    begin
      DrawRect := Rect;
      Canvas.FillRect(DrawRect);
      Inc(DrawRect.Left, 2);
      Inc(DrawRect.Top, 2);
      DrawText(Canvas.Handle, PChar(Column.Field.DisplayText), -1, DrawRect,
        DT_NOPREFIX or DT_WORDBREAK);
    end else
      DefaultDrawColumnCell(Rect, -1, Column, State);
  except

  end;
end;

procedure TbfaDBGrid.UpdateRowHeights;
var
  X, Y, RecNo: Integer;
  MaxHeight, TxtHeight: Integer;
  R: TRect;
begin
  if not FWordwrap then
    Exit;

  if (Self.DataSource = nil) or (Self.DataSource.DataSet = nil)  then
    Exit;
  if not (Self.DataSource.DataSet.Active) or (Self.DataSource.DataSet.RecordCount = 0) then
    Exit;
  RecNo := Self.DataSource.DataSet.RecNo;
  for Y := Self.FixedRows to Self.RowCount - 1 do
  begin
    if Y >= Self.RowCount then
    begin
      //OutputDebugString('Exit');
      Break;
    end;

    Self.DataSource.DataSet.RecNo := Y;
    MaxHeight := Self.DefaultRowHeight - 4;
    for X := Self.FixedCols to Self.ColCount - 1 do
    begin
      R := Rect(0, 0, Self.ColWidths[X] - 4, 0);

      TxtHeight := DrawText(Self.Canvas.Handle, PChar(Self.Fields[X].DisplayText), -1,
        R, DT_WORDBREAK or DT_CALCRECT);
      if TxtHeight > MaxHeight then
        MaxHeight := TxtHeight;
    end;
    try
      if Y >= Self.RowCount then
      begin
        //OutputDebugString('Exit');
        Break;
      end;
      Self.RowHeights[Y] := MaxHeight + 4;
    except
      Self.RowHeights[Y] := Self.DefaultRowHeight;
    end;
  end;
  Self.DataSource.DataSet.RecNo := RecNo;
end;

procedure TbfaDBGrid.AdjustColumns;
begin
  inherited;
  FBM1 := nil; //19.09.2024 SD <--bfacrm:ticket__show__40019  Nach dem Aktualisieren der Tabelle sollte das aktuelle Bookmark zurckgesetzt werden.
  UpdateRowHeights;
end;

procedure TbfaDBGrid.bfaOnKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
  lBM : TBookmark;
begin
 if DataSource = nil then
    Exit;
  if DataSource.DataSet = nil then
    Exit;
  if not DataSource.DataSet.Active then
    Exit;
  if Key = VK_TAB then
    Exit;


  if (Shift = [ssCtrl]) then
  begin
    if  ((Key = $41) or (key = $61)) and not SelectedAll then
    begin
       DataSource.DataSet.DisableControls;
       lBM := DataSource.DataSet.GetBookmark;
       DataSource.DataSet.First;
       while not DataSource.DataSet.Eof do
       begin
         SelectedRows.CurrentRowSelected := True;
         DataSource.DataSet.Next;
       end;
       DataSource.DataSet.GotoBookmark(lBM);
       DataSource.DataSet.EnableControls;
       SelectedAll := True;
       if Assigned(FOnSelectAll) then
         FOnSelectAll;
    end;
  end;
end;

procedure TbfaDBGrid.bfaOnKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if DataSource = nil then
    Exit;
  if DataSource.DataSet = nil then
    Exit;
  if not DataSource.DataSet.Active then
    Exit;

  if Shift = [] then
  begin
    case Key of
      vk_up, vk_down, vk_left, vk_right, vk_prior, vk_next, vk_home, vk_end:
      begin
        SelectedAll := False;
        SelectedRows.CurrentRowSelected := True;
        Self.Invalidate;
        FDelay.Enabled := False;
        FDelay.Enabled := True;
        if Assigned(FOnDeselect) then
          FOnDeselect;
      end;
    end;
    if Key = 27 then
    begin
      SelectedRows.Clear;
      SelectedAll := False;
      if Assigned(FBM1) then
      begin
        try
          DataSource.DataSet.GotoBookmark(FBM1);
          SelectedRows.CurrentRowSelected := True;
        except
          //09.11.2017 Crashlog
        end;
      end
      else
      begin
        DataSource.DataSet.First;
        FDelay.Enabled := False;
        FDelay.Enabled := True;
      end;
      if Assigned(FOnDeselect) then
        FOnDeselect;
    end;

  end;
end;

procedure TbfaDBGrid.bfaOnStateChange(Sender: TObject);
begin
  if (DataSource = nil) or (DataSource.DataSet = nil) then
    Exit;
  if not DataSource.DataSet.Active then
    Exit;

  if DataSource.DataSet.RecordCount = 0 then
  begin
    if Assigned(FOnClear) then
      FOnClear(Sender);
  end
  else
  begin
    FBM1 := DataSource.DataSet.GetBookmark;
    SelectedRows.CurrentRowSelected := True;
    if Assigned(FOnChange) then
      FOnChange(Sender);
  end;
  UpdateRowHeights;
end;

procedure TbfaDBGrid.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  lBM : TBookmark;
  Cell: TGridCoord;
  ActRec: Integer;
  doselect : Boolean;
  iLeftCol: Integer;
begin
  try
    if Assigned(OnMouseDown) then
      OnMouseDown(Self, Button, Shift, X, Y);

    if (Button = mbRight) then  // Task 15564 Verspringen der Trefferliste bei rechter Maustaste
      Exit;

    if ((ssCtrl in Shift) or (ssShift in Shift)) and FLockSelection then
        Exit;
    //if DataSource.DataSet.CompareBookmarks(lBM, FBM1) <> 0 then
    doselect := (ssCtrl in Shift) or (Y < 18);      // Column-Header
    Cell := Self.MouseCoord(X, Y);
    if dgIndicator in Self.Options then
      Dec(Cell.X);
    if dgTitles in Self.Options then
      Dec(Cell.Y);
    if Self.DataLink.Active and (Cell.X >= 0) and
      (Cell.Y >= 0) then
    begin
      ActRec := Self.DataLink.ActiveRecord;          // altiver Record Save
      try
        Self.DataLink.ActiveRecord := Cell.Y;       // Neuer Record
        if Self.Columns[Cell.X].Field <> nil then
          Caption := Self.Columns[Cell.X].Field.AsString;
        lBM := DataSource.DataSet.GetBookmark;           // lbm Aktiver Bookmark
        if DataSource.DataSet.CompareBookmarks(FBM1, lBM) <> 0 then  // neuer Bookmart (lBM) <> alter Bookmark (FMB1)
          doselect := True;                                          // Flag fr selektieren
      finally
        Self.DataLink.ActiveRecord := ActRec;   // selektiert alten Record zurck
      end;
    end;
  //07.04.2016  if doselect then
    if ((doselect) or (ssDouble in Shift)) {and (Button = mbLeft)} then
    begin
      iLeftCol := Self.LeftCol;
      inherited;
      if Self.LeftCol <> iLeftCol then
        Self.LeftCol := iLeftCol;
    end
    else if (Cell.Y = -1) and (dgeFilterBar in Self.OptionsEx) then
    begin
      inherited;
    end

    // Linksklick selektiert keinen Datensatz, geht aber zum Bookmark...und verschiebt die Scrollbar
    else if (DataSource <> nil) and (DataSource.DataSet <> nil) and (DataSource.DataSet.RecordCount > 0) then
    begin
      DataSource.DataSet.GotoBookmark(FBM1);
    end;
  except
    //11.07.2017 weil manchmal Abstrzt...
  end;
end;

procedure TbfaDBGrid.MouseUp(Button: TMouseButton; Shift: TShiftState; X,
  Y: integer);
var
  lBM : TBookmark;
  bUp : Boolean;
  colsized : Boolean;
  i : Integer;
begin
  try
    if (FGridState = gsColSizing) then
      colsized := True;
    if (DataSource = nil) or (DataSource.DataSet = nil) or (not DataSource.DataSet.Active) then
      Exit;

    FDeselectedMyself := False;
    FDelay.Enabled:= False;
    SelectedAll := False;
    lBM := DataSource.DataSet.GetBookmark;
    if (ssShift in Shift) then
    begin
      DataSource.DataSet.DisableControls;
      DataSource.DataSet.GotoBookmark(FBM1);
      if DataSource.DataSet.CompareBookmarks(FBM1, lBM) <> 0 then
      begin
        bUp := DataSource.DataSet.CompareBookmarks(FBM1, lBM) = 1;

        while DataSource.DataSet.CompareBookmarks(DataSource.DataSet.GetBookmark, lBM) <> 0 do
        begin
           SelectedRows.CurrentRowSelected := True;
          if bUp then
              DataSource.Dataset.Prior
          else
              DataSource.Dataset.Next;
        end;
      end;
      SelectedRows.CurrentRowSelected := True;
      DataSource.DataSet.EnableControls;
    end;
    if ssCtrl in Shift then
    begin
       DataSource.DataSet.DisableControls;
       if SelectedRows.CurrentRowSelected then
       begin
         DataSource.DataSet.GotoBookmark(lBM);
         SelectedRows.CurrentRowSelected := True;
       end
       else
       begin
          if DataSource.DataSet.CompareBookmarks(lBM, FBM1) = 0 then
          begin
            if SelectedRows.Count < 1 then
               SelectedRows.CurrentRowSelected := True
            else
            begin
              FDeselectedMyself := True;
//              DataSource.DataSet.GotoBookmark(SelectedRows.Items[SelectedRows.Count -1]);
//              FBM1 := SelectedRows.Items[SelectedRows.Count -1];
              DataSource.DataSet.GotoBookmark(SelectedRows.Items[0]);
              FBM1 := SelectedRows.Items[0];
              FDelay.Enabled:= True;
            end;
          end
          else
          begin
            DataSource.DataSet.GotoBookmark(FBM1);
          end;
       end;
       DataSource.DataSet.EnableControls;
    end;
    if Shift = [] then
    begin

      if DataSource.DataSet.CompareBookmarks(lBM, FBM1) <> 0 then
        FDelay.Enabled:= True;
      FBM1 := DataSource.DataSet.GetBookmark;
    end;
  finally
    inherited;
    if colsized then
    begin
      if Assigned(FOnColSizing) then
        FOnColSizing;
      UpdateRowHeights;
    end;
    colsized := False;
  end;
end;

procedure TbfaDBGrid.bfaOnColumnMoved(Sender: TObject; FromIndex, ToIndex: Integer);
begin
  inherited;
   //OutputDebugString('OnColumnMoved');
end;

procedure TbfaDBGrid.bfaOnCellClick(Column: TColumn);
begin
  //inherited;
  Self.LeftCol := FLeftCol;
  //FLeftCol := 0;
end;

procedure TbfaDBGrid.bfaOnColExit(Sender: TObject);
begin
  //inherited;
  FLeftCol := Self.LeftCol;
end;

procedure TbfaDBGrid.WMMouseWheel(var Message: TWMMouseWheel);
var
  ptSTC: TPoint;
begin
  //OutputDebugString(PChar('TbfaDBGrid.WMMouseWheel'));
  ptSTC := Self.ScreenToClient(Mouse.CursorPos);

  if (ptSTC.x < -1) or (ptSTC.Y  < -1) or
      (ptSTC.x > Self.ClientWidth) or (ptSTC.Y > Self.ClientHeight) then
  begin
    TForm(Self.Owner).defocuscontrol(Self, False);
    Exit;
  end
  else
  begin

//  if Self.Focused then
//  begin
    if Message.WheelDelta > 0 then
      SendMessage(Self.Handle, WM_VSCROLL, SB_LINEUP, 0)
    else
      SendMessage(Self.Handle, WM_VSCROLL, SB_LINEDOWN, 0);
//  end;

    UpdateRowHeights;
  end;
end;

procedure TbfaDBGrid.SetBookmark;
begin
  if (DataSource = nil) or (DataSource.DataSet = nil) or (not DataSource.DataSet.Active) then
    Exit;
  FBM1 := DataSource.DataSet.GetBookmark;
end;

procedure TbfaDBGrid.SetDelay(Value: Integer);
begin
  if not Assigned(FDelay) then
    Exit;
  if Value < 0 then Value := 0;
  FDelay.Enabled := Value > 0;
  FDelay.Interval := Value;
  FDelay.Enabled:= False;
end;

procedure TbfaDBGrid.Paint;
var
  LDrawInfo: TGridDrawInfo;
begin
  try
    inherited Paint;
    CalcDrawInfo(LDrawInfo);
    if LDrawInfo.Horz.GridBoundary < LDrawInfo.Horz.GridExtent then
    begin
      Canvas.Brush.Color := Color; //use the Color property to paint the background
      Canvas.FillRect(Rect(LDrawInfo.Horz.GridBoundary, 0, LDrawInfo.Horz.GridExtent, LDrawInfo.Vert.GridBoundary));
    end;
    if LDrawInfo.Vert.GridBoundary < LDrawInfo.Vert.GridExtent then
    begin
      Canvas.Brush.Color := Color;//use the Color property to paint the background
      Canvas.FillRect(Rect(0, LDrawInfo.Vert.GridBoundary, LDrawInfo.Horz.GridExtent, LDrawInfo.Vert.GridExtent));
    end;
  except
    //26.10.2017 wegen ein CrashLog hier eingebaut
  end;
end;

procedure TbfaDBGrid.Resize;
begin
  inherited;
  UpdateRowHeights;
end;

end.




