unit u_bfaInputCombo;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, SpTBXItem, SpTBXControls, StdCtrls, SpTBXEditors;

type

    TOnNew = procedure(Sender: TObject; AValue: WideString) of object;
    TOnDouble = procedure(Sender: TObject; AValue: WideString) of object;
    TOnModify = procedure(Sender: TObject; AValue: WideString; AObject: TObject = nil) of object;
    TOnDelete = procedure(Sender: TObject; AValue: WideString; AObject: TObject = nil) of object;
    TOnChange = procedure(Sender: TObject) of object;
    TOnBeginEdit = procedure(Sender: TObject) of object;
    TOnEndEdit = procedure(Sender: TObject) of object;
    TOnDropDown = procedure (Sender: TObject) of object;
    TOnCloseUp = procedure (Sender: TObject) of object;

  TbfaInputCombo = class(TFrame)
    cbbInput: TSpTBXComboBox;
    btnNew: TSpTBXButton;
    btnModify: TSpTBXButton;
    btnDelete: TSpTBXButton;
    pnl1: TSpTBXPanel;
    constructor Create(AOwner: TComponent); override;
    procedure btnNewClick(Sender: TObject);
    procedure btnModifyClick(Sender: TObject);
    procedure cbbInputKeyPress(Sender: TObject; var Key: Char);
    procedure btnDeleteClick(Sender: TObject);
    procedure EnableButtons(enable: Boolean);
    procedure FrameResize(Sender: TObject);
  private
    { Private-Deklarationen }
    newmode, editmode: Boolean;
    editindex : Integer;
    FOnNew: TOnNew;
    FOnDouble: TOnDouble;
    FOnModify: TOnModify;
    FOnDelete: TOnDelete;

    FConfirmDelete: Boolean;
    FShowModify: Boolean;
    FShowDelete: Boolean;
    FModified: Boolean;
    FEditable: Boolean;
    FOnChange: TOnChange;
    FOnBeginEdit: TOnBeginEdit;
    FOnEndEdit: TOnEndEdit;
    FOnDropDown: TOnDropDown;
    FOnCloseUp: TOnCloseUp;
    FPreventDoubles: Boolean;
    FDisableButtons: Boolean;
    FDeleteEmptyEntry: Boolean;
    procedure SetShowModify(const Value: Boolean);
    procedure SetShowDelete(const Value: Boolean);
    procedure SetEditable(const Value: Boolean);
    procedure OnComboChange(Sender: TObject);
    procedure DoSave(Sender: TObject);

    procedure OnComboDropDown(Sender: TObject);
    procedure OnComboCloseUp(Sender: TObject);
    procedure SetPreventDoubles(const Value: Boolean);
    procedure SetDisableButtons(const Value: Boolean);
  public
    { Public-Deklarationen }
    procedure DoCancel(Sender: TObject);
    property Modified: Boolean read FModified write FModified;
    property Editable: Boolean read FEditable write SetEditable;
    procedure EnableAllControls(Value: Boolean);
    function IsInUse: Boolean;
  published
    property OnNew: TOnNew read FOnNew write FOnNew;
    property OnDouble: TOnDouble read FOnDouble write FOnDouble;
    property OnModify: TOnModify read FOnModify write FOnModify;
    property OnDelete: TOnDelete read FOnDelete write FOnDelete;
    property OnChange: TOnChange read FOnChange write FOnChange;
    property OnBeginEdit: TOnBeginEdit read FOnBeginEdit write FOnBeginEdit;
    property OnEndEdit: TOnEndEdit read FOnEndEdit write FOnEndEdit;
    property OnDropDown: TOnDropDown read FOnDropDown write FOnDropDown;
    property OnCloseUp : TOnCloseUp read FOnCloseUp write FOnCloseUp;
    property ConfirmDelete: Boolean read FConfirmDelete write FConfirmDelete;
    property ShowDelete: Boolean read FShowDelete write SetShowDelete default True;
    property ShowModify: Boolean read FShowModify write SetShowModify default True;
    property PreventDoubles: Boolean read FPreventDoubles write SetPreventDoubles default False;
    property DisableButtons: Boolean read FDisableButtons write SetDisableButtons default False;
    property DeleteEmptyEntry: Boolean read FDeleteEmptyEntry write FDeleteEmptyEntry default False;
    property ParentBackground;
    property ParentColor;
  end;

implementation

{$R *.dfm}

uses
    gnugettext, bfaCommonMethods, bfaCommonDialogs{, CommonGUIUtils};


function TbfaInputCombo.IsInUse: Boolean;
begin
  Result := newmode or editmode;
end;

procedure TbfaInputCombo.EnableAllControls(Value: Boolean);
begin
  cbbInput.Enabled := Value;
  btnNew.Enabled := Value;
  btnModify.Enabled := Value;
  btnDelete.Enabled := Value;
end;

procedure TbfaInputCombo.btnDeleteClick(Sender: TObject);
var
  Value: WideString;
begin
  Value := cbbInput.Text;
  if (cbbInput.Text <> '') or FDeleteEmptyEntry then
  begin
    if FConfirmDelete then
    begin
      if BfMessageDlg(Format('Soll der Eintrag %s entfernt werden',[cbbInput.Text]),mtConfirmation,mbOKCancel) = mrOk then
      begin
        if cbbInput.ItemIndex > -1 then
        begin
          if Assigned(OnDelete) then
            FOnDelete(Self, Value, cbbInput.Items.Objects[cbbInput.ItemIndex]);
          cbbInput.Items.Delete(cbbInput.ItemIndex);
        end;
      end;
    end else begin
      if Assigned(OnDelete) then
        if cbbInput.ItemIndex > -1 then
          FOnDelete(Self, Value, cbbInput.Items.Objects[cbbInput.ItemIndex]);
    end;
  end;
end;

procedure TbfaInputCombo.btnModifyClick(Sender: TObject);
begin
//  cbbInput.Text := Trim(cbbInput.Text);
  if newmode or editmode then
  begin
    DoCancel(Sender);
  end else begin
    if cbbInput.ItemIndex = -1 then
      Exit;
    newmode := False;
    editmode := True;
    editindex := cbbInput.ItemIndex;
    cbbInput.Style := csDropDown;
//    cbbInput.Text := '';

    cbbInput.SetFocus;
    if Assigned(FOnBeginEdit) then
      FOnBeginEdit(Self);
    EnableButtons(False);
  end;
end;

procedure TbfaInputCombo.btnNewClick(Sender: TObject);
var
  value : WideString;
begin
  if newmode or editmode then
  begin
      DoSave(Sender);
  end else begin
    newmode := True;
    editmode := False;
    cbbInput.Style := csDropDown;
    cbbInput.Text := '';

    cbbInput.SetFocus;
    if Assigned(FOnBeginEdit) then
      FOnBeginEdit(Self);
    EnableButtons(False);
  end;
end;

procedure TbfaInputCombo.cbbInputKeyPress(Sender: TObject; var Key: Char);
begin
  if (Key = #13) then
  begin
    DisableBeep(Key); //11.03.2024 bfacrm:ticket__show__38134

    if editmode or newmode then
      DoSave(Sender);
  end;

  if Key = #27 then
  begin
    if editmode or newmode then
      DoCancel(Sender);
    Key := #0;  // wegen Windows-Pling Ton
  end;
end;

constructor TbfaInputCombo.Create(AOwner: TComponent);
begin
  inherited;

  FShowModify := True;
  FShowDelete := True;
  FModified := False;
  FEditable := True;
  cbbInput.OnChange := OnComboChange;
  cbbInput.OnDropDown := OnComboDropDown;
  cbbInput.OnCloseUp := OnComboCloseUp;
end;

procedure TbfaInputCombo.DoCancel(Sender: TObject);
begin
  try
    cbbInput.Style := csDropDown;
    if newmode then
      btnNew.SetFocus;
    if editmode then
      btnModify.SetFocus;

    newmode := False;
    editmode := False;
    EnableButtons(True);
    cbbInput.Text := '';
    if editindex > -1 then
    begin
      cbbInput.ItemIndex := editindex;
      cbbInput.Text := cbbInput.Items.Strings[editindex];  // Bei Cancel wird der ursprngliche Text nochmal angezeigt
    end;

    if Assigned(FOnEndEdit) then
      FOnEndEdit(Self);
  except

  end;
end;

procedure TbfaInputCombo.DoSave(Sender: TObject);
var
  value : WideString;
begin
    cbbInput.Text := Trim(cbbInput.Text);
    if newmode then
    begin
      FModified := True;
      if (cbbInput.Text <> '') then
      begin
        if FPreventDoubles and (cbbInput.ItemIndex > -1) then
        begin
          if Assigned(FOnDouble) then
            FOnDouble(Self, cbbInput.Text);
          value := '';
          cbbInput.SetFocus;
          cbbInput.Style := csDropDown;
          newmode := False;
          EnableButtons(True);
          cbbInput.ItemIndex := editindex;
          try
            btnNew.SetFocus;
          except

          end;
          Exit;
        end else
        begin
          value := cbbInput.Text;
          cbbInput.ItemIndex := cbbInput.Items.Add(cbbInput.Text);
        end;
      end;
      cbbInput.Style := csDropDown;
      newmode := False;
      EnableButtons(True);
      try
        btnNew.SetFocus;
      except

      end;
      if Assigned(FOnNew) then
        FOnNew(Self, value);
      if Assigned(FOnEndEdit) then
        FOnEndEdit(Self);
    end;
    if editmode then
    begin
      FModified := True;
      if (cbbInput.Text <> '') then
      begin
        if FPreventDoubles and (cbbInput.ItemIndex > -1) then
        begin
          if Assigned(FOnDouble) then
            FOnDouble(Self, cbbInput.Text);
          value := '';
          cbbInput.Style := csDropDown;
          editmode := False;
          EnableButtons(True);
          cbbInput.ItemIndex := editindex;
          try
            btnModify.SetFocus;
          except
            //12.07.2018 wegen CrashLog eingebaut
          end;
          Exit;
        end else
        begin
          value := cbbInput.Text;
          cbbInput.Items[editindex] := cbbInput.Text;
        end;
      end;
      cbbInput.Style := csDropDown;
      editmode := False;
      EnableButtons(True);
      try
        btnModify.SetFocus;
      except
        //12.07.2018 wegen CrashLog eingebaut
      end;
      if Assigned(FOnModify) then
        FOnModify(Self, value, cbbInput.Items.Objects[editindex]);
      cbbInput.ItemIndex := editindex;
      if Assigned(FOnEndEdit) then
        FOnEndEdit(Self);
    end;
end;

procedure TbfaInputCombo.EnableButtons(enable: Boolean);
begin
  if newmode or editmode then
  begin
    btnNew.Caption := _('Ok');
    btnModify.Caption := _('Abbrechen');
    btnNew.Enabled := True;
    btnModify.Enabled := True;

    {btnNew.Font.Color := clGreen;
    btnNew.Font.Style := [fsBold];
    btnModify.Font.Color := clMaroon;
    btnModify.Font.Style := [fsBold];
    cbbInput.Color := clMoneygreen;}

  end else
  begin
    btnNew.Caption := _('Neu');
    btnModify.Caption := _('ndern');
    btnNew.Enabled := enable and FEditable;
    btnModify.Enabled := enable and FEditable;

    {btnNew.Font.Color := clWindowText;
    btnNew.Font.Style :=  btnNew.Font.Style - [fsBold];
    btnModify.Font.Color := clWindowText;
    btnModify.Font.Style := btnModify.Font.Style - [fsBold];
    cbbInput.Color := clWindow;}
  end;

  btnDelete.Enabled := enable and FEditable;
//  btnDelete.Visible := btnDelete.Enabled or not FEditable;
end;

procedure TbfaInputCombo.FrameResize(Sender: TObject);
begin
//14.01.2022 Anpassungen fr neue Admin Oberflche -->
  if cbbInput.Left + cbbInput.Width + 4 > btnNew.Left then
    cbbInput.Width := btnNew.Left - cbbInput.Left - 4;
//14.01.2022 <--
end;

procedure TbfaInputCombo.OnComboChange(Sender: TObject);
begin
  if newmode or editmode then
    Exit;

  if Assigned(FOnchange) then
    FOnChange(Sender);
end;

procedure TbfaInputCombo.OnComboCloseUp(Sender: TObject);
begin
  if Assigned(FOnCloseUp) then
    FOnCloseUp(Sender);
end;

procedure TbfaInputCombo.OnComboDropDown(Sender: TObject);
begin
  if Assigned(FOnDropDown) then
    FOnDropDown(Sender);
end;

procedure TbfaInputCombo.SetEditable(const Value: Boolean);
begin
  if Editable <> Value then
  begin
    FEditable := Value;
    EnableButtons(Value);
  end;
end;

procedure TbfaInputCombo.SetDisableButtons(const Value: Boolean);
begin
  FDisableButtons := Value;
  btnModify.Enabled := not FDisableButtons;
  btnDelete.Enabled := not FDisableButtons;

end;

procedure TbfaInputCombo.SetPreventDoubles(const Value: Boolean);
begin
  FPreventDoubles := Value;
end;

procedure TbfaInputCombo.SetShowDelete(const Value: Boolean);
begin
  FShowDelete := Value;
  btnDelete.Visible := Value;
end;

procedure TbfaInputCombo.SetShowModify(const Value: Boolean);
begin
  FShowModify := Value;
  btnModify.Visible := Value;
end;

end.
