unit u_bfaInputEdit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, SpTBXItem, SpTBXControls, StdCtrls, SpTBXEditors;

type

    TOnNew = procedure(Sender: TObject; AValue: WideString) of object;
    TOnDouble = procedure(Sender: TObject; AValue: WideString) of object;
    TOnModify = procedure(Sender: TObject; AValue: WideString) of object;
    TOnDelete = procedure(Sender: TObject; AValue: WideString) of object;
    TOnChange = procedure(Sender: TObject) of object;
    TOnBeginEdit = procedure(Sender: TObject) of object;
    TOnEndEdit = procedure(Sender: TObject) of object;

  TbfaInputEdit = class(TFrame)
    btnNew: TSpTBXButton;
    btnModify: TSpTBXButton;
    btnDelete: TSpTBXButton;
    edtInput: TSpTBXEdit;
    pnl1: TSpTBXPanel;
    constructor Create(AOwner: TComponent); override;
    procedure btnNewClick(Sender: TObject);
    procedure btnModifyClick(Sender: TObject);
    procedure edtInputKeyPress(Sender: TObject; var Key: Char);
    procedure btnDeleteClick(Sender: TObject);
    procedure EnableButtons(enable: Boolean);
    procedure edtInputKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);

  private
    { Private-Deklarationen }
    newmode, editmode, key_pressed, key_paste: Boolean;
    FOldValue : WideString;
    FOnNew: TOnNew;
    FOnModify: TOnModify;
    FOnDelete: TOnDelete;
    FConfirmDelete: Boolean;
    FShowModify: Boolean;
    FShowDelete: Boolean;
    FModified: Boolean;
    FEditable: Boolean;
    FOnChange: TOnChange;
    FOnBeginEdit: TOnBeginEdit;
    FOnEndEdit: TOnEndEdit;
    FPreventDoubles: Boolean;
    FOnDouble: TOnDouble;
    FDeleteEmptyEntry: Boolean;
    FDisableButtons: Boolean;
    FNotAllowedChar: string;
    FDirectMode: Boolean;
    FDontClearEdit: Boolean;
    procedure SetShowModify(const Value: Boolean);
    procedure SetShowDelete(const Value: Boolean);
    procedure SelEditable(const Value: Boolean);
    procedure OnEditChange(Sender: TObject);

    procedure DoSave(Sender: TObject);
    procedure SetDisableButtons(const Value: Boolean);
  public
    { Public-Deklarationen }
    procedure DoCancel(Sender: TObject);
    procedure ResetEdit;
    property Editable: Boolean read FEditable write SelEditable;
    property Modified: Boolean read FModified write FModified;
    property OldValue: WideString read FOldValue write FOldValue;
    property DontClearEdit: Boolean read FDontClearEdit write FDontClearEdit;
  published

    property OnNew: TOnNew read FOnNew write FOnNew;
    property OnDouble: TOnDouble read FOnDouble write FOnDouble;
    property OnModify: TOnModify read FOnModify write FOnModify;
    property OnDelete: TOnDelete read FOnDelete write FOnDelete;
    property OnChange: TOnChange read FOnChange write FOnChange;
    property ConfirmDelete: Boolean read FConfirmDelete write FConfirmDelete;
    property OnBeginEdit: TOnBeginEdit read FOnBeginEdit write FOnBeginEdit;
    property OnEndEdit: TOnEndEdit read FOnEndEdit write FOnEndEdit;
    property ShowDelete: Boolean read FShowDelete write SetShowDelete default True;
    property ShowModify: Boolean read FShowModify write SetShowModify default True;
    property PreventDoubles: Boolean read FPreventDoubles write FPreventDoubles default True;
    property DisableButtons: Boolean read FDisableButtons write SetDisableButtons default False;
    property DeleteEmptyEntry: Boolean read FDeleteEmptyEntry write FDeleteEmptyEntry default False;
    property NotAllowedChar: string read FNotAllowedChar write FNotAllowedChar;
    property DirectMode: Boolean read FDirectMode write FDirectMode default False;

  end;

implementation

{$R *.dfm}

uses
    gnugettext, bfaCommonMethods, bfaCommonDialogs;

procedure TbfaInputEdit.btnDeleteClick(Sender: TObject);
var
  Value: WideString;
begin
  Value := edtInput.Text;
  if (edtInput.Text <> '') or FDeleteEmptyEntry then
  begin
    if FConfirmDelete then
    begin
      if BfMessageDlg(Format('Soll der Eintrag %s entfernt werden',[edtInput.Text]),mtConfirmation,mbOKCancel) = mrOk then
      begin
        if Assigned(OnDelete) then
          FOnDelete(Self, Value);
        if not FDontClearEdit then
          edtInput.Text := '';
      end;
    end else
    begin
      if Assigned(OnDelete) then
        FOnDelete(Self, Value);
      if not FDontClearEdit then
          edtInput.Text := '';
    end;
  end;
end;

procedure TbfaInputEdit.btnModifyClick(Sender: TObject);
var
  Value: WideString;
begin
  if newmode or editmode then
  begin
    DoCancel(Sender);
  end else
  if FDirectMode then
  begin
    edtInput.Text := Trim(edtInput.Text);
    if (edtInput.Text = '') or not fModified then
    begin
      FModified := False;
      Exit;
    end;

    if (edtInput.Text <> '') and (FOldValue <> edtInput.Text) then
    begin
      Value := edtInput.Text;

      if Assigned(FOnModify) then
        FOnModify(Self, value);
    end;

    FModified := False;
  end else begin
    if edtInput.Text = '' then
      Exit;
    newmode := False;
    editmode := True;
    FOldValue := edtInput.Text;
    EnableButtons(False);
    edtInput.SetFocus;
    if Assigned(FOnBeginEdit) then
      FOnBeginEdit(Self);
  end;
end;

procedure TbfaInputEdit.btnNewClick(Sender: TObject);
var
  Value: WideString;
begin
  if newmode or editmode then
  begin
    DoSave(Sender);
  end
  else if FDirectMode then begin
    edtInput.Text := Trim(edtInput.Text);
    if (edtInput.Text = '') or not fModified then
    begin
      FModified := False;
      Exit;
    end;

    if (edtInput.Text <> '') and (FOldValue <> edtInput.Text) then
    begin
      Value := edtInput.Text;

      if Assigned(FOnNew) then
        FOnNew(Self, Value);

      edtInput.Text := '';
    end;

    FModified := False;
  end else begin
    newmode := True;
    editmode := False;
    FOldValue := '';
    edtInput.Text := '';
    EnableButtons(False);
    edtInput.SetFocus;
    if Assigned(FOnBeginEdit) then
      FOnBeginEdit(Self);
  end;
end;

procedure TbfaInputEdit.edtInputKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
//  OutputDebugString('TbfaInputEdit.edtInputKeyDown');
  key_pressed := True;
  if (Key in [VK_DELETE, VK_BACK]) then     // Entf
  begin
    FModified := True;
    //FOldValue := edtInput.Text;
  end;
end;

procedure TbfaInputEdit.edtInputKeyPress(Sender: TObject; var Key: Char);
var
  value : WideString;
begin
//  OutputDebugString('TbfaInputEdit.edtInputKeyPress');
  key_pressed := True;
  try
    if (Key = #13) then
    begin
      DisableBeep(Key); //11.03.2024 bfacrm:ticket__show__38134

      if editmode or newmode then
        DoSave(Sender);
      if FDirectMode then
      begin
        btnNew.Click;
        Exit;
      end;
    end;

    if Key = #27 then
    begin
      if editmode or newmode then
        DoCancel(Sender);
      if FDirectMode then
      begin
        FModified := False;
        if edtInput.Text = FOldValue then
          key_pressed := False    // wenn bei ESC keine nderung geschieht muss key_pressed zurck, weil kein OnChange mehr luft
        else
          edtInput.Text := FOldValue;
      end;
      Key := #0;  // wegen Windows-Pling Ton
    end;

    if fNotAllowedChar <> '' then
    begin
      if Pos(Key, fNotAllowedChar) <> 0 then
      begin
        Key := #0;
        key_pressed := False;
        Abort;
      end;
    end;

    if (Key = #$16) then     // STRG + V
    begin
      key_paste := True;
    end;

    if FDirectMode then
      FModified := True;
  finally
//    key_pressed := False;
  end;
end;



constructor TbfaInputEdit.Create(AOwner: TComponent);
begin
  inherited;

  FShowModify := True;
  FShowDelete := True;
  FModified := False;
  FEditable := True;
  key_pressed := False;
  FDontClearEdit := False;
  edtInput.OnChange := OnEditChange;
end;

procedure TbfaInputEdit.DoCancel(Sender: TObject);
begin
  if newmode then
    btnNew.SetFocus;
  if editmode then
    btnModify.SetFocus;

  newmode := False;
  editmode := False;
  EnableButtons(True);
  edtInput.Text := '';
  if Assigned(FOnEndEdit) then
    FOnEndEdit(Sender);
end;

procedure TbfaInputEdit.DoSave(Sender: TObject);
var
  value : WideString;
begin
    edtInput.Text := Trim(edtInput.Text);
    if newmode then
    begin
      FModified := True;
      if  (edtInput.Text <> '') then
      begin
        value := edtInput.Text;
      end;
      newmode := False;
      EnableButtons(True);
      btnNew.SetFocus;
      if Assigned(FOnNew) then
        FOnNew(Self, value);
      if Assigned(FOnEndEdit) then
        FOnEndEdit(Sender);
    end;
    if editmode then
    begin
      FModified := True;
      if (edtInput.Text <> '') then
      begin
        value := edtInput.Text;
      end;
      editmode := False;
      EnableButtons(True);
      btnModify.SetFocus;
      if Assigned(FOnModify) then
        FOnModify(Self, value);
      if Assigned(FOnEndEdit) then
        FOnEndEdit(Sender);
    end;
end;

procedure TbfaInputEdit.EnableButtons(enable: Boolean);
begin
  if newmode or editmode then
  begin
    btnNew.Caption := _('Ok');
    btnModify.Caption := _('Abbrechen');
    btnNew.Enabled := True;
    btnModify.Enabled := True;
  end else
  begin
    btnNew.Caption := _('Neu');
    btnModify.Caption := _('ndern');
    btnNew.Enabled := enable and FEditable;
    btnModify.Enabled := enable and FEditable;
  end;

  btnDelete.Enabled := enable and FEditable;
end;

procedure TbfaInputEdit.OnEditChange(Sender: TObject);
begin
//  OutputDebugString('TbfaInputEdit.OnEditChange');
  try
   if key_paste then
    begin
      edtInput.Text := Trim(ParseNotAllowedChars(edtInput.Text));
      FModified := edtInput.Text <> FOldValue;
      key_paste := False;
      Abort;
    end;

    if key_pressed then
      Exit;

    if newmode or editmode then
      Exit;

    if FDirectMode and not key_pressed then
    begin
      FModified := False;   // wenn der Wert von auerhalb setzen, dann setzen wir Modified zurck
      FOldValue := edtInput.Text;
    end;

    if Assigned(FOnchange) then
      FOnChange(Sender);

  finally
    key_pressed := False;
  end;

end;

procedure TbfaInputEdit.ResetEdit;
begin
  FModified := False;   // wenn der Wert von auerhalb setzen, dann setzen wir Modified zurck
  FOldValue := edtInput.Text;
end;

procedure TbfaInputEdit.SelEditable(const Value: Boolean);
begin
  FEditable := Value;
  EnableButtons(Value);
end;

procedure TbfaInputEdit.SetDisableButtons(const Value: Boolean);
begin
  FDisableButtons := Value;
  btnModify.Enabled := not FDisableButtons;
  btnDelete.Enabled := not FDisableButtons;
end;

procedure TbfaInputEdit.SetShowDelete(const Value: Boolean);
begin
  FShowDelete := Value;
  btnDelete.Visible := Value;
end;

procedure TbfaInputEdit.SetShowModify(const Value: Boolean);
begin
  FShowModify := Value;
  btnModify.Visible := Value;
end;

end.
