unit u_bfaSelector;

interface

uses
  gnugettext, Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, SpTBXControls, SpTBXItem, StdCtrls, SpTBXEditors, ComCtrls, TB2Item,
  Menus, SpTBXDkPanels;

type
  TSelectorKind = (skNormal, skGroupsUser, skGroups, skUser);

  TbfaSelector = class(TFrame)
    popFilter: TSpTBXPopupMenu;
    mnuGroupsUser: TSpTBXItem;
    mnuGroups: TSpTBXItem;
    mnuUser: TSpTBXItem;
    SpTBXPanel1: TSpTBXPanel;
    lblFilter1: TSpTBXLabel;
    edtFilter1: TSpTBXEdit;
    lvList1: TListView;
    SpTBXSplitter1: TSpTBXSplitter;
    SpTBXPanel2: TSpTBXPanel;
    edtFilter2: TSpTBXEdit;
    lblFilter2: TSpTBXLabel;
    btn1: TSpTBXSpeedButton;
    btn2: TSpTBXSpeedButton;
    lvList2: TListView;
    btnClear1: TSpTBXSpeedButton;
    btnClear2: TSpTBXSpeedButton;
    popInfo1: TSpTBXPopupMenu;
    popInfo2: TSpTBXPopupMenu;

    procedure edtFilter1KeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure btn1Click(Sender: TObject);
    procedure btn2Click(Sender: TObject);
    procedure lvList1KeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure lvList2KeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure lvList1DragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure lvList2DragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure lvList2DragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure lvList1DragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure edtFilter2KeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure lvList2SelectItem(Sender: TObject; Item: TListItem; Selected:
        Boolean);
    procedure mnuGroupsClick(Sender: TObject);
    procedure mnuGroupsUserClick(Sender: TObject);
    procedure mnuUserClick(Sender: TObject);
    procedure SpTBXSplitter1Moving(Sender: TObject; var NewSize: Integer; var
        Accept: Boolean);
    procedure lvList1SelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure lvList1DblClick(Sender: TObject);
    procedure lvList2DblClick(Sender: TObject);
    procedure btnClear1Click(Sender: TObject);
    procedure btnClear2Click(Sender: TObject);
    procedure lvList1ColumnClick(Sender: TObject; Column: TListColumn);
    procedure lvList1ColumnRightClick(Sender: TObject; Column: TListColumn;
      Point: TPoint);
    procedure popInfoPopup(Sender: TObject);
    procedure SpTBXSplitter1Moved(Sender: TObject);
    procedure edtFilter1KeyPress(Sender: TObject; var Key: Char);
    procedure edtFilter2KeyPress(Sender: TObject; var Key: Char);
  private
    { Private-Deklarationen }

    FListAll: TStringList;
    FListSelected: TStringList;
    FImageList: TImageList;
    FSelectorKind: TSelectorKind;
    FModified: Boolean;
    FEditable: Boolean;
    FDisableControls: Boolean;
    FAllowChangeKind: Boolean;
    FSelectedSortable: Boolean;
    FFieldTypes: TStringList;
    FShowCheckboxes: Boolean;
    FRejectDoubles: Boolean;
    FSkipSelecting: Boolean;

    procedure SetListAll(const Value: TStringList);
    procedure SetListSelected(const Value: TStringList);
    procedure ListAllChange(Sender: TObject);
    procedure ListSelectedChange(Sender: TObject);
    procedure AddToSelected;
    procedure RemoveFromSelected;
    procedure SetImageList(const Value: TImageList);
    procedure AddItem(AView: TListView; AValue: string; AObject: TObject);
    procedure ShowListView(AView: TListView; AList : TStringList; AExclude: TStringList = nil; search: string = '');
    procedure SetSelectorKind(const AValue: TSelectorKind);
    procedure SetColumnCaptionsListView1;

    procedure SetEditable(const Value: Boolean);
    procedure SetDisableControls(const Value: Boolean);
    procedure SetAllowChangeKind(const Value: Boolean);
    procedure SetSelectedSortable(const Value: Boolean);
    procedure SetShowCheckboxes(const Value: Boolean);
    procedure ExchangeItems(lv: TListView; const src, dst: Integer);
    procedure InitListFieldTypes;
  public
    { Public-Deklarationen }
    FSkipChecking : Boolean;

    OnSelectSelected: procedure (AValue: WideString; AObject: TObject) of object;
    OnSelectAll: procedure (AValue: WideString; AObject: TObject) of object;
    OnDblClickSelected: procedure (AValue: WideString; AObject: TObject) of object;
    OnDblClickAll: procedure (AValue: WideString; AObject: TObject) of object;
    OnSelectedEmpty: procedure of object;
    OnDeselect: procedure of object;
    OnAddToSelected: procedure (AValue: WideString; AObject: TObject) of object;
    OnRemoveFromSelected: procedure (AValue: WideString; AObject: TObject) of object;
    OnSelectedSorted: procedure(AObject: TObject; AItems: TListItems) of object;

    OnClickItems: procedure(Sender: TObject) of object;

    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure AddListAll(AValue: string); overload;
    procedure AddListAll(AValue: string; AObject: TObject); overload;
    procedure AddListSelected(AValue: string); overload;
    procedure AddListSelected(AValue: string; AObject: TObject); overload;
    procedure ClearSelected;
    procedure SetColumnCaptionSelected(Column1, Column2 : Widestring);
    procedure Retranslate;

    property ImageList: TImageList read FImageList write SetImageList;
    property ListAll: TStringList read FListAll write SetListAll;
    property  ListSelected: TStringList read FListSelected write SetListSelected;
    property Modified: Boolean read FModified write FModified;
    property Editable: Boolean read FEditable write SetEditable;
    property DisableControls: Boolean read FDisableControls write SetDisableControls;
    property ShowCheckboxes: Boolean read FShowCheckboxes write SetShowCheckboxes;
    property SkipChecking: Boolean read FSkipChecking;

  published
    property SelectorKind: TSelectorKind read FSelectorKind write SetSelectorKind default skNormal;
    property AllowChangeKind: Boolean read FAllowChangeKind  write SetAllowChangeKind default True;
    property SelectedSortable: Boolean read FSelectedSortable write SetSelectedSortable default False;
    property RejectDoubles: Boolean read FRejectDoubles write FRejectDoubles default True;

  end;

implementation

{$R *.dfm}
uses
  bfaBaseObjects, bfaObjects36, bfaCommonUtils, IOUtils, bfaCommonStyles, bfaCommonMethods;

procedure TbfaSelector.AddItem(AView: TListView; AValue: string; AObject: TObject);
var
  item : TListItem;
begin
  item := AView.Items.Add;

  if FShowCheckboxes and (AView = lvList2) then
    item.SubItems.Add(AValue)
  else
    item.Caption := AValue;
  item.Data := AObject;
  if TObject(item.Data) is TbfaUser then
  begin
    if Assigned(FImageList) then
      item.ImageIndex := Integer(icoidx_user);
  end;
  if TObject(item.Data) is TbfaGroup then
  begin
    if Assigned(FImageList) then
      item.ImageIndex := Integer(icoidx_group);
  end;
  if TObject(item.Data) is TBfaADObject then
  begin
    item.Checked := (TBfaADObject(item.Data).Flags and 8 = 8);
  end;
end;

procedure TbfaSelector.AddListAll(AValue: string);
begin
  // bekommen wir einen Eintrag ohne Objekt, legen wir ein TbfaInt Object an
  // einfach ein Integer ber TObject(int) casten funktioniert nicht, wenn der Integer 0 sein soll
  // da bei der Abfrage auf den Typ des Objects ein TObject(0) = nil ist
  // und die Objektprfung gegen nil eine Exception wirft
  if FListAll.IndexOf(AValue) = -1 then
    FListAll.AddObject(AValue, TbfaInt.Create(FListAll.Count + FListSelected.Count));
end;

procedure TbfaSelector.AddListAll(AValue: string; AObject: TObject);
begin
  if FListAll.IndexOfObject(AObject) = -1 then
    FListAll.AddObject(AValue, AObject);
end;

procedure TbfaSelector.AddListSelected(AValue: string);
begin
  // bekommen wir einen Eintrag ohne Objekt, legen wir ein TbfaInt Object an
  // einfach ein Integer ber TObject(int) casten funktioniert nicht, wenn der Integer 0 sein soll
  // da bei der Abfrage auf den Typ des Objects ein TObject(0) = nil ist
  // und die Objektprfung gegen nil eine Exception wirft
   if FListSelected.IndexOf(AValue) = -1 then
    FListSelected.AddObject(AValue, TbfaInt.Create(FListAll.Count + FListSelected.Count));
end;

procedure TbfaSelector.AddListSelected(AValue: string; AObject: TObject);
begin
   if FListSelected.IndexOfObject(AObject) = -1 then
     FListSelected.AddObject(AValue, AObject);
end;

procedure TbfaSelector.AddToSelected;
var
  idx, idx2, i, lastselected, selcount: Integer;
  sel: TStringList;
  j: Integer;
  found : Boolean;
begin
  if lvList1.SelCount = 0 then
    Exit;

  lastselected := -1;
  selcount := 0;

  sel := TStringList.Create;
  FSkipChecking := True;
  try
    for i := 0 to lvList1.Items.Count -1 do
    begin
      if lvList1.Items[i].Selected then
      begin
        sel.AddObject(lvList1.Items[i].Caption, lvList1.Items[i].Data);
        lastselected := i;
        inc(selcount);
      end;
    end;

    for i := 0 to sel.Count -1 do
    begin
      if FRejectDoubles then     // bei dieser Option verlassen wir die Routine wenn der selektiert Name schon in ListSelected vorhanden ist
      begin
        found := False;
        for j := 0 to FListSelected.Count -1 do
          if sel[i] = FListSelected[j] then
          begin
            found := True;
            Break;
          end;
          if found then
            Exit;
      end;

      idx := FListAll.IndexOfObject(sel.Objects[i]);  // Index des Eintrages in der Liste
      idx2 := FListSelected.IndexOfObject(sel.Objects[i]);  // Index des Eintrages in der Liste

      if (idx > -1) and (idx2 = -1) then     // wenn wir ein Object aus ListAll haben und das ist noch nicht in ListSelected
      begin
        FListSelected.AddObject(FListAll[idx],FListAll.Objects[idx]);
        if Assigned(OnAddToSelected) then
          OnAddToSelected(FListAll[idx],FListAll.Objects[idx]);

      end;

    end;

    lastselected := lastselected - (selcount -1);  // wurden mehr als eins selektiert, Anzahl (-1) abziehen
    lvList1.DeleteSelected;
    if (lastselected > -1) then
    begin
      if lastselected < lvList1.Items.Count then
        lvList1.ItemIndex := lastselected
      else
        lvList1.ItemIndex := lvList1.Items.Count -1;
    end;

    ShowListView(lvList2, FListSelected);
    FSkipSelecting := True;
    if sel.Count > 0 then
      lvList2.ItemIndex := FListSelected.IndexOfObject(sel.Objects[0]);

    FModified := True;
  finally
    FreeAndNil(sel);
    FSkipChecking := False;
    FSkipSelecting := False;
  end;
end;

procedure TbfaSelector.btn1Click(Sender: TObject);
begin
  AddToSelected;
end;

procedure TbfaSelector.btn2Click(Sender: TObject);
begin
  RemoveFromSelected;
end;

procedure TbfaSelector.btnClear1Click(Sender: TObject);
begin
  edtFilter1.Text := '';
  ShowListView(lvList1, FListAll, FListSelected, LowerCase(Trim(edtFilter1.Text)));
end;

procedure TbfaSelector.btnClear2Click(Sender: TObject);
begin
  edtFilter2.Text := '';
  ShowListView(lvList2, FListSelected, nil, LowerCase(Trim(edtFilter2.Text)));
end;

procedure TbfaSelector.ReTranslate;
begin
  InitListFieldTypes;

  lblFilter1.Caption := _('Filter');
  lblFilter2.Caption := _('Filter');
  btnClear1.Hint := _('Filter entfernen (ESC)');
  btnClear2.Hint := _('Filter entfernen (ESC)');

  SetColumnCaptionsListView1;

  if lvList2.Columns.Count > 0 then
    lvList2.Column[0].Caption := _('Auswahl');
end;

constructor TbfaSelector.Create(AOwner: TComponent);
var
  Col: TListColumn;
begin
  inherited;

  FListAll := TStringList.Create;
  FListAll.OnChange := ListAllChange;
  FListAll.Sorted := True;
  FListAll.Duplicates := dupAccept;
  FListSelected := TStringList.Create;
  FListSelected.OnChange := ListSelectedChange;
  FListSelected.Sorted := True;
  FListSelected.Duplicates := dupAccept;

  FSkipChecking := False;
  FSkipSelecting := False;

  Col := lvList1.Columns.Add;
  //Col.Caption := _('Name');
  Col.Alignment := taLeftJustify;
  Col.Width := 140;
  Col.AutoSize := True;
  SetColumnCaptionsListView1;

  Col := lvList2.Columns.Add;
  Col.Caption := _('Auswahl');
  Col.Alignment := taLeftJustify;
  Col.Width := 140;
  Col.AutoSize := True;
  FModified := False;
  FEditable := True;

  FAllowChangeKind := True;
  FFieldTypes := TStringList.Create;
  InitListFieldTypes;

  TranslateComponent(Self,'Shared36');
end;

procedure TbfaSelector.InitListFieldTypes;
begin
  FFieldTypes.Clear;
  FFieldTypes.Add(_('Universal'));
  FFieldTypes.Add(_('Datum'));
  FFieldTypes.Add(_('Auswahl'));
  FFieldTypes.Add(_('Mehrfach'));
  FFieldTypes.Add(_('Invisible'));
  FFieldTypes.Add(_('Interval'));
  FFieldTypes.Add(_('Ganzzahl'));
  FFieldTypes.Add(_('Fliekomma'));
  FFieldTypes.Add(_('Whrung'));
  
  FFieldTypes.Add(_('9'));
  FFieldTypes.Add(_('10'));
  FFieldTypes.Add(_('DLM-Auswahlfeld'));
  FFieldTypes.Add(_('DLM-Datumsfeld'));
  FFieldTypes.Add(_('Tabellenfeld'));
  FFieldTypes.Add(_('Tabellenspalte'));

end;

destructor TbfaSelector.Destroy;
begin
  if Assigned(FListAll) then
    FListAll.Clear;
  if Assigned(FListSelected) then
    FListSelected.Clear;
  if Assigned(FFieldTypes) then
    FFieldTypes.Clear;
  FreeAndNil(FListAll);
  FreeAndNil(FListSelected);
  FreeAndNil(FFieldTypes);
  inherited Destroy;
end;

procedure TbfaSelector.edtFilter1KeyPress(Sender: TObject; var Key: Char);
begin
  DisableBeep(Key); //11.03.2024 bfacrm:ticket__show__38134
end;

procedure TbfaSelector.edtFilter1KeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
  search : String;
begin
  if Key = 27 then
  begin
    edtFilter1.Text := '';
    Key := 0;
  end;

  search := LowerCase(Trim(edtFilter1.Text));
  ShowListView(lvList1, FListAll, FListSelected, search);
end;

procedure TbfaSelector.edtFilter2KeyPress(Sender: TObject; var Key: Char);
begin
  DisableBeep(Key); //11.03.2024 bfacrm:ticket__show__38134
end;

procedure TbfaSelector.edtFilter2KeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
  search : String;
begin
  if Key = 27 then
  begin
    edtFilter2.Text := '';
    Key := 0;
  end;

  search := LowerCase(Trim(edtFilter2.Text));
  ShowListView(lvList2, FListSelected, nil, search);
end;

procedure TbfaSelector.ExchangeItems(lv: TListView; const src, dst: Integer);
var
  temp: TListItem;
begin
  lv.Items.BeginUpdate;
  try
    temp := TListItem.Create(lv.Items);
    temp.Assign(lv.Items.Item[src]);
    lv.Items.Item[src].Assign(lv.Items.Item[dst]);
    lv.Items.Item[dst].Assign(temp);
    temp.Free;
  finally
    lv.Items.EndUpdate
  end;
end;

procedure TbfaSelector.ListAllChange(Sender: TObject);
begin
  if FDisableControls then
    Exit;

//  for i := 0  to FListAll.Count -1 do
//  begin
//    // Elemente in ListSelected knnen nicht in ListAll sein
//
//    if FListSelected.IndexOfObject(FListAll.Objects[i]) > -1 then
//      FListSelected.Delete(FListSelected.IndexOfObject(FListAll.Objects[i]));
//  end;
  ShowListView(lvList1, FListAll, FListSelected);
end;


procedure TbfaSelector.ListSelectedChange(Sender: TObject);
begin
  if FDisableControls then
    Exit;

//  for i := 0  to FListSelected.Count -1 do
//  begin
//    if FListAll.IndexOfObject(FListSelected.Objects[i]) > -1 then
//      FListAll.Delete(FListAll.IndexOfObject(FListSelected.Objects[i]));
//  end;
  ShowListView(lvList2, FListSelected);

end;

procedure TbfaSelector.lvList1ColumnClick(Sender: TObject; Column: TListColumn);
begin
//09.11.2017  //OutputDebugString('lvList1ColumnClick');
end;

procedure TbfaSelector.lvList1ColumnRightClick(Sender: TObject;
  Column: TListColumn; Point: TPoint);
begin
//09.11.2017  //OutputDebugString('lvList1ColumnRightClick');

  if (FSelectorKind in [skUser, skGroups, skGroupsUser]) and FAllowChangeKind then
    popFilter.Popup(Mouse.CursorPos.X, Mouse.CursorPos.Y);

end;

procedure TbfaSelector.lvList1DblClick(Sender: TObject);
var
  Item: TListItem;
begin
  if not FEditable then
    Exit;

  if lvList1.Selected = nil then
    Exit;
  Item:= lvList1.Selected;
  if Assigned(OnDblClickAll) and (Item.Data <> nil) then
    OnDblClickAll(Item.Caption, Item.Data)
  else
    AddToSelected;
end;

procedure TbfaSelector.lvList1DragDrop(Sender, Source: TObject; X, Y: Integer);
begin
  RemoveFromSelected;
end;

procedure TbfaSelector.lvList1DragOver(Sender, Source: TObject; X, Y: Integer;
  State: TDragState; var Accept: Boolean);
begin
  Accept := (Source = lvList2) and FEditable;

end;

procedure TbfaSelector.lvList1KeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
  idx : Integer;
  //24.09.2024 SD -->bfacrm:ticket__show__38613
  inputChar: array[0..1] of WideChar;
  inputStr, search : String;
  KeyState: TKeyboardState;
  usesearch: Boolean;
  //24.09.2024 SD <--bfacrm:ticket__show__38613
begin
  if (Key = VK_RETURN) and (lvList1.SelCount > 0) and FEditable then
  begin
    idx := lvList1.Selected.Index;
    AddToSelected;
    if idx < lvList1.Items.Count then
      lvList1.Items[idx].Selected := True
    else if lvList1.Items.Count > 0 then
      lvList1.Items[lvList1.Items.Count -1].Selected := True;
  end
  //24.09.2024 SD -->bfacrm:ticket__show__38613
  else
  begin
    usesearch := False;
    if (Key in [32..126, 188..190]) then //['a'..'z', 'A'..'Z', '0'..'9', '-', '_', '.', ',', ';', ':']
    begin
      GetKeyboardState(KeyState);
      if ToUnicode(Key, MapVirtualKey(Key, 0), KeyState, @inputChar[0], 2, 0) > 0 Then
      begin
        inputStr := inputChar[0];
        edtFilter1.Text := edtFilter1.Text + inputStr;
        usesearch := True;
      end
    end
    else if Key = 8 then  //Backspace
    begin
      inputStr := edtFilter1.Text;
      Delete(inputStr, Length(inputStr), 1);
      edtFilter1.Text := inputStr;
      usesearch := True;
    end
    else if Key = 27 then //Esc
    begin
      if edtFilter1.Text <> '' then
      begin
        edtFilter1.Text := '';
        Key := 0;
        usesearch := True;
      end;
    end;
    if usesearch then
    begin
      search := LowerCase(Trim(edtFilter1.Text));
      ShowListView(lvList1, FListAll, FListSelected, search);
      if lvList1.Items.Count > 0 then
        lvList1.ItemIndex := 0;
    end;
   end;
   //24.09.2024 SD <--bfacrm:ticket__show__38613
end;

procedure TbfaSelector.lvList1SelectItem(Sender: TObject; Item: TListItem;
  Selected: Boolean);
begin
  if Assigned(OnSelectAll) and (Item.Data <> nil) then
    OnSelectAll(Item.Caption, Item.Data);
end;

procedure TbfaSelector.lvList2DblClick(Sender: TObject);
var
  Item: TListItem;
begin
  if not FEditable then
    Exit;

  Item:= lvList2.Selected;
  if Assigned(OnDblClickSelected) and (Item.Data <> nil) then
    OnDblClickSelected(Item.Caption, Item.Data)
  else
    RemoveFromSelected;
end;

procedure TbfaSelector.lvList2DragDrop(Sender, Source: TObject; X, Y: Integer);
var
  DragItem, DropItem, CurrentItem, NextItem: TListItem;
  idx : Integer;
begin
  idx := -1;
  if Source = lvList1 then
    AddToSelected;
  if Source = lvList2 then
  begin
    with Sender as TListView do
    begin
      DropItem    := GetItemAt(X, Y);
      CurrentItem := Selected;
      while CurrentItem <> nil do
      begin
        NextItem := GetNextItem(CurrentItem, SdAll, [IsSelected]);
        if DropItem = nil then DragItem := Items.Add
        else
          if (DropItem.Index = 0) or (CurrentItem.Index > DropItem.Index) then
            DragItem := Items.Insert(DropItem.Index)

          else
            DragItem := Items.Insert(DropItem.Index +1);
        DragItem.Assign(CurrentItem);
        CurrentItem.Free;
        CurrentItem := NextItem;
        if idx = -1 then
          idx := DragItem.Index;
      end;
      ItemIndex := idx;

      if Assigned(OnSelectedSorted) then
        OnSelectedSorted(Self, lvList2.Items);
     end;
  end;
end;

procedure TbfaSelector.lvList2DragOver(Sender, Source: TObject; X, Y: Integer;
  State: TDragState; var Accept: Boolean);
begin
  Accept := ((Source = lvList1) or FSelectedSortable) and FEditable;

end;

procedure TbfaSelector.lvList2KeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
  idx : Integer;
  //24.09.2024 SD -->bfacrm:ticket__show__38613
  inputChar: array[0..1] of WideChar;
  inputStr, search : String;
  KeyState: TKeyboardState;
  usesearch: Boolean;
  //24.09.2024 SD <--bfacrm:ticket__show__38613
begin
  // Fall: Es treten hier oft Exceptions auf (Drei Crashlogs: lvList2.Selected.Index)
  // Nicht reproduzierbar, daher Manahmen auf Verdacht:
  // a) try Except
  // b) Wenn nichts selektiert (Selcount = 0), dann auch kein Ermitteln des Selected.Index

  try
    usesearch := True;  //24.09.2024 SD <--bfacrm:ticket__show__38613
    if FEditable and (lvList2.SelCount > 0) then
    begin
      idx := lvList2.Selected.Index;
      if (Key = VK_DELETE) {and (lvList2.SelCount > 0)} then
      begin
        RemoveFromSelected;
        if idx < lvList2.Items.Count then
          lvList2.Items[idx].Selected := True
        else if lvList2.Items.Count > 0 then
          lvList2.Items[lvList2.Items.Count -1].Selected := True;
        usesearch := False; //24.09.2024 SD <--bfacrm:ticket__show__38613
      end;

      if (Key = VK_UP) and (ssCtrl in Shift) {and (lvList2.SelCount > 0)} then
      begin
        if idx > 0 then
        begin
          ExchangeItems(lvList2, idx, idx -1);
          lvList2.Items[idx].Selected := False;
          lvList2.ItemIndex := idx -1;
          if Assigned(OnSelectedSorted) then
            OnSelectedSorted(Self, lvList2.Items);
        end;
        usesearch := False; //24.09.2024 SD <--bfacrm:ticket__show__38613
      end;

      if (Key = VK_Down) and (ssCtrl in Shift) {and (lvList2.SelCount > 0)} then
      begin
        if idx < lvList2.Items.Count -1 then
        begin
          ExchangeItems(lvList2, idx, idx +1);
          lvList2.Items[idx].Selected := False;
          lvList2.ItemIndex := idx +1;
          if Assigned(OnSelectedSorted) then
            OnSelectedSorted(Self, lvList2.Items);
        end;
        usesearch := False; //24.09.2024 SD <--bfacrm:ticket__show__38613
      end;
    end;
    //24.09.2024 SD -->bfacrm:ticket__show__38613
    if usesearch then
    begin
      usesearch := False;
      if (Key in [32..126, 188..190]) then //['a'..'z', 'A'..'Z', '0'..'9', '-', '_', '.', ',', ';', ':']
      begin
        GetKeyboardState(KeyState);
        if ToUnicode(Key, MapVirtualKey(Key, 0), KeyState, @inputChar[0], 2, 0) > 0 Then
        begin
          inputStr := inputChar[0];
          edtFilter2.Text := edtFilter2.Text + inputStr;
          usesearch := True;
        end
      end
      else if Key = 8 then  //Backspace
      begin
        inputStr := edtFilter2.Text;
        Delete(inputStr, Length(inputStr), 1);
        edtFilter2.Text := inputStr;
        usesearch := True;
      end
      else if Key = 27 then //Esc
      begin
        if edtFilter2.Text <> '' then
        begin
          edtFilter2.Text := '';
          Key := 0;
          usesearch := True;
        end;
      end;
      if usesearch then
      begin
        search := LowerCase(Trim(edtFilter2.Text));
        ShowListView(lvList2, FListSelected, nil, search);
        if lvList2.Items.Count > 0 then
          lvList2.ItemIndex := 0;
      end;
     end;
     //24.09.2024 SD <--bfacrm:ticket__show__38613

  finally

  end;
end;

procedure TbfaSelector.mnuGroupsClick(Sender: TObject);
begin
  SelectorKind := skGroups;

end;

procedure TbfaSelector.mnuGroupsUserClick(Sender: TObject);
begin
  SelectorKind := skGroupsUser;
end;

procedure TbfaSelector.mnuUserClick(Sender: TObject);
begin
  SelectorKind := skUser;
end;


procedure TbfaSelector.popInfoPopup(Sender: TObject);
  function ModeToString(mode: Integer): string;
  begin
    try
      case mode of
        2: Result := _('Wortanfang');
        1: Result := _('Wortende');
        3: Result := _('Bestandteil');
        else
          Result := _('Exakt');
      end;
    except
      Result := '';
    end;
  end;
  function AddfieldOptionsToString(options: Integer): string;
  begin
    try
      Result := '';

      if (options and cAddfieldOptStrict) = cAddfieldOptStrict then
        Result := _('strikte Auswahl');

      if (options and cAddfieldOptMandatory) = cAddfieldOptMandatory then
      begin
        if Result <> '' then
          Result := Result + ', ';
        Result := Result + _('Pflichtfeld');
      end;

      if (options and cAddfieldOptLink) = cAddfieldOptLink then
      begin
        if Result <> '' then
          Result := Result + ', ';
        Result := Result + _('Verknpfungsfeld');
      end;

      if (options and cAddfieldOptNaming) = cAddfieldOptNaming then
      begin
        if Result <> '' then
          Result := Result + ', ';
        Result := Result + _('Dateiname bei Versand oder Speichern');
      end;
    except
      // nicht leer setzen.
    end;
  end;
var
//29.02.2024 bfacrm:ticket__show__35906  item: TTBRootItem;
  item: TSpTBXItem;
  litem : TListItem;
begin
  if (Sender = popInfo1) and (lvList1.Selected = nil) then
    Abort;
  if (Sender = popInfo2) and (lvList2.Selected = nil) then
    Abort;

  TSpTBXPopupMenu(Sender).Items.Clear;
  //29.02.2024 bfacrm:ticket__show__35906 -->
  //item := TTBRootItem.Create(TSpTBXPopupMenu(Sender));
  item := TSpTBXItem.Create(TSpTBXPopupMenu(Sender));
  if stlControl <> nil then
    TSpTBXItem(item).FontSettings.Color := stlControl.TextColor;
  //29.02.2024 bfacrm:ticket__show__35906 -->
  if Sender = popInfo1 then
  begin
    litem := lvList1.Selected;
  end
  else begin
    litem := lvList2.Selected;
    item.Caption := lvList2.Selected.Caption;
  end;

  if TObject(litem.Data) is TbfaUser then
  begin
    item.Caption := 'ID: ' + IntToStr(TbfaUser(litem.Data).ID);
    item.Tag := Integer(TbfaUser(litem.Data));
  end
  else if TObject(litem.Data) is TbfaGroup then
  begin
    item.Caption := 'ID: ' + IntToStr(TbfaGroup(litem.Data).ID);
    item.Tag := Integer(TbfaGroup(litem.Data));
  end
  else if TObject(litem.Data) is TbfaAddField then
  begin
    // ID
    item.Caption := 'ID: ' + IntToStr(TbfaAddField(litem.Data).ID);
    item.Tag := Integer(TbfaAddField(litem.Data));
    if Assigned(OnClickItems) then
      item.OnClick := OnClickItems;
    TSpTBXPopupMenu(Sender).Items.Add(item);

    // Typ
    //29.02.2024 bfacrm:ticket__show__35906 -->
    //item := TTBRootItem.Create(TSpTBXPopupMenu(Sender));
    item := TSpTBXItem.Create(TSpTBXPopupMenu(Sender));
    if stlControl <> nil then
      TSpTBXItem(item).FontSettings.Color := stlControl.TextColor;
    //29.02.2024 bfacrm:ticket__show__35906 -->
    item.Caption := _('Typ') + ': ' + FFieldTypes[TbfaAddfield(litem.Data).Typ];
    item.Tag := Integer(TbfaAddField(litem.Data));
    if Assigned(OnClickItems) then
      item.OnClick := OnClickItems;
    TSpTBXPopupMenu(Sender).Items.Add(item);

    // Beschreibung
    if TbfaAddfield(litem.Data).Description <> '' then
    begin
      //29.02.2024 bfacrm:ticket__show__35906 -->
      //item := TTBRootItem.Create(TSpTBXPopupMenu(Sender));
      item := TSpTBXItem.Create(TSpTBXPopupMenu(Sender));
      if stlControl <> nil then
        TSpTBXItem(item).FontSettings.Color := stlControl.TextColor;
      //29.02.2024 bfacrm:ticket__show__35906 -->
      item.Caption := _('Beschreibung:')+ ' ' + TbfaAddfield(litem.Data).Description;
      item.Tag := Integer(TbfaAddField(litem.Data));
      if Assigned(OnClickItems) then
        item.OnClick := OnClickItems;
      TSpTBXPopupMenu(Sender).Items.Add(item);
    end;

    // Suchmethode
    //29.02.2024 bfacrm:ticket__show__35906 -->
    //item := TTBRootItem.Create(TSpTBXPopupMenu(Sender));
    item := TSpTBXItem.Create(TSpTBXPopupMenu(Sender));
    if stlControl <> nil then
      TSpTBXItem(item).FontSettings.Color := stlControl.TextColor;
    //29.02.2024 bfacrm:ticket__show__35906 -->
    item.Caption := _('Suchmethode')+ ': ' + ModeToString(TbfaAddfield(litem.Data).Mode);
    item.Tag := Integer(TbfaAddField(litem.Data));
    if Assigned(OnClickItems) then
      item.OnClick := OnClickItems;
    TSpTBXPopupMenu(Sender).Items.Add(item);

    // Optionen
    if TbfaAddfield(litem.Data).Options > 0 then
    begin
      //29.02.2024 bfacrm:ticket__show__35906 -->
      //item := TTBRootItem.Create(TSpTBXPopupMenu(Sender));
      item := TSpTBXItem.Create(TSpTBXPopupMenu(Sender));
      if stlControl <> nil then
        TSpTBXItem(item).FontSettings.Color := stlControl.TextColor;
      //29.02.2024 bfacrm:ticket__show__35906 -->
      item.Caption := _('Optionen')+ ': ' + AddfieldOptionsToString(TbfaAddfield(litem.Data).Options);
      item.Tag := Integer(TbfaAddField(litem.Data));
      if Assigned(OnClickItems) then
        item.OnClick := OnClickItems;
      TSpTBXPopupMenu(Sender).Items.Add(item);
    end;

    // Checkprogramm
    if TbfaAddfield(litem.Data).Checkprogram <> '' then
    begin
      //29.02.2024 bfacrm:ticket__show__35906 -->
      //item := TTBRootItem.Create(TSpTBXPopupMenu(Sender));
      item := TSpTBXItem.Create(TSpTBXPopupMenu(Sender));
      if stlControl <> nil then
        TSpTBXItem(item).FontSettings.Color := stlControl.TextColor;
      //29.02.2024 bfacrm:ticket__show__35906 -->
      item.Caption := _('Checkprogramm')+ ': ' + TPath.GetFileName(TbfaAddfield(litem.Data).Checkprogram);
      item.Tag := Integer(TbfaAddField(litem.Data));
      if Assigned(OnClickItems) then
        item.OnClick := OnClickItems;
      TSpTBXPopupMenu(Sender).Items.Add(item);
    end;

    Exit;
  end
  else if TObject(litem.Data) is TbfaStatefield then
  begin
    item.Caption := 'ID: ' + IntToStr(TbfaStatefield(litem.Data).ID);
    item.Tag := Integer(TbfaStatefield(litem.Data));
  end
  else
  begin
    item.Caption := litem.Caption;
    item.Tag := -1;
  end;

  if Assigned(OnClickItems) then
    item.OnClick := OnClickItems;

  TSpTBXPopupMenu(Sender).Items.Add(item);
end;

procedure TbfaSelector.ClearSelected;
var
  state : Boolean;
begin
  state := FDisableControls;
  FDisableControls := True;
  try
    if FListSelected.Count > 0 then
      FListSelected.Clear;
  finally
    FDisableControls := state;
  end;
  if Assigned(OnSelectedEmpty) then
    OnSelectedEmpty;
end;

procedure TbfaSelector.lvList2SelectItem(Sender: TObject; Item: TListItem;
    Selected: Boolean);
begin
  if FDisableControls then
    Exit;
  if FSkipSelecting then
    Exit;

  try
//    lvList2.DragMode := dmManual;
    if not Selected then
    begin
      if Assigned(OnDeselect) then
        OnDeselect;

      if lvList2.SelCount = 0 then
        Exit;
      if lvList2.SelCount > 1 then
        Exit
      else if Assigned(OnSelectSelected) and (lvList2.Selected.Data <> nil) then
      begin
        OnSelectSelected(lvList2.Selected.Caption, lvList2.Selected.Data);
        Exit;
      end;
    end;

    if Assigned(OnSelectSelected) and (Item.Data <> nil) then
    begin
      if FShowCheckboxes then
        OnSelectSelected(Item.SubItems[0], Item.Data)
      else
        OnSelectSelected(Item.Caption, Item.Data);
    end;
  finally
//    lvList2.DragMode := dmAutomatic;
  end;
end;

procedure TbfaSelector.RemoveFromSelected;
var
  idx, i, lastselected, selcount: Integer;
  sel : TStringList;
  search : String;
begin
  if lvList2.SelCount = 0 then
    Exit;
  lastselected := -1;
  selcount := 0;
  FSkipChecking := True;
  sel := TStringList.Create;
  try
    for i := 0 to lvList2.Items.Count -1 do
    begin
      if lvList2.Items[i].Selected then
      begin
        if FShowCheckboxes then
          sel.AddObject(lvList2.Items[i].SubItems[0], lvList2.Items[i].Data)
        else
          sel.AddObject(lvList2.Items[i].Caption, lvList2.Items[i].Data);
        lastselected := i;
        inc(selcount);
      end;
    end;

    for i := 0 to sel.Count -1 do
    begin
      idx := FListSelected.IndexOfObject(sel.Objects[i]);  // Index des Eintrages in der Liste

      if idx > -1 then
      begin
        FListSelected.Delete(idx);
        AddItem(lvList1, sel[i], sel.Objects[i]);
        if Assigned(OnRemoveFromSelected) then
          OnRemoveFromSelected(sel[i], sel.Objects[i]);
      end;
    end;

    lastselected := lastselected - (selcount -1);  // wurden mehr als eins selektiert, Anzahl (-1) abziehen
    lvList2.DeleteSelected;
    if (lastselected > -1) then
    begin
      if lastselected < lvList2.Items.Count then
        lvList2.ItemIndex := lastselected
      else
        lvList2.ItemIndex := lvList2.Items.Count -1;
    end;

    //24.09.2024 SD -->bfacrm:ticket__show__38613
//    ShowListView(lvList1, FListAll, FListSelected);
    search := LowerCase(Trim(edtFilter1.Text));
    ShowListView(lvList1, FListAll, FListSelected, search);
    if lvList1.Items.Count > 0 then
      lvList1.ItemIndex := 0;
    //24.09.2024 SD <--bfacrm:ticket__show__38613

    if (lvList2.Items.Count = 0) and Assigned(OnSelectedEmpty) then
      OnSelectedEmpty;

  finally
    FreeAndNil(sel);
    FSkipChecking := False;
  end;

  FModified := True;
end;

procedure TbfaSelector.SetAllowChangeKind(const Value: Boolean);
begin
  FAllowChangeKind := Value;
end;

procedure TbfaSelector.SetColumnCaptionsListView1;
begin
  lvList1.Columns[0].Caption := _('Name');
  if FSelectorKind = skGroupsUser then
  begin
    lvList1.Columns[0].Caption := _('Gruppen und Benutzer')
  end else if FSelectorKind = skGroups then
  begin
    lvList1.Columns[0].Caption := _('Gruppen')
  end else if FSelectorKind = skUser then
  begin
    lvList1.Columns[0].Caption := _('Benutzer')
  end;

end;

procedure TbfaSelector.SetColumnCaptionSelected(Column1, Column2: Widestring);
begin
  lvList2.Columns[0].Caption := Column1;
  if Assigned(lvList2.Columns[1]) then
    lvList2.Columns[1].Caption := Column2;
end;

procedure TbfaSelector.SetDisableControls(const Value: Boolean);
begin
  FDisableControls := Value;
  if FDisableControls then
  begin
    lvList1.Items.BeginUpdate;
    lvList2.Items.BeginUpdate;
    FListAll.BeginUpdate; //19.02.2024
    FListSelected.BeginUpdate; //19.02.2024
  end else if not FDisableControls then  // wenn DisableControls zurckgesetzt wird, gehen wir davon aus, dass eine Liste geladen wurde
  begin                         // die wir jetzt auch sehen wollen
    lvList1.Items.EndUpdate;
    lvList2.Items.EndUpdate;
    FListAll.EndUpdate; //19.02.2024
    FListSelected.EndUpdate; //19.02.2024
//14.03.2024 wegen EndUpdate werden im Change aufgerufen -->
//    ShowListView(lvList1, FListAll, FListSelected);
//    ShowListView(lvList2, FListSelected);
//14.03.2024 wegen EndUpdate werden im Change aufgerufen <--
  end;
end;

procedure TbfaSelector.SetEditable(const Value: Boolean);
begin
  FEditable := Value;
    if Assigned(btn1) then
      btn1.Enabled := FEditable;
  if Assigned(btn2) then
    btn2.Enabled := FEditable;
end;

procedure TbfaSelector.SetImageList(const Value: TImageList);
begin
  FImageList := Value;
  lvList1.SmallImages := Value;
  lvList2.SmallImages := Value;
end;


procedure TbfaSelector.SetListAll(const Value: TStringList);
begin
  FListAll.Assign(Value);
end;

procedure TbfaSelector.SetListSelected(const Value: TStringList);
begin
  FListSelected.Assign(Value);
end;

procedure TbfaSelector.SetSelectedSortable(const Value: Boolean);
begin
  FSelectedSortable := Value;
  if FSelectedSortable then
    lvList2.SortType := stNone
  else
    lvList2.SortType := stText;
  FListSelected.Sorted := not Value;
end;

procedure TbfaSelector.SetSelectorKind(const AValue: TSelectorKind);
begin
  if AValue <> FSelectorKind then
  begin
    FSelectorKind := AValue;
    SetColumnCaptionsListView1;
    ShowListView(lvList1, FListAll, FListSelected);
  end;
end;

procedure TbfaSelector.SetShowCheckboxes(const Value: Boolean);
var
  columnitem: TListColumn;
begin
  FShowCheckboxes := Value;
  lvList2.Checkboxes := FShowCheckboxes;
  lvList2.RowSelect := FShowCheckboxes;
  lvList2.Columns.Clear;

  if FShowCheckboxes then
  begin
    columnitem := lvList2.Columns.Add;
    columnitem.MinWidth := 21;
    columnitem.Width := 121;
    columnitem := lvList2.Columns.Add;
    columnitem.MinWidth := 21;
    columnitem.Caption := _('Auswahl');
    columnitem.AutoSize := True;
  end else begin
    columnitem := lvList2.Columns.Add;
    columnitem.MinWidth := 21;
    columnitem.Caption := _('Auswahl');
    columnitem.AutoSize := True;
  end;

end;

procedure TbfaSelector.ShowListView(AView: TListView; AList : TStringList; AExclude: TStringList = nil; search: string = '');
var
  i: Integer;
begin
  try
    AView.Clear;
    if (SelectorKind = skNormal) then
    begin
      if FSelectedSortable then
        AView.SortType := stNone
      else
        AView.SortType := stText;
      for i := 0 to AList.Count -1 do
      begin
        if (AExclude <> nil) and (AExclude.IndexOfObject(AList.Objects[i]) > -1) then
          Continue;
        if ((Pos(search, LowerCase(AList[i])) > 0) or (search = '')) then
        begin
          AddItem(AView, AList[i], AList.Objects[i]);
        end;
      end;
    end else if SelectorKind in [skGroupsUser, skGroups, skUser] then
    begin
      AView.SortType := stNone;                    // in der selektierten Liste zeigen wir immer alles an
      if (SelectorKind in [skGroupsUser, skGroups])  or (AView = lvList2) then   // erst Gruppen
      begin
        for i := 0 to AList.Count -1 do
        begin
          if AList.Objects[i] = nil then
            continue;
          if (AExclude <> nil) and (AExclude.IndexOfObject(AList.Objects[i]) > -1) then
            Continue;
          if ((Pos(search, LowerCase(AList[i])) > 0) or (search = '')) then
          begin
            If AList.Objects[i] is TbfaGroup then
              AddItem(AView, AList[i], AList.Objects[i]);
          end;
        end;
      end;                                       // in der selektierten Liste zeigen wir immer alles an
      if (SelectorKind in [skGroupsUser, skUser]) or (AView = lvList2) then   // dann User
      begin
        for i := 0 to AList.Count -1 do
        begin
          if ((Pos(search, LowerCase(AList[i])) > 0) or (search = '')) then
          begin
            if AList.Objects[i] = nil then
              continue;
            if (AExclude <> nil) and (AExclude.IndexOfObject(AList.Objects[i]) > -1) then
              Continue;
            If AList.Objects[i] is TbfaUser then
              AddItem(AView, AList[i], AList.Objects[i]);
          end;
        end;
      end;
    end;
  except
    //25.09.2023 CrashLog
  end;
end;

procedure TbfaSelector.SpTBXSplitter1Moved(Sender: TObject);
var
  Diff : Integer;
begin
  if Self.Left + Self.Width >= Self.Parent.Width then
  begin
//      Diff := SpTBXSplitter1.Left - Self.SpTBXPanel1.Width;
//      Self.SpTBXPanel1.Width := Self.SpTBXPanel1.Width - Diff;
//      SpTBXSplitter1.Left := Self.SpTBXPanel1.Width;
//      Self.SpTBXPanel2.Width := Self.SpTBXPanel2.Constraints.MinWidth;
//      if Self.Left + Self.Width >= Self.Parent.Width then
//        Self.Width := Self.Width - Diff;

//     Self.Width := Self.Width - 10;
  end;
end;

procedure TbfaSelector.SpTBXSplitter1Moving(Sender: TObject; var NewSize:
    Integer; var Accept: Boolean);
var
  Diff : Integer;
begin
  if Self.Width - NewSize  < 120 then
    NewSize := Self.Width - 120;
  Accept := (Self.Width - NewSize >= 120);
//
//  if Accept and (NewSize >= Self.SpTBXPanel1.Width) then
//  begin
//    if Self.SpTBXPanel2.Width <= Self.SpTBXPanel2.Constraints.MinWidth then
//    begin
//      Diff := NewSize - Self.SpTBXPanel1.Width;
//      Self.SpTBXPanel1.Width := Self.SpTBXPanel1.Width - Diff;
//      NewSize := Self.SpTBXPanel1.Width;
//      Self.SpTBXPanel2.Width := Self.SpTBXPanel2.Constraints.MinWidth;
//      if Self.Left + Self.Width >= Self.Parent.Width then
//        Self.Width := Self.Width - Diff;
//      Accept := False;
//    end;
//  end;
end;

end.
