unit u_bfaTreeView;

interface

uses
    Windows, Messages, SysUtils, Variants, Classes, Graphics,
    Controls, Forms, Dialogs, ExtCtrls, ComCtrls, JvComCtrls, DB, Uni,
    bfaLog, bfaObjects36, bfaSP, Generics.Collections, bfaBaseObjects
    ;

type
  TOnNodeChecked = procedure of object;

  TbfaTreeView = class(TJVTreeView)
  private
    FViewKind: TbfaViewKind;
    FRight: Int64;
    FbfaLog: TbfaLog;
    FUser: TbfaUser;
    FConnection: TUniConnection;
    FLastSelected: Int64;
    FExpanded: TList<TbfaInt64>;
    FIniname, FIniSectionName: string;
    FCheckChilds: Boolean;
    FSkipExpand: Boolean;
    FOnNodeChecked: TOnNodeChecked;
    FRightsUserid: string;
    FRightsGrouplist: string;

    procedure SetViewKind(Value: TbfaViewKind);
    procedure TVExpanding(Sender: TObject; Node: TTreeNode; var AllowExpansion: Boolean);
    procedure TVCollapsing(Sender: TObject; Node: TTreeNode; var AllowExpansion: Boolean);
    function SelectTreenodeByLgrID(LgrID: Int64; UserID: string; Right : Int64;
      GroupList : WideString; HasChildren : Boolean) : TbfaSP;
    procedure ReadExpandedFromINI(SelectLastArchive: Boolean);
    function SearchNode(ArcID : Int64) : TTreeNode;
    procedure DeleteNodeData(Nd : TTreeNode);
    procedure NodeCheckedChange(Sender: TObject; Node: TJvTreeNode);
    procedure SelectionChanging(Sender: TObject; OldNode, NewNode: TJvTreeNode;
        Cause: TJvNodeSelectCause; var Allow: Boolean);
    procedure SelectionChange(Sender: TObject);
    procedure TreeViewKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    function NodeFullExpanded(NodeItemIndex: Integer): Boolean;
  public
    FAutoscroll : TTimer;
    fbscroll_down : Boolean;
    RootNode : TTreeNode;
    FSkipChangeEvents: Boolean;

    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;

    procedure Load(SelectLastArchive: Boolean = True);
    procedure ClearTreeview;
    procedure SaveToINI;
    function SelectTreeNode(ArcID : Int64) : TTreeNode;
    function GetTreeNodeByID(ArcID : Int64) : TTreeNode;
    procedure DoAutoscroll(Sender: TObject);
    procedure CheckOneNode(Node: TTreeNode);
    function CountChecked: Integer;

    function HasChildren(LgrID, Right : Int64) : Boolean;
    property ViewKind: TbfaViewKind read FViewKind write SetViewKind;
    property Cnt : TUniConnection read FConnection write FConnection;
    property bfaLog : TbfaLog read FbfaLog write FbfaLog;
    property User : TbfaUser read FUser write FUser;
    property LastSelected: Int64 read FLastSelected write FLastSelected;
    property Ininame: string read FIniname write FIniname;
    property IniSectionName: string read FIniSectionName write FIniSectionName;
    property CheckChilds: Boolean read FCheckChilds write FCheckChilds;
    property SkipExpand: Boolean read FSkipExpand write FSkipExpand;
    property OnNodeChecked: TOnNodeChecked read FOnNodeChecked write FOnNodeChecked;
    property Right: Int64 read FRight write FRight;
    property RightsUserid: string read FRightsUserid write FRightsUserid;
    property RightsGrouplist: string read FRightsGrouplist write FRightsGrouplist;
  end;

implementation

uses
  bfaCommonUtils, IniFiles, bfaCommonDB;


function TbfaTreeView.CountChecked: Integer;
var
  i : Integer;
begin
  Result := 0;
   for i := 0 to Self.Items.Count -1 do
     if Self.Checked[Self.Items.Item[i]] then
      inc(Result)
end;

constructor TbfaTreeView.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FAutoscroll := TTimer.Create(AOwner);
  FAutoscroll.OnTimer := DoAutoscroll;
  FAutoscroll.Interval := 150;
  FAutoscroll.Enabled := False;
  fbscroll_down := False;

  FViewKind := tvkViewer;
  FRight := (rightRead or rightImport);
  FConnection := nil;
  FbfaLog := nil;
  FExpanded := TList<TbfaInt64>.Create;
  FExpanded.Clear;
  FSkipExpand := False;

  Self.OnExpanding := TVExpanding;
  Self.OnCollapsing := TVCollapsing;

  Self.ChangeDelay := 0;
  Self.ForceClickSelect := False;
  Self.HideSelection := False;
  Self.MultiSelect := False;
  Self.ReadOnly := True;
  Self.Ininame := sSysTreeViewINI;
  Self.IniSectionName := sSysTreeView;
  FRightsUserid := '';
  FRightsGrouplist := '';
  Self.OnKeyDown := Self.TreeViewKeyDown;

  Self.OnNodeCheckedChange := NodeCheckedChange;

  // SJ 08.09.2020
  //  only for TreeViews with Checkboxes
  //  could also be assigned after creating them
  Self.OnSelectionChanging := SelectionChanging;
  Self.OnSelectionChange := SelectionChange;
  // SJ 08.09.2020
end;

destructor TbfaTreeView.Destroy;
var
  i : Integer;
begin
  for i := 0 to Self.Items.Count - 1 do
  begin
    if PbfaInt64(Self.Items[i].Data) <> nil then
       PbfaInt64(Self.Items[i].Data).Free;
  end;

  for i := 0 to FExpanded.Count - 1 do
  begin
    if FExpanded[i] <> nil then
       FExpanded[i].Free;
  end;
  FExpanded.Clear;

  FExpanded.Free;

  FAutoscroll.Enabled := False;
  FAutoscroll.Free;

  FConnection := nil;
  FbfaLog := nil;

  inherited Destroy;
end;

procedure TbfaTreeView.Load(SelectLastArchive: Boolean = True);
var
  dsc : TbfaSP;
  pID : PbfaInt64;
  tn : TTreeNode;
  i : Integer;
  lmq: TUniQuery;
begin
  if not Self.FSkipChangeEvents then
  begin
    Self.FSkipChangeEvents := True;
    Self.Items.BeginUpdate;
    try
      Self.Items.Clear;
    finally
      Self.FSkipChangeEvents := False;
      Self.Items.EndUpdate;
    end;
  end
  else
    Self.Items.Clear;

  if FbfaLog <> nil then
  begin
    FbfaLog.DebugBegin(llDebug, 'TbfaTreeView.Load', ClassName);
//14.03.2024    Self.Items.BeginUpdate; //19.02.2024
    try
      try
        if FConnection <> nil then
        begin
          lmq := DbCreateQuery;
          try
            if FViewKind = tvkAdministrator then
              lmq.SQL.Text := 'SELECT * FROM sys_arc where not lgrid in (select arcid from sys_arc) and (flags & 2 = 0) order by arcname desc'
            else
              lmq.SQL.Text := 'SELECT * FROM sys_arc where not lgrid in (select arcid from sys_arc) and type = 0 and (flags & 2 = 0) order by arcname desc';
            lmq.Execute;
            if lmq.RecordCount > 0 then
            begin
              lmq.First;

              while not lmq.Eof do
              begin
                New(pID);
                pID^ := TbfaInt64.Create;
                Self.Items.BeginUpdate;
                try
                  rootNode :=  Self.Items.AddFirst(nil, lmq.FieldByName(sColArcName).AsWideString);
                finally
                  Self.Items.EndUpdate;
                end;
                pID.Value := lmq.FieldByName(sColArcID).AsLargeInt;
                pID.ArcType := lmq.FieldByName('type').AsInteger; //25.11.2020
                rootNode.Data := pID;
                if lmq.FieldByName('type').AsInteger = 2 then
                begin
                  rootNode.ImageIndex := Integer(icoidx_admin_clock);
                  rootNode.SelectedIndex := Integer(icoidx_admin_clock_selected);
                end else begin
                  if FViewKind = tvkImporter then
                  begin
                    rootNode.ImageIndex := Integer(icoidx_warehouse_closed_importer);
                    rootNode.SelectedIndex := Integer(icoidx_warehouse_open_importer);
                  end
                  else
                  begin
                    rootNode.ImageIndex := Integer(icoidx_warehouse_closed);
                    rootNode.SelectedIndex := Integer(icoidx_warehouse_open);
                  end;
                end;

                if lmq.FieldByName(sColType).AsInteger = 0 then
                  rootNode.HasChildren := HasChildren(lmq.FieldByName(sColArcID).AsLargeInt, FRight);

                lmq.Next;
              end;
            end;
          finally
            lmq.Free;
          end;

          ReadExpandedFromINI(SelectLastArchive);

          if FExpanded.Count > 0 then
          begin
            for i:= 0 to FExpanded.Count - 1 do
            begin
              tn:= SearchNode(FExpanded[i].Value);
              if tn <> nil then
              begin
                FSkipExpand := True;
                Self.Items.BeginUpdate;
                try
                  tn.Expand(False);
                  Self.Update;
                finally
                  FSkipExpand := False;
                  Self.Items.EndUpdate;
                end;
              end;
            end;
          end else if (FExpanded.Count = 0) and (not FileExists(BD.AppdataBitfarm + '\' + Ininame)) then
          begin
            // erster Start des Clients? Wir machen die RootNode auf
            Self.Items.BeginUpdate;
            try
              FSkipExpand := True;
              Self.Items[0].Expand(False);
              Self.Update;
            finally
              FSkipExpand := False;
              Self.Items.EndUpdate;
            end;
          end;

          if SelectLastArchive then
          begin
            if FLastSelected > 0 then
            begin
              SelectTreeNode(FLastSelected);
              if (Self.FViewKind = tvkAdministrator) and (Self.Selected <> nil) and (not Self.Checked[Self.Selected]) then
                Self.Checked[Self.Selected] := True;
            end;
          end;

        end;
      except
        FbfaLog.Write(llNormal, ltException, 'Load', ClassName);
      end;
    finally
//14.03.2024      Self.Items.EndUpdate; //19.02.2024
      FbfaLog.DebugEnd(llDebug, 'TbfaTreeView.Load', ClassName);
    end;
  end;
end;

procedure TbfaTreeView.NodeCheckedChange(Sender: TObject; Node: TJvTreeNode);
var
  child: TTreeNode;
  check : Boolean;
  switchchilds: Boolean;
begin
  if FCheckChilds then
    Exit;

  switchchilds := Node.Selected and (Node.ImageIndex in [Integer(icoidx_warehouse_closed),
    Integer(icoidx_warehouse_open)]);

  check := Self.Checked[Node];

  FCheckChilds := True;
  try
    if Node.Selected then
      Self.Checked[Node] := True;

    if Node.HasChildren and (not FSkipExpand) then  // SJ 08.09.2020
    begin
      Node.Expand(False);

      child := Node.getFirstChild;
      while Assigned(child)  do
      begin
        if (child.Parent = Node) and (child.ImageIndex in [Integer(icoidx_folder_closed),
          Integer(icoidx_folder_open)]) then
        begin
          if switchchilds then
            Self.Checked[child] := not Self.Checked[child]
          else
            Self.Checked[child] := check or child.Selected;
        end;
        child := child.GetNext;
      end;
    end;

    if Assigned(FOnNodeChecked) then
      FOnNodeChecked;

  finally
    FCheckChilds := False;
    FSkipExpand := False;
  end;
end;

function TbfaTreeView.NodeFullExpanded(NodeItemIndex: Integer): Boolean;
var
  ATreeNode: TTreeNode;
begin
  try
    ATreeNode := nil;
    Result := False;

    if (Self.Items[NodeItemIndex].Parent <> nil) then
    begin
      ATreeNode := Self.Items[NodeItemIndex].Parent;
      Result := True;

      while (ATreeNode.Parent <> nil) do
      begin
        if (Result and ATreeNode.Expanded) then
          Result := True
        else
          Result := False;
        ATreeNode := ATreeNode.Parent;
      end;
    end;
  except
    FbfaLog.Write(llNormal, ltException, 'NodeFullExpanded', ClassName);
  end;
end;

procedure TbfaTreeView.CheckOneNode(Node: TTreeNode);
begin
  FCheckChilds := True;
  try
    if Node <> nil then //13.09.2017 absturz im AdminFrame beim speichern wenn man nur Benutzer sperren will
      Self.Checked[Node] := True;
  finally
    FCheckChilds := False;
  end;
end;

procedure TbfaTreeView.ClearTreeview;
var
  i: Integer;
begin
  for i:= 0 to Self.Items.Count - 1 do
  begin
    if PbfaInt64(Self.Items[i].Data) <> nil then
      PbfaInt64(Self.Items[i].Data).Free;
  end;
  Self.Items.Clear;
end;

procedure TbfaTreeView.DoAutoscroll(Sender: TObject);
begin
  if fbscroll_down then
    SendMessage(Self.Handle, WM_VSCROLL, SB_LINEUP, 0)
  else
    SendMessage(Self.Handle, WM_VSCROLL, SB_LINEDOWN, 0);
end;

function TbfaTreeView.HasChildren(LgrID, Right : Int64) : Boolean;
var
  subSP : TbfaSP;
  found : Boolean;
  userid : string;
  grouplist : string;
begin
  Found := False;
  if FRightsUserid <> '' then
    userid := FRightsUserid
  else
    userid := IntToStr(FUser.ID);

  if FRightsGrouplist <> '' then
    grouplist := FRightsGrouplist
  else
    grouplist := FUser.GroupList.CommaText;

  try
    if FConnection <> nil then
    begin
      subSP := SelectTreenodeByLgrID(LgrID, userid, Right, grouplist, True);
      if subSP.RecordCount > 0 then
      begin
        subSP.First;
        while ((not found) and (not subSP.Eof)) do
        begin
          if LgrID <> subSP.FieldByNameAsLargeInt(sColArcID) then //21.02.2023 bfacrm:ticket__show__34057
          begin
            if (subSP.FieldByNameAsInteger(sColType) = 0) then
              found := HasChildren(subSP.FieldByNameAsLargeInt(sColArcID), Right)
            else if subSP.FieldByNameAsInteger(sColType) = 1 then
              found := True;
          end;
          subSP.Next;
        end;
        subSP.Close;
      end;
    end;
    Result := found;
  except
    Result := False;
  end;
end;

procedure TbfaTreeView.TVExpanding(Sender: TObject; Node: TTreeNode; var AllowExpansion: Boolean);
var
  bSP : TbfaSP;
  pID : PbfaInt64;
  tn : TTreeNode;
  CanAdd : Boolean;
  userid : string;
  grouplist : string;
begin
  Screen.Cursor:= crHourGlass;
  try
    if (Node.Count = 0) then
    begin
      if FRightsUserid <> '' then
        userid := FRightsUserid
      else
        userid := IntToStr(FUser.ID);

      if FRightsGrouplist <> '' then
        grouplist := FRightsGrouplist
      else
        grouplist := FUser.GroupList.CommaText;

      try
        pID := PbfaInt64(Node.Data);
        bSP := SelectTreenodeByLgrID(pID.Value, userid, FRight, grouplist, False);
        if FConnection <> nil then
        begin
          if bSP.RecordCount > 0 then
          begin
            bSP.First;
            while not bSP.Eof do
            begin
              if bSP.FieldByNameAsInteger(sColType) = 0 then
                CanAdd := HasChildren(bSP.FieldByNameAsLargeInt(sColArcID), FRight)
              else
                CanAdd := True;

              if ((CanAdd) or (FRight = 0)) and
                  (Pos('deleted', bSP.FieldByNameAsString('flags')) = 0) then   // fge Archive hinzu, Lager, wenn sie Archive haben, oder alles,
              begin                               // wenn wir ohne rechte abfragen (Administrator) und es nicht deleted ist
                New(pID);
                pID^ := TbfaInt64.Create;
                tn :=  Self.Items.AddChild(Node, bSP.FieldByNameAsWideString(sColArcName));
                pID.Value := bSP.FieldByNameAsLargeInt(sColArcID);
                pID.ArcType := bSP.FieldByNameAsInteger(sColType);
                tn.Data := pID;
                if bSP.FieldByNameAsInteger(sColType) = 0 then
                begin
                  if FViewKind = tvkImporter then
                  begin
                    tn.ImageIndex := Integer(icoidx_warehouse_closed_importer);
                    tn.SelectedIndex := Integer(icoidx_warehouse_closed_importer);
                  end
                  else
                  begin
                    tn.ImageIndex := Integer(icoidx_warehouse_closed);
                    tn.SelectedIndex := Integer(icoidx_warehouse_closed);
                  end;
                  tn.HasChildren := True;
                end
                else
                begin
                  if FViewKind = tvkImporter then
                  begin
                    tn.ImageIndex := Integer(icoidx_folder_closed_importer);
                    tn.SelectedIndex := Integer(icoidx_folder_open_importer);
                  end
                  else
                  begin
                    tn.ImageIndex := Integer(icoidx_folder_closed);
                    tn.SelectedIndex := Integer(icoidx_folder_open);
                  end;
                end;
              end;

              bSP.Next
            end;
            bSP.close;
            if (Node.Count = 0) then   // Wenn zB Container sind ausgeschaltet...
              Node.HasChildren := False
            else if ((Node.Count > 0) and (not Node.Expanded) and AllowExpansion) then
              Node.Expand(False);
          end else
            Node.HasChildren := False;
        end else
          Node.HasChildren := False;
      except

      end;
    end;

    if Node.HasChildren then
    begin
      if FViewKind = tvkImporter then
      begin
        Node.ImageIndex := Integer(icoidx_warehouse_open_importer);
        Node.SelectedIndex := Integer(icoidx_warehouse_open_importer);
      end
      else
      begin
        Node.ImageIndex := Integer(icoidx_warehouse_open);
        Node.SelectedIndex := Integer(icoidx_warehouse_open);
      end;
    end;

    SaveToINI;
  finally
    Screen.Cursor:= crDefault;
  end;
end;

procedure TbfaTreeView.DeleteNodeData(Nd : TTreeNode);
var
  j : Integer;
begin
  for j := nd.Count - 1 downto 0 do
  begin
    if PbfaInt64(Nd[j].Data) <> nil then
      PbfaInt64(Nd[j].Data).Free;
  end;
end;

procedure TbfaTreeView.TreeViewKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
  FSelected, FNewSelected: Integer;
begin
  if (Key = VK_NEXT) or (Key = VK_PRIOR) then
  begin
    // erstmal gucken wo wir stehen
    // FSelectedIndex vom TreeNode ist leider private
    for FSelected := 0 to Self.Items.Count - 1 do
    begin
      if (Self.Items[FSelected].Selected) then
        break;
    end;

    if (Key = VK_NEXT) then
    begin
      for FNewSelected := FSelected + 1 to Self.Items.Count - 1 do
      begin
        if (Self.Items[FNewSelected].HasChildren) and (NodeFullExpanded(FNewSelected)) then
        begin
          Key := 0;
          Self.Items[FNewSelected].Selected := True;
          break;
        end;
      end;

    end
    else if (Key = VK_PRIOR) then
    begin
      for FNewSelected := FSelected - 1 downto 0 do
      begin
        if (Self.Items[FNewSelected].HasChildren) and (NodeFullExpanded(FNewSelected)) then
        begin
          Key := 0;
          Self.Items[FNewSelected].Selected := True;
          break;
        end;
      end;
    end;
  end;
end;

procedure TbfaTreeView.TVCollapsing(Sender: TObject; Node: TTreeNode; var AllowExpansion: Boolean);
begin
  try
    if Node.HasChildren then
    begin
      if FViewKind = tvkImporter then
      begin
        Node.ImageIndex := Integer(icoidx_warehouse_closed_importer);
        Node.SelectedIndex := Integer(icoidx_warehouse_closed_importer);
      end
      else
      begin
        Node.ImageIndex := Integer(icoidx_warehouse_closed);
        Node.SelectedIndex := Integer(icoidx_warehouse_closed);
      end;
    end;

    SaveToINI;
  except

  end;

end;

function TbfaTreeView.SelectTreenodeByLgrID(LgrID: Int64; UserID: string; Right : Int64;
  GroupList : WideString; HasChildren : Boolean) : TbfaSP;
var
  FbfaSP : TbfaSP;
begin
  FbfaSP := TbfaSP.Create(FConnection, sSpSelectTreenodeByLgrID);
  try
    FbfaSP.PrepareSQL;
       
    FbfaSP.SetParamByNameAsLargeInt(sPrmLgrID, LgrID);
    FbfaSP.SetParamByNameAsString(sPrmUserID, UserID);
    FbfaSP.SetParamByNameAsWideString(sPrmGroupList, GroupList);
    FbfaSP.SetParamByNameAsLargeInt(sPrmRight, Right);
    FbfaSP.SetParamByNameAsBoolean(sPrmHasChildren, HasChildren);
    FbfaSP.Open;
    Result:= FbfaSP;
  except
    FbfaLog.Write(llNormal, ltException, 'SelectTreenodeByLgrID - ErrorCode: ' +
      IntToStr(ecConnectionSelectTreenodeByLgrIDFailure), ClassName);
    Result := nil;
  end;
end;

procedure TbfaTreeView.SetViewKind(Value: TbfaViewKind);
begin
  FViewKind := Value;
  if FViewKind = tvkViewer then
    FRight := (rightRead or rightImport)
  else if FViewKind = tvkImporter then
    FRight := rightImport
  else if FViewKind = tvkAdministrator then
    FRight := 0;
end;

procedure TbfaTreeView.ReadExpandedFromINI(SelectLastArchive: Boolean);
var
  ini : TIniFile;
  ids, str, IniSection : string;
  i : Integer;
  Int64ID : TbfaInt64;
begin
  ini := TIniFile.Create(BD.AppdataBitfarm + '\' + Ininame);
  try
    FExpanded.Clear;
    if FileExists(ini.FileName) then
    begin
      //12.04.2022 Rückwärtskompatibilität 361 -->
      if ini.SectionExists(Self.IniSectionName) then
        iniSection := Self.IniSectionName
      else
        iniSection := sSysTreeView;
      //12.04.2022 Rückwärtskompatibilität 361 <--

      if FViewKind = tvkAdministrator then
      begin
        if SelectLastArchive then
          FLastSelected := StrToInt64(ini.ReadString(iniSection, sSysAdminLastID, '-1'));
        ids:= ini.ReadString(iniSection, sSysAdminExpanded, '');
      end
      else if FViewKind = tvkImporter then
      begin
        if SelectLastArchive then
          FLastSelected := StrToInt64(ini.ReadString(iniSection, sSysImportLastID, '-1'));
        ids := ini.ReadString(iniSection, sSysImportExpanded, '');
      end
      else if FViewKind = tvkViewer then
      begin
        if SelectLastArchive then
          FLastSelected := StrToInt64(ini.ReadString(iniSection, sSysViewLastID, '-1'));
        ids := ini.ReadString(iniSection, sSysViewExpanded, '');
      end;
      if ids <> '' then
      begin
        i := 1;
        str := '';
        while i <= Length(ids) do
        begin
          if ids[i] = ';' then
          begin
            Int64ID := TbfaInt64.Create;
            Int64ID.Value := StrToInt64(str);

            FExpanded.Add(Int64ID);
            str := '';
          end
          else
            str := str + ids[i];

          inc(i);
        end;
      end
      else
        FExpanded.Clear;
    end;
  finally
    Ini.Free;
  end;
end;

procedure TbfaTreeView.SaveToINI;
var
  ini : TIniFile;
  ids : string;
  i : Integer;
begin
  if FSkipExpand then
    Exit;

  ini := TIniFile.Create(BD.AppdataBitfarm + '\' + Ininame);
  try
    try
      ids := '';
      for i := 0 to Self.Items.Count - 1 do
      begin
        if Self.Items[i].Expanded then
          ids := ids + IntToStr(PbfaInt64(Self.Items[i].Data).Value) + ';';
      end;

      if FViewKind = tvkAdministrator then
      begin
        if FLastSelected > 0 then
          ini.WriteString(Self.IniSectionName, sSysAdminLastID, IntToStr(FLastSelected));
        ini.WriteString(Self.IniSectionName, sSysAdminExpanded, ids);
      end
      else if FViewKind = tvkImporter then
      begin
        if FLastSelected > 0 then
          ini.WriteString(Self.IniSectionName, sSysImportLastID, IntToStr(FLastSelected));
        ini.WriteString(Self.IniSectionName, sSysImportExpanded, ids);
      end
      else if FViewKind = tvkViewer then
      begin
        if FLastSelected > 0 then
          ini.WriteString(Self.IniSectionName, sSysViewLastID, IntToStr(FLastSelected));
        ini.WriteString(Self.IniSectionName, sSysViewExpanded, ids);
      end;
    except
      FbfaLog.Write(llNormal, ltException, 'SaveToINI: ' +
        BD.AppdataBitfarm + '\' + Ininame, ClassName);
    end;
  finally
    Ini.Free;
  end;
end;

function TbfaTreeView.SearchNode(ArcID : Int64) : TTreeNode;
var
  Node : TTreeNode;
begin
  try
    Result:= nil;
    if Self.Items.Count > 0 then
    begin
      Node:= Self.Items[0];
      while ((Node <> nil) and (Result = nil)) do
      begin
        if (Node.Data <> nil) then
        begin
          if PbfaInt64(Node.Data).Value = ArcID then
            Result:= Node;
        end;
        Node:= Node.GetNext;
      end;
    end;
  except
    Result:= nil;
  end;
end;

// SJ 08.09.2020
procedure TbfaTreeView.SelectionChange(Sender: TObject);
begin
// after the change we have to set the toogle to false, otherwise the last
// archive will not be saved as SaveToINI
  FSkipExpand := False;
end;

procedure TbfaTreeView.SelectionChanging(Sender: TObject; OldNode, NewNode: TJvTreeNode;
    Cause: TJvNodeSelectCause; var Allow: Boolean);
begin
// needs to be set for all TreeViews with Checkboxes, because otherwise
// NodeCheckedChange will Expand the Node when leaving it
  FSkipExpand := True;
end;
// SJ 08.09.2020

function TbfaTreeView.SelectTreeNode(ArcID : Int64) : TTreeNode;
var
  i: Integer;
  found: Boolean;
begin
  Result:= nil;
  try
    i:= 0;
    found:= false;
    while ((i < Self.Items.Count) and (not found)) do
    begin
      if PbfaInt64(Self.Items[i].Data).Value = ArcID then
      begin
        found:= true;
        Self.Items[i].Selected:= True;
        Self.Selected.Expand(False);
        Result:= Self.Items[i];
      end;
      inc(i);
    end;
  except
    Result:= nil;
  end;
end;

function TbfaTreeView.GetTreeNodeByID(ArcID : Int64) : TTreeNode;
var
  i: Integer;
  found: Boolean;
begin
  Result:= nil;
  try
    i:= 0;
    found:= false;
    while ((i < Self.Items.Count) and (not found)) do
    begin
      if PbfaInt64(Self.Items[i].Data).Value = ArcID then
      begin
        found:= true;
        Result:= Self.Items[i];
      end;
      inc(i);
    end;
  except
    Result:= nil;
  end;
end;

end.
