unit CommentaryFrame;

interface

uses
  gnugettext, Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, SpTBXItem, SpTBXControls, Uni, DB, MemDS, VirtualTable, Grids, DBClient,
  DBGrids, CRGrid, Vcl.StdCtrls,
  {$IFDEF COMMENTGRID}cxGrid, cxGridLevel, bfaGridDBTableView, cxLookAndFeels,
  dxPSCore, dxPScxGridLnk, cxGridMenuOperations, cxGridFooterPopupMenuItems, cxGridStdPopupMenu, //18.07.2024 SD <--bfacrm:ticket__show__12110
  dxCore, cxGridCustomTableView, cxGraphics, cxGridCustomView, cxGridTableView,
  cxClasses, cxGridCustomPopupMenu, cxGridPopupMenu, dxPSGlbl, dxPSUtl,
  dxPSEngn, dxPrnPg, dxBkgnd, dxWrap, dxPrnDev, dxPSCompsProvider,
  dxPSFillPatterns, dxPSEdgePatterns, dxPSPDFExportCore, dxPSPDFExport,
  cxDrawTextUtils, dxPSPrVwStd, dxPScxPageControlProducer,
  dxPScxEditorProducers, dxPScxExtEditorProducers, Vcl.Menus, TB2Item
  {$ELSE}u_bfaGrid{$ENDIF};


type
  TfraCommentary = class(TFrame)
    SpTBXPanel1: TSpTBXPanel;
    mmoComment: TMemo;
    btnAdd: TSpTBXButton;
    btnModify: TSpTBXButton;
    btnDelete: TSpTBXButton;
    SpTBXPanel2: TSpTBXPanel;

    //18.07.2024 SD -->bfacrm:ticket__show__12110
    cxGridPopupMenu1: TcxGridPopupMenu;
    popFilterBar: TSpTBXPopupMenu;
    procedure OnClickPrinter(Sender: TObject);
    procedure cxGridPopupMenu1Popup(ASenderMenu: TComponent;
      AHitTest: TcxCustomGridHitTest; X, Y: Integer; var AllowPopup: Boolean);
    //18.07.2024 SD <--bfacrm:ticket__show__12110

    procedure btnAddClick(Sender: TObject);
    procedure btnModifyClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);

  private
    { Private-Deklarationen }
    Editing: Boolean;
    activeDocId: Int64;
    procedure grdDiscussionMouseUp(Sender: TObject;
        Button: TMouseButton; Shift: TShiftState; X, Y: integer);
    procedure grdDiscussionDblClick(Sender: TObject);

    procedure ResetButtons(ClearText: Boolean = True);
    procedure SetButtons;
    procedure ModifyComment(memotxt : WideString);
    {$IFDEF COMMENTGRID}
    procedure tvwResultCustomDrawCell(Sender: TcxCustomGridTableView;
      ACanvas: TcxCanvas; AViewInfo: TcxGridTableDataCellViewInfo;
      var ADone: Boolean);
    procedure grdCommentarySelectionChanged(Sender: TcxCustomGridTableView);
    {$ENDIF}

  public
    { Public-Deklarationen }
    {$IFDEF COMMENTGRID}
    grdCommentary: TbfaGridDBTableView;
    grdCommentaryGrid: TcxGrid;
    grdCommentaryLevel: TcxGridLevel;

    {$ELSE}
    grdCommentary: TbfaDBGrid;
    {$ENDIF}
    ComDataSource : TDataSource;
    ComQuery : TUniQuery;

    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure LoadCommentary(gdocid: Int64);
    procedure RefreshCommentary;
    procedure Clear;
    procedure Retranslate;

  end;

implementation

{$R *.dfm}

uses
  bfaLog, bfaApplication, bfaPreview36, MainDataModule, bfaCommonUtils,
  bfaCommonStyles, bfaCommonMethods, bfaCommonDialogs, u_EditInfo, bfaIconsDM
  {$IFDEF COMMENTGRID}
  , cxControls
  {$ENDIF}
  ;

procedure TfraCommentary.btnAddClick(Sender: TObject);
var
  UserAction : PUserAction;
  memotxt: WideString;

begin
  memotxt := Trim(ParseSQlString(mmoComment.Text));
  if (memotxt <> '') and (TbfaPreview(Self.Owner).ActiveDoc <> nil) then
  begin
    UserAction := bitfarmApp.CreateUserAction;
    UserAction^.ActionName := 'actCreateComment';
    UserAction^.Params.Add(Inttostr(TbfaPreview(Self.Owner).ActiveDoc.ID));
    UserAction^.Params.Add(memotxt);
    bitfarmApp.MainThread.Add(UserAction);
//    LoadCommentary(TbfaPreview(Self.Owner).ActiveDoc.ID);
//    grdDiscussion.DataSource.DataSet.Refresh;
  end;

end;

procedure TfraCommentary.btnDeleteClick(Sender: TObject);
var
  UserAction : PUserAction;
  msg: string;
begin
  if grdCommentary{$IFDEF COMMENTGRID}.DataController{$ENDIF}.DataSource.DataSet.FieldByName('userid').AsInteger = bitfarmApp.User.ID then
  begin
    msg := Format(_('Soll der Kommentar wirklich gelscht werden?') ,[]);
    if BfMessageDlg(msg, mtConfirmation, [mbYes,mbNo], True) = mrYes then
    begin
      UserAction := bitfarmApp.CreateUserAction;
      UserAction^.ActionName := 'actDeleteComment';
      UserAction^.Params.Add(Inttostr(TbfaPreview(Self.Owner).ActiveDoc.ID));
      UserAction^.Params.Add(grdCommentary{$IFDEF COMMENTGRID}.DataController{$ENDIF}.DataSource.DataSet.FieldByName('commentid').AsString);
      bitfarmApp.MainThread.Add(UserAction);
    end;
  end;

end;

procedure TfraCommentary.btnModifyClick(Sender: TObject);
var
  memotxt : WideString;
begin
  if not Editing then
  begin
    memotxt := grdCommentary{$IFDEF COMMENTGRID}.DataController{$ENDIF}.DataSource.DataSet.FieldByName('comment').AsString;
    mmoComment.Text := StringReplace(memotxt, #10, #13#10, [rfReplaceAll]);
    Editing := True;
    btnModify.Caption := _('Speichern');
  end else
  begin
    memotxt := Trim(ParseSQlString(mmoComment.Text));
    ModifyComment(memotxt);
  end;
end;

procedure TfraCommentary.ModifyComment(memotxt : WideString);
var
  UserAction : PUserAction;
begin
  try
    if memotxt <> '' then
    begin
      UserAction := bitfarmApp.CreateUserAction;
      UserAction^.ActionName := 'actModifyComment';
      UserAction^.Params.Add(Inttostr(TbfaPreview(Self.Owner).ActiveDoc.ID));
      UserAction^.Params.Add(memotxt);
      UserAction^.Params.Add(grdCommentary{$IFDEF COMMENTGRID}.DataController{$ENDIF}.DataSource.DataSet.FieldByName('commentid').AsString);
      bitfarmApp.MainThread.Add(UserAction);
      Editing := False;
      btnModify.Caption := _('ndern');
    end;
  except

  end;
end;

procedure TfraCommentary.Clear;
begin
  try
    {$IFDEF COMMENTGRID}
    grdCommentary.Controller.ClearSelection;
    {$ELSE}
    grdCommentary.SelectedRows.Clear;
    {$ENDIF}
    grdCommentary{$IFDEF COMMENTGRID}.DataController{$ENDIF}.DataSource.DataSet := nil;

    ResetButtons;
  except
    bitfarmLog.Write(llNormal, ltException, 'Clear', ClassName);
  end;
end;


{$IFDEF COMMENTGRID}
procedure TfraCommentary.tvwResultCustomDrawCell(Sender: TcxCustomGridTableView;
      ACanvas: TcxCanvas; AViewInfo: TcxGridTableDataCellViewInfo;
      var ADone: Boolean);
begin
//20.09.2023 Muss hier sein, sonst TbfaDBGridTableView.DoCustomDrawCell wird nicht gettigt!
end;
{$ENDIF}

constructor TfraCommentary.Create(AOwner: TComponent);
var
  tbiPrinter: TSpTBXItem; //18.07.2024 SD <--bfacrm:ticket__show__12110
begin
  inherited Create(AOwner);
  {$IFDEF COMMENTGRID}
  grdCommentaryGrid := TcxGrid.Create(Self);
  grdCommentaryGrid.Parent := SpTBXPanel2;
  grdCommentaryGrid.Align := alClient;
  grdCommentaryGrid.Name := '';
  grdCommentaryGrid.LookAndFeel.ScrollbarMode := sbmClassic;
  grdCommentaryGrid.LookAndFeel.NativeStyle := True;
  grdCommentaryLevel := grdCommentaryGrid.Levels.Add;
  grdCommentaryLevel.Name := 'grdCommentary_LevelName';
  grdCommentary := grdCommentaryGrid.CreateView(TbfaGridDBTableView) as TbfaGridDBTableView;
  grdCommentary.Name := 'grdCommentary_ViewName';
  grdCommentary.Grd := grdCommentaryGrid;

  grdCommentaryLevel.GridView := grdCommentary;
//15.09.2023  grdCommentary.Width := 500;
  grdCommentary.Styles.Header := stlColumn;
  grdCommentary.Styles.Footer := stlColumn;
  grdCommentary.Styles.GroupByBox := stlGroupByBoxStyle;
  grdCommentary.Styles.Navigator := stlGroupByBoxStyle;
  grdCommentary.Styles.NavigatorInfoPanel := stlGroupByBoxStyle;
  grdCommentary.Styles.Background := stlBackgroundStyle;
  grdCommentary.Styles.Group := stlBackgroundStyle;
  grdCommentary.RowColorEvenOdd := bitfarmApp.ViewerOptions.GeneralOptions.RowColorEvenOdd;
  grdCommentary.ImageList := TImageList(bfaIcons.ilMainMenu);
  grdCommentary.SetGridDBTableViewDefaultSettings;
  grdCommentary.OnCustomDrawCell := tvwResultCustomDrawCell;
  grdCommentary.OptionsView.CellAutoHeight := True;
  grdCommentary.OptionsBehavior.RecordScrollMode := rsmByPixel; //11.10.2023 bfacrm:ticket__show__36597
  grdCommentary.OnSelectionChanged := grdCommentarySelectionChanged;
//18.09.2023  grdCommentary.ChangeDelay := 10;
  //18.07.2024 SD -->bfacrm:ticket__show__12110
  cxGridPopupMenu1.Grid := grdCommentaryGrid;
  tbiPrinter := TSpTBXItem.Create(Self.popFilterBar);
  tbiPrinter.Caption := _('Kommentare ausdrucken');
  tbiPrinter.FontSettings.Size := bfaFontSettingsSize;
  tbiPrinter.Enabled := True;
  tbiPrinter.Visible := True;
  tbiPrinter.OnClick := OnClickPrinter;
  Self.popFilterBar.Items.Add(tbiPrinter);
  //18.07.2024 SD <--bfacrm:ticket__show__12110

  {$ELSE}
  grdCommentary := TbfaDBGrid.Create(AOwner);
  grdCommentary.Parent := SpTBXPanel2;
  grdCommentary.Align := alClient;
  grdCommentary.Options := [dgTitles, dgColumnResize, dgColLines, dgRowLines, dgTitleClick, dgTabs, dgRowSelect]; // , {dgAlwaysShowSelection,} dgConfirmDelete, dgCancelOnExit];
  grdCommentary.OptionsEx := [dgeEnableSort, {dgeLocalFilter, dgeLocalSorting,} dgeStretch];
  grdCommentary.WordWrap := True;
  {$ENDIF}
  grdCommentary.Kind := gkNone;
  grdCommentary.OnMouseUp := grdDiscussionMouseUp;
  grdCommentary.OnDblClick := grdDiscussionDblClick;
  grdCommentary.RowColorEvenOdd := bitfarmApp.ViewerOptions.GeneralOptions.RowColorEvenOdd;

  {$IFDEF COMMENTGRID}
  grdCommentary.CreateDBColumn('tstamp', _('Datum'), 'tstamp', '', 130, True, soDescending);
  grdCommentary.CreateDBColumn('user', _('Benutzer'), 'user', '', 200);
  grdCommentary.CreateDBColumn('comment', _('Kommentar'), 'comment', '', 500);
  grdCommentary.OptionsView.ColumnAutoWidth := True;
  {$ELSE}
  grdCommentary.Columns.Add;
  grdCommentary.Columns[0].Title.Caption := _('Datum');
  grdCommentary.Columns[0].FieldName := 'tstamp';
  grdCommentary.Columns[0].Width := 25;

  grdCommentary.Columns.Add;
  grdCommentary.Columns[1].Title.Caption := _('Benutzer');
  grdCommentary.Columns[1].FieldName := 'user';
  grdCommentary.Columns[1].Width := 25;

  grdCommentary.Columns.Add;
  grdCommentary.Columns[2].Title.Caption := _('Kommentar');
  grdCommentary.Columns[2].FieldName := 'comment';
  grdCommentary.Columns[2].Width := 250;
  {$ENDIF}

  ComQuery := TUniQuery.Create(nil);
  ComDataSource := TDataSource.Create(nil);
  {$IFDEF COMMENTGRID}
  //15.09.2023
  {$ELSE}
  ComDataSource.OnStateChange := grdCommentary.bfaOnStateChange;
  {$ENDIF}
  ComDataSource.DataSet := ComQuery;
  grdCommentary{$IFDEF COMMENTGRID}.DataController{$ENDIF}.DataSource := ComDataSource;

  btnAdd.Caption := _('Hinzufgen');
  btnModify.Caption := _('ndern');
  btnDelete.Caption := _('Lschen');
end;

//18.07.2024 SD -->bfacrm:ticket__show__12110
procedure TfraCommentary.cxGridPopupMenu1Popup(ASenderMenu: TComponent;
  AHitTest: TcxCustomGridHitTest; X, Y: Integer; var AllowPopup: Boolean);

var
  s, selpos2 : TPoint;
  selrect : TRect;
begin
  try
    if (ASenderMenu is TcxGridStdHeaderMenu)
    then
    begin
      S.X := X;
      S.Y := Y;
      selpos2 := grdCommentaryGrid.ScreenToClient(s);

      popFilterBar.Popup(X, Y);
      AllowPopup := False;
    end;
  except

  end;
end;

procedure TfraCommentary.OnClickPrinter(Sender: TObject);
var
  dxComponentPrinter1 : TdxComponentPrinter;
  dxComponentPrinter1Link1 : TdxGridReportLink;
begin
  Screen.Cursor := crHourGlass;
  try
    dxComponentPrinter1 := TdxComponentPrinter.Create(Self);
    dxComponentPrinter1Link1 := TdxGridReportLink.Create(Self);
    with dxComponentPrinter1Link1 do
    begin
      ComponentPrinter := dxComponentPrinter1;
      Component := grdCommentaryGrid;
      Active := True;
    end;

    TranslateComponent(dxComponentPrinter1, 'Shared36');
    TranslateComponent(dxComponentPrinter1Link1, 'Shared36');

    dxComponentPrinter1.AddLink(dxComponentPrinter1Link1);
    dxComponentPrinter1.CurrentLinkIndex := 0;

    dxComponentPrinter1Link1.OptionsView.FilterBar := False;
    dxComponentPrinter1Link1.ReportTitleMode := tmOnFirstPage;
    if bitfarmApp.FSearchObj.SearchType in [1, 2] then
      dxComponentPrinter1Link1.ReportTitleText := bitfarmApp.FSearchObj.Name;

    dxComponentPrinter1Link1.ReportFootnotes.Text := FormatDateTime('dd.mm.yyyy', now()) +
      ' ' + _('Benutzer:') + ' ' + bitfarmApp.User.Name;
    dxComponentPrinter1Link1.ReportFootnotes.Mode := fnmOnEveryBottomPage;

    dxComponentPrinter1.Preview(True, dxComponentPrinter1Link1);
  finally
    Screen.Cursor := crDefault;
  end;
end;
//18.07.2024 SD <--bfacrm:ticket__show__12110

destructor TfraCommentary.Destroy;
begin
  if ComQuery.Active then
    ComQuery.Close;
  ComDataSource.Free;
  ComQuery.Free;
  inherited Destroy;
end;

procedure TfraCommentary.grdDiscussionMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: integer);
begin
  if (grdCommentary{$IFDEF COMMENTGRID}.DataController{$ENDIF}.DataSource = nil) or
     (grdCommentary{$IFDEF COMMENTGRID}.DataController{$ENDIF}.DataSource.DataSet = nil) then
    Exit;

  {$IFDEF COMMENTGRID}
  grdCommentary.Controller.ClearSelection;
  grdCommentary.Controller.FocusedRecord.Selected := True;
  {$ELSE}
  grdCommentary.SelectedRows.Clear;
  grdCommentary.SelectedRows.CurrentRowSelected := True;
  {$ENDIF}

  btnModify.Enabled :=  grdCommentary{$IFDEF COMMENTGRID}.DataController{$ENDIF}.DataSource.DataSet.FieldByName('userid').
    AsInteger = bitfarmApp.User.ID;
  btnDelete.Enabled :=  grdCommentary{$IFDEF COMMENTGRID}.DataController{$ENDIF}.DataSource.DataSet.FieldByName('userid').
    AsInteger = bitfarmApp.User.ID;
  mmoComment.Text := '';
  btnModify.Caption := _('ndern');
  Editing := False;
end;

procedure TfraCommentary.grdDiscussionDblClick(Sender: TObject);
var
  frm : Tfrm_EditInfo;
  memotxt : WideString;
  {$IFDEF COMMENTGRID}
  MouseDownInHeader: Boolean;
  AHitTest: TcxCustomGridHitTest;
  MouseDownPos : TPoint;
  {$ENDIF}
begin
  if (grdCommentary{$IFDEF COMMENTGRID}.DataController{$ENDIF}.DataSource = nil) or
     (grdCommentary{$IFDEF COMMENTGRID}.DataController{$ENDIF}.DataSource.DataSet = nil) then
    Exit;
  if grdCommentary{$IFDEF COMMENTGRID}.DataController{$ENDIF}.DataSource.DataSet.FieldByName('userid').AsInteger <> bitfarmApp.User.ID then
    Exit;

  {$IFDEF COMMENTGRID}
  MouseDownPos := grdCommentaryGrid.ScreenToClient(mouse.cursorpos);
  AHitTest := TcxGridSite(Sender).ViewInfo.GetHitTest(MouseDownPos.X, MouseDownPos.Y);
  MouseDownInHeader := (AHitTest.HitTestCode = htColumnHeader) or
    (AHitTest.HitTestCode = htColumnHeaderHorzSizingEdge) or
    (AHitTest.HitTestCode = htColumnHeaderFilterButton);

  if MouseDownInHeader then
    Exit;
  {$ENDIF}

  frm := Tfrm_EditInfo.Create(nil);
  try
    frm.FormTyp := 1;
    frm.Caption := _('Kommentar');
    {$IFDEF COMMENTGRID}
    frm.memo_Info.WantReturns := True;
    {$ENDIF}
    frm.memo_Info.Clear;
    frm.memo_Info.Text := grdCommentary{$IFDEF COMMENTGRID}.DataController{$ENDIF}.DataSource.DataSet.FieldByName('comment').AsString;
    frm.btn_speichern.ModalResult := mrOk;
    frm.Color := stlControl.Color;
    frm.Font.Color := stlControl.TextColor;
    frm.memo_Info.Color := stlControl.Color;
    frm.memo_Info.Font.Color := stlControl.TextColor;
    frm.btn_speichern.Color := stlControl.Color;
    frm.btn_speichern.Font.Color := stlControl.TextColor;
    frm.ShowModal;
    if frm.ModalResult = mrOK then
    begin
      memotxt := Trim(ParseSQlString(frm.memo_Info.Text));
      ModifyComment(memotxt);
    end;
  finally
    frm.Release;
  end;
end;

procedure TfraCommentary.LoadCommentary(gdocid: Int64);
var
  Cleartext: Boolean;
begin
  if ComDataSource.DataSet <> nil then
     ComDataSource.DataSet := nil;

  //21.05.2024 SD Kommentar wird beim Wechsel zu einer anderen Registerkarte geleert -->bfacrm:ticket__show__35710
  Cleartext := (activeDocId <> gdocid);
//13.06.2024 bfacrm:ticket__show__38209   Exit;
  activeDocId := gdocid;
  //21.05.2024 SD <--bfacrm:ticket__show__35710

  ComQuery.SQL.Text :=
                  'SELECT tstamp, If((u.displayname is Null or u.displayname = ' + #39#39 +
                  {$IFDEF COMMENTGRID}
                  '), u.username, u.displayname) as user, REPLACE(REPLACE(comment,' + #39 + '\\n' +
                  #39 + ', CHAR (13, 10)),' + #39 + '\\' +
                  #39 + ', ' + #39#39 + ') as comment, commentid, c.userid FROM sys_com as c '+
                  {$ELSE}
                  '), u.username, u.displayname) as user,comment, commentid, c.userid FROM sys_com as c '+
                  {$ENDIF}
                  'left join sys_user as u on c.userid = u.userid where gdocid = ' + IntToStr(gdocid) +
                  ' order by tstamp desc';

  ComQuery.Execute;
  if ComQuery.RecordCount > 0 then
  begin
    ComDataSource.DataSet := ComQuery;
    {$IFDEF COMMENTGRID}
    try
      grdCommentary.Controller.ClearSelection;
    finally
      grdCommentary.Invalidate;
    end;
    {$ELSE}
    grdCommentary.AdjustColumns;
    {$ENDIF}
  end;
  ResetButtons(ClearText);
end;

procedure TfraCommentary.RefreshCommentary;
begin
  try
    if grdCommentary{$IFDEF COMMENTGRID}.DataController{$ENDIF}.DataSource.DataSet <> nil then
    begin
      {$IFDEF COMMENTGRID}
      grdCommentary.DataController.BeginUpdate;
      try
        grdCommentary.DataController.DataSource.DataSet.Refresh;
        if grdCommentary.Controller.SelectedRecordCount > 0 then
        begin
          grdCommentary.Controller.ClearSelection;
          grdCommentary.Controller.FocusedRecord.Selected := True;
        end;
      finally
        grdCommentary.DataController.EndUpdate;
        grdCommentary.Invalidate;
      end;
      {$ELSE}
      grdCommentary.DataSource.DataSet.Refresh;
      if grdCommentary.SelectedRows.Count > 0 then
      begin
        grdCommentary.SelectedRows.Clear;
        grdCommentary.SelectedRows.CurrentRowSelected := True;
      end;
      {$ENDIF}
    end;

    SetButtons;
  except
    bitfarmLog.Write(llNormal, ltException, 'RefreshCommentary', ClassName);
  end;
end;

{$IFDEF COMMENTGRID}
procedure TfraCommentary.grdCommentarySelectionChanged(Sender: TcxCustomGridTableView);
begin
  if grdCommentary.DataController.RowCount = 0  then
    Exit;
end;
{$ENDIF}

procedure TfraCommentary.ResetButtons(ClearText: Boolean = True);
begin
  btnAdd.Enabled := True;
  btnModify.Enabled := False;
  btnDelete.Enabled := False;
  btnModify.Caption := _('ndern');
  Editing := False;
  if ClearText then
    mmoComment.Text := '';
end;

procedure TfraCommentary.SetButtons;
begin
  btnAdd.Enabled := True;
  btnModify.Enabled := False;
  btnDelete.Enabled := False;

  if (grdCommentary{$IFDEF COMMENTGRID}.DataController{$ENDIF}.DataSource <> nil) and
     (grdCommentary{$IFDEF COMMENTGRID}.DataController{$ENDIF}.DataSource.DataSet <> nil) then
  begin
    btnModify.Enabled :=  grdCommentary{$IFDEF COMMENTGRID}.DataController{$ENDIF}.DataSource.DataSet.FieldByName('userid').
      AsInteger = bitfarmApp.User.ID;
    btnDelete.Enabled :=  grdCommentary{$IFDEF COMMENTGRID}.DataController{$ENDIF}.DataSource.DataSet.FieldByName('userid').
      AsInteger = bitfarmApp.User.ID;
  end;

  btnModify.Caption := _('ndern');
  Editing := False;
  mmoComment.Text := '';
end;

procedure TfraCommentary.Retranslate;
begin
  ReTranslateComponent(Self, 'ViewerV36');
end;

end.
