unit CreateResubFrame;

interface

uses
  gnugettext, Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, cxGraphics, cxControls, cxLookAndFeels, cxLookAndFeelPainters, u_bfaAddfield,
  cxContainer, cxEdit, ComCtrls, dxCore, cxDateUtils, SpTBXControls, cxTextEdit,
  cxMaskEdit, cxDropDownEdit, cxCalendar, StdCtrls, SpTBXEditors, SpTBXItem,
  Mask, JvExMask, JvToolEdit, JvCombobox, bfaApplication, cxCheckBox,
  cxCheckComboBox;

type
  TfraCreateResub = class(TFrame)
    SpTBXPanel1: TSpTBXPanel;
    lblName: TSpTBXLabel;
    lblInfoCreate: TSpTBXLabel;
    lblInfoForward: TSpTBXLabel;
    Bezeichnung4: TSpTBXLabel;
    Bezeichnung5: TSpTBXLabel;
    edtAlarm1: TSpTBXSpinEdit;
    dtDate1: TcxDateEdit;
    Bezeichnung6: TSpTBXLabel;
    Bezeichnung7: TSpTBXLabel;
    chkReturnToSender: TSpTBXCheckBox;
    chkEmailreminder: TSpTBXCheckBox;
    chkEveryone: TSpTBXCheckBox;
    chkOnlyOne: TSpTBXCheckBox;
    btnCancel: TSpTBXButton;
    cbbUser: TbfaJvCheckedComboBox;
    cbbGroup: TbfaJvCheckedComboBox;
    cbbFavorites: TSpTBXComboBox;
    edtName: TSpTBXEdit;
    edtInfoCreate: TSpTBXEdit;
    edtInfoFoward: TSpTBXEdit;
    btnOK: TSpTBXButton;
    chkNotifyByMail: TSpTBXCheckBox;
    cbbFavIcon: TSpTBXComboBox;
    lblDateInDays: TSpTBXLabel;
    edtAdditionalDays: TSpTBXSpinEdit;

    constructor Create(AOwner: TComponent);
    procedure chkOnlyOneClick(Sender: TObject);
    procedure chkEveryoneClick(Sender: TObject);
    procedure edtNameChange(Sender: TObject);
    procedure edtInfoCreateChange(Sender: TObject);
    procedure edtInfoFowardChange(Sender: TObject);
    procedure dtDate1Editing(Sender: TObject; var CanEdit: Boolean);
    procedure chkReturnToSenderClick(Sender: TObject);
    procedure chkEmailreminderClick(Sender: TObject);
    procedure cbbUserChange(Sender: TObject);
    procedure cbbGroupChange(Sender: TObject);
    procedure cbbUserPopupShown(Sender: TObject);
    procedure cbbUserPopupHidden(Sender: TObject);
    procedure cbbGroupPopupHidden(Sender: TObject);
    procedure cbbGroupPopupShown(Sender: TObject);
    procedure cbbFavoritesChange(Sender: TObject);
    procedure edtAlarm1Change(Sender: TObject);
    procedure chkNotifyByMailClick(Sender: TObject);
    procedure cbbFavIconDrawItem(Sender: TObject; ACanvas: TCanvas;
      var ARect: TRect; Index: Integer; const State: TOwnerDrawState;
      const PaintStage: TSpTBXPaintStage; var PaintDefault: Boolean);
    procedure cbbFavIconChange(Sender: TObject);
    procedure dtDate1PropertiesCloseUp(Sender: TObject);
    procedure dtDate1PropertiesPopup(Sender: TObject);
    procedure cbbFavoritesCloseUp(Sender: TObject);
    procedure cbbFavoritesDropDown(Sender: TObject);
    procedure cbbFavIconDropDown(Sender: TObject);
    procedure cbbFavIconCloseUp(Sender: TObject);
    procedure edtAdditionalDaysChange(Sender: TObject);
    procedure edtNameKeyPress(Sender: TObject; var Key: Char);
    procedure edtInfoCreateKeyPress(Sender: TObject; var Key: Char);
    procedure edtInfoFowardKeyPress(Sender: TObject; var Key: Char);
    procedure cbbFavoritesKeyPress(Sender: TObject; var Key: Char);
  private
    { Private-Deklarationen }
    FModified: Boolean;
    procedure SetModified(const Value: Boolean);
  public
    { Public-Deklarationen }
    LoadResub, OpenPopup: Boolean;
    OnModified: procedure of object;
    SaveView: Boolean;
    InitialOptions: Integer;
    property Modified : Boolean read FModified write SetModified;
    procedure OnComboModified(Field: TBfaAddfieldComp);
    procedure Clear;
    function GetResub: PResub;
    procedure SetSmallSize;
    procedure SetSaveView(setview : Boolean = False);
  end;

implementation

{$R *.dfm}

uses
  DateUtils, MainForm, MainDataModule, bfaIconsDM, bfaCommonMethods,
  bfaCommonStyles;  //14.08.2024 SD <--bfacrm:ticket__show__39501
{ TfraCreateResub }


procedure TfraCreateResub.cbbUserPopupHidden(Sender: TObject);
begin
  frmMainForm.OnComboCloseUp(Sender);
  OpenPopup := False;
end;

procedure TfraCreateResub.cbbUserPopupShown(Sender: TObject);
begin
  frmMainForm.OnComboDropDown(Sender);
  OpenPopup := True;
end;

procedure TfraCreateResub.cbbFavIconChange(Sender: TObject);
begin
  if LoadResub then
    Exit;

  Modified := True;
end;

procedure TfraCreateResub.cbbFavIconCloseUp(Sender: TObject);
begin
  OpenPopup := False;
end;

procedure TfraCreateResub.cbbFavIconDrawItem(Sender: TObject; ACanvas: TCanvas;
  var ARect: TRect; Index: Integer; const State: TOwnerDrawState;
  const PaintStage: TSpTBXPaintStage; var PaintDefault: Boolean);
var
  ComboBox: TComboBox;
  Bitmap: TBitmap;
begin
  if PaintStage = pstPostPaint then
    Exit;

  if Index < 1 then   // erster Eintrag bleibt leer
    Exit;

  ComboBox := (Sender as TComboBox);
  Bitmap := TBitmap.Create;
  try
    //14.08.2024 SD -->bfacrm:ticket__show__39501
    if odSelected in State then
      Bitmap.Canvas.Brush.Color := stlSelected.Color
    else
      Bitmap.Canvas.Brush.Color := stlControl.Color;
    //14.08.2024 SD <--bfacrm:ticket__show__39501
    bfaIcons.ilPreview.GetBitmap(bitfarmapp.GetResubFavIcon(Index), Bitmap);
    with ComboBox.Canvas do
    begin
      FillRect(ARect);
      if Bitmap.Handle <> 0 then
        Draw(ARect.Left + 2, ARect.Top, Bitmap);
      ARect := Bounds(
        ARect.Left + ComboBox.ItemHeight + 2,
        ARect.Top,
        ARect.Right - ARect.Left,
        ARect.Bottom - ARect.Top
      );
      DrawText(
        handle,
        PChar(ComboBox.Items[Index]),
        length(ComboBox.Items[index]),
        ARect,
        DT_VCENTER + DT_SINGLELINE
      );
    end;
  finally
    Bitmap.Free;
  end;
end;

procedure TfraCreateResub.cbbFavIconDropDown(Sender: TObject);
begin
  OpenPopup := True;
end;

procedure TfraCreateResub.cbbFavoritesChange(Sender: TObject);
begin
  LoadResub := True;
  try
    bitfarmApp.LoadWVLFavorite(Self, cbbFavorites.ItemIndex);
  finally
    LoadResub := False;
  end;
end;

procedure TfraCreateResub.cbbFavoritesCloseUp(Sender: TObject);
begin
  OpenPopup := False;
end;

procedure TfraCreateResub.cbbFavoritesDropDown(Sender: TObject);
begin
  OpenPopup := True;
end;

procedure TfraCreateResub.cbbFavoritesKeyPress(Sender: TObject; var Key: Char);
begin
  DisableBeep(Key); //11.03.2024 bfacrm:ticket__show__38134
end;

procedure TfraCreateResub.cbbGroupChange(Sender: TObject);
var
  OldState: Boolean;
begin
  if LoadResub then
    Exit;

  Modified := True;

  OldState := chkOnlyOne.Enabled;

  chkEveryone.Enabled := (cbbGroup.Text <> '') or (cbbUser.CheckedCount > 1);  //mehr als ein Einzelbenutzer -> Gruppe
  chkOnlyOne.Enabled := (cbbGroup.Text <> '') or (cbbUser.CheckedCount > 1);  //mehr als ein Einzelbenutzer -> Gruppe

  if not OldState then  // war disabled (wird also das erste Mal eingeschaltet), dann option efa setzen
    chkOnlyOne.Checked := True;

//  if cbbGroup.Text <> '' then
//  begin
//    chkEveryone.Enabled := True;
//    chkOnlyOne.Enabled := True;
//  end
//  else
//  begin
//    chkEveryone.Enabled := False;
//    chkOnlyOne.Enabled := False;
//  end;
end;

procedure TfraCreateResub.cbbGroupPopupHidden(Sender: TObject);
begin
  frmMainForm.OnComboCloseUp(Sender);
  OpenPopup := False;
end;

procedure TfraCreateResub.cbbGroupPopupShown(Sender: TObject);
begin
  frmMainForm.OnComboDropDown(Sender);
  OpenPopup := True;
end;

procedure TfraCreateResub.cbbUserChange(Sender: TObject);
var
  OldState: Boolean;
begin
  if LoadResub then
    Exit;

  Modified := True;

  OldState := chkOnlyOne.Enabled;

  chkEveryone.Enabled := (cbbGroup.Text <> '') or (cbbUser.CheckedCount > 1);  //mehr als ein Einzelbenutzer -> Gruppe
  chkOnlyOne.Enabled := (cbbGroup.Text <> '') or (cbbUser.CheckedCount > 1);  //mehr als ein Einzelbenutzer -> Gruppe

  if not OldState and (cbbUser.CheckedCount > 1) then  // war disabled (wird also das erste Mal eingeschaltet), dann option efa setzen
    chkEveryone.Checked := True;    // jeder einzeln ist standard, wenn man eine spontane gruppe aus mehreren user hat

end;

procedure TfraCreateResub.chkEmailreminderClick(Sender: TObject);
begin
  if LoadResub then
    Exit;

  Modified := True;
end;

procedure TfraCreateResub.chkEveryoneClick(Sender: TObject);
begin
  if LoadResub then
    Exit;

  chkOnlyOne.Checked := not chkEveryone.Checked;
  Modified := True;
end;

procedure TfraCreateResub.chkNotifyByMailClick(Sender: TObject);
begin
  if LoadResub then
    Exit;

  Modified := True;
end;

procedure TfraCreateResub.chkOnlyOneClick(Sender: TObject);
begin
  if LoadResub then
    Exit;
  chkEveryone.Checked := not chkOnlyOne.Checked;
  Modified := True;
end;

procedure TfraCreateResub.chkReturnToSenderClick(Sender: TObject);
begin
  if LoadResub then
    Exit;

  Modified := True;
end;

procedure TfraCreateResub.Clear;
begin
  cbbFavorites.Text := '';
  edtAlarm1.Text := '7';
  edtAlarm1.Value := 7; // bfacrm:ticket__show__30688
  dtDate1.Date := Date();
  chkReturnToSender.Checked := False;
  chkReturnToSender.Enabled := True;
  chkEmailreminder.Checked := False;
  chkEveryone.Checked := False;
  chkOnlyOne.Checked := False;
  edtName.Text := '';
  edtInfoCreate.Text := '';
  edtInfoFoward.Text := '';
  cbbUser.SetUnCheckedAll;
  cbbGroup.SetUnCheckedAll;
  chkEveryone.Enabled := False;
  chkOnlyOne.Enabled := False;
  cbbFavIcon.ItemIndex := 0;
  InitialOptions := 0;
  edtAdditionalDays.Value := 0;
end;

constructor TfraCreateResub.Create(AOwner: TComponent);
var
  i : Integer;
begin
  inherited Create(AOwner);
//  cbbUser := TBfaAddfieldComp.Create(Self, zfld03_MehrfachAuswahl, '');
//  with cbbUser do
//  begin
//    Parent := Self;
//    Left := 68;
//    Top := 125;
//    Width := 117;
//    Height := 22;
//    Label1.Visible := False;
//    compCheckedCombo.Left := compCheckedCombo.Left - 10;
//    compCheckedCombo.Top := compCheckedCombo.Top - 15;
//    compCheckedCombo.Width := 117;
//    tlbNeg.Visible := False;
//    TSpTBXListBox(comp2).SetParentComponent(AOwner);
//    OnModified := OnComboModified;
//    DontSetColors := True;
//  end;
//  cbbGroup := TBfaAddfieldComp.Create(Self, zfld03_MehrfachAuswahl, '');
//  with cbbGroup do
//  begin
//    Parent := Self;
//    Left := 266;
//    Top := 125;
//    Width := 117;
//    Height := 22;
//    Label1.Visible := False;
//    compCheckedCombo.Left := compCheckedCombo.Left - 10;
//    compCheckedCombo.Top := compCheckedCombo.Top - 15;
//    compCheckedCombo.Width := 117;
//    tlbNeg.Visible := False;
//    TSpTBXListBox(comp2).SetParentComponent(AOwner);
//    OnModified := OnComboModified;
//    DontSetColors := True;
//  end;
  SaveView := False;
  OpenPopup := False;
  InitialOptions := 0;
  for i := 0 to 10 do
    cbbFavIcon.Items.Add('');

  // SJ 06.10.2021 - ikRegExpr deaktiviert SmartInput (Today, Yesterday, Sunday, ...)
  dtDate1.Properties.InputKind := ikRegExpr;
  dtDate1.Properties.EditFormat := FormatSettings.ShortDateFormat;

  dtDate1.Properties.SaveTime := True;
  dtDate1.Properties.ShowTime := False;

  // nur bei Favoriten-Verwaltung sichtbar
  lblDateInDays.Visible := False;
  edtAdditionalDays.Visible := False;
  edtAdditionalDays.Enabled := False;
end;

procedure TfraCreateResub.dtDate1Editing(Sender: TObject; var CanEdit: Boolean);
begin
  if LoadResub then
    Exit;

  Modified := True;
end;

procedure TfraCreateResub.dtDate1PropertiesCloseUp(Sender: TObject);
begin
  OpenPopup := False;
end;

procedure TfraCreateResub.dtDate1PropertiesPopup(Sender: TObject);
begin
  OpenPopup := True;
end;

procedure TfraCreateResub.edtAlarm1Change(Sender: TObject);
begin
  try
    if (frmMainform = nil) or (not frmMainform.fAppStarted) then
      Exit;
    if LoadResub then
      Exit;
    Modified := True;
  except

  end;
end;

procedure TfraCreateResub.edtAdditionalDaysChange(Sender: TObject);
begin
  try
    if (LoadResub) or (not frmMainForm.fAppStarted) then
      Exit;

    Modified := True;
  except

  end;
end;

procedure TfraCreateResub.edtInfoCreateChange(Sender: TObject);
begin
  if LoadResub then
    Exit;

  Modified := True;
end;

procedure TfraCreateResub.edtInfoCreateKeyPress(Sender: TObject; var Key: Char);
begin
  DisableBeep(Key); //11.03.2024 bfacrm:ticket__show__38134
end;

procedure TfraCreateResub.edtInfoFowardChange(Sender: TObject);
begin
  if LoadResub then
    Exit;

  Modified := True;
end;

procedure TfraCreateResub.edtInfoFowardKeyPress(Sender: TObject; var Key: Char);
begin
  DisableBeep(Key); //11.03.2024 bfacrm:ticket__show__38134
end;

procedure TfraCreateResub.edtNameChange(Sender: TObject);
begin
  if LoadResub then
    Exit;

  Modified := True;
end;

procedure TfraCreateResub.edtNameKeyPress(Sender: TObject; var Key: Char);
begin
  DisableBeep(Key); //11.03.2024 bfacrm:ticket__show__38134
end;

function TfraCreateResub.GetResub: PResub;
var
  resub: PResub;
  lst: TStringList;
  i, j, k, options: Integer;
begin
  New(resub);
  FillChar(resub^, SizeOf(resub^), 0);

  if SaveView then
  begin
    resub^.Info := edtInfoCreate.Text;
    resub^.Comment := edtInfoFoward.Text;
  end else
  begin
    resub^.Name := edtName.Text;
    resub^.Info := edtInfoCreate.Text;
    resub^.FwdInfo := edtInfoFoward.Text;
  end;

  lst := TStringList.Create;
  lst.Delimiter := ';';
  lst.StrictDelimiter := True;
  try
    resub^.Alarm := Round(edtAlarm1.Value);

    if dtDate1.Text = '' then
      dtDate1.Date := Now();

    resub^.Date := dtDate1.Date;
    resub^.AlarmDate := DateUtils.IncDay(dtDate1.Date, resub^.Alarm);

    for j := 0 to cbbUser.Items.Count -1 do
      if cbbUser.IsChecked(j) then
        lst.Add(IntToStr(integer(cbbUser.Items.Objects[j])));

    resub^.Users := lst.DelimitedText;
    lst.Clear;
    for j := 0 to cbbGroup.Items.Count -1 do
      if cbbGroup.IsChecked(j) then
        lst.Add(IntToStr(integer(cbbGroup.Items.Objects[j])));

    resub^.Groups := lst.DelimitedText;
    options := 0;
    if chkEveryone.Checked then
      options := (options or 1);
    if chkReturnToSender.Checked then
      options := (options or 2);
    if chkEmailreminder.Checked then
      options := (options or 4);
    if chkNotifyByMail.Checked then
      options := (options or 64);
    resub^.Options := options;
    resub^.FavIcon := cbbFavIcon.ItemIndex;
  finally
    FreeAndNil(lst)
  end;

  Result := resub;
end;

procedure TfraCreateResub.OnComboModified(Field: TBfaAddfieldComp);
begin
  if LoadResub then
    Exit;

  Modified := True;
end;

procedure TfraCreateResub.SetModified(const Value: Boolean);
begin
  FModified := Value;
  btnOK.Enabled := FModified;

  if Assigned(OnModified) then
    OnModified;
end;

procedure TfraCreateResub.SetSaveView(setview: Boolean);
begin
  SaveView := setview;

  if SaveView then
  begin
    lblName.Visible := True;
    edtName.Visible := False;
    cbbFavorites.Visible := True;
    lblName.Caption := _('Favorit');
    lblInfoCreate.Caption := _('Infotext');
    lblInfoForward.Caption := _('Kommentar');
  end else
  begin
    lblName.Visible := True;
    edtName.Visible := True;
    cbbFavorites.Visible := False;
    lblName.Caption := _('Bezeichnung');
    lblInfoCreate.Caption := _('Infotext bei Anlage');
    lblInfoForward.Caption := _('Infotext bei Weiterleitung');
  end;
end;

procedure TfraCreateResub.SetSmallSize;
begin
  Self.Height := Self.Height - 42;
  lblName.Visible := False;
  edtName.Visible := False;
  lblInfoForward.Visible := False;
  edtInfoFoward.Visible := False;
  lblInfoCreate.Top := lblInfoCreate.Top + 48;
  lblInfoCreate.Caption := 'Info';
  edtInfoCreate.Top := edtInfoCreate.Top + 48;
  edtInfoCreate.Left := dtDate1.Left;
  edtInfoCreate.Width := 322;
end;

end.
