unit DocLinksFrame;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms, 
  Dialogs, ComCtrls, SpTBXControls, StdCtrls, SpTBXEditors, SpTBXItem, ActnList,
  Buttons, u_bfaScrollBox;

type
  TfraDocLinksList = class(TFrame)
    scrbxDocLinkList: TbfaScrollBox;
    SpTBXPanel1: TSpTBXPanel;
    edtDocLink: TSpTBXEdit;
    SpTBXLabel1: TSpTBXLabel;
    btnAddAutoReference: TSpTBXButton;
    btnNewLink: TSpTBXButton;
    btnAddLink: TSpTBXButton;
    btnRemoveLink: TSpTBXButton;
    btnDeleteLink: TSpTBXButton;
    lvwDocLink: TListView;
    procedure lvwDocLinkDblClick(Sender: TObject);
    procedure edtDocLinkChange(Sender: TObject);
    procedure lvwDocLinkChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure btnNewLinkClick(Sender: TObject);
    procedure btnRemoveLinkClick(Sender: TObject);
    procedure btnDeleteLinkClick(Sender: TObject);
    procedure btnAddAutoReferenceClick(Sender: TObject);
    procedure btnAddLinkClick(Sender: TObject);
//    procedure edtDocLinkKeyUp(Sender: TObject; var Key: Word;
//      Shift: TShiftState);
    procedure edtDocLinkKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure lvwDocLinkKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure edtDocLinkKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
    FLastLink : WideString;

    procedure SetLastLink(Value : WideString);
  public
    { Public declarations }
    SettingLastLink : Boolean;

    //18.10.2024 -->
    constructor Create(AOwner: TComponent); override;
    procedure Retranslate;
    //18.10.2024 <--

    property LastLink : WideString read FLastLink write SetLastLink;

  end;

implementation

{$R *.dfm}

uses
  bfaIconsDM, MainForm, bfaLog,
  bfaCommonUtils, bfaCommonMethods, bfaCommonDialogs, gnugettext, newDocLink,
  Uni, bfaApplication, MainDataModule;

procedure TfraDocLinksList.btnAddAutoReferenceClick(Sender: TObject);
begin
  bitfarmApp.CreateNewLinkAction(False, True, False{, edtDocLink.Text // bfacrm:ticket__show__35602});
end;

procedure TfraDocLinksList.btnAddLinkClick(Sender: TObject);
begin
  frmMainForm.actAddLink.Execute;
end;

procedure TfraDocLinksList.btnDeleteLinkClick(Sender: TObject);
var
  msg: string;
begin
  if (lvwDocLink.Selected <> nil) then
  begin
    msg := Format(_('Soll die Verknpfung "%s" gelscht werden?') + #10#13 + _('Auch bei anderen Dokumenten wird diese Verknpfung dann entfernt!'),[lvwDocLink.Selected.Caption]);
    if BfMessageDlg(msg, mtWarning, [mbYes,mbNo], True) = mrYes then
      frmMainForm.actDeleteLink.Execute;
  end;
end;

procedure TfraDocLinksList.btnNewLinkClick(Sender: TObject);
var
  lb : Boolean;
  RefName : WideString;
begin
  if bitfarmApp.CurrentContainer.Count = 0 then //12.03.2019
    Exit;

  RefName := '';
  if edtDocLink.Text = '' then
    lb := GetNewDocLinkName(Self, bitfarmApp.ViewerOptions.SearchOptions.ShowDocLinks, RefName)
  else
  begin
    RefName := edtDocLink.Text;
    lb := true;
  end;

  RefName := ParseAddfields(refName);
  edtDocLink.Text := RefName;

  if RefName = '' then
    Exit;

  if lb then
  begin
    try
      edtDocLink.Enabled := False;

      bitfarmApp.CreateNewLinkAction(False, False, False, RefName);
      edtDocLink.Text := RefName;

    finally
      edtDocLink.Enabled := True;
      edtDocLink.SetFocus;
      bitfarmApp.SetDocLinkActions;
    end;
  end;
end;

procedure TfraDocLinksList.btnRemoveLinkClick(Sender: TObject);
var
  msg: string;
begin
  if lvwDocLink.Selected <> nil then
  begin
//    msg := Format(_('Soll das Dokument aus der Verknpfung %s entfernt werden?'),[lvwDocLink.Selected.Caption]);
//    if BfMessageDlg(msg, mtConfirmation, [mbYes,mbNo], True) = mrYes then
//      frmMainForm.actRemoveLink.Execute;
    if frmMainForm.actRemoveLink.Enabled then // SJ 20.08.2019 - Abfrage nur wenn Berechtigungen, sonst Meldung
    begin
      msg := Format(_('Soll das Dokument aus der Verknpfung %s entfernt werden?'),[lvwDocLink.Selected.Caption]);
      if BfMessageDlg(msg, mtConfirmation, [mbYes,mbNo], True) = mrYes then
        frmMainForm.actRemoveLink.Execute;
    end else
      frmMainForm.SetNotificationBar(_('Keine Berechtigungen zum Lschen der Verknpfung'));
  end;
end;


constructor TfraDocLinksList.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  try
    TranslateComponent(Self, 'ViewerV36');
  except

  end;
end;

procedure TfraDocLinksList.Retranslate;
begin
  ReTranslateComponent(Self, 'ViewerV36');
end;

procedure TfraDocLinksList.edtDocLinkChange(Sender: TObject);
begin
  if not SettingLastLink then
    bitfarmApp.SetDocLinkActions;
end;

//procedure TfraDocLinksList.edtDocLinkKeyUp(Sender: TObject; var Key: Word;
//  Shift: TShiftState);
//begin
//  if (Key = VK_RETURN) then
//    btnNewLinkClick(Sender);
//end;


procedure TfraDocLinksList.edtDocLinkKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
    (* SJ 15.08.2019 bfacrm:ticket__show__20238

    Problem: BfMessageDlg hat VK_RETURN ausgelst. Deswegen kam btnNewLinkClick doppelt
    Lsung: von KeyUp nach KeyDown *)
  if (Key = VK_RETURN) and btnNewLink.Enabled then
  begin
    btnNewLinkClick(Sender);
  end
end;

procedure TfraDocLinksList.edtDocLinkKeyPress(Sender: TObject; var Key: Char);
var
  not_allowed: string;
  FBalloonHint: TBalloonHint;
begin
  not_allowed := ';' + sNotAllowedChar;
  if Pos(Key, not_allowed) <> 0 then
  begin
    try
      if FBalloonHint = nil then
        FBalloonHint := TBalloonHint.Create(Self);
      FBalloonHint.Delay := 0;
      FBalloonHint.HideAfter := 800;
      FBalloonHint.Description := _(sMsgNotAllowedCharachters) + not_allowed;
      FBalloonHint.ShowHint(TSpTBXEdit(Sender));
      Key := #0;
      Beep;
      Abort;
    except
    //
    end;
  end;
end;

procedure TfraDocLinksList.lvwDocLinkChange(Sender: TObject; Item: TListItem;
  Change: TItemChange);
begin
  //OutputDebugString('lvwDocLinkChange');

  btnRemoveLink.Enabled := (lvwDocLink.Selected <> nil) and frmMainForm.actRemoveLink.Enabled;
  btnDeleteLink.Enabled := (lvwDocLink.Selected <> nil) and frmMainForm.actDeleteLink.Enabled;
end;

procedure TfraDocLinksList.lvwDocLinkDblClick(Sender: TObject);
begin
  bitfarmLog.DebugBegin(llDebug, 'lvwDocLinkDblClick', ClassName);
  try
    try
      if (lvwDocLink.Selected <> nil) then
        bitfarmApp.CreateRefSearchAction(lvwDocLink.Selected.Caption);
    except
      bitfarmLog.Write(llNormal, ltException, 'lvwDocLinkDblClick', ClassName);
    end;
  finally
    bitfarmLog.DebugEnd(llDebug, 'lvwDocLinkDblClick', ClassName);
  end;
end;

procedure TfraDocLinksList.lvwDocLinkKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_DELETE then
    btnRemoveLinkClick(Sender);
end;

procedure TfraDocLinksList.SetLastLink(Value : WideString);
begin
  SettingLastLink := True;
  try
    FLastLink := Value;
    edtDocLink.Text := FLastLink;
  finally
    SettingLastLink := False;
  end;
end;

end.
