unit EditHolidayForm;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, cxGraphics, cxLookAndFeels,
  cxLookAndFeelPainters, Vcl.Menus, Vcl.StdCtrls, cxButtons,
  cxGrid, bfaGridDBTableView, cxGridLevel, Vcl.ComCtrls;

type
  TbfaHoliday = class(TObject)
  public
    holid, userid_src, userid_dst: Integer;
    start_date, end_date: TDateTime;
    username_src, username_dst, displayname_src, displayame_dst: string;
    constructor Create(holid, userid_src, userid_dst: Integer;
      start_date, end_date: TDateTime;
      username_src, username_dst, displayname_src, displayame_dst: string);
  end;

  PbfaHoliday = ^TbfaHoliday;

  TfrmEditHoliday = class(TForm)
    ListView1: TListView;
    btnEdit: TcxButton;
    btnDelete: TcxButton;
    btnClose: TcxButton;
    btnAdd: TcxButton;
    procedure btnCloseClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure ListView1SelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure btnAddClick(Sender: TObject);
    procedure btnEditClick(Sender: TObject);
    procedure ListView1ColumnClick(Sender: TObject; Column: TListColumn);
    procedure ListView1Compare(Sender: TObject; Item1, Item2: TListItem;
      Data: Integer; var Compare: Integer);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private-Deklarationen }
    Descending: Boolean;
    SortedColumn: Integer;
    procedure FillHolidayForm;
  public
    { Public-Deklarationen }
  end;

var
  frmEditHoliday: TfrmEditHoliday;

implementation

uses
  Uni, MySQLUniProvider, MainDataModule, bfaLog, bfaBaseObjects, bfaApplication,
  bfaCommonDialogs, gnugettext, HolidayForm, bfaCommonDB;
{$R *.dfm}

procedure TfrmEditHoliday.btnDeleteClick(Sender: TObject);
var
  req: TbfaRequest;
  holid: integer;
begin
  try
    if (Self.ListView1.Selected <> nil) and (Self.ListView1.Selected.Data <> nil) then
      holid := TbfaHoliday(Self.ListView1.Selected.Data).holid
    else
      Exit;

    if holid = 0 then
      Exit;

    if (BfMessageDlg(_('Vertretung wirklich lschen?'), mtConfirmation, [mbYes,mbNo]) = mrYes) then
    begin
      try
        req := TbfaRequest.Create;
        req.ReqType := 'reqDeleteHoliday';
        req.UserID := bitfarmApp.User.ID;
        req.Profile := bitfarmApp.Profile;
        req.Parameter := IntToStr(holid) + ';' + TbfaHoliday(Self.ListView1.Selected.Data).userid_src.ToString;

        req.Lang := bitfarmApp.Lang;
        bitfarmApp.MainSocket.SendRequestToServer(req);

        req := bitfarmApp.MainSocket.ReceiveResponse;
        if req = nil then
          Exit;

        if (req.ReqType = 'respError') or (req.ReqType = 'respAuthError') then
        begin
          BfShowMessage(_('Server meldet Fehler: ') + req.DataString, nil);
          bitfarmLog.Write(llNormal, ltException, _('Server meldet Fehler: ') + req.DataString, ClassName);
        end else
          FillHolidayForm;
      finally
        FreeAndNil(req);
      end;
    end;
  except
    bitfarmLog.Write(llNormal, ltException, 'btnDeleteClick', ClassName);
  end;
end;

procedure TfrmEditHoliday.btnEditClick(Sender: TObject);
var
  req: TbfaRequest;
  Useraction: PUserAction;
  holiday: TbfaHoliday;
  frmHoliday: TfrmHoliday;
  i, idx: Integer;
  datestart, dateend: string;
begin
  try
    if (Self.ListView1.Selected = nil) or (Self.ListView1.Selected.Data = nil) then
      Exit;

    holiday := TbfaHoliday(Self.ListView1.Selected.Data);

    frmHoliday := TfrmHoliday.Create(Self);
    frmHoliday.cbxReplacement.ItemIndex := frmHoliday.cbxReplacement.Items.IndexOfObject(TObject(holiday.userid_dst));

    frmHoliday.cxDateEdit1.Date := holiday.start_date;
    frmHoliday.cxDateEdit2.Date := holiday.end_date;
    frmHoliday.ShowModal;

    if frmHoliday.ModalResult = mrOk then
    begin
      datestart := FormatDateTime('yyyy-mm-dd', frmHoliday.cxDateEdit1.Date);
      dateend := FormatDateTime('yyyy-mm-dd', frmHoliday.cxDateEdit2.Date);
      try
        req := TbfaRequest.Create;
        req.ReqType := 'reqModifyHoliday';
        req.UserID := bitfarmApp.User.ID;
        req.Profile := bitfarmApp.Profile;
        req.Parameter := IntToStr(holiday.holid)+';'+IntToStr(holiday.userid_src)+';'+
          IntToStr(Integer(frmHoliday.cbxReplacement.Items.Objects[frmHoliday.cbxReplacement.ItemIndex]))+';'+
          datestart+';'+dateend;
  //      slModifyHoliday.Add(holid+';'+IntToStr(user.ID)+';'+inttostr(repl.ID)+';'+datestart+';'+dateend);

        req.Lang := bitfarmApp.Lang;
        bitfarmApp.MainSocket.SendRequestToServer(req);

        req := bitfarmApp.MainSocket.ReceiveResponse;
        if req = nil then
          Exit;

        if req.ReqType = 'respAuthError' then
        begin
          BfShowMessage(_('Sie haben fr diesen Vorgang keine Berechtigung. Bitte melden Sie sich erneut an.'));
          UserAction := bitfarmApp.CreateUserAction;
          UserAction^.ActionName := 'actRelogin';
          bitfarmApp.MainThread.Add(UserAction);
          Exit;
        end
        else if req.ReqType = 'respError' then
          BfShowMessage(_('Fehler beim Speichern:') + ' ' + req.DataString)
        else
          FillHolidayForm;
      finally
        FreeAndNil(req);
      end;
    end;
  except
    bitfarmLog.Write(llNormal, ltException, 'btnEditClick', ClassName);
  end;
end;

procedure TfrmEditHoliday.btnAddClick(Sender: TObject);
begin
  try
    bitfarmApp.StartHoliday;
    FillHolidayForm;
  except
    bitfarmLog.Write(llNormal, ltException, 'btnAddClick', ClassName);
  end;
end;

procedure TfrmEditHoliday.btnCloseClick(Sender: TObject);
begin
  Self.Close;
end;

procedure TfrmEditHoliday.FillHolidayForm;
var
  cnt, cntSrc: TUniConnection;
  qry: TUniQuery;
  i: Integer;
  ListItem: TListItem;
  holiday: TbfaHoliday;
  banned, deleted: Boolean;
begin
  try
    try
      Self.ListView1.Clear;
      Self.ListView1.Items.BeginUpdate;

      qry := TUniQuery.Create(nil);
      qry.Connection := dmMain.cntMain;
      qry.SQL.Text := 'SELECT sh.holid, sh.start, sh.end, sh.userid_src, sh.userid_dst, ' +
                      '(SELECT username FROM sys_user WHERE userid = sh.userid_dst) AS uname_dst, ' +
                      '(SELECT username FROM sys_user WHERE userid = sh.userid_src) AS uname_src, ' +
                      // banned = 1 -> gesperrt
                      '(SELECT not(FIND_IN_SET('#39'enabled'#39', flags)) FROM sys_user WHERE userid = sh.userid_dst) AS banned, ' +
                      // deleted = 1 -> gelscht
                      '(SELECT find_in_set('#39'deleted'#39', flags) > 0 FROM sys_user WHERE userid = sh.userid_dst) AS deleted, ' +
                      '(SELECT displayname FROM sys_user WHERE userid = sh.userid_dst) AS dispname_dst, ' +
                      '(SELECT displayname FROM sys_user WHERE userid = sh.userid_src) AS dispname_src ' +
                      'FROM sys_holiday sh WHERE (sh.userid_dst = :userid OR sh.userid_src = :userid) '+
                      'AND (:today <= end)';
      qry.ParamByName('userid').AsInteger := bitfarmApp.User.ID;
      qry.ParamByName('today').AsDateTime := Date;
      qry.Execute;
      if qry.RecordCount = 0 then
      begin
        bitfarmLog.Write(llNormal, ltDebug, 'FillHolidayForm - no holidays found', ClassName);
        Exit;
      end else
        qry.First;

      while not qry.Eof do
      begin
        holiday := TbfaHoliday.Create(qry.FieldByName('holid').AsInteger,
            qry.FieldByName('userid_src').AsInteger,
            qry.FieldByName('userid_dst').AsInteger,
            qry.FieldByName('start').AsDateTime,
            qry.FieldByName('end').AsDateTime,
            qry.FieldByName('uname_src').AsString,
            qry.FieldByName('uname_dst').AsString,
            qry.FieldByName('dispname_src').AsString,
            qry.FieldByName('dispname_dst').AsString);
        banned := qry.FieldByName('banned').AsInteger = 1;
        deleted := qry.FieldByName('deleted').AsInteger = 1;
        ListItem := Self.ListView1.Items.Add;
//        ListItem.Caption := bitfarmApp.UserList[qry.FieldByName('userid_src').AsInteger];
//        ListItem.SubItems.Add(bitfarmApp.UserList[qry.FieldByName('userid_dst').AsInteger]);

        ListItem.Caption := bitfarmApp.UserList[bitfarmApp.UserList.IndexOfObject(TObject(qry.FieldByName('userid_src').AsInteger))];
        if bitfarmApp.UserList.IndexOfObject(TObject(qry.FieldByName('userid_dst').AsInteger)) <> -1 then
          ListItem.SubItems.Add(bitfarmApp.UserList[bitfarmApp.UserList.IndexOfObject(TObject(qry.FieldByName('userid_dst').AsInteger))])
        else
        begin
          if deleted then
            ListItem.SubItems.Add(DBGetUserName(qry.FieldByName('userid_dst').AsInteger, True) + ' ' + _('(gelscht)'))
          else if banned then
            ListItem.SubItems.Add(DBGetUserName(qry.FieldByName('userid_dst').AsInteger, True) + ' ' + _('(gesperrt)'))
          else
            ListItem.SubItems.Add(DBGetUserName(qry.FieldByName('userid_dst').AsInteger, True) + ' ' + _('(unbekannter Fehler)'));
        end;

        ListItem.SubItems.Add(DateToStr(qry.FieldByName('start').AsDateTime));
        ListItem.SubItems.Add(DateToStr(qry.FieldByName('end').AsDateTime));
        Self.ListView1.Items[ListItem.Index].Data := holiday;
        qry.Next;
      end;
      bitfarmApp.CheckHoliday;

      btnAdd.Enabled := True;
      btnEdit.Enabled := False;
      btnDelete.Enabled := False;
    except
      on E: Exception do
        bitfarmLog.Write(llNormal, ltException, 'FillHolidayForm: ' + E.Message, ClassName);
    end;
  finally
    Self.ListView1.Items.EndUpdate;
    if qry <> nil then
      FreeAndNil(qry);
  end;
end;

procedure TfrmEditHoliday.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  bitfarmApp.CheckHoliday; // bfacrm:ticket__show__35598
end;

procedure TfrmEditHoliday.FormCreate(Sender: TObject);
begin
  FillHolidayForm;
end;

procedure TfrmEditHoliday.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key = VK_DELETE) and btnDelete.Enabled then
    btnDelete.Click;
end;

procedure TfrmEditHoliday.ListView1ColumnClick(Sender: TObject;
  Column: TListColumn);
begin
  TListView(Sender).SortType := stNone;

  if Column.Index <> SortedColumn then
  begin
    SortedColumn := Column.Index;
    Descending := False;
  end
  else
    Descending := not Descending;

  TListView(Sender).SortType := stText;
end;

procedure TfrmEditHoliday.ListView1Compare(Sender: TObject; Item1,
  Item2: TListItem; Data: Integer; var Compare: Integer);
begin
  if SortedColumn = 0 then
    Compare := CompareText(Item1.Caption, Item2.Caption)
  else if SortedColumn <> 0 then
    Compare := CompareText(Item1.SubItems[SortedColumn-1], Item2.SubItems[SortedColumn-1]);

  if Descending then
    Compare := -Compare;
end;

procedure TfrmEditHoliday.ListView1SelectItem(Sender: TObject; Item: TListItem;
  Selected: Boolean);
begin
  if Selected and (Item <> nil) then
  begin
    if TbfaHoliday(Item.Data).userid_src = bitfarmApp.User.ID then
    begin
      btnEdit.Enabled := True;
      btnDelete.Enabled := True;
    end else
    begin
      btnEdit.Enabled := False;
      btnDelete.Enabled := False;
    end;
  end else if (Item = nil) then
  begin
    btnEdit.Enabled := False;
    btnDelete.Enabled := False;
  end;
end;

{ TbfaHoliday }

constructor TbfaHoliday.Create(holid, userid_src, userid_dst: Integer;
  start_date, end_date: TDateTime; username_src, username_dst, displayname_src, displayame_dst: string);
begin
  try
    Self.holid := holid;
    Self.userid_src := userid_src;
    Self.userid_dst := userid_dst;
    Self.start_date := start_date;
    Self.end_date := end_date;
    Self.username_src := username_src;
    Self.username_dst := username_dst;
    Self.displayname_src := displayname_src;
    Self.displayame_dst := displayame_dst;
  except

  end;
end;

end.
