unit FloatingForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, TB2Dock, SpTBXDkPanels, SpTBXItem, TB2Item, SpTBXSkins,
  System.Actions, Vcl.ActnList, Vcl.ExtCtrls;

type
  TbfaFloatingForm = class(TForm)
    dckFloating: TSpTBXMultiDock;
    SpTBXStatusBar1: TSpTBXStatusBar;
    itmFloatingInfo: TSpTBXLabelItem;
    SpTBXLabelItem2: TSpTBXLabelItem;
    ActionList1: TActionList;
    actSaveFloat: TAction;
    TBSeparatorItem1: TTBSeparatorItem;
    itmNotification: TSpTBXLabelItem;
    tmrNotification: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormShow(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormDeactivate(Sender: TObject);
// <!-- SJ 06.09.2019
//    procedure itmFloatingInfoDrawCaption(Sender: TObject; ACanvas: TCanvas;
//      ClientAreaRect: TRect; State: TSpTBXSkinStatesType; var ACaption: string;
//      var CaptionRect: TRect; var CaptionFormat: Cardinal;
//      IsTextRotated: Boolean; const PaintStage: TSpTBXPaintStage;
//      var PaintDefault: Boolean);
    procedure ResetNotificationBar(Sender: TObject);
    procedure actSaveFloatExecute(Sender: TObject);

  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure OnMove(var Msg: TWMMove); message WM_MOVE;

  private
    { Private declarations }
    FFloatingFullScreen: Boolean;
    FInShow: Boolean;

    procedure WMNCLButtonDblClk(var msg: TWMNCLButtonDblClk); message WM_NCLBUTTONDBLCLK;
    procedure WMSize(var Msg: TMessage); message WM_SIZE;
    procedure WMDisplayChange(var Msg: TMessage); message WM_DISPLAYCHANGE;

  public
    { Public declarations }
    Panel : TSpTBXDockablePanel;
    OldDock : TTBDock;
  end;

implementation

{$R *.dfm}

uses
  bfaLog, bfaApplication, bfaCommonUtils, bfaPreview36, MainDataModule, MainForm,
  Clipbrd, ActiveX, bfaBaseObjects, bfaObjects36,
  bfaCommonDB;  //01.08.2024 SD <--bfacrm:ticket__show__39252

// SJ 13.12.2019 bfacrm:ticket__show__21497
//procedure TbfaFloatingForm.WMNCLButtonDblClk(var Msg: TWMNCLButtonDblClk);
//var
//  Prev : TbfaPreview;
//begin
//  if (Msg.HitTest = HTCAPTION) then
//  begin
//    bitfarmApp.InDockChanged := True;
//    try
//      Prev := bitfarmApp.PreviewContainer.Containers[Panel.Tag].Preview;
//      Prev.ShowToolbars(False);
//
//      with bitfarmApp.ViewerOptions.GUIOptions do
//      begin
//        if (Self.WindowState <> wsMaximized) then
//        begin
//          StoredFloatingPosX := Self.Left;
//          StoredFloatingPosY := Self.Top;
//          StoredFloatingWidth := Self.Width;
//          StoredFloatingHeight := Self.Height;
//        end;
//        StoredFloatingFullScreen := (Self.WindowState = wsMaximized);
//      end;
//
//      Self.Hide;
//      Msg.Result := 0;
//      Panel.CurrentDock := OldDock;
//      if Self <> frmMainForm.frmFloatingSearch then
//        Self.Destroy;
//      bitfarmApp.Autozoom(Prev);
//    finally
//      bitfarmApp.InDockChanged := False;
//    end;
//  end
//  else
//    inherited;
//end;

procedure TbfaFloatingForm.WMNCLButtonDblClk(var Msg: TWMNCLButtonDblClk);
begin
  try
    if Self = frmMainForm.frmFloatingSearch then
    begin
      Self.Hide;
      Msg.Result := 0;
      Panel.CurrentDock := OldDock;
    end
    else
      inherited;
  except
    bitfarmLog.Write(llNormal, ltException, 'TbfaFloatingForm.WMNCLButtonDblClk', ClassName);
  end;
end;
// SJ 06.01.2020 bfacrm:ticket__show__21497

// SJ 13.10.2021 Monitor aus bei laufendem Betrieb? Fenster in sichtbaren Bereich
procedure TbfaFloatingForm.WMDisplayChange(var Msg: TMessage);
var
  ScreenWidth: Integer;
begin
  try
    inherited;
    ScreenWidth := GetSystemMetrics(SM_CXVIRTUALSCREEN);
    if (Self.Left > ScreenWidth) then
      Self.Left := 0;
  except
    bitfarmLog.Write(llNormal, ltException, 'TbfaFloatingForm.WMDisplayChange', ClassName);
  end;
end;

procedure TbfaFloatingForm.FormActivate(Sender: TObject);
begin
  if bitfarmApp.PreviewContainer = nil then
    Exit;

  bitfarmLog.DebugBegin(llDebug, 'TbfaFloatingForm.FormActivate', Classname);
  try
    try
      if ((not bitfarmApp.ClosingMainForm) and
          (not bitfarmApp.InCreateFloatingForm) and (Panel <> frmMainForm.pnlSearchOptions) and
          (bitfarmApp.PreviewContainer.ActiveItem <> Panel.Tag)) then      
        bitfarmApp.ShowActiveTabAction(Panel.Tag);
    except
      bitfarmLog.Write(llNormal, ltException, 'TbfaFloatingForm.FormActivate', ClassName);
    end;
  finally
    bitfarmLog.DebugEnd(llDebug, 'TbfaFloatingForm.FormActivate', Classname);
  end;
end;

procedure TbfaFloatingForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
var
  UserAction: PUserAction;
begin
  try
    bitfarmApp.InPanelClose := True;
    UserAction := bitfarmApp.CreateUserAction;
    UserAction^.ActionName := 'actPanelClose';
    UserAction^.Objects.Add(Sender);
    bitfarmApp.MainThread.Add(UserAction);
    CanClose := False;
    if (frmMainForm.fraScanFunctions.btnArchive.Enabled = False) then
      frmMainForm.fraScanFunctions.btnArchive.Enabled := True;
  except
    bitfarmLog.Write(llNormal, ltException, 'PanelClose', Classname);
  end;
end;

procedure TbfaFloatingForm.FormCreate(Sender: TObject);
begin
  Left := 1;
  Top := 1;
  Width := 300;
  Height := 300;
  FFloatingFullScreen := False;
  FInShow := False;
end;

procedure TbfaFloatingForm.FormDeactivate(Sender: TObject);
var
  Prev: TbfaPreview;
begin
  try
    if (Panel.Tag >=0) and (Panel.Tag <= bitfarmApp.PreviewContainer.Containers.Count - 1) then
    begin
      Prev := bitfarmApp.PreviewContainer.Containers[Panel.Tag].Preview;
      if Prev.PreviewKind = pvkSingle then
      begin
        if Prev.SingleView.gbAnnoNotSaved then
          Prev.SingleView.saveannostream;
//25.04.2019 hier muss in anderen previews auch aktualiaisert werden-->
        if Prev.SingleView.fRefreshInOtherPrevs then
        begin
          bitfarmApp.CreateRefreshDocumentInPreviewsAction(Prev.ActiveDoc,
            Prev, nil, False, False); //25.04.2019 von saveannostream raus, hier rein!
          Prev.SingleView.fRefreshInOtherPrevs := False;
        end;
//25.04.2019 <--
      end;
    end;
  except
    bitfarmLog.Write(llNormal, ltException, 'TbfaFloatingForm.FormDeactivate', Classname);
  end;
end;

procedure TbfaFloatingForm.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
var
  UserAction: PUserAction;
  Prev: TbfaPreview;
  i: Integer;
begin
  try
    if Key = VK_F2 then
    begin
      bitfarmLog.Write(llDebug, ltInfo, 'TbfaFloatingForm.FormKeyUp - VK_F2', ClassName);
      try
        bitfarmLog.Write(llDebug, ltInfo, 'TbfaFloatingForm.FormKeyUp - VK_F2 - Screen.ActiveControl.Name.ActiveControl: ' + 
          Screen.ActiveControl.Name, ClassName);
      except
        //
      end;
    end;        
  
    if Shift = [] then
    begin
      if Key = VK_F12 then
      begin
        if WindowState = wsMaximized then
          WindowState := wsNormal
        else
          WindowState := wsMaximized;
      end
      else if Key = VK_F11 then
        frmMainForm.actF11.Execute;
//12.09.2017 Ticket 11783 -->
// SJ 26.04.2022 - bfacrm:ticket__show__30789
//      else if Key = VK_F2 then
//      begin
//        Prev := bitfarmApp.PreviewContainer.Containers[Panel.Tag].Preview;
//        if (prev <> nil) and (Prev.PreviewKind = pvkSingle) then
//        begin
//          if Prev.ActiveDoc <> nil then
//          begin
//            if (Prev.ActiveDoc.ID <> -1) then
//            begin
//              if (not Prev.ActiveDoc.InEditMode) then
//                frmMainForm.actInProgress.Execute
//              else
//                frmMainForm.actSave.Execute;
//            end;
//          end;
//        end;
//      end;
// SJ 26.04.2022 - bfacrm:ticket__show__30789
//12.09.2017 Ticket 11783 <--
    end
    else if Shift = [ssAlt] then
    begin
      case Key of
        VK_F11:
          begin
            UserAction := bitfarmApp.CreateUserAction;
            UserAction^.ActionName := 'actAltF11';
            bitfarmApp.MainThread.Add(UserAction);
          end;
      end;
    end
    else if Shift = [ssCtrl] then
    begin
      case Key of
        // ACHTUNG: siehe auch TForm1.FormKeyDown
        Ord('V'):
        begin
          prev := bitfarmApp.ActivePreview;

          // Datei(en) in Zwischenablage? Importer.
          if Clipboard.HasFormat(CF_HDROP) then
              bitfarmApp.CreateLoadAction(DROPEFFECT_COPY, nil)

          else if (prev <> nil) and (prev.ActiveDoc <> nil) and (prev.SingleView <> nil) and
             (prev.SingleView.img_DocView <> nil) and
             (prev.SingleView.img_DocView.ObjIsClipboardAvailable) and
             (prev.SingleView.CheckAnnoRights) and (
               (prev.ActiveDoc.EditmodeUserID = bitfarmApp.User.ID) or
               (prev.ActiveDoc.ID = -1)
             )
          then
          begin
            var hobj, idx: Integer;
            var AnnoObj: TbfaAnnotation;

            if prev.SingleView.img_DocView.SelObjectsCount > 0 then
              prev.SingleView.img_DocView.UnSelAllObjects;

            prev.SingleView.img_DocView.ObjPasteFromClipboard(25,25);
            prev.SingleView.gbAnnoNotSaved := True;
            prev.ActiveDoc.Modified_Annos := True;
            if prev.SingleView.img_DocView.SelObjectsCount > 0 then
            begin
              AnnoObj := prev.ActiveDoc.GetAnnotation(prev.ActivePage, prev.CurrentLayer, idx);
              if AnnoObj = nil then
              begin
                AnnoObj := TbfaAnnotation.Create;
                AnnoObj.Page := prev.ActivePage;
                AnnoObj.Layer := prev.CurrentLayer;
                AnnoObj.Modified := True;
                //01.08.2024 SD -->bfacrm:ticket__show__39252
                if AnnoObj.Layer = 0 then
                  AnnoObj.rights := prev.ActiveDoc.Rights
                else if AnnoObj.Layer = bitfarmApp.User.ID then
                  AnnoObj.rights := (rightAPRead or rightAPWrite or rightAPModify)
                else
                begin
                  AnnoObj.rights := DbTableColumnGetInteger(Format('select bit_or(rights) from sys_docxs where gdocid = %d and src_userid = %d and userid = %d',
                    [prev.ActiveDoc.ID, AnnoObj.Layer, bitfarmApp.User.ID]));
                end;
                //01.08.2024 SD <--bfacrm:ticket__show__39252
                prev.ActiveDoc.Annotations.Add(AnnoObj);
              end else
                AnnoObj.Modified := True;

              for i := 0 to prev.SingleView.img_DocView.SelObjectsCount - 1 do
              begin
                hobj := prev.SingleView.img_DocView.SelObjects[i];
                prev.SingleView.createannodata(hobj);
              end;
              prev.SingleView.SetObjectPosition;
            end;
            prev.SingleView.saveannostream(True);
          end
          else
            Exit;
        end;
      end;
    end;
  except
    bitfarmLog.Write(llNormal, ltException, 'TbfaFloatingForm.FormKeyUp', ClassName);
  end;
end;

procedure TbfaFloatingForm.FormResize(Sender: TObject);
begin
  try
    if (not FInShow) and (Self = frmMainForm.frmFloatingSearch) and (not bitfarmApp.DockSearchPanel) then
    begin
      with bitfarmApp.ViewerOptions.GUIOptions do
      begin
        SearchPanelFloatingPosX := Self.Left;
        SearchPanelFloatingPosY := Self.Top;
        SearchPanelFloatingHeight := Self.Height;
        SearchPanelFloatingWidth := Self.Width;
      end;
    end;
  except
    bitfarmLog.Write(llNormal, ltException, 'TbfaFloatingForm.FormResize', Classname);
  end;
end;

procedure TbfaFloatingForm.OnMove(var Msg: TWMMove);
begin
  inherited;
  try
    if (not FInShow) and (Self = frmMainForm.frmFloatingSearch) and (not bitfarmApp.DockSearchPanel) then
    begin
      with bitfarmApp.ViewerOptions.GUIOptions do
      begin
        SearchPanelFloatingPosX := Self.Left;
        SearchPanelFloatingPosY := Self.Top;
        SearchPanelFloatingHeight := Self.Height;
        SearchPanelFloatingWidth := Self.Width;
      end;
    end;
  except
    bitfarmLog.Write(llNormal, ltException, 'TbfaFloatingForm.OnMove', Classname);
  end;
end;

procedure TbfaFloatingForm.ResetNotificationBar(Sender: TObject);
begin
  bitfarmLog.DebugBegin(llDebug, 'ResetNotificationBar', Classname);
  try
    try
      begin
        Self.tmrNotification.Enabled := False;
        Self.itmNotification.Visible := False;
        Self.itmNotification.Caption := '';
      end;
    except
      bitfarmLog.Write(llNormal, ltException, 'ResetNotificationBar', ClassName);
    end;
  finally
    bitfarmLog.DebugEnd(llDebug, 'ResetNotificationBar', Classname);
  end;
end;

procedure TbfaFloatingForm.FormShow(Sender: TObject);
var
  Prev: TbfaPreview;
  ScreenWidth: Integer;
begin
  FInShow := True;
  try
    ScreenWidth := -1;
    ScreenWidth := GetSystemMetrics(SM_CXVIRTUALSCREEN);
    if Self <> frmMainForm.frmFloatingSearch then
    begin
      with bitfarmApp.ViewerOptions.GUIOptions do
      begin
      // SJ 13.10.2021 kleinere Bildschirmauflsung?
        if (ScreenWidth <> -1) and (StoredFloatingPosX < ScreenWidth) then
          Left := StoredFloatingPosX
        else
          Left := 0;
        Top := StoredFloatingPosY;
        Height := StoredFloatingHeight;
        Width := StoredFloatingWidth;
        if StoredFloatingFullScreen then
          WindowState := wsMaximized;
          //07.05.2019 Ticket 19143
        if (frmMainForm.fraScanFunctions.btnArchive.Enabled = True) then
          frmMainForm.fraScanFunctions.btnArchive.Enabled := False;
      end;
    end
    else
    begin
      with bitfarmApp.ViewerOptions.GUIOptions do
      begin
      // SJ 13.10.2021 kleinere Bildschirmauflsung?
        if (ScreenWidth <> -1) and (SearchPanelFloatingPosX < ScreenWidth) then
          frmMainForm.frmFloatingSearch.Left := SearchPanelFloatingPosX
        else
          frmMainForm.frmFloatingSearch.Left := 0;
        frmMainForm.frmFloatingSearch.Top := SearchPanelFloatingPosY;
        frmMainForm.frmFloatingSearch.Height := SearchPanelFloatingHeight;
        frmMainForm.frmFloatingSearch.Width := SearchPanelFloatingWidth;
      end;
    end;

    bitfarmApp.SetStdStsInfo(itmFloatingInfo, nil);

  finally
    FInShow := False;
  end;
end;

// <!-- SJ 06.09.2019
//procedure TbfaFloatingForm.itmFloatingInfoDrawCaption(Sender: TObject;
//  ACanvas: TCanvas; ClientAreaRect: TRect; State: TSpTBXSkinStatesType;
//  var ACaption: string; var CaptionRect: TRect; var CaptionFormat: Cardinal;
//  IsTextRotated: Boolean; const PaintStage: TSpTBXPaintStage;
//  var PaintDefault: Boolean);
//begin
//  ACaption := StringReplace(ACaption, '&', '&&', [rfReplaceAll]);
//end;

procedure TbfaFloatingForm.actSaveFloatExecute(Sender: TObject);
begin
  if frmMainForm.actSave.Enabled then
    frmMainForm.actSave.execute;
end;

procedure TbfaFloatingForm.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.ExStyle := Params.ExStyle or WS_EX_APPWINDOW;
//24.10.2019  Params.WndParent := GetDesktopWindow;
  Params.WndParent := 0;
end;

procedure TbfaFloatingForm.WMSize(var Msg: TMessage);
begin
  inherited;
  try
//    if (Msg.WParam  = SIZE_MAXIMIZED) then // SJ 06.01.2020
    if (Msg.WParam  = SIZE_MAXIMIZED) or (Msg.WParam = SIZE_RESTORED) then
    begin
      if (Panel <> nil) and (bitfarmApp.PreviewContainer.Containers.Count > Panel.Tag) then
        bitfarmApp.PreviewContainer.Containers[Panel.Tag].Preview.bfaTlbPreview.sbmAutozoom.Click;
    end;
  except
    bitfarmLog.Write(llNormal, ltException, 'TbfaFloatingForm.WMSize', Classname);
  end;
end;

end.
