unit HistoryFrame;

interface

uses
  gnugettext, Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, SpTBXItem, SpTBXControls, Uni, DB, MemDS, VirtualTable, DBClient,
  {$IFDEF COMMENTGRID}cxGrid, cxGridLevel, bfaGridDBTableView, cxLookAndFeels,
  dxCore, cxGridCustomTableView, cxGraphics, cxGridCustomView, cxGridTableView,
  bfaCommonStyles, bfaIconsDM
  {$ELSE}CRGrid, DBGrids, u_bfaGrid{$ENDIF};

type
  TfraHistory = class(TFrame)
    SpTBXPanel1: TSpTBXPanel;
  private
    { Private-Deklarationen }
    {$IFDEF COMMENTGRID}
    procedure tvwHistoryCustomDrawCell(Sender: TcxCustomGridTableView;
      ACanvas: TcxCanvas; AViewInfo: TcxGridTableDataCellViewInfo;
      var ADone: Boolean);
    procedure grdHistorySelectionChanged(Sender: TcxCustomGridTableView);
    {$ELSE}
    procedure grdHistoryMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: integer);
    procedure grdHistoryTitelClick(Column: TColumn);
    {$ENDIF}
  public
    { Public-Deklarationen }
    {$IFDEF COMMENTGRID}
    grdHistory: TbfaGridDBTableView;
    grdHistoryGrid: TcxGrid;
    grdHistoryLevel: TcxGridLevel;
    {$ELSE}
    grdHistory: TbfaDBGrid;
    {$ENDIF}
    HistDataSet: TClientDataSet;
    HistDataSource : TDataSource;

    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure Clear;
    procedure Retranslate;
  end;

implementation

{$R *.dfm}

uses
  bfaLog, MainDataModule, bfaCommonUtils
  {$IFDEF COMMENTGRID}
  , cxControls
  {$ENDIF}
  ;

{ TfrmHistory }

procedure TfraHistory.Clear;
begin
  {$IFDEF COMMENTGRID}
  grdHistory.Controller.ClearSelection;
  {$ELSE}
  grdHistory.SelectedRows.Clear;
  {$ENDIF}
  if HistDataSet.Active then
    HistDataSet.EmptyDataSet;
end;

{$IFDEF COMMENTGRID}
procedure TfraHistory.tvwHistoryCustomDrawCell(Sender: TcxCustomGridTableView;
      ACanvas: TcxCanvas; AViewInfo: TcxGridTableDataCellViewInfo;
      var ADone: Boolean);
begin
//20.09.2023 Muss hier sein, sonst TbfaDBGridTableView.DoCustomDrawCell wird nicht gettigt!
end;

procedure TfraHistory.grdHistorySelectionChanged(Sender: TcxCustomGridTableView);
begin
  if grdHistory.DataController.RowCount = 0  then
    Exit;
end;
procedure TfraHistory.Retranslate;
begin
  ReTranslateComponent(Self, 'ViewerV36');
end;

{$ENDIF}


constructor TfraHistory.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  {$IFDEF COMMENTGRID}
  grdHistoryGrid := TcxGrid.Create(Self);
  grdHistoryGrid.Parent := SpTBXPanel1;
  grdHistoryGrid.Align := alClient;
  grdHistoryGrid.Name := '';
  grdHistoryGrid.LookAndFeel.ScrollbarMode := sbmClassic;
  grdHistoryGrid.LookAndFeel.NativeStyle := True;
  grdHistoryLevel := grdHistoryGrid.Levels.Add;
  grdHistoryLevel.Name := 'grdHistory_LevelName';
  grdHistory := grdHistoryGrid.CreateView(TbfaGridDBTableView) as TbfaGridDBTableView;
  grdHistory.Name := 'grdHistory_ViewName';
  grdHistory.Grd := grdHistoryGrid;

  grdHistoryLevel.GridView := grdHistory;
  grdHistory.Styles.Header := stlColumn;
  grdHistory.Styles.Footer := stlColumn;
  grdHistory.Styles.GroupByBox := stlGroupByBoxStyle;
  grdHistory.Styles.Navigator := stlGroupByBoxStyle;
  grdHistory.Styles.NavigatorInfoPanel := stlGroupByBoxStyle;
  grdHistory.Styles.Background := stlBackgroundStyle;
  grdHistory.Styles.Group := stlBackgroundStyle;
  grdHistory.RowColorEvenOdd := bitfarmApp.ViewerOptions.GeneralOptions.RowColorEvenOdd;
  grdHistory.ImageList := TImageList(bfaIcons.ilMainMenu);
  grdHistory.SetGridDBTableViewDefaultSettings;
  grdHistory.OnCustomDrawCell := tvwHistoryCustomDrawCell;
  grdHistory.OptionsView.CellAutoHeight := True;
  grdHistory.OptionsBehavior.RecordScrollMode := rsmByPixel; //11.10.2023 bfacrm:ticket__show__36597
  grdHistory.OnSelectionChanged := grdHistorySelectionChanged;
  {$ELSE}
  grdHistory := TbfaDBGrid.Create(AOwner);
  grdHistory.Parent := SpTBXPanel1;
  grdHistory.Align := alClient;
  grdHistory.Kind := gkNone;
  grdHistory.Options :=   [dgTitles, dgColumnResize, dgColLines, dgRowLines, dgTabs,
   dgRowSelect, dgConfirmDelete, dgCancelOnExit, dgTitleClick];
  grdHistory.OptionsEx := [dgeEnableSort, dgeLocalSorting, dgeStretch];
  grdHistory.OnMouseUp := grdHistoryMouseUp;
  grdHistory.OnTitleClick := grdHistoryTitelClick;
  grdHistory.WordWrap := True;
  grdHistory.RowColorEvenOdd := bitfarmApp.ViewerOptions.GeneralOptions.RowColorEvenOdd;
  {$ENDIF}
  HistDataSet := TClientDataSet.Create(nil);
  HistDataSource := TDataSource.Create(nil);
  {$IFDEF COMMENTGRID}
  //15.09.2023
  {$ELSE}
  HistDataSource.OnStateChange := grdHistory.bfaOnStateChange;
  {$ENDIF}
  HistDataSet.FieldDefs.Add('hist_date', ftDateTime, 0);
  HistDataSet.FieldDefs.Add('hist_user', ftWideString, 64);
  HistDataSet.FieldDefs.Add('hist_task', ftWideString, 512);
  HistDataSet.FieldDefs.Add('hist_datestring', ftString, 128);
  try
    HistDataSet.CreateDataSet;
    HistDataSource.DataSet := HistDataSet;
    {$IFDEF COMMENTGRID}
    grdHistory.CreateDBColumn('Datum', _('Datum'), 'hist_date', '', 130, True, soDescending);
    grdHistory.CreateDBColumn('Benutzer', _('Benutzer'), 'hist_user', '', 200);
    grdHistory.CreateDBColumn('Aktion', _('Aktion'), 'hist_task', '', 500);
    grdHistory.OptionsView.ColumnAutoWidth := True;
    grdHistory.DataController.DataSource := HistDataSource;
    {$ELSE}
    grdHistory.DataSource := HistDataSource;

    grdHistory.AdjustColumns;
    grdHistory.Columns.Add;
    grdHistory.Columns[0].Title.Caption := _('Datum');
    grdHistory.Columns[0].FieldName := 'Datum';
    grdHistory.Columns[0].SortOrder := soDesc;
    grdHistory.Columns[0].Width := 20;

    grdHistory.Columns.Add;
    grdHistory.Columns[1].Title.Caption := _('Benutzer');
    grdHistory.Columns[1].FieldName := 'Benutzer';
    grdHistory.Columns[1].Width := 10;
//    grdHistory.Columns[1].Width := 150;

    grdHistory.Columns.Add;
    grdHistory.Columns[2].Title.Caption := _('Aktion');
    grdHistory.Columns[2].FieldName := 'Aktion';
    grdHistory.Columns[1].Width := 150;
    {$ENDIF}

    HistDataSet.FilterOptions := [foCaseInsensitive];

    TranslateComponent(Self, 'ViewerV36');

  except
    on E: Exception do
      bitfarmLog.Write(llNormal, ltException, 'TfraHistory.Create - HistDataSet.CreateDataSet: ' + E.Message, ClassName);
  end;
end;

destructor TfraHistory.Destroy;
begin
  try
    if not bitfarmApp.ClosingMainForm then //05.12.2018
    begin
      bitfarmLog.DebugBegin(llDebug, 'TfraHistory.Destroy', ClassName);
      try
        try
          if Assigned(HistDataSet) then
          begin
            bitfarmLog.Write(llDebug, ltInfo, 'TfraHistory.Destroy - Assigned(HistDataSet)', ClassName);
            try
              if HistDataSet.Active then
              begin
                HistDataSet.EmptyDataSet;
                HistDataSet.Close;
              end;
            finally
              bitfarmLog.Write(llDebug, ltInfo, 'TfraHistory.Destroy - HistDataSet.Free', ClassName);
              HistDataSet.Free;
            end;
            bitfarmLog.Write(llDebug, ltInfo, 'TfraHistory.Destroy - HistDataSource.Free;', ClassName);
            HistDataSource.Free;
          end;
        except
          on E: Exception do
            bitfarmLog.Write(llNormal, ltException, 'TfraHistory.Destroy - ' + E.Message, ClassName);
        end;
      finally
        bitfarmLog.DebugEnd(llDebug, 'TfraHistory.Destroy', ClassName);
      end;
    end;
  finally
    inherited Destroy;
  end;
end;

{$IFDEF COMMENTGRID}
//25.09.2023
{$ELSE}
procedure TfraHistory.grdHistoryMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: integer);
begin
  try
    grdHistory.SelectedRows.Clear;
    grdHistory.SelectedRows.CurrentRowSelected := True;
  except
    bitfarmLog.Write(llNormal, ltException, 'grdHistoryMouseUp', ClassName);
  end;
end;

procedure TfraHistory.grdHistoryTitelClick(Column : TColumn);
begin
  try
    bitfarmApp.SortingClientDataSet(grdHistory, HistDataSet, Column);
  except
    bitfarmLog.Write(llNormal, ltException, 'grdHistoryTitelClick', ClassName);
  end;
end;
{$ENDIF}

end.
