unit HolidayForm;

interface

uses
  gnugettext, Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, cxGraphics, cxControls, cxLookAndFeels, cxLookAndFeelPainters,
  cxContainer, cxEdit, ComCtrls, dxCore, cxDateUtils, SpTBXControls, cxTextEdit,
  cxMaskEdit, cxDropDownEdit, cxCalendar, StdCtrls, SpTBXEditors, SpTBXItem,
  bfaBaseObjects, Generics.Collections;

type
  TGroupsAndUsers = class(TObject)
  private
    FGroups: TList<TbfaGroup>;
  public
    constructor Create;
    property Groups: TList<TbfaGroup> read FGroups;
  end;

  TfrmHoliday = class(TForm)
    SpTBXLabel1: TSpTBXLabel;
    cbxReplacement: TSpTBXComboBox;
    SpTBXLabel2: TSpTBXLabel;
    cxDateEdit1: TcxDateEdit;
    SpTBXLabel3: TSpTBXLabel;
    cxDateEdit2: TcxDateEdit;
    btnOK: TSpTBXButton;
    btnCancel: TSpTBXButton;
    SpTBXLabel4: TSpTBXLabel;
    cbxGroups: TSpTBXComboBox;
    procedure FormCreate(Sender: TObject);
    procedure cxDateEdit1PropertiesEditValueChanged(Sender: TObject);
    procedure cxDateEdit2PropertiesEditValueChanged(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure cbxGroupsClick(Sender: TObject);
  private
    { Private-Deklarationen }
    GroupsAndUsers: TGroupsAndUsers;
  public
    { Public-Deklarationen }
  end;

implementation

{$R *.dfm}

uses
  MainDataModule, bfaCommonDialogs, Uni, MySQLUniProvider, bfaLog,
  bfaCommonMethods;


procedure TfrmHoliday.btnOKClick(Sender: TObject);
begin
  if (cxDateEdit1.Text = '') or (cxDateEdit2.Text = '') then
    Exit;

  try
    if not bitfarmApp.checkholidayloop(bitfarmApp.user.ID, Integer(cbxReplacement.Items.Objects[cbxReplacement.ItemIndex]),
       cxDateEdit1.Date, cxDateEdit2.Date) then
    begin
      BfShowMessage(_('Dieser Eintrag ist nicht mglich, da sonst eine zirkulre Vertretung erzeugt wird.'));
      Exit;
    end else
      ModalResult := mrOk;
  except
    ModalResult := mrNone;
    BfShowMessage(_('Dieser Eintrag ist nicht mglich, da sonst eine zirkulre Vertretung erzeugt wird.'));
  end;
end;

procedure TfrmHoliday.cbxGroupsClick(Sender: TObject);
var
  I, J: Integer;
begin
  try
    cbxReplacement.Items.BeginUpdate;
    try
      cbxReplacement.Clear;
      cbxReplacement.Items.Clear;
      for I := 0 to GroupsAndUsers.Groups.Count - 1 do
        if GroupsAndUsers.Groups[I].ID = Integer(cbxGroups.Items.Objects[cbxGroups.ItemIndex]) then
          for J := 0 to GroupsAndUsers.Groups[I].Members.Count - 1 do
          begin
            // gelschte und gesperrte Benutzer sind nicht in der Liste enthalten
            // der Name darf aber auch nicht in der Liste auftauchen
            if bitfarmApp.UserList.IndexOfObject(TObject(GroupsAndUsers.Groups[I].Members[J].ToInteger)) = -1 then
              Continue;

            if GroupsAndUsers.Groups[I].Members[J].ToInteger <> bitfarmApp.User.ID then
              cbxReplacement.Items.AddObject(bitfarmApp.UserList[bitfarmApp.UserList.IndexOfObject(TObject(GroupsAndUsers.Groups[I].Members[J].ToInteger))], TObject(GroupsAndUsers.Groups[I].Members[J].ToInteger));
          end;
    finally
      cbxReplacement.Items.EndUpdate;
    end;
  except
    bitfarmApp.Log.Write(llNormal, ltException, 'cbxGroupsClick', ClassName);
  end;
end;

procedure TfrmHoliday.cxDateEdit1PropertiesEditValueChanged(Sender: TObject);
begin
  if cxDateEdit1.Date > cxDateEdit2.Date then
    cxDateEdit2.Date := cxDateEdit1.Date;
end;

procedure TfrmHoliday.cxDateEdit2PropertiesEditValueChanged(Sender: TObject);
begin
  if cxDateEdit2.Date < cxDateEdit1.Date then
    cxDateEdit1.Date := cxDateEdit2.Date;
end;

procedure TfrmHoliday.FormCreate(Sender: TObject);
var
  I: Integer;
begin
  TranslateComponent(Self, 'ViewerV36');

// SJ 17.09.2019 bfacrm:ticket__show__18925
//  cxDateEdit1.Date := Date;
//  cxDateEdit2.Date := Date;
  // SJ 06.10.2021 - ikRegExpr deaktiviert SmartInput (Today, Yesterday, Sunday, ...)
  cxDateEdit1.Properties.InputKind := ikRegExpr;
  cxDateEdit1.Properties.EditFormat := FormatSettings.ShortDateFormat;
  cxDateEdit2.Properties.InputKind := ikRegExpr;
  cxDateEdit2.Properties.EditFormat := FormatSettings.ShortDateFormat;
  GroupsAndUsers := TGroupsAndUsers.Create;

  for I := 0 to GroupsAndUsers.Groups.Count - 1 do
    cbxGroups.Items.AddObject(GroupsAndUsers.Groups[I].Name, TObject(GroupsAndUsers.Groups[I].ID));
  cbxGroups.ItemIndex := cbxGroups.Items.IndexOfObject(TObject(0)); // Alle (Builtin)

  if Assigned(cbxGroups.OnClick) then
    cbxGroups.OnClick(cbxGroups);
end;

procedure TfrmHoliday.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if ((Key = VK_ESCAPE) and (btnCancel.Visible)) then
    btnCancel.Click;
end;

{ TGroupsAndUsers }

constructor TGroupsAndUsers.Create;
var
  qry: TUniQuery;
  AGroup: TbfaGroup;
begin
  FGroups := TList<TbfaGroup>.Create;

  qry := TUniQuery.Create(nil);
  qry.Connection := dmMain.cntMain;
  qry.SQL.Text := 'SELECT groupid, groupname, (SELECT COUNT(*) FROM sys_groupuser sgu WHERE sgu.groupid = sg.groupid) AS cnt, '+
                    '(SELECT GROUP_CONCAT(sgu.userid SEPARATOR '#39';'#39') FROM sys_groupuser sgu '+
                    'LEFT JOIN sys_user su ON sgu.userid = su.userid '+
                    'WHERE sgu.groupid = sg.groupid AND FIND_IN_SET('#39'enabled'#39', su.flags)) AS members '+
                  'FROM sys_group sg WHERE NOT FIND_IN_SET('#39'deleted'#39', flags) HAVING cnt > 0';
  qry.Execute;
  if qry.RecordCount > 0 then
    qry.First;

  while not qry.Eof do
  begin
    AGroup := TbfaGroup.Create;
    AGroup.ID := qry.FieldByName('groupid').AsInteger;
//13.09.2024    AGroup.Name := qry.FieldByName('groupname').AsString;
    AGroup.Name := bitfarmApp.TranslateGroupName(qry.FieldByName('groupname').AsString);
    AGroup.Members.DelimitedText := qry.FieldByName('members').AsString;
    FGroups.Add(AGroup);
    qry.Next;
  end;
end;

end.
