unit ListItemsForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms, Vcl.ExtCtrls,
  Dialogs, u_bfaInputEdit, SpTBXControls, SpTBXItem, StdCtrls, SpTBXEditors, bfaObjects36,
  u_bfaScrollBox, bfaIconsDM, Clipbrd, StrUtils;

type
  TfrmListItems = class(TForm)
    lbItems: TSpTBXListBox;
    SpTBXPanel1: TSpTBXPanel;
    btnOK: TSpTBXButton;
    btnCancel: TSpTBXButton;
    SpTBXPanel2: TSpTBXPanel;
    bfaInputEdit1: TbfaInputEdit;
    btnSortUp: TSpTBXSpeedButton;
    btnSortDown: TSpTBXSpeedButton;
    btnLoadFromFile: TSpTBXButton;
    btnSaveToFile: TSpTBXButton;

    procedure FormCreate(Sender: TObject);
    procedure bfaInputEdit1Delete(Sender: TObject; AValue: WideString);
    procedure bfaInputEdit1Modify(Sender: TObject; AValue: WideString);
    procedure bfaInputEdit1New(Sender: TObject; AValue: WideString);
    procedure btnSortDownClick(Sender: TObject);
    procedure btnSortUpClick(Sender: TObject);
    procedure lbItemsClick(Sender: TObject);
    procedure lbItemsDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure lbItemsDragOver(Sender, Source: TObject; X, Y: Integer; State:
        TDragState; var Accept: Boolean);
    procedure lbItemsMouseDown(Sender: TObject; Button: TMouseButton; Shift:
        TShiftState; X, Y: Integer);
    procedure btnLoadFromFileClick(Sender: TObject);
    procedure btnSaveToFileClick(Sender: TObject);  //01.07.2024 SD <--bfacrm:ticket__show__38777
    function ShowStrictMessage(AValue: WideString): Boolean;
    procedure FormShow(Sender: TObject);
    procedure lbItemsStartDrag(Sender: TObject; var DragObject: TDragObject);
    procedure FormDestroy(Sender: TObject);
    procedure lbItemsKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure lbItemsKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure   bfaCopyToClipboard;
  private
    FEditable: Boolean;
    FStrictmode: Boolean;
    FAddName: Widestring;
    FAddIdx, FDragItem: Integer;
    fbscroll_down: Boolean;
    FAutoscroll : TTimer;
    FCurrentField: TbfaAddfield;
    procedure DoAutoscroll(Sender: TObject);
    procedure SetEditable(const Value: Boolean);
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
    procedure Retranslate;
    property Editable: Boolean read FEditable write SetEditable;
    property Strictmode: Boolean read FStrictmode write FStrictmode;
    property AddName: Widestring read FAddName write FAddName;
    property AddIdx: Integer read FAddIdx write FAddIdx;
    property CurrentField: TbfaAddfield read FCurrentField write FCurrentField;
  end;

var
  MousePos : TPoint;

implementation

{$R *.dfm}

uses
  bfaCommonMethods, bfaCommonDialogs, MainDataModule, bfaCommonUtils, gnugettext, bfaLog,
  Uni, bfaCommonDB, bfaBaseObjects, System.Generics.Collections;


procedure TfrmListItems.FormCreate(Sender: TObject);
begin
  TranslateComponent(Self, 'ViewerV36');
  btnSortUp.Images := bfaIcons.ilMainMenu;
  btnSortUp.ImageIndex := Integer(icoidx_arrow_up);
  btnSortDown.Images := bfaIcons.ilMainMenu;
  btnSortDown.ImageIndex := Integer(icoidx_arrow_down);
  //01.07.2024 SD -->bfacrm:ticket__show__38777
  btnLoadFromFile.Images := bfaIcons.ilMainMenu;
  btnLoadFromFile.ImageIndex := Integer(icoidx_files_download);
  btnLoadFromFile.Caption := '';
  btnSaveToFile.Images := bfaIcons.ilMainMenu;
  btnSaveToFile.ImageIndex := Integer(icoidx_files_load);
  btnSaveToFile.Caption := '';
  //01.07.2024 SD <--bfacrm:ticket__show__38777
//  bfaInputEdit1.NotAllowedChar := sNotAllowedChar;
  bfaInputEdit1.NotAllowedChar := sNotAllowedChar + #34;   //SD 11.06.24 auch Anfhrungszeichen sind in einer Auswahlliste nicht erlaubt
  FAutoscroll := TTimer.Create(nil);
  FAutoscroll.OnTimer := DoAutoscroll;
  FAutoscroll.Interval := 120;
  FAutoscroll.Enabled := False;
  fbscroll_down := False;
  bfaInputEdit1.DontClearEdit := True;
end;

procedure TfrmListItems.FormDestroy(Sender: TObject);
begin
  FAutoscroll.Enabled := False;
  FAutoscroll.Free;
end;

procedure TfrmListItems.Retranslate;
begin
  ReTranslateComponent(Self, 'ViewerV36');
end;

procedure TfrmListItems.FormShow(Sender: TObject);
begin
  if bfaInputEdit1.edtInput.Enabled then
    bfaInputEdit1.edtInput.SetFocus;
end;

procedure TfrmListItems.bfaInputEdit1Delete(Sender: TObject; AValue: WideString);
var
  idx, i: Integer;
  outlist, olditems: TStringList;
  siaResult: TStrictItemsAction;
  Addfields: TList<TbfaBaseObject>;
  field: TbfaAddfield;
  req: TbfaRequest;
begin
  idx := -1;

  if lbItems.Items.Count = 0 then    //nichts in der Liste? dann brauchen wir auch nichts lschen
    Exit;

  outlist := TStringList.Create;
  olditems := TStringList.Create;
  olditems.Delimiter := ';';
  olditems.StrictDelimiter := True;

  for i := lbItems.Items.Count -1 downto 0 do
  begin
    if lbItems.Selected[i] then
      olditems.Add(lbItems.Items[i])
  end;

  AValue := olditems.DelimitedText;

  outlist.Clear;
  siaResult := bitfarmApp.CheckStrictfieldItems(CurrentField, outlist, AValue);

  if siaResult in [siaNoChanges, siaDelete, siaNone] then
  begin
    if siaResult = siaDelete then
    begin
      Addfields := TList<TbfaBaseObject>.Create;
      for i := 0 to olditems.Count -1 do
      begin
        field := TbfaAddField.Create;
        field.Clone(CurrentField);
        field.Modified := True;
        field.OldValue :=  olditems[i];
        field.Value := '';
      Addfields.Add(field);
      end;
      req := TbfaRequest.Create;
      try
        req.ReqType := 'reqResetAddfields';
        req.UserID := bitfarmApp.User.ID;
        req.Profile := bitfarmApp.Profile;
        req.Lang := bitfarmApp.Lang;
        req.Data := Addfields;
        bitfarmApp.MainSocket.SendRequestToServer(req);

        req := bitfarmApp.MainSocket.ReceiveResponse;
        if req = nil then
          Exit;

        if req.ReqType = 'respError' then
          BfShowMessage(_('Fehler beim Zurcksetzen der Zusatzfelder') +#13#10 + req.DataString);

      finally
        FreeAndNil(req);
      end;
    end;

    for i := 0 to olditems.Count -1 do
    begin
      idx := lbItems.Items.IndexOf(olditems[i]);

      if idx > -1 then
      begin
        lbItems.Items.Delete(idx);
        if idx = lbItems.Items.Count then
          dec(idx);
      end else begin
        if (lbItems.ItemIndex > -1) and (lbItems.Items[lbItems.ItemIndex] = '') then  // ist trotzdem ein eintrag ausgewhlt und ist dieser leer?
          lbItems.Items.Delete(lbItems.ItemIndex);
      end;
    end;
  end else
  begin
    if Assigned(outlist) then
      FreeAndNil(outlist);
    Abort;
  end;

  for i := 0 to lbItems.Count -1 do
    lbItems.Selected[i] := False;

  if idx > -1 then
  begin
    lbItems.Selected[idx] := True;
    bfaInputEdit1.edtInput.Text := lbItems.Items[idx];
  end else
    bfaInputEdit1.edtInput.Text := '';

  if Assigned(outlist) then
    FreeAndNil(outlist);

end;

procedure TfrmListItems.bfaInputEdit1Modify(Sender: TObject; AValue:
    WideString);
var
  i, idx : Integer;
  outlist: TStringList;
  siaResult: TStrictItemsAction;
  Addfields: TList<TbfaBaseObject>;
  field: TbfaAddfield;
  req: TbfaRequest;
begin
  AValue := Trim(AValue);

  outlist := TStringList.Create;
  siaResult := bitfarmApp.CheckStrictfieldItems(CurrentField, outlist, bfaInputEdit1.OldValue, AValue);

  if siaResult in [siaAddToList, siaDelete, siaNoChanges, siaNone] then
  begin
    if siaResult in [siaAddToList, siaDelete] then
    begin
      Addfields := TList<TbfaBaseObject>.Create;
      field := TbfaAddField.Create;
      field.Clone(CurrentField);
      field.Modified := True;
      field.OldValue :=  bfaInputEdit1.OldValue;
      if siaResult = siaAddToList then
        field.Value := AValue
      else
        field.Value := '';
      Addfields.Add(field);

      req := TbfaRequest.Create;
      try
        req.ReqType := 'reqResetAddfields';
        req.UserID := bitfarmApp.User.ID;
        req.Profile := bitfarmApp.Profile;
        req.Lang := bitfarmApp.Lang;
        req.Data := Addfields;
        bitfarmApp.MainSocket.SendRequestToServer(req);

        req := BitfarmApp.MainSocket.ReceiveResponse;
        if req = nil then
          Exit;

        if req.ReqType = 'respError' then
          BfShowMessage(_('Fehler beim Zurcksetzen der Zusatzfelder') +#13#10 + req.DataString);

      finally
        FreeAndNil(req);
        FreeAndNil(outlist);
      end;
    end;

    for i := 0 to lbItems.Count -1 do
      lbItems.Selected[i] := False;

    idx := lbItems.Items.IndexOf(bfaInputEdit1.OldValue);
    if idx > -1 then
    begin
      if lbItems.Items.IndexOf(AValue) > -1 then   // wenn es den neuen Wert schon gibt > selektieren
      begin
        lbItems.Items.Delete(idx);
        lbItems.Selected[lbItems.Items.IndexOf(AValue)] := True;
      end else begin
        lbItems.Items[lbItems.Items.IndexOf(bfaInputEdit1.OldValue)] := Trim(AValue);
        lbItems.Selected[lbItems.Items.IndexOf(Trim(AValue))] := True;
      end;
      bfaInputEdit1.ResetEdit;
    end;
  end else
    Abort;
end;

procedure TfrmListItems.bfaInputEdit1New(Sender: TObject; AValue: WideString);
var
  i: Integer;
begin
  AValue := Trim(AValue);

  if AValue <> '' then
  begin
    for i := 0 to lbItems.Count -1 do
      lbItems.Selected[i] := False;
    if lbItems.Items.IndexOf(AValue) = -1 then
    begin
      lbItems.Items.Add(AValue);
      lbItems.Selected[lbItems.Items.Count -1] := True;
    end else
      lbItems.Selected[lbItems.Items.IndexOf(AValue)] := True;

    bfaInputEdit1.ResetEdit;
  end;
end;

procedure TfrmListItems.btnLoadFromFileClick(Sender: TObject);
var
  sl : TStringList;
  OpenDlg : TOpenDialog;
  fname: Widestring;
  i : Integer;
  entry : string;
begin
  OpenDlg := TOpenDialog.Create (nil);
  try
    OpenDlg.Options := [ofPathMustExist, ofFileMustExist];
    OpenDlg.Filter := _('Text-Dateien (*.TXT)|*.TXT');	//01.07.2024 SD <--bfacrm:ticket__show__38777
    if not OpenDlg.Execute then
      Exit;

    sl := TStringList.Create;

    try
      fname := OpenDlg.Files[0];
      //sl.LoadFromFile(fname, TEncoding.UTF8);   // Wenn das File UTF8-BOM (Byte Order Mark) encodiert ist, wird Unicode erkannt!
      sl.LoadFromFile(fname);

      // for i := sl.Count -1 downto 0 do   Warum rckwrts?
      for i := 0 to sl.Count -1 do
      begin
        entry := ParseNotAllowedChars(sl[i]);
        if Trim(entry) <> '' then
          if lbItems.Items.IndexOf(entry) = -1 then
            lbItems.Items.Add(entry);
      end;
    except
      BfMessageDlg(_('Die Datei konnte nicht geladen werden: ') + fname, mtError, [mbOK], False, _(sSysError));
      bitfarmLog.Write(llNormal, ltException, 'btnLoadFromFileClick - fname: ' + fname , ClassName);
    end;
  finally
    OpenDlg.Free;
    FreeAndNil(sl);
    lbItems.SetFocus;
  end;

end;

procedure TfrmListItems.btnSortDownClick(Sender: TObject);
var
  sl : TStringList;
begin
  try
    try
      sl := TStringList.Create;
      sl.Assign(lbItems.Items);
      sl.CustomSort(SortCompareDesc);
      lbItems.Items.Assign(sl);
    finally
      sl.Free
    end;
  except
    bitfarmLog.Write(llNormal, ltException, 'btnSortDownClick', ClassName);
  end;
end;

procedure TfrmListItems.btnSortUpClick(Sender: TObject);
var
  sl : TStringList;
begin
  try
    try
      sl := TStringList.Create;
      sl.Assign(lbItems.Items);
      sl.CustomSort(SortCompareAsc);
      lbItems.Items.Assign(sl);
    finally
      sl.Free
    end;
  except
    bitfarmLog.Write(llNormal, ltException, 'btnSortUpClick', ClassName);
  end;
end;

procedure TfrmListItems.DoAutoscroll(Sender: TObject);
begin
   if fbscroll_down then
      SendMessage(lbItems.Handle, WM_VSCROLL, SB_LINEUP, 0)
   else
      SendMessage(lbItems.Handle, WM_VSCROLL, SB_LINEDOWN, 0);
end;

procedure TfrmListItems.lbItemsClick(Sender: TObject);
begin
  if lbItems.ItemIndex > -1 then
  begin
    if lbItems.Selected[lbItems.ItemIndex] then
      bfaInputEdit1.edtInput.Text := lbItems.Items[lbItems.ItemIndex];

  end;
end;

procedure TfrmListItems.lbItemsDragDrop(Sender, Source: TObject; X, Y: Integer);
var
  Num1, Num2: Integer;
  Point1, Point2: TPoint;
begin
  FAutoscroll.Enabled := False;
  try
    Point2.X:=X;
    Point2.Y:=Y;
    with Source as TSpTBXListBox do
    begin
      Num2:=lbItems.ItemAtPos(MousePos,True);
      Num2 := FDragItem;
      Num1:=lbItems.ItemAtPos(Point2,True);
      if (Num2 = -1) then
        Exit;

      if Num1 = -1 then
        Num1 := lbItems.Items.Count -1;
      lbItems.Items.Move(Num2, Num1);
      lbItems.Selected[Num1] := True;
    end;
  except
    bitfarmLog.Write(llNormal, ltException, 'lbItemsDragDrop', ClassName);
  end;
end;

procedure TfrmListItems.lbItemsDragOver(Sender, Source: TObject; X, Y: Integer;
    State: TDragState; var Accept: Boolean);
begin
  Accept := Source = lbItems;

  if y < 15 then
  begin
     fbscroll_down := True;
     FAutoscroll.Enabled := True;
  end
  else
  if (lbItems.Height - y) < 15 then
  begin
     fbscroll_down := False;
     FAutoscroll.Enabled := True;
  end
  else
     FAutoscroll.Enabled := False;

end;

procedure TfrmListItems.lbItemsKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Shift = [ssCtrl]) and (Key = Ord('A')) then
  begin
    lbItems.Items.BeginUpdate;
    lbItems.SelectAll;
    lbItems.Items.EndUpdate;
  end;

  //01.07.2024 SD -->bfacrm:ticket__show__38777
  if (Shift = [ssCtrl]) and (Key = Ord('C')) then
    bfaCopyToClipboard;
  //01.07.2024 SD <--bfacrm:ticket__show__38777
end;

procedure TfrmListItems.lbItemsKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_DELETE then
  begin
    if (lbItems.Items.Count > 0) and (lbItems.ItemIndex > -1) then
       bfaInputEdit1Delete(nil, lbItems.Items[lbItems.ItemIndex]);
    end;
  if Key = VK_ESCAPE then
    Self.ModalResult := mrCancel;
end;

procedure TfrmListItems.lbItemsMouseDown(Sender: TObject; Button: TMouseButton;
    Shift: TShiftState; X, Y: Integer);
begin
  MousePos.X := X;
  MousePos.Y := Y;
end;

procedure TfrmListItems.lbItemsStartDrag(Sender: TObject;
  var DragObject: TDragObject);
begin
  FDragItem := lbItems.ItemAtPos(MousePos,True);
end;

procedure TfrmListItems.SetEditable(const Value: Boolean);
begin
  FEditable := Value;

  btnOK.Enabled := FEditable;
  bfaInputEdit1.Editable := FEditable;
  btnSortUp.Enabled := FEditable;
  btnSortDown.Enabled := FEditable;
  btnLoadFromFile.Enabled := FEditable;

end;

function TfrmListItems.ShowStrictMessage(AValue: WideString): Boolean;
var
  lmq : TUniQuery;
begin
  Result := True;
  if not FStrictmode or (AValue = '') then
    Exit;

  lmq := DbCreateQuery;
  try
    try
      lmq.SQL.Text := 'select count(*) from sys_docidx_' + IntToStr(FAddIdx) + ' where ' + FAddName + ' = ' + #39 + AValue + #39;;
      lmq.Execute;
      if lmq.RecordCount > 0 then
      begin
        lmq.First;
        if lmq.Fields[0].AsInteger > 0 then
          Result := BfMessageDlg(
                     Format(_('Dieser Wert ist bei %s Dokumente(n) gesetzt. Passen Sie diese Dokumente bitte vorher\nentsprechend an, da dieser Wert sonst automatisch der Liste wieder hinzugefgt wird\nWollen Sie die nderung durchfhren? '),[lmq.Fields[0].AsString]),
                     mtConfirmation, [mbYes,mbNo], True) = mrYes;
      end;
    except

    end;
  finally
    lmq.Free;
  end;
end;

//01.07.2024 SD -->bfacrm:ticket__show__38777
procedure TfrmListItems.bfaCopyToClipboard;
var
  i: Integer;
  listAsText: String;
begin
  listAsText := '';
  for i := 0 to lbItems.Count -1 do
    if (lbItems.Selected[i] = True) and (Trim(lbItems.Items[i]) <> '')  then
      listAsText := listAsText + Trim(lbItems.Items[i]) + #13 + #10;
  if listAsText <> '' then Clipboard.AsText := listAsText;
end;

procedure TfrmListItems.btnSaveToFileClick(Sender: TObject);
var
  sl : TStringList;
  OpenDlg : TOpenDialog;
  fname, entry : string;
  i : Integer;
begin
  try
    try
      sl := TStringList.Create;
      for i := 0 to lbItems.Count -1 do
      begin
        entry := ParseNotAllowedChars(lbItems.Items[i]);
        if Trim(entry) <> '' then
          sl.Add(entry);
      end;
      entry := ParseNotAllowedChars(FCurrentField.DisplayName);
      entry := ReplaceStr(entry, ' ', '_');
      fname:= _('bfaAuswahlliste_') + entry + '.txt';
      if PromptForFileName(fname, _('Text-Dateien (*.TXT)|*.TXT'), 'txt',
        _('Die Auswahlliste speichern...'), '', True) then
      begin
        if fname <> '' then
        begin
          sl.SaveToFile(fname);
          BfShowMessage(_('Die Auswahlliste wird gespeichert.'), nil);
        end;
      end;
    except
      BfMessageDlg(_('Die Datei konnte nicht speichern werden: ') + fname, mtError, [mbOK], False, _(sSysError));
      bitfarmLog.Write(llNormal, ltException, 'btnSaveToFileClick - fname: ' + fname , ClassName);
    end;
  finally
    OpenDlg.Free;
    FreeAndNil(sl);
    lbItems.SetFocus;
  end;
end;
//01.07.2024 SD <--bfacrm:ticket__show__38777

end.
