unit LoggedinUserForm;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, SpTBXControls, Vcl.StdCtrls,
  SpTBXEditors, SpTBXItem;

type
  TfrmLoggedinUser = class(TForm)
    pnl1: TSpTBXPanel;
    lstUser: TSpTBXListBox;
    btnLogout: TSpTBXButton;
    btnClose: TSpTBXButton;
    procedure lstUserClick(Sender: TObject);
    procedure btnLogoutClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
    lstloggedon : TStringList;
    procedure FillList;
  end;

var
  frmLoggedinUser: TfrmLoggedinUser;

implementation

{$R *.dfm}

uses
  bfaLog, bfaBaseObjects, bfaCommonUtils, MainDataModule, bfaCommonDialogs, gnugettext;

procedure TfrmLoggedinUser.btnLogoutClick(Sender: TObject);
var
  req: TbfaRequest;
  userid : string;
begin
  bitfarmLog.DebugBegin(llDebug, 'btnLogoutClick', ClassName);
  try
    try
      if (lstUser.ItemIndex > -1) and (lstUser.Items.Objects[lstUser.ItemIndex] <> nil) then
      begin
        req := TbfaRequest.Create;
        try
          req.ReqType := 'reqLogoutUser';
          req.UserID := bitfarmApp.User.ID;
          req.Profile := bitfarmApp.Profile;
          userid := IntToStr(TbfaInt(lstUser.Items.Objects[lstUser.ItemIndex]).Value);
          req.Parameter := userid;
          req.Lang := bitfarmApp.Lang;
          bitfarmApp.MainSocket.SendRequestToServer(req);

          req := bitfarmApp.MainSocket.ReceiveResponse;
          if req = nil then
            Exit;

          if req.ReqType = 'respError' then
            BfShowMessage(_('Fehler:') + req.DataString, nil)
          else
          begin
            //lstUser.Items.Delete(lstUser.ItemIndex);
            lstloggedon.Delete(lstloggedon.IndexOf(userid));
            lstUser.Clear;
            FillList;
          end;
        finally
          FreeAndNil(req);
        end;
      end;
    except
     on e: exception do
       bitfarmLog.Write(llNormal, ltException, 'btnLogoutClick' + e.Message, ClassName);
    end;
  finally
   bitfarmLog.DebugEnd(llDebug, 'btnLogoutClick', ClassName);
  end;
end;

procedure TfrmLoggedinUser.FormCreate(Sender: TObject);
begin
  lstloggedon := TStringList.Create;
  TranslateComponent(Self, 'ViewerV36');
end;

procedure TfrmLoggedinUser.FormDestroy(Sender: TObject);
begin
  lstUser.Items.Clear;
  lstloggedon.Clear;
  FreeAndNil(lstloggedon);
end;

procedure TfrmLoggedinUser.lstUserClick(Sender: TObject);
begin
  btnLogout.Enabled := (lstUser.ItemIndex > -1) and (lstUser.Items.Objects[lstUser.ItemIndex] <> nil);
end;

procedure TfrmLoggedinUser.FillList;
var
  i, cntNamed, cntConcurrent: Integer;
begin
  cntNamed := 0;
  cntConcurrent := 0;
  for i := 0 to bitfarmApp.GUser.Count -1 do
  begin
    if lstloggedon.IndexOf(IntToStr(bitfarmApp.GUser[i].ID)) > -1 then
    begin
      if (bitfarmApp.GUser[i].Flags and 64) = 0 then
        inc(cntNamed);
      if (bitfarmApp.GUser[i].Flags and 64) = 64 then
        inc(cntConcurrent);
    end;
  end;
  lstUser.Items.Add(_('Angemeldete Benutzer (named):') + ' ' + IntToStr(cntNamed));


  for i := 0 to bitfarmApp.GUser.Count -1 do
  begin
    if lstloggedon.IndexOf(IntToStr(bitfarmApp.GUser[i].ID)) > -1 then
    begin
      if (bitfarmApp.GUser[i].Flags and 64 = 0) then
      begin
        if (bitfarmApp.GUser[i].Flags and 128  = 0) and (bitfarmApp.GUser[i].ID <> bitfarmApp.User.ID) then    // nicht concurrent und nicht dmsadmin
          lstUser.Items.AddObject(bitfarmApp.GUser[i].DisplayName, TbfaInt.Create(bitfarmApp.GUser[i].ID))
        else
          lstUser.Items.Add(bitfarmApp.GUser[i].DisplayName);

      end;
    end;
  end;

  if cntConcurrent > 0 then
  begin
    lstUser.Items.Add('');
    lstUser.Items.Add(_('Angemeldete Benutzer (concurrent):') + ' ' + IntToStr(cntConcurrent));

    for i := 0 to bitfarmApp.GUser.Count -1 do
    begin
      if lstloggedon.IndexOf(IntToStr(bitfarmApp.GUser[i].ID)) > -1 then
      begin
        if (bitfarmApp.GUser[i].Flags and 64 = 64) then
        begin
          if (bitfarmApp.GUser[i].ID <> bitfarmApp.User.ID)  then    // concurrent aber nicht dmsadmin
            lstUser.Items.AddObject(bitfarmApp.GUser[i].DisplayName, TbfaInt.Create(bitfarmApp.GUser[i].ID))
          else
            lstUser.Items.Add(bitfarmApp.GUser[i].DisplayName);
        end;
      end;
    end;
  end;
  btnLogout.Enabled := False;
end;

end.
