unit LoginGPL;

interface

uses
  Forms, Buttons, StdCtrls, Controls, jpeg, JvProgressBar, ExtCtrls,
  Graphics, Classes, bfaApplication, Registry, windows, Vcl.Imaging.pngimage,
  Vcl.Imaging.GIFImg, cxGraphics, cxControls, cxLookAndFeels, cxLookAndFeelPainters,
  cxContainer, cxEdit, cxTextEdit, cxMaskEdit, cxDropDownEdit, Win11Forms,
  dxGDIPlusClasses
  ;

type
  TfrmLoginGPL = class (TForm)
    imgBitfarmLogo: TImage;
    imgBitfarmArchive: TImage;
    lbl_User: TLabel;
    lbl_Pass: TLabel;
    lbl_Profil: TLabel;
    lbl_Version: TLabel;
    btn_Login: TButton;
    btn_Exit: TButton;
    shp_Rand: TShape;
    lbl_URL: TLabel;
    play_mod_login: TBitBtn;
    tbi_Optionen: TButton;
    edt_User: TcxTextEdit;
    edt_Pass: TcxTextEdit;
    cbx_Profil: TcxComboBox;
    cbx_login_optionen: TcxComboBox;
    im_enterprise: TImage;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure lbl_URLClick(Sender: TObject);
    procedure lbl_URLMouseEnter(Sender: TObject);
    procedure lbl_URLMouseLeave(Sender: TObject);
    procedure play_mod_loginClick(Sender: TObject);
    procedure tbi_OptionenClick(Sender: TObject);
    procedure btn_LoginClick(Sender: TObject);
    procedure edt_PassPropertiesChange(Sender: TObject);
    procedure edt_PassKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
  private
    { Private-Deklarationen }
    FSavePass, //SavePass in Mylogin.ini
    FConSavePass, //SavePass In Con Datei
    FPasswordError,
    FUserFocus,
    FShowOpt: boolean;
    FFirstLoginAtProgrammStart: Boolean;
    FLastLoggedInUser: string;
    FLastLoggedInUserPassword: string;

    procedure InitLoginProfiles;
    procedure SetWindowHeight;
    function GetAccountSid(const Server, User: WideString; var Sid: PSID): DWORD;  // Auslagern in Unit
    function GetCurrentSid(): string;                                              // Auslagern in Unit
  public
    { Public-Deklarationen }
    procedure TryLoginAsCurrentWindowsUser;
    procedure ChangeProfile(Profile: string);
    procedure InitLogin;
    procedure Retranslate;
    class procedure GetLoginProfiles(slConFiles: TStringList);

  protected
    procedure CreateParams(var Params: TCreateParams); override;

  end;

  function ConvertSidToStringSidW(Sid: PSID; var StringSid: LPCWSTR): Bool; stdcall; external advapi32 name 'ConvertSidToStringSidW';  //auslagern in Unit

implementation

uses
  SysUtils, IniFiles, ShellAPI, MainDataModule,
  bfaObjects36, gnugettext, bitfarm_core,
  bfaCommonUtils, CommonGUIUtils, bfaCommonMethods, bfaLog,
  bfaCommonFiles, Dialogs, CryptoUtils, StrUtils;

{$R *.dfm}

function TfrmLoginGPL.GetAccountSid(const Server, User: WideString; var Sid: PSID): DWORD;
var
  dwDomainSize, dwSidSize: DWord;
  R                 : LongBool;
  wDomain           : WideString;
  Use               : DWord;
begin
  Result := 0;
  SetLastError(0);
  dwSidSize := 0;
  dwDomainSize := 0;
  R := LookupAccountNameW(PWideChar(Server), PWideChar(User), nil, dwSidSize,
    nil, dwDomainSize, Use);
  if (not R) and (GetLastError = ERROR_INSUFFICIENT_BUFFER) then
  begin
    SetLength(wDomain, dwDomainSize);
    Sid := GetMemory(dwSidSize);
    R := LookupAccountNameW(PWideChar(Server), PWideChar(User), Sid,
      dwSidSize, PWideChar(wDomain), dwDomainSize, Use);
    if not R then
    begin
      FreeMemory(Sid);
      Sid := nil;
    end;
  end
  else
    Result := GetLastError;
end;

function TfrmLoginGPL.GetCurrentSid(): string;
Var
  Sid: PSID;
  strSID: PWideChar;
begin
  GetAccountSid(GetEnvironmentVariable('USERDOMAIN'), bd.EnvUsername, Sid);
  ConvertSidToStringSidW(SID, strSID);
  Result := strSID;
end;


procedure TfrmLoginGPL.TryLoginAsCurrentWindowsUser;
var
  ini: TIniFile;
  sUserName: string;
  sSectionUserEncrypted, sIdentPwdEncrypted: string;
  sProfile, sPassword: string;
  IsLoggedIn: Boolean;
begin
  try
    bitfarmLog.DebugBegin(llDebug, 'TfrmLoginEnterprise.TryLoginAsCurrentWindowsUser');

    FFirstLoginAtProgrammStart := true;
    cbx_login_optionen.ItemIndex := 0; // Einstellung: Passwort nicht speichern
    InitLoginProfiles;

    ini := TIniFile.Create(BD.MyLoginIniName);
    sUsername := bd.EnvUsername;
    sSectionUserEncrypted := Encrypt(lowercase(sUsername), GlobalKey);
    sIdentPwdEncrypted := Encrypt(sSysPassword, GlobalKey);
    sProfile := ini.ReadString(sSectionUserEncrypted, sSysProfile, '');
    if sProfile = '' then  // Falls Sid noch nicht in Ini voerhanden, nehmen wir das Profil aus Section [Users]
      sProfile := ini.ReadString(sSysUser, sSysProfile, '');

    sPassword := ini.ReadString(sSectionUserEncrypted, sIdentPwdEncrypted, '');

    if cbx_Profil.Properties.Items.IndexOf(sProfile) > -1 then
      cbx_Profil.ItemIndex := cbx_Profil.Properties.Items.IndexOf(sProfile);

    edt_User.Text := sUsername;
    edt_Pass.Text := AES128_Decrypt(sPassword, GlobalKey);

    // Versuch sich anzumelden
    if (cbx_Profil.Text <> '') and (edt_User.Text <> '') and (edt_Pass.Text <> '') then
      btn_LoginClick(Self);

    // Wenn nicht erfolgreich automatisch eingelogged, dann zeige Fenster
    IsLoggedIn := ModalResult = mrOk;
    if not IsLoggedIn then
    begin
      ShowModal;
    end;

    // Wenn erfolgreich eingelogged, dann speichere mir die Einlogdaten (Credentials)
    IsLoggedIn := ModalResult = mrOk;
    if IsloggedIn then
    begin
      ini.EraseSection(sSysUser); // Wir lschen bisherige Section 'User' (Sicherheitsgrnde nach RS mit Heino)

      ini.WriteString(sSectionUserEncrypted, sSysProfil, cbx_profil.Text);
      ini.WriteString(sSectionUserEncrypted, sIdentPwdEncrypted, AES128_Encrypt(edt_Pass.Text, GlobalKey));
      FFirstLoginAtProgrammStart := false;
    end;
    ini.Free;
    bitfarmLog.DebugEnd(llDebug, 'TfrmLoginEnterprise.TryLoginAsCurrentWindowsUser');
  except
    on e: Exception do
      bitfarmLog.Write(llNormal, ltException, 'TryLoginAsCurrentWindowsUser:' + e.Message, ClassName);
  end;
end;

procedure TfrmLoginGPL.ChangeProfile(Profile: string);
Var
  Index: integer;
  IsLoggedIn: Boolean;
begin
  Index := cbx_Profil.Properties.Items.IndexOf(Profile);
  if (Index >= 0) then
  begin
    cbx_Profil.Itemindex := Index;

    edt_User.Text := FLastLoggedInUser;
    edt_Pass.Text := FLastLoggedInUserPassword;

    if (cbx_Profil.Text <> '') and (edt_User.Text <> '') and (edt_Pass.Text <> '') then
      btn_LoginClick(Self);
  end;

  IsLoggedIn := ModalResult = mrOk;
  if not IsLoggedIn then
  begin
    ShowModal;
  end;
end;

procedure TfrmLoginGPL.Retranslate;
begin
  gnugettext.RetranslateComponent(self, 'ViewerV36');
end;

procedure TfrmLoginGPL.btn_LoginClick(Sender: TObject);
var
   ini: TIniFile;
   reg : TRegistry;
begin
  Screen.Cursor := crHourGlass;
  try
    ModalResult := mrNone;
    if bitfarmApp.Login(BD.ExeDir + cbx_Profil.Text + '.con', cbx_Profil.Text, edt_User.Text,
                 edt_Pass.Text, GlobalKey) then
    begin
      ModalResult := mrOk;

      FLastLoggedInUser := edt_User.Text;
      FLastLoggedInUserPassword := edt_Pass.Text;

      // Windows-integrierte Authentifizierung hlt den zuletzt angemeldeten User nicht mehr mit (Sicherheitsgrnde)
      if not bitfarmApp.ViewerOptions.GeneralOptions.MsLogon then
      begin
        ini := TIniFile.Create(BD.ExeDir + cbx_Profil.Text + '.con');
        try
            FConSavePass :=
              (LowerCase(ini.ReadString(sSysOptions, sSysSavePass, 'true')) = LowerCase('true'))
              or
              (LowerCase(ini.ReadString(sSysOptions, sSysSavePass, 'true')) = LowerCase('1'))
              or
              (ini.ReadString(sSysOptions, sSysSavePass, 'true') = '');
        finally
          FreeAndNil(ini);
        end;

        ini := TIniFile.Create(BD.MyLoginIniName);
        try
          try
            ini.WriteString(sSysUser, sSysProfil, cbx_profil.Text);
            ini.WriteString(sSysUser, sSysUserName, edt_User.Text);
            ini.WriteString(sSysUser, sSysSavePass, BoolToStr(FSavePass,True));
            if (FSavePass) and (FConSavePass) and
               ((LowerCase(edt_User.Text) <> LowerCase(sSysDMSAdmin)) or IsDebuggerPresent)
            then
              ini.WriteString(sSysUser, sSysPassword, Encrypt(edt_Pass.Text, GlobalKey))
            else
              ini.WriteString(sSysUser, sSysPassword, '');
            ini.UpdateFile;
          except
             on e: Exception do
                bitfarmLog.Write(llNormal, ltException, 'btn_LoginClick:' + e.Message, ClassName);

          end;
        finally
          if Assigned(ini) then
             FreeAndNil(ini);
        end;
      end;

      reg := TRegistry.Create;
      try
        try
           reg.RootKey := HKEY_CURRENT_USER;
           reg.Access := KEY_ALL_ACCESS;
           reg.OpenKey('\Software\bitfarm\Archiv_V3\Viewer\Login', True);
           reg.WriteString('User', edt_User.Text);
           reg.WriteString('WindowsUser', BD.EnvUsername);
           reg.WriteString('mylogin', BD.MyLoginIniName); //m.frey
           reg.WriteString('Conn', cbx_Profil.Text);
           reg.WriteString('Confile', BD.ExeDir + cbx_Profil.Text);
           reg.CloseKey;

           reg.OpenKey('Software\bitfarm\Archiv_V3\Viewer\Optionen', True);
           reg.WriteString('ViewerFilePath', BD.ExeDir);
           reg.CloseKey;
        except
           on e: Exception do
              bitfarmLog.Write(llNormal, ltException, 'btn_LoginClick:' + e.Message, ClassName);
        end;
      finally
        FreeAndNil(reg);
      end;
    end
    else
      edt_Pass.SelectAll;
//  SJ 20.01.2020 - auf Heinos Wunsch nicht mehr leer setzen - das strt bei nicht erreichbaren Servern nur
//      edt_Pass.Text := '';
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TfrmLoginGPL.FormCreate(Sender: TObject);
begin
  RoundedCorners := rcOn; //03.06.2022 Win11

  FLastLoggedInUser := '';
  FLastLoggedInUserPassword := '';

  SetWindowHeight;

  try
    SpDxGetTextInitialize(bitfarmApp.Lang, '', [Self], True, True);
  except

  end;

  FPasswordError := False;
  FUserFocus := False;
  FFirstLoginAtProgrammStart := false;

  cbx_login_optionen.Clear;
  if not bitfarmApp.ViewerOptions.GeneralOptions.MsLogon then
  begin
    cbx_login_optionen.Properties.Items.AddObject(_('Kein Passwort speichern'), TObject(0));
    cbx_login_optionen.Properties.Items.AddObject(_('Passwort speichern'), TObject(1));
  end;
  cbx_login_optionen.Properties.Items.AddObject(_('Ansicht auf Werkseinstellungen zurcksetzen'), TObject(2));
//    cbx_login_optionen.AddItem(_('Kein Passwort speichern'), TObject(0));
//    cbx_login_optionen.AddItem(_('Passwort speichern'), TObject(1));
//  end;
//  cbx_login_optionen.AddItem(_('Ansicht auf Werkseinstellungen zurcksetzen'), TObject(2));
end;

procedure TfrmLoginGPL.tbi_OptionenClick(Sender: TObject);
begin
  FShowOpt := not FShowOpt;
  SetWindowHeight;
end;

procedure TfrmLoginGPL.SetWindowHeight;
begin
  if FshowOpt then
  begin
    ClientHeight := cbx_login_optionen.Top + cbx_login_optionen.Height + 8;
    cbx_login_optionen.Visible := True;
    play_mod_login.Visible := True;
  end
  else
  begin
    ClientHeight := cbx_login_optionen.Top - 2;
    cbx_login_optionen.Visible := False;
    play_mod_login.Visible := False;
  end;
end;

procedure TfrmLoginGPL.FormShow(Sender: TObject);
begin
  bitfarmLog.DebugBegin(llDebug, 'TfrmLoginEnterprise.FormShow');
  try
    cbx_login_optionen.ItemIndex := 0;

    if bitfarmApp.ViewerOptions.GeneralOptions.MsLogon then
    begin
      if not FFirstLoginAtProgrammStart then  // Nur beim Relogin
      begin
        edt_User.Text := bd.EnvUsername;
        edt_Pass.Text := '';
      end;
    end
    else
      InitLogin;

    lbl_Version.Caption := sSysVersion + ': ' + BfaGetVersion(Application.ExeName);
    lbl_URL.Caption := GetBitfarmURL;
    if FUserFocus or (edt_User.Text = '') then
    begin
      edt_User.SetFocus;
      edt_User.SelectAll;
    end
    else begin
      edt_Pass.SetFocus;
      edt_Pass.SelectAll;
    end;

    edt_Pass.Properties.ShowPasswordRevealButton := edt_Pass.Text = '';

    FUserFocus := True;
    FShowOpt := True;
    tbi_OptionenClick(Self);
    if FileExists(ExtractFilePath(Application.ExeName) + '\Viewer-files\SplashForm.gif')
    then
    begin
      imgBitfarmArchive.Picture.LoadFromFile(ExtractFilePath(Application.ExeName) +
        '\Viewer-files\SplashForm.gif');
      (imgBitfarmArchive.Picture.Graphic as TGIFImage).AnimationSpeed := 100;
      (imgBitfarmArchive.Picture.Graphic as TGIFImage).Animate := True;
      Self.DoubleBuffered := True;
    end;
  finally
    bitfarmLog.DebugEnd(llDebug, 'TfrmLoginEnterprise.FormShow');
  end;
end;

class procedure TfrmLoginGPL.GetLoginProfiles(slConFiles: TStringList);
var
  sFiles: TStrings;
  i, iCount: integer;
  ini : TIniFile;
  sFileName: string;
begin
  sFiles := TStringList.Create;
  try
    FindAllFiles(sFiles, BD.ExeDir, '*.con', False);
    for iCount := 0 to (sFiles.Count - 1) do
    begin
      ini := TIniFile.Create(sFiles[iCount]);
      try
        if ini.ValueExists(sSysServerSection, sSysBfaServerVersion) then
        begin
          if ini.ReadString(sSysServerSection, sSysBfaServerVersion, '') = '36' then
          begin
            sFileName := ExtractFileName(sFiles[iCount]);
            slConFiles.Add(copy(sFileName, 1, (Length(sFileName) - 4)));
          end;
        end;
      finally
        ini.Free;
      end;
    end;
  except
   on e: Exception do
    begin
      bitfarmLog.Write(llNormal, ltException, 'TfrmLoginEnterprise.InitLoginProfiles', ClassName);
    end;
  end;
  sFiles.Free;
end;

procedure TfrmLoginGPL.InitLoginProfiles;
var
  sFiles : TStrings;
  iCount : integer;
  ini : TIniFile;
  sFileName: string;
  slConFiles : TStringList;

begin
  try
    bitfarmLog.DebugBegin(llDebug, ltInfo, 'TfrmLoginEnterprise.InitLoginProfiles');
    sFiles := TStringList.Create;
    slConFiles := TStringList.Create;

    try
      GetLoginProfiles(slConFiles);

      cbx_Profil.Properties.Items.Clear;
      if slConFiles.Count > 0 then
      begin
        cbx_Profil.Properties.Items := slConFiles;
        cbx_Profil.Text := slConFiles[0];
      end;

    except
      on e: Exception do
      begin
        bitfarmLog.Write(llNormal, ltException, 'TfrmLoginEnterprise.InitLoginProfiles', ClassName);
      end;
    end;
  finally
    bitfarmLog.DebugEnd(llDebug, ltInfo, 'TfrmLoginEnterprise.InitLoginProfiles');
    if Assigned(slConFiles) then
      slConFiles.Free;
    if Assigned(sFiles) then
      sFiles.Free;
  end;
end;

procedure TfrmLoginGPL.InitLogin;
var
  sFiles : TStrings;
  lConFiles : TStringList;
  sFileName, slpass, sProfile : string;
  iCount : integer;
  ini : TIniFile;
begin
  bitfarmLog.DebugBegin(llDebug, ltInfo, 'TfrmLoginEnterprise.InitLogin');

  try
    try
      InitLoginProfiles; // ausgelagerter Code

      ini := TIniFile.Create(BD.MyLoginIniName);

      if ini.ValueExists(sSysUser, sSysUserName) then
      begin
        slpass := ini.ReadString(sSysUser, sSysUserName, '');
        if slpass <> '' then
           edt_User.Text := slpass;
      end;
      FSavePass :=
        (LowerCase(ini.ReadString(sSysUser, sSysSavePass, 'false')) = 'true')
        or
        (LowerCase(ini.ReadString(sSysUser, sSysSavePass, 'false')) = '1');

      edt_Pass.Text := '';

      if FPasswordError then
        edt_Pass.Text := ''
      else
      begin
        if ini.ValueExists(sSysUser, sSysPassword) and FSavePass and
           ((LowerCase(edt_User.Text) <> LowerCase(sSysDMSAdmin)) or IsDebuggerPresent) then
        begin
          slpass := ini.ReadString(sSysUser, sSysPassword, '');
          if slpass <> '' then
            edt_Pass.Text := Decrypt(slpass, GlobalKey);
        end;
      end;

      // Profil-Einstellung
      sProfile := ini.ReadString(sSysUser, sSysProfile, '');
      if cbx_Profil.Properties.Items.IndexOf(sProfile) > -1 then
         cbx_Profil.ItemIndex := cbx_Profil.Properties.Items.IndexOf(sProfile);

    except
      on e: Exception do
      begin
        bitfarmLog.Write(llNormal, ltException, 'TfrmLoginEnterprise.InitLogin', ClassName);
      end;
    end;
  finally
    bitfarmLog.DebugEnd(llDebug, ltInfo, 'TfrmLoginEnterprise.InitLogin');
    if Assigned(ini) then
      FreeAndNil(ini);
  end;
end;

procedure TfrmLoginGPL.lbl_URLClick(Sender: TObject);
begin
  ShellExecute(0, nil, PChar('https://www.bitfarm-archiv.de/'), PChar(''),
    PChar(''), SW_NORMAL);
end;

procedure TfrmLoginGPL.lbl_URLMouseEnter(Sender: TObject);
begin
  lbl_Url.Font.Color := clSkyBlue;
end;

procedure TfrmLoginGPL.lbl_URLMouseLeave(Sender: TObject);
begin
  lbl_URL.Font.Color := gclBlue;
end;

procedure TfrmLoginGPL.play_mod_loginClick(Sender: TObject);
var
  ini: TIniFile;
begin
  case Integer(cbx_login_optionen.Properties.Items.Objects[cbx_login_optionen.ItemIndex]) of
    //dont save password
    0:
    begin
      try
        FSavePass := False;
        ini := TIniFile.Create(BD.MyLoginIniName);
        ini.WriteString('User', 'Profil', cbx_profil.Text);
        ini.WriteString('User', 'SavePass', BoolToStr(FSavePass, True));
        ini.WriteString('User', 'Password', '');
        ini.UpdateFile;
      finally
        if Assigned(ini) then
           FreeAndNil(ini);
      end;
    end;
    // save password
    1:
    begin
      FSavePass := True;
    end;
    2: //Ansicht reset
    begin
      bitfarmLog.DebugBegin(llDebug, 'play_mod_loginClick - DeleteFile', ClassName);
      try
        if FileExists(BD.AppdataBitfarmPd + 'ViewerLayout.ini') then
          DeleteFile(PWideChar(BD.AppdataBitfarmPd + 'ViewerLayout.ini'));
        if FileExists(BD.AppdataBitfarmPd + 'Viewer.ini') then
          DeleteFile(PWideChar(BD.AppdataBitfarmPd + 'Viewer.ini'));
        if FileExists(BD.AppdataBitfarmPd + 'ViewerOptions.ini') then
          DeleteFile(PWideChar(BD.AppdataBitfarmPd + 'ViewerOptions.ini'));
        if FileExists(BD.AppdataBitfarmPd + 'TreeView.ini') then
          DeleteFile(PWideChar(BD.AppdataBitfarmPd + 'TreeView.ini'));
        if FileExists(BD.AppdataBitfarmPd + 'layout.ini') then
          DeleteFile(PWideChar(BD.AppdataBitfarmPd + 'layout.ini'));
        if FileExists(BD.AppdataBitfarmPd + 'MainForm.ini') then
          DeleteFile(PWideChar(BD.AppdataBitfarmPd + 'MainForm.ini'));
        if FileExists(BD.AppdataBitfarmPd + 'Sortings.ini') then
          DeleteFile(PWideChar(BD.AppdataBitfarmPd + 'Sortings.ini'));
      except
        bitfarmLog.Write(llNormal, ltException, 'play_mod_loginClick - DeleteFile', ClassName);
      end;
      bitfarmLog.DebugEnd(llDebug, 'play_mod_loginClick - DeleteFile', ClassName);
    end;
    //end of case of
  end;
end;

procedure TfrmLoginGPL.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.ExStyle := Params.ExStyle or WS_EX_APPWINDOW;
  Params.WndParent := 0;
end;

procedure TfrmLoginGPL.edt_PassKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if Key = VK_ESCAPE then   // verhindert, dass ein eingetragenes Passwort entfernt und per ESC zurckgeholt
    Key := 0;               // und eingesehen werden kann
end;

procedure TfrmLoginGPL.edt_PassPropertiesChange(Sender: TObject);
begin
  if edt_Pass.Text = '' then             // sollte der text geleert worden sein, blenden wir den RevealButton ein
    edt_Pass.Properties.ShowPasswordRevealButton := True;
end;

end.

