unit MainDataModule;

interface

uses
  SysUtils, Classes, UniProvider, MySQLUniProvider, DB, DBAccess, Uni, bfaIconsDM,
  DASQLMonitor, UniSQLMonitor, bfaApplication, {$IFDEF ENTERPRISE} bfaApplicationEnterprise, {$ENDIF} bfaLog,
  SpTBXItem, TB2Item, Menus, MemData{, CRVio, CRSSLIOHandler, ScBridge};

type
  TdmMain = class(TDataModule)
    cntMain: TUniConnection;
    trnMain: TUniTransaction;
    cntSearch: TUniConnection;
    monUniSQL: TUniSQLMonitor;
    cntBackThread: TUniConnection;
    cntResubSearch: TUniConnection;
    procedure cntMainError(Sender: TObject; E: EDAError; var Fail: Boolean);
    procedure cntSearchError(Sender: TObject; E: EDAError; var Fail: Boolean);
    procedure cntBackThreadError(Sender: TObject; E: EDAError;
      var Fail: Boolean);
    procedure cntMainConnectionLost(Sender: TObject; Component: TComponent;
      ConnLostCause: TConnLostCause; var RetryMode: TRetryMode);
    procedure cntSearchConnectionLost(Sender: TObject; Component: TComponent;
      ConnLostCause: TConnLostCause; var RetryMode: TRetryMode);
    procedure cntBackThreadConnectionLost(Sender: TObject;
      Component: TComponent; ConnLostCause: TConnLostCause;
      var RetryMode: TRetryMode);
    procedure cntResubSearchConnectionLost(Sender: TObject;
      Component: TComponent; ConnLostCause: TConnLostCause;
      var RetryMode: TRetryMode);
    procedure cntResubSearchError(Sender: TObject; E: EDAError;
      var Fail: Boolean);
  private
    { Private declarations }
//10.04.2024    procedure HandleMySQLError;
    procedure HandleMySQLError(cnt: TUniConnection);
  public
    { Public declarations }
  end;

var
  dmMain: TdmMain;

  {$IFNDEF ENTERPRISE}
    bitfarmApp : TbfaApplication;
  {$ELSE}
    bitfarmApp : TbfaApplicationEnterprise;
  {$ENDIF}

implementation

{$R *.dfm}

uses
  Controls, Windows, bfaCommonUtils, gnugettext, MainForm;

procedure TdmMain.cntBackThreadConnectionLost(Sender: TObject;
  Component: TComponent; ConnLostCause: TConnLostCause;
  var RetryMode: TRetryMode);
begin
  bitfarmLog.Write(llNormal, ltInfo, 'cntBackThreadConnectionLost', ClassName);
  try
    cntBackThread.Ping;
  except
//10.04.2024    HandleMySQLError;
    HandleMySQLError(cntBackThread);
  end;
end;

procedure TdmMain.cntBackThreadError(Sender: TObject; E: EDAError;
  var Fail: Boolean);
begin
  bitfarmLog.Write(llNormal, ltInfo, 'cntBackThread:' + E.Message, ClassName);
  try
    cntMain.Ping;
  except
//10.04.2024    HandleMySQLError;
    HandleMySQLError(cntMain);
  end;
end;

procedure TdmMain.cntMainConnectionLost(Sender: TObject; Component: TComponent;
  ConnLostCause: TConnLostCause; var RetryMode: TRetryMode);
begin
  bitfarmLog.Write(llNormal, ltInfo, 'cntMainConnectionLost', ClassName);
  try
    cntMain.Ping;
  except
//10.04.2024    HandleMySQLError;
    HandleMySQLError(cntMain);
  end;
end;

procedure TdmMain.cntMainError(Sender: TObject; E: EDAError; var Fail: Boolean);
begin
  bitfarmLog.Write(llNormal, ltInfo, 'cntMain:' + E.Message, ClassName);
  try
    cntMain.Ping;
  except
//10.04.2024    HandleMySQLError;
    HandleMySQLError(cntMain);
  end;
end;

procedure TdmMain.cntSearchConnectionLost(Sender: TObject;
  Component: TComponent; ConnLostCause: TConnLostCause;
  var RetryMode: TRetryMode);
begin
  bitfarmLog.Write(llNormal, ltInfo, 'cntSearchConnectionLost', ClassName);
  try
    cntSearch.Ping;
  except
//10.04.2024    HandleMySQLError;
    HandleMySQLError(cntSearch);
  end;
end;

procedure TdmMain.cntSearchError(Sender: TObject; E: EDAError;
  var Fail: Boolean);
begin
  bitfarmLog.Write(llNormal, ltInfo, 'cntSearch:' + E.Message, ClassName);
  try
    cntSearch.Ping;
  except
//12.04.2018 ab Version 3.6.0.1019 eingebaut, aber wenn man die Suche gecancelt hat,
//           dann muss man sich neu anmelden   HandleMySQLError;
  end;
end;

//10.04.2024 procedure TdmMain.HandleMySQLError;
procedure TdmMain.HandleMySQLError(cnt: TuniConnection);
var
  mdresult: TModalResult;
  hd: HWND;
  UserAction : PUserAction;
begin
  if Assigned(frmMainForm) then
  begin
    if frmMainForm.FShowMySQLError then
      Exit;

    bitfarmLog.Write(llNormal, ltInfo, 'HandleMySQLError', ClassName);
    bitfarmLog.Write(llNormal, ltInfo, 'Try to Reconnect: ' + cnt.Name, ClassName);
    try
      if cnt.Connected then // wenn schon verbunden war, bleibt True, dann kann man nochmal verbinden, sonst muss neu anmelden
        cnt.Connect;

      if cnt.Connected then
        Exit;
    finally
      if cnt.Connected then
        bitfarmLog.Write(llNormal, ltInfo, 'Reconnected: ' + cnt.Name, ClassName)
      else
        bitfarmLog.Write(llNormal, ltInfo, 'Reconnect not possible: ' + cnt.Name, ClassName);

    end;

    frmMainForm.FShowMySQLError := True;
    try
      bitfarmLog.Write(llNormal, ltInfo, 'my_conError', ClassName);

      bitfarmLog.DebugBegin(llDebug, 'HandleMySQLError - ', Classname);
      try
        try
          UserAction := bitfarmApp.CreateUserAction;
          UserAction^.ActionName := 'actReloginMySQLError';
          bitfarmApp.MainThread.Add(UserAction);
        except
          bitfarmLog.Write(llNormal, ltException, 'HandleMySQLError - ', ClassName);
        end;
      finally
        bitfarmLog.DebugEnd(llDebug, 'HandleMySQLError - ', Classname);
      end;
    finally
//      frmMainForm.FShowMySQLError := False;
    end;
  end;
end;

procedure TdmMain.cntResubSearchConnectionLost(Sender: TObject;
  Component: TComponent; ConnLostCause: TConnLostCause;
  var RetryMode: TRetryMode);
begin
  bitfarmLog.Write(llNormal, ltInfo, 'cntResubSearchConnectionLost', ClassName);
  try
    cntResubSearch.Ping;
  except
//10.04.2024    HandleMySQLError;
    HandleMySQLError(cntResubSearch);
  end;
end;

procedure TdmMain.cntResubSearchError(Sender: TObject; E: EDAError;
  var Fail: Boolean);
begin
  bitfarmLog.Write(llNormal, ltInfo, 'cntResubSearch:' + E.Message, ClassName);
  try
    cntResubSearch.Ping;
  except
//12.04.2018 ab Version 3.6.0.1019 eingebaut, aber wenn man die Suche gecancelt hat,
//           dann muss man sich neu anmelden   HandleMySQLError;
  end;
end;

end.
