unit MoveDocForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, SpTBXControls, u_bfaScrollBox, SpTBXItem, ExtCtrls, bfaObjects36,
  bfaApplication, Generics.Collections, gnugettext, cxGraphics, cxControls,
  cxLookAndFeels, cxLookAndFeelPainters, cxScrollBox;

type
  TfrmMoveDoc = class(TForm)
    Panel1: TPanel;
    grpIndexing: TSpTBXGroupBox;
    btnIndexing: TSpTBXButton;
    btnIndexingReset: TSpTBXButton;
    scrbxMoveDocAddfields: TbfaScrollBox;
    btnMoveDoc: TSpTBXButton;
    btnCancel: TSpTBXButton;
    procedure FormShow(Sender: TObject);
    procedure btnIndexingClick(Sender: TObject);
    procedure btnIndexingResetClick(Sender: TObject);
    procedure btnMoveDocClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure CreateAddfields(id: Integer);
  private
    { Private declarations }

  public
    { Public declarations }
    Doc: TbfaDocument;
    ArcID : Integer;
    TmpAddfields : TList<TbfaAddField>;
    TmpCurrentFields: TList<TbfaAddField>;
    procedure DeleteAllAddfields;
  end;

implementation

uses
  Uni, u_bfaAddfield, MainDataModule, MainForm, bfaIconsDM, bfaCommonUtils, bfaLog;

{$R *.dfm}

procedure TfrmMoveDoc.btnIndexingClick(Sender: TObject);
var
  so : TbfaSearch;
  add : TbfaAddField;
  searchproc: TbfaSearchProcessing;
  dsc : TUniDataSource;
  i : Integer;
  IdxChk : Boolean;
  ErrorStr: string;
begin
  so := TbfaSearch.Create;
  searchproc := TbfaSearchProcessing.Create;
  dsc := TUniDataSource.Create(nil);

  so.UserContext := bitfarmApp.User.ID;
  so.SearchType := 5;
  so.Direction := 'ASC';
  so.UseSearch35 := bitfarmApp.FUseSearch35;// 11.01.2019 Ticket 15015, 17504   
  
  try
    IdxChk := False;
    for i := 0 to TmpAddfields.Count -1 do
    begin
      if TmpAddfields[i].Component.chkIndexing.Checked then
      begin
        add := TbfaAddField.Create;
        add.Name := TmpAddfields[i].Component.Fieldname;
        add.DisplayName := TmpAddfields[i].Component.Dispname;
        add.Value := TmpAddfields[i].Component.Value;
        add.Modified := True;
        add.IndexTableID := TmpAddfields[i].Component.IndexTableID;
        add.Typ := Ord(TmpAddfields[i].Component.AddfieldType);
        so.AddFields.Add(add);
        IdxChk := True;
      end else
      begin
         if VarIsNull(so.Order) then
           so.Order := TmpAddfields[i].Component.Fieldname
         else
           so.Order := so.Order + ',' + TmpAddfields[i].Component.Fieldname;
         if VarIsNull(so.SelectedCols) then
           so.SelectedCols := TmpAddfields[i].Component.Fieldname + ':' +
            IntToStr(TmpAddfields[i].Component.IndexTableID)
         else
           so.SelectedCols := so.SelectedCols + ',' + TmpAddfields[i].Component.Fieldname + ':' +
            IntToStr(TmpAddfields[i].Component.IndexTableID);
//         TmpAddfields[i].Component.Indexing := True;

      end;
    end;

    if IdxChk then
    begin
      dsc.DataSet := searchproc.Search(dmMain.cntSearch, so, nil, ErrorStr);

      if dsc.DataSet <> nil then
      begin
        for i := 0 to TmpAddfields.Count -1 do
        begin
          if not TmpAddfields[i].Component.chkIndexing.Checked then
          begin
             TmpAddfields[i].Component.ShowIndexingCombo(True);
             TmpAddfields[i].Component.cbxIndexing.Properties.Items.BeginUpdate;
             try
               TmpAddfields[i].Component.cbxIndexing.Properties.Items.Clear;
               dsc.DataSet.First;
               while not dsc.DataSet.Eof do
               begin
                  if dsc.DataSet.FieldByName(TmpAddfields[i].Name).AsString <> '' then
//10.10.2017 wird DISTINCT gesucht, deswegen brauchen wir das nicht                     if TmpAddfields[i].Component.cbxIndexing.Items.IndexOf(dsc.DataSet.FieldByName(TmpAddfields[i].Name).AsString) = -1 then
                        TmpAddfields[i].Component.cbxIndexing.Properties.Items.Add(dsc.DataSet.FieldByName(TmpAddfields[i].Name).AsString);
                  dsc.DataSet.Next;
               end;
             finally
               TmpAddfields[i].Component.cbxIndexing.Properties.Items.EndUpdate;
             end;
             if TmpAddfields[i].Component.cbxIndexing.Properties.Items.Count = 1 then
             begin
               TmpAddfields[i].Component.cbxIndexing.ItemIndex := 0;
               TmpAddfields[i].Component.cbxIndexing.Properties.OnChange(nil);
             end else
               TmpAddfields[i].Component.cbxIndexing.ItemIndex := -1;
          end;
        end;
        scrbxMoveDocAddfields.Update;
      end;
    end;
  finally
    FreeAndNil(so);
    FreeAndNil(searchproc);
    FreeAndNil(dsc);
  end;
end;

procedure TfrmMoveDoc.btnIndexingResetClick(Sender: TObject);
var
  i : Integer;
begin
  try
    for i := 0 to TmpAddfields.Count -1 do
    begin
      if TmpAddfields[i].Component <> nil then
      begin
        TmpAddfields[i].Component.Reset;
        TmpAddfields[i].Component.ClearValue;
        TmpAddfields[i].Component.OnValueSet := TmpAddfields[i].SetValue;
      end;
      TmpAddfields[i].Modified := False;
      TmpAddfields[i].ModifiedSearch := False;
      TmpAddfields[i].Value := '';
      TmpAddfields[i].OldValue := '';
    end;

    scrbxMoveDocAddfields.Update;
  finally

  end;
end;

procedure TfrmMoveDoc.btnMoveDocClick(Sender: TObject);
begin
  try
    if bitfarmApp.CheckAddFields(nil, Doc, TmpAddfields, True) then
      ModalResult := mrOk
    else
      ModalResult := mrNone;
  finally

  end;
end;

procedure TfrmMoveDoc.FormCreate(Sender: TObject);
begin
  TmpAddfields := TList<TbfaAddfield>.Create;
  TmpCurrentFields := TList<TbfaAddfield>.Create;

  TranslateComponent(Self, 'ViewerV36');
end;

//21.09.2022 Ticket 32291 -->
procedure TfrmMoveDoc.DeleteAllAddfields;
begin
  bitfarmLog.DebugBegin(llDebug, 'TfrmMoveDoc.DeleteAllAddfields', ClassName);
  try
    if TmpAddfields.Count > 0 then
    begin
      for var i := 0 to TmpAddfields.Count - 1 do
      begin
        try
          var comp : TComponent;
          comp := TmpAddfields[i].Component;
          comp := nil;
          try
            if TmpAddfields[i].Component <> nil then
            begin
              TmpAddfields[i].Component.Free;
              TmpAddfields[i].Component := nil;
            end;
          finally
            TmpAddfields[i].Component := nil;
          end;
        except
          bitfarmLog.Write(llNormal, ltException, 'DeleteAddfields - Free Comp', ClassName);
        end;
      end;
    end;

    if scrbxMoveDocAddfields.ComponentCount > 0 then
    begin
      for var i := scrbxMoveDocAddfields.ComponentCount - 1 downto 0 do
        scrbxMoveDocAddfields.Components[i].Free;
    end;
  finally
    bitfarmLog.DebugEnd(llDebug, 'TfrmMoveDoc.DeleteAllAddfields', ClassName);
  end;
end;
//21.09.2022 Ticket 32291 <--

procedure TfrmMoveDoc.CreateAddfields(id: Integer);
var
  i, j, icount, lastpos : Integer;
  AddField : TbfaAddfield;
  addfieldcomp : TBfaAddfieldComp;
  addid : Integer;
  qry : TUniQuery;
  orderby : string;
begin
  bitfarmLog.DebugBegin(llDebug, 'TfrmMoveDoc.CreateAddfields', ClassName);
  try
    Arcid := id;
    qry := TUniQuery.Create(nil);
    try
      orderby := ' order by pos, displayname';
      qry.Connection := dmMain.cntMain;
//14.08.2024      qry.SQL.Text := 'SELECT distinct(sys_addfieldxs.fieldid) FROM sys_addfieldxs ' +
      qry.SQL.Text := 'SELECT distinct(sys_addfieldxs.fieldid), pos, displayname FROM sys_addfieldxs ' +
                      'left join sys_addfields on sys_addfieldxs.fieldid = sys_addfields.addfieldid ' +
                      'where arcid in ('+ IntToStr(ArcID) + ') ' +
                      'and (userid in (' + bitfarmApp.User.HolidayList.CommaText + ') ' +
                      ' or (userid = 0 and groupid in (' + bitfarmApp.User.GroupList.CommaText +
                      '))) and ((sys_addfields.flags & 2) = 0)' + orderby ;

      qry.Execute;
      if qry.RecordCount > 0 then
      begin
  //      SetLength(TmpAddfields, qry.RecordCount);
        TmpAddfields.Clear;
        scrbxMoveDocAddfields.AutoScroll := False;

        qry.First;
        i := 0;
        lastpos := 0;
        while not qry.Eof do
        begin
          addid := qry.FieldByName('fieldid').AsInteger;
          addfieldcomp := bitfarmApp.DoCreateAddfieldComp(scrbxMoveDocAddfields,
            addid, i, True, lastpos, False);
          AddField := TbfaAddField.Create;
          AddField.Clone(aAddfields[addid]);
          TmpAddfields.Add(AddField);
          if addfieldcomp <> nil then
          begin
            AddField.Component := addfieldcomp;
            AddField.Component.EditMode := True; //08.02.2019 Ticket 17565

//11.09.2017            if (Addfield.Component.Rights and 4 = 4) then   //nderungen nur bei Modify
            if (Addfield.Component.Rights and 6 > 0) then   //nderungen nur bei Modify und Schreiben wenn noch leer ist 11.09.2017
            begin
              if AddField.Component.AddfieldType = zfld13_Grid then
                AddField.Component.compGrid.OnValueSet := AddField.SetValue
              else
                AddField.Component.OnValueSet := AddField.SetValue;
            end;
            if AddField.Typ in [2,3,11] then //14.03.2018 wegen die Neue Auswahlfelder
              addfieldcomp.Reset;

            // Mehrfachauswahl Pflichtfeld? "" entfernen
            if (AddField.Component.AddfieldType = zfld03_MehrfachAuswahl) and
               ((AddField.Options and cAddfieldOptMandatory) = cAddfieldOptMandatory) then
            begin
              while (TbfaCheckedListBox(AddField.Component.comp2).Items.IndexOf('""') <> - 1) do
              begin
                var idx: Integer;
                idx := TbfaCheckedListBox(AddField.Component.comp2).Items.IndexOf('""');
                TbfaCheckedListBox(AddField.Component.comp2).Items.Delete(idx);
              end;
              while (TbfaCheckedListBox(AddField.Component.comp2).Items.IndexOf('') <> - 1) do
              begin
                var idx: Integer;
                idx := TbfaCheckedListBox(AddField.Component.comp2).Items.IndexOf('');
                TbfaCheckedListBox(AddField.Component.comp2).Items.Delete(idx);
              end;
            end;
          end;

          inc(i);
          qry.Next;
        end;

        scrbxMoveDocAddfields.AutoScroll := True;
        scrbxMoveDocAddfields.AfterScrollVert := frmMainForm.AfterScrollVert;
        scrbxMoveDocAddfields.AfterScrollHorz := frmMainForm.AfterScrollHorz;
  //      scrbxMoveDocAd.Show;

      end;
    finally
      FreeAndNil(qry);
    end;
  finally
    bitfarmLog.DebugEnd(llDebug, 'TfrmMoveDoc.CreateAddfields', ClassName);
  end;
end;

procedure TfrmMoveDoc.FormShow(Sender: TObject);
var
  i, icount : Integer;
  AddField : TbfaAddfield;
  addfieldcomp : TBfaAddfieldComp;
  addid : Integer;
  qry : TUniQuery;
  orderby : string;
begin
//  qry := TUniQuery.Create(nil);
//  try
//
//    orderby := ' order by pos';
//    qry.Connection := dmMain.cntMain;
//    qry.SQL.Text := 'SELECT distinct(sys_addfieldxs.fieldid) FROM sys_addfieldxs ' +
//                    'left join sys_addfields on sys_addfieldxs.fieldid = sys_addfields.addfieldid ' +
//                    'where arcid in ('+ IntToStr(ArcID) + ') ' +
//                    'and ((userid = 0 and groupid = 0) or userid = ' + InttoStr(bitfarmApp.User.ID) +
//                    ' or groupid in (' + bitfarmApp.User.GroupList.CommaText +
//                    ')) and ((sys_addfields.flags & 2) = 0)' + orderby ;
//
//    qry.Execute;
//    if qry.RecordCount > 0 then
//    begin
////      SetLength(TmpAddfields, qry.RecordCount);
//      TmpAddfields.Clear;
//      scrbxMoveDocAddfields.AutoScroll := False;
//
//      qry.First;
//      i := 0;
//      while not qry.Eof do
//      begin
//        addid := qry.FieldByName('fieldid').AsInteger;
//        addfieldcomp := bitfarmApp.DoCreateAddfieldComp(scrbxMoveDocAddfields,
//          addid, i, True, False);
//        AddField := TbfaAddField.Create;
//        AddField.Clone(aAddfields[addid]);
//        TmpAddfields.Add(AddField);
//        if addfieldcomp <> nil then
//        begin
//          AddField.Component := addfieldcomp;
//          if (Addfield.Component.Rights and 4 = 4) then   //nderungen nur bei Modify
//            AddField.Component.OnValueSet := AddField.SetValue;
//        end;
//
//        inc(i);
//        qry.Next;
//      end;
//
//      scrbxMoveDocAddfields.AutoScroll := True;
//      scrbxMoveDocAddfields.AfterScrollVert := frmMainForm.AfterScrollVert;
//      scrbxMoveDocAddfields.AfterScrollHorz := frmMainForm.AfterScrollHorz;
//      scrbxMoveDocAddfields.Show;
//
//    end;
//  finally
//    FreeAndNil(qry);
//  end;


  scrbxMoveDocAddfields.Show;

end;

end.
