unit SplashForm;

interface

uses
  Forms, StdCtrls, ExtCtrls, Classes, Controls, Vcl.Imaging.GIFImg,
  Vcl.Imaging.pngimage, Messages, Winapi.Windows, Vcl.BaseImageCollection,
  Vcl.ImageCollection, cxGraphics, cxControls, cxLookAndFeels,
  cxLookAndFeelPainters, cxContainer, cxEdit, cxProgressBar, Vcl.ComCtrls,
  Vcl.Samples.Gauges, SpTBXItem, SpTBXControls, dxGDIPlusClasses,
  System.ImageList, Vcl.ImgList, Vcl.VirtualImageList, Win11Forms, cxLabel,
  bfaLog;

type
  TbfaFormType = (sftNone, sftImporter, sftWorkplace, sftViewer, sftDeleteDocs,
                  sftArchiveDocs, sftAdmin);

  TfrmSplash = class (TForm)
    imgLogo: TImage;
    shp_Rand: TShape;
    lblLoading: TcxLabel;
    lblInfo: TcxLabel;
    ImageCollection1: TImageCollection;
    btnCancel: TSpTBXButton;
    pgbDocuments: TcxProgressBar;
    pgbBytes: TcxProgressBar;
    procedure FormHide(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private-Deklarationen }
    FTimer: TTimer;
    FPicCount: Integer;
    FUseTimer: Boolean;
    FFormType: TbfaFormType;
    FCanceled: Boolean;
    procedure SetUseTimer(Value : Boolean);
    procedure SetFormType(AType : TbfaFormType);
    procedure DoTimer(Sender : TObject);
  public
    { Public-Deklarationen }
    procedure IncProgressBar(Value : Double = -1);
    procedure Update; override;

    property UseTimer: Boolean read FUseTimer write SetUseTimer;
    property FormType: TbfaFormType read FFormType write SetFormType;
    property Canceled: Boolean read FCanceled write FCanceled;
  end;



implementation

{$R *.dfm}

uses
  System.SysUtils, MainDataModule, bfaIconsDM, bfaCommonUtils;

procedure TfrmSplash.btnCancelClick(Sender: TObject);
begin
  bitfarmApp.CancelProcess := True;
  btnCancel.Enabled := False;
  FCanceled := True;
end;

procedure TfrmSplash.DoTimer(Sender: TObject);
begin
//  if (FUseTimer) and (FFormType = sftViewer) then
//  begin
//    if (FPicCount < 0) or (FPicCount = 8) then
//    begin
//      FPicCount := 0;
//      imgProgress.Picture.Graphic := nil;
//    end
//    else
//    begin
//      imgProgress.Picture.Graphic :=
//        ImageCollection1.GetSourceImage(FPicCount, 311, 48);
//      Inc(FPicCount);
//    end;
//  end;
end;

procedure TfrmSplash.FormCreate(Sender: TObject);
begin
  RoundedCorners := rcOn; //03.06.2022 Win11

  lblInfo.Caption := '';
  FUseTimer := False;
  FTimer := nil;
  FPicCount := -1;
  FFormType := sftNone;
  btnCancel.Images := bfaIcons.ilMainMenu16;
  btnCancel.ImageIndex := Integer(icoidx_abort);
  FCanceled := False;

//05.10.2022 erst wenn Devex vollstndig benutzt wird, sonst Fehler bei DarkSkin!
//  lblLoading.Style.StyleController := ctlGeneral;
//  lblLoading.Style.BorderStyle := ebsNone;
//  lblInfo.Style.StyleController := ctlGeneral;
//  lblInfo.Style.BorderStyle := ebsNone;
end;

procedure TfrmSplash.FormHide(Sender: TObject);
begin
  bitfarmLog.Write(llDebug, ltDebug, 'frmSplash.Hide: ' + Self.lblLoading.Caption + ' ' + Self.lblInfo.Caption, Classname);
  lblInfo.Caption := '';
//  FUseTimer := False;
//  if FTimer <> nil then
//    FTimer.Enabled := False;
//  FPicCount := -1;
//  FFormType := ftNone;
end;

procedure TfrmSplash.FormShow(Sender: TObject);
begin
  bitfarmLog.Write(llDebug, ltDebug, 'frmSplash.Show: ' + Self.lblLoading.Caption + ' ' + Self.lblInfo.Caption, Classname);
end;

procedure TfrmSplash.SetFormType(AType : TbfaFormType);
begin
  FFormType := AType;
  pgbDocuments.Width:= 249;
  lblLoading.Top := 68;
  pgbDocuments.Position := 0;
  pgbDocuments.Properties.Max := 100;
  if FFormType = sftImporter then begin
    imgLogo.Picture.Graphic :=
      ImageCollection1.GetSourceImage(0, 311, 48);
    pgbDocuments.Width:= 249;
    lblLoading.Top := 55;
  end
  else if FFormType = sftWorkplace then begin
    imgLogo.Picture.Graphic :=
      ImageCollection1.GetSourceImage(1, 311, 48);
    pgbDocuments.Width:= 240;
    lblLoading.Top := 55;
  end
  else if FFormType = sftArchiveDocs then begin
    imgLogo.Picture.Graphic :=
      ImageCollection1.GetSourceImage(2, 311, 48);
    pgbDocuments.Width:= 244;
    lblLoading.Top := 55;
  end
  else if FFormType = sftDeleteDocs then begin
    imgLogo.Picture.Graphic :=
      ImageCollection1.GetSourceImage(3, 311, 48);
    pgbDocuments.Width:= 240;
    lblLoading.Top := 55;
  end
  else if FFormType = sftAdmin then begin
    imgLogo.Picture.Graphic :=
      ImageCollection1.GetSourceImage(4, 311, 48);
    pgbDocuments.Width:= 249;
    pgbDocuments.Properties.Max := 7;
    lblLoading.Top := 55;
  end
  else if FFormType = sftViewer then begin
    imgLogo.Picture.Graphic :=
      ImageCollection1.GetSourceImage(5, 311, 48);
    pgbDocuments.Width:= 245;
    pgbDocuments.Properties.Max := 5;
  end
  else begin
    imgLogo.Picture.Graphic :=
      ImageCollection1.GetSourceImage(5, 311, 48);
    pgbDocuments.Width:= 240;
  end;
end;

procedure TfrmSplash.SetUseTimer(Value : Boolean);
begin
  FUseTimer := Value;
//28.05.2021  if FFormType <> sftViewer then
//    pgbDocuments.Visible := Value;
  pgbDocuments.Visible := True;
  if Value then
  begin
    if FTimer = nil then
    begin
      FTimer := TTimer.Create(Self);
      FTimer.Interval := 100;
      FTimer.Enabled := False;
    end;
    if not FTimer.Enabled then
    begin
//      if FFormType = sftViewer then
//        FTimer.OnTimer := DoTimer
//      else
//        FTimer.OnTimer := nil;
      FPicCount := 0;
      FTimer.Enabled := True;
    end;
  end
  else begin
    if FTimer <> nil then
      FTimer.Enabled := False;
    FPicCount := -1;
    FFormType := sftNone;
    imgLogo.Picture.Graphic :=
      ImageCollection1.GetSourceImage(3, 311, 48);
    btnCancel.Visible := False;
    btnCancel.Enabled := False;
    lblLoading.Top := 68;
  end;
end;

procedure TfrmSplash.Update;
begin
  IncProgressBar;
  inherited;
end;

procedure TfrmSplash.IncProgressBar(Value : Double = -1);
begin
  if Value = -1 then
  begin
    if (pgbDocuments.Position + 1) <= pgbDocuments.Properties.Max then
      pgbDocuments.Position := pgbDocuments.Position + 1
    else
      pgbDocuments.Position := 0;
  end
  else
  begin
    if (pgbDocuments.Position + Value) <= pgbDocuments.Properties.Max then
      pgbDocuments.Position := pgbDocuments.Position + Value
    else
      pgbDocuments.Position := 0;

  end;
end;

end.
