(*
  All the code presented in this unit is Copyright Peter D Johnson
  www.delphidabbler.com, (c) 2004.

  The code may be used in any way providing it is not sold or passed off as
  the work of another person.

  The code is used at the user's own risk. No guarantee or warranty is provided.
*)

unit USingleInst;

interface

uses
   Windows, Controls, Messages, Dialogs, Classes;

type
//   TSingleInstParamHandler = procedure(const Param: string) of object;
   TSingleInstParamHandler = procedure(const Params: TStringList) of object;

type
   TSingleInstClass = class of TSingleInst;

   TSingleInst = class (TObject)
   private
      fOnProcessParam: TSingleInstParamHandler;
      fEnsureRestoreMsg: UINT;
   protected
      function WdwClassName: string; virtual;
      function WaterMark: DWORD; virtual;
      function FindDuplicateMainWdw: HWND; virtual;
      function SendParamsToPrevInst(Wdw: HWND): boolean; virtual;
      function SwitchToPrevInst(Wdw: HWND): boolean;
      procedure EnsureRestore(var Msg: TMessage); dynamic;
      procedure WMCopyData(var Msg: TMessage); dynamic;
   public
      constructor Create;
      procedure CreateParams(var Params: TCreateParams);
      function HandleMessages(var Msg: TMessage): boolean;
      function CanStartApp: boolean;
      property OnProcessParam: TSingleInstParamHandler
         Read fOnProcessParam Write fOnProcessParam;
   end;

function SingleInst: TSingleInst;

procedure RegisterSingleInstClass(Cls: TSingleInstClass);

implementation

uses
   SysUtils, Forms, MainForm {$IFDEF ENTERPRISE}, MainFormEnterprise {$ENDIF};

var
   // Globals storing SingleInst singleton and class of SingleInst
   gSingleInst: TSingleInst = nil;
   gSingleInstClass: TSingleInstClass = nil;

function SingleInst: TSingleInst;
begin
   if not Assigned(gSingleInst) then
   begin
      if Assigned(gSingleInstClass) then
         gSingleInst := gSingleInstClass.Create
      else
         gSingleInst := TSingleInst.Create;
   end;
   Result := gSingleInst;
end;

procedure RegisterSingleInstClass(Cls: TSingleInstClass);
begin
   gSingleInstClass := Cls;
end;

{ TSingleInst }

function TSingleInst.CanStartApp: boolean;
var
   Wdw: HWND;
begin
   Wdw := FindDuplicateMainWdw;
   if Wdw = 0 then
      Result := True
   else
      Result := not SwitchToPrevInst(Wdw);
end;

constructor TSingleInst.Create;
begin
   inherited;
   fEnsureRestoreMsg := RegisterWindowMessage('USINGLEINST_ENSURERESTORE');
end;

procedure TSingleInst.CreateParams(var Params: TCreateParams);
begin
   inherited;
//10.08.2016  StrPLCopy(Params.WinClassName, WdwClassName, SizeOf(Params.WinClassName) - 1);
  StrPLCopy(
    Params.WinClassName,
    WdwClassName,
    SizeOf(Params.WinClassName) div SizeOf(Char) - 1
  );
end;

procedure TSingleInst.EnsureRestore(var Msg: TMessage);
begin
   if IsIconic(Application.Handle) then
      Application.Restore;
   if Assigned(Application.MainForm) and not Application.MainForm.Visible then
      Application.MainForm.Visible := True;
   Application.BringToFront;

end;

function TSingleInst.FindDuplicateMainWdw: HWND;
begin
   Result := FindWindow(PChar(WdwClassName), nil);
end;

function TSingleInst.HandleMessages(var Msg: TMessage): boolean;
begin
   result := False;

  (*if Msg.Msg = WM_QUERYENDSESSION then
  begin
     //ShowMessage('WM_QUERYENDSESSION');
     Msg.Result := 1;
	   result := False;
      Exit;

  end;   *)

  if Msg.Msg = WM_ENDSESSION then
  begin
    //ShowMessage('WM_ENDSESSION');
    if WordBool (Msg.WPARAM) then
    begin 
      { Windows is closing down - clean up!! } 
      
      { This should execute all ExitProcs, close windows and call destructors.. }

//09.08.2016      while gbSaveDoc or gbDocMove do
//      begin
//         Application.ProcessMessages;
//         Sleep(10);
//      end;
//      //Sleep(100);
//
//      frm_MainForm.ShutDownApp(1);

      frmMainForm.Close;

      Msg.Result := 0;
      //ShowMessage('WM_ENDSESSION');
      Halt;
      Application.Terminate;


      { This should close things down properly,                                 }
      { but do we have enough time to handle any posted messages before Windows }
      { is already down??  This will result in a PostQuitMessage that might     }
      { never arrive!                                                           } 

      { This doesn't always work - avoid it } 
      {      Close;}      
    end;
  end;

   if Msg.Msg = WM_COPYDATA then
   begin
      WMCopyData(Msg);
      Result := True;
   end
   else
   if Msg.Msg = fEnsureRestoreMsg then
   begin
      EnsureRestore(Msg);
      Result := True;
   end
   else
      Result := False;
end;

function TSingleInst.SendParamsToPrevInst(Wdw: HWND): boolean;
var
   CopyData: TCopyDataStruct;
   I: integer;
   DataSize: integer;
   Data: PChar;
   PData: PChar;
begin
  DataSize := 0;
  for I := 1 to ParamCount do
    Inc(DataSize, Length(ParamStr(I)) + 1);
  Inc(DataSize);
  Data := StrAlloc(DataSize);
  try
    PData := Data;
    for I := 1 to ParamCount do
    begin
      StrPCopy(PData, ParamStr(I));
      Inc(PData, Length(ParamStr(I)) + 1);
    end;
    PData^ := #0;
    CopyData.lpData := Data;
    CopyData.cbData := DataSize * SizeOf(Char);
    CopyData.dwData := WaterMark;
    Result := SendMessage(Wdw, WM_COPYDATA, 0, LPARAM(@CopyData)) = 1;
  finally
    StrDispose(Data);
  end;
end;

function TSingleInst.SwitchToPrevInst(Wdw: HWND): boolean;
begin
  Assert(Wdw <> 0);
  if ParamCount > 0 then
    Result := SendParamsToPrevInst(Wdw)
  else
    Result := True;
  if Result then
    SendMessage(Wdw, fEnsureRestoreMsg, 0, 0);
end;

function TSingleInst.WaterMark: DWORD;
begin
   Result := $DDED;
end;

function TSingleInst.WdwClassName: string;
begin
   Result := 'bitfarm.Viewer.V36';
end;

procedure TSingleInst.WMCopyData(var Msg: TMessage);
var
   PData: PChar;
   Param: string;
   Params: TStringList;
begin
  if TWMCopyData(Msg).CopyDataStruct.dwData = WaterMark then
  begin
    Params := TStringList.Create;
    PData := TWMCopyData(Msg).CopyDataStruct.lpData;
    while PData^ <> #0 do
    begin
      Param := StrPas(PData);
      Params.Add(Param);
      Inc(PData, Length(Param) + 1);
    end;
    if Assigned(fOnProcessParam) then
      fOnProcessParam(Params);
    Msg.Result := 1;
  end
  else
    Msg.Result := 0;
//10.08.2016   application.ProcessMessages;
end;

initialization
   RegisterSingleInstClass(TSingleInst);


finalization

   gSingleInst.Free;

end.
