unit bfaQuickPreview;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, SpTBXItem, SpTBXControls,
  bfaObjects36, Generics.Collections, Vcl.ExtCtrls, gnugettext,  //15.10.2024 SD <--bfacrm:ticket__show__39532
  {$IFDEF IMAGEEN1140}
  iexBitmaps, iesettings, iexLayers, iexRulers, iexToolbars,
  iexUserInteractions, iexProcEffects,
  {$ENDIF}
  ieview, imageenview, hyiedefs, imageenio, imageenproc, hyieutils, bfaImageEnVect,
  ievect
  ;

type
  TfrmQuickPreview = class(TForm)
    imgQuickPreview: TbfaImageEnVect;
    lblPageCount: TSpTBXLabel;
    lblDocument: TSpTBXLabel;
    procedure FormMouseWheel(Sender: TObject; Shift: TShiftState;
      WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
    procedure FormShow(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    { Private declarations }
    FFirstResize : Boolean;
  public
    CurrentLayer, CurrentLayerRights : Integer;
    ImgCount, ImageIdx : Integer;
    //15.10.2024 SD <--bfacrm:ticket__show__39532
    DocLoadTimer, HideLabelTimer: TTimer;
    DocIdx, PageDirection: Integer;
    DocumentList: TList<TbfaDocument>;
    procedure SetActiveDoc(Direction: Integer);
    procedure RepaintActiveDoc(Sender: TObject);
    procedure DocLabelShow;
    procedure DocLabelHide(Sender: TObject);
    //15.10.2024 SD <--bfacrm:ticket__show__39532
    { Public declarations }
  end;

implementation

{$R *.dfm}

uses
  bfaLog, MainDataModule, bfaCommonUtils, {07.11.2022 IMAGEEN1140, hyieutils}
  bfaCommonMethods, bfaIconsDM  //15.10.2024 SD <--bfacrm:ticket__show__39532
  ;

procedure TfrmQuickPreview.FormCreate(Sender: TObject);
begin
  FFirstResize := False;
  CurrentLayer := -1;
  CurrentLayerRights := -1;
//  imgQuickPreview.DelayZoomFilter := True; // via .dfm
//  imgQuickPreview.ZoomFilter := rfFastLinear; // via .dfm
//  imgQuickPreview.BitmapResampleFilter := rfFastLinear; // via .dfm
  //15.10.2024 SD <--bfacrm:ticket__show__39532
  lblDocument.Color := clBlack;
  lblDocument.ParentFont := False;
  lblDocument.Font.Size := 14;
  lblDocument.Font.Color := clWhite;
  lblDocument.Alignment := taCenter;
  lblDocument.Visible := False;

  DocLoadTimer := TTimer.Create(Self);
  DocLoadTimer.OnTimer := Self.RepaintActiveDoc;
  DocLoadTimer.Interval := 100;
  DocLoadTimer.Enabled := False;

  HideLabelTimer := TTimer.Create(Self);
  HideLabelTimer.OnTimer := Self.DocLabelHide;
  HideLabelTimer.Interval := 3000;
  HideLabelTimer.Enabled := False;
  //15.10.2024 SD <--bfacrm:ticket__show__39532
end;

{$IFDEF ImageEn1140}
procedure TfrmQuickPreview.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
  Idx, mx, my, xTmp, nPos: Integer;
  iebmp2: TIEBitmap;
  Scroll, SetTop, SetBottom: Boolean;
begin
  if Key = VK_F2 then
  begin
    bitfarmLog.Write(llDebug, ltInfo, 'TfrmQuickPreview.FormKeyDown - VK_F2', ClassName);
    try
      bitfarmLog.Write(llDebug, ltInfo, 'TfrmQuickPreview.FormKeyDown - VK_F2 - Screen.ActiveControl.Name.ActiveControl: ' + 
        Screen.ActiveControl.Name, ClassName);
    except

    end;
  end

  // Alt +
  else if (ssAlt in Shift) and (Key in [VK_ADD, VK_OEM_PLUS])  then
    imgQuickPreview.ZoomIn

  // Alt -
  else if (ssAlt in Shift) and (Key in [VK_SUBTRACT, VK_OEM_MINUS]) then
    imgQuickPreview.ZoomOut

  else if (Key in [VK_HOME, VK_END, VK_PRIOR, VK_NEXT, VK_UP, VK_DOWN]) then
  begin
    Idx := ImageIdx;
    Scroll := False;
    SetTop := False;
    SetBottom := False;

    imgQuickPreview.GetMaxViewXY(mx, my);

    if Key = VK_HOME then
    begin
      Idx := 0;
      SetTop := True;
    end else if Key = VK_END then
    begin
      Idx := ImgCount - 1;
      SetTop := True;
    end else if (Key = VK_PRIOR) or (Key = VK_UP) then
    begin
      if (Key = VK_UP) and (imgQuickPreview.ViewY > 0) then
      begin
        xTmp := imax(round(imgQuickPreview.ClientHeight * imgQuickPreview.MouseWheelParams.Value / 100), 1);
        nPos := imgQuickPreview.ViewY + -1 * xTmp;
        imgQuickPreview.ViewY := nPos;
        Scroll := True;
      end

      else if (Key = VK_PRIOR) and (imgQuickPreview.ViewY > 0) then
      begin
        imgQuickPreview.DoVertScroll(iescPageUp, 0);
        Scroll := True;
      end

      else if Idx > 0 then
      begin
        Dec(Idx);
        SetBottom := True;
      end;
    end
    else if (Key = VK_NEXT) or (Key = VK_DOWN) then
    begin
      if (Key = VK_DOWN) and (imgQuickPreview.ViewY < my) then
      begin
        xTmp := imax(round(imgQuickPreview.ClientHeight * imgQuickPreview.MouseWheelParams.Value / 100), 1);
        nPos := imgQuickPreview.ViewY + 1 * xTmp;
        imgQuickPreview.ViewY := nPos;
        Scroll := True;
      end

      else if (Key = VK_NEXT) and (imgQuickPreview.ViewY < my) then
      begin
        imgQuickPreview.DoVertScroll(iescPageDown, 0);
        Scroll := True;
      end

      else if Idx < ImgCount - 1 then
      begin
        Inc(Idx);
        SetTop := True;
      end;
    end;

    if (Idx > -1) and (not Scroll) then
    begin
      imgQuickPreview.RemoveAllObjects;
      imgQuickPreview.IO.Params.ImageIndex := idx;
      ImageIdx := Idx;

      if (bitfarmApp.CurrentContainer[0].ScanPoolView <> nil) and
         bitfarmApp.CurrentContainer[0].ScanPoolViewLoaded then
      begin
        iebmp2 := TIEBitmap.Create;
        try
          bitfarmApp.CurrentContainer[0].ScanPoolView.CopyToIEBitmap(ImageIdx, iebmp2);
          {$IFDEF ImageEN1140}
          imgQuickPreview.Bitmap.Assign(iebmp2.VclBitmap);
          {$ELSE}
          imgQuickPreview.IO.Bitmap.Assign(iebmp2.VclBitmap);
          {$ENDIF}
          imgQuickPreview.IO.Params.Width := imgQuickPreview.Bitmap.Width;
          imgQuickPreview.IO.Params.Height := imgQuickPreview.Bitmap.Height;
          if SetTop then
            imgQuickPreview.ViewY := 0
          else if SetBottom then
            imgQuickPreview.ViewY := my;
          imgQuickPreview.Update;
        finally
          FreeandNil(iebmp2);
        end;
      end else
      begin
        bitfarmApp.CurrentContainer[0].DocStream.Position := 0;
        imgQuickPreview.IO.LoadFromStream(bitfarmApp.CurrentContainer[0].DocStream);
        if SetTop then
          imgQuickPreview.ViewY := 0
        else if SetBottom then
          imgQuickPreview.ViewY := my;
      end; 
      
      lblPageCount.Caption := IntToStr(idx + 1) + ' / ' + IntToStr(ImgCount);
      bitfarmApp.ShowQuickPreviewAnnos;
    end;
  end
  else if (Key = VK_F11) or (Key = VK_ESCAPE) then
    Close;
end;
{$ELSE}
procedure TfrmQuickPreview.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
  Idx: Integer;
  iebmp2: TIEBitmap;
begin
  if Shift = [] then
  begin
    if Key = VK_F2 then
    begin
      bitfarmLog.Write(llDebug, ltInfo, 'TfrmQuickPreview.FormKeyDown - VK_F2', ClassName);
      try
        bitfarmLog.Write(llDebug, ltInfo, 'TfrmQuickPreview.FormKeyDown - VK_F2 - Screen.ActiveControl.Name.ActiveControl: ' +
          Screen.ActiveControl.Name, ClassName);
      except

      end;
    end

    else if (Key in [VK_HOME, VK_END, VK_PRIOR, VK_UP, VK_NEXT, VK_DOWN]) then
    begin
      Idx := ImageIdx;
      if Key = VK_HOME then
        Idx := 0
      else if Key = VK_END then
        Idx := ImgCount - 1
      else if (Key = VK_PRIOR) or (Key = VK_UP) then
      begin
        if Idx > 0 then
          Dec(Idx);
      end
      else if (Key = VK_NEXT) or (Key = VK_DOWN) then
      begin
        if Idx < ImgCount - 1 then
          Inc(Idx);
      end;

      if Idx > -1 then
      begin
        imgQuickPreview.RemoveAllObjects;
        imgQuickPreview.IO.Params.ImageIndex := idx;
        ImageIdx := Idx;

        if (bitfarmApp.CurrentContainer[0].ScanPoolView <> nil) and
           bitfarmApp.CurrentContainer[0].ScanPoolViewLoaded then
        begin
          iebmp2 := TIEBitmap.Create;
          try
            bitfarmApp.CurrentContainer[0].ScanPoolView.CopyToIEBitmap(ImageIdx, iebmp2);
            imgQuickPreview.IO.Bitmap.Assign(iebmp2.VclBitmap);
            imgQuickPreview.IO.Params.Width := imgQuickPreview.Bitmap.Width;
            imgQuickPreview.IO.Params.Height := imgQuickPreview.Bitmap.Height;
            imgQuickPreview.Update;
          finally
            FreeandNil(iebmp2);
          end;
        end else
        begin
          bitfarmApp.CurrentContainer[0].DocStream.Position := 0;
          imgQuickPreview.IO.LoadFromStream(bitfarmApp.CurrentContainer[0].DocStream);
        end;

        lblPageCount.Caption := IntToStr(idx + 1) + ' / ' + IntToStr(ImgCount);
        bitfarmApp.ShowQuickPreviewAnnos;
      end;
    end
    else if Key <> VK_CONTROL then
      Close;
  end
  //15.10.2024 SD <--bfacrm:ticket__show__39532
  else if (Shift = [ssCtrl]) and (DocumentList.Count > 1) then
  begin
    if (Key in [VK_PRIOR, VK_NEXT]) then
    begin
      if Key = VK_NEXT then
        SetActiveDoc(-1)
      else if Key = VK_PRIOR then
        SetActiveDoc(1);
    end;
  end;
  //15.10.2024 SD <--bfacrm:ticket__show__39532
end;

{$ENDIF}

procedure TfrmQuickPreview.FormMouseWheel(Sender: TObject; Shift: TShiftState;
  WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
begin
  try
    try
      imgQuickPreview.RemoveAllObjects;
      bitfarmApp.PreviewMouseWheel(nil, Self, imgQuickPreview, Shift,
        WheelDelta, Handled);
      bitfarmApp.ShowQuickPreviewAnnos;
      imgQuickPreview.SetFocus;
    finally
      {$IFNDEF ImageEn1140}
      Handled := True;
      {$ENDIF}
    end;
  except
    bitfarmLog.Write(llNormal, ltException, 'FormMouseWheel', ClassName);
  end;
end;

procedure TfrmQuickPreview.FormResize(Sender: TObject);
begin
  try
    if not FFirstResize then
    begin
      case bitfarmApp.GetActivePreview.bfaTlbPreview.sbmAutozoom.ImageIndex of
        Integer(icoidx_preview_width):
          bitfarmApp.ZoomWidth(nil, imgQuickPreview);

        Integer(icoidx_preview_auto):
          bitfarmApp.ZoomPage(nil, imgQuickPreview);

        Integer(icoidx_preview_original):
          bitfarmApp.Zoom100(nil, imgQuickPreview);

        Integer(icoidx_preview_height):
          bitfarmApp.ZoomHeight(nil, imgQuickPreview);

        Integer(icoidx_select):
          bitfarmApp.ZoomSelect(nil, imgQuickPreview);

        Integer(icoidx_preview_no_autozoom):
          bitfarmApp.ZoomOff(nil, imgQuickPreview);
      end;
    end;
  except
    bitfarmLog.Write(llNormal, ltException, 'FormResize', ClassName);
  end;
end;

procedure TfrmQuickPreview.FormShow(Sender: TObject);
begin
  try
    imgQuickPreview.IO.AutoAdjustDPI := True;
    imgQuickPreview.IO.Aborting := true;
    imgQuickPreview.ObjBoxInnerSelectable := True;
    imgQuickPreview.MaxSelectionDistance := 30;
    imgQuickPreview.MaxMovingDistance := 30;
    imgQuickPreview.AllowOutOfBitmapMoving := False;
    imgQuickPreview.AllowOutOfBitmapPolylines := False;
    imgQuickPreview.MouseWheelParams.ZoomPosition := iemwMouse;
    {$IFDEF ImageEn1140}
    if bitfarmApp.ViewerOptions.EditorOptions.DocEditorZoom then
    begin
      imgQuickPreview.MouseWheelParams.Action := iemwVScroll;
      imgQuickPreview.MouseWheelParams.InvertDirection := False;
      imgQuickPreview.MouseWheelParamsAlt.Action := iemwZoom;
      imgQuickPreview.MouseWheelParamsAlt.InvertDirection := not bitfarmApp.ViewerOptions.EditorOptions.InvertDirection;
      imgQuickPreview.MouseWheelParamsAlt.ZoomPosition := iemwMouse;
    end else
    begin
      imgQuickPreview.MouseWheelParams.Action := iemwZoom;
      imgQuickPreview.MouseWheelParams.InvertDirection := not bitfarmApp.ViewerOptions.EditorOptions.InvertDirection;
      imgQuickPreview.MouseWheelParamsAlt.Action := iemwVScroll;
      imgQuickPreview.MouseWheelParamsAlt.InvertDirection := False;
      imgQuickPreview.MouseWheelParamsAlt.ZoomPosition := iemwMouse;
    end;
    {$ELSE}
    imgQuickPreview.MouseWheelParams.InvertDirection := bitfarmApp.ViewerOptions.EditorOptions.InvertDirection;
    {$ENDIF}
    imgQuickPreview.MouseInteract := [miScroll];

    case bitfarmApp.GetActivePreview.bfaTlbPreview.sbmAutozoom.ImageIndex of
      Integer(icoidx_preview_width):
        bitfarmApp.ZoomWidth(nil, imgQuickPreview);

      Integer(icoidx_preview_auto):
        bitfarmApp.ZoomPage(nil, imgQuickPreview);

      Integer(icoidx_preview_original):
        bitfarmApp.Zoom100(nil, imgQuickPreview);

      Integer(icoidx_preview_height):
        bitfarmApp.ZoomHeight(nil, imgQuickPreview);

      Integer(icoidx_select):
        bitfarmApp.ZoomSelect(nil, imgQuickPreview);

      Integer(icoidx_preview_no_autozoom):
        bitfarmApp.ZoomOff(nil, imgQuickPreview);
    end;

    WindowState := wsMaximized;
  finally
    FFirstResize := True;
    //15.10.2024 SD -->bfacrm:ticket__show__39532
    if DocumentList.Count > 1 then
      DocLabelShow;
    //15.10.2024 SD <--bfacrm:ticket__show__39532
  end;
end;

//15.10.2024 SD -->bfacrm:ticket__show__39532
procedure TfrmQuickPreview.SetActiveDoc(Direction: Integer);
var
  mx, my: Integer;
begin
  if ((DocIdx + Direction) < 0) or ((DocIdx + Direction) > DocumentList.Count - 1) or DocLoadTimer.Enabled or (bitfarmApp.CurrentContainer[0] = nil)then
    Exit;
  PageDirection := Direction;
  try
    HideLabelTimer.Enabled := False;
    lblPageCount.Visible := False;
    lblDocument.Visible := False;
    DocLoadTimer.Enabled := True;
    bitfarmApp.CurrentContainer[0] :=  DocumentList[DocIdx + Direction];
    if (bitfarmApp.CurrentContainer[0] <> nil) and (bitfarmApp.CurrentContainer[0].Loaded) then
    begin
      imgQuickPreview.Clear;
      imgQuickPreview.RemoveAllObjects;
      ImageIdx := 0;
      imgQuickPreview.IO.Params.ImageIndex := 0;
      bitfarmApp.CurrentContainer[0].DocStream.Position := 0;
      imgQuickPreview.IO.LoadFromStream(bitfarmApp.CurrentContainer[0].DocStream);
      if (Direction = -1) and (DocumentList.Count > 1) then
      begin
        ImageIdx := imgQuickPreview.IO.Params.ImageCount - 1;
        imgQuickPreview.Clear;
        imgQuickPreview.RemoveAllObjects;
        imgQuickPreview.IO.Params.ImageIndex := ImageIdx;
        bitfarmApp.CurrentContainer[0].DocStream.Position := 0;
        imgQuickPreview.IO.LoadFromStream(bitfarmApp.CurrentContainer[0].DocStream);
      end;
      case bitfarmApp.GetActivePreview.bfaTlbPreview.sbmAutozoom.ImageIndex of
        Integer(icoidx_preview_width):
          bitfarmApp.ZoomWidth(nil, imgQuickPreview);

        Integer(icoidx_preview_auto):
          bitfarmApp.ZoomPage(nil, imgQuickPreview);

        Integer(icoidx_preview_original):
          bitfarmApp.Zoom100(nil, imgQuickPreview);

        Integer(icoidx_preview_height):
          bitfarmApp.ZoomHeight(nil, imgQuickPreview);

        Integer(icoidx_select):
          bitfarmApp.ZoomSelect(nil, imgQuickPreview);

        Integer(icoidx_preview_no_autozoom):
          bitfarmApp.ZoomOff(nil, imgQuickPreview);
      end;
      if (Direction = -1) and (DocumentList.Count > 1) then
      begin
      imgQuickPreview.GetMaxViewXY(mx, my);
      imgQuickPreview.SetViewXY(imgQuickPreview.ViewX, my);
      end;
      imgQuickPreview.Update;
      ImgCount := imgQuickPreview.IO.Params.ImageCount;
    end
    else
    begin
      imgQuickPreview.Clear;
      imgQuickPreview.RemoveAllObjects;
    end;
  finally
    DocIdx := DocIdx + PageDirection;
    PageDirection := 0;
    if ImgCount > 1 then
    begin
      lblPageCount.Caption := IntToStr(ImageIdx + 1) + ' / ' + IntToStr(ImgCount);
      lblPageCount.Visible := True;
    end
    else
    begin
      lblPageCount.Caption := '';
      lblPageCount.Visible := False;
    end;
    DocLabelShow;
  end;
end;

procedure TfrmQuickPreview.RepaintActiveDoc(Sender: TObject);
begin
  DocLoadTimer.Enabled := False;
  DocIdx := DocIdx + PageDirection;
  PageDirection := 0;
end;

procedure TfrmQuickPreview.DocLabelShow;
var
  str, FileSize: WideString;
begin
  try
    str := str + bitfarmApp.CurrentContainer[0].DocName + ' - ' + _('Datum: ') +
      FormatDateTime('dd.mm.yyyy', VarToDateTime(bitfarmApp.CurrentContainer[0].DocDate)) + ' - ' +
      _('Dateityp: ') + StringReplace(bitfarmApp.CurrentContainer[0].FileType + ' - ' +
      _('Anzahl Seiten: ') + IntToStr(ImgCount), '.', '', [rfReplaceAll]);
    str := StringReplace(str, '&', '&&', [rfReplaceAll]);
    lblDocument.Caption := str;
  except
    lblDocument.Caption := bitfarmApp.CurrentContainer[0].DocName;
  end;
  lblDocument.Visible := True;
  lblDocument.Left := Round((Self.Width / 2) - (lblDocument.Width / 2));
  HideLabelTimer.Enabled := True;
end;

procedure TfrmQuickPreview.DocLabelHide(Sender: TObject);
begin
  HideLabelTimer.Enabled := False;
  lblDocument.Visible := False;
end;
//15.10.2024 SD <--bfacrm:ticket__show__39532
end.
