unit bfaSearchThread;

interface

uses
  System.Classes, VCL.ExtCtrls, VCL.ActnList, Uni, cxGridTableView,
  gnugettext,
  bfaBaseObjects, bfaObjects36, u_bfaGrid, bfaGridDBTableView,
  bfaApplication {$IFDEF ENTERPRISE} , bfaApplicationEnterprise {$ENDIF};


type
  TQueryThread = class(TThread)
  protected
    procedure Execute; override;
  public
    cnt: TUniConnection;
    SearchObj: TbfaSearch;
    HistorySearchObj: TbfaSearch;
    dsc: TUniDataSource;
    SearchProc: TbfaSearchProcessing;
    bfagrid: TbfaDBGrid;
    dbView : TbfaGridDBTableView;
    tmr: TTimer;
    pnl: TPanel;
    actHide: TAction;
    locate_gdocid: Int64;
    AutoCheckout: Boolean; // SJ 01.12.2021 - bfacrm:ticket__show__28712

    constructor Create(createsuspended: Boolean); overload;
//22.01.2024    procedure ExpandGroupRecursive(var AGroup: TcxGridGroupRow; var b: Boolean);
    procedure ExpandGroupRecursive(AGroup: TcxGridGroupRow; var Max: Integer);
  end;

implementation

uses
 System.SysUtils, VCL.Forms, VCL.Graphics, VCL.Controls,
 cxGrid,
 MainDataModule,
 MainForm {$IFDEF ENTERPRISE}, MainFormEnterprise {$ENDIF};




 { TQueryThread }

constructor TQueryThread.Create(createsuspended: Boolean);
begin
  inherited Create(createsuspended);
  locate_gdocid := 0;
  AutoCheckout := False; // SJ 01.12.2021 - bfacrm:ticket__show__28712
end;

//22.01.2024procedure TQueryThread.ExpandGroupRecursive(var AGroup: TcxGridGroupRow; var b: Boolean);
procedure TQueryThread.ExpandGroupRecursive(AGroup: TcxGridGroupRow; var Max: Integer);
var
  b: Boolean;
begin
  try
    if (Max < 10) and (AGroup <> nil) and not AGroup.IsData then
    begin
      dbView.BeginUpdate;
      try
        AGroup.Expand(False);
        try
          inc(Max);
          b := AGroup.Expanded;
        except
          b := False; //22.07.2022 Ticket 31661/1 manchmal Expanded nicht erreichbar
        end;
      finally
        dbview.EndUpdate;
      end;
      if (b and (AGroup.ValueCount > 1)) (*22.01.2024 or AGroup.IsData*) then
        dbView.Controller.FocusNextRecordWithSelection(AGroup.Index, True, False, True);

      if (Max < 10) and (dbView.Controller.FocusedRecord <> nil) and not dbView.Controller.FocusedRecord.IsData then
      begin
//31.01.2024        AGroup := (dbView.Controller.FocusedRecord as TcxGridGroupRow);
        AGroup := TcxGridGroupRow(dbView.Controller.FocusedRecord);
        (*22.01.2024
        try
          b := AGroup.Expanded;
        except
          b := False; //22.07.2022 31661/1 manchmal Expanded nicht erreichbar
        end;
        if not b and not AGroup.IsData then
          ExpandGroupRecursive(AGroup, b);
        22.01.2024*)
        if not AGroup.IsData then
          ExpandGroupRecursive(AGroup, Max);
      end;
    end;
  except

  end;
end;

procedure TQueryThread.Execute;
var
  dsc2: TUniDataSource;
  tmStart: TTime;
  sTime: string;
  OldColor: TColor;
  arcid: Int64;
  SL : TStringList;
  j, k, idx : Integer;
  foundsymarcid, colsorted : boolean;
  ErrorStr: string;
  //14.12.2023 -->
  GridGroup: TcxGridGroupRow;
  GroupExpanded: Boolean;
  Max : Integer;
  //14.12.2023 <--
begin
  inherited;
  try
    if not cnt.Connected then
      cnt.Connect;

    while not Self.Terminated do
    begin
      try
        OldColor := pnl.Color;
        tmr.Enabled := True;
        tmStart := GetTime;
        Synchronize(
          procedure
          begin
            Screen.Cursor := crHourGlass;
            dbView.DataController.DataSource := nil;
            pnl.Caption := ' ' + _('Suche aktiv ...');
            bitfarmApp.ResubListLoaded := False;
          end);

        try
          if dsc <> nil then
            dsc.Free;

          dsc := TUniDataSource.Create(nil);

          SL := TStringList.Create;
          try
            SL.CommaText := SearchObj.SelectedCols;
            for j := SL.Count - 1 downto 0 do
            begin
              if LowerCase(copy(sl[j], 1, 4)) <> 'add_' then
                SL.Delete(j)
              else
              begin
                for k := Low(aAddfields) to High(aAddfields) do
                begin
                  idx := Pos(':', sl[j]);
                  if idx > 0 then
                    sl[j] := Copy(Sl[j], 0, Pos(':', sl[j]) - 1);

                  if (aAddfields[k] <> nil) and (aAddfields[k].Name = sl[j]) then
                  begin
                    if not ((aAddfields[k].Typ = 3) or (aAddfields[k].Typ = 0)) then  //21.01.2019 die mehr als 255 Charachter sein knnen sonst wird (WideMEMO) in der Trefferliste
                      SL.Delete(j);
                    Break;
                  end;
                end;
              end;
            end;
            dsc.DataSet := SearchProc.Search(cnt, SearchObj, HistorySearchObj,
              ErrorStr, SL, bitfarmApp.Log);

            if ErrorStr <> '' then
            begin
              Synchronize(
              procedure
              begin
                frmMainForm.SetNotificationBar(ErrorStr);
              end);
            end;
          finally
            FreeAndNil(SL);
          end;

          Synchronize(
            procedure
            var
              i: Integer;
            begin
              tmr.Enabled := False;
              pnl.Color := OldColor;
              pnl.ParentBackground := True;

              tmStart := GetTime - tmStart;
              DateTimeToString(sTime, 'nn:ss:zzz', tmStart);
              pnl.Caption := ' ' + sTime;

              if dsc.DataSet <> nil then
              begin
                dbView.DataController.DataSource := dsc;
                if dbView.DataController.DataSource.DataSet.RecordCount > 0 then
                begin
                  if bitfarmApp.CurrentView = tvkResub then
                  begin
                    if frmMainForm.fraResub.btnMyTasks.Checked then
                    begin
                      if bitfarmApp.BMResub > 0 then
                      begin
                        dbView.Controller.ClearSelection;
                        dbView.DataController.DataSource.DataSet.Locate('timid', IntToStr(bitfarmApp.BMResub), []);
                      end;
                    end
                    else if frmMainForm.fraResub.btnDelegateTasks.Checked then
                    begin
                      if bitfarmApp.BMDelegateResub > 0 then
                      begin
                        dbView.Controller.ClearSelection;
                        dbView.DataController.DataSource.DataSet.Locate('timid', IntToStr(bitfarmApp.BMDelegateResub), []);
                      end;
                    end;
                  end
                  else
                    if locate_gdocid > 0 then
                      dbView.DataController.DataSource.DataSet.Locate('gdocid', IntToStr(locate_gdocid), []);
                  if bitfarmApp.CurrentView = tvkViewer then
                  begin
                    if bitfarmApp.GridLastSelectedGdDocID > 0 then
                    begin
                      dbView.Controller.ClearSelection;
                      dbView.DataController.DataSource.DataSet.Locate('gdocid', IntToStr(bitfarmApp.GridLastSelectedGdDocID), []);
                      bitfarmApp.GridLastSelectedGdDocID := 0;
                    end;
                  end;

                  dbview.SetBookmark;
//16.02.2024 -->
//                  dbview.Controller.FocusedRecord.Selected := True;
//                  dbView.OnSelectionChanged(dbview);
//16.02.2024 <--

                  if (dbView.Controller.FocusedRecord <> nil) and not dbView.Controller.FocusedRecord.IsData then
                  begin
                    try
//16.02.2024 -->
//                      GridGroup := TcxGridGroupRow(dbView.Controller.FocusedRecord);
//                      GroupExpanded := GridGroup.Expanded;
//                      Max := 0;
//                      ExpandGroupRecursive(GridGroup, Max);
                      if dbView.DataController.RecordCount >= dbView.ActRec then
                        idx := dbView.ActRec
                      else
                        idx := 0;
                      dbView.DataController.MakeRecordVisible(idx);
                      dbView.DataController.FocusedRecordIndex := idx;
                      dbView.Invalidate;
//16.02.2024 <--
                    except
                      //22.01.2024
                    end;
                  end;

                  //16.02.2024 -->
                  dbview.Controller.FocusedRecord.Selected := True;
                  dbView.OnSelectionChanged(dbview);
                  //16.02.2024 <--

                  // bfacrm:ticket__show__24708
                  if (bitfarmApp.GetActivePreview = bitfarmApp.PreviewContainer.ViewerItem.Preview) and
                     frmMainForm.grdResultlistDevEx.Enabled and frmMainForm.grdResultlistDevEx.Visible
                  then
                    if frmMainForm.ShowFloatingResub and //23.02.2024
                       (Screen.ActiveControl is TbfaGridSite) and
                       (TbfaGridSite(Screen.ActiveControl).Parent <> nil) and
                       (TbfaGridSite(Screen.ActiveControl).Parent is TcxGrid) and
                       (TcxGrid(TbfaGridSite(Screen.ActiveControl).Parent).Owner <> nil) and
                       (TcxGrid(TbfaGridSite(Screen.ActiveControl).Parent).Owner = frmMainForm.fraFloatingResub)
                    then
                    begin
                      try
                        TcxGrid(TbfaGridSite(Screen.ActiveControl).Parent).SetFocus;
                      except
                        //10.05.2023 CrashLog
                      end;
                    end
                    else
                    begin
                      try
                        frmMainForm.grdResultlistDevEx.SetFocus;
                      except
                        //10.05.2023 CrashLog
                      end;
                    end;
                end else
                begin
                  if (pnl.Caption <> '') then
                    pnl.Caption := '';

                  for i := 0 to aCurrentViewerAddfields.Count - 1 do
                  begin
                    if (not aCurrentViewerAddfields[i].ModifiedSearch) and
                       (aCurrentViewerAddfields[i].Component <> nil)
                    then
                      aCurrentViewerAddfields[i].Component.ClearValue;
                  end;
                  bitfarmApp.GetActivePreview.SetActions;

                  frmMainForm.SetNotificationBar(_('Die Suche ergab keinen Treffer!'), nil); //01.08.2022 Ticket 27900
                end;

                locate_gdocid := 0;
              end else
              begin
                if (pnl.Caption <> '') then
                  pnl.Caption := '';
                dbView.DataController.DataSource := nil;
                bitfarmApp.GetActivePreview.SetActions;
              end;
            end);
        finally
          Synchronize(
            procedure
            begin
              actHide.Enabled := True;
              Screen.Cursor := crDefault;
            end);
        end;

        // SJ 01.12.2021 - bfacrm:ticket__show__28712
        try
          if Self.AutoCheckout and bitfarmApp.ViewerOptions.GeneralOptions.CheckoutNewDocuments then
          begin
            var UserAction: PUserAction;
            UserAction := bitfarmApp.CreateUserAction;
            UserAction^.ActionName := 'actCheckout';
            bitfarmApp.MainThread.Add(UserAction);

            UserAction := bitfarmApp.CreateUserAction;
            UserAction^.ActionName := 'actOriginalfile';
            UserAction^.Params.Add('edit');
            bitfarmApp.MainThread.Add(UserAction);
          end;
        finally
          Self.AutoCheckout := False;
        end;
        // SJ 01.12.2021 - bfacrm:ticket__show__28712

        Self.Suspend;
      except
        dbView.DataController.DataSource := nil;
        tmr.Enabled := False;
        Self.Suspend;
      end;
    end;
  finally
    cnt.Close;
    Synchronize(
      procedure
      begin
        dbView.Invalidate;
      end);
  end;
end;

end.
