unit bfaThreadedThumbnail;

interface

uses
  Classes, Windows, Forms, Graphics, ievect, iemview, bfaBaseObjects, bfaObjects36, Generics.Collections,
  StdCtrls, bfaImageEnVect, bfaImageENMView;

type
  TThreadedThumbnails = class(TThread)
  private
    FViewer: TFrame;
    FFtxPos: TList<TbfaFtxPos>;
    FThumbnailView, FScanPoolView: TbfaImageEnMView;
    FInProcess: Boolean;
    FPage, FTotalPages, FSelectedIdx: Integer;
    FDoc, FOriDoc: TbfaDocument;
    FException: Boolean;
    FStopThread: Boolean;
    procedure DoThumbs(ActPage: Integer; img: Graphics.TBitmap;
      imgVect: TbfaImageEnVect; imgOri: Graphics.TBitmap);
    procedure FreeObjects;
  public
    destructor Destroy; override;
    procedure SetObjects(Thumbnail: TbfaImageEnMView; Ftx: TList<TbfaFtxPos>; Vwr: TFrame; SelectedImageIdx: Integer);
    procedure SetObjectsWithList(Thumbnail: TbfaImageEnMView; Ftx: TList<TbfaFtxPos>; Vwr: TFrame; Document: TbfaDocument; SelectedImageIdx: Integer);
    procedure KillThread;
    property Terminated;
    property InProcess: Boolean read FInProcess;
  protected
    /// <summary>
    ///   hgdhfgjs
    /// </summary>
    /// <exception cref="sdfsdf">
    ///   sdfsdf
    /// </exception>
    procedure Execute; override;
  end;

implementation

uses
  SysUtils, Dialogs, MainDataModule, bfaPreview36, bfaLog, bfaCommonStyles,
  bfaCommonImageEn, bfaApplication, bfaCommonUtils, gnugettext, Controls, 
  hyieutils, imageenio
  {$IFDEF IMAGEEN1140}
  , iexBitmaps
  {$ENDIF}
  ;

procedure TThreadedThumbnails.SetObjects(Thumbnail: TbfaImageEnMView; Ftx: TList<TbfaFtxPos>;
  Vwr: TFrame; SelectedImageIdx: Integer);
begin
  FreeOnTerminate := False;
  FViewer := Vwr;
  FThumbnailView := Thumbnail;
  FFtxPos := Ftx;
  Priority := tpLowest;
  FInProcess := False;
  FException := False;
  FStopThread := False;
  FSelectedIdx := SelectedImageIdx;
end;

destructor TThreadedThumbnails.Destroy;
begin
  FreeObjects;
  inherited Destroy;
end;

procedure TThreadedThumbnails.SetObjectsWithList(Thumbnail: TbfaImageEnMView; Ftx: TList<TbfaFtxPos>; Vwr: TFrame; Document: TbfaDocument; SelectedImageIdx: Integer);
var
  i: Integer;
begin
  try
    SetObjects(Thumbnail, Ftx, Vwr, SelectedImageIdx);
    FOriDoc := Document;
    FDoc := TbfaDocument.Create;
    FDoc.Clone(Document, False, False, True);
    if (not FOriDoc.ScanPoolViewLoaded) and (FOriDoc.HasPreview) and
       ((TbfaPreview(Vwr) = bitfarmApp.PreviewContainer.ImporterItem.Preview) or
       (TbfaPreview(Vwr) = bitfarmApp.PreviewContainer.WorkplaceItem.Preview)) then
    begin
      if FOriDoc.ScanPoolView <> nil then
        FScanPoolView := FOriDoc.ScanPoolView
      else
        FScanPoolView := CreateInvisibleScanPoolMView(nil);
    end
    else
    begin
      FScanPoolView := nil;
    end;
  except
    on E: Exception do
    begin
      if bitfarmApp <> nil then
      begin
        if E <> nil then
          bitfarmLog.Write(llNormal, ltException, 'Message : ' + E.Message, Classname)
        else
          bitfarmLog.Write(llNormal, ltException, 'TThreadedThumbnails.Create', Classname)
      end;

      FreeObjects;

    end;
  end;
end;

procedure TThreadedThumbnails.KillThread;
var
  i: Integer;
begin
  FStopThread := True;
  try
    Self.Terminate();

//15.10.2018 Eigentlich brauchen wir das nicht mehr, wegen Clone, und manchmal strzt hier auch ab    Self.WaitFor;
//18.04.2023 im Importer/Arbeitsplatz strzt leider manchmal immer noch ab
//10.05.2023 Self.WaitFor();  //18.01.2019 nochmal zurck weil ClearGarbage ist jetzt raus

//10.07.2023 ist auch im Execute im finally    FreeObjects;
  except
    bitfarmLog.Write(llNormal, ltInfo, 'KillThread', Classname);
  end;
end;
                      
procedure TThreadedThumbnails.DoThumbs(ActPage: Integer; img: Graphics.TBitmap;
  imgVect: TbfaImageEnVect; imgOri: Graphics.TBitmap);
begin
  if (Self.Terminated) or (bitfarmApp = nil) or (not Assigned(bitfarmApp)) or (bitfarmApp.ClosingMainForm) then
    Exit;
  try
    if Assigned(FThumbnailView) then
    begin
      FThumbnailView.LockPaint;
      try
        if (FThumbnailView.ImageCount = ActPage) then
        begin
          FThumbnailView.InsertImageEx(ActPage);
          FThumbnailView.SetImage(ActPage, img);

          FThumbnailView.Mio.Params[ActPage].BMP_Version := ioBMP_BMOS2V2;
          FThumbnailView.Mio.Params[ActPage].BMP_Compression := ioBMP_RLE;
          
          FThumbnailView.Mio.Params[ActPage].TIFF_Compression := 
            imgVect.IO.Params.TIFF_Compression;
          FThumbnailView.Mio.Params[ActPage].BitsPerSample := 
            imgVect.IO.Params.BitsPerSample;
          FThumbnailView.Mio.Params[ActPage].SamplesPerPixel := 
            imgVect.IO.Params.SamplesPerPixel;

          {$IFDEF IMAGEEN1140}
          FException := FThumbnailView.Exception;
          {$ELSE}
          FException := FThumbnailView.FException;
          {$ENDIF}

          if (FScanPoolView <> nil) then
          begin
            FScanPoolView.LockPaint;
            try
              FScanPoolView.InsertImageEx(ActPage);
              FScanPoolView.SetImage(ActPage, imgOri);

              FScanPoolView.Mio.Params[ActPage].BMP_Version := ioBMP_BMOS2V2;
              FScanPoolView.Mio.Params[ActPage].BMP_Compression := ioBMP_RLE;
              

              FScanPoolView.Mio.Params[ActPage].TIFF_Compression := 
                imgVect.IO.Params.TIFF_Compression;
              FScanPoolView.Mio.Params[ActPage].BitsPerSample := 
                imgVect.IO.Params.BitsPerSample;
              FScanPoolView.Mio.Params[ActPage].SamplesPerPixel :=
                imgVect.IO.Params.SamplesPerPixel;
//12.03.2019 Ticket 17979 -->
              FScanPoolView.Mio.Params[ActPage].DpiX :=
                imgVect.IO.Params.DpiX;
              FScanPoolView.Mio.Params[ActPage].DpiY :=
                imgVect.IO.Params.DPIY;
//12.03.2019 Ticket 17979 <--
            finally
              FScanPoolView.UnLockPaint;
              FScanPoolView.Update;
            end;
          end;
//23.05.2018 Ticket 14576 <--
        end
        else
          FException := True;
      finally
        FThumbnailView.UnLockPaint;
        FThumbnailView.Update;
      end;
    end;
  except
    bitfarmLog.Write(llNormal, ltException, 'ShowThumbnails Exception 0 DoThumbs', Classname);
  end;
end;

procedure TThreadedThumbnails.FreeObjects;
begin
  try
    if Assigned(FDoc) and (not bitfarmApp.ClosingMainForm)  then
    begin
      try
//29.08.2023 -->
////10.05.2023 -->
////        FDoc.Free;
//        Synchronize(
//          procedure
//          begin
//            FDoc.Free;
//          end);
////10.05.2023 <--
        {$IFDEF IMAGEEN_SYNCHRONIZED}
        Synchronize(
          procedure
          begin
            FDoc.Free;
          end);
        {$ELSE}
        FDoc.Free;
        {$ENDIF}
//29.08.2023 <--
      finally
        FDoc := nil;
      end;
    end;
  finally
    FInProcess := False;
  end;
end;

procedure TThreadedThumbnails.Execute;
var
  idx, i, ActPage, MActPage, ListCount, k, l, LoadedPages: Integer;
  foundFtx: Boolean;
  myBmp, OriBmp, tmpbmp : Graphics.TBitmap;
  Fimg: TbfaImageEnVect;
  FLabel: TLabel;
  iebmp2 : TIEBitmap;
begin
  bitfarmLog.DebugBegin(llDebug, 'TThreadedThumbnails.ShowThumbnails', Classname);

  FInProcess := True;

  Synchronize(
    procedure
    begin
      Fimg := nil;
      Fimg := CreateInvisibleVect(nil);

      FLabel := TLabel.Create(nil);
      FLabel.Caption := '';
      FLabel.AutoSize := False;
      FLabel.Width := FThumbnailView.Width - 15;
      FLabel.Height := FLabel.Height * 2;
      FLabel.Alignment := taCenter;
      FLabel.Color := clWhite;
      FLabel.Transparent := False;

      if FScanPoolView <> nil then
        FScanPoolView.LockPaint;
    end);

  try
    if FDoc = nil then
      Exit;

    Synchronize(
      procedure
      var
        h, g: Integer;
        iebmp: TIEBitmap;
        bmp: Graphics.TBitmap;
        Anno: TbfaAnnotation;
        img2: TbfaImageEnVect;
      begin
        try
////24.05.2018 Ticket 14576 -->
          LoadedPages := 0;
          if (FDoc.ID = -1) and (FScanPoolView <> nil) and 
             (FViewer = bitfarmApp.PreviewContainer.ImporterItem.Preview) then
          begin
            if (FScanPoolView.ImageCount = FThumbnailView.ImageCount) or (not FDoc.ScanPoolViewLoaded) then
            begin
              LoadedPages := FScanPoolView.ImageCount;

              if (FScanPoolView.ImageCount <> FThumbnailView.ImageCount) then
              begin
                if FScanPoolView.ImageCount > 0 then
                begin
                  FThumbnailView.Clear;
                  FThumbnailView.LockPaint;
                  try
                    for h := 0 to FScanPoolView.ImageCount - 1  do
                    begin
                      iebmp := TIEBitmap.Create;
//22.02.2024                      bmp := Graphics.TBitmap.Create;
                      try
                        FScanPoolView.CopyToIEBitmap(h, iebmp);

                        Anno := FDoc.GetAnnotation(h, 0, idx);
                        if (Anno <> nil) and (idx > -1) then
                        begin
                          img2 := CreateInvisibleVect(nil);
                          try
                            img2.IO.Params.ImageIndex := 0;
                            {$IFDEF ImageEN1140}
                            img2.Bitmap.Assign(iebmp.VclBitmap);
                            {$ELSE}
                            img2.IO.Bitmap.Assign(iebmp.VclBitmap);
                            {$ENDIF}
                            img2.IO.Params.BMP_Version := ioBMP_BMOS2V2;
                            img2.IO.Params.BMP_Compression := ioBMP_RLE;
                            
                            if DrawOneThumbWithAnnos(img2) then
                            begin
                              FreeAndnil(iebmp);
                              iebmp := TIEBitmap.Create;
                              iebmp.Assign(img2.Bitmap);
                            end;
                          finally
                            img2.RemoveAllObjects;
                            img2.Clear;
//29.08.2023 -->
////10.05.2023 -->
////                            try
////                              for g := img2.LayersCount - 1 downto 0 do
////                                img2.Layers[g].Free;
////                              img2.SelectionMask.Free;
////                            except
////                            end;
//                            img2.Free;
////10.05.2023 <--
                            {$IFDEF IMAGEEN_SYNCHRONIZED}
                            img2.Free;
                            {$ELSE}
                            try
                              for g := img2.LayersCount - 1 downto 0 do
                                img2.Layers[g].Free;
                              img2.SelectionMask.Free;
                            except
                            end;
                            {$ENDIF}
//29.08.2023 <--
                            img2 := nil;
                          end;
                        end;
            
                        FThumbnailView.InsertImageEx(h);
                        FThumbnailView.SetIEBitmap(h, iebmp);

                        FThumbnailView.Mio.Params[h].BMP_Version := ioBMP_BMOS2V2;
                        FThumbnailView.Mio.Params[h].BMP_Compression := ioBMP_RLE;

                        FThumbnailView.Mio.Params[h].TIFF_Compression :=
                          FScanPoolView.MIO.Params[h].TIFF_Compression;
                        FThumbnailView.Mio.Params[h].BitsPerSample :=
                          FScanPoolView.MIO.Params[h].BitsPerSample;
                        FThumbnailView.Mio.Params[h].SamplesPerPixel :=
                          FScanPoolView.MIO.Params[h].SamplesPerPixel;
                      finally
                        FreeAndNil(iebmp);
//22.02.2024                        FreeAndNil(bmp);
                      end;
                    end;
                  finally
                    FThumbnailView.UnLockPaint;
                    FThumbnailView.Update;
                    if not FThumbnailView.Visible then
                      FThumbnailView.Visible := True;
                  end;
                end;
              end;

              if not FThumbnailView.Visible then
                FThumbnailView.Visible := True;

            end
            else
            begin
              if (FScanPoolView.ImageCount > 0) then
                FScanPoolView.Clear;
              if (FThumbnailView.ImageCount > 0) then
                FThumbnailView.Clear;
            end;
          end
          else
           FThumbnailView.Clear;

          Fimg.Clear;
////24.05.2018 Ticket 14576 <--


          FLabel.Caption := '';
          FLabel.Visible := False;

        except
          bitfarmLog.Write(llNormal, ltException, 'ShowThumbnails Exception 1', Classname);
        end;
      end);

    Synchronize(
      procedure
      begin
        FLabel.Parent := FThumbnailView;
//        FLabel.Width := FThumbnailView.Width;
        FLabel.Color := FThumbnailView.Background;
        FLabel.Font.Color := stlFree.TextColor; //13.09.2022
        FLabel.Caption := _('ldt...  ');
        FLabel.Visible := True;
      end);

    FThumbnailView.ThumbWidth := 66;
    FThumbnailView.ThumbHeight := 89;

    if (not FException) and (FDoc.HasPreview) then
    begin
      try
        if (FDoc.ScanPoolView <> nil) and FDoc.ScanPoolViewLoaded then
        begin
          Fimg.IO.Params.ImageIndex := 0;
          iebmp2 := TIEBitmap.Create;
          try
            FDoc.ScanPoolView.CopyToIEBitmap(0, iebmp2);
            {$IFDEF ImageEN1140}
            Fimg.Bitmap.Assign(iebmp2.VclBitmap);
            {$ELSE}
            Fimg.IO.Bitmap.Assign(iebmp2.VclBitmap);
            {$ENDIF}
          finally
            FreeandNil(iebmp2);
          end;
        end else
        begin
          FDoc.DocStream.Position := 0;
//18.04.2023          Fimg.IO.LoadFromStreamTIFF(FDoc.DocStream);
          Fimg.IO.LoadFromStream(FDoc.DocStream);
        end;
      except
        FException := True;
        try
          Synchronize(
            procedure
            begin
              FLabel.Caption := '';
              FLabel.Visible := False;
            end);
        except
          bitfarmLog.Write(llNormal, ltException, 'ShowThumbnails Terminated Exception 3', Classname);
        end;

        bitfarmLog.Write(llNormal, ltException, 'ShowThumbnails Exception 4', Classname);
      end;                                                                              

      if (FDoc.ScanPoolView <> nil) and FDoc.ScanPoolViewLoaded then
      begin
        FTotalPages := FDoc.ScanPoolView.ImageCount;
      end else
      begin
        if Fimg.IO.Params.ImageCount = 0 then
          FTotalPages := 1
        else
          FTotalPages := Fimg.IO.Params.ImageCount;
      end;

      ActPage := LoadedPages;

      while (not Self.Terminated) and (bitfarmApp <> nil) and (not bitfarmApp.ClosingMainForm) and (ActPage < FTotalPages) do
      begin
        OriBmp := Graphics.TBitmap.Create;
        try
          try
            if (Assigned(Fimg) and (not FException)) then
            begin
              foundFtx := False;
              if ((bitfarmApp.ViewerOptions.SearchOptions.FtxPos) and (FFtxPos <> nil) and (FFtxPos.Count > 0)) then
              begin
                k := 0;
                while ((not foundFtx) and (k <= FFtxPos.Count - 1)) do
                begin
                  l := 0;
                  if ((FFtxPos[k].WordList <> nil) and (FFtxPos[k].WordList.Count > 0)) then
                  begin
                    while ((not foundFtx) and (l <= FFtxPos[k].WordList.Count - 1)) do
                    begin
                      if Self.FFtxPos[k].WordList[l].Page = ActPage then
                        foundFtx := True
                      else
                        inc(l);
                    end;
                  end;

                  inc(k);
                end;
              end;
              try
                Synchronize(
                  procedure
                  begin
                    if (not Self.Terminated) and (bitfarmApp <> nil) and (not bitfarmApp.ClosingMainForm) then
                    begin
                      if Fimg.ObjectsCount > 0 then
                        Fimg.RemoveAllObjects;
                    end;
                   end);

                if (FDoc <> nil) and
                   (FDoc.ScanPoolView <> nil) and FDoc.ScanPoolViewLoaded then
                begin
                  Fimg.IO.Params.ImageIndex := ActPage;
                  iebmp2 := TIEBitmap.Create;
                  try
                    Synchronize(
                      procedure
                      begin
                        if (not Self.Terminated) and (bitfarmApp <> nil) and (not bitfarmApp.ClosingMainForm) then
                        begin
                          FDoc.ScanPoolView.CopyToIEBitmap(ActPage, iebmp2);
                          {$IFDEF ImageEN1140}
                          Fimg.Bitmap.Assign(iebmp2.VclBitmap);
                          {$ELSE}
                          Fimg.IO.Bitmap.Assign(iebmp2.VclBitmap);
                          {$ENDIF}
                        end;
                      end);
                  finally
                    FreeandNil(iebmp2);
                  end;
                  OriBmp.Assign(Fimg.Bitmap);
                end else
                begin
                  try
                    if (not Self.Terminated) and (bitfarmApp <> nil) and (not bitfarmApp.ClosingMainForm) then
                    begin
                      Fimg.IO.Params.ImageIndex := ActPage;
                      if (not Self.Terminated) and (FDoc <> nil) then
                        FDoc.DocStream.Position := 0;
                      if (not Self.Terminated) and (FDoc <> nil) then
                        Fimg.IO.LoadFromStream(FDoc.DocStream);
                      if (not Self.Terminated) and (FDoc <> nil) then
                        Fimg.IO.IEBitmap.CopyToTBitmap(OriBmp); //Workaround, sonst strzt ab 2. Seite LoadFromstream ab!
                    end;
                  except
                  end;
                end;

                Synchronize(
                  procedure
                  begin
                    try
                      if (not Self.Terminated) and (bitfarmApp <> nil) and (not bitfarmApp.ClosingMainForm)
                         and (not FStopThread) //18.04.2023
                      then
                      begin
                      // SJ 27.04.2022 - bfacrm:ticket__show__25749
                        if (FDoc <> nil) and (bitfarmApp.GetActivePreview <> nil) then
//18.04.2023                        FDoc.GetAnnotation(ActPage, bitfarmApp.GetActivePreview.CurrentLayer, idx);
                        FDoc.GetAnnotation(ActPage, TbfaPreview(FViewer).CurrentLayer, idx);

                        if (idx > -1)
                           and (not FStopThread) //18.04.2023
                        then
                        begin
                          if not DrawOneThumbWithAnnos(Fimg)
                             and (not FStopThread) //18.04.2023
                          then
                          begin
                            FThumbnailView.Clear;
                            FLabel.Caption := '';
                            FLabel.Visible := False;
                            FreeObjects;
                            Exit;
                          end;
                        end;
                      end;
                    except
                      //
                    end;
                  end);

                if foundFtx then
                begin
                  Synchronize(
                    procedure
                    begin
                      if (not Self.Terminated) and (bitfarmApp <> nil) and (not bitfarmApp.ClosingMainForm) then
                      begin
                        if not DrawOneThumbWithFtxPos(Fimg) then
                        begin
                          FThumbnailView.Clear;
                          FLabel.Caption := '';
                          FLabel.Visible := False;
                          FreeObjects;
                          Exit;
                        end;
                      end;
                    end);
                end;

              except
                bitfarmLog.Write(llNormal, ltException, 'CreateThumbs', ClassName);
              end;

              if Fimg.Bitmap.Width = 0 then
              begin
                Inc(ActPage);
                bitfarmLog.Write(llNormal, ltException, 'ShowThumbnails Exception Continue', Classname);
                Continue;
              end;

              Synchronize(
                procedure
                begin
                  try
                    if (not Self.Terminated) and (bitfarmApp <> nil) and (not bitfarmApp.ClosingMainForm) then
                    begin
                      if not FThumbnailView.Visible then
                      begin
                        if (bitfarmApp.CurrentContainer.Count > 0) and (bitfarmApp.CurrentContainer[0] <> nil) and 
                           (FDoc = bitfarmApp.CurrentContainer[0]) then
                        begin
                          FThumbnailView.Visible := True;
                          FThumbnailView.Parent.Update;
                        end;
                      end;
                      DoThumbs(ActPage, Fimg.Bitmap, Fimg, OriBmp);
                      if (FDoc <> nil) and
                      (FDoc.ScanPoolView <> nil) and FDoc.ScanPoolViewLoaded then
                        FLabel.Caption := _('ldt... ') + #10 + #13 + '(' + IntToStr(ActPage + 1) + 
                          ' / ' + IntToStr(FDoc.ScanPoolView.ImageCount) + ')'
                      else
                        FLabel.Caption := _('ldt... ') + #10 + #13 + '(' + IntToStr(ActPage + 1) + 
                          ' / ' + IntToStr(Fimg.IO.Params.ImageCount) + ')';
                    end;      
                  except
                    //
                  end;
                end);

              if FException then
              begin
                bitfarmLog.Write(llNormal, ltException, 'ShowThumbnails Exception 6', Classname);

                Synchronize(
                  procedure
                  begin
                    try
                      FLabel.Caption := '';
                      FLabel.Visible := False;
                    except
                      bitfarmLog.Write(llNormal, ltException, 'ShowThumbnails Exception 7', Classname);
                    end;
                  end);

                Exit;
              end;

            end;
          except
            Synchronize(
              procedure
              begin
                try
                  FLabel.Caption := '';
                  FLabel.Visible := False;
                except
                  bitfarmLog.Write(llNormal, ltException, 'ShowThumbnails Exception 8', Classname);
                end;
              end);
            bitfarmLog.Write(llNormal, ltException, 'ShowThumbnails Exception 9', Classname);
            Break;
          end;
        finally
          FreeAndNil(OriBmp);
        end;

        if (ActPage = 0) and Assigned(bitfarmApp) and (bitfarmApp.ActivePreview <> nil) then
          bitfarmApp.ActivePreview.ThumbnailView.TabStop := True;

        Inc(ActPage);
      end;
    end
    else if not FException and (FDoc.ID = -1) then
    begin
      myBmp := Graphics.TBitmap.create;
      try
        Synchronize(
          procedure
          var
            FType : Boolean;
          begin
            Fimg.Clear;
            Fimg.IO.Params.OriginalWidth := 0;
            Fimg.IO.Params.OriginalHeight := 0;
            FType := CreateEmptyImage(Fimg, (*15.12.2020'.' + *)FDoc.FileType,
              bitfarmApp.BgColor);//16.12.2020 clWhite);
            myBmp.Assign(Fimg.Bitmap);
            myBmp.PixelFormat := pf24bit;
            if FType then
              IETextOut(myBmp.Canvas, 10, 2200, 0, ' ' + FDoc.FileType + ' ',
                280, clWhite, clBlack, 'Arial', []);
            DoThumbs(ActPage, myBmp, Fimg, Fimg.Bitmap);
          end);
      finally
        myBmp.Free;
      end;
    end;

    if (not Self.Terminated) and //18.04.2023
      (FThumbnailView.SelectedImage = -1) and (FSelectedIdx = -1) then
    begin
      Synchronize(
        procedure
        begin
          try
            FThumbnailView.SelectedImage := 0;
          except
          end;
        end);
    end;
// 14.03.2017 -->
    if (not Self.Terminated) and //18.04.2023
      (FSelectedIdx >= 0) and (FSelectedIdx < FThumbnailView.ImageCount) then  // Ticket 15770
    begin
      Synchronize(
        procedure
        begin
          try
//        <!-- SJ 06.01.2020 - bfacrm:ticket__show__21342
//        Wenn whrend des ladens der Voransicht Auswahl erfolgt ist, nicht auf Index 0
            if (TbfaPreview(FViewer) <> nil) then
              FThumbnailView.SelectedImage := TbfaPreview(FViewer).ActivePage
            else
              FThumbnailView.SelectedImage := FSelectedIdx; // nur Fallback
//        SJ 06.01.2020 -->
//            FThumbnailView.SelectedImage := FSelectedIdx; // SJ 06.01.2020
//            if FScanPoolView <> nil then
//              FScanPoolView.SelectedImage := FSelectedIdx;
          except
          end;
        end);
    end
    else
    begin
      Synchronize(
        procedure
        begin
          try
            if (not Self.Terminated) //18.04.2023
            then
              FThumbnailView.SelectedImage := 0;
          except
          end;
        end);
    end;
// 14.03.2017 <--

  finally
    if Assigned(bitfarmApp) and (not bitfarmApp.ClosingMainForm) then
    begin
      Synchronize(
        procedure
        begin
          if FScanPoolView <> nil then
          begin
            FScanPoolView.UnLockPaint;
            FScanPoolView.Update;
          end;
        end);
      try
        if (FScanPoolView <> nil) and Assigned(FOriDoc) then
        begin
          FOriDoc.ScanPoolView := FScanPoolView;
          if (not Self.Terminated) and (not FException) then
            FOriDoc.ScanPoolViewLoaded := True;
        end;
      except
      end;

      if (Self.Terminated) or (FException) then
      begin
        Synchronize(
          procedure
          begin
            try
              FLabel.Visible := False;
              FLabel.Caption := '';
            except
              bitfarmLog.Write(llNormal, ltException, 'ShowThumbnails Exception 1', Classname);
            end;
          end);
      end
      else
      begin
        Synchronize(
          procedure
          begin
            FLabel.Caption := '';
            FLabel.Visible := False;
            FreeAndNil(FLabel);
          end);
      end;

      FreeObjects;
      FStopThread := False;

      Fimg.RemoveAllObjects;
      Fimg.Clear;

//29.08.2023 -->
//      //10.05.2023 zum testen -->
////  //      FreeAndNil(Fimg);  /// Access Violation beim zB Ctrl+C in anderen Anwendungen und strzt ab!!!
////
////  //08.08.2017 Workaround, sonst wird das Speicher immer grer-->
////      try
////        for i := Fimg.LayersCount - 1 downto 0 do
////          Fimg.Layers[i].Free;
////        Fimg.SelectionMask.Free;
////      except
////      end;
////  //08.08.2017 <--
////
////      Fimg := nil;
//
//      try
//        Synchronize(
//          procedure
//          begin
//            Fimg.Free;
//          end);
//      finally
//        Fimg := nil;
//      end;
//      //10.05.2023 <--

      {$IFDEF IMAGEEN_SYNCHRONIZED}
      try
        Synchronize(
          procedure
          begin
            Fimg.Free;
          end);
      finally
        Fimg := nil;
      end;

      {$ELSE}
      try
        for i := Fimg.LayersCount - 1 downto 0 do
          Fimg.Layers[i].Free;
        Fimg.SelectionMask.Free;
      except
      end;

      Fimg := nil;
      {$ENDIF}
//29.08.2023 <--
      bitfarmLog.DebugEnd(llDebug, 'TThreadedThumbnails.ShowThumbnails ThreadID:' + IntToStr(Self.ThreadID), Classname);
    end;
  end;
end;

end.

