//-----------------------------------------------------------------------------
// Projekt:    Bitfarm-Archiv Administrator
// Entwickler: Frey
// Erstellt:   2009-09-11
// Name:       u_ActivediretoryUtils
// Typ:        Unit
// Zweck:      Hilfroutinen fr Active Directory
//-----------------------------------------------------------------------------
//
//----------------------------------------------------------------------------}

unit u_ActivediretoryUtils;

{$optimization off}
//{$RANGECHECKS ON}

{###############################} INTERFACE {###############################}

uses
   Classes, StrUtils, SysUtils, ActiveX, Variants, ComObj, Forms,
   ActiveDs_TLB, adshlp, gnugettext;


var bAdPause:Boolean;

function AduGetUserName(AOwner: TComponent): WideString;
function AduGetFullName(const Domain, UserName: string): string;
function AduGetEmail(const sUser, lsmyq: string): string;

function GetLDAPClass(AValue: WideString): WideString;
function GetLDAPName(AValue: WideString): WideString;
function GetLDAPDisplayname(AValue: WideString): WideString;
function GetLDAPObjectname(AValue: WideString): WideString;
function EscapeDNName(AValue: WideString): WideString;
function EscapeLDAPPath(AValue: WideString): WideString;


type
   TADsHelper = class
    public
      bLogRq: Boolean;
      sADRoot: string;    //[2609]
      LdapTextDomain : string;
      fiPause: integer;
      constructor Create(ADomain: string);
      function LdapString(Item, Range:string):string;
      procedure GetGroups(List:TStringList);
      procedure GetGroupInfo(const Gruppe:string; List:TStringList);
      procedure GetGroupMembersNameAndMail(const Gruppe, adclass:string; ListName,ListMail,MemberOf,ListDispName:TStringList);
      procedure GetGroupMembersNameAndMail2(const Gruppe, adclass:string; ListName,ListMail,MemberOf,ListDispName:TStringList);
      procedure GetUserInfo(const User:string; List:TStringList);
      function GetUserMail(const User:string):string;
      procedure GetLdap(List:TStringList; Angabe:string);
      procedure GetLdapGroup(List:TStringList; Angabe:string);
      procedure GetTest(List:TStringList; Angabe1,Angabe2,Angabe3:string);
      procedure AddMemberToGroupNameAndMail(disp: IADs);
      procedure AddMemberToGroupNameUndMail2(disp: IADs; list1,list2,list3,list4: TStringList);
      procedure LDAPHasChild(disp: IADs);
    private
      FDomain: string;
      ActList,ActList2,ActList3,ActList4: TStringList;
      AktGruppe: string;
      gHasChild: Boolean;
      procedure AddToGroupList(disp: IADs);
      procedure AddMemberToGroupList(disp: IADs);
      procedure AddToLdapList(disp: IADs);
      procedure AddToLdapGroupList(disp: IADs);

      procedure GetSubGroupMembers(const Gruppe, LDAP:string);
      procedure AddToTestList(disp: IADs);
   end;

var iAddMemberToGroupNameUndMailCounter: integer;

{############################} IMPLEMENTATION {#############################}

uses
  bfaLog, MainDataModule, bfaCommonUtils, bfaCommonMethods, bfaCommonDialogs, bfaBaseObjects;

//-----------------------------------------------------------------------------
function AduGetUserName(AOwner: TComponent): WideString;
var
   ADSystemInfo: TADSystemInfo;
begin
   bitfarmLog.Write(llNormal, ltDebug, 'AduGetUserName - start', '');

   ADSystemInfo := TADSystemInfo.Create(AOwner);
   try
      try
         Result := ADSystemInfo.UserName;
      except on e: exception do
         begin
            bitfarmLog.Write(llNormal, ltDebug, 'error AduGetUserName.ADSystemInfo ' + e.Message);
         end;
      end;
   finally
      //ADSystemInfo.FreeOnRelease;
      {
      aus der OH: FreeOnRelease wird aufgerufen, wenn eine von der Komponente
      implementierte Schnittstelle freigegeben wird. FreeOnRelease wird intern
      verwendet und ruft die entsprechende Schnittstellenmethode auf. Es ist
      nicht erforderlich, FreeOnRelease direkt aufzurufen.
      }
      ADSystemInfo.Free;
   end;
   bitfarmLog.Write(llNormal, ltDebug, 'AduGetUserName - end Result=' + Result);
end;


//-----------------------------------------------------------------------------
function AduGetFullName(const Domain, UserName: string): string;
var
   _aUser: IADsUser;
   lss: string;
begin
   bitfarmLog.Write(llNormal, ltDebug, 'AduGetFullName(' + Domain + ',' + UserName + ')');
   try
      lss := 'WinNT://' + Domain + '/' + UserName;
      Result := '';
      try
         if SUCCEEDED(ADsGetObject(lss, IADsUser, _aUser)) then
            try
               Result := _aUser.FullName; // z.B. 'WinNT://INTRANET/h.schneider'
            finally
               _aUser._Release;
               _aUser := nil; // brauch man das? Wenn das nicht in einer function wie hier ist, dann ja
            end;
      except on e: exception do
            bitfarmLog.Write(llNormal, ltDebug, 'AduGetFullName: error ADsGetObject (' + lss + ')' + e.Message);
      end;
   finally
      bitfarmLog.Write(llNormal, ltDebug, 'AduGetFullName(' + Domain + ',' + UserName + ')');
   end;
end;

//-----------------------------------------------------------------------------
function AduGetEmail(const sUser, lsmyq: string): string;
var
   _aUser: IADsUser;
   lsADsGetObject: string;
begin
   Result := '';
   try
      bitfarmLog.Write(llNormal, ltDebug, 'AduGetFullName(' + sUser + ',' + lsmyq + ')');
      try
         lsADsGetObject := 'LDAP://CN=' + EscapeDNName(sUser) + ',CN=Users,' + EscapeDNName(lsmyq);
         bitfarmLog.Write(llNormal, ltDebug, 'AduGetEmail lsADsGetObject=' + lsADsGetObject);
         ADsGetObject(lsADsGetObject, IADsUser, _aUser);
         bitfarmLog.Write(llNormal, ltDebug, 'btn_updateClick - nach ADsGetObject lsADsGetObject=' + lsADsGetObject);
         try
            try
               Result := _aUser.Get_EmailAddress;
            except
               on e: Exception do
                  bitfarmLog.Write(llNormal, ltDebug, 'error Get_EmailAddress ' + e.Message);
            end;
         except
            on e: Exception do
            begin
               bitfarmLog.Write(llNormal, ltDebug, 'error ADsGetObject (' + lsADsGetObject + ')' + e.Message);
            end;
         end;
      finally
         _aUser._Release;
         _aUser := nil; // brauch man das? Wenn das nicht in einer function wie hier ist, dann ja
      end;
   finally
      bitfarmLog.Write(llNormal, ltDebug, 'AduGetFullName(' + sUser + ',' + lsmyq + ')');
   end;
end;


function GetLDAPClass(AValue: WideString): WideString;
var
  p : Integer;
begin
  Result := AValue;
  if (LeftStr(AValue,2) = '+ ') or (LeftStr(AValue,2) = '  ') then
    AValue := Copy(AValue,3, Length(AValue)-2);

  p := Pos(':',AValue);
  if p=0 then Exit;

  Result := Copy(AValue,1,p-1);
end;

function GetLDAPDisplayname(AValue: WideString): WideString;
var
  p : Integer;
begin
  Result := AValue;
  p := Pos('=',AValue);
  if p=0 then Exit;

  Result := RightStr(AValue,length(AValue) - p);
  Result := Trim(Result);
end;

function GetLDAPName(AValue: WideString): WideString;
var
  p : Integer;
begin
  Result := AValue;
  p := Pos(':',AValue);
  if p=0 then Exit;

  Result := RightStr(AValue,length(AValue) - p);
  Result := Trim(Result);
end;

function GetLDAPObjectname(AValue: WideString): WideString;
var
  p : Integer;
begin
  Result := AValue;

  p := Pos(',',Result);
  if p=0 then Exit;

  Result := Copy(Result,1,p-1);
end;


function EscapeDNName(AValue: WideString): WideString;
var
  p : Integer;
begin
  Result := '';
  // erst die Sonderzeichen die fr DistinguishedNames escaped werden mssen (plus der / fr ADSI Abfragen)
  for p := 1 to length(AValue) do
  begin
    if AValue[p] in [',', '\', '#', '+', '<', '>', ',', ';', '"', '=', '/'] then
      Result := Result + '\' + AValue[p]
    else
      Result := Result + AValue[p];
  end;

  // jetzt auf fhrendes # prfen
  if Result[1] = '#' then
    Result := '\#' + Copy(Result, 2, Length(Result) -1);

  // jetzt alle Fhrenden oder Nachfolgenden Leerzeichen escapen
  p := 1;
  while p < length(Result) do
  begin
    if Result[p] <> ' ' then
      Break;

    if Result[p] = ' ' then
    begin
      Result := Copy(Result, 0, p -1) + '\ ' + Copy(Result,p +1, Length(Result) -p);
      inc(p);
    end;

    inc(p);
  end;

  p := length(Result);
  while p > 1 do
  begin
    if Result[p] <> ' ' then
      Break;

    if Result[p] = ' ' then
    begin
      Result := Copy(Result, 0, p -1) + '\ ' + Copy(Result,p +1, Length(Result) -p);

    end;

    dec(p);
  end;
end;

function EscapeLDAPPath(AValue: WideString): WideString;
var
  p : Integer;
  parts : TStringList;
  part, escaped: WideString;
begin
  parts := TStringList.Create;
  parts.Delimiter := ',';
  parts.StrictDelimiter := True;
  parts.DelimitedText := AValue;

  for p := 0 to parts.Count -1 do
  begin
    part := GetLDAPDisplayname(parts[p]);
    escaped := EscapeDNName(part);
    if part <> escaped then
      parts[p] := StringReplace(parts[p], part, escaped, [rfReplaceAll]);
  end;

  Result := parts.CommaText;
  FreeAndNil(parts);
end;

//-----------------------------------------------------------------------------
constructor TADsHelper.Create(ADomain: string);
//var p:integer;
begin
   fDomain := ADomain;

   // p:=Pos('.',fDomain);
   // LdapTextDomain := 'dc='+copy(fDomain,1,p-1)+',dc='+copy(fDomain,p+1,maxint);
   LdapTextDomain := 'dc='+StringReplace(fDomain, '.', ',dc=', [rfReplaceAll]);
   gHasChild := False;
end;


//-----------------------------------------------------------------------------
procedure TADsHelper.LDAPHasChild(disp: IADs);
var
  lsClass : WideString;
begin
  lsClass := disp.Class_;
  if  (lsClass='group') or (lsClass='container') or (lsClass='organizationalUnit') then
    gHasChild := True;


end;

function TADsHelper.LdapString(Item, Range:string):string;
begin
   Result := Item;
   if Item>'' then Result := Result + ',';
   Result := Result + Range;
   if Range>'' then Result := Result + ',';
   Result := Result + LdapTextDomain;
end;

//-----------------------------------------------------------------------------
procedure TADsHelper.GetGroups(List:TStringList);
var ls:string;  dom_groups: IADsContainer;
begin
   bitfarmLog.Write(llNormal, ltDebug, 'GetGroups begin', ClassName);
   try
      ActList := List;
      ActList.Clear;
      //ls := 'WinNT://' + FDomain;
      ls := sADRoot + EscapeDNName(FDomain);    //[2609]
      ADsGetObject(ls, IADsContainer, dom_groups);
      if bLogRq then
        bitfarmLog.Write(llNormal, ltDebug, 'A-ADsGetObject('''+ls+''', IADsContainer, dom_groups);', ClassName);
      dom_groups.Filter := VarArrayOf(['Group']);
      ADsEnumerateObjects(dom_groups, AddToGroupList);
   except on e: exception do
      begin
         bitfarmLog.Write(llNormal, ltDebug, 'Error on GetGroups ' + e.message, ClassName);
      end;
   end;
   bitfarmLog.Write(llNormal, ltDebug, 'GetGroups end', ClassName);
end;

procedure TADsHelper.AddToGroupList(disp: IADs);
begin
   if lowercase(Disp.Class_) = 'group' then
   begin
      ActList.Add(disp.Name);
   end;
end;

//-----------------------------------------------------------------------------
procedure TADsHelper.GetGroupInfo(const Gruppe:string; List:TStringList);
var
   ls: string;
   dom_group: IADsGroup;
   //Members : IADsMembers;
begin
//   GgfLogEvntBeg('GetGroupInfo begin');
   try
      ActList := List;
      ActList.Clear;
      //ls := 'WinNT://' + FDomain + '/' + Gruppe;
      //ls := 'LDAP://' + Gruppe;
      ls := sADRoot + EscapeLDAPPath(Gruppe);     //[2609]
      ADsGetObject(ls , IADsGroup, dom_group);
      if bLogRq then
        bitfarmLog.Write(llNormal, ltDebug, 'B-ADsGetObject('''+ls+''', IADsGroup, dom_group);');

      //List.Add(dom_group.AdsPath);
      //List.Add(dom_group.Description);
      //List.Add(dom_group.Name);
      AdsEnumerateMembers(dom_group.Members, AddMemberToGroupList);

      // Members._Release;
   except on e: exception do
      bitfarmLog.Write(llNormal, ltDebug, 'Error on GetGroups ' + e.message);
   end;
   bitfarmLog.Write(llNormal, ltDebug, 'GetGroupInfo end');
end;

procedure TADsHelper.AddMemberToGroupList(disp: IADs);
var ls: string;  //User: IADsUser;
begin
   //if lowercase(Disp.Class_) = 'group' then
   begin
      //ls := disp.Name;
      //User := disp as IADsUser;
      //ls := ls + ' | ' + User.EmailAddress;
      //ls := ls + ' | ' + disp.ADsPath;
      ls := (disp as IADsUser).FullName;
      ActList.Add(ls);
   end;
end;

//-----------------------------------------------------------------------------
procedure TADsHelper.GetGroupMembersNameAndMail(const Gruppe, adclass:string; ListName,ListMail,MemberOf,ListDispName:TStringList);
var
   ls: string;   hr:HRESULT;
   dom_group: IADsGroup;
   dom_container: IADsContainer;
   dom_ou: IADsOU;
   //Members : IADsMembers;
begin
   bitfarmLog.DebugBegin(llNormal, 'GetGroupMembersNameAndMail begin Gruppe='+ Gruppe +' adclass='+ adclass, ClassName);
   CoInitialize(nil);
   try
      try
         ActList := ListName;
         ActList2 := ListMail;
         ActList3 := MemberOf;
         ActList4 := ListDispName;
         AktGruppe := Gruppe;
         //ls := 'LDAP://' + Gruppe;
         ls := sADRoot + EscapeLDAPPath(Gruppe);      //[2609]
//         ls := sADRoot + Gruppe;      //[2609]

         bitfarmLog.Write(llNormal, ltDebug, 'C-ADsGetObject('+ls+')', ClassName);

         if adclass = 'group' then
           hr := ADsGetObject(ls,IID_IADsGroup, dom_group)
         else if adclass = 'container' then
//           hr := ADsGetObject(ls, IID_IADsContainer, dom_container)
           hr := ADsGetObject(ls, IID_IADsContainer, dom_container)
         else if adclass = 'organizationalUnit' then
           hr := ADsGetObject(ls, IID_IADsContainer, dom_container)
         else
            Exit;

         bitfarmLog.Write(llNormal, ltDebug, 'C-ADsGetObject HRESULT='+ IntToStr(hr), ClassName);
         if (SUCCEEDED(hr)) then
         begin
            if bLogRq then
              bitfarmLog.Write(llNormal, ltDebug, 'C-ADsGetObject('''+ls+''', IADsGroup, dom_group.Description)',ClassName);

            bitfarmLog.DebugBegin(llNormal, 'vor AdsEnumerateMembers AktGruppe='+AktGruppe, ClassName);
//            CoInitialize(nil);
//            try
              if adclass = 'group' then
  	            AdsEnumerateMembers(dom_group.Members, AddMemberToGroupNameAndMail)
              else if (adclass = 'container') then
//                AdsEnumerateObjects(ls, AddMemberToGroupNameAndMail);
                AdsEnumerateObjects(dom_container, AddMemberToGroupNameAndMail)
              else if (adclass = 'organizationalUnit') then
                AdsEnumerateObjects(dom_container, AddMemberToGroupNameAndMail);
//            finally
//              CoUninitialize;
//            end;

            bitfarmLog.DebugEnd(llNormal, 'nach AdsEnumerateMembers AktGruppe='+AktGruppe, ClassName);

//            dom_group._Release;
         end
         else
            bitfarmLog.Write(llNormal, ltDebug, 'not Succeeded ADsGetObject('''+ls+''', ...', ClassName);
      except on e: exception do
         begin
			   bitfarmLog.Write(llNormal, ltDebug, 'Error on GetGroups ' + e.message, ClassName);
            BfShowMessage(Format(_('Die Gruppe %s konnte nicht im AD gefunden werden. Entweder wurde sie entfernt oder der Pfad hat sich gendert.'),[ls]));
         end;
      end;
   finally

      CoUninitialize;
      bitfarmLog.DebugEnd(llNormal, 'GetGroupMembersNameAndMail end', ClassName);
   end;
end;

//-----------------------------------------------------------------------------
procedure TADsHelper.GetGroupMembersNameAndMail2(const Gruppe, adclass:string; ListName,ListMail,MemberOf,ListDispName:TStringList);
var
   ls: string;   hr:HRESULT;
   dom_group: IADsGroup;
   //Members : IADsMembers;
begin
   bitfarmLog.Write(llNormal, ltDebug, 'GetGroupMembersNameAndMail2 begin Gruppe='+Gruppe, ClassName);
//   CoInitialize(nil);
   try
      try
         ls := sADRoot + EscapeLDAPPath(Gruppe);
         bitfarmLog.Write(llNormal, ltDebug, 'C-ADsGetObject('+ls+')', ClassName);
//         hr := ADsGetObject(ls, IADsGroup, dom_group);

         if adclass = 'group' then
           hr := ADsGetObject(ls, IADsGroup, dom_group)
         else if adclass = 'container' then
           hr := ADsGetObject(ls, IID_IADsContainer, dom_group)
         else if adclass = 'organizationalUnit' then
           hr := ADsGetObject(ls, IID_IADsOU, dom_group)
         else
            Exit;


         bitfarmLog.Write(llNormal, ltDebug, 'C-ADsGetObject HRESULT='+ IntToStr(hr), ClassName);
         if (SUCCEEDED(hr)) then
         begin
            if bLogRq then
              bitfarmLog.Write(llNormal, ltDebug, 'C-ADsGetObject('''+ls+''', IADsGroup, dom_group.Description)', ClassName);

//            CoInitialize(nil);
//            try
	            AdsEnumerateMembers2(dom_group.Members, AddMemberToGroupNameUndMail2, ListName,ListMail,MemberOf,ListDispName);
//            finally
//              CoUninitialize;
//            end;

         end
         else
            bitfarmLog.Write(llNormal, ltDebug, 'not Succeeded ADsGetObject('''+ls+''', ...', ClassName);
      except on e: exception do
         begin
			   bitfarmLog.Write(llNormal, ltDebug, 'Error on GetGroups ' + e.message, ClassName);
            BfShowMessage(Format(_('Die Gruppe %s konnte nicht im AD gefunden werden. Entweder wurde sie entfernt oder der Pfad hat sich gendert.'),[ls]));
         end;
      end;
   finally
//      CoUninitialize;
      bitfarmLog.Write(llNormal, ltDebug, 'GetGroupMembersNameAndMail2 end', ClassName);
   end;
end;



procedure TADsHelper.AddMemberToGroupNameAndMail(disp: IADs);
var
  lsClass, lsName, lsMail, lsGrp, dispName: WideString;
  usr:IADsUser;
  v:variant;
  vt:TVarType; //User: IADsUser;
  isLocked: Boolean;
begin
  bitfarmLog.DebugBegin(llNormal, 'AddMemberToGroupNameAndMail begin '+IntToStr(iAddMemberToGroupNameUndMailCounter)+' AktGruppe='+AktGruppe, ClassName);
//  Application.ProcessMessages;
  try
    disp.GetInfo;
    lsClass := lowercase(disp.Class_);
    bitfarmLog.Write(llNormal, ltDebug, 'AddMemberToGroupNameAndMail lsClass='+lsClass, ClassName);

    if lsClass = 'user' then
    begin
       try
          lsName := disp.Get('sAMAccountName');
       except
          on e: Exception do
             bitfarmLog.Write(llNormal, ltDebug, 'error AddMemberToGroupNameAndMail Get(''sAMAccountName'') ' + e.Message, ClassName);
       end;
       // wenn User schon in Liste dann nicht nochmals rein
       if ActList.IndexOf(lsName) >= 0 then
       begin
          bitfarmLog.Write(llNormal, ltDebug, 'AddMemberToGroupNameAndMail schon drin - lsName='+lsName);
          Exit;
       end;

       isLocked := False;
       try
          v := disp.Get('userAccountControl');
       except
          on e:EOleException do
             bitfarmLog.Write(llNormal, ltDebug, 'Keine userAccountControl vorhanden' + e.Message);
          on e: Exception do
             bitfarmLog.Write(llNormal, ltDebug, 'error AddMemberToGroupNameUndMail Get(''userAccountControl'') ' + e.Message, ClassName);

       end;
        bitfarmLog.Write(llNormal, ltDebug, 'AddMemberToGroupNameUndMail userAccountControl VarType holen...');
        vt := VarType(v);
        bitfarmLog.Write(llNormal, ltDebug, 'AddMemberToGroupNameUndMail userAccountControl VarType='+IntToStr(vt));

        bitfarmLog.Write(llNormal, ltDebug, 'AddMemberToGroupNameAndMail Add lsName='+lsName);
        if vt = varInteger then
          ActList.AddObject(lsName, TbfaInt.Create(v))
        else
         // User hinzu
         ActList.Add(lsName);

       try
//          dispName := disp.Get('displayName');
          v := disp.Get('displayName');
       except
          on e:EOleException do
             bitfarmLog.Write(llNormal, ltDebug, 'Keine displayName vorhanden' + e.Message);
          on e: Exception do
             bitfarmLog.Write(llNormal, ltDebug, 'error AddMemberToGroupNameAndMail Get(''displayName'') ' + e.Message, ClassName);
       end;

        bitfarmLog.Write(llNormal, ltDebug, 'AddMemberToGroupNameAndMail VarType holen...');
        vt := VarType(v);
        bitfarmLog.Write(llNormal, ltDebug, 'AddMemberToGroupNameAndMail VarType='+IntToStr(vt));

        case vt of
           varEmpty,varNull : dispName := '';
           varOleStr, varString, varStrArg: dispName := v;
        else
            if VarArrayDimCount(v) > 0 then
              dispName := v[0];
        end;

       bitfarmLog.Write(llNormal, ltDebug, 'AddMemberToGroupNameAndMail Add dispName='+dispName);
       ActList4.Add(dispName);

       //E-Mail abfragen
       lsMail:='';

       try
          bitfarmLog.Write(llNormal, ltDebug, 'AddMemberToGroupNameAndMail mail 0');
          lsMail:='';
          v := Null;
          //v := (disp as IADsUser).EmailAddress;
          //v := disp.Get('mail');
          try
            v := disp.GetEx('mail');

          except
            on E:EOleException do
            begin
               v := NULL;
               bitfarmLog.Write(llNormal, ltDebug, 'Keine Email vorhanden');
            end;
          end;

          bitfarmLog.Write(llNormal, ltDebug, 'AddMemberToGroupNameAndMail VarType holen...');
          if not VarIsNull(v) then
          begin
            vt := VarType(v);
            bitfarmLog.Write(llNormal, ltDebug, 'AddMemberToGroupNameAndMail VarType='+IntToStr(vt));

            case vt of
               varEmpty,varNull : lsMail := '';
               varOleStr, varString, varStrArg: lsMail := v;
            else
                if VarArrayDimCount(v) > 0 then
                  lsMail := v[0];
            end;
          end else begin
            lsMail := '';

          end;
          bitfarmLog.Write(llNormal, ltDebug, 'AddMemberToGroupNameAndMail lsMail='+lsMail);
       except
          on e: Exception do
             bitfarmLog.Write(llNormal, ltDebug, 'error AddMemberToGroupNameAndMail Get_EmailAddress ' + e.Message);
       end;
       bitfarmLog.Write(llNormal, ltDebug, 'AddMemberToGroupNameAndMail Add lsMail='+lsMail, ClassName);
       ActList2.Add(lsMail);

       //MemberOf abfragen
       if Assigned(ActList3) then
       begin
         lsGrp := AktGruppe;
         bitfarmLog.Write(llNormal, ltDebug, 'AddMemberToGroupNameAndMail Add lsGrp='+lsGrp, ClassName);
         ActList3.Add(lsGrp);
       end;
//    end
//    else
//    if lowercase(lsClass) = 'group' then    //Untergruppen werden nicht mhr aufgelst
//    begin
//       //Pfad abfragen
////       lsName:='';
////       try
////          lsName := (disp as IADsGroup).Get('distinguishedName');
////       except
////          on e: Exception do
////             bitfarmLog.Write(llNormal, ltDebug, 'error AddMemberToGroupNameAndMail Get(''distinguishedName'') ' + e.Message, ClassName);
////       end;
////
////       GetSubGroupMembers(lsName, disp.ADsPath);
//    end
//    else
//    if lowercase(lsClass) = 'foreignsecurityprincipal' then
//    begin
//      //********   hier ggf eine Behandlung fr foreignsecurityprincipal    hin
//      BfShowMessage(_('Hinweis ! Es befinden sich Objekte in diesem Container die\n'
//                    + 'nicht aufgelst werden knnen (ForeignSecurityPrincipals)\n[')
//                    + disp.Name + _(']\nBitte berprfen Sie ihre AD-Konfiguration'));
//      bitfarmLog.Write(llNormal, ltDebug, _('AddMemberToGroupNameAndMail: FSP enthalten [') + disp.Name + ']', ClassName);
//    end
//    else
//    // alles was nicht user oder group ist fgen wir nicht hinzu
//    begin
//    //       ActList.Add(disp.Name);
//    //       ActList2.Add('-');
//    //       ActList3.Add('-');    //[2609]
    end;
  finally
    inc(iAddMemberToGroupNameUndMailCounter);
    bitfarmLog.DebugEnd(llNormal, 'AddMemberToGroupNameAndMail end AktGruppe='+AktGruppe, ClassName);
  end;
end;



procedure TADsHelper.AddMemberToGroupNameUndMail2(disp: IADs; list1,list2,list3,list4: TStringList);
var
  lsClass, lsName, lsMail, lsGrp, dispName: WideString;
  usr:IADsUser;
  v:variant;
  vt:TVarType; //User: IADsUser;
begin
   try
     disp.GetInfo;
     lsClass := lowercase(Disp.Class_);
     bitfarmLog.Write(llNormal, ltDebug, 'AddMemberToGroupNameUndMail2 lsClass='+lsClass, ClassName);

     if lsClass = 'user' then
     begin
       try
          (* ls := (disp as IADsUser).FullName;
          ls := (disp as IADsUser).Name;
          ls := (disp as IADsUser).AdsPath;           *)
          //lsName := (disp as IADsUser).Get('sAMAccountName');
          lsName := disp.Get('sAMAccountName');
       except
          on e: Exception do
             bitfarmLog.Write(llNormal, ltDebug, 'error AddMemberToGroupNameUndMail2 Get(''sAMAccountName'') ' + e.Message, ClassName);
       end;
       // wenn User schon in Liste dann nicht nochmals rein
       if list1.IndexOf(lsName) >= 0 then
       begin
          bitfarmLog.Write(llNormal, ltDebug, 'AddMemberToGroupNameUndMail2 schon drin - lsName='+lsName, ClassName);
          Exit;
       end;

       // User hinzu
       bitfarmLog.Write(llNormal, ltDebug, 'AddMemberToGroupNameUndMail2 Add lsName='+lsName, ClassName);
       list1.Add(lsName);

       try
//          dispName := disp.Get('displayName');
          v := disp.Get('displayName');
       except
          on e:EOleException do
             bitfarmLog.Write(llNormal, ltDebug, 'Keine displayName vorhanden' + e.Message);
          on e: Exception do
             bitfarmLog.Write(llNormal, ltDebug, 'error AddMemberToGroupNameUndMail2 Get(''displayName'') ' + e.Message, ClassName);
       end;

        bitfarmLog.Write(llNormal, ltDebug, 'AddMemberToGroupNameUndMail2 VarType holen...');
        vt := VarType(v);
        bitfarmLog.Write(llNormal, ltDebug, 'AddMemberToGroupNameUndMail2 VarType='+IntToStr(vt));

        case vt of
           varEmpty,varNull : dispName := '';
           varOleStr, varString, varStrArg: dispName := v;
        else
            if VarArrayDimCount(v) > 0 then
              dispName := v[0];
        end;

       bitfarmLog.Write(llNormal, ltDebug, 'AddMemberToGroupNameUndMail2 Add dispName='+dispName);
       list4.Add(dispName);


       //E-Mail abfragen
       lsMail:='';
       try

          bitfarmLog.Write(llNormal, ltDebug, 'AddMemberToGroupNameUndMail2 mail 0', ClassName);
          lsMail:='';

          try
            v := disp.GetEx('mail');

          except
            on e:EOleException do
			      begin
           		 bitfarmLog.Write(llNormal, ltDebug, 'Keine Email vorhanden 2', ClassName);
             end;
          end;

          bitfarmLog.Write(llNormal, ltDebug, 'AddMemberToGroupNameUndMail2 X', ClassName);
          vt := VarType(v);
          bitfarmLog.Write(llNormal, ltDebug, 'AddMemberToGroupNameUndMail2 VarType='+IntToStr(vt), ClassName);

          case vt of
             varEmpty,varNull : ;
             varOleStr, varString, varStrArg: lsMail := v;
          else
              if VarArrayDimCount(v)>0 then
                lsMail := v[0]
          end;
          bitfarmLog.Write(llNormal, ltDebug, 'AddMemberToGroupNameUndMail2 lsMail='+lsMail, ClassName);
       except
          on e: Exception do
             bitfarmLog.Write(llNormal, ltDebug, 'error AddMemberToGroupNameUndMail2 Get_EmailAddress ' + e.Message, ClassName);
       end;
       bitfarmLog.Write(llNormal, ltDebug, 'AddMemberToGroupNameUndMail2 Add lsMail='+lsMail, ClassName);
       list2.Add(lsMail);

       //MemberOf abfragen
       if Assigned(list3) then
       begin
          lsGrp := AktGruppe;
          bitfarmLog.Write(llNormal, ltDebug, 'AddMemberToGroupNameUndMail2 Add lsGrp='+lsGrp, ClassName);
          list3.Add(lsGrp);
       end;
     end else if lowercase(lsClass) = 'group' then
     begin
       //Pfad abfragen
       lsName:='';
       try
          lsName := (disp as IADsGroup).Get('distinguishedName');
       except
          on e: Exception do
             bitfarmLog.Write(llNormal, ltDebug, 'error AddMemberToGroupNameUndMail2 Get(''distinguishedName'') ' + e.Message, ClassName);
       end;

//       BfShowMessage('GetSubGroupMembers');
       //GetSubGroupMembers(lsName, disp.ADsPath);
     end else if lowercase(lsClass) = 'foreignsecurityprincipal' then
     begin
      BfShowMessage(_('Hinweis ! Es befinden sich Objekte in diesem Container die\n'
                    + 'nicht aufgelst werden knnen (ForeignSecurityPrincipals)\n[')
                    + disp.Name + _(']\nBitte berprfen Sie ihre AD-Konfiguration'));
      bitfarmLog.Write(llNormal, ltDebug, _('AddMemberToGroupNameAndMail: FSP enthalten [') + disp.Name + ']', ClassName);
     end
     else
      //   ********   hier ggf eine Behandlung fr foreignsecurityprincipal    hin
      begin
         list1.Add(disp.Name);
         list2.Add('-');
         list3.Add('-');    //[2609]
      end;
   finally

   end;
end;


procedure TADsHelper.GetSubGroupMembers(const Gruppe, LDAP:string);
var
   ls: string;
   dom_group: IADsGroup;
   //Members : IADsMembers;
begin
   bitfarmLog.Write(llNormal, ltDebug, 'GetSubGroupMembersNameUndMail begin Gruppe='+Gruppe+' LDAP='+LDAP, ClassName);
   try
      try
         AktGruppe := Gruppe;
         //ls := 'LDAP://' + Gruppe;
         ls := EscapeLDAPPath(LDAP);
         ADsGetObject(ls , IADsGroup, dom_group);
         if bLogRq then
            bitfarmLog.Write(llNormal, ltDebug, 'D-ADsGetObject('''+ls+''', IADsGroup, dom_group);', ClassName);

         AdsEnumerateMembers(dom_group.Members, AddMemberToGroupNameAndMail);

         // Members._Release;
      except on e: exception do
         bitfarmLog.Write(llNormal, ltDebug, 'Error on GetSubGroupMembersNameUndMail ' + e.message, ClassName);
      end;
   finally
      bitfarmLog.Write(llNormal, ltDebug, 'GetSubGroupMembersNameUndMail end', ClassName);
   end;
end;

//-----------------------------------------------------------------------------
procedure TADsHelper.GetUserInfo(const User:string; List:TStringList);
var ls:string;  dom_user: IADsUser;
begin
   bitfarmLog.Write(llNormal, ltDebug, 'GetUserInfo begin User='+User, ClassName);
   try
      ActList := List;
      //ls := 'WinNT://' + FDomain + '/' + User;
      //ls := 'LDAP://CN='+User+',CN=users,'+LdapTextDomain;
      ls := sADRoot + 'CN='+ EscapeDNName(User)+',CN=users,'+ EscapeDNName(LdapTextDomain);     //[2609]
      ADsGetObject(ls, IADsUser, dom_user);
      List.Add(dom_user.EmailAddress);
      dom_user._release;
   except on e: exception do
      begin
         bitfarmLog.Write(llNormal, ltDebug, 'Error on GetUserInfo ' + e.message, ClassName);
      end;
   end;
   bitfarmLog.Write(llNormal, ltDebug, 'GetUserInfo end', ClassName);
end;

//-----------------------------------------------------------------------------
function TADsHelper.GetUserMail(const User:string):string;
var ls:string;  dom_user: IADsUser;
begin
   bitfarmLog.Write(llNormal, ltDebug, 'GetUserMail begin', ClassName);
   try
      Result := '';
      //ls := 'WinNT://' + FDomain + '/' + User;
      //ls := 'LDAP://CN='+User+',CN=users,'+LdapTextDomain;
      ls := sADRoot + 'CN='+ EscapeDNName(User)+',CN=users,'+ EscapeDNName(LdapTextDomain);     //[2609]
      ADsGetObject(ls, IADsUser, dom_user);
      if bLogRq
      then bitfarmLog.Write(llNormal, ltDebug, 'E-ADsGetObject('''+ls+''', IADsUser, dom_user);', ClassName);
      try
         Result := dom_user.EmailAddress;
      except on e: EOleException do
         Result := '';
      end;
   except on e: exception do
      begin
         bitfarmLog.Write(llNormal, ltDebug, 'Error on GetUserMail ' + e.message, ClassName);
      end;
   end;
   bitfarmLog.Write(llNormal, ltDebug, 'GetUserMail end', ClassName);
end;

//-----------------------------------------------------------------------------
procedure TADsHelper.GetLdap(List:TStringList; Angabe:string);
var ls: string;  dom: IADsContainer;
begin
   bitfarmLog.DebugBegin(llDebug, 'GetLdap begin', ClassName);
   try
      ActList := List;
      ActList.Clear;
      //ls := 'LDAP://'+Angabe;
      ls := sADRoot + EscapeLDAPPath(Angabe);     //[2609]
      ADsGetObject(ls, IADsContainer, dom);

      if bLogRq then
        bitfarmLog.Write(llNormal, ltDebug, 'GetLdap('+ls+', IADsContainer, dom);', ClassName);

      ADsEnumerateObjects(dom, AddToLdapList);
   except on e: exception do
      begin
         bitfarmLog.Write(llNormal, ltDebug, 'Error on GetLdap ' + e.message, ClassName);
      end;
   end;
   bitfarmLog.DebugEnd(llDebug, 'GetLdap end', ClassName);
end;

procedure TADsHelper.AddToLdapList(disp: IADs);
var
  lsClass:string;
  hasChild : Boolean;
  dom: IADsContainer;
begin
   lsClass := disp.Class_;
//   bitfarmLog.Write(llNormal, ltDebug, 'AddToLdapList lsClass=' + lsClass);
   gHasChild := False;
   if  (lsClass='user')then
    Exit;

    if (lsClass='group') or (lsClass='container') or (lsClass='organizationalUnit') then
    begin
      ADsGetObject(disp.ADsPath, IADsContainer, dom);
      ADsEnumerateObjects(dom, LDAPHasChild);
    end;

    if gHasChild then
//      ActList.Add('+ ' + disp.Class_+': '+disp.Name)
      ActList.AddObject(disp.Class_+': '+disp.Name, TbfaInt.Create(1))
    else
//      ActList.Add('  ' + disp.Class_+': '+disp.Name);
      ActList.AddObject(disp.Class_+': '+disp.Name, TbfaInt.Create(0));
   bitfarmLog.Write(llNormal, ltDebug, 'AddToLdapList ' + disp.Class_+': '+disp.Name);


end;

//-----------------------------------------------------------------------------
procedure TADsHelper.GetLdapGroup(List:TStringList; Angabe:string);
var ls: string;  dom: IADsContainer;
begin
   bitfarmLog.Write(llNormal, ltDebug, 'GetLdapGroup begin', ClassName);
   try
      ActList := List;
      ActList.Clear;

      ls := sADRoot+ EscapeLDAPPath(Angabe);     //[2609]
      ADsGetObject(ls, IADsContainer, dom);

      if bLogRq then
       bitfarmLog.Write(llNormal, ltDebug, 'GetLdapGroup('+ls+', IADsContainer, dom);', ClassName);

      ADsEnumerateObjects(dom, AddToLdapGroupList);
   except on e: exception do
      begin
         bitfarmLog.Write(llNormal, ltDebug, 'Error on GetLdapGroup ' + e.message, ClassName);
      end;
   end;
   bitfarmLog.Write(llNormal, ltDebug, 'GetLdapGroup end', ClassName);
end;

procedure TADsHelper.AddToLdapGroupList(disp: IADs);
var p: integer;  s: string;
begin
   if lowercase(Disp.Class_) = 'group' then
   begin
      s := disp.Name;
      p := Pos('=',s);
      if p>0 then
         Delete(s,1,p);
      ActList.Add(s);
   end;
end;

//-----------------------------------------------------------------------------
procedure TADsHelper.GetTest(List:TStringList; Angabe1, Angabe2, Angabe3:string);
var ls: string;  dom_group: IADsContainer;
begin
   bitfarmLog.Write(llNormal, ltDebug, 'GetTest begin', ClassName);
   try
      ActList := List;
      ActList.Clear;

      ls := EscapeLDAPPath(Angabe1);
      ADsGetObject(ls, IADsContainer, dom_group);

      if bLogRq then
        bitfarmLog.Write(llNormal, ltDebug, 'F-ADsGetObject('''+ls+''', IADsContainer, dom_group);', ClassName);

      if Angabe2>'' then
         dom_group.Filter := VarArrayOf([Angabe2])
      else
         dom_group.Filter := VarArrayOf([]);
      ADsEnumerateObjects(dom_group, AddToTestList);
   except on e: exception do
      begin
         bitfarmLog.Write(llNormal, ltDebug, 'Error on GetGroups ' + e.message, ClassName);
      end;
   end;
   bitfarmLog.Write(llNormal, ltDebug, 'GetTest end', ClassName);
end;

procedure TADsHelper.AddToTestList(disp: IADs);
begin
   if lowercase(Disp.Class_) = 'group' then
   begin
      ActList.Add(disp.Name);
   end
   else
   begin
      ActList.Add('-'+disp.Name);
   end;
end;


initialization
  CoInitialize(nil); // <-- manually call CoInitialize()

finalization
  CoUnInitialize; // <-- free memory





//-----------------------------------------------------------------------------
end.
//-----------------------------------------------------------------------------
