unit bfaApplication;

interface

uses
  Classes, bfaLog, bfaConnection, bfaUser,
  bfaCommonUtils, 
  bfaOptions;

type
  TbfaApplication = class;

  PbfaApplication = ^TbfaApplication; 
    
  /// <Beschreibung>bitfarm Application Klasse</Beschreibung>
  /// <Description>bitfarm Application class</Description>
  TbfaApplication = class(TObject)
  protected
    /// <Beschreibung>Fehlercode</Beschreibung>
    /// <Description>Error code</Description>
    FErrorCode: Integer;
    /// <Beschreibung>Wenn das Connection Objekt zustande gekommen ist, dann true, sonst false.</Beschreibung>
    /// <Description>If the Connection object is initialized, then true, else false</Description>
    FConnectionInitialized: Boolean;
    /// <Beschreibung>Wenn das User Objekt zustande gekommen ist, dann true, sonst false.</Beschreibung>
    /// <Description>If the User object is initialized, then true, else false</Description>
    FUserInitialized: Boolean;
    /// <Beschreibung>Wenn das Log Objekt zustande gekommen ist, dann true, sonst false.</Beschreibung>
    /// <Description>If the Log object is initialized, then true, else false</Description>
    FLogInitialized: Boolean;
    /// <Beschreibung>Der aktuell eingeloggte Benutzer</Beschreibung>
    /// <Description>The actual user</Description>
    FUser: TbfaUser;
    /// <Beschreibung>Connection Objekt. Durch dieses Objekt kann man die Datenbank und den bitfarm-Server erreichen.</Beschreibung>
    /// <Description>Connection Object. Through this object are the database and the bitfarm-Server accessible.</Description>
    FConnection: TbfaConnection;
    /// <Beschreibung>Log Objekt zum loggen</Beschreibung>
    /// <Description>Log Object for logging</Description>
    FLog: TbfaLog;
    /// <Beschreibung>Vorbergende LogDir</Beschreibung>
    /// <Description>Temporary LogDir</Description>
    FTmpLogDir: WideString;
    /// <Beschreibung>Vorbergehende Loglevel</Beschreibung>
    /// <Description>Temporary LogLevel</Description>
    FTmpLogLevel: Integer;
    /// <Beschreibung>Klient Logpfad</Beschreibung>
    /// <Description>Client log path</Description>
    FClientLogPath: WideString;
    /// <Beschreibung>Klient Applikationspfad</Beschreibung>
    /// <Description>client application path</Description>
    FClientApplicationPath: WideString;
    /// <Beschreibung>Serverpfad</Beschreibung>
    /// <Description>Server path</Description>
    FServerDir : WideString;
    /// <Beschreibung>Temporary Pfad</Beschreibung>
    /// <Description>Temporary directory</Description>
    FTempDir: WideString;
    /// <Beschreibung>Applikationspfad</Beschreibung>
    /// <Description>Application path</Description>
    FAppDataDir: WideString;
    /// <Beschreibung>Applikationsoptionen wie z.B. Klient Logpfad, Klient Applikationspfad, Loglevel, DBMonitor</Beschreibung>
    /// <Description>Application options for exapmle: Client log path, Klient application path, log level, DB monitor</Description>
    FApplicationOptions: TbfaApplicationOptions;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function GetApplicationPath: WideString;
    /// <Beschreibung>Abbruch die Datenbank Verbindung</Beschreibung>
    /// <Description>Destroyes the database connection</Description>
    procedure DestroyConnection; 
    /// <Beschreibung>Initialisiert das Log und das Connection Objekt</Beschreibung>
    /// <Description>Initialize the Log and the Connection objects</Description>
    function Init(LogDir: WideString; LogLevel: Integer): Integer; 
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function Connect(SQLMonitor: Boolean): Boolean; 
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function ReadSettingsFromINI(ConFileName: string): Integer;



  public
    /// <Beschreibung>Setzt die Standardwerte und erstellt die Objekte</Beschreibung>
    /// <Description>Sets the default values and creates the objects</Description>
    constructor Create; 
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure WriteLog(LogLevel: Integer; LogType, LogMessage, ClsName: WideString);
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function Login(PbfaApp : PbfaApplication; ConFile: WideString; UserName, Password: WideString): Integer;
    /// <Beschreibung>Schliet die Verbindung</Beschreibung>
    /// <Description>Closes the Connection</Description>
    procedure CloseConnection;
    /// <Beschreibung>Abbruch das Datenbank und wartet bis das LogThread fertig wird</Beschreibung>
    /// <Description>Destroyes the Database and wait for the Log thread</Description>
    procedure FreeApp; 
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
//    function GetArchiveNamesByID(IDs: string): WideString;
    /// <Beschreibung>Eingeloggte Benutzer hat Useremanagement Berechtigung</Beschreibung>
    /// <Description>The actual user has user management right</Description>
    function HasUserManagementRight: Boolean;
    /// <Beschreibung>Eingeloggte Benutzer hat Groupmanagement Berechtigung</Beschreibung>
    /// <Description>The actual user has group management right</Description>
    function HasGroupManagementRight: Boolean;
    /// <Beschreibung>Eingeloggte Benutzer hat Archivemanagement Berechtigung</Beschreibung>
    /// <Description>The actual user has archive management right</Description>
    function HasArchiveManagementRight: Boolean;
    /// <Beschreibung>Eingeloggte Benutzer hat Fieldmanagement Berechtigung</Beschreibung>
    /// <Description>The actual user has field management right</Description>
    function HasFieldManagementRight: Boolean;
    /// <Beschreibung>Eingeloggte Benutzer hat Import Berechtigung</Beschreibung>
    /// <Description>The actual user has import right</Description>
    function HasImportRight: Boolean;
    /// <Beschreibung>Eingeloggte Benutzer hat Suchen Berechtigung</Beschreibung>
    /// <Description>The actual user has field search right</Description>
    function HasViewRight: Boolean;
    /// <Beschreibung>Log Objekt zum loggen</Beschreibung>
    /// <Description>Log Object for logging</Description>
    /// <associates>bfaApplication.TbfaApplication.FLog</associates>
    property Log: TbfaLog read FLog;
    /// <Beschreibung>Connection Objekt. Durch dieses Objekt kann man die Datenbank und den bitfarm-Server erreichen.</Beschreibung>
    /// <Description>Connection Object. Through this object are the database and the bitfarm-Server accessible.</Description>
    /// <associates>bfaApplication.TbfaApplication.FConnection</associates>
    property Connection: TbfaConnection read FConnection;
    /// <Beschreibung>Der aktuell eingeloggte Benutzer</Beschreibung>
    /// <Description>The actual user</Description>
    /// <associates>bfaApplication.TbfaApplication.FUser</associates>
    property User: TbfaUser read FUser;
    /// <Beschreibung>Klient Applikationspfad</Beschreibung>
    /// <Description>client application path</Description>
    /// <associates>bfaApplication.TbfaApplication.FClientApplicationPath</associates>
    property ClientApplicationPath: WideString read FClientApplicationPath;
    /// <Beschreibung>Wenn das Connection Objekt zustande gekommen ist, dann true, sonst false.</Beschreibung>
    /// <Description>If the Connection object is initialized, then true, else false</Description>
    /// <associates>bfaApplication.TbfaApplication.FConnectionInitialized</associates>
    property ConnectionInitialized: Boolean read FConnectionInitialized;
    /// <Beschreibung>Fehlercode</Beschreibung>
    /// <Description>Error code</Description>
    /// <associates>bfaApplication.TbfaApplication.FErrorCode</associates>
    property ErrorCode: Integer read FErrorCode;
    /// <Beschreibung>Wenn das User Objekt zustande gekommen ist, dann true, sonst false.</Beschreibung>
    /// <Description>If the User object is initialized, then true, else false</Description>
    /// <associates>bfaApplication.TbfaApplication.FUserInitialized</associates>
    property UserInitialized: Boolean read FUserInitialized;
    /// <Beschreibung>Klient Logpfad</Beschreibung>
    /// <Description>Client log path</Description>
    /// <associates>bfaApplication.TbfaApplication.FClientLogPath</associates>
    property ClientLogPath: WideString read FClientLogPath;
    /// <Beschreibung>Temporary Pfad</Beschreibung>
    /// <Description>Temporary directory</Description>
    /// <associates>bfaApplication.TbfaApplication.FTempDir</associates>
    property TemporaryDir: WideString read FTempDir;
    /// <Beschreibung>Serverpfad</Beschreibung>
    /// <Description>Server directory</Description>
    /// <associates>bfaApplication.TbfaApplication.FServerDir</associates>
    property ServerDir: WideString read FServerDir;
    /// <Beschreibung>Applikationspfad</Beschreibung>
    /// <Description>Application path</Description>
    /// <associates>bfaApplication.TbfaApplication.FAppDataDir</associates>
    property AppDataDir: WideString read FAppDataDir;
    /// <Beschreibung>Applikationsoptionen wie z.B. Klient Logpfad, Klient Applikationspfad, Loglevel, DBMonitor</Beschreibung>
    /// <Description>Application options for exapmle: Client log path, Klient application path, log level, DB monitor</Description>
    /// <associates>bfaApplication.TbfaApplication.FApplicationOptions</associates>
    property Options: TbfaApplicationOptions read FApplicationOptions write FApplicationOptions;
  end;

implementation

uses SysUtils, Registry, Windows, MySQLUniProvider, iniFiles, bfaBaseObjects;
//
//  Name: Create
//  Parameters: -
//  Beschreibung: Setzt die Standardwerte und erstellt die Objekte
//  Description: Sets the default values and creates the objects
//  Return value:
// [] 17.04.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
constructor TbfaApplication.Create;
begin
  inherited Create;
  FErrorCode:= 0;
  FConnectionInitialized:= False;
  FLogInitialized:= False;
  FUserInitialized:= False;
  FTmpLogLevel:= llNormal;
  FConnection:= nil;
  FLog:= nil;
  FUser:= nil;
  FLog:= TbfaLog.Create;  
  FClientApplicationPath:= GetApplicationPath;
  FApplicationOptions:= TbfaApplicationOptions.Create(ostIniFile, FClientApplicationPath + '\' + sSysBfaApplication + '.ini');
  FTmpLogDir:= WideString(FApplicationOptions.GetOption(sSysClientLogPath, okString, '', sSysSystem).Value); 
  FTmpLogLevel:= FApplicationOptions.GetOption(sSysLogLevel, okInteger, llNormal, sSysSystem).Value;
  Init(FTmpLogDir,FTmpLogLevel);
  FClientLogPath:= FLog.LogDir;
  if FTmpLogLevel < 0 then
    FTmpLogLevel:= llNormal;
  FServerDir := FApplicationOptions.GetOption(sSysServerPath, okString, '', sSysSystem).Value;
  FTempDir:= GetEnvironmentVariable(sSysTemp) + '\' + sSysBitfarmDirectory;
  if not DirectoryExists(FTempDir) then
    ForceDirectories(FTempDir);
  FAppDataDir:= GetEnvironmentVariable(sSysAppData) + '\' + sSysBitfarmDirectory;
  if not DirectoryExists(FAppDataDir) then
    ForceDirectories(FAppDataDir);
  FConnection:= TbfaConnection.Create(FLog);
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaApplication.GetApplicationPath: WideString;
var
  buf : array[0..MAX_PATH] of WideChar;
begin
  GetModuleFileName(0, buf, SizeOf(buf));
  Result:= ExtractFilePath(buf);
end;
//
//  Name: DestroyConnection
//  Parameters: - 
//  Beschreibung: Abbruch die Datenbank Verbindung
//  Description: Destroyes the database connection
//  Return value: -
// [] 17.04.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
procedure TbfaApplication.DestroyConnection;
begin
  FErrorCode:= 0;
  if FLogInitialized then
  begin
    WriteLog(llHigh, ltInfo, 'DestroyConnection', ClassName);
    if FConnectionInitialized then
    begin
      if FConnection <> nil then
        FConnectionInitialized:= FConnection.DestroyDB;
    end
    else
    begin
      FErrorCode:= ecApplicationConnectionNotInitialized;
      WriteLog(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecApplicationLogNotInitialized; 
  end;
end;
//
//  Name: Free
//  Parameters: -
//  Beschreibung: Abbruch das Datenbank und wartet bis das LogThread fertig wird
//  Description: Destroyes the Database and wait for the Log thread
//  Return value: - 
// [] 17.04.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
procedure TbfaApplication.FreeApp;
begin
  WriteLog(llHigh, ltInfo, 'FreeApp', ClassName);
  if FConnectionInitialized then
  begin
    FConnectionInitialized:= FConnection.DisconnectDB;
  end;
  if FConnection <> nil then
    FConnection.DestroyDB;
    
  if FLog <> nil then
  begin
    if FLog.QueueCount > 0 then                                     
    begin
      while FLog.QueueCount > 0 do
        Sleep(100);
    end;
  end;
  FApplicationOptions.Save;
  FApplicationOptions.Free;
  
  Self.Free;
  
end;
//
//  Name: Init
//  Parameters: LogDir
//              LogLevel
//  Beschreibung: Initialisiert das Log und das Connection Objekt
//  Description: Initialize the Log and the Connection objects
//  Return value: FErrorCode
// [] 17.04.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaApplication.Init(LogDir: WideString; LogLevel: Integer): Integer;
begin
  FErrorCode:= 0;
  FLogInitialized:= FLog.Init(LogDir,'', LogLevel);
  if Not FLogInitialized then
    FErrorCode:= ecApplicationFailureinInit;
  Result:= FErrorCode;  
end;
//
function TbfaApplication.ReadSettingsFromINI(ConFileName: string): Integer;
var
  ini: TIniFile;
begin
  FErrorCode:=0;
  ini:= TIniFile.Create(FClientApplicationPath + '\' + ConFileName);
  try
    if not FileExists(ini.FileName) then
      FErrorCode:= ecApplicationINIFileNotFound;
    if FErrorCode = 0 then
    begin  
      if (ini.ReadInteger(sSysServer, sSysVersion, 0) = sSysVersion40) then
      begin
        FConnection.SetConnection(                                                  
          ini.ReadString(sSysConnectionSection,sSysUserName,''),
          ini.ReadString(sSysConnectionSection,sSysPassword,''),
          ini.ReadString(sSysConnectionSection,sSysDatabase,''),
          ini.ReadString(sSysConnectionSection,sSysServer,''),
          ini.ReadInteger(sSysConnectionSection,sSysPort,3306),
          ini.ReadInteger(sSysConnectionSection,sSysProvider,0));
      end
      else
        FErrorCode := ecApplicationBadConVersion;
    end;
  finally
    Ini.Free;
    Result:= FErrorCode;
  end;
end;
//
function TbfaApplication.Login(PbfaApp : PbfaApplication; ConFile: WideString; UserName, Password: WideString): Integer;
begin
  FErrorCode:= 0;
  if FLogInitialized then
  begin
    WriteLog(llHigh, ltInfo, 'Login', ClassName);
      
    FErrorCode := ReadSettingsFromINI(ConFile);  
    if (FErrorCode = 0) then
    begin
      FConnectionInitialized := FConnection.ConnectDB(FApplicationOptions.GetOption(sSysUseDBMonitor, okBoolean, False, sSysSystem).Value);

      if FConnectionInitialized then
      begin
        if FUser = nil then
          FUser := TbfaUser.Create(PbfaApp)
        else
          FUser.Reset;
      
        FUser.Name := UserName;
        FUser.Password := Password;
        FErrorCode := FConnection.UserLogin(@FUser);
      
        if FErrorcode = 0 then
        begin
          FUser.Load(FUser.ID, False);          
          if FUser.Loaded then
            FUserInitialized := True
          else
            FUserInitialized := False; 
        
          Result := FErrorCode;
        end
        else
        begin
          FUserInitialized := False;
          if Connection.ErrorCode = ecConnectionUserLocked then
            FErrorCode := ecConnectionUserLocked
          else
            FErrorCode := ecApplicationLoginFailure;
          Result := FErrorCode;
          FUser := nil;
          WriteLog(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
        end;
      end
      else
      begin
        FErrorCode := ecApplicationConnectionNotInitialized;
        Result := FErrorCode;
        WriteLog(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
      end;
    end
    else
    begin
//      FErrorCode := ecApplicationConnectionNotInitialized;
      Result := FErrorCode;
      WriteLog(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode := ecApplicationLogNotInitialized;
    Result := FErrorCode;
  end;  
end;
//
//  Name: CloseConnection
//  Parameters: -
//  Beschreibung: Schliet die Verbindung
//  Description: Closes the Connection
//  Return value: -
// [] 17.04.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
procedure TbfaApplication.CloseConnection;
begin
  FErrorCode:= 0;
  if FLogInitialized then
  begin
    WriteLog(llHigh, ltInfo, 'CloseConnection', ClassName);
    if FConnectionInitialized then
    begin
      if FConnection <> nil then
        FConnectionInitialized:= FConnection.DisconnectDB;
    end
    else
    begin
      FErrorCode:= ecApplicationConnectionNotInitialized;
      WriteLog(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecApplicationLogNotInitialized;
  end;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
procedure TbfaApplication.WriteLog(LogLevel: Integer; LogType, LogMessage, ClsName: WideString);
begin
  FLog.Write(LogLevel, LogType, LogMessage, ClsName);
end;
//
//  Name: InitConnection
//  Parameters: SQLMonitor  
//  Beschreibung: Einstellt die Datenbank Verbindung
//  Description: Sets the Database connection
//  Return value: Connected
// [] 17.04.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaApplication.Connect(SQLMonitor: Boolean): Boolean;
begin
  FErrorCode:= 0;
  if FLogInitialized then
  begin
    WriteLog(llHigh, ltInfo, 'InitConnection', ClassName);
    FConnectionInitialized:= False;
    FUserInitialized:= False;

    try
      FConnectionInitialized:= FConnection.ConnectDB(SQLMonitor);
      WriteLog(llDebug, ltInfo, 'InitConnection Connected: ' + BoolToStr(FConnectionInitialized,true), ClassName);

      Result:= FConnectionInitialized;
    except
      FConnectionInitialized:= False;
      FErrorCode:= ecApplicationConnectionNotInitialized;
      WriteLog(llNormal, ltException, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
      Result:= False;
    end;
  end
  else
  begin
    FErrorCode:= ecApplicationLogNotInitialized;
    Result:= False;
  end;
end;
(*
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 02.01.2013 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaApplication.GetArchiveNamesByID(IDs: string): WideString;
begin
  try
    Result:= FConnection.SelectArchivenamesByIDs(IDs);
  except
    Result:= '';
  end;
end;
*)
//
function TbfaApplication.HasUserManagementRight: Boolean;
begin
  try
    if FUserInitialized then
    begin
      Result:= FUser.Rights.ApplicationRights.UserManagement or FUser.DMSAdmin;
    end
    else
      Result:= False;
  except

  end;
end;
//
function TbfaApplication.HasGroupManagementRight: Boolean;
begin
  try
    if FUserInitialized then
    begin
      Result:= FUser.Rights.ApplicationRights.GroupManagement or FUser.DMSAdmin;
    end
    else
      Result:= False;
  except

  end;
end;
//
function TbfaApplication.HasArchiveManagementRight: Boolean;
begin
  try
    if FUserInitialized then
    begin
      Result:= FUser.Rights.ApplicationRights.ArchiveManagement or FUser.DMSAdmin;
    end
    else
      Result:= False;
  except

  end;
end;
//
function TbfaApplication.HasFieldManagementRight: Boolean;
begin
  try
    if FUserInitialized then
    begin
      Result:= FUser.Rights.ApplicationRights.FieldManagement or FUser.DMSAdmin;
    end
    else
      Result:= False;
  except

  end;
end;
//
function TbfaApplication.HasImportRight: Boolean;
begin
  try
    if FUserInitialized then
    begin
      Result:= FUser.Rights.ApplicationRights.AppImport or FUser.DMSAdmin;
    end
    else
      Result:= False;
  except

  end;
end;
//
function TbfaApplication.HasViewRight: Boolean;
begin
  try
    if FUserInitialized then
    begin
      Result:= FUser.Rights.ApplicationRights.AppView or FUser.DMSAdmin;
    end
    else
      Result:= False;
  except

  end;
end;
//

end.
