unit bfaArchive;

interface

uses
  bfaApplication, bfaMapping, bfaUser,
  Classes, DB, Generics.Collections, bfaField,
  bfaBaseObjects;

type
  /// <Beschreibung>bitfarm Archive Klasse</Beschreibung>
  /// <Description>bitfarm Archive class</Description>
  TbfaBaseArchive = class(TbfaBaseObject)
  protected
    FbfaApplication: PbfaApplication;
    /// <Beschreibung>bergeordnetes Lager ID</Beschreibung>
    /// <Description>Store ID</Description>
    FParentID: Int64;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FParentType: Integer;
    /// <Beschreibung>Pfad und Dateiname des Briefpapiers</Beschreibung>
    /// <Description>Path and filename of the notepaper</Description>
    FBgrPicture: WideString;
    /// <Beschreibung>Vordefinierte Dokumentname fr das jeweilige Archiv</Beschreibung>
    /// <Description>Default document name of the archive</Description>
    FDocName: WideString;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FLayoutID: Int64;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FKind: TbfaTreeViewKind;
    /// <Beschreibung></Beschreibung> 
    /// <Description></Description>
    FIcon: Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FArchiveLayout: TMemoryStream;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FUserContainerID: Int64;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FFieldContainerID: Int64;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FInheritPermissions: Boolean;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FInheritFields: Boolean;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FInLoadProcess : Boolean;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FMaxUserColumn: Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FMaxGroupColumn: Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FGUILayoutModified : Boolean;

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure ClearAllProperties;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure SetGUILayoutModified(Value : boolean);

    
  public

    /// <Beschreibung>Pfad und Dateiname des Briefpapiers</Beschreibung>
    /// <Description>Path and filename of the notepaper</Description>
    property BgrPicture: WideString read FBgrPicture;
    /// <Beschreibung>Vordefinierte Dokumentname fr das jeweilige Archiv</Beschreibung>
    /// <Description>Default document name of the archive</Description>
    property DocName: WideString read FDocName;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property LayoutID: Int64 read FLayoutID;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property ParentID: Int64 read FParentID;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property ParentType: Integer read FParentType;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property ArchiveLayout: TMemoryStream read FArchiveLayout;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property Kind: TbfaTreeViewKind read FKind;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property ArchiveIcon: Integer read FIcon;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property UserContainerID: Int64 read FUserContainerID;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property FieldContainerID: Int64 read FFieldContainerID;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property InLoadProcess : Boolean read FInLoadProcess;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property MaxUserColumn : Integer read FMaxUserColumn;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property MaxGroupColumn : Integer read FMaxGroupColumn;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property GUILayoutModified : Boolean read FGUILayoutModified write SetGUILayoutModified;
    
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure Refresh;


    /// <Beschreibung>Setzt die Standardwerte und erstellt die Objekte</Beschreibung>
    /// <Description>Sets the default values and creates the objects</Description>
    constructor Create(bfaApp: PbfaApplication);
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure Clone(Source: TbfaBaseArchive);
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function Load(LayoutID: Int64): Integer;

  end;

  /// <Beschreibung>bitfarm AdminArchive Klasse</Beschreibung>
  /// <Description>bitfarm AdminArchive class</Description>
  TbfaAdminArchive = class(TbfaBaseArchive)
  private
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FFieldsList: TList<TbfaFieldMapping>;
//    FFieldsList: TList<TbfaMapping>;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FPermissionsList: TList<TbfaMapping>;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FContainerPermissionsList: TList<TbfaMapping>;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FContainerFieldsList: TList<TbfaMapping>;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FKind: TbfaTreeViewKind;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FInheritPermissionsModified: Boolean;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FInheritFieldsModified : Boolean;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FGeneralPropertiesModified : Boolean;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FPermissionsListModified : Boolean;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FFieldsListModified : Boolean;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FContainerPermissionsListModified : Boolean;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FContainerFieldsListModified : Boolean;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FListOfModifications : TList<TbfaModification>;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FInSaveProcess : Boolean;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FFrameType : Integer;
    
    
    
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function LoadFields : Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function LoadPermissions : Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure ClearAllProperties;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure SetArchiveName(Value: WideString);
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure SetBgrPicture(Value: WideString);
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure SetDocName(Value: WideString);
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure SetInheritPermissions(Value: Boolean);
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure SetInheritFields(Value: Boolean);
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure SetDescription(Value: WideString);
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure SetIcon(Value: Integer);    
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure SetArcType(Value: Integer);    
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure SetParentType(Value: Integer);
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure SetParentID(Value: Int64);
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure SetObjectRights(List : TList<TbfaMapping>; Idx : Integer; Obj : PbfaBaseObject; Rights : Int64; RoleID : Integer; RoleName : WideString);
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure GetObjectRights(List : TList<TbfaMapping>; Idx : Integer; Obj : PbfaBaseObject; var Rights : Int64; var RoleID : Integer; var RoleName : WideString);
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function AddObject(List : TList<TbfaMapping>; ObjID, Rights : Int64; ObjType : Integer; RoleID : Int64; RName : WideString): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function AddFieldObject(List : TList<TbfaFieldMapping>; PObj : TbfaAdminField; ControlID : Int64; ControlName : WideString) : Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure SetObjectControlType(List : TList<TbfaMapping>; Idx : Integer; Obj : PbfaBaseObject; ControlID : Integer; ControlName : WideString);
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function RemoveObject(List : TList<TbfaMapping>; Idx : Integer): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure SetGeneralPropertiesModified(Value : Boolean);
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure SetPermissionsListModified(Value : Boolean);
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure SetFieldsListModified(Value : Boolean);
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure SetContainerPermissionsListModified(Value : Boolean);
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure SetContainerFieldsListModified(Value : Boolean);   
    

    
  public
    /// <Beschreibung>Bezeichnung fr die Benutzer</Beschreibung>
    /// <Description>Displayname for the user</Description>
    property ArchiveName: WideString read FName write SetArchiveName;
    /// <Beschreibung>Pfad und Dateiname des Briefpapiers</Beschreibung>
    /// <Description>Path and filename of the notepaper</Description>
    property BgrPicture: WideString read FBgrPicture write SetBgrPicture;
    /// <Beschreibung>Vordefinierte Dokumentname fr das jeweilige Archiv</Beschreibung>
    /// <Description>Default document name of the archive</Description>
    property DocName: WideString read FDocName write SetDocName;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property ArcType: Integer read FObjType write SetArcType;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property InheritPermissions: Boolean read FInheritPermissions write SetInheritPermissions;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property InheritPermissionsModified: Boolean read FInheritPermissionsModified write FInheritPermissionsModified;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property InheritFields: Boolean read FInheritFields write SetInheritFields;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property InheritFieldsModified: Boolean read FInheritFieldsModified write FInheritFieldsModified;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property ParentID: Int64 read FParentID write SetParentID;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property ParentType: Integer read FParentType write SetParentType;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property Description: WideString read FDescription write SetDescription;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property FieldsList: TList<TbfaFieldMapping> read FFieldsList;
//    property FieldsList: TList<TbfaMapping> read FFieldsList;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property PermissionsList: TList<TbfaMapping> read FPermissionsList;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property ContainerFieldsList: TList<TbfaMapping> read FContainerFieldsList;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property ContainerPermissionsList: TList<TbfaMapping> read FContainerPermissionsList;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property Kind: TbfaTreeViewKind read FKind write FKind;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property ArchiveIcon: Integer read FIcon write SetIcon;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property UserContainerID: Int64 read FUserContainerID write FUserContainerID;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property FieldContainerID: Int64 read FFieldContainerID write FFieldContainerID;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property Modified: Boolean read FModified write FModified;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property GeneralPropertiesModified : Boolean read FGeneralPropertiesModified write SetGeneralPropertiesModified;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property PermissionsListModified : Boolean read FPermissionsListModified write SetPermissionsListModified;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property FieldsListModified : Boolean read FFieldsListModified write SetFieldsListModified;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property ContainerPermissionsListModified : Boolean read FContainerPermissionsListModified write SetContainerPermissionsListModified;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property ContainerFieldsListModified : Boolean read FContainerFieldsListModified write SetContainerFieldsListModified;  
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property ListOfModifications : TList<TbfaModification> read FListOfModifications write FListOfModifications;  
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property InSaveProcess : Boolean read FInSaveProcess;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property FrameType : Integer read FFrameType write FFrameType;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property ArchiveLayout: TMemoryStream read FArchiveLayout write FArchiveLayout;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function LoadContainerFields : Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function LoadContainerPermissions : Integer;
    
    
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure Refresh;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SaveGUILayout: Integer;
    

    /// <Beschreibung>Setzt die Standardwerte und erstellt die Objekte</Beschreibung>
    /// <Description>Sets the default values and creates the objects</Description>
    constructor Create(bfaApp: PbfaApplication);
    /// <Beschreibung>Einstellt das Log und das Connection Objekte</Beschreibung>
    /// <Description>Sets the Log and the connection objects</Description>
    procedure Clone(Source: TbfaAdminArchive);
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function Load(LayoutID: Int64): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function Save: Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SavePermissionsList: Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SaveContainerPermissionsList: Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SaveFieldsList: Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SaveContainerFieldsList : Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function AddObjectToContainerPermissionsList(OID, Rights: Int64; OType: Integer; RoleID: Int64; RName: WideString): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function AddObjectToPermissionsList(OID, Rights: Int64; OType: Integer; RoleID: Int64; RName: WideString): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function AddObjectToContainerFieldsList(OID, Rights: Int64; OType: Integer; RoleID: Int64; RName: WideString): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function AddObjectToFieldsList(PObj : TbfaAdminField; ControlID: Int64; ControlName: WideString): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure SetObjectFieldControlType(Idx : Integer; Obj : PbfaBaseObject; ControlID : Integer; ControlName : WideString);
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure SetObjectContainerFieldControlType(Idx : Integer; Obj : PbfaBaseObject; ControlID : Integer; ControlName : WideString);
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function RemoveObjectFromContainerPermissionsList(Idx : Integer): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function RemoveObjectFromContainerFieldsList(Idx : Integer): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function RemoveObjectFromFieldsList(Idx : Integer): Integer;
    
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function RemoveObjectFromPermissionsList(Idx : Integer): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function AddField(FieldID: Int64): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function RemoveField(Idx: Integer): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure SetObjectPermissionRights(Idx : Integer; Obj : PbfaBaseObject; Rights : Int64; RoleID : Integer; RoleName : WideString);
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure SetObjectContainerPermissionRights(Idx : Integer; Obj : PbfaBaseObject; Rights : Int64; RoleID : Integer; RoleName : WideString);
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure GetObjectContainerPermissionRights(Idx : Integer; Obj : PbfaBaseObject; var Rights : Int64; var RoleID : Integer; var RoleName : WideString);
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure GetObjectPermissionRights(Idx : Integer; Obj : PbfaBaseObject; var Rights : Int64; var RoleID : Integer; var RoleName : WideString);
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function RefreshObject(Obj : TbfaBaseObject; Mdt : TbfaModificationType): Integer;
    

  end;

  /// <Beschreibung>bitfarm ClientArchive Klasse</Beschreibung>
  /// <Description>bitfarm ClientArchive class</Description>
  TbfaClientArchive = class(TbfaBaseArchive)
  private
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
//    FFieldsList: TList<TbfaClientField>;
    FFieldsList: TList<TbfaMapping>;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FHasFieldUserRights: Boolean;
    
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure ClearAllProperties;

    
  public
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure Refresh;

    /// <Beschreibung>Setzt die Standardwerte und erstellt die Objekte</Beschreibung>
    /// <Description>Sets the default values and creates the objects</Description>
    constructor Create(bfaApp: PbfaApplication);
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure Clone(Source: TbfaClientArchive);
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function Load(LayoutID: Int64): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function LoadFields: Integer;
    

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property ArchiveLayout: TMemoryStream read FArchiveLayout write FArchiveLayout;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
//    property FieldsList: TList<TbfaClientField> read FFieldsList;
    property FieldsList: TList<TbfaMapping> read FFieldsList;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    property HasFieldUserRights: Boolean read FHasFieldUserRights;
    

  end;

  PbfaClientArchive = ^TbfaClientArchive;  
  PbfaAdminArchive = ^TbfaAdminArchive;
  PbfaBaseArchive = ^TbfaBaseArchive;

implementation

uses 
  SysUtils, bfaSP, uni, bfaCommonMethods, bfaMethods, bfaCommonUtils;


//
//  Name: Initialize
//  Parameters: -
//  Beschreibung: Setzt die Standardwerte und erstellt die Objekte
//  Description: Sets the default values and creates the objects
//  Return value: -
// [] 17.04.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
constructor TbfaBaseArchive.Create(bfaApp : PbfaApplication);
begin
  try
    New(FbfaApplication);
    FbfaApplication := bfaApp;

    inherited Create;

    FParentID := -1;
    FParentType := -2;
    FBgrPicture := '';
    FKind := tvNormal;
    FDocName := '';
    FLayoutID := -1;
    FIcon := 0;
    FArchiveLayout := TMemoryStream.Create;
    FUserContainerID := -1;
    FFieldContainerID := -1;  
    FInitialized := True;  
    FInLoadProcess := False;
    FMaxUserColumn:= -1;
    FMaxGroupColumn:= -1;    
    FGUILayoutModified := False;
  except
    FbfaApplication := nil;
  end;
end;
//
procedure TbfaBaseArchive.ClearAllProperties;
begin
  FErrorCode := -1;
  FID := -1;
  FParentID := -1;
  FParentType := -2;
  FName := '';
  FBgrPicture := '';
  FObjType := -2;
  FDocName := '';
  FLoaded := False;
  FLayoutID := -1;
  FDescription := '';
  FUserContainerID := -1;
  FFieldContainerID := -1;  
  Finitialized := True;
  FInLoadProcess := False;
  FMaxUserColumn:= -1;
  FMaxGroupColumn:= -1;  
  FGUILayoutModified := False;  
end;
//
//  Name: Load
//  Parameters: ArcID
//  Beschreibung: Ldt ein Archiv mit der bergebenen LayoutID aus der Datenbank
//  Description: Loads an Archive with the LayoutID from the Database
//  Return value: FErrorCode
// [] 17.04.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaBaseArchive.Load(LayoutID: Int64): Integer;
var
  ArcRec: TArchiveRec;
  LayoutRec: TLayoutRec;
begin
  FErrorCode:= 0;
  if FInitialized then
  begin
    FbfaApplication^.WriteLog(llHigh, ltInfo, 'Load(LayoutID: ' + IntToStr(LayoutID) + ')', ClassName);
    try
      FErrorCode:= FbfaApplication^.Connection.LoadArchive(LayoutID, ArcRec);
      if FErrorCode = 0 then
      begin
        FID:= ArcRec.ArchiveID;
        FParentID:= ArcRec.ParentID;
        FbfaApplication^.Connection.SelectLayoutObjectDetailsByID(ParentID, LayoutRec);
        FParentType:= LayoutRec.ObjType;        
        FName:= ArcRec.ArcName;
        FBgrPicture:= ArcRec.BgrPicture;
        FDocName:= ArcRec.DocName;
        FLayoutID:= ArcRec.LayoutID;
        FObjType:= ArcRec.ArcType;
        FDescription:= ArcRec.Description;
        FKind:= ArcRec.Kind;
        FIcon:= ArcRec.Icon;
        FInheritPermissions:= ArcRec.InheritPermissions;
        FInheritFields:= ArcRec.InheritFields;
        
        if ((FObjType <> otArchive) and (FObjType <> otRecordArchive)) then   // weil Archive selbst hat kein Container deswegen muss Parent nehmen
        begin
          FUserContainerID:= FbfaApplication^.Connection.SelectLayoutIDByObjectIDs(-1, FLayoutID, otUserContainer);
          FFieldContainerID:= FbfaApplication^.Connection.SelectLayoutIDByObjectIDs(-1, FLayoutID, otAddFieldContainer);
        end
        else
        begin
          FUserContainerID:= FbfaApplication^.Connection.SelectLayoutIDByObjectIDs(-1, FParentID, otUserContainer);
          FFieldContainerID:= FbfaApplication^.Connection.SelectLayoutIDByObjectIDs(-1, FParentID, otAddFieldContainer);
        end;
        FArchiveLayout.Clear;
        if ArcRec.ArchiveLayout <> nil then
          FArchiveLayout.CopyFrom(ArcRec.ArchiveLayout, ArcRec.ArchiveLayout.Size);

        FbfaApplication^.Connection.SelectMaxUsrGrpIdx(FMaxUserColumn, FMaxGroupColumn);  

        FGUILayoutModified := False;
        FLoaded:= True;
      end
      else
        ClearAllProperties;       

    except
      FErrorCode:= ecArchiveFailureInLoadArchive;
      FLoaded:= False;
      FbfaApplication^.WriteLog(llNormal, ltException, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
      Result:= FErrorCode;
    end;
  end
  else
  begin
    FErrorCode:= ecArchiveConnectionOrLogNotInitialized;
    FLoaded:= False;
  end;
  if FErrorCode <> 0 then
    FbfaApplication^.WriteLog(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
  Result:= FErrorCode;
end;
//
procedure TbfaBaseArchive.SetGUILayoutModified(Value : boolean);
begin
  FGUILayoutModified := Value;
  FModified := True;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
procedure TbfaBaseArchive.Clone(Source: TbfaBaseArchive);
begin
  try
    FID:= Source.ID;
    FParentID:= Source.ParentID;
    FName:= Source.Name;
    FBgrPicture:= Source.BgrPicture;
    FObjType:= Source.ObjType;
    FDocName:= Source.DocName;
    FErrorCode:= Source.ErrorCode;
    FInitialized:= Source.FInitialized;
    FLoaded:= Source.Loaded;
    FinloadProcess := Source.InLoadProcess;
    FLayoutID:= Source.LayoutID;
    FDescription:= Source.Description;
    FArchiveLayout:= Source.ArchiveLayout;
    FKind:= Source.Kind;
    FIcon:= Source.ArchiveIcon;
    FUserContainerID:= Source.UserContainerID;
    FFieldContainerID:= Source.FieldContainerID;
    FMaxUserColumn:= Source.FMaxUserColumn;
    FMaxGroupColumn:= Source.FMaxGroupColumn;
    FGUILayoutModified := Source.GUILayoutModified;

  except
    FbfaApplication^.WriteLog(llNormal, ltException, 'Clone', ClassName);
  end;
end;
//
procedure TbfaBaseArchive.Refresh;
begin
  try
    FErrorCode:= Load(FLayoutID);
  except
    FbfaApplication^.WriteLog(llNormal, ltException, 'Refresh', ClassName);
  end;
end;
//
//  Name: Initialize
//  Parameters: -
//  Beschreibung: Setzt die Standardwerte und erstellt die Objekte
//  Description: Sets the default values and creates the objects
//  Return value: -
// [] 17.04.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
constructor TbfaAdminArchive.Create(bfaApp: PbfaApplication);
begin
  inherited Create(bfaApp);
  FInheritPermissions:= False;
  FInheritFields:= False;
  FInSaveProcess := False;
  FFieldsList:= TList<TbfaFieldMapping>.Create;
//  FFieldsList:= TList<TbfaMapping>.Create;
  FFieldsList.Clear;
  FPermissionsList:= TList<TbfaMapping>.Create;
  FPermissionsList.Clear;  
  FContainerPermissionsList:= TList<TbfaMapping>.Create;
  FContainerPermissionsList.Clear;
  FContainerFieldsList:= TList<TbfaMapping>.Create;
  FContainerFieldsList.Clear;
  FModified:= False;
  FInheritPermissionsModified:= False;
  FInheritFieldsModified:= False;
  FGeneralPropertiesModified := False;
  FPermissionsListModified := False;
  FFieldsListModified := False;
  FContainerPermissionsListModified := False;
  FContainerFieldsListModified := False; 
  FListOfModifications := TList<TbfaModification>.Create;
  FListOfModifications.Clear; 
  FFrameType := -1;
end;
//
procedure TbfaAdminArchive.ClearAllProperties;
var
  i : Integer;
begin
  inherited ClearAllProperties;
  FInheritPermissions:= False;
  FInheritFields:= False;
  for i := 0 to FFieldsList.Count - 1 do
    FFieldsList.Items[i].Free;
  FFieldsList.Clear;
  for i := 0 to FPermissionsList.Count - 1 do
    FPermissionsList.Items[i].Free;
  FPermissionsList.Clear;
  for i := 0 to FContainerFieldsList.Count - 1 do
    FContainerFieldsList.Items[i].Free;
  FContainerFieldsList.Clear;
  for i := 0 to FContainerPermissionsList.Count - 1 do
    FContainerPermissionsList.Items[i].Free;
  FContainerPermissionsList.Clear;
  FModified:= False;
  FInheritPermissionsModified:= False;
  FInheritFieldsModified:= False;
  FGeneralPropertiesModified := False;
  FPermissionsListModified := False;
  FFieldsListModified := False;
  FContainerPermissionsListModified := False;
  FContainerFieldsListModified := False;  
  FInSaveProcess := False;
  FFrameType := -1;
end;
//
//  Name: Load
//  Parameters: ArcID
//  Beschreibung: Ldt ein Archiv mit der bergebenen LayoutID aus der Datenbank
//  Description: Loads an Archive with the LayoutID from the Database
//  Return value: FErrorCode
// [] 17.04.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaAdminArchive.Load(LayoutID: Int64): Integer;
var
  i : Integer;
begin
  FErrorCode:= 0;
  if FInitialized then
  begin
    FbfaApplication^.WriteLog(llHigh, ltInfo, 'Load(LayoutID: ' + IntToStr(LayoutID) + ')', ClassName);
    FInLoadProcess := True;
    try
      try
        for i := 0 to FFieldsList.Count - 1 do
          FFieldsList.Items[i].Free;
        FFieldsList.Clear;
        for i := 0 to FPermissionsList.Count - 1 do
          FPermissionsList.Items[i].Free;
        FPermissionsList.Clear;

        FErrorCode:= inherited Load(LayoutID);
        if FErrorCode = 0 then
        begin
          FbfaApplication^.User.Rights.SetApplicationRights(FbfaApplication, FbfaApplication^.User);
          FbfaApplication^.User.Rights.SetArchiveRights(FbfaApplication, FbfaApplication^.User, FLayoutID);

          FModified:= False;
          FInheritPermissionsModified:= False;
          FInheritFieldsModified:= False;
          FGeneralPropertiesModified := False;
          FPermissionsListModified := False;
          FFieldsListModified := False;
          FContainerPermissionsListModified := False;
          FContainerFieldsListModified := False;  
        

          FErrorCode:= LoadPermissions;
          FErrorCode:= LoadFields;
          if FUserContainerID > 0 then
            FErrorCode:= LoadContainerPermissions;
          if FFieldContainerID > 0 then
            FErrorCode:= LoadContainerFields;

        end
        else
          ClearAllProperties;       

      except
        FErrorCode:= ecArchiveFailureInLoadArchive;
        FLoaded:= False;
        FbfaApplication^.WriteLog(llNormal, ltException, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
        Result:= FErrorCode;
      end;
    finally
      FInLoadProcess := False;
    end;
  end
  else
  begin
    FErrorCode:= ecArchiveConnectionOrLogNotInitialized;
    FLoaded:= False;
  end;
  if FErrorCode <> 0 then
    FbfaApplication^.WriteLog(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
  Result:= FErrorCode;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaAdminArchive.Save: Integer;
var
  TmpArc
  : TbfaAdminArchive;
  
  FPermissionListErrorCode,
  FFieldListErrorCode,
  FContainerPermissionListErrorCode,
  FContainerFieldListErrorCode
  : Integer;
  
begin
  FErrorCode:= 0;
  FPermissionListErrorCode := 0;
  FFieldListErrorCode := 0;
  FContainerPermissionListErrorCode := 0;
  FContainerFieldListErrorCode := 0;
  if FInitialized then
  begin
    if FLoaded then
    begin
      FbfaApplication^.WriteLog(llHigh, ltInfo, 'Save(ArchiveID: ' + IntToStr(FID) + ')', ClassName);
      FbfaApplication^.Connection.MySQLTransaction.StartTransaction;
      try

        if FGeneralPropertiesModified then
        begin
          if FbfaApplication^.Connection.UpdateArchive(FLayoutID, FID, FName, FBgrPicture, FDocName, FDescription, FInheritPermissions, FInheritFields, False, FIcon, FKind) >= 0 then
            FListOfModifications.Add(TbfaModification.Create(TbfaBaseObject(Self), mdtNameModified, -1, -1))
          else
            FErrorCode := ecArchiveFailureInSave;
        end;

        if FGUILayoutModified then
        begin
          FErrorCode := SaveGUILayout;
        end;


        if FErrorCode = 0 then
        begin
          FPermissionListErrorCode := SavePermissionsList;
          if FPermissionListErrorCode <> 0 then
            FErrorCode := FPermissionListErrorCode;
          FFieldListErrorCode := SaveFieldsList;
          if FFieldListErrorCode <> 0 then
            FErrorCode := FFieldListErrorCode;
          FContainerPermissionListErrorCode := SaveContainerPermissionsList;
          if FContainerPermissionListErrorCode <> 0 then
            FErrorCode := FContainerPermissionListErrorCode;
          FContainerFieldListErrorCode := SaveContainerFieldsList;
          if FContainerFieldListErrorCode <> 0 then
            FErrorCode := FContainerFieldListErrorCode;
        end;
        
        if ((FInheritPermissionsModified) or (FInheritFieldsModified)) then
        begin
          if ((FInheritPermissions) or (FInheritFields)) then
          begin
            TmpArc:= TbfaAdminArchive.Create(FbfaApplication);
            try
              TmpArc.Load(FParentID);
              TmpArc.PermissionsListModified := FInheritPermissions;
              TmpArc.FieldsListModified := FInheritFields;
              if FInheritPermissions then
                FErrorCode := TmpArc.SavePermissionsList;
              if FInheritFields then
                FErrorCode := TmpArc.SaveFieldsList;
            finally
              FreeAndNil(TmpArc);
            end;
          end;
        end;
        
        if FErrorCode = 0 then
        begin
          if FInheritPermissionsModified then
          begin
            if FInheritPermissions then
            begin
              LoadPermissions;
            end;
          end;        
          if FInheritFieldsModified then
          begin
            if FInheritFields then
            begin
              LoadFields;
            end;
          end;        
          FbfaApplication^.Connection.MySQLTransaction.Commit;
          FModified:= False;
          FInheritPermissionsModified:= False;
          FInheritFieldsModified:= False;
          FGeneralPropertiesModified := False;
          FPermissionsListModified := False;
          FFieldsListModified := False;
          FContainerPermissionsListModified := False;
          FContainerFieldsListModified := False; 
          FGUILayoutModified := False; 
        end
        else
        begin
          FbfaApplication^.Connection.MySQLTransaction.Rollback;
          FbfaApplication^.WriteLog(llNormal, ltException, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
        end;
        
        
        Result:= FErrorCode;
      except
        FbfaApplication^.Connection.MySQLTransaction.Rollback;
        FErrorCode:= ecArchiveFailureInSave;
        FbfaApplication^.WriteLog(llNormal, ltException, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
        Result:= FErrorCode;
      end;
    end
    else
    begin
      FErrorCode:= ecArchiveIsNotLoaded;
      Result:= FErrorCode;
    end;
  end
  else
  begin
    FErrorCode:= ecArchiveConnectionOrLogNotInitialized;
    Result:= FErrorCode;
  end;
end;
//
function TbfaAdminArchive.SavePermissionsList: Integer;
var
  i: Integer;
begin
  FErrorCode:= 0;
  if FInitialized then
  begin
    if FLoaded then
    begin
      if FPermissionsListModified then
      begin
        FbfaApplication^.WriteLog(llHigh, ltInfo, 'SavePermissionsList', ClassName);
        try
          for i:= 0 to FPermissionsList.Count - 1 do
          begin
            if ((FPermissionsList.Items[i].Modified) {or (FPermissionsListModified)})then
            begin
              if (FPermissionsList.Items[i].Status = stDeleted) then
                FbfaApplication^.Connection.DeleteObjectFromUserMapping(FLayoutID, FPermissionsList.Items[i].Obj.ID, FPermissionsList.Items[i].Obj.ObjType)
              else
                FbfaApplication^.Connection.AddObjectToUserMapping(FLayoutID, FPermissionsList.Items[i].Obj.ID, FPermissionsList.Items[i].Right, FPermissionsList.Items[i].MappingID, FPermissionsList.Items[i].Obj.ObjType);
            end;
            if FPermissionsList.Items[i].Obj.Modified then // Wenn Benutzer oder Gruppe ObjectProperties sind gendert
            begin
              if FPermissionsList.Items[i].Obj.ObjType = otUser then
                TbfaUser(FPermissionsList.Items[i].Obj).Save
              else if FPermissionsList.Items[i].Obj.ObjType = otGroup then
                TbfaGroup(FPermissionsList.Items[i].Obj).Save;

              FListOfModifications.Add(TbfaModification.create(FPermissionsList.Items[i].Obj, mdtNameModified, -1, -1));

              RefreshObject(FPermissionsList.Items[i].Obj, mdtObjectModified);
            end;
          end;

          for i := FPermissionsList.Count - 1 downto 0 do
          begin
            if FPermissionsList.Items[i].Modified then
            begin
              if FPermissionsList.Items[i].Status = stDeleted then
                FPermissionsList.Delete(i)
              else
                FPermissionsList.Items[i].Modified := False;
            end;
          end;          
          
          FPermissionsListModified := False;
          
          Result:= FErrorCode;
        except
          FErrorCode:= ecArchiveFailureInSaveUsersAndGroupsContainer;
          FbfaApplication^.WriteLog(llNormal, ltException, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
          Result:= FErrorCode;
        end;
      end
      else
        Result := FErrorCode;
    end
    else
    begin
      FErrorCode:= ecArchiveIsNotLoaded;
      Result:= FErrorCode;
    end;
  end
  else
  begin
    FErrorCode:= ecArchiveConnectionOrLogNotInitialized;
    Result:= FErrorCode;
  end;
end;
//
function TbfaAdminArchive.SaveContainerPermissionsList: Integer;
var
  i : Integer;
  NewID : Int64;
begin
  FErrorCode := 0;
  if FInitialized then
  begin
    if FLoaded then
    begin
      if FContainerPermissionsListModified then
      begin
        FbfaApplication^.WriteLog(llHigh, ltInfo, 'SaveContainerPermissionsList', ClassName);
        try
          for i := 0 to FContainerPermissionsList.Count - 1 do
          begin
            if FContainerPermissionsList.Items[i].Modified then
            begin
              if FContainerPermissionsList.Items[i].Status = stActive then
              begin
                if FContainerPermissionsList.Items[i].LayoutID = -1 then
                begin
                  NewID := FbfaApplication^.Connection.AddObjectToLayout(FUserContainerID, FContainerPermissionsList.Items[i].Obj.ID, FContainerPermissionsList.Items[i].Obj.ObjType, '', False, False);
                  FListOfModifications.Add(TbfaModification.Create(FContainerPermissionsList.Items[i].Obj, mdtAdded, FUserContainerID, NewID));
                end
                else
                  NewID:= FContainerPermissionsList.Items[i].LayoutID;

                if FbfaApplication^.Connection.UpdateLayoutRights(NewID, FContainerPermissionsList.Items[i].Right, FContainerPermissionsList.Items[i].MappingID) = 0 then            
                  FContainerPermissionsList.Items[i].LayoutID := NewID;
                
              end
              else if FContainerPermissionsList.Items[i].Status = stDeleted then
              begin
                if FContainerPermissionsList.Items[i].LayoutID <> -1 then
                begin
                  FbfaApplication^.Connection.DeleteObject(FContainerPermissionsList.Items[i].LayoutID);
                  FListOfModifications.Add(TbfaModification.Create(FContainerPermissionsList.Items[i].Obj, mdtDeleted, FUserContainerID, FContainerPermissionsList.Items[i].LayoutID));
                end;
              end;            
                
            end;
            if FContainerPermissionsList.Items[i].Obj.Modified then // Wenn Benutzer oder Gruppe ObjectProperties sind gendert
            begin
              if FContainerPermissionsList.Items[i].Obj.ObjType = otUser then
                TbfaUser(FContainerPermissionsList.Items[i].Obj).Save
              else if FContainerPermissionsList.Items[i].Obj.ObjType = otGroup then
                TbfaGroup(FContainerPermissionsList.Items[i].Obj).Save;

              FListOfModifications.Add(TbfaModification.Create(FContainerPermissionsList.Items[i].Obj, mdtNameModified, -1, -1));

              RefreshObject(FContainerPermissionsList.Items[i].Obj, mdtObjectModified);
            end;
          end;
          
          for i := FContainerPermissionsList.Count - 1 downto 0 do
          begin
            if FContainerPermissionsList.Items[i].Modified then
            begin
              if FContainerPermissionsList.Items[i].Status = stDeleted then
                FContainerPermissionsList.Delete(i)
              else
                FContainerPermissionsList.Items[i].Modified := False;
              
            end;
          end;

          FContainerPermissionsListModified := False;

          Result:= FErrorCode;
        except
          FErrorCode:= ecArchiveFailureInSaveContainerPermissionsList;
          FbfaApplication^.WriteLog(llNormal, ltException, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
          Result:= FErrorCode;
        end;
      end
      else
        Result := FErrorCode;
    end
    else
    begin
      FErrorCode:= ecArchiveIsNotLoaded;
      Result:= FErrorCode;
    end;
  end
  else
  begin
    FErrorCode:= ecArchiveConnectionOrLogNotInitialized;
    Result:= FErrorCode;
  end;
end;
//
function TbfaAdminArchive.SaveFieldsList: Integer;
var
  i, j, k, Itms, Max, cnt: Integer;
  UReadRights, UWriteRights, UModifyRights,
  UUnsetRights,
  GReadRights, GWriteRights, GModifyRights,
  GUnsetRights: UInt64;
begin
  FErrorCode:= 0;
  if FInitialized then
  begin
    if FLoaded then
    begin
      if FFieldsListModified then
      begin
        FbfaApplication^.WriteLog(llHigh, ltInfo, 'SaveFieldsList', ClassName);
        try
          for i:= 0 to FFieldsList.Count - 1 do
          begin
            if ((FFieldsList.Items[i].Modified){ or (FFieldsListModified)}) then
            begin
              if FFieldsList.Items[i].Status = stDeleted then
                FbfaApplication^.Connection.DeleteObjectFromFieldMapping(FLayoutID, TbfaAdminField(FFieldsList.Items[i].Obj).FieldID)
              else
              begin
                FbfaApplication^.Connection.AddObjectToFieldMapping(FLayoutID, TbfaAdminField(FFieldsList.Items[i].Obj).FieldID, FID, TbfaAdminField(FFieldsList.Items[i].Obj).FieldType, 
                          TbfaAdminField(FFieldsList.Items[i].Obj).DefaultValue, TbfaAdminField(FFieldsList.Items[i].Obj).Description, 
                          TbfaAdminField(FFieldsList.Items[i].Obj).ControlItems, TbfaAdminField(FFieldsList.Items[i].Obj).Mode, 
                          TbfaAdminField(FFieldsList.Items[i].Obj).Options);
              end;
            end;

            if (FFieldsList.Items[i].Obj.FieldPermissionsModified) then
            begin
              if FMaxUserColumn >= 0 then
              begin
                Itms:= 0;

                for j := 0 to FMaxUserColumn do
                begin
                  UReadRights:= 0;
                  UWriteRights:= 0;
                  UModifyRights:= 0;
                  UUnsetRights:= 0;
                  cnt:= 0;

                  for k := 0 to FFieldsList.Items[i].Obj.FieldPermissions.Count - 1 do
                  begin
                    if FFieldsList.Items[i].Obj.FieldPermissions.Items[k].Obj.Modified then
                    begin
                      if FFieldsList.Items[i].Obj.FieldPermissions.Items[k].Obj.ObjType = otUser then
                      begin
                        if ((FFieldsList.Items[i].Obj.FieldPermissions.Items[k].Obj.ID >= Itms) and (FFieldsList.Items[i].Obj.FieldPermissions.Items[k].Obj.ID <= Itms + 63)) then
                        begin
                          cnt := FFieldsList.Items[i].Obj.FieldPermissions.Items[k].Obj.ID - 1;
                          if FFieldsList.Items[i].Obj.FieldPermissions.Items[k].Obj.Status = stActive then
                          begin
                            if (FFieldsList.Items[i].Obj.FieldPermissions.Items[k].ReadRight) then
                              UReadRights:= UReadRights + bfaPower(cnt);
                            if (FFieldsList.Items[i].Obj.FieldPermissions.Items[k].WriteRight) then
                              UWriteRights:= UWriteRights + bfaPower(cnt);
                            if (FFieldsList.Items[i].Obj.FieldPermissions.Items[k].ModifyRight) then
                              UModifyRights:= UModifyRights + bfaPower(cnt);                        
                          end
                          else if FFieldsList.Items[i].Obj.FieldPermissions.Items[k].Obj.Status = stDeleted then
                          begin
                            UUnsetRights:= UUnsetRights + bfaPower(cnt);
                          end;  
                        end;

                      end;
                    end;
                  
                  end;
                
                  FbfaApplication^.Connection.UpdateInheritedFieldRights(FLayoutID, FID, TbfaAdminField(FFieldsList.Items[i].Obj).FieldID, j, otUser, UReadRights, UWriteRights, UModifyRights, UUnsetRights);

                  Itms:= Itms + 64;
                end;
              end;

              if FMaxGroupColumn >= 0 then
              begin
                Itms:= 0;

                for j := 0 to FMaxGroupColumn do
                begin
                  GReadRights:= 0;
                  GWriteRights:= 0;
                  GModifyRights:= 0;
                  GUnsetRights:= 0;
                  cnt:= 0;

                  for k := 0 to FFieldsList.Items[i].Obj.FieldPermissions.Count - 1 do
                  begin
                    if FFieldsList.Items[i].Obj.FieldPermissions.Items[k].Obj.Modified then
                    begin
                      if FFieldsList.Items[i].Obj.FieldPermissions.Items[k].Obj.ObjType = otGroup then
                      begin
                        if ((FFieldsList.Items[i].Obj.FieldPermissions.Items[k].Obj.ID >= Itms) and (FFieldsList.Items[i].Obj.FieldPermissions.Items[k].Obj.ID <= Itms + 63)) then
                        begin
                          cnt := FFieldsList.Items[i].Obj.FieldPermissions.Items[k].Obj.ID - 1;
                          if FFieldsList.Items[i].Obj.FieldPermissions.Items[k].Obj.Status = stActive then
                          begin
                            if (FFieldsList.Items[i].Obj.FieldPermissions.Items[k].ReadRight) then
                              GReadRights := GReadRights + bfaPower(cnt);
                            if (FFieldsList.Items[i].Obj.FieldPermissions.Items[k].WriteRight) then
                              GWriteRights := GWriteRights + bfaPower(cnt);
                            if (FFieldsList.Items[i].Obj.FieldPermissions.Items[k].ModifyRight) then
                              GModifyRights := GModifyRights + bfaPower(cnt);                        
                          end
                          else if FFieldsList.Items[i].Obj.FieldPermissions.Items[k].Obj.Status = stDeleted then
                          begin
                            GUnsetRights := GUnsetRights + bfaPower(cnt);
                          end;  
                        end;

                      end;
                    end;
                  
                  end;
                
                  FbfaApplication^.Connection.UpdateInheritedFieldRights(FLayoutID, FID, TbfaAdminField(FFieldsList.Items[i].Obj).FieldID, j, otGroup, GReadRights, GWriteRights, GModifyRights, GUnsetRights);

                  Itms:= Itms + 64;
                end;
              end;                
            
            end;


            if FFieldsList.Items[i].Obj.Modified then // Wenn der Feld ObjectProperties sind gendert
            begin
              TbfaAdminField(FFieldsList.Items[i].Obj).Save;

              FListOfModifications.Add(TbfaModification.create(FFieldsList.Items[i].Obj, mdtNameModified, -1, -1));

              RefreshObject(FFieldsList.Items[i].Obj, mdtObjectModified);
            end;
            
            
          end;

          for i := FFieldsList.Count - 1 downto 0 do
          begin
            if FFieldsList.Items[i].Modified then
            begin
              if FFieldsList.Items[i].Status = stDeleted then
                FFieldsList.Delete(i)
              else
                FFieldsList.Items[i].Modified := False;
            end;
            if FFieldsList.Items[i].Obj.FieldPermissionsModified then
            begin
              for j := FFieldsList.Items[i].Obj.FieldPermissions.Count - 1 downto 0 do
              begin
                if FFieldsList.Items[i].Obj.FieldPermissions.Items[j].Obj.Status = stDeleted then
                  FFieldsList.Items[i].Obj.FieldPermissions.Delete(j)
                else
                begin
                  FFieldsList.Items[i].Obj.FieldPermissionsModified := False;
                  FFieldsList.Items[i].Obj.Modified := False;
                end;
              end;
            end;
          end;   
          FFieldsListModified := False;
          
          Result:= FErrorCode;
        except
          FErrorCode:= ecArchiveFailureInSaveAdditionalFields;
          FbfaApplication^.WriteLog(llNormal, ltException, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
          Result:= FErrorCode;
        end;
      end
      else
        Result := FErrorCode;      
    end
    else
    begin
      FErrorCode:= ecArchiveIsNotLoaded;
      Result:= FErrorCode;
    end;
  end
  else
  begin
    FErrorCode:= ecArchiveConnectionOrLogNotInitialized;
    Result:= FErrorCode;
  end;
end;
//
function TbfaAdminArchive.SaveContainerFieldsList: Integer;
var
  i : Integer;
  NewID : Int64;
begin
  FErrorCode:= 0;
  if FInitialized then
  begin
    if FLoaded then
    begin
      if FContainerFieldsListModified then
      begin
        FbfaApplication^.WriteLog(llHigh, ltInfo, 'SaveContainerFieldsList', ClassName);
        try
          for i:= 0 to FContainerFieldsList.Count - 1 do
          begin
            if FContainerFieldsList.Items[i].Modified then
            begin
              if FContainerFieldsList.Items[i].Status = stActive then
              begin
                NewID:= FContainerFieldsList.Items[i].LayoutID;  
                if NewID = -1 then
                begin
                  NewID:= FbfaApplication^.Connection.AddObjectToLayout(FFieldContainerID, FContainerFieldsList.Items[i].Obj.ID, FContainerFieldsList.Items[i].Obj.ObjType, '', False, False);
                  FListOfModifications.Add(TbfaModification.Create(FContainerFieldsList.Items[i].Obj, mdtAdded, FFieldContainerID, NewID));                  
                  if NewID > 0 then
                    FContainerFieldsList.Items[i].LayoutID := NewID;
                end;

              end
              else if FContainerFieldsList.Items[i].Status = stDeleted then
              begin
                if FContainerFieldsList.Items[i].LayoutID <> -1 then
                begin
                  FbfaApplication^.Connection.DeleteObject(FContainerFieldsList.Items[i].LayoutID);
                end;
              end;              
            end;
            if FContainerFieldsList.Items[i].Obj.Modified then // Wenn der Feld ObjectProperties sind gendert
            begin
              (FContainerFieldsList.Items[i].Obj as TbfaField).Save;

              FListOfModifications.Add(TbfaModification.Create(FContainerFieldsList.Items[i].Obj, mdtNameModified, -1, -1));

              RefreshObject(FContainerFieldsList.Items[i].Obj, mdtObjectModified);
            end;
          end;

          for i := FContainerFieldsList.Count - 1 downto 0 do
          begin
            if FContainerFieldsList.Items[i].Modified then
            begin
              if FContainerFieldsList.Items[i].Status = stDeleted then
                FContainerFieldsList.Delete(i)
              else
                FContainerFieldsList.Items[i].Modified := False;
            end;
          end;   
          FFieldsListModified := False;
                    
          Result:= FErrorCode;
        except
          FErrorCode:= ecArchiveFailureInSaveContainerFieldsList;
          FbfaApplication^.WriteLog(llNormal, ltException, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
          Result:= FErrorCode;
        end;
      end
      else
        Result := FErrorCode;
    end
    else
    begin
      FErrorCode:= ecArchiveIsNotLoaded;
      Result:= FErrorCode;
    end;
  end
  else
  begin
    FErrorCode:= ecArchiveConnectionOrLogNotInitialized;
    Result:= FErrorCode;
  end;
end;
//
function TbfaAdminArchive.SaveGUILayout: Integer;
begin
  FErrorCode:= 0;
  if FInitialized then
  begin
    if FLoaded then
    begin
      FbfaApplication^.WriteLog(llHigh, ltInfo, 'SaveGUILayout(ArchiveID: ' + IntToStr(FID) + ')', ClassName);
      try

        FErrorCode:= FbfaApplication^.Connection.UpdateArchiveGUILayout(FLayoutID, FID, FArchiveLayout);

        Result:= FErrorCode;
      except
        FErrorCode:= ecArchiveFailureInSaveGUILayout;
        FbfaApplication^.WriteLog(llNormal, ltException, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
        Result:= FErrorCode;
      end;
    end
    else
    begin
      FErrorCode:= ecArchiveIsNotLoaded;
      Result:= FErrorCode;
    end;
  end
  else
  begin
    FErrorCode:= ecArchiveConnectionOrLogNotInitialized;
    Result:= FErrorCode;
  end;
end;
//
function TbfaAdminArchive.AddObjectToContainerPermissionsList(OID, Rights: Int64; OType: Integer; RoleID: Int64; RName: WideString): Integer;
begin
  if AddObject(FContainerPermissionsList, OID, Rights, OType, RoleID, RName) = 0 then
    SetContainerPermissionsListModified(True);
end;
//
function TbfaAdminArchive.AddObjectToPermissionsList(OID, Rights: Int64; OType: Integer; RoleID: Int64; RName: WideString): Integer;
begin
  if AddObject(FPermissionsList, OID, Rights, OType, RoleID, RName) = 0 then
    SetPermissionsListModified(True);
end;
//
function TbfaAdminArchive.AddObjectToContainerFieldsList(OID, Rights: Int64; OType: Integer; RoleID: Int64; RName: WideString): Integer;
begin
  if AddObject(FContainerFieldsList, OID, Rights, OType, RoleID, RName) = 0 then
    SetContainerFieldsListModified(True);
end;
//
function TbfaAdminArchive.AddObjectToFieldsList(PObj : TbfaAdminField; ControlID: Int64; ControlName: WideString): Integer;
begin
  if AddFieldObject(FFieldsList, PObj, ControlID, ControlName) = 0 then
    SetFieldsListModified(True);
end;
//
function TbfaAdminArchive.AddObject(List : TList<TbfaMapping>; ObjID, Rights: Int64; ObjType: Integer; RoleID: Int64; RName: WideString): Integer;
var
  UserMapping : TbfaMapping;
  Usr : TbfaUser;
  Grp : TbfaGroup;
  Fld : TbfaField;
  Name, Description : WideString;
  FieldType : Integer;
begin
  FErrorCode:= 0;
  if FInitialized then
  begin
    if FLoaded then
    begin
      FbfaApplication^.WriteLog(llHigh, ltInfo, 'AddObject(' + IntToStr(ObjID) + ')', ClassName);
      try
        FieldType := -1;
        if ObjType = otUser then
        begin  
          Usr := TbfaUser.Create(FbfaApplication);
          Usr.Load(ObjID, False);

          Name := Usr.Name;
          Description:= Usr.Description;
        end
        else if ObjType = otGroup then
        begin
          Grp := TbfaGroup.Create(FbfaApplication);
          Grp.Load(ObjID, False);

          Name := Grp.Name;
          Description:= Grp.Description;
        end
        else 
        begin
          Fld := TbfaField.Create(FbfaApplication);
          Fld.Load(ObjID(*, False*));

          Name := Fld.DisplayName;
          Description:= Fld.Description;
        end;  

        UserMapping := TbfaMapping.Create(FbfaApplication, Name, Description, -1, ObjID, Rights, ObjType, stActive, RoleID, RName, True, FieldType);
        List.Add(UserMapping);

        Result := FErrorCode;
      except
        FErrorCode := ecArchiveFailureInAddObject;
        FbfaApplication^.WriteLog(llNormal, ltException, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
        Result := FErrorCode;
      end;
    end
    else
    begin
      FErrorCode := ecArchiveFailureInAddObject;
      Result := FErrorCode;
    end;
  end
  else
  begin
    FErrorCode := ecArchiveConnectionOrLogNotInitialized;
    Result := FErrorCode;
  end;
end;
//
function TbfaAdminArchive.AddFieldObject(List : TList<TbfaFieldMapping>; PObj : TbfaAdminField; ControlID : Int64; ControlName : WideString) : Integer;
var
  FieldMapping : TbfaFieldMapping;
begin
  FErrorCode:= 0;
  if FInitialized then
  begin
    if FLoaded then
    begin
      FbfaApplication^.WriteLog(llHigh, ltInfo, 'AddFieldObject(' + IntToStr(PObj.ID) + ')', ClassName);
      try
        FieldMapping := TbfaFieldMapping.Create(PObj, -1, stActive, ControlID, ControlName, True);
        List.Add(FieldMapping);

        Result := FErrorCode;
      except
        FErrorCode := ecArchiveFailureInAddObject;
        FbfaApplication^.WriteLog(llNormal, ltException, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
        Result := FErrorCode;
      end;
    end
    else
    begin
      FErrorCode := ecArchiveFailureInAddObject;
      Result := FErrorCode;
    end;
  end
  else
  begin
    FErrorCode := ecArchiveConnectionOrLogNotInitialized;
    Result := FErrorCode;
  end;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
procedure TbfaAdminArchive.Clone(Source: TbfaAdminArchive);
begin
  try
    inherited Clone(Source);
    FFieldsList := Source.FieldsList;
    FPermissionsList := Source.PermissionsList;
    FModified := Source.Modified;
    FInheritPermissionsModified := Source.InheritPermissionsModified;
    FInheritFieldsModified := Source.InheritFieldsModified;
    FModified := Source.Modified;
    FGeneralPropertiesModified := Source.GeneralPropertiesModified;
    FPermissionsListModified := Source.PermissionsListModified;
    FFieldsListModified := Source.FieldsListModified;
    FContainerPermissionsListModified := Source.ContainerPermissionsListModified;
    FContainerFieldsListModified := Source.ContainerFieldsListModified; 
    FFrameType := Source.FrameType; 
  except

  end;
end;
//
function TbfaAdminArchive.RemoveObjectFromContainerPermissionsList(Idx : Integer): Integer;
begin
  if RemoveObject(FContainerPermissionsList, Idx) = 0 then
    SetContainerPermissionsListModified(True);
end;
//
function TbfaAdminArchive.RemoveObjectFromPermissionsList(Idx : Integer): Integer;
begin
  if RemoveObject(FPermissionsList, Idx) = 0 then
    SetPermissionsListModified(True);
end;
//
function TbfaAdminArchive.RemoveObjectFromContainerFieldsList(Idx : Integer): Integer;
begin
  if RemoveObject(FContainerFieldsList, Idx) = 0 then
    SetContainerFieldsListModified(True);
end;
//
function TbfaAdminArchive.RemoveObjectFromFieldsList(Idx : Integer): Integer;
begin
  if RemoveObject(TList<TbfaMapping>(FFieldsList), Idx) = 0 then
    SetFieldsListModified(True);
end;
//
function TbfaAdminArchive.RemoveObject(List : TList<TbfaMapping>; Idx : Integer): Integer;
begin
  FErrorCode:= 0;
  if FInitialized then
  begin
    if FLoaded then
    begin
      FbfaApplication^.WriteLog(llHigh, ltInfo, 'RemoveObject(' + IntToStr(Idx) + ')', ClassName);
      try

        List.Items[idx].Status := stDeleted;
        List.Items[idx].Modified := True;

        Result := FErrorCode;
      except
        FErrorCode := ecArchiveFailureInRemoveObject;
        FbfaApplication^.WriteLog(llNormal, ltException, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
        Result := FErrorCode;
      end;
    end
    else
    begin
      FErrorCode := ecArchiveFailureInRemoveObject;
      Result := FErrorCode;
    end;
  end
  else
  begin
    FErrorCode := ecArchiveConnectionOrLogNotInitialized;
    Result := FErrorCode;
  end;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaAdminArchive.AddField(FieldID: Int64): Integer;
var
  Fld: TbfaAdminField;
  BaseField: TbfaBaseField;
  Mapping : TbfaMapping;
begin
  FErrorCode:= 0;
  if FInitialized then
  begin
    if FLoaded then
    begin
      FbfaApplication^.WriteLog(llHigh, ltInfo, 'AddField(' + IntToStr(FieldID) + ')', ClassName);
      try
      (*
        BaseField:= TbfaBaseField.Create(FbfaApplication);
//        BaseField.Init(FbfaApplication^.Connection, FbfaApplication.Log);
        FErrorCode:= BaseField.Load(FieldID);      

        Fld:= TbfaAdminField.Create(FbfaApplication);
        Fld.ArchiveID:= -1;
        Fld.ArchiveLayoutID:= -1;
        Fld.Load(FieldID);
        Fld.Modified:= True; // wegen die Containers muss true gesetzt werden
  
        Mapping := TbfaMapping.Create(FbfaApplication, );
  
        FFieldsList.Add(Fld);   

        SetFieldsListModified(True);
                                *)
        Result:= FErrorCode;
      except
        FErrorCode:= ecArchiveFailureInAddAdditionalField;
        FbfaApplication^.WriteLog(llNormal, ltException, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
        Result:= FErrorCode;
      end;
    end
    else
    begin
      FErrorCode:= ecArchiveFailureInAddAdditionalField;
      Result:= FErrorCode;
    end;
  end
  else
  begin
    FErrorCode:= ecArchiveConnectionOrLogNotInitialized;
    Result:= FErrorCode;
  end;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaAdminArchive.RemoveField(Idx: Integer): Integer;
begin
  FErrorCode:= 0;
  if FInitialized then
  begin
    if FLoaded then
    begin
      FbfaApplication^.WriteLog(llHigh, ltInfo, 'RemoveField(' + IntToStr(Idx) + ')', ClassName);
      try

        TbfaAdminField(FFieldsList.Items[Idx]).Status:= stDeleted;
        TbfaAdminField(FFieldsList.Items[Idx]).Modified:= True;

        SetFieldsListModified(True);

        Result:= FErrorCode;
      except
        FErrorCode:= ecArchiveFailureInRemoveAdditionalField;
        FbfaApplication^.WriteLog(llNormal, ltException, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
        Result:= FErrorCode;
      end;
    end
    else
    begin
      FErrorCode:= ecArchiveFailureInRemoveAdditionalField;
      Result:= FErrorCode;
    end;
  end
  else
  begin
    FErrorCode:= ecArchiveConnectionOrLogNotInitialized;
    Result:= FErrorCode;
  end;
end;
//
function TbfaAdminArchive.LoadFields: Integer;
var
  dsc: TbfaSP;
  FieldMapping : TbfaFieldMapping;
  i: Integer;
  Field : TbfaAdminField;
begin
  FErrorCode:= 0;
  if FInitialized then
  begin
    if FLoaded then
    begin
      try
        try  
          for i := 0 to FFieldsList.Count - 1 do
            FFieldsList.Items[i].Free;
          FFieldsList.Clear;
          dsc:= FbfaApplication^.Connection.SelectFields(FID);
          if dsc.RecordCount > 0 then
          begin
            dsc.First;
            while not dsc.Eof do
            begin
              Field := TbfaAdminField.Create(FbfaApplication, FMaxUserColumn, FMaxGroupColumn);
              Field.ID := dsc.FieldByNameAsLargeInt(sColAddFieldID);
              Field.Name := dsc.FieldByNameAsWideString(sColDisplayName);
              Field.Description := dsc.FieldByNameAsWideString(sColDescription);
              Field.FieldType := dsc.FieldByNameAsInteger(sColFieldType);
              Field.ArchiveID := FID;
              Field.ArchiveLayoutID := FLayoutID;

              FieldMapping := TbfaFieldMapping.Create(Field, -1, stActive, 0, '', False);
              
              FieldMapping.Obj.ControlType := dsc.FieldByNameAsInteger(sColControlType);  //Im LayoutDesigner ist es wichtig

              if dsc.FieldByNameAsInteger(sColControlType) = ctEditBox then
                FieldMapping.MappingName:= sCaptionEditBox
              else if dsc.FieldByNameAsInteger(sColControlType) = ctDateTime then
                FieldMapping.MappingName:= sCaptionDateTime
              else if dsc.FieldByNameAsInteger(sColControlType) = ctComboBox then
                FieldMapping.MappingName:= sCaptionComboBox
              else if dsc.FieldByNameAsInteger(sColControlType) = ctCheckComboBox then
                FieldMapping.MappingName:= sCaptionCheckComboBox
              else if dsc.FieldByNameAsInteger(sColControlType) = ctInterval then
                FieldMapping.MappingName:= sCaptionInterval
              else if dsc.FieldByNameAsInteger(sColControlType) = ctInteger then
                FieldMapping.MappingName:= sCaptionInteger
              else if dsc.FieldByNameAsInteger(sColControlType) = ctFloat then
                FieldMapping.MappingName:= sCaptionFloat
              else if dsc.FieldByNameAsInteger(sColControlType) = ctCurrency then
                FieldMapping.MappingName:= sCaptionCurrency
              else
                FieldMapping.MappingName:= '';  
                                                                       

              FFieldsList.Add(FieldMapping);        
              dsc.Next;
            end;
            dsc.Close;
          end;

          Result:= FErrorCode;
        except
          FErrorCode:= ecArchiveFailureInLoadAdditionalFields;
          Result:= FErrorCode;
        end;
      finally
        FreeAndNil(dsc);
      end;
    end
    else
    begin
      FErrorCode:= ecArchiveFailureInLoadAdditionalFields;
      Result:= FErrorCode;
    end;
  end
  else
  begin
    FErrorCode:= ecArchiveConnectionOrLogNotInitialized;
    Result:= FErrorCode;
  end;   
end;  
//
function TbfaAdminArchive.LoadContainerFields: Integer;
var
  dsc: TbfaSP;
  Mapping: TbfaMapping;
  i: Integer;
begin
  FErrorCode:= 0;
  if FInitialized then
  begin
    if FLoaded then
    begin
      try
        try  
          for i := 0 to FcontainerFieldsList.Count - 1 do
            FContainerFieldsList.Items[i].Free;
          FContainerFieldsList.Clear;
          dsc:= FbfaApplication^.Connection.GetContainerObjects(FLayoutID, otAddFieldContainer, 0, 1, '');
          if dsc.RecordCount > 0 then
          begin
            dsc.First;
            while not dsc.Eof do
            begin

              Mapping:= TbfaMapping.Create(FbfaApplication, dsc.FieldByNameAsWideString(sColDisplayname), dsc.FieldByNameAsWideString(sColFieldDescription), dsc.FieldByNameAsLargeInt(sColID), dsc.FieldByNameAsLargeInt(sColAddFieldID), dsc.FieldByNameAsLargeInt(sColRights), dsc.FieldByNameAsInteger(sColFieldType), stActive, 0, '', False);

              if dsc.FieldByNameAsInteger(sColFieldType) = otAdditionalField then
              begin
                if dsc.FieldByNameAsInteger(sColControlType) = ctEditBox then
                  Mapping.MappingName:= sCaptionEditBox
                else if dsc.FieldByNameAsInteger(sColControlType) = ctDateTime then
                  Mapping.MappingName:= sCaptionDateTime
                else if dsc.FieldByNameAsInteger(sColControlType) = ctComboBox then
                  Mapping.MappingName:= sCaptionComboBox
                else if dsc.FieldByNameAsInteger(sColControlType) = ctCheckComboBox then
                  Mapping.MappingName:= sCaptionCheckComboBox
                else if dsc.FieldByNameAsInteger(sColControlType) = ctInterval then
                  Mapping.MappingName:= sCaptionInterval
                else if dsc.FieldByNameAsInteger(sColControlType) = ctInteger then
                  Mapping.MappingName:= sCaptionInteger
                else if dsc.FieldByNameAsInteger(sColControlType) = ctFloat then
                  Mapping.MappingName:= sCaptionFloat
                else if dsc.FieldByNameAsInteger(sColControlType) = ctCurrency then
                  Mapping.MappingName:= sCaptionCurrency;
              end
              else if dsc.FieldByNameAsInteger(sColFieldType) = otStatusField then
                Mapping.MappingName:= sCaptionStatusField;
          
            
              FContainerFieldsList.Add(Mapping);
        
              dsc.Next;
            end;
            dsc.Close;
          end;

          Result:= FErrorCode;
        except
          FErrorCode:= ecArchiveFailureInLoadContainerFields;
          Result:= FErrorCode;
        end;
      finally
        FreeAndNil(dsc);
      end;
    end
    else
    begin
      FErrorCode:= ecArchiveFailureInLoadAdditionalFields;
      Result:= FErrorCode;
    end;
  end
  else
  begin
    FErrorCode:= ecArchiveConnectionOrLogNotInitialized;
    Result:= FErrorCode;
  end;  
end; 
//
function TbfaAdminArchive.RefreshObject(Obj : TbfaBaseObject; Mdt : TbfaModificationType): Integer;
var
  i, idx : Integer;
  found : Boolean;

  function GetIdx(List : TList<TbfaMapping>) : Integer;
  begin
    try
      i := 0;
      found := False;
      while ((not found) and (i <= List.Count - 1)) do
      begin
        if ((List.Items[i].Obj.ID = Obj.ID) and (List.Items[i].Obj.ObjType = Obj.ObjType) and (List.Items[i].Status = stActive)) then
        begin
          found := True;
        end;
        inc(i);
      end;
      if found then 
        Result := i - 1
      else
        Result := -1;
    except 
      Result := -1;      
    end;
  end;

  procedure UpdateObject(List : TList<TbfaMapping>; ix : Integer);
  begin
    if Mdt = mdtDeleted then
    begin
      List.Items[ix].Status := stDeleted;
    end
    else if (Obj.ObjType = otUser) then
    begin
      if (Mdt = mdtObjectModified) then
        PbfaUser(@List.Items[ix].Obj).Load(List.Items[ix].Obj.ID, PbfaUser(@List.Items[ix].Obj).ListLoaded);
    end
    else if (Obj.ObjType = otGroup) then
    begin
      if (Mdt = mdtObjectModified) then
        PbfaGroup(@List.Items[ix].Obj).Load(List.Items[ix].Obj.ID, PbfaGroup(@List.Items[ix].Obj).ListLoaded);
    end
    else if (Obj.ObjType = otAdditionalField) or (Obj.ObjType = otStatusField) then
    begin
      if (Mdt = mdtObjectModified) then
      begin
        PbfaField(@List.Items[ix].Obj).Load(List.Items[ix].Obj.ID);
      end;
    end 
  end;

  procedure UpdateFieldObject(List : TList<TbfaFieldMapping>; ix : Integer; typ : Integer = -1);
  begin
    if Mdt = mdtDeleted then
    begin
      List.Items[ix].Status := stDeleted;
    end
    else if (Obj.ObjType = otAdditionalField) or (Obj.ObjType = otStatusField) then
    begin
      if (Mdt = mdtObjectModified) then
      begin
        PbfaAdminField(@List.Items[ix].Obj).ArchiveID := FID;
        PbfaAdminField(@List.Items[ix].Obj).ArchiveLayoutID := FLayoutID;
        PbfaAdminField(@List.Items[ix].Obj).Load(List.Items[ix].Obj.ID)

      end;
    end  
  end;  

  
begin
  FErrorCode:= 0;
  if FInitialized then
  begin
    if FLoaded then
    begin
      if ((Obj.ObjType = otUser) or (Obj.ObjType = otGroup)) then
      begin
        idx := GetIdx(FPermissionsList);
        if idx > -1 then
          UpdateObject(FPermissionsList, idx);

        idx := GetIdx(FContainerPermissionsList);
        if idx > -1 then
          UpdateObject(FContainerPermissionsList, idx);
      end
      else if ((Obj.ObjType = otAdditionalField) or (Obj.ObjType = otStatusField)) then
      begin
        idx := GetIdx(TList<TbfaMapping>(FFieldsList));
        if idx > -1 then
          UpdateFieldObject(FFieldsList, idx);

        idx := GetIdx(FContainerFieldsList);
        if idx > -1 then
          UpdateObject(FContainerFieldsList, idx);
      end;
    end
    else
    begin
      FErrorCode:= ecArchiveFailureInRefreshObject;
      Result:= FErrorCode;
    end;
  end
  else
  begin
    FErrorCode:= ecArchiveConnectionOrLogNotInitialized;
    Result:= FErrorCode;
  end;
end;
//
function TbfaAdminArchive.LoadPermissions: Integer;
var
  dsc: TbfaSP;
  UserMapping: TbfaMapping;
  i : Integer;
begin
  FErrorCode:= 0;
  if FInitialized then
  begin
    if FLoaded then
    begin
      dsc:= FbfaApplication^.Connection.GetObjectPermission(FLayoutID);
      try

        for i := 0 to FPermissionsList.Count - 1 do
          FPermissionsList.Items[i].Free;
        FPermissionsList.Clear;      
        if dsc.RecordCount > 0 then
        begin
          dsc.First;
          while not dsc.Eof do
          begin
            UserMapping:= TbfaMapping.Create(FbfaApplication, dsc.FieldByNameAsWideString(sColName), dsc.FieldByNameAsWideString(sColDescription), -1, 
                                             dsc.FieldByNameAsLargeInt(sColID), dsc.FieldByNameAsLargeInt(sColRights), dsc.FieldByNameAsInteger(sColObjType), 
                                             stActive, dsc.FieldByNameAsLargeInt(sColRoleID), dsc.FieldByNameAsWideString(sColObjValue), False);
            if UserMapping.MappingID = 0 then
              UserMapping.MappingName:= sCaptionUserDefined;
            FPermissionsList.Add(UserMapping);
        
            dsc.Next;
          end;
          dsc.Close;
        end;

        Result:= FErrorCode;
      except
        FErrorCode:= ecArchiveFailureInLoadUsersAndGroups;
        Result:= FErrorCode;
      end;
    end
    else
    begin
      FErrorCode:= ecArchiveFailureInLoadUsersAndGroups;
      Result:= FErrorCode;
    end;
  end
  else
  begin
    FErrorCode:= ecArchiveConnectionOrLogNotInitialized;
    Result:= FErrorCode;
  end;
end;
//
function TbfaAdminArchive.LoadContainerPermissions: Integer;
var
  dsc: TbfaSP;
  UserMapping: TbfaMapping;
  i : Integer;
begin
  FErrorCode:= 0;
  if FInitialized then
  begin
    if FLoaded then
    begin
      dsc:= FbfaApplication^.Connection.GetContainerObjects(FLayoutID, otUserContainer, 0, 1, '');
      try

        for i := 0 to FContainerPermissionsList.Count - 1 do
          FContainerPermissionsList.Items[i].Free;
        FContainerPermissionsList.Clear;      
        if dsc <> nil then
        begin
          dsc.First;
          while not dsc.Eof do
          begin

            if dsc.FieldByNameAsInteger(sColType) = otUser then
            begin          
              UserMapping:= TbfaMapping.Create(FbfaApplication, dsc.FieldByNameAsWideString(sColUserName), dsc.FieldByNameAsWideString(sColUserDescription), dsc.FieldByNameAsLargeInt(sColID), dsc.FieldByNameAsLargeInt(sColUserID), dsc.FieldByNameAsLargeInt(sColRights), otUser, stActive, dsc.FieldByNameAsLargeInt(sColRoleID), dsc.FieldByNameAsWideString(sColObjValue), False);
            end
            else if dsc.FieldByNameAsInteger(sColType) = otGroup then
            begin          
              UserMapping:= TbfaMapping.Create(FbfaApplication, dsc.FieldByNameAsWideString(sColGrpName), dsc.FieldByNameAsWideString(sColGroupDescription), dsc.FieldByNameAsLargeInt(sColID), dsc.FieldByNameAsLargeInt(sColGrpID), dsc.FieldByNameAsLargeInt(sColRights), otGroup, stActive, dsc.FieldByNameAsLargeInt(sColRoleID), dsc.FieldByNameAsWideString(sColObjValue), False);
            end;
            
            if UserMapping.MappingID = 0 then
              UserMapping.MappingName:= sCaptionUserDefined;
            FContainerPermissionsList.Add(UserMapping); 
                    
            dsc.Next;
          end;
          dsc.Close;
        end;

        Result:= FErrorCode;
      except
        FErrorCode:= ecArchiveFailureInLoadContainerPermissions;
        Result:= FErrorCode;
      end;
    end
    else
    begin
      FErrorCode:= ecArchiveFailureInLoadUsersAndGroups;
      Result:= FErrorCode;
    end;
  end
  else
  begin
    FErrorCode:= ecArchiveConnectionOrLogNotInitialized;
    Result:= FErrorCode;
  end;   
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 02.01.2013 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
procedure TbfaAdminArchive.Refresh;
begin
  try
    FErrorCode:= Load(FLayoutID);
  except

  end;
end;
//
procedure TbfaAdminArchive.SetArchiveName(Value: WideString);
begin
  FName:= Value;
  SetGeneralPropertiesModified(True);
end;
//
procedure TbfaAdminArchive.SetBgrPicture(Value: WideString);
begin
  FBgrPicture:= Value;
  SetGeneralPropertiesModified(True);
end;
//
procedure TbfaAdminArchive.SetDocName(Value: WideString);
begin
  FDocName:= Value;
  SetGeneralPropertiesModified(True);
end;
//
procedure TbfaAdminArchive.SetInheritPermissions(Value: Boolean);
begin
  FInheritPermissions:= Value;
  FInheritPermissionsModified:= True;
  SetGeneralPropertiesModified(True);
end;
//
procedure TbfaAdminArchive.SetInheritFields(Value: Boolean);
begin
  FInheritFields:= Value;
  FInheritFieldsModified:= True;
  SetGeneralPropertiesModified(True);
end;
//
procedure TbfaAdminArchive.SetDescription(Value: WideString);
begin
  FDescription:= Value;
  SetGeneralPropertiesModified(True);
end;
//
procedure TbfaAdminArchive.SetIcon(Value: Integer);
begin
  FIcon:= Value;
  SetGeneralPropertiesModified(True);
end;
//
procedure TbfaAdminArchive.SetArcType(Value: Integer);
begin
  FObjType:= Value;
  SetGeneralPropertiesModified(True);
end;
//
procedure TbfaAdminArchive.SetParentType(Value: Integer);
begin
  FParentType:= Value;
  SetGeneralPropertiesModified(True);
end;
//
procedure TbfaAdminArchive.SetParentID(Value: Int64);
begin
  FParentID:= Value;
  SetGeneralPropertiesModified(True);
end;
//
procedure TbfaAdminArchive.SetGeneralPropertiesModified(Value : Boolean);                       
begin
  if Value then
  begin
    FGeneralPropertiesModified := True;
    FModified := True;
  end
  else
    FGeneralPropertiesModified := False;
end;
//
procedure TbfaAdminArchive.SetPermissionsListModified(Value : Boolean);
begin
  if Value then
  begin
    FPermissionsListModified := True;
    FModified := True;
  end
  else
    FPermissionsListModified := False;
end;
//
procedure TbfaAdminArchive.SetFieldsListModified(Value : Boolean);
begin
  if Value then
  begin
    FFieldsListModified := True;
    FModified := True;
  end
  else
    FFieldsListModified := False;
end;
//
procedure TbfaAdminArchive.SetObjectFieldControlType(Idx : Integer; Obj : PbfaBaseObject; ControlID : Integer; ControlName : WideString);
begin
  SetObjectControlType(TList<TbfaMapping>(FFieldsList), Idx, Obj, ControlID, ControlName);                                                                              
  SetFieldsListModified(True);
end;
//
procedure TbfaAdminArchive.SetObjectContainerFieldControlType(Idx : Integer; Obj : PbfaBaseObject; ControlID : Integer; ControlName : WideString);
begin
  SetObjectControlType(FContainerFieldsList, Idx, Obj, ControlID, ControlName);                                                                              
  SetContainerFieldsListModified(True);
end;
//
procedure TbfaAdminArchive.SetObjectControlType(List : TList<TbfaMapping>; Idx : Integer; Obj : PbfaBaseObject; ControlID : Integer; ControlName : WideString);
var
  i : Integer;
  found : Boolean;

  procedure GetIdx;
  begin
    found := False;
    i := 0;
    while ((not found) and (i <= List.Count - 1)) do
    begin
      if ((List.Items[i].Obj.ID = Obj^.ID) and (List.Items[i].Obj.ObjType = Obj^.ObjType)) then
      begin      
        Idx := i;
        found := True;
      end;
      inc(i);
    end;
  end;
  
begin
  if Idx = -1 then
  begin
    GetIdx;
  end;

  if Idx > -1 then
  begin  
    if (
        (List.Items[idx].MappingID <> ControlID) or
        (List.Items[idx].MappingName <> ControlName)
       )then
    begin

      List.Items[idx].MappingID := ControlID;
      List.Items[idx].MappingName := ControlName;

      List.Items[idx].Right := 0;
      List.Items[idx].Modified := True;
    end;    
  end;
end;
//
procedure TbfaAdminArchive.SetContainerPermissionsListModified(Value : Boolean);
begin
  if Value then
  begin
    FContainerPermissionsListModified := True;
    FModified := True;
  end
  else
    FContainerPermissionsListModified := False;
end;
//
procedure TbfaAdminArchive.SetContainerFieldsListModified(Value : Boolean);        
begin
  if Value then
  begin
    FContainerFieldsListModified := True;
    FModified := True;
  end
  else
    FContainerFieldsListModified := False;
end;
//
procedure TbfaAdminArchive.SetObjectPermissionRights(Idx : Integer; Obj : PbfaBaseObject; Rights : Int64; RoleID : Integer; RoleName : WideString);
begin
  SetObjectRights(FPermissionsList, Idx, Obj, Rights, RoleID, RoleName);                                                                              
  SetPermissionsListModified(True);
end;
//
procedure TbfaAdminArchive.SetObjectContainerPermissionRights(Idx : Integer; Obj : PbfaBaseObject; Rights : Int64; RoleID : Integer; RoleName : WideString);
begin
  SetObjectRights(FContainerPermissionsList, Idx, Obj, Rights, RoleID, RoleName);                                                                              
  SetContainerPermissionsListModified(True);
end;
//
procedure TbfaAdminArchive.SetObjectRights(List : TList<TbfaMapping>; Idx : Integer; Obj : PbfaBaseObject; Rights : Int64; RoleID : Integer; RoleName : WideString);
var
  i : Integer;
  found : Boolean;

  procedure GetIdx;
  begin
    found := False;
    i := 0;
    while ((not found) and (i <= List.Count - 1)) do
    begin
      if ((List.Items[i].Obj.ID = Obj^.ID) and (List.Items[i].Obj.ObjType = Obj^.ObjType)) then
      begin      
        Idx := i;
        found := True;
      end;
      inc(i);
    end;
  end;
  
begin
  if Idx = -1 then
  begin
    GetIdx;
    if Idx = -1 then
    begin
      try
        AddObject(List, Obj.ID, Rights, Obj.ObjType, RoleID, RoleName);
        Idx := List.Count - 1;
      except
        Idx := -1;
      end;  
    end;
  end;

  if Idx > -1 then
  begin  
    if (
        (List.Items[idx].Right <> Rights) or
        (List.Items[idx].MappingID <> RoleID)
       )then
    begin

      List.Items[idx].MappingID := RoleID;
      List.Items[idx].MappingName := RoleName;

      List.Items[idx].Right := Rights;
      List.Items[idx].Modified := True;
    end;    
  end;
end;
//
procedure TbfaAdminArchive.GetObjectPermissionRights(Idx : Integer; Obj : PbfaBaseObject; var Rights : Int64; var RoleID : Integer; var RoleName : WideString);
begin
  GetObjectRights(FPermissionsList, Idx, Obj, Rights, RoleID, RoleName);
end;
//
procedure TbfaAdminArchive.GetObjectContainerPermissionRights(Idx : Integer; Obj : PbfaBaseObject; var Rights : Int64; var RoleID : Integer; var RoleName : WideString);
begin
  GetObjectRights(FContainerPermissionsList, Idx, Obj, Rights, RoleID, RoleName);
end;
//
procedure TbfaAdminArchive.GetObjectRights(List : TList<TbfaMapping>; Idx : Integer; Obj : PbfaBaseObject; var Rights : Int64; var RoleID : Integer; var RoleName : WideString);
var
  i : Integer;
  found : Boolean;
begin
  if Idx = -1 then
  begin
    found := False;
    i := 0;
    while ((not found) and (i <= List.Count - 1)) do
    begin
      if ((List.Items[i].Obj.ID = Obj^.ID) and (List.Items[i].Obj.ObjType = Obj^.ObjType)) then
      begin      
        Idx := i;
        found := True;
      end;
      inc(i);
    end;
  end;

  if Idx > -1 then
  begin  
    RoleID := List.Items[idx].MappingID;
    RoleName := List.Items[idx].MappingName;
    Rights := List.Items[idx].Right;
  end
  else
  begin
    RoleID := -1;
    RoleName := sCaptionNone;
    Rights := -1;
  end;
end;
//
constructor TbfaClientArchive.Create(bfaApp: PbfaApplication);
begin
  inherited Create(bfaApp);
  FHasFieldUserRights:= False;
//  FFieldsList:= TList<TbfaClientField>.Create;
  FFieldsList:= TList<TbfaMapping>.Create;
  FFieldsList.Clear;
end;
//
procedure TbfaClientArchive.ClearAllProperties;
var
  i : Integer;
begin
  inherited ClearAllProperties;
  FHasFieldUserRights:= False;
  for i := 0 to FFieldsList.Count - 1 do
    FFieldsList.Items[i].Free;
  FFieldsList.Clear;
  
end;
//
//  Name: Load
//  Parameters: ArcID
//  Beschreibung: Ldt ein Archiv mit der bergebenen LayoutID aus der Datenbank
//  Description: Loads an Archive with the LayoutID from the Database
//  Return value: FErrorCode
// [] 17.04.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaClientArchive.Load(LayoutID: Int64): Integer;
var
  i : Integer;
begin
  FErrorCode:= 0;
  if FInitialized then
  begin
    FbfaApplication^.WriteLog(llHigh, ltInfo, 'Load(LayoutID: ' + IntToStr(LayoutID) + ')', ClassName);
    FInLoadProcess := True;
    try
      try
        for i := 0 to FFieldsList.Count - 1 do
          FFieldsList.Items[i].Free;
        FFieldsList.Clear;

        FHasFieldUserRights := False;

        FErrorCode:= inherited Load(LayoutID);
        if FErrorCode = 0 then
        begin
          FbfaApplication^.User.Rights.SetApplicationRights(FbfaApplication, FbfaApplication^.User);
          FbfaApplication^.User.Rights.SetArchiveRights(FbfaApplication, FbfaApplication^.User, FLayoutID);
        
          FErrorCode:= LoadFields;

          if FErrorCode <> 0 then
          begin
            ClearAllProperties;
          end;
        end
        else
          ClearAllProperties;       

      except
        FErrorCode:= ecArchiveFailureInLoadArchive;
        FLoaded:= False;
        FbfaApplication^.WriteLog(llNormal, ltException, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
        Result:= FErrorCode;
      end;
    finally
      FInLoadProcess:= False;
    end;
  end
  else
  begin
    FErrorCode:= ecArchiveConnectionOrLogNotInitialized;
    FLoaded:= False;
  end;
  if FErrorCode <> 0 then
    FbfaApplication^.WriteLog(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
  Result:= FErrorCode;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
procedure TbfaClientArchive.Clone(Source: TbfaClientArchive);
begin
  try
    inherited Clone(Source);
    FHasFieldUserRights:= Source.HasFieldUserRights;

    FFieldsList:= Source.FieldsList;

  except

  end;
end;
//
function TbfaClientArchive.LoadFields: Integer;
var
  dsc: TbfaSP;
  Fld: TbfaClientField;
  Mpg : TbfaMapping;
  i, Idx: Integer;
  FbfaMember: TbfaSP;
  Rights, UserRight: UInt64;
  found: Boolean;  
begin
  FErrorCode:= 0;
  if FInitialized then
  begin
    if FLoaded then
    begin
      try
        try  
          for i := 0 to FFieldsList.Count - 1 do
            FFieldsList.Items[i].Free;
          FFieldsList.Clear;

          dsc:= FbfaApplication^.Connection.SelectFields(FID);
          if dsc <> nil then
          begin
            dsc.First;
            while not dsc.Eof do
            begin

            
              Fld := TbfaClientField.Create(FbfaApplication);
              Fld.ArchiveID := FID;
              Fld.ArchiveLayoutID := FLayoutID;
              Fld.MaxUserIdx := FMaxUserColumn;
              Fld.MaxGroupIdx := FMaxGroupColumn;

              Fld.ID:= dsc.FieldByNameAsLargeInt(sColAddFieldID);
              Fld.DisplayName:= dsc.FieldByNameAsWideString(sColDisplayname);
              Fld.FieldType:= dsc.FieldByNameAsInteger(sColFieldType);
              Fld.ValueType:= dsc.FieldByNameAsInteger(sColValueType);
              Fld.ControlType:= dsc.FieldByNameAsInteger(sColControlType);
              Fld.Mode:= dsc.FieldByNameAsInteger(sColMode);
              Fld.Options:= dsc.FieldByNameAsInteger(sColOptions);
              Fld.DefaultValue:= dsc.FieldByNameAsWideString(sColDefaultValue);
              Fld.IndexTableID:= dsc.FieldByNameAsInteger(sColIndexTableID);
              Fld.Description:= dsc.FieldByNameAsWideString(sColDescription);
              if dsc.FieldByNameAsInteger(sColEnabled) = 0 then
                Fld.Enabled:= False
              else if dsc.FieldByNameAsInteger(sColEnabled) = 1 then
                Fld.Enabled:= True;
              Fld.Size:= dsc.FieldByNameAsInteger(sColSize);
              Fld.ControlItems:= dsc.FieldByNameAsWideString(sColControlItems);
              Fld.Status:= dsc.FieldByNameAsInteger(sColState);
              Fld.UserRead := False;
              Fld.UserWrite := False;
              Fld.UserModify := False;
              Fld.HasRights := False;              
              
//              Fld.Load(dsc.FieldByNameAsLargeInt(sColAddFieldID));

                    if FMaxUserColumn <> -1 then
                    begin
                      i := 0;
                      while (i <= FMaxUserColumn) and (not FHasFieldUserRights) do
                      begin
                        if (not FHasFieldUserRights) then  
                        begin
                          if (dsc.FieldByNameAsLargeInt(sSysRead_ + IntToStr(i)) > 0 ) then
                            FHasFieldUserRights := True;
                        end;
                        if (not FHasFieldUserRights) then  
                        begin
                          if (dsc.FieldByNameAsLargeInt(sSysWrite_ + IntToStr(i)) > 0 ) then
                            FHasFieldUserRights := True;
                        end;
                        if (not FHasFieldUserRights) then  
                        begin
                          if (dsc.FieldByNameAsLargeInt(sSysModify_ + IntToStr(i)) > 0 ) then
                            FHasFieldUserRights := True;
                        end;
                        inc(i);
                      end;
                    end;

                    if FMaxGroupColumn <> -1 then
                    begin
                      i := 0;
                      while (i <= FMaxGroupColumn) and (not FHasFieldUserRights) do
                      begin
                        if (not FHasFieldUserRights) then  
                        begin
                          if (dsc.FieldByNameAsLargeInt(sSysGRead_ + IntToStr(i)) > 0 ) then
                            FHasFieldUserRights := True;
                        end;
                        if (not FHasFieldUserRights) then  
                        begin
                          if (dsc.FieldByNameAsLargeInt(sSysGWrite_ + IntToStr(i)) > 0 ) then
                            FHasFieldUserRights := True;
                        end;
                        if (not FHasFieldUserRights) then  
                        begin
                          if (dsc.FieldByNameAsLargeInt(sSysGModify_ + IntToStr(i)) > 0 ) then
                            FHasFieldUserRights := True;
                        end;
                        inc(i);
                      end;
                    end;
                          
                    if (FbfaApplication^.User.ID > 0) and (FHasFieldUserRights) then // angemeldete UserID
                    begin
                      idx:= (FbfaApplication^.User.ID div 64);
                      Rights:= dsc.FieldByNameAsLargeInt(sSysRead_ + IntToStr(idx));
                      UserRight:= bfaPower(FbfaApplication^.User.ID - 1);
                      if ((Rights and UserRight) = UserRight) then
                        Fld.UserRead:= True;
                      Rights:= dsc.FieldByNameAsLargeInt(sSysWrite_ + IntToStr(idx));
                      if ((Rights and UserRight) = UserRight) then
                        Fld.UserWrite:= True;
                      Rights:= dsc.FieldByNameAsLargeInt(sSysModify_ + IntToStr(idx));
                      if ((Rights and UserRight) = UserRight) then
                        Fld.UserModify:= True;


                      if ((not Fld.UserRead) and (not Fld.UserWrite) and (not Fld.UserModify)) then
                      begin
                        FbfaMember:= FbfaApplication^.Connection.SelectWhereObjMember(FbfaApplication^.User.ID, otUser);
                        try
                          if FbfaMember.RecordCount > 0 then
                          begin
                            FbfaMember.First;
                            while ((not FbfaMember.Eof) and ((not Fld.UserRead) and (not Fld.UserWrite) and (not Fld.UserModify)) )do
                            begin
                              idx:= (FbfaMember.FieldByNameAsLargeInt(sColGrpID) div 64);
                              Rights:= dsc.FieldByNameAsLargeInt(sSysGRead_ + IntToStr(idx));
                              UserRight:= bfaPower(FbfaMember.FieldByNameAsLargeInt(sColGrpID) - 1);
                              if ((Rights and UserRight) = UserRight) then
                                Fld.UserRead:= True;
                              Rights:= dsc.FieldByNameAsLargeInt(sSysGWrite_ + IntToStr(idx));
                              if ((Rights and UserRight) = UserRight) then
                                Fld.UserWrite:= True;
                              Rights:= dsc.FieldByNameAsLargeInt(sSysGModify_ + IntToStr(idx));
                              if ((Rights and UserRight) = UserRight) then
                                Fld.UserModify:= True;
              
            
                              FbfaMember.Next;
                            end;
                          end;          
                        finally
                          FbfaMember.Close;
                        end;
                      end;
                    end;


              FFieldsList.Add(TbfaMapping.Create(Fld, -1, -1, stActive, -1, '', False));

//              if Fld.HasRights then
//                FHasFieldUserRights := True;
        
              dsc.Next;
            end;
            dsc.Close;
          end;

          Result:= FErrorCode;
        except
          FErrorCode:= ecArchiveFailureInLoadAdditionalFields;
          Result:= FErrorCode;
        end;
      finally
        FreeAndNil(dsc);
      end;
    end
    else
    begin
      FErrorCode:= ecArchiveFailureInLoadAdditionalFields;
      Result:= FErrorCode;
    end;
  end
  else
  begin
    FErrorCode:= ecArchiveConnectionOrLogNotInitialized;
    Result:= FErrorCode;
  end;    
end;  
//
procedure TbfaClientArchive.Refresh;
begin
  try
    FErrorCode:= Load(FLayoutID);
  except

  end;
end;
//


end.
