unit bfaConfigConnection;

interface

uses bfaDBConnection, ActiveX, Windows, bfaConnection,
  bfaLog, DB, bfaDataAccessDM, bfaMySQLDataAccessDM, bfaSP;

type
  TbfaConfigConnection = class(TbfaDBConnection)
  public
    constructor Create(Log: TbfaLog; ConFileName: WideString);
    /// <Beschreibung>Liest die Standard Einstellungen aus der INI Datei</Beschreibung>
    /// <Description>Reads the default settings from the INI file</Description>
    function ReadSettingsFromINI(ConFileName: string): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SelectProfiles: TbfaSP;  
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SetApplicationDBConnection(ClientID: Int64; Conn: TbfaConnection): Integer;

    
    
  
  end;


implementation

uses bfaMySQLDatabaseScripts, SysUtils, IniFiles, bfaCommonUtils, bfaMethods, 
  bfaCommonMethods;

//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
constructor TbfaConfigConnection.Create(Log: TbfaLog; ConFileName: WideString);
begin
  inherited Create;
  ReadSettingsFromINI(ConFileName);
end;
//
//  Name: ReadSettingsFromRegistry
//  Parameters: - 
//  Beschreibung: Liest die Standard Einstellungen aus der Registry
//  Description: Reads the default settings from the Registry
//  Return value: FErrorCode
// [] 17.04.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConfigConnection.ReadSettingsFromINI(ConFileName: string): Integer;
var
  ini: TIniFile;
begin
  FErrorCode:=0;
  ini:= TIniFile.Create(ConFileName);
  try
    if not FileExists(ini.FileName) then
      FErrorCode:= ecApplicationINIFileNotFound;
    if FErrorCode = 0 then
    begin                                                       
      FUserName:= ini.ReadString(sSysConnectionSection,sSysUserName,'');
      FPassword:= ini.ReadString(sSysConnectionSection,sSysPassword,'');
      FProvider:= ini.ReadInteger(sSysConnectionSection,sSysProvider,0);
      FServer:= ini.ReadString(sSysConnectionSection,sSysServer,'');
      FPort:= ini.ReadInteger(sSysConnectionSection,sSysPort,3306);
      FDatabase:= ini.ReadString(sSysConnectionSection,sSysDatabase,'');
      FClientVersion:= ini.ReadString(sSysConnectionSection,sSysClientVersion,'');
    end;
  finally
    Ini.Free;
    Result:= FErrorCode;
  end;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConfigConnection.SelectProfiles: TbfaSP;
begin
  FErrorCode:= 0;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'SelectProfiles', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.SelectProfiles(FbfaDataAccess.cntBfaServer, FLog);        
      end;      

    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
      Result:= nil;
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
    Result:= nil;
  end;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConfigConnection.SetApplicationDBConnection(ClientID: Int64; Conn: TbfaConnection): Integer;
begin
  FErrorCode:= 0;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'SetApplicationDBConnection', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.SelectProfileCfgByID(FbfaDataAccess.cntBfaServer, ClientID, Conn, FLog);        
      end;      

    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
      Result:= FErrorCode;
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
    Result:= FErrorCode;
  end;
end;
//
end.


