unit bfaConnection;

interface

uses 
  Generics.Collections, Classes, DB, 
  bfaDBConnection, SysUtils, bfaLog, bfaBaseObjects, bfaSP, Uni,
  bfaDataAccessDM, bfaMySQLDataAccessDM, bfaUser;

type
  TbfaConnection = class(TbfaDBConnection)
  private 
    function GetMySQLConnection : TUniConnection;    
    function GetMySQLTransaction : TUniTransaction;    
  public
    {$if defined(VERSION40)}
      property MySQLDataAccess : TbfaMySQLDataAccess read FbfaMySQLDataAccess;
    {$elseif defined(VERSION36)}
      property bfaDataAccess : TbfaDataAccess read FbfaDataAccess;
    {$ifend}
    property MySQLConnection : TUniConnection read GetMySQLConnection;
    property MySQLTransaction : TUniTransaction read GetMySQLTransaction;
    property Log : TbfaLog read FLog;
 
    /// <Beschreibung>Setzt die Standardwerte und erstellt die Objekte</Beschreibung>
    /// <Description>Sets the default values and creates the objects</Description>
    constructor Create(Log: TbfaLog);   
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SetConnection(UserName, Password, Database, Server: String; 
                           Port, Provider: Integer): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function UserLogin(UserObj : PbfaUserGroupBase): Integer;                             
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function LoadUser(UserObj: PbfaUserGroupBase): Integer; 
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function LoadGroup(GroupObj : PbfaUserGroupBase): Integer;
    /// <Beschreibung>Luft den sp_select_archivebyarcid Stored Procedure</Beschreibung>
    /// <Description>Calls the sp_select_archivebyarcid stored procedure</Description>
    function LoadArchive(LayoutID: Int64; var ArcRec: TArchiveRec): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SelectDMSRoot: Int64;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function CheckIfDMSAdmin(ParentID, ObjID: Int64; ObjType: Integer): Boolean;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SelectRoles(Role: WideString): TbfaSP;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function CheckIfObjectExsitsOnLevel(ParentID, ObjID: Int64; ObjType: Integer; TblName: string): Boolean;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SelectLayoutObjectDetailsByID(ID: Int64; var LayoutRec: TLayoutRec): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SelectLayoutIDByObjectIDs(ObjID, ParentID : Int64; OType : Integer) : Int64;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function GetUserArchiveApplicationRights(UserID, ParentID : Int64) : TbfaSP;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function GetLayoutRootObjects(Typ : Integer) : TbfaSP;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SelectUserContainers: TbfaSP;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function CheckIfExistsAndHasRight(ParentID, ObjID: Int64; ObjType, Rights: Integer): Boolean;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function GetLayoutArchivesWithIdxParams(LeftIdx, RightIdx: Int64; Level, ObjType: Integer): TbfaSP;    
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function UpdateUserPwd(UserID, MandantID : Int64; UserName, Password : WideString): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function AddObjectToLayout(ParentID, ObjID: Int64; ObjType: Integer; ObjName: WideString; InheritPermission, InheritAdditionalField: Boolean): Int64;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function CreateUser(FirstName, MiddleName, LastName, UserName, Password, Email, Description: WideString; Status: Integer; MandantID, LayoutID : Int64; SaveImmediately: Boolean; var UserLayoutID, ContainerID : Int64): Int64;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function CheckIfUserAdminInOtherContainer(ObjID, ParentID: Int64; ObjType: Integer): Boolean;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SelectObjectsMandant(ObjID: Int64; ObjType : Integer; var MandantName: WideString): Int64;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SelectArchiveNameByID(ArcID: Int64) : WideString;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function CheckUser(UserID : Int64; UserName, Password : WideString; MandantID : Int64): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function CheckGroup(GroupName : WideString; GroupID, MandantID : Int64): Integer;
    
    


    

    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function GetUserApplicationRights(UserID: Int64): TbfaSP;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function GetArchivesByUserID(UserID: Int64; UserType: Integer; LeftIdx, RightIdx: Int64; UserRight: Integer) : TbfaSP;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function GetArchivesByUserIDReturnWithList(UserID: Int64; UserType: Integer; LeftIdx, RightIdx: Int64; UserRight: Integer; GrpList : Boolean = False; ArcIDs : WideString = '') : WideString;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function CreateArchive(ArcName, BgrPicture, DefDocName, Description: WideString; ParentID, UserID: Int64; ArchiveType, Icon: Integer; DMSAdmin, InheritPermission, InheritAdditionalField: Boolean; Kind: TbfaTreeViewKind; var ArcID, LayoutID : Int64): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SelectObjectsUnderObject(ID: Int64): TbfaSP;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SelectUsers: TDataSet;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SelectGroups: TDataSet;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SelectAllFields: TbfaSP;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SelectAddUsers: TDataSet;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SelectAddGroups: TDataSet;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function CreateGroup(GroupName, Description, Email : WideString; MandantID, LayoutID : Int64; SaveImmediately : Boolean; var GroupLayoutID, ContainerID : Int64) : Int64;   
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SelectGroupMembers(GroupID, MandantID: Int64): TbfaSP;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function InsertObjectToSysGrpUsr(ParentID, ObjID, MandantID: Int64; ObjType: Integer): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SelectGroupSysGrpUsrID(GroupID: Int64): Int64;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SelectObjectRights(ParentID, ObjID: Int64; ObjType: Integer): Int64;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function UpdateRightsToObject(ArchiveID, ObjID, Rights: Int64; ObjType: Integer): Int64;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
//    function SelectArchivenamesByIDs(IDs: string): WideString;     
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
//    function SelectObjectsWithMemberIDs(IDs, TableName: string): TDataSet;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function UpdateGroup(GroupID, MandantID : Int64; GroupName, Description, Email : WideString; OnlyGroupName : Boolean) : Int64;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function UpdateUser(UserID, MandantID: Int64; UserName, Email, Description, FirstName, LastName, MiddleName, Password: WideString; Status: Integer; OnlyUserName: Boolean = False): Int64;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SelectWhereObjMember(ObjID: Int64; ObjType: Integer): TbfaSP;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function DeleteArchive(LayoutID: Int64): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function DeleteContainer(LayoutID: Int64): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function DeleteObject(LayoutID: Int64): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function GetArchiveStoreName(ArchiveLayoutID: Int64): WideString;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function UpdateArchive(LayoutID, ArcID: Int64; ArcName, BgrPicture, DocName, Description: WideString; InheritPermission, InheritAdditionalField, OnlyArcName: Boolean; Icon: Integer; Kind: TbfaTreeViewKind): Int64;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function UpdateArchiveGUILayout(LayoutID, ArcID: Int64; ArchiveLayout: TMemoryStream):Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
//    function DeleteUserOrGroupFromSysLayout(ParentID, ObjID: Int64; ObjType: Integer): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
//    function DeleteUserOrGroup(ObjID: Int64; ObjType: Integer): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SelectLayoutWhereObjMember(ObjID: Int64; ObjType: Integer): TbfaSP;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
 //   function SelectAdditionalAndStatusFields: TDataSet;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function CreateBaseField(DisplayName, Description, DefaultValue, ControlItems: WideString; 
        FieldType, ControlType, ValueType, Mode, FieldOptions, IndexTableID, Size: Integer; ControlEnabled: Boolean; 
        LayoutID : Int64; SaveImmediately: Boolean; var ContainerID: Int64): Int64;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function GetValueTypeString(ValueType: Integer; Size: Integer = 255): String;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SelectBaseFieldByID(FieldID: Int64; var FldRec: TBaseFldRec): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function UpdateBaseField(Field: TBaseFldRec; OnlyName : Boolean): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function UpdateField(Field: TFldRec): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function UpdateFieldInheritedMapping(LayoutID: Int64; Field: TFldRec): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function DeleteUserOrGroupFromGroup(ObjID, ParentID: Int64; ObjType: Integer): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function AddObjectToFieldMapping(LayoutID, AddFieldID, ArchiveID: Int64; ObjType: Integer; DefaultValue, Description, ControlItems: WideString; Mode, PrmOptions: Integer): Int64;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SelectFields(ArchiveID: Int64): TbfaSp;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function DeleteObjectFromFieldMapping(LayoutID, AddFieldID: Int64): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function GetFieldValue(Typ: Integer; ID: Int64; ColType: Integer = 0): WideString;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SelectFieldByArchiveID(FieldID, ArchiveID, UserID: Int64; UserColumn, GroupColumn: Integer; var FldRec: TFldRec): TbfaSP;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SelectAdminFieldByArchiveID(FieldID, ArchiveID, UserID: Int64; UserColumn, GroupColumn: Integer; var FldRec: TFldRec; List : TList<TbfaPermission>): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function LoadDocument(GDocID: Int64; var DocRec: TDocumentRec): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function GetObjectPermission(ID: Int64): TbfaSP;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function GetContainerObjects(ArchiveLayoutID: Int64; ContainerType, ListType, SQLType: Integer; LayoutIDs: string): TbfaSP;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function AddObjectToUserMapping(ArcID, ObjID, Rights, RoleID: Int64; ObjType: Integer): Int64;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function UpdateObjectInUserMapping(ArcID, ObjID, Rights: Int64; ObjType: Integer): Int64;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SelectAvailableUsers(ArchiveLayoutID: Int64; FullList: Integer): TDataSet;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SelectAvailableGroups(ArchiveLayoutID: Int64; FullList: Integer): TDataSet;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SelectArchiveAncestorObjects(LayoutID: Int64): TbfaSP;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function DeleteObjectFromUserMapping(LayoutID, ObjID: Int64; ObjType: Integer): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function CheckIfGrpUsrObjectExsitsOnLevel(ParentID, ObjID, MandantID: Int64; ObjType: Integer): Boolean;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function UpdateLayoutRights(LayoutID, Rights, RoleID: Int64): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function UpdateContainerDescription(LayoutID: Int64; Description: WideString): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SelectContainerDescription(ID: Int64): WideString;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SelectFieldPermissions(ArcID, AddFieldID: Int64; var UsrIdx, GrpIdx: Integer): TBfaSP;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function UpdateInheritedFieldRights(LayoutID, ArcID, AddfieldID: Int64; Idx, ObjType: Integer; Read, Write, Modify, UnsetRights: UInt64): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SelectMaxUsrGrpIdx(var UsrIdx, GrpIdx: Integer): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function CopyNode(ParentID: Int64; ObjType: Integer; NewParentID, NewObjID: Int64): Int64;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function SelectDMSRootArchiveID: Int64;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function CreateDocument(DocLoc, OriLoc, ExtLoc, SrcLoc, DocName: WideString; 
                            UserID: Int64; CheckOut, RevSave: Boolean; ArcID: Int64; PageCount, DocType: Integer;
                            FileType: string; DocTxt, DocTags: WideString): Int64;
    


  end;

  PbfaConnection = ^TbfaConnection;

implementation

uses 
  bfaMySQLDatabaseScripts, bfaCommonUtils;

//
constructor TbfaConnection.Create(Log: TbfaLog);
begin
  inherited Create;   
  Init(Log);  
end;
//
function TbfaConnection.GetMySQLConnection : TUniConnection;
begin
  Result := FbfaDataAccess.cntBfaServer;
end;
//
function TbfaConnection.GetMySQLTransaction : TUniTransaction;
begin
  FbfaMySQLDataAccess.trnBfaServer.DefaultConnection := GetMySQLConnection;
  Result := FbfaMySQLDataAccess.trnBfaServer;
end;
//
function TbfaConnection.SetConnection(UserName, Password, Database, Server : String; 
                                      Port, Provider : Integer) : Integer;
begin
  FErrorCode := 0;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltDebug, 'SetConnection', ClassName);
    FUserName := UserName;
    FPassword := Password;
    FProvider := Provider;
    FServer := Server;
    FPort := Port;
    FDatabase := Database;
  end
  else
  begin
    FErrorCode := ecConnectionLogIsNotInitialized;
  end;
  Result := FErrorCode;
end;
//
function TbfaConnection.LoadUser(UserObj : PbfaUserGroupBase): Integer;
begin
  FErrorCode := 0;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltDebug, 'LoadUser(' + IntToStr(UserObj^.ID) + ')', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        FErrorCode := MySQL.LoadUser(@Self, UserObj);
      end;
      
    end
    else
    begin
      FErrorCode := ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode := ecConnectionLogIsNotInitialized;
  end;
  Result := FErrorCode;
end;
//
function TbfaConnection.UserLogin(UserObj : PbfaUserGroupBase): Integer;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltDebug, 'Login(' + UserObj^.Name + ')', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        FErrorCode := MySQL.Login(@Self, UserObj);
        if FErrorCode = ecConnectionUserLocked then
          FLog.Write(llNormal, ltInfo, sMsgLoginErrorUserLocked + sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName)
        else if (FErrorCode <> 0) or (UserObj^.ID <= 0) then
        begin
          FErrorCode := ecConnectionLoginFailure;
          FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode) + '; spErrorCode: ' + IntToStr(UserObj^.ID), ClassName);
        end;
        Result := FErrorCode;
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;  
end;
//
function TbfaConnection.SelectDMSRoot: Int64;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltDebug, 'SelectDMSRoot', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.GetDMSRoot(@Self, sSysDmsRoot);
      end;   
               
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
function TbfaConnection.CheckIfDMSAdmin(ParentID, ObjID: Int64; ObjType: Integer): Boolean;
begin
  FErrorCode:= 0;
  Result:= False;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'CheckIfDMSAdmin', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.CheckIfDMSAdmin(@Self, ParentID, ObjID, ObjType);
      end;   
               
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
function TbfaConnection.SelectRoles(Role: WideString): TbfaSP;
begin
  FErrorCode:= 0;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'SelectRoles', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.SelectRoles(@Self, Role);        
      end;      

    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
      Result:= nil;
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
    Result:= nil;
  end;
end;
//
function TbfaConnection.CheckIfObjectExsitsOnLevel(ParentID, ObjID: Int64; ObjType: Integer; TblName: string): Boolean;
begin
  FErrorCode:= 0;
  Result:= False;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltDebug, 'CheckIfObjectExsitsOnLevel', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.CheckIfObjectExsitsOnLevel(@Self, ParentID, ObjID, ObjType, TblName);
      end;   
               
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;     
//
function TbfaConnection.SelectLayoutObjectDetailsByID(ID: Int64; var LayoutRec: TLayoutRec): Integer;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'SelectLayoutObjectDetailsByID', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.GetObjectDetailsByID(@Self, ID, LayoutRec, sTblSysLayout);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
function TbfaConnection.SelectLayoutIDByObjectIDs(ObjID, ParentID: Int64; OType: Integer): Int64;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'SelectLayoutIDByObjectIDs', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.SelectLayoutIDByObjectIDs(@Self, ObjID, ParentID, OType);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
function TbfaConnection.GetUserArchiveApplicationRights(UserID, ParentID: Int64): TbfaSP;
begin
  FErrorCode:= 0;
  Result:= nil;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltDebug, 'GetUserArchiveApplicationRights', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.GetUserArchiveApplicationRights(@Self, UserID, ParentID);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
function TbfaConnection.GetLayoutRootObjects(Typ: Integer): TbfaSP;
begin
  FErrorCode:= 0;
  Result:= nil;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltDebug, 'GetLayoutRootObjects', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.GetLayoutRootObjects(@Self, Typ);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
function TbfaConnection.SelectUserContainers: TbfaSP;
begin
  FErrorCode:= 0;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'SelectUserContainers', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.SelectUserContainers(@Self);        
      end;      

    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
      Result:= nil;
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
    Result:= nil;
  end;
end;
//
function TbfaConnection.CheckIfExistsAndHasRight(ParentID, ObjID: Int64; ObjType, Rights: Integer): Boolean;
begin
  FErrorCode:= 0;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'CheckIfExistsAndHasRight', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.CheckIfExistsAndHasRight(@Self, ParentID, ObjID, ObjType, Rights);        
      end;      

    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
      Result:= False;
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
    Result:= False;
  end;
end;
//
function TbfaConnection.GetLayoutArchivesWithIdxParams(LeftIdx, RightIdx: Int64; Level, ObjType: Integer): TbfaSp;
begin                                 
  FErrorCode:= 0;
  Result:= nil;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltDebug, 'GetLayoutWithIdxParams', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.GetLayoutArchivesWithIdxParams(@Self, LeftIdx, RightIdx, Level, ObjType);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
function TbfaConnection.UpdateUserPwd(UserID, MandantID : Int64; UserName, Password : WideString): Integer;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'UpdateUserPwd', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.UpdateUserPwd(@Self, UserID, MandantID, UserName, Password);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
function TbfaConnection.AddObjectToLayout(ParentID, ObjID: Int64; ObjType: Integer; ObjName: WideString; InheritPermission, InheritAdditionalField: Boolean): Int64;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltDebug, 'AddObjectToLayout', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.AddObjectToLayout(@Self, ParentID, ObjID, ObjType, ObjName, InheritPermission, InheritAdditionalField);
      end;   

      if Result = -1 then
      begin
        FErrorCode:= ecConnectionAddObjectToLayoutFailure;
      end
      else if Result = -2 then
      begin      
        FErrorCode:= ecConnectionObjectAlreadyExsits;
      end;
               
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
function TbfaConnection.CreateUser(FirstName, MiddleName, LastName, UserName, Password, Email, Description: WideString; Status: Integer; MandantID, LayoutID : Int64; SaveImmediately: Boolean; var UserLayoutID, ContainerID : Int64): Int64;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltDebug, 'CreateUser', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.CreateUser(@Self, FirstName, MiddleName, LastName, UserName, Password, Email, Description, Status, MandantID, LayoutID, SaveImmediately, UserLayoutID, ContainerID);
      end;   

      if Result = -1 then
      begin
        FErrorCode:= ecConnectionCreateUserFailure;
      end
      else if Result = -2 then
      begin      
        FErrorCode:= ecConnectionUserNameAlreadyExsits;
      end;
               
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
function TbfaConnection.CheckIfUserAdminInOtherContainer(ObjID, ParentID: Int64; ObjType: Integer): Boolean;
begin
  FErrorCode:= 0;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'CheckIfUserAdminInOtherContainer', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.CheckIfUserAdminInOtherContainer(@Self, ObjID, ParentID, ObjType);        
      end;      

    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
      Result:= False;
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
    Result:= False;
  end;
end;
//
function TbfaConnection.SelectObjectsMandant(ObjID: Int64; ObjType : Integer; var MandantName: WideString): Int64;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'SelectObjectsMandant', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.SelectObjectsMandant(@Self, ObjID, ObjType, MandantName);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
function TbfaConnection.SelectArchiveNameByID(ArcID: Int64) : WideString;
begin
  FErrorCode:= 0;
  Result := '';
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'SelectArchiveNameByID', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.SelectArchiveNameByID(@Self, ArcID);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
function TbfaConnection.CheckUser(UserID : Int64; UserName, Password : WideString; MandantID : Int64): Integer;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'CheckUser', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.CheckUser(@Self, UserID, UserName, Password, MandantID);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
function TbfaConnection.CheckGroup(GroupName : WideString; GroupID, MandantID : Int64): Integer;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'CheckGroup', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.CheckGroup(@Self, GroupName, GroupID, MandantID);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;



//
//  Name: LoadArchive
//  Parameters: LayoutID
//              ArcRec
//  Beschreibung: Luft den sp_select_archivebyarcid Stored Procedure 
//  Description: Calls the sp_select_archivebyarcid stored procedure
//  Return value: FErrorCode
// [] 06.01.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.LoadArchive(LayoutID: Int64; var ArcRec: TArchiveRec): Integer;
begin
  FErrorCode:= 0;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltDebug, 'LoadArchive(' + IntToStr(LayoutID) + ')', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        FErrorCode:= MySQL.LoadArchive(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, LayoutID, ArcRec, FLog);
      end;      
      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
  Result:= FErrorCode;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 17.04.2013 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.LoadDocument(GDocID: Int64; var DocRec: TDocumentRec): Integer;
begin
  FErrorCode:= 0;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltDebug, 'LoadDocument(' + IntToStr(GDocID) + ')', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        FErrorCode:= MySQL.LoadDocument(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, GDocID, DocRec, FLog);
      end;      
      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
  Result:= FErrorCode;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.GetUserApplicationRights(UserID: Int64): TbfaSP;
begin
  FErrorCode:= 0;
  Result:= nil;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltDebug, 'GetUserApplicationRight', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.GetUserApplicationRights(FbfaDataAccess.cntBfaServer, FLog, UserID);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;


//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.GetArchivesByUserID(UserID: Int64; UserType: Integer; LeftIdx, RightIdx: Int64; 
  UserRight: Integer) : TbfaSP;
begin
  FErrorCode:= 0;
  Result:= nil;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltDebug, 'GetArchivesByUserID', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.GetArchivesByUserID(FbfaDataAccess.cntBfaServer, FLog, UserID, LeftIdx, RightIdx, 
                UserRight);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;       
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.GetArchivesByUserIDReturnWithList(UserID: Int64; UserType: Integer; LeftIdx, RightIdx: Int64; 
  UserRight: Integer; GrpList : Boolean; ArcIDs : WideString) : WideString;
begin
  FErrorCode:= 0;
  Result := '';
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltDebug, 'GetArchivesByUserIDReturnWithList', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.GetArchivesByUserIDReturnWithList(FbfaDataAccess.cntBfaServer, FLog, UserID, LeftIdx, RightIdx, 
                UserRight, GrpList, ArcIDs);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.CreateArchive(ArcName, BgrPicture, DefDocName, Description: WideString; ParentID, UserID: Int64; ArchiveType, Icon: Integer; DMSAdmin, InheritPermission, InheritAdditionalField: Boolean; Kind: TbfaTreeViewKind; var ArcID, LayoutID : Int64): Integer;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltDebug, 'CreateArchive', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.CreateArchive(@Self, ArcName, BgrPicture, DefDocName, Description, ParentID, UserID, ArchiveType, Icon, DMSAdmin, InheritPermission, InheritAdditionalField, Kind, ArcID, LayoutID);
      end;   

      if Result = -1 then
      begin
        FErrorCode:= ecConnectionCreateArchiveFailure;
      end
      else if Result = -2 then
      begin      
        FErrorCode:= ecConnectionArchiveNameAlreadyExsits;
      end;
               
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.CreateGroup(GroupName, Description, Email : WideString; MandantID, LayoutID: Int64; SaveImmediately: Boolean; var GroupLayoutID, ContainerID : Int64): Int64;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltDebug, 'CreateGroup', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.CreateGroup(@Self, GroupName, Description, Email, MandantID, LayoutID, SaveImmediately, GroupLayoutID, ContainerID);
      end;   

      if Result = -1 then
      begin
        FErrorCode:= ecConnectionCreateGroupFailure;
      end
      else if Result = -2 then
      begin      
        FErrorCode:= ecConnectionGroupNameAlreadyExsits;
      end
      else if Result = -3 then
      begin      
        FErrorCode:= ecConnectionInsertGroupToSysGrpUsrFailure;
      end;               
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.SelectGroupMembers(GroupID, MandantID: Int64): TbfaSP;
begin
  FErrorCode:= 0;
  Result:= nil;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltDebug, 'SelectGroupMembers', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.SelectGroupMembers(FbfaDataAccess.cntBfaServer, FLog, GroupID, MandantID);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.SelectUsers: TDataSet;
begin
  FErrorCode:= 0;
  Result:= nil;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltDebug, 'SelectUsers', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.SelectUsers(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.SelectGroups: TDataSet;
begin
  FErrorCode:= 0;
  Result:= nil;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltDebug, 'SelectGroups', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.SelectGroups(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
function TbfaConnection.SelectAllFields: TbfaSP;
begin
  FErrorCode:= 0;
  Result:= nil;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltDebug, 'SelectAllFields', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.SelectAllFields(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.SelectAddUsers: TDataSet;
begin
  FErrorCode:= 0;
  Result:= nil;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltDebug, 'SelectAddUsers', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.SelectAddUsers(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.SelectAddGroups: TDataSet;
begin
  FErrorCode:= 0;
  Result:= nil;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltDebug, 'SelectAddGroups', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.SelectAddGroups(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.CheckIfGrpUsrObjectExsitsOnLevel(ParentID, ObjID, MandantID: Int64; ObjType: Integer): Boolean;
begin
  FErrorCode:= 0;
  Result:= False;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltDebug, 'CheckIfGrpUsrObjectExsitsOnLevel', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.CheckIfGrpUsrObjectExsitsOnLevel(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog, ParentID, ObjID, MandantID, ObjType);
      end;   
               
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;

//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.SelectObjectsUnderObject(ID: Int64): TbfaSP;
begin
  FErrorCode:= 0;
  Result:= nil;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'SelectObjectsUnderObject', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.GetLayoutUsersAndGroups(FbfaDataAccess.cntBfaServer, FLog, ID);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.GetObjectPermission(ID: Int64): TbfaSP;
begin
  FErrorCode:= 0;
  Result:= nil;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'GetObjectPermission', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.GetLayoutUsersAndGroups(FbfaDataAccess.cntBfaServer, FLog, ID);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.InsertObjectToSysGrpUsr(ParentID, ObjID, MandantID: Int64; ObjType: Integer): Integer;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'InsertObjectToSysGrpUsr', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.InsertObjectToSysGrpUsr(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog, ParentID, ObjID, MandantID, ObjType);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.SelectGroupSysGrpUsrID(GroupID: Int64): Int64;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'SelectGroupSysGrpUsrID', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.SelectGroupSysGrpUsrID(@Self, GroupID);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.SelectObjectRights(ParentID, ObjID: Int64; ObjType: Integer): Int64;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'SelectObjectRights', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.SelectObjectRights(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog, ParentID, ObjID, ObjType);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.UpdateRightsToObject(ArchiveID, ObjID, Rights: Int64; ObjType: Integer): Int64;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'UpdateRightsToObject', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.UpdateRightsToObject(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog, ArchiveID, ObjID, Rights, ObjType);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
(*
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.SelectArchivenamesByIDs(IDs: string): WideString;
begin
  FErrorCode:= 0;
  Result:= '';
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'SelectArchivenamesByIDs', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.SelectArchivenamesByIDs(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog, IDs);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;

//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.SelectObjectsWithMemberIDs(IDs, TableName: string): TDataSet;
begin
  FErrorCode:= 0;
  Result:= nil;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'SelectObjectsWithMemberIDs', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.SelectObjectsWithMemberIDs(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog, IDS, TableName);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
*)
//
function TbfaConnection.UpdateGroup(GroupID, MandantID : Int64; GroupName, Description, Email : WideString; OnlyGroupName : Boolean) : Int64;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'UpdateGroup', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.UpdateGroup(@Self, GroupID, MandantID, GroupName, Description, Email, OnlyGroupName);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
function TbfaConnection.UpdateUser(UserID, MandantID : Int64; UserName, Email, Description, FirstName, LastName, MiddleName, Password : WideString; Status: Integer; OnlyUserName: Boolean = False): Int64;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'UpdateUser', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.UpdateUser(@Self, UserID, MandantID, UserName, Email, Description, FirstName, LastName, MiddleName, Password, Status, OnlyUserName);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;

//
function TbfaConnection.SelectWhereObjMember(ObjID: Int64; ObjType: Integer): TbfaSP;
begin
  FErrorCode:= 0;
  Result:= nil;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'SelectObjMember', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.SelectWhereObjMember(FbfaDataAccess.cntBfaServer, FLog, ObjID, ObjType);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
function TbfaConnection.DeleteArchive(LayoutID: Int64): Integer;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'DeleteArchive', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.DeleteArchive(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog, LayoutID);
        if Result = -1 then
          FErrorCode:= ecConnectionRootCannotBeDeleted
        else if Result = -2 then
          FErrorCode:= ecConnectionObjectHasChildNodes
        else if Result = -3 then
          FErrorCode:= ecConnectionDocumentsBelongsToArchive;
        
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
function TbfaConnection.DeleteContainer(LayoutID: Int64): Integer;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'DeleteContainer', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.DeleteContainer(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog, LayoutID);
        
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
function TbfaConnection.DeleteObject(LayoutID: Int64): Integer;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'DeleteObject', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.DeleteObject(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog, LayoutID);
        
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
function TbfaConnection.GetArchiveStoreName(ArchiveLayoutID: Int64): WideString;
var
  LayoutRec: TLayoutRec;
begin
  FErrorCode:= 0;
  Result:= '';
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'GetArchiveStoreName', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        FErrorCode:= MySQL.GetObjectDetailsByID(@Self, ArchiveLayoutID, LayoutRec, sTblSysLayout); // Archive Parameters
        FErrorCode:= MySQL.GetObjectDetailsByID(@Self, LayoutRec.ParentID, LayoutRec, sTblSysLayout); // Parent Parameters

        Result:= GetFieldValue(otArchive, LayoutRec.ObjID); // Select ArchiveName from sys_arc
        
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.UpdateArchive(LayoutID, ArcID: Int64; ArcName, BgrPicture, DocName, Description: WideString; InheritPermission, InheritAdditionalField, OnlyArcName: Boolean; Icon: Integer; Kind: TbfaTreeViewKind): Int64;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'UpdateArchive', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        FErrorCode:= MySQL.UpdateArchive(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog, LayoutID, ArcID, ArcName, BgrPicture, DocName, Description, InheritPermission, InheritAdditionalField, OnlyArcName, Icon, Kind);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
  Result:= FErrorCode;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.UpdateArchiveGUILayout(LayoutID, ArcID: Int64; ArchiveLayout: TMemoryStream):Integer;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'UpdateArchiveGUILayout', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        FErrorCode:= MySQL.UpdateArchiveGUILayout(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog, ArcID, ArchiveLayout);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
  Result:= FErrorCode;
end;
(*
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.DeleteUserOrGroupFromSysLayout(ParentID, ObjID: Int64; ObjType: Integer): Integer;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'DeleteUserOrGroupFromSysLayout', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.DeleteUserOrGroupFromSysLayout(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog, ParentID, ObjID, ObjType);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.DeleteUserOrGroup(ObjID: Int64; ObjType: Integer): Integer;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'DeleteUserOrGroup', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.DeleteUserOrGroup(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog, ObjID, ObjType);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
*)
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.SelectLayoutWhereObjMember(ObjID: Int64; ObjType: Integer): TbfaSP;
begin
  FErrorCode:= 0;
  Result:= nil;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'SelectLayoutObjMember', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.SelectLayoutWhereObjMember(FbfaDataAccess.cntBfaServer, FLog, ObjID, ObjType);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
function TbfaConnection.LoadGroup(GroupObj : PbfaUserGroupBase): Integer;
begin
  FErrorCode:= 0;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'LoadGroup(' + IntToStr(GroupObj^.ID) + ')', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        FErrorCode:= MySQL.LoadGroup(@Self, GroupObj);
      end;      
      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
  Result:= FErrorCode;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
(*
function TbfaConnection.SelectAdditionalAndStatusFields: TDataSet;
begin
  FErrorCode:= 0;
  Result:= nil;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'SelectAdditionalAndStatusFields', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.SelectAdditionalAndStatusFields(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;*)
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.CreateBaseField(DisplayName, Description, DefaultValue, ControlItems: WideString; 
    FieldType, ControlType, ValueType, Mode, FieldOptions, IndexTableID, Size: Integer; ControlEnabled: Boolean; 
    LayoutID : Int64; SaveImmediately: Boolean; var ContainerID: Int64): Int64;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'CreateField', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.CreateBaseField(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog, DisplayName, Description, DefaultValue, ControlItems, 
            FieldType, ControlType, ValueType, Mode, FieldOptions, IndexTableID, Size, ControlEnabled, LayoutID, SaveImmediately, ContainerID);
      end;   

      if Result = -1 then
      begin
        FErrorCode:= ecConnectionCreateAdditionalFieldFailure;
      end
      else if Result = -2 then
      begin      
        FErrorCode:= ecConnectionAdditionalFieldAlreadyExsits;
      end;
               
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.GetValueTypeString(ValueType: Integer; Size: Integer = 255): String;
begin
  FErrorCode:= 0;
  Result:= '';
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'GetValueTypeString', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.ValueTypeToString(ValueType, Size);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;  
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.SelectBaseFieldByID(FieldID: Int64; var FldRec: TBaseFldRec): Integer;
begin
  FErrorCode:= 0;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'SelectFieldByID(' + IntToStr(FieldID) + ')', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        FErrorCode:= MySQL.SelectBaseFieldByID(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog, FieldID, FldRec);
      end;      
      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
  Result:= FErrorCode;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.UpdateBaseField(Field: TBaseFldRec; OnlyName : Boolean): Integer;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'UpdateBaseField', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        FErrorCode:= MySQL.UpdateBaseField(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog, Field, OnlyName);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
  Result:= FErrorCode;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.UpdateField(Field: TFldRec): Integer;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'UpdateField', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        FErrorCode:= MySQL.UpdateField(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog, Field);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
  Result:= FErrorCode;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.UpdateFieldInheritedMapping(LayoutID: Int64; Field: TFldRec): Integer;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'UpdateFieldInheritedMapping', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        FErrorCode:= MySQL.UpdateFieldInheritedMapping(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog, LayoutID, Field);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
  Result:= FErrorCode;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.DeleteUserOrGroupFromGroup(ObjID, ParentID: Int64; ObjType: Integer): Integer;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'DeleteUserOrGroupFromGroup', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.DeleteUserOrGroupFromGroup(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog, ObjID, ParentID, ObjType);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.AddObjectToFieldMapping(LayoutID, AddFieldID, ArchiveID: Int64; ObjType: Integer; DefaultValue, Description, ControlItems: WideString; Mode, PrmOptions: Integer): Int64;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'InsertFieldToMapping', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.AddObjectToFieldMapping(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog, LayoutID, AddFieldID, ArchiveID, ObjType, DefaultValue, Description, ControlItems, Mode, PrmOptions);
      end;   
               
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.DeleteObjectFromFieldMapping(LayoutID, AddFieldID: Int64): Integer;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'DeleteFieldFromMapping', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.DeleteObjectFromFieldMapping(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog, LayoutID, AddFieldID);
      end;   
               
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.SelectFields(ArchiveID: Int64): TbfaSP;
begin
  FErrorCode:= 0;
  Result:= nil;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'SelectFields(' + InttoStr(ArchiveID) + ')', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.SelectFields(FbfaDataAccess.cntBfaServer, FLog, ArchiveID);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.GetFieldValue(Typ: Integer; ID: Int64; ColType: Integer = 0): WideString;
begin
  FErrorCode:= 0;
  Result:= '';
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'GetFieldValue(' + InttoStr(ID) + ')', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.GetFieldValue(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog, Typ, ID, ColType);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.SelectFieldByArchiveID(FieldID, ArchiveID, UserID: Int64; UserColumn, GroupColumn: Integer; var FldRec: TFldRec): TbfaSp;
begin
  Result:= nil;
  FErrorCode:= 0;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'SelectFieldByID(' + IntToStr(FieldID) + ')', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.SelectFieldByArchiveID(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog, FieldID, ArchiveID, UserID, UserColumn, GroupColumn, FldRec);
      end;      
      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.SelectAdminFieldByArchiveID(FieldID, ArchiveID, UserID: Int64; UserColumn, GroupColumn: Integer; var FldRec: TFldRec; List : TList<TbfaPermission>): Integer;
begin
  FErrorCode:= 0;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'SelectFieldByID(' + IntToStr(FieldID) + ')', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        FErrorCode:= MySQL.SelectAdminFieldByArchiveID(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog, FieldID, ArchiveID, UserID, UserColumn, GroupColumn, FldRec, List);
      end;      
      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
  Result:= FErrorCode;
end;
//function TbfaConnection.SelectAdminFieldByArchiveID(FieldID, ArchiveID, UserID: Int64; var FldRec: TFldRec; var UsrArray, GrpArray: array of Integer): Integer;
//begin
//  FErrorCode:= 0;
//  if FLogInitialized then
//  begin
//    FLog.Write(llHigh, ltInfo, 'SelectFieldByID(' + IntToStr(FieldID) + ')', ClassName);
//    if FConnected then
//    begin
//      if FProvider = providerMySQL then
//      begin
//        FErrorCode:= MySQL.SelectAdminFieldByArchiveID(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog, FieldID, ArchiveID, UserID, FldRec, UsrArray, GrpArray);
//      end;      
//      
//    end
//    else
//    begin
//      FErrorCode:= ecConnectionDatabaseFailure;
//      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
//    end;
//  end
//  else
//  begin
//    FErrorCode:= ecConnectionLogIsNotInitialized;
//  end;
//  Result:= FErrorCode;
//end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.GetContainerObjects(ArchiveLayoutID: Int64; ContainerType, ListType, SQLType: Integer; LayoutIDs: string): TbfaSP;
begin
  FErrorCode:= 0;
  Result:= nil;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'GetContainerObjects', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.GetContainerObjects(FbfaDataAccess.cntBfaServer, FLog, ArchiveLayoutID, ContainerType, ListType, SQLType, LayoutIDs);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 17.04.2013 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.AddObjectToUserMapping(ArcID, ObjID, Rights, RoleID: Int64; ObjType: Integer): Int64;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'InsertUserMapping', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.AddObjectToUserMapping(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog, ArcID, ObjID, Rights, RoleID, ObjType);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 17.04.2013 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.UpdateObjectInUserMapping(ArcID, ObjID, Rights: Int64; ObjType: Integer): Int64;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'UpdateUserMapping', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.UpdateObjectInUserMapping(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog, ArcID, ObjID, Rights, ObjType);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.SelectAvailableUsers(ArchiveLayoutID: Int64; FullList: Integer): TDataSet;
begin
  FErrorCode:= 0;
  Result:= nil;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'SelectAvailableUsers', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.SelectAvailableUsers(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog, ArchiveLayoutID, FullList);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.SelectAvailableGroups(ArchiveLayoutID: Int64; FullList: Integer): TDataSet;
begin
  FErrorCode:= 0;
  Result:= nil;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'SelectAvailableGroups', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.SelectAvailableGroups(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog, ArchiveLayoutID, FullList);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.SelectArchiveAncestorObjects(LayoutID: Int64): TbfaSP;
begin
  FErrorCode:= 0;
  Result:= nil;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'SelectArchiveAncestorObjects', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.SelectArchiveAncestorObjects(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog, LayoutID);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;

//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 23.05.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaConnection.DeleteObjectFromUserMapping(LayoutID, ObjID: Int64; ObjType: Integer): Integer;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'DeleteObjectFromUserMapping', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.DeleteObjectFromUserMapping(FbfaMySQLDataAccess, FbfaDataAccess.cntBfaServer, FLog, LayoutID, ObjID, ObjType);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;

//
function TbfaConnection.UpdateLayoutRights(LayoutID, Rights, RoleID: Int64): Integer;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'UpdateLayoutRights', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.UpdateLayoutRights(FbfaDataAccess.cntBfaServer, FLog, LayoutID, Rights, RoleID);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
function TbfaConnection.UpdateContainerDescription(LayoutID: Int64; Description: WideString): Integer;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'UpdateContainerDescription', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.UpdateContainerDescription(FbfaDataAccess.cntBfaServer, FLog, LayoutID, Description);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;

//
function TbfaConnection.SelectContainerDescription(ID: Int64): WideString;
begin
  FErrorCode:= 0;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'SelectContainerDescription', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.SelectContainerDescription(FbfaDataAccess.cntBfaServer, FLog, ID);        
      end;      

    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
      Result:= '';
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
    Result:= '';
  end;
end;


//
function TbfaConnection.SelectFieldPermissions(ArcID, AddFieldID: Int64; var UsrIdx, GrpIdx: Integer): TBfaSP;
begin
  FErrorCode:= 0;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'SelectFieldPermissions', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.SelectFieldPermissions(FbfaDataAccess.cntBfaServer, FLog, ArcID, AddFieldID, UsrIdx, GrpIdx);        
      end;      

    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
function TbfaConnection.UpdateInheritedFieldRights(LayoutID, ArcID, AddfieldID: Int64; Idx, ObjType: Integer; Read, Write, Modify, UnsetRights: UInt64): Integer;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'UpdateFieldRights', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.UpdateInheritedFieldRights(FbfaDataAccess.cntBfaServer, FLog, LayoutID, ArcID, AddfieldID, Idx, ObjType,Read, Write, Modify, UnsetRights);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
function TbfaConnection.SelectMaxUsrGrpIdx(var UsrIdx, GrpIdx: Integer): Integer;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'SelectMaxUsrGrpIdx', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.SelectMaxUsrGrpIdx(FbfaDataAccess.cntBfaServer, FLog, UsrIdx, GrpIdx);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
function TbfaConnection.SelectDMSRootArchiveID: Int64;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'SelectDMSRootArchiveID', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.SelectDMSRootArchiveID(FbfaDataAccess.cntBfaServer, FLog);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
function TbfaConnection.CopyNode(ParentID: Int64; ObjType: Integer; NewParentID, NewObjID: Int64): Int64;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'CopyNode', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.CopyNode(FbfaDataAccess.cntBfaServer, FLog, ParentID, ObjType, NewParentID, NewObjID);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//
function TbfaConnection.CreateDocument(DocLoc, OriLoc, ExtLoc, SrcLoc, DocName: WideString; 
                            UserID: Int64; CheckOut, RevSave: Boolean; ArcID: Int64; PageCount, DocType: Integer;
                            FileType: string; DocTxt, DocTags: WideString): Int64;
begin
  FErrorCode:= 0;
  Result:= -1;
  if FLogInitialized then
  begin
    FLog.Write(llHigh, ltInfo, 'CreateDocument', ClassName);
    if FConnected then
    begin
      if FProvider = providerMySQL then
      begin
        Result:= MySQL.CreateDocument(FbfaDataAccess.cntBfaServer, FLog, DocLoc, OriLoc, ExtLoc, SrcLoc, DocName,
                                      UserID, CheckOut, RevSave, ArcID, PageCount, DocType, FileType, DocTxt, DocTags);
      end;      
    end
    else
    begin
      FErrorCode:= ecConnectionDatabaseFailure;
      FLog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
  end;
end;
//

end.
