unit bfaDBConnection;

interface

uses bfaLog, bfaCommonUtils// , bfaDataAccessDM

  {$if defined(VERSION40)}
    , bfaMySQLDataAccessDM
  {$elseif defined(VERSION36)}
    , bfaDataAccessDM
  {$ifend}

;

type
  TbfaMySQLDataAccess36 = class(TbfaDataAccess)

  end;

  TbfaDBConnection = class(TObject)
  protected
    /// <Beschreibung>Fehlercode</Beschreibung>
    /// <Description>Error Code</Description>
    FErrorCode: Integer;
    /// <Beschreibung>UniDac Provider</Beschreibung>
    /// <Description>UniDac Provider</Description>
    FProvider: Integer;
    /// <Beschreibung>SQL Benutzername</Beschreibung>
    /// <Description>SQL Username</Description>
    FUserName: WideString;
    /// <Beschreibung>SQL Passwort</Beschreibung>
    /// <Description>SQL password</Description>
    FPassword: WideString;
    /// <Beschreibung>SQL Servername oder IP Adresse</Beschreibung>
    /// <Description>SQL servername or IP address</Description>
    FServer: WideString;
    /// <Beschreibung>SQL Server Port</Beschreibung>
    /// <Description>SQL Server Port</Description>
    FPort: Integer;
    /// <Beschreibung>SQL Datenbankname</Beschreibung>
    /// <Description>SQL Databasename</Description>
    FDatabase: WideString;
    /// <Beschreibung>Klient Versionsnummer</Beschreibung>
    /// <Description>Client Version</Description>
    FClientVersion: WideString;
    /// <Beschreibung>Die Datenbank Verbindung ist zustande gekommen</Beschreibung>
    /// <Description>The Database Connection exists</Description>
    FConnected: Boolean;
    /// <Beschreibung>Wenn das Log Objekt zustande gekommen ist, dann true, sonst false.</Beschreibung>
    /// <Description>If the Log object is initialized, then true, else false</Description>
    FLogInitialized: Boolean;
    /// <Beschreibung>Das Log Objekt</Beschreibung>
    /// <Description>The Log object</Description>
    FLog: TbfaLog;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FbfaDataAccess: TBfaDataAccess;
  {$if defined(VERSION40)}
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FbfaMySQLDataAccess : TbfaMySQLDataAccess;
  {$elseif defined(VERSION36)}
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FbfaMySQLDataAccess : TbfaMySQLDataAccess36;
  {$ifend}


  public
    /// <Beschreibung>Fehlercode</Beschreibung>
    /// <Description>Error Code</Description>
    property ErrorCode: Integer read FErrorCode;
    /// <Beschreibung>SQL Datenbankname</Beschreibung>
    /// <Description>SQL Databasename</Description>
    property Database: WideString read FDatabase;
    /// <Beschreibung>SQL Passwort</Beschreibung>
    /// <Description>SQL password</Description>
    property Password: WideString read FPassword;
    /// <Beschreibung>SQL Server Port</Beschreibung>
    /// <Description>SQL Server Port</Description>
    property Port: Integer read FPort;
    /// <Beschreibung>SQL Servername oder IP Adresse</Beschreibung>
    /// <Description>SQL servername or IP address</Description>
    property Server: WideString read FServer;
    /// <Beschreibung>SQL Benutzername</Beschreibung>
    /// <Description>SQL Username</Description>
    property UserName: WideString read FUserName;
    /// <Beschreibung>UniDac Provider</Beschreibung>
    /// <Description>UniDac Provider</Description>
    property Provider: Integer read FProvider;
    /// <Beschreibung>Log Objekt</Beschreibung>
    /// <Description>Log object</Description>
    property Log : TbfaLog read FLog;


    /// <Beschreibung>Setzt die Standardwerte und erstellt die Objekte</Beschreibung>
    /// <Description>Sets the default values and creates the objects</Description>
    constructor Create; 
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function ConnectDB(SQLMonitor: Boolean): Boolean; virtual;
    /// <Beschreibung>Abbruch die Datenbank Verbindung</Beschreibung>
    /// <Description>Destroyes the Database Connection</Description>
    function DestroyDB: Boolean;
    /// <Beschreibung>Schliet die Datenbank Verbindung</Beschreibung>
    /// <Description>Closes the Database Connection</Description>
    function DisconnectDB: Boolean;
    /// <Beschreibung>Initialisiert das Log und das Connection Objekt</Beschreibung>
    /// <Description>Initialize the Log and the Connection objects</Description>
    function Init(const Log: TbfaLog): Integer;
  end;

implementation

uses SysUtils, Uni, ADODB, bfaCommonMethods
  {$if defined(VERSION40)}
    , bfaMySQLDatabaseScripts, bfaMethods
  {$ifend}
;

//
//  Name: Initialize
//  Parameters: -
//  Beschreibung: Setzt die Standardwerte und erstellt die Objekte
//  Description: Sets the default values and creates the objects
//  Return value: -
// [] 17.04.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
constructor TbfaDBConnection.Create;
begin
  inherited Create;
  FErrorCode:= -1;
  FProvider:= -1;
  FUserName:= '';
  FPassword:= '';
  FServer:= '';
  FPort:= -1;
  FDatabase:= '';
  FClientVersion:= '';
  FConnected:= False;
  FLogInitialized:= False;
  FLog:= nil;
  FbfaDataAccess:= TbfaDataAccess.Create(FbfaDataAccess);
  FbfaMySQLDataAccess:= nil;
end; 
//
//  Name: ConnectDB
//  Parameters: Provider
//              UserName
//              Password
//              SQLServer
//              Port
//              Database
//              ClientVersion
//              SQLMonitor  
//  Beschreibung: Einstellt die Datenbank Verbindung
//  Description: Sets the Database connection
//  Return value: True if connected
// [] 17.04.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaDBConnection.ConnectDB(SQLMonitor: Boolean): Boolean;

  function SetProvider(Provider: integer): WideString;
  begin
    if Provider = providerMySQL then
      Result:= sSysMySQL
    else if Provider = providerMsSQL then
      Result:= sSysSQLServer
    else if Provider = providerPostgreSQL then
      Result:= sSysPostgreSQL
    else if Provider = providerOracleSQL then
      Result:= sSysOracle;
  end;

begin
  FErrorCode:= 0;
  if FLogInitialized then
  begin
    Flog.Write(llHigh, ltInfo, 'ConnectDB', ClassName);
    try

      Flog.Write(llDebug, ltDebug, 'SQLMonitor: ' + BoolToStr(SQLMonitor, true), ClassName);
      FbfaDataAccess.monUniSQL.Active:= SQLMonitor;

      with FbfaDataAccess.cntBfaServer do
      begin
        Connected := False;
        ProviderName := SetProvider(FProvider);
        Server := FServer;
        Port := FPort;
        Username := FUserName;
        Password := FPassword;
        Database := FDatabase;
        Connect;
        Flog.Write(llDebug, ltDebug, 'Connected: ' + BoolToStr(Connected, true), ClassName);
        FConnected := Connected;
        Result := FConnected;

        if FConnected then
        begin
          if FProvider = providerMySQL then
          begin
            {$if defined(VERSION40)}
               FbfaMySQLDataAccess := TbfaMySQLDataAccess.Create(FbfaMySQLDataAccess);
            {$elseif defined(VERSION36)}
               FbfaMySQLDataAccess := TbfaMySQLDataAccess36.Create(FbfaMySQLDataAccess);
            {$ifend}
          end;
        end;
      end;

    except
      FErrorCode:= ecConnectionDatabaseFailure;
      Flog.Write(llNormal, ltError, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
      Result:= False;
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
    Result:= False;
  end;
end;
//
//  Name: DestroyDB
//  Parameters: - 
//  Beschreibung: Abbruch die Datenbank Verbindung
//  Description: Destroyes the Database Connection
//  Return value: Connected
// [] 17.04.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaDBConnection.DestroyDB: Boolean;
begin
  FErrorCode:= 0;
  if FLogInitialized then
  begin
    Flog.Write(llHigh, ltDebug, 'DestroyDB', ClassName);
    with FbfaDataAccess.cntBfaServer do
    begin
      if FConnected then
      begin
        Result:= DisConnectDB;
      end
      else
        Result:= False;
    end;
    if FProvider = providerMySQL then
    begin
      if FbfaMySQLDataAccess <> nil then
      begin
        FbfaMySQLDataAccess.Destroy;
        FbfaMySQLDataAccess:= nil;
      end;
    end;    
    if FbfaDataAccess <> nil then
    begin
      FbfaDataAccess.Destroy;
      FbfaDataAccess:= nil;
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
    Result:= False;
  end;
end;
//
//  Name: DisconnectDB
//  Parameters: - 
//  Beschreibung: Schliet die Datenbank Verbindung
//  Description: Closes the Database Connection
//  Return value: Connected
// [] 17.04.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaDBConnection.DisconnectDB: Boolean;
begin
  FErrorCode:= 0;
  if FLogInitialized then
  begin
    Flog.Write(llHigh, ltDebug, 'DisconnectDB', ClassName);
    with FbfaDataAccess.cntBfaServer do
    begin
      if FConnected then
        DisConnect;
      Connected:= False;
      FConnected:= False;
      Result:= False;
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
    Result:= False;
  end;
end;
//
//  Name: Init
//  Parameters: Log
//  Beschreibung: Einstellt das Log Objekt
//  Description: Sets the Log object
//  Return value: FErrorCode
// [] 17.04.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaDBConnection.Init(const Log: TbfaLog): Integer;
begin
  FErrorCode:= 0;
  if ( Log <> nil ) then
  begin
    try
      FLog:= Log as TbfaLog;
      Result:= FErrorCode;
      if FErrorCode = 0 then
        FLogInitialized:= True
      else
        FLogInitialized:= False;
    except
      FErrorCode:= ecConnectionFailureInInit;
      FLogInitialized:= False;
      Result:= FErrorCode;
    end;
  end
  else
  begin
    FErrorCode:= ecConnectionLogIsNotInitialized;
    FLogInitialized:= False;
  end;    
  Result:= FErrorCode;
end;
//

end.
