unit bfaDocument;

interface

uses
  Classes, 
  Generics.Collections,
  bfaApplication,
  bfaBaseObjects,
  bfaCommonUtils,
  bfaMapping;

type
  /// <Beschreibung>bitfarm Document Klasse</Beschreibung>
  /// <Description>bitfarm Document class</Description>
  TbfaDocument = class(TbfaBaseDocument)
  private
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    FbfaApplication: PbfaApplication;
    
    
	public
    
    /// <Beschreibung>Setzt die Standardwerte und erstellt die Objekte</Beschreibung>
    /// <Description>Sets the default values and creates the objects</Description>
		constructor Create(bfaApp: PbfaApplication); overload;
    /// <Beschreibung>Ldt ein Dokument mit der bergebenen DocID aus der Datenbank.</Beschreibung>
    /// <Description>Loads a Document with the passed DocID from the database</Description>
    function Load(GDocID: Int64): Integer;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure Clone(Source: TbfaDocument);
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    function Archive: Integer; 


  end;

  PbfaDocument = ^TbfaDocument;

implementation

uses 
  SysUtils, bfaMethods, bfaCommonMethods;

//
//  Name: Create
//  Parameters: -
//  Beschreibung: Setzt die Standardwerte und erstellt die Objekte
//  Description: Sets the default values and creates the objects
//  Return value: -
// [] 17.04.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
constructor TbfaDocument.Create(bfaApp: PbfaApplication);
begin
  try
    New(FbfaApplication);
    FbfaApplication:= bfaApp;
    
    inherited Create;

  except
    FbfaApplication:= nil;
  end;
end;
//
//  Name:
//  Parameters:
//  Beschreibung:
//  Description:
//  Return value:
// [] 17.04.2012 b.takacs 4.0.0.0
//------------------------------------------------------------------------------
function TbfaDocument.Load(GDocID: Int64): Integer;
var
  DocRec : TDocumentRec;
  CopyResult : Boolean;
begin
  FErrorCode:= 0;
  if FInitialized then
  begin
    FbfaApplication^.WriteLog(llHigh, ltInfo, 'Load(GDocID: ' + IntToStr(GDocID) + ')', ClassName);
    try

      FErrorCode:= FbfaApplication^.Connection.LoadDocument(GDocID, DocRec);
      if FErrorCode = 0 then
      begin
        FID:= GDocID;
        FArcID:= DocRec.ArcID;
        FDocLoc:= DocRec.DocLoc;
        FExtLoc:= DocRec.ExtLoc;
        FOriLoc:= DocRec.OriLoc;
        FDocumentName:= DocRec.Documentname;
        FDocTXT:= DocRec.DocTXT;
        FDocTags:= DocRec.Vocabulary;
        FCheckedOut:= DocRec.CheckedOut;
        FLoaded:= True;
//        Docrec.DocStream.Position:= 0;
//        FStreamTIF.LoadFromFile(FDocLoc);
//        FStreamORI.CopyFrom(DocRec.OriStream, DocRec.OriStream.Size);
//        FStreamIEV.CopyFrom(DocRec.AnnoStream, DocRec.AnnoStream.Size);
//        FStreamLetter.CopyFrom(DocRec.LetterStream, DocRec.LetterStream.Size);
        FCheckOutUserID:= DocRec.CheckOutUserID;                                       
        
        FPageCount:= DocRec.PageCount;
        FDocType:= DocRec.DocType;
        FFileType:= DocRec.FileType;
        FFilePath:= DocRec.FilePath;
        FImage:= DocRec.Image;

        FUserID:= DocRec.UserID;
        FDocDate:= DocRec.DocDate;
        FRevSave:= DocRec.RevSave;
        FModified:= DocRec.Modified;
        FModifiedUserID:= DocRec.ModifiedUserID;

        FInitialized:= True;        
        

////////////////////////////////////////////////////////////////////////////////
///  Local speichern ins Temp Verzeichnis... 

//        If not DirectoryExists(FbfaApplication^.TemporaryDir + '\DATA\' +
//                  FbfaApplication^.Connection.Database + '\') then
//          ForceDirectories(FbfaApplication^.TemporaryDir + '\DATA\' +
//                  FbfaApplication^.Connection.Database + '\');
(*
        FbfaApplication^.WriteLog(llDebug, ltDebug, 'Copy: ' + FbfaApplication^.ServerDir + '\' +
          FDocLoc + '\' + IntToStr(FID) + '.' + FFileType + '; To: ' + FbfaApplication^.TemporaryDir + '\DATA\' +
                  FbfaApplication^.Connection.Database + '\' + IntToStr(FID) + '.' + FFileType, ClassName);

        CopyResult := CopyBfaFile(FbfaApplication^.ServerDir + '\' + FDocLoc + '\', IntToStr(FID) + '.' + FFileType,
                  FbfaApplication^.TemporaryDir + '\DATA\' +
                  FbfaApplication^.Connection.Database + '\' + IntToStr(FID) + '.' + FFileType);

        FbfaApplication^.WriteLog(llDebug, ltDebug, 'CopyResult: ' + BoolToStr(CopyResult), ClassName);

        if FileExists(FbfaApplication^.TemporaryDir + '\DATA\' +
                  FbfaApplication^.Connection.Database + '\' + IntToStr(FID) + '.' + FFileType) then
          FStreamTIF.LoadFromFile(FbfaApplication^.TemporaryDir + '\DATA\' +
                  FbfaApplication^.Connection.Database + '\' + IntToStr(FID) + '.' + FFileType);
*)
//        if FileExists(FbfaApplication^.ServerDir + '\DATA\' +
//                  FbfaApplication^.Connection.Database + '\' + IntToStr(FID) + '.' + FFileType) then
//          FStreamTIF.LoadFromFile(FbfaApplication^.ServerDir + '\DATA\' +
//                  FbfaApplication^.Connection.Database + '\' + IntToStr(FID) + '.' + FFileType);

////////////////////////////////////////////////////////////////////////////////


        FLoaded:= True;
      end;

    except
      FErrorCode:= ecDocumentFailureInLoadArchive;
      FLoaded:= False;
      FbfaApplication^.WriteLog(llNormal, ltException, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
      Result:= FErrorCode;
    end;
  end
  else
  begin
    FErrorCode:= ecDocumentConnectionOrLogIsNotInitialized;
    FLoaded:= False;
  end;
  Result:= FErrorCode;
end;
//
procedure TbfaDocument.Clone(Source : TbfaDocument);
begin
  try
    inherited Clone(Source);
  except

  end;
end;
//
function TbfaDocument.Archive: Integer;
var
  ID: Int64;
begin
  FErrorCode:= 0;
  if FInitialized then
  begin
    FbfaApplication^.WriteLog(llHigh, ltInfo, 'Save', ClassName);
    try

      FDocLoc := '\DATA\' + FbfaApplication^.Connection.Database + '\';

      ID:= FbfaApplication^.Connection.CreateDocument(FDocLoc, FOriLoc, FextLoc, '',
                                FDocumentName, FUserID, FCheckedOut, FRevSave,
                                FArcID, FPageCount, FDocType, FFileType, FDocTXT, FDocTags);
      if ID > 0 then
      begin
        FID := ID;
////////////////////////////////////////////////////////////////////////////////

//        If not DirectoryExists(FbfaApplication^.ServerDir + '\DATA\' + FbfaApplication^.Connection.Database + '\') then
//          ForceDirectories(FbfaApplication^.ServerDir + '\DATA\' + FbfaApplication^.Connection.Database + '\');
//        FStreamTIF.SaveToFile(FbfaApplication^.ServerDir + '\DATA\' + FbfaApplication^.Connection.Database + '\' +
//                  IntToStr(ID) + '.' + FFileType);
//
//        CopyBfaFile(FbfaApplication^.ServerDir + '\DATA\' + FbfaApplication^.Connection.Database + '\',
//                  FDocLoc, IntToStr(ID) + '.' + FFileType);
////////////////////////////////////////////////////////////////////////////////
        
      end;

    except
      FErrorCode:= ecDocumentFailureInLoadArchive;
      FLoaded:= False;
      FbfaApplication^.WriteLog(llNormal, ltException, sSysErrorCode + ': ' + IntToStr(FErrorCode), ClassName);
      Result:= FErrorCode;
    end;
  end
  else
  begin
    FErrorCode:= ecDocumentConnectionOrLogIsNotInitialized;
    FLoaded:= False;
  end;
  Result:= FErrorCode;
end;
//
end.
