unit bfaMapping;

interface

uses
  bfaBaseObjects, bfaField;

type
  TbfaMapping = class(TObject)
  protected
    FObj : TbfaBaseObject;
    FLayoutID : Int64;
    FRight : Int64;      
    FStatus : Integer;   
    FMappingID : Int64;
    FMappingName : WideString;
    FModified : Boolean;
  public
    constructor Create(bfaApp : Pointer; PrmName, PrmDesc: WideString; LytID, PrmObjID, PrmRight : Int64; PrmObjType, Sts : Integer; MpgID : Int64; MpgName : WideString; Modified : Boolean; FieldType : Integer = -1); overload; virtual;
    constructor Create(PObj : TbfaBaseObject; LytID, PrmRight : Int64; Sts : Integer; MpgID : Int64; MpgName : WideString; Modified : Boolean); overload;

    property Obj : TbfaBaseObject read FObj;
    property LayoutID : Int64 read FLayoutID write FLayoutID;
    property Right : Int64 read FRight write FRight;
    property Status : Integer read FStatus write FStatus;
    property MappingID : Int64 read FMappingID write FMappingID;
    property MappingName : WideString read FMappingName write FMappingName;
    property Modified : Boolean read FModified write FModified;
  
  end; 

  TbfaFieldMapping = class(TbfaMapping)
  protected
    FObj : TbfaAdminField;
  public
    constructor Create(bfaApp : Pointer; PrmName, PrmDesc: WideString; LytID, PrmObjID, PrmRight : Int64; PrmObjType, Sts : Integer; MpgID : Int64; MpgName : WideString; Modified : Boolean; FieldType : Integer = -1); override;
    constructor Create(PObj : TbfaAdminField; LytID : Int64; Sts : Integer; MpgID : Int64; MpgName : WideString; Modified : Boolean); overload;

    property Obj : TbfaAdminField read FObj;
  end; 
  
  TbfaModificationType = (mdtNameModified, mdtAdded, mdtDeleted, mdtObjectModified, mdtNonde);

  TbfaModification = class(TObject)
  private
    FObj : TbfaBaseObject;
    FModificationType : TbfaModificationType;
    FParentID,
    FLytID : Int64;
  public
    constructor Create(Obj : TbfaBaseObject; Modification : TbfaModificationType; PID, LID : Int64);

    property Obj : TbfaBaseObject read FObj;
    property ModificationType : TbfaModificationType read FModificationType;
    property ParentID : Int64 read FParentID;
    property LytID : Int64 read FLytID;
  end;   

implementation

uses
  bfaCommonUtils, bfaUser;

//
constructor TbfaMapping.Create(bfaApp : Pointer; PrmName, PrmDesc: WideString; LytID, PrmObjID, PrmRight : Int64; PrmObjType, Sts : Integer; MpgID : Int64; MpgName : WideString; Modified : Boolean; FieldType : Integer = -1);
begin
  FLayoutID := LytID;
  FRight := PrmRight;
  FStatus := Sts;
  FMappingID := MpgID;
  FMappingName := MpgName;
  FModified := Modified;
  if FieldType = -1 then
  begin
    if PrmObjType = otUser then
      FObj := TbfaUser.Create(bfaApp)
    else if PrmObjType = otGroup then
      FObj := TbfaGroup.Create(bfaApp)
    else if ((PrmObjType = otAdditionalField) or (PrmObjType = otStatusField)) then
      FObj := TbfaField.Create(bfaApp);
  end
  else 
  begin
    if FieldType = 0 then
    begin
      FObj := TbfaAdminField.Create(bfaApp);  
    end
    else if FieldType = 1 then
    begin
      FObj := TbfaClientField.Create(bfaApp);  
    end;
  end;
  FObj.ID := PrmObjID;
  FObj.Name := PrmName;
  FObj.Description := PrmDesc;
  FObj.ObjType := PrmObjType;
  FObj.GeneralPropertiesModified := False;
  FObj.Modified := False;
end;
//
constructor TbfaMapping.Create(PObj : TbfaBaseObject; LytID, PrmRight : Int64; Sts : Integer; MpgID : Int64; MpgName : WideString; Modified : Boolean);
begin
  FObj := PObj;
  FObj.GeneralPropertiesModified := False;
  FObj.Modified := False;  

  FLayoutID := LytID;
  FRight := PrmRight;
  FStatus := Sts;
  FMappingID := MpgID;
  FMappingName := MpgName;
  FModified := Modified;
end;
//
constructor TbfaFieldMapping.Create(bfaApp : Pointer; PrmName, PrmDesc: WideString; LytID, PrmObjID, PrmRight : Int64; PrmObjType, Sts : Integer; MpgID : Int64; MpgName : WideString; Modified : Boolean; FieldType : Integer = -1);
begin
  FObj := TbfaAdminField.Create(bfaApp);
  TbfaBaseObject(FObj).ID := PrmObjID;
  FObj.Name := PrmName;
  FObj.Description := PrmDesc;
  TbfaBaseObject(FObj).ObjType := PrmObjType;  

  inherited Create(FObj ,LytID, PrmRight, Sts, MpgID, MpgName, Modified);  
end;
//
constructor TbfaFieldMapping.Create(PObj : TbfaAdminField; LytID : Int64; Sts : Integer; MpgID : Int64; MpgName : WideString; Modified : Boolean);
begin
  FObj := PObj;
  FObj.GeneralPropertiesModified := False;
  FObj.Modified := False;  

  FLayoutID := LytID;
  FRight := -1;
  FStatus := Sts;
  FMappingID := MpgID;
  FMappingName := MpgName;
  FModified := Modified;
end;
//
constructor TbfaModification.Create(Obj : TbfaBaseObject; Modification : TbfaModificationType; PID, LID : Int64);
begin
  FObj := Obj;
  FModificationType := Modification;
  FParentID := PID;
  FLytID := LID;
end;
//

end.
