unit bfaSearch;

interface

uses
  Classes,
  Generics.Collections,
  DB,
  uni,
  XMLIntf,
  bfaBaseObjects,
  bfaApplication,
  bfaObjects36;

type
  TbfaSearch = class(TbfaBaseSearch)
  private
    FbfaApplication : PbfaApplication;
    /// <Beschreibung>Liste mit BitfarmAddfield Objekten</Beschreibung>
    /// <Description></Description>
    FAddFields : TList<TbfaAddField>;
    /// <Beschreibung>Liste mit BitfarmStatefield Objekten</Beschreibung>
    /// <Description></Description>
    FStsFields : TList<TbfaStateField>;

    function GetUserContext : Int64;
    function GetDMSADmin : Integer;

  public
    constructor Create(bfaApp: PbfaApplication);

    property UserContext : Int64 read GetUserContext;
    property DMSAdmin : Integer read GetDMSADmin;

    property AddFields : TList<TbfaAddField> read FAddFields write FAddFields;
    property StsFields : TList<TbfaStateField> read FStsFields write FStsFields;
     
  end;

  TbfaSearchProcessing = class(TbfaBaseSearchProcessing)
  private 
    Fqry : TUniStoredProc;
  
  public
    constructor Create;    

    /// <Beschreibung>Funktion um eine Lesezeichendatei einzulesen. Gibt eine Liste an SearchObjekten zurck. Optional kann man den Namen oder die ID eines bestimmten Lesezeichen angeben, welches dann zurckgegeben wird. Wird eine UserId/GroupID bzw eine Liste von IDs angegeben, so werden nur die Lesezeichen zurckgegeben, auf die mindestens eine (oder mehrere) ID berechtigt ist. Die Eingrenzung auf UserId hat Vorrang vor GroupID</Beschreibung>
    /// <Description></Description>
//    function ReadXMLFile(Path : WideString; Name: WideString = ''; ID : Int64 = -1; UserID : TStringList = nil; GroupID : TStringList = nil) : TList<TbfaSearch>; overload;
    /// <Beschreibung>Funktion um ein SearchObject direkt mit einem XML-Knoten (fav-Node) zu fllen, bei Erfolg wird ein SearchObject zurckgegeben, andernfalls None. Der Knoten muss ein ElementTree.Element Objekt sein</Beschreibung>
    /// <Description></Description>
//    function FillSearchWithNode(FbfaApp : PbfaApplication; Fav : IXMLNode) : TbfaSearch;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
//    function AddSearchFields(SObj : TbfaSearch; ChildNode : IXMLNode) : TbfaSearch;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
    procedure Reset;
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
//    function Search(SearchObj : TbfaSearch) : TDataSet;

  
  end;  


implementation

uses
  bfaCommonUtils, Variants, SysUtils;

//
constructor TbfaSearch.Create(bfaApp: PbfaApplication);
begin
  try
    inherited Create;

    New(FbfaApplication);
    
    FbfaApplication := bfaApp;
  except
    FbfaApplication:= nil;
  end;

//  FAddFields := TList<TbfaAddField>.Create;
//  FAddFields.Clear;
end;
//
function TbfaSearch.GetUserContext : Int64;
begin
  Result := FbfaApplication^.User.ID;
end;
//
function TbfaSearch.GetDMSADmin : Integer;
begin
  if FbfaApplication^.User.DMSAdmin then
    Result := 1
  else
    Result := 0;
end;
//
constructor TbfaSearchProcessing.Create;
begin
  Fqry := TUniStoredProc.Create(nil);
  Fqry.Name := 'spSearch40';
  Reset;
end;
//
procedure TbfaSearchProcessing.Reset;
begin
  FVersion := 40;
end;
//
(*function TbfaSearchProcessing.Search(SearchObj : TbfaSearch) : TDataSet;
var
  i, idx : Integer;
  ArcIDs, GDocIDs : string; 
  UseRights, found : Boolean;
  qry : TUniQuery;
  TmpSList : TStringList;
begin
  if (SearchObj <> nil) then
  begin
    ArcIDs := '';
    GDocIDs := '';
    try
      UseRights := False;
      found := False;
      if (SearchObj.UserID.Count > 0) then
      begin
        UseRights := True;
        SearchObj.UserID.Sort;
        found := SearchObj.UserID.Find(IntToStr(SearchObj.UserContext), Idx);
      end;

      if ((SearchObj.GroupID.Count > 0) and (not found)) then
      begin
        UseRights := True;
        SearchObj.GroupID.Sort;
        qry := TUniQuery.Create(nil);
        try
          qry.SQL.Text := '';
            
          qry.Open;
          TmpSList.CommaText := qry.FieldByName('ids').AsString;
          qry.Close;
          if (TmpSList.Count > 0) then
          begin
            i := 0;
            while ((i <= TmpSList.Count - 1) and (not found)) do
            begin 
              found := SearchObj.GroupID.Find(TmpSList[i], Idx);
              inc(i);
            end;
          end;

        finally
          FreeAndNil(qry);
        end; 
      end;

      if ((not UseRights) or (UseRights and found)) then
      begin
      
        if (SearchObj.ArcID.Count > 1) then
          ArcIDs := Stringreplace(SearchObj.ArcID.Text, Char(13) + Char(10), ',', [rfReplaceAll])
        else
          ArcIDs := Stringreplace(SearchObj.ArcID.Text ,Char(13) + Char(10), '', [rfReplaceAll]);

        if (ArcIDs <> '') then
        begin
          if (ArcIDs[Length(ArcIDs)] = ',') then
            delete(ArcIDs, length(ArcIDs), 1);
        end;

        if Fqry.Active then
          Fqry.Close;
                     
        Fqry.Connection := SearchObj.FbfaApplication^.Connection.MySQLConnection;
        Fqry.Debug := Fqry.Connection.Debug;

        Fqry.StoredProcName := 'sp_search';
        Fqry.PrepareSQL;

        Fqry.ParamByName('prm_searchtype').AsInteger := SearchObj.SearchType;

        Fqry.ParamByName('prm_version').AsInteger := sSysVersion40;
            
        Fqry.ParamByName('prm_arcid').AsString := ArcIDs;
        Fqry.ParamByName('prm_order').AsWideString := SearchObj.Order;
        Fqry.ParamByName('prm_direction').AsWideString := SearchObj.Direction;
        Fqry.ParamByName('prm_limit').AsInteger := SearchObj.MaxValues;
          
        Fqry.ParamByName('prm_userid').AsLargeInt := SearchObj.UserContext;

        Fqry.ParamByName('prm_searchuserid').AsLargeInt := SearchObj.SearchUserID;
        Fqry.ParamByName('prm_searchuser').AsWideString := SearchObj.SearchUser;
          
        if SearchObj.SearchUserNegated then
          Fqry.ParamByName('prm_searchuser_negated').AsSmallInt := 1
        else
          Fqry.ParamByName('prm_searchuser_negated').AsSmallInt := 0;

        Fqry.ParamByName('prm_searchgroupid').AsLargeInt := SearchObj.SearchGroupID;
        Fqry.ParamByName('prm_searchgroup').AsWideString := SearchObj.SearchGroup;
          
        if SearchObj.SearchGroupNegated then
          Fqry.ParamByName('prm_searchgroup_negated').AsSmallInt := 1
        else
          Fqry.ParamByName('prm_searchgroup_negated').AsSmallInt := 0;            
            
        if VarIsNull(SearchObj.SearchDate) then
          Fqry.ParamByName('prm_searchdate').AsWideString := ''
        else
          Fqry.ParamByName('prm_searchdate').AsWideString := SearchObj.SearchDate;
        if VarIsNull(SearchObj.SearchDateHigh) then
          Fqry.ParamByName('prm_searchdate_high').AsWideString := ''
        else
          Fqry.ParamByName('prm_searchdate_high').AsWideString := SearchObj.SearchDateHigh;
        if SearchObj.SearchDateNegated then
          Fqry.ParamByName('prm_searchdate_negated').AsSmallInt := 1
        else
          Fqry.ParamByName('prm_searchdate_negated').AsSmallInt := 0;
            
        if SearchObj.SQLWhereOnly then
          Fqry.ParamByName('prm_sqlwhere_only').AsSmallInt := 1
        else
         Fqry.ParamByName('prm_sqlwhere_only').AsSmallInt := 0;
        Fqry.ParamByName('prm_sqlwhere').AsWideString := SearchObj.SQLWhere;
        Fqry.ParamByName('prm_sqlwhere_operator').AsString := SearchObj.SQLWhereOperator;
        Fqry.ParamByName('prm_ftx_fields').AsInteger := SearchObj.FtxFields;
        Fqry.ParamByName('prm_ftx').AsWideString := SearchObj.SearchString;
        if SearchObj.SearchStringNegated then
          Fqry.ParamByName('prm_ftx_negated').AsSmallInt := 1
        else
          Fqry.ParamByName('prm_ftx_negated').AsSmallInt := 0;
        if SearchObj.FtxMatchAgainst then
          Fqry.ParamByName('prm_matchagainst').AsSmallInt := 1
        else
          Fqry.ParamByName('prm_matchagainst').AsSmallInt := 0;
        if (SearchObj.FtxMatchAgainst) and (SearchObj.FtxNaturalMode) then
          Fqry.ParamByName('prm_natural_mode').AsSmallInt := 1
        else
          Fqry.ParamByName('prm_natural_mode').AsSmallInt := 0;
        if (SearchObj.FtxMatchAgainst) and (SearchObj.FtxBooleanMode) and (not SearchObj.FtxNaturalMode) then
          Fqry.ParamByName('prm_boolean_mode').AsSmallInt := 1
        else
          Fqry.ParamByName('prm_boolean_mode').AsSmallInt := 0;
        if (SearchObj.FtxMatchAgainst) and (SearchObj.FtxWithQueryExpansion) then
          Fqry.ParamByName('prm_with_query_expansion').AsSmallInt := 1
        else
          Fqry.ParamByName('prm_with_query_expansion').AsSmallInt := 0;
        Fqry.ParamByName('prm_result_type').AsWideString := SearchObj.ResultType;
        Fqry.ParamByName('prm_selected_cols').AsWideString := SearchObj.SelectedCols;
        Fqry.ParamByName('prm_addfield_xml').AsWideString := SearchObj.AddFieldXML;
        Fqry.ParamByName('prm_xmlpath').AsWideString := '';
        if SearchObj.Count then
          Fqry.ParamByName('prm_count').AsSmallInt := 1
        else
          Fqry.ParamByName('prm_count').AsSmallInt := 0;

        if not VarIsNull(SearchObj.CheckOut) then
        begin
          if SearchObj.CheckOut then
            Fqry.ParamByName('prm_checkout').AsSmallInt := 1
          else
            Fqry.ParamByName('prm_checkout').AsSmallInt := 0;
        end
        else
          Fqry.Params.ParamValues['prm_checkout'] := NULL;  

        Fqry.ParamByName('prm_stsfield_xml').AsWideString := SearchObj.StsFieldXML;
        Fqry.ParamByName('prm_status').AsInteger := SearchObj.State;  
        Fqry.ParamByName('prm_notstatus').AsInteger := SearchObj.NotState;  

        if (SearchObj.GdocID.Count > 1) then
          GDocIDs := Stringreplace(SearchObj.GDocID.Text, Char(13) + Char(10), ',', [rfReplaceAll])
        else
          GDocIDs := Stringreplace(SearchObj.GDocID.Text ,Char(13) + Char(10), '', [rfReplaceAll]);

        Fqry.ParamByName('prm_gdocid').AsString := GDocIDs;
        Fqry.ParamByName('prm_refsearch').AsWideString := SearchObj.RefSearch;
        Fqry.ParamByName('prm_timref').AsString := SearchObj.TimRef.CommaText;

        if VarIsNull(SearchObj.ResubDate) then
          Fqry.ParamByName('prm_resubdate').AsWideString := ''
        else
          Fqry.ParamByName('prm_resubdate').AsWideString := SearchObj.ResubDate;
        if VarIsNull(SearchObj.ResubDateHigh) then
          Fqry.ParamByName('prm_resubdate_high').AsWideString := ''
        else
          Fqry.ParamByName('prm_resubdate_high').AsWideString := SearchObj.ResubDateHigh;
        if SearchObj.ResubDateNegated then
          Fqry.ParamByName('prm_resubdate_negated').AsSmallInt := 1
        else
          Fqry.ParamByName('prm_resubdate_negated').AsSmallInt := 0;        
            
        if VarIsNull(SearchObj.ResubAlarm) then
          Fqry.ParamByName('prm_resubalarm').AsWideString := ''
        else
          Fqry.ParamByName('prm_resubalarm').AsWideString := SearchObj.ResubAlarm;
        if VarIsNull(SearchObj.ResubAlarmHigh) then
          Fqry.ParamByName('prm_resubalarm_high').AsWideString := ''
        else
          Fqry.ParamByName('prm_resubalarm_high').AsWideString := SearchObj.ResubAlarmHigh;
        if SearchObj.ResubAlarmNegated then
          Fqry.ParamByName('prm_resubalarm_negated').AsSmallInt := 1
        else
          Fqry.ParamByName('prm_resubalarm_negated').AsSmallInt := 0;   

        Fqry.ParamByName('prm_resubstatus').AsInteger := SearchObj.ResubState;
        Fqry.ParamByName('prm_resubuserid').AsLargeInt := SearchObj.ResubUserID;
        Fqry.ParamByName('prm_resubcontrollerid').AsLargeInt := SearchObj.ResubControllerID;
          
        Fqry.ParamByName(sPrmGroupType).AsInteger := otGroup;
        Fqry.ParamByName(sPrmUserType).AsInteger := otUser;
        Fqry.ParamByName(sPrmRoot).AsInteger := otRoot;
        Fqry.ParamByName(sPrmMandant).AsInteger := otMandant;
        Fqry.ParamByName(sPrmStore).AsInteger := otStore;
        Fqry.ParamByName(sPrmArchive).AsInteger := otArchive;
        Fqry.ParamByName(sPrmRecordArchive).AsInteger := otRecordArchive;
        Fqry.ParamByName(sPrmDMSAdmin).AsSmallInt := SearchObj.DMSAdmin;

        Fqry.ParamByName('prm_docname').AsWideString := SearchObj.DocName;
        if SearchObj.DocNameNegated then
          Fqry.ParamByName('prm_docname_negated').AsSmallInt := 1
        else
          Fqry.ParamByName('prm_docname_negated').AsSmallInt := 0;

            
        Fqry.Execute;   

        Result := Fqry;  
      end
      else
        Result := nil;
      
    except
      Result := nil;
    end;

  end
  else
    Result := nil;
end;
//
function TbfaSearchProcessing.AddSearchFields(SObj : TbfaSearch; ChildNode : IXMLNode) : TbfaSearch;
var
  Fld : TbfaAddField;
  FieldNode : IXMLNode;
  Attr : OleVariant;
  j : Integer;
begin
  try
    Fld := TbfaAddField.Create;

    Attr := ChildNode.Attributes['text'];
    if not VarIsNull(Attr) then
      Fld.Name := Attr;
    for j := 0 to ChildNode.ChildNodes.Count - 1 do
    begin
      FieldNode := ChildNode.ChildNodes[j];
      Fld.FieldType := otAdditionalField;
      if (Version = 40) then
      begin
        if FieldNode.NodeName = 'fieldtype' then
          Fld.FieldType := FieldNode.NodeValue      // kann auch Statusfeld sein      
      end;
      if FieldNode.NodeName = 'type' then
      begin
        Fld.ValueType := GetSearchFieldTypes(FieldNode.NodeValue);
      end
      else if ((FieldNode.NodeName = 'value') or (FieldNode.NodeName = 'valueUG')) then
      begin
        if Fld.ValueType = 1 then
        begin
          if (ChildNode.NodeName = 'zusatzname') then
            Fld.Value := FormatDateTime('yyyy-mm-dd', StrToDate(FieldNode.NodeValue))
          else
          begin
            try
              Fld.Value := FormatDateTime('yyyy-mm-dd  hh:nn:ss', StrToDate(FieldNode.NodeValue))
            except
              Fld.Value := NULL
            end;
          end;
        end
        else
          Fld.Value := FieldNode.NodeValue;
      end
      else if ((FieldNode.NodeName = 'valuehigh') or (FieldNode.NodeName = 'valueOG')) then
      begin
        if Fld.ValueType = 1 then
        begin
          if (ChildNode.NodeName = 'zusatzname') then
            Fld.ValueHigh := FormatDateTime('yyyy-mm-dd', StrToDate(FieldNode.NodeValue))
          else
          begin
            try
              Fld.ValueHigh := FormatDateTime('yyyy-mm-dd  hh:nn:ss', StrToDate(FieldNode.NodeValue))
            except
              Fld.ValueHigh := NULL
            end;
          end;
        end
        else
          Fld.ValueHigh := FieldNode.NodeValue;
      end
      else if (FieldNode.NodeName = 'NegBtn') then
      begin
        Fld.Negate := FieldNode.NodeValue = '1'
      end
      else if (FieldNode.NodeName = 'negated') then
      begin
        Fld.Negate := FieldNode.NodeValue = 'True';
      end;

      Fld.ModifiedSearch := True;
            
    end;
    TbfaSearch(SObj).AddFields.Add(Fld);
  except
    Result := nil;
  end;
end;
//
function TbfaSearchProcessing.ReadXMLFile(Path : WideString; Name: WideString = ''; ID : Int64 = -1; UserID : TStringList = nil; GroupID : TStringList = nil) : TList<TbfaSearch>;
begin
//  Result := TList<TbfaSearch>(inherited ReadXMLFile(Path, Name, ID, UserID, GroupID));
end;
//
(*function TbfaSearchProcessing.FillSearchWithNode(FbfaApp : PbfaApplication; Fav : IXMLNode) : TbfaBaseSearch;
var
  SObj : TbfaSearch;
begin
  try
    SObj := TbfaSearch.Create(FbfaApp);

    Result := TbfaSearch(FillSearchObjectWithNode(SObj, Fav));

  except
    Result := nil;
  end;
end;
*)
//



end.
