unit bfaSocketConnection;

interface

uses IdTCPClient, IdSSLOpenSSL, Classes, XMLIntf, bfaCommonUtils, bfaObjects36,
     Windows, SysUtils, bfaBaseObjects, Generics.Collections, bfaBaseSocketConnection,
     bfaParser, bfaLog, SplashForm;


type
  /// <Beschreibung>bitfarm SocketConnection Klasse</Beschreibung>
  /// <Description>bitfarm SocketConnection class</Description>
	TSocketConnection = class(TBaseSocketConnection)
  private
    FReceivedData: Int64;
    FDataLength: Int64;
    procedure InitSplashForm(SplashForm: TfrmSplash; datalen: Int64);
  published
    function ReceiveResponse(AThread : TThread = nil; ignore_error: Boolean = False): TbfaRequest; overload;
    function ReceiveHistory(hist : TList<TbfaHistory>): TbfaRequest;
    function ReceiveFtxPos(FtxPos: TList<TbfaFtxPos>): TbfaRequest;
//26.05.2023    function ReceiveData(Doc: TbfaDocument; AThread : TThread = nil; ShowErrorMsg: Boolean = True): TbfaRequest;
    function ReceiveData(Doc: TbfaDocument; AThread : TThread = nil; ShowErrorMsg: Boolean = True; OriFile: Boolean = False): TbfaRequest;
    function ReceiveDataToFile(Doc: TbfaDocument; filename: string; AThread : TThread = nil): TbfaRequest;
    procedure SendRequestToServer(req: TbfaRequest);  override;
    function SendDataToServer(req: TbfaRequest; Stream: TStream; AThread : TThread = nil; SplashForm: TfrmSplash = nil): TbfaRequest;
    function ReceiveDoc(Doc: TbfaDocument; AThread : TThread = nil;
      OnlyDoc : Boolean = False; ShowErrorMsg: Boolean = True): TbfaRequest;
    function WriteStreamViaSocket(Stream: TStream; AThread: TThread = nil; SplashForm: TfrmSplash = nil): Boolean; overload;
    function ReceiveStream(Doc: TbfaDocument;  FileName: string; datalen: Int64;
      AThread: TThread; SplashForm: TfrmSplash): Boolean; //07.06.2023
  public
    /// <Beschreibung></Beschreibung>
    /// <Description></Description>
  	constructor Create(ConnectionType: byte; SHost: string; SPort: integer;
    								   CertificateFile: string); overload;
//    procedure SetSid(value: string);
    property ReceivedData: Int64 read FReceivedData; // SJ Abschlussprojekt
    property DataLength: Int64 read FDataLength; // SJ Abschlussprojekt

  end;

implementation

uses
  bfaApplication, gnugettext, MainDataModule, bfaCommonDialogs;

constructor TSocketConnection.Create(ConnectionType: byte; SHost: string;
																		 SPort: integer; CertificateFile: string);
begin
	inherited Create;

  FConnectionType := ConnectionType;
  FServerHost := SHost;
  FServerPort := SPort;
  FCertFile := CertificateFile;
  FServerReadTimeout := 60000; // 60 sec Timeout
  FConnected := False;
  InitializeCriticalSection(FReadSocket_CS);
  FSid := bitfarmApp.Sid;
  FAutoreconnect := bitfarmApp.Autoreconnect;
end;

function TSocketConnection.ReceiveData(Doc: TbfaDocument; AThread : TThread = nil;
//26.05.2023   ShowErrorMsg: Boolean = True): TbfaRequest;
  ShowErrorMsg: Boolean = True; OriFile: Boolean = False): TbfaRequest;
var
  i, j: integer;
  s, xml: string;
  req: TbfaRequest;
  dataLen: Int64;
  lst: TList<TbfaBaseObject>;
  UserAction: PUserAction;
  TmpStream: TMemoryStream;
  TmpOriStream: TFileStream;
  TempDir: string;

  BytesCopied, AllBytesCopied: Int64;
  fs: TMemoryStream;
  ChunkSize: Integer;
begin
  bitfarmLog.Debug(llDebug, 'SocketConnection - ReceiveData', Classname);
  Result := nil;
  dataLen := 0;
  Bfr.Clear;
  if Doc.DocStream <> nil then
    Doc.DocStream.Clear;

  try
    xml := ReceiveXML(AThread);
  finally
    if xml <> '' then
    begin
      try
        lst := XMLStringToObject(xml);
        if lst <> nil then
        begin
          if lst.Count > 0 then
          begin
            if lst.Count > 1 then
            begin
              req := TbfaRequest(lst[1]);
              Result := req;
              try
                if Doc = nil then
                  Result := nil
                else
                  Doc.Clone(TbfaDocument(lst[0]), False, True, False);
                try
                  if (lst[0] <> nil) and (TbfaDocument(lst[0]).ID > 0) then
                    TbfaDocument(lst[0]).Free;  ///10.05.2023 sollte nicht mit Synchronize???
                except
                  //17.01.2019
                end;
              finally
                lst[0] := nil;
              end;
              dataLen := req.DataLen;
            end else
            begin
              Req := TbfaRequest(lst[0]);
              Result := req;
              dataLen := req.DataLen;
            end;
            lst.Clear;
          end
          else
            Result := nil;
          FreeAndNil(lst);
        end else begin
          Result := nil;
          bitfarmLog.Error(llDebug, 'Fehler beim Parsen der XML (1)', Classname);
        end;
      except
        Result := nil;
        bitfarmLog.Error(llDebug, 'Fehler beim Parsen der XML (2)', Classname);
      end;
    end else
    begin
      bitfarmLog.Error(llDebug, 'SocketConnection - ReceiveData keine Antwort vom Server erhalten', Classname);
      if ShowErrorMsg then
        BfShowMessage(_('Es hat einen Fehler bei der Verbindung mit dem Server gegeben. Bitte melden Sie sich erneut an.'), AThread);
    end;
  end;


  if Result = nil then
  begin
    if ShowErrorMsg then
      BfShowMessage(_('Es hat einen Fehler bei der Verbindung mit dem Server gegeben.'), AThread);
    Exit;
  end;

  if (Result.ReqType = 'respDenied') then
  begin
    if ShowErrorMsg then
      BfShowMessage(_('Keine Berechtigung auf das Dokument.') + #13#10 + Result.DataString, AThread);
    Exit;
  end;

  if (Result.ReqType = 'respDocError') or (Result.ReqType = 'respError') or
     ((Result.ReqType <> 'respOK') and ((Result.ReqType <> 'respDoc')))
  then
    Exit;

  if not OriFile then
  begin
    EnterCriticalSection(FReadSocket_CS);
    TmpStream := TMemoryStream.Create;
    try
      try
        TmpStream.Clear;
        TCPComp.Socket.ReadStream(TmpStream, dataLen, false);
        TmpStream.Position := 0;
        if Doc <> nil then
        begin
          Doc.DocStream.Clear;
          Doc.DocStream := TmpStream;
        end;
      except
        bitfarmLog.Error(llDebug, 'SocketConnection - Fehler beim Streamlesen', Classname);
        if ShowErrorMsg then
          BfShowMessage(_('Das Dokument konnte nicht geladen werden.'), AThread);
      end;
    finally
      if Doc.DocStream <> TmpStream then   //22.11.2018 wenn fr das Document weitergegeben ist, darf man hier nicht leer setzen
        FreeAndNil(TmpStream);
      LeaveCriticalSection(FReadSocket_CS);
    end;
  end;
end;

function TSocketConnection.ReceiveDataToFile(Doc: TbfaDocument; filename: string; AThread : TThread = nil): TbfaRequest;
var
  i, j, blocksize: integer;
  dataLen, recv: Int64;
  s, xml: string;
  req: TbfaRequest;
  lst: TList<TbfaBaseObject>;
  UserAction: PUserAction;
  myFile: TFileStream;
  buffer: PByte;
begin
  bitfarmLog.Debug(llDebug, 'SocketConnection - ReceiveData', Classname);
  Result := nil;
  blocksize := 524288;
  dataLen:= 0;
  recv := 0;
  GetMem(buffer, blocksize);
  Bfr.Clear;

  try
    xml := ReceiveXML(AThread);
  finally
    if xml <> '' then
    begin
      try
        lst := XMLStringToObject(xml);
        if lst <> nil then
        begin
          if lst.Count > 0 then
          begin
            if lst.Count > 1 then
            begin
              req := TbfaRequest(lst[1]);
              Result := req;
              if Doc = nil then
                Result := nil
              else
                Doc.Clone(TbfaDocument(lst[0]));
              TbfaDocument(lst[0]).Free;
              lst[0] := nil; //23.01.2018 wegen CrashLog
              dataLen := req.DataLen;
              FDataLength := req.DataLen; // SJ Abschlussprojekt
            end else
            begin
              Req := TbfaRequest(lst[0]);
              Result := req;
              dataLen := req.DataLen;
              FDataLength := req.DataLen; // SJ Abschlussprojekt
            end;
            lst.Clear;
          end
          else
            Result := nil;
          FreeAndNil(lst);
        end else begin
          Result := nil;
          bitfarmLog.Error(llDebug, 'Fehler beim Parsen der XML (1)', Classname);
        end;
      except
        Result := nil;
        bitfarmLog.Error(llDebug, 'Fehler beim Parsen der XML (2)', Classname);
      end;
    end else
    begin
      bitfarmLog.Error(llDebug, 'SocketConnection - ReceiveData keine Antwort vom Server erhalten', Classname);
      BfShowMessage(_('Es hat einen Fehler bei der Verbindung mit dem Server gegeben. Bitte melden Sie sich erneut an.'), AThread);
    end;
  end;

  if Result = nil then
  begin
    BfShowMessage(_('Es hat einen Fehler bei der Verbindung mit dem Server gegeben.'), AThread);
    Exit;
  end;

  if (Result.ReqType = 'respDenied') then
  begin
    BfShowMessage(_('Keine Berechtigung auf die Datei.') + #13#10 + Result.DataString, AThread);
    Exit;
  end;

  if (Result.ReqType = 'respDocError') or (Result.ReqType = 'respError')
      or ((Result.ReqType <> 'respOK') and ((Result.ReqType <> 'respDoc'))) then
  begin
    if not bitfarmApp.DoDropping then
    begin
      BfShowMessage(_('Das Dokument konnte nicht geladen werden.') + #13#10 + Result.DataString, AThread);
      if (Result.ReqType = 'respAuthError') then
        BfShowMessage(_('Bitte melden Sie sich neu an (Datei - Neu anmelden).'), AThread);
    end;

    Exit;
  end;
  try
    myFile := TFileStream.Create(filename, fmOpenWrite or fmCreate);
  except
    //
  end;

  Bfr.Position:= 0;
  try
    try
      while recv < dataLen do
      begin
        if dataLen - recv < blocksize then
          blocksize := dataLen - recv;

        TCPComp.Socket.ReadStream(Bfr,blocksize,False);
        Bfr.Position := 0;

        Bfr.ReadBuffer(buffer^,blocksize);
        myFile.WriteBuffer(buffer^ , blocksize);

        recv := recv + blocksize;
        FReceivedData := recv;
        Bfr.Clear;
      end;
    except
      //
    end;
  finally
    FreeMem(buffer);
    myFile.Free;
  end;
end;


function TSocketConnection.ReceiveDoc(Doc: TbfaDocument; AThread : TThread = nil;
  OnlyDoc : Boolean = False; //13.03.2019 Ticket 18518 wenn nur gespeichert wir und bleibt stehen, dann muss nicht alles neugeladen werden
  ShowErrorMsg: Boolean = True): TbfaRequest;
var
  i, j: integer;
  s, xml: string;
  req: TbfaRequest;
  dataLen: Int64;
  lst: TList<TbfaBaseObject>;
  UserAction: PUserAction;
begin
  bitfarmLog.Debug(llDebug, 'SocketConnection - ReceiveDoc', Classname);
  Result := nil;

  try
    xml := ReceiveXML(AThread);
  finally
    if xml <> '' then
    begin
      lst := XMLStringToObject(xml);

      if lst.Count > 1 then
      begin
        Result := TbfaRequest(lst[1]);
        Doc.Clone(TbfaDocument(lst[0]), False, True, True, True, OnlyDoc);
      end else
        Result := TbfaRequest(lst[0]);
    end else
    begin
      bitfarmLog.Error(llDebug, 'SocketConnection - ReceiveDoc keine Antwort vom Server erhalten', Classname);
      Sleep(25);
      if ShowErrorMsg then
        BfShowMessage(_('Es hat einen Fehler bei der Verbindung mit dem Server gegeben. Bitte melden Sie sich erneut an.'), AThread);
    end;
  end;
end;

function TSocketConnection.ReceiveResponse(AThread : TThread = nil; ignore_error: Boolean = False): TbfaRequest;
var
  xml: String;
  lst: TList<TbfaBaseObject>;
  UserAction: PUserAction;
  i: Integer;
begin
  bitfarmLog.Debug(llDebug, 'SocketConnection - ReceiveResponse', Classname);
  Result := nil;
  try
    xml := ReceiveXML(AThread);
  finally
    if xml = '' then
      Result := nil
    else
    begin
      lst := XMLStringToObject(xml);
      try
        for i := 0 to lst.Count - 1 do
        begin
          if lst[i] is TbfaRequest then
          begin
            Result := TbfaRequest(lst[i]);
            Result.Data := TList<TbfaBaseObject>.Create;
          end;
        end;

        if Result <> nil then
        begin
          for i := 0 to lst.Count - 1 do
          begin
            if lst[i] is TbfaAddfield then
              Result.Data.Add(TbfaAddfield(lst[i]));
            if lst[i] is TbfaStateField then
              Result.Data.Add(TbfaStateField(lst[i]));
            if lst[i] is TbfaWorkflow then
              Result.Data.Add(TbfaWorkflow(lst[i]));
            if lst[i] is TbfaSearch then
              Result.Data.Add(TbfaSearch(lst[i]));
            if lst[i] is TbfaADEntry then
              Result.Data.Add(TbfaADEntry(lst[i]));
          end;
        end;
      finally
        FreeAndNil(lst);
      end;
    end;

    if not ignore_error and (Result = nil) then
    begin
      bitfarmLog.Error(llDebug, 'SocketConnection - ReceiveResponse keine Antwort vom Server erhalten', Classname);
      UserAction := bitfarmApp.CreateUserAction;
      UserAction^.ActionName := 'actRelogin';
      bitfarmApp.MainThread.Add(UserAction);
    end;
	end;
end;

function TSocketConnection.SendDataToServer(req: TbfaRequest; Stream: TStream;
  AThread : TThread = nil; SplashForm: TfrmSplash = nil): TbfaRequest;
var
  xml: string;
  lst: TList<TbfaBaseObject>;
  i, idx: Integer;
begin
  Result := nil;
  lst := TList<TbfaBaseObject>.Create;
  idx := 0;
  try
    Self.DestroyComponents;
    Self.CreateComponents;
    Self.Connect;

    req.Sessionid := FSid;
    lst.Add(req);
    xml := ObjectToXML(lst);
    try
      if Assigned(bitfarmLog) then
        bitfarmLog.Write(llDebug, ltInfo, 'SendDataToServer: ' + xml, ClassName);

      WriteStringsViaSocket(xml, Length(xml));

      xml := ReceiveXML(AThread);
      if xml <> '' then
      begin
        lst := XMLStringToObject(xml);
        if TbfaRequest(lst[0]).ReqType = 'respSendFile' then
        begin
          WriteStreamViaSocket(Stream, Athread, SplashForm);
          if (SplashForm <> nil) and SplashForm.Canceled then
            Exit;
          xml := ReceiveXML(AThread);
          lst.Clear;
          lst := XMLStringToObject(xml);
        end;
        if lst.Count = 1 then
          Result := TbfaRequest(lst[0])
        else begin
          for i := 0 to lst.Count -1 do
            if lst[i] is TbfaRequest then
              idx := i;
          Result := TbfaRequest(lst[idx]);
          Result.Data := TList<TbfaBaseObject>.Create;
          for i := 0 to lst.Count -1 do
          begin
            if i = idx then
              Continue;
            Result.Data.Add(lst[i])
          end;
        end;

        if Assigned(bitfarmLog) then
          bitfarmLog.Write(llDebug, ltInfo, 'SendDataToServer: ' + xml, ClassName);
      end;
    except
      //
    end;
  finally
    FreeAndNil(lst);
  end;
end;

procedure TSocketConnection.SendRequestToServer(req: TbfaRequest);
var
  xmlstr: string;
  lst: TList<TbfaBaseObject>;
begin
  lst := TList<TbfaBaseObject>.Create;
  try
    CheckAutoreconnect;

    req.Sessionid := FSid;
    lst.Add(req);
    xmlstr := ObjectToXML(lst);

    try
      if Assigned(bitfarmLog) and (req.reqType <> 'reqAuthenticate') and
         (req.ReqType <> 'reqADUser') and (req.ReqType <> 'reqADTree')
      then
        bitfarmLog.Write(llDebug, ltInfo, 'SendRequestToServer: ' + xmlstr, ClassName);
      WriteStringsViaSocket(xmlstr, Length(xmlstr));
    except
      if Assigned(bitfarmLog) then
        bitfarmLog.Write(llNormal, ltException, 'SendRequestToServer', ClassName);
    end;
  finally
    FreeAndNil(lst);
  end;
end;

//procedure TSocketConnection.SetSid(value: string);
//begin
//  FSid := value;
//end;

function TSocketConnection.ReceiveHistory(hist : TList<TbfaHistory>): TbfaRequest;
var
  xml: String;
  lst: TList<TbfaBaseObject>;
  UserAction: PUserAction;
  i: Integer;
begin
  bitfarmLog.Debug(llDebug, 'SocketConnection - ReceiveHistory', Classname);
  Result := nil;
  try
    xml := ReceiveXML;
  finally
    if xml = '' then
      Result := nil
    else
    begin
      lst := XMLStringToObject(xml);

      if lst.Count > 1 then
      begin
        for i := 0 to lst.count - 2 do
          hist.Add(TbfaHistory(lst[i]));
        Result := TbfaRequest(lst[lst.count - 1]);
      end else
        Result := TbfaRequest(lst[0]);
    end;

    if Result = nil then
    begin
      bitfarmLog.Error(llDebug, 'SocketConnection - ReceiveHistory keine Antwort vom Server erhalten', Classname);
      BfShowMessage(_('Es hat einen Fehler bei der Verbindung mit dem Server gegeben. Bitte melden Sie sich erneut an.'));
      UserAction := bitfarmApp.CreateUserAction;
      UserAction^.ActionName := 'actRelogin';
      bitfarmApp.MainThread.Add(UserAction);
    end;
	end;
end;

function TSocketConnection.ReceiveFtxPos(FtxPos: TList<TbfaFtxPos>): TbfaRequest;
var
  xml: String;
  lst: TList<TbfaBaseObject>;
  UserAction: PUserAction;
  i: Integer;
begin
  bitfarmLog.Debug(llDebug, 'SocketConnection - ReceiveFtxPos', Classname);
  Result := nil;
  try
    xml := ReceiveXML;
  finally
    if xml = '' then
      Result := nil
    else
    begin
      lst := XMLStringToObject(xml);

      if lst.Count > 1 then
      begin
        for i := 0 to lst.count - 2 do
          FtxPos.Add(TbfaFtxPos(lst[i]));
        Result := TbfaRequest(lst[lst.count - 1]);
      end else
        Result := TbfaRequest(lst[0]);
    end;

    if Result = nil then
    begin
      bitfarmLog.Error(llDebug, 'SocketConnection - ReceiveResponse keine Antwort vom Server erhalten', Classname);
      BfShowMessage(_('Es hat einen Fehler bei der Verbindung mit dem Server gegeben. Bitte melden Sie sich erneut an.'));
      UserAction := bitfarmApp.CreateUserAction;
      UserAction^.ActionName := 'actRelogin';
      bitfarmApp.MainThread.Add(UserAction);
    end;
	end;
end;

//05.06.2023 -->
function TSocketConnection.WriteStreamViaSocket(Stream: TStream; AThread : TThread = nil;
  SplashForm: TfrmSplash = nil): Boolean;
var
  BytesCopied, AllBytesCopied: Int64;
  fs: TMemoryStream;
  ChunkSize: Integer;
begin
  if Assigned(bitfarmLog) then
    bitfarmLog.Debug(llDebug, 'SocketConnection - WriteStreamViaSocket', Classname);

  fs := TMemoryStream.Create;
  BytesCopied := 0;
  AllBytesCopied := 0;
  try
    Stream.Position := 0;
    ChunkSize := 524288;
    AThread.Synchronize(nil,
      procedure
      begin
        if SplashForm <> nil then
        begin
          try
            SplashForm.pgbBytes.Properties.Min := 0;
            SplashForm.pgbBytes.Properties.Max := Stream.Size;
            SplashForm.pgbBytes.Position := 0;
            SplashForm.pgbBytes.Visible := True;
            SplashForm.btnCancel.Enabled := True;
            SplashForm.btnCancel.Visible := True;
            SplashForm.Update;
          except
            //07.07.2023
          end;
        end;
      end);
    repeat
      if ChunkSize > (Stream.Size - Stream.Position) then
        ChunkSize := (Stream.Size - Stream.Position);
      fs.CopyFrom(Stream, ChunkSize);
      TCPComp.Socket.Write(fs);
      AllBytesCopied := AllBytesCopied + ChunkSize;
      fs.Clear;
      AThread.Synchronize(nil,
        procedure
        begin
          if SplashForm <> nil then
          begin
            try
              SplashForm.pgbBytes.Position := AllBytesCopied;
              SplashForm.pgbBytes.Update;
            except
              //07.07.2023
            end;
          end;
        end);
    until (AllBytesCopied >= Stream.Size) or ((SplashForm <> nil) and SplashForm.Canceled); { until we run out of chunks }
  finally
    //14.11.2023 -->
    AThread.Synchronize(nil,
      procedure
      begin
        if SplashForm <> nil then
        begin
          try
            SplashForm.pgbBytes.Visible := False;
            SplashForm.Update;
          except
            //07.07.2023
          end;
        end;
      end);
    //14.11.2023 <--

    FreeAndNil(fs);
  end;
end;

procedure TSocketConnection.InitSplashForm(SplashForm: TfrmSplash; datalen: Int64);
begin
  try
    SplashForm.lblLoading.Caption := _('Originaldatei wird geladen...');
    SplashForm.pgbBytes.Properties.Min := 0;
    SplashForm.pgbBytes.Properties.Max := datalen;
    SplashForm.pgbBytes.Position := 0;
    SplashForm.pgbBytes.Visible := True;
    SplashForm.btnCancel.Enabled := True;
    SplashForm.Canceled := False;
    SplashForm.btnCancel.Visible := True;
    SplashForm.Update;
    if not SplashForm.Showing then
     SplashForm.Show;
  except
    //07.07.2023
  end;
end;

function TSocketConnection.ReceiveStream(Doc: TbfaDocument; FileName: string;
  datalen: Int64; AThread : TThread; SplashForm: TfrmSplash): Boolean;
var
  BytesCopied, AllBytesCopied: Int64;
  fs: TMemoryStream;
  ChunkSize: Integer;
begin
  EnterCriticalSection(FReadSocket_CS);
  try
    try
      if Doc <> nil then
      begin
//27.07.2023        Doc.OriStream := TFileStream.Create(FileName, fmCreate);
        if not DirectoryExists(ExtractFilePath(FileName)) then
          ForceDirectories(ExtractFilePath(FileName));

        Doc.OriStream := TFileStream.Create(FileName, fmCreate or fmShareDenyNone);
        bitfarmLog.Info(llDebug, 'SocketConnection - ReceiveStream - FileName: ' + FileName, Classname);

        BytesCopied := 0;
        AllBytesCopied := 0;
        ChunkSize := 524288;
        if AThread <> nil then
        begin
          AThread.Synchronize(nil,
            procedure
            begin
              InitSplashForm(SplashForm, datalen);
            end);
        end
        else
          InitSplashForm(SplashForm, datalen);
        if ChunkSize > datalen then
          ChunkSize := datalen;
        repeat
          if ChunkSize > datalen - AllBytesCopied then
            ChunkSize := datalen - AllBytesCopied;
          TCPComp.Socket.ReadStream(Doc.OriStream, ChunkSize, false);
          AllBytesCopied := AllBytesCopied + ChunkSize;
          if AThread <> nil then
          begin
            AThread.Synchronize(nil,
              procedure
              begin
                SplashForm.pgbBytes.Position := AllBytesCopied;
                SplashForm.pgbBytes.Update;
              end);
          end
          else
          begin
            SplashForm.pgbBytes.Position := AllBytesCopied;
            SplashForm.pgbBytes.Update;
          end;
        until (AllBytesCopied >= datalen) or SplashForm.Canceled;
        if AThread <> nil then
        begin
          AThread.Synchronize(nil,
            procedure
            begin
              SplashForm.pgbBytes.Visible := False;
              SplashForm.Hide;
            end);
        end
        else
        begin
          SplashForm.pgbBytes.Visible := False;
          SplashForm.Hide;
        end;

        if SplashForm.Canceled then
        begin
          try
            bitfarmLog.Info(llDebug, 'SocketConnection - ReceiveStream CANCELED - FileName: ' + FileName, Classname);
            Doc.OriStream.Free;
            if FileExists(FileName) then
              DeleteFile(FileName);
          finally
            Doc.OriStream := nil;
          end;
        end
        else
          bitfarmLog.Info(llDebug, 'SocketConnection - ReceiveStream RECEIVED - FileName: ' + FileName, Classname);
      end;
    except
      bitfarmLog.Error(llDebug, 'SocketConnection - Fehler beim Streamlesen', Classname);
      BfShowMessage(_('Das Dokument konnte nicht geladen werden.'), AThread);
    end;
  finally
    LeaveCriticalSection(FReadSocket_CS);
  end;
end;

//05.06.2023 <--


end.
