{ Hauptansicht, wenn Updater mit Parameter (Profil) gestartet wird }
unit MainForm;

interface

uses
  Winapi.Windows, Winapi.Messages, Winapi.ShellApi,
  System.SysUtils, System.Classes, System.IOUtils, IniFiles, gnugettext, SynZip,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.ComCtrls, Vcl.ExtCtrls, Vcl.Graphics,

  bfaBaseObjects, bfaLog, bfaCommonUtils, bfaCommonFiles, bfaCommonMethods,
  bfaBaseSocketConnection, bfaObjects36, ChooseProfile;

type
  { Working Thread - required for not freezing GUI }
  MyThread = class(TThread)
  private
    /// <summary>
    /// Sollte eigentlich nur bei Einstieg via updater.ini gesetzt sein
    /// </summary>
    RetrySslConf: Boolean;
  protected
    procedure Execute; override;
  end;

  { Die Haupt-Form des Updaters }
  TUpdateForm = class(TForm)
    { Fortschrittsbalken }
    ProgressBar1: TProgressBar;
    { @exclude }
    Label1: TLabel;
    { Abbrechen }
    btnCancel: TButton;
    { Status }
    lblStatus: TLabel;
    grpbx_Connection: TGroupBox;
    lblServer: TLabel;
    lblPort: TLabel;
    chkSSL: TCheckBox;
    edtServer: TEdit;
    edtPort: TEdit;
    btnConnect: TButton;
    edtProfile: TEdit;
    lblProfile: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    { MainThread will be started here }
    procedure FormShow(Sender: TObject);
    { Create new Instance of Thread and start it }
    procedure ThreadStart(Sender: TObject; Retry: Boolean = False);
    { terminate Thread, clean up and terminate application }
    procedure CloseApplication(Sender: TObject);
    procedure btnConnectClick(Sender: TObject);
    procedure KeyPressReturnClick(Sender: TObject; var Key: Word; Shift: TShiftState);
  protected
    { Application always in taskbar }
    procedure CreateParams(var Params: TCreateParams); override;
  private
    { Private-Deklarationen }

      (* ENV-Variablen *)
      Con, ViewerIni: TIniFile;
      MainSocket: TBaseSocketConnection;
      bfaServer: String;
      bfaServerPort: Integer;
      fProfile: String;
      ChooseProfile: TProfileForm;
      bfaServerNoSSL: Integer; // sctpSSL = 0, sctpNoSSL = 1

      (* Programm-Variablen *)
      fHotSeachRunning, fViewerRunning: Boolean;
      fFileList, fRequestFileList, fRestartApplicationList: TStringList;
//      fServerRequest: TbfaRequest;
      fUpdateStatus: TbfaUpdateStatus;

//      tmrStartThread, tmrCloseForm, tmrRefreshBar: TTimer;
      tmrCloseForm, tmrRefreshBar: TTimer;
      MainThread: MyThread;
      req: TbfaRequest;
      { Binary-File from ReceiveDataToFile }
      Doc: TbfaDocument;
      { call before connect to make hostname changes in .con files possible }
      procedure MoveConFiles;
      procedure ShowConnectionSettings(Show: Boolean);
      { no use until now }
      { @exclude }
//      function CountFilesInFolder (path: string ): integer;
  public
    { Public-Deklarationen }
    { Setzt Fortschrittsbalken }
    procedure RefreshBar(Sender: TObject);
    { Startet zu Applikationsstart laufende Anwendungen neu. }
    function StartApplication(Filename: string; AsExplorerUser: Boolean = False): Boolean;
    { auf dem Server zum Update konfigurierte Dateien. }
    property FileList: TStringList read fFileList write fFileList;
    { Dateien die angefordert werden. }
    property RequestFileList: TStringList read fRequestFileList write fRequestFileList;
    property RestartApplicationList: TStringList read fRestartApplicationList write fRestartApplicationList;
    { zum bestimmen ob Update fehlgeschlagen ist. }
    property UpdateStatus: TbfaUpdateStatus read fUpdateStatus write fUpdateStatus;
    { HotSearch muss beendet werden, falls es aktuellisiert werden muss. Und danach neu gestartet werden. }
    property HotSeachRunning: Boolean read fHotSeachRunning;
    { Viewer muss @bold(immer) beendet werden. Und neu gestartet werden, sofern er lief. }
    property ViewerRunning: Boolean read fViewerRunning;
    { Anhand des Profils identifizieren wir den Server gegen den wir aktualisieren. }
    property Profile: String read fProfile write fProfile;

  end;

const
  WM_CLOSETIMER = WM_APP + 4;

var
  { @exclude }
  UpdateForm: TUpdateForm;

implementation

{$R *.dfm}

uses
  ShellExecuteUnelevated;

procedure TUpdateForm.btnCancelClick(Sender: TObject);
begin
  try
    bitfarmLog.Write(llDebug, ltInfo, 'btnCancelClick', 'bfaUpdater');
    if btnCancel.Caption = _('Ok') then
    begin
      ViewerIni.WriteBool('General', 'UpdateFailed', False);
      CloseApplication(Sender);
      Exit;
    end;

    if tmrCloseForm.Enabled = True then
      CloseApplication(Sender);

    case MessageDlg(_('Mchten Sie den Vorgang wirklich abbrechen?'), mtWarning, mbOkCancel, 2) of
      mrOk:
        begin
          bitfarmLog.Write(llDebug, ltInfo, 'Application.Terminate', 'bfaUpdater');
          CloseApplication(Sender);
        end;
    end;
  except on E: Exception do
    bitfarmLog.Write(llDebug, ltException, 'btnCancelClick: ' + E.Message, ClassName);
  end;
end;

procedure TUpdateForm.btnConnectClick(Sender: TObject);
begin
  Self.bfaServer := Self.edtServer.Text;
  Self.Profile := Self.edtProfile.Text;
  if Self.chkSSL.Checked then
    Self.bfaServerNoSSL := 0
  else
    Self.bfaServerNoSSL := 1;
  try
    Self.bfaServerPort := StrToInt(Self.edtPort.Text);
    ThreadStart(Sender);
  except
    MessageDlg(_('Ungltiger Serverport'), mtError, [mbOk], 0);
  end;
end;

procedure TUpdateForm.CloseApplication(Sender: TObject);
var
  RestartAppication: String;
  RestartRslt: Boolean;
begin
  try
    tmrCloseForm.Enabled := False;
    if (UpdateForm.RestartApplicationList <> nil) and (UpdateForm.RestartApplicationList.Count > 0) then
      for RestartAppication in UpdateForm.RestartApplicationList do
      begin
        StartApplication(RestartAppication, True);
        bitfarmLog.Write(llDebug, ltDebug, 'Start Application: ' + RestartAppication{ + ' - ' + BoolToStr(RestartRslt, True)}, ClassName);
      end;

    if (UpdateStatus <> bfaObjects36.TbfaUpdateStatus.eUpdateError) then
      ViewerIni.WriteBool('General', 'UpdateFailed', False);
    if (UpdateStatus = bfaObjects36.TbfaUpdateStatus.eUpdateError) then
      ExitCode := 1
    else
      ExitCode := 0;

    Application.Terminate;
  except on E: Exception do
    bitfarmLog.Write(llDebug, ltException, 'CloseApplication: ' + E.Message, ClassName);
  end;
end;

//function TUpdateForm.CountFilesInFolder(path: string): Integer;
//var
//  search: TSearchRec;
//  Subdir: TStringList;
//begin
//  try
//    path := IncludeTrailingPathDelimiter(ExtractFilePath(path));
//    Subdir := TStringList.Create;
//    Result := 0;
//
//    if FindFirst(path + '*.*', faDirectory, search) = 0 then
//    begin
//      repeat
//        Subdir.Add(search.Name);
//      until FindNext(search) <> 0;
//      FindClose(search);
//    end;
//
//    if FindFirst(path + '*.*', faAnyFile and not faDirectory, search) = 0 then
//    begin
//      repeat
//        inc(Result);
//      until FindNext(search) <> 0;
//      FindClose(search);
//    end;
//  except on E: Exception do
//    bitfarmLog.Write(llDebug, ltException, 'CountFilesInFolder: ' + E.Message, ClassName);
//  end;
//end;

procedure TUpdateForm.MoveConFiles;
var
  ConDatei: TSearchRec;
begin
  try
    if FindFirst(BD.AppdataBitfarmPd + 'Updates\*.con', faAnyFile, ConDatei) = 0 then
    begin
      repeat
        MoveFile(PChar(BD.AppdataBitfarmPd + 'Updates\' + ConDatei.Name), PChar(BD.ProgramFilesBitfarmClientPd + ConDatei.Name));
        if GetLastError = 183 then
        begin
          bitfarmLog.Write(llDebug, ltInfo, 'Dateiname bereits vorhanden: '+ ConDatei.Name, ClassName);
          if FileExists(BD.ProgramFilesBitfarmClientPd + ConDatei.Name + '.bak') then
          begin
            DeleteFile(BD.ProgramFilesBitfarmClientPd + ConDatei.Name + '.bak');
            bitfarmLog.Write(llDebug, ltInfo, 'Datei wird gelscht: '+ ConDatei.Name +'.bak', ClassName);
          end;
          bitfarmLog.Write(llDebug, ltInfo, 'Benenne Datei um: "'+ ConDatei.Name +'" > "'+ ConDatei.Name +'.bak"', ClassName);
          RenameFile(BD.ProgramFilesBitfarmClientPd + ConDatei.Name, BD.ProgramFilesBitfarmClientPd + ConDatei.Name + '.bak');
          MoveFile(PChar(BD.AppdataBitfarmPd + 'Updates\' + ConDatei.Name), PChar(BD.ProgramFilesBitfarmClientPd + ConDatei.Name));
        end else if GetLastError <> 0 then
          bitfarmLog.Write(llDebug, ltException, 'Returncode: '+ GetLastError.ToString +' - '+ SysErrorMessage(GetLastError), ClassName)
        else
          bitfarmLog.Write(llDebug, ltInfo, 'bertragung erfolgreich: '+ ConDatei.Name, ClassName)
      until FindNext(ConDatei) <> 0;
      FindClose(ConDatei);
    end;
  except on E: Exception do
    bitfarmLog.Write(llDebug, ltException, 'MoveConFiles: ' + E.Message, ClassName);
  end;
end;

procedure TUpdateForm.RefreshBar(Sender: TObject);
var
  percent: Integer;
begin
  try
    if (MainSocket.DataLength * MainSocket.ReceivedData) > 0 then
    begin
      percent := Round(100 / MainSocket.DataLength * MainSocket.ReceivedData);
      ProgressBar1.Position := Round(percent * 0.9);
      lblStatus.Caption := _('Download:') + ' ' + percent.ToString + '%';
    end
    else
      ProgressBar1.Position := 0;
  except on E: Exception do
    bitfarmLog.Write(llDebug, ltException, 'RefreshBar: ' + E.Message, 'bfaUpdater');
  end;
end;

procedure TUpdateForm.ShowConnectionSettings(Show: Boolean);
begin
  for var Idx := 0 to UpdateForm.grpbx_Connection.ControlCount - 1 do
  begin
    UpdateForm.grpbx_Connection.Controls[Idx].Enabled := Show;
  end;
  UpdateForm.grpbx_Connection.Enabled := Show;

  if Show then
  begin
    UpdateForm.edtServer.Text := Self.bfaServer;
    UpdateForm.edtProfile.Text := Self.Profile;
    UpdateForm.edtPort.Text := Self.bfaServerPort.ToString;
    UpdateForm.chkSSL.Checked := not(Self.bfaServerNoSSL).ToBoolean; // sctpSSL = 0, sctpNoSSL = 1
    UpdateForm.ClientHeight := MulDiv(263, UpdateForm.Monitor.PixelsPerInch, 96);
  end
  else
    UpdateForm.ClientHeight := MulDiv(102, UpdateForm.Monitor.PixelsPerInch, 96);
end;

function TUpdateForm.StartApplication(Filename: string; AsExplorerUser: Boolean = False): Boolean;
var
  ShellResult: Integer;
  ErrorMsg: string;
begin
  if not AsExplorerUser then
  begin
    if (FileExists(BD.ProgramFilesBitfarmClientPd + Filename)) then
      ShellResult := ShellExecute(0, nil, PChar(BD.ProgramFilesBitfarmClientPd + Filename), nil, nil, SW_SHOWNORMAL)
    else if (FileExists(BD.ExeDir + Filename)) then
      ShellResult := ShellExecute(0, nil, PChar(BD.ExeDir + Filename), nil, nil, SW_SHOWNORMAL)
    else
      ShellResult := ERROR_FILE_NOT_FOUND;
  end else
  begin
    if (FileExists(BD.ProgramFilesBitfarmClientPd + Filename)) then
    begin
      ShellExecuteFromExplorer(BD.ProgramFilesBitfarmClientPd + Filename);
      Result := True;
      Exit;
    end else if (FileExists(BD.ExeDir + Filename)) then
    begin
      ShellExecuteFromExplorer(BD.ExeDir + Filename);
      Result := True;
      Exit;
    end else
      ShellResult := ERROR_FILE_NOT_FOUND;
  end;

  ErrorMsg := ShellExecToString(ShellResult);

  if ShellResult > 32 then
  begin
    bitfarmLog.Write(llDebug, ltInfo, 'Starting Application: '+ Filename, 'bfaUpdater');
    Result := True;
  end
  else
  begin
    bitfarmLog.Write(llDebug, ltException, 'Problem beim Starten von: ' + Filename + ': ' + ErrorMsg, 'bfaUpdater');
    Result := False;
  end;
end;

procedure TUpdateForm.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.ExStyle := Params.ExStyle or WS_EX_APPWINDOW;
end;

procedure TUpdateForm.KeyPressReturnClick(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if Key = VK_RETURN then
    btnConnect.Click;
end;

procedure TUpdateForm.FormCreate(Sender: TObject);
begin
  try
    bitfarmLog.Init(BD.EnvTemp, '', 5, 'bfaUpdater', False);
    bitfarmLog.Write(llDebug, ltInfo, 'START', 'bfaUpdater');

    { Verzeichnisse erzeugen, falls nicht vorhanden }
    if (not DirectoryExists(BD.EnvTempUpdatePd)) then
      ForceDirectories(BD.EnvTempUpdatePd);
    if (not DirectoryExists(BD.AppdataBitfarmPd + 'Updates')) then
      ForceDirectories(BD.AppdataBitfarmPd + 'Updates');

    { Objekte fr den Request initialisieren }
    req := TbfaRequest.Create;
    Doc := TbfaDocument.Create;
    FileList := TStringList.Create('"', ';', [soStrictDelimiter]);
    RequestFileList := TStringList.Create('"', ';', [soStrictDelimiter]);

    tmrCloseForm := TTimer.Create(Application);
    tmrCloseForm.Interval := 4000;
    tmrCloseForm.OnTimer := CloseApplication;
    tmrCloseForm.Enabled := False;

    tmrRefreshBar := TTimer.Create(Application);
    tmrRefreshBar.Interval := 200;
    tmrRefreshBar.OnTimer := RefreshBar;
    tmrRefreshBar.Enabled := False;

    // Kein Profil bergeben? Auswahl anbieten
    if (ParamStr(1) = '') then
    begin
      bitfarmLog.Write(llDebug, ltInfo, 'kein Profil via Parameter - Modalfenster', ClassName);
      UpdateForm.lblStatus.Caption := _('Bitte Profil whlen...');

      UpdateForm.ChooseProfile := TProfileForm.Create(nil);
      case UpdateForm.ChooseProfile.ShowModal of
        mrCancel:
          begin
            UpdateForm.ShowConnectionSettings(False);
            bitfarmLog.Write(llDebug, ltInfo, 'Modal-Result: mrCancel - terminate', ClassName);
            UpdateForm.lblStatus.Caption := _('Kein Profil ausgewhlt. Updater wird beendet.');
            UpdateForm.tmrCloseForm.Enabled := True;
            Exit;
          end;
        mrOk:
          begin
            UpdateForm.Profile := UpdateForm.ChooseProfile.Profile;
            UpdateForm.lblStatus.Caption := _('Verbindung wird hergestellt...');
            bitfarmLog.Write(llDebug, ltInfo, 'Profil via Modal: '+ UpdateForm.Profile, ClassName);
          end;
      end;
    end else
    begin
      UpdateForm.Profile := ParamStr(1);
      bitfarmLog.Write(llDebug, ltInfo, 'Profil via Parameter: '+ UpdateForm.Profile, ClassName);
    end;

    { keine lokale Installation }
    if (not DirectoryExists(BD.ProgramFilesX86BitfarmClient)) then
    begin
      UpdateForm.ShowConnectionSettings(False);
      lblStatus.Caption := _('Keine lokale Installation gefunden.')+sLineBreak+_('Updater schliet in 5 Sekunden.');
      tmrCloseForm.Enabled := True;
    end else

    { lokale Installation }
    begin
      lblStatus.Caption := _('Download wird gestartet...');

      { auf Fehler setzen - bei Erfolg zurck setzen }
      ViewerIni := TIniFile.Create(BD.AppdataBitfarmPd + 'Viewer.ini');
      ViewerIni.WriteBool('General', 'UpdateFailed', True);

      MoveConFiles;

    end;
  except on E: Exception do
    bitfarmLog.Write(llDebug, ltException, 'FormCreate: ' + E.Message, 'bfaUpdater');
  end;
end;

procedure TUpdateForm.FormShow(Sender: TObject);
begin
  if (not tmrCloseForm.Enabled) then
    ThreadStart(Sender);
end;

procedure TUpdateForm.ThreadStart(Sender: TObject; Retry: Boolean);
begin
  MainThread := MyThread.Create(True);
  MainThread.FreeOnTerminate := True;
  MainThread.Resume;
  MainThread.RetrySslConf := Retry;
end;


{ MyThread }

procedure MyThread.Execute;
var
  i, rslt, pid1: Integer;
  Filename: String;
  ViewerExeName, HotSearchExeName: String;
  ViewerExe, HotSearchExe: PWideChar;
  handles: THandleArray;
  h: Cardinal;
  cmdline: Widestring;
  hd: HWND;

    function ConnectionString(noSSL: Integer): string;
    begin
      if noSSL = 1 then
       Result := _('Verbindung wird hergestellt...')
      else if noSSL = 0 then
       Result := _('SSL-Verbindung wird hergestellt...');
    end;
begin
  try
    inherited;
    UpdateForm.ShowConnectionSettings(False);

    // wenn bfaServer gesetzt ist, dann kommen wir aus dem neu verbinden Modus und lesen nichts aus
    if (UpdateForm.bfaServer = '') then
    begin
      // updater.ini laden
      if FileExists(Paramstr(1)) then
      begin
        UpdateForm.con := TIniFile.Create(Paramstr(1));
        if (UpdateForm.Profile <> '') then
        begin
          UpdateForm.bfaServer := UpdateForm.Con.ReadString(sSysServerSection, sSysBfaServer, 'localhost');
          UpdateForm.bfaServerPort := UpdateForm.Con.ReadInteger(sSysServerSection, sSysBfaServerPort, 50001);
          UpdateForm.Profile := UpdateForm.Con.ReadString('Updater', 'profile', '');
          bitfarmLog.Write(llHigh, ltInfo, 'Server: '+ UpdateForm.bfaServer +'::'+ UpdateForm.bfaServerPort.ToString, ClassName);
          Self.RetrySslConf := True;
        end;

      // .con laden
      end else if FileExists(BD.ProgramFilesBitfarmClientPd + UpdateForm.Profile +'.con') then
      begin
        if FileExists(BD.ExeDir + UpdateForm.Profile +'.con') then
          UpdateForm.con := TIniFile.Create(BD.ExeDir + UpdateForm.Profile +'.con')
        else
          UpdateForm.con := TIniFile.Create(BD.ProgramFilesBitfarmClientPd + UpdateForm.Profile +'.con');

        if (UpdateForm.Profile <> '') then
        begin
          UpdateForm.bfaServer := UpdateForm.con.ReadString(sSysServerSection, sSysBfaServer, 'localhost');
          UpdateForm.bfaServerPort := UpdateForm.con.ReadInteger(sSysServerSection, sSysBfaServerPort, 50001);
          UpdateForm.bfaServerNoSSL := ((LowerCase(UpdateForm.con.ReadString(sSysServerSection, sSysNoSSL, 'false')) = 'true') or (UpdateForm.con.ReadString(sSysServerSection, sSysNoSSL, 'false') = '1')).ToInteger;
          bitfarmLog.Write(llHigh, ltInfo, 'Server: '+ UpdateForm.bfaServer +'::'+ UpdateForm.bfaServerPort.ToString, ClassName);
        end;

      end else
      begin
        bitfarmLog.Write(llNormal, ltException, 'Kein Profil bergeben', ClassName);
        UpdateForm.lblStatus.Caption := _('Profil konnte nicht geladen werden.');
        UpdateForm.tmrCloseForm.Enabled := True;
        Exit;
      end;

      { die neu zu startenden Anwendungen }
      ViewerExeName := 'ViewerV3.exe';
      HotSearchExeName := 'HotSearch.exe';
      ViewerExe := 'bitfarm.Viewer.V36';
      HotSearchExe := 'bitfarm.hotsearch.V36';

      { ermitteln ob Viewer und HotSearch laufen }

      UpdateForm.RestartApplicationList := TStringList.Create('"', ';', [soStrictDelimiter]);

      hd := FindWindow(ViewerExe, nil);
      try
        if hd <> 0 then
        begin
          UpdateForm.lblStatus.Caption := _('Viewer wird beendet...');
          UpdateForm.RestartApplicationList.Add(ViewerExeName);
          getWindowThreadProcessId(hd, @pid1);
          cmdline := Trim(GetCommandLineFromPid(pid1));
          var exepath: TStringList;
          exepath := TStringList.Create('"', ';', [soStrictDelimiter]);
          exepath.DelimitedText := cmdline;
          ShellExecute(0, nil, PChar(exepath[0]), PChar('-c:'), nil, SW_SHOWNORMAL);
          bitfarmLog.Write(llNormal, ltDebug, 'Path: '+ exepath[0], ClassName);
        end;
      except

      end;

      { 10 Sekunden warten auf Viewer-Ende }
      i := 0;
      while (FindWindow(ViewerExe, nil) <> 0) and (i < 10) do
      begin
        i := i+1;
        Sleep(1000);
      end;

      { Benutzer informieren dass Viewer nicht beendet werden konnte }
      if (FindWindow(ViewerExe, nil) <> 0) then
      begin
        FlashWindow(UpdateForm.Handle, True);
        UpdateForm.lblStatus.Caption := _('ViewerV3 konnte nicht beendet werden.') + sLineBreak + _('Bitte alle Instanzen manuell schlieen');
      end;

      { warten bis Viewer geschlossen ist }
      while (FindWindow(ViewerExe, nil) <> 0) do
        Sleep(1000);
    end;

    if (Trim(UpdateForm.Profile) = '') then
    begin
      UpdateForm.lblStatus.Caption := _('Bitte Profil angeben!');
      UpdateForm.ShowConnectionSettings(True);
      Exit;
    end;

//    UpdateForm.lblStatus.Caption := _('Verbindung wird hergestellt...');
    UpdateForm.lblStatus.Caption := ConnectionString(UpdateForm.bfaServerNoSSL);

    { Socket-Connection erzeugen }
    UpdateForm.MainSocket := TBaseSocketConnection.Create(UpdateForm.bfaServerNoSSL, UpdateForm.bfaServer, UpdateForm.bfaServerPort, '');
    bitfarmLog.Write(llHigh, ltInfo, 'MainSocket erstellt', ClassName);

    UpdateForm.MainSocket.CreateComponents;
    UpdateForm.MainSocket.ServerReadTimeout := 7000;

    if (not UpdateForm.MainSocket.Connect)  then
    begin
      if (not Self.RetrySslConf) then
      begin
        bitfarmLog.Write(llHigh, ltException, 'retry - konnte keine Verbindung zum bfaServer aufbauen', ClassName);
        UpdateForm.lblStatus.Caption := _('Fehler bei der Serververbindung!');
        UpdateForm.UpdateStatus := bfaObjects36.TbfaUpdateStatus.eUpdateError;
        UpdateForm.ShowConnectionSettings(True);
        Exit;
      end;

      if UpdateForm.bfaServerNoSSL = 0 then
      begin
        // es wurde versucht SSL aufzubauen, jetzt NoSSL
        bitfarmLog.Write(llHigh, ltException, 'konnte keine SSL Verbindung zum bfaServer auf bauen, versuche ohne SSL', ClassName);
        UpdateForm.lblStatus.Caption := _('Verbindung (SSL) nicht erfolgreich.') + sLineBreak + _('Versuche Verbindung ohne SSL...');
        UpdateForm.MainSocket := TBaseSocketConnection.Create(sctpNoSSL, UpdateForm.bfaServer, UpdateForm.bfaServerPort, '')
      end else if UpdateForm.bfaServerNoSSL = 1 then
      begin
        // es wurde versucht NoSSL aufzubauen, jetzt SSL
        bitfarmLog.Write(llHigh, ltException, 'konnte keine Verbindung zum bfaServer auf bauen, versuche ohne SSL', ClassName);
        UpdateForm.lblStatus.Caption := _('Verbindung nicht erfolgreich.') + sLineBreak + _('Versuche mit SSL...');
        UpdateForm.MainSocket := TBaseSocketConnection.Create(sctpSSL, UpdateForm.bfaServer, UpdateForm.bfaServerPort, '');
      end;
      bitfarmLog.Write(llHigh, ltInfo, 'MainSocket erstellt', ClassName);

      UpdateForm.MainSocket.CreateComponents;
      UpdateForm.MainSocket.ServerReadTimeout := 7000;
      if not UpdateForm.MainSocket.Connect then
      begin
        bitfarmLog.Write(llHigh, ltException, 'konnte keine Verbindung zum bfaServer auf bauen, beende...', ClassName);
        UpdateForm.lblStatus.Caption := _('Fehler bei der Serververbindung!');
//        UpdateForm.tmrCloseForm.Enabled := True;
        UpdateForm.UpdateStatus := bfaObjects36.TbfaUpdateStatus.eUpdateError;
        UpdateForm.ShowConnectionSettings(True);
        Exit;
      end;
    end;

    UpdateForm.req.ReqType := 'reqCheckUpdates';
    UpdateForm.req.Profile := UpdateForm.Profile;
    bitfarmLog.Write(llHigh, ltInfo, 'send reqCheckUpdates', ClassName);
    UpdateForm.MainSocket.SendRequestToServer(UpdateForm.req);
    UpdateForm.req := UpdateForm.MainSocket.ReceiveResponse;

    if (UpdateForm.req = nil) then
    begin
      // wir mssen den req neu erzeugen wenn Antwort nil ist
      UpdateForm.req := TbfaRequest.Create;

      bitfarmLog.Write(llHigh, ltException, 'ReceiveResponse.req = nil', ClassName);
      UpdateForm.lblStatus.Caption := _('Fehler bei der Serververbindung!'){ + sLineBreak + _('Updater wird beendet.')};
//      UpdateForm.tmrCloseForm.Enabled := True;
      UpdateForm.UpdateStatus := bfaObjects36.TbfaUpdateStatus.eUpdateError;
      UpdateForm.MainSocket.Disconnect;
      UpdateForm.ShowConnectionSettings(True);
      Exit;
    end else if (UpdateForm.req.ReqType = 'respAuthError') then
    begin
      UpdateForm.lblStatus.Caption := Format(_('Profil %s nicht vorhanden'), [UpdateForm.Profile]);
      UpdateForm.MainSocket.Disconnect;
      UpdateForm.ShowConnectionSettings(True);
      Exit;
    end else if (UpdateForm.req.ReqType = 'respError') then
    begin
      bitfarmLog.Write(llHigh, ltError, 'respError - req.DataString: ' + UpdateForm.req.DataString, ClassName);
      UpdateForm.lblStatus.Caption := _('Fehler beim Update!') + sLineBreak + _('Updater wird beendet.');
      UpdateForm.tmrCloseForm.Enabled := True;
      UpdateForm.fUpdateStatus := bfaObjects36.TbfaUpdateStatus.eUpdateError;
      Exit;
    end else if (UpdateForm.req.ReqType = 'respNoUpdateConfiguration') then
    begin
      bitfarmLog.Write(llHigh, ltError, 'respNoUpdateConfiguration: ' + UpdateForm.req.DataString, ClassName);
      UpdateForm.lblStatus.Caption := _('Updates nicht konfiguriert.') + ' ' + _('Updater wird beendet.');
      UpdateForm.btnCancel.Caption := _('Ok');
      UpdateForm.tmrCloseForm.Enabled := True;
      UpdateForm.UpdateStatus := bfaObjects36.TbfaUpdateStatus.eUpdateOK;
      Exit;
    end else
    begin
      UpdateForm.FileList.DelimitedText := UpdateForm.req.DataString;
      if(UpdateForm.FileList.Count = 0) then
      begin
        UpdateForm.lblStatus.Caption := _('Updates nicht konfiguriert.') + ' ' + _('Updater wird beendet.');
        UpdateForm.btnCancel.Caption := _('Ok');
        UpdateForm.tmrCloseForm.Enabled := True;
        UpdateForm.UpdateStatus := bfaObjects36.TbfaUpdateStatus.eUpdateOK;
        Exit;
      end else
      begin
        try
          for Filename in UpdateForm.FileList do
          begin
            var AFileNameStr: string := '';
            var AFileDateStr: string := '';
            var AModifiedUTC: TDateTime := 0;
            try
               AFileNameStr := Filename.Substring(0, Filename.Length-20);
               AFileDateStr := Filename.Substring(Filename.Length-19, Filename.Length);
               { Der Updater selbst kommt nicht auf die Liste }
               if AFileNameStr = 'Updater.exe' then
                Continue;

              if FileExists(BD.ProgramFilesBitfarmClientPd + AFileNameStr) = True then
              begin
                AModifiedUTC := GetModifiedUTCEx(BD.ProgramFilesBitfarmClientPd + AFileNameStr);
                bitfarmLog.Write(llHigh, ltInfo, 'Datei: ' + AFileNameStr + ' | Client: ' + DateTimeToStr(AModifiedUTC) + ' | Server: ' + DateTimeToStr(MySQLDateToDate(AFileDateStr)), 'Thread.Execute')
              end else
                bitfarmLog.Write(llHigh, ltInfo, 'Datei: ' + AFileNameStr + ' nicht gefunden', 'Thread.Execute');

              if LowerCase(ExtractFileExt(AFileNameStr)) = '.con' then
              begin
                if ((not FileExists(BD.ProgramFilesBitfarmClientPd + AFileNameStr)) or
                   (AModifiedUTC < MySQLDateToDate(AFileDateStr))) then
                begin
                  bitfarmLog.Write(llHigh, ltInfo, 'zu RequestFileList hinzugefgt: ' + AFileNameStr, 'Thread.Execute');
                  UpdateForm.RequestFileList.Add(AFileNameStr);
                end;
                Continue;
              end;

              if (not FileExists(BD.ProgramFilesBitfarmClientPd + AFileNameStr)) or
                 (DateTimeToStr(AModifiedUTC) <> DateTimeToStr(MySQLDateToDate(AFileDateStr))) then
              begin
                bitfarmLog.Write(llHigh, ltInfo, 'zu RequestFileList hinzugefgt: ' + AFileNameStr, 'Thread.Execute');
                UpdateForm.RequestFileList.Add(AFileNameStr);
              end;
            except
              bitfarmLog.Write(llHigh, ltException, 'failed checking Filename: ' + AFileNameStr + ' ('+Filename+')', ClassName);
            end;
          end;

          for Filename in UpdateForm.RequestFileList do
          begin
            if (Filename = HotSearchExeName) then
            begin
              bitfarmLog.Write(llHigh, ltInfo, 'KillTask ' +  HotSearchExeName);
              KillTask(HotSearchExeName);
              UpdateForm.RestartApplicationList.Add(HotSearchExeName);
            end;
          end;

          if (UpdateForm.RequestFileList.Count > 0) then
          begin
            bitfarmLog.Write(llHigh, ltInfo, 'reqSendUpdates');
            bitfarmLog.Write(llHigh, ltInfo, UpdateForm.RequestFileList.DelimitedText);
            UpdateForm.req.ReqType := 'reqSendUpdates';
            UpdateForm.req.Profile := UpdateForm.Profile;
            UpdateForm.req.Parameter := UpdateForm.RequestFileList.DelimitedText;
            UpdateForm.req.DataString := '';
            UpdateForm.MainSocket.SendRequestToServer(UpdateForm.req);
            UpdateForm.tmrRefreshBar.Enabled := True;

            Filename := BD.EnvTempUpdatePd + 'Updates.zip';

            UpdateForm.req := UpdateForm.MainSocket.ReceiveDataToFile(UpdateForm.Doc, Filename, nil);

            UpdateForm.ProgressBar1.Position := 90;
            UpdateForm.tmrRefreshBar.Enabled := False;
            UpdateForm.lblStatus.Caption := _('Update wird entpackt...');
            bitfarmLog.Write(llHigh, ltInfo, 'Update wird entpackt > ' + Filename);
            try
              try
                with TZipRead.Create(Filename) do
                begin
                  try
                    UnZipAll(BD.AppdataBitfarmPd + 'Updates');
                  finally
                    Free;
                  end;
                end;

                { Count von zip vs directory }
                UpdateForm.ProgressBar1.Position := Round(UpdateForm.ProgressBar1.Position + (UpdateForm.ProgressBar1.Max - UpdateForm.ProgressBar1.Position) / 2);
                UpdateForm.lblStatus.Caption := _('Update wird verschoben...');
                bitfarmLog.Write(llHigh, ltInfo, 'Update wird verschoben...');
                try
                  BfMoveDir(BD.AppdataBitfarmPd + 'Updates\', BD.ProgramFilesX86BitfarmClientPd, True);
                  bitfarmLog.Write(llHigh, ltInfo, BD.AppdataBitfarmPd + 'Updates\ > ' + BD.ProgramFilesX86BitfarmClientPd);
                  UpdateForm.ProgressBar1.Position := UpdateForm.ProgressBar1.Max;
                  UpdateForm.lblStatus.Caption := _('Update erfolgreich');
                  UpdateForm.btnCancel.Caption := _('Ok');
                  bitfarmLog.Write(llHigh, ltInfo, 'Update erfolgreich');
                  UpdateForm.tmrCloseForm.Enabled := True;
                  UpdateForm.UpdateStatus := bfaObjects36.TbfaUpdateStatus.eUpdateOK;
                  Exit;
                except on E: Exception do
                  bitfarmLog.Write(llHigh, ltException, 'Failed to move update: ' + E.Message, ClassName);
                end;
              finally
                DeleteFile(Filename);
              end;
            except on E: Exception do
              begin
                bitfarmLog.Write(llHigh, ltException, 'Failed to extract zip file: ' + E.Message, ClassName);
                UpdateForm.lblStatus.Caption := _('Entpacken fehlgeschlagen!') + sLineBreak + _('Updater wird beendet.');
                UpdateForm.tmrCloseForm.Enabled := True;
                UpdateForm.UpdateStatus := bfaObjects36.TbfaUpdateStatus.eUpdateError;
                Exit;
              end;
            end;
          end else
          begin
            UpdateForm.lblStatus.Caption := _('Keine Updates verfgbar.') + ' ' + _('Updater wird beendet.');
            UpdateForm.btnCancel.Caption := _('Ok');
            UpdateForm.tmrCloseForm.Enabled := True;
            UpdateForm.UpdateStatus := bfaObjects36.TbfaUpdateStatus.eNoUpdate;
            Exit;
          end;
        except on E: Exception do
          bitfarmLog.Write(llHigh, ltException, 'Thread.Execute Update 1: ' + E.Message, ClassName);
        end;
      end;
    end;
  except on E: Exception do
    bitfarmLog.Write(llHigh, ltException, 'Thread.Execute Main: ' + E.Message, ClassName);
  end;
end;

end.


