# -*- encoding: UTF-8 -*-

from bs4 import BeautifulSoup
import configparser
import logging
import logging.handlers
import subprocess
import io
import tempfile
import time
import os
import queue
import shutil
import codecs
import base64
import pyzmail
import re
from dateutil import parser
import dateutil
import email
import cgi
import sys
import warnings
warnings.filterwarnings(action='ignore', category=UnicodeWarning)

try:
    import RendererEnterprise
    RE = RendererEnterprise.Renderer()
except:
    RE = None

OFFICEEXT = [".doc", ".docx", ".dotx", ".odt", ".xls", ".xlsx", ".ppt",
             ".pptx", ".odt", ".ods", ".odp", ".rtf", ".txt", ".wps", ".asc", ".xml"]
OFFICEEXT_OUT = ["txt", "pdf", "tif", "jpg", "bmp", "png", "html"]
GS_DEFAULTOPTIONS = {}
GS_DEFAULTOPTIONS["tif"] = {"color": "color",
                            "compression": "lzw", "resolution": 300}
GS_DEFAULTOPTIONS["jpg"] = {"color": "color", "quality": 80}
GS_DEFAULTOPTIONS["png"] = {"color": "color", "page": 0}
GS_DEFAULTOPTIONS["bmp"] = {"color": "color"}

PDFEXT = [".pdf", ".ps"]
PDFEXT_OUT = ["txt", "tif", "jpg", "bmp", "png"]
PDF_DEFAULTOPTIONS = {}
PDF_DEFAULTOPTIONS["tif"] = {"color": "color",
                             "compression": "lzw", "resolution": 300}
PDF_DEFAULTOPTIONS["jpg"] = {"color": "color", "quality": 80}
PDF_DEFAULTOPTIONS["png"] = {"color": "color"}
PDF_DEFAULTOPTIONS["bmp"] = {"color": "color"}

HTMLEXT = [".htm", ".html"]
HTMLEXT_OUT = ["txt", "pdf", "tif", "jpg", "bmp", "png"]
HTML_DEFAULTOPTIONS = {}
HTML_DEFAULTOPTIONS["tif"] = {"color": "color",
                              "compression": "lzw", "resolution": 300}
HTML_DEFAULTOPTIONS["jpg"] = {"color": "color", "quality": 80}
HTML_DEFAULTOPTIONS["png"] = {"color": "color"}
HTML_DEFAULTOPTIONS["bmp"] = {"color": "color"}

IMGEXT = [".tif",  ".jpg", ".png", ".bmp", ".gif"]
IMGEXT_OUT = ["tif", "jpg", "bmp", "png", "pdf"]
IMG_DEFAULTOPTIONS = {}
IMG_DEFAULTOPTIONS["tif"] = {"color": "color",
                             "compression": "lzw", "resolution": 300}
IMG_DEFAULTOPTIONS["jpg"] = {"color": "color", "quality": 80}
IMG_DEFAULTOPTIONS["png"] = {"color": "color"}
IMG_DEFAULTOPTIONS["bmp"] = {"color": "color"}

GS_TIF_DEVICES = {}
GS_TIF_DEVICES["bw"] = "--sDevice=tiffg4"
GS_TIF_DEVICES["gray"] = "--sDevice=tiffgray"
GS_TIF_DEVICES["color"] = "--sDevice=tiff24nc"

GS_PNG_DEVICES = {}
GS_PNG_DEVICES["color"] = "--sDevice=png16m"
GS_PNG_DEVICES["gray"] = "--sDevice=pnggray"
GS_PNG_DEVICES["bw"] = "--sDevice=pngmonod"

GS_BMP_DEVICES = {}
GS_BMP_DEVICES["color"] = "--sDevice=bmp32b"
GS_BMP_DEVICES["gray"] = "--sDevice=bmpgray"
GS_BMP_DEVICES["bw"] = "--sDevice=bmpmono"

GS_JPG_DEVICES = {}
GS_JPG_DEVICES["color"] = "--sDevice=jpeg"
GS_JPG_DEVICES["gray"] = "--sDevice=jpeggray"


GS_COMPRESSION = {}
GS_COMPRESSION["lzw"] = "--compress=lzw"


DEFAULT_GS_SDEVICE = "--sDevice=tiff24nc"
DEFAULT_GS_COMPRESSION = "--compress=lzw"
DEFAULT_GS_RESOLUTION = "--resolution=300"

DEFAULTPNGOPTIONS = {"sdevice": "--sDevice=pnggray"}
DEFAULTBMPOPTIONS = {"sdevice": "--sDevice=bmpgray"}

DEFAULTJPGOPTIONS = {"sdevice": "--sDevice=jpeggray"}
DEFAULTTIFOPTIONS = {"sdevice": "--sDevice=tiffgray",
                     "compression": "--compress=lzw", "resolution": "--resolution=300", "maxpages": None}
DEFAULTTIFFILTEROPTIONS = {"color": "-gray",
                           "resolution": "-res=300", "compression": "-complzw"}

DEFAULTPNGFILTEROPTIONS = {"color": "-color"}
DEFAULTBMPFILTEROPTIONS = {"color": "-color"}

DEFAULTJPGFILTEROPTIONS = {"color": "-color", "quality": "-compjpg=80"}


COLORVALUES = {}
COLORVALUES["gray"] = "gray"
COLORVALUES["grey"] = "gray"
COLORVALUES["bw"] = "bw"
COLORVALUES["sw"] = "bw"
COLORVALUES["color"] = "color"
COLORVALUES["colour"] = "color"

COMPRESSIONVALUES = {}
COMPRESSIONVALUES["lzw"] = "lzw"


#sys.stdout=open(r"C:\bitfarm-archiv\stdout-r.txt", "wb")
#sys.stderr=open(r"C:\bitfarm-archiv\stderr-r.txt", "wb")


class Renderer(object):

    def __init__(self, configfile):
        self.__version__ = "3.6.0"

        self.allfiletypes = []
        self.allfiletypes.extend(OFFICEEXT)
        self.allfiletypes.extend(PDFEXT)
        self.allfiletypes.extend(HTMLEXT)
        self.allfiletypes.extend(IMGEXT)

        config = configparser.ConfigParser(interpolation=None)
        config.read(configfile)

        # Logging config
#        logdir = os.path.expandvars(config.get("main", "logdir"))
#        if not os.path.isdir(logdir):
#            os.makedirs(logdir)
#        logfilepath = os.path.join(logdir, "BitfarmRenderServer.log")
#        self.log = logging.getLogger('RenderService')
#        self.log.setLevel(logging.DEBUG)
#        formatter = logging.Formatter(
#            '%(asctime)s - %(levelname)s - %(message)s')
#        handler = logging.handlers.RotatingFileHandler(
#            logfilepath, backupCount=5)
#        handler.setFormatter(formatter)
#        self.log.addHandler(handler)
        self.log = logging.getLogger("bfaServer")
        self.log.info("starting RenderService %s" % self.__version__)
        self.tempdir = os.path.expandvars(config.get("main", "tmpdir"))
        if not os.path.isdir(self.tempdir):
            os.makedirs(self.tempdir)
        self.cleanup()
        if "minimal" in config.options("main"):
            self.minimal = config.getboolean("main", "minimal")
        else:
            self.minimal = False
        # sonstige config
        if self.minimal is False:
            # sonstige config
            self.sofficebin = config.get("soffice", "binpath")
            self.pythonpath = config.get("soffice", "pythonpath")

            # Pfade für Standardconverter
            self.wkhtmltopdfbin = os.path.join(
                os.getcwd(), "bin", "wkhtmltopdf", "bin", "wkhtmltopdf.exe")
            self.pdf2tifbin = os.path.join(
                os.getcwd(), "bin", "bfa_pdf2tif.exe")
            self.pdf2txtbin = os.path.join(
                os.getcwd(), "bin", "pdftotext.exe")
            self.ooextract = os.path.join(
                os.getcwd(), "bin", "ooextract.py")
            self.filterv3bin = os.path.join(
                os.getcwd(), "bin", "FilterV3.exe")
            self.filterv4bin = os.path.join(
                os.getcwd(), "bin", "FilterV4.exe")
            self.filterv4bin = os.path.join(
                os.getcwd(), "bin", "FilterV4.exe")

            missingbins = []
            if not os.path.isfile(self.wkhtmltopdfbin):
                missingbins.append(self.wkhtmltopdfbin)
            if not os.path.isfile(self.pdf2tifbin):
                missingbins.append(self.pdf2tifbin)
            if not os.path.isfile(self.pdf2txtbin):
                missingbins.append(self.pdf2txtbin)
            if not os.path.isfile(self.ooextract):
                missingbins.append(self.ooextract)
            if not os.path.isfile(self.filterv3bin):
                missingbins.append(self.filterv3bin)
            if not os.path.isfile(self.filterv4bin):
                missingbins.append(self.filterv4bin)

            if len(missingbins) > 0:
                self.log.critical(
                    "cannot start renderer, files not found: %s" % (str(missingbins)))
                raise Exception(
                    "cannot start renderer, files not found: %s" % (str(missingbins)))

            ports_string = config.get("soffice", "listenports")
            try:
                self.ports = [int(port) for port in ports_string.split(",")]
            except:
                self.log.critical(
                    "config error, [soffice] listenports wrong format. correct value example: 2002,2003")
                raise Exception(
                    "config error, [soffice] listenports wrong format. correct value example: 2002,2003")
            self.soffice_instances = []
            self.sofficequeue = Queue.Queue()
            self.log.info("RenderService starting libreoffice")
            self.kill_all_soffice(others=True)
            self.cleanup()
            self.start_all_soffice()
            if RE is not None:
                RE.loadconfig(configfile)
                RE.mainrenderer = self
            self.log.info("RenderService ready")
        else:
            self.filterv4bin = os.path.join(
                os.path.expandvars("%bitfarm-archiv%"), "FilterV4.exe")

    def _create_tif_options(self, tifoptionsstr):
        # gs settings for pdf to tif
        tifoptions = GS_DEFAULTOPTIONS["tif"]
        optarr = tifoptionsstr.split(",")
        for optstr in optarr:
            opt, val = optstr.split("=")
            if opt in tifoptions:
                if opt.lower() == "color":
                    if val.lower() in COLORVALUES:
                        tifoptions[opt] = COLORVALUES[val]
                elif opt.lower() == "resolution":
                    if val.isnumeric():
                        tifoptions[opt] = val
                elif opt.lower() == "maxpages":
                    if val.isnumeric():
                        tifoptions[opt] = val
                elif opt.lower() == "compression":
                    if val in ("none", "lzw", "g4", "g3", "crle", "pack"):
                        tifoptions[opt] = val
        if "color" in tifoptions:
            if tifoptions["color"] in ("color", "gray"):
                if "compression" in tifoptions:
                    if tifoptions["compression"] not in ("lzw", "pack", "none"):
                        tifoptions["compression"] = "lzw"
                else:
                    tifoptions["compression"] = None
            else:  # bw
                if "compression" in tifoptions:
                    if tifoptions["compression"] not in ("crle", "g4", "g3", "lzw", "pack", "none"):
                        tifoptions["compression"] = "lzw"

        # bw compression mdes: crle | g3 | g4 | lzw | pack
        # color compression modes: lzw,

        return tifoptions

    def _create_bmp_options(self, bmpoptionsstr):
        bmpoptions = GS_DEFAULTOPTIONS["bmp"]
        optarr = bmpoptionsstr.split(",")
        for optstr in optarr:
            opt, val = optstr.split("=")
            if opt in bmpoptions:
                if opt.lower() == "color":
                    if val.lower() in COLORVALUES:
                        bmpoptions[opt] = COLORVALUES[val]

                elif opt.lower() == "maxpages":
                    if val.isnumeric():
                        bmpoptions[opt] = val
        return bmpoptions

    def _create_png_options(self, pngoptionsstr):
        pngoptions = GS_DEFAULTOPTIONS["png"]
        optarr = pngoptionsstr.split(",")
        for optstr in optarr:
            opt, val = optstr.split("=")
            if opt in pngoptions:
                if opt.lower() == "color":
                    if val.lower() in COLORVALUES:
                        pngoptions[opt] = COLORVALUES[val]

                elif opt.lower() == "maxpages":
                    if val.isnumeric():
                        pngoptions[opt] = val
        return pngoptions

    def _create_jpg_options(self, jpgoptionsstr):
        jpgoptions = GS_DEFAULTOPTIONS["jpg"]
        optarr = jpgoptionsstr.split(",")
        for optstr in optarr:
            opt, val = optstr.split("=")
            if opt in jpgoptions:
                if opt.lower() == "color":
                    if val.lower() in COLORVALUES:
                        jpgoptions[opt] = COLORVALUES[val]
                elif opt.lower() == "maxpages":
                    if val.isnumeric():
                        jpgoptions[opt] = val
                elif opt.lower() == "quality":
                    if val.isnumeric():
                        jpgoptions["quality"] = val
        return jpgoptions

    def _html_to_pdf(self, uid, infile, outfile):
        args = [
            self.wkhtmltopdfbin,
            "--dpi",
            "300",
            # "--minimum-font-size",
            # "32",
            "--no-pdf-compression",
            "--disable-javascript",
            # "--no-images",
            infile,
            outfile]
        self.log.debug("%s _html_to_pdf %s" % (uid, str(args)))
        # try:
        timeout = 120  # 1min
        cnt = 0
        is_timeout = False
        pipe = subprocess.Popen(
            args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        txt = ""
        while pipe.returncode is None:
            time.sleep(0.5)
            cnt += 1
            if cnt > timeout:
                is_timeout = True
                break
            txt = pipe.communicate()
            self.log.debug("%s wkhtmltopdf output: %s" % (uid, txt))
            pipe.poll()
        if is_timeout:
            pipe.terminate()
            #self.log.debug("%s wkhtmltopdf output: %s" %(uid, txt))
            return None
        else:
            #self.log.debug("%s wkhtmltopdf output: %s" %(uid, txt))
            return True
        # except:
            #self.log.debug("%s wkhtmltopdf output: %s" %(uid, txt))
            # return None

    def _save_inputfile(self, srcfile, pth=None, uid=""):
        self.log.debug("%s _save_inputfile srcfile type: %s" %
                       (uid, type(srcfile)))
        self.log.debug("%s _save_inputfile pth: %s" % (uid, str(pth)))
        try:
            if type(srcfile) is str:
                if os.path.isfile(srcfile):
                    return srcfile
        except:
            pass
        try:
            if pth is None:
                tempdir = tempfile.mkdtemp(dir=self.tempdir)
                srcfile_ext = os.path.splitext(
                    os.path.basename(srcfile.filename))[1].lower()
                fd, pth = tempfile.mkstemp(dir=tempdir, suffix=srcfile_ext)
                os.close(fd)
                tempf = open(pth, "wb")
            else:
                tempf = open(pth, "wb")
            while True:
                if isinstance(srcfile, io.BytesIO):
                    newdata = srcfile.read(8192)
                else:
                    newdata = srcfile.file.read(8192)
                if not newdata or len(newdata) == 0:
                    break
                else:
                    tempf.write(newdata)
            tempf.close()
        except:
            pth = None
        return pth

    def _options2gsargs(self, options, typ):
        args = []
        if typ == "tif":
            if "color" in options:
                args.append(GS_TIF_DEVICES[options["color"]])
            if "compression" in options:
                if options["compression"] in ("crle", "g4", "g3", "lzw", "pack", "none"):
                    args.append("--compress=%s" % options["compression"])
                else:
                    args.append("--compress=lzw")
            if "resolution" in options:
                args.append("--resolution=%s" % options["resolution"])
            if "maxpages" in options:
                args.append("--maxpages=%s" % options["maxpages"])
        elif typ == "jpg":
            if "color" in options:
                args.append(GS_JPG_DEVICES[options["color"]])
            if "maxpages" in options:
                args.append("--maxpages=%s" % options["maxpages"])
            if "quality" in options:
                args.append("--quality=%s" % options["quality"])
        elif typ == "png":
            if "color" in options:
                args.append(GS_PNG_DEVICES[options["color"]])
            if "maxpages" in options:
                args.append("--maxpages=%s" % options["maxpages"])
        elif typ == "bmp":
            if "color" in options:
                args.append(GS_BMP_DEVICES[options["color"]])
            if "maxpages" in options:
                args.append("--maxpages=%s" % options["maxpages"])
        return args

    def _timeoutprocess(self, args, timeout=120, uid=""):
        pipe = subprocess.Popen(
            args, stdout=subprocess.PIPE, stderr=subprocess.PIPE, stdin=subprocess.PIPE)
        cnt = 0
        is_timeout = False
        # timeout wird in Sekunden angegeben, wir prüfen aber alle 0,1 sek
        timeout = 10 * timeout
        while pipe.returncode is None:
            # while cnt < 30:
            time.sleep(0.1)
            cnt += 1
            #self.log.debug("_timeoutprocess run %d" %cnt)
            #self.log.debug("_timeoutprocess pipe.returncode %s" %str(pipe.returncode))
            if cnt > timeout:
                is_timeout = True
                break
            txt = pipe.communicate()
            self.log.debug("%s _timeoutprocess output %s" % (uid, str(txt)))
            pipe.poll()
        if is_timeout:
            self.log.error("TIMEOUT!!!!")
            pipe.terminate()
            return False
        else:
            return True

    def cleanup(self):
        for root, dirs, files in os.walk(self.tempdir):
            for d in dirs:
                try:
                    shutil.rmtree(os.path.join(self.tempdir, d))
                except:
                    self.log.error("cleanup error, %s could not be removed" % (
                        os.path.join(self.tempdir, d)))

    def convert(self, uid, tempdir, **params):
        self.log.debug("%s Renderer.convert started" % (uid))
        self.log.debug("%s tempdir %s" % (uid, tempdir))
        self.log.debug("%s params %s" % (uid, str(params)))
        outtypes = params["outtypes"].split(",")
        srcfile = params["myFile"]
        srcfile_ext = os.path.splitext(
            os.path.basename(srcfile.filename))[1].lower()
        # if "inputfile" in params:
        #srcfile = params["inputfile"]
        if srcfile_ext not in self.allfiletypes:
            if RE is None:
                return {"error": "%s cannot be converted" % (srcfile_ext)}
            elif srcfile_ext not in RE.allfiletypes:
                return {"error": "%s cannot be converted" % (srcfile_ext)}

        if srcfile_ext in OFFICEEXT:
            for outtype in outtypes:
                if outtype not in OFFICEEXT_OUT:
                    return {"error": "%s cannot be converted to %s" % (srcfile_ext, outtype)}
            # wir haben ein Soffice-Typ und outtypes sind okay
            # Konverter starten
            del params["outtypes"]
            del params["myFile"]
            result = self.office_convert(
                uid, srcfile, tempdir, outtypes, **params)
            return result
        elif srcfile_ext in PDFEXT:
            for outtype in outtypes:
                if outtype not in PDFEXT_OUT:
                    return {"error": "%s cannot be converted to %s" % (srcfile_ext, outtype)}
            del params["outtypes"]
            del params["myFile"]
            result = self.pdf_convert(
                uid, srcfile, tempdir, outtypes, **params)
            return result
        elif srcfile_ext in HTMLEXT:
            for outtype in outtypes:
                if outtype not in HTMLEXT_OUT:
                    return {"error": "%s cannot be converted to %s" % (srcfile_ext, outtype)}
            del params["outtypes"]
            del params["myFile"]
            result = self.html_convert(
                uid, srcfile, tempdir, outtypes, **params)
            # print result
            return result
        elif srcfile_ext in IMGEXT:
            for outtype in outtypes:
                if outtype not in IMGEXT_OUT:
                    return {"error": "%s cannot be converted to %s" % (srcfile_ext, outtype)}
            del params["outtypes"]
            del params["myFile"]
            result = self.img_convert(
                uid, srcfile, tempdir, outtypes, **params)
            return result
        else:
            if RE is not None:
                if srcfile_ext not in RE.allfiletypes:
                    return {"error": "%s cannot be converted" % (srcfile_ext)}
                else:
                    del params["outtypes"]
                    del params["myFile"]
                    result = RE.convert(
                        uid, srcfile, tempdir, outtypes, **params)
                    return result
        return {"error": "unknown converter error"}

    def html_convert(self, uid, srcfile, tempdir, outtypes, **params):
        self.log.debug("%s start html_convert" % (uid))

        if type(srcfile) is not str:
            srcfile_ext = os.path.splitext(
                os.path.basename(srcfile.filename))[1].lower()
            fd, pth = tempfile.mkstemp(dir=tempdir, suffix=srcfile_ext)
            os.close(fd)
            infile = self._save_inputfile(srcfile, pth=pth, uid=uid)
        else:
            infile = srcfile

        if infile is None:
            self.log.error("%s could not save infile" % uid)
            return {"error": "could not save infile"}

        result = {}

        pdfname = os.path.join(tempdir, "temp.pdf")
        htmltopdfresult = self._html_to_pdf(uid, infile, pdfname)
        if htmltopdfresult is not True or not os.path.isfile(pdfname):
            return {"error": "html to pdf error"}
        else:
            if "pdf" in outtypes:
                if os.path.isfile(pdfname):
                    f = open(pdfname, "rb")
                    pdfresult = f.read()
                    f.close()
                    result["pdf"] = base64.b64encode(pdfresult)
                    self.log.info("%s pdf %s created" % (uid, pdfname))
                else:
                    self.log.error(
                        "%s office_convert could not create pdf" % (uid))
                    result["pdf"] = None
                    pdfresult = None

            if "tif" in outtypes or "bmp" in outtypes or "png" in outtypes or "jpg" in outtypes:
                pdfconvertresult = self.pdf_convert(
                    uid, pdfname, tempdir, outtypes, **params)
                if not "error" in pdfconvertresult:
                    result.update(pdfconvertresult)

        return result

    # def img_convert(self, *args, **kwargs):
    def img_convert(self, uid, srcfile, tempdir, outtypes, **params):
        # TODO: Tif -> irgendwas -> muss Seite angegeben werden,
        # da Zielformat nicht geeignet für multiple Seiten
        self.log.debug("%s start img_convert" % (uid))
        self.log.debug("%s img_convert params: %s" % (uid, str(params)))
        self.log.debug("%s img_convert outtypes: %s" % (uid, str(outtypes)))
        #print params
        # return {"error":"FilterV3/FilterV4 not installed/configured"}
        if self.filterv4bin is None:
            self.log.error("%s FilterV4 not installed/configured" % (uid))
            return {"error": "FilterV4 not installed/configured"}

        # srcfile kann mytif aus request-body sein oder eine vorhandene Datei
        if type(srcfile) is not str:
            srcfile_ext = os.path.splitext(
                os.path.basename(srcfile.filename))[1].lower()
            fd, pth = tempfile.mkstemp(dir=tempdir, suffix=srcfile_ext)
            os.close(fd)
            infile = self._save_inputfile(srcfile, pth=pth, uid=uid)
        else:
            infile = srcfile
            srcfile_ext = os.path.splitext(
                os.path.basename(srcfile))[1].lower()

        if infile is None:
            self.log.error("%s could not save infile" % uid)
            return {"error": "could not save infile"}

        infiletype = srcfile_ext

        if "iev" in params:
            fd, pth = tempfile.mkstemp(dir=tempdir, suffix=".iev")
            os.close(fd)
            self._save_inputfile(params["iev"], pth=pth, uid=uid)
            ievfile = pth
            self.log.debug("%s ievfile %s saved" % (uid, ievfile))
        else:
            ievfile = None

        # tif with ievlist
        ievdict = {}
        if srcfile_ext == ".tif":
            for param in params:
                if param.startswith("iev_"):
                    try:
                        page = int(param.split("_")[1])
                        ievdict[page] = params[param]
                    except:
                        ievdict[page] = None
        if len(ievdict) == 0:
            ievdict = None
            self.log.debug("%s no ievdict" % (uid))
        else:
            self.log.debug("%s ievdict: %s" % (uid, str(ievdict)))

        if "tif" in outtypes:
            createtif = True
            if "tifoptions" in params:
                tifoptions = params["tifoptions"]
            else:
                tifoptions = None
        else:
            createtif = False
            tifoptions = None

        if "jpg" in outtypes:
            createjpg = True
            if "jpgoptions" in params:
                jpgoptions = params["jpgoptions"]
            else:
                jpgoptions = None
        else:
            createjpg = None
            jpgoptions = None

        if "bmp" in outtypes:
            createbmp = True
            if "bmpoptions" in params:
                bmpoptions = params["bmpoptions"]
            else:
                bmpoptions = None
        else:
            createbmp = False
            bmpoptions = None

        if "png" in outtypes:
            createpng = True
            if "pngoptions" in params:
                pngoptions = params["pngoptions"]
            else:
                pngoptions = None
        else:
            createpng = False
            pngoptions = None

        if "pdf" in outtypes:
            createpdf = True
        else:
            createpdf = False
        result = {}
        if True in (createtif, createpng, createbmp, createjpg, createpdf):
            tifname = os.path.join(tempdir, "temp.tif")
            bmpname = os.path.join(tempdir, "temp.bmp")
            pngname = os.path.join(tempdir, "temp.png")
            jpgname = os.path.join(tempdir, "temp.jpg")
            pdfname = os.path.join(tempdir, "temp.pdf")
            self.log.debug("%s img_convert infiletype: %s" %
                           (uid, str(infiletype)))

            if infiletype.lower() == ".tif":
                if createtif is True:
                    # Tif nach Tif
                    args = [self.filterv4bin, "-in=" + infile, "-format=tif", "-out=" + tifname]
                    color = None
                    resolution = None
                    compression = None

                    if tifoptions is not None:
                        options = tifoptions.split(",")
                        for item in options:
                            key, val = item.split("=")
                            if key == "color":
                                if val.lower() in ["gray", "grey"]:
                                    color = "-gray"
                                elif val.lower() in ["bw", "schwarzweiss", "sw"]:
                                    color = "-bw"
                                elif val.lower() in ["color", "farbe"]:
                                    color = "-color"
                            if key == "resolution":
                                if val.isnumeric():
                                    resolution = "-res="+val
                            if key == "compression":
                                if val.lower() in ["lzw", "complzw"]:
                                    compression = "-complzw"
                                elif val.lower() in ["g4", "compg4"]:
                                    compression = "-compg4"
                                elif val.lower() in ["none"]:
                                    compression = ""
                                # todo: jpg-Komprimierung mit quality-option
                                else:
                                    compression = None
                    if compression is None:
                        compression = DEFAULTTIFFILTEROPTIONS["compression"]
                    if color is None:
                        color = DEFAULTTIFFILTEROPTIONS["color"]
                    if resolution is None:
                        resolution = DEFAULTTIFFILTEROPTIONS["resolution"]
                    args.append(compression)
                    args.append(color)
                    args.append(resolution)

                    if ievdict is not None:
                        lastkey = sorted(ievdict.keys())[-1]
                        self.log.debug("%s img_convert lastkey (ievpage): %s" % (uid, str(lastkey)))
                        annoargs = (lastkey+1)*["None"]

                        # save single ievfiles in tempdir replace in anno-args
                        for ievpage, stream in ievdict.items():
                            try:
                                stream.seek(0)
                            except:
                                pass
                            if stream != u"":
                                self.log.debug(
                                    "%s iev is not None, saving file" % (uid))
                                fname = os.path.join(
                                    tempdir, str(ievpage)+".iev")
                                pth = self._save_inputfile(
                                    stream, pth=fname, uid=uid)
                                self.log.debug("%s saved iev %s" % (uid, pth))
                                annoargs[ievpage] = fname
                            # else:
                                #self.log.debug("%s iev is None" %(uid))
                                #annoargs[ievpage] = "None"
                        args.append("anno="+",".join(annoargs))
                        self.log.debug("%s img_convert annoargs: %s" % (uid, str(annoargs)))

                    self.log.debug(
                        "%s starting process with args: %s" % (uid, str(args)))
                    timeout = self._timeoutprocess(args, timeout=120, uid=uid)

                    if os.path.isfile(tifname):
                        self.log.debug("%s img_convert tif found" % (uid))
                        f = open(tifname, "rb")
                        tifresult = f.read()
                        f.close()
                        result["tif"] = base64.b64encode(tifresult)
                        self.log.debug("%s img_convert got tif-result" % (uid))
                    else:
                        self.log.debug("%s img_convert no tif found" % (uid))
                        result["tif"] = None

                if createpng is True:
                    # Tif nach PNG
                    args = [self.filterv4bin, "-in=" + infile, "-format=png", "-out=" + pngname]
                    color = None

                    if pngoptions is not None:
                        options = pngoptions.split(",")
                        self.log.debug("%s pngoptions: %s" %
                                       (uid, str(options)))
                        for item in options:
                            key, val = item.split("=")
                            if key == "color":
                                if val.lower() in ["gray", "grey"]:
                                    color = "-gray"
                                elif val.lower() in ["bw", "schwarzweiss", "sw"]:
                                    color = "-bw"
                                elif val.lower() in ["color", "farbe"]:
                                    color = "-color"
                            elif key == "page":
                                args.append("-savepage="+val)

                    if color is None:
                        color = DEFAULTPNGFILTEROPTIONS["color"]

                    args.append(color)

                    if ievfile is not None:
                        args.append("-anno="+ievfile)

                    self.log.debug("%s starting process with args: %s" % (uid, str(args)))
                    timeout = self._timeoutprocess(args, timeout=120, uid=uid)
                    if os.path.isfile(pngname):
                        f = open(pngname, "rb")
                        pngresult = f.read()
                        f.close()
                        result["png"] = base64.b64encode(pngresult)
                        self.log.debug("%s img_convert got pngresult" % (uid))
                    else:
                        result["png"] = None

                if createbmp is True:
                    # Tif nach BMP
                    args = [self.filterv4bin, "-in=" + infile, "-format=bmp", "-out=" + bmpname]
                    color = None

                    if bmpoptions is not None:
                        options = bmpoptions.split(",")
                        self.log.debug("%s bmpoptions: " % (uid, str(options)))
                        for item in options:
                            key, val = item.split("=")
                            if key == "color":
                                if val.lower() in ["gray", "grey"]:
                                    color = "-gray"
                                elif val.lower() in ["bw", "schwarzweiss", "sw"]:
                                    color = "-bw"
                                elif val.lower() in ["color", "farbe"]:
                                    color = "-color"

                    if color is None:
                        color = DEFAULTBMPFILTEROPTIONS["color"]

                    args.append(color)
                    self.log.debug(
                        "%s starting process with args: %s" % (uid, str(args)))
                    timeout = self._timeoutprocess(args, timeout=120, uid=uid)
                    if os.path.isfile(bmpname):
                        f = open(bmpname, "rb")
                        bmpresult = f.read()
                        f.close()
                        result["bmp"] = base64.b64encode(bmpresult)
                        self.log.debug("%s img_convert got bmpresult" % (uid))
                    else:
                        result["bmp"] = None

                if createjpg is True:
                    # Tif nach PNG
                    args = [self.filterv4bin, "-in=" + infile, "-format=jpg", "-out=" + jpgname]
                    color = None
                    quality = None

                    if jpgoptions is not None:
                        options = jpgoptions.split(",")
                        self.log.debug("%s jpgoptions: %s" % (uid, str(options)))
                        for item in options:
                            key, val = item.split("=")
                            if key == "color":
                                if val.lower() in ["gray", "grey"]:
                                    color = "-gray"
                                elif val.lower() in ["bw", "schwarzweiss", "sw"]:
                                    color = "-bw"
                                elif val.lower() in ["color", "farbe"]:
                                    color = "-color"
                            elif key == "quality":
                                quality = "-compjpg="+val

                    if quality is None:
                        quality = DEFAULTJPGFILTEROPTIONS["quality"]
                    if color is None:
                        color = DEFAULTJPGFILTEROPTIONS["color"]

                    args.append(color)
                    args.append(quality)
                    self.log.debug("%s starting process with args: %s" % (uid, str(args)))
                    timeout = self._timeoutprocess(args, timeout=120, uid=uid)
                    if os.path.isfile(jpgname):
                        f = open(jpgname, "rb")
                        jpgresult = f.read()
                        f.close()
                        result["jpg"] = base64.b64encode(jpgresult)
                        self.log.debug("%s img_convert got jpgresult" % (uid))
                    else:
                        result["jpg"] = None
                if createpdf is True:
                    args = [self.filterv4bin, "-in=" + infile, "-format=pdf", "-out=" + pdfname]

                    if ievdict is not None:
                        lastkey = sorted(ievdict.keys())[-1]
                        self.log.debug(
                            "%s img_convert lastkey (ievpage): %s" % (uid, str(lastkey)))
                        annoargs = (lastkey+1)*[None]

                        # save single ievfiles in tempdir replace in anno-args
                        for ievpage, stream in ievdict.items():
                            if stream != u"":
                                self.log.debug(
                                    "%s iev is not None, saving file" % (uid))
                                fname = os.path.join(
                                    tempdir, str(ievpage)+".iev")
                                pth = self._save_inputfile(
                                    stream, pth=fname, uid=uid)
                                self.log.debug("%s saved iev %s" % (uid, pth))
                                annoargs[ievpage] = fname
                            else:
                                self.log.debug("%s iev is None" % (uid))
                                annoargs[ievpage] = "None"
                        args.append("anno="+",".join(annoargs))
                        self.log.debug("%s img_convert annoargs: %s" %
                                       (uid, str(annoargs)))
                    self.log.debug(
                        "%s starting process with args: %s" % (uid, str(args)))
                    timeout = self._timeoutprocess(args, timeout=120, uid=uid)
                    if os.path.isfile(pdfname):
                        f = open(pdfname, "rb")
                        pdfname = f.read()
                        f.close()
                        result["pdf"] = base64.b64encode(pdfname)
                        self.log.debug("%s img_convert got pdf" % (uid))
                    else:
                        self.log.error("%s img_convert no pdf created" % (uid))
                        result["pdf"] = None

            elif infiletype.lower() in (".bmp", ".png", ".gif", ".jpg"):
                if createtif is True:
                    # Tif nach Tif
                    args = [self.filterv3bin, "-in=" + infile, "-format=tif", "-out=" + tifname]
                    color = None
                    resolution = None
                    compression = None

                    if tifoptions is not None:
                        options = tifoptions.split(",")
                        for item in options:
                            key, val = item.split("=")
                            if key == "color":
                                if val.lower() in ["gray", "grey"]:
                                    color = "-gray"
                                elif val.lower() in ["bw", "schwarzweiss", "sw"]:
                                    color = "-bw"
                                elif val.lower() in ["color", "farbe"]:
                                    color = "-color"
                            if key == "resolution":
                                if val.isnumeric():
                                    resolution = "-res="+val
                            if key == "compression":
                                if val.lower() in ["lzw", "complzw"]:
                                    compression = "-complzw"
                                elif val.lower() in ["g4", "compg4"]:
                                    compression = "-compg4"
                                elif val.lower() in ["none"]:
                                    compression = ""
                                # todo: jpg-Komprimierung mit quality-option
                                else:
                                    compression = None
                    if compression is None:
                        compression = DEFAULTTIFFILTEROPTIONS["compression"]
                    if color is None:
                        color = DEFAULTTIFFILTEROPTIONS["color"]
                    if resolution is None:
                        resolution = DEFAULTTIFFILTEROPTIONS["resolution"]
                    args.append(compression)
                    args.append(color)
                    args.append(resolution)
                    self.log.debug(
                        "%s starting process with args: %s" % (uid, str(args)))
                    timeout = self._timeoutprocess(args, timeout=120, uid=uid)

                    if os.path.isfile(tifname):
                        f = open(tifname, "rb")
                        tifresult = f.read()
                        f.close()
                        result["tif"] = base64.b64encode(tifresult)
                        self.log.debug("%s img_convert got tif-result" % (uid))
                    else:
                        self.log.error("%s img_convert no tif found" % (uid))
                        result["tif"] = None

                if createpng is True:
                    # ".bmp", ".png", ".gif", ".jpg" nach PNG
                    args = [self.filterv4bin, "-in=" + infile, "-format=png", "-out=" + pngname]
                    color = None

                    if pngoptions is not None:
                        options = pngoptions.split(",")
                        self.log.debug("%s pngoptions: %s" %
                                       (uid, str(options)))
                        for item in options:
                            key, val = item.split("=")
                            if key == "color":
                                if val.lower() in ["gray", "grey"]:
                                    color = "-gray"
                                elif val.lower() in ["bw", "schwarzweiss", "sw"]:
                                    color = "-bw"
                                elif val.lower() in ["color", "farbe"]:
                                    color = "-color"

                    if color is None:
                        color = DEFAULTPNGFILTEROPTIONS["color"]

                    args.append(color)
                    if ievfile is not None:
                        args.append("-anno="+ievfile)
                    self.log.debug(
                        "%s starting process with args: %s" % (uid, str(args)))
                    start = time.time()
                    timeout = self._timeoutprocess(args, timeout=120, uid=uid)
                    stop = time.time() - start
                    if os.path.isfile(pngname):
                        f = open(pngname, "rb")
                        pngresult = f.read()
                        f.close()
                        result["png"] = base64.b64encode(pngresult)
                        self.log.debug("%s img_convert got pngresult" % (uid))
                    else:
                        result["png"] = None

                if createbmp is True:
                    # Tif nach BMP
                    args = [self.filterv4bin, "-in=" + infile, "-format=bmp", "-out=" + bmpname]
                    color = None

                    if bmpoptions is not None:
                        options = bmpoptions.split(",")
                        self.log.debug("%s bmpoptions: %s" %
                                       (uid, str(options)))
                        for item in options:
                            key, val = item.split("=")
                            if key == "color":
                                if val.lower() in ["gray", "grey"]:
                                    color = "-gray"
                                elif val.lower() in ["bw", "schwarzweiss", "sw"]:
                                    color = "-bw"
                                elif val.lower() in ["color", "farbe"]:
                                    color = "-color"

                    if color is None:
                        color = DEFAULTBMPFILTEROPTIONS["color"]

                    args.append(color)
                    self.log.debug(
                        "%s starting process with args: %s" % (uid, str(args)))
                    timeout = self._timeoutprocess(args, timeout=120, uid=uid)
                    if os.path.isfile(bmpname):
                        f = open(bmpname, "rb")
                        bmpresult = f.read()
                        f.close()
                        result["bmp"] = base64.b64encode(bmpresult)
                        self.log.debug("%s img_convert got bmpresult" % (uid))
                    else:
                        result["bmp"] = None

                if createjpg is True:
                    # Tif nach PNG
                    args = [self.filterv4bin, "-in=" + infile, "-format=jpg", "-out=" + jpgname]
                    color = None
                    quality = None

                    if jpgoptions is not None:
                        options = jpgoptions.split(",")
                        self.log.debug("%s jpgoptions: %s" %
                                       (uid, str(options)))
                        for item in options:
                            key, val = item.split("=")
                            if key == "color":
                                if val.lower() in ["gray", "grey"]:
                                    color = "-gray"
                                elif val.lower() in ["bw", "schwarzweiss", "sw"]:
                                    color = "-bw"
                                elif val.lower() in ["color", "farbe"]:
                                    color = "-color"
                            elif key == "quality":
                                quality = "-compjpg="+val

                    if quality is None:
                        quality = DEFAULTJPGFILTEROPTIONS["quality"]
                    if color is None:
                        color = DEFAULTJPGFILTEROPTIONS["color"]

                    args.append(color)
                    args.append(quality)
                    self.log.debug(
                        "%s starting process with args: %s" % (uid, str(args)))
                    timeout = self._timeoutprocess(args, timeout=120, uid=uid)
                    if os.path.isfile(jpgname):
                        f = open(jpgname, "rb")
                        jpgresult = f.read()
                        f.close()
                        result["jpg"] = base64.b64encode(jpgresult)
                        self.log.debug("%s img_convert got jpgresult" % (uid))
                    else:
                        result["jpg"] = None
        self.log.debug("%s img_convert finished" % (uid))
        return result

    def kill_all_soffice(self, others=False):

        self.log.debug("killing all libreoffice instances")
        for x in self.soffice_instances:
            self.log.debug("killing libreoffice listening on port %d" % x[0])
            x[1].kill()
            try:
                shutil.rmtree(x[2])
            except:
                pass
        self.soffice_instances = []
        if others is True:
            cmd = ["taskkill", "/F", "/IM", "soffice.exe"]
            try:
                subprocess.check_call(cmd)
            except:
                pass

    def office_convert(self, uid, srcfile, tempdir, outtypes, **params):
        self.log.info("%s starting office_convert" % uid)
        self.log.debug("%s office_convert srcfile: %s" % (uid, srcfile))
        self.log.debug("%s office_convert tempdir: %s" % (uid, tempdir))
        self.log.debug("%s office_convert outtypes: %s" % (uid, outtypes))
        self.log.debug("%s office_convert params: %s" % (uid, params))
        port = self.sofficequeue.get()
        self.log.debug("%s office_convert soffice-port: %d" % (uid, port))

        ooargs = [self.pythonpath, self.ooextract, "-c",
                  "socket,host=localhost,port=%d" % port]
        self.log.debug("1")
        ooargs.append("--pdf")
        if "txt" in outtypes:
            ooargs.append("--txt")
        if "html" in outtypes:
            ooargs.append("--html")
        self.log.debug("2")
        # srcfile kann mytif aus request-body sein oder eine vorhandene Datei
        if type(srcfile) is not str:
            self.log.debug("3")
            srcfile_ext = os.path.splitext(
                os.path.basename(srcfile.filename))[1].lower()
            self.log.debug("4")
            fd, pth = tempfile.mkstemp(dir=tempdir, suffix=srcfile_ext)
            self.log.debug("5")
            os.close(fd)
            self.log.debug("6")
            infile = self._save_inputfile(srcfile, pth=pth, uid=uid)
            self.log.debug("7")
        else:
            infile = srcfile
#        else:
#            self.log.debug("8")
#            shutil.copyfile(infile, os.path.join(tempdir, os.path.basename(infile)))
#            self.log.debug("9")
#            infile = os.path.join(tempdir, os.path.basename(infile))
#            self.log.debug("10")
        self.log.debug("11")
        if infile is None:
            self.log.error("%s could not save infile" % uid)
            return {"error": "could not save infile"}
        self.log.debug("12")
        ooargs.append(infile)
        self.log.debug("13")
        self.log.info("%s office_convert starting soffice-conversion" % (uid))
        self.log.debug("%s office_convert client args: %s" %
                       (uid, " ".join(ooargs)))
        self.log.debug("14")
        ret = self._timeoutprocess(ooargs, timeout=480, uid=uid)
        self.log.debug("15")
        if ret is False:
            self.log.error(
                "%s office_convert timeout, restarting soffice process" % (uid))
            startnew = self.restart_soffice(port)
            if startnew is True:
                self.log.info("%s office_convert restart successs" % (uid))
            else:
                self.log.critical("%s office_convert restart failed" % (uid))
            return {"error": "timeout while converting office document"}
        else:
            self.log.info("%s soffice terminated gracefully" % (uid))

        pdfname = os.path.splitext(infile)[0] + u".pdf"
        txtname = os.path.splitext(infile)[0] + u".txt"
        htmlname = os.path.splitext(infile)[0] + u".html"

        if not os.path.isfile(pdfname):
            self.log.error(
                "%s could not create pdf, but need pdf to continue" % (uid))
            return {"error": "could not create pdf, but need pdf to continue"}

        # Instanz wieder freigeben
        self.sofficequeue.put(port)

        result = {}
        if "txt" in outtypes:
            if os.path.isfile(txtname):
                self.log.info("%s txt %s created" % (uid, txtname))
                f = open(txtname, "rb")
                txtresult = f.read()
                f.close()
                if txtresult != "":
                    self.log.debug(
                        "%s soffice textextraction got result (size %d)" % (uid, len(txtresult)))
                    result["txt"] = base64.b64encode(txtresult)
                else:
                    txtresult = None
            if not os.path.isfile(txtname) or txtresult is None:
                self.log.error("%s soffice txt-conversion failed!" % (uid))
                # try txt-extraction via pdf
                if os.path.isfile(pdfname):
                    self.log.debug(
                        "%s trying txt-conversion via pdftotext" % (uid))
                    pdftotextargs = [self.pdf2txtbin, pdfname, txtname]
                    self.log.debug("%s starte pdf2txtbin: %s" %
                                   (uid, " ".join(pdftotextargs)))

                    ret = self._timeoutprocess(
                        pdftotextargs, timeout=120, uid=uid)
                    if ret is True:
                        if os.path.isfile(txtname):
                            self.log.info("%s txt %s created" % (uid, txtname))
                            f = open(txtname, "rb")
                            txtresult = f.read()
                            f.close()
                            result["txt"] = base64.b64encode(txtresult)
                        else:
                            self.log.error(
                                "%s pdf2txt failed, no textfile" % (uid))
                            result["txt"] = None
                    else:
                        self.log.error("%s timeout pdf2txt" % (uid))
                        result["txt"] = None
                else:
                    self.log.error("%s pdf2txt failed, no pdf" % (uid))
                    result["txt"] = None

        if "html" in outtypes:
            if os.path.isfile(htmlname):
                f = open(htmlname, "rb")
                htmlresult = f.read()
                f.close()
                result["html"] = base64.b64encode(htmlresult)
                self.log.info("%s html %s created" % (uid, htmlname))
            else:
                self.log.error(
                    "%s office_convert could not create html" % (uid))
                result["html"] = None

        if "pdf" in outtypes:
            if os.path.isfile(pdfname):
                f = open(pdfname, "rb")
                pdfresult = f.read()
                f.close()
                result["pdf"] = base64.b64encode(pdfresult)
                self.log.info("%s pdf %s created" % (uid, pdfname))
            else:
                self.log.error(
                    "%s office_convert could not create pdf" % (uid))
                result["pdf"] = None
                pdfresult = None

        # if "tif" in outtypes or "bmp" in outtypes or "png" in outtypes or "jpg" in outtypes:
        if "tif" in outtypes:
            pdfconvertresult = self.pdf_convert(
                uid, pdfname, tempdir, "tif", **params)
            if "error" not in pdfconvertresult:
                if "tif" in pdfconvertresult:
                    result["tif"] = pdfconvertresult["tif"]
        return result

    def pdf_convert(self, uid, srcfile, tempdir, outtypes, **params):
        self.log.info("%s starting pdf_convert" % uid)

        if "txt" in outtypes:
            createtxt = True
        else:
            createtxt = False

        if "html" in outtypes:
            createhtml = True
        else:
            createhtml = False

        if "tif" in outtypes:
            createtif = True
            if "tifoptions" in params:
                self.log.debug("%s pdfconvert tifoptions vor _create_tif_options: %s" % (
                    uid, params["tifoptions"]))
                tifoptions = self._create_tif_options(params["tifoptions"])
                self.log.debug(
                    "%s pdfconvert tifoptions nach _create_tif_options: %s" % (uid, tifoptions))
            else:
                tifoptions = PDF_DEFAULTOPTIONS["tif"]
        else:
            createtif = False

        if "jpg" in outtypes:
            createjpg = True
            if "jpgoptions" in params:
                jpgoptions = self._create_jpg_options(params["jpgoptions"])
            else:
                jpgoptions = PDF_DEFAULTOPTIONS["jpg"]
        else:
            createjpg = False

        if "bmp" in outtypes:
            createbmp = True
            if "bmpoptions" in params:
                bmpoptions = self._create_bmp_options(params["bmpoptions"])
            else:
                bmpoptions = PDF_DEFAULTOPTIONS["bmp"]
        else:
            createbmp = False

        if "png" in outtypes:
            createpng = True
            if "pngoptions" in params:
                pngoptions = self._create_png_options(params["pngoptions"])
                #pngoptions = params["pngoptions"]
            else:
                pngoptions = PDF_DEFAULTOPTIONS["png"]
        else:
            createpng = False

        if "pdf" in outtypes:
            createpdf = True
        else:
            createpdf = False

        # srcfile kann mytif aus request-body sein oder eine vorhandene Datei
        if type(srcfile) is not str:
            srcfile_ext = os.path.splitext(
                os.path.basename(srcfile.filename))[1].lower()
            fd, pth = tempfile.mkstemp(dir=tempdir, suffix=srcfile_ext)
            os.close(fd)
            infile = self._save_inputfile(srcfile, pth=pth, uid=uid)
        else:
            infile = srcfile

        if infile is None:
            self.log.error("%s could not save infile" % uid)
            return {"error": "could not save infile"}

        result = {}
        if "jpg" in outtypes:
            #jpgname = os.path.splitext(infile)[0] + ".jpg"
            jpgname = os.path.join(tempdir, "temp.jpg")
            pdf2tifargs = [self.pdf2tifbin, "-i", infile]
            pdf2tifargs.append("-o")
            pdf2tifargs.append(jpgname)
            pdf2tifargs.extend(self._options2gsargs(jpgoptions, "jpg"))

            self.log.debug("%s starte pdfconvert: %s" %
                           (uid, " ".join(pdf2tifargs)))
            ret = self._timeoutprocess(pdf2tifargs, timeout=300, uid=uid)
            self.log.debug("%s pdfconvert done" % (uid))
            if ret is True:
                if os.path.isfile(jpgname):
                    f = open(jpgname, "rb")
                    jpgresult = f.read()
                    f.close()
                    result["jpg"] = base64.b64encode(jpgresult)
                else:
                    self.log.error(
                        "%s pdfconvert konnte kein jpg erzeugen" % (uid))
                    jpgresult = None
            else:
                self.log.error("%s PDF-conversion: no jpg" % (uid))
                result["jpg"] = None
        if "bmp" in outtypes:
            #bmpname = os.path.splitext(infile)[0] + ".bmp"
            bmpname = os.path.join(tempdir, "temp.bmp")
            pdf2tifargs = [self.pdf2tifbin, "-i", infile]
            pdf2tifargs.append("-o")
            pdf2tifargs.append(bmpname)
            pdf2tifargs.extend(self._options2gsargs(bmpoptions, "bmp"))
            self.log.debug("%s starte pdfconvert: %s" %
                           (uid, " ".join(pdf2tifargs)))

            ret = self._timeoutprocess(pdf2tifargs, timeout=300, uid=uid)
            self.log.debug("%s pdfconvert done" % (uid))
            if ret is True:
                if os.path.isfile(bmpname):
                    self.log.debug("%s bmp found" % (uid))
                    f = open(bmpname, "rb")
                    bmpresult = f.read()
                    f.close()
                    self.log.debug("%s bmp read" % (uid))
                    result["bmp"] = base64.b64encode(bmpresult)
                else:
                    self.log.error(
                        "%s pdfconvert konnte kein bmp erzeugen" % (uid))
                    bmpresult = None
            else:
                self.log.error("%s PDF-conversion: no bmp" % (uid))
                result["bmp"] = None
        if "png" in outtypes:
            #pngname = os.path.splitext(infile)[0] + ".png"
            pngname = os.path.join(tempdir, "temp.png")
            pdf2tifargs = [self.pdf2tifbin, "-i", infile]
            pdf2tifargs.append("-o")
            pdf2tifargs.append(pngname)
            self.log.debug("%s pngoptions: %s" % (uid, str(pngoptions)))
            pdf2tifargs.extend(self._options2gsargs(pngoptions, "png"))
            self.log.debug("%s starte pdfconvert: %s" %
                           (uid, " ".join(pdf2tifargs)))

            ret = self._timeoutprocess(pdf2tifargs, timeout=300, uid=uid)
            self.log.debug("%s  pdfconvert done" % (uid))
            if ret is True:
                if os.path.isfile(pngname):
                    self.log.debug("%s png found" % (uid))
                    f = open(pngname, "rb")
                    pngresult = f.read()
                    f.close()
                    self.log.debug("%s png read (%d)" % (uid, len(pngresult)))
                    result["png"] = base64.b64encode(pngresult)
                else:
                    self.log.error(
                        "%s pdfconvert konnte kein png erzeugen" % (uid))
                    pngresult = None
            else:
                self.log.error("%s PDF-conversion: no png" % (uid))
                result["png"] = None

        if "tif" in outtypes:
            #tifname = os.path.splitext(infile)[0] + ".tif"
            tifname = os.path.join(tempdir, "temp.tif")
            pdf2tifargs = [self.pdf2tifbin, "-i", infile]
            pdf2tifargs.append("-o")
            pdf2tifargs.append(tifname)
            self.log.debug("%s pdfconvert tifoptions: %s" % (uid, tifoptions))
            pdf2tifargs.extend(self._options2gsargs(tifoptions, "tif"))
            self.log.debug("%s starte pdfconvert: %s" %
                           (uid, " ".join(pdf2tifargs)))

            ret = self._timeoutprocess(pdf2tifargs, timeout=300, uid=uid)
            self.log.debug("%s pdfconvert done" % (uid))
            if ret is True:

                if os.path.isfile(tifname):
                    self.log.debug("%s tif found" % (uid))
                    f = open(tifname, "rb")
                    tifresult = f.read()
                    f.close()
                    self.log.debug("%s tif read" % (uid))
                    result["tif"] = base64.b64encode(tifresult)
                else:
                    self.log.error(
                        "%s pdfconvert konnte kein Tif erzeugen" % (uid))
                    tifresult = None
            else:
                self.log.error("%s PDF-conversion: no tif" % (uid))
                result["tif"] = None

        if "txt" in outtypes:
            self.log.debug("%s trying PDF-text-extraction" % (uid))
            #txtname = os.path.splitext(infile)[0] + ".txt"
            txtname = os.path.join(tempdir, "temp.txt")
            pdftotextargs = [self.pdf2txtbin, infile, txtname]
            self.log.debug("%s starte pdf2txtbin: %s" %
                           (uid, " ".join(pdftotextargs)))
            p = subprocess.Popen(pdftotextargs, stdout=subprocess.PIPE)

            p.communicate()
            p.wait()
            self.log.debug("%s pdf2txtbin finished" % (uid))
            if os.path.isfile(txtname):
                f = open(txtname, "rb")
                txtresult = f.read()
                # TODO: Ergebnis checken, ggfs. ocrclient anschmeissen
                f.close()
                if txtresult != "" and len(txtresult.replace("\f", "")) > 1:
                    self.log.debug("%s PDF-text-extraction: success" % (uid))
                    result["txt"] = base64.b64encode(txtresult)
                else:
                    self.log.error("%s PDF-text-extraction: no text" % (uid))
                    result["txt"] = None
            else:
                self.log.error("%s PDF-text-extraction: no text" % (uid))
                result["txt"] = None

        return result

    def start_all_soffice(self):
        self.sofficequeue.empty()
        for port in self.ports:
            proc, env = self.startlistener(port)
            self.soffice_instances.append([port, proc, env])
            self.sofficequeue.put(port)

    def startlistener(self, port):
        d = self.tempdir
        envpath = tempfile.mkdtemp(dir=d).replace("\\", "/")
        cmd = [
            self.sofficebin,
            "-env:UserInstallation=file:///" +
            envpath,
            "--headless",
            "--accept=socket,host=localhost,port=%d;urp;" %
            port]
        cmdstring = " ".join(cmd)
        self.log.debug(
            "starting libreoffice instance, listening on port %d" % port)
        self.log.debug(cmdstring)
        p = subprocess.Popen(cmd)
        return [p, envpath]

    def restart_renderer(self):
        self.kill_all_soffice()
        self.start_all_soffice()

    def restart_soffice(self, restartport):
        oldindex = newproc = newenv = None
        i = 0
        for port, proc, env in self.soffice_instances:
            if port == restartport:
                oldindex = i
                try:
                    proc.kill()
                    shutil.rmtree(env)
                except:
                    self.log.critical(
                        "Office Instanz konnte nicht beendet werden!")
                    return False
                time.sleep(.5)
                newproc, newenv = self.startlistener(port)
                self.sofficequeue.put(port)
                break
            i += 1
        if oldindex is not None and newproc is not None and newenv is not None:
            self.soffice_instances[oldindex][1] = newproc
            self.soffice_instances[oldindex][2] = newenv
            return True
        else:
            self.log.error("Office Instanz konnte nicht beendet werden.")
            return False

    """
    def tif_convert(self, **kwargs):
        result = {}
        if "infile" in kwargs:
            infile = kwargs["infile"]
        else:
            return {"error": "infile needed"}
        if "createtxt" in kwargs:
            if kwargs["createtxt"] is True:
                args = ["tesseract", infile, "stdout", "-l", "deu"]
                p = subprocess.Popen(args, stdout=subprocess.PIPE)
                out = p.communicate()
                result["txt"] = base64.b64encode(out[0])
        if "createocrpdf" in kwargs:
            if kwargs["createocrpdf"] is True:
                args = [
                    "tiff2pdf",
                    "-o",
                    os.path.join(
                        os.path.dirname(infile),
                        "temp.pdf"),
                    infile]
                p = subprocess.Popen(args, stdout=subprocess.PIPE)
                out = p.communicate()
                p.wait()

                args = ["pdfsandwich", os.path.join(
                    os.path.dirname(infile), "temp.pdf"), "-lang", "deu"]
                p = subprocess.Popen(args, stdout=subprocess.PIPE)
                out = p.communicate()
                p.wait()
        return result    
    """


if __name__ == "__main__":
    # pass
    renderer = Renderer(r"modules.cfg")
    time.sleep(5)
    renderer.kill_all_soffice()
