import sqlite3
import os
import pickle
import sys
import base64
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.metrics.pairwise import linear_kernel
# import thottolib
import re

__version__ = "v3.6.1.0"
_VERSION = "0.1"

CREATE_SQL_SYS_CLASSDOCS = \
    """
CREATE TABLE sys_classdocs (
 gdocid integer PRIMARY KEY,
 classname text NOT NULL,
 doctxt text NOT NULL,
 rules BLOB NOT NULL
);
"""

CREATE_SQL_SYS_CLASSIFIER = \
    """
CREATE TABLE sys_classifier (
 matrix text NOT NULL,
 vectorizer text NOT NULL,
 labels text NOT NULL
);
"""


class NamingRule(object):
    def __init__(self, rules_dict={}):
        self.proplist = ["add_id", "replace_chars", "maxval", "wordsleft", "wordsright",
                         "thottotype", "thottosubtype", "txtvalue", "regex", "index", "search_regex", "position",
                         "value"]
        self.defaults = [None, [], False, [], [], None, None, None, None, None, None, None]
        for k, v in zip(self.proplist, self.defaults):
            if k in rules_dict:
                setattr(self, k, rules_dict[k])
            else:
                setattr(self, k, v)

    def create(self, text, fields):
        # sanitize text by removing linebreaks and multiple spaces
        txt = re.sub('\s+', ' ', text)
        # find all date-like objects in text
        dates = thottolib.find_dates_in_text(txt)
        # find all decimal-like objects in text
        decimals = thottolib.find_decimals_in_text(txt)

        fieldrules = {}
        # find all date-like objects in text
        dates = thottolib.find_dates_in_text(txt)
        # find all decimal-like objects in text
        decimals = thottolib.find_decimals_in_text(txt)
        fieldrules = {}
        rules = {}
        for field in fields:
            print(field, fields[field][1], type(fields[field][1]))
            if fields[field][1] in ("", None, []):  # ignore empty field
                continue

            # we have a value, now lets check if that value is present in txt
            # if the value is found, we store its start-end positions in positions
            # it is possible, that we have multiple occurances.
            positions = []
            if fields[field][0] in (0, 2, 3, 11):  # universal, auswahl mehrfach -> utf8 string
                value = fields[field][1]
                print(value)
                value_re = re.escape(value)
                startpos = 0
                cchars = r"\/_-.:,; "
                cchars_re = "[" + re.escape(cchars) + "]*"
                # cchars_re = "["+cchars+"]*"
                value_re = "[\s^]" + "(" + cchars_re.join([re.escape(c) for c in value]) + ")[\s$]"
                matches = re.findall(value_re, txt[startpos:])
                # print matches
                pos = 0
                for match in matches:
                    props = dict(list(zip(self.proplist, self.defaults)))

                    props["value"] = value
                    pos = txt.find(match, pos)
                    fromindex = pos
                    toindex = pos + len(match)
                    replchars = [x for x in cchars if x in match]
                    props["replace_chars"] = replchars
                    value_re = "[\s^]" + "(" + re.escape("".join(replchars)).join(
                        [re.escape(c) for c in value]) + ")[\s$]"
                    props["regex"] = value_re
                    props["txtvalue"] = match
                    # props["add_id"] = field
                    # props["type"] = fields[field][0]

                    words_left = [_f for _f in re.split("\s+", txt[:fromindex]) if _f]
                    words_left_count = min(len(words_left), 5)

                    words_right = [_f for _f in re.split("\s+", txt[toindex:]) if _f]
                    words_right_count = min(len(words_right), 5)
                    props["position"] = (fromindex, toindex)
                    props["wordsleft"] = words_left[-words_left_count:]
                    props["wordsright"] = words_right[:words_right_count]
                    props["search_regex"] = "\S+\s+" * words_left_count + "(" + "\S+\s+" * len(
                        re.split("\s", match)) + ")" + "\S+\s+" * words_right_count
                    print(props)
                    print(match)
                    positions.append(((pos, pos + len(match)), props, None))

            elif fields[field][0] in (1, 12):  # datetime
                value = fields[field][1]
                # check if any date found with thottolib matches the date-value
                indexes = [i for i, date in enumerate([row[0] for row in dates]) if date == fields[field][1]]
                for ind in indexes:
                    date, dtstr, (fromindex, toindex), dttype = dates[ind]
                    props = dict(list(zip(self.proplist, self.defaults)))
                    props["value"] = value
                    props["regex"] = thottolib.PDATA_DATETIME[dttype]["regex"]
                    props["txtvalue"] = dtstr
                    # props["add_id"] = field
                    # props["type"] = fields[field][0]
                    props["thottotype"] = "PDATA_DATETIME"
                    props["thottosubtype"] = dttype
                    props["position"] = (fromindex, toindex)
                    words_left = filter(None, re.split("\s+", txt[:fromindex]))
                    words_left_count = min(len(words_left), 5)

                    words_right = filter(None, re.split("\s+", txt[toindex:]))
                    words_right_count = min(len(words_right), 5)

                    props["wordsleft"] = words_left[-words_left_count:]
                    props["wordsright"] = words_right[:words_right_count]
                    props["search_regex"] = "\S+\s+" * words_left_count + "(" + "\S+\s+" * len(
                        re.split("\s", dtstr)) + ")" + "\S+\s+" * words_right_count
                    # print props
                    positions.append(((fromindex, toindex), props, None))

            elif fields[field][0] == 8:  # decimal.Decimal
                value = fields[field][1]
                # check if any date found with thottolib matches the date-value
                indexes = [i for i, dcm in enumerate([row[0] for row in decimals]) if dcm == value]
                for ind in indexes:
                    dcm, dcmstr, (fromindex, toindex), dcmtype = decimals[ind]
                    props = dict(list(zip(self.proplist, self.defaults)))
                    props["value"] = value
                    props["regex"] = thottolib.PDATA_DECIMAL[dcmtype]["regex"]
                    props["txtvalue"] = dcmstr
                    # props["add_id"] = field
                    # props["type"] = fields[field][0]
                    props["thottotype"] = "PDATA_DECIMAL"
                    props["maxval"] = max([x[0] for x in decimals]) == dcm
                    props["thottosubtype"] = dcmtype
                    words_left = filter(None, re.split("\s+", txt[:fromindex]))
                    words_left_count = min(len(words_left), 5)

                    words_right = filter(None, re.split("\s+", txt[toindex:]))
                    words_right_count = min(len(words_right), 5)
                    props["position"] = (fromindex, toindex)
                    props["wordsleft"] = words_left[-words_left_count:]
                    props["wordsright"] = words_right[:words_right_count]
                    props["search_regex"] = "\S+\s+" * words_left_count + "(" + "\S+\s+" * len(
                        re.split("\s", dcmstr)) + ")" + "\S+\s+" * words_right_count
                    positions.append(((fromindex, toindex), props, None))
                pass
            else:
                print(fields[field][0])
                pass
            # print positions
            if len(positions) > 0:
                if field not in rules:
                    rules[field] = {"type": fields[field][0], "value": str(fields[field][1]), "hits": []}
                for ((fromindex, toindex), props, _) in positions:
                    rules[field]["hits"].append(props)
        # pprint.pprint(fieldrules)
        # for field, matches in fieldrules.iteritems():
        #     fname = fields[field].name
        #     ftype = fields[field][0]
        #     for match in matches:
        #         ((start,end), props, _) = match
        # txtvalue = txt[start:end]
        # # print fname, txtvalue, regex

        # words_left = filter(None, re.split("\s+", txt[:start]))
        # words_left_count = min(len(words_left), 5)

        # words_right = filter(None, re.split("\s+", txt[end:]))
        # words_right_count = max(len(words_right), 5)

        # # print txt
        # # print words_left
        # # print words_left_count
        # # print words_right
        # # print words_right_count
        # left_re = "(\S+\s+)" * words_left_count
        # right_re = "(\S+\s+)" * words_right_count
        # if field not in rules:
        #     rules[field] = []
        # rules[field].append({"type":ftype, "thotto_type":thotto_type, "wordsleft":words_left[-words_left_count:], "wordsright":words_right[:words_right_count], "txtvalue":txtvalue, "regex":regex})

        # pprint(rules)
        # print rules
        return rules

    def applyrule(self, rules, txt, gdocid):
        # pprint.pprint(rules)
        result = {}
        doctxt = re.sub('\s+', ' ', txt)
        docwords = re.split("\s+", doctxt)
        for add_id in rules:

            print(add_id)
            oritype = rules[add_id]["type"]
            orival = rules[add_id]["value"]
            for hit in rules[add_id]["hits"][:1]:
                wordsleft = hit["wordsleft"]
                wordsright = hit["wordsright"]
                txtvalue = hit["txtvalue"]
                thottotype = hit["thottotype"]
                thottosubtype = hit["thottosubtype"]
                maxval = hit["maxval"]
                replace_chars = hit["replace_chars"]
                search_regex = hit["search_regex"]
                regex = hit["regex"]
                if oritype in (0, 2, 3):
                    leftpos = {}
                    for index, wd in enumerate(wordsleft[::-1], 1):
                        try:
                            wdpos = docwords.index(wd)
                        except:
                            continue
                        # goodpos.append(wdpos+index)
                        print(wordsleft)
                        print(docwords[wdpos + index - len(wordsleft):wdpos + index])
                        print(list(map(lambda x, y: x == y, wordsleft,
                                       docwords[wdpos + index - len(wordsleft):wdpos + index])))
                        if wdpos + index not in leftpos:
                            leftpos[wdpos + index] = []
                        leftpos[wdpos + index].append(sum(map(lambda x, y: x == y, wordsleft, docwords[
                                                                                              wdpos + index - len(
                                                                                                  wordsleft):wdpos + index])) / float(
                            len(wordsright)))
                    rightpos = {}
                    for index, wd in enumerate(wordsright, 0):
                        try:
                            wdpos = docwords.index(wd)
                        except:
                            continue
                        # goodpos.append(wdpos+index)
                        print(wordsright)
                        print(docwords[wdpos - index:wdpos + len(wordsright) - index])
                        print(list(map(lambda x, y: x == y, wordsright,
                                       docwords[wdpos - index:wdpos + len(wordsright) - index])))
                        if wdpos - index not in rightpos:
                            rightpos[wdpos - index] = []
                        rightpos[wdpos - index].append(sum(map(lambda x, y: x == y, wordsright, docwords[
                                                                                                wdpos - index:wdpos + len(
                                                                                                    wordsright) - index])) / float(
                            len(wordsright)))

                    print(leftpos)
                    print(rightpos)
                    beststart = None
                    bestend = None
                    bestval = 0
                    for k, v in list(leftpos.items()):
                        val = sum(v) / float(len(v))
                        if val > bestval:
                            bestval = val
                            beststart = k

                    bestval = 0
                    for k, v in list(rightpos.items()):
                        val = sum(v) / float(len(v))
                        if val > bestval:
                            bestval = val
                            bestend = k

                    print(beststart)
                    print(bestend)
                    rawvalue = " ".join(docwords[beststart:bestend])
                    newvalue = rawvalue
                    for c in replace_chars:
                        newvalue = newvalue.replace(c, "")
                    print(newvalue)
                    print(add_id)
                    if add_id not in result:
                        result[add_id] = []
                    result[add_id].append(newvalue)

                    print()
                elif oritype == 1:
                    # pprint.pprint(hit)
                    # Datum mit thoottype suchen
                    dates = thottolib.find_dates_in_text(doctxt)
                    bestdt = None
                    bestval = 0
                    for dt, matchtext, (start, end), subtype in dates:
                        word_left_pos = len(doctxt[:start].strip().split(" "))
                        word_right_pos = len(doctxt[:end].strip().split(" ")) + 1
                        wl = docwords[word_left_pos - len(wordsleft) + 1:word_left_pos + 1]
                        wr = docwords[word_right_pos:word_right_pos + len(wordsright)]

                        wl = docwords[word_left_pos - len(wordsleft):word_left_pos]
                        wr = docwords[word_right_pos - 1:word_right_pos + len(wordsright) - 1]

                        resultr = sum(map(lambda x, y: x == y, wordsright, wr)) / float(len(wordsright))
                        resultl = sum(map(lambda x, y: x == y, wordsleft, wl)) / float(len(wordsleft))
                        print(wl)
                        print(wordsleft)
                        print(resultl)
                        print(wr)
                        print(wordsright)
                        print(resultr)

                        print(matchtext)
                        if resultr + resultl > bestval:
                            bestdt = dt
                            bestval = resultl + resultr
                        print()
                    if bestdt is not None:
                        if add_id not in result:
                            result[add_id] = []
                        result[add_id].append(bestdt)
                elif oritype == 8:
                    # pprint.pprint(hit)
                    # Datum mit thoottype suchen
                    decs = thottolib.find_decimals_in_text(doctxt)
                    bestdec = None
                    bestval = 0
                    for dec, matchtext, (start, end), subtype in decs:
                        word_left_pos = len(doctxt[:start].strip().split(" "))
                        word_right_pos = len(doctxt[:end].strip().split(" ")) + 1
                        wl = docwords[word_left_pos - len(wordsleft):word_left_pos]
                        wr = docwords[word_right_pos - 1:word_right_pos + len(wordsright) - 1]
                        resultr = sum(map(lambda x, y: x == y, wordsright, wr)) / float(len(wordsright))
                        resultl = sum(map(lambda x, y: x == y, wordsleft, wl)) / float(len(wordsleft))
                        print(wl)
                        print(wordsleft)
                        print(resultl)
                        print(wr)
                        print(wordsright)
                        print(resultr)

                        print(matchtext)
                        if resultr + resultl > bestval:
                            bestdec = dec
                            bestval = resultl + resultr
                        print()
                    if bestdec is not None:
                        if add_id not in result:
                            result[add_id] = []
                        result[add_id].append(bestdec)
        return result


class BfaClassifier(object):
    def __init__(self, db=None):

        self.vectorizer = None
        self.labels = None
        self.train_matrix = None

        self.ref_labels = {}

        if db is None:
            if not os.path.isfile("classifier.db"):
                self.db = sqlite3.connect("classifier.db", check_same_thread=False)
                self.cursor = self.db.cursor()
                self.cursor.execute(CREATE_SQL_SYS_CLASSDOCS)
                self.cursor.execute(CREATE_SQL_SYS_CLASSIFIER)
                self.has_rules = False
            else:
                self.db = sqlite3.connect("classifier.db", check_same_thread=False)
                self.cursor = self.db.cursor()
                sql = "SELECT matrix, vectorizer, labels from sys_classifier;"
                result = self.cursor.execute(sql)
                row = self.cursor.fetchone()
                if row:
                    self.train_matrix, self.vectorizer, self.labels = [
                        pickle.loads(base64.b64decode(x)) for x in row]
                    self.has_rules = True
                else:
                    self.has_rules = False
        else:
            self.db = db
            # sql-alchemy db-objekt gibts keinen cursor...
            # self.cursor = self.db.cursor()
            sql = "SELECT matrix, vectorizer, labels from sys_classifier;"
            result = self.db.execute(sql)
            row = result.fetchone()
            if row:
                self.train_matrix, self.vectorizer, self.labels = [
                    pickle.loads(base64.b64decode(x)) for x in row]
                self.has_rules = True
                self._load_refs()
            else:
                self.has_rules = False

    def _load_refs(self, db=None):
        if db is not None:
            self.db = db
        self.ref_labels = {}
        sql = "SELECT gdocid, classname, rules from sys_classdocs"
        result = self.db.execute(sql)
        for row in result.fetchall():
            gdocid, classname, rules = row
            rules = pickle.loads(base64.b64decode(rules))
            self.ref_labels[gdocid] = [classname, rules]

    def get_classes(self):
        return self.ref_labels

    def remove_classdoc(self, gdocid, db=None):
        if db is not None:
            self.db = db
        sql = "DELETE FROM sys_classdocs WHERE gdocid = %s"
        self.db.execute(sql, (gdocid))

        sql = "SELECT COUNT(*) from sys_classdocs"
        result = self.db.execute(sql)
        cnt = result.fetchone()
        if cnt == 0:
            sql = "Truncate sys_classifier;"
            self.db.execute(sql)
            self.vectorizer = None
            self.train_matrix = None
            self.labels = None
            self.has_rules = False
        self._load_refs()
        self._calc_n_save()

    def classify(self, txt, top_n=5):
        if self.has_rules is False:
            return False

        test_matrix = self.vectorizer.transform([txt])
        cosine_similarities = linear_kernel(
            self.train_matrix, test_matrix).flatten()
        related_label_indices = [i for i in cosine_similarities.argsort()[::-1]]

        result = [(self.ref_labels[self.labels[index]][0], self.labels[index], float(cosine_similarities[index]))
                  for index in related_label_indices[0:top_n]]
        # print cosine_similarities
        # print result
        best = result[0]
        classname, gdocid, thresh = best
        # rules = self.ref_labels[gdocid][1]
        # naming = rules["ref"]["naming"]
        # nr = NamingRule()
        # nr_result = nr.applyrule(naming, txt, None)
        return result, None
        # naming_result = {}
        # for addid, add_rules in naming.iteritems():
        #     naming_result[addid] = []
        #     print addid
        #     nr = NamingRule()
        #     nr.applyrule(add_rules, txt, None)
        #     for rule in add_rules:
        #         naming_result[addid].append(self.get_bestmatch(rule, txt))
        # return (result, naming_result)

    def get_bestmatch(self, rule, text):
        print(rule)
        prewords = rule["pre"]
        postwords = rule["post"]
        position = rule["position"]
        strval = rule["strval"]
        words = re.split("\s+", text.strip())

        # prewords von links nach rechts gegen words laufen lassen, dabei beste Match Position merken
        bestmatch = -1
        startindex = -1
        for i in range(0, len(words) - len(prewords)):
            cmp_text = words[i:len(prewords) + i]
            matchcount = sum([1 if x[0] == x[1] else 0 for x in zip(cmp_text, prewords)])
            if matchcount > 0 and matchcount > bestmatch:
                bestmatch = matchcount
                startindex = i + 10
                print(matchcount)
                print(cmp_text)
                print(prewords)
                print()
        if startindex == -1:
            return None
        restwords = words[startindex:]
        bestmatch = -1
        endindex = -1
        for i in range(0, len(restwords) - len(postwords)):
            cmp_text = restwords[i:len(postwords) + i]
            # print cmp_text
            # print postwords
            # print [1 if x[0]==x[1] else 0 for x in zip(cmp_text, postwords)]
            matchcount = sum([1 if x[0] == x[1] else 0 for x in zip(cmp_text, postwords)])

            if matchcount > bestmatch:
                print(matchcount)
                print(cmp_text)
                print(postwords)
                print()
                bestmatch = matchcount
                endindex = i + startindex
        print(words[startindex:endindex])
        return words[startindex:endindex]

        # thottotype = rule["thotto_type"]

        return {}

    def create_naming_rules(self, value_dict, text, pre=10, post=10):
        rules = {}
        rule = NamingRule()
        rules = rule.create(text, value_dict)

        # decimals = thottolib.find_decimals_in_text(text)
        # dates = thottolib.find_dates_in_text(text)
        # for label, (ftype, value) in value_dict.iteritems():
        #     rules[label] = []

        #     if ftype in [0, 2, 3, 11]:
        #         pass
        #     elif ftype in [1, 12]:
        #         for row in dates:
        #             print row
        #             val, strval, (start, end), thotto_type = row
        #             if val == value:
        #                 # Treffer im Text, jetzt Worte davor und dahinter ermitteln
        #                 pretext = text[:start].strip()
        #                 posttext = text[end:].strip()
        #                 prewords = re.split("\s+", pretext)[-pre:]
        #                 postwords = re.split("\s+", posttext)[:post]
        #                 if len(prewords) < pre:
        #                     prewords = [None] * (pre - len(prewords)) + prewords
        #                 if len(postwords) < post:
        #                     postwords = postwords + [None] * (pre - len(postwords))
        #                 rules[label].append({"pre":prewords, "post":postwords, "strval":strval, "val":value, "thotto_type":thotto_type, "position":(start, end)})
        #     elif ftype in [8]:
        #         for row in decimals:
        #             print row
        #             val, strval, (start, end), thotto_type = row
        #             if val == value:
        #                 # Treffer im Text, jetzt Worte davor und dahinter ermitteln
        #                 pretext = text[:start].strip()
        #                 posttext = text[end:].strip()
        #                 prewords = re.split("\s+", pretext)[-pre:]
        #                 postwords = re.split("\s+", posttext)[:post]
        #                 if len(prewords) < pre:
        #                     prewords = [None] * (pre - len(prewords)) + prewords
        #                 if len(postwords) < post:
        #                     postwords = postwords + [None] * (pre - len(postwords))
        #                 rules[label].append({"pre":prewords, "post":postwords, "strval":strval, "val":value, "thotto_type":thotto_type, "position":(start, end)})

        return rules

    def add_class_doc(self, gdocid, text, classname, rules=None, calc=False, db=None):
        if db is not None:
            self.db = db
        if type(self.db) is sqlite3.Connection:
            sql = "INSERT into sys_classdocs (gdocid, classname, doctxt, rules) VALUES (?, ?, ?, ?)"
        else:
            sql = "INSERT into sys_classdocs (gdocid, classname, doctxt, rules) VALUES (%s, %s, %s, %s)"

        try:
            self.db.execute(sql, (gdocid, classname, text, base64.b64encode(pickle.dumps(rules))))
            # self.db.commit()
        except Exception as e:
            print(e)
            return False
        # if calc is True:
        if calc == 'calc':
            self._calc_n_save()

    def _calc_n_save(self, db=None):
        if db is not None:
            self.db = db
        corpus = []
        sql = "SELECT gdocid, classname, doctxt FROM sys_classdocs"
        result = self.db.execute(sql)
        rows = result.fetchall()
        if len(rows) == 0:
            self.vectorizer = None
            self.train_matrix = None
            self.labels = None
            self.has_rules = False
            if type(self.db) is sqlite3.Connection:
                sql = "Delete from sys_classifier;"
            else:
                sql = "Truncate sys_classifier;"
            self.db.execute(sql)
            return
        for row in rows:
            gdocid, label, txt = row
            corpus.append((gdocid, txt))

        self.vectorizer = TfidfVectorizer(analyzer='word', ngram_range=(1, 3), min_df=0)
        self.train_matrix = self.vectorizer.fit_transform([txt for label, txt in corpus])
        self.labels = [label for label, txt in corpus]
        dblabels = base64.b64encode(pickle.dumps([label for label, txt in corpus]))
        dbmatrix = base64.b64encode(pickle.dumps(self.train_matrix))
        dbvectorizer = base64.b64encode(pickle.dumps(self.vectorizer))
        if type(self.db) is sqlite3.Connection:
            sql = "Delete from sys_classifier;"
        else:
            sql = "Truncate sys_classifier;"
        self.db.execute(sql)
        # self.db.commit()
        # self.cursor.execute(CREATE_SQL_SYS_CLASSIFIER)
        # self.db.commit()
        if type(self.db) is sqlite3.Connection:
            sql = "INSERT INTO sys_classifier (matrix, vectorizer, labels) VALUES (?, ?, ?)"
        else:
            sql = "INSERT INTO sys_classifier (matrix, vectorizer, labels) VALUES (%s,%s,%s)"
        self.db.execute(sql, (dbmatrix, dbvectorizer, dblabels))
        # self.db.commit()
        self.has_rules = True
        self._load_refs()
        return True


if __name__ == "__main__":
    import codecs
    import glob
    import pprint
    import argparse

    parser = argparse.ArgumentParser()
    parser.add_argument("-t", dest="train", help="train mode, lern new classdocs", action='store_true')
    parser.add_argument("-t2", dest="train2", help="train mode 2, lern new classdocs in subfolder per class", action='store_true')
    parser.add_argument("-c", dest="classify", help="classify mode, classify new docs", action='store_true')
    parser.add_argument("-d", dest="dir", help="directory with textfiles to train/classify.\nIn train mode, the txt base-filename will be used as the new classname", type=str)
    parser.add_argument("--dbfile", dest="dbfile", type=str, default=os.path.join(os.path.dirname(sys.argv[0]), "classifier.db"))
    args = parser.parse_args()

    if not os.path.isfile(args.dbfile):
        print("creating new db")
        db = sqlite3.connect(args.dbfile, check_same_thread=False)
        cursor = db.cursor()
        cursor.execute(CREATE_SQL_SYS_CLASSDOCS)
        cursor.execute(CREATE_SQL_SYS_CLASSIFIER)
        db.commit()
    else:
        print("using existing db")
        db = sqlite3.connect(args.dbfile, check_same_thread=False)

    c = BfaClassifier(db=db)

    if args.train is True:
        for i, fn in enumerate(glob.glob(os.path.join(args.dir, "*.txt"))):
            classname = os.path.basename(fn)
            print("adding class", os.path.basename(fn))
            with codecs.open(fn, "r", encoding="cp1252") as doc:
                c.add_class_doc(i, doc.read(), classname)
        c._calc_n_save()
        db.commit()
        db.close()
    elif args.train2 is True:
        c.vectorizer = None
        c.train_matrix = None
        c.labels = None
        c.has_rules = False
        corpus = []
        c.ref_labels = {}
        dirs = os.listdir(args.dir)
        for i, dir in enumerate(dirs):
            c.ref_labels[i] = dir
            for fn in glob.glob(os.path.join(args.dir, dir, "*.txt")):
                classname = i
                print("adding class", i)
                with codecs.open(fn, "r", encoding="cp1252") as doc:
                    corpus.append((i, doc.read()))

        c.vectorizer = TfidfVectorizer(analyzer='word', ngram_range=(1, 3), min_df=0)
        c.train_matrix = c.vectorizer.fit_transform([txt for label, txt in corpus])
        c.labels = [label for label, txt in corpus]
        dblabels = base64.b64encode(pickle.dumps([label for label, txt in corpus]))
        dbmatrix = base64.b64encode(pickle.dumps(c.train_matrix))
        dbvectorizer = base64.b64encode(pickle.dumps(c.vectorizer))
        if type(db) is sqlite3.Connection:
            sql = "Delete from sys_classifier;"
        else:
            sql = "Truncate sys_classifier;"
        db.execute(sql)
        db.commit()
        if type(db) is sqlite3.Connection:
            sql = "Delete from sys_classdocs;"
        else:
            sql = "Truncate sys_classdocs;"
        db.execute(sql)
        db.commit()

        # self.cursor.execute(CREATE_SQL_SYS_CLASSIFIER)
        # self.db.commit()
        if type(db) is sqlite3.Connection:
            sql = "INSERT INTO sys_classifier (matrix, vectorizer, labels) VALUES (?, ?, ?)"
        else:
            sql = "INSERT INTO sys_classifier (matrix, vectorizer, labels) VALUES (%s,%s,%s)"
        db.execute(sql, (dbmatrix, dbvectorizer, dblabels))
        db.commit()

        # self.db.commit()
        for x,y in c.ref_labels.items():
            if type(db) is sqlite3.Connection:
                sql = "INSERT INTO sys_classdocs (gdocid, classname, doctxt, rules) VALUES (?, ?, 'bla', ?)"
            else:
                sql = "INSERT INTO sys_classdocs (gdocid, classname, doctxt, rules) VALUES (%s, %s, 'bla', %s)"
            db.execute(sql, (x, y, base64.b64encode(pickle.dumps(None))))
        db.commit()

        has_rules = True
        c._load_refs()

        db.commit()
        db.close()
    elif args.classify is True:

        testfiles = glob.glob(os.path.join(args.dir, "*.txt"))
        for testfile in testfiles:

            with codecs.open(testfile, "r", encoding="cp1252") as f:
                txt = f.read()
                print("----------------------------------------------------")
                print("file ", os.path.basename(testfile))
                result = c.classify(txt)
                if result is False:
                    print("classifying failed")
                    continue
                best_matches, rules = result
                classname, classid, score = best_matches[0]
                classname2, classid2, score2 = best_matches[1]

                print("best match:", classname, score)
                print("second best match:", classname2, score2)

        # pprint.pprint(c.classify(txt)[0])
            # pprint.pprint(c.classify(txt)[1])
            # print
    else:
        print(parser.print_usage())
