﻿#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#       bfaDocLauncher.py
#
#
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

import os
import sys
import subprocess
import time
import datetime
import ctypes
import psutil
import logging
import logging.handlers

def is_locked(filepath):
    """Checks if a file is locked by opening it in append mode.
    If no exception thrown, then the file is not locked.
    """
    locked = None
    file_object = None
    # if os.path.exists(filepath):
    try:
        #print "Trying to open %s." % filepath
        buffer_size = 8
        # Opening file in append mode and read the first 8 characters.
        file_object = open(filepath, 'a', buffer_size)
        if file_object:
            #print "%s is not locked." % filepath
            locked = False
    except IOError as message:
        # print "File is locked (unable to open in append mode). %s." %  message
        locked = True
    finally:
        if file_object:
            file_object.close()
            #print "%s closed." % filepath
    #else:
        #print "%s not found." % filepath
    #print('is_locked %s' % str(locked))
    return locked


mypath = os.path.split(os.path.abspath(sys.argv[0]))[0]
logdir = os.environ['temp']
if not os.path.exists(logdir):
    logdir = mypath
try:
    pid = os.getpid()
except:
    pid = 0
log = logging.getLogger('bfaDocLauncher %d' % pid)

formatter = logging.Formatter('%(asctime)s %(name)-12s %(levelname)-8s %(message)s')
fh = logging.handlers.RotatingFileHandler(os.path.join(logdir, "bfaDocLauncher_%s.log" % (datetime.datetime.now().strftime('%Y-%d-%m %H-%M-%S'))), maxBytes=1024 * 1024, backupCount=10)
fh.setLevel(logging.DEBUG)
fh.setFormatter(formatter)
log.addHandler(fh)
log.setLevel(logging.DEBUG)

xml = None
socketlog = None
checkin = False
sslSwitch = True

try:
    params = " ".join([x for x in sys.argv[1:] if x != sys.argv[7]])
    log.debug("bfaDocLauncher v3.6.2.18840")
    log.debug("Parameter: %s" % params)
    profile = sys.argv[1]
    ip = sys.argv[2]
    port = int(sys.argv[3])
    userid = int(sys.argv[4])
    gdocid = int(sys.argv[5])
    fpath = sys.argv[6]
    password = sys.argv[7]
    if len(sys.argv) > 8:
        if sys.argv[8] == 'checkin':
            checkin = True
        if sys.argv[8] == 'nossl':
            sslSwitch = False
    if len(sys.argv) > 9:
        if sys.argv[9] == 'nossl':
            sslSwitch = False
except:
    log.exception("Fehler bei der Parameterübergabe")
    sys.exit(1)

msgfile = None
errorfile = None

if os.path.exists(os.path.join(mypath, 'bfaMessage.exe')):
    msgfile = os.path.join(mypath, 'bfaMessage.exe')
if os.path.exists(os.path.join(mypath, 'bfaLauncherError.exe')):
    errorfile = os.path.join(mypath, 'bfaLauncherError.exe')
errorproc = None
msgproc = None

if not os.path.exists(fpath):
    msg = "Datei konnte nicht gefunden werden."
    log.exception("%s Pfad %s" % (msg, fpath))
    ctypes.windll.user32.MessageBoxW(0, msg, 'Fehler', 0)
    sys.exit(1)

try:
    # pipe = subprocess.Popen(fpath, shell=False)
    # pipe.wait()
    # os.system('start %s' % fpath)

    pipe = subprocess.Popen(['cmd', '/C', fpath], shell=False)
    # pipe.wait()
except:
    msg = "Datei konnte nicht geöffnet werden."
    log.exception(msg)
    ctypes.windll.user32.MessageBoxW(0, msg, 'Fehler', 0)
    sys.exit(1)

if pipe is None:
    msg = "Fehler beim Datei öffnen."
    log.debug(msg)
    ctypes.windll.user32.MessageBoxW(0, msg, 'Fehler', 0)
    sys.exit(1)

mypid = None
try:
    startproc = psutil.Process(pipe.pid)
    startchilds = startproc.children(recursive=True)
except psutil.NoSuchProcess as err:
    log.exception('kein Startprozess 1')
except:
    log.exception('kein Startprozess 2')
else:
    mypid = pipe.pid
log.debug('mypid: %s' % str(mypid))
time.sleep(.2)      # kurz abwarten, damit der Prozess starten kann

fbase = os.path.basename(fpath)
fext = os.path.splitext(fbase)[1]
error = False
is_open = True
opened = False
opened_once = False         # das Dokument wurde einmal wenigstens gefunden, und zwar unter den geöffneten Dateien der Prozessüberwachung
i = 0
while not opened and i < 10:
    if is_locked(fpath):     # sollte eine anwendung die datei locken, dann brauchen wir den timeout nicht bis zum ende durchlaufen
        log.debug('file locked')
        break
    i += 1
    time.sleep(.2)

time.sleep(.2)      # kurz abwarten, ob nicht an anderen Prozess übergeben wird
if mypid:   # nochmal prüfen, ob der prozess noch vorhanden ist
    try:
        startproc = psutil.Process(mypid)
        startchilds = startproc.children(recursive=False)
        if startchilds:
            read_bytes = 0
            timeout = 0
            inactive = 0
            diff_bytes = 0
            while timeout < 200:
                if not startproc.is_running() and not startchilds[0].is_running():
                    log.debug('Startprozess beendet, Übergabe an andere Instanz (1)')
                    mypid = None
                    break
                if not psutil.pid_exists(mypid):
                    log.debug('Startprozess beendet, Übergabe an andere Instanz (2)')
                    mypid = None
                    break
                with startchilds[0].oneshot():
                    read_bytes = startchilds[0].io_counters()[2]
                    diff_bytes = read_bytes - diff_bytes
                    if diff_bytes > 0:      # wenn wir in dieser Runde mehr Bytes gelesen haben als zuvor
                        inactive = 0
                    else:
                        inactive += 1
                    log.debug('read_bytes %d  diff_bytes %d  inactive %d' % (read_bytes, diff_bytes, inactive))
                    diff_bytes = read_bytes     # wir merken uns den Wert dieser Runde
                    timeout += 1                # zur Sicherheit
                    if inactive > 10:           # eine Sekunde nichts mehr gelesen? raus hier
                        break
                    time.sleep(.1)

    except psutil.NoSuchProcess as err:
        log.debug('Startprozess beendet, Übergabe an andere Instanz (3)')
        mypid = None
    except:
        log.exception('Startprozess beendet, Übergabe an andere Instanz (4)')
        mypid = None

log.debug('Starte Überwachungsloop')
loop = 0
startchilds = None
mychild = None
proc = None
childproc = None

try:
    while is_open:
        loop += 1
        log.debug('loop: %s' % str(loop))
        is_open = False

        if mypid:
            try:
                if not psutil.pid_exists(mypid):
                    log.debug('Process nicht gefunden (1) %s' % str(mypid))
                    mypid = None
                    is_open = True  # vielleicht wurde doch an einem anderen Prozess übergeben. Damit wir wenigstens einmal auf ein file_lock prüfen > is_open = True
                    continue
                if not proc:
                    proc = psutil.Process(mypid)
                if not proc:
                    log.debug('Process nicht gefunden (2) %s' % str(mypid))
                    mypid = None
                    is_open = True  # vielleicht wurde doch an einem anderen Prozess übergeben. Damit wir wenigstens einmal auf ein file_lock prüfen > is_open = True
                    continue

                # print('proc.pid == %s' % str(mypid))
                is_open = True  # der Prozess ist noch offen
                opened = True
                found = False
                # myproc = psutil.Process(mypid)
                # mychilds = myproc.children(recursive=True)
                if not mychild:
                    mychilds = proc.children(recursive=True)
                    # print('Mychilds = %s' % str(len(mychilds)))
                    for child in mychilds:
                        found = False
                        # print('child.pid %s' % str(child.pid))
                        childproc = psutil.Process(child.pid)
                        try:
                            # print('vor open_files()')
                            open_files = childproc.open_files()
                            # print('nach open_files()')
                        except psutil.AccessDenied:
                            log.debug('Acces denied bei open_files')
                            continue
                        if not open_files:
                            log.debug('not open_files()')
                            continue
                        for open_file in open_files:
                            # print('open_file.path %s' % open_file.path)
                            if open_file.path == fpath:
                                found = True
                                mychild = child.pid
                                break
                        openfiles = None
                        if found:
                            break
                else:
                    if not psutil.pid_exists(mychild):
                        log.debug('mychild Process nicht gefunden (1) %s' % str(mychild))
                        continue
                    if not childproc:
                        childproc = psutil.Process(mychild)
                    if not childproc:
                        log.debug('mychild Process nicht gefunden (2) %s' % str(mychild))
                        continue
                    try:
                        # print('vor open_files() (2)')
                        open_files = childproc.open_files()
                        # print('nach open_files() (2)')
                    except psutil.AccessDenied:
                        log.debug('Acces denied bei open_files (2)')
                        continue
                    if not open_files:
                        log.debug('not open_files() (2)')
                        continue
                    for open_file in open_files:
                        #print('open_file.path (2) %s' % open_file.path)
                        if open_file.path == fpath:
                            found = True
                            break
                    openfiles = None

                if found:
                    # print('found')
                    if not opened_once:
                        log.debug('opened_once')
                    opened_once = True
                    opened = True
                    time.sleep(.4)

                else:       # wurde der Pfad in den Process schonmal gefunden (opened_once) jetzt aber nicht mehr
                    log.debug('not found')
                    opened = not opened_once
                    is_open = not opened_once


            # This catches a race condition where a process ends
            # before we can examine its files
            except psutil.NoSuchProcess as err:
                log.debug('psutil.NoSuchProcess')
                is_open = True      # wir versuchen noch eine Runde
            except:
                log.exception('Unbekannter Fehler')
                is_open = True      # wir versuchen noch eine Runde

        if not is_open:
            log.debug('is_open= %s' % str(is_open))
            if is_locked(fpath):
                log.debug('is_locked %s' % fpath)
                is_open = True
                opened = True
                time.sleep(.5)
            elif not opened_once:   #sicherheitshalber nochmal prüfen, aber nur, wenn wir nicht über die Prozessüberwachung kommen (sondern nur über die file_locked Überwachung)
                log.debug('is not opened_once')
                for x in range(10):
                    time.sleep(.1)
                    if is_locked(fpath):
                        log.debug('is_locked %s' % fpath)
                        is_open = True
                        opened = True
                        break

        if not is_open and not opened and not opened_once:
            log.debug('error is True: is_open %s opened %s opened_once %s' % (str(is_open), str(opened), str(opened_once)))
            error = True
except:
    log.exception('Exception im Mainloop')

if error:
    if errorfile:
        args = [errorfile, fext]
        if checkin:
            args.append('checkin')
        errorproc = subprocess.Popen(args)
        errorproc.wait()
    else:
        msg = "Dieser Dateityp (%s) kann nicht automatisch zurückgespielt werden, wenn die dazugehörige Anwendung schon geöffnet war. Sie müssen die Datei nach der Bearbeitung manuell wieder einchecken." % fext
        log.error(msg)
        ctypes.windll.user32.MessageBoxW(0, msg, 'Fehler', 0)
else:
    log.debug('is_open %s opened %s opened_once %s' % (str(is_open), str(opened), str(opened_once)))
    log.debug("Datei geschlossen, sende jetzt zum Server")

    if not os.path.exists(fpath):
        msg = "Die Datei %s kann nicht gefunden werden.\nZurückspielen nicht möglich." % fpath
        log.error(msg)
        ctypes.windll.user32.MessageBoxW(0, msg, 'Fehler', 0)
        sys.exit(1)

    stats = os.stat(fpath)

    if stats.st_size == 0:
        msg = "Die Datei %s ist leer / 0 Kb groß.\nZurückspielen nicht möglich." % fpath
        log.error(msg)
        ctypes.windll.user32.MessageBoxW(0, msg, 'Fehler', 0)
        sys.exit(1)

    resyncproc = os.path.join(mypath, 'bfaResyncDoc.exe')
    if not os.path.exists(resyncproc):
        msg = "%s wurde nicht gefunden.\nZurückspielen nicht möglich." % resyncproc
        log.error(msg)
        ctypes.windll.user32.MessageBoxW(0, msg, 'Fehler', 0)
        sys.exit(1)

    args = [resyncproc, profile, str(ip), str(port), str(userid), str(gdocid), fpath, password]
    if checkin:
        args.append('checkin')
    if not sslSwitch:
        args.append('nossl')

    try:
        pipe = subprocess.Popen(args, shell=False)
    except:
        log.exception('Fehler bei bfaResyncDoc ')
    else:
        log.debug('ReSyncDoc gestartet...')
        sys.exit(0)
