#!/usr/bin/env python
# -*- coding: utf-8 -*-

import socket
import os, sys
import bitfarm_doc
import bitfarm_proc
import bitfarm_xml
# from requestHandler import *
from time import *
import argparse
import io
import warnings
import ssl

warnings.filterwarnings(action='ignore', category=UnicodeWarning)
warnings.filterwarnings(action='ignore', module='sqlalchemy')

if __name__ == "__main__":
    mypath = os.path.split(os.path.abspath(sys.argv[0]))[0]
    hostname = socket.getfqdn()
    parser = argparse.ArgumentParser(description=u"bitfarm Archiv Clienttool, um Dokumente aus der Datenbank zu holen. User muss über eine Vieweranmeldung authentifiziert sein")
    parser.add_argument("-p", "--profile", help=u"Name des Profils, an dem sich angemeldet werden soll", required=True)
    parser.add_argument("-u", "--user", help=u"Name des User, der das Dokument anfordert", required=True)
    parser.add_argument("-e", "--encryptedpass", help=u"Passwort des User, verschlüsselt", required=True)
    parser.add_argument("-g", "--gdocid", help=u"Gdocid des Documents", required=True)
    parser.add_argument("-s", "--source", help=u"Quelle des Dokument (docloc,oriloc)", choices=['docloc', 'oriloc'],  default='docloc')
    parser.add_argument("-f", "--filepath", help=u"Der Zielpfad für das Dokument (optional)")
    parser.add_argument("-r", "--revision", help=u"Version des Dokuments, nur bei oriloc (optional)", type=int, default = 0)
    parser.add_argument("-n", "--nossl", help=u"Serververbindung ohne SSL", type=bool, default = False)

    args = parser.parse_args()
    
    if args.revision != 0 and args.source != 'oriloc':
        args.revision = 0
        sys.stdout.write("Revision kann nur bei Originaldateien verwendet werden\n")

    try:
        con = bitfarm_proc.BitfarmCon(args.profile)
    except:
        sys.stdout.write("Konnte nicht mit Profil %s verbinden\n" % args.profile)
        sys.exit(1)
    else:
        sys.stdout.write("Mit Profil %s verbunden\n" % con.profile)

    buff_size = 1024
    split_size = int(1024*1024/2)

    if not args.nossl:
        context = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
        context.check_hostname = False
        context.verify_mode = ssl.CERT_NONE
        sock_nossl = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock = context.wrap_socket(sock_nossl, server_hostname=socket.gethostname())
        sock.connect((con.bfaserver, con.bfaserverport))
    else:
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.connect((con.bfaserver, con.bfaserverport))

    if sock is None:
        sys.stdout.write("Socket konnte nicht verbunden werden\n")
        sys.exit(7)
    # else:
    #     sys.stdout.write(str(sock) +"\n")
    
    con.create_db_engine(bind_tables=False)
    con.connect_db()
    user = con.read_user(name=args.user)
    if user is None:
        sys.stdout.write("User %s konnte nicht gefunden werden\n" % args.user)
        sys.exit(10)

    xmlstr = bitfarm_xml.create_xml_request(reqtype="reqAuthenticate", profile=con.profile, parameter=args.user, data=args.encryptedpass)
    try:
        sock.sendall(xmlstr)
    except:
        sys.stdout.write("Request konnte nicht versendet weden\n")
        sys.exit(2)
    nachricht = ''
    try:
        while nachricht[-9:] != '</bfaReq>':
            nachricht += sock.recv(1).decode()       # da die XML zusammen mit dem Stream kommt, müssen wir Byteweise lesen
            if nachricht == '':             # bis die XML komplett ist
                break
    except:
        nachricht = None

    if not nachricht:
        sys.stdout.write("Keine Nachricht vom Server empfangen\n")
        sys.exit(6)
    else:
        resp = bitfarm_xml.xml_to_object(nachricht)

        if resp.reqtype == 'respAuthError':
            sys.stdout.write("Nicht authentifiziert!\n")
            sys.exit(4)


    userid = 170
    sessionid = con._bitdecrypt(resp.sessionid)
    if args.source == 'oriloc':
        xmlstr = bitfarm_xml.create_xml_request(reqtype="reqOri", profile=con.profile, userid=user.id, gdocid=int(args.gdocid),revision=args.revision, sessionid=sessionid)
    elif args.source == 'docloc':
        xmlstr = bitfarm_xml.create_xml_request(reqtype="reqPreview", profile=con.profile, userid=user.id, gdocid=int(args.gdocid), sessionid=sessionid)

    try:
        sock.sendall(xmlstr)
    except:
        sys.stdout.write("Request konnte nicht versendet weden\n")
        sys.exit(2)
        
    nachricht = ''
    try:
        while nachricht[-9:] != '</bfaReq>':
            nachricht += sock.recv(1).decode()       # da die XML zusammen mit dem Stream kommt, müssen wir Byteweise lesen
            if nachricht == '':             # bis die XML komplett ist
                break
    except:
        nachricht = None
    datalen = 0
    ip = sock.getpeername()[0]
    if not nachricht:
        sys.stdout.write("Keine Nachricht vom Server empfangen\n")
        sys.exit(6)
    else:
        resp = bitfarm_xml.xml_to_object(nachricht)

        if resp.reqtype == 'respAuthError':
            sys.stdout.write("Nicht authentifiziert!\n")
            sys.exit(4)

        if resp.reqtype == 'respDenied':
            sys.stdout.write("Kein Zugriff aufs Dokument erlaubt!\n")
            sys.exit(4)

        if resp.reqtype == 'respDocError':
            sys.stdout.write("Dokument konnte nicht empfangen werden\n")
            sys.exit(8)

        if resp.reqtype == 'respDoc':
            datalen = int(resp.datalen)
        if args.filepath is None:
            args.filepath = resp.filename
        
        if datalen > 0:
            try:
                binarystream = io.BytesIO()

                while binarystream.tell() < datalen:
                    buff = sock.recv(buff_size)
                    binarystream.write(buff)
                sys.stdout.write("Daten empfangen\n")
            except:
                sys.stdout.write("Client scheint weg zu sein, Empfangen wird abgebrochen\n")
                sys.exit(3)

            try:
                binarystream.seek(0)
                f = open(args.filepath, "wb")
                while f.tell() < datalen:
                    f.write(binarystream.read(buff_size))
                f.close()
                binarystream.close()
                sys.stdout.write("Daten gespeichert in %s\n" % args.filepath)
            except:
                try:
                    f.close()
                    binarystream.close()
                except:
                    pass
                sys.stdout.write("Konnte Datei nicht speichern %s\n" % args.filepath)
        else:
            sys.stdout.write("Leeres Dokument empfangen (0 Bytes)\n")
            sys.exit(5)            
                    
    con.disconnect_db()
                    



