# -*- encoding: UTF-8 -*-
#
# Form based authentication for CherryPy. Requires the
# Session tool to be loaded.
#
from urllib import request
import cherrypy
# import urllib
# import socket
# import bitfarm_doc
# import bitfarm_baseobj
import bitfarm_xml
import bitfarm_errors
import os
# import cgi
import json
import inspect
import decimal
import logging
import time
import tempfile

serverlogger = logging.getLogger('serverlogger')

from bfaServerClient import BfaServerClient
from xml.dom import minidom
__version__ = "3.6.2.0"

SESSION_KEY = '_cp_username'



class DecimalEncoder(json.JSONEncoder):
    def default(self, o):
        if isinstance(o, decimal.Decimal):
            return str(float(o))
        elif isinstance(o, set):
            return list(o)
        return super(DecimalEncoder, self).default(o)


class ResponseWebHandler():
    
    def _printxml(self, webclient, response):
        parsed = minidom.parseString(webclient.xmlresponse)
        print(parsed.toprettyxml())
    
    def _savexml(self, webclient, response):
        parsed = minidom.parseString(webclient.xmlresponse)
        nicexml = parsed.toprettyxml()
        default = "%d.xml" %webclient.responseobject.gdocid
        filepath = input("Dateiname: (default %s): " %default)
        if filepath == "":
            filepath = default
        f = open(filepath, "wb")
        f.write(nicexml)
        f.close()
    
    def _openxml(self, webclient, response):
        parsed = minidom.parseString(webclient.xmlresponse)
        nicexml = parsed.toprettyxml()
        handle,  temppath = tempfile.mkstemp(suffix=".xml")
        os.close(handle)
        f = open(temppath, "wb")
        f.write(nicexml)
        f.close()
        os.system(temppath)
        os.remove(temppath)
        
        
    def _savebin(self, webclient, response):
        filepath = None
        if webclient.responseobject.filename:
            if webclient.responseobject.filename != "":
                filepath = input("Dateiname (default %s): " %webclient.responseobject.filename)
        if filepath == "":
            filepath = webclient.responseobject.filename
        while filepath in (None, ""):
            filepath = input("Dateiname: ")
        f = open(filepath, "wb")
        f.write(webclient.streamdata.read())
        f.close()
        
    def _openbin(self, webclient, response):
        filepath = None
        ext = None
        if webclient.responseobject.filename:
            if webclient.responseobject.filename != "":
                filepath = webclient.responseobject.filename
                ext = os.path.splitext(filepath)[1]
        if ext:
            handle,  temppath = tempfile.mkstemp(suffix=ext)
            os.close(handle)
            f = open(temppath, "wb")
            webclient.streamdata.seek(0)
            f.write(webclient.streamdata.read())
            webclient.streamdata.seek(0)
            f.close()
            os.system(temppath)
            os.remove(temppath)
    
    def handle_respAuth(self, webclient, response):
        serverlogger.debug("handle_respAuth")
        webclient.userid = response.userid
        webclient.loggedin = True
        return [True, 200, "", 0, ""]
    
    def handle_respDenied(self, webclient, response):
        serverlogger.debug("handle_respDenied")
        return [False, 403, "denied", 403, "denied"]

    def handle_respEditDenied(self, webclient, response):
        serverlogger.debug("handle_respEditDenied")
        return [False, 200, response.userid, 403, "denied"]
    
    def handle_respAccess(self, webclient, response):
        serverlogger.debug("handle_respAccess")
        doc = response.data   
        try:
            del doc.extloc
        except:
            pass
        try:
            del doc.oriloc
        except:
            pass
        try:
            del doc.docloc
        except:
            pass
        try:
            del doc.srcloc
        except:
            pass
        try:
            del doc.fulltxt
        except:
            pass
            
        doc = bitfarm_xml.obj_to_dict(doc)
        # 
        return [True, 200, doc, 0, ""]
        
            
    def handle_respAuthError(self, webclient, response):
        serverlogger.debug("handle_respAuthError")
        #serverlogger.debug("handle_respAuthError response %s" %str(response))
        if hasattr(response, "data"):
            #serverlogger.debug("handle_respAuthError response.data %s" %str(response.data))
            data = response.data
        else:
            data = webclient.xmlresponse
        return [False, 401, data, 401, "authentication error"]
    
    def handle_respUsers(self, webclient, response):
        serverlogger.debug("handle_respUsers")
        userdata = response.data

        if response.parameter:
            if "xml" in response.parameter:
                userdata = bitfarm_xml.obj_to_dict(userdata)

        for user in userdata:
            if "password" in user:
                del user["password"]
            if "grouplist" in user:
                del user["grouplist"]
            if "rights" in user:
                del user["rights"]
            if "role" in user:
                del user["role"]
        return [True, 200, userdata, 0, ""]
    
    def handle_respAddfields(self, webclient, response):
        serverlogger.debug("handle_respAddfields")
        if response.parameter:
            if "json" in response.parameter:
                
                return [True, 200, response.data, 0, ""]
            else:
                if response.data is None:
                    return [True, 200, [], 0, ""]
                # cleanresponse = []
                # for obj in response.data:
                #     cleanresponse.append(bitfarm_xml.obj_to_dict(obj))
                serverlogger.debug("handle_respAddfields obj_to_dict")
                # start = time.process_time()
                cleanresponse = bitfarm_xml.obj_to_dict(response.data)
                # serverlogger.info('process time object_to_dict %s' % str(time.process_time() - start))
                serverlogger.debug("handle_respAddfields return")
                return [True, 200, cleanresponse, 0, ""]
        if webclient.xmlresponse:
            cherrypy.response.headers['Content-Type'] = "application/xml"
            return [True, 200, webclient.xmlresponse, 0, ""]
        else:
            return [False, 500, ""]
            
    def handle_respHistory(self, webclient, response):
        serverlogger.debug("handle_respHistory")
        if response.parameter:
            if "json" in response.parameter:
                
                return [True, 200, response.data, 0, ""]
                
        if webclient.xmlresponse:
            cherrypy.response.headers['Content-Type'] = "application/xml"
            return [True, 200, webclient.xmlresponse, 0, ""]
        else:
            return [False, 500, "", 500, "unknown error"]
    
    def handle_respGetHoliday(self, webclient, response):
        serverlogger.debug("handle_respGetHoliday")
        if response.parameter:
            if "json" in response.parameter:
                obj = bitfarm_xml.object_to_xml(response.data)
                if response.data is None:
                    response.data = []
                
                return [True, 200, response.data, 0, ""]
                
        if webclient.xmlresponse:
            cherrypy.response.headers['Content-Type'] = "application/xml"
            return [True, 200, webclient.xmlresponse, 0, ""]
        else:
            return [False, 500, "", 500, "unknown error"]
    
    def handle_respGroups(self, webclient, response):
        serverlogger.debug("handle_respGroups")
        
        if response.parameter:
            if "json" in response.parameter:
                if response.data and type(response.data) == list:
                    cleanresponse = []
                    for group in response.data:
                        # groupdata = {"profile": group["profile"], "role": group["role"], "name": group["name"], "id": group["id"], "member": []}
                        groupdata = {"profile": group["profile"], "name": group["name"], "id": group["id"]}
                        if "members" in group:
                            groupdata["member"] = group["members"]
                        else:
                            groupdata["member"] = []
                        cleanresponse.append(groupdata)
                    
                    return [True, 200, cleanresponse, 0, ""]
                else:
                    return [False, 500, "", 500, "unknown error"]
            else:
                cleanresponse = []
                for obj in response.data:
                    # group = bitfarm_xml.obj_to_dict(obj)
                    # groupdata = {"profile": group["profile"], "name": group["name"], "id": group["id"]}
                    # if "members" in group:
                    #     groupdata["member"] = group["members"]
                    # else:
                    #     groupdata["member"] = []

                    groupdata = {"profile": obj.profile, "name": obj.name, "id": obj.id}
                    if obj.members:
                        groupdata["member"] = obj.members
                    else:
                        groupdata["member"] = []

                    cleanresponse.append(groupdata)
                
                return [True, 200, cleanresponse, 0, ""]
        else:
            if webclient.xmlresponse:
                cherrypy.response.headers['Content-Type'] = "application/xml"
                return [True, 200, webclient.xmlresponse, 0, ""]
            else:
                return [False, 500, "", 500, "unknown error"]
        return [False, 500, "", 500, "unknown error"]
        
    def handle_respArchives(self, webclient, response):
        serverlogger.debug("handle_respArchives")
        if response.parameter:
            if "json" in response.parameter:
                # obj = bitfarm_xml.object_to_xml(response.data)
                #
                # # cleanup object
                # cleanresponse = []
                # for arc in response.data:
                #     try:
                #         del arc["rights"]["_BitfarmRights__user"]
                #     except:
                #         pass
                #     cleanresponse.append(arc)
                
                
                # return [True, 200, json.dumps(cleanresponse, cls=DecimalEncoder)]
                return [True, 200, response.data, 0, ""]
            else:
                cleanresponse = []
                if response.data is None:
                    return [True, 200, [], 0, ""]
                # for obj in response.data:
                #     cleanresponse.append(bitfarm_xml.obj_to_dict(obj))
                cleanresponse = bitfarm_xml.obj_to_dict(response.data)
                return [True, 200, cleanresponse, 0, ""]

        if response.data is not None:
            for index, arc in enumerate(response.data):
                try:         
                    del response.data[index].rights._BitfarmRights__user.password    
                except:
                    pass
        else:
            return [True, 200, [], 0, ""]
        xml = bitfarm_xml.object_to_xml(response)
        webclient.xmlresponse = xml
        if webclient.xmlresponse:
            cherrypy.response.headers['Content-Type'] = "application/xml"
            return [True, 200, webclient.xmlresponse, 0, ""]
        else:
            return [False, 500, "", 500, "unknown error"]
    
    def handle_respSearch(self, webclient, response):
        serverlogger.debug("handle_respSearch")
        if response.parameter:
            if "json" in response.parameter:
                if response.data is None:
                    response.data = []
                if response.selected_cols is None:
                    response.selected_cols = []
                
                responsedata = {"docs": response.data, "selected_cols": response.selected_cols}
                data = responsedata
                try:
                    serverlogger.debug("handle_respSearch response.data: %s" % str(response.data))
                    serverlogger.debug("handle_respSearch sending json data: %s" % data)
                except:
                    pass
                return [True, 200, data, 0, ""]
            else:
                if response.data is None:
                    response.data = []
                if response.selected_cols is None:
                    response.selected_cols = []
                cleanresponse = []
                for obj in response.data:
                    cleanresponse.append(bitfarm_xml.obj_to_dict(obj))
                
                responsedata = {"docs": cleanresponse, "selected_cols": response.selected_cols}
                data = responsedata
                try:
                    serverlogger.debug("handle_respSearch response.data: %s" % str(response.data))
                    serverlogger.debug("handle_respSearch sending json data: %s" % data)
                except:
                    pass
                return [True, 200, data, 0, ""]

        if webclient.xmlresponse:
            cherrypy.response.headers['Content-Type'] = "application/xml"
            return [True, 200, webclient.xmlresponse, 0, ""]
        else:
            return [False, 500, "", 500, "unknown error"]
    
    def handle_respDocError(self, webclient, response):
        serverlogger.debug("handle_respDocError")
        if hasattr(response, "errortext") and response.errortext:
            return [False, 500, response.errortext, 500, "respDocError"]
        elif hasattr(response, "data"):
            return [False, 500, response.data, 500, "respDocError"]
        else:
            return [False, 500, "respDocError", 500, "respDocError"]
    
    def handle_respDoc(self, webclient, response):
        serverlogger.debug("handle_respDoc")
        try:
            webclient.streamdata.seek(0, os.SEEK_END)
            #print "webclient streamdata"
            if webclient.streamdata.tell()>0:
                #print "webclient streamdata.tell > 0 -> handle_default"
                return self.handle_default(webclient, response)
        except:
            pass
        if response.parameter:
            if "json" in response.parameter:
                #serverlogger.debug("response.data type: %s" %type(response.data))
                #serverlogger.debug("response.data: %s" %response.data)
                doc = response.data
               
                if "docloc" in doc:
                    del doc["docloc"]
                    serverlogger.debug("found docloc and deleted docloc")
                if "oriloc" in doc:
                    if doc["oriloc"].strip() !="":
                        doc["type"] = os.path.splitext(doc["oriloc"])[1][1:]
                    del doc["oriloc"]
                    serverlogger.debug("found oriloc and deleted oriloc")
                else:
                    doc["type"] = "tif"
                if "extloc" in doc:
                    del doc["extloc"]
                    serverlogger.debug("found extloc and deleted extloc")
                if "srcloc" in doc:
                    del doc["srcloc"]
                    serverlogger.debug("found srcloc and deleted srcloc")
                
                return [True, 200, doc, 0, ""]
            else:
                if response.data.oriloc != "":
                    response.data.type = os.path.splitext(response.data.oriloc)[1][1:]
                else:
                    response.data.type = "tif"
                del response.data.oriloc
                del response.data.docloc
                del response.data.extloc

                data = bitfarm_xml.obj_to_dict(response.data)

                return [True, 200, data, 0, ""]

        if response.data:
            if response.data.oriloc != "":
                response.data.type = os.path.splitext(response.data.oriloc)[1][1:]
            else:
                response.data.type = "tif"
            del response.data.oriloc
            del response.data.docloc
            del response.data.extloc
                
            cherrypy.response.headers['Content-Type'] = "application/xml"
            return [True, 200, bitfarm_xml.object_to_xml(response.data), 0, ""]
        else:
            return [False, 500, "handle_respDoc error", 500, "unknown error"]
    
    def handle_respTif(self, webclient, response):
        serverlogger.debug("handle_respTif")
        if webclient.streamdata:
            data = b''
            webclient.streamdata.seek(0)
            while True:
                buff = webclient.streamdata.read()
                if buff:
                    data += buff
                else:
                    break
            serverlogger.debug("handle_respTif streamdata length: %d" % len(data))
            cherrypy.response.headers['Content-Disposition'] = 'attachment; filename="%s"' % response.filename
            cherrypy.response.headers['Content-Type'] = "application/" + "tif"
            return [True, 200, data, 0, ""]
    
    def handle_respSendFile(self, webclient, response):
        return [True, 200, response.data, 0, ""]
    
    def handle_respOK(self, webclient, response):
        serverlogger.debug("handle_respOK")
        try:
            
            return [True, 200, True, 0, ""]
        except:
            return [False, 500, "unknown error", 500, "unknown error"]
 
    def handle_respLastModified(self, webclient, response):
        serverlogger.debug("handle_respLastModified")
        try:
            return [True, 200, response.modified_date, 0, ""]
        except:
            return [False, 500, "error handling respLastModified", 500, "unknown error"]
    
    def handle_respGetBookmarks(self, webclient, response):
        serverlogger.debug("handle_respGetBookmarks")
        if response.parameter:
            if "json" in response.parameter:
                if response.data is None:
                    response.data = []
                
                data = response.data
                try:
                    serverlogger.debug("handle_respGetBookmarks response.data: %s" % str(response.data))
                    serverlogger.debug("handle_respGetBookmarks sending json data: %s" % data)
                except:
                    pass
                return [True, 200, data, 0, ""]
        if webclient.xmlresponse:
            cherrypy.response.headers['Content-Type'] = "application/xml"
            return [True, 200, webclient.xmlresponse, 0, ""]
        else:
            return [False, 500, "", 500, "unknown error"]
            
    def handle_respError(self, webclient, response):
        serverlogger.debug("handle_respError")
        if webclient.xmlresponse:
            serverlogger.error(webclient.xmlresponse)
        if hasattr(response, "data"):
            data = response.data
            try:
                if type(data) in (str, str):
                    data = data
                else:
                    data = bitfarm_xml.obj_to_dict(data)
                return [False, 500, data, 500, "backend error"]
            except:
                pass
        else:
            data = "unknown error"
        return [False, 500, None, 500, "generic/unknown backend error"]

    def handle_respCancelDenied(self, webclient, response):
        serverlogger.debug("handle_respCancelDenied")
        # success, statuscode, result, errorcode, message 
        return [False, 200, "denied", 105, "cancel denied"]


    def handle_default(self, webclient, response):
        # default handler, used if theres no special handler found
        serverlogger.debug("handle_default")
        if webclient.streamdata:
            serverlogger.debug("response filename %s" %response.filename)
            data = b""
            webclient.streamdata.seek(0)
            while True:
                buff = webclient.streamdata.read()
                if buff:
                    data+=buff
                else:
                    break
            if response.datatype in ("png", "jpg"):
                cherrypy.response.headers['Content-Type'] = "image/"+response.datatype
            else:
                if response.filename:
                    ext = os.path.splitext(response.filename)[1]
                    cherrypy.response.headers['Content-Disposition'] = 'attachment; filename="%s"' %(cherrypy.session["bfaserverclient"].responseobject.data.title+ext)
                    cherrypy.response.headers['Content-Type'] = "application/"+ext[1:]
            return [True, 200, data, 0, ""]
            
        else:
            if "error" not in response.reqtype.lower():
                if webclient.xmlresponse:
                    cherrypy.response.headers['Content-Type'] = "application/xml"
                    return [True, 200, webclient.xmlresponse, 0, ""]
            else:
                parsed = minidom.parseString(webclient.xmlresponse)
                parsed = parsed.toprettyxml()
                # success, statuscode, result, errorcode, message 
                if hasattr(response, "data"):
                    data = response.data
                else:
                    data = "unknown error"
                return [False, 400, data, 400, data]
        return [False, 400, "", 400, "unknown error"]


def check_credentials(profile, username, password, auto_heartbeat=None):
    #print username, password
    webclient = BfaServerClient(auto_heartbeat=auto_heartbeat)
    webclient.load_profile(profile)
    webclient.responsehandler =  ResponseWebHandler()
    result = webclient.connect(username=username, userpass=password)
    
    return result, webclient
#    if result[0] is True:
#        return webclient
#    else:
#        return None
#    
    
    

def check_auth(*args, **kwargs):
    """A tool that looks in config for 'auth.require'. If found and it
    is not None, a login is required and the entry is evaluated as alist of
    conditions that the user must fulfill"""

    
    conditions = cherrypy.request.config.get('auth.require', None)
    # get_parmas = urllib.quote(cherrypy.request.request_line.split()[1])
    if conditions is not None:
        username = cherrypy.session.get(SESSION_KEY)
        serverlogger.debug("check_auth sessions:")
        serverlogger.debug(cherrypy.session)
        serverlogger.debug("check_auth user %s" %(username))
        #print "check_auth", cherrypy.session
        if username:
            cherrypy.request.login = username
            for condition in conditions:
                if not condition():
                    cherrypy.request.handler = None
                    cherrypy.response.status = 401
                    cherrypy.response.headers['Content-Type'] = "application/json"
                    cherrypy.response.body = json.dumps({'data': 'auth error', 'errorcode': 401, 'message': 'auth error', 'result': False}).encode('utf8')
                    # cherrypy.response.status = '401 Unauthorized'
                    # raise cherrypy.HTTPError("401 Unauthorized")
        else:
            #cherrypy.lib.sessions.expire()
            cherrypy.request.handler = None
            cherrypy.response.status = 401
            cherrypy.response.headers['Content-Type'] = "application/json"
            cherrypy.response.body = json.dumps({'data': 'auth error', 'errorcode': 401, 'message': 'auth error', 'result': False}).encode('utf8')
            # cherrypy.response.status = '401 Unauthorized'
            # raise cherrypy.HTTPError("401 Unauthorized")
    #cherrypy.session.regenerate()
    


cherrypy.tools.auth = cherrypy.Tool('before_handler', check_auth)

def require(*conditions):
    """A decorator that appends conditions to the auth.require config
    variable."""
    def decorate(f):
        if not hasattr(f, '_cp_config'):
            f._cp_config = dict()
        if 'auth.require' not in f._cp_config:
            f._cp_config['auth.require'] = []
        f._cp_config['auth.require'].extend(conditions)
        return f
    return decorate


# Conditions are callables that return True
# if the user fulfills the conditions they define, False otherwise
#
# They can access the current username as cherrypy.request.login
#
# Define those at will however suits the application.

def member_of(groupname):
    def check():
        # replace with actual check if <username> is in <groupname>
        return cherrypy.request.login == 'bitfarm' and groupname == 'admin'
    return check

def name_is(reqd_username):
    return lambda: reqd_username == cherrypy.request.login

# These might be handy

def any_of(*conditions):
    """Returns True if any of the conditions match"""
    def check():
        for c in conditions:
            if c():
                return True
        return False
    return check

# By default all conditions are required, but this might still be
# needed if you want to use it inside of an any_of(...) condition
def all_of(*conditions):
    """Returns True if all of the conditions match"""
    def check():
        for c in conditions:
            if not c():
                return False
        return True
    return check


# Controller to provide login and logout actions

class AuthController(object):

    def on_login(self, username):
        """Called on successful login"""
        pass

    def on_logout(self, username):
        """Called on logout"""
        pass


    @cherrypy.expose
    @cherrypy.tools.json_out()
    # @cherrypy.tools.json_in()
    def login(self, *args, **kwargs):
        params = cherrypy.request.params
        body = cherrypy.request.body.read()
    
        """
        3.6.2 mobile client:
        params: {}
        body: b'{"profile":"vmth5","username":"dmsadmin","password":""}'
        
        3.6.1 mobile client:
        params: {'profile': 'vmth5', 'username': 'dmsadmin', 'password': ''}
        body: b''
        """
        if cherrypy.request.headers["User-Agent"] == 'bitfarm_Archive_mobile_client':
            if body==b"":
                # return {"result": False, "errorcode": 3, "error":"please upgrade to version 2!", "data":{}}
                return [False, "login error", "please upgrade to version 2"]
           
            
        try:
            kwargs = json.loads(body)
        except:
            return {"result": False, "errorcode": 2, "message": "error loading json", "data":{}}
        try:
            profile=kwargs["profile"]
            username=kwargs["username"]
            password=kwargs["password"]
        except Exception as e:
            return {"result": False, "errorcode": 2, "message": "missing argument(s)", "data":{}}

        if username is None or password is None or profile is None:
            return {"result": False, "errorcode": 2, "message": "missing argument(s)", "data":{}}
            
        """
        for _id, (data, expiration_time) in cherrypy._cpcompat.copyitems(cherrypy.session.cache):
            knownusername = data.get(SESSION_KEY, None)
            if username == knownusername:
                serverlogger.debug("User already logged in!")
                return {"result": False, "errorcode": 11, "message": "User already logged in", "data":False}
        """
        try:
            auto_heartbeat = int(cherrypy.request.config.get("auto_heartbeat_interval"))
        except Exception as e:
            auto_heartbeat = 30
        try:
            loginresult, wc = check_credentials(profile, username, password, auto_heartbeat=auto_heartbeat)
        except bitfarm_errors.FindConError as e:
            return {"result": False, "errorcode": 1, "message": "unbekanntes Profil", "data":{}}

        
        if loginresult[0] is False:
            serverlogger.info("login from %s with user %s failed!" %(cherrypy.request.remote.ip, username))
            errorstring = loginresult[2]
            serverlogger.error(errorstring)
            if errorstring == "Das Passwort ist falsch.":
                errorcode = 2
            elif errorstring == "Der Benutzer ist nicht vorhanden.":
                errorcode = 3
            elif errorstring == "Der Benutzer wurde deaktiviert.":
                errorcode = 4
            elif errorstring == "Anmeldung mit dem Profil vmth4 an diesem Server nicht möglich":
                errorcode = 5
            else:
                errorcode = 6
            return {"result": False, "errorcode": errorcode, "message": errorstring, "data":{}}
        else:
            cherrypy.session.regenerate()
            cherrypy.session[SESSION_KEY] = cherrypy.request.login = username
            cherrypy.session['bfaserverclient']= wc
            serverlogger.info("successful login from %s with user %s" %(cherrypy.request.headers.get("X-REAL-IP", cherrypy.request.remote.ip), username))
            cherrypy.session['bfaserverclient'].clientmethods = list(filter((lambda x: x[0].startswith("get")), [(name, method) for name, method in inspect.getmembers(cherrypy.session['bfaserverclient'], inspect.ismethod)]))
            cherrypy.session['bfaserverclient'].clientmethodd = {key: value for (key, value) in cherrypy.session['bfaserverclient'].clientmethods}    
            self.on_login(username)
            return {"result": True, "errorcode": 0, "message": "login success", "data":{}}


    @cherrypy.expose
    @cherrypy.tools.json_out()
    def checklogin(self):
        # print "bfaHttpAuth checklogin called"
        sess = cherrypy.session
        username = sess.get(SESSION_KEY, None)
        if username:
            # print "if username ist True"
            result = sess["bfaserverclient"].test_socket()
            # print result
            serverlogger.info("checklogin %s: %s" %(username, str(result)))
            # print result
            return {"result": True, "errorcode": 0, "message": "loggedin", "data":True}
        else:
            # print("no username")
            return {"result": False, "errorcode": 0, "message": "not loggedin", "data":False}
        
        
    @cherrypy.expose
    @cherrypy.tools.json_out()
    def logout(self, from_page="/"):
        # print "bfaHttpAuth logout called"
        sess = cherrypy.session
        username = sess.get(SESSION_KEY, None)
        
        if username:
            sess[SESSION_KEY] = None
            cherrypy.session['usr_id']=None
            cherrypy.session['_cp_username']=None
            cherrypy.session["bfaserverclient"].logout()
            cherrypy.session["bfaserverclient"] = None
            cherrypy.request.login = None
            serverlogger.info("%s logged out" %username)
            self.on_logout(username)
                
            
        return {"result": True, "errorcode": 0, "message": "logout success", "data":{}}
