#!/usr/bin/env python
# -*- coding: utf-8*-
import requests
import warnings
import json
import atexit
from pprint import pprint
import os
import sys, time

warnings.filterwarnings(action='ignore', module='requests')

login_path = "/auth/login"
online_path = "/anybodyOutThere"
logout_path = "/auth/logout"
get_doc_data_path = "/api/gdocidstore"
get_doc_text_path = "/api/gettext"
set_edit_path = "/api/getDocEdit"
set_cancel_path = "/api/getdoccancel"
save_doc_path = "/api/getSave"
get_users_path = "/api/getusers"
get_groups_path = "/api/getgroups"
get_archives_path = "/api/getarctree"
get_addfields_path = "/api/getaddfields"
get_wvl_path = "/api/getwvl"
get_history_path = "/api/gethistory"
get_search_path = "/api/search"
get_docpage_path = "/api/preview"
get_mastmodified_path = "/api/getlastmodified"
get_save_holiday_path = "/api/getsaveholiday"
get_modify_holiday_path = "/api/getmodifyholiday"
get_holiday_path = "/api/getholiday"
get_delete_holiday_path = "/api/getdeleteholiday"

method_dict = {}
method_dict[login_path] = "post"
method_dict[logout_path] = "get"
method_dict[online_path] = "get"
method_dict[get_doc_data_path] = "get"
method_dict[set_edit_path] = "get"
method_dict[set_cancel_path] = "get"
method_dict[get_doc_text_path] = "get"
method_dict[save_doc_path] = "post"
method_dict[get_users_path] = "get"
method_dict[get_groups_path] = "get"
method_dict[get_archives_path] = "get"
method_dict[get_addfields_path] = "get"
method_dict[get_wvl_path] = "get"
method_dict[get_history_path] = "get"
method_dict[get_search_path] = "post"
method_dict[get_docpage_path] = "get"
method_dict[get_mastmodified_path] = "get"
method_dict[get_save_holiday_path] = "get"
method_dict[get_modify_holiday_path] = "get"
method_dict[get_holiday_path] = "get"
method_dict[get_delete_holiday_path] = "get"

class ApiError(Exception):
    def __init__(self, message, *args):
        super(ApiError, self).__init__(message)
    
    
class NoLoginError(Exception):
    def __init__(self, message, *args):
        super(NoLoginError, self).__init__(message)

def needlogin(func):
    """
    decorator function check login state of client
    raises Exception if not logged in
    """
    def checkauth(client, *args, **kwargs):
        if client._logged_in is True:
            return func(client, *args, **kwargs)
        else:
            loginresult = client.login(client._username, client._userpass)
            if loginresult is True:
                return func(client, *args, **kwargs)
            else:
                raise NoLoginError("not logged_in, relogin failed")
    return checkauth
    
    
class BfaHttpClient():
    _logged_in = False
    _server_url = ""
    _profile = ""
    _session = None
    _verify = False
    _username = None
    _userpass = None
    
    def __init__(self, serverurl, profile):
        self._serverurl = serverurl
        self._profile = profile
        self.arcs = {}
        self.users = {}
        self.groups = {}
        
    def _request(self, path, data=None, jsonify=False):
        if self._session is not None:
            url = self._serverurl + path
            
            if method_dict[path] == "get":
                method = self._session.get
                #print "requesting %s with GET" %url
            elif method_dict[path] == "post":
                method = self._session.post
                #print "requesting %s with POST" %url
            if data is not None:
                if method_dict[path] == "post":
                    if jsonify is True:
                        result = method(url, verify = self._verify, json=data)
                    else:
                        result = method(url, verify = self._verify, data=data)
                else:
                    if jsonify is True:
                        result = method(url, verify = self._verify, json=data)
                    else:
                        result = method(url, verify = self._verify, params=data)
            else:
                result = method(url, verify = self._verify)
            #print "request got result with status code %d" %result.status_code
            if "json" in result.headers["Content-Type"]:
                return result
            elif "image" in result.headers["Content-Type"]:
                return result.content
            else:
                return result
        else:
            return None
            
    
    def _loard_arcs(self):
        self.arcs = {}
        arctree = self.get_archives()
        for arc in arctree:
            pprint(arc)
            
    def login(self, username, password):
        if self._logged_in is not False:
            raise Exception("already logged in")
        else:
            data = {"username":username, "password":password, "profile":self._profile}
            self._session = requests.Session()
            result = self._request(login_path, data)
            if result.status_code == 200:
                resultobject = result.json()
                if resultobject[0] == True:
                    self._logged_in = True
                    self._username = username
                    self._userpass = password
                    #self._loard_arcs()
                    return True
                else:
                    return False
            else:
                return False
    
    
    @needlogin
    def logout(self):
        print("logging out")
        if self._logged_in is not True:
            raise Exception("no session to logout from")
        else:
            result = self._request(logout_path)
            if result.status_code == 200:
                resultobject = result.json()
                if resultobject == True:
                    self._logged_in = False
                    self._session = None
                    return True
                else:
                    return False
            else:
                return False
   


        
    @needlogin
    def get_tif(self, gdocid):
        print("get_tif called")

    @needlogin      
    def set_edit(self, gdocid):
        # return document as json if successfull
        data = {"gdocid":gdocid}
        result = self._request(set_edit_path, data)
        if result.status_code == 200:
            try:
                return result.json()
            except:
                raise ApiError("json decode docdata failed, server said %s" %result.text)
        else:
            raise ApiError("error loading docdata, http status code %d" %result.status_code)
    
    #@needlogin      
    def set_cancel(self, gdocid):
        # returns json True if successfull
        data = {"gdocid":gdocid}
        result = self._request(set_cancel_path, data)
        if result.status_code == 200:
            try:
                return result.json()
            except:
                raise ApiError("json decode docdata failed, server said %s" %result.text)
        else:
            raise ApiError("error loading docdata, http status code %d" %result.status_code)
            
    @needlogin        
    def get_doc_data(self, gdocid):
        data = {"gdocid":gdocid, "asjson":"j"}
        result = self._request(get_doc_data_path, data)
        if result.status_code == 200:
            try:
                return result.json()
            except:
                raise ApiError("json decode docdata failed, server said %s" %result.text)
        else:
            raise ApiError("error loading docdata, http status code %d" %result.status_code)
            
    @needlogin        
    def get_doc_with_text(self, gdocid):
        data = {"gdocid":gdocid, "asjson":"j"}
        result = self._request(get_doc_text_path, data)
        if result.status_code == 200:
            try:
                return result.json()
            except:
                raise ApiError("json decode docdata failed, server said %s" %result.text)
        else:
            raise ApiError("error loading docdata, http status code %d" %result.status_code)
                
      
    @needlogin
    def savedoc(self, gdocid, addfields, statefields):
        
        data = {"gdocid":gdocid, "asjson":"j", "addfields":addfields, "statefields":statefields}
        result = self._request(save_doc_path,data, True)
        if result.status_code == 200:
            try:
                return result.json()
            except:
                raise ApiError("json decode docdata failed, server said %s" %result.text)
        else:
            raise ApiError("error loading docdata, http status code %d" %result.status_code)
        pass


    @needlogin
    def get_users(self):
        data = {"asjson":"j"}
        result = self._request(get_users_path, data)
        if result.status_code == 200:
            try:
                return result.json()
            except:
                raise ApiError("json decode docdata failed, server said %s" %result.text)
        else:
            raise ApiError("error loading docdata, http status code %d" %result.status_code)
    
    @needlogin
    def get_groups(self):
        data = {"asjson":"j"}
        result = self._request(get_groups_path, data)
        if result.status_code == 200:
            try:
                return result.json()
            except:
                raise ApiError("json decode docdata failed, server said %s" %result.text)
        else:
            raise ApiError("error loading docdata, http status code %d" %result.status_code)
            
    @needlogin
    def get_archives(self):
        data = {"asjson":"j"}
        result = self._request(get_archives_path, data)
        if result.status_code == 200:
            try:
                return result.json()
            except:
                raise ApiError("json decode docdata failed, server said %s" %result.text)
        else:
            raise ApiError("error loading docdata, http status code %d" %result.status_code)
    
    @needlogin
    def get_addfields(self):
        data = {"asjson":"j"}
        result = self._request(get_addfields_path, data)
        if result.status_code == 200:
            try:
                return result.json()
            except:
                raise ApiError("json decode docdata failed, server said %s" %result.text)
        else:
            raise ApiError("error loading docdata, http status code %d" %result.status_code)
            
    @needlogin
    def get_wvl(self):
        data = {"asjson":"j"}
        result = self._request(get_wvl_path, data)
        if result.status_code == 200:
            try:
                return result.json()
            except:
                raise ApiError("json decode docdata failed, server said %s" %result.text)
        else:
            raise ApiError("error loading docdata, http status code %d" %result.status_code)   
    
    @needlogin
    def get_history(self, gdocid):
        data = {"asjson":"j", "gdocid":gdocid}
        result = self._request(get_history_path, data)
        if result.status_code == 200:
            try:
                return result.json()
            except:
                raise ApiError("json decode docdata failed, server said %s" %result.text)
        else:
            raise ApiError("error loading docdata, http status code %d" %result.status_code)    
    
    @needlogin
    def dosearch(self,  selected_cols=u'gdocid,docdate,arcname', order=u"docdate", \
                            name=None, id=None, date=None, searchtype=0, userid=None, groupid=None, \
                            urights=None, grights=None, sqlwhere=None, sqlwhere_operator=None, \
                            sqlwhere_only=None, nonconstructive=None, maxvalues=50, refsearch=None, \
                            refsearch_negated=None, searchuser=None, searchuserid=None, searchuser_negated=None, \
                            searchgroupid=None, searchgroup=None, searchgroup_negated=None, arcid=None, \
                            gdocid=None, timref=None, checkout=None, state=None, notstate=None, \
                            searchdate=None, searchdatehigh=None, searchdate_negated=None, ftx_fields=3, \
                            searchstring=None, searchstring_negated=None, ftx_matchagainst=None, ftx_naturalmode=None, \
                            ftx_booleanmode=None, ftx_withqueryexpansion=None, count=None, resulttype=None, \
                            direction="desc", addfields=None, statefields=None, resubdate=None, \
                            resubdate_high=None, resubalarm_high=None, resubalarm=None, resubuserid=None, \
                            resubuserid_negated=None, resubcontrollerid=None, resubcontrollerid_negated=None, \
                            resubstate=None, resubdate_negated=None, resubalarm_negated=None, docname=None, \
                            docname_negated=None, filetype=None, filetype_negated=None, grouplist=None, \
                            usemodifieddate=None, selectedarcid=None, asjson=False):
        data = locals().copy()
        data.pop("self", None)
        result = self._request(get_search_path, data, jsonify=True)
        if result.status_code == 200:
            try:
                return result.json()
            except:
                raise ApiError("json decode docdata failed, server said %s" %result.text)
        else:
            raise ApiError("error loading docdata, http status code %d" %result.status_code)  
    
    @needlogin
    def get_doc_page(self, gdocid, pageno, color="color", ievlayer=None, size=None, scale=None, rotation=None, imgtype = "png"):
        # color "grey", "gray", "bw", "color"
        # size "1920x1080"
        # rotation integer zw. 0 und 359
        # scale float > 0,1 <= 1,0
        # bei scale und size gewinnt size
        data = {"gdocid":gdocid,  "pageno":pageno, "ievlayer":ievlayer, "imgtype":imgtype, "rotation":rotation, "scale":scale, "size":size}
        result = self._request(get_docpage_path, data, jsonify=False)
        return result
    
    @needlogin
    def get_save_holiday(self, replid, startdate, enddate):
        data = {"replid":replid, "startdate":startdate, "enddate":enddate}
        result = self._request(get_save_holiday_path, data, jsonify=False)
        return result

    @needlogin
    def get_delete_holiday(self, holid, userid):
        data = {"holid":holid, "userid":userid}
        result = self._request(get_delete_holiday_path, data, jsonify=False)
        return result
    
        
    @needlogin
    def get_modify_holiday(self, holid, replid, startdate, enddate):
        data = {"holid":holid, "replid":replid, "startdate":startdate, "enddate":enddate}
        result = self._request(get_modify_holiday_path, data, jsonify=False)
        return result
    
    @needlogin
    def get_holiday(self, asjson=False):
        if asjson is True:
            data={"asjson":"j"}
        else:
            data = {}
        result = self._request(get_holiday_path, data)
        return result.json()
        
    @needlogin
    def get_lastmodified(self, gdocid, asjson=False, revision=""):
        data = {"gdocid":gdocid, "asjson":asjson}
        result = self._request(get_mastmodified_path, data, jsonify=False)
        return result

def docload_speedtest(client, gdocid, loadcount):
    starttime = time.time()
    for i in range(loadcount):
        result = client.get_doc_data(gdocid)  
        sys.stdout.write("\r%d loaded    " %i) 
    sys.stdout.write("                             \r") 
    print
    return (time.time()-starttime)/float(loadcount)
    
def savedoc_test(client):        
    gdocid = 1458
    addfields = [{"id":34, "value":u"noch in der Schwebe;Auftrag", 'indexid':0, "type":3}]
    #statefields = [{"id":2, "bitmask":2, "value":True, "indexid":0}]
    statefields = []
    try:
        doc = client.set_edit(gdocid)
        result = client.savedoc(gdocid, addfields, statefields)
        
        client.set_cancel(gdocid)
    except Exception as e:
        print("error savedoc", e)


def main():
    
    server = sys.argv[1]
    profile = sys.argv[2]
    username = sys.argv[3]
    password = sys.argv[4]
    client = BfaHttpClient(server, profile)
    client.login(username, password)
    client.logout()
    sys.exit()

if __name__=="__main__":
    main()
            
        

