 #!/usr/bin/env python
# -*- coding: latin-1 -*-

import threading
import queue
import smtplib
import logging
import gettext
import time


try:
#    trans = gettext.translation("bfaArchivierung", "locale", ["de"])
    trans = gettext.translation("bfaArchiving", "locale", ['en'],  fallback = True)
    trans.install()
except:
    print(u'Keine bersetzung gefunden/no translation found')
    _ = lambda x: x
else:
    _ = trans.gettext


class BitfarmMailServer(threading.Thread):
    log = logging.getLogger(u"Main.BitfarmMailServer")
    queue = queue.Queue(0)

    def __init__(self, _con):
        threading.Thread.__init__(self)
        self.stopsignal = False
        self.server = 'localhost'
        self.port = 25
        self.con = _con
        self.smtp = None
        if self.con.main.smtpserver is not None:
            self.server = self.con.main.smtpserver
        if self.con.main.smtpport is not None:
            self.port = self.con.main.smtpport

        self.log.debug(_(u'Versuche Connect zum SMTP-Server %s %s') %(self.server, self.port))
#        bfaProcessing.sendmaillock.acquire()
        try:
            try:
                self.smtp = smtplib.SMTP(self.server, int(self.port))
            except:
                self.log.exception(_(u'%s: Fehler beim Verbinden mit dem SMTP-Server %s %s') %(self.name, self.server, self.port))
                self.smtp = None
            else:
                if self.con.main.authuser and self.con.main.authuser:
                    try:
                        self.smtp.login(self.con.main.authuser, self.con.main.authuser)
                    except:
                        self.log.exception(_(u'%s: Fehler beim Login zum SMTP-Server') %self.name)
                        self.smtp = None

        finally:
            if self.smtp:
                self.log.debug(_(u'Connect erfolgreich'))

    def run(self):
        self.log.info(_(u'Starte MailServer-Thread'))

        while self.is_alive():
            self.log.debug(_(u'bereit'))
            mail = BitfarmMailServer.queue.get()
            self.log.debug(_(u'neue Mail entgegengenommen %s' %str(mail)))
            sended = False

            while not sended:
                try:
                    self.smtp.sendmail(mail[0], mail[1], mail[2].as_string())
                except:
                    self.log.exception(_(u'%s: Fehler beim Versenden einer Email') %self.name)
                    self.log.warning(_(u'Versuche Reconnect'))
                    try:
                        self.smtp = smtplib.SMTP(self.server, int(self.port))
                    except:
                        self.log.exception(_(u'%s: Fehler beim Verbinden mit dem SMTP-Server %s %s') %(self.name, self.server, self.port))
                        time.sleep(3)
                    else:
                        if self.con.main.authuser and self.con.main.authuser:
                            try:
                                self.smtp.login(self.con.main.authuser, self.con.main.authuser)
                            except:
                                self.log.exception(_(u'%s: Fehler beim Login zum SMTP-Server') %self.name)
                                time.sleep(3)
                else:
                    self.log.debug(_(u'Email erfolgreich versendet'))
                    sended = True


        self.log.info(_(u'Beende MailServer-Thread'))
        if self.smtp is not None:
            self.smtp.quit()
