#!/usr/bin/env python
# -*- coding: latin-1 -*-

import sys
import os
import shutil
import threading
import Queue
import logging
import time
import datetime
import win32com
import win32com.client
from win32com.client import Dispatch, GetActiveObject
import pythoncom
import bitfarm_doc
import bitfarm_proc
#import bitfarm_doc_shared
#import bitfarm_workflow
import bfaCustomizing
import gettext
import cStringIO
import hashlib
import zbar
#import Image
import FreeImagePy as FIPY
import zipfile

try:
    trans = gettext.translation("bfaArchiving", "locale", ['en'],  fallback = True)
    trans.install()
except:
    print(u'Keine bersetzung gefunden/no translation found')
    _ = lambda x: x
else:
    _ = trans.ugettext

class bfaProcessing(threading.Thread, bfaCustomizing.bfaCustomizing):
    """bfaProcessing-Klasse stellt eine Thread-Klasse zur Verfgung, mit grundlegenden, dokumentverarbeitenden 
    Funktionen auf Applikationsebene. Jede Instanz erzeugt sich eine Instanz des bfaDocProcessing-Objekts und hinterlegt
    diese in self.bfaProc. Mit dem Parameter _con_db kann man ein Dictionary optional weitergeben, in dem fr jedes Profil
    schon eine verbundene DB-Connection hinterlegt ist (z.B, von einem bergeordneten Thread). {'Profil':sqla.engine.base.Connection}"""
    
    result = []
    resultlock = threading.Lock()
    writelock = threading.Lock()
    sendmaillock = threading.Lock()
    queue_cnt = 1
    queue = Queue.Queue(0)
    split_size = 1024*1024/2
    NRS_EXT = {0:['.xls', '.docx', '.xlsx', '.txt','.log']}    #hier kann pro Archiv angegeben werden, welche Dokumente in NRS liegen bleiben (nderbar)
    log = logging.getLogger("Main.bfaProcessing")


    def __init__(self, _con, _con_db = None):
        threading.Thread.__init__(self)
        self._job = None
        self.con = _con
        self.bfaProc = bitfarm_proc.BitfarmDocProcessing(self.con, _con_db)
        self.timeout = 0
        self.qsbc = None
        self.log.info(_(u'%s: Processing-Thread initialisiert') % self.name)


    def run(self):

        while self.is_alive():
            self._job = bfaProcessing.queue.get()
            self.log.debug(_(u"%s: processing [%s] (%i aktive task)") %(self.name, self._job.gdocid, bfaProcessing.queue.unfinished_tasks))

            try:
                if hasattr(self,  self._job.tasks[0].task):
                    getattr(self,  self._job.tasks[0].task)()
                else:
                    self.result = False
                    self._job.error = True
                    self._job.errortext = _("Die Funktion %s konnte nicht gefunden werden") %self._job.tasks[0].task
                    self.log.error(self._job.errortext)

                self.log.debug(_('%s: verarbeitet: %s') %(self.name, self._job))
            except:
                self.log.exception(_('%s: Fehler in der Verarbeitung') %self.name)

            if self._job is not None:
                bfaProcessing.resultlock.acquire()
                try:
                    if self._Thread__stopped:
                        break
                    if type(self._job) is list:
                        self.__class__.result.extend(self._job)
                    else:
                        self.__class__.result.append(self._job)
                    if bfaProcessing.queue.unfinished_tasks > 0:
                        bfaProcessing.queue.task_done()
                    self._job = None
                    self.timeout = 0
                finally:
                    bfaProcessing.resultlock.release()
                    self.log.debug(_(u"%s: processing fertig : %s") %(self.name, str(self._job)))
            else:
                self.log.error(_('%s: kein Job fr die Rckgabe mehr vorhanden.') %self.name)

        self.log.warning(_('%s: Thread wurde gestoppt') %self.name)


    def archive(self):
        self.log.info(_(u'Dokument zur Archivierung bergeben [%s]') %self._job.gdocid)
        # gdocid erzeugen
        try:
            if self._job.gdocid is None:
                try:
                    self.bfaProc.create(self._job, self._job.arcid)
                except:
                    self.errortext = _('Dokument konnte nicht erzeugt werden')
                    self.log.exception(self.errortext)
                    self.error = True
                    return self._job

            # Dateiablage

            # wenn wir einen srcloc haben, der schon auf einen der Dateiablageorte verteilt wurde, kann dieser entfernt werden
            if self._job.srcloc in [self._job.docloc,  self._job.oriloc]:
                self._job.srcloc = None

            #In Version 3.5 haben wir (noch) keinen scrloc-Pfad
            _version = self._job.version if self._job.version is not None else self.con.version
            if _version in (3, 35, '3.0'):
                self._job.oriloc = self._job.srcloc if self._job.oriloc == '' else self._job.oriloc
                self._job.srcloc = u''
            # wir bermitteln den Pfad der Quell bzw. Originaldatei um zu entscheiden ob die ggf. nichtrevisionsicher bleiben soll
#            _fpath = self._job.oriloc if self._job.oriloc != '' else self._job.srcloc if self._job.srcloc != '' else self._job.docloc if self._job.docloc != '' else self._job.path
#            if _fpath == '':    # kein Pfad? dann in self._job.path schauen
#                self.log.warning(_('Dokument hat keinen Pfad'))
#                self.error = True
#                self.errortext = _('Dokument hat keinen Pfad')
#                return

            if self._job.oriloc:
                _fext = os.path.splitext(self._job.oriloc)[1]

                if not self._job.is_revisionsave and not self._job.is_changeable:
                    _nrs = self.NRS_EXT[self._job.arcid] if self._job.arcid in self.NRS_EXT else self.NRS_EXT[0]
                    if _fext not in _nrs:
                        self._do_make_revisionsave()
            else:
                if not  self._job.is_changeable:
                    self._do_make_revisionsave()

            #Archivierung

            if self._job.title is None or self._job.title == u'':
                self._job.title = _('Dokument')

            try:
                self.bfaProc.write(self._job)
            except:
                self.error = True
                self.errortext = _('Dokument %s konnte nicht archiviert werden' %self._job)
                self.log.exception(self.errortext )
            else:
                self.log.info(_('Dokument erfolgreich archiviert: %s') %self._job)
                self.bfaProc.write_history(self._job, _('Dokument erfolgreich archiviert'), 1)

            if self._job.tmploc:
                try:
                    os.remove(self._job.tmploc)
                except:
                    self.log.warning(_('Temporre Datei %s konnte nicht gelscht werden') %self._job.tmploc)

        finally:
            return self._job


    def make_revisionsave(self):
        self.log.debug('make_revisionsave')
        try:
            self._do_make_revisionsave()
        except:
            self.log.exception(_('Fehler bei do_make_revisionsave'))
        try:
            self.bfaProc.write(self._job)
        except:
            self.error = True
            self.errortext = _('Dokument %s konnte nicht archiviert werden' %self._job)
            self.log.exception(self.errortext )
        else:
            self.log.info(_('Dokument erfolgreich archiviert: %s') %self._job)
            self.result = True


    def _do_make_revisionsave(self):
        self.log.debug(u'%s: _do_make_revisionsave' %self.name)
        try:
            if self._job.profile is None or self._job.profile =='':
                _con = self.con.main
                self._job.profile = _con.profile
            else:
                _con = self.con[self._job.profile]

            if self._job.srcloc and self._job.srcloc != self._job.extloc:    # wenn wir srcloc haben, und dieser nicht mit extloc identisch ist (extloc bleibt in nrs liegen, daher wird dann auch srcloc nicht umgelagert!)
                if self._job.srcloc.find(_con.archive_rs) == -1:         # die Behandlung von srcloc unterscheidet sich ein bischen von docloc und oriloc. srcloc wird nach rs kopiert
                    _fext = os.path.splitext(self._job.srcloc)[1]          # ohne einen neuen basisnamen zu erzeugen, es wird einfach der pfad von nrs auf rs umgebogen,
                                                                                        # der eigentliche name darf nicht gendert werden, da ggf. mehrere Dateien (barcodesplit, unzip)
                    #_fpath = os.path.dirname(self._job.srcloc)                         # den gleichen srcloc haben. so kann geprft werden, ob dieser schon kopiert wurde.
                    #_fpath = _fpath.replace(_con.archive_nrs, _con.archive_rs)
                    _fpath = self.bfaProc.create_arcpath(self._job, is_revisionsave = True)
                    _basename = os.path.splitext(os.path.basename(self._job.srcloc))[0]
                    _fname= os.path.join(_fpath, _basename + _fext)
                    _ind = 0
                    bfaProcessing.writelock.acquire()
                    try:
                        if not os.path.exists(os.path.expandvars(self._job.srcloc)) and os.path.exists(os.path.expandvar(_fname)):      # offensichtlich schon umkopiert
                            self._job.srcloc = _fname
                        else:
                            while os.path.exists(os.path.expandvars(_fname)):
                                _fname = os.path.join(_fpath, _basename + '_' + str(_ind) + _fext)
                                _ind += 1
                            try:
                                shutil.move(os.path.expandvars(self._job.srcloc), os.path.expandvars(_fname))
                            except:
                                self._job.errortext = _(u'%s: Fehler bei make_revisionsave (srcloc) [%s]') %(self.name, self._job)
                                self.log.exception(self._job.errortext)
                                try:
                                    self.log.warning(_(u'%s: Zweiter Versuch (srcloc) [%s]') %(self.name, self._job))
                                    time.sleep(2)
                                    while os.path.exists(os.path.expandvars(_fname)):
                                        _fname= os.path.join(_fpath, self._job.basename + '_' + str(_ind) + _fext)
                                        _ind += 1
                                    shutil.move(os.path.expandvars(self._job.docloc), os.path.expandvars(_fname))
                                except:
                                    self._job.errortext = _(u'%s: Fehler 2 bei make_revisionsave (srcloc) [%s]') %(self.name, self._job)
                                    self.log.exception(self._job.errortext)
                                    self._job.error = True
                                    return False
                                else:
                                    self.log.warning(_(u'%s: Zweiter Versuch erfolgreich [%s]') %(self.name, self._job))
                                    self._job.docloc = _fname
                            else:
                                self._job.srcloc = _fname
                    finally:
                        bfaProcessing.writelock.release()

            if self._job.docloc:
                if self._job.docloc.find(_con.archive_rs) == -1:
                    _fext = os.path.splitext(self._job.docloc)[1]
                    _fpath = self.bfaProc.create_arcpath(self._job, is_revisionsave = True)
                    _fname= os.path.join(_fpath, self._job.basename + _fext)
                    _ind = 0
                    bfaProcessing.writelock.acquire()
                    try:
                        while os.path.exists(os.path.expandvars(_fname)):
                            _fname = os.path.join(_fpath, self._job.basename + '_' + str(_ind) + _fext)
                            _ind += 1
                        try:
                            shutil.move(os.path.expandvars(self._job.docloc), os.path.expandvars(_fname))
                        except:
                            self._job.errortext = _(u'%s: Fehler bei make_revisionsave (docloc) [%s]') %(self.name, self._job)
                            self.log.exception(self._job.errortext)
                            try:
                                self.log.warning(_(u'%s: Zweiter Versuch (docloc) [%s]') %(self.name, self._job))
                                time.sleep(2)
                                while os.path.exists(os.path.expandvars(_fname)):
                                    _fname= os.path.join(_fpath, self._job.basename + '_' + str(_ind) + _fext)
                                    _ind += 1
                                shutil.move(os.path.expandvars(self._job.docloc), os.path.expandvars(_fname))
                            except:
                                self._job.errortext = _(u'%s: Fehler 2 bei make_revisionsave (docloc) [%s]') %(self.name, self._job)
                                self.log.exception(self._job.errortext)
                                self._job.error = True
                                return False
                            else:
                                self.log.warning(_(u'%s: Zweiter Versuch erfolgreich [%s]') %(self.name, self._job))
                                self._job.docloc = _fname
                        else:
                            self._job.docloc = _fname
                    finally:
                        bfaProcessing.writelock.release()

            if self._job.oriloc:
                if self._job.oriloc.find(_con.archive_rs) == -1:
                    _fext = os.path.splitext(self._job.oriloc)[1]
                    _fpath = self.bfaProc.create_arcpath(self._job, is_revisionsave = True)
                    _fname = os.path.join(_fpath, self._job.basename + _fext)
                    _ind = 0
                    bfaProcessing.writelock.acquire()
                    try:
                        while os.path.exists(os.path.expandvars(_fname)):
                            _fname= os.path.join(_fpath, self._job.basename + '_' + str(_ind) + _fext)
                            _ind += 1
                        try:
                            shutil.move(os.path.expandvars(self._job.oriloc), os.path.expandvars(_fname))
                        except:
                            self._job.errortext = _(u'%s: Fehler bei make_revisionsave (oriloc) [%s]') %(self.name, self._job)
                            self.log.exception(self._job.errortext)
                            try:
                                self.log.warning(_(u'%s: Zweiter Versuch (oriloc) [%s]') %(self.name, self._job))
                                time.sleep(2)
                                while os.path.exists(os.path.expandvars(_fname)):
                                    _fname = os.path.join(_fpath, self._job.basename + '_' + str(_ind) + _fext)
                                    _ind += 1
                                shutil.move(os.path.expandvars(self._job.docloc), os.path.expandvars(_fname))
                            except:
                                self._job.errortext = _(u'%s: Fehler 2 bei make_revisionsave (oriloc) [%s]') %(self.name, self._job)
                                self.log.exception(self._job.errortext)
                                self._job.error = True
                                return False
                            else:
                                self.log.warning(_(u'%s: Zweiter Versuch erfolgreich [%s]') %(self.name, self._job))
                                self._job.docloc = _fname
                        else:
                            self._job.oriloc = _fname
                    finally:
                        bfaProcessing.writelock.release()
        finally:
            self._job.is_revisionsave = True
            return True


    def unzip(self):
        self.log.debug(_('%s: unzip') %self.name)
        if os.path.splitext(self._job.path)[1].lower() != '.zip':
            self.log.warning(_(u'%s: unzip kein untersttztes Dateiformat %s') %(self.name, self._job.path))
            self._job.result = False
            return None

        if not zipfile.is_zipfile(self._job.path):
            self.log.warning(_(u'%s: bergebene Datei kein Zip-File') %self.name)
            self.result = False
            return None

        jobs = []

        if self._job.title is None or self._job.title == _('Dokument'):
            self._job.title = self._job.filename if self._job.filename else _('Zip-Datei')


        if not self._job.tasks[0].params.zip_dontcreatelinks:
            if self._job.tasks[0].params.zip_linkmask:
                # TODO: prfen ob noch andere Zeichen geparst werden mssen in links
                if self._job.tasks[0].params.zip_linkmask.find("'") >= 0: #hochkomma machen rger in links
                    self._job.tasks[0].params.zip_linkmask = self._job.tasks[0].params.zip_linkmask.replace("'", '')
                newlink = [self.bfaProc.parse_txt(self._job, self._job.tasks[0].params.zip_linkmask)]
            else:
                newlink = ['ZIP: %s %s' %(self._job.filename, datetime.datetime.now().strftime('%d.%m.%Y %H:%M:%S'))]
            self._job.links.extend(newlink)

        zf = zipfile.ZipFile(self._job.path, 'r')
        self.log.debug(u'Zipfile content: %s' %str(zf.namelist()))
        jobs.extend(self._do_unzip(zf, zf.namelist(), newlink))
        zf.close()

        if len(jobs) == 0:
            self.log.warning(_(u'%s: es konnte nichts entpackt werden') %self.name)
            self._job.result = False
            return None

        if self._job.tasks[0].params.zip_deleteafterdecomp:
            self._job.error = True
            self._job.errortext = _('Zip-Datei in Garbage verschoben')
            self.log.warning(_('Zip-Datei in Garbage verschoben'))

        jobs.insert(0, self._job)

        for job in jobs:    # nach dem entpacken zum rendern schicken
            task = bitfarm_doc.BitfarmTask('render_by_type')
            job.tasks.insert(1, task)

        #zf.extractall(os.path.dirname(self._job.path))
        self._job = jobs
        return self._job

    def _do_unzip(self, zf, filelist, newlink):
        new_jobs = []
        sublinks = []
#        links = newlink[:]
        for zfile in filelist:
            if self._job.tasks[0].params.zip_extractrecursive:
                if os.path.splitext(zfile)[1].lower() == '.zip':

                    self.bfaProc.create_tmploc(self._job, os.path.splitext(zfile)[1].lower())
    #                data = zf.read(zfile)
    #                f = open(self._job.outfile, 'wb')
    #                f.write(data)
    #                f.close()
                    source = zf.open(zfile)
                    target = file(self._job.tmploc, 'wb')
                    with source, target:
                        shutil.copyfileobj(source, target)

                    if not self._job.tasks[0].params.zip_dontcreatelinks:
                        sublinks = newlink[:]
                        if self._job.tasks[0].params.zip_linkmask:
                            # TODO: prfen ob noch andere Zeichen geparst werden mssen in links
                            if self._job.tasks[0].params.zip_linkmask.find("'") >= 0: #hochkomma machen rger in links
                                self._job.tasks[0].params.zip_linkmask = self._job.tasks[0].params.zip_linkmask.replace("'", '')
                            sublinks.append(self.bfaProc.parse_txt(self._job, self._job.tasks[0].params.zip_linkmask))
                        else:
                            sublinks.append('ZIP: %s %s' %(zfile, datetime.datetime.now().strftime('%d.%m.%Y %H:%M:%S')))

                    new_zf = zipfile.ZipFile(self._job.tmploc, 'r')
                    print new_zf.namelist()
                    new_jobs.extend(self._do_unzip(new_zf, new_zf.namelist(), sublinks))
                    new_zf.close()

                    if self._job.tasks[0].params.zip_deleteafterdecomp:
                        try:
                            os.remove(self._job.tmploc)
                        except:
                            self.log.warning(_(u'%s: Fehler bei zip_deleteafterdecomp, konnte Datei nicht lschen') %self.name)
                        continue    # interne zip datei nicht nochmal entpacken


            if os.path.splitext(zfile)[1] == '' and not self._job.tasks[0].params.zip_acceptnoext:
                continue

            if len(self._job.tasks[0].params.zip_blacklistext) > 0:
                if os.path.splitext(zfile)[1] in self._job.tasks[0].params.zip_blacklistext:
                    continue

            if len(self._job.tasks[0].params.zip_whitelistext) > 0:
                if os.path.splitext(zfile)[1] not in self._job.tasks[0].params.zip_whitelistext:
                    continue

            doc = bitfarm_doc.BitfarmDocument()
            self.bfaProc.create(doc)
            self.bfaProc.create_outfile(doc, os.path.splitext(zfile)[1].lower())
            if os.path.splitext(zfile)[1].lower() == '.zip':
                doc.links.extend(sublinks)
            else:
                doc.links.extend(newlink)
            source = zf.open(zfile)
            target = file(doc.outfile, 'wb')
            with source, target:
                shutil.copyfileobj(source, target)

            #doc.srcloc = zf.filename
            doc.title = os.path.basename(zfile)
            doc.tasks = self._job.tasks[1:]     # alle bis auf den aktuellen Task dem neuen Objekt bertragen
            doc.filename = source.name
            new_jobs.append(doc)

        return new_jobs


    def check_duplicates(self):
        self.log.debug(u'check_duplicates')
        memory = 1000
        sums = []

        if self._job.profile is None or self._job.profile == '':
            _con = self.con.main
        else:
            _con = self.con[self._job.profile]

        if _con.ini['scripts'].preventdoubledocsmemory is not None:
            memory = int(_con.ini['scripts'].preventdoubledocsmemory)

        sha = hashlib.sha1()
        with open(os.path.expandvars(self._job.path)) as f:
            sha.update(f.read(self.split_size))
        digest = sha.hexdigest()
        print digest
        self._job.sha = digest
        sumfile = os.path.join(_con.bfapath, _con.profile + '-checksums.txt')
        if os.path.exists(sumfile):
            with open(os.path.join(sumfile), 'r') as f:
                sums = f.readlines()

        digest += '\n'
        if digest in sums:
            self.log.warning(_(u'%s: Duplette erkannt [%s]') %(self.name, self._job.gdocid))
            _task1 = None
            _task2 = None
            if self._job.tasks[0].warning is not None:
                if self._job.tasks[0].warning.create_resub:
                    _task1 = bitfarm_doc.BitfarmTask('notify')
                    _task1.message = self._job.tasks[0].warning
                if self._job.tasks[0].warning.send_mail:
                    _task2 = bitfarm_doc.BitfarmTask('send_mail')
                    _task2.message = self._job.tasks[0].warning

            self._job.tasks = []
            if _task1 is not None:
                self._job.tasks.append(_task1)
            if _task2 is not None:
                self._job.tasks.append(_task2)

            if _con.sys_duplicates is None:
                self.log.warning(_(u'Dubletten-Archiv nicht geladen'))
            else:
                if not self._job.title:
                     self._job.title = _('Dublette')

                if not self.bfaProc.move(self._job, _con.garbage_id):
                    self.error = True
                    self.errortext = _('Dokument konnte nicht ins Garbage-Archiv veschoben werden')
                    self.log.exception(self.errortext )
                else:
                    self.log.info(_('Dokument ins Garbage-Archiv veschoben: %s') %self._job)
                    self.bfaProc.write_history(self._job, _('Dokument ins Garbage-Archiv veschoben'), 1)
        else:
            while len(sums) > memory -1:
                sums.pop(0)
            sums.append(digest)
            with open(sumfile, 'w') as f:
                for sum in sums:
                    f.write(sum)

        return self._job

    def read_zbar(self):
        self.log.debug(u'%s: read_zbar' %self.name)
        if os.path.splitext(self._job.path)[1] != '.tif':
            self.log.warning(_(u'%s: read_zbar Tif-Format wird bentigt %s') %(self.name, self._job.path))
            self._job.result = False
            return None

        scanner = zbar.ImageScanner()
        # configure the reader
        scanner.parse_config('enable')

        try:
            F = FIPY.Image(self._job.path)
        except:
            self.log.exception(_(u'%s: Datei konnte nicht geladen werden') %self.name)
            self._job.error = True
            self._job.errortext(_(u'read_zbar: Datei konnte nicht geladen werden'))
            return None
        if F.ok()[0] != 0:
            page = 0
        pages = F.getNumPages()
        stream = cStringIO.StringIO()
        start = time.time()
        for page in range(pages):
        #    stream = cStringIO.StringIO()
            stream.seek(0)
            stream.truncate()
            F.splitPageToBufferImage(stream, page = page +1,  flag = FIPY.TIFF_NONE)
            self.log.debug(u"%s: ZBar Seite %i: %i" %(self.name, page, stream.tell()))

            bm = FIPY.Image()
            bm.fromBuffer(stream.getvalue(), FIPY.FIF_TIFF, FIPY.TIFF_NONE)

            # obtain image data
            width, height = bm.size
            dib2 = bm.ConvertToGreyscale(bm.getBitmap())

            bm2 = FIPY.Image()
            bm2.loadFromBitmap(dib2)

            width, height = bm2.size
            image = zbar.Image(width, height, 'Y800', bm2.getBuffer())

            # scan the image for barcodes
            scanner.scan(image)

            # extract results
            for symbol in image:
                # do something useful with results
                self.log.debug(u'%s: decoded %s symbol "%s"' %(self.name, str(symbol.type),  symbol.data))
                bar = bitfarm_doc.BitfarmBarcode()
                bar.barcode = symbol.data
                bar.type = str(symbol.type)
                bar.page = page
                self._job.barcodes.append(bar)
                if self._job.tasks[0].params.tags_add_barcode:
                    if symbol.data not in self._job.tags:
                        self._job.tags.append(symbol.data)
            # clean up
            del(image)
        #    raw.close()
        self.log.debug(_(u"%s: dauer:%f") %(self.name, time.time() - start))
        stream.close()
        try:
            F.quit()
        except:
            self.log.exception(_(u'%s: Fehler bei FreeImagePy.quit()') %self.name)
        del F


    def get_fileinfo(self):
        self.log.debug(u'%s: get_fileinfo' %self.name)
        bitfarm_doc_shared.get_fileinfo(self._job)


    def raise_processing_error(self):
        self.log.debug(u'%s: raise_processing_error' %self.name)
        self._job.error = True


