﻿#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#       bfaDocLauncher.py
#
#
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

import os
import sys
import encodings
# import socket
import subprocess

import time
import ssl
import bitfarm_con
import bitfarm_xml
import bitfarm_doc

import datetime
import ctypes
import psutil
import logging
import logging.handlers
import shutil

def is_locked(filepath):
    """Checks if a file is locked by opening it in append mode.
    If no exception thrown, then the file is not locked.
    """
    locked = None
    file_object = None
    # if os.path.exists(filepath):
    try:
        #print "Trying to open %s." % filepath
        buffer_size = 8
        # Opening file in append mode and read the first 8 characters.
        file_object = open(filepath, 'a', buffer_size)
        if file_object:
            #print "%s is not locked." % filepath
            locked = False
    except IOError as message:
        # print "File is locked (unable to open in append mode). %s." %  message
        locked = True
    finally:
        if file_object:
            file_object.close()
            #print "%s closed." % filepath
    #else:
        #print "%s not found." % filepath
    print('is_locked %s' % str(locked))
    return locked


mypath = os.path.split(os.path.abspath(sys.argv[0]))[0]
logdir = os.environ['temp']
if not os.path.exists(logdir):
    logdir = mypath
try:
    pid = os.getpid()
except:
    pid = 0
log = logging.getLogger('bfaResyncDoc %d' % pid)

formatter = logging.Formatter('%(asctime)s %(name)-12s %(levelname)-8s %(message)s')
fh = logging.handlers.RotatingFileHandler(os.path.join(logdir, "bfaResyncDoc_%s.log" % (datetime.datetime.now().strftime('%Y-%d-%m %H-%M-%S'))), maxBytes=1024 * 1024, backupCount=10)
fh.setLevel(logging.DEBUG)
fh.setFormatter(formatter)
log.addHandler(fh)
log.setLevel(logging.DEBUG)

xml = None
socketlog = None
checkin = False
sslSwitch = True

try:
    params = " ".join([x for x in sys.argv[1:] if x != sys.argv[7]])
    log.debug("bfaResyncDoc v3.6.2.18840")
    log.debug("Parameter: %s" % params)
    profile = sys.argv[1]
    ip = sys.argv[2]
    port = int(sys.argv[3])
    userid = int(sys.argv[4])
    gdocid = int(sys.argv[5])
    fpath = sys.argv[6]
    password = sys.argv[7]
    if len(sys.argv) > 8:
        if sys.argv[8] == 'checkin':
            checkin = True
        if sys.argv[8] == 'nossl':
            sslSwitch = False
    if len(sys.argv) > 9:
        if sys.argv[9] == 'nossl':
            sslSwitch = False
except:
    log.exception("Fehler bei der Parameterübergabe")
    sys.exit(1)

msgfile = None
errorfile = None

if os.path.exists(os.path.join(mypath, 'bfaMessage.exe')):
    msgfile = os.path.join(mypath, 'bfaMessage.exe')
if os.path.exists(os.path.join(mypath, 'bfaLauncherError.exe')):
    errorfile = os.path.join(mypath, 'bfaLauncherError.exe')
errorproc = None
msgproc = None

if not os.path.exists(fpath):
    msg = "Datei konnte nicht gefunden werden."
    log.exception("%s Pfad %s" % (msg, fpath))
    ctypes.windll.user32.MessageBoxW(0, msg, 'Fehler', 0)
    sys.exit(1)

fbase = os.path.basename(fpath)
fext = os.path.splitext(fbase)[1]

if not os.path.exists(fpath):
    msg = "Die Datei %s kann nicht gefunden werden.\nZurückspielen nicht möglich." % fpath
    log.error(msg)
    ctypes.windll.user32.MessageBoxW(0, msg, 'Fehler', 0)
    sys.exit(1)

stats = os.stat(fpath)

if stats.st_size == 0:
    msg = "Die Datei %s ist leer / 0 Kb groß.\nZurückspielen nicht möglich." % fpath
    log.error(msg)
    ctypes.windll.user32.MessageBoxW(0, msg, 'Fehler', 0)
    sys.exit(1)

try:
    if msgproc is None:
        if msgfile:
            msgproc = subprocess.Popen(msgfile)
        else:
            msgproc = None
except:
    log.exception("Fehler beim Start von bfaMessage")
else:
    log.debug("bfaMessage gestartet %s" % str(msgproc))

try:
    import socket
except:
    log.exception('socket modul konnte nicht importiert werden')
else:
    log.debug('socket modul importiert')

con = bitfarm_con.BitfarmCon(profile)

sock = None
for tries in range(2):
    if sock is not None:
        try:
            sock.close()
        except:
            log.exception("Socket konnte nicht geschlossen werden")
        else:
            log.debug("Socket geschlossen round %d" % tries)
        time.sleep(.5)

    try:
        req = bitfarm_doc.bfaReq()
        req.reqtype = "reqAuthenticate"
        req.userid = userid
        req.data = password
        req.datatype = "text"
        req.profile = profile

        xml = bitfarm_xml.object_to_xml(req, with_lock=True)
        log.debug("reqAuthenticate Request erstellt")
        time.sleep(.1)

        sock = None

        if sslSwitch:
            context = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
            context.check_hostname = False
            context.verify_mode = ssl.CERT_NONE
            sock_nossl = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            sock = context.wrap_socket(sock_nossl, server_hostname=socket.gethostname())
            sockid = hex(id(sock)).upper()
            log.debug('socket object SSL %s' % sockid)
            sock.connect((ip, port))
            log.debug('socket.connect SSL %s %i' % (ip, port))
        else:
            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            sockid = hex(id(sock)).upper()
            log.debug('socket object %s' % sockid)
            sock.connect((ip, port))
            log.debug('socket.connect %s %i' % (ip, port))

        # for res in socket.getaddrinfo(ip, port, socket.AF_UNSPEC, socket.SOCK_STREAM):
        #     af, socktype, proto, canonname, sa = res
        #     log.debug("getaddrinfo %s" % canonname)
        #     try:
        #         sock = socket.socket(af, socktype, proto)
        #     except socket.error as msg:
        #         sock = None
        #         continue
        #     sockid = hex(id(sock)).upper()
        #     log.debug('socket object %s' % sockid)
        #     try:
        #         sock.connect(sa)
        #     except socket.error as msg:
        #         log.debug(str(msg))
        #         sock.close()
        #         sock = None
        #         continue
        #     log.debug('socket.connect %s %i' % (ip, port))
        #     break

        if sock is None:
            msg = u"Fehler bei der Socketerstellung (1), Server oder Netzwerk nicht erreichbar."
            log.exception(msg)
            if msgproc:
                msgproc.kill()
            ctypes.windll.user32.MessageBoxW(0, msg, u'Fehler', 0)
            sys.exit(1)

        sock.settimeout(30)
    except:
        msg = "Fehler bei der Socketerstellung (2), Server oder Netzwerk nicht erreichbar."
        log.exception(msg)
        if msgproc:
            msgproc.kill()
        ctypes.windll.user32.MessageBoxW(0, msg, 'Fehler', 0)
        sys.exit(1)

    i = 0
    buff_size = 1024

    log.debug("send reqAuthenticate round %d" % tries)

    try:
        sock.send(xml.encode())
    except:
        msg = "konnte nicht Anmelden, Server nicht erreichbar"
        log.exception(msg)
        if tries == 0:
            time.sleep(.5)
            continue
        if msgproc:
            msgproc.kill()
        ctypes.windll.user32.MessageBoxW(0, msg, 'Fehler', 0)
        sys.exit(1)

    nachricht = ''
    try:
        while nachricht[-9:] != '</bfaReq>':
            buff = sock.recv(1024).decode()
            if buff == '':
                break
            nachricht += buff
    except:
        log.exception("Fehler bei der Anmeldung (sock.recv)")
        nachricht = ''
        if tries == 0:
            time.sleep(.5)
            continue

        # nur wenn xmlStream komplett ist parsen
    sid = ''
    if "</bfaReq>" in nachricht:
        # log.debug("received:")
        # log.debug(nachricht)
        if 'respAuthError' in nachricht:
            msg = "Anmeldung vom Server abgelehnt"
            log.error(msg)
            log.debug(nachricht)
            if msgproc:
                msgproc.kill()
            ctypes.windll.user32.MessageBoxW(0, msg, 'Fehler', 0)
            sys.exit(1)
        else:
            log.debug("Anmeldung erfolgt")
            resp = bitfarm_xml.xml_to_object(nachricht)
            if resp.sessionid:
                sid = con.bitdecrypt(resp.sessionid)
            break   # erfolgreich, dann wiederholungsschleife verlassen
    else:
        msg = "Fehler bei der Anmeldung am Server"
        log.error(msg)
        log.debug(nachricht)
        if tries == 0:
            time.sleep(.5)
            continue
        if msgproc:
            msgproc.kill()
        ctypes.windll.user32.MessageBoxW(0, msg, 'Fehler', 0)
        sys.exit(1)

req = bitfarm_doc.bfaReq()
req.reqtype = "reqSyncDoc"
req.gdocid = gdocid
req.userid = userid
req.profile = profile
req.datalen = stats.st_size
req.filename = os.path.basename(fpath)
fdate = datetime.datetime.fromtimestamp(stats.st_mtime)
fdate = fdate.replace(microsecond=0)
req.filedate = fdate
req.parameter = str(userid)
if checkin:
    req.parameter += ';;;checkin'
xmllog = bitfarm_xml.object_to_xml(req)
log.debug(xmllog)

if sid:
    req.sessionid = sid

xml = bitfarm_xml.object_to_xml(req)

try:
    start = time.time()
    sock.send(xml.encode())
    log.debug("reqSyncDoc gesendet")
    nachricht = ''
    try:
        while nachricht[-9:] != '</bfaReq>':
            buff = sock.recv(1024).decode()
            if buff == '':
                break
            nachricht += buff
    except:
        log.exception("Fehler bei der reqSyncDoc (sock.recv)")
        nachricht = ''
    # nur wenn xmlStream komplett ist parsen
    if "</bfaReq>" in nachricht:
        resp = bitfarm_xml.xml_to_object(nachricht)
        log.debug("Response wurde vollständig empfangen: %s" % resp.reqtype)
#                bitfarm_xml.object_to_xmlfile(resp, 'test123.xml')
        if resp.reqtype == 'respError':
            msg = "Serverfehler: %s" % resp.data
            log.error(msg)
            if msgproc:
                msgproc.kill()
            ctypes.windll.user32.MessageBoxW(0, msg, 'Fehler', 0)
            sys.exit(1)

        if resp.reqtype == 'respSendFile':
            streamed = 0
            with open(fpath, 'rb') as f:
                while streamed < stats.st_size:
                    f.seek(streamed)
                    buff = f.read(8192)
                    if buff == "":      # aus irgendwelchen gründen ist die Datei kleiner als beim Öffnen. (Wenn zb noch darauf gearbeitet wird (was eigentlich nicht sein darf))
                        log.error("Die Datei konnte nicht vollständig gelesen werden")
                        break
                    sock.send(buff)
                    streamed += len(buff)

        nachricht = ''
        try:
            while nachricht[-9:] != '</bfaReq>':
                buff = sock.recv(1024).decode()
                if buff == '':
                    break
                nachricht += buff
        except:
            log.exception("Fehler bei der respSendFile (sock.recv)")
            nachricht = ''
        # nur wenn xmlStream komplett ist parsen

        # if not checkin and msgproc:
        if msgproc:
            time.sleep(2)
            msgproc.kill()

        if "</bfaReq>" in nachricht:
            resp = bitfarm_xml.xml_to_object(nachricht)
            if resp.reqtype == 'respOK':
                log.debug("Übertragung erfolgreich")

                try:
                    os.remove(fpath)
                    #os.rmdir(os.path.dirname(fpath))
                    shutil.rmtree(os.path.dirname(fpath), ignore_errors=True)
                except:
                    msg = "Die Datei oder das Verzeichnis %s konnte nicht aufgeräumt werden" % fpath
                    log.exception(msg)
                    ctypes.windll.user32.MessageBoxW(0, msg, 'Fehler', 0)
                    sys.exit(1)
            else:
                msg = "Fehler bei der Übertragung der Datei\n%s" % resp.data
                log.error(msg)
                ctypes.windll.user32.MessageBoxW(0, msg, 'Fehler', 0)
                sys.exit(1)

    else:
        log.debug("unvollständiger Response!")

    #log.debug( "Dauer %f" %(time.time() - start)
except SystemExit:
    log.exception("SystemExit")
    if msgproc:             # keine Meldung mehr, wenn wir über sys.exit() hier hingekommen sind
        msgproc.kill()

except:
    log.exception('Fehler beim Senden der Datei')
    msg = "Server konnte nicht erreicht werden.\nSenden wurde abgebrochen."
    log.error(msg)
    if msgproc:
        msgproc.kill()
    ctypes.windll.user32.MessageBoxW(0, msg, 'Fehler', 0)
    sys.exit(1)

finally:
    sock.close()

