﻿#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Dieses Modul stellt den bfaServer als Applikationsklasse zur Verfügung.
Außerdem ist noch die bfaservice Klasse vorhanden (win32serviceutil.ServiceFramework),
um den bfaServer als Windows Dienst registrieren und starten zu können
"""

__version__ = "v3.6.2.0"

from bfa_version import *

#import cProfile
#import pstats
#import trace
#import timeit
#import traceback
import os
import sys
import threading
import queue
import logging
import logging.config
import time
import datetime
import configparser
import sqlalchemy as sqla
import subprocess
import socket
import base64
import dpapi
from collections import deque
from collections import OrderedDict
import bitfarm_doc
import bitfarm_proc
import bitfarm_xml
from bitfarm_translate import *
import bitfarm_configobj
import bitfarm_history
from bitfarm_errors import *
import bfaMailServer

import ctypes
import win32service
import win32serviceutil
import win32api
import win32file
import win32con
import win32event
import win32evtlogutil
import win32timezone
import win32clipboard
import winerror
import servicemanager
import msvcrt
import warnings
import psutil
from Crypto.Cipher import PKCS1_OAEP
from Crypto.PublicKey import RSA
from threading import Timer
from nslookup import Nslookup

import multiprocessing
multiprocessing.freeze_support()
#https://docs.python.org/2/library/multiprocessing.html#multiprocessing.freeze_support

warnings.filterwarnings(action='ignore', category=UnicodeWarning)
warnings.filterwarnings(action='ignore', module='sqlalchemy')

#GPL / Enterprise
try:
    import bfaServerEnterprise as bfa_classes
    print("bfaServerEnterprise loaded!")
except ImportError:
    print("bfaServerEnterprise not loaded, running GPL-Edition")
    import bfaServerCore as bfa_classes
# import bfaServerCore as bfa_classes

trans = bfaTranslate("bfaServer36")

BUFFER_SIZE = 1024
SPLIT_SIZE = 524288     # 1024*1024/2 #Geschwindigkeit mit 1024*1024/2 am Besten auf meiner Maschine, ggfs mal testen
#filelen_splitsize = 1073741824   #dateien größer ein Megabyte werden partiell gestreamt
CACHE_MAXLEN = 2097152  # 1024*1024*2
CACHE_SIZE = 100
running = False

levels = {'debug': logging.DEBUG,
          'info': logging.INFO,
          'warning': logging.WARNING,
          'error': logging.ERROR,
          'critical': logging.CRITICAL}

class Timer(Timer):
    """
    See: https://hg.python.org/cpython/file/2.7/Lib/threading.py#l1079
    """

    def run(self):
        while not self.finished.is_set():
            self.finished.wait(self.interval)
            self.function(*self.args, **self.kwargs)

        self.finished.set()


class BitfarmServer(threading.Thread):
    requestqueue = queue.Queue(0)
    resultqueue = queue.Queue(0)
    log = logging.getLogger("Main.BitfarmServer")
    reconnect_lock = threading.Lock()
    ch = None

    def __init__(self, service=None, app=None):
        threading.Thread.__init__(self)
        self.service = service
        self.app = app
        self.stopsignal = False
        self.stoptimeout = 0
        self.profiles = None
        self.ports = None
        self.con = None
        self._job = None
        self.bfaSockets = []
        self.bfaWorkers = []
        self.converterhostslist = []
        self.omphostslist = []
        self.converterhost = None
        self.converterhosts = queue.Queue()
        self.omphosts = queue.Queue()
        self.useremoterenderer = False
        self.bfapath = None
        self.mypath = None
        self.tempdir = None
        self.bfaLogin = None
        self.received = 0
        self.processed = 0
        self.server = None
        self.server_ip = None
        self.use_servername = False
        self.mail_server = None
        self.running = False
        self.starting = True
        self.timeout = False
        self.stopping = False
        self.stopped = False
        self.authuser = {}          #dict mit {profile: {(userid,ip):[Sockets]}}
        self.userrights = {}        #dict mit {profile: {arcid: int}}
        self.user = {}              #dict mit {profile: {userid:BitfarmUser}}
        self.in_progress = {}       #dict mit {profile: {gdocid:userid}}
        self.active_user = {}       # dict mit {profile: {(userid,ip):timestamp}}
        self.archives = {}
        self.cnt_named = {}
        self.cnt_concurrent = {}
        self.waitingperiod = {}
        self.starttime = ''
        self.split_size = SPLIT_SIZE
        self.buffer_size = BUFFER_SIZE
        self.cache_maxlen = 1048576
        self.cache_size = 10
        self.ldap_sizelimit = 500
        self.cnt_worker_threads = 5
        self.cache = {}             #dict mit {(gdocid,filepath):stream}
        self.cached = deque()
        self.annofunc = None
        self.gridfunc = None
        self.gridlib = None
        self.gridinit = None
        self.gridfinal = None
        self.gridconnect = None
        self.griddisconnect = None
        self.gridload = None
        self.develop = False
        self.ldapdebug = False
        self.need_reconnect = False
        self.logfile = None
        self.lic_valid = False
        self.bfaDLM_timer = None
        self.bfaDocproc = None
        self.bfaQueue_timer = None
        self.bfaModelLoader = None
        self.queue_counts = {}
        self.basemodel = {}
        self.custommodel = {}
        self.model_loaded = {}
        self.model_training = {}
        self.prediction_threshold = 0.9
        self.training_results = {}
        self.doc_classes = {}
        self.vectorizer = {}
        self.train_matrix = {}
        self.classids = {}
        self.nossl = False
        self.keep_resubs = 90

    def get_status(self):
        _ret = _("Status-Informationen:   (ESC zum Beenden)") + '\n'
        _ret += _("Gestartet %s" % self.starttime) + '\n'
        print(_ret)
        for _profile in self.con.profiles:
            sys.stdout.write(_("Profil: %s (Angemeldete User: %i)\n") % (_profile, len(self.user[_profile])))
            sys.stdout.write(_("Profil: %s (Dokumente in Bearbeitung: %i)\n") % (_profile, len(self.in_progress[_profile])))
        while bfaServer.running:
            time.sleep(.01)

            sys.stdout.write("Empfangene Request %i, verarbeitet %i\r" %(self.received, self.processed))

            x = kbfunc()
            if x == 27:  # ESC
                sys.stdout.write("\n")
                break

    def get_info(self):
        _ret = _("Server-Informationen:") + '\n'
        _ret += _("bfaServer3.6 Version %s" % version) + '\n'
        _ret += _("Gestartet %s" % self.starttime) + '\n'
        _ret += _("bitfarm-Archiv Programmverzeichnis %s" % self.bfapath) + '\n'
        _ret += _("Logdatei %s" % self.logfile) + '\n'

        for _profile in self.con.profiles:
            _ret += _("Geladenes Profil: %s") % _profile + '\n'
            _ret += _("Datenbank Profil: %s auf %s_%s") % (self.con.connections[_profile].db_name, self.con.connections[_profile].server, self.con.connections[_profile].port) + '\n'
            _ret += _("Profil: %s Anzahl named User: %i" % (_profile, self.con.connections[_profile].named)) + '\n'
            if self.con.connections[_profile].concurrent > 0:
                _ret += _("Profil: %s Anzahl concurrent User: %i" % (_profile, self.con.connections[_profile].concurrent)) + '\n'
            if self.cnt_named[_profile] > 0:
                _ret += _("Profil: %s Anzahl angemeldeter Benutzer (named): %i" % (_profile, self.cnt_named[_profile])) + '\n'
            if self.cnt_concurrent[_profile] > 0:
                _ret += _("Profil: %s Anzahl angemeldeter Benutzer (concurrent): %i" % (_profile, self.cnt_concurrent[_profile])) + '\n'
            _ret += _("Profil: %s auto_reconnect: %s") % (_profile,  self.con.connections[_profile].auto_reconnect) + '\n'
        if self.con.connections[_profile].ldap_ssl:
            _ret += _("LDAP Abfrage benutzt SSL auf Port %d" % self.con.connections[_profile].ldap_ssl_port) + '\n'
        _ret += _("Anzahl SocketConnectoren: %i") % len(self.bfaSockets) + '\n'
        _ret += _("Anzahl WorkerThreads: %i") % len(self.bfaWorkers) + '\n'
        _ret += _("Größe Dateicache: %i") % self.cache_size + '\n'
        _ret += _("Maximale Dateigröße Dateicache: %i") % self.cache_maxlen + '\n'
        if self.useremoterenderer:
            _ret += _("Converterhosts: %i [%s]") % (len(self.converterhostslist), [str(x) for x in self.converterhostslist]) + '\n'
        else:
            _ret += _('Ein lokaler Renderer wird verwendet.')

        return _ret

    def get_user(self):
        _ret = ""
        for _profile in self.con.profiles:
            _ret += _("Profil: %s Anzahl mögliche Benutzer (named): %i") % (_profile, self.con.connections[_profile].named) + '\n'
            if self.con.connections[_profile].concurrent > 0:
                _ret += _("Profil: %s Anzahl mögliche Benutzer (concurrent): %i") % (_profile, self.con.connections[_profile].concurrent) + '\n'
            _ret += _("Profil: %s insgesamt angemeldete User: %i") % (_profile, len(self.user[_profile])) + '\n'
            for user in list(self.user[_profile].values()):
                if 'dmsadmin' in user.flags:
                    _ret += _("  DMSAdmin (wird nicht gezählt)") + '\n'
            if self.cnt_named[_profile] > 0:
                _ret += _("Profil: %s Anzahl angemeldeter Benutzer (named): %i") % (_profile, self.cnt_named[_profile]) + '\n'
                for user in list(self.user[_profile].values()):
                    if 'concurrent' not in user.flags:
                        _ret += "  %s" % (user.displayname if user.displayname else user.name) + '\n'
            if self.cnt_concurrent[_profile] > 0:
                _ret += _("Profil: %s Anzahl angemeldeter Benutzer (concurrent): %i") % (_profile, self.cnt_concurrent[_profile]) + '\n'
                for user in list(self.user[_profile].values()):
                    if 'concurrent' in user.flags:
                        _ret += "  %s" % (user.displayname if user.displayname else user.name) + '\n'
        return _ret

    def get_userids(self, _profile):
        _ret = []

        for user in list(self.user[_profile].values()):
            if user.id not in _ret:
                _ret.append(user.id)

        return _ret

    # def get_annotext(self, profile, gdocid):
    #     _ret = ""
    #     if self.annofunc is None:
    #         return _ret
    #     args = [self.annofunc, profile, str(gdocid), '-q']
    #     try:
    #         _ret = subprocess.check_output(args, universal_newlines=True)
    #     except:
    #         self.log.exception(_('Aufruf fehlgeschlagen: AnnotextCMD'))
    #     else:
    #         return _ret

    def get_annotext(self, server, port, db, user, password, gdocid):
        _ret = u""
        try:
            if self.annofunc:
                user = self.con.main.bitencrypt(user)
                password = self.con.main.bitencrypt(password)
                _ret = self.annofunc(ctypes.c_wchar_p(server), ctypes.c_int(port), ctypes.c_wchar_p(db),  ctypes.c_wchar_p(user), ctypes.c_wchar_p(password), ctypes.c_int64(gdocid))
        except:
            self.log.exception(_('Aufruf fehlgeschlagen: Annotext64.dll'))
            self.log.warning('Parameter: %s %d %s %s %s %d' % (server, port, db, user, password, gdocid))
        else:
            return _ret

    def reconnect(self, profile):
        self.log.debug("versuche reconnect für %s" % profile)
        _success = True
        for thread in self.bfaWorkers:
            _success = _success & thread.reconnect(profile)
        return _success

    def EventLogInfo(self, msg):
        if self.service:
            self.service.LogInfoMsg(msg)
        else:
            self.log.info(msg)

    def EventLogWarning(self, msg):
        if self.service:
            self.service.LogWarningMsg(msg)
        else:
            self.log.info(msg)

    def EventlogError(self, msg):
        if self.service:
            self.service.LogErrorMsg(msg)
        else:
            self.log.info(msg)

    def check_dlm(self):
        self.log.info("check_dlm")
        for profile in self.profiles:
            self.log.info("check_dlm für %s" % profile)

    def reconnect_workerthreads(self, profile):
        _success = True
        BitfarmServer.reconnect_lock.acquire()
        try:
            for thread in self.bfaWorkers:
                _success = _success & thread.reconnect(profile)
            self.need_reconnect = not _success
        finally:
            BitfarmServer.reconnect_lock.release()
        self.log.debug(_("%s Reconnect der Worker-Threads durchgeführt. Ergebnis %s") % (self.name, _("erfolgreich") if _success else _("nicht erfolgreich")))

    def read_serverconfig(self, confile):
        level = 'info' #default = info
        try:
            logging.config.fileConfig(confile)
        except:
            self.EventlogError("Konnte Logging Konfiguration nicht lesen, ein TimedRotationFileHandler wird automatisch angelegt")

        self.log = logging.getLogger("Main")
        self.log.propagate = False
        if len(self.log.handlers) == 0:     # keinen Handler geladen (wegen fehlerhafter bfaServer36.ini)
            if not os.path.exists(os.path.join(self.mypath,'logs','bfaServer.log')):
                try:
                    os.mkdir(os.path.join(self.mypath,'logs'))
                except:
                    pass
            fhandler = logging.handlers.TimedRotatingFileHandler(os.path.join(self.mypath,'logs','bfaServer.log'), when='midnight')
            fhandler.setFormatter(MyFormatter('%(asctime)s - %(name)s - %(levelname)s: %(message)s'))
            fhandler.setLevel(logging.INFO)
            fhandler.propagate = False
            self.log.addHandler(fhandler)

        self.log.setLevel(self.log.handlers[0].level)
        self.ch = logging.StreamHandler(sys.stdout)
        if not self.service:
            # create console handler and set level to debug
            if len(sys.argv) > 1:
                for arg in sys.argv:
                    if '-log:' in arg:
                        level = arg[5:].lower()
                        break
            self.ch.setLevel(levels[level])
            formatter = MyFormatter('%(name)s-%(levelname)s:  %(message)s')
            self.ch.setFormatter(formatter)
            self.ch.args = sys.stdout
            self.log.addHandler(self.ch)

        for handler in self.log.handlers:
            if isinstance(handler, logging.handlers.TimedRotatingFileHandler):
                self.EventLogInfo(handler.baseFilename)
                if self.service and not os.path.exists(handler.baseFilename):
                    handler.baseFilename = os.path.join(self.mypath, 'logs', 'bfaServer.log')
                elif not os.path.exists(os.path.dirname(handler.baseFilename)):
                    handler.baseFilename = os.path.join(self.mypath, 'logs', 'bfaServer.log')
                    handler.suffix += '.log'
                    self.logfile = handler.baseFilename
                    self.EventLogInfo("Log-Datei: %s" % handler.baseFilename)

        if self.develop:
            logging.getLogger('sqlalchemy.engine').setLevel(logging.DEBUG)

        config = configparser.ConfigParser({'cache_size': str(self.cache_size), 'cache_maxlen': str(self.cache_maxlen)}, allow_no_value=True)
        try:
            # config.read_file(open(confile))
            config.read([confile], encoding='utf-8')
            # config.readfp(open(confile))
        except:
            self.EventlogError(_('Konnte bfaServer36.ini nicht öffnen %s' % confile))
            self.starting = False
            return None
        else:
            self.log.info(_("Konfigurationsfile: %s " % confile))
        if not config.has_section('Server'):
            self.EventlogError(_('Section Server in bfaServer36.ini nicht gefunden'))
            self.starting = False
            return None
        try:
            profiles = None
            if config.has_option('Server', 'profiles'):
                profiles = config.get('Server', 'profiles')
            if profiles is None or profiles == "":
                self.EventlogError(_("Es wurde kein Profil übergeben"))
                self.starting = False
                return None
            self.profiles = profiles.lower().split(';')

            ports = None
            if config.has_option('Server', 'ports'):
                ports = config.get('Server', 'ports')
            if ports is None or ports == "":
                self.EventlogError(_("Es wurde kein Port übergeben"))
                self.starting = False
                return None
            self.ports = [int(x) for x in ports.split(';')]

            if config.has_option('Server', 'server'):
                self.server = config.get('Server', 'server')

            if config.has_option('Server', 'nossl'):
                try:
                    self.nossl = config.getboolean('Server', 'nossl')
                except:
                    self.nossl = False

            if config.has_option('Server', 'use_servername'):
                try:
                    self.use_servername = config.getboolean('Server', 'use_servername')
                except:
                    self.use_servername = False

            omphosts = []
            if config.has_option('Server', 'ocrhosts'):
                omphosts = config.get('Server', 'ocrhosts')
            if len(omphosts) > 0:
                omphosts = [host for host in omphosts.split(';')]

            if config.has_option('Server', 'cache_size'):
                self.cache_size = config.getint('Server', 'cache_size')
            if config.has_option('Server', 'cache_maxlen'):
                self.cache_maxlen = config.getint('Server', 'cache_maxlen')
            if config.has_option('Server', 'ldap_sizelimit'):
                try:
                    self.ldap_sizelimit = config.getint('Server', 'ldap_sizelimit')
                except:
                    pass
            if config.has_option('Server', 'cnt_worker_threads'):
                self.cnt_worker_threads = config.getint('Server', 'cnt_worker_threads')
            if "isolated" in config.options("Server"):
                isolated = [host for host in config.get('Server', 'isolated').split(';')]
            else:
                isolated = []
            if config.has_option('Server', 'keep_resubs'):
                try:
                    self.keep_resubs = config.getint('Server', 'keep_resubs')
                except:
                    pass
            chosts = []
            if config.has_option('Server', 'converterhosts'):
                chosts = config.get('Server', 'converterhosts')
            converterhosts = []
            if len(chosts) > 0:
                for host in chosts.split(';'):
                    ip, port = host.split(":")
                    if ip in isolated:
                        converterhosts.append((host, True))
                    else:
                        converterhosts.append((host, False))
            if bfa_classes.EDITION == 'Enterprise':
                if config.has_option('Server', 'httppreviewdir'):
                    self.httppreviewdir = config.get('Server', 'httppreviewdir')
                else:
                    self.httppreviewdir = os.path.join(self.bfapath, 'bin', 'httppreviews')
                if not os.path.isdir(self.httppreviewdir):
                    os.makedirs(self.httppreviewdir)

            for host in converterhosts:
                if host != "":
                    self.converterhosts.put(host)
            if len(converterhosts) == 0:
                self.useremoterenderer = False
            else:
                self.useremoterenderer = True
            for host in omphosts:
                if host != "":
                    self.omphosts.put(host)

        except Exception as e:
            self.EventlogError(_('Konnte bfaServer.ini nicht lesen\n%s' % str(e)))
            self.starting = False
            return None

        return True

    def load_gridfield_dll(self):
        gridlibpath = os.path.join(self.mypath, 'bfaGridfield64.dll')
        self.log.debug("load %s" % gridlibpath)
        if not os.path.exists(gridlibpath):
            self.log.error(_('Konnte bfaGridfield64.dll nicht finden'))
            self.gridfunc = None
            return False
        else:
            self.gridlib = ctypes.WinDLL(gridlibpath)
            self.gridfunc = self.gridlib['GetExport']
            self.gridfunc.restype = ctypes.c_wchar_p
            self.gridinit = self.gridlib['InitializeDXLibrary']
            self.gridfinal = self.gridlib['FinalizeDXLibrary']
            self.gridconnect = self.gridlib['ConnectDB']
            self.gridconnect.restype = ctypes.c_int
            self.griddisconnect = self.gridlib['DisconnectDB']
            self.griddisconnect.restype = ctypes.c_int
            self.gridload = self.gridlib['LoadGridfields']
            self.gridinit()
            return True

    def connect_gridfield_dll(self):
        _rc = self.gridconnect(ctypes.c_wchar_p(";".join(self.con.profiles)))
        if _rc != 0:
            self.log.error(_("Achtung, konnte bfaGridfield64.dll nicht an Datenbank anmelden"))
            self.close_gridfield_dll()
            return False
        else:
            self.gridload()
            return True

    def close_gridfield_dll(self):
        if self.gridlib:
            try:
                self.griddisconnect()
            except:
                self.log.exception(_("Fehler bei griddisconnect"))
            try:
                self.gridfinal()
            except:
                self.log.exception(_("Fehler bei gridfinal"))
            gridlib_handle = self.gridlib._handle
            del self.gridlib
            try:
                ctypes.windll.kernel32.FreeLibrary(gridlib_handle)
            except:
                self.log.exception(_("Fehler bei FreeLibrary"))
        self.gridlib = None
        self.gridfunc = None

    def run(self):
        self.starting = True
        self.running = False
        dirversion = False
        mypath = os.path.split(os.path.abspath(sys.argv[0]))[0]
        mypath = win32file.GetLongPathName(mypath)
        if os.path.split(mypath)[1] == 'bfaServer36':   # laufen wir als OneDir Version?
            mypath = os.path.split(mypath)[0]           # dann ist mypath das übergeordnete Verzeichnis
            dirversion = True
        # level = 'info' #default = info
        # levels = {'debug': logging.DEBUG,
        #           'info': logging.INFO,
        #           'warning': logging.WARNING,
        #           'error': logging.ERROR,
        #           'critical': logging.CRITICAL}
        try:
            bfapath = os.environ["bitfarm-archiv"]
        except:
            if self.service:
                self.service.LogErrorMsg(_('Konnte Systemvariable %bitfarm-archiv% nicht auflösen'))
            bfapath = mypath
        if self.service:
            #TODO: ich bau hier den Pfad mal zusammen, besser wäre aus HKLM\CurrentControlSet\Services\bfaServer36\PythonClass zu lesen
            mypath = os.path.join(bfapath, 'bin')
            self.service.LogInfoMsg(_('Initialisiere Dienst bfaServer. bfapath %s mypath %s cwd %s') % (bfapath, mypath, os.getcwd()))
        confile = os.path.join(mypath, "bfaServer36.ini")
        # confile = os.path.join(bfapath, "bin", "bfaServer36.ini")

        self.bfapath = bfapath
        self.mypath = mypath
        self.tempdir = os.path.join(bfapath, 'temp')

        if not os.path.exists(self.tempdir):
            try:
                os.mkdir(self.tempdir)
            except:
                self.log.exception(_('Konnte %s nicht anlegen') % self.tempdir)

        tempdir = os.path.join(mypath, 'temp')
        if not os.path.exists(tempdir):
            try:
                os.mkdir(tempdir)
            except:
                self.log.exception(_('Konnte %s nicht anlegen') % tempdir)

        if not os.path.exists(confile):
            confile = os.path.join(self.bfapath, "bin", "bfaServer36.ini")
            # confile = os.path.join(mypath, "bfaServer36.ini")
        if os.path.exists(confile):
            if not self.read_serverconfig(confile):
                return None
        else:
            self.EventlogError(_('Konnte bfaServer36.ini nicht finden. Start wird abgebrochen'))
            self.starting = False
            print(_('Konnte bfaServer36.ini nicht finden. Start wird abgebrochen'))
            return None

        self.log.info('+++++++++++++++++ Bitfarm Server ++++++++++++++++++++++++')
        self.log.info("bitfarm Informationsysteme GmbH, 2024")
        self.log.info("bitfarm Server 36 Version %s %s" % (bfa_classes.__name__, version))
        self.log.info("Edition: %s" % bfa_classes.EDITION)
        if self.service:
            self.log.info(_('als Dienst gestartet'))
        else:
            self.log.info(_('im Modus %s gestartet') % self.app)
        self.log.info(_('Server läuft in der Verzeichnisversion'))
        myuser = win32api.GetUserNameEx(win32con.NameSamCompatible)
        self.log.info(_("Ausführender Windowsbenutzer %s") % myuser)
        servername = socket.getfqdn()
        self.log.info("Server: %s " % self.server)
        self.log.info("Server Hostname: %s " % servername)

        dns_query = Nslookup()
        try:
            ips_record = dns_query.dns_lookup(servername)
            self.server_ip = ips_record.answer[0]
            self.log.info(_("Server IP: %s (per NsLookup)") % self.server_ip)
        except:
            try:
                self.server_ip = socket.gethostbyname(servername)
                self.log.info(_("Server IP: %s (per gethostbyname)") % self.server_ip)
            except:
                self.log.exception(_('Server IP konnte nicht ermittelt werden, wird auf 0.0.0.0 gesetzt'))
                self.server_ip = '0.0.0.0'

        self.log.debug(_('Server initialisiert bfapath = %s') % self.bfapath)
        self.log.debug(_('Server initialisiert mypath = %s') % self.mypath)

        try:
            self.con = bitfarm_proc.BitfarmConnections(self.profiles)
        except:
            self.log.exception(_('Konnte Profil(e) nicht lesen: %s') % self.profiles)
            sys.exit(1)
        loop = 0
        while self.con.main.engine is None:
            self.con.create_db_engine(pool_size=(self.cnt_worker_threads * 2) + 4)
            if self.con.main.engine is None:
                if loop < 10:
                    loop += 1
                    time.sleep(3)
                else:
                    self.log.critical(_('Konnte keine DatenbankEngine erstellen, Timeout! Beende Dienst...'))
                    self.EventlogError(_('Konnte keine DatenbankEngine erstellen, Timeout! Beende Dienst...'))
                    self.starting = False
                    return None

        self.log.info(_('Datenbank-Engine erstellt.'))
        # self.con.connect_db()
        loop = 0
        while self.con.main.db is None:
            self.con.connect_db()
            if self.con.main.db is None:
                if loop < 10:
                    loop += 1
                    time.sleep(3)
                else:
                    self.log.critical(_('Konnte nicht mit der MySQL verbinden, Timeout! Beende Dienst...'))
                    self.EventlogError(_('Konnte nicht mit der MySQL verbinden, Timeout! Beende Dienst...'))
                    self.starting = False
                    return None
        self.log.info(_('Datenbankverbindung hergestellt.'))

        if not self.con.read_ini():
            self.log.error(_("scripts.ini konnte nicht gelesen werden"))
        else:
            for profile in self.profiles:
                if 'svn' not in self.con[profile].ini:
                    self.log.warning(_("Keine SVN-Konfiguration für Profil %s gefunden. Profil läuft ohne Versionierung.") % profile)

        self.bfaDocProc = bitfarm_proc.BitfarmDocProcessing(self.con, _con_db=self.con.db, name=self.name)

        for profile in self.profiles:
            self.authuser[profile] = {}
            self.active_user[profile] = {}
            self.userrights[profile] = {}
            self.user[profile] = {}
            self.in_progress[profile] = {}
            self.archives[profile] = {}
            self.cnt_named[profile] = 0
            self.cnt_concurrent[profile] = 0
            self.waitingperiod[profile] = None
            self.queue_counts[profile] = OrderedDict()
            self.basemodel[profile] = None
            self.custommodel[profile] = None
            self.model_loaded[profile] = False
            self.model_training[profile] = False
            self.training_results[profile] = {'success' : False, 'error' : '', 'accuracy' : 0, 'loss' : 0, 'classes': '',
                                              'docs_per_class' : 0, 'running' : False, 'results' : '', 'classifier_running': False,
                                              'classifier_success': False, 'classifier_error': ''}
            self.doc_classes[profile] = {}  # Key classid, Value Tupel (classname, typ)
            self.vectorizer[profile] = None
            self.train_matrix[profile] = None
            self.classids[profile] = None

        for profile in self.con.profiles:
            self.log.info(_("Lese Archivkonfiguration für %s") % profile)
            if self.develop:
                start = time.perf_counter()
            archives = self.con.connections[profile].read_arctree(self.con.connections[profile].db)
            for archive in archives:
                if archive.id not in self.archives[profile]:
                    self.archives[profile][archive.id] = archive
            if self.develop:
                self.log.info('Process time: %s' % str(time.perf_counter() - start))
                with open(os.path.join(self.bfapath, 'archives.xml'),'w') as fn:
                    bitfarm_xml.object_to_xmlfile(archives, fn)

        for profile in self.con.profiles:
            if self.con[profile].gpl:
                if not self.con[profile].bfauser:   # None or b''
                    self.con[profile].bfauser = b'dmsadmin'
                    dmsadmin = self.con[profile].read_user(name='dmsadmin')
                    if dmsadmin and 'dmsadmin' in dmsadmin.flags:
                        password = self.con[profile].bitdecrypt(dmsadmin.password)
                        if password:
                            self.con[profile].bfapass = password.encode()
                            if os.path.exists(self.con[profile].confile):
                                ini = bitfarm_configobj.ConfigObj(infile=self.con[profile].confile, encoding="cp1252",
                                                                  write_empty_values=True)
                                if 'client' not in ini:
                                    ini['client'] = {}
                                ini['client']['bfauser'] = base64.b64encode(dpapi.cryptData(self.con[profile].bfauser)).decode()
                                ini['client']['bfapass'] = base64.b64encode(dpapi.cryptData(self.con[profile].bfapass)).decode()
                                ini.write()
                                ini = None
                                dmsadmin = None
                                password = None

#        self.log.info(_(u"starte Mailserver"))
#        self.mail_server = bfaMailServer.BitfarmMailServer(self.con)
#        if self.mail_server.smtp:
#            self.mail_server.start()

        # os.chdir(self.mypath)
        # annolibpath = os.path.join(self.bfapath, 'AnnoTextCMD.exe')
        # if not os.path.exists(annolibpath):
        #     self.log.exception(_('Konnte AnnoTextCMD.exe nicht laden'))
        #     self.annofunc = None
        # else:
        #     self.annofunc = annolibpath

        annolibpath = os.path.join(self.mypath, 'AnnoText64.dll')
        self.log.debug("load %s" % annolibpath)
        if not os.path.exists(annolibpath):
            self.log.error(_('Konnte AnnoText64.dll nicht finden'))
            self.annofunc = None
        else:
            try:
                # annolib = ctypes.PyDLL(os.path.join(self.mypath, 'AnnoText64.dll'))
                annolib = ctypes.WinDLL(os.path.join(self.mypath, 'AnnoText64.dll'))
                self.annofunc = annolib['GetAnnoText']
                self.annofunc.restype = ctypes.c_wchar_p
            except:
                self.log.exception(_('Konnte AnnoText64.dll nicht laden'))
                self.annofunc = None

        if self.load_gridfield_dll():
            self.connect_gridfield_dll()

        # Lizenz einlesen
        if not os.path.exists(os.path.join(self.mypath, 'certs', 'bfaServer36.pem')):
            self.log.warning("Achtung, bfaServer36.pem nicht gefunden")
            for profile in self.con.profiles:
                self.con[profile].named = 1     # hier versucht jemand zu tricksen? das zertifikat ist eigentlich immer da...
                self.con[profile].concurrent = 0
        else:
            key = RSA.importKey(open(os.path.join(self.mypath, 'certs', 'bfaServer36.pem')).read())
            cipher = PKCS1_OAEP.new(key)

            for profile in self.profiles:
                bfa_classes.read_registry_file(self, profile, cipher)

        # erst LoginThread starten, wenn keine DB Verbindung besteht ist hier dann auch schon wieder Schluss
        try:
            self.bfaLogin = bfa_classes.BitfarmLoginThread(self)
            self.bfaLogin.start()
        except:
            self.log.exception(_('Der LoginThread konnte nicht gestartet werden. Der Server wird wieder beendet'))
            self.starting = False
            return None

        # dann die WorkerThreads
        try:
            self.log.info(_('Starte WorkerThreads'))
            self.bfaWorkers = [bfa_classes.BitfarmWorkerThread(self, bfa_classes.BfaClassifier) for x in range(self.cnt_worker_threads)]
            for bfaWorker in self.bfaWorkers:
                bfaWorker.start()
        except:
            self.log.exception(_('Die WorkerThreads konnten nicht gestartet werden. Der Server wird wieder beendet'))
            self.starting = False
            return None

        _running = False
        _timeout = False
        _starting = True
        while _starting and not _running and not _timeout:
            for bfaWorker in self.bfaWorkers:
                _starting = _starting and bfaWorker.starting
                _timeout = _timeout or (time.time() > bfaWorker.timeout > 0)
                _running = _running or bfaWorker.running
            time.sleep(0.1)

        if not _running or _timeout:
            self.log.error(_("Die WorkerThreads konnten nicht gestartet werden. Der Server wird wieder beendet"))
            self.log.debug("starting %s runnning %s timeout %s" %(_starting, _running, str(self.timeout)))
            self.starting = False
            return None

        # dannn die SocketConnectoren
        for port in self.ports:
            self.log.info(_("Starte SocketConnector auf Port %i") % port)
            self.bfaSockets.append(bfa_classes.BitfarmSocketConnector(self, port))
        for bfaSocket in self.bfaSockets:
            bfaSocket.start()

        _running = False
        _timeout = False
        _starting = True
        while _starting and not _running and not _timeout:
            for bfaSocket in self.bfaSockets:
                _starting = _starting and bfaSocket.starting
                _timeout = _timeout or (time.time() > bfaSocket.timeout > 0)
                _running = _running or bfaSocket.running
            time.sleep(0.1)

        if not _running or _timeout:
            self.log.error(_("Die SocketConnectoren konnten nicht gestartet werden. Der Server wird wieder beendet"))
            self.log.debug("starting %s runnning %s timeout %s" % (_starting, _running, str(self.timeout)))
            self.starting = False
            return None

        # Timer Thread starten

        self.bfaDLM_timer = bfa_classes.BitfarmDLMCheck(self)      # einmal pro Tag
        self.bfaDLM_timer.start()

        self.bfaQueue_timer = bfa_classes.BitfarmQueueCheck(self)      # alle 30sec
        self.bfaQueue_timer.start()

        # if 'BitfarmModelLoader' in bfa_classes.__dict__:
        #     self.bfaModelLoader = bfa_classes.BitfarmModelLoader(self)
        #     self.bfaModelLoader.start()

        for profile in self.profiles:
            if self.con[profile].dont_check_expired:
                self.log.info(_('Profil %s: das DLM wurde deaktiviert') % profile)

        # for profile in self.profiles:
        #     self.bfaLogin.clear_state(profile)

        self.running = True
        self.starting = False
        self.starttime = datetime.datetime.now().strftime('%H:%M:%S %d.%m.%Y')
        self.log.info(self.get_info())
        self.log.info('BitfarmServer running, ready for requests...')
        self.need_reconnect = False
        while self.is_alive():
            _job = None
            _result = None
            xml = None
            try:
                _job = self.requestqueue.get(False)
            except queue.Empty:
                # self.log.debug(u"Nichts vom MP übergeben, dann weiter")
                # da wir nicht blockierend arbeiten, machen wir bei leerer Queue etwas Pause
                time.sleep(0.01)
            except:
                self.log.exception(_('%s Fehler bei get_next_job') % self.name)

            if _job:
                self.received += 1
                self.log.debug(_("%s Job empfangen <%s>") % (self.name, hex(id(_job.sock))))

                # Datenbankverbindung noch aktiv?
                try:
                    self.con.connections[_job.profile].db.execute('select now()')
                except sqla.exc.OperationalError:
                    self.log.error(_("%s Keine Datenbankverbindung zu %s, versuche erneute Abfrage...") % (self.name, _job.profile))
                    self.need_reconnect = True
                    time.sleep(1)
                    try:
                        self.con.connections[_job.profile].db.execute('select now()')
                    except sqla.exc.OperationalError:
                        self.log.error(_("%s Keine Datenbankverbindung zu %s, versuche reconnect (2)...") % (self.name, _job.profile))
                        self.need_reconnect = True
                        try:
                            self.con.connections[_job.profile].engine.connect()
                        except sqla.exc.OperationalError:
                            self.log.error(_("%s Reconnect gescheitert") % self.name)
                            xml = bitfarm_xml.create_xml_request(reqtype="respError", data=_("Keine Datenbankverbindung, Funktion kann nicht ausgeführt werden."))
                            try:
                                _job.sock.send(xml)
                            except:
                                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                            else:
                                self.processed += 1
                            _job = None
                        else:
                            self.log.warning(_("%s Reconnect erfolgreich") % self.name)
                    else:
                        self.log.warning(_("%s Zweite Abfrage erfolgreich") % self.name)

            if _job:
                if self.need_reconnect:       # jetzt alle Workerthreads neu Verbinden lassen
                    self.reconnect_workerthreads(_job.profile)

                    if self.gridlib:
                        self.connect_gridfield_dll()
                    else:
                        if self.load_gridfield_dll():
                            self.connect_gridfield_dll()

                try:
                    # hier Fragen wir ggf weitere Bedingungen ab und stellen den Job in die WorkerQueue
                    # eine Reihe von Requests werden im Servermainloop direkt verarbeitet
                    if _job.reqtype == 'reqReconnect' and _job.userid in self.user[_job.profile]:
                        if 'admin' in self.user[_job.profile][_job.userid].flags:
                            if self.reconnect(_job.profile):
                                xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("Reconnect durchgeführt"))
                            else:
                                xml = bitfarm_xml.create_xml_request(reqtype="respError", data=_("Reconnect nicht erfolgreich"))
                        else:
                            xml = bitfarm_xml.create_xml_request(reqtype="respAuthError", data=_("Keine Berechtigung"))
                        try:
                            _job.sock.send(xml)
                        except:
                            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                        else:
                            self.processed += 1

                    elif _job.reqtype == 'reqSetLogLevel' and _job.userid in self.user[_job.profile]:
                        if 'admin' in self.user[_job.profile][_job.userid].flags:
                            if _job.data == 'info':
                                setlog = logging.getLogger('Main')
                                for hnd in setlog.handlers:
                                    hnd.setLevel(logging.INFO)
                            elif _job.data == 'debug':
                                setlog = logging.getLogger('Main')
                                for hnd in setlog.handlers:
                                    hnd.setLevel(logging.DEBUG)
                            xml = bitfarm_xml.create_xml_request(reqtype="respOK", data= _("Loglevel gesetzt"))
                        else:
                            xml = bitfarm_xml.create_xml_request(reqtype="respAuthError", data= _("Keine Berechtigung"))
                        try:
                            _job.sock.send(xml)
                        except:
                            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                        else:
                            self.processed += 1
                    elif _job.reqtype == 'reqStartSocketLog' and _job.userid in self.user[_job.profile]:
                        if 'admin' in self.user[_job.profile][_job.userid].flags:
                            if _job.data:
                                socketHandler = logging.handlers.SocketHandler(_job.data,
                                    logging.handlers.DEFAULT_TCP_LOGGING_PORT)
                                setlog = logging.getLogger('Main')
                                setlog.addHandler(socketHandler)

                            xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("Loglevel gesetzt"))
                        else:
                            xml = bitfarm_xml.create_xml_request(reqtype="respAuthError", data=_("Keine Berechtigung"))
                        try:
                            _job.sock.send(xml)
                        except:
                            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                        else:
                            self.processed += 1
                    elif _job.reqtype == 'reqStopSocketLog' and _job.userid in self.user[_job.profile]:
                        if 'admin' in self.user[_job.profile][_job.userid].flags:
                            if _job.data:
                                setlog = logging.getLogger('Main')
                                for hnd in setlog.handlers[:]:
                                        if type(hnd) is logging.handlers.SocketHandler:
                                            if hnd.host == _job.data:
                                                hnd.close()
                                                setlog.handlers.remove(hnd)

                            xml = bitfarm_xml.create_xml_request(reqtype= "respOK", data= _("Loglevel gesetzt"))
                        else:
                            xml = bitfarm_xml.create_xml_request(reqtype= "respAuthError", data= _("Keine Berechtigung"))
                        try:
                            _job.sock.send(xml)
                        except:
                            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                        else:
                            self.processed += 1
                    elif _job.reqtype == 'shutdownServer' and _job.userid in self.user[_job.profile]:
                        if 'admin' in self.user[_job.profile][_job.userid].flags:
                            self.stopsignal = True
                            xml = bitfarm_xml.create_xml_request(reqtype="respOK", data= _("Shutdown eingeleitet"))
                        else:
                            xml = bitfarm_xml.create_xml_request(reqtype="respAuthError", data= _("Keine Berechtigung"))
                        try:
                            _job.sock.send(xml)
                        except:
                            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                        else:
                            self.processed += 1
                        if self.stopsignal:
                            self.shutdown()
                    elif _job.reqtype == 'reqServerInfo' and _job.userid in self.user[_job.profile]:
                        if 'admin' in self.user[_job.profile][_job.userid].flags:
                            data = self.get_info()
                            xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=data)
                        else:
                            xml = bitfarm_xml.create_xml_request(reqtype="respAuthError", data= _("Keine Berechtigung"))
                        try:
                            _job.sock.send(xml)
                        except:
                            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                        else:
                            self.processed += 1
                    elif _job.reqtype == 'reqUserInfo' and _job.userid in self.user[_job.profile]:
                        if 'admin' in self.user[_job.profile][_job.userid].flags:
                            data = self.get_user()
                            xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=data)
                        else:
                            xml = bitfarm_xml.create_xml_request(reqtype="respAuthError", data= _("Keine Berechtigung"))
                        try:
                            _job.sock.send(xml)
                        except:
                            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                        else:
                            self.processed += 1
                    elif _job.reqtype == 'reqLoggedonUser' and _job.userid in self.user[_job.profile]:
                        if 'admin' in self.user[_job.profile][_job.userid].flags:
                            data = self.get_userids(_job.profile)
                            data = ",".join([str(x) for x in data])
                            xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=data)
                        else:
                            xml = bitfarm_xml.create_xml_request(reqtype="respAuthError", data= _("Keine Berechtigung"))
                        try:
                            _job.sock.send(xml)
                        except:
                            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                        else:
                            self.processed += 1
                    elif _job.reqtype == 'reqLogoutUser' and _job.userid in self.user[_job.profile]:
                        if 'admin' in self.user[_job.profile][_job.userid].flags:
                            userid = _job.parameter
                            found = False
                            for userkey in self.authuser[_job.profile]:
                                if userkey[0] == int(_job.parameter):
                                    found = True
                                    break
                            if found:
                                for sock in self.authuser[_job.profile][userkey]:
                                    self.bfaLogin.queue.put(('clear_authlist', sock))
                                self.bfaLogin.queue.put(('remove_userkey', (_job.profile, userkey)))
                                xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=data)
                            else:
                                xml = bitfarm_xml.create_xml_request(reqtype="respError", data=data)
                        else:
                            xml = bitfarm_xml.create_xml_request(reqtype="respAuthError", data= _("Keine Berechtigung"))
                        try:
                            _job.sock.send(xml)
                        except:
                            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                        else:
                            self.processed += 1
                    elif _job.reqtype == 'reqRegistration' and _job.userid in self.user[_job.profile]:
                        if 'admin' in self.user[_job.profile][_job.userid].flags:
                            reg = "%s;%s" % (self.con[_job.profile].named, self.con[_job.profile].concurrent)

                            xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=reg)
                        else:
                            xml = bitfarm_xml.create_xml_request(reqtype="respAuthError", data= _("Keine Berechtigung"))
                        try:
                            _job.sock.send(xml)
                        except:
                            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                        else:
                            self.processed += 1
                    elif _job.reqtype in ['reqSaveAdmin', 'reqResetAddfields', 'reqADTree'] and _job.userid in self.user[_job.profile]:
                        if 'admin' in self.user[_job.profile][_job.userid].flags:
                            bfa_classes.BitfarmWorkerThread.queue.put(_job)
                        else:
                            xml = bitfarm_xml.create_xml_request(reqtype="respAuthError", data= _("Keine Berechtigung"))
                            try:
                                _job.sock.send(xml)
                            except:
                                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                            else:
                                self.processed += 1
                    elif _job.reqtype == 'reqReloadConfig' and _job.userid in self.user[_job.profile]:
                        if 'admin' not in self.user[_job.profile][_job.userid].flags:
                            xml = bitfarm_xml.create_xml_request(reqtype="respAuthError", data= _("Keine Berechtigung"))
                        else:
                            if not os.path.exists(confile):
                                xml = bitfarm_xml.create_xml_request(reqtype="respError", data= _("Die Konfigurationsdatei %s ist nicht mehr vorhanden. Neuladen nicht möglich.") % confile)
                            else:
                                if not self.read_serverconfig(confile):
                                    xml = bitfarm_xml.create_xml_request(reqtype="respError", data= _("Fehler beim Lesen der Konfigurationsdatei %s. Neuladen nicht möglich.") % confile)
                                else:
                                    if _job.profile not in self.profiles:
                                        xml = bitfarm_xml.create_xml_request(reqtype="respError", data= _("Achtung! Aktuelles Profil %s nicht mehr in der Konfigurationsdatei vorhanden.\nNach dem nächsten Serverneustart kann sich nicht mehr über dieses Profil angemeldet werden..") % _job.profile)
                                    else:
                                        error = ""
                                        reload_profiles = [_job.profile]        # wurde ein neues Profil hinzugefügt. Dann muss das auch geladen werden und eine DB Engine erstellt werden
                                        for profile in self.profiles:
                                            if profile not in self.con.profiles:
                                                reload_profiles.append(profile)

                                        # aktuelles Profil neuladen (und ggf zusätzliches)
                                        for profile in reload_profiles:
                                            if profile in self.con.connections:
                                                self.con[profile].disconnect_db()
                                                self.con[profile].dispose_engine()
                                            self.con.connections[profile] = bitfarm_proc.BitfarmCon(profile)

                                            while self.con[profile].engine is None:
                                                try:
                                                    self.con[profile].create_db_engine(pool_size=(self.cnt_worker_threads * 2) + 4)
                                                    round = 0
                                                    if self.con[profile].engine is None:
                                                        if round < 12:
                                                            round += 1
                                                            time.sleep(4)
                                                        else:
                                                            self.log.critical(_('Konnte keine Datenbank-Engine erstellen, Timeout!'))
                                                            self.EventlogError(_('Konnte keine Datenbank-Engine erstellen, Timeout!'))
                                                            error += _("Datenbank-Engine für Profil %s konnte nicht erstellt werden.\n") % profile
                                                except:
                                                    self.log.exception(_("%s Datenbank-Engine für Profil %s konnte nicht erstellt werden.\n") % (self.name, profile))
                                                    error += _("Datenbank-Engine für Profil %s konnte nicht erstellt werden.\n") % profile
                                                    break

                                            if self.con[profile].engine is None:
                                                error += _("Datenbank für Profil %s konnte nicht verbunden werden.\n") % profile
                                                self.log.error(_("%s Datenbank für Profil %s konnte nicht verbunden werden.") % (self.name, profile))
                                                # xml = bitfarm_xml.create_xml_request(reqtype="respError", data= _(u"Achtung! Datenbank konnte nicht wieder verbunden werden."))
                                            else:
                                                self.log.info(_('%s Datenbank-Engine erstellt.') % self.name)

                                                round = 0
                                                while self.con[profile].db is None:
                                                    try:
                                                        self.con[profile].connect_db()
                                                        if self.con[profile].db is None:
                                                            if round < 12:
                                                                round += 1
                                                                time.sleep(4)
                                                            else:
                                                                self.log.critical(_('Konnte nicht mit der MySQL verbinden, Timeout! '))
                                                                self.EventlogError(_('Konnte nicht mit der MySQL verbinden, Timeout!'))
                                                                error += _("Datenbank für Profil %s konnte nicht wieder verbunden werden.\n") % profile
                                                    except:
                                                        self.log.exception(_("%s Datenbank für Profil %s konnte nicht wieder verbunden werden.\n") % (self.name, profile))
                                                        error += _("Datenbank für Profil %s konnte nicht wieder verbunden werden.\n") % profile
                                                        break

                                                if self.con[profile].db is None:
                                                    # xml = bitfarm_xml.create_xml_request(reqtype="respError", data= _(u"Achtung! Datenbank konnte nicht wieder verbunden werden."))
                                                    error += _("Datenbank für Profil %s konnte nicht wieder verbunden werden.\n") % profile
                                                else:
                                                    self.log.info(_('Datenbank-Connection erstellt.'))
                                                    self.con.db[profile] = self.con[profile].db     # neu zuweisen

                                                    self.con[profile].read_ini()

                                                    if 'svn' not in self.con[profile].ini and not self.con[profile].gpl:
                                                        self.log.warning(_("Keine SVN-Konfiguration für Profil %s gefunden. Profil läuft ohne Versionierung.") % profile)

                                                    if profile not in self.authuser:     # sollten die array noch nicht gefüllt sein (neues Profil ist hinzugegkommen)
                                                        self.authuser[profile] = {}
                                                    if profile not in self.active_user:
                                                        self.active_user[profile] = {}
                                                    if profile not in self.userrights:
                                                        self.userrights[profile] = {}
                                                    if profile not in self.user:
                                                        self.user[profile] = {}
                                                    if profile not in self.in_progress:
                                                        self.in_progress[profile] = {}
                                                    if profile not in self.archives:
                                                        self.archives[profile] = {}
                                                    if profile not in self.cnt_named:
                                                        self.cnt_named[profile] = 0
                                                    if profile not in self.cnt_concurrent:
                                                        self.cnt_concurrent[profile] = 0
                                                    if profile not in self.waitingperiod:
                                                        self.waitingperiod[profile] = None
                                                    if profile not in self.queue_counts:
                                                        self.queue_counts[profile] = OrderedDict()

                                                    self.log.info(_("Lese Archivkonfiguration neu ein für %s") % profile)
                                                    archives = self.con.connections[profile].read_arctree(self.con.connections[profile].db)
                                                    for archive in archives:
                                                        self.archives[profile][archive.id] = archive

                                                    self.log.info(_("Lese Benutzerkonfiguration neu ein für %s") % profile)
                                                    for user in list(self.user[profile].values()):
                                                        obj = self.con[profile].read_user(db=self.con.connections[profile].db, userid=user.id, logid=self.name)
                                                        if obj:
                                                            self.user[profile][obj.id] = obj

                                                    self.log.info(_("Lese Benutzerberechtigungen für %s") % profile)
                                                    for userid in list(self.userrights[profile].keys()):
                                                        self.bfaLogin.read_userrights(userid, profile, db=self.con.connections[profile].db)

                                                    bfa_classes.read_registry_file(self, profile, cipher)

                                                    # jetzt die Threads reconnecten
                                                    BitfarmServer.reconnect_lock.acquire()
                                                    try:
                                                        self.bfaLogin.reconnect(profile)
                                                        self.bfaDLM_timer.reconnect(profile)
                                                        self.bfaDLM_timer.checked[profile] = False          # zurücksetzen, damit nach neuladen DLM und MaintainDB wieder laufen
                                                        self.bfaDLM_timer.maintained[profile] = False
                                                    finally:
                                                        BitfarmServer.reconnect_lock.release()
                                                    self.reconnect_workerthreads(profile)

                                                    if self.gridlib:
                                                        self.connect_gridfield_dll()
                                                    else:
                                                        if self.load_gridfield_dll():
                                                            self.connect_gridfield_dll()

                                                    self.basemodel[profile] = None
                                                    self.custommodel[profile] = None

                                                    if self.bfaModelLoader is not None:
                                                        self.bfaModelLoader.queue.put('loadall')

                                        if error:
                                            xml = bitfarm_xml.create_xml_request(reqtype="respError", data= error)
                                        else:
                                            xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("Konfiguration neu geladen"))

                        try:
                            _job.sock.send(xml)
                        except:
                            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                        else:
                            self.processed += 1

                        if not error:
                            hist = bitfarm_doc.BitfarmHistory()
                            hist.userid = _job.userid
                            hist.gdocid = 0
                            hist.taskid = 135
                            self.bfaDocProc.write_history(history=hist, profile=_job.profile, logid=self.name)

                    elif _job.reqtype == 'reqEdit' and _job.userid in self.user[_job.profile]:
                        if _job.gdocid in self.in_progress[_job.profile] and _job.userid != self.in_progress[_job.profile][_job.gdocid]:    # wenn es schon in bearbeitung ist, aber nicht von mir
                            xml = bitfarm_xml.create_xml_request(reqtype="respEditDenied", userid=self.in_progress[_job.profile][_job.gdocid])
                            self.log.debug("%s %s" % (self.name, xml))
                            try:
                                _job.sock.send(xml)
                            except:
                                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                            else:
                                self.processed += 1
                        else:
                             bfa_classes.BitfarmWorkerThread.queue.put(_job)

                    elif _job.reqtype == 'reqSave' and _job.userid in self.user[_job.profile]:
                        if _job.gdocid in self.in_progress[_job.profile]:
                            if _job.userid != self.in_progress[_job.profile][_job.gdocid]:     # man kann nur speichern, was man selbst in edit gesetzt hat
                                xml = bitfarm_xml.create_xml_request(reqtype="respSaveDenied", data=self.in_progress[_job.profile][_job.gdocid])
                                self.log.debug("%s %s" % (self.name, xml))
                                try:
                                    _job.sock.send(xml)
                                except:
                                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                                else:
                                        self.processed += 1
                            else:
                                bfa_classes.BitfarmWorkerThread.queue.put(_job)
                        else:
                            xml = bitfarm_xml.create_xml_request(reqtype="respSaveError", data=_("Dokument war nicht in Bearbeitung"), datatype="text")
                            self.log.debug("%s %s" % (self.name, xml))
                            try:
                                _job.sock.send(xml)
                            except:
                                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                            else:
                                self.processed += 1
                    elif _job.reqtype == 'reqCancel' and _job.userid in self.user[_job.profile]:
                        if _job.gdocid in self.in_progress[_job.profile]:
                            if _job.userid != self.in_progress[_job.profile][_job.gdocid]:     # man kann nur abbrechen, was man selbst in edit gesetzt hat
                                xml = bitfarm_xml.create_xml_request(reqtype="respCancelDenied", data=self.in_progress[_job.profile][_job.gdocid])
                                self.log.debug("%s %s" % (self.name, xml))
                                try:
                                    _job.sock.send(xml)
                                except:
                                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                                else:
                                        self.processed += 1
                            else:
                                bfa_classes.BitfarmWorkerThread.queue.put(_job)
                        else:
                            xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("reqCancel ausgeführt"))
                            self.log.debug("%s %s" % (self.name, xml))
                            try:
                                _job.sock.send(xml)
                            except:
                                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                            else:
                                self.processed += 1
                    elif _job.reqtype == 'reqShareLayer' and _job.userid in self.user[_job.profile]:
                        if _job.gdocid in self.in_progress[_job.profile]:
                            if _job.userid != self.in_progress[_job.profile][_job.gdocid]:     # man kann nur speichern, was man selbst in edit gesetzt hat
                                xml = bitfarm_xml.create_xml_request(reqtype="respShareLayerDenied", data=self.in_progress[_job.profile][_job.gdocid])
                                self.log.debug("%s %s" %(self.name, xml))
                                try:
                                    _job.sock.send(xml)
                                except:
                                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                                else:
                                    self.processed += 1
                            else:
                                bfa_classes.BitfarmWorkerThread.queue.put(_job)
                        else:
                            xml = bitfarm_xml.create_xml_request(reqtype="respError", data= _("Dokument war nicht in Bearbeitung"), datatype="text")
                            self.log.debug("%s %s" %(self.name, xml))
                            try:
                                _job.sock.send(xml)
                            except:
                                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                            else:
                                self.processed += 1
                    # wenn wir ein reqProcessing oder reqWorkflow rein bekommen, dann wird das Objekt im Data-Node weitergegeben
                    elif _job.reqtype == 'reqProcessing' or _job.reqtype == 'reqWorkflow':
                        if type(_job.data) in [bitfarm_doc.BitfarmDocument, bitfarm_doc.BitfarmWorkflow]:
                            bfa_classes.BitfarmWorkerThread.queue.put(_job.data)

                    elif _job.reqtype in ["reqCalculateExpiredDate", "reqCheckUpdates", "reqSendUpdates"]:  # anonyme Funktionen, die keiner Useranmeldung bedürfen
                        self.log.debug('%s Übergebe %s an WorkerThreads <%s>' % (self.name, _job.reqtype, hex(id(_job.sock))))
                        bfa_classes.BitfarmWorkerThread.queue.put(_job)
                    else:   #alles was nicht im Mainloop selbst erledigt wird, geht an die WorkerThreads
                        self.log.debug('%s Übergebe Job an WorkerThreads <%s>' % (self.name, hex(id(_job.sock))))
                        bfa_classes.BitfarmWorkerThread.queue.put(_job)
                except:
                    self.log.exception(_("Konnte Job nicht zum WorkerThread geben"))

            # überprüfen der WorkerThreads auf Timeout
            for thread in self.bfaWorkers:
                if (time.time() > thread.timeout > 0) and not self.stopsignal:  # neustart bei timeout, aber nicht wenn wir runterfahren sollen
                    self.log.warning(_("%s WorkerThread in Timeout gelaufen") % thread.name)
                    if thread._job is not None: # wir versuchen den job zurückzubekommen
                        _result = thread._job
                        thread._job = None
                        self.log.warning(_("%s Job %s vom stehenden WorkerThread entgegen genommen %s") % (thread.name, _result))
                    else:
                        # es kann sein, dass der job doch noch in die resultqueue gegeben wurde, oder aber es ist wirklich ein Fehler mit dem Job aufgetreten
                        self.log.warning(_("%s WorkerThread hatte keinen Job") % thread.name)
                    thread.shutdown()
                    thread._Thread__stop()
                    self.bfaWorkers.remove(thread)
                    new_thread = bfa_classes.BitfarmWorkerThread(self)
                    self.bfaWorkers.append(new_thread)
                    new_thread.start()
                    break   # wenn ein thread in ein timeout gelaufen ist und wir dessen job zurückholen
                            # dann müssen wir diesen direkt weiterverarbeiten (bei 2 threads im Timeout wird sonst _result des ersten überschrieben)

            if _result is None:  # sollten wir nach der überprüfung der threads auf timeout keinen _result entgegen genommen haben
                try:             # dann hole einen fertigen job aus der resultqueue
                    _result = self.resultqueue.get(False)
                except queue.Empty:
        #            self.log.debug(u"Nichts vom MP übergeben, dann weiter")
                    # da wir nicht blockierend arbeiten, machen wir bei leerer Queue etwas Pause
                    time.sleep(0.01)
                except:
                    self.log.exception(_('Fehler bei get_result'))

            if _result is not None:
                self.log.debug(_("%s Job verarbeitet <%s>") % (self.name, hex(id(_result.sock))))
                self.processed += 1
                self.log.debug('%s processed %i received %i' % (self.name, self.processed, self.received))
                if (self.processed == self.received) and True in [thread._job is None for thread in self.bfaWorkers]:
                    self.log.debug("%s Es wurden soviel Jobs verarbeitet wie empfangen (%i), setze Zähler zurück" % (self.name, self.processed))
                    self.received = 0       # damit wir nach 3Jahren Dauerbetrieb kein Integer Overflow haben :-)
                    self.processed = 0

             # wenn wir ein stopsignal haben und alles verarbeitet ist
            if self.stopsignal:
                if (self.processed >= self.received or time.time() > self.stoptimeout > 0):
                    if time.time() > self.stoptimeout > 0:
                        self.log.warning(_("Es wurden nicht alle Jobs verarbeitet - Timeout"))
                    else:
                        self.log.info(_("Stopsignal erhalten und alles verarbeitet"))
                    break   # hier wird das stopsignal verarbeitet, das unterbricht den mainloop des threads

        # SHUTDOWN
        if self.received != self.processed:
            self.log.warning(_("Achtung, Anzahl der empfangenen(%i) und verarbeiteten(%i) Jobs differiert") % (self.received, self.processed))
        else:
            self.log.info(_("Es wurden alle empfangenen Jobs verarbeitet %i") % self.received)

        # self.shutdown()

        self.running = False
        self.stopping = False
        self.stopped = True
        self.log.info('BitfarmServer stopped')

    def shutdown(self):
        if self.stopping:    # sind wir im shutdown, dann muss shutdown nicht nochmal laufen
            return
        # wenn der Server gestoppt werden soll beenden wir erstmal den empfang von jobs
        self.log.info(_("bfaServer36 Shutdown"))
        self.stopping = True
        try:
            if self.bfaDLM_timer is not None:
                self.bfaDLM_timer.shutdown()
                self.bfaDLM_timer.join(2)
                self.log.info(_("DLM Thread beendet"))

            if self.bfaQueue_timer is not None:
                self.bfaQueue_timer.shutdown()
                self.bfaQueue_timer.join(1)
                self.log.info(_("QueueTimer Thread beendet"))

            if self.bfaModelLoader is not None:
                self.bfaModelLoader.shutdown()
                self.bfaModelLoader.join(1)
                self.log.info(_("ModelLoader Thread beendet"))

            if self.bfaLogin:
                self.bfaLogin.shutdown()
                self.bfaLogin.join(1)
                self.log.info(_("LoginThread beendet"))

            for bfaSocket in self.bfaSockets:
                bfaSocket.shutdown()
                bfaSocket.join(1)
                self.log.info(_("SocketConnector beendet"))
            self.log.info(_("SocketConnectoren beendet"))

            for profile in self.authuser:
                for sockets in list(self.authuser[profile].values()):
                    for sock in sockets:
                        try:
                            sock.close()  # sockets schliessen
                        except:
                            self.log.exception("Fehler beim Socket schliessen während des Shutdowns.")
            self.log.info(_("Sockets geschlossen"))

            for bfaWorker in self.bfaWorkers:
                bfaWorker.shutdown()
                bfaWorker.join(2)
                self.log.info("bfaWorker beendet")
            self.log.info("Workerthreads beendet")

            if self.received - self.processed > 0:
                timeout = 5
            else:
                timeout = 1
            self.stoptimeout = time.time() + timeout
            self.log.info(_("Stopsignal erhalten, Timeout = %s sec.") % timeout)

            self.log.debug("close_gridfield_dll")
            try:
                self.close_gridfield_dll()
            except:
                self.log.exception("Fehler beim Entaden der bfaGridfield64.dll")

        except:
            self.log.exception('Fehler beim Shutdown')
        finally:
            self.log.debug("Alle Threads gestoppt")
            self.stopsignal = True  # Server Mainloop verlassen
            self.stopping = False

class bfaservice(win32serviceutil.ServiceFramework):

    _svc_name_ = "bfaServer36"
    _svc_display_name_ = "bitfarm Server36"
    _svc_description_ = "bitfarm Server für die Version 36"
    #_svc_deps_ = ["EventLog"]

    def __init__(self, args):
        win32serviceutil.ServiceFramework.__init__(self, args)
        self.hWaitStop = win32event.CreateEvent(None, 0, 0, None)           
        #servicemanager.LogInfoMsg("INIT!")
        self.timeout = 1000     #1 second
        self.bfasvr = None


    def SvcStop(self):
        # import servicemanager
        # servicemanager.LogInfoMsg("STOP!")
        self.ReportServiceStatus(win32service.SERVICE_STOP_PENDING)
        win32event.SetEvent(self.hWaitStop)
        #self.bfasrv.stopsignal = True
         
    def SvcDoRun(self):
        import servicemanager
        servicemanager.LogInfoMsg("bfaServer36 Service starting...")
        # servicemanager.LogMsg(servicemanager.EVENTLOG_INFORMATION_TYPE,servicemanager.PYS_SERVICE_STARTED,(self._svc_name_, ''))
        win32evtlogutil.ReportEvent(self._svc_name_,servicemanager.PYS_SERVICE_STARTED,0, servicemanager.EVENTLOG_INFORMATION_TYPE,(self._svc_name_, ''))

        self.ReportServiceStatus(win32service.SERVICE_START_PENDING)


        # This is how long the service will wait to run / refresh itself (see script below)

        self.bfasvr = BitfarmServer(service=servicemanager)
        self.bfasvr.start()

        while self.bfasvr.starting and not self.bfasvr.running and not self.bfasvr.timeout:
            time.sleep(.5)
            self.ReportServiceStatus(win32service.SERVICE_START_PENDING)

        self.ReportServiceStatus(win32service.SERVICE_RUNNING)
        servicemanager.LogInfoMsg("Running")

        while self.bfasvr.running:
            # Wait for service stop signal, if I timeout, loop again
            rc = win32event.WaitForSingleObject(self.hWaitStop, self.timeout)
            # Check to see if self.hWaitStop happened
            if rc == win32event.WAIT_OBJECT_0:
                # Stop signal encountered
                break

        servicemanager.LogInfoMsg("bfaServer - stopping")  #For Event Log
        # log.info("bfaServer - stopping")
        
        if self.bfasvr.is_alive() or not self.bfasvr.stopped:
            self.bfasvr.shutdown()
            self.ReportServiceStatus(win32service.SERVICE_STOP_PENDING)
            time.sleep(.5)

        while self.bfasvr.stopping:
            if self.bfasvr.stopped:
                break
            self.ReportServiceStatus(win32service.SERVICE_STOP_PENDING)
            time.sleep(.5)

            self.bfasvr.join(2)

        servicemanager.LogInfoMsg("bfaServer - stopped")
        self.ReportServiceStatus(win32service.SERVICE_STOPPED)

        win32evtlogutil.ReportEvent(self._svc_name_,servicemanager.PYS_SERVICE_STOPPED,0, servicemanager.EVENTLOG_INFORMATION_TYPE,(self._svc_name_, ''))

        
def kbfunc():
    x = msvcrt.kbhit()
    if x:
        ret = ord(msvcrt.getch())
        #  log.debug('keyboard %i' %ret)
    else:
        ret = 0
    return ret

    
def ctrlHandler(ctrlType):
    return True


if __name__ == '__main__':
    
    # multiprocessing.freeze_support()
    # ctypes.windll.kernel32.SetDllDirectoryW(None)
    # das Log und die Pfade definieren wir hier global, damit es sowohl in der Dienstklasse, als auch standalone in den anderen Klassen 
    # zur Verfügung steht
    # servicemanager.Initialize()

    # win32api.SetConsoleCtrlHandler(ctrlHandler, True)

    # servicemanager.LogInfoMsg(_(u'Konfiguration gelesen.'))
    # log.info(_(u'Konfiguration gelesen.'))
    # with open(r'D:\Development\bitfarm-archiv\bin\beep.log', 'w') as f:
    #     f.write('Konfiguration gelesen.')

    if len(sys.argv) == 1:
        # servicemanager.LogInfoMsg(_(u'Starte Dienst...'))
        # servicemanager.PrepareToHostSingle(bfaservice)
        # servicemanager.StartServiceCtrlDispatcher()
        try:
            evtsrc_dll = os.path.abspath(servicemanager.__file__)
            servicemanager.PrepareToHostSingle(bfaservice)
            servicemanager.Initialize('bfaServer36', evtsrc_dll)
            servicemanager.StartServiceCtrlDispatcher()
        except win32service.error as details:
            if details[0] == winerror.ERROR_FAILED_SERVICE_CONTROLLER_CONNECT:
                win32serviceutil.usage()
    else:
        if sys.argv[1] in ['install', 'stop', 'start', 'remove']:
            servicemanager.LogInfoMsg(_('Parameter für Dienststeuerung: %s') % sys.argv[1])
            win32serviceutil.HandleCommandLine(bfaservice)

        else:
            log = logging.getLogger("Main")

            if sys.argv[1] == 'console':
                app = 'console'
            else:
                app = 'standalone'

            if len(sys.argv) >= 3:
                if sys.argv[2][:6] == '-lang:':
                    loc = sys.argv[2][6:]
                    trans.change_language(loc)

            # log.info(_(u'bfapath= %s mypath=  %s cwd= %s') % (bfapath, mypath, os.getcwd()))
            bfaServer = BitfarmServer(app=app)
            bfaServer.start()
            
            while bfaServer.starting and not bfaServer.running:
                time.sleep(0.1)

            if bfaServer.running:
                if app == 'console':
                    log.info(_("(Geben Sie 'help' ein für eine Liste der Befehle)"))
                    bfaServer.ch.setLevel(logging.ERROR)
                else:
                    bfaServer.ch.setLevel(logging.DEBUG)
                    log.info(_("Drücken Sie STRG + ALT + Q um den Server zu stoppen"))

            while bfaServer.running:
#                if ch.level == 10:   # debug
                if app == 'standalone' or (app == "console" and bfaServer.ch.level != logging.ERROR):
                    #log.debug("bfaServer - still running")  
                
                    time.sleep(.5)
                    
                    x = kbfunc()
                    if x == 27:  # ESC
                        bfaServer.ch.setLevel(levels['error'])
                    if x == 64:  # Ctrl-Alt-q
                        bfaServer.ch.setLevel(levels['info'])
                        break
                else:
                    cmd = input('bfaServer>>').lower()
                    if not bfaServer.running:
                        print(_("bitfarm Server wurde beendet"))
                        break
                    if cmd == 'log':
                        log.info(_("Sie können die Debug-Ausgabe mit ESC beenden"))
                        bfaServer.ch.setLevel(levels['debug'])
                    elif cmd == 'quit':
                        bfaServer.ch.setLevel(levels['info'])
                        break
                    elif cmd == 'info':
                        print((bfaServer.get_info()))
                    elif cmd == 'user':
                        print((bfaServer.get_user()))
                    elif cmd == 'status':
                        bfaServer.get_status()
                    elif cmd[:9] == 'loglevel=':
                        if cmd[9:] in levels:
                            for handler in log.handlers:
                                handler.setLevel(levels[cmd[9:]])
                    elif cmd == 'help':
                        print(_("Mögliche Befehle:"))
                        print(_("log: schaltet in den Debug-Modus, ESC zum verlassen drücken oder\n     STRG + ALT + Q zum beenden"))
                        print(_("loglevel=x: Schaltet das Loglevel aller Logger auf den angebenen Wert"))
                        print(_("            x = {debug,info,warning,error,critical"))
                        print(_("info: zeigt Status-Informationen des laufenden Servers"))
                        print(_("user: zeigt die angemeldeten User an"))
                        print(_("quit: beendet den Server"))
                    else:
                        print(_("unbekannter Befehl"))

            log.info("bfaServer - stopping %s" % app)

            if not bfaServer.is_alive() or not bfaServer.stopped:
                bfaServer.shutdown()
                bfaServer.join(3)
            log.info("bfaServer - stopped %s" % app)
            sys.exit(0)
