#!/usr/bin/env python
# -*- coding: utf-8*-
#
#       bfaServerClient.py
#
#
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

#from bfaHttpAuth import require
from pprint import pprint
import base64
import socket
import ssl
import os
import sys
import time
import io
import inspect
import codecs
import tempfile
import datetime
import uuid
import subprocess
import threading
import logging
import unicodedata
import string
from collections import OrderedDict

from xml.dom import minidom

import bitfarm_doc
import bitfarm_baseobj
import bitfarm_xml

__version__ = "3.6.2.4"

VALID_FILTERS = ["color", "highres"]

def input_wstrip(msg):
    return input(msg).strip()


def _set_logger(logger):
    global serverlogger
    serverlogger = logger
    serverlogger.debug("logger switched")


validFilenameChars = "-_.() %s%s" % (string.ascii_letters, string.digits)


def removeDisallowedFilenameChars(filename):
    cleanedFilename = unicodedata.normalize(
        'NFKD', filename)  # .encode('ASCII', 'ignore')
    return ''.join(c for c in cleanedFilename if c in validFilenameChars)


def timeoutprocess(args, timeout=120):

    def kill_proc(proc, is_timeout):
        is_timeout["value"] = True
        proc.kill()

    proc = subprocess.Popen(
        args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

    is_timeout = {"value": False}
    stdout = ""
    stderr = ""
    timer = threading.Timer(timeout, kill_proc, [proc, is_timeout])
    try:
        timer.start()
        stdout, stderr = proc.communicate()
    finally:
        timer.cancel()
    return proc.returncode, stdout, stderr, is_timeout["value"]


serverlogger = logging.getLogger("serverlogger")


class CustomCaster():
    def todate(self, arg):
        try:
            return datetime.datetime.strptime(arg, "%Y-%m-%d")
        except:
            raise Exception("error casting datevalue. Expected format '%Y-%m-%d'")

    def attachment_list(self, arg):
        if type(arg) != list:
            raise Exception("error casting attachment_list. wrong datatype, expected list")
        for sub in arg:
            if type(sub) is tuple:
                if len(sub) == 2:
                    if type(sub[0]) is str and type(sub[1]) is str:
                        return arg
        raise Exception("error casting attachment_list.")

    def check_bookmark_str(self, arg):
        if arg in (None, ""):
            return None
        if type(arg) not in (str, str):
            raise Exception("error casting bookmark-string. wrong datatype")
        else:
            try:
                values = arg.split(",")
                int_vals = [int(val) for val in values]
            except Exception as e:
                raise Exception("error casting bookmark-string. wrong content. '%s'" % (str(e)))
            return arg

    def check_searchdatetype(self, arg):
        if arg not in (4, 8, 1024):
            raise Exception("error searchdatetyp, possible values: 4, 8, 1024")
        return arg

    def wvlparams(self, arg):
        """"
        wvlparams": {
                "info": "bla", -> doc.info
                "users": "dmsadmin;#Gruppe1;§Gruppe2", -> doc.editor
                "wvldate": "2021-07-30 00:00:00", -> doc.target_date
                "alarmdate": "2021-08-04 00:00:00", -> doc.alarm_date
                "options": 0 -> doc.timeroptions (0,1,2,4,8,64)
            }
        """
        required_keys = ["info", "users",
                         "wvldate", "alarmdate", "options"]
        if arg == {}:
            return None
        for key in list(arg.keys()):
            if key not in required_keys:
                del arg[key]
        for key in required_keys:
            if key not in arg:
                raise Exception("wvlparams casting error, missing key '%s'" % key)
            if key in ("info", "comment", "users"):
                arg[key] = self.unknownencodingtounicode(arg[key])
            elif key in ("wvldate", "alarmdate"):
                arg[key] = self.strtodatetime(arg[key], "%Y-%m-%d %H:%M:%S")
            elif key == "options":
                if type(arg[key]) is not int:
                    raise Exception(
                        "wvlparam '%s' wrong type, need int" % key)

        return arg

    def todate_str(self, arg):
        try:
            dateobj = datetime.datetime.strptime(arg, "%Y-%m-%d")
            return arg
        except:
            raise Exception(
                "error casting datevalue. Expected format '%Y-%m-%d'")

    def verify_intstr(self, arg):
        try:
            return str(int(arg))
        except:
            raise Exception("casting error verify_intstr, integer required")

    def unknownencodingtounicode_or_none(self, arg):
        if arg is None:
            return None
        else:
            return self.unknownencodingtounicode(arg)

    def pipe(self, value):
        return value

    def unknownencodingtounicode(self, arg):
        # print "1", arg, type(arg)
        if type(arg) is str:
            return arg

        for enc in ["latin1", "cp1252", "uft-8"]:
            try:
                arg = arg.decode(enc)
                break
            except:
                pass
        arg = str(arg)
        # print "2", arg, type(arg)
        return arg

    def stringto_linebreakstring(self, arg):
        if type(arg) in (str, str):
            arg = arg.replace(";", r"\n")
        return arg

    def stringtounicodelist(self, arg):
        if arg is None:
            return []
        if type(arg) in (str, str):
            arg = arg.split(";")
        return arg

    def to_int_or_none(self, arg):
        if arg is None:
            return None
        else:
            return int(arg)

    def stringtointlist(self, arg):
        return arg

    def intlist_to_str(self, arg):
        return ";".join([str(x) for x in arg])

    def strtodatetime(self, datestring, formatstring):
        if datestring == "":
            return None
        return datetime.datetime.strptime(datestring, formatstring)

    def strtodatetime_with_current_time(self, datestring):
        formatstr = "%Y-%m-%d"
        try:
            dt = datetime.datetime.strptime(datestring, formatstr)
            current_time_str = datetime.datetime.now().strftime("%H:%M:%S")
            dt = datetime.datetime.strptime(
                datestring + " " + current_time_str, "%Y-%m-%d %H:%M:%S")
            return dt
        except:
            raise Exception(
                "error casting datevalue. Expected format '%Y-%m-%d'")

    def strtodirection(self, arg):
        if type(arg) is str:
            return str(arg)
        elif type(arg) is str:
            return arg
        else:
            raise Exception("error casting direction")

    def strtostatfields(self, arg):
        return arg

    def cast_addfield_list_search(self, arg):
        if type(arg) is not list:
            raise Exception("casting error, addfields as list expected")
        new_addfield_list = []
        required_keys = ["id", "indexid", "type", "value", "mode"]

        for add_dict in arg:

            for key in required_keys:
                if key not in add_dict:
                    raise Exception("missing addfield key %s" % key)

            newadd = bitfarm_doc.BitfarmAddfield()
            if add_dict["type"] not in list(range(13)):
                raise Exception("add-property 'type' expects integer (0-13)")
            newadd.type = add_dict["type"]
            if "negated" in add_dict:
                if type(add_dict["negated"]) is not bool:
                    raise Exception("add-property 'negated' expects boolean")
                newadd.negated = add_dict["negated"]
            if type(add_dict["mode"]) is not int:
                raise Exception("add-property 'mode' expects integer")
            newadd.mode = add_dict["mode"]
            if type(add_dict["indexid"]) is not int:
                raise Exception("add-property 'indexid' expects integer")
            newadd.indextableid = add_dict["indexid"]
            if type(add_dict["id"]) is not int:
                raise Exception("add-property 'id' expects integer")
            newadd.add = "add_"+str(add_dict["id"])
            newadd.id = add_dict["id"]

            # value check/casting
            # (date, dlm-date, integer, floating point number, currency)
            if add_dict["type"] in (1, 12, 6, 7, 8):
                if "valuehigh" not in add_dict:
                    raise Exception("missing addfield key valuehigh")
                if add_dict["type"] in (1, 12):
                    strdate_from = add_dict["value"]
                    strdate_to = add_dict["valuehigh"]
                    # possible values:
                    # empty string, date-string, null/None
                    # one datevalue None: search docs with empty date
                    if strdate_from == "":
                        newadd.value = ""
                    if strdate_to == "":
                        newadd.valuehigh = ""
                    if strdate_from is None or strdate_to is None:
                        newadd.value = "NULL"
                    else:
                        if type(strdate_from) in (str, str) and strdate_from != "":
                            try:
                                newadd.value = datetime.datetime.strptime(strdate_from, "%Y-%m-%d")
                            except:
                                raise Exception(
                                    "error casting addfield date value. %Y-%m-%d expected")
                        if type(strdate_to) in (str, str) and strdate_to != "":
                            try:
                                newadd.valuehigh = datetime.datetime.strptime(strdate_to, "%Y-%m-%d")
                            except:
                                raise Exception("error casting addfield date value. %Y-%m-%d expected")
                else:
                    newadd.value = add_dict["value"]
                    newadd.valuehigh = add_dict["valuehigh"]
            else:
                newadd.value = add_dict["value"]
                #newadd.valuehigh = add_dict["valuehigh"]
            if newadd.value is None:
                newadd.value = "null"
            newadd.modified = True
            newadd.modified_search = True
            new_addfield_list.append(newadd)
        return new_addfield_list

    def strtoaddfields(self, arg):
        return arg

    def strtobool(self, arg):
        if type(arg) is bool:
            return arg
        elif type(arg) in (str, str):
            if arg.lower() in ['true', '1', 't', 'y', 'yes', 'ja', 'j']:
                return True
            else:
                return False
        else:
            raise Exception
    
    def check_filter(self, arg):
        if type(arg) is not str:
            raise Exception
        else:
            arg = arg.lower()
            filters = [x for x in arg.split(";") if x!=""]
            filterargs = []
            for filter in filters:
                if filter not in VALID_FILTERS:
                    raise Exception
                elif filter == "color":
                    filterargs.append("Color")
                elif filter == "highres":
                    filterargs.append("HighRes")
            return ";".join(filterargs)

    def stringlist_to_string(self, arg):
        if type(arg) is list:
            arg = [str(x) for x in arg]
            return ";".join(arg)
        return arg

    def to_list_of_ints(self, arg):
        if type(arg) in (str, str):
            return arg
        elif type(arg) is list:
            return ",".join([str(x) for x in arg])

    def to_statefielddict(self, arg):
        statefielddict = OrderedDict()
        # statefielddict = {}
        for sfdict in arg:
            for key, required_type in zip(("bitmask", "id", "indexid", "value", "name"), ((int, int), (int), (int), (bool), (str, str))):
                if key == "value":
                    if type(sfdict[key]) in (str, str):
                        if sfdict[key].lower() in ("", "", "false", "false"):
                            sfdict[key] = False
                        elif sfdict[key].lower() in ("true", "true"):
                            sfdict[key] = True

                if key not in sfdict:
                    raise TypeError(
                        "%s is missing in statefield-dictionay" % key)
                if type(required_type) is type:
                    if type(sfdict[key]) is not required_type:
                        raise TypeError("wrong type for key %s" % key)
                elif type(required_type) is tuple:
                    if type(sfdict[key]) not in required_type:
                        raise TypeError("wrong type for key %s" % key)
        for sfdict in arg:
            statefield = bitfarm_doc.BitfarmStatefield()
            statefield.indexcolumnid = sfdict["indexid"]
            statefield.id = sfdict["id"]
            val = self.strtobool(sfdict["value"])
            statefield.value = val
            statefield.bitmask = sfdict["bitmask"]
            statefield.name = sfdict["name"]
            statefielddict[sfdict["name"]] = statefield
        return statefielddict

    def to_addfielddict(self, arg):
        addfield_dict = {}
        """
        "indexid":1,
        "id": 3,
        "value": "test",
        "add": "add_27",
        """
        for adddict in arg:
            for key, required_type in zip(("indexid", "id", "add", "name"), ((int), (int), (str, str), (str, str))):
                if key not in adddict:
                    raise TypeError("%s is missing in addfield-dictionay" % key)
                if type(required_type) is type:
                    if type(adddict[key]) is not required_type:
                        raise TypeError("wrong type for key %s" % key)
                elif type(required_type) is tuple:
                    if type(adddict[key]) not in required_type:
                        raise TypeError("wrong type for key %s" % key)
        for adddict in arg:
            addfield = bitfarm_doc.BitfarmAddfield()
            addfield.indextableid = adddict["indexid"]
            addfield.id = adddict["id"]
            addfield.value = adddict["value"]
            addfield.add = "add_"+str(adddict["id"])
            addfield.name = adddict["name"]
            addfield_dict[adddict["id"]] = addfield
        return addfield_dict

    def b64to_stream(self, arg):
        if type(arg) is str:
            arg = bytes(arg, "ascii")
        if type(arg) is bytes:
            # print arg[:100]
            stream = io.BytesIO()
            stream.write(base64.b64decode(arg))
            stream.seek(0, os.SEEK_END)
            return stream
        else:
            raise TypeError("expect string or unicode, got %s" % (type(arg)))

    def to_stream(self, arg):
        if type(arg) in (str, str):
            stream = io.StringIO()
            stream.write(arg)
            stream.seek(0, os.SEEK_END)
            return stream
        else:
            raise TypeError("expect string or unicode, got %s" % (type(arg)))


class ResponseConsoleHandler():
    def _printxml(self, webclient, response):
        parsed = minidom.parseString(webclient.xmlresponse)
        print(parsed.toprettyxml())

    def _savexml(self, webclient, response):
        parsed = minidom.parseString(webclient.xmlresponse)
        nicexml = parsed.toprettyxml()
        if response.reqtype == "respSearch":
            default = "search.xml"
        else:
            default = "response.xml"
        filepath = input_wstrip("Dateiname: (default %s): " % default).strip()
        if filepath == "":
            filepath = default
        f = codecs.open(filepath, "wb", encoding="utf8")
        f.write(nicexml)
        f.close()

    def _openxml(self, webclient, response):
        parsed = minidom.parseString(webclient.xmlresponse)
        nicexml = parsed.toprettyxml()
        handle,  temppath = tempfile.mkstemp(suffix=".xml")
        os.close(handle)
        f = codecs.open(temppath, "wb", encoding="utf8")
        f.write(nicexml)
        f.close()
        os.system(temppath)
        os.remove(temppath)

    def _savebin(self, webclient, response):
        filepath = None
        if webclient.responseobject.filename:
            if webclient.responseobject.filename != "":
                filepath = input_wstrip("Dateiname (default %s): " % webclient.responseobject.filename)
        if filepath == "":
            filepath = webclient.responseobject.filename
        while filepath in (None, ""):
            filepath = input_wstrip("Dateiname: ")
        f = open(filepath, "wb")
        f.write(webclient.streamdata.read())
        f.close()

    def _openbin(self, webclient, response):
        filepath = None
        ext = None
        if webclient.responseobject.filename:
            if webclient.responseobject.filename != "":
                filepath = webclient.responseobject.filename
                ext = os.path.splitext(filepath)[1]
        if ext:
            handle,  temppath = tempfile.mkstemp(suffix=ext)
            os.close(handle)
            f = open(temppath, "wb")
            webclient.streamdata.seek(0)
            f.write(webclient.streamdata.read())
            webclient.streamdata.seek(0)
            f.close()
            os.system(temppath)
            os.remove(temppath)

    def _handle_respGroups(self, webclient, response):
        return self.handle_respDoc(webclient, response)

    def _handle_respUsers(self, webclient, response):
        return self.handle_respDoc(webclient, response)

    def handle_respAuthError(self, webclient, response):
        webclient.loggedin = False
        webclient.userid = None
        if webclient.sock is not None:
            webclient.sock.close()
            webclient.sock = None
        # print "auth error!"
        return [False, webclient.xmlresponse]

    def _handle_respDocError(self, webclient, response):
        # print "response DocError!"
        return [False, response.errortext if response.errortext else response.data]

    def _handle_respIsLatest(self, webclient, response):
        # print "is latest!"
        return [True, response.data]

    def handle_respAuth(self, webclient, response):
        webclient.userid = response.userid
        webclient.loggedin = True
        # print "you are logged in with userid %d" %webclient.userid
        # print response.data
        return [True, response.data]

    def _handle_respImg(self, webclient, response):
        menulines = {}
        menulines[0] = "back"
        menulines[1] = "show response"
        menulines[2] = "show xml"
        menulines[3] = "save xml"
        menulines[4] = "open xml in standard app"
        if webclient.streamdata:
            menulines[5] = "save image"
            menulines[6] = "open image in standard app"
        action = None
        menutxt = ""
        for key in sorted(menulines.keys()):
            menutxt += "%d. %s\n" % (key, menulines[key])
        menutxt += "your choice: "
        while action != 0:
            action = input_wstrip(menutxt)
            try:
                action = int(action)
            except:
                continue
            if action == 1:
                print(response)
            elif action == 2:
                parsed = minidom.parseString(webclient.xmlresponse)
                print(parsed.toprettyxml())
            elif action == 3:
                parsed = minidom.parseString(webclient.xmlresponse)
                nicexml = parsed.toprettyxml()
                default = "%d.xml" % webclient.responseobject.gdocid
                filepath = input_wstrip("Dateiname: (default %s): " % default)
                if filepath == "":
                    filepath = default
                f = open(filepath, "wb")
                f.write(nicexml)
                f.close()
            elif action == 4:
                parsed = minidom.parseString(webclient.xmlresponse)
                nicexml = parsed.toprettyxml()
                handle,  temppath = tempfile.mkstemp(suffix=".xml")
                os.close(handle)
                f = open(temppath, "wb")
                f.write(nicexml)
                f.close()
                os.system(temppath)
                os.remove(temppath)
            elif action == 5:
                filepath = None
                if webclient.responseobject.filename:
                    if webclient.responseobject.filename != "":
                        filepath = input_wstrip(
                            "Dateiname (default %s): " % webclient.responseobject.filename)
                if filepath == "":
                    filepath = webclient.responseobject.filename
                while filepath in (None, ""):
                    filepath = input_wstrip("Dateiname: ")
                f = open(filepath, "wb")
                f.write(webclient.streamdata.read())
                f.close()
            elif action == 6:
                filepath = None
                ext = None
                if webclient.responseobject.filename:
                    if webclient.responseobject.filename != "":
                        filepath = webclient.responseobject.filename
                        ext = os.path.splitext(filepath)[1]
                if ext:
                    handle,  temppath = tempfile.mkstemp(suffix=ext)
                    os.close(handle)
                    f = open(temppath, "wb")
                    webclient.streamdata.seek(0)
                    f.write(webclient.streamdata.read())
                    webclient.streamdata.seek(0)
                    f.close()
                    os.system(temppath)
                    os.remove(temppath)

    def _handle_respDoc(self, webclient, response):
        menulines = {}
        menulines[0] = "back"
        menulines[1] = "show response"
        menulines[2] = "show xml"
        menulines[3] = "save xml"
        menulines[4] = "open xml in standard app"
        if webclient.streamdata:
            menulines[5] = "save binary data"
        action = None
        menutxt = ""
        for key in sorted(menulines.keys()):
            menutxt += "%d. %s\n" % (key, menulines[key])
        menutxt += "your choice: "
        while action != 0:
            action = input_wstrip(menutxt)
            try:
                action = int(action)
            except:
                continue
            if action == 1:
                print(response)
            elif action == 2:
                parsed = minidom.parseString(webclient.xmlresponse)
                print(parsed.toprettyxml())
            elif action == 3:
                parsed = minidom.parseString(webclient.xmlresponse)
                nicexml = parsed.toprettyxml()
                default = "%d.xml" % webclient.responseobject.gdocid
                filepath = input_wstrip("Dateiname: (default %s): " % default)
                if filepath == "":
                    filepath = default
                f = open(filepath, "wb")
                f.write(nicexml)
                f.close()
            elif action == 4:
                parsed = minidom.parseString(webclient.xmlresponse)
                nicexml = parsed.toprettyxml()
                handle,  temppath = tempfile.mkstemp(suffix=".xml")
                os.close(handle)
                f = open(temppath, "wb")
                f.write(nicexml)
                f.close()
                os.system(temppath)
                os.remove(temppath)
            elif action == 5:
                filepath = None
                if webclient.responseobject.filename:
                    if webclient.responseobject.filename != "":
                        filepath = input_wstrip(
                            "Dateiname (default %s): " % webclient.responseobject.filename)
                if filepath == "":
                    filepath = webclient.responseobject.filename
                while filepath in (None, ""):
                    filepath = input_wstrip("Dateiname: ")
                f = open(filepath, "wb")
                f.write(webclient.streamdata.read())
                f.close()

        return True

    def handle_respSendFile(self, webclient, response):
        return True

    def _handle_respArchives(self, webclient, response):
        menulines = {}
        menulines[0] = "back"
        menulines[1] = "show response"
        menulines[2] = "show xml"
        menulines[3] = "save xml"
        menulines[4] = "open xml in standard app"
        if webclient.streamdata:
            menulines[5] = "save binary data"
        action = None
        menutxt = ""
        for key in sorted(menulines.keys()):
            menutxt += "%d. %s\n" % (key, menulines[key])
        menutxt += "your choice: "
        while action != 0:
            action = input_wstrip(menutxt)
            try:
                action = int(action)
            except:
                continue
            if action == 1:
                print(response)
            elif action == 2:
                parsed = minidom.parseString(webclient.xmlresponse)
                print(parsed.toprettyxml())
            elif action == 3:
                parsed = minidom.parseString(webclient.xmlresponse)
                nicexml = parsed.toprettyxml()
                default = "%d.xml" % webclient.responseobject.gdocid
                filepath = input_wstrip("Dateiname: (default %s): " % default)
                if filepath == "":
                    filepath = default
                f = open(filepath, "wb")
                f.write(nicexml)
                f.close()
            elif action == 4:
                parsed = minidom.parseString(webclient.xmlresponse)
                nicexml = parsed.toprettyxml()
                handle,  temppath = tempfile.mkstemp(suffix=".xml")
                os.close(handle)
                f = open(temppath, "wb")
                f.write(nicexml)
                f.close()
                os.system(temppath)
                os.remove(temppath)
            elif action == 5:
                default = "archives.xml"
                filepath = input_wstrip("Dateiname (default %s): " % default)
                if filepath == "":
                    filepath = default
                parsed = minidom.parseString(webclient.xmlresponse)
                nicexml = parsed.toprettyxml()
                f = open(filepath, "wb")
                f.write(nicexml)
                f.close()

    def handle_respOri(self, webclient, response):
        if webclient.streamdata:
            print("Bilddatei empfangen")
            webclient.streamdata.seek(0)
            save = input_wstrip("Bilddatei speichern (j/n)?: ")
            if save.lower() == "j":
                filepath = input_wstrip(
                    "Dateiname (default %s): " % webclient.responseobject.filename)
                if filepath == "":
                    filepath = webclient.responseobject.filename
                f = open(filepath, "wb")
                f.write(webclient.streamdata.read())
                f.close()

    def handle_default(self, webclient, response):
        # default handler, used if theres no special handler found
        # print "no special handler found, using default handler"
        # print "response type: %s" %response.reqtype
        menulines = {}
        index = 0
        menulines[index] = "back"
        index += 1
        responsefunctions = {}
        responsefunctions["show xml"] = self._printxml
        responsefunctions["save xml"] = self._savexml
        responsefunctions["open xml in standard app"] = self._openxml
        responsefunctions["save binary data"] = self._savebin
        responsefunctions["open binary data"] = self._openbin

        try:
            xml = webclient.xmlresponse
            if xml != "":
                menulines[index] = "show xml"
                index += 1
                menulines[index] = "save xml"
                index += 1
                menulines[index] = "open xml in standard app"
                index += 1
        except:
            pass

        try:
            stream = webclient.streamdata
            if stream is not None:
                stream.seek(0)
                menulines[index] = "save binary data"
                index += 1
                menulines[index] = "open binary data"
                index += 1
        except:
            pass

        menutxt = ""
        for key in sorted(menulines.keys()):
            menutxt += "%d. %s\n" % (key, menulines[key])
        menutxt += "your choice: "
        action = None

        while True:
            func = None
            action = input_wstrip(menutxt)
            if action == "0":
                break
            try:
                action = int(action)
                func = responsefunctions[menulines[action]]
            except:
                print("invalid choice")
                action = None
                func = None
            if func is not None:
                func(webclient, response)


class BfaServerClient():
    def __init__(self, profile=None, auto_heartbeat=None):
        self.auto_heartbeat = auto_heartbeat
        self.sock = None
        self.loggedin = False
        self.con = None
        self.__uid = None
        self.__sid = None
        self.userpass = None
        self.profile = profile
        self.args = {}
        if self.profile is not None:
            self.load_profile(profile)
        self.caster = CustomCaster()
        self._set_args()
        self.responsehandler = None
        self.lastsentxml = None
        self.lastrecvxml = None
        self.tempsock = None
        self.uuid = uuid.uuid1()
        serverlogger.info("client instance created with uuid %s" % self.uuid)
        serverlogger.info("client profile %s" % self.profile)
        

    def run_auto_heartbeat(self):
        while self.loggedin is True:
            time.sleep(self.auto_heartbeat)
            if self.loggedin is True:
                xml = bitfarm_xml.create_xml_request(
                        reqtype="reqHeartBeat", profile=self.profile, userid=self.userid)
                self._sendxml(xml)

        
    def _init_auto_heartbeat(self):
        t = threading.Thread(target=self.run_auto_heartbeat)
        t.start()


    def _get_userid(self):
        return self.__uid

    def _set_userid(self, args):
        if self.__uid is None:
            try:
                self.__uid = args
            except:     # wenn was beim setzen der USerid schief geht, sagen wir nicht was. Der Server wird richtig anliefern, alles andere können manuelle Versuche sein
                pass

    def _del_userid(self):
        self.__uid = None
        self.__sid = None   # keinen User keine Session, so ist das

    def _get_sessionid(self):
        return self.__sid

    def _set_sessionid(self, args):
        if self.__sid is None:
            try:
                self.__sid = args
            except:     # wenn was beim setzen der Sessionid schief geht, sagen wir nicht was. Der Server wird richtig anliefern, alles andere können manuelle Versuche sein
                pass

    def _del_sessionid(self):
        self.__sid = None

    userid = property(_get_userid, _set_userid, _del_userid)
    sessionid = property(_get_sessionid, _set_sessionid, _del_sessionid)

    def _set_args(self):
        self.args["logout"] = {}
        self.args["login"] = {"userid": int, "username": str, "userpass": str}
        self.args["getDoc"] = {"gdocid": int, "asjson": self.caster.strtobool, "tim_id": int, "fields_arcid": int,
                               "include_text": self.caster.strtobool, "include_annostream": self.caster.strtobool}
        self.args["getTxt"] = {"gdocid": int,
                               "asjson": self.caster.strtobool, "tim_id": int}
        self.args["getBookmarks"] = {"global_bm": self.caster.strtobool, "local_bm": self.caster.strtobool,
                                     "history_bm": self.caster.strtobool, "asjson": self.caster.strtobool}
        self.args["getExecuteBookmarks"] = {
            "bmtype": int, "userid": int, "groupid": int, "indexid": int, "asjson": self.caster.strtobool, "guid": str, "usercontext": self.caster.to_int_or_none}
        self.args["getFtxPos"] = {
            "gdocid": int, "word": self.caster.unknownencodingtounicode, "tim_id": int}
        self.args["getHistory"] = {
            "gdocid": int, "asjson": self.caster.strtobool, "lang": str, "tim_id": int}
        self.args["getImg"] = {"gdocid": int, "imgtype": str, "page": int,
                               "ievlayer": int, "forcerendering": self.caster.strtobool, "tim_id": int}
        self.args["getOri"] = {"gdocid": int, "revision": int, "tim_id": int}
        self.args["getLastModified"] = {"gdocid": int, "revision": str}
        self.args["getDocCommentary"] = {"gdocid": int}
        self.args["getArchive"] = {"gdocid": int}
        self.args["getAccountingData"] = {"gdocid": int}
        self.args["getPreview"] = {
            "gdocid": int, "ievlayer": int, "tim_id": int}
        self.args["getFile"] = {"gdocid": int, "tim_id": int}
        self.args["getIEV"] = {"gdocid": int, "ievlayer": int}
        self.args["getDocOcr"] = {"gdocid": int, "top_left_x": int, "top_left_y": int,
                                  "bottom_right_x": int, "bottom_right_y": int, "page": int}
        self.args["getImgOcr"] = {"b64img": str, "filetype": str}
        self.args["getReadGroups"] = {"asjson": self.caster.strtobool}
        self.args["getReadArchives"] = {"rootid": int, "mask": int, "asjson": self.caster.strtobool,
                                        "no_addfields": self.caster.strtobool, "no_fielditems": self.caster.strtobool}
        self.args["getReadUsers"] = {"asjson": self.caster.strtobool}
        self.args["getReadAddfields"] = {
            "asjson": self.caster.strtobool, "no_fielditems": self.caster.strtobool, "all_fields": self.caster.strtobool}
        self.args["getSave"] = {"gdocid": int, "addfields": list,
                                "statefields": list, "links": list, "slw": list, "tim_id": int}
        self.args["getWvl"] = {"selected_cols": str,
                               "asjson": self.caster.strtobool}
        self.args["getSearch"] = {
            "selected_cols": str,
            "name": str,
            "id": int,
            "date": (self.caster.strtodatetime, "%Y-%m-%d"),
            # Typ der  Suche, 0 = normale Suche, 1 = lokales Lesezeichen, 2 = globales Lesezeichen,  3 = Verlaufs-Lesezeichen, 4= WVL, 5 = Zusatzfeldsuche (Verschlagwortungshilfe)
            "searchtype": int,
            "userid": int,
            "groupid": int,
            "urights": self.caster.stringtounicodelist,
            "grights": self.caster.stringtounicodelist,
            "sqlwhere": str,
            "sqlwhere_operator": str,
            "sqlwhere_only": self.caster.strtobool,
            "nonconstructive": self.caster.strtobool,
            "maxvalues": int,
            "refsearch": str,
            "refsearch_negated": self.caster.strtobool,
            "searchuser": str,
            "searchuserid": int,
            "searchuser_negated": self.caster.strtobool,
            "searchgroupid": int,
            "searchgroup": str,
            "searchgroup_negated": self.caster.strtobool,
            "arcid": self.caster.to_list_of_ints,
            "gdocid": self.caster.to_list_of_ints,
            "timref": int,
            "checkout": self.caster.strtobool,
            "state": int,
            "notstate": self.caster.strtobool,
            "searchdate": (self.caster.strtodatetime, "%Y-%m-%d"),
            "searchdatehigh": (self.caster.strtodatetime, "%Y-%m-%d"),
            "searchdate_negated": self.caster.strtobool,
            "ftx_fields": int,
            "searchstring": self.caster.unknownencodingtounicode,
            "searchstring_negated": self.caster.strtobool,
            "ftx_matchagainst": self.caster.strtobool,
            "ftx_naturalmode": self.caster.strtobool,
            "ftx_booleanmode": self.caster.strtobool,
            "ftx_withqueryexpansion": self.caster.strtobool,
            "count": self.caster.strtobool,
            "resulttype": int,
            "direction": self.caster.strtodirection,
            "addfields": self.caster.cast_addfield_list_search,
            "statefields": self.caster.strtostatfields,
            "user_context": int,
            "resubdate": self.caster.todate,
            "resubnotstate": int,
            "resubdate_high": (self.caster.strtodatetime, "%Y-%m-%d"),
            "resubalarm_high": (self.caster.strtodatetime, "%Y-%m-%d"),
            "resubalarm": (self.caster.strtodatetime, "%Y-%m-%d"),
            "resubuserid": int,
            "resubuserid_negated": self.caster.strtobool,
            "resubcontrollerid": int,
            "resubcontrollerid_negated": self.caster.strtobool,
            "resubstate": int,
            "resubdate_negated": self.caster.strtobool,
            "resubstate_negated": self.caster.strtobool,
            "resubalarm_negated": self.caster.strtobool,
            "resubuseridlist": self.caster.pipe,
            "docname": str,
            "order": str,
            "docname_negated": self.caster.strtobool,
            "filetype": str,
            "filetype_negated": self.caster.strtobool,
            "grouplist": self.caster.stringtointlist,
            "usemodifieddate": bool,
            "selectedarcid": int,
            "asjson": self.caster.strtobool,
            "debug": self.caster.strtobool,
            "rightsmask": int,
            "bookmark": self.caster.check_bookmark_str,
            "searchdatetype": self.caster.check_searchdatetype,
            "groupby": str,
            "selectedcols_inresub": self.caster.strtobool
        }

        self.args["getCancel"] = {"gdocid": int}
        self.args["getEdit"] = {"gdocid": int, "tim_id": int}
        self.args["send_mail"] = {"to":str, "html":str, 'text':str, "attachments":self.caster.attachment_list, "subject":str}
        self.args["getSaveTitle"] = {"gdocid":str, "title":str}
        self.args["getSaveResub"] = {
            "gdocid": int,
            "action": str,
            "tim_id": int,
            "options": int,
            "users": self.caster.intlist_to_str,
            "groups": self.caster.intlist_to_str,
            "info": str,
            "comment": str,
            "wvl_date": ((self.caster.strtodatetime, "%Y-%m-%d %H:%M")),
            "alarm_date": ((self.caster.strtodatetime, "%Y-%m-%d %H:%M")),
            "status": int,
            "asjson": self.caster.strtobool}
        self.args["getSaveHoliday"] = {"replid": self.caster.verify_intstr,
                                       "startdate": self.caster.todate_str, "enddate": self.caster.todate_str}
        self.args["getModifyHoliday"] = {"replid": self.caster.verify_intstr, "startdate": self.caster.todate_str,
                                         "enddate": self.caster.todate_str, "holid": self.caster.verify_intstr}
        self.args["getDeleteHoliday"] = {"holid": self.caster.verify_intstr}
        self.args["getHoliday"] = {"asjson": self.caster.strtobool}
        self.args["getMoveDoc"] = {"gdocid": int,
                                   "src_arcid": int, "dest_arcid": int}
        self.args["getRemoveLink"] = {
            "gdocid": int, "linkstring": self.caster.unknownencodingtounicode}
        self.args["getClassify"] = {
            "gdocid": int, "text": self.caster.unknownencodingtounicode_or_none}
        self.args["getAddClassDoc"] = {"gdocid": int, "text": self.caster.unknownencodingtounicode_or_none,
                                       "classname": self.caster.unknownencodingtounicode, "calc": self.caster.strtobool}
        self.args["getClassDocs"] = {}
        self.args["getCreateNamingRule"] = {
            "gdocid": int, "text": self.caster.unknownencodingtounicode_or_none, "classname": self.caster.unknownencodingtounicode_or_none}
        self.args["getAutoname"] = {
            "gdocid": int, "text": self.caster.unknownencodingtounicode_or_none}
        self.args["getNewDoc"] = {"arcid": int, "archive": self.caster.unknownencodingtounicode_or_none, "title": self.caster.unknownencodingtounicode_or_none, "filename": self.caster.unknownencodingtounicode_or_none,
                                  "addfields": self.caster.to_addfielddict,
                                  "statefields": self.caster.to_statefielddict,
                                  "docstream": self.caster.b64to_stream,
                                  "lang": str,
                                  "doctags": self.caster.stringlist_to_string,
                                  "links": list,
                                  "astif": self.caster.strtobool,
                                  "wvlparams": self.caster.wvlparams,
                                  "ocr": self.caster.strtobool,
                                  "filter": self.caster.check_filter}

    def _sendrequest(self, req):
        serverlogger.info("%s sending %s to server" % (self.uuid, req.reqtype))
        req.sessionid = self.sessionid
        xml = bitfarm_xml.object_to_xml(req, shortxml=False)
        if '<sessionid' in xml:
            xmllog = xml[:xml.find('<sessionid')] + xml[xml.find('/sessionid>') + 11:]
        else:
            xmllog = xml
        try:
            parsed = minidom.parseString(xmllog)
            serverlogger.debug("%s - sending xml:\n%s" % (self.uuid, parsed.toprettyxml()))
        except:
            pass
        # serverlogger.debug("sending xml: %s" % xml)
        if req.reqtype in ("reqMoveDoc", "reqAuthenticate", "reqReadArchives", "reqStopSocketLog", "reqStartSocketLog", "reqSetLogLevel", "reqMoveDoc"):
            sock = self.sock

        elif req.reqtype in ("reqArchive", "reqDoc", "reqPreview", "reqOri", "reqImg"):
            # sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            # sock.connect((self.con.bfaserver, self.con.bfaserverport))

            if self.con.nossl:
                sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                sock.connect((self.con.bfaserver, self.con.bfaserverport))
            else:
                context = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
                context.check_hostname = False
                context.verify_mode = ssl.CERT_NONE
                sock_nossl = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                sock = context.wrap_socket(sock_nossl, server_hostname=socket.gethostname())
                sock.connect((self.con.bfaserver, self.con.bfaserverport))
            self.tempsock = sock
        else:
            sock = self.sock
        sock.sendall(xml.encode())

        resp = self._getresponse(sock)
        if resp:
            return self._handleresponse(resp)
        else:
            return False

    def _senddata(self, stream, socket=None):
        if socket is None:
            sock = self.sock
        else:
            sock = socket
        try:
            stream.seek(0)
            sock.sendall(stream.read())
            return True
        except:
            return False

    def _sendxml(self, xml):
        if type(xml) is str:
            xml = xml.encode()
        if b'<sessionid' in xml:
            xmllog = xml[:xml.find(b'<sessionid')] + xml[xml.find(b'/sessionid>') + 11:]
        else:
            xmllog = xml
        parsed = minidom.parseString(xmllog)
        req = bitfarm_xml.xml_to_object(xml.decode())
        # if req is None:
        #     print(parsed.toprettyxml())
        #     print()
        serverlogger.info("%s - sending %s to server" %
                          (self.uuid, req.reqtype))

        if req.reqtype == "reqAuthenticate":
            profile = req.profile
            user = req.parameter
            serverlogger.debug("%s - login user '%s' - profile '%s'" % (self.uuid, user, profile))
            pass
        else:
            serverlogger.debug("%s - sending xml: %s" % (self.uuid, parsed.toprettyxml()))

        if req.profile is None or req.sessionid is None:
            if req.profile is None:
                req.profile = self.profile
            if req.sessionid is None:
                req.sessionid = self.sessionid
        xml = bitfarm_xml.object_to_xml(req, shortxml=False).encode()

        if req.reqtype in ("reqDoc", "reqPreview", "reqOri", "reqImg", "reqTif", "reqSavePreview", "reqHeartBeat"):
            # sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            # sock.connect((self.con.bfaserver, self.con.bfaserverport))

            if self.con.nossl:
                sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                sock.connect((self.con.bfaserver, self.con.bfaserverport))
            else:
                context = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
                context.check_hostname = False
                context.verify_mode = ssl.CERT_NONE
                sock_nossl = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                sock = context.wrap_socket(sock_nossl, server_hostname=socket.gethostname())
                sock.connect((self.con.bfaserver, self.con.bfaserverport))
        else:
            sock = self.sock
        serverlogger.debug("%s - self.sock %s" % (self.uuid, self.sock))
        serverlogger.debug("%s - sending xml with socket %s" % (self.uuid, self.sock))
        sock.sendall(xml)
        # reqHeartBeat has no response
        if req.reqtype in ["reqHeartBeat"]:
            time.sleep(.5)
            sock.close()
            return None
        resp = self._getresponse(sock)

        if resp:
            try:
                self.lastrecvxml = self.xmlresponse
            except:
                self.lastrecvxml = "no xml"
            return self._handleresponse(resp)
        else:
            self.lastrecvxml = "no xml"
            return None

    def _getresponse(self, sock):
        self.streamdata = None
        self.responseobject = None
        start = time.time()
        nachricht = ''
        rest = None
        serverlogger.info("%s - receiving response" % self.uuid)
        try:
            while nachricht[-9:] != '</bfaReq>':
                # buff = sock.recv(1024*1024).decode()
                buff = sock.recv(8192)
                try:
                    buff = buff.decode()
                except:
                    pass
                # print buff
                if buff == '':
                    break
                if "</bfaReq>" in buff:
                    pos = buff.find("</bfaReq>")+9
                    rest = buff[pos:]
                    nachricht += buff[:pos]
                    break

                nachricht += buff
                # if "</bfaReq>" in nachricht:
                #     rest = nachricht[nachricht.find("</bfaReq>")+9:]
                #     nachricht = nachricht[:nachricht.find("</bfaReq>")+9]
                #     break
        except socket.error as e:
            serverlogger.error("%s - exception reading socket1: %s" % (self.uuid, str(e)))
            raise e
            nachricht = ''
        except Exception as e:
            serverlogger.error("%s - exception reading socket2: %s" % (self.uuid, str(e)))
            raise e
            nachricht = ''

        if nachricht.strip() == "":
            raise socket.error("socket connection broken, empty string received")

        serverlogger.info("%s - response complete" % self.uuid)
        if "</bfaReq>" in nachricht:
            stream = None

            if serverlogger.getEffectiveLevel() == logging.DEBUG:
                parsed = minidom.parseString(nachricht)
                serverlogger.debug("%s - xml response:\n%s" % (self.uuid, parsed.toprettyxml()))

            self.xmlresponse = nachricht

            # print "got xmlresponse"
            try:
                resp = bitfarm_xml.xml_to_object(nachricht)
            except Exception as e:
                serverlogger.error("%s - exception in bitfarm_xml.xml_to_object: %s" % (self.uuid, str(e)))
                serverlogger.error("%s - response parser failed, no responseobject. server-message: %s" % (self.uuid, nachricht))
                return None
            self.responseobject = resp
            serverlogger.info("%s - response type: %s" % (self.uuid, resp.reqtype))
            if resp.reqtype in ("reqPreview", 'respImg', 'respTif', "respDoc") and resp.datalen:
                serverlogger.debug("%s - received imagestream length %d" % (self.uuid, resp.datalen))
                datalen = resp.datalen
                if datalen:
                    stream = io.BytesIO()
                    if rest is not None:
                        datalen = datalen-len(rest)
                        try:
                            stream.write(rest.encode("utf8"))
                        except:
                            pass

                    recv = 0
                    while recv < datalen:
                        buff = sock.recv(4096)
                        try:
                            stream.write(buff)
                        except:
                            pass
                        recv += len(buff)
                        #sys.stdout.write("received: %i" %recv)
                        # sys.stdout.write("\r")
                        # sys.stdout.flush()
                    # print
                    # print "Dauer %f" %(time.time() - start)
                    self.streamdata = stream
                return resp
            else:
                #serverlogger.debug("response (raw): %s" %nachricht)
                return resp

        else:
            serverlogger.error("%s - received incomplete xml:%s" % (self.uuid, nachricht))
            self.lastrecvxml = "no xml"
            pass
            # print "unvollstaendiger Response!"
        return None

    # @profile

    def _handleresponse(self, response):
        if response is None:
            return [False, 404, "response-Fehler"]
        if response.reqtype == 'respAuth':
            self.userid = response.userid
            if response.sessionid:
                self.sessionid = self.con._bitdecrypt(response.sessionid)
            self.loggedin = True
            if isinstance(self.auto_heartbeat, int):
                if self.con.auto_reconnect:
                    self._init_auto_heartbeat()
        if response.reqtype == 'respAuthError':
            del self.userid
            del self.sessionid
            self.loggedin = False

        if self.responsehandler is not None:
            handlers = list(filter((lambda x: x[0].startswith("handle_")), [
                (name, method) for name, method in inspect.getmembers(self.responsehandler, inspect.ismethod)]))
            handlerfunctions = {key: value for (key, value) in handlers}
            if "handle_"+response.reqtype in handlerfunctions:
                serverlogger.info("%s - calling handler %s" % (self.uuid, "handle_"+response.reqtype))
                result = handlerfunctions["handle_" + response.reqtype](self, response)
                return result
            else:
                serverlogger.info("%s - calling handler handle_default" % (self.uuid))
                result = self.responsehandler.handle_default(self, response)
                return result
        else:
            # print "unhandled response", response
            return True
        """
        if response.reqtype=="respAuth":
            print self, "successfully logged in"
            self.userid = response.userid
            self.loggedin = True
            return [True, response.data]
        elif response.reqtype=="respAuthError":
            print self, "login error:\n", response.data
            self.loggedin = False
            return [False, response.data]
        elif response.reqtype=="respDoc":
            print self, "document loaded"
            return [True, response.data]
        elif response.reqtype=="respImg":
            print self, "image loaded"
            return [True, response.data]
        elif response.reqtype=="respArchives":
            print "Archives loaded"
            return [True, response.data]
            
        elif response.reqtype=="respDocError":
            if not response.data:
                response.data="unbekannter Fehler respDocError"
            elif response.data==u"":
                response.data="unbekannter Fehler respDocError"
            return [False, response.data]
        """

        return None

    def load_profile(self, profile):
        """This function loads a Bitfarm profile.

        :param profile: the profiles name (basename of the con-file).
        :type profile: str.
        :returns:  True on success, None if loading failes

        """
        if self.loggedin is True:
            self.logout()
            self.con = None
            self.sock = None
        try:
            self.con = bitfarm_baseobj.BitfarmBaseCon(profile)
            # print self.con
            # print profile
            self.profile = self.con.profile
        except Exception as e:
            raise e
            self.con = None
            self.profile = None
            return None
        
        return True

    def test_socket(self):
        serverlogger.debug("testing socket for userid %d" % self.userid)
        if self.sock:
            self.sock.settimeout(0)
            try:
                data = self.sock.recv(1)
                # self.sock.settimeout(None)
                serverlogger.debug("socket test failed, logout")
                self.logout()
                return False
            except:
                serverlogger.debug("socket test ok")
                self.sock.settimeout(None)
                return True
        return False

    def connect(self, userid=None, username=None, userpass=None):
        serverlogger.debug("bfaserverClient connect called")
        if not self.con:
            serverlogger.error("bfaserverClient connect unknown Profile")
            return [False, 200, "unknown Profile", 600, "unknown Profile"]
        if self.loggedin is True:
            serverlogger.error("bfaserverClient connect already connected")
            return [False, 200, "already connected", 601, "already connected"]
        if self.sock is None:
            # print self.con.bfaserver, self.con.bfaserverport
            try:
                if self.con.nossl:
                    self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                    self.sock.connect((self.con.bfaserver, self.con.bfaserverport))
                else:
                    context = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
                    context.check_hostname = False
                    context.verify_mode = ssl.CERT_NONE
                    sock_nossl = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                    self.sock = context.wrap_socket(sock_nossl, server_hostname=socket.gethostname())
                    self.sock.connect((self.con.bfaserver, self.con.bfaserverport))
            except Exception as e:
                serverlogger.error(
                    "Keine Verbindung zum bfaServer, Fehler %s" % str(e))
                self.sock = None
                return [False, 503, "no connection to backend", 503, "no connection to backend"]
        serverlogger.debug("bfaserverClient connect socket created")
        req = bitfarm_doc.bfaReq()
        if userid is not None:

            self.userid = int(userid)
            serverlogger.info("%s - connected with userid %d" % (self.uuid, self.userid))
            req.userid = int(userid)
        else:
            if username is not None:
                serverlogger.info("%s - connected with username %s" % (self.uuid, username))
                req.parameter = username

        req.reqtype = "reqAuthenticate"
        #req.parameter = username
        if userpass is not None:
            req.data = self.con._bitencrypt(userpass)
        else:
            req.data = ""
        req.datatype = "text"
        req.profile = self.profile
        return self._sendrequest(req)

    def logout(self):
        # TODO: bfaserver logout request
        # print "Beende Verbindung"
        serverlogger.info("logging out userid %d" % (self.userid))
        if self.sock is not None:
            # self.sock.send("quitClient")
            req = bitfarm_doc.bfaReq()
            req.userid = self.userid
            req.reqtype = 'reqDisconnectUser'
            req.profile = self.profile
            xml = bitfarm_xml.object_to_xml(req, shortxml=True)
            try:
                self.sock.sendall(xml)
            except:
                serverlogger.debug("could not send reqDisconnectUser")

            try:
                self.sock.close()
            except:
                serverlogger.debug("dead socket could not be closed")
        self.sock = None
        #self.con = None
        self.userid = None
        del self.sessionid
        self.loggedin = False

    def getClassnames(self):
        if self.loggedin is True:
            xml = bitfarm_xml.create_xml_request(reqtype="reqGetClasses", profile=self.profile, userid=self.userid)
            return self._sendxml(xml)
        else:
            return [False, "authentication error, not logged in"]

    def getAddClassDoc(self, gdocid, text, classname, calc=False):
        if self.loggedin is True:
            xml = bitfarm_xml.create_xml_request(reqtype="reqAddClassDoc", profile=self.profile, userid=self.userid, gdocid=gdocid, data=[text, classname, calc])
            return self._sendxml(xml)
        else:
            return [False, "authentication error, not logged in"]

    def send_mail(self, to, text=None, html=None, subject=None, attachments=[]):
        if self.loggedin is True:
            params = {}
            params["to"] = to
            params["text"] = text
            params["html"] = html
            params["subject"] = subject
            xml = bitfarm_xml.create_xml_request(reqtype="reqSMTPMail", profile=self.profile, userid=self.userid,
                                                 parameter=params, data=attachments)
            # print(params)
            self.lastsentxml = xml
            return self._sendxml(xml)
        else:
            return [False, "authentication error, not logged in"]

    def getCreateDocument(self, docmeta, docstreams):
        if self.loggedin is True:
            # [arcid, 'quellprofile', quell-gdocic, ('docloc',65535,'.tif','rs'),  ('extloc',12348,'.tif','nrs')]
            mainmeta = docmeta[:3]
            locs = docmeta[3:]
            xml = bitfarm_xml.create_xml_request(reqtype="reqCreateDocument", profile=self.profile, userid=self.userid, parameter=docmeta)
            oldsock = self.sock

            try:
                # self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                # self.sock.connect((self.con.bfaserver, self.con.bfaserverport))

                if self.con.nossl:
                    self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                    self.sock.connect((self.con.bfaserver, self.con.bfaserverport))
                else:
                    context = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
                    context.check_hostname = False
                    context.verify_mode = ssl.CERT_NONE
                    sock_nossl = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                    self.sock = context.wrap_socket(sock_nossl, server_hostname=socket.gethostname())
                    self.sock.connect((self.con.bfaserver, self.con.bfaserverport))
            except Exception as e:
                serverlogger.error("Keine Verbindung zum bfaServer, Fehler %s" % str(e))
                self.sock = None
            result = self._sendxml(xml)
            # print result
            doc = None
            if result is True:
                for loc, stream in zip(locs, docstreams):
                    # print loc, len(stream)
                    # print self.sock
                    self.sock.sendall(stream)
                    result = self._getresponse(self.sock)
                    if result.reqtype not in ("respSendFile", "respOK", "respDoc"):
                        self.sock = oldsock
                        return [False, result]
                self.sock = oldsock
                return [True, result]
            else:
                self.sock = oldsock
                return [False, result]

    def getClassDocs(self):
        if self.loggedin is True:
            xml = bitfarm_xml.create_xml_request(reqtype="reqGetClassDocs", profile=self.profile, userid=self.userid)
            return self._sendxml(xml)
        else:
            return [False, "authentication error, not logged in"]

    def getCreateNamingRule(self, gdocid, classname=None):
        if self.loggedin is True:
            xml = bitfarm_xml.create_xml_request(reqtype="reqAddNamingRule", profile=self.profile, userid=self.userid)
            return self._sendxml(xml)
        else:
            return [False, "authentication error, not logged in"]

    def getAutoname(self, gdocid=None, text=None):
        if self.loggedin is True:
            xml = bitfarm_xml.create_xml_request(reqtype="reqAutoname", profile=self.profile, userid=self.userid)
            return self._sendxml(xml)
        else:
            return [False, "authentication error, not logged in"]

    def getClassify(self, gdocid=None, text=None):
        if self.loggedin is True:
            xml = bitfarm_xml.create_xml_request(reqtype="reqClassify", profile=self.profile, userid=self.userid, gdocid=gdocid, data=text)
            return self._sendxml(xml)
        else:
            return [False, "authentication error, not logged in"]

    def getDocOcr(self, gdocid, page, top_left_x, top_left_y, bottom_right_x, bottom_right_y):
        # [{'tl': (203, 649), 'br': (522, 691)}],
        if self.loggedin is True:
            data = (page, (top_left_x, top_left_y),
                    (bottom_right_x, bottom_right_y))
            xml = bitfarm_xml.create_xml_request(reqtype="reqDocOcr", profile=self.profile, userid=self.userid, gdocid=gdocid, data=data)
            # print xml
            return self._sendxml(xml)
        else:
            return [False, "authentication error, not logged in"]

    def getImgOcr(self, b64img, filetype):
        # self._job.data -> (image - Stream(base64 - codiert), filetype(z.B.".tif"))
        data = (b64img, filetype)
        if self.loggedin is True:
            xml = bitfarm_xml.create_xml_request(reqtype="reqImgOcr", profile=self.profile, userid=self.userid, data=data)
            # print xml
            return self._sendxml(xml)
        else:
            return [False, "authentication error, not logged in"]

    def getReadArchives(self, rootid=None, mask=None, asjson=False, no_addfields=False, no_fielditems=False):
        if self.loggedin is True:
            right = None
            if mask is not None:
                right = bitfarm_doc.BitfarmRights()
                right.rights = int(mask)
            params = [rootid]
            if asjson:
                params.append('asjson')
            if no_addfields:
                params.append('no_addfields')
            if no_fielditems:
                params.append('no_fielditems')
            # xml = bitfarm_xml.create_xml_request(reqtype="reqReadArchives", profile=self.profile, userid=self.userid, data=right, parameter=params, shortxml=False)
            # return self._sendxml(xml)
            # xml = bitfarm_xml.create_xml_request(reqtype="reqReadArchives", profile=self.profile, userid=self.userid, data=right, parameter=params, shortxml=False)
            # req_cmp = bitfarm_xml.xml_to_object(xml.decode())

            req = bitfarm_xml.create_request(reqtype="reqReadArchives", profile=self.profile, userid=self.userid, data=right, parameter=params, shortxml=False)
            # print(req.parameter)
            # print(req_cmp.parameter)

            return self._sendrequest(req)
        else:
            return [False, "authentication error, not logged in"]

    def getReadArchives_(self, rootid=None, mask=None, asjson=False, no_addfields=False, no_fielditems=False):
        if self.loggedin is True:
            right = None
            if mask is not None:
                right = bitfarm_doc.BitfarmRights()
                right.rights = int(mask)
            params = [rootid]
            if asjson:
                params.append('asjson')
            if no_addfields:
                params.append('no_addfields')
            if no_fielditems:
                params.append('no_fielditems')
            xml = bitfarm_xml.create_xml_request(reqtype="reqReadArchives", profile=self.profile, userid=self.userid, data=right, parameter=params, shortxml=False)
            return self._sendxml(xml)
        else:
            return [False, "authentication error, not logged in"]

    def getLoadAccountingData(self, gdocid):
        if self.loggedin is True:
            xml = bitfarm_xml.create_xml_request(reqtype="reqLoadAccountingData", profile=self.profile, userid=self.userid, gdocid=gdocid, shortxml=False)
            return self._sendxml(xml)
        else:
            return [False, "authentication error, not logged in"]

    def getReadAddfields(self, asjson=False, no_fielditems=False, all_fields=False):
        if self.loggedin is True:
            params = []
            if asjson:
                params.append('asjson')
            if no_fielditems:
                params.append('no_fielditems')
            if all_fields:
                params.append('all_fields')

            # xml = bitfarm_xml.create_xml_request(
            #     reqtype="reqReadAddfields", profile=self.profile, userid=self.userid, parameter=params, shortxml=False)
            # return self._sendxml(xml)
            req = bitfarm_xml.create_request(reqtype="reqReadAddfields", profile=self.profile, userid=self.userid, parameter=params, shortxml=False)
            return self._sendrequest(req)

        else:
            return [False, "authentication error, not logged in"]

    def getFile(self, gdocid, tim_id=None):  # not implemented
        return [False, "not implemented"]

    def getIEV(self, gdocid, ievlayer):  # not implemented
        return [False, "not implemented"]

    def getReadGroups(self, asjson=False):
        if self.loggedin is True:
            if asjson is True:
                params = ["asjson"]
            else:
                params = []
            # xml = bitfarm_xml.create_xml_request(
            #     reqtype="reqReadGroups", profile=self.profile, userid=self.userid, parameter=params, shortxml=False)
            # return self._sendxml(xml)
            req = bitfarm_xml.create_request(reqtype="reqReadGroups", profile=self.profile, userid=self.userid, parameter=params, shortxml=False)
            return self._sendrequest(req)
        else:
            return [False, "authentication error, not logged in"]

    def getFtxPos(self, gdocid, word, tim_id=None):
        if self.loggedin is True:
            # print type(word)
            # print word.encode("latin1")
            params = str(word)
            xml = bitfarm_xml.create_xml_request(reqtype="reqFtxPos", tim_id=tim_id, profile=self.profile, gdocid=gdocid, userid=self.userid, data=params)
            # print xml
            return self._sendxml(xml)
        else:
            return [False, "authentication error, not logged in"]
        return [False, "not implemented"]

    def getReadUsers_(self, asjson=False):
        if self.loggedin is True:
            if asjson is True:
                params = ["asjson"]
            else:
                params = []
            xml = bitfarm_xml.create_xml_request(reqtype="reqReadUsers", profile=self.profile, userid=self.userid, parameter=params)
            return self._sendxml(xml)
        else:
            return [False, "authentication error, not logged in"]
        return [False, "not implemented"]

    def getReadUsers(self, asjson=False):
        if self.loggedin is True:
            if asjson is True:
                params = ["asjson"]
            else:
                params = []
            # xml = bitfarm_xml.create_xml_request(reqtype="reqReadUsers", profile=self.profile, userid = self.userid, parameter = params)
            req = bitfarm_xml.create_request(reqtype="reqReadUsers", profile=self.profile, userid=self.userid, parameter=params)
            # return self._sendxml(xml)
            return self._sendrequest(req)
        else:
            return [False, "authentication error, not logged in"]
        return [False, "not implemented"]

    def getWvl(self, selected_cols='gdocid,docdate,arcname,docname,tmr.date,tmr.alarm,userid_work,userid_shed,tmr.timid,tmr.timref', asjson=False):
        ###############################
        # Suche nach Wiedervorlagen
        fav = bitfarm_doc.BitfarmSearch()
        fav.searchtype = 4

        fav.user_context = self.userid
        fav.debug = True
        # selected_cols okay?
        # selected_cols=gdocid,docdate,arcname,docname,date,alarm,timid,timref,info,comment,tmr.options,status
        # https://vmth1/api/getWvl?selected_cols=gdocid,docdate,arcname,docname,date,alarm,timid,timref,info,comment,tmr.options,status&asjson=j
        knowncols = ['gdocid', 'docdate', 'arcname', 'arcid', 'docname', 'tmr.date', 'tmr.groupid', 'tmr.alarm',
                     'userid_work', 'userid_shed', 'tmr.timid', 'tmr.timref', 'info', 'comment', 'tmr.options', 'tmr.status']
        cols = [x.strip() for x in selected_cols.split(",")]
        for (i, col) in enumerate(cols):
            if col == 'date':
                cols[i] = 'tmr.date'
            if col == 'alarm':
                cols[i] = 'tmr.alarm'
        for col in cols:
            if col not in knowncols:
                # success, statuscode, result, errorcode, message
                return [False, 500, False, 616, "param selected_cols error %s" % col]
        fav.selected_cols = ",".join(cols)
        #fav.selected_cols = u'gdocid,docdate,arcname,docname,date,alarm,userid_work,userid_shed,timid,timref'
        #fav.selected_cols = u'gdocid,docdate,arcname'
        #fav.inresub = True
        #fav.resulttype = u''
        fav.resubnotstate = 4
        fav.resubstate_negated = True
        fav.resubuserid = self.userid

        if asjson is True:
            params = ["asjson"]
        else:
            params = []
        xml = bitfarm_xml.create_xml_request(
            reqtype="reqSearch", profile=self.profile, userid=self.userid, data=fav, parameter=params)
        return self._sendxml(xml)
        ###############################

    def getSearch(self, selected_cols='gdocid,docdate,arcname', order="docdate desc", bookmark=None,
                  name=None, id=None, date=None, searchtype=0, userid=None, groupid=None,
                  urights=None, grights=None, sqlwhere=None, sqlwhere_operator=None,
                  sqlwhere_only=None, nonconstructive=None, maxvalues=50, refsearch=None,
                  refsearch_negated=None, searchuser=None, searchuserid=None, searchuser_negated=None,
                  searchgroupid=None, searchgroup=None, searchgroup_negated=None, arcid=None,
                  gdocid=None, timref=None, checkout=None, state=None, notstate=None,
                  searchdate=None, searchdatehigh=None, searchdate_negated=None, ftx_fields=3,
                  searchstring=None, searchstring_negated=None, ftx_matchagainst=None, ftx_naturalmode=None,
                  ftx_booleanmode=None, ftx_withqueryexpansion=None, count=None, resulttype=None,
                  direction="desc", addfields=None, statefields=None, resubdate=None,
                  resubdate_high=None, resubalarm_high=None, resubalarm=None, resubuserid=None,
                  resubuserid_negated=None, resubcontrollerid=None, resubcontrollerid_negated=None,
                  resubstate=None, resubstate_negated=None, resubnotstate=None, resubdate_negated=None, resubalarm_negated=None,
                  resubuseridlist=None, docname=None, docname_negated=None, filetype=None, filetype_negated=None, grouplist=None,
                  usemodifieddate=None, selectedarcid=None, asjson=False, debug=False, rightsmask=None, user_context=None,
                  searchdatetype=4, groupby=None, selectedcols_inresub=False):
        # , user_context=None

        # searchtype
        #  0 = normale Suche, 1 = lokales Lesezeichen, 2 = globales Lesezeichen,  3 = Verlaufs-Lesezeichen, 4= WVL, 5 = Zusatzfeldsuche (Verschlagwortungshilfe)

        user_context = self.userid
        fav = bitfarm_doc.BitfarmSearch()
        fav.profile = self.profile
        if bookmark is not None:
            fav.bookmark = bookmark

        fav.debug = debug

        if gdocid is not None:
            fav.gdocid = gdocid
        for arg, value in list(locals().items()):
            if arg not in ("self", "fav", "addfields", "statefields", "asjson", "direction", "usemodifieddate"):
                if value is not None:
                    fav.__setattr__(arg, value)

        if asjson is True:
            params = ["asjson"]
        else:
            params = []

        if statefields is not None:
            fav.statefields = []
            for sf in statefields:
                newsf = bitfarm_doc.BitfarmStatefield()
                newsf.value = sf["value"]
                newsf.id = sf["id"]
                if 'indexid' in sf:
                    newsf.indexcolumnid = sf["indexid"]
                if 'bitmask' in sf:
                    newsf.bitmask = sf["bitmask"]
                newsf.modified_search = True
                fav.statefields.append(newsf)

        if addfields is not None:
            fav.addfields = addfields
        else:
            fav.addfields = []

        xml = bitfarm_xml.create_xml_request(
            reqtype="reqSearch", userid=self.userid, profile=self.profile, data=fav, parameter=params, shortxml=False)

        #parsed = minidom.parseString(xml)
        # print parsed.toprettyxml()
        return self._sendxml(xml)

    def getSearchTest(self):
        fav = bitfarm_doc.BitfarmSearch()
        fav.searchtype = 0
        #fav.resulttype = u''
        fav.arcid = 2
        fav.user_context = self.userid  # > eigene UserID
        # fav.ftx_fields = 3        # Bitmaske - Erste Bit Volltext, zweite Bit Schlagwörter
        #fav.searchstring = u"python"
        fav.order = 'docdate'
        #fav.direction = 'DESC'
        # fav.searchstring_negated = False -> negierte Suche
        fav.addfields = []
        fav.selected_cols = 'gdocid,docdate,arcname,docname,add_1:0,add_2:0,add_3:0,add_4:0'
        add1 = bitfarm_doc.BitfarmAddfield()
        add1.add = "add_1"
        add1.indextableid = 0
        # add1.fieldname=u"Techniker"
        add1.addfieldid = 1
        add1.value = "heranbrachte"
        fav.debug = True
        fav.addfields.append(add1)

        #fav.selected_cols = u'gdocid,docdate,arcname,docname'
        fav.count = False
        xml = bitfarm_xml.create_xml_request(
            reqtype="reqSearch", userid=self.userid, data=fav)
        parsed = minidom.parseString(xml)
        # print parsed.toprettyxml()
        return self._sendxml(xml)

    def getSendPreview(self):  # not implemented
        return [False, "not implemented"]

    def getSendIev(self):  # not implemented
        return [False, "not implemented"]

    def getIevCheck(self):  # not implemented
        return [False, "not implemented"]

    def getSendOri(self):  # not implemented
        return [False, "not implemented"]

    def getMoveDoc(self, gdocid, src_arcid, dest_arcid):
        if self.loggedin is True:
            parameter = ",".join(
                [str(gdocid), str(src_arcid), str(dest_arcid)])
            xml = bitfarm_xml.create_xml_request(
                reqtype="reqMoveDoc", profile=self.profile, userid=self.userid, parameter=parameter)
            return self._sendxml(xml)
        else:
            return [False, "authentication error, not logged in"]

    def getSetLogLevel(self):  # not implemented
        return [False, "not implemented"]

    def getStartSocketLog(self):  # not implemented
        return [False, "not implemented"]

    def getStopSocketLog(self):  # not implemented
        return [False, "not implemented"]

    def getSaveHoliday(self, replid, startdate, enddate):
        # start- and enddate as string "%Y-%m-%d"
        if self.loggedin is True:
            parameter = ";".join(
                [str(self.userid), replid, startdate, enddate])
            xml = bitfarm_xml.create_xml_request(
                reqtype="reqSaveHoliday", profile=self.profile, userid=self.userid, parameter=parameter)
            return self._sendxml(xml)
        else:
            return [False, "authentication error, not logged in"]

    def getHoliday(self, asjson=False):
        if self.loggedin is True:
            if asjson is True:
                parameter = "asjson"
            else:
                parameter = None
            xml = bitfarm_xml.create_xml_request(
                reqtype="reqGetHoliday", profile=self.profile, userid=self.userid, parameter=parameter)
            return self._sendxml(xml)
        else:
            return [False, "authentication error, not logged in"]

    def getModifyHoliday(self, holid, replid, startdate, enddate):
        # start- and enddate as string "%Y-%m-%d"
        if self.loggedin is True:
            parameter = ";".join(
                [holid, str(self.userid), replid, startdate, enddate])
            xml = bitfarm_xml.create_xml_request(
                reqtype="reqModifyHoliday", profile=self.profile, userid=self.userid, parameter=parameter)
            return self._sendxml(xml)
        else:
            return [False, "authentication error, not logged in"]

    def getDeleteHoliday(self, holid):
        if self.loggedin is True:
            xml = bitfarm_xml.create_xml_request(
                reqtype="reqDeleteHoliday", profile=self.profile, userid=self.userid, parameter=";".join([str(holid), str(self.userid)]))
            return self._sendxml(xml)
        else:
            return [False, "authentication error, not logged in"]

    def getEdit(self, gdocid, tim_id=None):
        if self.loggedin is True:
            xml = bitfarm_xml.create_xml_request(
                reqtype="reqEdit", tim_id=tim_id, profile=self.profile, userid=self.userid, gdocid=gdocid)
            return self._sendxml(xml)
        else:
            return [False, "authentication error, not logged in"]

    def getSaveDoc(self, doc, gdocid):
        kwargs = {"reqtype": "reqSave", "userid": self.userid,
                  "gdocid": gdocid, "data": doc, "profile": self.profile}
        xml = bitfarm_xml.create_xml_request(**kwargs)

        parsed = minidom.parseString(xml)
        # print parsed.toprettyxml()

        return self._sendxml(xml)

    def saveDoc(self, docobj):

        kwargs = {"reqtype": "reqSave", "userid": self.userid,
                  "gdocid": docobj.gdocid, "data": docobj, "profile": self.profile}
        if docobj.arcid is not None:
            kwargs["arcid"] = docobj.arcid
        if docobj.fields_arcid is not None:
            kwargs["fields_arcid"] = docobj.fields_arcid
        if docobj.symbolic_arcid is not None:
            kwargs["symbolic_arcid"] = docobj.symbolic_arcid
        xml = bitfarm_xml.create_xml_request(**kwargs)
        return self._sendxml(xml)
    
    def getSaveTitle(self, gdocid, title):
        if self.loggedin is True:
            kwargs = {"reqtype": "reqSaveDocName", "userid": self.userid,
                  "parameter": gdocid, "data": title, "profile": self.profile}
            xml = bitfarm_xml.create_xml_request(**kwargs)
            return self._sendxml(xml)
        else:
            # success, statuscode, result, errorcode, message
            return [False, 404, False, 404, "authentication error, not logged in"]

    def getSave(self, gdocid, addfields, statefields, tim_id=None):
        # history Object wird mit add- / statefield geliefert
        if self.loggedin is True:

            data = []
            for addfield in addfields:
                add = bitfarm_doc.BitfarmAddfield()
                indexid = addfield["indexid"]
                addid = addfield["id"]
                value = addfield["value"]

                try:
                    add.name = addfield["name"]
                except:
                    add.name = "unbekannt"
                add.add = "add_"+str(addid)
                add.id = addid
                add.indextableid = int(indexid)
                add.addfieldid = int(addid)
                add.value = value
                data.append(add)

            for sf in statefields:
                statefield = bitfarm_doc.BitfarmStatefield()
                try:
                    statefield.indexcolumnid = sf["indexid"]
                    statefield.id = sf["id"]
                    statefield.value = sf["value"]
                    statefield.bitmask = sf["bitmask"]
                    if "func" in sf:
                        statefield.func = sf["func"]
                    try:
                        statefield.name = sf["name"]
                    except:
                        statefield.name = "unbekannt"
                except Exception as e:
                    serverlogger.error("error on getSave: %s" % str(e))
                    return [False, 400, False, 40001, "parameter error"]
                data.append(statefield)
            kwargs = {"reqtype": "reqSave", "userid": self.userid, "gdocid": gdocid,
                      "data": data, "profile": self.profile, "tim_id": tim_id}
            serverlogger.debug("getSave kwargs: %s" % str(kwargs))
            xml = bitfarm_xml.create_xml_request(**kwargs)
            return self._sendxml(xml)
        else:
            # success, statuscode, result, errorcode, message
            return [False, 404, False, 404, "authentication error, not logged in"]

    def getNewDoc(self, arcid, archive, title, filename, addfields, statefields, docstream, links=[], doctags=[], lang="de", astif=True, ocr=True, wvlparams={}, filter=""):
        """
        data = {
            "arcid": 15,
            "archive": "wfdtest",
            "title": "mein Dokumententitel",
            "filename": "filename.png",
            "doctags": "Schlagwort1;Schlagwort2",
            "links": "test-Verknüpfung1, test-Verknüpfung2",
            "addfields": [
                    {   "indexid":1,
                        "id": 3,
                        "value": "test",
                        "add": "add_27",
                    },
                    ...
            ]
            "statefields": [
                    {   "indexid":0,
                        "id": 27,
                        "value": True,
                        "bitmask": 4,
                    },
                    ...
            ]
            "docstream": <base64-codierter String, der das Dokument enthält>,
            "wvlparams": {
                "info": "bla", -> doc.info
                "users": "dmsadmin;#Gruppe1;§Gruppe2", -> doc.editor
                "wvldate": "2021-07-30 00:00:00", -> doc.target_date
                "alarmdate": "2021-08-04 00:00:00", -> doc.alarm_date
                "options": 0 -> doc.timeroptions (0,1,2,4,8,64)
            }
        }
        """
        #serverlogger.debug("getnewdoc addfields:")
        # serverlogger.debug(str(addfields))
        if self.loggedin is True:
            docstream.seek(0, os.SEEK_END)
            datalen = docstream.tell()
            docstream.seek(0)
            filename = removeDisallowedFilenameChars(filename)
            ext = os.path.splitext(filename)[1].lower()
            if astif is True:
                if ext == ".png":
                    png2tiffexe = os.path.join(
                        self.con.clientprogram, "png2tiff.exe")
                    if not os.path.isfile(png2tiffexe):
                        serverlogger.warning("png2tiff.exe not found, archiving as %s..." % ext)
                    else:
                        tempdir = tempfile.mkdtemp()
                        orifile = os.path.join(tempdir, filename)
                        neworifile = os.path.splitext(orifile)[0]+".tif"
                        with open(orifile, "wb") as fobj:
                            fobj.write(docstream.read())
                        cmd = [png2tiffexe, orifile]

                        returncode, stdout, stderr, timedout = timeoutprocess(cmd, 10)
                        if returncode != 0:
                            serverlogger.error("error converting png to tif with png2tiff.exe.")
                            serverlogger.error("png2tiff.exe returncode: %d, output: %s, err: %s" % (returncode, stdout, stderr))

                        if timedout is False and os.path.isfile(neworifile):
                            docstream = io.BytesIO()
                            with open(neworifile, "rb") as fobj:
                                docstream.write(fobj.read())
                                datalen = docstream.tell()
                                docstream.seek(0)
                                filename = os.path.splitext(filename)[0]+".tif"
                        else:
                            serverlogger.warning("tif-conversion failed, archiving %s" % ext)
                elif ext in (".bmp", ".jpg", ".jpeg"):
                    filterv4 = os.path.join(self.con.clientprogram, "filterv4.exe")
                    if not os.path.isfile(filterv4):
                        serverlogger.warning("FilterV4.exe not found, archiving as %s..." % ext)
                    else:
                        tempdir = tempfile.mkdtemp()
                        orifile = os.path.join(tempdir, filename)
                        neworifile = os.path.splitext(orifile)[0]+".tif"
                        with open(orifile, "wb") as fobj:
                            fobj.write(docstream.read())
                        cmd = [filterv4, "-in="+orifile, "-out="+neworifile, "format=tif"]

                        returncode, stdout, stderr, timedout = timeoutprocess(cmd, 10)
                        if returncode != 0:
                            serverlogger.error("error converting to tif with FilterV4.exe.")
                            serverlogger.error("FilterV4.exe returncode: %d, output: %s, err: %s" % (returncode, stdout, stderr))

                        if timedout is False and os.path.isfile(neworifile):
                            docstream = io.BytesIO()
                            with open(neworifile, "rb") as fobj:
                                docstream.write(fobj.read())
                                datalen = docstream.tell()
                                docstream.seek(0)
                                filename = os.path.splitext(filename)[0]+".tif"
                        else:
                            serverlogger.warning(
                                "tif-conversion failed, archiving %s" % ext)
            req = bitfarm_doc.bfaReq()
            req.reqtype = "reqArchive"
            req.profile = self.profile
            req.userid = self.userid
            req.datalen = datalen
            req.parameter = os.path.splitext(filename)[1]
            req.lang = lang

            doc = bitfarm_doc.BitfarmDocument()
            if wvlparams is not None:
                doc.info = wvlparams["info"]
                doc.editor = wvlparams["users"]
                doc.target_date = wvlparams["wvldate"]
                doc.alarm_date = wvlparams["alarmdate"]
                doc.timeroptions = wvlparams["options"]

            doc.tempid = uuid.uuid1()
            doc.arcid = arcid
            doc.profile = self.profile
            if title is None or title == "":
                title = "Dokument"
            doc.title = title
            doc.userid = self.userid
            doc.user = ''  # username
            doc.doctype = 1
            doc.filter = filter
            if ocr is True:
                doc.ocrtype = "OMP"
            else:
                doc.ocrtype = "TXT"
            doc.filetype = req.parameter
            doc.archive = archive
            doc.date = datetime.datetime.now()

            doc.fields = addfields
            doc.statefields = statefields
            doc.doctags = doctags
            doc.links = links
            req.data = doc

            # initialen Request senden
            result = self._sendrequest(req)
            try:
                if result[0] is False:
                    return result
            except:
                pass
            # if result is not True:
            #     return [False, "Request konnte nicht verarbeitet werden"]
            result = self._senddata(docstream, self.tempsock)
            # print result
            if result is not True:
                return [False, "Request konnte nicht verarbeitet werden"]
            response = self._getresponse(self.tempsock)
            result = self._handleresponse(response)
            return result

        else:
            return [False, "authentication error, not logged in"]

    def getSaveResub(self, gdocid, action, tim_id=None, options=None, users=None, groups=None, info=None, comment=None, wvl_date=None, alarm_date=None, status=None, asjson=False):
        """
        gdocid: Dokumenten-ID
        action: Gibt an, was mit einem Workflow gemacht werden soll
            - forward, create, change, set_done, cancel

            forward (wvl weiterleiten):
                - erfordert tim_id, options, users und oder groups
            create (neue wvl erzeugen):
                - erfordert users und oder groups, info, comment, options, date, alarm_date
            change (wvl bearbeiten):
                - erfordert info, comment, users, groups, userid_shed, date, alarm_date, status, tim_id
            set_done, cancel:
                - erfordert tim_id

        - tim_id: tim_id der zu bearbeitenden Wiedervorlagen
        - date: Wvl-Datum "YYYY-MM-DD HH:MM"
        - alarm_date: wvl-Alarm-Datum "YYYY-MM-DD HH:MM"
        - info: wvl-Information
        - comment: wvl-Kommentar
        - users: String mit ";"-getrennten userids
        - groups: String mit ";"-getrennten groupids
        - options: wvl-Option als Integer, mapping ???
        - status: ???

        ####### controller? ##############
        Chatprotokoll 25.08.2016
        f.schneider
        (11:59:51) wenn kein controller bei dem request mitgegeben wird, dann ist man selbst der controller
        (12:01:05) wenn man vom client aus eine wiedervorlage anlegt, ist das ja dann immer der fall


        ####### status und options? ##############
        welche Ausprägungen von status und options gibt es in dem Zusammenhang und was bedeuten die?
        Chatprotokoll 25.08.2016
        f.schneider
        (12:32:08) options 0 = einer für alle, 1 = jeder einzelnd, gilt nur bei gruppen
        (12:32:14) status:
        (12:32:24) 1 = offen
        (12:32:29) 2 = in Arbeit
        (12:32:33) 4 = fertig
        (12:32:40) 8 = in Rückfrage
        (12:33:30) siehe Doku: file:///X:/bin/docs/python-api/bitfarm_doc.BitfarmWorkflow-class.html
        (12:36:05) oben stehen noch die anderen möglichen options return to sender und erinnerungsmail


        also:
        options:
            0 Einer für Alle
            1 Jeder einzeln (required wird ausgewertet, 0= Alle, 1-n= Anzahl der fertig zu setzenden User
            2 Return to Sender
            4 Erinnerungsmail
            einfach aufaddieren und als options mitgeben
        status:
            1 = offen
            2 = in Arbeit
            4 = fertig
            8 = in Rückfrage

        """
        param = action.lower()
        if param not in ("forward", "create", "change", "set_done", "cancel"):
            return [False, 404, "unknown param"]
        param = [param]

        wf = bitfarm_doc.BitfarmWorkflow()
        wf.profile = self.profile.lower()
        wf.gdocid = gdocid
        if tim_id is not None:
            wf.tim_id = tim_id
        if users is not None:
            wf.users = users
        if groups is not None:
            wf.groups = groups
        if status is not None:
            wf.status = status
        if options is not None:
            wf.options = options
        if info is not None:
            wf.info = info
        if comment is not None:
            wf.comment = comment
        if wvl_date is not None:
            wf.date = wvl_date
        if alarm_date is not None:
            wf.alarm_date = alarm_date
        if asjson is True:
            param.append("asjson")
        xml = bitfarm_xml.create_xml_request(reqtype="reqSaveResub", profile=self.profile,
                                             userid=self.userid, gdocid=gdocid, parameter=param, data=wf, shortxml=False)
        # print xml
        return self._sendxml(xml)

    def getCancel(self, gdocid):  # not implemented
        # print gdocid
        if self.loggedin is True:
            xml = bitfarm_xml.create_xml_request(
                reqtype="reqCancel", profile=self.profile, userid=self.userid, gdocid=gdocid)
            # print xml
            return self._sendxml(xml)
        else:
            return [False, "authentication error, not logged in"]

    def getshutdownServer(self):  # not implemented
        return [False, "not implemented"]

    def getBookmarks(self, global_bm=True, local_bm=True, history_bm=True, asjson=False):
        params = []
        if global_bm is True:
            params.append("globale")
        if local_bm is True:
            params.append("locale")
        if history_bm is True:
            params.append("history")
        if asjson is True:
            params.append("asjson")
        paramstr = ";".join(params)
        xml = bitfarm_xml.create_xml_request(
            reqtype="reqGetBookmarks", userid=self.userid, parameter=paramstr)
        return self._sendxml(xml)

    # def getExecuteBookmarks(self, bmtype, userid, groupid, indexid, asjson=False, guid=None):
    #     params = []
    #     if guid is None:
    #         params.append(
    #             ",".join([str(bmtype), str(userid), str(groupid), str(indexid)]))
    #     else:
    #         params.append(
    #             ",".join([str(bmtype), str(userid), str(groupid), str(guid)]))
    #     if asjson is True:
    #         params.append("asjson")
    #     paramstr = ";".join(params)
    #     xml = bitfarm_xml.create_xml_request(
    #         reqtype="reqExecuteBookmark", userid=self.userid, parameter=paramstr)
    #     return self._sendxml(xml)

    def getExecuteBookmarks(self, bmtype, userid, groupid, indexid, asjson=False, guid=None, usercontext=None):
        params = []

        if guid is None:
            params.append(",".join([str(bmtype), str(userid), str(groupid), str(indexid)]))
        else:
            params.append(",".join([str(bmtype), str(userid), str(groupid), str(guid)]))
        # params.append(",".join([str(bmtype), str(userid), str(groupid), str(indexid)]))
        if asjson is True:
            params.append("asjson")
        if usercontext is not None:
            params.append("usercontext:%d" % usercontext)
        paramstr = ";".join(params)

        xml = bitfarm_xml.create_xml_request(reqtype="reqExecuteBookmark", userid=self.userid, parameter=paramstr)
        return self._sendxml(xml)

    def getHistory(self, gdocid, asjson=False, lang=None, tim_id=None):
        if self.loggedin is True:
            if asjson is True:
                xml = bitfarm_xml.create_xml_request(
                    reqtype="reqHistory", tim_id=tim_id, profile=self.profile, userid=self.userid, gdocid=gdocid, parameter=['asjson'])
                return self._sendxml(xml)
            else:
                xml = bitfarm_xml.create_xml_request(
                    reqtype="reqHistory", tim_id=tim_id, profile=self.profile, userid=self.userid, gdocid=gdocid, parameter=[], lang=lang)
                return self._sendxml(xml)
        else:
            return [False, "authentication error, not logged in"]

    def getDeleteLink(self, linkstring):
        pass

    def getRemoveLink(self, linkstring, gdocid):
        if self.loggedin is True:
            args = {"reqtype": "reqRemoveLink",
                    "profile": self.profile, "userid": self.userid}
            # gdocid muss als string übergeben werden...
            parameter = linkstring+";"+str(gdocid)

            args["parameter"] = parameter
            xml = bitfarm_xml.create_xml_request(**args)
            return self._sendxml(xml)
        else:
            return [False, "authentication error, not logged in"]

    def getDocCommentary(self, gdocid):
        if self.loggedin is True:
            args = {"reqtype": "reqDoc", "profile": self.profile,
                    "userid": self.userid, "gdocid": gdocid}
            parameter = ['base', 'commentary']
            args["parameter"] = parameter
            xml = bitfarm_xml.create_xml_request(**args)
            return self._sendxml(xml)
        else:
            return [False, "authentication error, not logged in"]

    def getDoc(self, gdocid, asjson=False, tim_id=None,  fields_arcid=None, include_text=False, include_annostream=False):
        if self.loggedin is True:
            args = {"reqtype": "reqDoc", "tim_id": tim_id,
                    "profile": self.profile, "userid": self.userid, "gdocid": gdocid}
            parameter = ['base', 'fields', 'links',
                         'iev', 'paths', 'commentary']
            if include_text is True:
                parameter.append("text")
            if include_annostream is True:
                parameter.append("include_annostream")
            if asjson is True:
                parameter.append('asjson')
            args["parameter"] = parameter
            if fields_arcid is not None:
                args["fields_arcid"] = fields_arcid
            xml = bitfarm_xml.create_xml_request(**args)
            return self._sendxml(xml)
        else:
            return [False, "authentication error, not logged in"]

    def getTxt(self, gdocid, asjson=False, tim_id=None):
        if self.loggedin is True:
            if asjson is True:
                xml = bitfarm_xml.create_xml_request(
                    reqtype="reqDoc", tim_id=tim_id, profile=self.profile, userid=self.userid, gdocid=gdocid, parameter=['text', 'asjson'])
                return self._sendxml(xml)
            else:
                xml = bitfarm_xml.create_xml_request(
                    reqtype="reqDoc", tim_id=tim_id, profile=self.profile, userid=self.userid, gdocid=gdocid, parameter=['text'])
                return self._sendxml(xml)
        else:
            return [False, "authentication error, not logged in"]

    def getPreview(self, gdocid, ievlayer=None, tim_id=None):
        if self.loggedin is True:
            req = bitfarm_doc.bfaReq()
            req.reqtype = "reqTif"
            req.userid = self.userid
            req.gdocid = gdocid
            req.profile = self.profile
            req.tim_id = tim_id
            req.parameter = {'ievlayer': ievlayer}
            # req.parameter=['base','fields']
            xml = bitfarm_xml.object_to_xml(req)
            result = self._sendxml(xml)
            return result
        else:
            return [False, "authentication error, not logged in"]

    def getImg(self, gdocid, imgtype="png", page=1, ievlayer=None, forcerendering=False, tim_id=None):
        if self.loggedin is True:
            req = bitfarm_doc.bfaReq()
            req.reqtype = "reqImg"
            req.userid = self.userid
            req.gdocid = gdocid
            req.tim_id = tim_id
            req.profile = self.profile
            req.parameter = {"imgtype": imgtype, 'page': page-1,
                             'ievlayer': ievlayer, "forcerendering": forcerendering}
            xml = bitfarm_xml.object_to_xml(req)
            return self._sendxml(xml)
        else:
            return [False, "authentication error, not logged in"]

    def getOri(self, gdocid, revision=None, tim_id=None):
        if self.loggedin is True:
            req = bitfarm_doc.bfaReq()
            req.reqtype = "reqOri"
            req.userid = self.userid
            req.tim_id = tim_id
            if revision is None:
                req.revision = 0
            else:
                req.revision = int(revision)
            req.gdocid = gdocid
            req.profile = self.profile
            #xml = bitfarm_xml.create_xml_request(reqtype="reqOri", userid = self.userid, gdocid = gdocid, revision=0)
            xml = bitfarm_xml.object_to_xml(req)
            parsed = minidom.parseString(xml)
            # print parsed.toprettyxml()
            return self._sendxml(xml)
        else:
            return [False, "authentication error, not logged in"]

    def getOcrXml(self, gdocid):
        if self.loggedin is True:
            req = bitfarm_doc.bfaReq()
            req.reqtype = "reqOcrXml"
            req.userid = self.userid
            req.gdocid = gdocid
            req.profile = self.profile
            #xml = bitfarm_xml.create_xml_request(reqtype="reqOri", userid = self.userid, gdocid = gdocid, revision=0)
            xml = bitfarm_xml.object_to_xml(req)
            parsed = minidom.parseString(xml)
            # print parsed.toprettyxml()
            return self._sendxml(xml)
        else:
            return [False, "authentication error, not logged in"]

    def getLastModified(self, gdocid, revision=None):
        # print "getLastModified called"
        if self.loggedin is True:
            req = bitfarm_doc.bfaReq()
            req.reqtype = "reqLastModified"
            req.userid = self.userid
            req.gdocid = gdocid
            req.profile = self.profile
            #req.gdocid = '1458;INSERT INTO sys_abo (gdocid, userid, tstamp,info) VALUES (123,123,"2016-09-24 00:00:00","pwned");--'
            xml = bitfarm_xml.object_to_xml(req)
#                            xml = """<?xml version='1.0' encoding='us-ascii'?>
#                <bfaReq>
#                    <_bfaReq__reqtype typ="str">reqLastModified</_bfaReq__reqtype>
#                    <prio typ="int">2</prio>
#                    <_BitfarmBaseObject__gdocid typ="str">1458;INSERT INTO sys_abo (gdocid, userid, tstamp,info) VALUES (123,123,"2016-09-24 00:00:00","pwned");--</_BitfarmBaseObject__gdocid>
#                    <version typ="int">36</version>
#                    <alarm typ="int">7</alarm>
#                    <userid typ="int">12</userid>
#                </bfaReq>"""
#                            xml = """<?xml version='1.0' encoding='us-ascii'?>
#                <bfaReq>
#                    <_bfaReq__reqtype typ="str">reqLastModified</_bfaReq__reqtype>
#                    <prio typ="int">2</prio>
#                    <_BitfarmBaseObject__gdocid typ="str">1458;SELECT username, userpass from sys_user;</_BitfarmBaseObject__gdocid>
#                    <version typ="int">36</version>
#                    <alarm typ="int">7</alarm>
#                    <userid typ="int">12</userid>
#                </bfaReq>"""
            return self._sendxml(xml)
        else:
            return [False, "authentication error, not logged in"]


actionloggedin = """0. Logout
1. Archivbaum laden
2. Dokument laden
3. Voransicht laden
4. Originaldatei laden
5. getImg
***************************
Auswahl: """
actionloggedout = """0. Beenden
1. Profil wählen %s
2. Login
***************************
Auswahl: """


if __name__ == "__main__":

    bfaclient = BfaServerClient()
    try:
        bfaclient.load_profile(sys.argv[1].strip())
    except:
        pass
    bfaclient.responsehandler = ResponseConsoleHandler()
    direct = True
    if direct is False:
        try:
            actionType = ""
            loop = 0
            loops = 0
            gdocid = 0
            save = False
            lastrecv = 0
            message = ""
            while actionType != "0":
                i = 0
                xml = ""
                if actionType == "16" and loop < loops:
                    loop += 1
                else:
                    actionType = ""
                    loops = 0
                    loop = 0
                    gdocid = 0
                    lastrecv = 0
                    if message != "":
                        # print(message)
                        message = ""

                    # nicht eingeloggt
                    if bfaclient.loggedin is not True:
                        while actionType not in ("0", "1", "2"):
                            menu = actionloggedout
                            if bfaclient.profile is not None:
                                menu = menu % (
                                    "(aktives Profil: "+bfaclient.profile.encode("cp1252")+")")
                            else:
                                menu = menu % "(kein aktives Profil)"
                            actionType = input_wstrip(menu)
                            if actionType == "1":
                                profile = input_wstrip("Profilname: ").strip()
                                if bfaclient.load_profile(profile) is not True:
                                    message = "Profil konnte nicht geladen werden"
                                else:
                                    message = "mit Profil %s verbunden" % bfaclient.profile
                            elif actionType == "2":
                                if bfaclient.profile is None:
                                    print("Bitte zuerst Profil wählen!")
                                else:
                                    userid = input_wstrip("user-id: ")
                                    if userid == "":
                                        username = input_wstrip("user-name: ")
                                    userpass = input_wstrip("Passwort: ")
                                    if userid != "":
                                        result = bfaclient.connect(userid=userid, userpass=userpass)
                                    else:
                                        result = bfaclient.connect(username=username, userpass=userpass)
                                    print(bfaclient.xmlresponse)
                                    message = "Mit bfaserver verbunden"
                    # eingeloggt, Verbindung steht
                    else:
                        actionType = input_wstrip(actionloggedin)
                        if actionType == "0":
                            bfaclient.logout()
                            actionType = ""
                        elif actionType == "1":
                            archives = bfaclient.getArchives()
                            if archives[0] is True:
                                print("Archivbaum empfangen")
                                save = input_wstrip(
                                    "Archivbaum xml speichern (j/n)?: ").strip()
                                if save.lower() == "j":
                                    filepath = input_wstrip(
                                        "Dateiname (default archives.xml): ").strip()
                                    if filepath == "":
                                        filepath = "archives.xml"
                                    f = open(filepath, "wb")
                                    f.write(bitfarm_xml.object_to_xml(archives[1]))
                                    f.close()
                            else:
                                print(
                                    "Fehler, Archivbaum konnte nicht geladen werden.")
                        elif actionType == "2":
                            gdocid = input_wstrip("gdocid: ").strip()
                            try:
                                gdocid = int(gdocid)
                            except:
                                message = "Fehler beim Laden eines Dokuments, gdocid muss numerisch sein!"
                                continue
                            doc = bfaclient.getDoc(gdocid)
                            print("doc:", doc)
                            if doc:
                                print("Dokument empfangen")
                                save = input_wstrip(
                                    "Dokument als xml speichern (j/n)?: ").strip()
                                if save.lower() == "j":
                                    filepath = input_wstrip("Dateiname (default %d.xml): " % gdocid).strip()
                                    if filepath == "":
                                        filepath = "%s.xml" % gdocid
                                    f = open(filepath, "wb")
                                    f.write(bitfarm_xml.object_to_xml(doc))
                                    f.close()
                            else:
                                print("Fehler beim Laden des Dokuments!")
                                print(bfaclient.responseobject.data.decode("utf8"))
                        elif actionType == "3":
                            gdocid = input_wstrip("gdocid: ").strip()
                            try:
                                gdocid = int(gdocid)
                            except:
                                message = "Fehler beim Laden eines Dokuments, gdocid muss numerisch sein!"
                                continue
                            result = bfaclient.getTif(gdocid)
                            print("result:", result)
                            if result[0] is True:

                                if bfaclient.streamdata:
                                    print("Voransicht empfangen")
                                    bfaclient.streamdata.seek(0)
                                    save = input_wstrip(
                                        "Voransicht speichern (j/n)?: ").strip()
                                    if save.lower() == "j":
                                        filepath = input_wstrip("Dateiname (default %s): " % bfaclient.responseobject.filename).strip()
                                        if filepath == "":
                                            filepath = bfaclient.responseobject.filename
                                        f = open(filepath, "wb")
                                        while True:
                                            data = bfaclient.streamdata.read()
                                            if data:
                                                f.write(data)
                                            else:
                                                break
                                        f.close()
                            else:
                                message = "Fehler beim Laden der Voransicht!\n" + result[1]
                        elif actionType == "4":
                            gdocid = input_wstrip("gdocid: ").strip()
                            try:
                                gdocid = int(gdocid)
                            except:
                                message = "Fehler beim Laden eines Dokuments, gdocid muss numerisch sein!"
                                continue
                            result = bfaclient.getOri(gdocid)
                            print("result:", result)
                            if result[0] is True:

                                if bfaclient.streamdata:
                                    print("Originaldatei empfangen")
                                    bfaclient.streamdata.seek(0)
                                    save = input_wstrip("Originaldatei speichern (j/n)?: ").strip()
                                    if save.lower() == "j":
                                        filepath = input_wstrip("Dateiname (default %s): " % bfaclient.responseobject.filename).strip()
                                        if filepath == "":
                                            filepath = bfaclient.responseobject.filename
                                        f = open(filepath, "wb")
                                        f.write(bfaclient.streamdata.read())
                                        f.close()
                            else:
                                message = "Fehler beim Laden der Originaldatei!\n" + result[1]

                        elif actionType == "5":
                            gdocid = input_wstrip("gdocid: ").strip()
                            try:
                                gdocid = int(gdocid)
                            except:
                                message = "Fehler beim Laden eines Dokuments, gdocid muss numerisch sein!"
                                continue
                            # TODO: Seite, img-Typ und IEV abfragen
                            result = bfaclient.getImg(
                                gdocid, page=0, ievlayer=0)
                            print("result:", result)
#                            if result[0] is True:
#
#                                if bfaclient.streamdata:
#                                    print "Originaldatei empfangen"
#                                    bfaclient.streamdata.seek(0)
#                                    save = input_wstrip("Originaldatei speichern (j/n)?: ")
#                                    if save.lower()=="j":
#                                        filepath = input_wstrip("Dateiname (default %s): " %bfaclient.responseobject.filename)
#                                        if filepath == "":
#                                            filepath = bfaclient.responseobject.filename
#                                        f = open(filepath, "wb")
#                                        f.write(bfaclient.streamdata.read())
#                                        f.close()
#                            else:
#                                message = "Fehler beim Laden der Originaldatei!\n"+result[1]

        finally:
            bfaclient.logout()

    else:

        #servermethods = filter((lambda x: x.startswith("req")), [name for name, method in inspect.getmembers(BitfarmWorkerThread, inspect.ismethod)])
        clientmethods = list(filter((lambda x: x[0].startswith("get")), [(
            name, method) for name, method in inspect.getmembers(bfaclient, inspect.ismethod)]))
        clientmethodd = {key: value for (key, value) in clientmethods}
        clientmethodd["login"] = bfaclient.connect
        clientmethodd["logout"] = bfaclient.logout

        deadactions = []
        # for method in servermethods:
        # if "get"+method[3:] not in clientmethodd:
        #deadactions.append([method, None])
        # pprint(deadactions)

        deadactions = []

        actions = {}
        index = 0

        for key in sorted(clientmethodd.keys()):
            actions[index] = [key, clientmethodd[key]]
            index += 1

        menutxt = 20 * "*"+"\n"
        for key in sorted(actions.keys()):
            menutxt += str(key)+". " + actions[key][0]+"\n"
        menutxt += "x. quit\n"
        menutxt += 20 * "*"+"\n"
        menutxt += "your choice: "

        actionType = None
        while actionType != "x":
            invalid = True
            actions = {}
            index = 0

            while bfaclient.con is None:
                profile = input_wstrip("profilename: ").strip()
                result = bfaclient.load_profile(profile)
                print(result)

            for key in sorted(clientmethodd.keys()):
                if key == "login":
                    if bfaclient.loggedin is False:
                        actions[index] = [key, clientmethodd[key]]
                        index += 1
                elif key == "logout":
                    if bfaclient.loggedin is True:
                        actions[index] = [key, clientmethodd[key]]
                        index += 1
                else:
                    if bfaclient.loggedin is True:
                        actions[index] = [key, clientmethodd[key]]
                        index += 1

            menutxt = 20 * "*"+"\n"
            for key in sorted(actions.keys()):
                menutxt += str(key)+". " + actions[key][0]+"\n"
            menutxt += "x. quit\n"
            menutxt += 20 * "*"+"\n"
            menutxt += "your choice: "

            while invalid:
                actionType = input_wstrip(menutxt).strip()
                try:
                    index = int(actionType)
                    if index in actions:
                        invalid = False
                    else:
                        print("\ninvalid choice, try again...\n")
                except:
                    if actionType.lower() != "x":
                        print("\ninvalid choice, try again...\n")
                    else:
                        print("quit")
                        clientmethodd["logout"]()
                        sys.exit()

            args, varargs, keywords, defaults = inspect.getargspec(
                actions[int(actionType)][1])
            args.remove("self")
            myargs = {}
            process = True
            for index, arg in enumerate(args):
                if defaults is not None:
                    if -len(args)+len(defaults)+index < 0:
                        choice = input_wstrip(
                            arg+": ").strip()
                    else:
                        choice = input_wstrip(arg+" (default: " + str(defaults[-len(args)+len(
                            defaults)+index]) + "): ").strip()
                        if len(choice) == 0:
                            choice = None
                            myargs[arg] = defaults[-len(args) +
                                                   len(defaults)+index]
                else:
                    choice = input_wstrip(
                        arg+": ").strip()
#                if choice=="über":
#                    print choice, type(choice)

                if choice is not None:
                    if actions[int(actionType)][0].replace("req", "get") in bfaclient.args:
                        myargs[arg] = bfaclient.args[actions[int(
                            actionType)][0].replace("req", "get")][arg](choice)
                    else:
                        myargs[arg] = choice
            if process is True:
                try:
                    func = actions[int(actionType)][1]
                except:
                    func = bfaclient.responsehandler.handle_default
                result = func(**myargs)
                print(result)

            """
            for method in servermethods:
                if "get"+method[3:] in clientmethodd:
                    actions[index] = [method, clientmethodd["get"+method[3:]]]
                    index+=1
                else:
                    deadactions.append([method, None])
            """
