#!/usr/bin/env python
# -*- coding: latin-1 -*-

import os
import sys
import threading
import select
import shutil
import threading
import Queue
import logging
import logging.config
import datetime,  time
import cStringIO
import warnings
import subprocess
import re
import socket
import gettext
import decimal
from collections import deque


import bitfarm_doc
import bitfarm_proc
import bitfarm_errors
#import bitfarm_files
import bitfarm_xml
#import bitfarm_workflow
import bfaCustomizing

import glob
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.mime.image import MIMEImage
from email.mime.base import MIMEBase
from email import encoders

try:
    trans = gettext.translation("bfaArchiving", "locale", ['en'],  fallback = True)
    trans.install()
except:
    print(u'Keine bersetzung gefunden/no translation found')
    _ = lambda x: x
else:
    _ = trans.ugettext

import bfaOCRCore as ocr_class



class bfaWorkflow(threading.Thread, bfaCustomizing.bfaCustomizing):
    """bfaWorkflow-Klasse stellt eine Thread-Klasse zur Verfgung, mit grundlegenden, workflowverarbeitenden 
    Funktionen auf Applikationsebene. Jede Instanz erzeugt sich eine Instanz des bfaDocProcessing und bfaWflowProcessing-Objekts und hinterlegt
    diese in self.bfaProc und bfaWflow. Mit dem Parameter _con_db kann man ein Dictionary optional weitergeben, in dem fr jedes Profil
    schon eine verbundene DB-Connection hinterlegt ist (z.B, von einem bergeordneten Thread). {'Profil':sqla.engine.base.Connection}"""
    
    
    
    result = []
    resultlock = threading.Lock()
    queue_cnt = 1
    #queue = Queue.Queue(BitfarmArchivierung.cnt_wf_threads)
    queue = Queue.Queue(0)
    log = logging.getLogger("Main.bfaWorkflow")

    def __init__(self, _con, _mailserver, _con_db = None):
        threading.Thread.__init__(self)
        self._job = None
        self.con = _con
        #self.db = self.con.main.engine.connect()
        self.bfaProc = bitfarm_proc.BitfarmDocProcessing(self.con, _con_db)
        self.bfaWflow = bitfarm_proc.BitfarmWorkflowProcessing(self.con, _con_db)
        self.timeout = 0
        self.mailserver = _mailserver
        self.log.info(_(u'%s: Workflow-Thread initialisiert') %self.name)


    def run(self):
#        self.log = logging.getLogger("Main.bfaWorkflow")
        while self.is_alive():
            self._job = bfaWorkflow.queue.get()
            self.log.debug(_(u"%s: workflow [%s] (%i aktive tasks)") %(self.name, self._job.gdocid, bfaWorkflow.queue.unfinished_tasks))

#            start = time.time()
#            self.timeout = start +1
#            for i in range(200):
#                self.log.debug("w %s:pausiere Workflow %i" %(self.name, i))
#                time.sleep(0.01)
#            qtime = int(self.name[7:])
#            for i in range(qtime):
#                self.log.debug("w %s:verarbeite Workflow %i" %(self.name, i))
#                time.sleep(0.2)

            try:
                if hasattr(self, self._job.tasks[0].task):
                    getattr(self, self._job.tasks[0].task)()
                else:
                    self.result = False
                    self._job.error = EC_FUNCTIONNOTFOUND
                    self._job.errortext = _(u"Die Funktion %s konnte nicht gefunden werden") %self._job.tasks[0].task
                    self.log.error(self._job.errortext)

                self.log.debug(_(u'%s: verarbeitet: %s') %(self.name, self._job))
            except:
                self.log.exception(_(u'%s: Fehler in der Verarbeitung') %self.name)

            if self._job is not None:
                bfaWorkflow.resultlock.acquire()
                try:
                    if self._Thread__stopped:
                        break
                    if type(self._job) is list:
                        self.__class__.result.extend(self._job)
                    else:
                        self.__class__.result.append(self._job)
                    if bfaWorkflow.queue.unfinished_tasks > 0:
                        bfaWorkflow.queue.task_done()
                    self._job = None
                    self.timeout = 0
                finally:
                    bfaWorkflow.resultlock.release()
                    self.log.debug(_(u"%s: workflow fertig : %s") %(self.name, str(self._job)))
            else:
                self.log.error(_(u'%s: kein Job fr die Rckgabe mehr vorhanden.') %self.name)

        self.log.warning(_(u'%s: Thread wurde gestoppt') %self.name)


    def notify(self):
        """Erstellt eine Wiedervorlage die per BitfarmNotification-Objekt definiert wurde (Text, Empfnger, etc)"""
        self.log.debug(u'%s: notify' %self.name)

        if self._job.tasks[0].message is None:
            if self._job.tasks[0].warning is not None:
                self._job.tasks[0].message = self._job.tasks[0].message
            else:
                self.result = False
                return None

        if type(self._job.tasks[0].message) is not bitfarm_workflow.BitfarmNotification:
            self.result = False
            return None

        if isinstance(self._job, bitfarm_doc.BitfarmWorkflow):
            tempdoc = bitfarm_doc.BitfarmDocument()
            self.bfaProc.load(tempdoc, self._job.gdocid)

        if type(self._job.tasks[0].message.recipient) is not list:
            recp = [self._job.tasks[0].message.recipient]
        else:
            recp = self._job.tasks[0].message.recipient

        if len(recp) == 0:
            recp.append(self.con.main.admin)

        for i in range(len(recp)):
            if type(recp[i]) is str:
                if recp[i][0] == '%' and recp[i][-1] == '%':  #Variable im Usernamen: parsen
                    if tempdoc is None:
                        recp[i] = self.bfaProc.parse_txt(self._job, recp[i])
                    else:
                        recp[i] = self.bfaProc.parse_txt(tempdoc, recp[i])

        if tempdoc is None:
            subject = self.bfaProc.parse_txt(self._job, self._job.tasks[0].message.subject)
        else:
            subject = self.bfaProc.parse_txt(tempdoc, self._job.tasks[0].message.subject)
        if tempdoc is None:
            text = self.bfaProc.parse_txt(self._job, self._job.tasks[0].message.text)
        else:
            text = self.bfaProc.parse_txt(tempdoc, self._job.tasks[0].message.text)

        if not self.bfaWflow.create_resub(self._job, recp, subject, text):
            self.log.error(_(u'%s: Wiedervorlage konnte nicht angelegt werden'))
            self._job.result = False

        return True

    def do_workflow(self):
        """ kann einen beliebigen Workflow ausfhren. Der Workflow wird per ID oder Name geladen, falls vorhanden wird eine Condition geprft,
        und entsprechend der Task oder Else_Task ausgefhrt. Zum Ausfhren des Task wird dieser in die Task-Liste eingefgt
        und von der Funktion zusammen mit dem Ursprungs Objekt zurckgegeben. Um Vernderungen an dem Workflow zu sichern, wird zuletzt der
        Workflow wieder zurckgeschrieben."""
        self.log.debug(_(u'%s: do_workflow') %self.name)
        wf = None
        if self._job.tasks[0].params.wf_id is None and self._job.tasks[0].params.wf_name is None:
            self.log.warning(_(u'%s: es wurde keine wf_id oder wf_name bergeben'))
            return None

        _profile = self._job.profile
        if self._job.tasks[0].params.wf_id:
            wfs = self.bfaWflow.get_workflow_by_id(_profile, wf_id = self._job.tasks[0].params.wf_id, wf_trigger = self._job.tasks[0].params.wf_trigger)
            
        elif self._job.tasks[0].params.wf_name:
            wfs = self.bfaWflow.get_workflow_by_id(_profile, wf_name = self._job.tasks[0].params.wf_name, wf_trigger = self._job.tasks[0].params.wf_trigger)
        
        if wfs:
            for wf in wfs:
                self._job.wf_id = wf.wf_id
                if not self._job.gdocid:
                    self._job.gdocid = wf.gdocid
                if isinstance(self._job, bitfarm_doc.BitfarmWorkflow):     # wenn der Job selbst ein Workflow-Objekt ist
                    self._job.wf_name = wf.wf_name                              # bernehmen wir alle Parameter von dem geladenen Workflow-Objekt
                    self._job.wf_type = wf.wf_type
                    self._job.wf_pre.extend(wf.wf_pre)
                    self._job.wf_origin = wf.wf_origin
                    self._job.usr_counter = wf.usr_counter
                    self._job.grp_counter = wf.grp_counter
                    self._job.doc_counter = wf.doc_counter
                    self._job.status = wf.status
                    self._job.importpath = wf.importpath
                    self._job.wf_prio = wf.wf_prio
                    self._job.wf_trigger = wf.wf_trigger
                    self._job.inherited_trigger = wf.inherited_trigger

                if wf.condition is not None:
                    if wf.condition.execute(wf):
                        i = 1
                        for task in wf.tasks:
                            self._job.tasks.insert(i, task)
                            i += 1
                    else:
                        if wf.else_tasks is not None:
                            if type(wf.else_tasks) is list:
                                i = 1
                                for task in wf.else_tasks:
                                    self._job.tasks.insert(i, task)
                                    i += 1
                            else:
                                self._job.tasks.insert(1, wf.else_tasks)
                else:
                    i = 1
                    for task in wf.tasks:
                        self._job.tasks.insert(i, task)
                        i += 1

                self.bfaWflow.save_workflow_by_id(_profile, wf, wf.wf_id, wf_trigger = self._job.tasks[0].params.wf_trigger)

        else:
            self.log.warning(_(u'%s: do_workflow: es konnte kein Workflow mit wf_id %s wf_trigger %s gefunden werden ') %(self.name, self._job.tasks[0].params.wf_id, self._job.tasks[0].params.wf_trigger))
            self._job.result = False
            return False

        self._job.result = True
        return True

    def create_workflow_step(self):
        """Erzeugt einen Eintrag in der sys_workflow und legt somit einen temporren Workflowschritt an.
        Ist die Ausfhrung erfolgreich wird ein Eintrag in der sys_timer erzeugt (bei wf_type = 1 Userworkflow),
        ggf. wird ein vorhandener Eintrag in der sys_timer entfernt (task.params.wf_options).
        Die wf_id des auslsenden Workflows (also der Workflow, der den create_workflow_step-Task trgt,
        falls der Task nicht ber ein BitfarmDocument kommt)
        wird in die wf_pre Liste gespeichert, damit die end_workflow_step die Eintrge aus der sys_workflow wieder
        entfernen kann."""
        self.log.debug(_(u'%s: create_workflow_step') %self.name)
        if self._job.tasks[0].params.wf_id is None and self._job.tasks[0].params.wf_name is None:
            self.log.warning(_(u'%s: es wurde keine wf_id oder wf_name bergeben'))
            return None
        recp = []
        ctrl = None
        info = u""
        info_ext = u""
        date = None
        alarm = None
        delta = None
        tempdoc = None
        
        # wir laden einen Workflow Schritt aus der sys_wfd (dort ist die Definition aller Schritte hinterlegt)
        _profile = self._job.profile
        new_wf = None
        if self._job.tasks[0].params.wf_id:
            new_wfs = self.bfaWflow.get_workflow_by_id(_profile, wf_id = self._job.tasks[0].params.wf_id)
        elif self._job.tasks[0].params.wf_name:
            new_wfs = self.bfaWflow.get_workflow_by_id(_profile, wf_name = self._job.tasks[0].params.wf_name)

        insertids = []
        wf_pres = []
        if new_wfs:
            for new_wf in new_wfs:
                # wir tragen unsere Workflow-ID in die wf_pre Liste ein)
                if isinstance(self._job, bitfarm_doc.BitfarmWorkflow):
                    wf_pres.extend([x for x in self._job.wf_pre if x not in wf_pres])
                if self._job.wf_origin is not None:   #ist der ausgefhrte Workflow schon ein Schritt eines mehrstufigen WFs?
                    if self._job.wf_id not in wf_pres:
                        wf_pres.append(self._job.wf_id)
            
            for new_wf in new_wfs:
                new_wf.wf_origin = new_wf.wf_id
                if not new_wf.gdocid:
                    new_wf.gdocid = self._job.gdocid
                new_wf.wf_pre = wf_pres
                # wir schreiben diesen Workflowschritt in die sys_workflow
                try:
                    result = self.bfaWflow.save_workflow_by_id(_profile, wf = new_wf , wf_id = None,  wf_trigger = 3)
                except:
                    msg = _(u'%s: konnte Workflow nicht in sys_workflow speichern') %self.name
                    self.log.exception(msg)
                    self._job.result = False
                    self._job.error = EC_SAVEWORKFLOWERROR
                    self._job.errortext = msg
                    return False
                else:
                    if new_wf.wf_type == 1:                 # haben wir einen Userworkflow? dann soll fr diesen einen Eintrag in sys_timer erzeugt werden 
                        insertids.append(result.lastrowid)  # und die wf_id(s) hinterlegt werden
                    self.log.info(_(u'%s: create_workflow_step Workflowschritt angelegt id %s origin %s') %(self.name, insertids[-1], new_wf.wf_origin))

                #TODO: bei wf_type = 1 (Userworkflow) mssen wir noch in die sys_timer schreiben, je nach  wf_options mssen wir alte Eintrge aus der sys_timer entfernen

        if len(insertids) > 0:     #wir haben ein oder mehrere  Workflows angelegt, die User-Workflows sind, Wiedervorlage in sys_timer anlegen
            if isinstance(self._job, bitfarm_doc.BitfarmWorkflow):
                tempdoc = bitfarm_doc.BitfarmDocument()
                self.bfaProc.load(tempdoc, self._job.gdocid)
                new_wf.wf_name = tempdoc.title
                new_wf.arcid = tempdoc.arcid
                new_wf.gdocid = tempdoc.gdocid
            else:
                new_wf.wf_name = self._job.title

            recp = self._job.tasks[0].params.wf_recipient
            if len(recp) == 0:
                self.log.warning(_(u'%s: create_workflow_step: kein Empfnger hinterlegt.') %(self.name, self._job.tasks[0].params.wf_id))
                self._job.result = False
                return False
            ctrl = self._job.tasks[0].params.wf_controller
            if ctrl is None:
                ctrl = recp[0]
            
            if type(recp) is list:
                for i in range(len(recp)):
                    if type(recp[i]) is str:
                        if recp[i][0] == '%' and recp[i][-1] == '%':  #Variable im Usernamen: parsen
                            if tempdoc is None:
                                recp[i] = self.bfaProc.parse_txt(self._job, recp[i])
                            else:
                                recp[i] = self.bfaProc.parse_txt(tempdoc, recp[i])
            elif type(recp) is str:
                if recp[0] == '%' and recp[-1] == '%':  #Variable im Usernamen: parsen
                    if tempdoc is None:
                        recp = self.bfaProc.parse_txt(self._job, recp)
                    else:
                        recp = self.bfaProc.parse_txt(tempdoc, recp)
            
            if type(ctrl) is str:
                if ctrl[0] == '%' and ctrl[-1] == '%':  #Variable im Usernamen: parsen
                    if tempdoc is None:
                        ctrl = self.bfaProc.parse_txt(self._job, ctrl)
                    else:
                        ctrl = self.bfaProc.parse_txt(tempdoc, ctrl)
                          
            
            info = self._job.tasks[0].params.wf_info
            info_ext = self._job.tasks[0].params.wf_info_ext
            options = self._job.tasks[0].params.wf_options
            required = self._job.tasks[0].params.wf_required
            date = self._job.tasks[0].params.wf_date if self._job.tasks[0].params.wf_date is not None else datetime.datetime.now()
            if self._job.tasks[0].params.wf_alarm is None:
                delta = self._job.tasks[0].params.wf_delta if self._job.tasks[0].params.wf_delta is not None else 7
                alarm = date + datetime.timedelta(delta)
            else:
                alarm = self._job.tasks[0].params.wf_alarm
            date = date.replace(microsecond = 0)
            alarm = alarm.replace(microsecond = 0)
            
            tim_ref = self.bfaWflow.create_resub(new_wf, recp, info, info_ext, options, insertids, ctrl, date, alarm, required)
            if not tim_ref:
                self.log.warning(_(u'%s: create_workflow_step: es konnte keine Wiedervorlage fr wf_id %s angelegt werden.') %(self.name, self._job.tasks[0].params.wf_id))
                self._job.result = False
                return False
            else:
                self.log.info(_(u'%s: Wiedervorlage mit tim_ref %s angelegt') %(self.name, tim_ref))
        else:
            self.log.warning(_(u'%s: create_workflow_step: es konnte kein Workflow mit wf_id %s in der sys_wfd gefunden werden ') %(self.name, self._job.tasks[0].params.wf_id))
            self._job.result = False
            return False

        self._job.result = True
        return True


    def end_workflow(self):
        """Beendet einen temporren Workflow aus der sys_workflow (anhand von BitfarmWorkflow.wf_id)"""
        self.log.debug(_(u'%s: end_workflow') %self.name)

        if self.bfaWflow.delete_workflow_by_id(self._job.profile, wf_id = self._job.wf_id, wf_trigger = 3):
            self.log.info(_(u'%s: end_workflow entferne id %s in sys_workflow') %(self.name, self._job.wf_id))
        else:
            self.log.warning(_(u'%s: end_workflow id %s in sys_workflow konnte nicht entfernt werden') %(self.name, self._job.wf_id))

    def end_workflow_step(self):
        """Beendet eine mehrstufigen Workflow und entfernt alle temporren Eintrge aus der sys_workflow (anhand von BitfarmWorkflow.wf_pre)"""
        self.log.debug(_(u'%s: end_workflow_step') %self.name)
        for id in self._job.wf_pre:
            if self.bfaWflow.delete_workflow_by_id(self._job.profile, wf_id = id, wf_trigger = 3):
                self.log.info(_(u'%s: end_workflow_step entferne id %s in sys_workflow') %(self.name, id))
            else:
                self.log.warning(_(u'%s: end_workflow_step id %s in sys_workflow konnte nicht entfernt werden') %(self.name, id))

        if self.bfaWflow.delete_workflow_by_id(self._job.profile, wf_id = self._job.wf_id, wf_trigger = 3):
            self.log.info(_(u'%s: end_workflow_step entferne id %s in sys_workflow') %(self.name, self._job.wf_id))
        else:
            self.log.warning(_(u'%s: end_workflow_step id %s in sys_workflow konnte nicht entfernt werden') %(self.name, self._job.wf_id))

    def do_create_resub(self):
        """Legt eine WVL in der sys_timer an, jedoch ohne korrespondierenden Workflowschritt in der sys_workflow. Somit sind keine weiteren Schritte
        nach Fertig/Fortsetzen des Workflows mglich"""
        self.log.debug(_(u'%s: do_create_resub') %self.name)

        recp = self._job.tasks[0].params.wf_recipient
        if len(recp) == 0:
            self.log.warning(_(u'%s: do_create_resub: kein Empfnger hinterlegt.') %(self.name, self._job.tasks[0].params.wf_id))
            self._job.result = False
            return False
        ctrl = self._job.tasks[0].params.wf_controller
        if ctrl is None:
            ctrl = recp[0]
            
        if type(recp) is list:
            for i in range(len(recp)):
                if type(recp[i]) is str:
                    if recp[i][0] == '%' and recp[i][-1] == '%':  #Variable im Usernamen: parsen
                        recp[i] = self.bfaProc.parse_txt(self._job, recp[i])
        elif type(recp) is str:
            if recp[0] == '%' and recp[-1] == '%':  #Variable im Usernamen: parsen
                recp = self.bfaProc.parse_txt(self._job, recp)

        if type(ctrl) is str:
            if ctrl[0] == '%' and ctrl[-1] == '%':  #Variable im Usernamen: parsen
                ctrl = self.bfaProc.parse_txt(self._job, ctrl)
            
        info = self._job.tasks[0].params.wf_info
        info_ext = self._job.tasks[0].params.wf_info_ext
        options = self._job.tasks[0].params.wf_options
        required = self._job.tasks[0].params.wf_required
        date = self._job.tasks[0].params.wf_date if self._job.tasks[0].params.wf_date is not None else datetime.datetime.now()
        if self._job.tasks[0].params.wf_alarm is None:
            delta = self._job.tasks[0].params.wf_delta if self._job.tasks[0].params.wf_delta is not None else 7
            alarm = date + datetime.timedelta(delta)
        else:
            alarm = self._job.tasks[0].params.wf_alarm
        date = date.replace(microsecond = 0)
        alarm = alarm.replace(microsecond = 0)
        tim_ref = self.bfaWflow.create_resub(self._job, recp, info, info_ext, options, 0, ctrl, date, alarm, required)
        if not tim_ref:
            self.log.warning(_(u'%s: create_workflow_step: es konnte keine Wiedervorlage fr wf_id %s angelegt werden.') %(self.name, self._job.tasks[0].params.wf_id))
            self._job.result = False
            return False
        else:
            self.log.info(_(u'%s: Wiedervorlage mit tim_ref %s angelegt') %(self.name, tim_ref))


    def do_change_resub(self):
        """ndert eine WVL in der sys_timer. Mglich sind Text, Kommentar, Datum, Alarm und Controller.
        Achtung Controller nicht vollstndig untersttzt, nicht bei weitergeleiteten WVL ndern"""
        
        #TODO: Controller untersttzen
        self.log.debug(_(u'%s: do_change_resub') %self.name)
        subject = self._job.tasks[0].params.wf_info
        text = self._job.tasks[0].params.wf_info_ext
        date = self._job.tasks[0].params.wf_date
        alarm = self._job.tasks[0].params.wf_alarm
        ctrl = self._job.tasks[0].params.wf_controller
        if self.bfaWflow.change_resub(self._job, subject=subject, text=text, date=date, alarm=alarm, controller=ctrl):
            self.log.info(_(u'%s: Wiedervorlage gendert %s') %(self.name, self._job.tim_id)) 
            self._job.result = True
            return True
        else:
            self.log.info(_(u'%s: Wiedervorlage konnte nicht gendert werden %s') %(self.name, self._job.tim_id)) 
            self._job.result = False
            return False

    def do_setstate_done(self):
        """Prft ob ein Wiedervorlageneintrag auf Fertig gesetzt werden kann. Dabei werden die Optionen (Einer fr Alle, Jeder einzeln und Return to sender)
                sowie ggf. Urlaubsvertretung bercksichtigt. Zuerst wird geprft ob der Workflowschritt (wird per ID oder Name aus der sys_workflow gelesen)
                eine Condition trgt und diese zutreffend ist. Dann wird setstate_done aufgerufen, um zu ermitteln, ob der Workflowschritt
                komplett fertig gesetzt ist, dann wird der/die Task(s) des Workflows ausgefhrt. Hierzu werden neuen Workflowobjekte
                erzeugt, die dem Taskmanager bergeben werden."""
        self.log.debug(_(u'%s: setstate_done') %self.name)
        wf_ids = None
        if self._job.tasks[0].params.wf_id is not None:
            if type(self._job.tasks[0].params.wf_id) is int:
                wf_ids = [self._job.tasks[0].params.wf_id]
            if type(self._job.tasks[0].params.wf_id) is list:
                wf_ids = [int(x) for x in self._job.tasks[0].params.wf_id]
               
        
        
        if wf_ids is not None:     # wenn wir setstate_done fr einen mehrstufigen workflow durchfhren sollen
                                    # dann trgt dieser task die wf_id, ber den wf_trigger = 3 werden temporre WF abgefragt
            wfs = self.bfaWflow.get_workflow_by_id(wf_id = wf_ids,  wf_trigger = 3)
            if wfs is None:
                self.log.warning(_(u'%s: do_setstate_done es konnte kein Workflow gefunden werden') %self.name)
                self._job.result = False
                return False
        else:   # setstate_done soll fr eine einfache Wiedervorlage ausgefhrt werden
            wfs = None    # kein geladener WF
            
        wf_ids = []
        new_wfs = []
        if wfs is not None:
            do_setstate = False
            for wf in wfs:
                new_wf = None
                if wf.gdocid is not None:
                    if isinstance(self._job, bitfarm_doc.BitfarmDocument) and self._job.gdocid != wf.gdocid:
                        self.log.warning(_(u'%s: do_setstate_done das bergebene Dokument [%s] entspricht nicht dem Dokument des WFs [%s]') %(self.name, self._job.gdocid, wf.gdocid))
                        self._job.result = False
                        return False 
                    tempdoc = bitfarm_doc.BitfarmDocument()
                    self.bfaProc.load(tempdoc, wf.gdocid)
                    tempdoc.condition = bitfarm_xml.copy_bitfarmobj(wf.condition)
                if wf.condition is not None:
                    # wir bernehmen von allen wfs mit zutreffenden condition die tasks, bzw else_tasks
                    if wf.condition.execute(wf) or tempdoc.condition.execute(tempdoc):
                        do_setstate = True
                        new_wf = bitfarm_doc.BitfarmWorkflow()
                        new_wf.wf_id = wf.wf_id
                        new_wf.wf_pre = wf.wf_pre
                        new_wf.wf_pre.append(wf.wf_id)
                        new_wf.gdocid = wf.gdocid
                        
                        wf_ids.append(wf.wf_id) # wenn wir die Tasks bernehmen, mssen wir auch die wf_id in die wf_pre-Liste schreiben
                        i = 1
                        for task in wf.tasks:
                            new_wf.tasks.insert(i, task)
                            i += 1
                    else:
                        if wf.else_tasks is not None:
                            do_setstate = True
                            new_wf = bitfarm_doc.BitfarmWorkflow()
                            new_wf.wf_id = wf.wf_id
                            new_wf.wf_pre = wf.wf_pre
                            new_wf.wf_pre.append(wf.wf_id)
                            new_wf.gdocid = wf.gdocid
                                
                            wf_ids.append(wf.wf_id) # wenn wir die Tasks bernehmen, mssen wir auch die wf_id in die wf_pre-Liste schreiben
                            if type(wf.else_tasks) is list:
                                i = 1
                                for task in wf.else_tasks:
                                    new_wf.tasks.insert(i, task)
                                    i += 1
                            else:
                                new_wf.tasks.insert(1, wf.else_tasks)
                else:   # keine Conditon? Dann bernehme alle Tasks (falls vorhanden)
                    do_setstate = True
                    new_wf = bitfarm_doc.BitfarmWorkflow()
                    new_wf.wf_id = wf.wf_id
                    new_wf.wf_pre = wf.wf_pre
                    new_wf.wf_pre.append(wf.wf_id)
                    new_wf.gdocid = wf.gdocid

                    wf_ids.append(wf.wf_id) # wenn wir die Tasks bernehmen, mssen wir auch die wf_id in die wf_pre-Liste schreiben
                    i = 1
                    for task in wf.tasks:
                        new_wf.tasks.insert(i, task)
                        i += 1
                if new_wf is not None:
                    new_wfs.append(new_wf)
                
        else:
            do_setstate = True  #keine Workflows, dann fhren wir generell setstate_done aus
        
        if do_setstate:
            if self.bfaWflow.setstate_done(self._job):
                self.log.info(_(u'%s: do_setstatedone erfolgreich ausgefhrt') %self.name)
                if self._job.result:    # der WVL wurde vollstndig fertiggesetzt, jetzt die Tasks an den Job hngen
                    self.log.info(_(u'%s: die WVL mit der tim_ref %s wurde fertig gesetzt') %(self.name, self._job.tim_ref))

                    if len(new_wfs) > 0:
                        self._job = [self._job]
                        self._job.extend(new_wfs)
                        
                    # gdocid des WFs bernehmen
                    # wenn wir selbst vom Type BitfarmWorkflow sind, dann bernehmen wir die gdocid direkt
                    if isinstance(self._job, bitfarm_doc.BitfarmWorkflow):
                        self._job.gdocid = wf.gdocid
                        self._job.wf_pre.extend(wf.wf_pre)
                        self._job.wf_pre.extend(wf_ids)   # die wf_ids der geladenen workflows in die wf_pre Liste packen, falls der nchste Task ein 
                                                            # create_workflow_step sein soll (bei mehrstufigen Workflows ist das so)
                    if isinstance(self._job, bitfarm_doc.BitfarmDocument):  # bei Dokumenten nur, wenn wir nicht schon eine haben
                        if self._job.gdocid is None:
                            self._job.gdocid = wf.gdocid
                else:
                    self.log.info(_(u'%s: die WVL mit der tim_ref %s ist noch offen') %(self.name, self._job.tim_ref))
        else:
            self.log.info(_(u'%s: do_setstatedone Bedingung(en) des(der) Workflows nicht zutreffend') %self.name)
            self._job.result = False
            return False
                
    def do_forward_resub(self):
        """Leitet eine Wiedervorlage weiter. Die WVL wird ber die tim_id des Objekts definiert, Weiterleitungsziel und Optionen ber
        task.params.wf_recipient, task.params.wf_options, task.params.wf_required"""
        self.log.debug(_(u'%s: do_forward_resub') %self.name)

        if self.bfaWflow.forward_resub(self._job, self._job.tasks[0].params.wf_recipient, self._job.tasks[0].params.wf_options, self._job.tasks[0].wf_required):
            self.log.info(_(u'%s: Wiedervorlage erfolgreich weitergeleitet, tim_ref %s') %(self.name, self._job.tim_ref))
        else:
            self.log.warning(_(u'%s: Fehler beim Weiterleiten, tim_ref %s') %(self.name, self._job.tim_ref))


    def send_mail(self):
        """Sendet eine Email die per BitfarmNotification-Objekt definiert wurde (Text, Empfnger, etc)"""
        self.log.debug(_(u'%s: send_mail') %self.name)
        part1 = None
        part2 = None
        part3 = None
        server = 'localhost'
        port = 25
        fpath = None
        html = None
#        self.timeout = time.time() + 5    # 5 sec Timeout

        # TODO: bei einer bergebenen MessagID oder MessageName wird der Text/HTML-Body und
        #       Empfngerliste etc aus der Datenbank geholt

        if self._job.tasks[0].message is None:
            if self._job.tasks[0].warning is not None:
                self._job.tasks[0].message = self._job.tasks[0].message
            else:
                self.result = False
                return None

        if type(self._job.tasks[0].message) is not bitfarm_workflow.BitfarmNotification:
            self.result = False
            return None

        if isinstance(self._job, bitfarm_doc.BitfarmWorkflow):
            tempdoc = bitfarm_doc.BitfarmDocument()
            self.bfaProc.load(tempdoc, self._job.gdocid)

        msg = MIMEMultipart('multipart')
        msg['Subject'] = self.bfaProc.parse_txt(self._job, self._job.tasks[0].message.subject)
        me = self.con.main.sendername
        msg['From'] = me
        recp = []

        if type(self._job.tasks[0].message.recipient) != list:
            self._job.tasks[0].message.recipient = [self._job.tasks[0].message.recipient]

        for item in self._job.tasks[0].message.recipient:
            if type(item) in [str,  unicode]:
                if item.find(';') >= 0:
                    recp.extend(item.split(';'))
                else:
                    recp.append(item)

            elif type(item) == bitfarm_doc.BitfarmUser:
                if item.email is not None:
                    recp.append(item.email)
            elif type(item) == bitfarm_doc.BitfarmGroup:
                for member in item.members:
                    if member.email is not None:
                        recp.append(member.email)

        for i in range(len(recp)):
            if type(recp[i]) is str:
                if recp[i][0] == '%' and recp[i][-1] == '%':  #Variable im Usernamen: parsen
                    if tempdoc is None:
                        recp[i] = self.bfaProc.parse_txt(self._job, recp[i])
                    else:
                        recp[i] = self.bfaProc.parse_txt(tempdoc, recp[i])

        if len(recp) == 0:
            recp.append(self.con.main.adminmail)

        msg['To'] = ', '.join(recp)
        if self._job.tasks[0].message.text is not None:
            if tempdoc is None:
                self._job.tasks[0].message.text = self.bfaProc.parse_txt(self._job, self._job.tasks[0].message.text)
            else:
                self._job.tasks[0].message.text = self.bfaProc.parse_txt(tempdoc, self._job.tasks[0].message.text)
            part1 = MIMEText(self._job.tasks[0].message.text, 'plain', 'utf-8')
        if self._job.tasks[0].message.html is not None:
            if tempdoc is None:
                html = self.bfaProc.parse_txt(self._job, self._job.tasks[0].message.html)
            else:
                html = self.bfaProc.parse_txt(tempdoc, self._job.tasks[0].message.html)
        if self._job.tasks[0].message.attach_link:
            if self._job.gdocid is not None:
                if html is None:
                    html = ''
                html += '<BR><DIV><A HREF="gdocid:%s">Zum Dokument</A></DIV>' % self._job.gdocid
        if html is not None:
            part2 = MIMEText(html, 'html', 'utf-8')

        if self._job.tasks[0].message.attach_document:
            fpath = None
                
            if isinstance(self._job, bitfarm_doc.BitfarmWorkflow):
                if tempdoc is None:
                    self.log.warning(_(u'%s: es wurde ein Workflow-Objekt bergeben, wenn ein Dokument verarbeitet werden soll, muss dieses erst mit load_doc geladen werden') %self.name)
                else:
                    fpath = tempdoc.oriloc if tempdoc.oriloc else tempdoc.docloc
            else:
                fpath = self._job.path
            
            if fpath is not None and os.path.exists(os.path.expandvars(fpath)):
                fp = open(os.path.expandvars(fpath), 'rb')
                part3 = MIMEBase('application', 'octet-stream')
                part3.set_payload(fp.read())
                fp.close()
                # Encode the payload using Base64
                encoders.encode_base64(part3)
                # Set the filename parameter
                part3.add_header('Content-Disposition', 'attachment', filename= os.path.basename(fpath))
            else:
                self.log.warning(u'Dokument konnte nicht angehangen werden, Pfad nicht vorhanden %s' %fpath)

        if part1 is None and part2 is None: #Fallback
            part1 = MIMEText(u'Automatische Email vom bitfarm-Server', 'plain')

        if part1 is not None:
            msg.attach(part1)

        if part2 is not None:
            msg.attach(part2)

        if part3 is not None:
            msg.attach(part3)

        self.mailserver.queue.put((me, recp, msg))

        return self._job


    def load_doc(self, gdocid = None):
        """ ldt ein Dokument anhand der bergebenen Gdocid oder ber eine Params.gdocid und hngt alle Tasks an dieses Dokument 
            und bergibt es dem Task_Manager (es wird als zweites Objekt an self._job gehangen).
            Der Ursprungs-Task wird danach beendet"""
        self.log.debug(_(u'%s: load_doc') %self.name)
        if not isinstance(self._job, bitfarm_doc.BitfarmDocument) and not isinstance(self._job,  bitfarm_doc.BitfarmWorkflow):
            self.log.warning(_(u'Objekt kann nicht verarbeitet werden, weder Doc noch Workflow-Objekt'))
            self._job.result = False
            return None
        
        if gdocid is None:
            gdocid = self._job.tasks[0].params.gdocid
        
        if gdocid is None:
            self.log.warning(_(u'%s: keine gdocid bergeben') %self.name)
            self.result = False
            return None
        _doc = bitfarm_doc.BitfarmDocument()
        if not self.bfaProc.load(_doc, int(gdocid)):
            self.log.warning(_(u'%s: Dokument konnte nicht geladen werden') %self.name)
            self.result = False
            return None

        if _doc is None:
            self.log.warning(_(u'%s: Dokument konnte nicht geladen werden') %self.name)
            self.result = False
            return None

        _doc.tasks = self._job.tasks[1:]        # alle bis auf den aktuellen Task dem neuen Objekt bertragen
        self._job.tasks = [self._job.tasks[0]]  # die eigenen Tasks auf den aktuellen beschrnken, wird dann vom Taskmanager entfernt
        self._job.result = True
        self._job = [self._job]
        self._job.append(_doc)

        return True

    def copy_doc(self):
        """ ldt ein Dokument ber eine gdocid, erzeugt eine Kopie und hngt alle Tasks an dieses Dokument und bergibt es dem Task_Manager.
            Mit Params.gdocid wird die gdocid bergeben und mit Params.source wird angegeben, welche Datei geladen werden soll. Die Datei wird kopiert und in srcloc hinterlegt.
            Der Ursprungs-Task wird danach beendet"""
        self.log.debug(_(u'%s: copy_doc') %self.name)
        if not isinstance(self._job, bitfarm_doc.BitfarmDocument) and not isinstance(self._job,  bitfarm_doc.BitfarmWorkflow):
            self.log.warning(_(u'Objekt kann nicht verarbeitet werden, weder Doc noch Workflow-Objekt'))
            self._job.result = False
            return None

        if self._job.tasks[0].params.gdocid is None:
            self.log.warning(_(u'%s: keine gdocid bergeben') %self.name)
            self._job.result = False
            return None
        _src = bitfarm_doc.BitfarmDocument()
        if not  self.bfaProc.load(_src, int(self._job.tasks[0].params.gdocid)):
            self.log.warning(_(u'%s: Ursprungs-Dokument konnte nicht geladen werden') %self.name)
            self._job.result = False
            return None

        if _src is None:
            self.log.warning(_(u'%s: Ursprungs-Dokument konnte nicht geladen werden') %self.name)
            self._job.result = False
            return None

        _doc = bitfarm_doc.BitfarmDocument()
        self.bfaProc.create(_doc, _src.arcid)
        self.bfaProc.create_tempname(_doc)
        if self._job.tasks[0].params.source == 'docloc':
            _srcloc = _src.docloc
        if self._job.tasks[0].params.source == 'extloc':
            _srcloc = _src.extloc
        if self._job.tasks[0].params.source == 'oriloc':
            _srcloc = _src.oriloc

        if not _srcloc:
           _srcloc = _src.docloc

        # nun kopieren wir
        try:
            shutil.copy(_srcloc, _doc.tempname + os.path.splitext(_srcloc)[1])
        except:
            self.log.exception(_(u'%s: Ursprungs-Dokument konnte nicht kopiert werden') %self.name)
            self._job.result = False
        else:
            _doc.srcloc = _srcloc

            _doc.tasks = self._job.tasks[1:]        # alle bis auf den aktuellen Task dem neuen Objekt bertragen
            self._job.tasks = [self._job.tasks[0]]
            self._job.result = True
            self._job = [self._job]
            self._job.append(_doc)
        return True

    def move_doc(self):
        """verschiebt ein Dokument aus seinem Ursprungsarchiv in ein ber params.sortto angegebenen Archiv"""
        self.log.debug(_(u'%s: move_doc') %self.name)
        if not isinstance(self._job, bitfarm_doc.BitfarmDocument) and not isinstance(self._job,  bitfarm_doc.BitfarmWorkflow):
            self.log.warning(_(u'Objekt kann nicht verarbeitet werden, weder Doc noch Workflow-Objekt'))
            self._job.result = False
            return None

        if self._job.tasks[0].params.sortto is None:
            self.log.warning(u'%s: Zielarchiv nicht angegeben' %self.name)
            self._job.result = False
            return None

        self._job.result = self.bfaProc.move(self._job, self._job.tasks[0].params.sortto)

        return self._job.result


    def check_workflows(self):
        """Sucht anhand der bergebenen Informationen (doc oder wf-objekt) alle Workflows die diesen Bedingungen entsprechend
        ausgefhrt werden knnen. Es werden auch Default-Workflows gefunden. Die Funktion findet per default alle Workflows, die fr die
        Archivierung relevant sein knnen (wf_trigger=1), check_workflows ist daher ein Standard-Task in der Dokumentenverarbeitung.
        Es kann aber ber BitfarmParameter.wf_trigger auch nach anderen Workflows gesucht werden (zB wf_trigger = 2 sucht nach late oder interval-workflows,
        oder nach Workflows die fr das Vertragsmanagement eingerichtet wurden.)
        Dafr werden alle Workflows geladen, falls vorhanden die Bedingungen geprft und entsprechend der Task oder Else_Task zur Verarbeitung gegeben.
        Workflows ohne Tasks werden nicht zur Verarbeitung gegeben (zB. Bedingung ergab False und kein Else_task definiert.
        Wie die gefundenen Workflows verarbeitet werden ist vom Typ des aufrufenden Objekts (das Objekt, welches den check_workflows-Task trgt) abhngig:
        Es der gefundene Workflow gleichen Typs, dann werden die Task dieses Workflows in die Taskliste des Aufrufershinzugefgt,
        ist er unterschiedlichen Typs, dann wird dieser Workflow als eigenstndiges Objekt in die Verarbeitung gegeben.
        (Beispiel: Doc-Objekt ruft check_workflows auf, es wird ein Dokumenten-Workflow gefunden: Tasks werden in die eigene Taskliste aufgenommen.
        oder es wird ein User-Workflow gefunden: der User-Workflow wird als einzelnes Objekt der Verarbeitung bergeben.
        Ob Dokumenten oder User-Workflows gesucht werden kann mit Params.wf_type bergeben werden, wird nichts bergeben, wird beides gesucht"""
        self.log.debug(_(u'%s: check_workflows') %self.name)
        if not isinstance(self._job, bitfarm_doc.BitfarmDocument) and not isinstance(self._job,  bitfarm_doc.BitfarmWorkflow):
            self.log.warning(_(u'Objekt kann nicht verarbeitet werden, weder Doc noch Workflow-Objekt'))
            self._job.result = False
            return None

        jobs = []

        metype = 0 if isinstance(self._job, bitfarm_doc.BitfarmDocument) else 1
        metrigger = 1 if self._job.tasks[0].params.wf_trigger is None else self._job.tasks[0].params.wf_trigger
        i = 1

        if self._job.tasks[0].params.wf_type is None:
            wfs = self.bfaWflow.get_workflow(profile = self._job.profile, wf_type = 0, \
                                            wf_trigger = metrigger, arcid = self._job.arcid, \
                                            gdocid = self._job.gdocid, userid = self._job.userid, \
                                            groupid = self._job.groupid, date = self._job.date, \
                                            use_default = True)
            wfs.extend(self.bfaWflow.get_workflow(profile = self._job.profile, wf_type = 1, \
                                                wf_trigger = metrigger, arcid = self._job.arcid, \
                                                gdocid = self._job.gdocid, userid = self._job.userid, \
                                                groupid = self._job.groupid, date = self._job.date, \
                                                use_default = True))

        elif self._job.tasks[0].params.wf_type == 0:
            wfs = self.bfaWflow.get_workflow(profile = self._job.profile, wf_type = 0, \
                                            wf_trigger = metrigger, arcid = self._job.arcid, \
                                            gdocid = self._job.gdocid, userid = self._job.userid, \
                                            groupid = self._job.groupid, date = self._job.date, \
                                            use_default = True)
        elif self._job.tasks[0].params.wf_type == 1:
            wfs = self.bfaWflow.get_workflow(profile = self._job.profile, wf_type = 1, \
                                            wf_trigger = metrigger, arcid = self._job.arcid, \
                                            gdocid = self._job.gdocid, userid = self._job.userid, \
                                            groupid = self._job.groupid, date = self._job.date, \
                                            use_default = True)

        for wf in wfs:
            if self._job.wf_id is not None and wf.wf_id == self._job.wf_id:
                continue        #wir fgen uns selbst nicht hinzu, sonst werden wir unendlich
            if wf.tasks is None or len(wf.tasks) == 0:
                continue        # workflows ohne tasks werden nicht hinzugefgt

            if wf.condition is not None:
                if not wf.condition.execute(wf):    #condition gab false
                    if wf.else_tasks is not None:
                        if type(wf.else_tasks) is list:
                            wf.tasks = wf.else_tasks
                        else:
                            wf.tasks = [wf.else_tasks]
                    else:
                        continue    # workflows ohne tasks werden nicht hinzugefgt

            # hinzufgen
            if wf.wf_type == metype:   # ist der workflow vom selben typ wie das aufrufende objekt?
                self.log.info(_(u'Workflow gefunden, zur Taskliste hinzugefgt %s %s') %(wf, self._job))
                for task in wf.tasks:   # dann fge die gefundenen task ein
                    if task.task == 'check_workflows':      # wenn wir ein check_workflows task sind, sollten wir keinen weiteren
                        self.log.warning(_(u'gefundener check_workflows-Task wird verworfen, da sonst Endlosschleife erzeut wrde'))
                        continue                            # hinzufgen, sonst werden wir unendlich
                    self._job.tasks.insert(i, task)
                    i += 1

            else:   # workflows von jeweils anderen typ werden als einzelne objekte in die verarbeitung gegeben
                self.log.info(_(u'Workflow gefunden, zur Verarbeitung hinzugefgt %s') %wf)
                jobs.append(wf)

        if len(jobs) > 0:
            self.log.debug(_(u'Workflows gefunden %s') %jobs)
            self._job.result = True
            jobs.insert(0, self._job)
            self._job = jobs

        return True

    def set_displayname(self):
        self.log.debug(_(u'%s: set_displayname') %self.name)
        title = self.bfaProc.parse_txt(self._job, self._job.tasks[0].params.set_displayname)
        if self._job.title is None or self._job.title == u'':
            self._job.title = title

    def naming(self):
        """Fhrt naming-Regeln auf das Dokument durch. Naming-Regeln werden in GetField-Objekten gekapselt.
        Volltext oder Barcodes oder Emails oder mask muss vorhanden sein."""
        self.log.debug(_(u'%s: naming') %self.name)
        if not isinstance(self._job, bitfarm_doc.BitfarmDocument):
            self.log.warning(_(u'Objekt kann nicht verarbeitet werden, kein Document-Object'))
            self._job.result = False
            return None

        if type(self._job.tasks[0].params.getfield) not in [list,  bitfarm_workflow.BitfarmGetField]:
            self.log.warning(_(u'Task enthlt keine Getfield-Anweisung'))
            self._job.result = False
            return None
        #TODO: komplexe Abbruchbedingung definieren, Volltext und searchpattern, Barcode und searchbarcode, using und Email bzw. DOc-Objekt
#        if self._job.ftx.is_empty and len(self._job.barcodes) == 0:
#            self.log.warning(_(u'Dokument enthlt kein Volltext und/oder Barcodes'))
#            self._job.result = False
#            return None

        if type(self._job.tasks[0].params.getfield) ==  bitfarm_workflow.BitfarmGetField:
            self._job.tasks[0].params.getfield = [self._job.tasks[0].params.getfield]

#        loaded = []
        getfields = self.resolve_getfields(self._job.tasks[0].params.getfield)
        
        
#        for getfield in getfields:     #wir lsen falls angegeben id oder name auf und erhalten ggf. mehrere getfield-objekte aus der datenbank
#            if getfield.id is not None or getfield.name is not None:
#                newfield = self.bfaWflow.get_naming(id = getfield.id, name = getfield.name)
#                loaded.append(getfield)
#            else:
#                newfield = None
#            if newfield is None:
#                continue
#            if type(newfield) != list:
#                newfield = [newfield]
#
#            for field in newfield:          # wir fgen uns nicht nochmal hinzu
#                if field.id is not None:
#                    if field.id in [x.id for x in getfields]:
#                       continue 
#                if field.name is not None:
#                    if field.name in [x.name for x in getfields]:
#                        continue
#                getfields.append(field)
#        
#        for getfield in loaded:
#            getfields.remove(getfield)
        
        
        for getfield in getfields:
            self.log.debug(_(u'%s GetField-Anweisung, %s %s') %(self.name, u'using' if getfield.using else _(u'suche'), getfield.searchpattern if getfield.searchpattern else getfield.searchbarcode if getfield.searchbarcode else getfield.using))
            found = None
            ftx = u""

            if getfield.searchpattern is not None:
                rex = re.compile(getfield.searchpattern)

                if type(self._job.ftx) is bitfarm_doc.BitfarmFulltext:
                    if getfield.pages is not None:
                        if type(getfield.pages) == int:
                            if getfield.pages < self._job.ftx.pages:
                                if getfield.pages < 0:    # minus-angabe heisst von hinten
                                    ftx = self._job.ftx[self._job.ftx.pages - getfield.pages]
                                else:
                                    ftx = self._job.ftx[getfield.pages]
                            else:
                                ftx = self._job.ftx[self._job.ftx.pages -1]
                        elif type(getfield.pages) == list:
                            for page in getfield.pages:
                                if int(page) < self._job.ftx.pages:
                                    if page < 0:    # minus-angabe heisst von hinten
                                        ftx += self._job.ftx[self._job.ftx.pages - page]
                                    else:
                                        ftx += self._job.ftx[int(page)]
                                else:
                                    ftx += self._job.ftx[self._job.ftx.pages -1]
                        elif type(getfield.pages) == str:
                            if getfield.pages.find('-') > -1:
                               first, last = getfield.pages.split('-')
                               for page in range(int(first), int(last)):
                                    if int(page) < self._job.ftx.pages:
                                        ftx += self._job.ftx[int(page)]
                    else:
                        ftx = self._job.ftx.text
                        if ftx == u"":
                            self.log.warning(_(u'Dokument enthlt kein Volltext, Getfield-Anweisung wird bersprungen'))
                            continue
#                            self._job.result = False
#                            return None
                else:
                    if type(self._job.ftx) in [str,  unicode]:
                        ftx = self._job.ftx
                    else:
                        self.log.warning(_(u'Dokument enthlt kein Volltext, Getfield-Anweisung wird bersprungen'))
                        continue
#                        self._job.result = False
#                        return None
                found = rex.findall(ftx)

            elif getfield.searchbarcode is not None:
                if len(self._job.barcodes) > 0:
                    found = []
                    for barcode in self._job.barcodes:
                        if re.search(getfield.searchbarcode,  barcode.barcode):
                            found.append(barcode.barcode)

            elif getfield.using is not None:
                if getfield.using.lower() in [_(u"empfaenger"), _(u"absender"), _(u"empfangsdatum"), _(u"versanddatum"), _(u"empfangsuhrzeit"), _(u"versanduhrzeit"), _(u"cc"), _(u"bcc")] and self._job.email is not None:
                    if getfield.using.lower() == _(u"empfaenger"):
                        found = [self._job.email.recipient]
                    elif getfield.using.lower() == _(u"absender"):
                        found = [self._job.email.sender]
                    elif getfield.using.lower() == _(u"empfangsdatum"):
                        found = [self._job.email.delivery_date]
                    elif getfield.using.lower() == _(u"versanddatum"):
                        found = [self._job.email.date]
                    elif getfield.using.lower() == _(u"empfangsuhrzeit"):
                        found = [self._job.email.delivery_date]
                    elif getfield.using.lower() == _(u"versanduhrzeit"):
                        found = [self._job.email.date]
                    elif getfield.using.lower() == _(u"cc"):
                        found = [self._job.email.cc]
                    elif getfield.using.lower() == _(u"bcc"):
                        found = [self._job.email.bcc]
                elif getfield.using.lower() in [_(u"originaldatei"), _(u"richtung"), _(u"extension"), _(u"dateidatum"), _(u"herkunft"), _(u"benutzer"), _(u"tabelle"), _(u"autor"), _(u"titel"), _(u"groesse"), _(u"faxnummer")] and isinstance(self._job, bitfarm_doc.BitfarmDocument):
                    if getfield.using.lower() == _(u"originaldatei"):
                        found = [self._job.filename]
                    elif getfield.using.lower() == _(u"richtung"):
                        found = [self._job.origin]
                    elif getfield.using.lower() == _(u"extension"):
                        found = [os.path.splitext(self._job.path)[1]]
                    elif getfield.using.lower() == _(u"dateidatum"):
                        found = [self._job.file_created]
                    elif getfield.using.lower() == _(u"herkunft"):
                        found = [self._job.origin]
                    elif getfield.using.lower() == _(u"benutzer"):
                        found = [self._job.user]
                    elif getfield.using.lower() == _(u"tabelle"):
                        found = [self._job.archive]
                    elif getfield.using.lower() == _(u"autor"):
                        found = [self._job.user]
                    elif getfield.using.lower() == _(u"titel"):
                        found = [self._job.title]
                    elif getfield.using.lower() == _(u"groesse"):
                        found = [self._job.file_size]
#                    elif getfield.using.lower() == _(u"faxnummer"):
#                        found = [self._job.abc]
                elif isinstance(self._job, bitfarm_doc.BitfarmDocument) and hasattr(self._job, getfield.using.lower()):
                    if type(getattr(self._job, getfield.using.lower())) is str:
                        found = [getattr(self._job, getfield.using.lower())]
                    else:
                        found = [unicode(getattr(self._job, getfield.using.lower()))]

            elif getfield.mask is not None:
                found = []
                if type(getfield.mask) is not list:
                    getfield.mask = [getfield.mask]

                if self._job.ocrxmlpath and os.path.isfile(os.path.expandvars(self._job.ocrxmlpath)):   #wenn wir ein ocr_xml File haben, machen wir die Maskenextraktion darber
                    pass
                    #TODO: Maskenextraktion aus ocr_xml
                else:   # ansonsten oldschool mit ausschneiden und ocr
                    if not os.path.exists(os.path.join(self.con.main.bfapath, 'FilterV4.exe')):
                        self.log.warning(_(u'%s: konnte FilterV4.exe nicht finden') %(self.name))
                        self._job.result = False
                        return None
                    else:
                        for mask in getfield.mask:
                            if not isinstance(mask, bitfarm_doc.BitfarmRect):
                                continue
                            if mask.top is None or mask.left is None or mask.width is None or mask.height is None:
                                self.log.warning(_(u'%s: Maskenanweisung nicht vollstndig, wird bersprungen') %self.name)
                                continue

                            self.bfaProc.create_tmploc(self._job, '.tif')
                            args =[os.path.join(self.con.main.bfapath, 'FilterV4.exe'), '-in=' + os.path.expandvars(self._job.path)]
                            args.append('-out=' + os.path.expandvars(self._job.tmploc))

                            if getfield.pages is not None:
                                if type(getfield.pages) == int:
                                    args.append('-page=' + str(getfield.pages))
                                elif type(getfield.pages) == list:
                                     args.append('-page=' + ",".join([str(x) for x in getfield.pages]))
                                elif type(getfield.pages) == str:
                                    args.append('-page=' + getfield.pages)
                                else:
                                    args.append('-page=0')
                            else:
                                args.append('-page=0')

                            if  mask.left is not None:
                                args.append('-cutleft=' + str(mask.left))

                            if mask.width is not None:
                                args.append('-cutwidth=' + str(mask.width))

                            if mask.top is not None:
                                args.append('-cuttop=' + str(mask.top))

                            if mask.height is not None:
                                args.append('-cutheight=' + str(mask.height))

                            if mask.rotate is not None:
                                args.append('-rotate=' + str(mask.rotate))

                            timeout = 3000    #5min.
                            cnt = 0
                            is_timeout = False

                            try:
                                self.log.debug(_('%s: starte FilterV4 args = %s') %(self.name, " ".join(args)))
                                pipe = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

                                while pipe.returncode is None:
                                    time.sleep(0.1)
                                    cnt += 1
                                    if cnt > timeout:
                                        self.log.error(_(u'%s: timeout bei Filter') %self.name)
                                        is_timeout = True
                                        break
                                    pipe.poll()
                                if is_timeout:
                                    self.log.warning(_('%s: beende Filter Prozess') %self.name)
                                    pipe.terminate()
                                    continue
                            except:
                                self.log.exception(_(u'%s Filterv4.exe konnte nicht ausgefhrt werden'))
                                continue
                            if not os.path.exists(os.path.expandvars(self._job.tmploc)):
                                self.log.error(_(u'%s: Filtern nicht erfolgreich, keine Datei erzeugt') %self.name)
                                continue

                            if not getfield.mask_use_cre and hasattr(ocr_class.bfaOCR, '_do_omp'):
                                _text, _xml = ocr_class.bfaOCR._do_omp(self._job, os.path.expandvars(self._job.tmploc), self.name)
                                if _text != u'':
                                    found.append(self._parse_whitechars(_text))
                                    self.log.debug(_(u'%s: Maskentext erhalten [%s]') %(self.name, self._job))
                                else:
                                    self.log.debug(_(u'%s: kein Maskentext erhalten [%s]') %(self.name, self._job))
                            else:
                                _text = ocr_class.bfaOCR._do_cre(self.con, os.path.expandvars(self._job.tmploc), self.name)
                                if _text != u'':
                                    found.append(self._parse_whitechars(_text))
                                    self.log.debug(_(u'%s: Maskentext erhalten [%s]') %(self.name, self._job))
                                else:
                                    self.log.debug(_(u'%s: kein Maskentext erhalten [%s]') %(self.name, self._job))


            if found is None or len(found) == 0:
                if getfield.warning is not None:
                    _task1 = None
                    _task2 = None
                    if getfield.warning.create_resub:
                        _task1 = bitfarm_doc.BitfarmTask('notify')
                        _task1.message = getfield.warning
                    if getfield.warning.send_mail:
                        _task2 = bitfarm_doc.BitfarmTask('send_mail')
                        _task2.message = getfield.warning
                    if _task1:
                        self._job.tasks.insert(1, _task1)
                    if _task2:
                        self._job.tasks.insert(1, _task2)
                continue
            if len(found) > 0:
                resind = 0
                resfirst = 0
                reslast = len(found)
                if getfield.resultindex:
                    if type(getfield.resultindex) == int:
                        if getfield.resultindex < 0:    #negativ
                            if getfield.resultindex < - len(found):
                                getfield.resultindex = - len(found)
                        else:
                            if getfield.resultindex > len(found):
                                getfield.resultindex = len(found)
                        resind = getfield.resultindex
                    if type(getfield.resultindex) == str:
                        if getfield.resultindex.find('-') > -1:
                            resfirst, reslast =  getfield.resultindex.split('-')
                            resfirst = int(resfirst)
                            reslast = int(reslast)

                #TODO: in doc.fields ist jetzt field.add der key des dictionary. ggf muss hier was angepasst werden
                # bzw, die funktion sollte nicht den namen sondern den add-Bezeichner verlangen
                if getfield.addfield is not None:
                    if getfield.addfield in self._job.fields:
                        if getfield.set_continue:
                            self._job.fields[getfield.addfield].format = 'continue'
                        if getfield.set_writeable:
                            self._job.fields[getfield.addfield].format = 'writeable'
                        if getfield.nodeletespaces:
                            if 'deletespaces' in self._job.fields[getfield.addfield].format:
                                self._job.fields[getfield.addfield].format.remove('deletespaces')
                        else:   #TODO: msste deletespaces, dashes und dots nicht standard fr ein zusatzfeld sein?
                            if 'deletespaces' not in self._job.fields[getfield.addfield].format:
                                self._job.fields[getfield.addfield].format.append('deletespaces')
                        if getfield.nodeletedashes:
                            if 'deletedashes' in self._job.fields[getfield.addfield].format:
                                self._job.fields[getfield.addfield].format.remove('deletedashes')
                        if getfield.nodeletedots:
                            if 'deletedots' in self._job.fields[getfield.addfield].format:
                                self._job.fields[getfield.addfield].format.remove('deletedots')
                        if getfield.set_continue:
                            for ind in range(resfirst, reslast):
                            #for group in found:
                                self._job.fields[getfield.addfield].value = self._format_value(getfield, found[ind])
                        else:
                            self._job.fields[getfield.addfield].value = self._format_value(getfield, found[resind])
                    else:
                        new_field = self.bfaProc.get_addfield(name=getfield.addfield, profile=self._job.profile)
                        if new_field is None:
                            new_field = bitfarm_doc.BitfarmAddfield()
                            new_field.name = getfield.addfield
                        if getfield.set_continue:
                            new_field.format = 'continue'
                        if getfield.set_writeable:
                            new_field.format = 'writeable'
                        if getfield.nodeletespaces:
                            if 'deletespaces' in new_field.format:
                                new_field.format.remove('deletespaces')
                        else:
                            if 'deletespaces' not in new_field.format:
                                new_field.format.append('deletespaces')
                        if getfield.nodeletedashes:
                            if 'deletedashes' in new_field.format:
                                new_field.format.remove('deletedashes')
                        if getfield.nodeletedots:
                            if 'deletedots' in new_field.format:
                                new_field.format.remove('deletedots')
                        if getfield.set_continue:
                            for ind in range(resfirst, reslast):
                            #for group in found:
                                new_field.value = self._format_value(getfield, found[ind])
                        else:
                            new_field.value = self._format_value(getfield, found[resind])
                        if len(new_field.rights) == 0:
                            right = bitfarm_doc.BitfarmFieldRight()
                            right.modify = True
                            new_field.rights.add(right)    #auf Defaultmodus setzen
                        self._job.fields[new_field.name] = new_field
                else:   #zb. linkstring, displayname anweisungen bentigen kein addfield
                    for group in found:
                        self._format_value(getfield, group)

    def resolve_getfields(self, getfields) :
        _ret =[]
        for getfield in getfields:     #wir lsen falls angegeben id oder name auf und erhalten ggf. mehrere getfield-objekte aus der datenbank
            if getfield.id is not None or getfield.name is not None:
                newfield = self.bfaWflow.get_naming(id = getfield.id, name = getfield.name)
                newfield = self.resolve_getfields(newfield) 
            else:
                newfield = [getfield]

            if len(newfield) == 0:
                continue

            for field in newfield:          # wir fgen uns nicht nochmal hinzu
                if field.id is not None:
                    if field.id in [x.id for x in getfields]:
                       continue 
                if field.name is not None:
                    if field.name in [x.name for x in getfields]:
                        continue
                _ret.append(field)
                     # wenn wir was neues haben, dann wieder auflsen
        return _ret

    def _format_value(self, getfield, value):
        """Interne Hilfsfunktion zum Formatieren von Searchstrings"""
        _ret = value
        try:
            if getfield.cutleft is not None or getfield.cutright is not None:
                value = value[getfield.cutleft : - getfield.cutright]
            if getfield.getleft is not None:
                value = value[: getfield.getleft]
            if getfield.getright is not None:
                value = value[ - getfield.getright : ]
            if getfield.makedate:
                try:
                    if getfield.formatstring is not None:
                        value = datetime.datetime.strptime(value, getfield.formatstring)
                    else:   #TODO: hier mssen ggf. weitere Makedate-Formatangaben hin, die durchgetestet werden
                        value = datetime.datetime.strptime(value, '%d.%m.%Y')
                except:
                    value = _ret
                finally:
                    getfield.formatstring = None

            if getfield.numeric:
                # die locale einstellungen fr dieses dokument holen
                if self._job.profile is None or self._job.profile == '':
                    dec_point = self.con.main.localeconv['decimal_point']
                else:
                    dec_point = self.con[self._job.profile].localeconv['decimal_point']
                if value.find(dec_point) >= 0:
                    value = value.replace(dec_point, '.')
                if value.find('.') >= 0:
                    value = float(value)
                else:
                    value = int(value)

            if getfield.decimal:
                # die locale einstellungen fr dieses dokument holen
                if self._job.profile is None or self._job.profile == '':
                    th_sep = self.con.main.localeconv['thousands_sep']
                    dec_point = self.con.main.localeconv['decimal_point']
                else:
                    th_sep = self.con[self._job.profile].localeconv['thousands_sep']
                    dec_point = self.con[self._job.profile].localeconv['decimal_point']
                if value.find(th_sep) >= 0 and value.find(dec_point) >= 0:  #TODO: da kann man vielleicht auch eine regexp bauen
                    value = value.replace(th_sep, '')
                if value.find(dec_point) >= 0:
                    value = value.replace(dec_point, '.')
                value = decimal.Decimal(value)

            if getfield.formatstring is not None:
                if type(value) == list:
                    value = getfield.formatstring % ";".join(value)
                else:
                    value = getfield.formatstring % value

            if getfield.linkstring:
                if value not in self._job.links:
                    self._job.links.append(value)

            if getfield.addtag:
                if value not in self._job.tags:
                    self._job.tags.append(value)

            if getfield.displayname:
                if self._job.title is None or self._job.title == u"":
                    self._job.title = value

        finally:
            return value

    def _parse_whitechars(self, _text):
        _text = _text.replace('\t',' ')
        _text = _text.replace('\r\n',  ' ')
        _text = _text.replace('\n', ' ')
        while _text.find('  ') > -1:
            _text = _text.replace('  ', ' ')
        return _text

    def raise_workflow_error(self):
        self.log.debug(u'%s: raise_workflow_error' %self.name)
        self._job.error = EC_WORKFLOWERROR
