
import bitfarm_xml
import codecs
import sys
import os
import argparse

def get_pagebreaks(xmfile, textfile):
    with codecs.open(textfile, "r", encoding="cp1252") as f:
        text = f.read()
    xmlparser = bitfarm_xml.OcrXmlParser()
    xmlparser.readfile(xmfile)

    seitenumbrueche = []
    highestindex = 0
    oldindex = 0
    highestindexwordlength = 0
    for page in sorted(xmlparser.pages):
        oldindex = highestindex + highestindexwordlength
        xmlwords = xmlparser.pages[page]["words"]
        highestindexwordlength = 0
        for word, positions in xmlwords.items():
            
            wordpos = []
            myindex = oldindex
            for i in range(len(positions)):
                wordlen = 0
                try:
                    index = text.index(word, myindex)
                    wordlen = len(word)
                except:
                    index = None
                if index is None:
                    try:
                        index = text.index(word.encode("cp1252"), myindex)
                        wordlen = len(word)
                    except:
                        index = None
                if index is None:
                    try:
                        index = text.index(word.replace("\\", "\\\\"), myindex)
                        wordlen = len(word)
                    except:
                        index = None
                if index is None:
                    try:
                        index = text.index(word.replace("\\", "\\\\").encode("cp1252"), myindex)
                        wordlen = len(word)
                    except:
                        index = None    
                if index is not None:
                    wordpos.append(index)
                    myindex = index + wordlen
            index = max(wordpos)
            if index > highestindex:
                highestindex = index
                highestindexwordlength = len(word)
        seitenumbrueche.append(highestindex+highestindexwordlength)
    return seitenumbrueche
    
def add_pagebreaks(xmlfile, textfile, separator, outfile=None):
    seitenumbrueche = get_pagebreaks(xmlfile, textfile)
    with codecs.open(textfile, "r", encoding="cp1252") as f:
        text = f.read()
    newtext = ""
    oldindex = 0
    for i, index in enumerate(sorted(seitenumbrueche)):
        pagetext = text[oldindex:index]
        oldindex = index
        if "%d" in separator:
            pagesep = separator %(i+1)
        else:
            pagesep = separator
        newtext = newtext + pagetext + "\r\n" + pagesep + "\r\n"
    if outfile is None:
        outfile=textfile
    with codecs.open(outfile, "w", encoding="cp1252") as f:
        f.write(newtext)
        



if __name__=="__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("xmlfile", help="input xml-file")
    parser.add_argument("txtfile", help="input txt-file")
    parser.add_argument("pbstring", nargs='?', help="optional, string that should be used as pagebreak indicator. use %%d to refer to the pagenumber for example \"- Page %%d -\". Defaults to \"- Seite %%d -\"")
    parser.add_argument("-o", "--outfile", help="if you use this argument an new textfile with the given name will be created. If you omit this argument, the txtfile itself will be edited.", type=str)
    args = parser.parse_args()
    if args.pbstring is None:
        args.pbstring = "- Seite %d -"
    add_pagebreaks(args.xmlfile, args.txtfile, args.pbstring, args.outfile)
    
    #xmlfilepath = r"c:\users\t.hassler\Desktop\converterdist\ocrtemp\out0\1.ocr_xml"
    #textfile = r"c:\users\t.hassler\Desktop\converterdist\ocrtemp\out0\1.txt"
    #add_pagebreaks(xmlfilepath, textfile)

    

