import argparse
import codecs
import locale
import sys

from thottolib import find_dates_in_text

VERSION = "3.6.1.1"

class ArgumentParserWriteUTF8(argparse.ArgumentParser):
    """Subclass of ArgumentParser that outputs UTF-8 properly

    Standard ArgumentParser in argparse outputs help messages without
    converting to UTF-8. This fails if the message is a unicode string
    containing non-ASCII characters and output is redirected (to a
    pipe or a file). Unredirected output seems OK. See
    http://bugs.python.org/issue9779
    This class fixes it. It will probably not be needed in Python 3.
    """

    def _print_message(self, message, file=None):
        """Output message to file, encoded as UTF-8 """
        if message:
            if file is None:
                file = sys.stderr
            file.write(message.encode(locale.getpreferredencoding()))

parser = ArgumentParserWriteUTF8()
parser.add_argument("-text", help=u'text to parse', default=None)
parser.add_argument("-textfile", help=u'txt-file containing the text to be parsed (utf8!).', default=None)
parser.add_argument("-dateformat", help=u'formatstring for dates', default="%d.%m.%Y")

args = parser.parse_args()
if args.textfile != None:
    with codecs.open(args.textfile, "r", encoding="utf8") as fobj:
        txt = fobj.read()
elif args.text is not None:
    # u'05. M\xe4rz 2020'
    # -> works: M\xc3\xa4rz
    txt = args.text.decode("cp1252").encode("utf8")
else:
    parser.print_usage()

#for cdc in ("cp1252", "cp")

dates = find_dates_in_text(txt)
for d, dstr, (startpos, endpos), dtype in dates:
    print d.strftime(args.dateformat)
