#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
import os
import glob
import tempfile
import time
import shutil
import subprocess
import codecs 


from bfaServerClient import BfaServerClient, serverlogger
from xml_import import XML_Import
from pprint import pprint
import ConfigParser
import getpass
from dpapi import cryptData, decryptData
import base64

version = "3.6.1.2"



class ResponseHandler():
    def handle_respAuth(self, webclient, response):
        webclient.userid = response.userid
        webclient.loggedin = True
        return True
    
    def handle_default(self, client, response):
        return response.data
    
    def handle_respAddfields(self, client, response):
        fields = response.data
        addfields = {}
        for field in fields:
            addfields[field["id"]] = {"name":field["name"], "add": field["add"], "indextableid":field["indextableid"], "type":field["type"]}
            arcs = []
            for arc in field['rights']["_rights"]:
                try:
                    arcs.append({"arcid":arc["_BitfarmFieldRight__arcid"], "fieldpos": arc["_BitfarmFieldRight__fieldpos"]})
                except:
                    pass
            addfields[field["id"]]["arcs"] = arcs
        return addfields

    def handle_respAuthError(self, webclient, response):
        return False
        
        
    def handle_respArchives(self, webclient, response):
        addfields = webclient.getReadAddfields(asjson=True)
        arcs = {}
        for arc in response.data:
            if arc["enabled"] == u"enabled" or u"enabled" in arc["enabled"]:
                arcs[arc["id"]] = {"name":arc["arcname"], "lgrid":arc["lgrid"]}
                if "statefields" in arc:
                    sf = {}
                    for field in arc["statefields"]:
                        sf[field["id"]] = {"name":field["name"], "pos":field["pos"], "indexcolumnid":field["indexcolumnid"], "bitmask":field["_BitfarmStatefield__bitmask"]}
                    arcs[arc["id"]]["statefields"] = sf
                else:
                    arcs[arc["id"]]["statefields"] = []
            arcs[arc["id"]]["addfields"] = []
        for field in addfields:
            for fieldarc in addfields[field]["arcs"]:
                if fieldarc["arcid"] in arcs:
                    cleanfield = addfields[field].copy()
                    del cleanfield["arcs"]
                    arcs[fieldarc["arcid"]]["addfields"].append(cleanfield)
        return arcs
        
import os,codecs,sys

class zusPaste():

    def __init__(self):
        self.archivname = ""
        self.fileType = None
        self.filePath = None
        self.zusfields = {}
        self.lines = []
        self.links = []
        self.originaldatei=""
        self.stoparc = False
        self.nrs = None

    def dump(self):
        for line in self.lines:
            print line

    def tostr(self):
        zus_str = ""
        for line in self.lines:
            zus_str+=line
        return zus_str


    def open(self, filePath):
        if os.path.isfile(filePath):
            ext = os.path.splitext(filePath)[1].lower()
            if ext in [".var", ".job", ".move"]:
                try:
                    fileObj = codecs.open(filePath, "r", "latin1")
                    self.lines = fileObj.read().split("\n")
                    fileObj.close()
                    cleanlines=[]
                    for line in self.lines:
                        cleanlines.append(line.replace("\r", ""))
                    self.lines = cleanlines
                    
                    self.filePath = filePath
                    if ext==".var":
                        self.fileType = "var"
                    elif ext==".job":
                        self.fileType = "job"
                    elif ext==".move":
                        self.fileType = "move"
                    lastfield = ""
                    if self.fileType=="job":
                        for line in self.lines:
                            #line = line.replace("\r", "")
                            if line[:8].lower()=="zustitel":
                                lastfield = line.split("=")[1]
                            elif line[:7].lower()=="zuswert":
                                self.zusfields[lastfield]=line.split("=")[1]
                            elif line[:13].lower()=="originaldatei":
                                self.originaldatei = line.split("=")[1].strip()
                            elif line[:7].lower()=="tabelle":
                                self.archivname = line.split("=")[1].strip()  
                            elif line[:7].lower()=="gdoc_id":
                                self.gdocid=line.split("=")[1].strip()    
                            elif line[:7].lower()=="orginal":
                                self.original=line.split("=")[1].strip()  
                            elif line[:4].lower()=="name":
                                self.archivname=line.split("=")[1].strip() 
                            elif line[:5].lower()=="docid":
                                self.docid=line.split("=")[1].strip()
                            elif line[:6].lower()=="profil":
                                self.profil=line.split("=")[1].strip()
                    elif self.fileType=="var":
                        for line in self.lines:
                            #line = line.replace("\r", "")
                            if line[:10].lower()=="zusatzfeld":
                                lastfield = line.split("=")[1]
                            elif line[:4].lower()=="wert":
                                self.zusfields[lastfield]=line.split("=")[1]
                            elif line[:7].lower()=="tabelle":
                                self.archivname = line.split("=")[1]
                            elif line[:7].lower()=="docname":
                                self.docname = line.split("=")[1]
                            elif line[:8].lower()=="benutzer":
                                self.username = line.split("=")[1]
                            elif line[:10].lower()=="statusflag":
                                self.statusflag = line.split("=")[1]
                            elif line[:13].lower()=="originaldatei":
                                self.originaldatei = line.split("=")[1]
                            elif line[:5].lower()=="link=":
                                self.links.append(line.split("=")[1])
                except:
                    return None
                return True
            else:
                return None
        else:
            return None

    def write(self, filePath=None):
        self.paste()
        if not filePath:
            if self.filePath:
                try:
                    fileObj = codecs.open(self.filePath, "w", "latin1")
                except:
                    return None
            else:
                return None
        else:
            try:
                fileObj = open(filePath, "w")
            except:
                return None
        if self.stoparc is True:
            fileObj.write("STOPARC\r\n")
        if self.nrs is True:
            fileObj.write("nrs=true\r\n")
        if self.nrs is False:
            fileObj.write("nrs=false\r\n")
        for line in self.lines:
            if len(line.strip())>0:
                try:
                    fileObj.write(line+"\r\n")
                except:
                    print line,type(line)
                    sys.exit()
        fileObj.close()
        return True


    def paste(self):
        zusFeld =self.zusfields
        if self.fileType=="var":
            i=0
            line=self.lines[i]
            while line:
                if line[0:11].lower()=="zusatzfeld=":
                    val = line.split("=")[1].lower()
                    if val in zusFeld.keys():
                        if self.lines[i+1][0:5].lower()=="wert=":
                            try:
                                self.lines[i+1]="Wert="+str(zusFeld[val])
                            except:
                                self.lines[i+1]="Wert="+str(zusFeld[val])
                elif line[:7].lower()=="tabelle":
                    self.lines[i] = line.split("=")[0]+"="+self.archivname
                elif line[:7].lower()=="docname":
                    self.lines[i] = line.split("=")[0]+"="+self.docname
                elif line[:8].lower()=="benutzer":
                    self.lines[i] = line.split("=")[0]+"="+self.username
                elif line[:10].lower()=="statusflag":
                    self.lines[i] = line.split("=")[0]+"="+self.statusflag
                else:
                    self.lines[i] = line
                i+=1
                try:
                    line=self.lines[i]
                except:
                    line=None
            return True
        if self.fileType in ("job", "move"):
            i=0
            line=self.lines[i]
            while line:
                if line[0:8].lower()=="zustitel":
                    ref = line.split("=")[0][8:]
                    val = line.split("=")[1]			
                    zuskeys = [key.encode("latin1") for key in zusFeld.keys()]
                    if val.encode("latin1") in zuskeys:
                        if self.lines[i+2][0:7].lower()=="zuswert":
                            
                            try:
                                self.lines[i+2]="ZusWert"+ref.encode("latin1")+"="+zusFeld[val].encode("latin1")
                                self.lines[i+2]=self.lines[i+2].decode("latin1")
                            except:
                                #print val,type(val)
                                print zusFeld[val.encode("latin1")],type(zusFeld[val.encode("latin1")])
                                print zusFeld[val.encode("latin1")].encode('latin-1')
                                #print zusFeld[val.encode("latin1")],type(zusFeld[val.encode("latin1")])
                                sys.exit()
                                self.lines[i+2]="ZusWert"+ref+"="+str(zusFeld[val.encode("latin1")])
                i+=1
                try:
                    line=self.lines[i]
                except:
                    line=None
            return True
        return None

    def set_user(self, username):
        for i in range(0, len(self.lines)):
            if self.lines[i].lower()=="benutzer=":
                self.lines[i]="Benutzer="+username
                break

    def set_ocr_typ(self, ocr_typ):
        for i in range(0, len(self.lines)):
            if self.lines[i].lower()=="ocr_typ=":
                self.lines[i]="OCR_Typ="+ocr_typ
                break

    def add_link(self, link):
        """

        :type self: object
        """
        if self.fileType == "job":
            for i in range(0, len(self.lines)):
                if self.lines[i].lower() == "[referenz]":
                    ref = self.lines[i + 1].split("=")[0]
                    val = self.lines[i + 1].split("=")[1]
                    if val == "" or val is None:
                        val = "0"
                    tmp = int(val)
                    tmp += 1
                    self.lines[i + 1] = ref + "=" + str(tmp)
                    self.lines.insert(i + 1 + tmp, ("Referenz" + str(tmp) + "=" + link))
                    self.links.append(link)
                    
        if self.fileType == "var":
            if link is not None and link != "":
                self.lines.append((r"Link=" + link.replace("\n", "")))
                self.links.append(link)

    def link_count(self):
        return len(self.links)
        """
        iLinkCounter = 0
        iLineCounter = 0
        if self.fileType == "var":
            for line in self.lines:
                if line[0:5] == "Link=": iLinkCounter += 1
                iLineCounter += 1
        elif self.fileType == "job":
            for line in self.lines:
                if line.strip() == "[Referenz]":
                    if self.lines[iLineCounter + 1].split("=")[0] == "Anzahl":
                        iLinkCounter = int(self.lines[iLineCounter + 1].split("=")[1])
                iLineCounter += 1
        else: pass
        return(iLinkCounter)
        """
        
    #gibt true zurueck wenn ein Link ersetzt wurde und false wenn nicht
    def replace_link(self, oldlink, newlink):
        success = False
        iLineCounter = 0
        if self.fileType == "var":
            for line in self.lines:
                if line[0:5] == "Link=":
                    if line.split("=")[1] == oldlink:
                        self.lines[iLineCounter] = ("Link=" + newlink)
                        success = True
                        self.links.remove(oldlink)
                        self.links.append(newlink)
                iLineCounter += 1
        elif self.fileType == "job":
            for line in self.lines:
                if line[0:8] == "Referenz":
                    if line.split("=")[1] == oldlink:
                        self.lines[iLineCounter] = (line.split("=")[0] + "=" + newlink)
                        success = True
                        self.links.remove(oldlink)
                        self.links.append(newlink)
                iLineCounter += 1
        else: pass
        return(success)
    
    def setstatus(self, val):
        for i in range(0, len(self.lines)):
            if self.lines[i].strip().lower()=="status=":
                self.lines[i]="Status="+val
                break
                
    def delete_link(self, link):
        iLineCounter = 0
        tempList = self.lines
        if self.fileType == "var":
            for line in self.lines:
                if line[0:5] == "Link=":
                    if line.split("=")[1] == link:
                        print tempList[iLineCounter]
                        tempList.remove(line)
                        self.links.remove(link)
                iLineCounter += 1
        elif self.fileType == "job":
            iAmount = 0
            iAmountIndex = 0
            for line in self.lines:
                if line[0:7] == "Anzahl=":
                    iAmount = int(line.split("=")[1])
                    iAmount -= 1
                    iAmountIndex = iLineCounter
                if line[0:8] == "Referenz":
                    if line.split("=")[1] == link:
                        tempList.remove(line)
                        self.links.remove(link)
                        tempList[iAmountIndex] = ("Anzahl=%s" %(str(iAmount)))
                iLineCounter += 1
        else: pass
        self.lines = tempList
                









if len(sys.argv) < 2:
    print "import         : bfa_xmlimport.py <ini-file> <xml-file or xml-dir>"
    print "set credentials: bfa_xmlimport.py -cred <ini-file>"
    print "get version    : bfa_xmlimport.py -v"
    sys.exit(1)
    
if sys.argv[1] == "-v":
    print version
    sys.exit(0)
    
    
if len(sys.argv) < 3:
    print "import         : bfa_xmlimport.py <ini-file> <xml-file or xml-dir>"
    print "set credentials: bfa_xmlimport.py -cred <ini-file>"
    print "get version    : bfa_xmlimport.py -v"
    sys.exit(1)


if sys.argv[1] == "-cred":
    ini = ConfigParser.RawConfigParser()
    # ini.optionxform=unicode
    try:
        with codecs.open(sys.argv[2], "r", "utf8") as fobj:
            ini.readfp(fobj)
    except:
        print "ini '%s' not found" %sys.argv[2]
        sys.exit(1)
    username = raw_input("username: ")
    password = getpass.getpass("password (not shown): ")
    username = base64.b64encode(cryptData(username))
    password = base64.b64encode(cryptData(password))
    ini.set("main", "bfauser", username)
    ini.set("main", "bfapass", password)
    try:
        with codecs.open(sys.argv[2], "w", "utf8") as fobj:
            ini.write(fobj)
    except:
        print "error writing ini '%s'" %sys.argv[2]
        sys.exit(1)
    sys.exit()
    
ini = sys.argv[1]
importitem = sys.argv[2]



xml = XML_Import(ini)
serverlogger.disabled=1

xml.log.info(35*"-")   

    
    
xml.log.info("connecting bfaServer")
client = BfaServerClient(xml.profilename)
client.responsehandler = ResponseHandler()
try:
    clientconnect = client.connect(username=xml.bfauser, userpass=xml.bfapass)
except:
    xml.log.error("error connecting bfaServer")
    sys.exit(1)
if clientconnect is not True:
    if clientconnect == [False, 503]:
        xml.log.error("error connecting bfaServer")
    else:
        xml.log.error("loginerror")
    sys.exit(1)
xml.log.info("loading archives")
arcs = client.getReadArchives(asjson=True)
xml.log.info("got archives")
client.logout()
xml.log.info("bfaServer logout")
 
xmlfiles = []
if os.path.isdir(importitem):
    xml.log.info("processing all xml-files in %s" %importitem.encode("utf8"))
    for f in glob.glob(os.path.join(importitem.encode("utf8"), u"*.xml")):
        xmlfiles.append(f)
    for root, dirs, files in os.walk(importitem):
        for subdir in dirs:
            cdir = os.path.join(root.decode("cp1252"), subdir.decode("cp1252"))
            for f in glob.glob(os.path.join(cdir, "*.xml*")):
                xmlfiles.append(f)

elif os.path.isfile(importitem):
    xmlfiles.append(importitem)


xml.log.info(35*"-")
for xmlf in xmlfiles:
    xmlbasename = os.path.basename(xmlf)
    if type(xmlbasename) == str:
        xmlbasename = xmlbasename.decode(sys.getfilesystemencoding())
    xml.log.info("parsing xml-file '%s'" %xmlbasename)
    if xml.load_xml(xmlf) is not True:
        xml.log.error("parsing error, '%s' will not be imported" %xmlbasename)
        continue
    else:
        xml.log.info("xml-file parsed sucessfully, starting import")
    xml.log.info(20*"-")
    if xml.archive not in xml.templatearcids:
        xml.log.error("template '%s' not found, xml-file '%s' will not be imported" %(xml.archive, xmlbasename))
        continue
    if  xml.templatearcids[xml.archive] in arcs:
        try:
            
            configerror = False
            arc = arcs[xml.templatearcids[xml.archive]]
            if len(xml.zusatzfelder) > 0:
                addfields = [x["name"] for x in arc["addfields"]]
                for key in xml.zusatzfelder:
                    if key not in addfields:
                        xml.log.error("addfield '%s' is not availabe in archive '%s'" %(key, arc["name"]))
                        configerror = True
                        
                        
            statval = 0
            if len(xml.statusfelder) > 0:
                statfieldnames = [x["name"] for x in arc["statefields"].values()]
                for field in xml.statusfelder:
                    if field not in statfieldnames:
                        xml.log.error("statefield '%s' is unknown and cannot be set to true" %field)
                        configerror = True
                    else:
                        try:
                            statval += [v["bitmask"] for v in arc["statefields"].values() if v["name"] ==field][0]
                        except Exception as e:
                            xml.log.error("error setting statefield '%s' to true: %s" %(field, str(e)))
                            configerror = True
            if configerror is True:
                xml.log.error("xml-file '%s' will not be imported" %(xmlbasename))
                continue
            tempdir = tempfile.mkdtemp()
            temptpl = os.path.join(tempdir, "temp.tpl")
            tempjob = os.path.join(tempdir, "temp.job")
            shutil.copyfile(xml.templates[xml.archive], tempjob)
            xml.log.debug("copying template '%s'" %xml.templates[xml.archive])
            
            zp = zusPaste()
            zp.open(tempjob)
            zp.zusfields = xml.zusatzfelder
            zp.setstatus("0:"+str(statval))
            zp.set_user(xml.user)
            zp.set_ocr_typ("OMP")
            zp.write()
            xml.log.debug("jobfile '%s' created" %tempjob)
            if xml.archives[xml.archive]["plugin"] is not None:
                if os.path.isfile(xml.archives[xml.archive]["plugin"]):
                    proc = subprocess.Popen([xml.archives[xml.archive]["plugin"], tempjob])
                    xml.log.info("starting plugin '%s'" %xml.archives[xml.archive]["plugin"])
                    proc.wait()
                    if proc.returncode != 0:
                        xml.log.error("plugin error, returncode %s" %str(proc.returncode))
                    else:
                        xml.log.info("plugin terminated gracefully with returncode 0")
            basename = os.path.splitext(os.path.basename(xml.docfile))[0]
            
            ziel = os.path.join(xml.uebergabe, basename)
            jobziel = ziel+".job"
            docziel = ziel+os.path.splitext(os.path.basename(xml.docfile))[1]
            c = 0
            while os.path.isfile(jobziel) or os.path.isfile(docziel):
                jobziel = ziel+"_"+str(c)+"_.job"
                docziel = ziel+"_"+str(c)+"_"+os.path.splitext(os.path.basename(xml.docfile))[1]
                c+=1
            xml.log.debug("copy jobfile to '%s'" %jobziel)
            shutil.copyfile(tempjob, jobziel)
            xml.log.debug("copy docfile to '%s'" %docziel)
            shutil.copyfile(xml.docfile, docziel)
            xml.log.debug("deleting tempdir '%s'" %tempdir)
            shutil.rmtree(tempdir)
            xml.log.info("%s imported with xml '%s'" %(os.path.basename(xml.docfile), xmlbasename))
        except:
            xml.log.error("error while importing '%s'" %xmlbasename)
            continue
        if xml.deletedoc is True:
            xml.log.info("deleting document source '%s'" %xml.docfile)
            os.remove(xml.docfile)
        if xml.deletexml is True:
            xml.log.info("deleting xml source '%s'" %xml.xmlfile)
            os.remove(xml.xmlfile) 
        xml.log.info("import of xml-file '%s' finished" %xmlbasename)
        
    else:
        xml.log.error("template '%s' not found, ignoring '%s'" %(xml.archive, xmlbasename))
    xml.log.info(35*"-")
xml.log.info(60*"-")
xml.log.info(60*"-")



