#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Dieses Modul enthält die einfache BitfarmCon und BitfarmConnections Klasse, von bitfarm_baseobj geerbt,
als einfache Repräsentation eines Profils..
Sie kann in Anwendungen verwendet werden, die keinen Datenbankzugriff benötigen (siehe bitfarm_proc)"""

import os
import sys
import logging
import bitfarm_baseobj

__version__ = 'v3.6.2'


class BitfarmCon(bitfarm_baseobj.BitfarmBaseCon):
    """
    Bitfarm Configurations-Subklasse.
    Diese Klasse stellt ein einzelnes Profil zur Verfügung, beim Initialisieren kann der Name oder der komplette Pfad auf
    ein Profil übergeben werden.
    con = BitfarmCon('bitfarm')
    Eine Datenbank-Engine wird über die Methode create_db_engine hergestellt, diese wird in con.engine hinterlegt.
    Hier sind auch die Funktionen hinterlegt um User und Gruppen zu lesen, da diese profilabhängig und nicht
    workflow- oder dokumentabhängig sind. Dafür muss aber die DB-Connection übergeben werden, da diese ja erst zur Laufzeit
    erzeugt wird.
    Per Default wird das Con-Objekt als Version 35 erstellt, um abwärtskompatibiltät zur Datenbank 35 zu gewährleisten.
    Jede höhere Version (36, 40) muss in der Con-Datei in der Sektion [Server] angegeben werden.
    @version:
    """
    count = 0
    log = logging.getLogger("Main.BitfarmCon")
    log.setLevel(logging.WARNING)

    if sys.platform == 'win32':
        mypath = os.path.split(os.path.abspath(sys.argv[0]))[0]
    else:
        mypath = os.getcwd()
    try:
        bfapath = os.environ['bitfarm-archiv']
    except:
        bfapath = mypath

    # locale.setlocale(locale.LC_ALL, '')

    def __init__(self, *args):
        """
        Erstellt eine Profil-Instanz. Es kann entweder ein Profilname oder der Pfad auf eine .Con-Datei angegeben werden
        """
        self.log.debug("Initialisiere ein bitfarm_con Object")
        bitfarm_baseobj.BitfarmBaseCon.__init__(self, *args)


class BitfarmConnections(bitfarm_baseobj.BitfarmBaseConnections):
    """
        BitfarmConnections-Klasse. Diese Klasse stellt den Anwendungen ein oder mehrere Profile zur Verfügung.
        Das erste geladene Profil wird in BitfarmConnections.main abgelegt.
        Es können weitere Profile geladen werden in dem weitere Instanzen mit einem Profilnamen erzeugt werden.
        Auf alle Profile kann auch über den Namen zugegriffen werden  BitfarmConnections[Profil].
        Die create_db_engine-Methode erstellt eine Datenbank-Engine für alle geladenen Profile.
        Soll in Threads auf die Datenbank zugegriffen werden, so hinterlegt man innerhalb des Threads in eine lokale Variable
        ein Connection-Objekt, zB self.db = BitfarmConnections.main.engine.connect()
    @version:
    """
    version = __version__
    log = logging.getLogger("Main.BitfarmConnections")
    log.setLevel(logging.WARNING)

    def __init__(self, *name):
        """Erzeugt eine BitfarmConnections Instanz, es kann der Name / oder Pfad eine Profils übergeben werden, bzw. eine Liste von Profilen"""
        bitfarm_baseobj.BitfarmBaseConnections.__init__(self, *name)

