#!/usr/bin/env python
# -*- coding: utf-8 -*-

u"""Dieses Modul bietet Hilfs-Funktionen die für ein BitfarmDocument-Objekt zur Verfügung stehen. Die Funktionen sind aus allen Klassen und Modulen heraus aufrufbar, es muss nur ein BitfarmDocument übergeben werden."""

import os
import sys
import datetime
import gettext
import FreeImagePy as FIPY
import logging
import bitfarm_doc
from bitfarm_errors import *

try:
    trans = gettext.translation("bfaArchiving", "locale", ['en'], fallback=True)
    trans.install()
except:
    print(u'Keine Übersetzung gefunden/no translation found')
    _ = lambda x: x
else:
    _ = trans.ugettext

log = logging.getLogger('Main')


def get_fileinfo(_doc):
    u"""Holt Dateiinformationen (file_size, file_created, file_modified) sowie Anzahl der Seiten bei TIF."""
    if not isinstance(_doc, bitfarm_doc.BitfarmDocument):
        raise NoBitfarmDocumentError(_doc)
    log.debug(u'get_fileinfo')

    _docpath = _doc.path if _doc.path else _doc.docloc

    if not os.path.exists(os.path.expandvars(_docpath)):
        log.warning(_(u'get_fileinfo: Datei konnte nicht gefunden werden %s') %_docpath)
        return None

    stats = os.stat(os.path.expandvars(_docpath))
    _doc.file_size = stats.st_size
    _doc.file_created = datetime.datetime.fromtimestamp(stats.st_ctime)
    _doc.file_modified = datetime.datetime.fromtimestamp(stats.st_mtime)

    if os.path.splitext(_docpath)[1] == '.tif':
        try:
            f = FIPY.Image(os.path.expandvars(_docpath))
        except:
            log.exception(_(u'get_fileinfo: Datei konnte nicht geladen werden'))
            _doc.error = True
            _doc.errortext(_(u'get_fileinfo: Datei konnte nicht geladen werden'))
            return None
        if f.ok()[0] != 0:
            _doc.pages = None
        try:
            _doc.pages = f.getNumPages()
        except:
            log.warning(_(u'get_fileinfo: Fehler bei getNumPages'))
            _doc.pages = None
        else:
            if f.isMulti:
                for page in range(_doc.pages):
                    f.setCurrentPage(page)
                    # print F.getSize()
                    #TODO: Infos sSeitenweise auslesen und in ein entsprechenden fileinfo-Objekt packen
        try:
            f.quit()
        except:
            log.exception(_(u'get_fileinfo: Fehler bei FreeImagePy.quit()'))
        del f
