#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Helfer Modul für OAuth2

import msal
import base64
import oauth2

import win32clipboard


def copy_clipboard():
    win32clipboard.OpenClipboard()
    data = win32clipboard.GetClipboardData(win32clipboard.CF_UNICODETEXT)
    win32clipboard.CloseClipboard()
    return data


def paste_clipboard(text):
    win32clipboard.OpenClipboard()
    win32clipboard.EmptyClipboard()
    win32clipboard.SetClipboardText(text, win32clipboard.CF_UNICODETEXT)
    win32clipboard.CloseClipboard()

def get_access_token(oauth2_provider, oauth2_client_id, oauth2_client_secret, oauth2_tenant_id, oauth2_refresh_token):
    access_token = None
    refresh_token = None
    if oauth2_provider == "microsoft":
        app = msal.PublicClientApplication(
            oauth2_client_id, authority=f"https://login.microsoftonline.com/{oauth2_tenant_id}")
        result = app.acquire_token_by_refresh_token(
            refresh_token=oauth2_refresh_token, scopes=["https://outlook.office.com/.default"])
        if "access_token" in result:
            access_token = result["access_token"]
            if "refresh_token" in result:
                refresh_token = result["refresh_token"]
    elif oauth2_provider == "google":
        result = oauth2.RefreshToken(
            oauth2_client_id, oauth2_client_secret, oauth2_refresh_token)
        if "access_token" in result:
            access_token = result["access_token"]
            if "refresh_token" in result:
                refresh_token = result["refresh_token"]
    return (access_token, refresh_token)


def generate_oauth2_string(username, access_token, as_base64=True):
    auth_string = f"user={username}\x01auth=Bearer {access_token}\x01\x01".encode('utf-8')
    if as_base64:
        auth_string = base64.b64encode(auth_string)
        auth_string = auth_string.decode()
    return auth_string
