#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Dieses Modul stellt Internationalisierung mittels gettext für die Python-API und dem bfaServer zur Verfügung"""

import gettext
import locale
import sys
import logging
import os
import win32file

class bfaTranslate(object):
    """Klasse liefert ein erweitertes Verhalten für gettext. Sie muss nur einmal am Anfang einer Application instanziert werden. Wird keine domain mitgegeben,
    wird der Applikationsname angenommen, dh. es sollten gleichnamige .mo Dateien vorhanden sein. Gesucht werden diese immer in .\locale
    nach der Instanzierung ist applikationsweit die Funktion _() implementiert. Diese kann allerdings mit einem zweiten optionalen Parameter genutzt werden,
    hierbei gibt man das Sprachkürzel der gewünschten Sprache an (zB 'en' oder 'fr', also _('Dies bitte übersetzen','en') ).
    Wird nichts angegeben wird in der systemdefault Sprache ausgegeben.
    Damit die erweiterten Übersetzungsstrings auch von pygettext extrahiert werden können, muss die modifizierte Version aus dem .\bin-Verzeichnis genommen werden"""
    def __init__(self, domain = None):
        if domain is None:
            self.domain = os.path.splitext(os.path.basename(sys.argv[0]))[0]

        loc = locale.getdefaultlocale()[0].split('_')[0]

        self.trans = {None: gettext.NullTranslations()}

        mypath = os.path.split(os.path.abspath(sys.argv[0]))[0]
        mypath = win32file.GetLongPathName(mypath)
        try:
            bfapath = os.environ["bitfarm-archiv"]
        except:
            bfapath = mypath

            if mypath.lower() == os.getcwd().lower():
                if mypath[-3:] == 'bin':
                    fpath = os.path.join(mypath, 'locale')
                else:
                    fpath = os.path.join(mypath, 'bin', 'locale')
            else:
                fpath = r'.\locale'
        else:
            fpath = os.path.join(bfapath, 'bin', 'locale')

        locale_dir = 'locale'
        if os.path.exists(fpath):
            locale_dir = fpath

        if os.path.exists(locale_dir):
            dirs = os.listdir(locale_dir)

            for lang in dirs:
                if gettext.find(domain, locale_dir, [lang]):
                    self.trans[lang] = gettext.translation(domain, locale_dir, [lang],  fallback=True)

            if loc in self.trans:
                self.trans[None] = self.trans[loc]

        import builtins
        builtins.__dict__['_'] = lambda x, y = None: self.trans[y].gettext(x) if y in self.trans else x

    def change_language(self, loc):
        if loc in self.trans:
            self.trans[None] = self.trans[loc]


class MyFormatter(logging.Formatter):
    def __init__(self, fmt=None, datefmt=None, encoding='windows-1250'):
        logging.Formatter.__init__(self, fmt, datefmt)
        # self.encoding = encoding
        # 'windows-1250'
    # def formatException(self, ei):
    #     r = logging.Formatter.formatException(self, ei)
    #     if type(r) in [str]:
    #         r = r.decode(self.encoding, 'replace') # Convert to unicode
    #     return r
    #
    # def format(self, record):
    #     t = record
    #     try:
    #         t = logging.Formatter.format(self, record)
    #     except:
    #         return t
    #     # if type(t) in [unicode]:
    #     #     t = t.encode(self.encoding, 'replace')
    #     if type(t) in [str]:
    #         t = t.decode(self.encoding)
    #     return t
