﻿#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Modul zum Parsen von Bitfarm-Daten-Objekten.
Wird benötigt für die Kommunikation mit anderen Applikationen.
Die RequestHandler-Klasse für Version V4 wird hier integriert.
Folgende Bitfarm-Objekte können geparst werden:
    - BitfarmWorkflow
    - BitfarmTask
    - BitfarmParameter
    - BitfarmConditionObject
    - BitfarmConditionSet
    - BitfarmDocument
    - BitfarmAddfield
    - bfaReq
    - BitfarmStream
    - BitfarmUser
    - BitfarmGroup
    - BitfarmBarcode
    - BitfarmNotification
    - BitfarmGetField
    - BitfarmFulltext
    - BitfarmEmail
    - BitfarmAttachment
    - BitfarmAnnotation
    - BitfarmDocClass
    - BitfarmRights
    - BitfarmFieldRight
    - BitfarmFieldRights
    - BitfarmSearch
    - BitfarmArchive
    - BitfarmHistory
    - BitfarmADEntry
    - BitfarmPredictionResult
    - BitframGridConfig
    - BitfarmColumnConfig
"""
# import timeit

import os
# import sys
# import logging
# import logging.config
import datetime
# import time
import io
import base64
import decimal
import codecs
# import json

import xml.etree.cElementTree as eTree  # ich probiere cElementTree

from defusedxml import cElementTree as ET  # ich probiere cElementTree

#import xml.etree.ElementTree as eTree  # ich probiere cElementTree
try:      #wir laden schon mal den XML-Parser, da das dynamische Laden durch etree.fromstring in einem Thread Probleme macht
    from xml.parsers import expat
except ImportError:
    try:
        import pyexpat as expat
    except ImportError:
        raise ImportError("No module named expat; use SimpleXMLTreeBuilder instead")
import re
import html.entities

from collections import OrderedDict
import threading

import bitfarm_doc
import bitfarm_con

lock = threading.Lock()
lock2 = threading.Lock()
lock3 = threading.Lock()
lock4 = threading.Lock()

datetime.datetime.strptime('2013-01-31 14:42:42', '%Y-%m-%d %H:%M:%S')

SHORTXML = True     # beim erzeugen des XML-Strings werden alle Attribute die None sind weggelassen.
                    # Ist SHORTXML gesetzt werden auch False, 0, Leerstring, leere Listen oder Dictionaries weggelassen
                    # wird hinterher ein Objekt anhand des XLS-Strings wieder erzeugt, dann werden diese Attribute
                    # sowieso wieder leer (mit None oder False) initialisiert
BFACLASSES = [bitfarm_doc.BitfarmFulltext, bitfarm_doc.BitfarmDocument,  bitfarm_doc.BitfarmAddfield, bitfarm_doc.bfaReq,
              bitfarm_doc.BitfarmWorkflow, bitfarm_con.BitfarmCon,  bitfarm_con.BitfarmConnections,  bitfarm_doc.BitfarmEmail,
              bitfarm_doc.BitfarmAttachment, bitfarm_doc.BitfarmAnnotation, bitfarm_doc.BitfarmTask, bitfarm_doc.BitfarmParameter,
              bitfarm_doc.BitfarmStream, bitfarm_doc.BitfarmRect, bitfarm_doc.BitfarmHistory, bitfarm_doc.BitfarmConditionObject,
              bitfarm_doc.BitfarmConditionSet, bitfarm_doc.BitfarmDLM, bitfarm_doc.BitfarmUser,  bitfarm_doc.BitfarmGroup,
              bitfarm_doc.BitfarmBarcode, bitfarm_doc.BitfarmSvnField, bitfarm_doc.BitfarmNotification, bitfarm_doc.BitfarmGetField,
              bitfarm_doc.BitfarmDocClass, bitfarm_doc.BitfarmRights, bitfarm_doc.BitfarmSearch, bitfarm_doc.BitfarmStatefield,
              bitfarm_doc.BitfarmArchive, bitfarm_doc.BitfarmFieldRight, bitfarm_doc.BitfarmFieldRights, bitfarm_doc.BitfarmADEntry,
              bitfarm_doc.BitfarmPredictionResult, bitfarm_doc.BitfarmGridConfig, bitfarm_doc.BitfarmColumnConfig,
              'BitfarmCon', 'BitfarmConnections',  'BitfarmFulltext',  'BitfarmEmail', 'BitfarmAttachment', 'BitfarmAnnotation',
              'BitfarmWorkflow', 'BitfarmTask', 'BitfarmParameter', 'BitfarmConditionObject', 'BitfarmConditionSet', 'BitfarmHistory',
              'BitfarmDocument', 'BitfarmAddfield',  'bfaReq', 'BitfarmStream', 'BitfarmRect', 'BitfarmUser', 'BitfarmGroup', 'BitfarmBarcode', 'BitfarmSvnField',
              'BitfarmNotification', 'BitfarmGetField', 'BitfarmDocClass', 'BitfarmRights', 'BitfarmSearch', 'BitfarmStatefield', 'BitfarmArchive',
              'BitfarmFieldRight', 'BitfarmFieldRights', 'BitfarmDLM', 'BitfarmADEntry', 'BitfarmPredictionResult', 'BitfarmGridConfig', 'BitfarmColumnConfig', 'fav']


def _add_element_json(obj):
    if str(type(obj))[8:-2].split('.')[-1] == 'RowProxy':
        obj = list(obj)
    if type(obj) in BFACLASSES:
        result = obj_to_dict(obj)
    
    elif type(obj) in (dict, OrderedDict):
        result = {}
        for elem in obj:
            result[elem] = _add_element_json(obj[elem])
    elif type(obj) in [list, tuple]:
        result = []
        for elem in obj:
            result.append(_add_element_json(elem))
    elif isinstance(obj, io.BytesIO) or isinstance(obj, io.StringIO):
        obj.seek(0)
        result = base64.b64encode(obj.read()).decode()
    elif type(obj) == decimal.Decimal:
        result = obj
    elif type(obj) == datetime.datetime:
        try:
            result = obj.strftime("%Y-%m-%d %H:%M:%S")
        except:
            result = obj.isoformat()
            result = result[:10] + " " + result[11:19]
    else:
        result = obj
    return result
        
        
def obj_to_dict(obj, log=None):
    
    if log is not None:
        log.debug(type(obj))
    if str(type(obj))[8:-2].split('.')[-1] == 'RowProxy':
        obj = list(obj)
    if type(obj) in (tuple, list):
        d = []
        for subobj in obj:
            d.append(_add_element_json(subobj))
        return d
    d = {}
    if isinstance(obj, bitfarm_doc.BitfarmRights):
        d['_BitfarmRights__userid'] = obj.__dict__['_BitfarmRights__userid']
        d['_BitfarmRights__groupid'] = obj.__dict__['_BitfarmRights__groupid']
        d['_BitfarmRights__arcid'] = obj.__dict__['_BitfarmRights__arcid']
        d['_BitfarmRights__gdocid'] = obj.__dict__['_BitfarmRights__gdocid']
        d['_BitfarmRights__rights'] = obj.__dict__['_BitfarmRights__rights']
        if obj.__dict__['_BitfarmRights__user']:
            d['_BitfarmRights__user'] = obj.__dict__['_BitfarmRights__user']
        if obj.__dict__['_BitfarmRights__group']:
            d['_BitfarmRights__group'] = obj.__dict__['_BitfarmRights__group']
        if obj.__dict__['userlist']:
            d['userlist'] = obj.__dict__['userlist']
        if obj.__dict__['grouplist']:
            d['grouplist'] = obj.__dict__['grouplist']
        return d

    if isinstance(obj, bitfarm_doc.BitfarmFieldRight):
        d['_BitfarmFieldRight__userid'] = obj.__dict__['_BitfarmFieldRight__userid']
        d['_BitfarmFieldRight__groupid'] = obj.__dict__['_BitfarmFieldRight__groupid']
        d['_BitfarmFieldRight__arcid'] = obj.__dict__['_BitfarmFieldRight__arcid']
        d['_BitfarmFieldRight__rights'] = obj.__dict__['_BitfarmFieldRight__rights']
        d['_BitfarmFieldRight__fieldpos'] = obj.__dict__['_BitfarmFieldRight__fieldpos']
        if obj.__dict__['_BitfarmFieldRight__gdocid']:
            d['_BitfarmFieldRight__gdocid'] = obj.__dict__['_BitfarmFieldRight__gdocid']
        if obj.__dict__['_BitfarmFieldRight__user']:
            d['_BitfarmFieldRight__user'] = obj.__dict__['_BitfarmFieldRight____user']
        if obj.__dict__['_BitfarmFieldRight__group']:
            d['_BitfarmFieldRight__group'] = obj.__dict__['_BitfarmFieldRight__group']
        return d

    for attr in obj.__dict__:
        if obj.__dict__[attr] is not None:
            # if isinstance(obj, bitfarm_doc.BitfarmRights):
            #     if type(obj.__dict__[attr]) is bool:
            #         continue    #beim BitfarmRights-Objekt brauchen wir die ganzen Booleans nicht
            if type(obj.__dict__[attr]) is bool and obj.__dict__[attr] is False:
                continue
            if type(obj.__dict__[attr]) is str and obj.__dict__[attr] == '':
                continue
            #if type(obj.__dict__[attr]) in [int, long, float] and obj.__dict__[attr] == 0:
                #continue
            #if type(obj.__dict__[attr]) in [list, dict] and len(obj.__dict__[attr])== 0:
                #continue
            if attr in d:
                if type(d[attr]) is not list:
                    d[attr] = [d[attr]].extend(_add_element_json(obj.__dict__[attr]))
            else:
                d[attr] = _add_element_json(obj.__dict__[attr])
    return d


def xml_to_object(xmlstr):
    """Funktion zum Parsen eines XML-Strings mit Bitfarm-Objekten. Die Objekte werden dabei erzeugt und zurückgegeben.
    Wenn der XML-String eine Liste von Bitfarm-Objekten (unterhalb eines <objlist> Tags) enthält werden diese in einer Liste zurückgegeben.
    """
    if xmlstr is None or xmlstr == '':
        return None

    if type(xmlstr) is bytes:
        xmlstr = xmlstr.decode()

    if type(xmlstr) is not str:
        return None

    xmlstr = _parse_nonprintable(xmlstr)

    encoding = re.match("<\?xml\s\S*\sencoding='(\S*)'\?>", xmlstr)
    if encoding:
        encoding = encoding.group(1)
    else:
        encoding = 'us-ascii'
    lock.acquire()
    try:
        tree = eTree.fromstring(xmlstr)

#        if tree.tag == 'objlist':
#            _ret =[]
#            for elem in tree.getchildren():
#                obj = _create_elem(elem, encoding)
#                if obj is not None:
#                    obj = _set_class_attributes(obj, elem, encoding)
#                    _ret.append(obj)
#        else:
#            obj = _create_elem(tree, encoding)
#            if obj is not None:
#                obj = _set_class_attributes(obj, tree, encoding)
#            _ret = obj

        obj = _create_elem(tree, encoding)
        if obj is not None:
            obj = _set_class_attributes(obj, tree, encoding)
        _ret = obj    
    
    finally:
        lock.release()
    del tree
    return _ret


def fromstring(xmlstr):
    """Funktion zum Parsen eines XML-Strings, gibt einen eTree.tree zurück.
    """
    if xmlstr is None or xmlstr == '':
        return None

    if type(xmlstr) is not str:
        return None

    # encoding = re.match("<\?xml\s\S*\sencoding='(\S*)'\?>", xmlstr)
    # if encoding:
    #     encoding = encoding.group(1)
    # else:
    #     encoding = 'us-ascii'
    lock3.acquire()
    try:
        tree = eTree.fromstring(xmlstr)

    finally:
        lock3.release()

    return tree


def tostring(tree, encoding='us-ascii'):
    """Methode zum Parsen von eine eTree.tree zu einem XML-String. Optional kann das encoding angegeben werden,
    das Standard-Encoding ist 'us-ascii'. Encodings müssen standardkonform angegeben werden, also 'utf-8' oder 'iso-5589-1'
    siehe http://www.iana.org/assignments/character-sets
    Es kann auch eine Liste von Objekten übergeben werden.
    Es wird ein XML-String zurückgegeben.
    """
    with lock4:
        _ret = None
        if tree is not None:
            if encoding == 'us-ascii':
                _ret = "<?xml version='1.0' encoding='%s'?>" % encoding + eTree.tostring(tree, encoding=encoding).decode()
            else:
                _ret = eTree.tostring(tree, encoding=encoding).decode()
    return _ret


def _create_elem(elem, _enc):
    _ret = None
    itemtyp = (elem.attrib['typ'] if 'typ' in elem.attrib else elem.tag)

    if itemtyp in BFACLASSES:
        if itemtyp == 'BitfarmWorkflow':
            _ret = bitfarm_doc.BitfarmWorkflow()
        elif itemtyp == 'BitfarmTask':
            _ret = bitfarm_doc.BitfarmTask()
        elif itemtyp == 'BitfarmParameter':
            _ret = bitfarm_doc.BitfarmParameter()
        elif itemtyp == 'BitfarmDocument':
            _ret = bitfarm_doc.BitfarmDocument()
        elif itemtyp == 'BitfarmAddfield':
            _ret = bitfarm_doc.BitfarmAddfield()
        elif itemtyp == 'BitfarmStatefield':
            _ret = bitfarm_doc.BitfarmStatefield()
        elif itemtyp == 'bfaReq':
            _ret = bitfarm_doc.bfaReq()
        elif itemtyp == 'BitfarmStream':
            _ret = bitfarm_doc.BitfarmStream()
        elif itemtyp == 'BitfarmConditionSet':
            _ret = bitfarm_doc.BitfarmConditionSet()
        elif itemtyp == 'BitfarmConditionObject':
            _ret = bitfarm_doc.BitfarmConditionObject()
        elif itemtyp == 'BitfarmGroup':
            _ret = bitfarm_doc.BitfarmGroup()
        elif itemtyp == 'BitfarmUser':
            _ret = bitfarm_doc.BitfarmUser()
        elif itemtyp == 'BitfarmGetField':
            _ret = bitfarm_doc.BitfarmGetField()
        elif itemtyp == 'BitfarmBarcode':
            _ret = bitfarm_doc.BitfarmBarcode()
        elif itemtyp == 'BitfarmNotification':
            _ret = bitfarm_doc.BitfarmNotification()
        elif itemtyp == 'BitfarmConnections':
            _ret = bitfarm_con.BitfarmConnections()
        elif itemtyp == 'BitfarmCon':
            _ret = bitfarm_con.BitfarmCon()
        elif itemtyp == 'BitfarmFulltext':
            _ret = bitfarm_doc.BitfarmFulltext()
        elif itemtyp == 'BitfarmEmail':
            _ret = bitfarm_doc.BitfarmEmail()
        elif itemtyp == 'BitfarmAttachment':
            _ret = bitfarm_doc.BitfarmAttachment()
        elif itemtyp == 'BitfarmAnnotation':
            _ret = bitfarm_doc.BitfarmAnnotation()
        elif itemtyp == 'BitfarmRect':
            _ret = bitfarm_doc.BitfarmRect()
        elif itemtyp == 'BitfarmDocClass':
            _ret = bitfarm_doc.BitfarmDocClass()
        elif itemtyp == 'BitfarmRights':
            _ret = bitfarm_doc.BitfarmRights()
        elif itemtyp == 'BitfarmSearch':
            _ret = bitfarm_doc.BitfarmSearch()
        elif itemtyp == 'bookmark':
            _ret = []
        elif itemtyp == 'fav':
            _ret = bitfarm_doc.BitfarmSearch()
        elif itemtyp == 'BitfarmArchive':
            _ret = bitfarm_doc.BitfarmArchive()
        elif itemtyp == 'BitfarmFieldRight':
            _ret = bitfarm_doc.BitfarmFieldRight()
        elif itemtyp == 'BitfarmFieldRights':
            _ret = bitfarm_doc.BitfarmFieldRights()            
        elif itemtyp == 'BitfarmHistory':
            _ret = bitfarm_doc.BitfarmHistory()
        elif itemtyp == 'BitfarmSvnField':
            _ret = bitfarm_doc.BitfarmSvnField()
        elif itemtyp == 'BitfarmDLM':
            _ret = bitfarm_doc.BitfarmDLM()
        elif itemtyp == 'BitfarmADEntry':
            _ret = bitfarm_doc.BitfarmADEntry()
        elif itemtyp == 'BitfarmPredictionResult':
            _ret = bitfarm_doc.BitfarmPredictionResult()
        elif itemtyp == 'BitfarmGridConfig':
            _ret = bitfarm_doc.BitfarmGridConfig()
        elif itemtyp == 'BitfarmColumnConfig':
            _ret = bitfarm_doc.BitfarmColumnConfig()
    elif itemtyp in [list, 'list', 'bookmark']:
        _ret = []
    elif itemtyp in [set,  'set']:
        _ret = set()
    elif itemtyp in [frozenset, 'frozenset']:
        _ret = frozenset()
    elif itemtyp in [dict, 'dict']:
        _ret = {}
    elif itemtyp in [tuple, 'tuple']:
        _ret = ()
    elif itemtyp in [OrderedDict, 'OrderedDict',  'collections.OrderedDict']:
        _ret = OrderedDict()
    else:
        if 'typ' in elem.attrib:
            _ret = _create_value(elem.text, elem.attrib['typ'], _enc)
    return _ret


def _create_value(value, typ, _enc):
    _ret = None
    if value is not None:
        if typ == 'datetime.datetime' or typ == 'datetime':
            try:
                _ret = datetime.datetime.strptime(value, '%Y-%m-%d %H:%M:%S')
            except:
                _ret = None
        elif typ == 'bool':
            _ret = (True if value == 'True' else False)
        elif typ == 'unicode':
            if type(value) is str:
                _ret = value
                # if 'ascii' in _enc and '<?xml version="1.0"?>' not in value:
                #     _ret = _unescape(value)
                #     if type(_ret) == str:
                #         _ret = _ret.decode(_enc)
                # else:
                #     _ret = value.decode(_enc)
            else:
                _ret = value
        elif typ == 'base64':
            _ret = io.BytesIO()
            _ret.write(base64.b64decode(value))
        elif typ == 'str':
            _ret = str(value)
        elif typ == 'Decimal':
            _ret = decimal.Decimal(value)
        elif typ == 'int':
            _ret = int(value)
        elif typ == 'float':
            _ret = float(value)
        elif typ == 'long':
            _ret = int(value)   # long ist jetzt int :-) (in Python3)
        elif typ == 'NoneType':
            _ret = None
        else:
            try:
                _ret = eval("%s('%s')" % (typ, value))
            except:
                _ret = None
    return _ret


def _set_class_attributes(obj, elem, _enc):
    tmp = []
    itemtyp = None
    key = None
    for child in elem:
        itemtyp = (elem.attrib['typ'] if 'typ' in elem.attrib else elem.tag)

        if itemtyp in [dict, 'dict',  OrderedDict, 'OrderedDict', 'collections.OrderedDict']:
            if 'tagtyp' in child.attrib:
                key = _create_value(child.attrib['name'], child.attrib['tagtyp'], _enc)
            else:
                key = child.tag
            obj[key] = _create_elem(child, _enc)
        elif itemtyp in [list, 'list', 'bookmark']:
            obj.append(_create_elem(child, _enc))
        elif itemtyp in [set, frozenset, 'set', 'frozenset']:
            tmp.append(_create_elem(child, _enc))
        elif itemtyp in [tuple, 'tuple']:
            tmp.append(_create_elem(child, _enc))
        elif itemtyp in BFACLASSES:
            if isinstance(obj, bitfarm_doc.BitfarmRights) and child.tag == '_BitfarmRights__rights':   #Behandlung für verkürzte BitfarmRights Objekte (SHORTXML)
                obj.rights = _create_elem(child, _enc)
            elif isinstance(obj, bitfarm_doc.BitfarmFieldRight) and child.tag == '_BitfarmFieldRight__rights':   #Behandlung für verkürzte BitfarmFieldRight Objekte (SHORTXML)
                obj.rights = _create_elem(child, _enc)
            else:
                obj.__dict__[child.tag] = _create_elem(child, _enc)

        # if len(list(child)) > 0:    # wenn ein element mehrere unterlemente hat, dann rekursiv
        if len([x for x in child.iter()]) > 0:
            if itemtyp in [dict, 'dict',  OrderedDict, 'OrderedDict', 'collections.OrderedDict']:
                obj[key] = _set_class_attributes(obj[key], child, _enc)
            elif itemtyp in [list, 'list', 'bookmark']:
                obj[-1] = _set_class_attributes(obj[-1], child, _enc)
            elif itemtyp in [set, frozenset, 'set', 'frozenset']:
                tmp[-1] = _set_class_attributes(tmp[-1], child, _enc)
            elif itemtyp in [tuple, 'tuple']:
                tmp[-1] = _set_class_attributes(tmp[-1], child, _enc)
            elif itemtyp in BFACLASSES:
                obj.__dict__[child.tag] = _set_class_attributes(obj.__dict__[child.tag], child, _enc)

    if itemtyp in [tuple, 'tuple']:
        obj = tuple(tmp)
    if itemtyp in [set, 'set']:
        obj = set(tmp)
    if itemtyp in [frozenset, 'frozenset']:
        obj = frozenset(tmp)

    return obj

# Removes HTML or XML character references and entities from a text string.
#
# @param text The HTML (or XML) source text.
# @return The plain text, as a Unicode string, if necessary.


def _unescape(val):
    def fixup(m):
        value = m.group(0)
        if value[:2] == "&#":
            # character reference
            try:
                if value[:3] == "&#x":
                    return chr(int(value[3:-1], 16))
                else:
                    return chr(int(value[2:-1]))
            except ValueError:
                pass
        else:
            # named entity
            try:
                value = chr(html.entities.entitydefs.name2codepoint[value[1:-1]])
            except KeyError:
                pass
        return value    # leave as is
    return re.sub("&#?\w+;", fixup, val)


def _parse_nonprintable(string):
    try:
        # if type(string) is str:
        #     string = re.sub(r'[\x00-\x08\x0b-\x1f\x7f]', '', string)    # ascii steuerzeichen parsen
        # elif type(string) is unicode:
        #     string = re.sub(r'[\x00-\x08\x0b-\x1f\x7f-\x9f]', '', string)   # unicode steuerzeichen parsen
        if type(string) is str:
            string = re.sub(r'[\x00-\x08\x0b-\x1f\x7f-\x9f]', '', string)   # unicode steuerzeichen parsen
    finally:
        return string


def object_to_xml(obj, encoding='us-ascii', shortxml=SHORTXML, with_lock=True):
    """Methode zum Parsen von Bitfarm-Objekten zu einem XML-String. Optional kann das encoding angegeben werden,
    das Standard-Encoding ist 'us-ascii'. Encodings müssen standardkonform angegeben werden, also 'utf-8' oder 'iso-5589-1'
    siehe http://www.iana.org/assignments/character-sets
    Es kann auch eine Liste von Objekten übergeben werden.
    Es wird ein XML-String zurückgegeben.
    """
    if with_lock:
        with lock2:
            _ret = None
            root = _object_to_xml(obj, shortxml)
            if root is not None:
                if encoding == 'us-ascii':
                    _ret = '<?xml version="1.0" encoding="%s"?>' % encoding + eTree.tostring(root, encoding=encoding).decode()
                else:
                    _ret = '<?xml version="1.0"?>' + eTree.tostring(root, encoding=encoding).decode()

        del root
        _ret = _parse_nonprintable(_ret)
    else:
        _ret = None
        root = _object_to_xml(obj, shortxml)
        if root is not None:
            if encoding == 'us-ascii':
                _ret = '<?xml version="1.0" encoding="%s"?>' % encoding + eTree.tostring(root, encoding=encoding).decode()
            else:
                _ret = '<?xml version="1.0"?>' + eTree.tostring(root, encoding=encoding).decode()

        del root
        _ret = _parse_nonprintable(_ret)
    return _ret


def object_to_xmlfile(obj, filename, encoding='us-ascii', shortxml=SHORTXML):
    """Methode zum Parsen von Bitfarm-Objekten zu einem XML-String und Speicherung in eine Datei.
    Es kann eine Dateiname oder ein zum Schreiben geöffnetes File-Object übergeben werden.
    Optional kann das encoding angegeben werden, das Standard-Encoding ist 'us-ascii'.
    Encodings müssen standardkonform angegeben werden, also 'utf-8' oder 'iso-5589-1'
    Es kann auch eine Liste von Objekten übergeben werden.
    Über den optionalen Parameter kann man auch SHORTXML beeinflussen (default = True)
    Es wird ein XML-String zurückgegeben.
    """
    with lock2:
        _ret = None
        root = _object_to_xml(obj, shortxml)
        if root is not None:
            #tree = eTree.ElementTree(root)
            #tree.write(filename, encoding=encoding,  xml_declaration=True)
            _ret = "<?xml version='1.0' encoding='%s'?>" % encoding + eTree.tostring(root, encoding=encoding).decode()
            if type(filename) is str:
                with open(filename,'w') as f:
                    f.write(_ret)
            else:
                filename.write(_ret)
        del root
        _ret = _parse_nonprintable(_ret)
    return _ret


def _object_to_xml(obj, shortxml):
    found = False

    if type(obj) == list:
        root = eTree.Element('objlist')
        root.attrib['typ'] = 'list'
        for elem in obj:
            if type(elem) in BFACLASSES:
                found = True
                listnode = _add_element('item', elem, shortxml)
                root.append(listnode)
            if str(elem.__class__)[8:-2].split('.')[-1] == 'RowProxy':
                found = True
                elem = tuple(elem)
                listnode = _add_element('tuple', elem, shortxml)
                root.append(listnode)
                
        if not found:
            return None
    else:
        if type(obj) in BFACLASSES:
            root = eTree.Element(str(obj.__class__)[8:-2].split('.')[1])
            _get_class_attributes(root, obj, shortxml)
        else:
            return None

    return root


def _get_class_attributes(root, obj, shortxml):
    for attr in obj.__dict__:
        if obj.__dict__[attr] is not None:
            if shortxml:
                if isinstance(obj, bitfarm_doc.BitfarmRights) or isinstance(obj, bitfarm_doc.BitfarmFieldRight):
                # if isinstance(obj, bitfarm_doc.BitfarmFieldRight):
                    if attr[:16] == '_BitfarmRights__' or attr[:20] == '_BitfarmFieldRight__':
                    # if attr[:20] == '_BitfarmFieldRight__':
                        if type(obj.__dict__[attr]) is bool:
                            continue    #beim BitfarmRights-Objekt brauchen wir die ganzen Booleans nicht
                if type(obj.__dict__[attr]) is bool and obj.__dict__[attr] is False:
                    if not isinstance(obj, bitfarm_doc.BitfarmStatefield):    # bei Statefields brauchen wir die Booleans
                        continue
                if type(obj.__dict__[attr]) is str and obj.__dict__[attr] == '':
                    continue
                # in vielen Fällen brauchen wir die 0 als Information (weil viele Indexe bei 0 beginnen)
                # if type(obj.__dict__[attr]) in [int, float] and obj.__dict__[attr] == 0:
                #     if not isinstance(obj, bitfarm_doc.BitfarmAddfield):    # bei Addfields brauchen wir die 0
                #         continue
                if type(obj.__dict__[attr]) in [list, dict] and len(obj.__dict__[attr]) == 0:
                    continue

            tag = _add_element(attr, obj.__dict__[attr], shortxml)
            if tag is not None:
                root.append(tag)


def _add_element(elem, value, shortxml):
    value_type = type(value)
    elem_type = type(elem)
    if shortxml is True:
        if value is None:
            return None
        if value_type is list and len(value) == 0:
            return None
        if value_type is str and value == '':
            return None

    # Tuple als Key eines dict (einzig denkbare möglichkeit wie hier ein element tuple hingeraten könnte) wird nicht unterstützt
    if elem_type is tuple:
        return None

    if elem_type is not str:
        elem = str(elem)

    tag = eTree.Element(elem)
    if 'typ' not in tag.attrib:
        tag.attrib["typ"] = str(type(value))[8:-2] if str(type(value))[1:6] == 'class' else str(type(value))[7:-2] if str(type(value))[1:4] == 'type' else ''
        if '.' in tag.attrib["typ"]:
            tag.attrib["typ"] = tag.attrib["typ"].split('.')[1]
    if value_type in [list, tuple]:
        for item in value:
            if str(type(item))[8:-2].split('.')[-1] == 'RowProxy':
                item = tuple(item)
            
            listtag = _add_element('item', item, shortxml)
            if listtag is not None:
                tag.append(listtag)
    elif value_type in [set, frozenset]:
        for item in value:
            settag = _add_element('item', item, shortxml)
            if settag is not None:
                tag.append(settag)
    elif value_type in [dict,  OrderedDict]:
        for key in value:
            # tuple als key wird nicht unterstützt
            if type(key) == tuple:
                continue
            if value_type is dict:
                tag.attrib['typ'] = 'dict'
            else:
                tag.attrib['typ'] = 'OrderedDict'

            dicttag = _add_element('item', value[key], shortxml)
            if dicttag is not None:
                if 'tagtyp' not in dicttag.attrib:
                    dicttag.attrib['tagtyp'] = str(type(key))[7:-2] if str(type(key))[:5] == '<type' else str(type(key))[8:-2] if str(type(key))[:6] == '<class' else str(type(key))[8:-2].split('.')[1]
                if type(key) is str:
                    dicttag.attrib['name'] = key
                else:
                    dicttag.attrib['name'] = str(key)
                tag.append(dicttag)
    elif str(value_type) == "<type 'cStringIO.StringO'>":
        tag.attrib['typ'] = 'base64'
        tag.text = base64.b64encode(value.getvalue()).decode()
    elif value_type in [io.BytesIO, io.StringIO]:
        tag.attrib['typ'] = 'base64'
        tag.text = base64.b64encode(value.getvalue()).decode()
    elif value_type in BFACLASSES:
        _get_class_attributes(tag, value, shortxml)
    elif value_type is decimal.Decimal:
        tag.text = str(value)
    elif value_type is int:
        tag.text = str(value)
        tag.attrib['typ'] = 'int'
    elif value_type is bool:
        tag.text = str(value)
    else:
        if value_type is str:
            tag.text = value
        else:
            tag.text = str(value)
        # tag.attrib['typ'] = 'str'

    return tag


def copy_bitfarmobj_xml(_obj):
    _ret = None
    _new = None
    if type(_obj) is list:
        _ret = []
        for _elem in _obj:
            xml = object_to_xml(_elem)
            _new = xml_to_object(xml)
            if _new is not None:
                _ret.append(_new)
        if len(_ret) == 0:
            return None

    elif type(_obj) in BFACLASSES:
        xml = object_to_xml(_obj)
        _ret = xml_to_object(xml)
    return _ret


def copy_bitfarmobj(_obj):
    """Kopiert ein BitfarmObjekt und gibt dieses zurück. Die Kopie inkl. aller Unterobjekte ist komplett neu instanziert, somit wird eine
    unabhängige, echte Kopie erstellt."""
    _ret = None
    _new = None
    if type(_obj) is list:
        _ret = []
        for _elem in _obj:
            _new = _copy_obj(_elem)
            if _new is not None:
                _ret.append(_new)
        if len(_ret) == 0:
            return None

    elif type(_obj) in BFACLASSES:
        _ret = _copy_obj(_obj)
    return _ret


def _copy_obj(_obj):
    _ret = _create_obj(_obj)
    if _ret is None:
        return None
    for _elem in _obj.__dict__:
        _ret.__dict__[_elem] = _get_value(_obj.__dict__[_elem])

    return _ret


def _get_value(_elem):
    if type(_elem) in BFACLASSES:
        return _copy_obj(_elem)
    elif type(_elem) == list:
        _list = []
        for _item in _elem:
            if type(_item) in BFACLASSES:
                _list.append(_copy_obj(_item))
            else:
                _list.append(_get_value(_item))
        return _list
    elif type(_elem) == dict:
        _dict = {}
        for _item in _elem:
            if type(_elem[_item]) in BFACLASSES:
                _dict[_item] = _copy_obj(_elem[_item])
            else:
                _dict[_item] = _get_value(_item)
        return _dict
    else:
        return _elem


def _create_obj(_obj):
    _ret = None
    itemtyp = type(_obj)
    if itemtyp in BFACLASSES:
        if itemtyp is bitfarm_doc.BitfarmWorkflow:
            _ret = bitfarm_doc.BitfarmWorkflow()
        elif itemtyp is bitfarm_doc.BitfarmTask:
            _ret = bitfarm_doc.BitfarmTask()
        elif itemtyp is bitfarm_doc.BitfarmParameter:
            _ret = bitfarm_doc.BitfarmParameter()
        elif itemtyp is bitfarm_doc.BitfarmDocument:
            _ret = bitfarm_doc.BitfarmDocument()
        elif itemtyp is bitfarm_doc.BitfarmAddfield:
            _ret = bitfarm_doc.BitfarmAddfield()
        elif itemtyp is bitfarm_doc.BitfarmStatefield:
            _ret = bitfarm_doc.BitfarmStatefield()
        elif itemtyp is bitfarm_doc.bfaReq:
            _ret = bitfarm_doc.bfaReq()
        elif itemtyp is bitfarm_doc.BitfarmStream:
            _ret = bitfarm_doc.BitfarmStream()
        elif itemtyp is bitfarm_doc.BitfarmConditionSet:
            _ret = bitfarm_doc.BitfarmConditionSet()
        elif itemtyp is bitfarm_doc.BitfarmConditionObject:
            _ret = bitfarm_doc.BitfarmConditionObject()
        elif itemtyp is bitfarm_doc.BitfarmGroup:
            _ret = bitfarm_doc.BitfarmGroup()
        elif itemtyp is bitfarm_doc.BitfarmUser:
            _ret = bitfarm_doc.BitfarmUser()
        elif itemtyp is bitfarm_doc.BitfarmGetField:
            _ret = bitfarm_doc.BitfarmGetField()
        elif itemtyp is bitfarm_doc.BitfarmBarcode:
            _ret = bitfarm_doc.BitfarmBarcode()
        elif itemtyp is bitfarm_doc.BitfarmNotification:
            _ret = bitfarm_doc.BitfarmNotification()
        elif itemtyp is bitfarm_con.BitfarmConnections:
            _ret = bitfarm_con.BitfarmConnections()
        elif itemtyp is bitfarm_con.BitfarmCon:
            _ret = bitfarm_con.BitfarmCon()
        elif itemtyp is bitfarm_doc.BitfarmFulltext:
            _ret = bitfarm_doc.BitfarmFulltext()
        elif itemtyp is bitfarm_doc.BitfarmEmail:
            _ret = bitfarm_doc.BitfarmEmail()
        elif itemtyp is bitfarm_doc.BitfarmAttachment:
            _ret = bitfarm_doc.BitfarmAttachment()
        elif itemtyp is bitfarm_doc.BitfarmAnnotation:
            _ret = bitfarm_doc.BitfarmAnnotation()
        elif itemtyp is bitfarm_doc.BitfarmRect:
            _ret = bitfarm_doc.BitfarmRect()
        elif itemtyp is bitfarm_doc.BitfarmDocClass:
            _ret = bitfarm_doc.BitfarmDocClass()
        elif itemtyp is bitfarm_doc.BitfarmRights:
            _ret = bitfarm_doc.BitfarmRights()
        elif itemtyp is bitfarm_doc.BitfarmSearch:
            _ret = bitfarm_doc.BitfarmSearch()
        elif itemtyp is bitfarm_doc.BitfarmArchive:
            _ret = bitfarm_doc.BitfarmArchive()
        elif itemtyp is bitfarm_doc.BitfarmFieldRight:
            _ret = bitfarm_doc.BitfarmFieldRight()
        elif itemtyp is bitfarm_doc.BitfarmFieldRights:
            _ret = bitfarm_doc.BitfarmFieldRights()
        elif itemtyp is bitfarm_doc.BitfarmHistory:
            _ret = bitfarm_doc.BitfarmHistory()
        elif itemtyp is bitfarm_doc.BitfarmSvnField:
            _ret = bitfarm_doc.BitfarmSvnField()
        elif itemtyp is bitfarm_doc.BitfarmDLM:
            _ret = bitfarm_doc.BitfarmDLM()
        elif itemtyp is bitfarm_doc.BitfarmADEntry:
            _ret = bitfarm_doc.BitfarmADEntry()
        elif itemtyp is bitfarm_doc.BitfarmPredictionResult:
            _ret = bitfarm_doc.BitfarmPredictionResult()
        elif itemtyp is bitfarm_doc.BitfarmGridConfig:
            _ret = bitfarm_doc.BitfarmGridConfig()
        elif itemtyp is bitfarm_doc.BitfarmColumnConfig:
            _ret = bitfarm_doc.BitfarmColumnConfig()
    return _ret


def create_request(reqtype="", profile=None, userid=0, arcid=None, data=None, datatype=None, datalen=None, gdocid=None, filename=None, filehash=None, filedate=None, parameter=None, lang=None, tim_id=None, in_editmode=None, selected_cols=None, shortxml=SHORTXML, fields_arcid=None, errortext=None, revision=None, sessionid=None):
    req = bitfarm_doc.bfaReq()
    req.reqtype = reqtype
    req.userid = userid
    if profile:
        req.profile = profile
    if arcid:
        req.arcid = arcid        
    if gdocid:
        req.gdocid = gdocid
    if data:
        req.data = data                       
    if datatype:
        req.datatype = datatype
    if datalen:
        req.datalen = datalen
    if filename:
        req.filename = filename
    if filehash:
        req.filehash = filehash
    if filedate:
        req.filedate = filedate
    if parameter:
        req.parameter = parameter
    if lang:
        req.lang = lang
    if tim_id:
        req.tim_id = tim_id
    if in_editmode:
        req.in_editmode = in_editmode
    if selected_cols:
        req.selected_cols = selected_cols
    if fields_arcid:
        req.fields_arcid = fields_arcid
    if errortext:
        req.errortext = errortext
    if revision:
        req.revision = revision
    if sessionid:
        req.sessionid = sessionid
    return req


def create_xml_request(reqtype="", profile=None, userid=0, arcid=None, data=None, datatype=None, datalen=None, gdocid=None, filename=None, filehash=None, filedate=None, parameter=None, lang=None, tim_id=None, in_editmode=None, selected_cols=None, shortxml=SHORTXML, fields_arcid=None, errortext=None, revision=None, sessionid=None):
    req = create_request(reqtype=reqtype, profile=profile, userid=userid, arcid=arcid, data=data, datatype=datatype, datalen=datalen, gdocid=gdocid, filename=filename, filehash=filehash, filedate=filedate, parameter=parameter, lang=lang, tim_id=tim_id, in_editmode=in_editmode, selected_cols=selected_cols, shortxml=shortxml, fields_arcid=fields_arcid, errortext=errortext, revision=revision, sessionid=sessionid)
    _xml = object_to_xml(req, shortxml=shortxml)
    req = None
    return _xml.encode()

class OcrXmlParser():
    def __init__(self):
        self.pages = {}
        self.xmldom = None
        self.t = None

    def remove_ns(self, xmlfilepath):
        f = codecs.open(xmlfilepath, 'r', 'utf-16')
        s = f.read()
        f.close()
        if '<document xmlns="http://www.scansoft.com/omnipage/xml/ssdoc-schema3.xsd" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">' in s:
            f = codecs.open(xmlfilepath, 'w', 'utf-16')
            s = s.replace('<document xmlns="http://www.scansoft.com/omnipage/xml/ssdoc-schema3.xsd" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">', '<document>')
            f.write(s)
            f.close()
        else:
            f.close()

    def readfile(self, xmlfilepath, removens=False, resolution=None):
        self.xmlfilepath = xmlfilepath
        self.t = None
        if os.path.isfile(xmlfilepath):
            if removens is True:
                try:
                    self.remove_ns(xmlfilepath)
                except:
                    return False
            try:
                self.t = ET.parse(xmlfilepath)
            except:
                return False
        else:
            try:
                self.t = ET.fromstring(xmlfilepath.encode("utf-16"))
            except:
                return False
        if self.t is not None:
            pages = self.t.findall("{http://www.scansoft.com/omnipage/xml/ssdoc-schema3.xsd}page")
            if len(pages) == 0:
                pages = self.t.findall("page")
            words = []
            counter = 0
            for page in pages:
                try:
                    (width, height) = resolution[counter]
                except:
                    width = height = None
                counter += 1
                self.pages[counter] = {}
                page_iter = page.iter()
                self.pages[counter]['sections'] = []
                self.pages[counter]['orientation'] = "0"
                self.pages[counter]['columns'] = []
                self.pages[counter]['tables'] = []
                self.pages[counter]['paras'] = []
                self.pages[counter]['lns'] = []
                self.pages[counter]['words'] = {}
                self.pages[counter]['size'] = None
                self.pages[counter]['dpi'] = None
                self.pages[counter]['theoreticalsize'] = None
                self.pages[counter]['factor'] = None

                for element in page_iter:
                    if element.tag.endswith("source"):
                        if width is not None and height is not None:
                            self.pages[counter]['size'] = [width, height]
                        else:
                            try:
                                self.pages[counter]['size'] = [int(element.attrib["sizex"]), int(element.attrib["sizey"])]
                            except:
                                pass
                        try:
                            self.pages[counter]['dpi'] = [int(element.attrib["dpix"]), int(element.attrib["dpiy"])]
                        except:
                            pass
                        try:
                            self.pages[counter]['orientation'] = element.attrib["orientation"]
                        except:
                            pass
                    elif element.tag.endswith("theoreticalPage"):
                        try:
                            if "offsetX" in element.attrib:
                                offsetx = int(element.attrib['offsetX'])
                            else:
                                offsetx = 0
                            if "offsetY" in element.attrib:
                                offsety = int(element.attrib['offsetY'])
                            else:
                                offsety = 0
                            
                            if self.pages[counter]['orientation'] in ("90", "270"):
                                self.pages[counter]['theoreticalsize'] = [int(element.attrib['height'])-offsety, int(element.attrib['width'])-offsetx]
                                self.pages[counter]['factor'] = [float(self.pages[counter]['size'][1])/float(self.pages[counter]['theoreticalsize'][1]), float(self.pages[counter]['size'][0])/float(self.pages[counter]['theoreticalsize'][0])]
                            else:
                                self.pages[counter]['theoreticalsize'] = [int(element.attrib['width'])-offsetx, int(element.attrib['height'])-offsety]
                                self.pages[counter]['factor'] = [float(self.pages[counter]['size'][0])/float(self.pages[counter]['theoreticalsize'][0]), float(self.pages[counter]['size'][1])/float(self.pages[counter]['theoreticalsize'][1])]
                        except:
                            pass
                    elif element.tag.endswith("wd"):
                        text = element.text
                        
                        if text is None or text == "":
                            text = "".join([el.text for el in list(element)])
                        if text is None:
                            continue
                        elif type(text) is str:
                            text = text.strip()
                            if len(text) == 0:
                                continue

                        if text not in self.pages[counter]['words']:
                            self.pages[counter]['words'][text] = []
                        x1 = int(float(element.attrib["l"])*self.pages[counter]['factor'][0])
                        y1 = int(float(element.attrib["t"])*self.pages[counter]['factor'][1])
                        x2 = int(float(element.attrib["r"])*self.pages[counter]['factor'][0])
                        y2 = int(float(element.attrib["b"])*self.pages[counter]['factor'][1])
                        position = self.calcrepos([(x1, y1), (x2, y2)], self.pages[counter]['orientation'], self.pages[counter]['size'][0], self.pages[counter]['size'][1])
                        self.pages[counter]['words'][text].append(position)
                    elif element.tag.endswith("section"):
                        x1 = int(float(element.attrib["l"])*self.pages[counter]['factor'][0])
                        y1 = int(float(element.attrib["t"])*self.pages[counter]['factor'][1])
                        x2 = int(float(element.attrib["r"])*self.pages[counter]['factor'][0])
                        y2 = int(float(element.attrib["b"])*self.pages[counter]['factor'][1])
                        self.pages[counter]['sections'].append([(x1, y1), (x2, y2)])
                    elif element.tag.endswith("column"):
                        x1 = int(float(element.attrib["l"])*self.pages[counter]['factor'][0])
                        y1 = int(float(element.attrib["t"])*self.pages[counter]['factor'][1])
                        x2 = int(float(element.attrib["r"])*self.pages[counter]['factor'][0])
                        y2 = int(float(element.attrib["b"])*self.pages[counter]['factor'][1])
                        self.pages[counter]['columns'].append([(x1, y1), (x2, y2)])
                    elif element.tag.endswith("table"):
                        x1 = int(float(element.attrib["l"])*self.pages[counter]['factor'][0])
                        y1 = int(float(element.attrib["t"])*self.pages[counter]['factor'][1])
                        x2 = int(float(element.attrib["r"])*self.pages[counter]['factor'][0])
                        y2 = int(float(element.attrib["b"])*self.pages[counter]['factor'][1])
                        self.pages[counter]['tables'].append([(x1, y1), (x2, y2)])
                    elif element.tag.endswith("para"):
                        x1 = int(float(element.attrib["l"])*self.pages[counter]['factor'][0])
                        y1 = int(float(element.attrib["t"])*self.pages[counter]['factor'][1])
                        x2 = int(float(element.attrib["r"])*self.pages[counter]['factor'][0])
                        y2 = int(float(element.attrib["b"])*self.pages[counter]['factor'][1])
                        self.pages[counter]['paras'].append([(x1, y1), (x2, y2)])
                    elif element.tag.endswith("ln"):
                        x1 = int(float(element.attrib["l"])*self.pages[counter]['factor'][0])
                        y1 = int(float(element.attrib["t"])*self.pages[counter]['factor'][1])
                        x2 = int(float(element.attrib["r"])*self.pages[counter]['factor'][0])
                        y2 = int(float(element.attrib["b"])*self.pages[counter]['factor'][1])
                        self.pages[counter]['lns'].append([(x1, y1), (x2, y2)])
                    
        else:
            return None
            
    def readfile_minidom(self, xmlfilepath):
        from defusedxml import minidom
        if os.path.isfile(xmlfilepath):
            self.xmldom = minidom.parse(str(xmlfilepath))
            pages = self.xmldom.getElementsByTagName('page')
            pageno = 1
            for page in pages:
                self.pages[pageno] = {}
                source = page.getElementsByTagName("source")
                self.pages[pageno]['size'] = [int(source[0].attributes['sizex'].value), int(source[0].attributes['sizey'].value)]
                self.pages[pageno]['dpi'] = [int(source[0].attributes['dpix'].value), int(source[0].attributes['dpiy'].value)]
                try:
                    self.orientation = int(source[0].attributes['orientation'].value)
                except:
                    self.orientation = 0

                theoreticalPage = page.getElementsByTagName("theoreticalPage")
                self.pages[pageno]['theoreticalsize'] = [int(theoreticalPage[0].attributes['width'].value), int(theoreticalPage[0].attributes['height'].value)]
                self.pages[pageno]['factor'] = [float(self.pages[pageno]['size'][0])/float(self.pages[pageno]['theoreticalsize'][0]), float(self.pages[pageno]['size'][1])/float(self.pages[pageno]['theoreticalsize'][1])]
                self.pages[pageno]['sections'] = []
                self.pages[pageno]['columns'] = []
                self.pages[pageno]['tables'] = []
                self.pages[pageno]['paras'] = []
                self.pages[pageno]['lns'] = []
                self.pages[pageno]['words'] = {}

                sections = page.getElementsByTagName('section')
                columns = page.getElementsByTagName('column')
                tables = page.getElementsByTagName('table')
                paras = page.getElementsByTagName('para')
                lns = page.getElementsByTagName('ln')
                words = page.getElementsByTagName('wd')

                for node in sections:
                    position = self.getPosition(node, self.pages[pageno]['factor'][0], self.pages[pageno]['factor'][1])
                    if position:
                        self.pages[pageno]['sections'].append(position)
                for node in columns:
                    position = self.getPosition(node, self.pages[pageno]['factor'][0], self.pages[pageno]['factor'][1])
                    if position:
                        self.pages[pageno]['columns'].append(position)
                for node in tables:
                    position = self.getPosition(node, self.pages[pageno]['factor'][0], self.pages[pageno]['factor'][1])
                    if position:
                        self.pages[pageno]['tables'].append(position)
                for node in paras:
                    position = self.getPosition(node, self.pages[pageno]['factor'][0], self.pages[pageno]['factor'][1])
                    if position:
                        self.pages[pageno]['paras'].append(position)
                for node in lns:
                    position = self.getPosition(node, self.pages[pageno]['factor'][0], self.pages[pageno]['factor'][1])
                    if position:
                        self.pages[pageno]['lns'].append(position)
                for node in words:
                    absposition = self.getPosition(node, self.pages[pageno]['factor'][0], self.pages[pageno]['factor'][1])
                    position = self.calcrepos(absposition, self.pages[pageno]['orientation'], self.pages[pageno]['sizex'], self.pages[pageno]['sizey'])
                    if position:
                        if node.firstChild.nodeValue not in self.pages[pageno]['words']:
                            self.pages[pageno]['words'][node.firstChild.nodeValue] = []
                        self.pages[pageno]['words'][node.firstChild.nodeValue].append(position)
                        
                pageno += 1

    def reduce_xmlfile(self,  xmlfilepath,  removens=False):
        if not os.path.isfile(xmlfilepath):
            return xmlfilepath

        if removens is True:
            try:
                self.remove_ns(xmlfilepath)
            except:
                return xmlfilepath
        try:
            self.t = ET.parse(xmlfilepath)
        except:
            return xmlfilepath

        pages = self.t.findall("{http://www.scansoft.com/omnipage/xml/ssdoc-schema3.xsd}page")
        if len(pages) == 0:
            pages = self.t.findall("page")

        if os.path.splitext(xmlfilepath)[1] == u'.ocr_xml':
            newxmlfile = os.path.splitext(xmlfilepath)[0] + '.ocr.xml'
        else:
            newxmlfile = xmlfilepath

        eTree.register_namespace("", "http://www.scansoft.com/omnipage/xml/ssdoc-schema3.xsd")
        newxml = eTree.Element('document')
        for page in pages:
            pagenode = eTree.Element('page')

            iterator = page.iter()
            for element in iterator:
                if element.tag.endswith("description"):
                    pagenode.append(element)
                elif element.tag.endswith("wd"):
                    pagenode.append(element)
#                elif element.tag.endswith("section"):
#
#                elif element.tag.endswith("column"):
#
#                elif element.tag.endswith("table"):
#
#                elif element.tag.endswith("para"):
#
#                elif element.tag.endswith("ln"):

            newxml.append(pagenode)

        try:
            os.remove(xmlfilepath)
        except:
            return xmlfilepath

        try:
            tree = eTree.ElementTree(newxml)
            tree.write(newxmlfile, encoding = 'UTF-8',  xml_declaration = True)
        except:
            return xmlfilepath

        return newxmlfile

    def calcrepos(self, absposition, orientation, sizex, sizey):
        if orientation == "0":
            return absposition
        elif orientation == "270":
            x1 = sizex - absposition[1][1]
            x2 = sizex - absposition[0][1]
            
            y1 = absposition[0][0]
            y2 = absposition[1][0]
            return [(x1, y1), (x2, y2)]
        elif orientation == "90":
            x1 = absposition[0][1]
            y1 = sizey - absposition[1][0]
            x2 = absposition[1][1]
            y2 = sizey - absposition[0][0]
            return [(x1, y1), (x2, y2)]
        elif orientation == "180":
            x1 = sizex-absposition[1][0]
            x2 = sizex-absposition[0][0]
            y1 = sizey-absposition[1][1]
            y2 = sizey-absposition[0][1]
            return [(x1, y1), (x2, y2)]

    def getWordPos(self, word, pageno=None, exact=False, ignorecase=True, returndict=False):
        resultpagedict = {}
        flatresult = []
        dictresult = {}
        word = word.strip()
        if word == "":
            if returndict is True:
                return {}
            else:
                return []
        if pageno is not None:
            if pageno in self.pages:
                for realword in self.pages[pageno]['words']:
                    if realword:
                        if ignorecase is True:
                            iword = word.lower()
                            irealword = realword.lower()
                        else:
                            iword = word
                            irealword = realword
                        if exact is False:
                            if iword in irealword:
                                if pageno not in resultpagedict:
                                    resultpagedict[pageno] = []
                                    dictresult[pageno] = []
                                resultpagedict[pageno].extend(self.pages[pageno]['words'][realword])
                                #dictresult[pageno].append({"tl":self.pages[pageno]['words'][realword][0][0], "br":self.pages[pageno]['words'][realword][0][1]})
                                for w in self.pages[pageno]['words'][realword]:
                                    dictresult[pageno].append({"tl":w[0], "br":w[1]})
                        else:
                            if iword == irealword:
                                if pageno not in resultpagedict:
                                    resultpagedict[pageno] = []
                                    dictresult[pageno] = []
                                resultpagedict[pageno].extend(self.pages[pageno]['words'][realword])
                                #dictresult[pageno].append({"tl":self.pages[pageno]['words'][realword][0][0], "br":self.pages[pageno]['words'][realword][0][1]})
                                for w in self.pages[pageno]['words'][realword]:
                                    dictresult[pageno].append({"tl":w[0], "br":w[1]})
        else:
            pageno = 0
            for page in self.pages:
                pageno += 1
                for realword in self.pages[page]['words']:
                    if realword:
                        if ignorecase is True:
                            iword = word.lower()
                            irealword = realword.lower()
                        else:
                            iword = word
                            irealword = realword
                        if exact is False:
                            #print irealword
                            if iword in irealword:
                                
                                if pageno not in resultpagedict:
                                    resultpagedict[pageno] = []
                                    dictresult[pageno] = []
                                resultpagedict[pageno].extend(self.pages[pageno]['words'][realword])
                                #dictresult[pageno].append({"tl":self.pages[pageno]['words'][realword][0][0], "br":self.pages[pageno]['words'][realword][0][1]})
                                #print len(self.pages[pageno]['words'][realword])
                                for w in self.pages[pageno]['words'][realword]:
                                    dictresult[pageno].append({"tl":w[0], "br":w[1]})
                        else:
                            if iword == irealword:
                                if pageno in resultpagedict:
                                    resultpagedict[pageno] = []
                                    dictresult[pageno] = []
                                
                                resultpagedict[pageno].extend(self.pages[pageno]['words'][realword])
                                #dictresult[pageno].append({"tl":self.pages[pageno]['words'][realword][0][0], "br":self.pages[pageno]['words'][realword][0][1]})
                                for w in self.pages[pageno]['words'][realword]:
                                    dictresult[pageno].append({"tl":w[0], "br":w[1]})
        for key, resultlist in resultpagedict.items():
            flatresult.append([key, resultlist])
        if returndict is True:
            for key in dictresult:
                l = sorted(dictresult[key], key=self.getdictresultKey)
                dictresult[key] = l
            return dictresult
        else:
            sortedresult = []
            for pageresult in flatresult:
                
                page = pageresult[0]
                poslist = pageresult[1]
                sortedresult.append([page])
                sortedresult[-1].append(sorted(poslist, key=self.getflatresultKey))
                #print pageresult
            return sortedresult
    
    def getdictresultKey(self, item):
        return item["tl"][1]

    def getflatresultKey(self, item):
        return item[0][1]
        
    def getWordPosInRange(self, word, area, pageno=None, exact=False, ignorecase=True):
        wordpos = self.getWordPos(word, pageno, exact, ignorecase)
        result = []
        rx1, ry1 = area[0]
        rx2, ry2 = area[1]
        for page, positions in wordpos:
            pagepositions = []
            for singlepos in positions:
                x1, y1 = singlepos[0]
                x2, y2 = singlepos[1]
                if rx1 < x1 and rx2 > x2 and ry1 < y1 and ry2 > y2:
                    pagepositions.append(singlepos)
            if len(pagepositions) > 0:
                result.append([page, pagepositions])
        return result

    def getWordsInRange(self, area, pageno):
        rx1, ry1 = area[0]
        rx2, ry2 = area[1]
        rangewords = []
        for word in self.pages[pageno]['words']:
            if len(self.pages[pageno]['words'][word]) > 0:
                for appearance in self.pages[pageno]['words'][word]:
                    x1, y1 = appearance[0]
                    x2, y2 = appearance[1]
                    if rx1 < x1 and rx2 > x2 and ry1 < y1 and ry2 > y2:
                        rangewords.append(word)
                        break

        return rangewords

    def getPosition(self, node, xfactor, yfactor):
        try:
            x1pos = int(xfactor*int(node.getAttribute("l")))
            x2pos = int(xfactor*int(node.getAttribute("r")))
            y1pos = int(yfactor*int(node.getAttribute("t")))
            y2pos = int(yfactor*int(node.getAttribute("b")))
            result = [(x1pos, y1pos), (x2pos, y2pos)]
        except:
            result = False
        return result

    def displayResult(self, imagepath, resultlist, otherrects=None):
        pass    # kein PIL verfügbar in diesem Module, Funktion ggf in einer höheren Anwendungsklasse verschieben
        # #TODO: geht (wegen PIL) nur für die erste Seite eines Tifs
        # if os.path.isfile(imagepath):
        #     img = Image.open(imagepath)
        #     draw = ImageDraw.Draw(img)
        #     if resultlist:
        #         for page, pagepositions in resultlist:
        #             for hit in pagepositions:
        #                 draw.rectangle(hit, outline="#FF0000")
        #                 draw.rectangle([(hit[0][0]-1, hit[0][1]-1), (hit[1][0]+1, hit[1][1]+1)], outline="#FF0000")
        #                 draw.rectangle([(hit[0][0]-2, hit[0][1]-2), (hit[1][0]+2, hit[1][1]+2)], outline="#FF0000")
        #                 draw.rectangle([(hit[0][0]-3, hit[0][1]-3), (hit[1][0]+3, hit[1][1]+3)], outline="#FF0000")
        #     if otherrects:
        #         for pos, color in otherrects:
        #             draw.rectangle(pos, outline=color)
        #             draw.rectangle([(pos[0][0]-1, pos[0][1]-1), (pos[1][0]+1, pos[1][1]+1)], outline=color)
        #             draw.rectangle([(pos[0][0]-2, pos[0][1]-2), (pos[1][0]+2, pos[1][1]+2)], outline=color)
        #             draw.rectangle([(pos[0][0]-3, pos[0][1]-3), (pos[1][0]+3, pos[1][1]+3)], outline=color)
        #     img.show()

    def changeOrientation(self, pagedict={}):
        if not self.t:
            return False
        else:
            self.remove_ns(self.xmlfilepath)
            pages = self.t.findall("page")
            counter = 0
            if "all" in pagedict:
                for page in pages:
                    counter += 1
                    iter = page.iter()
                    for element in iter:
                        if element.tag.endswith("source"):
                            element.attrib["orientation"] = str(pagedict["all"])
            else:
                for page in pages:
                    counter += 1
                    if counter not in pagedict:
                        continue
                    else:
                        iter = page.iter()
                        for element in iter:
                            if element.tag.endswith("source"):
                                element.attrib["orientation"] = str(pagedict[counter])
            resultstring = ET.tostring(self.t.getroot())
            f = codecs.open(self.xmlfilepath, 'wb', 'utf-16')
            f.write(resultstring)
            f.close()

# class OcrXmlParser_old():   # übernommen aus bfa_ocr_xml_parser
#
#     def __init__(self):
#         self.pages = {}
#         self.xmldom = None
#
#     def remove_ns(self, xmlfilepath):
#         f = codecs.open(xmlfilepath, 'r', 'utf-16')
#         s = f.read()
#         f.close()
#         if '<document xmlns="http://www.scansoft.com/omnipage/xml/ssdoc-schema3.xsd" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">' in s:
#             f = codecs.open(xmlfilepath, 'w', 'utf-16')
#             s = s.replace('<document xmlns="http://www.scansoft.com/omnipage/xml/ssdoc-schema3.xsd" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">', '<document>')
#             f.write(s)
#             f.close()
#         else:
#             f.close()
#
#     def readfile(self, xmlfilepath, removens=False):
#         self.xmlfilepath = xmlfilepath
#         if os.path.isfile(xmlfilepath):
#             if removens is True:
#                 try:
#                     self.remove_ns(xmlfilepath)
#                 except:
#                     return False
#             try:
#                 self.t = ET.parse(xmlfilepath)
#             except:
#                 return False
#             pages = self.t.findall("{http://www.scansoft.com/omnipage/xml/ssdoc-schema3.xsd}page")
#             if len(pages) == 0:
#                 pages = self.t.findall("page")
#             words = []
#             counter = 0
#             for page in pages:
#                 self.pages[counter] = {}
#                 iter = page.getiterator()
#                 self.pages[counter]['sections'] = []
#                 self.pages[counter]['orientation'] = 0
#                 self.pages[counter]['columns'] = []
#                 self.pages[counter]['tables'] = []
#                 self.pages[counter]['paras'] = []
#                 self.pages[counter]['lns'] = []
#                 self.pages[counter]['words'] = {}
#                 self.pages[counter]['size'] = None
#                 self.pages[counter]['dpi'] = None
#                 self.pages[counter]['theoreticalsize'] = None
#                 self.pages[counter]['factor'] = None
#
#                 for element in iter:
#                     if element.tag.endswith("source"):
#                         try:
#                             self.pages[counter]['size'] = [int(element.attrib["sizex"]), int(element.attrib["sizey"])]
#                         except:
#                             pass
#                         try:
#                             self.pages[counter]['dpi'] = [int(element.attrib["dpix"]), int(element.attrib["dpiy"])]
#                         except:
#                             pass
#                         try:
#                             self.pages[counter]['orientation'] = element.attrib["orientation"]
#                         except:
#                             pass
#                     elif element.tag.endswith("theoreticalPage"):
#                         try:
#                             self.pages[counter]['theoreticalsize'] = [int(element.attrib['width']), int(element.attrib['height'])]
#                             self.pages[counter]['factor'] = [float(self.pages[counter]['size'][0])/float(self.pages[counter]['theoreticalsize'][0]), float(self.pages[counter]['size'][1])/float(self.pages[counter]['theoreticalsize'][1])]
#                         except:
#                             pass
#                     elif element.tag.endswith("wd"):
#                         if element.text not in self.pages[counter]['words']:
#                             self.pages[counter]['words'][element.text] = []
#                         x1 = int(float(element.attrib["l"])*self.pages[counter]['factor'][0])
#                         y1 = int(float(element.attrib["t"])*self.pages[counter]['factor'][1])
#                         x2 = int(float(element.attrib["r"])*self.pages[counter]['factor'][0])
#                         y2 = int(float(element.attrib["b"])*self.pages[counter]['factor'][1])
#                         position = self.calcrepos([(x1, y1), (x2, y2)], self.pages[counter]['orientation'], self.pages[counter]['size'][0], self.pages[counter]['size'][1])
#                         self.pages[counter]['words'][element.text].append(position)
#                elif element.tag.endswith("section"):
#                         x1 = int(float(element.attrib["l"])*self.pages[counter]['factor'][0])
#                         y1 = int(float(element.attrib["t"])*self.pages[counter]['factor'][1])
#                         x2 = int(float(element.attrib["r"])*self.pages[counter]['factor'][0])
#                         y2 = int(float(element.attrib["b"])*self.pages[counter]['factor'][1])
#                         self.pages[counter]['sections'].append([(x1, y1), (x2, y2)])
#                elif element.tag.endswith("column"):
#                         x1 = int(float(element.attrib["l"])*self.pages[counter]['factor'][0])
#                         y1 = int(float(element.attrib["t"])*self.pages[counter]['factor'][1])
#                         x2 = int(float(element.attrib["r"])*self.pages[counter]['factor'][0])
#                         y2 = int(float(element.attrib["b"])*self.pages[counter]['factor'][1])
#                         self.pages[counter]['columns'].append([(x1, y1), (x2, y2)])
#                elif element.tag.endswith("table"):
#                         x1 = int(float(element.attrib["l"])*self.pages[counter]['factor'][0])
#                         y1 = int(float(element.attrib["t"])*self.pages[counter]['factor'][1])
#                         x2 = int(float(element.attrib["r"])*self.pages[counter]['factor'][0])
#                         y2 = int(float(element.attrib["b"])*self.pages[counter]['factor'][1])
#                         self.pages[counter]['tables'].append([(x1, y1), (x2, y2)])
#                elif element.tag.endswith("para"):
#                         x1 = int(float(element.attrib["l"])*self.pages[counter]['factor'][0])
#                         y1 = int(float(element.attrib["t"])*self.pages[counter]['factor'][1])
#                         x2 = int(float(element.attrib["r"])*self.pages[counter]['factor'][0])
#                         y2 = int(float(element.attrib["b"])*self.pages[counter]['factor'][1])
#                         self.pages[counter]['paras'].append([(x1, y1), (x2, y2)])
#                     elif element.tag.endswith("ln"):
#                         x1 = int(float(element.attrib["l"])*self.pages[counter]['factor'][0])
#                         y1 = int(float(element.attrib["t"])*self.pages[counter]['factor'][1])
#                         x2 = int(float(element.attrib["r"])*self.pages[counter]['factor'][0])
#                         y2 = int(float(element.attrib["b"])*self.pages[counter]['factor'][1])
#                         self.pages[counter]['lns'].append([(x1, y1), (x2, y2)])
#                 counter += 1
#
#
#     def reduce_xmlfile(self,  xmlfilepath,  removens = False):
#         if not os.path.isfile(xmlfilepath):
#             return xmlfilepath
#
#         if removens is True:
#             try:
#                 self.remove_ns(xmlfilepath)
#             except:
#                 return xmlfilepath
#         try:
#             self.t = ET.parse(xmlfilepath)
#         except:
#             return xmlfilepath
#         pages = self.t.findall("{http://www.scansoft.com/omnipage/xml/ssdoc-schema3.xsd}page")
#         if len(pages) == 0:
#             pages = self.t.findall("page")
#
#         if os.path.splitext(xmlfilepath)[1]  == u'.ocr_xml':
#             newxmlfile = os.path.splitext(xmlfilepath)[0] + '.ocr.xml'
#         else:
#             newxmlfile = xmlfilepath
#
#         eTree.register_namespace("", "http://www.scansoft.com/omnipage/xml/ssdoc-schema3.xsd")
#         newxml = eTree.Element('document')
#         for page in pages:
#             pagenode = eTree.Element('page')
#
#             iter = page.getiterator()
#             for element in iter:
#                 if element.tag.endswith("description"):
#                     pagenode.append(element)
#                 elif element.tag.endswith("wd"):
#                     pagenode.append(element)
# #                elif element.tag.endswith("section"):
# #
# #                elif element.tag.endswith("column"):
# #
# #                elif element.tag.endswith("table"):
# #
# #                elif element.tag.endswith("para"):
# #
# #                elif element.tag.endswith("ln"):
#
#             newxml.append(pagenode)
#
#         try:
#             os.remove(xmlfilepath)
#         except:
#             return xmlfilepath
#
#         try:
#             tree = eTree.ElementTree(newxml)
#             tree.write(newxmlfile, encoding = 'UTF-8',  xml_declaration = True)
#         except:
#             return xmlfilepath
#
#         return newxmlfile
#
#
#     def calcrepos(self, absposition, orientation, sizex, sizey):
#         if orientation == "0":
#             return absposition
#         elif orientation == "270":
#             x2 = sizey-absposition[0][1]
#             x1 = sizey-absposition[1][1]
#             y1 = absposition[0][0]
#             y2 = absposition[1][0]
#             return [(x1, y1), (x2, y2)]
#         elif orientation == "90":
#             x1 = absposition[0][1]
#             y1 = sizex - absposition[1][0]
#             x2 = absposition[1][1]
#             y2 = sizex - absposition[0][0]
#             return [(x1, y1), (x2, y2)]
#         elif orientation == "180":
#             x1 = sizex-absposition[1][0]
#             x2 = sizex-absposition[0][0]
#             y1 = sizey-absposition[1][1]
#             y2 = sizey-absposition[0][1]
#             return [(x1, y1), (x2, y2)]
#
#     def getWordPos(self, word, pageno=None, exact=False, ignorecase=True):
#         resultpagedict = {}
#         flatresult = []
#         if pageno is not None:
#             if pageno in self.pages:
#                 for realword in self.pages[pageno]['words']:
#                     if realword:
#                         if ignorecase is True:
#                             iword = word.lower()
#                             irealword = realword.lower()
#                         else:
#                             iword = word
#                             irealword = realword
#                         if exact is False:
#                             if iword in irealword:
#                                 if pageno not in resultpagedict:
#                                     resultpagedict[pageno] = []
#                                 resultpagedict[pageno].extend(self.pages[pageno]['words'][realword])
#                         else:
#                             if iword == irealword:
#                                 if pageno not in resultpagedict:
#                                     resultpagedict[pageno] = []
#                                 resultpagedict[pageno].extend(self.pages[pageno]['words'][realword])
#         else:
#             pageno = 0
#             for page in self.pages:
#                 for realword in self.pages[page]['words']:
#                     if realword:
#                         if ignorecase is True:
#                             iword = word.lower()
#                             irealword = realword.lower()
#                         else:
#                             iword = word
#                             irealword = realword
#                         if exact is False:
#                             if iword in irealword:
#                                 if pageno not in resultpagedict:
#                                     resultpagedict[pageno] = []
#                                 resultpagedict[pageno].extend(self.pages[pageno]['words'][realword])
#                         else:
#                             if iword == irealword:
#                                 if pageno in resultpagedict:
#                                     resultpagedict[pageno] = []
#                                 resultpagedict[pageno].extend(self.pages[pageno]['words'][realword])
#                 pageno += 1
#
#         for key, resultlist in resultpagedict.iteritems():
#             flatresult.append([key, resultlist])
#         return flatresult
#
#     def getWordPosInRange(self, word, area, pageno=None, exact=False, ignorecase=True):
#         wordpos = self.getWordPos(word, pageno, exact, ignorecase)
#         result = []
#         rx1, ry1 = area[0]
#         rx2, ry2 = area[1]
#         for page, positions in wordpos:
#             pagepositions = []
#             for singlepos in positions:
#                 x1, y1 = singlepos[0]
#                 x2, y2 = singlepos[1]
#                 if rx1 < x1 and rx2 > x2 and ry1 < y1 and ry2 > y2:
#                     pagepositions.append(singlepos)
#             if len(pagepositions) > 0:
#                 result.append([page, pagepositions])
#         return result
#
#     def getWordsInRange(self, area, pageno):
#         rx1, ry1 = area[0]
#         rx2, ry2 = area[1]
#         rangewords = []
#         for word in self.pages[pageno]['words']:
#             if len(self.pages[pageno]['words'][word]) > 0:
#                 for appearance in self.pages[pageno]['words'][word]:
#                     x1, y1 = appearance[0]
#                     x2, y2 = appearance[1]
#                     if rx1 < x1 and rx2 > x2 and ry1 < y1 and ry2 > y2:
#                         rangewords.append(word)
#                         break
#
#         return rangewords
#
#     def getPosition(self, node, xfactor, yfactor):
#         try:
#             x1pos = int(xfactor*int(node.getAttribute("l")))
#             x2pos = int(xfactor*int(node.getAttribute("r")))
#             y1pos = int(yfactor*int(node.getAttribute("t")))
#             y2pos = int(yfactor*int(node.getAttribute("b")))
#             result = [(x1pos, y1pos), (x2pos, y2pos)]
#         except:
#             result = False
#         return result
#
#     def displayResult(self, imagepath, resultlist, otherrects=None):
#         #TODO: geht (wegen PIL) nur für die erste Seite eines Tifs
#         if os.path.isfile(imagepath):
#             img = Image.open(imagepath)
#             draw = ImageDraw.Draw(img)
#             if resultlist:
#                 for page, pagepositions in resultlist:
#                     for hit in pagepositions:
#                         draw.rectangle(hit, outline="#FF0000")
#                         draw.rectangle([(hit[0][0]-1, hit[0][1]-1), (hit[1][0]+1, hit[1][1]+1)], outline="#FF0000")
#                         draw.rectangle([(hit[0][0]-2, hit[0][1]-2), (hit[1][0]+2, hit[1][1]+2)], outline="#FF0000")
#                         draw.rectangle([(hit[0][0]-3, hit[0][1]-3), (hit[1][0]+3, hit[1][1]+3)], outline="#FF0000")
#             if otherrects:
#                 for pos, color in otherrects:
#                     draw.rectangle(pos, outline=color)
#                     draw.rectangle([(pos[0][0]-1, pos[0][1]-1), (pos[1][0]+1, pos[1][1]+1)], outline=color)
#                     draw.rectangle([(pos[0][0]-2, pos[0][1]-2), (pos[1][0]+2, pos[1][1]+2)], outline=color)
#                     draw.rectangle([(pos[0][0]-3, pos[0][1]-3), (pos[1][0]+3, pos[1][1]+3)], outline=color)
#             img.show()
#
#     def changeOrientation(self, pagedict={}):
#         if not self.t:
#             return False
#         else:
#             #self.remove_ns(self.xmlfilepath)
#             pages = self.t.findall("{http://www.scansoft.com/omnipage/xml/ssdoc-schema3.xsd}page")
#             if len(pages) == 0:
#                 pages = self.t.findall("page")
#             counter = 0
#             if "all" in pagedict:
#                 for page in pages:
#                     iter = page.getiterator()
#                     for element in iter:
#                         if element.tag.endswith("source"):
#                             element.attrib["orientation"] = str(pagedict["all"])
#                             break
#                     counter += 1
#             else:
#                 for page in pages:
#                     if counter not in pagedict:
#                         counter += 1
#                         continue
#                     else:
#                         iter = page.getiterator()
#                         for element in iter:
#                             if element.tag.endswith("source"):
#                                 element.attrib["orientation"] = str(pagedict[counter])
#                                 break
#                     counter += 1
# #            resultstring = ET.tostring(self.t.getroot())
# #            f = codecs.open(self.xmlfilepath, 'wb', 'utf-16')
# #            f.write(resultstring)
# #            f.close()
#
#         try:
#             tree = eTree.ElementTree(self.t._root)
#             tree.write(self.xmlfilepath, encoding='UTF-8',  xml_declaration = True)
#         except:
#             return False
