import sys
import os
import subprocess
import re
import shutil
import datetime

mydir = os.path.dirname(sys.argv[0])
scriptname = os.path.join(mydir, "bfa_xmlimport.py")

build_args = ["pyinstaller", "--onefile", "--icon=ServiceV3.ico", "--version-file=bfa_xmlimport_version.txt", "--clean"]

version_regex = r"version\s+=\s+[\"|\'](\d\.\d\.\d+\.\d+)[\"|\']"



build_args.append(scriptname)
dirname = os.path.dirname(scriptname)

exefile = os.path.splitext(os.path.basename(scriptname))[0]+".exe"

with open(scriptname, "r") as f:
    lines = f.read().split("\n")

version = None
for line in lines:
    line = line.strip().lower()

    match = re.match(version_regex, line)
    if match:
        version = match.groups(0)[0]
        break

if version is None:
    print( "no version found, cannot build...")
    sys.exit(1)

version_ext = version.replace(".", ", ")

info = """VSVersionInfo(
            ffi=FixedFileInfo(
            filevers=(%s),
            prodvers=(3, 6, 0, 0),
            mask=0x3f,
            flags=0x0,
            OS=0x40004,
            fileType=0x1,
            subtype=0x0,
            date=(0, 0)
            ),
            kids=[
            StringFileInfo(
                [
                StringTable(                    u'040904B0',
                [StringStruct(u'CompanyName', u'bitfarm GmbH'),
                StringStruct(u'FileDescription', u'%s'),
                StringStruct(u'FileVersion', u'%s'),
                StringStruct(u'InternalName', u''),
                StringStruct(u'LegalCopyright', u'bitfarm GmbH, 2005 - %d'),
                StringStruct(u'LegalTrademarks', u''),
                StringStruct(u'OriginalFilename', u'%s'),
                StringStruct(u'ProductName', u'bitfarm Archiv'),
                StringStruct(u'ProductVersion', u'3.6.1.0'),
                StringStruct(u'Comments', u'')])
                        ]),
            VarFileInfo([VarStruct(u'Translation', [1031, 1200])])
            ]
        )""" % (version_ext, exefile, version, datetime.datetime.now().year, exefile)



with open('bfa_xmlimport_version.txt', 'wb') as f:
    f.write(info)

builddir = os.path.join(dirname, "build")
distdir = os.path.join(dirname, "dist")
for d in (builddir, distdir):
    if os.path.isdir(d):
        shutil.rmtree(d)

try:
    proc = subprocess.check_call(build_args)
except Exception as e:
    print ("error while building exe: %s" %str(e))
