import sys
import os
import subprocess
import re
import shutil
import datetime
mydir = os.path.dirname(sys.argv[0])
scriptname = os.path.join(mydir, "bfaHttpServer.py")

build_args = ["pyinstaller", "--onedir", "--exclude-module=tkinter", "--exclude-module=tcl", "--exclude-module=FixTk", "--exclude-module=_tkinter", "--exclude-module=Tkinter", "--hidden-import=win32timezone","--icon=ServiceV3.ico", "--version-file=version.txt", "--clean"]

version_regex = r"_*version_*\s+=\s+[\"|\'](\d\.\d\.\d+\.\d+)b?(\d{0,4})[\"|\']"


build_args.append(scriptname)
dirname = os.path.dirname(scriptname)

exefile = os.path.splitext(os.path.basename(scriptname))[0]+".exe"


with open(scriptname, "r") as f:
    lines = f.read().split("\n")

version = None
for line in lines:
    line = line.strip().lower()

    match = re.match(version_regex, line)
    if match:
        groups = match.groups(0)
        version = groups[0]
        groups = [x for x in groups if x!=""]
        if len(groups) > 1:
            beta_version = groups[1]
            description = exefile + f" (beta {beta_version})"
        else:
            description = exefile
        break

if version is None:
    print("no version found, cannot build...")
    sys.exit(1)

version_ext = version.replace(".", ", ")
py_version = f"py_{sys.version_info.major}.{sys.version_info.minor}.{sys.version_info.micro}"
exefile += f" {py_version}"
version_ext = version.replace(".", ", ")

info = """VSVersionInfo(
            ffi=FixedFileInfo(
            filevers=(%s),
            prodvers=(3, 6, 2, 0),
            mask=0x3f,
            flags=0x0,
            OS=0x40004,
            fileType=0x1,
            subtype=0x0,
            date=(0, 0)
            ),
            kids=[
            StringFileInfo(
                [
                StringTable(                    u'040904B0',
                [StringStruct(u'CompanyName', u'bitfarm GmbH'),
                StringStruct(u'FileDescription', u'%s'),
                StringStruct(u'FileVersion', u'%s'),
                StringStruct(u'InternalName', u''),
                StringStruct(u'LegalCopyright', u'bitfarm Informationssysteme GmbH, 2005 - %d'),
                StringStruct(u'LegalTrademarks', u''),
                StringStruct(u'OriginalFilename', u'%s'),
                StringStruct(u'ProductName', u'bitfarm Archiv'),
                StringStruct(u'ProductVersion', u'3.6.2.0'),
                StringStruct(u'Comments', u'%s')])
                        ]),
            VarFileInfo([VarStruct(u'Translation', [1031, 1200])])
            ]
        )""" % (version_ext, exefile, version, datetime.datetime.now().year, exefile, f"py_{sys.version_info.major}.{sys.version_info.minor}.{sys.version_info.micro}")



with open('version.txt', 'w') as f:
    f.write(info)

builddir = os.path.join(dirname, "build\\bfaHttpServer")
distdir = os.path.join(dirname, "dist\\bfaHttpServer")
for d in (builddir, distdir):
    if os.path.isdir(d):
        shutil.rmtree(d)

try:
    proc = subprocess.check_call(build_args)
except Exception as e:
    print("error while building exe: %s" % str(e))
