#!/usr/bin/env python
# -*- coding: utf-8 -*-
import bitfarm_doc
import datetime
import logging
import os
import errno

from bitfarm_configobj import ConfigObj

log = logging.getLogger()


class Document(bitfarm_doc.BitfarmDocument):
    def __init__(self, jobname):
        bitfarm_doc.BitfarmDocument.__init__(self)

        if not os.path.isfile(jobname):
            raise OSError(
                errno.ENOENT, os.strerror(errno.ENOENT), jobname)
        self.config = ConfigObj(jobname, list_values=False, write_empty_values=self.write_empty_values,
                                encoding='iso8859', default_encoding='iso8859')
        #     except:
        #         log.error("Die Datei %s konnte nicht gelesen werden" % (jobname))
        #         return False
        # else:
        #     log.debug("Die Datei %s konnte nicht gefunden werden" % (jobname))

        if 'Version' in self.config:
            if 'Version' in self.config['Version']:
                self.version = int(float(self.config['Version']['Version']))
        if 'Archiv' in self.config:
            if 'Profil' in self.config['Archiv']:
                self.profile = self.config['Archiv']['Profil']
            if 'Name' in self.config['Archiv']:
                self.archive = self.config['Archiv']['Name']

            if "ArcID" in self.config['Archiv']:
                try:
                    self.arcid = int(self.config['Archiv']['ArcID'])
                except:
                    self.arcid = None
            elif "Arcid" in self.config['Archiv']:
                try:
                    self.arcid = int(self.config['Archiv']['Arcid'])
                except:
                    self.arcid = None
            if 'Tabelle' in self.config['Archiv']:
                self.tabelle = self.config['Archiv']['Tabelle']
                self.tblname = self.tabelle

        if 'Document' in self.config:
            if 'Titel' in self.config['Document']:
                self.title = self.config['Document']['Titel']
            if 'Datum' in self.config['Document']:
                try:
                    self.date = datetime.datetime.strptime(
                        self.config['Document']['Datum'], "%Y-%m-%d %H:%M:%S")
                except:
                    log.warning(u"konnte kein doc.date erzeugen")
            else:
                self.date = datetime.datetime.now().replace(microsecond=0)

                # wird an dieser Stelle nicht mehr benötigt, die gdocid wird vom Objekt selbst erzeugt (über self.create())
            if 'gdoc_id' in self.config['Document']:
                try:
                    self.gdocid = int(self.config['Document']['gdoc_id'])
                except:
                    self.gdocid = None
            if 'DOCID' in self.config['Document']:
                _id = self.config['Document']['DOCID']
                if len(_id) > 0:
                    if '.' in _id:
                        self.arcid = int(_id.split('.')[0])
                        self._docid = int(_id.split('.')[1])
                    else:
                        self._docid = int(_id)
                else:
                    _id = None
                    self._docid = None
            if 'Orginal' in self.config['Document']:
                self.docloc = self.config['Document']['Orginal']
            if 'Kopie' in self.config['Document']:
                self.extloc = self.config['Document']['Kopie']
            if 'Quelle' in self.config['Document']:
                self.oriloc = self.config['Document']['Quelle']
            if 'Volltext' in self.config['Document']:
                self.txtloc = self.config['Document']['Volltext']
            if 'Schlagworte' in self.config['Document']:
                self.tagloc = self.config['Document']['Schlagworte']
            if 'Benutzer' in self.config['Document']:
                self.user = self.config['Document']['Benutzer']
            if 'Status' in self.config['Document']:
                self.status = self.config['Document']['Status']
            if 'StatStr' in self.config['Document']:
                self.statstr = self.config['Document']['StatStr']
            if 'OCR_Typ' in self.config['Document']:
                self.ocrtype = self.config['Document']['OCR_Typ']
            if 'OCR_QF' in self.config['Document']:
                self.ocrqf = self.config['Document']['OCR_QF']
            if 'Herkunft' in self.config['Document']:
                self.origin = self.config['Document']['Herkunft']
            if 'Dateiname' in self.config['Document']:
                self.filename = self.config['Document']['Dateiname']
            if 'type' in self.config['Document']:
                self.filetype = self.config['Document']['type']
            if 'Filter' in self.config['Document']:
                self.filter = self.config['Document']['Filter'].split(";")

        if 'Task' in self.config:
            if 'Filter' in self.config['Task']:
                self.target_date = self.config['Task']['Termin']
            if 'Alarm' in self.config['Task']:
                self.alarm = self.config['Task']['Alarm']
            if 'Bearbeiter' in self.config['Task']:
                self.editor = self.config['Task']['Bearbeiter']
            if 'Notiz' in self.config['Task']:
                self.info = self.config['Task']['Notiz']
            if 'TimerOptionen' in self.config['Task']:
                self.timeroptions = self.config['Task']['TimerOptionen']

        if 'Hash' in self.config:
            if 'SHA1' in self.config['Hash']:
                self.sha1 = self.config['Hash']['SHA1']

        if 'SVN' in self.config:
            if 'SVN_Link' in self.config['SVN']:
                self.svn_link = self.config['SVN']['SVN_Link']
            if 'SVN_Version' in self.config['SVN']:
                self.svn_version = self.config['SVN']['SVN_Version']
            if 'SVN_Revision' in self.config['SVN']:
                self.svn_revision = self.config['SVN']['SVN_Revision']
            if 'SVN_Datum' in self.config['SVN']:
                self.svn_date = self.config['SVN']['SVN_Datum']
            if 'SVN_Info' in self.config['SVN']:
                self.svn_info = self.config['SVN']['SVN_Info']

        if 'Referenz' in self.config:
            if 'Anzahl' in self.config['Referenz']:
                _anz = self.config['Referenz']['Anzahl']
                for _ind in range(1, int(_anz)):
                    if 'Referenz%d' % _ind in self.config['Referenz']:
                        self.links.append(
                            self.config['Referenz']['Referenz%d' % _ind])

        if 'Zusatzfelder' in self.config:
            if 'Felder' in self.config['Zusatzfelder']:
                _anz = self.config['Zusatzfelder']['Felder']
                for _ind in range(1, int(_anz) + 1):
                    _add = bitfarm_doc.BitfarmAddfield()
                    if 'ZusTitel%d' % _ind in self.config['Zusatzfelder']:
                        _add.name = self.config['Zusatzfelder']['ZusTitel%d' % _ind]
                    if 'ZusFeld%d' % _ind in self.config['Zusatzfelder']:
                        _add.add = self.config['Zusatzfelder']['ZusFeld%d' % _ind]
                    if 'ZusWert%d' % _ind in self.config['Zusatzfelder']:
                        if ';' in self.config['Zusatzfelder']['ZusWert%d' % _ind]:
                            _add.value = self.config['Zusatzfelder']['ZusWert%d' % _ind].split(
                                ';')
                        else:
                            _add.value = self.config['Zusatzfelder']['ZusWert%d' % _ind]

                    self.fields[_add.name] = _add

        if 'Parameter' in self.config:
            for item in self.config['Parameter'].items():
                self.params[item[0]] = item[1]

        self.__tempname = os.path.splitext(jobname)[0]
        self.__queue = os.path.dirname(jobname)
        self.path = jobname

    def write_to_file(self, jobname=None):
        """
        Schreibt das Dokument in eine Jodatei.
        Der Dateiname ist in self.tempname hinterlegt.
        Dynamisch hinzugefügte Werte werden unter der Section [Document]ergänzt
        Wenn keine entsprechenden Zusatzfelder in der Job-Datei vorhanden sind,
        so werden diese unter der Section [Document] ergänzt
        Zusatzfeldinhalte in Form einer Liste werden mit Semikolon getrennt geschrieben
        @param jobname: (optional) der Pfad auf die .job-Datei. Wird nur ein Basisname übergeben,
        wird die Endung .job angehangen. Wird nichts übergeben, so wird self.tempname ausgelesen.
        """
        if jobname is None:
            jobname = self.path

        if os.path.splitext(jobname)[1] == '':
            jobname += '.job'

        if not self.config:
            self.config = ConfigObj(list_values=False, write_empty_values=self.write_empty_values, encoding='cp1252',
                                    default_encoding='cp1252')

        self.config.filename = jobname

        if self.version in (35, 30, 3.0, 3.5, "35", "30", "3.0", "3.5"):
            if len(self.statusdict) > 0:
                self.status = 0
                _ind = 1
                for val in self.statusdict.values():
                    if val:
                        self.status |= (2 ** _ind)
                    _ind += 1

            if True:
                self.status |= (1 if self.in_progress else 0)
        else:
            self.config['Document']['Status'] = self.status
        if self.version or self.write_empty_values:
            if 'Version' not in self.config:
                self.config['Version'] = {}
            self.config['Version']['Version'] = (
                self.version if self.version else '')

        if self.archive or self.tblname or self.profile or self.arcid or self.write_empty_values:
            if 'Archiv' not in self.config:
                self.config['Archiv'] = {}
            self.config['Archiv']['Profil'] = (
                self.profile if self.profile else '')
            self.config['Archiv']['Name'] = (
                self.archive if self.archive else '')
            self.config['Archiv']['Tabelle'] = (
                self.tblname if self.tblname else '')
            self.config['Archiv']['ArcID'] = (self.arcid if self.arcid else '')

        if 'Document' not in self.config:
            self.config['Document'] = {}
        _docid = ''
        if self.version in (35, 30, 3.0, 3.5, "35", "30", "3.0", "3.5"):
            if self._docid or self.write_empty_values:
                if self.arcid and self._docid:
                    _docid = "%d.%d" % (self.arcid, self._docid)
                else:
                    _docid = str(self._docid if self._docid else '')
                self.config['Document']['DOCID'] = (_docid if _docid else '')
        elif self.version in (40, 4.0, 3.6, 36, "36", "40", "3.6", "4.0"):
            if self.arcid and self.gdocid:
                _docid = "%d.%d" % (self.arcid, self.gdocid)
            elif self.gdocid:
                _docid = "%d.%d" % (0, self.gdocid)
            self.config['Document']['DOCID'] = (_docid if _docid else '')
        if self.gdocid or self.write_empty_values:
            self.config['Document']['gdoc_id'] = (
                self.gdocid if self.gdocid else '')
        if self.title or self.write_empty_values:
            title_enc = self.title.encode('cp1252', 'replace')
            title_enc = title_enc.decode('cp1252')
            self.config['Document']['Titel'] = (title_enc if title_enc else '')
        if self.docloc or self.write_empty_values:
            self.config['Document']['Orginal'] = (
                self.docloc if self.docloc else '')
        if self.extloc or self.write_empty_values:
            self.config['Document']['Kopie'] = (
                self.extloc if self.extloc else '')
        if self.oriloc or self.write_empty_values:
            self.config['Document']['Quelle'] = (
                self.oriloc if self.oriloc else '')
        if self.user or self.write_empty_values:
            self.config['Document']['Benutzer'] = (
                self.user if self.user else '')
        if self.userid or self.write_empty_values:
            self.config['Document']['userid'] = (
                self.userid if self.userid else '')

        if self.ftx or self.write_empty_values:
            if type(self.ftx) == bitfarm_doc.BitfarmFulltext:
                self.config['Document']['Volltext'] = (
                    self.ftx.text if self.ftx.text else '')
            else:
                self.config['Document']['Volltext'] = (
                    self.ftx if self.ftx else '')
        self.config['Document']['Volltext'] = ""
        if self.status or self.write_empty_values:
            self.config['Document']['Status'] = (
                self.status if self.status else '')
        if self.statstr or self.write_empty_values:
            self.config['Document']['StatStr'] = (
                self.statstr if self.statstr else '')
        if self.ocrqf or self.write_empty_values:
            self.config['Document']['OCR_QF'] = (
                self.ocrqf if self.ocrqf else -1)
        if self.ocrtype or self.write_empty_values:
            self.config['Document']['OCR_Typ'] = (
                self.ocrtype if self.ocrtype else 'OMP')
        if self.tags or self.write_empty_values:
            self.config['Document']['Schlagworte'] = (
                ";".join(self.tags) if self.tags else '')
        if self.date or self.write_empty_values:
            if type(self.date) in [str]:
                self.config['Document']['Datum'] = (
                    self.date if self.date else '')
            else:
                self.config['Document']['Datum'] = (
                    self.date.strftime('%Y-%m-%d %H:%M:%S') if self.date else '')
        if self.filter or self.write_empty_values:
            if type(self.filter) is list:
                filter = ";".join(self.filter)
            else:
                filter = self.filter
            self.config['Document']['Filter'] = (filter if filter else '')
        #        if self.version  in (35,  30, 3.0,  3.5,  "35", "30",  "3.0",  "3.5"):
        if self.pages or self.write_empty_values:
            self.config['Document']['Pages'] = (
                self.pages if self.pages else '')
        #        else:
        #            if self.pagecount or self.write_empty_values:
        #                self.config['Document']['Pages'] = (self.pagecount if self.pagecount else '')
        if self.sha1 or self.write_empty_values:
            if 'Hash' not in self.config:
                self.config['Hash'] = {}
            self.config['Hash']['SHA1'] = (self.sha1 if self.sha1 else '')
        if len(self.links) > 0 or self.write_empty_values:
            if 'Referenz' not in self.config:
                self.config['Referenz'] = {}
            self.config['Referenz']['Anzahl'] = len(self.links)
            for _ind in range(len(self.links)):
                self.config['Referenz']['Referenz%d' %
                                        (_ind + 1)] = self.links[_ind]
        if self.svn_link or self.svn_version or self.svn_revision or self.svn_date or self.svn_info or self.write_empty_values:
            if 'SVN' not in self.config:
                self.config['SVN'] = {}
            if self.svn_link or self.write_empty_values:
                self.config['SVN']['SVN_Link'] = (
                    self.svn_link if self.svn_link else '')
            if self.svn_version or self.write_empty_values:
                self.config['SVN']['SVN_Version'] = (
                    self.svn_version if self.svn_version else '')
            if self.svn_revision or self.write_empty_values:
                self.config['SVN']['SVN_Revision'] = (
                    self.svn_revision if self.svn_revision else '')
            if self.svn_date or self.write_empty_values:
                self.config['SVN']['SVN_Datum'] = (
                    self.svn_date if self.svn_date else '')
            if self.svn_info or self.write_empty_values:
                self.config['SVN']['SVN_Info'] = (
                    self.svn_info if self.svn_info else '')

        if self.target_date or self.alarm or self.editor or self.info or self.timeroptions or self.write_empty_values:
            if 'Tasks' not in self.config:
                self.config['Tasks'] = {}
            if self.target_date or self.write_empty_values:
                self.config['Tasks']['Termin'] = (
                    self.target_date if self.target_date else '')
            if self.alarm or self.write_empty_values:
                if self.alarm_date is not None:
                    self.config['Tasks']['Alarm'] = (
                        self.alarm_date if self.alarm_date else '')
                else:
                    alarm = None
                    if self.alarm is not None:
                        alarm = self.target_date + \
                            datetime.timedelta(self.alarm)
                    self.config['Tasks']['Alarm'] = (alarm if alarm else '')
            if self.editor or self.write_empty_values:
                self.config['Tasks']['Bearbeiter'] = (
                    self.editor if self.editor else '')
            if self.info or self.write_empty_values:
                self.config['Tasks']['Notiz'] = (
                    self.info if self.info else '')
            if self.timeroptions or self.write_empty_values:
                self.config['Tasks']['TimerOptionen'] = (
                    self.timeroptions if self.timeroptions else '0')

        if len(self.fields) > 0 or self.write_empty_values:
            # if 'Zusatzfelder' not in self.config:
            self.config['Zusatzfelder'] = {}
            if self.version in (3, 35, '3.0'):
                self.config['Zusatzfelder']['Felder'] = len(self.fields)
                _ind = 1
                for field in self.fields.values():

                    self.config['Zusatzfelder']['ZusTitel%d' %
                                                (_ind)] = field.name
                    self.config['Zusatzfelder']['ZusFeld%d' %
                                                (_ind)] = field.add
                    if field.value:
                        val = field.value
                        if type(val) is list:
                            if field.type in (7, 8):
                                self.config['Zusatzfelder']['ZusWert%d' % (_ind)] = ";".join(
                                    [str(x).replace(".", ",") for x in val])
                            else:
                                self.config['Zusatzfelder']['ZusWert%d' % (
                                    _ind)] = ";".join(val)
                        else:
                            # Datums-Felder unterscheiden sich in 3.5 und 3.6 job-Files
                            # 3.5 Datum dd.mm.yyyy, 3.6 Datum dd.mm.yyyy HH:MM:SS
                            if field.type == 1:
                                if self.version in (35, 30, 3.0, 3.5, "35", "30", "3.0", "3.5"):
                                    datestring = val.strftime('%d.%m.%Y')
                                    self.config['Zusatzfelder']['ZusWert%d' % (
                                        _ind)] = datestring
                                elif self.version in (36, 3.6, "36", "3.6", 40, 4.0, "40", "4.0"):
                                    self.config['Zusatzfelder']['ZusWert%d' % (
                                        _ind)] = val
                            elif field.type in (7, 8):
                                self.config['Zusatzfelder']['ZusWert%d' % (
                                    _ind)] = str(val).replace(".", ",")
                            else:
                                self.config['Zusatzfelder']['ZusWert%d' % (
                                    _ind)] = val

                    else:
                        self.config['Zusatzfelder']['ZusWert%d' % (_ind)] = ''
                    _ind += 1
            else:
                self.config['Zusatzfelder']['Felder'] = len(self.fields)
                _ind = 1
                for field in self.fields.values():
                    self.config['Zusatzfelder']['ZusTitel%d' %
                                                (_ind)] = field.name
                    self.config['Zusatzfelder']['ZusFeld%d' %
                                                (_ind)] = field.add
                    if field.value:
                        val = field.value
                        if type(val) is list:
                            if field.type in (7, 8):
                                self.config['Zusatzfelder']['ZusWert%d' % (_ind)] = ";".join(
                                    [str(x).replace(".", ",") for x in val])
                            else:
                                self.config['Zusatzfelder']['ZusWert%d' % (
                                    _ind)] = ";".join(val)
                        else:
                            if field.type in (7, 8):
                                self.config['Zusatzfelder']['ZusWert%d' % (
                                    _ind)] = str(val).replace(".", ",")
                            else:
                                if type(val) in [str]:
                                    val_enc = val.encode('cp1252', 'replace')
                                    val = val_enc.decode('cp1252')
                                self.config['Zusatzfelder']['ZusWert%d' % (
                                    _ind)] = val
                    else:
                        self.config['Zusatzfelder']['ZusWert%d' % (_ind)] = ''
                    _ind += 1

        if len(self.params) > 0 or self.write_empty_values:
            if 'Parameter' not in self.config:
                self.config['Parameter'] = {}
            for item in self.params.items():
                self.config['Parameter'][item[0]] = item[1]

        self.config.write_empty_values = True
        try:
            self.config.write()
        except:
            log.exception("Fehler beim Jobfile" + self.config.filename)
            return False
        else:
            log.debug("Jobfile geschrieben: " + self.config.filename)
            return True
