#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Dieses Modul enthält die einfache BitfarmCon und BitfarmConnections Klasse, von bitfarm_baseobj geerbt,
als einfache Repräsentation eines Profils..
Sie kann in Anwendungen verwendet werden, die keinen Datenbankzugriff benötigen (siehe bitfarm_proc)"""

import os
import sys
import logging
import bitfarm_baseobj

# try:
#     import bitfarm_enterprise as bitfarm_base
# except:
#     import bitfarm_core as bitfarm_base

__version__ = 'v3.6.1'


class BitfarmCon(bitfarm_baseobj.BitfarmBaseCon):
    """
    Bitfarm Configurations-Subklasse.
    Diese Klasse stellt ein einzelnes Profil zur Verfügung, beim Initialisieren kann der Name oder der komplette Pfad auf
    ein Profil übergeben werden.
    con = BitfarmCon('bitfarm')
    Eine Datenbank-Engine wird über die Methode create_db_engine hergestellt, diese wird in con.engine hinterlegt.
    Hier sind auch die Funktionen hinterlegt um User und Gruppen zu lesen, da diese profilabhängig und nicht
    workflow- oder dokumentabhängig sind. Dafür muss aber die DB-Connection übergeben werden, da diese ja erst zur Laufzeit
    erzeugt wird.
    Per Default wird das Con-Objekt als Version 35 erstellt, um abwärtskompatibiltät zur Datenbank 35 zu gewährleisten.
    Jede höhere Version (36, 40) muss in der Con-Datei in der Sektion [Server] angegeben werden.
    @version:
    """
    count = 0
    log = logging.getLogger("Main.BitfarmCon")
    log.setLevel(logging.WARNING)

    if sys.platform == 'win32':
        mypath = os.path.split(os.path.abspath(sys.argv[0]))[0]
    else:
        mypath = os.getcwd()
    try:
        bfapath = os.environ['bitfarm-archiv']
    except:
        bfapath = mypath

    # locale.setlocale(locale.LC_ALL, '')



    def __init__(self, *args):
        """
        Erstellt eine Profil-Instanz. Es kann entweder ein Profilname oder der Pfad auf eine .Con-Datei angegeben werden
        """
        self.log.debug("Initialisiere ein bitfarm_con Object")
        bitfarm_baseobj.BitfarmBaseCon.__init__(self, *args)
        # self.__tempname = tempname

    #     self.config = None
    #     self.profile = None
    #     """@ivar: Name des Profils"""
    #     self.confile = None
    #     """@ivar: Pfad auf die .Con-Datei"""
    #     self.engine = None
    #     """@ivar: hinterlegte DB-Engine"""
    #     self.sys_queue = None
    #     """@ivar: Binding der sys_queue Tabelle, falls vorhanden, ansonsten None"""
    #     self.sys_arc = None
    #     """@ivar: Binding der sys_arc-Tabelle, sollte vorhanden sein :-) """
    #     self.sys_arcxs = None
    #     """@ivar: Binding der sys_arcxs-Tabelle"""
    #     self.sys_gdocid = None
    #     """@ivar: Binding der sys_gdocid-Tabelle, nur v35"""
    #     self.sys_doc = None
    #     """@ivar: Binding der sys_doc-Tabelle, ab v36"""
    #     self.sys_docxs = None
    #     """@ivar: Binding der sys_docxs-Tabelle, ab v36"""
    #     self.sys_doctxt = None
    #     """@ivar: Binding der sys_doctxt-Tabelle, ab v36"""
    #     self.sys_docidx = []
    #     """@ivar: Binding der sys_docidx-Tabelle, ab v36
    #         @type: list"""
    #     self.arc_tbl = None
    #     """@ivar: Binding an die aktuell ausgewählte Archiv-Tabelle, wird ggf. innerhalb BitfarmDocProcessing oder BitfarmWorkflowProcessing erzeugt, nur v35"""
    #     self.sys_doclock = None
    #     """@ivar: Binding der sys_doclock-Tabelle, nur v35"""
    #     self.sys_doclink = None
    #     """@ivar: Binding der sys_doclink-Tabelle"""
    #     self.sys_bookmarks = None
    #     """@ivar: Binding der sys_bookmarks-Tabelle"""
    #     self.sys_statefieldxs = None
    #     """@ivar: Binding der sys_statefieldxs-Tabelle"""
    #     self.sys_statefields = None
    #     """@ivar: Binding der sys_statefields-Tabelle"""
    #     self.sys_addfieldxs = None
    #     """@ivar: Binding der sys_addfieldxs-Tabelle"""
    #     self.sys_addfields = None
    #     """@ivar: Binding der sys_addfields-Tabelle"""
    #     self.sys_user = None
    #     """@ivar: Binding der sys_user-Tabelle"""
    #     self.sys_group = None
    #     """@ivar: Binding der sys_group-Tabelle"""
    #     self.sys_groupuser = None
    #     """@ivar: Binding der sys_groupuser-Tabelle"""
    #     self.sys_group = None
    #     """@ivar: Binding der sys_group-Tabelle"""
    #     self.history = None
    #     """@ivar: Binding an die aktuelle History-Tabelle, nur v35"""
    #     self.sys_history = None
    #     """@ivar: Binding an die History-Tabelle, ab v36 """
    #     self.sys_timer = None
    #     """@ivar: Binding an die sys_timer-Tabelle """
    #     self.sys_timerhist = None
    #     """@ivar: Binding an die sys_timerhist-Tabelle """
    #     self.sys_wfd = None
    #     """@ivar: Binding der sys_wfd-Tabelle, ansonsten None"""
    #     self.sys_workflow = None
    #     """@ivar: Binding der sys_workflow-Tabelle, ansonsten None"""
    #     self.sys_garbage = None
    #     """@ivar:  Binding der sys_garbage-Tabelle, ansonsten None"""
    #     self.sys_duplicates = None
    #     """@ivar: Binding der sys_duplicates-Tabelle, ansonsten None"""
    #     self.sys_sap = None
    #     """@ivar: Binding der sys_sap-Tabelle, ansonsten None"""
    #     self.sys_holiday = None
    #     """@ivar: Binding der sys_holiday-Tabelle, ansonsten None"""
    #     self.sys_superior = None
    #     """@ivar: Binding der sys_superior-Tabelle, ansonsten None"""
    #     self.sys_svn = None
    #     """@ivar: Binding der sys_svn-Tabelle, ansonsten None"""
    #     self.sys_abo = None
    #     """@ivar: Binding der sys_abo- Tabelle"""
    #     self.sys_anno = None
    #     """@ivar: Binding der sys_annotations- Tabelle"""
    #     self.db = None
    #     """@ivar: Datenbank-Connection, über connect_db() erzeugt, sollte nicht innerhalb von Threads verwendet werden """
    #     self.ini = {}
    #     """@ivar: Dictionary mit dem Inhalt der scripts.ini """
    #     self.locale = locale.getdefaultlocale()
    #     """@ivar: lokale Sprache und Codepage"""
    #     self.localeconv = locale.localeconv()
    #     """@ivar: lokale Einstellungen des Gebietsschemas"""
    #     self.meta = None
    #     self.history_map = None
    #     """@ivar: ans Profil gebundene Instanz der HISTORY_MAP Tabelle (dictionary)"""
    #     self.server_config = {}
    #     """@ivar: Inhalt der sys_config als Dictionary."""
    #
    #
    #
    #     try:
    #         self.history_map = HISTORY_MAP
    #     except:
    #         self.history_map = None
    #         self.log.warning('keine History_map geladen, Historyeinträge nicht möglich')
    #
    #     self.emptydocs = {}
    #     """@ivar: die vorhandenen Vorlagen als Bezeichner:Pfad Paare"""
    #
    #     # bfaServer [SERVER]
    #     self.bfaserver = None
    #     """@ivar: Hostname des bfaServers"""
    #     self.bfaserverport = None
    #     """@ivar: Port des bfaServers"""
    #     self.bfaserverports = None
    #     """@ivar: mehrere semikolongetrennte Ports des bfaServers36"""
    #     self.certificate = None
    #     """@ivar: Zertifikatsfile ds bfaServers"""
    #     self.concurrent = 0
    #     """@ivar: Anzahl concurrent Nutzer, 1 Nutzer sind default"""
    #     self.named = 5
    #     """@ivar: Anzahl named Nutzer, 5 Nutzer sind default"""
    #     self.ssl = None
    #     """@ivar: nutzt SSL-Verschlüsselung(Achtung Viewer unterstützt derzeit keine SSL Verschlüsselung ab v3.5.2"""
    #
    #     # db connection [CONNECTION]
    #     self.server = None
    #     """@ivar: Host des Datenbank-Servers"""
    #     self.port = None
    #     """@ivar: Port ders Datenbank-Servers"""
    #     self.db_name = None
    #     """@ivar: Datenbankname"""
    #     self.user = None
    #     """@ivar: Datenbank-User"""
    #     self.password = None
    #     """@ivar: Passwort des Datenbankusers"""
    #     self.domain = None
    #     """@ivar: Domäne des bitfarm Servers"""
    #     self.db_engine = None
    #     """@ivar: Name der Datenbank-Engine, derzeit nur 'mysql' verfügbar"""
    #     self.db_encoding = None
    #     """@ivar: das Datenbank-Encoding, bei 'mysql' wird 'latin-1' als default eingetragen"""
    #     self.tables = None
    #     """@ivar: Liste der Datenbank-Tabellen der verbundenen Datenbank, wird bei create_db_engine eingelesen"""
    #     self.version = 35
    #     """@ivar: Datenbank, bzw bitfarm Serverversion, aktuell Version 3"""
    #     self.undistributed_arcid = None
    #     """@ivar: arcid des unverteilt-Archives"""
    #     self.max_row_size = None
    #     """@ivar: die maximale Spaltengröße einer Datenbank-Tabelle, der geladenen Datenbank, ist von der Datenbank-Engine abhängig (MySQL = 65535)"""
    #
    #     # archivierung [ARCHIVING]
    #     self.archiveserver = None
    #     """@ivar: Host des Archivierungsserver"""
    #     self.archiveport = None
    #     """@ivar: Port des Archivierungsserver"""
    #
    #     # [SCRIPTS]
    #     self.serverprogram = None
    #     """@ivar: Lokaler Pfad zum bitfarm-Archiv Installationsverzeichnis (nicht UNC)"""
    #     self.clientprogram = None
    #     """@ivar: UNC Pfad zum bitfarm-Archiv Installationsverzeichnis"""
    #     self.clientspooler = None
    #     """@ivar: Lokaler Pfad zum Übergabe-Verzeichnis (nicht UNC)"""
    #     self.uncclientspooler = None
    #     """@ivar: UNC Pfad zum Übergabe-Verzeichnis """
    #     self.archive_rs = None
    #     """@ivar: UNC Pfad zur revisionssicheren Archivablage"""
    #     self.archive_nrs = None
    #     """@ivar: UNC Pfad zur nicht revisionssicheren Archivablage"""
    #     self.storages = []
    #     """@ivar: Systemvariablen mit UNC Pfad zum Storage als Liste angelegt, ein Dokument verweist über die Variable doc.storage auf sein Storage, v36"""
    #     self.queue = None
    #     """@ivar: Pfad des Queue-Verzeichnisses, wenn nicht angegeben dann %bitfarm-archiv%\queue"""
    #     self.omnipage = None
    #     """@ivar: Lokaler Pfad und Dateiname zur Omnipageinstallation"""
    #     self.ghostscript = None
    #     """@ivar: Lokaler Pfad und Dateiname zur Ghostscriptinstallation (wird auch von gs_detect.exe gesetzt)"""
    #     self.gsview = None
    #     """@ivar: Lokaler Pfad und Dateiname zur GSView-Installation"""
    #     self.openofficepath = None
    #     """@ivar: Pfad zur OpenOfficeInstallation (oder Portable bei GPL) """
    #     self.smtpserver = None
    #     """@ivar: Hostname oder IP-Adresse eines SMTP Servers, der die Erinnerungsmails aus der Wiedervorlage oder Mails aus dem Workflow versendet oder Verknüpfungsmails aus dem Viewer heraus."""
    #     self.sendername = None
    #     """@ivar: Absenderadresse für Server-Mails."""
    #     self.smtpport = None
    #     """@ivar: Port des smtp-Servers (normalerweise 25)"""
    #     self.authuser = None
    #     """@ivar: Benutzername für SMTP Server, falls eine Authentifizierung nötig ist."""
    #     self.authpass = None
    #     """@ivar: Passwort für SMTP Server, falls eine Authentifizierung nötig ist."""
    #     self.duplicate_testing = False
    #     """@ivar: aktiviert die Duplettenprüfung"""
    #     self.adminmail = None
    #     """@ivar: Empfänger für warnings, falls nichts anderes angegeben"""
    #     self.templates = None
    #     """@ivar: UNC Pfad zum bitfarm-Archiv Templates-Verzeichnis"""
    #     self.workflowfile = None
    #     """@ivar:  UNC-Pfad und Dateiname der zentralen WFD-Datei (Nur nötig bei Multi-Server Umgebungen)"""
    #     self.undistributed = None
    #     """@ivar: Name des unverteilt Archivs"""
    #     self.workflows_dir = None
    #     """@ivar: UNC Pfad zum zentralen bitfarm-Archiv Workflow-Verzeichnis (Nur nötig bei Multi-Server Umgebungen)"""
    #     self.tempdir = None
    #     """@ivar: Pfad zum Temp-Verzeichnis"""
    #     self.garbage_id = None
    #     """@ivar: Arcid des Garbagearchives"""
    #     self.duplicates_id = None
    #     """@ivar: Arcid des Dublettenarchives"""
    #
    #     # [BACKUP]
    #     self.backuppath = None
    #     """@ivar: Zielpfad in dem die RAR-Dateien, die vom Backupscript  erstellt werden, abgelegt werden sollen."""
    #     self.sqldatapath = None
    #     """@ivar: Lokaler Pfad zu den MySQL Datenbank Dateien. """
    #     self.backuparchive = False
    #     """@ivar: """
    #     self.backupnrsarchive = False
    #     """@ivar: Soll die revisionssichere Archivablage mit im Backup eingeschlossen werden? In den meisten Fällen false, da dies über einen normales Fileserverbackup läuft."""
    #     self.backupversion = False
    #     """@ivar: Soll das Bitfarm-Programmverzeichnis mit im Backup eingeschlossen werden?"""
    #     self.backupworkflows = False
    #     """@ivar: Sicherung des workflow-Verzeichnisses"""
    #     self.backupsvnrepository = False
    #     """@ivar: Sicherung des SVN-Repositorys (sinnvoll bei Einsatz der Versionierung). In den meisten Fällen false, da dies über einen normales Fileserverbackup läuft"""
    #     self.backupmysqllog = False
    #     """@ivar: Backup der MySQL Logdateien"""
    #     self.filesize = None
    #     """@ivar: Maximale Größe in KB einer RAR Datei, bevor eine neue RAR Datei erstellt wird."""
    #     self.backupinterval = None
    #     """@ivar: Anzahl der Tage, die die RAR Dateien im backuppath aufbewahrt werden, bevor sie automatisch per Script gelöscht werden."""
    #
    #     # [OPTIONS]
    #     self.logdir = None
    #     """@ivar: UNC-Pfad zum zentralen Verzeichnis für Client-Logs (Schreibbrechtigung für User setzen; nur zum Debuggen verwenden (Performance))"""
    #     self.savepass = None
    #     """@ivar: Hier schaltet man im Viewer die Möglichkeit das Passwort zu speichern aus (0)"""
    #     self.manuscanplugin = None
    #     """@ivar: Dateiname des Manuscanplugins, das eingebunden werden soll."""
    #     self.importerplugin = None
    #     """@ivar: Dateiname des Importerplugins, das eingebunden werden soll."""
    #     self.svninfo = None
    #     """@ivar: Schaltet die SVNInfo-Funktion im Viewer ein (1)"""
    #     self.linkchars = None
    #     """@ivar: Ab der hier angegebenen Anzahl von Zeichen beginnt der Viewer bei Verknüpfungen mit der Autovervollständigung."""
    #     self.regkeyarchived = None
    #     """@ivar: Wenn dieser Wert höher ist, als der lokal in der Registry gespeicherte Wert, überträgt der Viewer seine Registrierungsdaten an den Server"""
    #     self.ignorerightsatimport = None
    #     """@ivar: Hier schaltet man für Importer und Manuscan die Überprüfung der Zusatzfeldrechte aus (1)"""
    #     self.usesearch35 = False
    #     """@ivar: Schaltet für Zusatzfelder (Text) auf das Suchverhalten der Version 35"""
    #     self.resultlist_showinedit = False
    #     """@ivar: Zeigt in der Trefferliste die Dokumente in Bearbeitung grün an, anstatt Dokumente der Wiedervorlgae"""
    #
    #     BitfarmCon.count += 1
    #
    #     if len(args) > 0:
    #         confile = args[0]
    #         if not os.path.isfile(confile):
    #             confile = os.path.join(self.bfapath, confile + '.con')
    #             if not os.path.exists(confile):
    #                 confile = os.path.join(self.mypath, args[0] + '.con')
    #
    #         self.config = ConfigParser.ConfigParser(allow_no_value=True)
    #         if os.path.isfile(confile) and os.path.exists(confile):
    #             try:
    #                 self.config.readfp(open(confile))
    #             except:
    #                 self.log.debug("Die Datei %s konnte nicht gelesen werden" % confile)
    #                 raise ParseConError(confile)
    #         else:
    #             self.log.debug("Die Datei %s konnte nicht gefunden werden" % confile)
    #             raise FindConError(confile)
    #
    #         profile = os.path.splitext(os.path.basename(confile))[0]
    #         self.profile = profile.lower()
    #         self.confile = confile
    #
    #         if self.config.has_section('Server'):
    #             try:
    #                 if self.config.has_option('Server', 'Version'):
    #                     self.version = self.config.getint('Server', 'Version')
    #             except:
    #                 self.version = 35
    #             if self.config.has_option('Server', 'Server'):
    #                 self.bfaserver = self.config.get('Server', 'Server')
    #             if self.config.has_option('Server', 'Ports'):
    #                 ports = self.config.get('Server', 'Ports')
    #                 self.bfaserverports = [int(x) for x in ports.split(";")]
    #                 self.bfaserverport = self.bfaserverports[0]
    #             elif self.config.has_option('Server', 'Port'):
    #                 self.bfaserverport = self.config.getint('Server', 'Port')
    #             if self.config.has_option('Server', 'Certificate'):
    #                 self.certificate = self.config.get('Server', 'Certificate')
    #             if self.version == 35 and self.config.has_option('Server', 'Concurrent'): # in v36 wird die profil.rc datei gelesen
    #                 try:
    #                     self.concurrent = self.config.getint('Server', 'Concurrent')
    #                 except:
    #                     self.concurrent = 5     # 5 war default bei v35
    #
    #             if self.config.has_option('Server', 'SSL'):
    #                 try:
    #                     self.ssl = self.config.getboolean('Server', 'SSL')
    #                 except:
    #                     self.ssl = False
    #
    #         if self.config.has_section('Connection'):
    #             if self.config.has_option('Connection', 'server'):
    #                 self.server = self.config.get('Connection', 'server')
    #             if self.config.has_option('Connection', 'Port'):
    #                 self.port = self.config.getint('Connection', 'Port')
    #             if self.config.has_option('Connection', 'DB'):
    #                 self.db_name = self.config.get('Connection', 'DB')
    #             if self.config.has_option('Connection', 'User'):
    #                 self.user = self._bitdecrypt(self.config.get('Connection', 'User'))
    #             if self.config.has_option('Connection', 'Pass'):
    #                 self.password = self._bitdecrypt(self.config.get('Connection', 'Pass'))
    #             if self.config.has_option('Connection', 'Domain'):
    #                 self.domain = self.config.get('Connection', 'Domain')
    #             if self.config.has_option('Connection', 'db_engine'):
    #                 self.db_engine = self.config.get('Connection', 'db_engine')
    #             else:
    #                 self.db_engine = 'mysql'
    #             if self.config.has_option('Connection', 'max_row_size'):
    #                 self.max_row_size = self.config.get('Connection', 'max_row_size')
    #             else:
    #                 self.max_row_size = 61440   # eigentlich 65535 in der MySQL, wir machen aber unsere Tabellenspalten
    #                                             # nicht so voll, damit beim Feldtypändern nicht direkt eine neue Tabelle angelegt werden muss
    #             if self.config.has_option('Connection', 'db_encoding'):
    #                 self.db_encoding = self.config.get('Connection', 'db_encoding')
    #             else:
    #                 if self.version == 35:
    #                     self.db_encoding = 'latin1'
    #                 else:
    #                     self.db_encoding = 'utf8'
    #
    #         if self.config.has_section('Archiving'):
    #             if self.config.has_option('Archiving', 'Server'):
    #                 self.archiveserver = self.config.get('Archiving', 'Server')
    #             if self.config.has_option('Archiving', 'Port'):
    #                 self.archiveport = self.config.getint('Archiving', 'Port')
    #
    #         if self.config.has_section('Scripts'):
    #             if self.config.has_option('Scripts', 'serverprogramm'):
    #                 self.serverprogram = self.config.get('Scripts', 'serverprogramm')
    #             if self.config.has_option('Scripts', 'serverprogram'):
    #                 self.serverprogram = self.config.get('Scripts', 'serverprogram')
    #             if self.config.has_option('Scripts', 'clientprogramm'):
    #                 self.clientprogram = self.config.get('Scripts', 'clientprogramm')
    #             if self.config.has_option('Scripts', 'clientprogram'):
    #                 self.clientprogram = self.config.get('Scripts', 'clientprogram')
    #             if self.config.has_option('Scripts', 'clientspooler'):
    #                 self.clientspooler = self.config.get('Scripts', 'clientspooler')
    #             if self.config.has_option('Scripts', 'uncclientspooler'):
    #                 self.uncclientspooler = self.config.get('Scripts', 'uncclientspooler')
    #             if self.config.has_option('Scripts', 'archive_rs'):
    #                 self.archive_rs = self.config.get('Scripts', 'archive_rs')
    #             elif self.config.has_option('Scripts', 'archivverzeichnis'):
    #                 self.archive_rs = self.config.get('Scripts', 'archivverzeichnis')
    #             if self.config.has_option('Scripts', 'archive_nrs'):
    #                 self.archive_nrs = self.config.get('Scripts', 'archive_nrs')
    #             elif self.config.has_option('Scripts', 'archivkopie'):
    #                 self.archive_nrs = self.config.get('Scripts', 'archivkopie')
    #             if self.config.has_option('Scripts', 'storages'):
    #                 self.storages = self.config.get('Scripts', 'storages').split(';')
    #             else:
    #                 self.storages = ['bfastore0']
    #             if self.config.has_option('Scripts', 'queue'):
    #                 self.queue = self.config.get('Scripts', 'queue')
    #             else:
    #                 if self.serverprogram is not None and os.path.isdir(self.serverprogram):
    #                     self.queue = os.path.join(self.serverprogram, 'queue')
    #                 else:
    #                     self.queue = os.path.join(self.mypath, 'queue')
    #             if self.config.has_option('Scripts', 'omnipage'):
    #                 self.omnipage = self.config.get('Scripts', 'omnipage')
    #             if self.config.has_option('Scripts', 'ghostscript'):
    #                 self.ghostscript = self.config.get('Scripts', 'ghostscript')
    #             if self.config.has_option('Scripts', 'gsview'):
    #                 self.gsview = self.config.get('Scripts', 'gsview')
    #             if self.config.has_option('Scripts', 'smtpserver'):
    #                 self.smtpserver = self.config.get('Scripts', 'smtpserver')
    #             if self.config.has_option('Scripts', 'smtpport'):
    #                 try:
    #                     self.smtpport = self.config.getint('Scripts', 'smtpport')
    #                 except:
    #                     self.smtpport = 25
    #             else:
    #                 self.smtpport = 25
    #             if self.config.has_option('Scripts', 'sendername'):
    #                 self.sendername = self.config.get('Scripts', 'sendername')
    #             if self.config.has_option('Scripts', 'authuser'):
    #                 self.authuser = self.config.get('Scripts', 'authuser')
    #             if self.config.has_option('Scripts', 'authpass'):
    #                 self.authpass = self.config.get('Scripts', 'authpass')
    #             #                if self.config.has_option('Scripts', 'schattenarchiv'):
    #             #                    try:
    #             #                        self.schattenarchiv = self.config.getboolean('Scripts', 'schattenarchiv')
    #             #                    except:
    #             #                        self.schattenarchiv = False
    #             if self.config.has_option('Scripts', 'duplicate_testing'):
    #                 try:
    #                     self.duplicate_testing = self.config.getboolean('Scripts', 'duplicate_testing')
    #                 except:
    #                     self.duplicate_testing = False
    #             if self.config.has_option('Scripts', 'adminmail'):
    #                 self.adminmail = self.config.get('Scripts', 'adminmail')
    #             if self.config.has_option('Scripts', 'templates'):
    #                 self.templates = self.config.get('Scripts', 'templates')
    #             if self.config.has_option('Scripts', 'workflowfile'):
    #                 self.workflowfile = self.config.get('Scripts', 'workflowfile')
    #             if self.config.has_option('Scripts', 'openofficepath'):
    #                 self.openofficepath = self.config.get('Scripts', 'openofficepath')
    #             if self.config.has_option('Scripts', 'undistributed'):
    #                 self.undistributed = self.config.get('Scripts', 'undistributed')
    #             elif self.config.has_option('Scripts', 'unverteilt'):
    #                 self.undistributed = self.config.get('Scripts', 'unverteilt')
    #             if self.config.has_option('Scripts', 'undistributed_arcid'):
    #                 try:
    #                     self.undistributed_arcid = self.config.getint('Scripts', 'undistributed_arcid')
    #                 except:
    #                     self.undistributed_arcid = None
    #             if self.config.has_option('Scripts', 'workflows'):
    #                 self.workflows_dir = self.config.get('Scripts', 'workflows')
    #             if self.config.has_option('Scripts', 'tempdir'):
    #                 self.tempdir = self.config.get('Scripts', 'tempdir')
    #             else:
    #                 self.tempdir = os.path.join(self.mypath, 'temp')
    #             if self.config.has_option('Scripts', 'garbage_id'):
    #                 try:
    #                     self.garbage_id = self.config.getint('Scripts', 'garbage_id')
    #                 except:
    #                     self.garbage_id = None
    #             if self.config.has_option('Scripts', 'duplicates_id'):
    #                 try:
    #                     self.duplicates_id = self.config.getint('Scripts', 'duplicates_id')
    #                 except:
    #                     self.duplicates_id = None
    #         if self.config.has_section('Backup'):
    #             if self.config.has_option('Backup', 'backuppath'):
    #                 self.backuppath = self.config.get('Backup', 'backuppath')
    #             if self.config.has_option('Backup', 'sqldatapath'):
    #                 self.sqldatapath = self.config.get('Backup', 'sqldatapath')
    #             if self.config.has_option('Backup', 'backuparchiv'):
    #                 try:
    #                     self.backuparchive = self.config.getboolean('Backup', 'backuparchiv')
    #                 except:
    #                     self.backuparchive = False
    #             if self.config.has_option('Backup', 'backupnrsarchiv'):
    #                 try:
    #                     self.backupnrsarchive = self.config.getboolean('Backup', 'backupnrsarchiv')
    #                 except:
    #                     self.backupnrsarchive = False
    #             if self.config.has_option('Backup', 'backupversion'):
    #                 try:
    #                     self.backupversion = self.config.getboolean('Backup', 'backupversion')
    #                 except:
    #                     self.backupversion = False
    #             if self.config.has_option('Backup', 'backupworkflows'):
    #                 try:
    #                     self.backupworkflows = self.config.getboolean('Backup', 'backupworkflows')
    #                 except:
    #                     self.backupworkflows = False
    #             if self.config.has_option('Backup', 'backupsvnrepository'):
    #                 try:
    #                     self.backupsvnrepository = self.config.getboolean('Backup', 'backupsvnrepository')
    #                 except:
    #                     self.backupsvnrepository = False
    #             if self.config.has_option('Backup', 'backupmysqllog'):
    #                 try:
    #                     self.backupmysqllog = self.config.getboolean('Backup', 'backupmysqllog')
    #                 except:
    #                     self.backupmysqllog = False
    #             if self.config.has_option('Backup', 'filesize'):
    #                 try:
    #                     self.filesize = self.config.getint('Backup', 'filesize')
    #                 except:
    #                     self.filesize = 1024*1024
    #             if self.config.has_option('Backup', 'backupinterval'):
    #                 try:
    #                     self.backupinterval = self.config.getint('Backup', 'backupinterval')
    #                 except:
    #                     self.backupinterval = 14
    #             elif self.config.has_option('Backup', 'backupzeitraum'):
    #                 try:
    #                     self.backupinterval = self.config.getint('Backup', 'backupinterval')
    #                 except:
    #                     self.backupinterval = 14
    #
    #         if self.config.has_section('Options'):
    #             if self.config.has_option('Options', 'logdir'):
    #                 self.logdir = self.config.get('Options', 'logdir')
    #             if self.config.has_option('Options', 'savepass'):
    #                 try:
    #                     self.savepass = self.config.getboolean('Options', 'savepass')
    #                 except:
    #                     self.savepass = False
    #             if self.config.has_option('Options', 'manuscanplugin'):
    #                 self.manuscanplugin = self.config.get('Options', 'manuscanplugin')
    #             if self.config.has_option('Options', 'importerplugin'):
    #                 self.importerplugin = self.config.get('Options', 'importerplugin')
    #             if self.config.has_option('Options', 'SvnInfo'):
    #                 try:
    #                     self.svninfo = self.config.getboolean('Options', 'SvnInfo')
    #                 except:
    #                     self.svninfo = False
    #             if self.config.has_option('Options', 'LinkChars'):
    #                 self.linkchars = self.config.get('Options', 'LinkChars')
    #             if self.config.has_option('Options', 'regkeyarchived'):
    #                 try:
    #                     self.regkeyarchived = self.config.getint('Options', 'regkeyarchived')
    #                 except:
    #                     self.regkeyarchived = 0
    #             if self.config.has_option('Options', 'ignorerightsatimport'):
    #                 try:
    #                     self.ignorerightsatimport = self.config.getboolean('Options', 'ignorerightsatimport')
    #                 except:
    #                     self.ignorerightsatimport = False
    #             if self.config.has_option('Options', 'usesearch35'):
    #                 try:
    #                     self.usesearch35 = self.config.getboolean('Options', 'usesearch35')
    #                 except:
    #                     self.usesearch35 = False
    #             if self.config.has_option('Options', 'resultlist_showinedit'):
    #                 try:
    #                     self.resultlist_showinedit = self.config.getboolean('Options', 'resultlist_showinedit')
    #                 except:
    #                     self.resultlist_showinedit = False
    #
    #         if self.config.has_section('History_ext'):
    #             for option in self.config.options('History_ext'):
    #                 if int(option) < 900:
    #                     raise WrongHistoryIndexError(confile)
    #                 if option in self.history_map:
    #                     raise DuplicateHistoryIndexError(confile)
    #                 else:
    #                     self.history_map[int(option)] = self.config.get('History_ext', option).decode('1252')
    #
    #         if self.config.has_section('EmptyDocs'):
    #             for option in self.config.options('EmptyDocs'):
    #                 self.emptydocs[option] = self.config.get('EmptyDocs', option).decode('1252')
    #                 self.emptydocs[option] = os.path.expandvars(self.emptydocs[option])
    #
    #     if self.config is not None:
    #         self.config = None
    #
    # def __del__(self):
    #     self.log.debug("delete bitfarm_con Object")
    #     BitfarmCon.count -= 1
    #
    # def __str__(self):
    #     return unicode(self).encode('cp1252')
    #
    # def __unicode__(self):
    #     _ret = u""
    #     if len(self.__dict__) > 0:
    #         for tupel in self.__dict__.items():
    #             if tupel[1] is not None and tupel[1] != '':
    #                 _ret += u'%s = %s' % tupel + '\n'
    #         _ret = u'Inhalt des Profils ' + self.confile + '\n' + _ret
    #     else:
    #         _ret = u'Es wurde kein Profil geladen\njob.tempname = ' + str(self.confile)
    #     return _ret
    #
    # @staticmethod
    # def _bitdecrypt(key):
    #     """
    #     Methode zum entschlüsseln des User und Passworts in einer .con-Datei
    #     """
    #     gkey = bitfarm_base.GLOBALKEY
    #     _ret = u''
    #     if len(key) > len(gkey):
    #         lilen = len(key) / 2
    #     else:
    #         lilen = len(gkey) / 2
    #
    #     pos = 0
    #     for i in range(0, len(key) / 2):
    #         _ret = _ret + unichr(int(gkey[pos * 2:pos * 2 + 2], 16) ^ int(key[i * 2:i * 2 + 2], 16))
    #         pos += 1
    #         if pos > 127:
    #             pos = 0
    #     return _ret
    #
    # @staticmethod
    # def _bitencrypt(key):
    #     """
    #     Methode zum verschlüsseln des User und Passworts
    #     """
    #     gkey = bitfarm_base.GLOBALKEY
    #
    #     _ret = u''
    #     next = ""
    #     pos = 0
    #     for i in range(0, len(key)):
    #         next = str(hex(ord(key[i]) ^ int(gkey[pos * 2:pos * 2 + 2], 16)))
    #         next = next.replace("0x", "")
    #         if len(next) == 1:
    #             next = "0" + next
    #         _ret = _ret + next
    #         pos += 1
    #         if pos > 127:
    #             pos = 0
    #     return _ret.upper()
    #
    # def read_ini(self, inifile=None):
    #     """Hilfsfunktion die die scritps.ini aus dem Programmverzeichnis (default)
    #     oder eine beliebige INI-Datei liest und in der Con-Instanz unter con.ini als BitfarmINIObject hinterlegt."""
    #     if inifile is None:
    #         inifile = os.path.join(self.bfapath, 'scripts.ini')
    #
    #     if not os.path.exists(inifile):
    #         return None
    #
    #     self.config = ConfigParser.ConfigParser(allow_no_value=True)
    #     if os.path.isfile(inifile) and os.path.exists(inifile):
    #         try:
    #             self.config.readfp(open(inifile))
    #         except:
    #             pass
    #
    #     for section in self.config.sections():
    #         self.ini[section.lower()] = BitfarmINIObject()
    #         for option in self.config.options(section):
    #             if option not in self.ini[section.lower()].__dict__:
    #                 if self.config.get(section, option).lower() == 'true':
    #                     self.ini[section.lower()].__dict__[option] = True
    #                 elif self.config.get(section, option).lower() == 'false':
    #                     self.ini[section.lower()].__dict__[option] = False
    #                 elif self.config.get(section, option).lower() == '':
    #                     self.ini[section.lower()].__dict__[option] = None
    #                 elif self.config.get(section, option).isdigit():
    #                     self.ini[section.lower()].__dict__[option] = int(self.config.get(section, option))
    #                 else:
    #                     self.ini[section.lower()].__dict__[option] = self.config.get(section, option)
    #     if self.config is not None:
    #         self.config = None

class BitfarmConnections(bitfarm_baseobj.BitfarmBaseConnections):
    """
        BitfarmConnections-Klasse. Diese Klasse stellt den Anwendungen ein oder mehrere Profile zur Verfügung.
        Das erste geladene Profil wird in BitfarmConnections.main abgelegt.
        Es können weitere Profile geladen werden in dem weitere Instanzen mit einem Profilnamen erzeugt werden.
        Auf alle Profile kann auch über den Namen zugegriffen werden  BitfarmConnections[Profil].
        Die create_db_engine-Methode erstellt eine Datenbank-Engine für alle geladenen Profile.
        Soll in Threads auf die Datenbank zugegriffen werden, so hinterlegt man innerhalb des Threads in eine lokale Variable
        ein Connection-Objekt, zB self.db = BitfarmConnections.main.engine.connect()
    @version:
    """
    version = __version__
    log = logging.getLogger("Main.BitfarmConnections")
    log.setLevel(logging.WARNING)

    def __init__(self, *name):
        """Erzeugt eine BitfarmConnections Instanz, es kann der Name / oder Pfad eine Profils übergeben werden, bzw. eine Liste von Profilen"""
        bitfarm_baseobj.BitfarmBaseConnections.__init__(self, *name)


#         self.connections = {}
#         """@ivar: Dictonary, welches alle Connections trägt
#             @type: dictonary"""
#         self.main = None
#         """@ivar: hier ist das erste geladene Profil hinterlegt, die Main.Connection. Über diese Connection wird die sys_queue und die config_tabellen erreicht
#             @type: BitfarmCon Instanz"""
#         self.profiles = []
#         """@ivar: die Namen der geladenen Profile
#         @type: list"""
#
#         if len(name) > 0:
#             if type(name[0]) == list:
#                 name = list(name[0])
#             for profile in name:
#                 if os.path.isfile(profile) and os.path.exists(profile):
#                     profilename = os.path.basename(profile)
#                     profilename = os.path.splitext(profilename)[0]
#                 else:
#                     profilename = profile
#                 profilename = profilename.lower()
#                 self.connections[profilename] = BitfarmCon(profile)
#                 self.profiles.append(profilename)
#
#                 if self.main is None:
#                     self.main = self.connections[profilename]
#
#     def __getitem__(self, name):
#         if name not in self.connections:
#             return self.main
#         else:
#             return self.connections[name]
#
#     def __iter__(self):
#         for profile in self.profiles:
#             try:
#                 yield self.connections[profile]
#             except Exception:
#                 pass
#
#     def create_db_engine(self, bind_tables=True, autocommit=True, echo=False, user=None, password=None):
#         """create_db_engine Methode: erzeugt für alle geladenen Profile eine Datenbank-Engine.
#             Optional kann bind_tables (default=True) auf False gesetzt werden. Dann werden keine Tabellen-Bindings ans con Objekt
#             vorgenommen, man muss selbst für die benötigten Bindings sorgen (bsp: self.sys_timer = sqla.Table('sys_timer', self.meta,  autoload = True))
#             oder über die Funktion bind_tables()
#         """
#         for profile in self.connections:
#             try:
#                 self.connections[profile].create_db_engine(bind_tables=bind_tables, autocommit=autocommit, echo=echo, user=user, password=password)
#             except:
#                 self.log.exception(u'Fehler bei create_db_engine')
#                 self.connections[profile].engine = None
#             else:
#                 if profile == self.main.profile:
#                     self.main.engine = self.connections[profile].engine
#
#     def connect_db(self):
#         """connect_db Methode. erzeugt für alle geladenenen Profile eine Datenbank-Connection
#         und hinterlegt diese in BitfarmConnections[profile].db bzw. BitfarmConnections.main.db
#         Achtung! sollte nicht in Multi-Thread-Anwendungen
#         verwendet werden. Dort immer in den einzelnen Threads eine eigene Connection erzeugen."""
#         for profile in self.connections:
#             try:
#                 self.connections[profile].connect_db()
#             except:
#                 self.log.exception(u'Fehler bei connect_db')
#                 self.connections[profile].db = None
#             else:
#                 if profile == self.main.profile:
#                     self.main.db = self.connections[profile].db
#
#     def bind_tables(self):
#         """bind_tables Methode. Bindet für alle geladenenen Profile die Datenbank-Tabellen an
#         die entsprechenden Attribute der BitfarmCon Objekte.
#         """
#         for profile in self.connections:
#             try:
#                 self.connections[profile].bind_tables()
#             except:
#                 self.log.exception(u'Fehler bei bind_tables')
#
#     def disconnect_db(self):
#         """disconnect_db Methode. Schliesst für alle geladenenen Profile die Datenbank-Connection."""
#         for profile in self.connections:
#             try:
#                 self.connections[profile].disconnect_db()
#             except:
#                 self.log.exception('Fehler bei disconnect_db')
#                 self.connections[profile].db = None
#             else:
#                 if profile == self.main.profile:
#                     self.main.db = None
#
#     def add(self, name):
#         """add - Methode: fügt eine weiteres Profil hinzu"""
#         if name not in self.connections:
#             self.connections[name] = BitfarmCon(name)
#             self.connections[name].create_db_engine()
#             self.profiles.append(name)
#
#     def read_ini(self):
#         """read_ini Methode: liest aus der Scripts.ini und speichert im ConfigDictonary jedes Profils"""
#         for profile in self.connections.keys():
#             self.connections[profile].read_ini()
# #
# class BitfarmINIObject(object):
#     """Hilfsklasse um eine dynamische INI-Klasse aufzubauen"""
#     version = __version__
#
#     def __init__(self):
#         self = None
#
#     def __getattr__(self, name):
#         if name in self.__dict__:
#             return self.__dict__[name]
#         else:
#             return None