#!/usr/bin/env python
# -*- coding: utf-8 -*-
u"""
Dieses Modul enthält folgende Klassen alle Datenobjekte der bitfarm-API.
Datenobjekte haben in der Regel keine Funktionen und dienen nur der Darstellung
von Entitäten. Die Datenobkjekte werden mittels der Processing Klassen
(siehe bitfarm_proc) verarbeitet.
"""

import os
import sys
import logging
#import logging.config
import decimal
import io
from collections import OrderedDict
import re
import time
import datetime

import bitfarm_baseobj


__version__ = 'v3.6.1'
__author__ = 'F.Schneider'
__copyright = 'bitfarm Informationssystem GmbH, 57072 Siegen'
__license__ = 'GPL v2'

datetime.datetime.strptime('2013-01-31 14:42:42', '%Y-%m-%d %H:%M:%S')

REQTYPELIST = ["reqSendPreview", "reqSendIev", "reqSendOri", 'respSendReady', 'respSendError', 'respSendComplete',
               'shutdownServer', 'reqAuthenticate', 'respAuth', 'respAuthError', 'reqDoc', 'reqDocTH', 'reqOri',
               'reqOriTH', 'reqIev', 'reqIevCheck', 'reqIevTH', 'reqPrevies', 'reqPreviewTH', 'reqQueuecount',
               'reqDuplicateDoc', 'reqResetOriginal', 'reqMoveDoc', 'respDoc', 'respOK', 'respError',
               'respDocError', 'respMySQLError', 'respIevCheck', 'reqIevDelete', 'reqAdminUpdate', 'respMoveError',
               'respGetHoliday', 'reqSetLogLevel', 'reqStartSocketLog', 'reqStopSocketLog', 'reqPreview', 'respIsLatest',
               'reqReadArchives', 'respArchives', 'reqReadUsers', 'respUsers', 'reqReadGroups', 'respGroups',
               'reqCreateCommentary', 'reqModifyCommentary', 'reqDeleteCommentary', 'reqReadAddfields',
               'respAddfields', 'reqImg', 'respImg', 'respImgError', 'respSendFile', 'reqSyncDoc',
               'reqCheckin', 'reqCheckout', 'reqTif', 'respTif', 'respTifError', 'reqSearch', 'respSearch', 'reqSavePreview',
               'reqEdit', 'respEditDenied', 'reqSave', 'respSaveDenied', 'respSaveError', 'reqCancel', 'respCancelDenied', 'respAccess', 'respDenied',
               'respHistory', 'reqHistory', 'reqFtxPos', 'reqShareLayer', 'respShareLayerDenied', 'respPending', 'respResetDLMError',
               'respFtxPos', 'respFtxPosError', 'reqSaveResub', 'reqSaveAdmin', 'reqSaveHoliday', 'reqGetHoliday', 'reqModifyHoliday',
               'reqDeleteHoliday', 'reqLastModified', 'respLastModified', 'reqArchive', 'reqSaveHistory', "reqGetBookmarks",
               "respGetBookmarks", "reqExecuteBookmark", "reqOcrXml", "respOcrXml", "reqRemoveLink", "reqDeleteLink", "reqResetDLM",
               "set_expired_for_archive", "recalculate_expired_for_archives", "reqCreateDocument",
               "reqLoadAccountingData", "respLoadAccountingData", "reqGetClasses", "reqAddClassDoc", "reqClassify", "respGetClasses", 
               "respAddClassDoc", "respClassify", "reqRemoveClassDoc", "respRemoveClassDoc", "respAddNamingRule", "respAutoname",
               "reqAddNamingRule", "reqAutoname", "reqDisconnectUser", "reqHeartBeat", "reqDocOcr", "reqImgOcr", "respDocOcrError", "respImgOcrError", "respOcr",
               "reqCreateDocument", "reqDuplicateDoc", "respDuplicateError", "reqCheckUpdates", "reqSendUpdates",
               "reqServerInfo", "reqUserInfo", "reqAccessable", "respAccessable", "reqReloadConfig", "reqLoggedonUser", "reqLogoutUser",
               "reqResetAddfields", "reqQueueCount", "reqPredictClass", "reqAddClass", "reqTrainClass", "reqCheckTraining", "reqSqlMapper","respSqlMapper", "reqSMTPMail"]

RIGHT_LIST = 1
RIGHT_READ = 2
RIGHT_MODIFY = 4
RIGHT_WRITE = 8
RIGHT_MAIL = 16
RIGHT_PRINT = 32
RIGHT_LINK = 64
RIGHT_IMPORT = 128
RIGHT_HISTORY = 256
RIGHT_MOVE = 512
RIGHT_AWRITE = 1024
RIGHT_AREAD = 2048
RIGHT_AMODIFY = 4096
RIGHT_APWRITE = 8192
RIGHT_APREAD = 16384
RIGHT_WORKFLOWBREAK = 32768
RIGHT_LOGREAD = 65536
RIGHT_ADMIN = 131072
RIGHT_DELETE = 262144
RIGHT_EXPORT = 524288
RIGHT_REVSAVE = 1048576
RIGHT_APMODIFY = 2097152

RESOLVE_ARCRIGHTS = OrderedDict()
RESOLVE_ARCRIGHTS[1]       = u'list'
RESOLVE_ARCRIGHTS[2]       = u'read'
RESOLVE_ARCRIGHTS[4]       = u'modify'
RESOLVE_ARCRIGHTS[8]       = u'write'
RESOLVE_ARCRIGHTS[16]      = u'mail'
RESOLVE_ARCRIGHTS[32]      = u'print'
RESOLVE_ARCRIGHTS[64]      = u'link'
RESOLVE_ARCRIGHTS[128]     = u'import'
RESOLVE_ARCRIGHTS[256]     = u'history'
RESOLVE_ARCRIGHTS[512]     = u'move'
RESOLVE_ARCRIGHTS[1024]    = u'awrite'
RESOLVE_ARCRIGHTS[2048]    = u'aread'
RESOLVE_ARCRIGHTS[4096]    = u'amodify'
RESOLVE_ARCRIGHTS[8192]    = u'apwrite'
RESOLVE_ARCRIGHTS[16384]   = u'apread'
RESOLVE_ARCRIGHTS[32768]   = u'wfbreak'
RESOLVE_ARCRIGHTS[65536]   = u'logread'
RESOLVE_ARCRIGHTS[131072]  = u'admin'
RESOLVE_ARCRIGHTS[262144]  = u'delete'
RESOLVE_ARCRIGHTS[524288]  = u'export'
RESOLVE_ARCRIGHTS[1048576] = u'revsave'
RESOLVE_ARCRIGHTS[2097152] = u'apmodify'

RIGHT_FIELD_READ = 1
RIGHT_FIELD_WRITE = 2
RIGHT_FIELD_MODIFY = 4

RESOLVE_FIELDRIGHTS = OrderedDict()
RESOLVE_FIELDRIGHTS[1] = 'read'
RESOLVE_FIELDRIGHTS[2] = 'write'
RESOLVE_FIELDRIGHTS[4] = 'modify'

ROLE_SEARCH = RIGHT_LIST | RIGHT_READ | RIGHT_AREAD | RIGHT_APREAD
ROLE_IMPORT = ROLE_SEARCH | RIGHT_MODIFY | RIGHT_IMPORT | RIGHT_LINK | RIGHT_AWRITE | RIGHT_APWRITE
ROLE_STANDARD = ROLE_IMPORT | RIGHT_WRITE | RIGHT_MAIL | RIGHT_PRINT | RIGHT_HISTORY
ROLE_POWER = ROLE_STANDARD | RIGHT_MOVE | RIGHT_REVSAVE | RIGHT_WORKFLOWBREAK | RIGHT_DELETE | RIGHT_EXPORT
ROLE_ADMIN = ROLE_POWER | RIGHT_ADMIN | RIGHT_AMODIFY | RIGHT_APMODIFY

RESOLVE_ROLE = {ROLE_SEARCH: u"Recherche",
                ROLE_IMPORT: u"Importer",
                ROLE_STANDARD: u"Standard",
                ROLE_POWER: u"Erweitert",
                ROLE_ADMIN: u"Admin"}

RESOLVE_FIELDTYPE = {0: u"Universal",
                     1: u"Datum",
                     2: u"Auswahl",
                     3: u"Mehrfachauswahl",
                     4: u"Hidden",
                     5: u"Interval",
                     6: u"Ganzzahl",
                     7: u"Fließkomma",
                     8: u"Währung",
                     11: u"Document Lifecycle",
                     12: u'DLM Datum'}

RESOLVE_SEARCHMODE = {0: u"Exakt",
                      1: u"Wortende",
                      2: u"Wortanfang",
                      3: u"Wortbestandteil"}

RESOLVE_FUNCTION = {1: u"Uhrzeit",
                      2: u"Datum",
                      3: u"Zeitstempel",
                      4: u"Benutzer",
                      5: u"Benutzer + Zeitstempel",
                      6: u"Document Lifecycle"}

RESOLVE_ARC_OPTIONS = {1: u"Titelpflicht",
                       2: u"Titel wie Archivbezeichnung",
                       4: u"Plugin aktiv"}

FLAG_ENABLED = 1
FLAG_DELETED = 2
FLAG_FROM_AD = 4
FLAG_ADMIN = 8
FLAG_SUPERIOR = 16
FLAG_MUST_CHANGE_PASSWORD = 32
FLAG_CONCURRENT = 64

FLAGS = OrderedDict()
FLAGS[1] = u'enabled'
FLAGS[2] = u'deleted'
FLAGS[4] = u'from_ad'
FLAGS[8] = u'admin'
FLAGS[16] = u'superior'
FLAGS[32] = u'must_change_pass'
FLAGS[64] = u'concurrent'

FLAGS_GROUP = OrderedDict()
FLAGS_GROUP[1] = u'enabled'
FLAGS_GROUP[2] = u'deleted'
FLAGS_GROUP[4] = u'from_ad'
FLAGS_GROUP[8] = u'concurrent'

FLAGS_FIELD = OrderedDict()
FLAGS_FIELD[1] = u'enabled'
FLAGS_FIELD[2] = u'deleted'
FLAGS_FIELD[4] = u'dlm'

FIELD_OPTIONS = OrderedDict()
FIELD_OPTIONS[0] = u"leer"
FIELD_OPTIONS[1] = u"Pflicht"
FIELD_OPTIONS[2] = u"Verknüpfung"
FIELD_OPTIONS[4] = u"Dateiname"
FIELD_OPTIONS[8] = u"Strict"

FIELD_SIZES = {0: 767, 1: 8, 2: 767, 3: 3098, 4: 767, 5: 38, 6: 8, 7: 4,  8: 8, 11: 767, 12: 8}
FIELD_TYPES = {0: 'VARCHAR(255)', 1: 'DateTime', 2: 'VARCHAR(255)', 3: 'VARCHAR(1024)', 4: 'VARCHAR(255)', 5:'VARCHAR(12)', 6: 'BIGINT', 7: 'DOUBLE', 8: 'DECIMAL(15, 2)', 11: 'VARCHAR(255)', 12: 'DateTime'}

RESUB_OPTIONS = OrderedDict()
RESUB_OPTIONS[0] = u'Einer für Alle'
RESUB_OPTIONS[1] = u'Jeder einzeln'
RESUB_OPTIONS[2] = u'Nach Erledigung zurück'
RESUB_OPTIONS[4] = u'Erinnerungsmail senden'
RESUB_OPTIONS[8] = u'deprecated'
RESUB_OPTIONS[64] = u'Benachrichtigung versenden'
RESUB_OPTIONS[128] = u'Wiedervorlage in Rückfrage'
RESUB_OPTIONS[256] = u'Abonnement'

DLM_INTERVALS = {'': 'unkonfiguriert', 'd': 'Tag(e)', 'w': 'Woche(n)', 'm': 'Monat(e)', 'y': 'Jahr(e)', 'c': 'Nie (leeren)'}

# Modul-Logger
log = logging.getLogger(__name__)
ch = logging.StreamHandler(sys.stdout)
ch.setLevel(logging.DEBUG)
formatter = logging.Formatter('%(name)s-%(levelname)s:  %(message)s')
ch.setFormatter(formatter)
ch.args = sys.stdout
log.root.addHandler(ch)
log.root.setLevel(logging.WARNING)


def bfa_strftime(_date, _format):
    if type(_date) in [datetime.datetime, datetime.date]:
        if _format.find('%Y') > -1:
            _format = _format.replace('%Y', '%04d' % _date.year)
        if _format.find('%m') > -1:
            _format = _format.replace('%m', '%02d' % _date.month)
        if _format.find('%d') > -1:
            _format = _format.replace('%d', '%02d' % _date.day)
    if type(_date) is datetime.datetime:
        if _format.find('%H') > -1:
            _format = _format.replace('%H', '%02d' % _date.hour)
        if _format.find('%M') > -1:
            _format = _format.replace('%M', '%02d' % _date.minute)
        if _format.find('%S') > -1:
            _format = _format.replace('%S', '%02d' % _date.second)
        if _format.find('%f') > -1:
            _format = _format.replace('%f', '%06d' % _date.microsecond)
    elif type(_date) is datetime.date:
        if _format.find('%H') > -1:
            _format = _format.replace('%H', '00')
        if _format.find('%M') > -1:
            _format = _format.replace('%M', '00')
        if _format.find('%S') > -1:
            _format = _format.replace('%S', '00')
        if _format.find('%f') > -1:
            _format = _format.replace('%f', '000000')

    return _format


def parse_filename(fname):
    _ret = fname
    try:
        _ret = re.sub(r'[\/:*?"<>|&]', '', fname)   # alle Zeichen die nicht in Dateinamen erlaubt sind und das UND Zeichen (damit funktioniert der Kommandozeilenaufruf nicht)
    except:
        pass
    return _ret


def ListFiles(folder, *extension):
    _ret = None

    if os.path.exists(folder):

        flist = os.walk(folder).next()
        if extension == '' or len(extension) == 0:
            _ret = flist[2]
        else:
            pythonfiles = re.compile("\w*"+extension[0]+"$", re.IGNORECASE)
            listfiltered = filter(pythonfiles.search, flist[2])
            _ret = listfiltered
        if sys.platform != "win32":
            _ret.sort()
    return _ret


def ListBasefiles(folder, *basename):
    _ret = None

    if os.path.exists(folder):

        flist = os.walk(folder).next()
        if basename == '' or len(basename) == 0:
            _ret = flist[2]
        else:

            pythonfiles = re.compile(basename[0]+".", re.IGNORECASE)
            listfiltered = filter(pythonfiles.search, flist[2])

            if not listfiltered:
                _ret = None
            else:
                _ret = listfiltered
        if sys.platform != "win32":
            _ret.sort()
    return _ret


def GetFirstFile(folder, *extension):
    _ret = None
    if os.path.exists(folder):

        flist = ListFiles(folder, *extension)
        if len(flist) > 0:
            _ret = flist[0]

    return _ret


def GetLastFile(folder, *extension):
    _ret = None
    if os.path.exists(folder):

        flist = ListFiles(folder, *extension)
        if len(flist) > 0:
            _ret = flist[len(flist)-1]

    return _ret


def GetFiFo(fifopath, basename, *len_index):
    _ret = None
    basename = basename.lower()
    highest = 0
    biggest = 0
    i = 1
    tempnumber = ''
    flist = []
    if len(len_index) == 0:
        len_index = 5
    else:
        len_index = len_index[0]

    biggest = (10 ** len_index) -1

    fifofile = GetLastFile(fifopath)
    if not fifofile:
        tempnumber = '1'
        tempnumber = tempnumber.zfill(len_index)
    else:
        strfifo = os.path.splitext(os.path.basename(fifofile))[0]

        while highest == 0:

            if strfifo.find('_0') != 0:
                strfifo = strfifo.replace('_0','')
            if strfifo.find('-archiv') != 0:
                strfifo = strfifo.replace('-archiv', '')
            if strfifo.find('-temp') != 0:
                strfifo = strfifo.replace('-temp', '')

            if len(strfifo) > len_index-1:
                strnumber = strfifo[len(strfifo)-len_index:]
                if strnumber.isdigit():
                    highest = int(strnumber)

                    if highest >= biggest:
                        if basename == 'prio':
                            fifofolder = None
                        else:
                            basename = 'prio'
                            _ret = GetFiFo(fifopath, basename)
                            return _ret

                    highest += 1
                    tempnumber = str(highest)
                    tempnumber = tempnumber.zfill(len_index)
                    time.sleep(0.1000)

                    # jetzt gegenprüfen, ob der name immer noch frei ist
                    flist = ListFiles(fifopath)
                    if flist:
                        suchedatei = re.compile(basename + tempnumber, re.IGNORECASE)
                        listfiltered = filter(suchedatei.search, flist)
#                        print listfiltered
                        if len(listfiltered):
                            #print 'Slot besetzt'
                            _ret = GetFiFo(fifopath, basename, *len_index)
                            return _ret
                        else:
                            break

            if not flist:
                flist = ListFiles(fifopath)
            i += 1
            if i <= len(flist):
                strfifo = os.path.splitext(os.path.basename(flist[-i]))[0]
            else:
                strfifo = '0'
                strfifo = strfifo.zfill(len_index)

    _ret = os.path.join(fifopath, basename + tempnumber)
    return _ret


class bfaReq(bitfarm_baseobj.BitfarmBaseObject):
    u"""Bitfarm Request-Handler Klasse. Diese Klasse wird verwendet um eine Request-Struktur aufzubauen, die als XML geparst
    zur Kommunikation zwischen bitfarm Servern oder zwischen Server und (Viewer)-Client verwendet wird.
    Sie ist NICHT identisch mit dem RequestHandler des bfaServers, wenn auch sehr ähnlich"""

    #    __slots__ = ("__reqType", "__user", "__fileName", "__dataLen", "__dataType", "__gdocId", "__page", "__revision" , "__arcid",  \
    #    "__ievType", "__ievUser",  "__pubAnno", "__privAnno", "__data", "__fields", "__tasklist", "__taskparams","__status", "annoDict")
    #
    #    __dict__ =  ("reqType", "user", "fileName", "dataLen", "dataType", "gdocId", "page", "revision" , "arcid",  \
    #    "ievType", "ievUser",  "pubAnno", "privAnno", "data", "fields", "tasklist", "taskparams","status", "annoDict")
    version = __version__

    def __init__(self):
        bitfarm_baseobj.BitfarmBaseObject.__init__(self)
        self.__reqtype = None
        self.__anno = None
        self.__data = None
        self.__fields = None
        #        self.__tasks = None

        self.resubuser = None
        """@ivar:Wiedervorlage-Benutzer (Userid) der den Request stellt, falls vom Benutzer abweichend.
        @type: int"""
        self.controllerid = None
        """@ivar:Wiedervorlage-Benutzer (userid_shed), wird bei der Rechteabfrage benötigt.
        @type: int"""
        self.filename = None
        """@ivar:Filename einer zu übermittelnden Datei.
        @type: str, unicode"""
        self.datalen = None
        """@ivar:Länge in Bytes einer Datenübertragung.
        @type: int, long"""
        self.filehash = None
        """@ivar:Hashwert einer zu übermittelnden Datei. Wird in einem Request ein filehash übergeben,
        so wird das Dokument nur geladen, wenn der aktuelle filehash der Datei unterschiedlich ist, 
        also die Datei geändert wurde.
        @type: str"""
        self.filedate = None
        """@ivar:Dateidatum einer zu übermittelnden Datei. Wird in einem Request ein filedate übergeben,
        so wird das Dokument nur geladen, wenn der Modified-Zeitstempel jünger ist (os.stat.st_mtime)
        @type: datetime.datetime"""
        self.datatype = None
        """@ivar:Bezeichner des Datentyps der Übertragung, meistens FileExtension.
        @type: str"""
        self.page = 0
        """@ivar:Die Seite des betreffenden Dokuments.
        @type: int"""
        self.revision = 0
        """@ivar:Die Revision des betreffenden Dokuments.
        @type: int"""
        self.annotype = None
        """@ivar:der Typ einer angefragten oder zu übertragenden Annotation 0 = public 1 = private.
        @type: int"""
        self.annouser = None
        """@ivar:der Benutzer (Userid) der Annotation, falls vom Benutzer abweichend.
        @type: int"""
        self.sock = None
        """@ivar: der Socket, über den der Request kam, für Rückantworten.
        @type: socket"""
        self.parameter = None
        """@ivar: optionaler Parameter, um Request weiter zu definieren
        @type: str/list"""
        self.history = []
        """@ivar: Liste aller History-Einträge, als BitfarmHistory Objekte.
        Kann mit BitfarmDocProcessing.read_history gelesen und BitfarmDocProcessing.write_history geschrieben werden"""
        self.lang = None
        """@ivar: Sprachkürzel für die Kommunikation mit dem Client.
        @type: str"""
        self.fields_arcid = None
        """@ivar: Arcid für Zusatzfelder, die zusätzlich zu den Feldern des Archives angezeigt werden sollen.
        @type: int"""
        self.symbolic_arcid = None
        """@ivar: das Archiv, in dem das Dokument liegt, wenn es ein SymbolicLink ist.
        Wird für die Rechteabfrage oder die Zusatzfelder benötigt
        @type: Integer"""
        self.in_editmode = None
        """@ivar: eine kommagetrennte Liste an gdocids der Dokumente, die momentan im EditMode sind
        @type: str"""
        self.selected_cols = None
        """@ivar: eine kommagetrennte Liste der Spaltennamen eines Resultsets einer Suche
        @type: unicode"""
        self.need_reconnect = False
        """@ivar: wird bei ausführen des Jobs ein Verlust der Datenbankverbindung erkannt, so wird need_reconnect gesetzt. IM Mainloop des verarbeitenden Threads kann dann ein reconnect versucht werden.
        @type: bool"""
        self.sessionid = None

    def __del__(self):
        pass

    def get_reqtype(self):
        return self.__reqtype

    def get_anno(self):
        return self.__tasklist

    def get_data(self):
        return self.__data

    def get_fields(self):
        return self.__fields

    #    def get_tasks(self):
    #        return self.__reqtype

    def set_reqtype(self, *args):
        if args:
            if args[0] in REQTYPELIST:
                self.__reqtype = args[0]
        else:
            self.__reqtype = None

    def set_anno(self, *args):
        if args:
            self.__anno = args[0]
        else:
            self.__anno = None

    def set_data(self, *args):
        if args:
            self.__data = args[0]
        else:
            self.__data = None

    def set_fields(self, *args):
        if args:
            self.__fields = args[0]
        else:
            self.__fields = None

        #    def set_tasks(self,  *args):
        #        if args:
        #            self.__tasks = args[0]
        #        else:
        #            self.__tasks = None

    reqtype = property(get_reqtype, set_reqtype)
    """@ivar:Der Typ des Request, muss in der REQTYPELIST eingetragen sein.
    @type: str"""
    anno = property(get_anno, set_anno)
    """@ivar:ein Liste mit BitfarmAnnotation-Objekten.
    @type: BitfarmAnnotation"""
    data = property(get_data, set_data)
    """@ivar:ein beliebiges Bitfarm-Objekt.
    @type: BitfarmDocument, BitfarmWorkflow etc."""
    fields = property(get_fields, set_fields)
    """@ivar:eine Liste von BitfarmAddfield oder BitfarmStatefield Objekten.
    @type: BitfarmAddfield"""


# tasks = property(get_tasks,  set_tasks)
#    """@ivar:eine Liste von BitfarmTask Objekten.
#    @type: BitfarmTask"""


class BitfarmSearch(object):
    """Klasse um eine Suchausdruck zu erstellen. Ein Suchausdruck kann der BitfarmSearchProcessing-Klasse übergeben werden.
    Diese übersetzt die Suchparamter für die StoredProcedures die datenbankseitig die Suche durchführen und gibt ein Resultset zurück.
    Das SearchObjekt ist so aufgebaut, dass eine XML Repräsentation ein Lesezeichen erzeugt. Es können Lesezeichen der Version 3.5
    gelesen werden, viele Properties bestehen daher aus Gründen der Abwärtskompatibiltät"""

    def __init__(self):
        self.profile = None
        """@ivar: das Profil, für das gesucht werden soll
        @type: str"""
        self.selected_cols = None
        """@ivar: die für die Suche selektierten Spalten der Tabelle, wird nichts angegeben wird * gesetzt, ansonsten Liste von Bezeichnern
        @type: list"""
        self.order = None
        """@ivar:  die Spalte nach der das Ergebnis sortiert sein soll  TODO Liste mit Spalten
        @type: list"""
        self.name = None
        """@ivar: Name der Suche/Lesezeichen
        @type: unicode"""
        self.id = None
        """@ivar: ID der Suche/Lesezeichen, entspricht in v36 dem aktuellen Index innerhalb der XML
        @type: int"""
        self.date = None
        """@ivar: Datum der Suche/Lesezeichen
        @type: datetime"""
        self.searchtype = None
        """@ivar:  Typ der  Suche, 0 = normale Suche, 1 = lokales Lesezeichen, 2 = globales Lesezeichen,  3 = Verlaufs-Lesezeichen, 4= WVL, 5 = Zusatzfeldsuche (Verschlagwortungshilfe)
        @type: int"""
        self.userid = None
        """@ivar:  die UserID, die berechtigt ist
        @type: int"""
        self.groupid = None
        """@ivar:  die GroupID, die berechtigt ist
        @type: int"""
        self.urights = None
        """@ivar: eine Liste berechtigter Usernamen, deprecated, zur abwärtskompatibilität mit 3.5, die tatsächlich vorhandenen User stehen in userid
        @type: unicode"""
        self.grights = None
        """@ivar: eine Liste berechtigter Gruppennamen, deprecated, zur abwärtskompatibilität mit 3.5, die tatsächlich vorhandenen Gruppen stehen in groupid
        @type: int"""
        self.sqlwhere = None
        """@ivar: ein sql-where  Statement, welches direkt ausgeführt werden soll
        @type: unicode"""
        self.sqlwhere_operator = None
        """@ivar: wenn sqlwhere_only ist false, und mehrere filter ist gesetzt, dann sqlwhere statement wird mit diesem operator hinzugefügt
        @type: unicode"""
        self.sqlwhere_only = False
        """@ivar: es soll nur das Where-Statement ausgeführt werden, alle anderen ggf vorhandenen Suchparametern werden ignoriert, entspricht gbadvusrsrc, führt zwangsläufig zu einer nicht-aufbauenden Suche
        @type: bool"""
        self.constructive = False
        """@ivar:  kann gesetzt werden, um zu verhindern, dass aufbauend gesucht werden kann, wird sqlwhere_only gesetzt, dann ist auch constructive gesetzt
        @type: bool"""
        self.maxvalues = None
        """@ivar: die maximalen Treffer der Suche, none = alle
        @type: int"""
        self.refsearch = None
        """@ivar:  Suchparameter Verknüpfungsname
        @type: unicode"""
        self.refsearch_negated = False
        """@ivar:  Suchparameter Verknüpfungsname negiert
        @type: bool"""
        self.searchuserid = None
        """@ivar:  Suchparameter UserID
        @type: int"""
        self.searchuser = None
        """@ivar:  Suchparameter Username (früher 'Benutzer')
        @type: unicode"""
        self.searchuser_negated = False
        """@ivar:  Suchparameter Username (früher 'notBenutzer')
        @type: bool"""
        self.searchgroupid = None
        """@ivar:  Suchparameter Gruppe ID
        @type: int"""
        self.searchgroup = None
        """@ivar:  Suchparameter Gruppe
        @type: unicode"""
        self.searchgroup_negated = False
        """@ivar:  Suchparameter Gruppe
        @type: bool"""
        self.arcid = None
        """@ivar:  Suchparameter ArchivID, kann einzelne ID oder Liste an IDs sein
        @type: int, list"""
        self.gdocid = None
        """@ivar:  Suchparameter gdocid, kann einzelne ID oder Liste an IDs sein
        @type: int, list"""
        self.gdocidwithsymlink = False
        """@ivar:  Suchparameter für Symboliclinks
        @type: bool"""
        self.timref = None
        """@ivar:  Suchparameter timref, kann einzelne ID oder Liste an IDs sein
        @type: int, list"""
        self.checkout = None
        """@ivar:  Suchparameter Ausgecheckt (früher 'iCheckout')
        @type: bool"""
        self.inresub = False
        """@ivar:  Suchparameter in Bearbeitung/auf Wiedervorlage (früher docstate & 1)
        @type: bool"""
        self.inresub_negated = False
        """@ivar: Zur Suche von Dokumenten, die nicht auf Wiedervorlage liegen
        @type: bool"""
        self.state = 0
        """@ivar:  Suchparameter Status  (früher istat)
        @type: int"""
        self.notstate = 0
        """@ivar:  Suchparameter negiert Status  (früher nistat)
        @type: int"""
        #        self.tags = None
        #        """@ivar:  Suchparameter Schlagworte    (früher slw)
        #        @type: int"""
        #        self.tags_negated = False
        #        """@ivar:  Suchparameter negiert Schlagworte    (früher notslw)
        #        @type: bool"""
        self.searchdate = None
        """@ivar:  Suchparameter Datum (früher 'Datevom')
        @type: datetime"""
        self.searchdatehigh = None
        """@ivar:  Suchparameter Datum (früher 'Datebis')
        @type: datetime"""
        self.searchdate_negated = False
        """@ivar:  Suchparameter Datum negiert  (früher 'notdatevom')
        @type: bool"""
        self.ftx_fields = None
        """@ivar:  Bitmaske - Erste Bit Volltext, zweite Bit Schlagwörte
        @type: int"""
        self.searchstring = None
        """@ivar:  Suchparameter Volltext/Schlagwörte (FtxFields Bitmaske wird auswerten), die gesamte Suchphrase wird hier eingegeben
        @type: unicode"""
        self.searchstring_negated = False
        """@ivar:  Suchparameter negiert Volltext/Schlagworte (FtxFields Bitmaske wird auswerten)
        @type: unicode"""
        self.ftx_matchagainst = False
        """@ivar:  richtige Volltextsuche verwenden (MySQL)
        @type: bool"""
        self.ftx_naturalmode = False
        """@ivar:  richtige Volltextsuche verwenden (MySQL) - Natural Mode
        @type: bool"""
        self.ftx_booleanmode = False
        """@ivar:  richtige Volltextsuche verwenden (MySQL) - Boolean Mode
        @type: bool"""
        self.ftx_withqueryexpansion = False
        """@ivar:  richtige Volltextsuche verwenden (MySQL) - With Query Expansion
        @type: bool"""
        #        self.ftxandtags = False
        #        """@ivar:  Volltextsuche auf Schlagworte erweitern
        #        @type: bool"""
        self.count = False
        """@ivar:  statt einem Suchergebnis wird nur die Anzahl als int zurückgegeben
        @type: bool"""
        self.resulttype = None
        """@ivar:  '' - Dataset, 'temp' - MySql temporary table, 'tablename' - MySql table
        @type: unicode"""
        self.direction = None
        """@ivar:  die Sortrierreihenfolge asc oder desc
        @type: str"""
        self.addfields = []
        """@ivar:  Liste mit BitfarmAddfield Objekten
        @type: list, BitfarmAddfield"""
        self.statefields = []
        """@ivar:  Liste mit BitfarmStatefield Objekten
        @type: list, BitfarmStatefield"""
        self.user_context = None
        """@ivar:  Benutzer ID der die Suche ausgefürt hat
        @type: int"""
        self.resubdate = None
        """@ivar: Suchparameter WVL Termin
        @type: datetime"""
        self.resubdate_high = None
        """@ivar: Suchparameter WVL Termin
        @type: datetime"""
        self.resubalarm = None
        """@ivar: Suchparameter WVL Erinnerung
        @type: datetime"""
        self.resubalarm_high = None
        """@ivar: Suchparameter WVL Erinnerung
        @type: datetime"""
        self.resubuserid = None
        """@ivar: Suchparameter WVL Benutzer  ID
        @type: int"""
        self.resubuserid_negated = False
        """@ivar: Suchparameter negiert WVL Benutzer  ID
        @type: bool"""
        self.resubuserid_list = None
        """@ivar: Suchparameter Userids als kommagetrennte Liste
        @type: str"""
        self.resubcontrollerid = None
        """@ivar: Suchparameter WVL Kontroller  ID
        @type: int"""
        self.resubcontrollerid_negated = False
        """@ivar: Suchparameter negiert WVL Kontroller  ID
        @type: bool"""
        self.resubstate = None
        """@ivar: Suchparameter WVL Status
        @type: int"""
        self.resubstate_negated = False
        """@ivar: Suchparameter negiert WVL Status
        @type: bool"""
        self.resubnotstate = None
        """@ivar: Suchparameter WVL nicht gesetzter Status
        @type: int"""
        self.resubdate_negated = False
        """@ivar: Suchparameter negiert WVL Termin
        @type: bool"""
        self.resubalarm_negated = False
        """@ivar: Suchparameter negiert WVL Erinnerung
        @type: bool"""
        self.docname = None
        """@ivar: Suchparameter Dokumentname
        @type: str"""
        self.docname_negated = False
        """@ivar: Suchparameter negiert Dokumentname
        @type: bool"""
        self.filetype = None
        """@ivar: Suchparameter Dateityp
        @type: str"""
        self.filetype_negated = False
        """@ivar: Suchparameter Dateityp negiert 
        @type: bool"""
        self.grouplist = []
        """@ivar: Liste mit Gruppen IDs wo der Benutzer mitglied ist
        @type: list"""
        self.userlist = []
        """@ivar: Liste mit Gruppen IDs wo der Benutzer mitglied ist
        @type: list"""
        self.usemodifieddate = None
        """@ivar: Suchparameter Bearbeitet 
        @type: bool"""
        self.searchdatetype = None
        """@ivar: Suchparameter Datum, Bitmaske die angibt nach welchem Datum gesucht wird, 4 = Datum 8 = Geändert 1024 = Löschdatum, ab v36.1 
        @type: int"""
        self.selectedarcid = None
        """@ivar: Ausgewählte ArchiveID bei der Suche (kann im Viewer wichtig sein, oder bei Lesezeichen)
        @type: int"""
        self.verify = False
        """@ivar: das Suchobjekt muss überprüft werden (zB nach einer Migration)
        @type: bool"""
        self.debug = False
        """@ivar: die Suche wird in einer temporären Tabelle gespeichert (tmp_fullsql)
        @type: bool"""
        self.abo_userid = None
        """@ivar: die Userid, deren Abonnements gesucht werden sollen, 0 = alle
        @type: int"""
        self.__guid = None
        """@ivar: eine GUID des Searchobjects, wird für in der Datenbank gespeichert Suchen (Bookmarks) verwendet. 
        @type: str (xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx)"""
        self.rightsmask = None
        """@ivar: eine Bitmask die die Archivrechte bei der Suche um weitere rechte erweitert
        @type: int"""
        self.bookmark = None
        """@ivar: das zugrundeliegende Lesezeichen bei einer aufbauenden Suche. Kommagetrennter String mit den Parametern typ, userid, groupid, index 
        @type: str"""

    def get_guid(self):
        return self.__guid

    def set_guid(self, *args):
        if self.__guid is None:
            if type(args[0]) is str:
                self.__guid = args[0]

    guid = property(get_guid, set_guid)


class BitfarmRights(object):
    """Klasse um Archivberechtigungen zu Speichern. Eine Instanz enthält den Berechtigungssatz für ein Archiv bzw. Dokument, für einen User oder eine Gruppe.
    Pro Instanz wird der Rechtesatz einmal als Bitmaske (rights) dargestellt, als auch jedes einzelne Rechte als Boolean"""

    def __init__(self):
        self.__user = None
        self.__group = None
        self.__userid = None
        self.__groupid = None
        self.__arcid = None
        self.__gdocid = None
        self.__rights = None
        self.__list = False
        self.__read = False
        self.__modify = False
        self.__write = False
        self.__mail = False
        self.__docprint = False
        self.__link = False
        self.__docimport = False
        self.__history = False
        self.__move = False
        self.__awrite = False
        self.__aread = False
        self.__amodify = False
        self.__apwrite = False
        self.__apread = False
        self.__workflowbreak = False
        self.__logread = False
        self.__admin = False
        self.__delete = False
        self.__export = False
        self.__revsave = False
        self.__apmodify = False
        self.modified = False
        self.userlist = []
        """Eine List an Userids, für die die Berechtigung gelten.
        @type: list, int"""
        self.grouplist = []
        """Eine List an Groupids, für die die Berechtigung gelten.
        @type: list, int"""

    def __str__(self):
        _ret = []
        if self.__list:
            _ret.append('list')
        if self.__read:
            _ret.append('read')
        if self.__modify:
            _ret.append('modify')
        if self.__write:
            _ret.append('write')
        if self.__mail:
            _ret.append('mail')
        if self.__docprint:
            _ret.append('docprint')
        if self.__link:
            _ret.append('link')
        if self.__docimport:
            _ret.append('docimport')
        if self.__history:
            _ret.append('history')
        if self.__move:
            _ret.append('move')
        if self.__awrite:
            _ret.append('awrite')
        if self.__aread:
            _ret.append('aread')
        if self.__amodify:
            _ret.append('amodify')
        if self.__apwrite:
            _ret.append('apwrite')
        if self.__apread:
            _ret.append('apread')
        if self.__workflowbreak:
            _ret.append('workflowbreak')
        if self.__logread:
            _ret.append('logread')
        if self.__admin:
            _ret.append('admin')
        if self.__delete:
            _ret.append('delete')
        if self.__export:
            _ret.append('export')
        if self.__revsave:
            _ret.append('revsave')
        if self.__apmodify:
            _ret.append('apmodify')
        return "BitfarmRights Mask %s Arcid %s Gdocid %s User %s Group %s\n%s" \
               % (self.__rights, self.__arcid, self.__gdocid, self.__user, self.__group, str(_ret))

    def _clear_rights(self):
        self.__list = False
        self.__read = False
        self.__modify = False
        self.__write = False
        self.__mail = False
        self.__docprint = False
        self.__link = False
        self.__docimport = False
        self.__history = False
        self.__move = False
        self.__awrite = False
        self.__aread = False
        self.__amodify = False
        self.__apwrite = False
        self.__apread = False
        self.__workflowbreak = False
        self.__logread = False
        self.__admin = False
        self.__delete = False
        self.__export = False
        self.__revsave = False
        self.__apmodify = False

    def get_list(self):
        return self.__list

    def get_read(self):
        return self.__read

    def get_modify(self):
        return self.__modify

    def get_write(self):
        return self.__write

    def get_mail(self):
        return self.__mail

    def get_docprint(self):
        return self.__docprint

    def get_link(self):
        return self.__link

    def get_docimport(self):
        return self.__docimport

    def get_history(self):
        return self.__history

    def get_move(self):
        return self.__move

    def get_awrite(self):
        return self.__awrite

    def get_aread(self):
        return self.__aread

    def get_amodify(self):
        return self.__amodify

    def get_apwrite(self):
        return self.__apwrite

    def get_apread(self):
        return self.__apread

    def get_workflowbreak(self):
        return self.__workflowbreak

    def get_logread(self):
        return self.__logread

    def get_admin(self):
        return self.__admin

    def get_delete(self):
        return self.__delete

    def get_export(self):
        return self.__export

    def get_revsave(self):
        return self.__revsave

    def get_user(self):
        return self.__user

    def get_group(self):
        return self.__group

    def get_userid(self):
        return self.__userid

    def get_groupid(self):
        return self.__groupid

    def get_arcid(self):
        return self.__arcid

    def get_gdocid(self):
        return self.__gdocid

    def get_apmodify(self):
        return self.__apmodify

    def get_rights(self):
        _ret = 0
        if self.__list:
            _ret = _ret | RIGHT_LIST
        if self.__read:
            _ret = _ret | RIGHT_READ
        if self.__modify:
            _ret = _ret | RIGHT_MODIFY
        if self.__write:
            _ret = _ret | RIGHT_WRITE
        if self.__mail:
            _ret = _ret | RIGHT_MAIL
        if self.__docprint:
            _ret = _ret | RIGHT_PRINT
        if self.__link:
            _ret = _ret | RIGHT_LINK
        if self.__docimport:
            _ret = _ret | RIGHT_IMPORT
        if self.__history:
            _ret = _ret | RIGHT_HISTORY
        if self.__move:
            _ret = _ret | RIGHT_MOVE
        if self.__awrite:
            _ret = _ret | RIGHT_AWRITE
        if self.__aread:
            _ret = _ret | RIGHT_AREAD
        if self.__amodify:
            _ret = _ret | RIGHT_AMODIFY
        if self.__apwrite:
            _ret = _ret | RIGHT_APWRITE
        if self.__apread:
            _ret = _ret | RIGHT_APREAD
        if self.__workflowbreak:
            _ret = _ret | RIGHT_WORKFLOWBREAK
        if self.__logread:
            _ret = _ret | RIGHT_LOGREAD
        if self.__admin:
            _ret = _ret | RIGHT_ADMIN
        if self.__delete:
            _ret = _ret | RIGHT_DELETE
        if self.__export:
            _ret = _ret | RIGHT_EXPORT
        if self.__revsave:
            _ret = _ret | RIGHT_REVSAVE
        if self.__apmodify:
            _ret = _ret | RIGHT_APMODIFY
        return _ret

    def set_list(self, *args):
        if args[0]:
            self.__list = True
        else:
            self.__list = False
        self.__rights = self.get_rights()

    def set_read(self, *args):
        if args[0]:
            self.__read = True
        else:
            self.__read = False
        self.__rights = self.get_rights()

    def set_modify(self, *args):
        if args[0]:
            self.__modify = True
        else:
            self.__modify = False
        self.__rights = self.get_rights()

    def set_write(self, *args):
        if args[0]:
            self.__write = True
        else:
            self.__write = False
        self.__rights = self.get_rights()

    def set_mail(self, *args):
        if args[0]:
            self.__mail = True
        else:
            self.__mail = False
        self.__rights = self.get_rights()

    def set_docprint(self, *args):
        if args[0]:
            self.__docprint = True
        else:
            self.__docprint = False
        self.__rights = self.get_rights()

    def set_link(self, *args):
        if args[0]:
            self.__link = True
        else:
            self.__link = False
        self.__rights = self.get_rights()

    def set_docimport(self, *args):
        if args[0]:
            self.__docimport = True
        else:
            self.__docimport = False
        self.__rights = self.get_rights()

    def set_history(self, *args):
        if args[0]:
            self.__history = True
        else:
            self.__history = False
        self.__rights = self.get_rights()

    def set_move(self, *args):
        if args[0]:
            self.__move = True
        else:
            self.__move = False
        self.__rights = self.get_rights()

    def set_awrite(self, *args):
        if args[0]:
            self.__awrite = True
        else:
            self.__awrite = False
        self.__rights = self.get_rights()

    def set_aread(self, *args):
        if args[0]:
            self.__aread = True
        else:
            self.__aread = False
        self.__rights = self.get_rights()

    def set_amodify(self, *args):
        if args[0]:
            self.__amodify = True
        else:
            self.__amodify = False
        self.__rights = self.get_rights()

    def set_apwrite(self, *args):
        if args[0]:
            self.__apwrite = True
        else:
            self.__apwrite = False
        self.__rights = self.get_rights()

    def set_apread(self, *args):
        if args[0]:
            self.__apread = True
        else:
            self.__apread = False
        self.__rights = self.get_rights()

    def set_workflowbreak(self, *args):
        if args[0]:
            self.__workflowbreak = True
        else:
            self.__workflowbreak = False

    def set_logread(self, *args):
        if args[0]:
            self.__logread = True
        else:
            self.__logread = False
        self.__rights = self.get_rights()

    def set_admin(self, *args):
        if args[0]:
            self.__admin = True
        else:
            self.__admin = False
        self.__rights = self.get_rights()

    def set_delete(self, *args):
        if args[0]:
            self.__delete = True
        else:
            self.__delete = False
        self.__rights = self.get_rights()

    def set_export(self, *args):
        if args[0]:
            self.__export = True
        else:
            self.__export = False
        self.__rights = self.get_rights()

    def set_revsave(self, *args):
        if args[0]:
            self.__revsave = True
        else:
            self.__revsave = False
        self.__rights = self.get_rights()

    def set_apmodify(self, *args):
        if args[0]:
            self.__apmodify = True
        else:
            self.__apmodify = False
        self.__rights = self.get_rights()

    def set_rights(self, *args):
        self._clear_rights()
        self.__list = args[0] & RIGHT_LIST == RIGHT_LIST
        self.__read = args[0] & RIGHT_READ == RIGHT_READ
        self.__modify = args[0] & RIGHT_MODIFY == RIGHT_MODIFY
        self.__write = args[0] & RIGHT_WRITE == RIGHT_WRITE
        self.__mail = args[0] & RIGHT_MAIL == RIGHT_MAIL
        self.__docprint = args[0] & RIGHT_PRINT == RIGHT_PRINT
        self.__link = args[0] & RIGHT_LINK == RIGHT_LINK
        self.__docimport = args[0] & RIGHT_IMPORT == RIGHT_IMPORT
        self.__history = args[0] & RIGHT_HISTORY == RIGHT_HISTORY
        self.__move = args[0] & RIGHT_MOVE == RIGHT_MOVE
        self.__awrite = args[0] & RIGHT_AWRITE == RIGHT_AWRITE
        self.__aread = args[0] & RIGHT_AREAD == RIGHT_AREAD
        self.__amodify = args[0] & RIGHT_AMODIFY == RIGHT_AMODIFY
        self.__apwrite = args[0] & RIGHT_APWRITE == RIGHT_APWRITE
        self.__apread = args[0] & RIGHT_APREAD == RIGHT_APREAD
        self.__workflowbreak = args[0] & RIGHT_WORKFLOWBREAK == RIGHT_WORKFLOWBREAK
        self.__logread = args[0] & RIGHT_LOGREAD == RIGHT_LOGREAD
        self.__admin = args[0] & RIGHT_ADMIN == RIGHT_ADMIN
        self.__delete = args[0] & RIGHT_DELETE == RIGHT_DELETE
        self.__export = args[0] & RIGHT_EXPORT == RIGHT_EXPORT
        self.__revsave = args[0] & RIGHT_REVSAVE == RIGHT_REVSAVE
        self.__apmodify = args[0] & RIGHT_APMODIFY == RIGHT_APMODIFY
        self.__rights = args[0]

    def set_user(self, *args):
        if self.__user is None:
            if type(args[0]) is BitfarmUser:
                self.__user = args[0]

    def set_group(self, *args):
        if self.__group is None:
            if type(args[0]) is BitfarmGroup:
                self.__group = args[0]

    def set_userid(self, *args):
        if self.__userid is None:
            if type(args[0]) is int:
                self.__userid = args[0]

    def set_groupid(self, *args):
        if self.__groupid is None:
            if type(args[0]) is int:
                self.__groupid = args[0]

    def set_arcid(self, *args):
        if self.__arcid is None:
            if type(args[0]) is int:
                self.__arcid = args[0]

    def set_gdocid(self, *args):
        if self.__gdocid is None:
            if type(args[0]) is int:
                self.__gdocid = args[0]

    list = property(get_list, set_list)
    read = property(get_read, set_read)
    modify = property(get_modify, set_modify)
    write = property(get_write, set_write)
    mail = property(get_mail, set_mail)
    docprint = property(get_docprint, set_docprint)
    link = property(get_link, set_link)
    docimport = property(get_docimport, set_docimport)
    history = property(get_history, set_history)
    move = property(get_move, set_move)
    awrite = property(get_awrite, set_awrite)
    aread = property(get_aread, set_aread)
    amodify = property(get_amodify, set_amodify)
    apwrite = property(get_apwrite, set_apwrite)
    apread = property(get_apread, set_apread)
    workflowbreak = property(get_workflowbreak, set_workflowbreak)
    logread = property(get_logread, set_logread)
    admin = property(get_admin, set_admin)
    delete = property(get_delete, set_delete)
    export = property(get_export, set_export)
    revsave = property(get_revsave, set_revsave)
    apmodify = property(get_apmodify, set_apmodify)
    rights = property(get_rights, set_rights)
    """Die Bitmaske der gesetzten Rechte. Ergibt sich aus den gesetzten Einzelberechtigungen oder kann als Bitmaske gesetzt werden,
    was die Einzelberechtigung wiederum setzt.
    @type: int"""
    user = property(get_user, set_user)
    """Der User für den die Berechtigung gilt (optional) (kann nur initial gesetzt werden)
    @type: BitfarmUser"""
    group = property(get_group, set_group)
    """Der Gruppe für die die Berechtigung gilt (optional) (kann nur initial gesetzt werden)
    @type: BitfarmGroup"""
    userid = property(get_userid, set_userid)
    """Der ID des Users, für den die Berechtigung gilt (optional) (kann nur initial gesetzt werden)
    @type: int"""
    groupid = property(get_groupid, set_groupid)
    """Der ID der Gruppe, für die die Berechtigung gilt (optional) (kann nur initial gesetzt werden)
    @type: int"""
    arcid = property(get_arcid, set_arcid)
    """Der Archiv für das die Berechtigung gilt (mandatory) (kann nur initial gesetzt werden)
    @type: int"""
    gdocid = property(get_gdocid, set_gdocid)
    """Der Dokument für das die Berechtigung gilt (optional) (kann nur initial gesetzt werden)
    @type: int"""


class BitfarmFieldRight(object):
    """Klasse um eine Feldberechtigung zu Speichern. Eine Instanz enthält den Berechtigungssatz für ein Feld. 
    Dieser Berechtigungssatz wird dann einem Addfield, einem StateField oder einer Annotation zugeordnet.
    Pro Instanz wird der Rechtesatz einmal als Bitmaske (rights) dargestellt, als auch jedes einzelne Rechte als Boolean"""

    def __init__(self):
        self.__userid = 0
        self.__groupid = 0
        self.__user = None
        self.__group = None
        self.__arcid = None
        self.__gdocid = None
        self.__fieldid = None
        self.__fieldpos = None
        self.__rights = None

        self.__read = False
        self.__write = False
        self.__modify = False

        self.modified = False
        self.deleted = False

    #    def __repr__(self):
    #        return self.__str__()

    # def __str__(self):
    #     return self

    def __unicode__(self):
        _ret = []

        if self.__read:
            _ret.append('read')
        if self.__modify:
            _ret.append('modify')
        if self.__write:
            _ret.append('write')
        if self.__user:
            user = self.__user.name
        else:
            user = u'None'
        if self.__group:
            group = self.__group.name
        else:
            group = u'None'
        return u"BitfarmFieldRight(rights %s arcid %s fieldid %s user %s group %s %s)" \
               % (self.__rights, self.__arcid, self.__fieldid, user, group, str(_ret))

    def _clear_rights(self):
        self.__read = False
        self.__write = False
        self.__modify = False
        self.__rights = 0

    def get_read(self):
        return self.__read

    def get_write(self):
        return self.__write

    def get_modify(self):
        return self.__modify

    def get_user(self):
        return self.__user

    def get_group(self):
        return self.__group

    def get_arcid(self):
        return self.__arcid

    def get_gdocid(self):
        return self.__gdocid

    def get_fieldid(self):
        return self.__fieldid

    def get_fieldpos(self):
        return self.__fieldpos

    def get_userid(self):
        return self.__userid

    def get_groupid(self):
        return self.__groupid

    def get_rights(self):
        _ret = 0
        if self.__read:
            _ret = _ret | RIGHT_FIELD_READ
        if self.__modify:
            _ret = _ret | RIGHT_FIELD_MODIFY
        if self.__write:
            _ret = _ret | RIGHT_FIELD_WRITE
        return _ret

    def set_read(self, *args):
        if args[0]:
            self.__read = True
        else:
            self.__read = False
        self.__rights = self.get_rights()

    def set_write(self, *args):
        if args[0]:
            self.__write = True
        else:
            self.__write = False
        self.__rights = self.get_rights()

    def set_modify(self, *args):
        if args[0]:
            self.__modify = True
        else:
            self.__modify = False
        self.__rights = self.get_rights()

    def set_rights(self, *args):
        self._clear_rights()
        if type(args[0]) in [set, str]:  # wundervolles Python: egal ob str oder set mit strings, es funktioniert
            self.__read = 'read' in args[0]
            self.__write = 'write' in args[0]
            self.__modify = 'modify' in args[0]

            self.__rights = 4 if self.__modify else 2 if self.__write else 1 if self.__read else 0

        else:
            if args[0] & RIGHT_FIELD_READ == RIGHT_FIELD_READ:
                self.__read = True
            else:
                self.__read = False

            if args[0] & RIGHT_FIELD_WRITE == RIGHT_FIELD_WRITE:
                self.__write = True
            else:
                self.__write = False

            if args[0] & RIGHT_FIELD_MODIFY == RIGHT_FIELD_MODIFY:
                self.__modify = True
            else:
                self.__modify = False

            self.__rights = args[0]

    def set_user(self, *args):
        if self.__user is None:
            if type(args[0]) is BitfarmUser:
                self.__user = args[0]

    def set_group(self, *args):
        if self.__group is None:
            if type(args[0]) is BitfarmGroup:
                self.__group = args[0]

    def set_arcid(self, *args):
        if self.__arcid is None:
            if type(args[0]) is int:
                self.__arcid = args[0]

    def set_gdocid(self, *args):
        if self.__gdocid is None:
            if type(args[0]) is int:
                self.__gdocid = args[0]

    def set_fieldid(self, *args):
        if self.fieldid is None:
            if type(args[0]) is int:
                self.__fieldid = args[0]

    def set_fieldpos(self, *args):
        if self.fieldpos is None:
            if type(args[0]) is int:
                self.__fieldpos = args[0]

    def set_userid(self, *args):
        if self.userid == 0:
            if type(args[0]) is int:
                self.__userid = args[0]

    def set_groupid(self, *args):
        if self.groupid == 0:
            if type(args[0]) is int:
                self.__groupid = args[0]

    rights = property(get_rights, set_rights)
    """Die Bitmaske der gesetzten Rechte. Ergibt sich aus den gesetzten Einzelberechtigungen oder kann als Bitmaske gesetzt werden,
    was die Einzelberechtigung wiederum setzt. Die möglichen Rechte (inkl Bitmask) sind: read(1), write(2), modify(4)
    @type: int"""
    userid = property(get_userid, set_userid)
    """Die Userid für die die Berechtigung gilt (optional) (kann nur initial gesetzt werden)
    @type: int"""
    groupid = property(get_groupid, set_groupid)
    """Die Groupid für die die Berechtigung gilt (optional) (kann nur initial gesetzt werden)
    @type: int"""
    user = property(get_user, set_user)
    """Der User für den die Berechtigung gilt (optional) (kann nur initial gesetzt werden)
    @type: BitfarmUser"""
    group = property(get_group, set_group)
    """Die Gruppe für die die Berechtigung gilt (optional) (kann nur initial gesetzt werden)
    @type: BitfarmGroup"""
    arcid = property(get_arcid, set_arcid)
    """Das Archiv für das die Berechtigung gilt (mandatory) (kann nur initial gesetzt werden)
    @type: int"""
    gdocid = property(get_gdocid, set_gdocid)
    """Das Dokument für das die Berechtigung gilt (optional) (kann nur initial gesetzt werden)
    @type: int"""
    fieldid = property(get_fieldid, set_fieldid)
    """Das Feld für das die Berechtigung gilt (optional) (kann nur initial gesetzt werden)
    @type: int"""
    fieldpos = property(get_fieldpos, set_fieldpos)
    """Die archivabhängige Position des Feldes (optional) (kann nur initial gesetzt werden)
    @type: int"""
    read = property(get_read, set_read)
    write = property(get_write, set_write)
    modify = property(get_modify, set_modify)


class BitfarmFieldRights(object):
    """Klasse um die Feldberechtigungen eines Feldes zu speichern. Es können ein oder mehrere BitfarmFieldRight Instanzen
    hinzugefügt werden. Über die Berechtigungssätze kann iteriert werden und per Index zugegriffen werden.
    Mit add(BitfarmFieldRight) können Feldrechte hinzugefügt werden, es ist auch eine Liste möglich.
    Mit get_right() ermittelt man gezielt ein Recht abhängig vom übergebenen Parameter"""

    def __init__(self):
        self._rights = []

    #    def __repr__(self):
    #        return self.__str__()

    def __unicode__(self):
        #        return unicode([x for x in self._rights])
        _ret = u'BitfarmFieldRights('
        for right in self._rights:
            _ret += u'%s, ' % (right)
        _ret += _ret[:-2] + u')'
        return (_ret)

    # def __str__(self):
    #     return unicode(self).encode('cp1252')

    def __del__(self):
        self._rights = []

    def __len__(self):
        return len(self._rights)

    def __getitem__(self, index):
        if type(index) is not int:
            raise TypeError
        if index < 0:
            raise IndexError
        if index >= len(self._rights):
            raise IndexError

        return self._rights[index]

    def __setitem__(self, index, value):
        if type(index) is not int:
            raise TypeError
        if index < 0:
            raise IndexError
        if index > len(self._rights):
            raise IndexError
        if type(value) is list:
            for val in value:
                if isinstance(val, BitfarmFieldRight):
                    self._rights.append(val)
        else:
            if isinstance(value, BitfarmFieldRight):
                self._rights.append(value)

    def __iter__(self):
        for _right in self._rights:
            try:
                yield _right
            except Exception:
                pass

    def __delitem__(self, index):
        self.delete(index)

    def add(self, value):
        """fügt eine BitfarmFieldRight Instanz hinzu, welche einen Berechtigungssatz trägt"""
        if type(value) is list:
            for val in value:
                if isinstance(val, BitfarmFieldRight):
                    self._rights.append(val)
        else:
            if isinstance(value, BitfarmFieldRight):
                self._rights.append(value)

    def get_right(self, arcid=None, userid=None, groupid=None):
        """gibt eine Liste an BitfarmFieldRight-Objekte zurück, entsprechend der angegebenen Paramter.
        Wird keine arcid angegeben, werden alle Objekte zurückgeliefert, ansonsten nur die der entsprechenden arcid.
        Diese können noch weiter auf userid oder groupid eingschränkt werden."""
        _ret = []
        if len(self._rights) == 0:
            return _ret
        if arcid is None:
            _ret = self._rights
        elif arcid is not None:
            for _right in self._rights:
                if _right.arcid == arcid:
                    if userid is not None:
                        if _right.userid is not None and _right.userid == userid:
                            _ret.append(_right)
                        elif _right.user is not None and _right.user.id == userid:
                            _ret.append(_right)
                    elif groupid is not None:
                        if _right.groupid is not None and _right.groupid == groupid:
                            _ret.append(_right)
                        elif _right.group is not None and _right.group.id == groupid:
                            _ret.append(_right)
                    else:
                        _ret.append(_right)
        return _ret


class BitfarmEmail(object):
    """Klasse zum Speichern von Email-Repräsentationen, wird benötigt zur eml und msg Verarbeitung. Die Klasse trägt alle Headerdaten,
    die Email-Bodies und eine Liste aller Attachements (BitfarmAttachment Instanzen)"""
    version = __version__

    def __init__(self):
        self.subject = None
        """@ivar: Betreff der Email
            @type:  unicode"""
        self.sender = None
        """@ivar: Absender der Email, wie im Header angegeben
            @type:  unicode"""
        self.recipient = None
        """@ivar: Empfänger der Email, wie im Header angegeben
            @type:  unicode"""
        self.cc = None
        """@ivar: CC der Email, wie im Header angegeben
            @type:  unicode"""
        self.bcc = None
        """@ivar: BCC der Email, wie im Header angegeben
            @type: unicode"""
        self.date = None
        """@ivar: Versanddatum der Email
            @type: datetime.dateime"""
        self.delivery_date = None
        """@ivar: Empfangsdatum (falls angegeben)
            @type: datetime.dateime"""
        self.email_content_type = None
        """@ivar: der Content-Type der gesamten Email
            @type:  str"""
        self.text_body = None
        """@ivar: Der Textbody der Email
            @type:  unicode"""
        self.text_charset = None
        """@ivar: das Charset des Text-Bodys
            @type: str"""
        self.text_transfer_encoding = None
        """@ivar: das Transfer-Encoding (falls angegeben)
            @type: str"""
        self.html_body = None
        """@ivar: der HTML-Body der Email
            @type: unicode"""
        self.html_charset = None
        """@ivar: das Charset des HTML-Bodys
            @type: str"""
        self.html_transfer_encoding = None
        """@ivar: das Transfer-Encoding (falls angegeben)
            @type: str"""
        self.messageid = None
        """@ivar: die Message-ID der Email (Header-Message-ID)
            @type: str"""
        self.rawheader = None
        """@ivar: der Email-Header als Text
            @type: str"""


# self.attachments = []
#        """@ivar: ein Liste  von BitfarmAttachment Instanzen
#            @type: list"""

class BitfarmAnnotation(object):
    """Klasse um Annotations zu speichern. Es gibt eine iev und eine svg Property um die Daten Annotation zu speichern"""
    version = __version__

    def __init__(self):
        self.text = None
        """@ivar: der Textinhalt der Annotation
            @type:  unicode"""
        self.layer = 0
        """@ivar: der Layer der Annotation. Layer 0 ist public, ansonsten die Userid der private Annotation.
        (Sollten mal Gruppen gewünscht werden, einfach einen negativen integer benutzen :-)
            @type:  int"""
        self.page = 0
        """@ivar: die Seite der Annotation
            @type:  int"""
        self.authorid = None
        """@ivar: die Userid der Erstellers der Annotation
            @type:  int"""
        self.created = None
        """@ivar: das Erstelldatum der Annotation
            @type:  datetime.datetime"""
        self.modified_date = None
        """@ivar: das Änderungsdatum der Annotation
            @type:  datetime.datetime"""
        self.iev = None
        """@ivar: die Daten der Annotation, als BitfarmStream-Objekt
            @type:  BitfarmStream"""
        self.filepath = None
        """@ivar: Pfad auf die iev-Datei in der Archivablage
            @type:  str"""
        self.svg = None
        """@ivar: der Daten der Annotation, als SVG
            @type:  str"""
        self.modified = False
        """@ivar: Flag, ob die Annotation geändert wurde.
        @type: bool"""
        self.rights = 0
        """@ivar: Der Rechtesatz für diese Annotation als Bitmask.
        @type: int"""
        self.history = []
        """@ivar: Liste aller History-Einträge, als BitfarmHistory Objekte.
        Kann mit BitfarmDocProcessing.read_history gelesen und BitfarmDocProcessing.write_history geschrieben werden"""


class BitfarmAttachment(object):
    """Klasse um ein Attachment zu speichern. Attachments sind in der Regel base64 kodiert."""
    version = __version__

    def __init__(self):
        self.attachment = None
        """@ivar: der Attachmentinhalt, entweder ein Filelike-Object oder ein ein Base64 codierter String
            @type:  str, cStringIO"""
        self.content_type = None
        """@ivar: der Content-Type des Attachments
            @type: str"""
        self.name = None
        """@ivar: der Name des Attachments (kann von filename abweichen)
            @type: unicode"""
        self.content_id = None
        """@ivar: Content-ID des Attachments
            @type: str"""
        self.filename = None
        """@ivar: der FileName des Attachments
            @type: unicode"""
        self.content_encoding = None
        """@ivar: das Content-Encoding des Attachments, in der Regel Base64
            @type: str"""
        self.inline = False
        """@ivar: Flag, ob es sich um ein Inline-Attachment handelt, falls zu ermitteln, default = False
            @type: bool"""
        self.from_email = None
        """@ivar: Flag, ob es sich um ein Email-Attachment handelt, default = False
            @type: bool"""


class BitfarmFulltext(object):
    """Klasse zum Speichern von Volltext. Das Document-Object trägt als Attribut doc.ftx eine Instanz der BitfarmFulltext Klasse.
    Der Volltext wird nach Seiten getrennt gespeichert, das Trennzeichen ist das FormFeed Steuerzeichen 0xC (ASCII 12).
    Auf die Seiten kann per Index zugegriffen werden (ftx[0]) oder iteriert werden (for page in ftx:print page).
    Der gesamte Volltext kann per ftx.text geschrieben und gelesen werden, ansonsten können per append und insert Seiten hinzugefügt
    werden, bei enthaltenen Trennzeichen wird jeweils in Einzelseiten aufgeteilt. Auf eine Einzelseite oder dem gesamten Text
    können die üblichen Stringmethoden angewandt werden.
    Volltext wird immer als Unicode gelesen oder ausgegeben. Das Standard-Encoding ist UTF-8. Das Encoding kann auch gesetzt werden,
    allerdings nur wenn die Instanz noch keinen Volltext trägt."""
    version = __version__

    def __init__(self):
        self._ftx = []
        self.__text = None
        self.__encoding = 'utf-8'
        """@ivar: hier wird das Encoding des Volltextes hinterlegt, default = utf-8
            @type: str"""

    def __del__(self):
        if self._ftx is not None:
            if type(self._ftx) == BitfarmFulltext:
                for _stream in self._ftx:
                    _stream.truncate()
                    _stream.close()

    def __len__(self):
        return len(self._ftx)

    def __getitem__(self, index):
        if type(index) is not int:
            raise TypeError
        if index < 0:
            raise IndexError
        if index >= len(self._ftx):
            raise IndexError

        return self._ftx[index].getvalue().decode(self.__encoding)

    def __setitem__(self, index, value):
        if type(index) is not int:
            raise TypeError
        if index < 0:
            raise IndexError
        if index > len(self._ftx):
            raise IndexError
        _pages = value.split(chr(12))
        _page = _pages.pop(0)
        _ftx = io.BytesIO()
        _ftx.write(_page.encode(self.__encoding, 'replace'))
        self._ftx[index] = _ftx
        _ind = 1
        for _page in _pages:
            _ftx = io.BytesIO()
            _ftx.write(_page.encode(self.__encoding, 'replace'))
            self._ftx.insert(index + _ind, _ftx)
            _ind += 1

    def __iter__(self):
        for _page in self._ftx:
            try:
                yield _page.getvalue().decode(self.__encoding)
            except Exception:
                pass

    def __delitem__(self, index):
        self.delete(index)

    def delete(self, index):
        """löscht eine Einzelseite per Index."""
        if type(index) is not int:
            raise TypeError
        if index < 0:
            raise IndexError
        if index > len(self._ftx):
            raise IndexError
        _page = self._ftx.pop(index)
        _page.truncate()
        _page.close()
        _page = None

    def append(self, value):
        """fügt ans Ende weitere Volltextseiten an, es werden bei enthaltenem Trennzeichen einzelne Seiten erzeugt."""
        _pages = value.split(chr(12))
        for _page in _pages:
            _ftx = io.BytesIO()
            _ftx.write(value.encode(self.__encoding, 'replace'))
            self._ftx.append(_ftx)

    def insert(self, index, value):
        """fügt eine Seite an die Stelle Index, es werden bei enthaltenem Trennzeichen einzelne Seiten erzeugt."""
        if type(index) is not int:
            raise TypeError
        if index < 0:
            raise IndexError
        if index > len(self._ftx):
            raise IndexError
        _ind = 0
        _pages = value.split(chr(12))
        for _page in _pages:
            _ftx = io.BytesIO()
            _ftx.write(_page.encode(self.__encoding, 'replace'))
            self._ftx.insert(index + _ind, _ftx)
            _ind += 1

    def clear(self):
        """löscht den gesamten Volltext."""
        for _page in self._ftx:
            _page.truncate()
            _page.close()
        self._ftx = []

    def count(self, value):
        """Zählt die Anzahl eines Wortes im Volltext."""
        _count = 0
        for _page in self._ftx:
            _ftx = _page.getvalue().decode(self.__encoding)
            _count += _ftx.count(value)
        return _count

    def find_all(self, value):
        """erzeugt eine Liste mit allen Fundstellen eines Wortes. Eine Fundstelle ist immer als Tupel(Seite, Position) angegeben."""
        _ret = []
        _ind = 0
        for _page in self._ftx:
            _ftx = _page.getvalue().decode(self.__encoding)
            _pos = 0
            for _hits in range(_ftx.count(value)):
                _pos = _ftx.find(value, _pos)
                if _pos != -1:
                    _ret.append((_ind, _pos))
                    _pos += 1
            _ind += 1
        return _ret

    def _get_text(self):
        if len(self._ftx) == 0:
            return u''
        else:
            return chr(12).join([_page.getvalue().decode(self.__encoding) for _page in self._ftx])

    def _set_text(self, value):
        _pages = value.split(chr(12))
        for _page in _pages:
            _ftx = io.BytesIO()
            _ftx.write(_page.encode(self.__encoding, 'replace'))
            self._ftx.append(_ftx)

    def _get_encoding(self):
        return self.__encoding

    def _set_encoding(self, _enc):
        if len(self._ftx) == 0:
            self.__encoding = _enc

    def _get_is_empty(self):
        _ret = True
        for _ftx in self._ftx:
            _ret = _ret and _ftx.tell() == 0
        return _ret

    def _get_pages(self):
        return len(self._ftx)

    encoding = property(_get_encoding, _set_encoding)
    """das Encoding des Volltextes, default ist UTF-8. Kann nur bei leerer Volltextinstanz gesetzt werden."""
    text = property(_get_text, _set_text)
    """der gesamte Volltext als Unicodestring."""
    is_empty = property(_get_is_empty)
    """gibt an, ob der Volltext leer ist oder nicht"""
    pages = property(_get_pages)
    """gibt an, wieviele Seiten der Volltext hat"""


class BitfarmStream(object):
    """Hilfsklasse zur Verwaltung von Streams. Intern wird unter BitfarmStream.stream ein cStringIO Stream zur Verfügung gestellt,
    alle Methoden von cStringIO können also auf BitfarmStream.stream angewandt werden. EIn Instanz enthält zu dem ein identifier- Property,
    der man eine eindeutige Kennung geben kann, sowie eine size-Property, die die  aktuelle Größe zurückgibt.
    Wird eine Instanz von BitfarmStream gelöscht, wird der Stream automatisch geschlossen.
    BitfarmStream-Instanzen eignen sich zum Transfer via XML, da die bfa_xml Klasse diese automatisch Base64 codiert"""
    version = __version__

    def __init__(self):
        self.identifier = ""
        """ein optionaler String, um einen Stream zu kennzeichnen
        @type: unicode"""
        self.stream = io.BytesIO()
        """ein cStringIO Stream, mit all seinen Methoden und Eigenschaften
        @type: cStringIO.StringIO"""
        self.__size = 0
        """gibt die aktuelle Größe des Streams in Byte zurück
        @type: int"""

    def __del__(self):
        self.stream.close()

    #    def _set_size:
    #        return self.__size

    def _get_size(self):
        pos = self.stream.tell()
        self.stream.seek(0, 2)
        self.__size = self.stream.tell()
        self.stream.seek(pos)
        return self.__size

    size = property(_get_size)
    """gibt die aktuelle Größe des Streams in Byte zurück
    @type: int"""


class BitfarmRect(object):
    """Hilfsklasse um Bildbereiche zu definieren. der Nullpunkt ist immer die linke, obere Ecke. Die Position kann mit Top, Bottom, Left, Right angegeben werden,
    ein Rechteck definiert man mit width und height. rotate legt fest, ob der Auschnitt noch gedreht werden soll (in Grad im Uhrzeigersinn), dpi gibt an, auf welche Auflösung
    sich die Angaben beziehen (default = None, es wird keine Umrechnung in relative Positionen durchgeführt"""
    version = __version__

    def __init__(self):
        self.top = None
        self.bottom = None
        self.left = None
        self.right = None
        self.width = None
        self.height = None
        self.rotate = None
        self.dpi = None


class BitfarmHistory(object):
    """Klasse um Bitfarm History Einträge zu kapseln, übergeben werden müssen die taskid, userid, optional können die gdocid und 1-3 Values als Unicodestring.
    Mit gdocid = 0 werden Admineinträge gespeichert (Administrator-History)
    History können einem Document-Objekt, Addfield-Objekt, Statefield-Objekt, Annotation-Objekt  oder einem bfaReq angehangen werden.
    Diese werden dann automatisch gespeichert.
    """

    def __init__(self, gdocid=None, taskid=None, userid=None, value1="", value2="", value3="", username=None):
        self._gdocid = gdocid
        self.taskid = taskid
        """@ivar: eindeutige TaskID für HISTORY_MAP
        @type: int"""
        self.userid = userid
        """@ivar: Userid des Benutzer, der den History erzeugt
        @type: int"""
        self._date = None
        """@ivar: aktueller Zeitstempel, nur lesend
        @type: int"""
        self._msg = None
        """@ivar: die Meldung als Plaintext, nur lesend
        @type: unicode"""
        self.value1 = value1
        """@ivar: die Values
        @type: unicode"""
        self.value2 = value2
        """@ivar: die Values
        @type: unicode"""
        self.value3 = value3
        """@ivar: die Values
        @type: unicode"""
        self.username = username
        """@ivar: Name des Users
        @type: unicode"""

    def _get_gdocid(self):
        return self._gdocid

    def _set_gdocid(self, value):
        if self._gdocid is None:
            self._gdocid = int(value)

    def _get_date(self):
        return self._date

    def _get_msg(self):
        return self._msg

    gdocid = property(_get_gdocid, _set_gdocid)
    date = property(_get_date)
    msg = property(_get_msg)


class BitfarmSvnField(object):
    """Klasse um SVN-Einträge zu kapseln. Alle Einträge können nur Initial gesetzt werden, außer die SVN_Info,
    die kann überschrieben werden (zB. zur Korrektur)"""

    def __init__(self):
        self._gdocid = None
        self._svn_id = None
        self._svn_version = None
        self._svn_revision = None
        self._svn_date = None
        self.svn_info = None  # Info kann überschrieben werden, alle anderen nur einmal gesetzt werden.
        self._svn_userid = None
        self._svn_link = None

    def _get_gdocid(self):
        return self._gdocid

    def _set_gdocid(self, value):
        if self._gdocid is None:
            self._gdocid = int(value)

    def _get_svn_id(self):
        return self._svn_id

    def _set_svn_id(self, value):
        if self._svn_id is None:
            self._svn_id = int(value)

    def _del_svn_id(self):
        self._svn_id = None

    def _get_svn_date(self):
        return self._svn_date

    def _set_svn_date(self, value):
        if self._svn_date is None:
            self._svn_date = value

    def _get_svn_version(self):
        return self._svn_version

    def _set_svn_version(self, value):
        if self._svn_version is None:
            self._svn_version = int(value)

    def _get_svn_revision(self):
        return self._svn_revision

    def _set_svn_revision(self, value):
        if self._svn_revision is None:
            self._svn_revision = int(value)

    def _get_svn_userid(self):
        return self._svn_userid

    def _set_svn_userid(self, value):
        if self._svn_userid is None:
            self._svn_userid = value

    def _get_svn_link(self):
        return self._svn_link

    def _set_svn_link(self, value):
        if self._svn_link is None:
            self._svn_link = value

    gdocid = property(_get_gdocid, _set_gdocid)
    svn_id = property(_get_svn_id, _set_svn_id, _del_svn_id)
    svn_date = property(_get_svn_date, _set_svn_date)
    svn_version = property(_get_svn_version, _set_svn_version)
    svn_revision = property(_get_svn_revision, _set_svn_revision)
    svn_userid = property(_get_svn_userid, _set_svn_userid)
    svn_link = property(_get_svn_link, _set_svn_link)


class BitfarmSAPField(object):
    """Klasse um SAP-Einträge zu kapseln. Alle Einträge können nur Initial gesetzt werden."""

    def __init__(self):
        self._gdocid = None
        self._sap_id = None
        self._sap_contrep = None
        self._sap_docid = None
        self._sap_compid = None
        self._sap_docprot = None
        self._sap_docstatus = None
        self._sap_tstampc = None
        self._sap_tstampm = None
        self._sap_content_path = None
        self._sap_content_type = None
        self._sap_charset = None
        self._sap_version = None

    def _get_gdocid(self):
        return self._gdocid

    def _set_gdocid(self, value):
        if self._gdocid is None:
            self._gdocid = int(value)

    def _get_sap_id(self):
        return self._sap_id

    def _set_sap_id(self, value):
        if self._sap_id is None:
            self._sap_id = int(value)

    def _del_sap_id(self):
        self._sap_id = None

    def _get_sap_contrep(self):
        return self._sap_contrep

    def _set_sap_contrep(self, value):
        if self._sap_contrep is None:
            self._sap_contrep = value

    def _get_sap_docid(self):
        return self._sap_docid

    def _set_sap_docid(self, value):
        if self._sap_docid is None:
            self._sap_docid = value

    def _get_sap_compid(self):
        return self._sap_compid

    def _set_sap_compid(self, value):
        if self._sap_compid is None:
            self._sap_compid = value

    def _get_sap_docprot(self):
        return self._sap_docprot

    def _set_sap_docprot(self, value):
        if self._sap_docprot is None:
            self._sap_docprot = value

    def _get_sap_docstatus(self):
        return self._sap_docstatus

    def _set_sap_docstatus(self, value):
        if self._sap_docstatus is None:
            self._sap_docstatus = value

    def _get_sap_tstampc(self):
        return self._sap_tstampc

    def _set_sap_tstampc(self, value):
        if self._sap_tstampc is None:
            self._sap_tstampc = value

    def _get_sap_tstampm(self):
        return self._sap_tstampm

    def _set_sap_tstampm(self, value):
        if self._sap_tstampm is None:
            self._sap_tstampm = value

    def _get_sap_content_path(self):
        return self._sap_content_path

    def _set_sap_content_path(self, value):
        if self._sap_content_path is None:
            self._sap_content_path = value

    def _del_sap_content_path(self):
        self._sap_content_path = None

    def _get_sap_content_type(self):
        return self._sap_content_type

    def _set_sap_content_type(self, value):
        if self._sap_content_type is None:
            self._sap_content_type = value

    def _get_sap_charset(self):
        return self._sap_charset

    def _set_sap_charset(self, value):
        if self._sap_charset is None:
            self._sap_charset = value

    def _get_sap_version(self):
        return self._sap_version

    def _set_sap_version(self, value):
        if self._sap_version is None:
            self._sap_version = value

    gdocid = property(_get_gdocid, _set_gdocid)
    sap_id = property(_get_sap_id, _set_sap_id, _del_sap_id)
    sap_contrep = property(_get_sap_contrep, _set_sap_contrep)
    sap_docid = property(_get_sap_docid, _set_sap_docid)
    sap_compid = property(_get_sap_compid, _set_sap_compid)
    sap_docprot = property(_get_sap_docprot, _set_sap_docprot)
    sap_docstatus = property(_get_sap_docstatus, _set_sap_docstatus)
    sap_tstampc = property(_get_sap_tstampc, _set_sap_tstampc)
    sap_tstampm = property(_get_sap_tstampm, _set_sap_tstampm)
    sap_content_path = property(_get_sap_content_path, _set_sap_content_path, _del_sap_content_path)
    sap_content_type = property(_get_sap_content_type, _set_sap_content_type)
    sap_charset = property(_get_sap_charset, _set_sap_charset)
    sap_version = property(_get_sap_version, _set_sap_version)


class BitfarmDLM(object):
    """Klasse um Document Lifecycle Einträge zu kapseln"""
    version = __version__

    def __init__(self):
        self.id = None
        self.name = None
        self.type = None
        self.fieldid = None
        self.item = None
        self.duration = None
        self.interval = None
        self.date = None
        self.prio = None
        self.mode = None
        self.modified = False
        self.flags = None


class BitfarmCommentary(object):
    """Klasse um Kommentar Einträge zu kapseln"""
    version = __version__

    def __init__(self):
        self.id = None
        self.gdocid = None
        self.userid = None
        self.tstamp = None
        self.comment = None
        self.modified = False


class BitfarmUser(object):
    """Hilfsklasse um einen User darzustellen"""
    version = __version__

    def __init__(self):
        self.name = None
        """@ivar: Username, default = None
        @type: unicode"""
        self.id = None
        """@ivar: Userid, default = None
        @type: integer"""
        self.src_userid = None
        """@ivar: Src_userid, wird benötigt, wenn die sys_docxs abgefragt wird und man den User benötigt, der die Rechte vergeben hat. default = None
        @type: integer"""
        self.displayname = None
        """@ivar: Anzeigename, default = None
        @type: unicode"""
        self.email = None
        """@ivar: Emailadresse des Users, default = None
        @type: unicode"""
        self.flags = None
        """@ivar: Kennzeichen in einer Liste zB enabled, deleted, from_ad, default = None, oder als Bitmaske
        @type: list/int"""
        self.password = None
        """@ivar: das verschlüsselte Passwort der Users
        @type: string"""
        self.profile = None
        """@ivar: das Profil, zu dem der User gehört
        @type: string"""
        self.rights = 0
        """@ivar: ein Berechtigungssatz als Bitmaske
        @type: integer"""
        self.role = 0
        """@ivar: ein Standard-Berechtigungssatz (Rolle) als Bitmaske
        @type: integer"""
        self.flags = None
        """@ivar: Kennzeichen in einer Liste zB enabled, deleted, default = None
        @type: list"""
        self.grouplist = []
        """@ivar: Eine Liste an GroupIDs, in den der User Mitglied ist
        @type: list, int"""
        self.employees = []
        """@ivar: Eine Liste an UserIDs, die Mitarbeiter des Users sind
        @type: list, int"""
        self.tempid = None
        """temoräre ID des Objekts als GUID String
        @type: str"""

    # def __repr__(self):
    #     return self.__str__()

    def __unicode__(self):
        return u"BitfarmUser([%s] %s)" % (self.id, self.name)

    # def __str__(self):
    #     #return self.name
    #     return self


class BitfarmGroup(object):
    """Hilfsklasse um Gruppen darzustellen"""
    version = __version__

    def __init__(self):
        self.name = None
        """@ivar: Name der Gruppe, default = None
        @type: unicode"""
        self.id = None
        """@ivar: ID der Gruppe, default = None
        @type: integer"""
        self.members = []
        """@ivar: Liste der Mitglieder, kann Gruppen oder User beinhalten, default = None, oder eine Liste an Userids
        @type: list mit BitfarmUser,BitfarmGroup, list mit Userids"""
        self.profile = None
        """@ivar: das Profil, zu dem die Gruppe gehört
        @type: str"""
        self.rights = 0
        """@ivar: ein Berechtigungssatz als Bitmaske
        @type: integer"""
        self.role = 0
        """@ivar: ein Standard-Berechtigungssatz (Rolle) als Bitmaske
        @type: integer"""
        self.flags = None
        """@ivar: Kennzeichen in einer Liste zB enabled, deleted, from_ad, default = None, oder als Bitmaske
        @type: list/int"""
        self.adpath = None
        """@ivar: Pfad im ADmit dem synchronisiert werden soll
        @type: list/int"""
        self.email = None
        """@ivar: Emailadresse der Gruppe, zB ein Mailverteiler
        @type: unicode"""
        self.tempid = None
        """temoräre ID des Objekts als GUID String
        @type: str"""

    def __unicode__(self):
        _ret = u"BitfarmGroup(%s" % self.name
        if type(self.members) is str:
            _ret += self.members
        else:
            _ret += u",".join([str(x) for x in self.members])
        _ret += u")"
        return _ret

    # def __str__(self):
    #     return unicode(self).encode('cp1252')

    # def __repr__(self):
    #     return self.__str__()


class BitfarmDocument(bitfarm_baseobj.BitfarmBaseObject):
    """ Bitfarm Document-Klasse:
    Diese Klasse stellt die Repräsentation eines Dokuments dar.
    Sie hat keine Methoden, nur Eigenschaften.
    """

    def __init__(self, **kargs):
        """
        Erzeugt ein leeres Document-Object.
        doc = BitfarmDocument({gdocid,tempname})
        Optional können auch die gdocid oder ein Pfad übergeben werden
        @param kargs: (optional) Dictonary mit  gdocid:Integer
                                                tempname:Pfad
                                                write_empty_values:Bool

        """
        bitfarm_baseobj.BitfarmBaseObject.__init__(self)
        self.config = None
        _gdocid = None
        _tempname = ""

        if kargs:
            if 'gdocid' in kargs:
                _gdocid = kargs['gdocid']
            if 'tempname' in kargs:
                _tempname = kargs['tempname']
            if 'write_empty_values' in kargs:
                self.write_empty_values = kargs['write_empty_values']

        self.__tempname = _tempname
        self.__path = ""
        self.gdocid = _gdocid
        self.__is_checkout = False
        self._docid = None
        """@ivar: Tabellenindex eines Dokuments in der jeweiligen arc_Tabelle  (in Version 3)
        @type: int"""
        self.tblname = None
        """@ivar: tblname aus des sys_arc (Version 3)"""
        self.archive = None
        """@ivar: Name des Archives"""
        self.title = ""
        """@ivar: Name des Dokuments"""
        self.doc_class = None
        """@ivar: Dokumentenklasse, kann von der Funktion classify ermittelt werden, falls Klassifizierungsregeln hinterlegt sind
        @type: str, unicode"""
        self.arcdate = None
        """@ivar: Archivierungsdatum, falls eins explizit gesetzt werden soll, ansonsten default = None (= aktueller Timestamp)
        @type: datetime.datetime"""
        self.expired = None
        """@ivar: Löschdatum, falls eins explizit gesetzt werden soll, ansonsten default = None 
        @type: datetime.datetime"""
        self.expired_setby = None
        """@ivar: wer das expired Datum gesetzt hat NULL = Archivierung, 0 = Admin, 1..n = Prio des DLM Eintrags, welches das Datum gesetzt hat, wobei gilt NULL < 0 < 1..n.
        Ein DLM Eintrag mit niedriger Prio (höhere Wert) darf kein Datum überschreiben mit höherer Prio (höchste Prio = 1)
        @type: int"""
        self.expired_userid = None
        """@ivar: die Userid, die das expired Datum gesetzt hat.
        @type: int"""
        self.expired_modified = False
        """@ivar: Flag, ob das expired Datum verändert wurde.
        @type: bool"""
        self.basename = ""
        """@ivar: Basisname der Datei in der Archivablage
        @type: string"""
        self.storageid = 0
        """@ivar: Index des Storage, wo sich das Dokument befindet, default = 0"""
        self.oriloc = ""
        """@ivar: Pfad der Originaldatei"""
        self.docloc = ""
        """@ivar: Pfad der Voransicht"""
        self.extloc = ""
        """@ivar: Pfad der geänderten Voransicht (Kopie)"""
        self._srcloc = ""
        """@ivar: Pfad der Quell-Datei"""
        self.tmploc = ""
        """@ivar: Pfad zu einer temporären Datei, wird beim Archivieren gelöscht (spätestens beim Löschen des Dok-Objekts)"""
        self.ocrxmlpath = ""
        """@ivar: Pfad zu einer OCR_XML Datei (Enterprise)"""
        self.txtloc = ""
        """@ivar: Pfad einer Volltext-Datei"""
        self.outfile = ""
        """@ivar: Pfad auf eine Ausgabedatei, wird mit BitfarmDocProcessing.create_outfile erzeugt. Der Pfad wird für die Taskverarbeitung benötigt wenn in einem Task
        eine neue Datei neu erzeugt wird.
        Die Ausgabedatei wird automatisch in docloc gespeichert, wenn docloc belegt ist in extloc. Soll der Pfad woanders gespeichert werden so muss das Ziel
        in Task.params.destination hinterlegt werden. Das geschieht normalerweise bei der Definition des Task, kann aber auch zur Laufzeit in einer Funktion gesetzt werden.
        """
        self.ftx = BitfarmFulltext()
        """@ivar: Volltext als Instanz von BitfarmFulltext, der Volltext wird intern als Unicode gespeichert
        @type: BitfarmFulltext, Unicode"""
        self.fulltxt = ""
        """@ivar: Volltext als Unicodestring, nur lesend, zum Speichern muss eine Instanz von doc.ftx übergeben werden.
        @type: Unicode"""
        self.tags = []
        """@ivar: Liste aller Schlagworte"""
        self.doctags = ""
        """@ivar: Schlagwörter als Unicodestring, getrennt per \n """
        self.user = ""
        """@ivar: Username"""
        self.status = 0
        """Der Status eines Dokuments als Integer, nur Version 35, ab Version 36 wird self.states benutzt
        @ivar: int"""
        self.states = {}
        """@ivar: Dictionary mit  den Status Integer (BitMaske) der einzelnen Statusspalten. readonly! 
        Da ab Version 36 mehrere Statusspalten in der Datenbank möglich sind,
        werden im Dictionary die Bitmasken der vorhandenen Statusspalten gespeichert. key = indexcolumnid
        @type: dict, int"""
        self.in_progress = 0
        """@ivar: die Userid des Benutzers , welcher das Dokument in Bearbeitung hat
        @type: int"""
        self.in_editmode = None
        """@ivar: eine kommagetrennte Liste an gdocids der Dokumente, die momentan im EditMode sind
         @type: str"""
        self.in_resub = False
        """@ivar: wird gesetzt wenn docstate_0 & 1 == 1 ist (erstes tatusbit gesetzt) oder das Dokumwent tatsächlich noch offen auf der Wiedervorlage liegt.
        @type: bool"""
        self.statusdict = OrderedDict()
        """@ivar: der Status als Dictionary {Name des Status:True/False} - deprecated
        @type:OrderedDict"""
        self.statefields = OrderedDict()
        """@ivar: die Statusfelder als Dictionary {Name des Status: BitfarmStatefield}, soll ab Version 36 benutzt werden.
        Änderung an Stati sollen nur innerhalb der Statefields gemacht werden, das Atrribut doc.states (doc.status v35) dagegen ist nur lesend.
        @type:OrderedDict, BitfarmStatefield"""
        self.statstr = None
        """@ivar: Status-Messagestring"""
        self.ocrtype = None
        """@ivar: die OCR, die das Dokument verarbeiten soll (OMP/TXT/CRE etc.)"""
        self.ocrqf = None
        """@ivar: unbekannt"""
        self.tagloc = ""
        """@ivar: Pfad auf die Schlagwortdatei"""
        self.target_date = None
        """@ivar: Termin für eine Wiedervorlage
        @type: datetime.datetime"""
        self.alarm = None
        """@ivar: Alarmzeitpunkt für eine Wiedervorlage
        @type: datetime.datetime"""
        self.editor = ""
        """@ivar: der Bearbeiter einer Wiedervorlage"""
        self.info = ""
        """@ivar: Die Info-Message zu einer Wiedervorlage"""
        self.timeroptions = None
        """@ivar: weitere Optionen Wiedervorlage (nach Beendigung zurück)"""
        self.origin = ""
        """@ivar: Importweg / Pfad des Dokuments"""
        self.filename = ""
        """@ivar: ursprünglicher Dateiname"""
        self.filetype = ""
        """@ivar: Typ des Dokuments"""
        self.sha1 = None
        """@ivar: SHA1-Hash des Dokuments"""
        self.options = None
        """@ivar: unbekannt"""
        self.links = []
        """@ivar: Liste aller Verknüpfungen
        @type: list"""
        self.filter = []
        """@ivar: abc"""
        self.fields = OrderedDict()
        """@ivar: ein Dictionary das die Zusatzfelder (AddField-Objekte) als Values trägt
        @type: dict / AddField"""
        self.fields_arcid = None
        """@ivar: die Arcid der Felder, die für dieses Dokument zusätzlich geladen werden sollen, zu den Feldern des Archives des Dokuments.
        Wird zB für die Wiedervorlage gebraucht, oder allgemein, um ein beliebigen Feldsatz pro Dokument zu laden.
        @type: int"""
        self.params = {}  # optionale Parameter
        """@ivar: Dictionary um zusätzliche, beliebige Parameter mitzuführen, als Schlüssel:Wert Paare"""
        self.barcodes = []
        """@ivar: Liste mit gefundenen Barcodes"""
        self.history = []
        """@ivar: Liste aller History-Einträge, als BitfarmHistory Objekte.
        Kann mit BitfarmDocProcessing.read_history gelesen und BitfarmDocProcessing.write_history geschrieben werden"""
        self.email = None
        """@ivar: eine BitfarmEmail-Instanz, falls das Dokument eine Email ist, wird für die eml oder msg-Verarbeitung benötigt"""
        self.svn_link = None
        """@ivar: svn_link-String des Dokuments"""
        self.svn_version = None
        """@ivar: die Version des Dokuments
        @type: int"""
        self.svn_revision = None
        """@ivar: die Revision des Dokuments
        @type: int"""
        self.svn_date = None
        """@ivar: das Datum des Dokuments
        @type: datetime.datetime"""
        self.svn_info = None
        """@ivar: Infostring zur Version des Dokuments"""
        self.svn_history = []
        """@ivar: der Versionsverlauf dieses Dokuments, eine Liste von BitfarmSvnField Objekten"""
        self.workflows = []
        """@ivar: Liste von Workflow-Objekten, die zu diesem Dokument gehören"""
        self.checkout_userid = None
        """@ivar: userid des User der das Dokument ausgecheckt hat"""
        self.is_revisionsave = False
        """@ivar: gibt an, ob das Dokument revisionssicher ist, default = False. Dokumente werden nur durchs Archivieren in den revisionssicheren Bereich übertragen
       @type: bool """
        self.is_changeable = False
        """@ivar: gibt an, ob ein Dokument zwingend im nicht-revisionssicheren Bereich gespeichert werden soll (änderbar), default = False
        @type: bool"""
        self.importpath = None
        """@ivar: gibt den Importweg (UNC-Pfad, TCP etc.) an, über den das Doc importiert wurde.
        @type: string"""
        self.annotations = []
        """@ivar: Liste aller Annotations als BitfarmAnnotation Objekte
        @type: BitfarmAnnotation"""
        self.pages = None
        """@ivar: Anzahl der Seiten bei Bilddateien
        @type: int"""
        self.file_size = None
        """@ivar: Größe der Dokumentdatei
        @type: int"""
        self.file_created = None
        """@ivar: Created-Timestamp der Datei
        @type: datetime.datetime"""
        self.file_modified = None
        """@ivar: Modiefied-Timestamp der Datei
        @type: datetime.datetime"""
        self.rights = 0
        """@ivar: der Berechtigungssatz für dieses Dokument (abhängig vom User) als Bitmaske
        @type: Integer"""
        self.symbolic_arcid = None
        """@ivar: das Archiv, in dem das Dokument liegt, wenn es ein SymbolicLink ist.
        Wird für die Rechteabfrage oder die Zusatzfelder benötigt
        @type: Integer"""
        self.symlinks = ""
        """@ivar: die Archive, als semikolon getrennter String, wo das Dokument als SymbolicLink zusätzlich liegt
        @type: string"""
        self.rights_list = []
        """@ivar: eine Liste an BitfarmRights Objekten um Benutzerrechte zu diesem Dokument zu hinterlegen
        @type: list/BitfarmRights"""
        self.abo_userid = None
        """@ivar: Liste an Userids, die das Dokument abonniert haben, als semikolongetrennter string
        @type: str"""
        self.sap = []
        """@ivar: Liste aller SAP Einträge als BitfarmSAPField Objekte
        @type: BitfarmSAPField"""
        self.commentary = []
        """@ivar: Liste aller Kommentare als BitfarmCommentary Objekte
        @type: BitfarmCommentary"""

        if self.version == 35:
            self.write_empty_values = True
        else:
            self.write_empty_values = False

    def __del__(self):
        if self.ftx is not None:
            if type(self.ftx) == BitfarmFulltext:
                for _stream in self.ftx._ftx:
                    if _stream is not None and not _stream.closed:
                        _stream.truncate()
                        _stream.close()
        if self.tmploc != '':
            if os.path.exists(os.path.expandvars(self.tmploc)):
                try:
                    os.remove(os.path.expandvars(self.tmploc))
                except:
                    pass
        del self.fields

    def __unicode__(self):
        _ret = u"BitfarmDocument"

        if self.gdocid:
            _ret += u" [%s]" % self.gdocid
        if self.archive:
            if type(self.archive) is str:
                _ret += u" Archive: %s" % self.archive
            else:
                _ret += u" Archive: %s" % self.archive
        if self.profile:
            if type(self.profile) is str:
                _ret += u" (%s)" % self.profile
            else:
                _ret += u" (%s)" % self.profile
        return _ret

    # def __str__(self):
    #     return unicode(self).encode('cp1252')

    def __repr__(self):
        _ret = "BitfarmDocument"
        if self.gdocid:
            _ret += " [%s]" % self.gdocid
        if self.q_id:
            _ret += " q_id:%s" % self.q_id
        return _ret

    def _get_tempname(self):
        return self.__tempname

    def _set_tempname(self, newname):
        if type(newname) is str:
            self.__tempname = newname
        else:
            self.__tempname = ""

    def _get_path(self):
        return self.__path

    def _set_path(self, jobpath):
        if os.path.exists(os.path.expandvars(jobpath)):
            self.__path = jobpath
        else:
            self.__path = None

        #    def _get_gdocid(self):
        #        return self.__gdocid
        #
        #    def _set_gdocid(self, _gdocid):
        #        if self.__gdocid is None:
        #            if type(_gdocid) is int:
        #                self.__gdocid = _gdocid
        #            else:
        #                self.__gdocid = None
        #
        #    def _get_arcid(self):
        #        return self.__arcid
        #
        #    def _set_arcid(self, _arcid):
        #        if type(_arcid) is int:
        #            self.__arcid = _arcid
        #        else:
        #            self.__arcid = None

    def _get_con(self):
        return self.__con

    #    def _get_profile(self):
    #        return self.__profile
    #
    #    def _set_profile(self, _profile):
    #        if _profile:
    #            self.__profile = _profile
    #        else:
    #            self.__profile = ""

    #    def _get_version(self):
    #        return self.__version
    #
    #    def _set_version(self, _version):
    #        if _version:
    #            self.__version = _version
    #        else:
    #            self.__version = 3

    def _get_is_checkout(self):
        return self.__is_checkout

    def _set_is_checkout(self, *args):
        self.__is_checkout = args[0]

    #    def _get_errortext(self):
    #        return self.__errortext
    #
    #    def _set_errortext(self, _txt):
    #        if _txt is not None:
    #            self.__errortext += _txt +'\r\n'

    #    def _get_encoding(self):
    #        return self.__encoding
    #
    #    def _set_encoding(self, _enc):
    #        if self.__encoding is None:
    #            self.__encoding = _enc

    #    queue = property(_get_queue, _set_queue)
    #    """ hier kann ein Queue-Verzeichnis hinterlegt werden, der Pfad wird beim setzen geprüft
    #    @type: str"""


    def _getsrcloc(self):
        return self._srcloc

    def _setsrcloc(self, value):
        self._srcloc = value

    def _delsrcloc(self):
        del self._srcloc

    srcloc = property(_getsrcloc, _setsrcloc, _delsrcloc, "i am the sourcelocation")

    tempname = property(_get_tempname, _set_tempname)
    """ hier wird ein Dateiname hinterlegt werden um das Dokument während der Verarbeitung (temporär) zu speichern
    @type: str"""
    path = property(_get_path, _set_path)
    """ der Ablage oder auch Quell-Pfad des Dokuments während der Verarbeitung. Die hinterlegte Datei muss existieren, ansonsten wird None zurückgegeben
    @type: str"""
    #    gdocid = property(_get_gdocid, _set_gdocid)
    #    """ die gdocid des Dokuments (readonly). Kann nicht gesetzt werden, sondern wird beim Laden (load) angegeben oder beim Erzeugen (create) erzeugt
    #    @type: int"""
    #    arcid = property(_get_arcid, _set_arcid)
    #    """ die Archiv-ID des aktuellen Dokuments
    #    @type: int"""
    #    profile = property(_get_profile,  _set_profile)
    #    """das aktuelle Profil  dem das Dokument zugehörig ist
    #    @type: str"""
    #    version = property(_get_version,  _set_version)
    #    """die Version des Dokuments, aktuell unterstützt ist Version 3
    #    @type: int"""
    is_checkout = property(_get_is_checkout, _set_is_checkout)
    """Gibt an ob das Dokument ausgecheckt ist (readonly). Dokument können über die Methoden doc.checkout() und doc.checkin()
    ein und ausgecheckt werden
    @type: bool"""


# errortext = property(_get_errortext,  _set_errortext)
#    """Fehlermeldungen die während der Taskverarbeitung erzeugt werden
#    @type: unicode"""
#    encoding = property(_get_encoding, _set_encoding)
#    """Das Encoding womit das Dokument geladen wurde. Wird bei Laden aus der Datenbank automatisch gesetzt,
#    entspricht dem Datenbank-Encoding (BitfarmCon.db_encoding)"""


class BitfarmStatefield(object):
    """Bitfarm Statusfeld-Klasse
    Diese Klasse stellt Statusfelder zur Verfügung. Diese Klasse soll ab Version 3.6 verwendet werden.
    Die Statusfelder eines Dokumentes werden als Dictonary mit einzelnen Statusfelder-Instanzen geführt,
    Key ist der Statusfeldname

    @version:
    """
    log = logging.getLogger("Main.BitfarmStatefield")
    log.setLevel(logging.WARNING)
    log.propagate = 0
    count = 0
    version = __version__

    def __init__(self, ):
        self.name = None
        """Statusfeldname
        @type: unicode"""
        self.id = None
        """ID des Statusfeldes.
        @type: int"""
        self.arcid = None
        """Archiv, wofür dieser Status gilt.
        @type: int"""
        self.gdocid = None
        """Dokument, wofür dieser Status gilt.
        @type: int"""
        self.__value = None
        """Wert des Status  True/False
        @type: bool"""
        self.__oldvalue = None
        """Initialwert des Feldes, wird beim Laden des Feldes einmalig gesetzt
        @type: bool"""
        self.__bitmask = None
        """Bitmaske des Statusfeldes, welches Bit im Status-Integer diesem Status entspricht
        @type: int"""
        self.modified = False
        """wurde das Feld geändert?
        @type: bool"""
        self.modified_search = False
        """wurde das Feld zur Suche geändert?
        @type: bool"""
        self.addfield = None
        """ein korrespondierendes Zusatzfeld, zB. für eine Function
        @type: str"""
        self.addfieldid = None
        """die ID des korrespondierenden Zusatzfeld, zB. für eine Function
        @type: int"""
        self.func = None
        """eine hinterlegte Funktion, Funktionen müssen noch definiert werden
        @type: int"""
        self._function = None
        """alte Schreibweise, deprecated
        @type: int"""
        self.rights = None
        """Ein Berechtigungssatz für das Feld.
        @type: BitfarmFieldRights"""
        self.indexcolumnid = 0
        """Die Indexspalte, auf die sich die Bitmask bezieht, default = 0.
        @type: int"""
        self.or_junction = False
        """Wert des Feldes wird per OR Verknüpfung der Suche hinzugefügt
        @type: bool"""
        self.history = []
        """@ivar: Liste aller History-Einträge, als BitfarmHistory Objekte.
        Kann mit BitfarmDocProcessing.read_history gelesen und BitfarmDocProcessing.write_history geschrieben werden"""
        self.tempid = None
        """temoräre ID des Feldes als GUID String
        @type: str"""
        self.flags = None
        """Flags des Statefields
        @type: int oder list"""


    def __unicode__(self):
        return u"BitfarmStatefield(%s=%s Mask %s)" % (self.name, self.value, self.bitmask)

    # def __str__(self):
    #     return unicode(self).encode('cp1252')

    # def __repr__(self):
    #     return self.__str__()

    def get_value(self):
        return self.__value

    def get_oldvalue(self):
        return self.__oldvalue

    def get_bitmask(self):
        return self.__bitmask

    def get_function(self):
        return self.func

    def set_value(self, *args):
        if len(args) > 0:
            if type(args[0]) is bool:
                self.__value = args[0]
                if self.__oldvalue is not None:
                    self.modified = self.__value != self.__oldvalue
                else:
                    self.modified = True
                    self.__oldvalue = args[0]

    def set_oldvalue(self, *args):
        if len(args) > 0:
            if type(args[0]) is bool and self.__oldvalue is None:
                self.__oldvalue = args[0]

    def set_bitmask(self, *args):
        if args[0]:
            if self.__bitmask is None:
                if type(args[0]) is int:
                    self.__bitmask = args[0]

    def set_function(self, *args):
        self.func = args[0]

    value = property(get_value, set_value)
    oldvalue = property(get_oldvalue, set_oldvalue)
    bitmask = property(get_bitmask, set_bitmask)
    function = property(get_function, set_function)


class BitfarmAddfield(object):
    """Bitfarm Zusatzfeldklasse.
    Diese Klasse stellt Zusatzfelder zur Verfügung. Zusatzfelder haben folgende Eigenschaften
    .name = Name des Zusatzfeldes
    .value = Wert des Zusatzfeldes, Liste
    .type = type des Zusatzfeldes entsprechend cfg_Tabelle
    .add = Add_Bezeichner der Datenbank
    .checkprogram = beim Setzen des Wertes wird dieses Programm ausgeführt und der Wert als Parameter übergeben,
                    die Rückgabe über stdout wird eingelesen
    .format =   eine der vordefiniereten Formateigenschaften oder ein Formatstring, der auf das Zusatzfeld angewendet wird
                folgende Vorbelegungen sind möglich     
                - dropemptystrings (default)
                - dropdoublet (default)
                - deletespaces
                - deletedashes
                - deletedots
                - numeric
                - continue (Werte werden aufgelistet und beim Schreiben mit ; zusammengefügt)
    Die Zusatzfelder eines Dokumentes werden als Dictonary mit einzelnen Addfields-Instanzen geführt,
    Key ist der Zusatzfeldname

    @version:
    """
    log = logging.getLogger("Main.BitfarmAddfield")
    log.setLevel(logging.WARNING)
    log.propagate = 0
    count = 0
    version = __version__

    def __init__(self, _encoding='utf-8'):
        #        self.log.debug("Initialisiere ein bitfarm_addfield Object")
        self.name = None
        """Zusatzfeldname
        @type: unicode"""
        self.__value = None
        """Wert des Zusatzfeldes, Typ entspricht dem des Datenbankfeldes
        @type: int, float, decimal, str, unicode, datetime"""
        self.valuehigh = None
        """Wert des oberen Zusatzfeldes zur Suche
        @type: int, float, decimal, str, unicode, datetime"""
        self.oldvalue = None
        """Initial-Wert des Zusatzfeldes wenn es angezeigt wird, wird verwendet beim Cancel
        @type: int, float, decimal, str, unicode, datetime"""
        self.negated = False
        """Wert des Zusatzfeldes wird negiert gesucht
        @type: bool"""
        self.or_junction = False
        """Wert des Feldes wird per OR Verknüpfung der Suche hinzugefügt
        @type: bool"""
        self.add = None
        """Add_xxx Feld-Bezeichner in der Datenbank
        @type: str"""
        self.id = 0
        """Eindeutige Feld ID ab Version 36
        @type: int"""
        self.type = 0
        """Typ entsprechend cfg_Tabelle 0=Universal, 1 = Datum, 2=Auswahl, 3=Mehrfachauswahl, 4 = invisible,5 = Interval, 6=Ganzzahl,7=Fließkomma, 8=Währung, 9=nicht vergeben, 10=nicht vergeben, 11 DLM Auswahl, 12 = DLM-Datum
        @type: int"""
        self.mode = 0
        """Suchmodus 0 = Exakt, 1 = Wortende, 2 = Wortanfang, 3 = Bestandteil
        @type: int"""
        self.checkprogram = None
        """optionales Checkprogramm zur Eingabeverarbeitung im Viewer
        @type: unicode"""
        self.description = None
        """optionale beschreibung zum Feld
        @type:  unicode"""
        self.__format = []
        """Formatierungseinstellungen für das Feld, werden beim Setzen des Value berücksichtigt ['dropemptystrings', 'dropdoublet', 'deletedashes', 'deletespaces', 'deletedots', 'deleteparanthese','numeric']
        @type: list"""
        self.options = 0
        """Zusatzfeld-Optionen entsprechend cfg_Tabelle als Bitmaske 1=Pflichtfeld, 2=Verknüpfung, 4=Dateiname bei Versand/Speichern, 8=Strictmode, 16=Auf zweiten Tab anzeigen
        @type: int"""
        self.defaultvalue = None
        """Vorgabewert bei Zusatzfelder (zur Anzeige)
        @type: unicode"""
        self.items = []
        """Werte bei Auswahlfelder
        @type: list"""
        self.pos = 0
        """Anzeigeposition im Viewer
        @type: int"""
        self.modified = False
        """wurde das Feld geändert
        @type: bool"""
        self.modified_search = False
        """wurde das Feld geändert, der Wert wird aber nur für die Suche verwendet.
        @type: bool"""
        self._encoding = _encoding
        """Das Encoding für value.count
        @type: str"""
        self.rights = None
        """Berechtigungen für das Feld als BitfarmFieldRights Instanz.
        @type: BitfarmFieldRights"""
        self.indextableid = 0
        """Der Index der Indexdatentabelle für dieses Zusatzfeld.
        @type: int"""
        self.history = []
        """@ivar: Liste aller History-Einträge, als BitfarmHistory Objekte.
        Kann mit BitfarmDocProcessing.read_history gelesen und BitfarmDocProcessing.write_history geschrieben werden"""
        self.values_query = None
        """für zukünftige Feldinterne Suchen, enthält das XML eines SearchObjects
        @type: str"""
        self.tempid = None
        """temoräre ID des Feldes als GUID String
        @type: str"""
        self.flags = None
        """Flags des Addfields
        @type: int oder list"""

        BitfarmAddfield.count += 1


    def __del__(self):
        #        self.log.debug("delete bitfarm_addfield Object")
        BitfarmAddfield.count -= 1

    # def __str__(self):
    #     return unicode(self).encode('cp1252')

    def __unicode__(self):
        _ret = u"BitfarmAddfield("
        _ret += u"Name:%s" % self.name if self.name is not None else ""
        if self.value is not None:
            _ret += u", Value:%s" % self.value
        if self.add is not None:
            _ret += u", Field:%s" % self.add
        if self.type is not None:
            _ret += u", Type:%i" % self.type
        _ret += u")"
        return _ret

#    def __repr__(self):
#        _ret = u"BitfarmAddfield("
#        _ret += u"Name:%s" % self.name if self.name is not None else ""
#        _ret += u")"
#        return _ret.encode('latin-1')

    def get_value(self):
        return self.__value

    def set_value(self, *args):
        if args:
            value = args[0]

            # if type(value) in [str]:
            #     if type(value) is str:
            #         value = value.decode(self._encoding)

            self.modified = not value == self.oldvalue

            if 'continue' in self.format:
                if type(self.__value) is not list:
                    self.__value = [self.__value]
                if type(value) is list:
                    self.__value.extend(value)
                else:
                    self.__value.append(value)
                self.formatvalue()
            else:
                self.__value = value
                self.formatvalue()

    def get_format(self):
        return self.__format

    def set_format(self, *args):
        """
        die ist der Setter um die Formatproperty zu setzen. Sie prüft, ob das ankommende Format 
        in 'dropemptystrings', 'dropdoublet', 'deletedashes', 'deletespaces', 'deletedots', 'deleteparanthese','numeric'
        vorhanden ist."""
        if args:
            format = []
            format.extend(args[:])  # auch liste von formaten möglich
            for f in format:
                if f is None:
                    format.remove(None)
            if format not in ['dropemptystrings', 'dropdoublet', 'deletedashes', 'deletespaces', 'deletedots',
                              'deleteparanthese', 'numeric']:
                self.__format.extend(format)
            #        else:
            #            self.__format.extend(['dropemptystrings', 'dropdoublet'])

    def clear(self):
        self.__value = None
        self.oldvalue = None
        self.modified = False

    def formatvalue(self, *args):
        """
        führt eine Formatierung gemäß der format-Eigenschaft für einen Wert durch.
        Diese Funktion wird durchgeführt wenn ein Wert in ein Zusatzfeld geschrieben wird,
        somit hat dieser Wert direkt das richtige Format
        """
        values = []

        if args:
            if type(args[0]) is list:
                values = args[0]
            else:
                values = [args[0]]
        else:
            if type(self.__value) is list:
                values.extend(self.__value)
            else:
                values = [self.__value]

        while None in values:
            values.remove(None)

        for value in values:
            if type(value) not in [datetime.datetime, int, float, decimal.Decimal]:
                for i in range(len(values)):
                    if type(values[i]) is str:
                        values[i] = values[i].strip()
                        if '\'' in values[i]:
                            values[i] = values[i].replace('\'', '')

                if 'dropdoublet' in self.__format:
                    for val in values:
                        if values.count(val) > 1:
                            self.log.debug('Dubletten entfernen')
                            values.remove(val)

                if 'dropemptystrings' in self.__format:
                    for val in values:
                        if val == '':
                            self.log.debug('leerstring entfernen')
                            values.remove('')

                if 'deletespaces' in self.__format:
                    for i in range(len(values)):
                        if ' ' in values[i]:
                            self.log.debug('leerzeichen entfernen')
                            values[i] = values[i].replace(' ', '')
                        if '\t' in values[i]:
                            self.log.debug('leerzeichen entfernen')
                            values[i] = values[i].replace('\t', '')

                if 'deletedots' in self.__format:
                    for i in range(len(values)):
                        if '.' in values[i]:
                            self.log.debug('punkte entfernen')
                            values[i] = values[i].replace('.', '')
                        if ',' in values[i]:
                            self.log.debug('punkte entfernen')
                            values[i] = values[i].replace(',', '')
                        if ':' in values[i]:
                            self.log.debug('punkte entfernen')
                            values[i] = values[i].replace(':', '')
                        if ';' in values[i]:
                            self.log.debug('punkte entfernen')
                            values[i] = values[i].replace(';', '')

                if 'deletedashes' in self.__format:
                    for i in range(len(values)):
                        if '\\' in values[i]:
                            self.log.debug('backslash entfernen')
                            values[i] = values[i].replace('\\', '')
                        if '/' in values[i]:
                            self.log.debug('slash entfernen')
                            values[i] = values[i].replace('/', '')
                        if '|' in values[i]:
                            self.log.debug('pipe entfernen')
                            values[i] = values[i].replace('|', '')
                        if '-' in values[i]:
                            self.log.debug('dash entfernen')
                            values[i] = values[i].replace('-', '')

                if 'deleteparanthese' in self.__format:
                    for i in range(len(values)):
                        if '(' in values[i]:
                            self.log.debug('klammer entfernen')
                            values[i] = values[i].replace('(', '')
                        if ')' in values[i]:
                            self.log.debug('klammer entfernen')
                            values[i] = values[i].replace(')', '')
                        if '[' in values[i]:
                            self.log.debug('klammer entfernen')
                            values[i] = values[i].replace('[', '')
                        if ']' in values[i]:
                            self.log.debug('klammer entfernen')
                            values[i] = values[i].replace(']', '')
                        if '{' in values[i]:
                            self.log.debug('klammer entfernen')
                            values[i] = values[i].replace('{', '')
                        if '}' in values[i]:
                            self.log.debug('klammer entfernen')
                            values[i] = values[i].replace('}', '')
                if 'numeric' in self.__format:
                    for i in range(len(values)):
                        values[i] = "".join([x for x in values[i] if x.isdigit() or x in ['.', ',']])

        for format in self.__format:
            if format not in ['dropemptystrings', 'dropdoublet', 'deletedashes', 'deletespaces', 'deletedots',
                              'deleteparanthese', 'numeric']:
                if format[0:9] == 'cutstring':
                    cut = format[10:-1]
                    a, b = cut.split(':')
                    if a.isdigit():
                        a = int(a)
                    else:
                        a = None
                    if b[:1] == '-':
                        b = b[1:]
                        if b.isdigit():
                            b = int(b) * -1
                        else:
                            b = None
                    elif b.isdigit():
                        b = int(b)
                    else:
                        b = None

                    for i in range(len(values)):
                        values[i] = values[i][a:b]

        if args:
            if len(values) == 0:
                return None
            elif len(values) == 1:
                return values[0]
            else:
                return values
        else:
            if len(values) == 0:
                return None
            elif len(values) == 1:
                self.__value = values[0]
            else:
                self.__value = values

    value = property(get_value, set_value)
    format = property(get_format, set_format)


class BitfarmBarcode(object):
    """Hilfsklasse, um Zugriff auf Barcodes zu erleichtern. Das Ergebniss eines Barcodlesevorgangs kann in
    einzelnen BitfarmBarcode-Instanzen gespeichert werden, es kann aber auch die Konfiguration damit übergeben werden.
    Die gefunden Barcodes können später in der naming-Funktion mittels entsprechender GetField-Anweisungen (ein searchbarcode muss
    angegeben sein) in beliebige Zusatzfelder gemappt werden (sowie linkstring, addtag etc.)"""
    version = __version__

    def __init__(self):
        self.barcode = None
        """@ivar: Klartextbezeichner des Barcodes", default = None
        @type: str, unicode"""
        self.keybarcode = False
        """@ivar: Trennbarcode, default = False
        @type: bool"""
        self.filebarcode = False
        """@ivar: Aktenbarcode, bleibt als Information auf allen getrennten Dokumenten, bis der nächste Aktenbarcode gefunden wird, default = False
        @type: bool"""
        self.stackbarcode = False
        """@ivar: Stapelbarcode, wird ein Stapelbarcode erkannt, werden alle folgenden Seiten vereinzelnd, default = False
        @type: bool"""
        self.dropbarcode = False
        """@ivar: zum Verwerfen von Trennseiten, default = False
        @type: bool"""

        self.page = None
        """@ivar: die Seite auf denen der Barcode gefunden wurde
        @type: integer"""
        self.type = None
        """@ivar: Typ des Barcodes.
        @type: str"""


# class BitfarmConnections(object):
#     """
#         BitfarmConnections-Klasse. Diese Klasse stellt den Anwendungen ein oder mehrere Profile zur Verfügung.
#         Das erste geladene Profil wird in BitfarmConnections.main abgelegt.
#         Es können weitere Profile geladen werden in dem weitere Instanzen mit einem Profilnamen erzeugt werden.
#         Auf alle Profile kann auch über den Namen zugegriffen werden  BitfarmConnections[Profil].
#         Die create_db_engine-Methode erstellt eine Datenbank-Engine für alle geladenen Profile.
#         Soll in Threads auf die Datenbank zugegriffen werden, so hinterlegt man innerhalb des Threads in eine lokale Variable
#         ein Connection-Objekt, zB self.db = BitfarmConnections.main.engine.connect()
#     @version:
#     """
#     version = __version__
#     log = logging.getLogger("Main.BitfarmConnections")
#     log.setLevel(logging.WARNING)
#
#     def __init__(self, *name):
#         """Erzeugt eine BitfarmConnections Instanz, es kann der Name / oder Pfad eine Profils übergeben werden, bzw. eine Liste von Profilen"""
#
#         self.connections = {}
#         """@ivar: Dictonary, welches alle Connections trägt
#             @type: dictonary"""
#         self.main = None
#         """@ivar: hier ist das erste geladene Profil hinterlegt, die Main.Connection. Über diese Connection wird die sys_queue und die config_tabellen erreicht
#             @type: BitfarmCon Instanz"""
#         self.profiles = []
#         """@ivar: die Namen der geladenen Profile
#         @type: list"""
#
#         if len(name) > 0:
#             if type(name[0]) == list:
#                 name = list(name[0])
#             for profile in name:
#                 if os.path.isfile(profile) and os.path.exists(profile):
#                     profilename = os.path.basename(profile)
#                     profilename = os.path.splitext(profilename)[0]
#                 else:
#                     profilename = profile
#                 profilename = profilename.lower()
#                 self.connections[profilename] = BitfarmCon(profile)
#                 self.profiles.append(profilename)
#
#                 if self.main is None:
#                     self.main = self.connections[profilename]
#
#     def __getitem__(self, name):
#         if name not in self.connections:
#             return self.main
#         else:
#             return self.connections[name]
#
#     def __iter__(self):
#         for profile in self.profiles:
#             try:
#                 yield self.connections[profile]
#             except Exception:
#                 pass
#
#     def create_db_engine(self, bind_tables=True, autocommit=True, echo=False, user=None, password=None):
#         """create_db_engine Methode: erzeugt für alle geladenen Profile eine Datenbank-Engine.
#             Optional kann bind_tables (default=True) auf False gesetzt werden. Dann werden keine Tabellen-Bindings ans con Objekt
#             vorgenommen, man muss selbst für die benötigten Bindings sorgen (bsp: self.sys_timer = sqla.Table('sys_timer', self.meta,  autoload = True))
#             oder über die Funktion bind_tables()
#         """
#         for profile in self.connections:
#             try:
#                 self.connections[profile].create_db_engine(bind_tables=bind_tables, autocommit=autocommit, echo=echo, user=user, password=password)
#             except:
#                 self.log.exception(u'Fehler bei create_db_engine')
#                 self.connections[profile].engine = None
#             else:
#                 if profile == self.main.profile:
#                     self.main.engine = self.connections[profile].engine
#
#     def connect_db(self):
#         """connect_db Methode. erzeugt für alle geladenenen Profile eine Datenbank-Connection
#         und hinterlegt diese in BitfarmConnections[profile].db bzw. BitfarmConnections.main.db
#         Achtung! sollte nicht in Multi-Thread-Anwendungen
#         verwendet werden. Dort immer in den einzelnen Threads eine eigene Connection erzeugen."""
#         for profile in self.connections:
#             try:
#                 self.connections[profile].connect_db()
#             except:
#                 self.log.exception(u'Fehler bei connect_db')
#                 self.connections[profile].db = None
#             else:
#                 if profile == self.main.profile:
#                     self.main.db = self.connections[profile].db
#
#     def bind_tables(self):
#         """bind_tables Methode. Bindet für alle geladenenen Profile die Datenbank-Tabellen an
#         die entsprechenden Attribute der BitfarmCon Objekte.
#         """
#         for profile in self.connections:
#             try:
#                 self.connections[profile].bind_tables()
#             except:
#                 self.log.exception(u'Fehler bei bind_tables')
#
#     def disconnect_db(self):
#         """disconnect_db Methode. Schliesst für alle geladenenen Profile die Datenbank-Connection."""
#         for profile in self.connections:
#             try:
#                 self.connections[profile].disconnect_db()
#             except:
#                 self.log.exception('Fehler bei disconnect_db')
#                 self.connections[profile].db = None
#             else:
#                 if profile == self.main.profile:
#                     self.main.db = None
#
#     def add(self, name):
#         """add - Methode: fügt eine weiteres Profil hinzu"""
#         if name not in self.connections:
#             self.connections[name] = BitfarmCon(name)
#             self.connections[name].create_db_engine()
#             self.profiles.append(name)
#
#     def read_ini(self):
#         """read_ini Methode: liest aus der Scripts.ini und speichert im ConfigDictonary jedes Profils"""
#         for profile in self.connections.keys():
#             self.connections[profile].read_ini()


class BitfarmArchive(object):
    """Bitfarm Hilfsklasse, um die Informationen eine Archives zu kapseln"""

    def __init__(self):
        self.arcname = None
        self.id = 0
        self.lgrid = 0
        self.tblname = None
        self.flags = 0
        self.backpic = None
        self.icon = 0
        self.type = 0
        self.options = 0
        self.defdocname = None
        self.storageid = 0
        self.rights = None
        self.addfields = []
        self.statefields = []
        self.modified = False
        self.tempid = None
        self.duration = None
        self.interval = None


class BitfarmWorkflow(bitfarm_baseobj.BitfarmBaseObject):
    """Workflow Klasse
       Erzeugt einzelne Instanzen eines Workflows, die von der BitfarmWorkflows-Klasse (oder anderen Programmen) verarbeitet
       werden können. Sie enthält keine Methoden.
       Die möglichen Options einer WVL: - 0     Einer für Alle
                                        - 1     Jeder einzeln (required wird ausgewertet, 0= Alle, 1-n= Anzahl der fertig zu setzenden User
                                        - 2     Return to Sender
                                        - 4     Erinnerungsmail
                                        - 8     nicht einzelmodus in bearbeitung (war nie implemntiert, keiner weiß was das sein soll)
                                        - 64    Benachrichtigungsmail
                                        - 128   Wiedervorlage in Rückfrage
                                        - 256   Abonnement

    """
    version = __version__

    def __init__(self):
        bitfarm_baseobj.BitfarmBaseObject.__init__(self)

        self.wf_name = None
        """@ivar:Name des WF, hierüber kann ein bestimmter Workflow direkt angesprochen werden, entspricht Templateverarbeitung (.tpl)"""
        self.wf_type = 0
        """@ivar:Typ des WF, 0 = Dokumentenworkflow, 1=Userworkflow"""
        self.wf_pre = []
        """@ivar:Liste der vorangegangenen wf_ids. Bei User-Workflows werden hier alle nacheinander folgenden Schritte gespeichert um den Verlauf nachzuverfolgen.
        Anhand dieser Liste kann die end_user_workflow alle Einträge aus der sys_workflow entfernen.
        @type: list, int"""
        self.wf_origin = None
        """@ivar:ID des Ursprungsworkflows aus der sys_wfd.
        @type: int"""
        self.usr_counter = 0
        """@ivar:Userzähler zum Auslösen eines WF"""
        self.grp_counter = 0
        """@ivar:Gruppenzähler zum Auslösen eines WF"""
        self.doc_counter = 0
        """@ivar:Dokumentenzähler zum Auslösen eines WF"""
        self.status = None
        """@ivar:Status des WF, Achtung ab v36 gelten neue Bitmasken: 1 = offen, 2= in Arbeit, 4= fertig, 8 = in Rückfrage, 16 = Weiterleitung"""
        #TODO: in Rückfrage über Status realisieren, war bisher unter optionen, hat da aber nichts zu suchen

        self.else_tasks = None
        """@ivar: eine BitfarmTask-Instanz oder eine Liste an BitfarmTask Instanzen. Wird geladen und ausgeführt, wenn eine Condition definiert war die False ergab.
            Der/die else_tasks kann natürlich wieder eine Condition tragen, die wiederum geprüft wird, so sind Oder-Ketten möglich.
            Trägt else_tasks einen einzelnen Task, so wird in der Ausführung durch den Task_Manager nur dieser geladen und ersetzt den aktuellen Task.
            Trägt else_tasks eine Liste so werden alle Tasks durch die Liste ersetzt.
            So kann man erreichen, dass nach einer erfolglosen Bedingungsprüfung entweder nur der aktuelle Task geänder wird, ohne die folgenden zu beeinflussen,
            oder aber eine völlig andere Task-Liste abgearbeitet wird. """
        self.condition = None
        """@ivar:Bedingung die zutreffend sein muss, damit der Workflow verarbeitet wird. Kann ein ConditionSet oder ConditionObject sein"""
        self.importpath = None
        """@ivar:der Importweg des Dokuments, entweder ein Pfad oder ein Bezeichner, zB scanner, importer etc)"""
        self.wf_prio = 0
        """@ivar:Priorität des Workflows (0= niedrigste), wird aus der sys_wfd geholt, diese wird verwendet um die Workflows bei der Suche nach Wichtigkeit zu sortieren.
        @type: int"""
        self.wf_trigger = None
        """@ivar:der Auslöser des Workflows (0= Spooler, 1,2= Archivierung/Server, 3 = Benutzer/temprärer WF, default = None).
        @type: int"""
        self.inherited_from = 0
        """@ivar: ID des Workflows von dem geerbt wurde, ansonsten 0.
        @type: int"""
        self.inherited_trigger = []
        """@ivar: eine Liste der geerbten wf_trigger wenn der Workflow von einem Basisworkflow geerbt hat,in der DB semikolongetrennt
        @type: list"""
        self.info = ""
        """@ivar: Infofeld eines Workflows.
        @type: unicode"""
        self.comment = ""
        """@ivar: Kommentarfeld eines Workflows.
        @type: Unicode"""
        self.userid_work = None
        """@ivar: userid_work, der User dem der Workflow zugeordnet ist.
        @type: int"""
        self.userid_shed = None
        """@ivar: der Controller des Workflows.
        @type: int"""
        self.hist_id = None
        """@ivar: hist_id, Referenz auf sys_timerhist für weitergeleitete Userworkflows.
        @type: int"""
        self.groups = None
        """@ivar: semikolon getrennte Liste von Groupids.
        @type: str"""
        self.users = None
        """@ivar: semikolon getrennte Liste von Userids.
        @type: str"""
        self.username = ""
        """@ivar: Username des Workflows, nur zur Darstellung.
        @type: Unicode"""
        self.groupname = ""
        """@ivar: Gruppen des Workflows, nur zur Darstellung.
        @type: Unicode"""


    def __str__(self):
        return "Bitfarm Workflow [%s] %s" % (self.wf_id, self.wf_name)

class BitfarmTask(object):
    """Tasks-Klasse (Hilfsklasse)
        Erstellt Task-Objekte. Ausgeführt werden die Task vom task_manager.
        Tasks können zu Listen zusammengefügt werden, um mehrere Verarbeitungsschritte nacheinander
        auszuführen (tasklist). Ein Task gehört zu einer Funktionsgruppe (zB r,o,w,p in bfaArchivierung)
        Ein Task kann über seinen Namen oder seiner ID aufgerufen werden. Jeder Task führt eine definierte
        Funktion durch, die einer Funktionsgruppe zugehörig ist.
        Die benötigten Parameter werden durch eine Instanz von BitfarmParameter dargestellt.
        Durch inherit kann von anderen Task und vorallem deren Parameter geerbt werden. Nur einfache, lineare
        Vererbung möglich. So können für alle Tasks Defaulteinstellungen definiert werden.
        show_tasks liefert die vorhandenen, verfügbaren Task zurück.
        Mögliche Tasks sind z.B:
                                            - naming
                                            - sorting
                                            - rendering (ps_to_tif, doc_to_tif etc)
                                            - ocr
                                            - run_plugin, run_vbsplugin
                                            - barcodesplit, readbarcodes, unzip, zip
                                            - unzip
                                            - check_workflows, do_workflow, create_workflow, create_task

        """
    groups = {  'r':['render_by_type', 'ps_to_tif', 'ps_to_txt', 'office_to_tif', 'pdf_to_tif','ps_to_pdf','cad_to_pdf', 'tif_to_pdf', \
                    'jpg_to_tif', 'bmp_to_tif', 'gif_to_tif', 'png_to_tif', 'eml_to_tif', 'msg_to_tif', 'oo_to_tif', \
                    'xml_to_pdf', 'doc_to_pdf', 'doc_to_ps', 'doc_to_tif', 'excel_to_pdf', 'excel_to_ps', 'excel_to_tif',  \
                    'powerpoint_to_tif', 'powerpoint_to_pdf', 'powerpoint_to_ps', 'odt_to_ps', 'odt_to_tif', 'odt_to_pdf', \
                    'filter', 'raise_render_error', 'html_to_pdf', 'html_to_tif', 'custom_converter', 'autorotate', 'render_to_cache'],
                'o':['omp', 'cre', 'pdf_to_txt', 'pdf_to_txt2', 'eml_to_txt', 'msg_to_txt', 'raise_ocr_error'],
                'w':['check_workflows', 'do_workflow', 'naming', 'sorting', 'set_displayname', 'create_workflow', 'create_task', \
                    'notify', 'send_mail', 'load_doc', 'copy_doc', 'move_doc', 'raise_workflow_error', 'create_workflow_step', \
                    'end_workflow_step', 'end_workflow', 'do_setstate_done', 'do_change_resub'],
                'p':['readbarcodes', 'read_zbar', 'bcplus', 'barcodesplit', 'unzip', 'run_plugin', 'run_vbsplugin', 'archive', \
                    'check_duplicates', 'raise_processing_error', 'get_fileinfo', 'extract_attachments'], \
                's':['req_preview', 'req_doc', 'req_iev',  'req_ori'] }

    version = __version__


    def __init__(self, _task = None):
        self.inherit = None
        """@ivar: eine Task-ID oder ein Taskname , von dem geerbt wird, nur einfache Vererbung (linear)"""
        self.group = None
        """@ivar: Funktionsgruppe des Task (r,o,w,p) p = default"""
        self.task = _task
        """@ivar: Der Task (die Funktion) der/die ausgeführt werden soll"""
        for group in self.groups:
            if _task in self.groups[group]:
                self.group = group

        # konnte keine Funktionsgruppe gefunden werden, wird als default 'p' angenommen
        # die Processing-Klasse kann dynamisch hinzugefügte Befehle zu verarbeiten
        if self.group is None:
            self.group = 'p'

        self.id = None
        """@ivar: eine Task-ID, über die der Task in der Datenbank hinterlegt ist"""
        self.name = None
        """@ivar: ein Taskname, mit dem der Task in der Datenbank hinterlegt ist"""
        self.params = BitfarmParameter()
        """@ivar: eine Instanz der BitfarmParameter-Klasse"""
        self.condition = None
        """@ivar: kann eine Instanz eines ConditionSets oder eines ConditionObjects tragen, die mit .execute(doc) geprüft werden kann
        Ist eine Condition vorhanden, wird diese geprüft und der Task nur bei True ausgeführt. Bei False wird der ElseTask ausgeführt, falls vorhanden """
        self.else_tasks = None
        """@ivar: eine BitfarmTask-Instanz oder eine Liste an BitfarmTask Instanzen. Wird geladen und ausgeführt, wenn eine Condition definiert war die False ergab.
        Der/die else_tasks kann natürlich wieder eine Condition tragen, die wiederum geprüft wird, so sind Oder-Ketten möglich.
        Trägt else_tasks einen einzelnen Task, so wird in der Ausführung durch den Task_Manager nur dieser geladen und ersetzt den aktuellen Task.
        Trägt else_tasks eine Liste so werden alle Tasks durch die Liste ersetzt.
        So kann man erreichen, dass nach einer erfolglosen Bedingungsprüfung entweder nur der aktuelle Task geänder wird, ohne die folgenden zu beeinflussen,
        oder aber eine völlig andere Task-Liste abgearbeitet wird. """
        self.message = None
        """@ivar: trägt ein BitfarmNotification Objekt, welches für den task notify oder send_mail verwendet wird.
        Will man explizit eine Benachrichtung wenn der vorherige Task mir result = False zurückkommt, erstellt man einen Notify oder send_mail-Task der in der Condition auf Result = False prüft :-)
        @type: BitfarmNotification"""
        self.warning = None
        """@ivar: trägt ein BitfarmNotification Objekt, welches ausgelöst wird, wenn der Task nicht ausgeführt werden konnte und mit job.error = True zurückkommt.
            Das Notification-Object aus task.warning wird in task.message hinterlegt und entsprechend BitfarmNotification.notify oder BitfarmNotification.send_mail ausgeführt
        @type: BitfarmNotification"""
        self.on_error_resume_next = False
        """@ivar: hier kann man festlegen, dass wenn ein Task einen Fehler verursacht hat, die weiteren Tasks abgearbeitet werden.
        Intern wird auf Job.result = False gesetzt. default = False
        @type: bool"""
        self.inherited = False
        """@ivar: hier wird angegeben, ob der Task von einem anderen geerbt wurde. Die Zugehörigkeit wird über die inherit_id hergestellt.
        Nur wenn der Schalter gesetzt ist wird von dem Basistask Änderungen übernommen.
        @type: bool"""
        self.inherit_id = datetime.datetime.now().strftime('%Y%m%d%H%M%S%f')[:-3]
        """@ivar: Eindeutiger microsekunden-genauer Timestamp, damit bei Vererbung die Nachfahren identifiziert werden können.
        @type: string"""

    def __str__(self):
        return "%s: %s" % (self.group, self.task if self.task is not None else '')

#    def __repr__(self):
#        return "%s: %s" %(self.group, self.task if self.task is not None else '')

    def show_tasks(self):
        """Gibt eine Liste aller Funktionen der verschiedenen Funktionsgruppen zurück"""
        _ret = ""
        for group in self.groups:
            _ret += 'Funktionsgruppe: %s \n' %group
            _ret += '%s\n' %str(self.groups[group])
        return _ret

class BitfarmParameter(object):
    """Hilfsklasse um zur Verfügung stehende Parameter zu kapseln"""
    version = __version__

    def __init__(self):
        self.page = None
        """@ivar: aktuelle Seite des Dokuments"""
        self.pages = None
        """@ivar: Gesamtanzahl der Seiten, bzw von-bis Bezeichner"""
        self.timeout = None
        """@ivar: hier kann man manuell einen Timeout setzen, default = 300sec."""
        self.timeout_per_page = None
        """@ivar: hier kann man manuell einen Timeout setzen, der mit der Anzahl der Seiten multipliziert wird, falls bekannt, default = 300sec."""
        self.set_displayname = None
        """@ivar: Dokumententitel für set_displayname"""
        self.gdocid = None
        """@ivar: gdocid für load_doc"""
        self.source = None
        """@ivar: Quelle des Dokuments, DOCLOC, EXTLOC, ORILOC, SRCLOC, oder TMPLOC, die Pfade dürfen nur in NRS liegen"""
        self.destination = None
        """@ivar: Ziel des zu verarbeitenden Dokuments, DOCLOC, EXTLOC, ORILOC, SRCLOC, oder TMPLOC, die Pfade dürfen nur in NRS liegen"""
        self.ocrtype = None
        """@ivar: OCR-Engine OMP, CSDK, CRE oder andere """
        self.ocrmasks = None
        """@ivar: verschiedene Masken, entweder Masken um das Leseergebnis zu verbessern, oder Masken die einzelne Metadaten auslesen"""
        self.ocrpages = None
        """@ivar: Anzahl der zu lesenden Seiten, wird nichts übergeben, dann wird alles gelesen"""
        self.ocrparams = None
        """@ivar: eine Liste an Bezeichnern zur Steuerung der OCR, welche Parameter unterstützt werden ist von der OCR-Engine abhängig.
        Ausgabeformate: txt (omp, csdk, cre) (default), xml, pdf (omp und csdk). Sprachen: en, de.
        @type: list, str"""
        self.pdftype = None
        """@ivar: PDF-Typ für ps_to_pdf Funktion, derzeit vorhanden pdfx, pdfa1, pdfa2. Default ist pdfx"""
        self.keybarcodes = None
        """@ivar: eine Liste von Keybarcodes-Objekten, dropbarcodepage oder aktbarcode oder searchbarcode sind Attributte"""
        self.extract_text = None
        """@ivar: es soll in der rendernden Funktion versucht werden, denn Volltext direkt zu extrahieren"""
        self.sortto = None
        """@ivar: Ziel einer sortto oder move Anweisung, kann eine arcid (int) oder ein Archivname (str) sein"""
        self.getfield = None
        """@ivar: eine getfield-Anweisung (oder eine Liste von getfield-Anweisungen) für naming"""
        self.mswordupdatefields = None
        """@ivar: Word-Elemente (zB Kopf und Fußzeilen) sollen bei der Dokumentverarbeitung geupdatet werden"""
        self.orientation = None
        """@ivar: Papierausrichtung (portrait / landscape) für die Ghostscript-Verarbeitung, default ist portrait"""

        # Filter
        self.img_resolution = None
        """@ivar: Auflösung DPI"""
        self.img_compression = None
        """@ivar: None,  LZW,  JPG,  ZIP, FAXG4 oder AUTO"""
        self.img_comprate = None
        """@ivar: Kompressionsrate bei JPG-Komprimierung"""
        self.img_dpix = None
        """@ivar: dpi X-Achse,  falls von resolution abweicht"""
        self.img_dpiy = None
        """@ivar: dpi Y-Achse,  falls von resolution abweicht"""
        self.img_colordepth = None
        """@ivar: bw, gray oder color"""
        self.img_bw_threshold = None
        """@ivar: Schwellwert bei Schwarz-Weiss Wandlung: -2 Entropie(default) -1 Durchschnitt 0-255 Schwellwert"""
        self.img_rotate = None
        """@ivar: Dreht das Bild in Grad"""
        self.img_turn = None
        """@ivar: Kippt oder spiegelt das Bild (left, right oder upsidedown)"""
        self.img_setfilter = None
        """@ivar:setzt eine Filter für Größen oder Auflösungänderung. Mögliche Filter
                none, triangle, hermite, bell, bspline, lanczos3, mitchell, nearest, linear, fastlinear, bilinear, bicubic, projectbw, projectwn"""
        self.img_removepix = None
        """@ivar: entfernt einzelnde Punkte, Größe in Pixel"""
        self.img_contrast = None
        """@ivar: Ändert den Kontrast -100 bis 100"""
        self.img_brightness = None
        """@ivar: Ändert die Helligkeit -255 bis 255"""
        self.img_skew = None
        """@ivar: Geraderichten. Bereich in Grad oder auto für automatisches Geraderichten"""
        self.img_white = None
        """@ivar:Automatischer Weiss-Ausgleich"""
        self.img_size = None
        """@ivar:Größenänderung in Prozent"""
        self.img_width = None
        """@ivar:neue (max.) Breite des Bildes"""
        self.img_height = None
        """@ivar:neue (max.) Höhe des Bildes"""
        self.img_nonproportional = False
        """@ivar:Größenänderung nicht propoportional (default = False)"""
        self.img_split = False
        """@ivar: Splittet das Bild in Einzelseiten mit Index _00001"""
        self.img_merge_anno = None
        """@ivar: Pfad auf eine IEV-Datei zum Mergen ins Bild"""
        self.img_merge_layer = None
        """@ivar:Pfad auf eine Tif-Fatei zum Mergen ins Bild"""
        self.img_negative = False
        """@ivar: erzeugt ein Negativ vom Bild (default = False)"""
        self.img_outpath = None
        """@ivar: optionaler Zielpfad (default = None)"""
        self.img_createatlas = False
        """@ivar: Atlas-Dateien erzeugen (default = False)"""
        self.img_autocrop = False
        """@ivar: entfernt automatisch einen Rand"""
        self.img_removegreen = None
        """@ivar: entfernt den Grünanteil (0-255)"""
        self.img_removered = None
        """@ivar: entfernt den Rotanteil (0-255)"""
        self.img_removeblue = None
        """@ivar:entfernt den Blauanteil (0-255)"""
        self.img_format = None
        """@ivar:Ausgabeformat tif/jpg/gif/png/bmp/pdf"""
        self.img_noise = False
        """@ivar: entfernt Bildrauschen (default = False)"""
        self.img_sharpen = False
        """@ivar: schärft das Bild"""
        self.img_dither = False
        """@ivar: führt Graustufenumwandlung mit Floyd-Steinberg Dithering durch"""
        self.img_cutleft = None
        """@ivar: Beschneidet die Seite auf der angegebenen Seite"""
        self.img_cutright = None
        """@ivar:Beschneidet die Seite auf der angegebenen Seite"""
        self.img_cuttop = None
        """@ivar: Beschneidet die Seite auf der angegebenen Seite"""
        self.img_cutbottom = None
        """@ivar: Beschneidet die Seite auf der angegebenen Seite"""
        self.img_cropleft = None
        """@ivar: Beschneidet den Bildinhalt auf der angegebenen Seite"""
        self.img_cropright = None
        """@ivar: Beschneidet den Bildinhalt auf der angegebenen Seite"""
        self.img_croptop = None
        """@ivar: Beschneidet den Bildinhalt auf der angegebenen Seite"""
        self.img_cropbottom = None
        """@ivar: Beschneidet den Bildinhalt auf der angegebenen Seite"""

        # ZIP
        self.zip_deleteafterdecomp = False
        """@ivar: Quell-Zipdatei nach entpacken löschen, default = False"""
        self.zip_extractrecursive = False
        """@ivar: Zipdateien rekursiv entpacken"""
        self.zip_blacklistext = []
        """@ivar: Extensions von Dateien die nicht entpackt werden sollen"""
        self.zip_whitelistext = []
        """@ivar: Extensions von Dateien die entpackt werden dürfen"""
        self.zip_acceptnoext = False
        """@ivar: Dateien ohne Extension werden entpackt deafult = True"""
        self.zip_dontcreatelinks = False
        """@ivar: beim Entpacken keine Verknüpfungen erzeugen"""
        self.zip_linkmask = None
        """@ivar: Maskenstring für Verknüpfungen, Variablen können geparst werden mittels %variable%"""

        # Tags (Schlagworte)
        self.tags_add_barcode = False
        """@ivar: fügt Barcodes zu den Schlagwörtern hinzu"""
        self.tags_add_fields = False
        """@ivar: fügt Zusatzfelder zu den Schlagworten hinzu"""
        self.tags_add_title = False
        """@ivar: fügt Dokumententitel zu den Schlagwörtern hinzu"""
        self.tags_add_filename = False
        """@ivar: fügt den Originaldateinamen zu den Schlagwörtern hinzu"""

        # Email-Verarbeitung
        self.email_dontcreatelinks = False
        """@ivar: beim Emailattachments entpacken keine Verknüpfungen erzeugen"""
        self.email_linkmask = None
        """@ivar: Maskenstring für Verknüpfungen, Variablen können geparst werden mittels %variable%"""
        self.email_attachments_dontextract = False
        """@ivar: True wenn Attachments nicht extrahiert werden sollen, default = False"""
        self.email_attachments_inline_dontextract = False
        """@ivar: True wenn Inline-Attachments (bei Emails) nicht extrahiert werden sollen, default = False"""
        self.email_attachments_changeable = False
        """@ivar: Attachments sollen nicht revisonssicher gespeichert werden, default = False"""
        self.email_attachments_archive = None
        """@ivar: ein bestimmtes Zielarchiv für Attachments, Name oder ArcID"""
        self.email_attachments_whitelist = None
        """@ivar: nur Attachments dieser Typen dürfen archiviert werden, Liste mit Extensions"""
        self.email_attachments_blacklist = None
        """@ivar: Attachments dieser Typen werden nicht archiviert, Liste mit Extensions"""
        self.email_attachments_acceptnoextension = False
        """@ivar: Attachments ohne Endungen werden etxtrahiert und ohne Voransicht gespeichert, default = False"""
        self.email_attachments_dontusefilename = False
        """@ivar: Es wird nicht der Attachmentname sondern 'Anlage zur E-Mail' verwendet, default = False"""
        self.email_attachments_renametobitfax= False
        """@ivar: Attachments mit der Endung .001 ff werden in Tif umbenannt, default = False"""
        self.email_ftxaddrawheader= False
        """@ivar: HEader in Volltext übernehmen, default = False"""

        # Barcode-Verarbeitung
        self.barcodesplit_dontcreatelinks = False
        """@ivar: beim Barcodetrennen keine Verknüpfungen erzeugen"""
        self.barcodesplit_linkmask = None
        """@ivar: Maskenstring für Verknüpfungen, Variablen können geparst werden mittels %variable%"""
        self.barcodesplit_config = []
        """@ivar: Liste an Barcode-Objekten zur Konfiguration der Barcodetrennung"""
        self.bc_typ = []
        """@ivar: Hier wird der Barcodetyp festgelegt, nach dem gesucht werden soll. Die Konstanten
        umfassen: BC_CODABAR, BC_CODE128, BC_CODE39, BC_CODE39EXT, BC_CODE93,
        BC_EAN128, BC_DATAMATRIX, BC_EAN8, BC_EAN13, BC_INDUSTRIE25,
        BC_INTERLEAVED25, BC_PDF417, BC_UPCA, BC_UPCE, BC_CODABLOCK,
        BC_25_IATA, BC_25_3MATRIX, BC_25_3DATALOGIC, BC_25_BCDMATRIX,
        BC_25_INVERTIERT, BC_CODE32, BC_PHARMA, BC_CODE93EXT, BC_PATCHCODE,
        BC_QR_CODE, BC_AZTEC.
        Mehrere Typen werden in als Liste übergeben, außer bei Patchcode, Pharmacode, Codablock F, 2D Barcodes DataMatrix,
        PDF417, QR Code, Aztec Code (iBC_Type wählt den 2D Barcode aus, iBC_Type2 muss BC_NONE sein)
        PostNet Codes (iBC_Type muss BC_NONE sein, iBC_Type2 wählt den PostNet Code aus)
        default = BC_ALL
        @type: list"""

        self.bc_typ2 = []
        """@ivar: BC_GS1_EAN13, BC_GS1_DATABAR,
            BC_GS1_DATABAR_STACKED, BC_GS1_DATABAR_EXP,
            BC_GS1_DATABAR_EXP_STACKED, BC_GS1_DATABAR_LIMITED,
            BC_POSTNET_IMB, BC_POSTNET_ROYAL, BC_POSTNET_KIX,
            BC_POSTNET_AUSTRALIAN, BC_POSTNET_CPC, BC_POSTNET_ZIP,
            BC_POSTNET_PLANET, default = BC_None
        @type: list"""
        self.bc_checksum = []
        """@ivar: Hier wird festgelegt, ob und mit welcher Prüfsumme gearbeitet wird.
            Erlaubt sind: BC_NONE, BC_MOD10, BC_MOD10_EXT, BC_MOD43, BC_EXISTENCE,
            BC_REPORT_CHECKSUM, BC_CODE11_C, BC_CODE11_C_K, default = BC_NONE
        @type: list"""
        self.bc_orientation = []
        """@ivar: Die Ausrichtung des Barcodes wird hier festgelegt. Die Orientation wird über Konstanten angegeben.
        Erlaubt sind: BC_0, BC_90, BC_180, BC_270, BC_360. Konstanten werden als Liste angegeben.
        Weicht ein Barcode von den vier Richtungen zu stark ab, kann er nicht erkannt werden. Dann kann man durch die Konstanten BC_SKEW_LIGHT (13°), BC_SKEW_MEDIUM
        (26°) und BC_SKEW_HEAVY (39°) die Suche verfeinern, default = BC_360
        @type: list"""
        self.bc_lenght = None
        """@ivar: Hier wird die Barcodelänge der Ergebnisse in Zeichen festgelegt. Es kann auch 0 übergeben werden. Dann wird das Intervall von bc_lenght_from bis bc_length_to benutzt.
            Sind diese Parameter auch auf 0 gesetzt, wird die Länge automatisch ermittelt, wo-bei als Standardwerte 4 und 64 verwendet werden.
            Der maximale Wert ist 64. Sollte der Barcode weniger als 4 Zeichen enthalten, muss
            bc_lenght_from explizit gesetzt werden.
        @type: integer"""
        self.bc_lenght_from = None
        """@ivar: Kleinste mögliche Länge des Ergebnisses bei unbekannter Barcodelänge in Zeichen.
            Dieser Wert kann auch 0 sein. Sind sowohl bc_lenght_from, bc_length_to, und
            bc_lenght auf 0 gesetzt, wird die Länge automatisch ermittelt
        @type: integer"""
        self.bc_length_to = None
        """@ivar: Größte mögliche Länge des Ergebnisses unbekannter Barcodelänge in Zeichen.
            Dieser Wert kann auch 0 sein. Sind sowohl bc_lenght_from, bc_length_to, und
            bc_lenght auf 0 gesetzt, wird die Länge automatisch ermittelt.
        @type: integer"""
        self.bc_light_margin = None
        """@ivar: Dies entspricht der Ruhezone um den Barcode, default = 30
        @type: integer"""
        self.bc_read_multiple = None
        """@ivar: Steuert die Abfrage mehrerer Barcodes auf einer Seite. default = BC_MULTI (liest alle Barcodes, liefert doppelte nur einmal zurück)
        BC_MULTI_MULTI liest alle Barcodes und liefert doppelte auch mehrfach zurück (zum Zählen)
        @type: str"""
        self.bc_scan_distance = None
        """@ivar: Dies entspricht dem Scanabstand, d.h., die Abtastung des Bildes in y-Richtung er-folgt mit Schritten in Höhe des Scanabstandes. Defaultwert ist 15 Pixel
        @type: integer"""
        self.bc_scan_distbarcode = None
        """@ivar: Dies entspricht der Abtastung des Barcodes in y-Richtung. Defaultwert ist 3 Pixel.
        Bei schlecht gedruckten Barcodes sollte dieser Wert verringert werden. Dies hat al-lerdings eine Verlangsamung zur Folge.
        @type: integer"""
        self.bc_minheight = None
        """@ivar: Hier wird die minimale Höhe des Barcodes spezifiziert, der gesucht werden soll. D e-fault Wert für lineare Barcodes ist 15 Pixel.
        @type: integer"""
        self.bc_maxheight = None
        """@ivar: Hier wird die maximale Höhe des Barcodes spezifiziert, der gesucht werden soll. De-fault Wert für lineare Barcodes ist 400 Pixel.
        @type: integer"""
        self.bc_percent = None
        """@ivar: Für sehr schlechte Barcodes kann durch iBC_Checksum mit dem Wert
            BC_EXISTENCE eine „Verdachtsmeldung“ aktiviert werden. Die Steuerung der Emp-findlichkeit erfolgt bei linearen Barcodes über iBC_Percent.
        @type: integer"""
        self.bc_maxnoval = None
        """@ivar: Erlaubte Größe der Lücke innerhalb eines Barcodes. Wird eine größere Anzahl Zei-len lang kein Wert erkannt, wird der Barcode als abgeschlossen angesehen.
            Defaultwert: 10 Pixel
        @type: integer"""
        self.bc_tolerance = None
        """@ivar: Maximale Schieflage des Barcodes (Linientoleranz), Defaultwert : 10
        @type: integer"""

        # Wiedervorlage
        self.wf_id = None
        """@ivar: wf_id für do_workflow, create_workflow oder check_workflow Tasks (int oder Liste int)"""
        self.wf_name = None
        """@ivar: wf_id für do_workflow, create_workflow oder check_workflow Tasks"""
        self.wf_type = None
        """@ivar: wf_type für check_workflows etc. 0 = Dokumenten, 1 = Userworkflows"""
        self.wf_trigger = None
        """@ivar: wf_trigger für das Ausführen des Workflows, bei 0 oder 1 wird ein Workflow der sys_wfd ausgeführt, bei 2 ein WF aus der sys_workflow"""
        self.wf_options = 0
        """Bitmaske für verschieden WorkflowOptionen (einer für alle, jeder einzeln etc), default = 0"""
        self.wf_recipient = []
        """Empfänger des Workflows, eine Liste an User oder Group-Objekten"""
        self.wf_controller = None
        """Controller eines Workflows, ein UserObjekt"""
        self.wf_info = None
        """Infotext eines Workflows
        @type: unicode"""
        self.wf_required = None
        """die Anzahl der benötigten User bei JE GruppenWVL, None bzw. 0 = Alle
        @type: int"""
        self.wf_info_ext = None
        """Weiterer Infotext (WF-Kommentar in 3.5)
        @type: unicode"""
        self.wf_date = None
        """Termin eines Workflows
        @type: datetime.datetime"""
        self.wf_alarm = None
        """Alarm eines Workflows
        @type: datetime.datetime"""
        self.wf_delta = None
        """Delta in Tagen zwischen Termin und Alarm, wird benutzt, wenn man weder Termin noch Alarm bzw. nur Termin angegeben hat
        @type: int"""



        # Plugins
        self.vbs_plugin = None
        """@ivar: Pfad auf ein VBS-Plugin"""

        # Custom Converter
        self.cc_converter = None
        """@ivar: Pfad oder Name des Converter-Tools. Wird nur ein Name angegeben, so wird das Tool im Programmverzeichnis erwartet"""
        self.cc_args = None
        """@ivar: die Argumente für das Tool. Mit %infile% und %outfile%, bzw %tmploc%, %docloc%, %oriloc%, %srcloc%, %extloc%
        lassen sich die entsprechenden Variablen in den Aufruf parsen. Mehrere Argument per Semikolon trennen
        (somit sind Argumente mit Leerzeichen möglich)."""
#        self.cc_source = None
#        """@ivar: die Quelle (DOCLOC, EXTLOC, ORILOC, SRCLOC, oder TMPLOC, die Pfade dürfen nur in NRS liegen) des zu konvertierenden Dokuments.
#        Wird nichts angegeben dann wird das aktuelle Dokument verwendet (self._job.path)"""
#        self.cc_destination = None
#        """@ivar: die Quelle (DOCLOC, EXTLOC, ORILOC, SRCLOC, oder TMPLOC, die Pfade dürfen nur in NRS liegen) des zu konvertierenden Dokuments.
#        Wird nichts angegeben dann wird der Outfile-Path verwendet (self._job.outfile)"""
        self.cc_ext = None
        """@ivar: die Dateiendung des konvertierten Dokuments. Wird nichts angegeben dann wird .tif erwartet"""

    def __class__(self):
        return "BitfarmParameter"


    def __str__(self):
        _ret = ""
        if self.page is not None:
            _ret +=  'page=%s\n' %str(self.page)
        if self.pages is not None:
            _ret +=  'pages=%s\n' %str(self.pages)
        if self.img_resolution is not None:
            _ret += 'resolution=%s\n' %str(self.img_resolution)
        if self.img_compression is not None:
            _ret +=  'compression=%s\n' %str(self.img_compression)
        if self.img_dpix is not None:
            _ret +=  'dpix=%s\n' %str(self.img_dpix)
        if self.img_dpiy is not None:
            _ret +=  'dpiy=%s\n' %str(self.img_dpiy)
        if self.img_dpiy is not None:
            _ret +=  'colordepth=%s\n' %str(self.img_colordepth)
        if self.ocrtype is not None:
            _ret +=  'ocrtype=%s\n' %str(self.ocrtype)
        if self.ocrmasks is not None:
            _ret +=  'ocrmasks=%s\n' %str(self.ocrmasks)
        if self.ocrpages is not None:
            _ret +=  'ocrpages=%s\n' %str(self.ocrpages)
        if self.keybarcodes is not None:
            _ret +=  'keybarcodes=%s\n' %str(self.keybarcodes)
        if self.source is not None:
            _ret +=  'source=%s\n' %str(self.source)
        if self.destination is not None:
            _ret +=  'destination=%s\n' %str(self.destination)
        if self.keybarcodes is not None:
            _ret +=  'keybarcodes=%s\n' %str(self.keybarcodes)
        if self.extract_text is not None:
            _ret +=  'extract_text=%s\n' %str(self.extract_text)
        if self.sortto is not None:
            _ret +=  'sortto=%s\n' %str(self.sortto)
        if self.getfield is not None:
            _ret +=  'getfield=%s\n' %str(self.getfield)

        return _ret

class BitfarmGetField(object):
    """Hilfsklasse um GetField-Anweisungen fürs naming zu kapseln.
    Die Mechanismen searchsteps, searchdirection, getstrings wird/kann bei searchpattern über den regulären Ausdruck
    abgebildet werden.Searchbarcode findet sich selbst, der searchbarcode wird auch als regulärer Ausdruck verarbeitet"""
    version = __version__

    def __init__(self):
        self.addfield = None
        """@ivar: Name des Zusatzfeldes, welches befüllt werden soll. Ist das Zusatzfeld nicht vorhanden, wird es angelegt"""
        self.addfieldid = None
        """@ivar: ID des Zusatzfeldes, welches befüllt werden soll. Ist das Zusatzfeld nicht vorhanden, wird es angelegt"""
        self.searchbarcode = None
        """@ivar: der Barcode, der gefunden werden muss, kann eine Liste an Barcodes sein"""
        self.id = None
        """@ivar: wird eine ID angegeben, so wird der Pattern aktuell aus der sys_namings geholt. Damit können Pattern verarbeitet
        werden, die sich ggf. durch Dokumententrainings noch verändern"""
        self.name = None
        """@ivar: wird ein Name angegeben, so wird der Pattern aktuell aus der sys_namings geholt. Damit können Pattern verarbeitet
        werden, die sich ggf. durch Dokumententrainings noch verändern"""
        self.ref_docs = None
        """@ivar: eine Liste an gdocids, die die Referenzdokumente für diese getfiled-Anweisung(es) beschreibt. Für automatische Naming-Regel-Erstellung
        @type: None, list of integer"""
        self.searchpattern = None
        """@ivar: der reguläre Ausdruck, der gefunden werden muss, kann eine Liste an Patterns sein"""
        self.using = None
        """Bezeichner um bestimmte, zur Laufzeit vorhandene Informationen in Zusatzfelder zu Schreiben. Mögliche Bezeichner:
        Dokument-Informationen:
            - Originaldatei Dateiname bei Dateiimport
            - Richtung  Eingang / Ausgang / Intern, soweit auswertbar (z.B. E-Mails, Faxe)
            - Extension Dateiendung (DOC, PDF, u.a.)
            - Dateidatum    Datum einer Datei
            - Herkunft  {Fax, EMail, Scan, Datei, Druck, unbekannt}
            - Benutzer  Benutzername (Windows-Anmeldename)
            - Tabelle    Archiv, DMS-Ordner
            - Autor
            - Titel
            - groesse
            - Faxnummer
        Email-Headerinformationen:
        Empfaenger, Absender, Empfangsdatum, Versanddatum, Empfangsuhrzeit, Versanduhrzeit, CC, BCC"""
        self.mask = None
        """@ivar: Maskenanweisung für OCR-Masken. Kann eine BitfarmRect Instanz oder eine Liste an BitfarmRect-Instanzen sein,
        default = None.
        @type: BitfarmRect"""
        self.mask_use_cre = False
        """@ivar: Flag um für Maskenextraktion die CRE zu benutzen, nur für Enterprise relevant, da CRE die sowieso Default-OCR in der GPL-Version ist.
        Maskenextraktion über die CRE geht nur auf der ersten Seite, per OmniPage sind mehrere Seiten möglich.
        @type: bool"""
        self.resultindex = None
        """@ivar: werden von einem Ausdruck mehrere Treffer gefunden, kann man gezielt angeben, welche(n) Treffer man als Ergebnis möchte.
        Wert kann eine Zahl (int) sein, bei negativer Zahl wird von hinten gezählt, oder ein von - bis Ausdruck (str)
        @type: int, str"""
        self.makedate = None
        """@ivar: soll der gefundene Searchstring als Datum formatiert werden, wenn formatstring angegeben ist, wird dieser zum Formatieren
        verwendet ( siehe datime.strptime)
        @type: bool"""
        self.numeric = None
        """@ivar: soll der gefundene Searchstring als Zahl formatiert werden, Kommazahlen werden zum Typ float, ganze Zahlen zum Typ int konvertiert.
        @type: bool"""
        self.decimal = None
        """@ivar: soll der gefundene Searchstring als Dezimal-Zahl formatiert werden.
        @type: bool"""
        self.formatstring = None
        """@ivar: ein Ausdruck um das Suchergebnis zu formatieren, entspricht einem Python Formatausdruck, der Wert kann mit %s oder %i etc. in  den String geparst werden,
        es können auch Platzhalter mit %Feld% angegeben sein"""
        self.linkstring = None
        """@ivar: soll der gefundene Searchstring als Verknüpfung dienen. Der Wert kann mit formatstring vorher formatiert werden.
        @type: bool"""
        self.displayname = None
        """@ivar: soll der gefundene Searchstring als Dokumententitel dienen. Der Wert kann mit formatstring vorher formatiert werden.
        @type: bool"""
        self.addtag = None
        """@ivar: soll der gefundene Searchstring als Schlagwort dienen. Der Wert kann mit formatstring vorher formatiert werden.
        @type: bool"""
        self.pages = None
        """kann eine Seitenzahl, eine Liste von Seitenzahlen, oder ein von - bis Ausdruck sein, um den Suchbereich einzuschränken
        @type: int, list, str"""
        self.cutleft = None
        """@ivar: Schneidet n Zeichen links vom Suchergebnis. Cutanweisungen haben vorrang vor Getanweisungen
        @type: int"""
        self.cutright = None
        """@ivar: Schneidet n Zeichen rechts vom Suchergebnis. Cutanweisungen haben vorrang vor Getanweisungen
        @type: int"""
        self.getleft = None
        """@ivar: nimmt n Zeichen links vom Suchergebnis. Cutanweisungen haben vorrang vor Getanweisungen
        @type: int"""
        self.getright = None
        """@ivar: nimmt n Zeichen rechts vom Suchergebnis. Cutanweisungen haben vorrang vor Getanweisungen
        @type: int"""
        self.nodeletedashes = False
        """@ivar: ändert die Formateigenschaften des Zusatzfeldes, so dass Strichzeichen nicht entfernt werden
        @type: bool"""
        self.nodeletespaces = False
        """@ivar: ändert die Formateigenschaften des Zusatzfeldes, so dass Leerzeichen nicht entfernt werden
        @type: bool"""
        self.nodeletedots = False
        """@ivar: ändert die Formateigenschaften des Zusatzfeldes, so dass Punktzeichen nicht entfernt werden
        @type: bool"""
        self.warning = None
        """@ivar: trägt ein BitfarmNotification Objekt, welches ausgelöst wird, wenn das Feld nicht befüllt werden konnte
            @type: BitfarmNotification"""
        self.set_continue = False
        """@ivar: fügt alle Suchtreffer semikolongetrennt ins Feld, das Feld muss Modify-Berechtigung haben"""
        self.set_writeable = False
        """@ivar: überschreibt ggf vorhandene Zusatzfeldwerte mit dem Suchtreffer, das Feld muss Read oder Write-Berechtigung haben"""



class BitfarmNotification(object):
    """Hilfsklasse über die Benachrichtigungen erstellt werden können. Wenn in einem Task mehrere gleiche Benachrichtigungen zusammenkommen,
    wird der Benachrichtigungstext zusammengefasst. Benachrichtigungen haben ein eindeutige ID, über die sie in der Datenbank zu finden sind.
    Benachrichtigungen können entweder eine einfache Wiedervorlage und/oder eine Email erzeugen"""

    version = __version__

    def __init__(self):
        self.id = None
        """@ivar: eindeutige ID der Benachrichtigung, default = None
        @type: integer"""
        self.name = None
        """@ivar: Name der Benachrichtigung, default = None
        @type: integer"""
        self.subject = None
        """@ivar: Betreff Benachrichtigung, default = None
        @type: integer"""
        self.text = None
        """@ivar: Plain-Text der Benachrichtigung, mittels Tags kann man Variablen einbauen, default = None
            Plain-Text wird vor Allem für Wiedervorlage benötigt
        @type: unicode"""
        self.html = None
        """@ivar: HTML-Text der Benachrichtigung, mittels Tags kann man Variablen einbauen, default = None
        @type: unicode"""
        self.recipient = None
        """@ivar: Liste von Empfänger als BitfarmUser oder BitfarmGroup-Objekte, oder Emailadresse, mehrere semikolongetrennt, default = None
        @type: list/BitfarmUser,BitfarmGroup,str,unicode"""
        self.create_resub = False
        """@ivar: wenn True wird die Nachricht als einfache Wiedervorlage erzeugt
        @type: bool"""
        self.send_mail = False
        """@ivar: wenn True wird die Nachricht als Email verschickt
        @type: bool"""
        self.attach_document = False
        """@ivar: wenn True wird das aktuelle Dokument angehangen
        @type: bool"""
        self.attach_link = False
        """@ivar: wenn True wird das aktuelle Dokument als gdocid-Link versendet
        @type: bool"""

class BitfarmConditionObject(object):
    """Condition-Klasse
        Kapselt alle Funktionen um Bedingungen zu prüfen.
        Ein ConditionObject trägt wenigstens ein ConditionSet (sowie die junction and/or/not)
        oder ein ConditionObject oder eine Liste an CoditionObjecten die mittels ihrer junction zusammengefasst
        werden. Sie ist es möglich verschachtelte Conditions zu erstellen.
        Beispiel:
                - erstelle cset1, cset2, cset3, cset4
                - erstelle
                - cobj1 = BitfarmConditionObject('and', cset1 )
                - cobj2 = BitfarmConditionObject('and', cset2 )
                - cobj3 = BitfarmConditionObject('and', cset3 )
                - cobj4 = BitfarmConditionObject('or', cset4 )
                - fasse jeweils 2 zusammenbauen
                - cobj5 = BitfarmConditionObject('and', [cobj1, cobj2] )
                - cobj6 = BitfarmConditionObject('or', [cobj3, cobj4] )
                - fasse die zwei zusammen
                - cobj = BitfarmConditionObject('or', [cobj5, cobj6])
                - prüfe
                - cobj.execute(t)
                - entspricht dem Ausdruck (cset1 and cset2) or (cset3 or cset4)
        """
    version = __version__
    log = logging.getLogger('Main.bfaConditionObject')

    def __init__(self, junc = None, item = None):
        self.junc = junc    # AND OR NOT
        self.item = item

    def __str__(self):
        if type(self.item) is list:
            return "(%s , %s)" % (self.junc, [item for item in self.item])
        else:
            return "(%s , %s)" % (self.junc, self.item)
        #return u"(%s , %s)" %(self.junc, str(self.item))

    def _get_junc(self):
        return self.__junc

    def _set_junc(self, junc):
        if junc is None:
            self.__junc = 'and'
        else:
            if junc.lower() in ['and', 'or', 'not']:
                self.__junc = junc.lower()
            else:
                self.__junc = 'and'

    def _get_item(self):
        return self.__item
    def _set_item(self, item):
        if type(item) in [BitfarmConditionSet,  bool]:
            self.__item = item
        if type(item) == list:
            self.__item = []
            for elem in item:
                if type(elem) in [BitfarmConditionObject, BitfarmConditionSet]:
                    self.__item.append(elem)


    def execute(self,  _doc):
        """Diese Funktion überprüft ein ConditionObject und liefert entsprechend True oder False zurück.
                Es wird das entsprechende Dokument oder Workflow-Objekt übergeben, gegen das geprüft wird"""

        # if type(_doc) is not bitfarm_doc.BitfarmDocument and type(_doc) is not BitfarmWorkflow:
        if type(_doc) is not BitfarmDocument and type(_doc) is not BitfarmWorkflow:
            raise(u'No BitfarmDocument / BitfarmWorkflow')

        # 1. auflösen  (resolve)
        self._resolve(self.item, _doc)

        # 2. auswerten  (interpret)
        result = self._interpret(self.item)
        self.log.debug(u"result: " + str(result))
        self.log.debug(u"-----------------------")
        return result

    def _resolve(self, _item, _doc):
        if type(_item) is not BitfarmConditionSet and type(_item) is not list:
            return False

        if type(_item) is list:
            for _elem in _item:
                if type(_elem.item) is list:
                    self.log.debug(u"_resolve: %s" %_elem)
                    _elem.item = self._resolve(_elem.item, _doc)
                    self.log.debug(u"_resolved: %s" %_elem.item)
                if type(_elem.item) is BitfarmConditionSet:
#                    _elem.item = _elem.item.execute(_doc)
                    self.log.debug(u"execute %s" %_elem)
                    _item[_item.index(_elem)].item = _elem.item.execute(_doc)
                    self.log.debug(u"executed %s" %_elem.item)
        if type(_item) is BitfarmConditionSet:
            self.log.debug(u"execute set %s" %_item)
            self.item = _item.execute(_doc)
            self.log.debug(u"executed set %s" %_item)

    def _interpret(self, _item):
        if type(_item) not in [BitfarmConditionObject,  BitfarmConditionSet, list,  bool]:
            return False

        if type(_item) is list:
            for _elem in _item:
                if type(_elem.item) is list:
                    #self.log.debug(u"_interpret: %s" %_elem.item)
                    _elem.item = self._interpret(_elem.item)
                    #self.log.debug(u"_interpreted: %s" %_elem.item)
                if type(_elem.item) is bool:
#                    _elem.item = _elem.item.execute(_doc)
                    #self.log.debug(u"interpret %s" %_elem)
                    if _item.index(_elem) == 0:  #das erste Element einer Klammer (liste) ist entweder True oder False
                        _ret = True and _elem.item
                    else:
                        if _elem.junc == 'and':
                            _ret = _ret and _elem.item
                        elif _elem.junc == 'or':
                            _ret = _ret or _elem.item
                        elif _elem.junc == 'not':
                            _ret = _ret and not _elem.item
                    #self.log.debug(u"interpreted %s" %_elem.item)

        if type(_item) is BitfarmConditionObject:
            #self.log.debug(u"interpret2 %s %s" %(self.junc, str(_item)))
            if _item.junc == 'and':
                _ret = _ret and _item.item
            elif _item.junc == 'or':
                _ret = _ret or _item.item
            elif _item.junc == 'not':
                _ret = _ret and not _item.item
            #self.log.debug(u"interpreted2 %s" %_item)

        if type(self) == BitfarmConditionObject and type(_item) == bool:
            #self.log.debug(u"interpret3 %s %s" %(self.junc, str(_item)))
            if self.junc == 'and':
                _ret = True and _item
            elif self.junc == 'or':
                _ret = True or _item
            elif self.junc == 'not':
                _ret = True and not _item
            #self.log.debug(u"interpreted3 %s" %str(_item))
        self.log.debug(u'return %s' %str(_ret))
        return _ret

    junc = property(_get_junc, _set_junc)
    item = property(_get_item, _set_item)

class BitfarmConditionSet(object):
    """ConditionSet-Klasse (Hilfsklasse)
        Diese Klasse stellt die einzelnen Prüfbedingungen zu Verfügung.
        Ein Set besteht aus einer Operator (param) und seinem Typ, sowie dem Operant und dem Prüfwert (value)
        Es kann noch definiert werden, dass die Prüfung im Fehlerfall tolerant oder nicht sein soll.
        Wird gegen eine Liste geprüft(value vom Typ list), so prüft =, is, contains, bzw. <>, isnot, containsnot auf Vorhandensein des Oprators in der Liste.
        Wird ein Prüfwert(value) gegen einen Operator vom Typ list geprüft, so prüft =, is, <>, isnot exakt und contains, bzw.
        containsnot auf einen Teilstring innerhalb der Elemente der Liste.
    """
    version = __version__

    def __init__(self, param = None, type = None, operator = None, value = None, tolerant = False,  ignorecase = False):
        self.ignorecase = ignorecase
        """@ivar: Groß/Kleinschreibung nicht beachten, default = False
        @type: bool"""
        self.param = param
        self.type = type     #FTX, FIELD, STATUS, DOCUMENT, WORKFLOW
        self.operator = operator    # <, =, >, <>, <=, >=, is, isnot, contains, containsnot
        self.value = value       # Wert der geprüft wird
        self.tolerant = tolerant
        """@ivar: wenn die Prüfung tolerant sein soll, dann gibt sie im Fehlerfall true zurück, default = False
        @type: bool"""

    def __str__(self):
        return "condition: %s (%s) %s %s" % (self.param, self.type, self.operator, self.value)

    def _set_param(self, param):
        if self.ignorecase:
            param = param.lower()
        if type(param) is str:
            self.__param = param
        else:
            self.param = str(param)

    def _get_param(self):
        return self.__param

    def _set_typ(self, type):
        if type is None:
            self.__typ = None
        elif type.lower() in ['ftx', 'field', 'status', 'document', 'workflow']:
            self.__typ = type.lower()
        else:
            self.__typ = 'ftx'

    def _get_typ(self):
        return self.__typ

    def _set_operator(self, operator):
        if operator is None:
            self.__operator = None
        elif operator.lower() in ['<', '=', '>', '<>', '<=', '>=', 'is', 'isnot', 'contains', 'containsnot', 'counts', 'counts_more', 'counts_less', 'matching', 'notmatching']:
            self.__operator = operator
        else:
            self.__operator = '='

    def _get_operator(self):
        return self.__operator

    def _set_value(self, value):
        if self.ignorecase:
            if type(value) is str:
                value = value.lower()
        self.__value = value
        if type(self.__value) in [int,  float]:
            if self.__operator in ['is', 'contains', 'counts']:
                self.operator = '='
            if self.__operator in ['isnot', 'containsnot']:
                self.operator = '<>'
            if self.__operator == 'counts_more':
                self.__operator = '>'
            if self.__operator == 'counts_less':
                self.__operator = '<'

        if self.__typ != 'ftx':
            if self.__operator == 'counts':
                self.__operator = '='
            if self.__operator == 'counts_more':
                self.__operator = '>'
            if self.__operator == 'counts_less':
                self.__operator = '<'


        if self.__typ == 'status':
            if self.__operator in ['is', 'contains', 'count']:
                self.operator = '='
            if self.__operator in ['isnot', 'containsnot', '<', '>', '<=', '>=']:
                self.operator = '<>'


    def _get_value(self):
        return self.__value

    def execute(self, _doc):
        """Diese Funktion überprüft das ConditionSet und liefert entsprechend True oder False zurück.
        Es wird das entsprechende Document übergeben, gegen das geprüft wird"""

        if type(_doc) is not BitfarmDocument and type(_doc) is not BitfarmWorkflow:
            raise('No BitfarmDocument / BitfarmWorkflow')

        if type(_doc) is BitfarmWorkflow:
            self.type = 'workflow'

        try:
            if self.type == 'ftx':
                _ftx = None
                if type(_doc.ftx) is str:
                    _ftx = _doc.ftx
                elif type(_doc.ftx) == BitfarmFulltext:
                    _ftx = _doc.ftx.text
                if _ftx is not None:
                    if self.operator in ['=', 'is']:
                        return _ftx == self.value
                    if self.operator in ['<>', 'isnot']:
                        return _ftx != self.value
                    if self.operator == '<':
                        return _ftx < self.value
                    if self.operator == '<=':
                        return _ftx <= self.value
                    if self.operator == '>':
                        return _ftx > self.value
                    if self.operator == '>=':
                        return _ftx >= self.value
                    if self.operator == 'contains':
                        return _ftx.find(self.value) > -1
                    if self.operator == 'containsnot':
                        return _ftx.find(self.value) == -1
                    if self.operator == 'counts':
                        return _ftx.count(self.param) == int(self.value)
                    if self.operator == 'counts_more':
                        return _ftx.count(self.param) > int(self.value)
                    if self.operator == 'counts_less':
                        return _ftx.count(self.param) < int(self.value)
                    if self.operator == 'matching':
                        if re.match(self.value, _ftx) is None:
                            return False
                        else:
                            return True
                    if self.operator == 'notmatching':
                        if re.match(self.value, _ftx) is None:
                            return True
                        else:
                            return False

            elif self.type == 'field':
                if self.param not in _doc.fields:
                    return self.tolerant
                if self.operator in ['=', 'is']:
                    if type(self.value) is list:
                        return _doc.fields[self.param].value in self.value
                    else:
                        return _doc.fields[self.param].value == self.value
                if self.operator in ['<>', 'isnot']:
                    if type(self.value) is list:
                        return _doc.fields[self.param].value not in self.value
                    else:
                        return _doc.fields[self.param].value != self.value
                if self.operator == '<':
                    return _doc.fields[self.param].value < self.value
                if self.operator == '<=':
                    return _doc.fields[self.param].value <= self.value
                if self.operator == '>':
                    return _doc.fields[self.param].value > self.value
                if self.operator == '>=':
                    return _doc.fields[self.param].value >= self.value
                if self.operator == 'contains':
                    if type(self.value) is list:
                        return _doc.fields[self.param].value in self.value
                    else:
                        if type(_doc.fields[self.param]) is list:
                            for param in _doc.fields[self.param]:
                                if type(param) is str:
                                    if param.find(self.value) > -1:
                                        return True
                                else:
                                    if param == self.value:
                                        return True
                            return False    # nicht gefunden? dann False
                        else:
                            return _doc.fields[self.param].value.find(self.value) > -1
                if self.operator == 'containsnot':
                    if type(self.value) is list:
                        return _doc.fields[self.param].value not in self.value
                    else:
                        if type(_doc.fields[self.param]) is list:
                            for param in _doc.fields[self.param]:
                                if type(param) is str:
                                    if param.find(self.value) > -1:
                                        return False
                                else:
                                    if param == self.value:
                                        return False
                            return True    # nicht gefunden? dann True
                        else:
                            return _doc.fields[self.param].value.find(self.value) == -1

            elif self.type == 'status':
                if _doc.version in [3, 35, '3.0']:
                    if self.param not in _doc.statusdict:
                        return self.tolerant
                    if self.operator in ['=', 'is']:
                        return _doc.statusdict[self.param].value == self.value
                    if self.operator in ['<>', 'isnot']:
                        return _doc.statusdict[self.param].value != self.value
                elif _doc.version == 36:
                    if self.param not in _doc.statefields:
                        return self.tolerant
                    if self.operator in ['=', 'is']:
                        return _doc.statefields[self.param].value == self.value
                    if self.operator in ['<>', 'isnot']:
                        return _doc.statefields[self.param].value != self.value

            elif self.type == 'document':
                if self.param not in _doc.__dict__:
                    if '_BitfarmDocument__' + self.param not in _doc.__dict__:
                        if '_BitfarmBaseObject__' + self.param not in _doc.__dict__:
                            return self.tolerant
                        else:
                            self.param = '_BitfarmBaseObject__' + self.param
                    else:
                        self.param = '_BitfarmDocument__' + self.param

                if self.operator in ['=', 'is']:
                    if type(self.value) is list:
                        return _doc.__dict__[self.param] in self.value
                    else:
                        return _doc.__dict__[self.param] == self.value
                if self.operator in ['<>', 'isnot']:
                    if type(self.value) is list:
                        return __dict__[self.param] not in self.value
                    else:
                        return _doc.__dict__[self.param] != self.value
                if self.operator == '<':
                    return _doc.__dict__[self.param] < self.value
                if self.operator == '<=':
                    return _doc.__dict__[self.param] <= self.value
                if self.operator == '>':
                    return _doc.__dict__[self.param] > self.value
                if self.operator == '>=':
                    return _doc.__dict__[self.param] >= self.value
                if self.operator == 'contains':
                    if type(self.value) is list:
                        return _doc.__dict__[self.param] in self.value
                    else:
                        if type(_doc.__dict__[self.param]) is list:
                            for param in _doc.__dict__[self.param]:
                                if type(param) is str:
                                    if param.find(self.value) > -1:
                                        return True
                                else:
                                    if param == self.value:
                                        return True
                            return False    # nicht gefunden? dann False
                        else:
                            return _doc.__dict__[self.param].find(self.value) > -1
                if self.operator == 'containsnot':
                    if type(self.value) is list:
                        return _doc.__dict__[self.param] not in self.value
                    else:
                        if type(_doc.__dict__[self.param]) is list:
                            for param in _doc.__dict__[self.param]:
                                if type(param) is str:
                                    if param.find(self.value) > -1:
                                        return False
                                else:
                                    if param == self.value:
                                        return False
                            return True    # nicht gefunden? dann True
                        else:
                            return _doc.__dict__[self.param].find(self.value) == -1

            elif self.type == 'workflow':
                if self.param not in _doc.__dict__:
                    return self.tolerant

                if self.operator in ['=', 'is']:
                    if self.param == 'doc_counter':
                        if _doc.doc_counter >= int(self.value) -1:
                            _doc.doc_counter = 0
                            return True
                        else:
                            _doc.doc_counter += 1
                            return False
                    elif self.param == 'usr_counter':
                        if _doc.usr_counter >= int(self.value) -1:
                            _doc.usr_counter = 0
                            return True
                        else:
                            _doc.usr_counter += 1
                            return False
                    elif self.param == 'grp_counter':
                        if _doc.grp_counter >= int(self.value) -1:
                            _doc.grp_counter = 0
                            return True
                        else:
                            _doc.grp_counter += 1
                            return False
                    else:
                        if type(self.value) is list:
                            return _doc.__dict__[self.param] in self.value
                        else:
                            return _doc.__dict__[self.param] == self.value
                if self.operator in ['<>', 'isnot']:
                    if type(self.value) is list:
                        return __dict__[self.param] not in self.value
                    else:
                        return _doc.__dict__[self.param] != self.value
                if self.operator == '<':
                    return _doc.__dict__[self.param] < self.value
                if self.operator == '<=':
                    return _doc.__dict__[self.param] <= self.value
                if self.operator == '>':
                    return _doc.__dict__[self.param] > self.value
                if self.operator == '>=':
                    return _doc.__dict__[self.param] >= self.value
                if self.operator == 'contains':
                    if type(self.value) is list:
                        return _doc.__dict__[self.param] in self.value
                    else:
                        if type(_doc.__dict__[self.param]) is list:
                            for param in _doc.__dict__[self.param]:
                                if type(param) is str:
                                    if param.find(self.value) > -1:
                                        return True
                                else:
                                    if param == self.value:
                                        return True
                            return False    # nicht gefunden? dann False
                        else:
                            return _doc.__dict__[self.param].find(self.value) > -1
                if self.operator == 'containsnot':
                    if type(self.value) is list:
                        return _doc.__dict__[self.param] not in self.value
                    else:
                        if type(_doc.__dict__[self.param]) is list:
                            for param in _doc.__dict__[self.param]:
                                if type(param) is str:
                                    if param.find(self.value) > -1:
                                        return False
                                else:
                                    if param == self.value:
                                        return False
                            return True    # nicht gefunden? dann True
                        else:
                            return _doc.__dict__[self.param].find(self.value) == -1

        except:
            return self.tolerant
        # wenn wir bis hierher gekommen sind, dann hat irgendwas nicht funktioniert
        return self.tolerant


    param = property(_get_param, _set_param)
    """der Parameter bzw der Name/Bezeichner der geprüft werden soll, wird gegen type = ftx (Volltext) geprüft kann param auch leer sein"
    @type: str, unicode"""
    type = property(_get_typ, _set_typ)
    """ type des zu prüfenden Parameters, 'ftx', 'field', 'status', 'document', 'workflow' \r\n
    'workflow' steht bei Prüfung einer Condition eines Workflowobjekts zur Verfügung, die anderen vier bei einer Condition eines DokumentenObjekts
    @type: str"""
    operator = property(_get_operator, _set_operator)
    """ein Operator aus der Menge '<', '=', '>', '<>', '<=', '>=', 'is', 'isnot', 'contains', 'containsnot', 'counts', 'counts_more', 'counts_less' \r\n
    Welcher Operator gesetzt werden kann bzw. wie er interpretiert wird, ist vom type abhängig.\r\n counts, counts_more, counts_less gibt es nur bei Volltext,\r\n dabei wird die Anzahl in params übergeben
    @type: str"""
    value = property(_get_value, _set_value)
    """der Wert gegen den geprüft wird, kann auch eine Liste mit Werten sein"""


class BitfarmADEntry(object):
    """Helferklasse für AD-Einträge zum Client zu übertragen"""
    def __init__(self):
        self.adclass = None
        self.adpath = None
        self.displayname = None
        self.samaccountname = None
        self.mail = None
        self.useraccountcontrol = 0
        self.has_user = False


class BitfarmDocClass(object):
    """Instanzen dieser Klasse tragen alle Informationen um eine Dokumentenklasse zu definieren und prüfen zu können"""
    version = __version__

    def __init__(self):
        self.classname = None
        """@ivar: Name einer Dokumenten-Klasse, wird auch in der Datenbank hinterlegt
        @type: str, unicode"""
        self.classid = None
        """@ivar: ID der Klasse in der Datenbank, kann hierüber geladen werden.
        @type: int"""
        self.ngrams = {}
        """@ivar: Dictonary zum Speichern der klassenrelevanten ngrams. Der Key gibt die Anzahl der Tupelelemente an.
        @type: dict, keys = int, values = set"""
        self.classdocs = []
        """@ivar: Liste aller Referenzsdokumente (ID in der sys_classdocs) für eine Dokumentenklasse.
        @type: list, int"""


class BitfarmPredictionResult(object):
    """Helferklasse um Ergebnisse der Prediction zu Übertragen"""
    def __init__(self):
        self.classid = None
        self.page = None
        self.guid = None

class BitfarmGridConfig(object):
    """Helferklasse um ein Zusatzfeld Grid zu definieren"""
    def __init__(self):
        self.displayname = None
        self.showsummary = False
        self.showheader = False
        self.showgoupbybox = False
        self.showindicator = False
        self.usepopups = False
        self.manualsorting = False
        self.alternate_rows = False
        self.multiselect = False
        self.columns = None     # Liste von BitfarmColumnConfig Objekten

class BitfarmColumnConfig(object):
    def __init__(self):
        self.name = None
        self.columntype = 0             # 0 = cltEdit, 1 = cltDate, 2 = cltNumber, 3 = cltCurreny, 4 = cltList, 5 = cltDBList, 6 = cltAutoIncrement
        self.valuetype = 0              # 0 = vltString, 1 = vltInteger, 2 = vltFloat, 3 = vltDate, 4 = vltCurrency
        self.summarytype = 0            # 0 = skNone, 1 = skSum, 2 = skMin, 3 = skMax, 4 = skCount, 5 = skAverage
        self.fieldname = None
        self.readonly = False
        self.displayformat = None
        self.editformat = None
        self.expression = None
        self.defaultvalue = None       # der default Wert in der AuswahlListe, falls gewünscht
        self.listtype = 0               # 0 = lstNone, 1 = lstItems, 2 = lstAddfield, 3 = lstQuery
        self.listitems = None
        self.listaddfield = 0         # falls Werte eines Zusatzfeldes
        self.listquery = None         # oder mittels einer Query ins Auswahlfeld sollen

class MyFormatter(logging.Formatter):
    def __init__(self, fmt=None, datefmt=None, encoding='windows-1250'):
        logging.Formatter.__init__(self, fmt, datefmt)
        self.encoding = encoding

    def formatException(self, ei):
        r = logging.Formatter.formatException(self, ei)
        if type(r) in [str]:
            r = r.decode('windows-1250', 'replace') # Convert to unicode
        return r

    def format(self, record):
        t = logging.Formatter.format(self, record)
        # if type(t) in [unicode]:
        #     t = t.encode(self.encoding, 'replace')
        if type(t) in [str]:
            t = t.decode('utf8')
        return t


def addattr(clsname,  attr ,  value):
    log.debug(u"--------- addattr")
    log.debug(clsname.__dict__)
    if not clsname.__dict__.has_key(attr):
        setattr(clsname, attr,  value)
    else:
        clsname.__dict__[attr] = value
    return clsname

def get_ngrams(txt, from_n, to_n=None):
    #TODO: reimplement wo nltk
    return []




class DynamicSleeptime(object):
    """
        Helper-Klasse um Sleep-Zeiten oder ähnliches langsam,
        immer schneller werdend im Bereich min - max ansteigen zu lassen
    """
    version = __version__

    def __init__(self, min, max, steps=25, delay=1):
        """ min  Minimum in sec. (float)
            max Maximum in sec (float)
            steps Verzögerungsschritte (int)
            delay Einsatz der Verzögerung delay * min (int)
            """

        self.__delay = delay
        self.__qs = [1 for x in range(self.__delay)]
        self.__qs.extend(range(steps * 2, 2, -2))
        self.__qs.append(1)
        self.__n = 0
        self.__initmin = min
        self.__min = min
        self.__max = max

    def _iter_(self):
        return self

    def next(self):
        if self.__delay < self.__n < len(self.__qs):
            self.__min += (self.__max - self.__min) / self.__qs[self.__n]

        self.__n += 1

        return self.__min

    def min(self):
        return self.__initmin

    def reset(self):
        self.__n = 0
        self.__min = self.__initmin


# import xml.etree.cElementTree as eTree
#import bitfarm_xml as bfaxml
#import bitfarm_workflow as bfawf

#from defusedxml import cElementTree as dTree
# import xml.etree.cElementTree as eTree
#
# Element = type(eTree.fromstring('<xml></xml>'))
