import sys
import os
import subprocess
import re
import shutil
import datetime
import stat
import argparse
import pathlib

__version__ = "3.6.2.3"


def on_rm_error(func, path, exc_info):
    # path contains the path of the file that couldn't be removed
    # let's just assume that it's read-only and unlink it.
    os.chmod(path, stat.S_IWRITE)
    os.unlink(path)


parser = argparse.ArgumentParser()
parser.add_argument("--include_bfa_xmlmapper",
                    action="store_true", default=False)
parser.add_argument("--include_example_plugin",
                    action="store_true", default=False)
parser.add_argument("--include_dubletten_check",
                    action="store_true", default=False)
parser.add_argument("--include_xml_export",
                    action="store_true", default=False)                    
parser.add_argument("--include_bfa_field_sync",
                    action="store_true", default=False)                     
parser.add_argument("--include_bfa_fname_mapper",
                    action="store_true", default=False) 
parser.add_argument("--include_send_mail",
                    action="store_true", default=False)  

plugin_runner_files = [
    ("plugin_runner_example.ini", "plugin_runner_example.ini"),
    ("changelog.txt", "changelog.txt")
    ]
plugin_runner_dirs = []

bfa_xmlmapper_files = [("plugins\\bfa_xmlmapper\\plugin.py", "plugin.py"),
                       ("plugins\\bfa_xmlmapper\\example_config.ini",
                        "example_config.ini"),
                       ("plugins\\bfa_xmlmapper\\casting.py", "example_casting.py")]
bfa_xmlmapper_dirs = [("plugins\\bfa_xmlmapper\\bin_nosvn", "bin")]

example_plugin_files = [("plugins\\example_plugin\\plugin.py", "plugin.py")]
example_plugin_dirs = []

dubletten_check_dirs = []
dubletten_check_files = [("plugins\\dubletten_check\\plugin.py", "plugin.py"), (
    "plugins\\dubletten_check\\dubletten_check_example.ini", "dubletten_check_example.ini")]

xml_export_plugin_files = [("plugins\\xml_export\\plugin.py", "plugin.py"), ("plugins\\xml_export\\xml_export_example.ini", "xml_export_example.ini")]
xml_export_plugin_dirs = []

bfa_field_sync_plugin_files = [("plugins\\bfa_field_sync\\plugin.py", "plugin.py"), ("plugins\\bfa_field_sync\\bfa_field_sync_example.ini", "bfa_field_sync_example.ini")]
bfa_field_sync_plugin_dirs = []


bfa_fname_mapper_plugin_files = [("plugins\\bfa_fname_mapper\\plugin.py", "plugin.py"), ("plugins\\bfa_fname_mapper\\example_config.ini", "bfa_fname_mapper_example.ini")]
bfa_fname_mapper_plugin_dirs = []

send_mail_files = [("plugins\\send_mail\\plugin.py", "plugin.py"),
                       ("plugins\\send_mail\\send_mail_example.ini",
                        "send_mail_example.ini"),
                       ("plugins\\send_mail\\send_mail_casting_example.py", "send_mail_casting_example.py"),
                       ("plugins\\send_mail\\templates\\example_template.html", "templates\\example_template.html"),
                       ("plugins\\send_mail\\templates\\example_template.txt", "templates\\example_template.txt"),
                       ("plugins\\send_mail\\readme.txt", "readme.txt")
                       ]
send_mail_dirs = []


args = parser.parse_args()

mydir = os.path.dirname(sys.argv[0])
scriptname = os.path.join(mydir, "plugin_runner.py")

build_args = ["pyinstaller", "--console", "--onedir", "--icon=icon.ico", "--version-file=version.txt",
              "--clean", "--hidden-import=lxml", "--hidden-import=facturx", "--hidden-import=pyodbc"]

version_regex = r"_*version_*\s+=\s+[\"|\'](\d\.\d\.\d+\.\d+)b?(\d{0,4})[\"|\']"


build_args.append(scriptname)
dirname = os.path.dirname(scriptname)

exefile = os.path.splitext(os.path.basename(scriptname))[0]+".exe"

with open(scriptname, "r") as f:
    lines = f.read().split("\n")

version = None
for line in lines:
    line = line.strip().lower()

    match = re.match(version_regex, line)
    if match:
        groups = match.groups(0)
        version = groups[0]
        if len(groups) > 1:
            beta_version = groups[1]
            description = exefile + f" (beta {beta_version})"
        else:
            description = exefile
        break

if version is None:
    print("no version found, cannot build...")
    sys.exit(1)


py_version = f"py_{sys.version_info.major}.{sys.version_info.minor}.{sys.version_info.micro}"
exefile += f" {py_version}"
version_ext = version.replace(".", ", ")

info = """VSVersionInfo(
            ffi=FixedFileInfo(
            filevers=(%s),
            prodvers=(3, 6, 2, 0),
            mask=0x3f,
            flags=0x0,
            OS=0x40004,
            fileType=0x1,
            subtype=0x0,
            date=(0, 0)
            ),
            kids=[
            StringFileInfo(
                [
                StringTable(                    u'040904B0',
                [StringStruct(u'CompanyName', u'bitfarm GmbH'),
                StringStruct(u'FileDescription', u'%s'),
                StringStruct(u'FileVersion', u'%s'),
                StringStruct(u'InternalName', u''),
                StringStruct(u'LegalCopyright', u'bitfarm Informationssysteme GmbH, 2005 - %d'),
                StringStruct(u'LegalTrademarks', u''),
                StringStruct(u'OriginalFilename', u'%s'),
                StringStruct(u'ProductName', u'bitfarm Archiv'),
                StringStruct(u'ProductVersion', u'3.6.2.0'),
                StringStruct(u'Comments', u'%s')])
                        ]),
            VarFileInfo([VarStruct(u'Translation', [1031, 1200])])
            ]
        )""" % (version_ext, exefile, version, datetime.datetime.now().year, exefile, f"py_{sys.version_info.major}.{sys.version_info.minor}.{sys.version_info.micro}")


with open('version.txt', 'w') as f:
    f.write(info)

builddir = os.path.join(dirname, "build\\plugin_runner")
distdir = os.path.join(dirname, "dist\\plugin_runner")
for d in (builddir, distdir):
    if os.path.isdir(d):
        shutil.rmtree(d)

try:
    proc = subprocess.check_call(build_args)
except Exception as e:
    print("error while building exe: %s" % str(e))
    sys.exit(-1)


for srcfile, dstfile in plugin_runner_files:
    shutil.copyfile(os.path.join(dirname, srcfile), os.path.join(
        dirname, "dist", "plugin_runner", dstfile))
for srcdir, dstdir in plugin_runner_dirs:
    shutil.copytree(os.path.join(dirname, srcdir), os.path.join(
        dirname, "dist", "plugin_runner", dstdir))

if args.include_bfa_xmlmapper is True:
    for srcfile, dstfile in bfa_xmlmapper_files:
        if not os.path.isdir(os.path.join(dirname, "dist", "plugin_runner", "plugins", "bfa_xmlmapper")):
            os.makedirs(os.path.join(dirname, "dist",
                        "plugin_runner", "plugins", "bfa_xmlmapper"))
        shutil.copyfile(os.path.join(dirname, srcfile), os.path.join(
            dirname, "dist", "plugin_runner", "plugins", "bfa_xmlmapper", dstfile))
    for srcdir, dstdir in bfa_xmlmapper_dirs:
        shutil.copytree(os.path.join(dirname, srcdir), os.path.join(
            dirname, "dist", "plugin_runner", "plugins", "bfa_xmlmapper", dstdir))

if args.include_send_mail is True:
    for srcfile, dstfile in send_mail_files:
        if not os.path.isdir(os.path.join(dirname, "dist", "plugin_runner", "plugins", "send_mail")):
            os.makedirs(os.path.join(dirname, "dist",
                        "plugin_runner", "plugins", "send_mail"))
        parent = pathlib.Path(os.path.join(dirname, "dist", "plugin_runner", "plugins", "send_mail", dstfile)).parent
        if not parent.exists():
            os.makedirs(str(parent))
        shutil.copyfile(os.path.join(dirname, srcfile), os.path.join(
            dirname, "dist", "plugin_runner", "plugins", "send_mail", dstfile))
    for srcdir, dstdir in send_mail_dirs:
        shutil.copytree(os.path.join(dirname, srcdir), os.path.join(
            dirname, "dist", "plugin_runner", "plugins", "send_mail", dstdir))

if args.include_dubletten_check is True:
    for srcfile, dstfile in dubletten_check_files:
        if not os.path.isdir(os.path.join(dirname, "dist", "plugin_runner", "plugins", "dubletten_check")):
            os.makedirs(os.path.join(dirname, "dist",
                        "plugin_runner", "plugins", "dubletten_check"))
        shutil.copyfile(os.path.join(dirname, srcfile), os.path.join(
            dirname, "dist", "plugin_runner", "plugins", "dubletten_check", dstfile))
    for srcdir, dstdir in dubletten_check_dirs:
        shutil.copytree(os.path.join(dirname, srcdir), os.path.join(
            dirname, "dist", "plugin_runner", "plugins", "dubletten_check", dstdir))

if args.include_example_plugin is True:
    for srcfile, dstfile in example_plugin_files:
        if not os.path.isdir(os.path.join(dirname, "dist", "plugin_runner", "plugins", "example_plugin")):
            os.makedirs(os.path.join(dirname, "dist",
                        "plugin_runner", "plugins", "example_plugin"))
        shutil.copyfile(os.path.join(dirname, srcfile), os.path.join(
            dirname, "dist", "plugin_runner", "plugins", "example_plugin", dstfile))
    for srcdir, dstdir in example_plugin_dirs:
        shutil.copytree(os.path.join(dirname, srcdir), os.path.join(
            dirname, "dist", "plugin_runner", "plugins", "example_plugin", dstdir))

if args.include_xml_export is True:
    for srcfile, dstfile in xml_export_plugin_files:
        if not os.path.isdir(os.path.join(dirname, "dist", "plugin_runner", "plugins", "xml_export")):
            os.makedirs(os.path.join(dirname, "dist",
                        "plugin_runner", "plugins", "xml_export"))
        shutil.copyfile(os.path.join(dirname, srcfile), os.path.join(
            dirname, "dist", "plugin_runner", "plugins", "xml_export", dstfile))

if args.include_bfa_field_sync is True:
    for srcfile, dstfile in bfa_field_sync_plugin_files:
        if not os.path.isdir(os.path.join(dirname, "dist", "plugin_runner", "plugins", "bfa_field_sync")):
            os.makedirs(os.path.join(dirname, "dist",
                        "plugin_runner", "plugins", "bfa_field_sync"))
        shutil.copyfile(os.path.join(dirname, srcfile), os.path.join(
            dirname, "dist", "plugin_runner", "plugins", "bfa_field_sync", dstfile))

if args.include_bfa_fname_mapper is True:
    for srcfile, dstfile in bfa_fname_mapper_plugin_files:
        if not os.path.isdir(os.path.join(dirname, "dist", "plugin_runner", "plugins", "bfa_fname_mapper")):
            os.makedirs(os.path.join(dirname, "dist",
                        "plugin_runner", "plugins", "bfa_fname_mapper"))
        shutil.copyfile(os.path.join(dirname, srcfile), os.path.join(
            dirname, "dist", "plugin_runner", "plugins", "bfa_fname_mapper", dstfile))

# shutil.copyfile(os.path.join(dirname, "plugin_runner_example.ini"),
#                 os.path.join(dirname, "dist", "plugin_runner", "plugin_runner_example.ini"))

# shutil.copyfile(os.path.join(dirname, "plugin_runner.bat"),
#                 os.path.join(dirname, "dist", "plugin_runner.bat"))

# shutil.copytree(os.path.join(dirname, "plugins"),
#                 os.path.join(dirname, "dist", "plugin_runner", "plugins"))


# for dirpath, dirnames, filenames in os.walk(os.path.join(dirname, "dist", "plugin_runner", "plugins")):
#     if dirpath.endswith(".svn") or dirpath.endswith("__pycache__"):
#         shutil.rmtree(dirpath, onerror=on_rm_error)
