#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  xml_import.py
#  
#  Copyright 2015 t.hassler <t.hassler@PCTH2>
#  
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#  
#  

import xml.etree.ElementTree as eTree
import sys,  os,  shutil
import configparser
import re
import glob
import codecs
import tempfile
import logging
import logging.handlers
import getpass
from dpapi import cryptData, decryptData
import base64

class XML_Import():
    def __init__(self, configpath):
        
        # config
        self.config = configparser.RawConfigParser()
        # self.config.optionxform=unicode
        try:
            self.config.readfp(codecs.open(configpath, "r", "utf8"))
        except:
            print "configpath '%s' not found" %configpath
            sys.exit(1)
        #self.config.read(configpath)
        
        # Logging
        self.log = logging.getLogger("XML-Import")
        self.log.propagate = False
        self.log.setLevel(getattr(logging, self.config.get("main", "loglevel").upper(), None))
        
        handler = logging.handlers.RotatingFileHandler(
              self.config.get("main", "logfile"), maxBytes=5000000, backupCount=5)
        
        formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
        handler.setFormatter(formatter)
        self.log.addHandler(handler)
        streamhandler = logging.StreamHandler(sys.stdout)
        streamformatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
        streamhandler.setFormatter(streamformatter)
        self.log.addHandler(streamhandler)
        
        
        #self.log.debug("debug test")
        #xml.log.info(100*"-")
        self.log.info("XML-Import init")
        
        if self.config.has_option("main", "bfauser") is False or self.config.has_option("main", "bfapass") is False:
            self.log.error("bfauser/bfapss not found in ini")
            raise Exception("bfauser/bfapss not found in ini")

        
        try:
            self.bfauser = decryptData(base64.b64decode(self.config.get("main", "bfauser")))
            self.user=self.bfauser
            self.bfapass = decryptData(base64.b64decode(self.config.get("main", "bfapass")))
        except Exception as e:
            self.log.debug(str(e))
            self.log.error("error decrypting bfauser/bfapss")
            sys.exit(1)
            
        
        # Templates
        self.templatepath = self.config.get("main", "templatepath")
        self.templates = {}
        for template in glob.glob(os.path.join(self.templatepath, "*.tpl")):
            self.templates[os.path.basename(template)[:-4]] = template
        self.templatearcids = {}
        for templatename, path in self.templates.items():
            with open(path, "r") as fobj:
                parser = configparser.RawConfigParser()
                parser.readfp(fobj)
            try:
                if parser.getint("Version", "Version") == 36:
                    arcid = parser.getint("Archiv", "Arcid")
                    self.templatearcids[templatename] = arcid
            except:
                self.log.error("could not parse template %s" %(path))
                continue
        self.log.debug("XML-Import templates loaded")
        
        self.bfapath = self.config.get("main", "bitfarmpath")
        self.profilename = self.config.get("main", "profile")
        # rootnode
        self.rootnode = self.config.get("main", "rootnode")[1:-1]

        self.document = self.config.get("main", "document")
        self.uebergabe = self.config.get("main", "uebergabe")
        self.deletexml = self.config.getboolean("main", "deletexml")
        self.deletedoc = self.config.getboolean("main", "deletedoc")
        
            
        
        
        # Archive
        self.archives = {}
        for arc in self.config.options("Archive"):
            # TODO: Prüfen, ob zu jedem konfigurierten Archiv auch ein Template existiert
            opt = self.config.get("Archive", arc)
            self.archives[arc] = {"sortcondition":opt, "addfields":{}, "statfields":{}, "plugin":None}
        self.log.debug("XML-Import archives loaded")
        
        # Zusatzfelder
        for arc in self.archives:
            zus_section = "zus_"+arc
            if zus_section in self.config.sections():
                zusopts = self.config.options(zus_section)
                for field in zusopts:                
                    self.archives[arc]["addfields"][field] = self.config.get(zus_section, field)
        self.log.debug("XML-Import addfield configuration loaded")            
        # Statusfelder
        for arc in self.archives:
            stat_section = "stat_"+arc
            if stat_section in self.config.sections():
                statopts = self.config.options(stat_section)
                for field in statopts:                
                    self.archives[arc]["statfields"][field] = self.config.get(stat_section, field)
        self.log.debug("XML-Import statefield configuration loaded")   
        # casting
        if "Plugins" in self.config.sections():
            for archiv in self.config.options("Plugins"):
                if archiv in self.archives:
                    self.archives[archiv]["plugin"]=self.config.get("Plugins", archiv) 
        self.log.debug("XML-Import plugin configuration loaded") 
        
    def load_xml(self, xmlpath):
        xmlFile = open(xmlpath, "r")
        self.xmlStr=xmlFile.read()
        xmlFile.close()
        self.xmlfile = xmlpath
        
        
        self.archive = None
        self.zusatzfelder = None
        self.statusfelder = None
        
        
        self.tree = eTree.fromstring(self.xmlStr)
        self.xmlroot = self.tree
        if self.rootnode != self.tree.tag:
            self.xmlroot = self.tree.find(self.rootnode)
        
#        if "user" not in self.config.options("main"):
#            self.user = "windowsuser"
#        else:
#            try:
#                self.user = self._get_node_by_string(self.config.get("main", "user")).text
#            except:
#                self.user = "windowsuser"
#        if len(self.user)==0:
#            self.user = "windowsuser"
#        if self.user == "windowsuser":
#            self.user = getpass.getuser()
        
        if self.xmlroot is None:
            self.log.error("rootnode '%s' not found, xml will not be parsed" %self.rootnode)
            return None
        if self.document == "file":
            self.log.debug("document referenced by filename")
            docfiles = glob.glob(os.path.join(os.path.dirname(xmlpath), os.path.splitext(os.path.basename(xmlpath))[0]+".*"))
            docfiles.remove(xmlpath)
            if len(docfiles) < 1:
                self.log.error("document-file not found! lookup-path: '%s'" %os.path.join(os.path.dirname(xmlpath), os.path.splitext(os.path.basename(xmlpath))[0]+".*"))
                return None
            elif len(docfiles) > 1:
                self.log.error("multiple document-files found, need exactly 1 file! lookup-path: '%s'" %os.path.join(os.path.dirname(xmlpath), os.path.splitext(os.path.basename(xmlpath))[0]+".*"))
                return None
            docfile = docfiles[0]
            self.log.debug("document found: '%s'" %docfile)
        else:
            self.log.debug("document referenced by node '%s" %self.document)
            #docnode = self._get_node_by_string(self.document)'
            try:
                docfile = self._get_node_by_string(self.document).text
            except Exception as e:
                self.log.error("error getting document-file: %s" %(str(e)))
                return None
            self.log.debug("document reference '%s'" %docfile)
            if not os.path.isfile(docfile):
                docfile = os.path.join(os.path.dirname(xmlpath), docfile)
                if not os.path.isfile(docfile):
                    self.log.error("document-file not found!")
                    return None
                else:
                    self.log.debug("document reference '%s' found" %docfile)
            else:
                self.log.debug("document reference '%s' found" %docfile)
           
        self.docfile = docfile
        self.archive = self.eval_archive()
        if self.archive is None:
            return None
        
        self.zusatzfelder = self.eval_addfields()
        self.statusfelder = self.eval_statusfields()
        return True
        
    def _get_value(self, node, nodestring= None):
        if nodestring is not None:
            attrib = re.search("@([^@]*)@$", nodestring)
            if attrib:
                attributename = attrib.group(1)
                if attributename in node.attrib:
                    value = node.attrib[attributename]
                    return value
        else:
            return node.text
        return None
        
        
    def eval_statusfields(self):
        self.log.info("determining statefields") 
        statusfelder = {}
        for field in self.archives[self.archive]["statfields"]:
            statusfelder[field] = False
            s = self.archives[self.archive]["statfields"][field].strip()
            self.log.debug("testing field '%s' using evalstring '%s'" %(field, s))
            sc = self._replace_nodes_with_val(s)
            result = self._eval_str(sc)
            if result is True:
                self.log.debug("parsed evalstring '%s' matched" %(sc))
                statusfelder[field] = True
            else:
                self.log.debug("parsed evalstring '%s' did not match" %(sc))
        self.log.info("statefields: %s" %(str(statusfelder)))
        return statusfelder
        
        
    def findnodes(self, valuenode):
        nodes = re.findall("<[^>]*>", valuenode)
        trenner = re.findall(">([^<]*)<",valuenode)
        evalnodes = []
        oldval = None
        for index, val in enumerate(nodes):
            if oldval is None:
                oldval = val
            else:
                if len(trenner[index-1])>0:
                    evalnodes.append(oldval)
                    oldval = val
                else:
                    oldval = oldval + val
        evalnodes.append(oldval)
        return evalnodes
    
    def _get_node_by_string(self, nodestring, root=None):
        if root is None:
            root = self.xmlroot
        if root.tag == nodestring.replace("<","").replace(">","").strip():
            return root
        nodestring = nodestring.strip()
        nodes = re.sub(">\s*<", "||", nodestring).replace("<","").replace(">","").split("||")
        e = root
        result = []
        for node in nodes:
#            attrib = re.search("(@[^@]*@)$", node)
#            if attrib and len(attrib.groups())==1:
#                node = re.sub("(@[^@]*@)$", "", node)
            e = e.find(node)
            if e is None:
                return None
        return e
        
    def _replace_nodes_with_val(self, s, rootnode=None):
        geklammert = re.findall("\([^[\)|\(]*\)", s)
        for ka in geklammert:
            #nodestrings = re.findall("<.*?>",ka)
            nodestrings=self.findnodes(ka)
            for nodestring in nodestrings:
                node = self._get_node_by_string(nodestring, rootnode)
                nodeval = node.text
                if nodeval is None:
                    nodeval = ""
                s = s.replace(nodestring, nodeval.replace("(", "&#40;").replace(")", "&#41;"))
        return s

    def _eval_str(self, s):
        operators1 = ["!=", "contains not", "contains", "<=", ">=", "=<", "=>", "="]
        operators2 = ["<", ">"]
        geklammert = re.findall("\([^[\)|\(]*\)", s)
        for expression in geklammert:
            opfound=False
            for op in operators1:
                parts = expression[1:-1].split(op)
                parts = [x.strip() for x in parts]
                if len(parts)==2:
                    opfound=True
                    if op == "=":
                        if parts[0] == parts[1]:
                            s = s.replace(expression, "True")
                        else:
                            s = s.replace(expression, "False")
                    elif op == "!=":
                        if parts[0] != parts[1]:
                            s = s.replace(expression, "True")
                        else:
                            s = s.replace(expression, "False") 
                    elif op == ">=" or op == "=>":
                        try:
                            if float(parts[0]) >= float(parts[1]):
                                s = s.replace(expression, "True")
                            else:
                                s = s.replace(expression, "False") 
                        except:
                            s = s.replace(expression, "False") 
                    elif op == "<=" or op == "=<":
                        try:
                            if float(parts[0]) <= float(parts[1]):
                                s = s.replace(expression, "True")
                            else:
                                s = s.replace(expression, "False") 
                        except:
                            s = s.replace(expression, "False")         
                    elif op == "contains":
                        if parts[1] in parts[0]:
                            s = s.replace(expression, "True")
                        else:
                            s = s.replace(expression, "False")   
                    elif op == "contains not":
                        if parts[1] in parts[0]:
                            s = s.replace(expression, "False")
                        else:
                            s = s.replace(expression, "True") 
                    break
            if opfound is False:
                for op in operators2:
                    parts = expression[1:-1].split(op)
                    parts = [x.strip() for x in parts]
                    if len(parts)==2:
                        opfound=True
                        if op == "<":
                            try:
                                if float(parts[0]) < float(parts[1]):
                                    s = s.replace(expression, "True")
                                else:
                                    s = s.replace(expression, "False") 
                            except:
                                s = s.replace(expression, "False") 
                        elif op == ">":
                            try:
                                if float(parts[0]) > float(parts[1]):
                                    s = s.replace(expression, "True")
                                else:
                                    s = s.replace(expression, "False") 
                            except:
                                s = s.replace(expression, "False") 
                        break
                    
        allowed = ["true", "false", "(", ")", "not", "and", "or", " "]
        total = len(s)
        for el in allowed:
            total -= s.lower().count(el) * len(el)
        if total != 0:
            return None
        return eval(s)
        
    def eval_archive(self):
        self.log.info("determining archive") 
        for arc in self.archives:
            
            if "sortcondition" in self.archives[arc]:
                self.log.debug("testing condition '%s'" %(self.archives[arc]["sortcondition"])) 
                sc = self._replace_nodes_with_val(self.archives[arc]["sortcondition"])
                self.log.debug("parsed condition: '%s'" %(sc))
                result = self._eval_str(sc)
                if result is True:
                    self.log.info("condition matched, archive '%s' determined" %arc) 
                    return arc
                else:
                    self.log.debug("condition did not match") 
        self.log.info("could not determine archive")         
        return None
        
    def eval_addfields(self):
        self.log.info("determining addfields using section '%s'" %("zus_"+self.archive)) 
        zusatzfelder = {}
        #for arc in self.archives:
        for field in self.archives[self.archive]["addfields"]:
            self.log.debug("determining addfield '%s'" %field)
            s = self.archives[self.archive]["addfields"][field].strip()
            self.log.debug("using evalstring '%s'" %s)
            
            evalstr = s
            if s.startswith("List"):
                self.log.debug("List-command found")
                l = re.match("List\s*\((.*)\)", evalstr) 
                if len(l.groups())==1:
                    evalstr = l.groups()[0]
                strings = evalstr.split(",")
                if len(strings)==3:
                    thisrootstring, iternodestring, valuenodestring = strings
                elif len(strings)>3:
                    thisrootstring, iternodestring= strings[:2]
                    valuenodestring = ",".join(strings[2:])
                valuenodestring = valuenodestring.strip()
                thisrootstring = thisrootstring.strip()
                iternodestring = iternodestring.strip()
                thisroot = self._get_node_by_string(thisrootstring, self.xmlroot)
                iternodetag = iternodestring.replace("<", "").replace(">", "").strip()
                
                if valuenodestring == "self":
                    result = []
                    for node in thisroot.findall(iternodetag):
                        if node.text.strip() != "":
                            result.append(node.text.strip())
                            self.log.debug("partial result found for '%s': '%s'" %(field, node.text.strip()))
                        else:
                            self.log.debug("no value found for '%s', node '%s' is empty" %(field, node.tag))
                        
                    if len(result)>0:
                        slicing = self.archives[self.archive]["addfields"][field].replace(evalstr,"").replace("List", "").replace("(","").replace(")","").replace(" ", "")
                        if len(slicing)>0:
                            self.log.debug("using slicing '%s'" %slicing)
                            match = re.match("\[(-?\d*):(-?\d*)\]", slicing)
                            if match:
                                left=match.groups()[0]
                                right=match.groups()[1]
                                if right != "":
                                    right = int(right)
                                else:
                                    right=None
                                if left != "":
                                    left = int(left)
                                else:
                                    left=None
                                zusatzfelder[field]=";".join(result[left:right])
                            else:
                                self.log.error("slicing command malformed, skipping field '%s'" %field)
                        else:
                            zusatzfelder[field]=";".join(result)
                        
                        
                else:
                    evalnodes = self.findnodes(valuenodestring)
                    result = []
                    
                    for node in thisroot.findall(iternodetag):
                        s = valuenodestring
                        for enodestr in evalnodes:
                            enode = self._get_node_by_string(enodestr, node)
                            if enode is not None:
                                if enode.text is not None:
                                    s = s.replace(enodestr, enode.text.strip())
                                    
                                else:
                                    s = s.replace(enodestr, "")
                                    self.log.debug("no value found for '%s', node '%s' is empty" %(field, enode.tag))
                            else:
                                s = None
                                self.log.error("error parsing evalstring '%s', skipping field '%s'" %(evalstr, field))
                                break
                        if s is not None:
                            self.log.debug("partial result found for '%s': '%s'" %(field, s))
                            result.append(s)
                    if len(result)>0:
                        slicing = self.archives[self.archive]["addfields"][field].replace(evalstr,"").replace("List", "").replace("(","").replace(")","").replace(" ", "")
                        if len(slicing)>0:
                            self.log.debug("using slicing '%s'" %slicing)
                            match = re.match("\[(-?\d*):(-?\d*)\]", slicing)
                            if match:
                                left=match.groups()[0]
                                right=match.groups()[1]
                                if right != "":
                                    right = int(right)
                                else:
                                    right=None
                                if left != "":
                                    left = int(left)
                                else:
                                    left=None
                                zusatzfelder[field]=";".join(result[left:right])
                            else:
                                self.log.error("slicing command malformed, skipping field '%s'" %field)
                                
                        else:
                            zusatzfelder[field]=";".join(result)
                        self.log.debug("value for field '%s': '%s'" %(field, zusatzfelder[field]))
            elif s.startswith("If") or s.startswith("If"):
                self.log.debug("If-command found")
                result = re.match("If\s*\((.*)\)", evalstr) 
                conditionresult = False
                if result is not None:
                    conditionstring = result.groups()[0]
                    a = conditionstring.split(",")
                    a = [x.strip() for x in a]
                    if len(a)==3:
                        self.log.debug("multi-node If-Expression")
                        multinode = True
                        conditionstring = a[0]
                        rootnode = self._get_node_by_string(a[1])
                        iternodestring = a[2].replace("<","").replace(">","")
                        valuestring = re.sub("If\s*\((.*)\)", "", evalstr).strip()
                    elif len(a)==1:
                        self.log.debug("single-node If-Expression")
                        multinode = False
                    else:
                        # darf nicht sein, falsch konfiguriert!
                        self.log.error("malformed If-Expression, skipping field '%s'" %field)
                        continue
                    if multinode is False:    
                        self.log.debug("testing condition '%s'" %(conditionstring))
                        condition = self._replace_nodes_with_val(conditionstring)
                        conditionresult = self._eval_str(condition)
                        if conditionresult is True:
                            self.log.debug("condition matched")
                            valuestring = evalstr.replace(conditionstring, "").replace("If", "").replace("(", "").replace(")", "").replace(" ", "")
                            evalnodes = self.findnodes(valuestring)
                            s = valuestring
                            for enodestr in evalnodes:
                                enode = self._get_node_by_string(enodestr)
                                if enode is not None:
                                    if enode.text is not None:
                                        s = s.replace(enodestr, enode.text.strip())
                                    else:
                                        s = s.replace(enodestr, "")
                                else:
                                    s = None
                                    break
                            if s is not None:
                                self.log.debug("value for field '%s': '%s'" %(field, s))
                                zusatzfelder[field]=s
                        else:
                            self.log.debug("condition did not match")
                    else:
                        for node in rootnode.findall(iternodestring):
                            condition = self._replace_nodes_with_val(conditionstring, node)
                            self.log.debug("testing condition '%s'" %(conditionstring))
                            conditionresult = self._eval_str(condition)
                            if conditionresult is True:
                                self.log.debug("condition matched")
                                evalnodes = self.findnodes(valuestring)
                                s = valuestring
                                for enodestr in evalnodes:
                                    enode = self._get_node_by_string(enodestr, node)
                                    if enode is not None:
                                        if enode.text is not None:
                                            s = s.replace(enodestr, enode.text.strip())
                                        else:
                                            s = s.replace(enodestr, "")
                                    else:
                                        s = None
                                        break
                                if s is not None:
                                    self.log.debug("value for field '%s': '%s'" %(field, s))
                                    zusatzfelder[field]=s
                                break
                            else:
                                self.log.debug("condition did not match")
                else:
                    self.log.error("malformed If-Expression, skipping field '%s'" %field)
            else:
                evalnodes = self.findnodes(s)
                if evalnodes == [None]:
                    self.log.info("no node found for evalstring '%s' (field '%s'), using as fixvalue." % (evalstr, field))
                    # self.log.error("error parsing evalstring '%s', skipping field '%s'" %(evalstr, field))
                    # continue
                else:
                    for enodestr in evalnodes:
                        enode = self._get_node_by_string(enodestr)
                        if enode is not None:
                            if enode.text is not None:
                                s = s.replace(enodestr, enode.text.strip())
                            else:
                                self.log.debug("no value found for '%s', node '%s' is empty" %(field, enode.tag))
                                s = s.replace(enodestr, "")
                        else:
                            self.log.error("error parsing evalstring '%s', skipping field '%s'" %(evalstr, field))
                            s = None
                            break
                if s is not None:
                    self.log.debug("value for field '%s': '%s'" %(field, s))
                    zusatzfelder[field]=s
        self.log.info("addfields: " + str(zusatzfelder))
        return zusatzfelder
            
        
def main():
    #xmlFilePath = r"C:\Users\t.hassler\projects\Tools\bfa_xml_importer\test3_list_self.xml"
    xmlFilePath = r"C:\Users\t.hassler\projects\Tools\bfa_xml_importer\test3.xml"
    configpath = r"C:\Users\t.hassler\projects\Tools\bfa_xml_importer\xml_import.ini"
    xmlimport = XML_Import(configpath)
    xmlimport.load_xml(xmlFilePath)
    print "ermitteltes Archiv:", xmlimport.archive
    from pprint import pprint
    print "ermittelte Zusatzfelder:"
    pprint(xmlimport.zusatzfelder)
    print "ermittelte Statusfelder:"
    pprint(xmlimport.statusfelder)
    
if __name__ == '__main__':
    main()

