unit u_bfaAddfield;

interface

uses
  Windows, Messages, Variants, Classes, Graphics, StdCtrls, StrUtils,
  Forms, Controls, JvCombobox, Dialogs, System.Generics.Collections,
  bfaBaseObjects, bfaCommonUtils, gnugettext, Vcl.Imaging.pngimage,
  SpTBXControls, SpTBXEditors, SpTBXItem, TB2Item, TB2Dock, TB2Toolbar, cxCalendar,
  cxDropDownEdit, cxMaskEdit, cxTextEdit, cxSpinEdit, cxLabel, dxTokenEdit, cxCheckListBox,
  dxCalloutPopup, cxCheckBox, cxLookAndFeelPainters, cxEdit, cxControls, cxGraphics,
  cxGridTableView, cxGridDBTableView, cxDataStorage, cxGrid, cxGridLevel, cxGridPopupMenu,
  cxCustomData, cxButtons, cxButtonEdit, cxGridCustomTableView, cxDBLookupComboBox,
  cxImage, cxStyles, cxImageList, VCL.ImgList, VirtualImageList, oleacc,
  u_bfaScrollBox, DBClient, Data.DB, System.RegularExpressions;


type
  GridColumnHeaderPainterAccess = class(TcxGridColumnHeaderPainter);

  TbfaJvCheckedComboBox = class(TJvCheckedComboBox)
  published
    property OnPopupShown;
    property OnPopupHidden;
  end;

  TFieldState = (fsNone, fsSet, fsUnset);

  TbfaLabel = class(TcxLabel)
  public
    constructor Create(AOwner: TComponent); override;
  end;

  TbfaTextEdit = class(TcxMaskEdit)
  private
    fModified: Boolean;
    function GetColor: TColor;
    procedure SetColor(const Value: TColor);
  public
    property Color: TColor read GetColor write SetColor;
    property Modified: Boolean read fModified write fModified;
  end;

  TbfaSpinEdit = class(TcxSpinEdit)
  private
    fModified: Boolean;
    fMaxDecimal: Integer;
    property EditingValue;

    function TrimLeadingZeros(const S: string): string;
    function GetColor: TColor;
    procedure SetColor(const Value: TColor);
    procedure OnValidate(Sender: TObject; var DisplayValue: Variant; var ErrorText: TCaption; var Error: Boolean);
    procedure SpinEditOnChange(Sender: TObject);
  protected
    function IsValidChar(AChar: Char): Boolean; override;
    procedure SpinEditOnKeyPress(Sender: TObject; var Key: Char; isIntervall: Boolean = false);
  public
    NegativeCurrencyRed: Boolean;
    property MaxDecimal: Integer read fMaxDecimal write fMaxDecimal;
    property Color: TColor read GetColor write SetColor;
    property Modified: Boolean read fModified write fModified;
    procedure PasteFromClipboard; override;
    procedure SpinEditOnValueChanged(Sender: TObject);

    constructor Create(AOwner: TComponent);
  end;

  TbfaComboBox = class(TcxComboBox)
  private
    FStoredItems: TStringList;
    function GetAutoComplete: Boolean;
    function GetAutoDropDown: Boolean;
    function GetColor: TColor;
    function GetItems: TStrings;
    procedure SetStoredItems(const Value: TStringList);
    procedure SetAutoComplete(const Value: Boolean);
    procedure SetAutoDropDown(const Value: Boolean);
    procedure SetColor(const Value: TColor);
    procedure SetItems(const Value: TStrings);
  public
    SkipChange: Boolean;
    property Color: TColor read GetColor write SetColor;
    property StoredItems: TStringList read FStoredItems write SetStoredItems;
    property AutoComplete: Boolean read GetAutoComplete write SetAutoComplete;
    property AutoDropDown: Boolean read GetAutoDropDown write SetAutoDropDown;
    property Items: TStrings read GetItems write SetItems;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

  TbfaCheckedListBox = class(TcxCheckListBox)
  private
    function GetItems: TStrings;
  private
    procedure CMCancelMode(var Message: TCMCancelMode); message CM_CANCELMODE;
    procedure WMLButtonDown(var Message: TWMLButtonDown); message WM_LBUTTONDOWN;
    procedure WndProc(var Message: TMessage); override;
  public
    Field: TComponent;
    PopupOpen: Boolean;
    OnDropDown: procedure (Sender: TObject) of object;
    OnCloseUp: procedure (Sender: TObject) of object;
    property ItemsString: TStrings read GetItems;
    procedure CheckAll(AState: TCheckBoxState);
  end;

  TbfaCheckedCombo = class(TdxTokenEdit)
  private
    FSkipSearchByReturn: Boolean;
    FOnBfaCanResize: TCanResizeEvent;
    function GetColor: TColor;
    procedure SetColor(const Value: TColor);
    procedure CheckComboCanResize(Sender: TObject; var NewWidth, NewHeight: Integer; var Resize: Boolean);
  public
    OnResizeHeight: procedure (Sender: TObject; NewWidth, NewHeight: Integer; IsSettingValue: Boolean) of object;
    property Color: TColor read GetColor write SetColor;
    property SkipSearchByReturn: Boolean read FSkipSearchByReturn write FSkipSearchByReturn;
    constructor Create(AOwner: TComponent); override;
  published
    // for accessing OnCanResize Event of tokenEdit - Currently not in use
    property OnBfaCanResize: TCanResizeEvent read FOnBfaCanResize write FOnBfaCanResize;
  end;


  TbfaGridfield = class(TComponent)
  private
    { Private declarations }
    FConfig: TbfaGridConfig;
    FBalloonHint: TBalloonHint;
    FMsgNotAllowedChar: string;
    FNotAllowChar: string;
    LookupDataSource: TDataSource;
    LookupDataSet: TClientDataSet;

    function GetFooterSummaryItems: TcxDataFooterSummaryItems;
    function GetGroupSummaryItems: TcxDataGroupSummaryItems;
  public
    View: TcxGridDBTableView;
    Level: TcxGridLevel;
    Grid: TcxGrid;
    DataSource: TDataSource;
    DataSet: TClientDataSet;
    StyleOdd, StyleBlack, StyleRed, StyleFree, StyleSelected: TcxStyle;
    PopupStandard: TcxGridPopupMenu;
    UseBuiltInPopupMenus : Boolean;
//    UseColumnNamesForExport : Boolean;
    NegativeCurrencyRed: Boolean;
    ColumnModified: Boolean;
    RecID: Integer;
    VisibleRows: Integer;
    NewHeight: Integer;

    OnModified: procedure(const Value: Boolean) of object;
    OnValueSet: procedure(value, valuehigh: Variant; modified, modifiedhigh: Boolean) of object;
    OnGridComboDropDown: procedure(Sender: TObject) of object;
    OnGridComboCloseup: procedure(Sender: TObject) of object;
    OnResizeHeight: procedure (Sender: TObject; NewWidth, NewHeight: Integer; IsSettingValue: Boolean) of object;

    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure ResetColumns;
    procedure ResetGridHeight;
    procedure bfaOnBeforeInsert(ADatacontroller: TcxCustomDataController);
    procedure OnGridValueChanged(Sender: TObject);
    procedure OnGridChanged(Sender: TObject);
    procedure OnCurrencyChange(Sender: TObject);
    procedure OnGridPopup(Sender: TObject);
    procedure OnGridCloseup(Sender: TObject);
    procedure OnGridAfterInsert(ADatacontroller: TcxCustomDataController);
    procedure GetContentStyle(Sender: TcxCustomGridTableView; ARecord: TcxCustomGridRecord;
        AItem: TcxCustomGridTableItem; var AStyle: TcxStyle);
    procedure GridCustomDrawColumnHeader(Sender: TcxGridTableView; ACanvas: TcxCanvas;
      AViewInfo: TcxGridColumnHeaderViewInfo; var ADone: Boolean);
    procedure GridCustomDrawFooterCell(Sender: TcxGridTableView;
      ACanvas: TcxCanvas; AViewInfo: TcxGridColumnHeaderViewInfo; var ADone: Boolean);
    procedure GridCustomDrawGroupCell(Sender : TcxCustomGridTableView;
     ACanvas: TcxCanvas;
      AViewInfo: TcxGridTableCellViewInfo; var ADone: Boolean);
    procedure GridCustomDrawCell(Sender: TcxCustomGridTableView;
      ACanvas: TcxCanvas; AViewInfo: TcxGridTableDataCellViewInfo; var ADone: Boolean);
    procedure GridColumnChanged(Sender: TcxGridTableView; AColumn: TcxGridColumn);
    procedure LoadConfig(AConfig: TbfaGridConfig; SkipFormat : Boolean = False);
    function GetValue(UseColumnNames: Boolean = False; WithReadonly : Boolean = False; OnlySelection: Boolean = False): string;
    function GetColumnConfigByFieldName(fieldname: string): TbfaColumnConfig;
    procedure GetExport(var header: string; var values: string; UseColumnNames: Boolean = False);

    property FooterSummaryItems: TcxDataFooterSummaryItems read GetFooterSummaryItems;
    property GroupSummaryItems: TcxDataGroupSummaryItems read GetGroupSummaryItems;
    property Config: TbfaGridConfig  read FConfig write FConfig;
  end;

// bfacrm:ticket__show__28579
  TbfaButtonEdit = class(TcxButtonEdit)
  private
    fLastDoubleClick: Cardinal;
    fSelStart, fSelLength: Integer;

    procedure ButtonEditDoubleClick(Sender: TObject);
    procedure ButtonEditClick(Sender: TObject);
  public
    constructor Create(AOwner: TComponent);
  end;

  TBfaAddfieldComp = class(TSpTBXPanel)
    chkIndexing: TSpTBXCheckBox;
    comp: TComponent;
    comp2: TComponent;
    Label1: TbfaLabel;
    Label2: TbfaLabel;
    compEdit: TbfaTextEdit;
    compSpinEditLow: TbfaSpinEdit;
    compSpinEditHigh: TbfaSpinEdit;
    compComboBox: TbfaComboBox;
    compDateLow: TcxDateEdit;
    compDateHigh: TcxDateEdit;
    compCheckedCombo: TbfaCheckedCombo;
    lstCheckedListbox: TbfaCheckedListBox;
//    compCalloutPopup: TdxCalloutPopup;
    compGrid: TbfaGridfield;
    compGridDataSet: TClientDataSet;
    imglGrid: TcxImageList;
    edtGridSearch: TbfaTextEdit; //TcxTextEdit;
    compGridPopupStandard: TcxGridPopupMenu;
    compGridPopupConfig: TSpTBXPopupMenu;
    cbxGridSearchField: TBfaComboBox;
//    StyleController : TcxEditStyleController;
    cbxIndexing: TBfaComboBox;
    tlbNeg: TSpTBXToolbar;
    cliNeg: TSpTBXColorItem;
    ctrlNeg: TTBControlItem;
    btnIndexing: TSpTBXButton;
    btnAdd: TSpTBXButton;
    btnGridAdd: TcxButton;
    btnGridDelete: TcxButton;
    btnGridStamp: TcxButton;
    btnGridExport: TcxButton;
    btnGridSearch: TcxButton;
    btnGridConfig: TcxButton;

  { Private-Deklarationen }
  private
    fModified: Boolean;
    FModifiedHigh: Boolean;
    fIndexing: Boolean;
    fAddbutton: Boolean;
    fIndexingOffset: Integer;  // wenn die Verschlagwortungshilfe angezeigt wird, mssen die Felder nach rechts rcken
    fLeft, fRight, fMin, fSpare, fFieldWidth : Integer;

    fIsIndexing: Boolean;   // wenn in einem einzelnem Feld gesucht wurde
    fIsSettingValue: Boolean;
    fOldValueSet: Boolean;
    fValue: string;
    fValueHigh: string;
    fOldValue: string;
    fItems: string;
    fState: Integer;   // 1= Suchen 2= Schreiben 4=Pflichtfeld
    fAddfieldType: eAddFieldType;
    fWidth: Integer;
    fTop: Integer;
    fCheckProgram: string;
    fFieldname: string;   // Fieldname
    fDispname: string;
    fListClose: Boolean;
    fOptions: Integer;
    fRights: Integer;
    fAllowedChar: string;
    fNotAllowedChar: string;
    fMsgAllowedChar: string;
    fMsgNotAllowedChar: string;
    fdteSearchDisable: Boolean;
    fDontSetColors: Boolean;
    fEditmode: Boolean;
    fComboitems: TStringList;
    fBalloonHint: TBalloonHint; //05.06.2019 Ticket 19429
    fLastDoubleClick: Cardinal;
    fSelStart, fSelLength: Integer;
    fIgnoreClick: Boolean;

    procedure NegSelect(Sender: TObject);
    procedure btnIndexingClick(Sender: TObject);
    function GetIndexing: Boolean;
    function GetIntervalByIndex(combo : TbfaComboBox) : string;
    procedure SetIndexing(const Value: Boolean);
    procedure CheckedComboClick(Sender: TObject; AButtonIndex: Integer);
    procedure CheckedComboOnExit(Sender: TObject);
    procedure CheckedComboOnKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure OnClickCheck(Sender: TObject; AIndex: Integer;
      APrevState, ANewState: TcxCheckBoxState);
    procedure OnComboMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure OnChange(Sender: TObject); overload;
    procedure OnChange(Sender: TObject; var CanEdit: Boolean); overload;
    procedure OnChangeHigh(Sender: TObject); overload;
    procedure OnChangeHigh(Sender: TObject; var CanEdit: Boolean); overload;
    procedure OnIndexingChange(Sender: TObject);
    procedure OnKeyPress(Sender: TObject; var Key: Char);
    procedure OnKeyPressHigh(Sender: TObject; var Key: Char);
    procedure OnKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure OnKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure OnKeyUpHigh(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure OnExit(Sender: TObject);
    procedure OnExitHigh(Sender: TObject);
    procedure OnEditing(Sender: TObject; var CanEdit: Boolean);
    procedure DateOnEditValueChanged(Sender: TObject);
    procedure OnResizeDate(Sender: TObject);
    procedure OnResizeInterval(Sender: TObject);
    procedure OnResizeNum(Sender: TObject);
    procedure OnDateCloseUp(Sender: TObject);
    procedure OnDatePopup(Sender: TObject);
    procedure OnPropertiesValidate(Sender: TObject;
      var DisplayValue: Variant; var ErrorText: TCaption; var Error: Boolean);
    procedure OnDoubleClick(Sender: TObject);
    procedure OnClick(Sender: TObject);
    procedure OnGridEditing(Sender: TcxCustomGridTableView; AItem: TcxCustomGridTableItem; var AAllow: Boolean);
    procedure TokenDelete(Sender: TObject; const ATokenText: string;
      AToken: TdxTokenEditToken; var AAllow: Boolean);
    procedure TokenAdd(Sender: TObject; const ATokenText: string;
      AToken: TdxTokenEditToken; var AAllow: Boolean);
    procedure TokenDraw(Sender: TObject;
      ACanvas: TcxCanvas; AViewInfo: TdxTokenEditCustomTokenViewInfo; var AHandled: Boolean);
    function GetCompValue: Variant;
    procedure SetCompValue(const Value: Variant);
    procedure SetOldValue(const Value: Variant);
    function GetItems: string;
    function GetStrings: TStrings;
    procedure SetItems(const Value: string);
    procedure SetStrings(const Value: TStrings);
    procedure SetColor(Acomp : TComponent; Acolor: TColor); overload;
    procedure SetAllowedChar(const Value: string);
    procedure SetNotAllowedChar(const Value: string);
    procedure SetDispname(const Value: string);
    procedure SetOptions(const Value: Integer);
    procedure SetRights(const Value: Integer);
    procedure SetModified(const Value: Boolean);
    procedure SetModifiedHigh(const Value: Boolean);
    function GetAddButton: Boolean;
    procedure SetAddButton(const Value: Boolean);
    function GetOldValue: Variant;
    function GetValueHigh: Variant;
    procedure SetValueHigh(const Value: Variant);
    function CheckValidChars(const Value: string): string;
    procedure OnCalenderPopup(Sender: TObject);
    procedure ClickCalenderClear(Sender: TObject);
    procedure AddfieldClick(Sender: TObject);
    procedure AddfieldDblClick(Sender: TObject);
    procedure AddfieldMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure OnConfigClick(Sender: TObject);
    procedure OnConfigPopup(Sender: TObject);
    procedure OnConfigClose(Sender: TObject);
    procedure OnConfigItemClick(Sender: TObject);
    procedure OnGridAddClick(Sender: TObject);
    procedure OnGridDeleteClick(Sender: TObject);
    procedure BtnGridExportClick(Sender: TObject);
    procedure BtnGridStampClick(Sender: TObject);
    procedure OnGridKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure OnGridEditKeyDown(Sender: TcxCustomGridTableView; AItem: TcxCustomGridTableItem;
              AEdit: TcxCustomEdit; var Key: Word; Shift: TShiftState);
    procedure GridSearchFieldChange(Sender: TObject);
    function CtrlPressed: Boolean;
    function AltGrPressed: Boolean;
    function AllowedCtrlKeys: Boolean;
    function IsDeleteOrBack: Boolean;
    procedure SetEditMode(const Value: Boolean);
    procedure AddfieldStyleChanged(Sender: TObject);
    procedure SetButtonColors(AButton: TcxButton; AStyle: TcxEditStyleController);
    function GridHeightCalculate(RowCount: Integer): Integer; //04.10.2024 SD <--bfacrm:ticket__show__39992

  { Public-Deklarationen }
  public
    Negate : Boolean;
    FieldID : Int64;
    IndexTableID : Integer;
    SearchField: string;
//     SearchItems: TStringList;

    OnModified: procedure(Field: TBfaAddfieldComp) of object;
    OnComboClick: procedure(Field: TBfaAddfieldComp) of object;
    OnFieldSearch: procedure(Field: TBfaAddfieldComp; value: string) of object;
    OnValueSet: procedure(value, valuehigh: Variant; modified, modifiedhigh: Boolean) of object;
    OnComboDropDown: procedure (Sender: TObject) of object;
    OnComboCloseUp: procedure (Sender: TObject) of object;
    OnReturnSearch: procedure of object;
    OnGridExportClick: procedure (Field: TBfaAddfieldComp) of object;
    OnGridStampClick: procedure (Field: TBfaAddfieldComp) of object;

    constructor Create(AOwner: TComponent); overload; override;
    constructor Create(AOwner: TComponent; Afieldtype: eAddFieldType;
      ADispname: string = 'Zusatzfeld'; AIndexing: Boolean = False;
      ImgList: TVirtualImageList = nil); overload;

    procedure SetColor(AColor: TColor); overload;
    procedure ShowIndexingCombo(Indexing: Boolean);
    procedure CreateControl(ImgList: TVirtualImageList = nil);
    procedure Reset;
    procedure ClearValue;
    procedure ClearOldValue;
    procedure SetCheckedComboValue;
    procedure LoadGridSearchField;
    procedure ShowBalloonHint(Sender: TObject; Message: string); //04.06.2019 Ticket 19429
    procedure SetcompGridPopupConfigCaptions; //13.09.2024

    property MsgNotAllowedChar: string read fMsgNotAllowedChar;
    property Indexing: Boolean read GetIndexing write SetIndexing;
    property AddButton: Boolean read GetAddButton write SetAddButton;
    property Modified: Boolean read fModified write SetModified;
    property ModifiedHigh: Boolean read FModifiedHigh write SetModifiedHigh;
//    property ModifiedHigh: Boolean read FModifiedHigh write FModifiedHigh;
    property OldValue: Variant read GetOldValue write SetOldValue;
    property Value: Variant read GetCompValue write SetCompValue;
    property ValueHigh: Variant read GetValueHigh write SetValueHigh;

    property Editmode: Boolean read fEditmode write SetEditMode;
    property State: integer read fState write fState;
    property AddfieldType: eAddFieldType read fAddfieldType write fAddfieldType;
    property Items: string read GetItems write SetItems;
    property Strings: TStrings read GetStrings write SetStrings;
    property Checkprogram: string  read fCheckProgram write fCheckProgram;
    property Fieldname: string read fFieldname write fFieldname;
    property Dispname: string read fDispname write SetDispname;
    property ListClose: Boolean read fListClose write fListClose;
    property Options: Integer read fOptions write SetOptions;
    property Rights: Integer read fRights write SetRights;
    property AllowedChar: string read fAllowedChar write SetAllowedChar;
    property NotAllowedChar: string read fNotAllowedChar write SetNotAllowedChar;
    property DontSetColors: Boolean read fDontSetColors write fDontSetColors;
    property IndexingOffset: Integer read fIndexingOffset;
    property IsIndexing: Boolean read fIsIndexing write fIsIndexing;
    property Comboitems: TStringList read fComboitems write fComboitems;
    property IsSettingValue: Boolean read fIsSettingValue;
  end;

  TBfaStatefieldComp = class(TSpTBXItem)
    private
      fOldValue: TFieldState;
      fValue: TFieldState;
      fRights: Integer;
      fModified: Boolean;
      fOldValueSet: Boolean;
      fEditMode: Boolean;
      fViewKind: TbfaViewKind; // Importer kein Click ohne Rechte
      procedure OnClickState(Sender: TObject);
      procedure SetRights(const Value: Integer);
      procedure SetOldValue(const Value: TFieldState);
      procedure SetModified(const Value: Boolean);
      procedure SetViewKind(const Value: TbfaViewKind);
    public
      Bitmask: Int64;
      Name: string;
      FieldID: Int64;
      IndexColumnID: Integer;
      Hint: string;

      OnValueSet: procedure(value: TFieldState; modified: Boolean) of object;

      constructor Create(AOwner: TComponent); overload; override;
      constructor Create(AOwner: TComponent; Bitmask: Int64; Name: string); overload;
      procedure Reset;
      procedure SetCompValue(state: TFieldState);
      procedure ClearValue;
      procedure ClearOldValue;
      procedure SetValueAsClick(state: TFieldState);

      property Value: TFieldState read fValue write SetCompValue;
      property Rights: Integer read fRights write SetRights;
      property Modified: Boolean read fModified write SetModified;
      property OldValue: TFieldState read fOldValue write SetOldValue;
      property EditMode: Boolean read fEditMode write fEditMode;
      property ViewKind: TbfaViewKind read fViewKind write SetViewKind;
    end;

{$R bfaIcons362.res}

{$R icons.RES icons.rc}

implementation

{ TBfaAddfieldComp }
uses
   SysUtils, bfaCommonMethods, bfaCommonStyles, ClipBrd, DateUtils, cxCurrencyEdit, dxCore, dxGDIPlusClasses, cxLookAndFeels,
   bfaGridDBTableView, VCL.GraphUtil, cxGridCommon, cxGridRows;

procedure TBfaAddfieldComp.AddfieldClick(Sender: TObject);
begin
  inherited;
  try
    if (Sender is TcxCustomTextEdit) then
    begin
      if (Double(GetTickCount) < Double(fLastDoubleClick + GetDoubleClickTime * 1.5)) and
         (TcxCustomTextEdit(Sender).GetTextLen > TcxCustomTextEdit(Sender).SelLength) and
         (TcxCustomTextEdit(Sender).SelStart >= fSelStart) and
         (TcxCustomTextEdit(Sender).SelStart <= (fSelStart + fSelLength)) then
      begin
        TcxCustomTextEdit(Sender).SelectAll;
        fLastDoubleClick := 0;
      end;
    end;
  except
    // nur zur Sicherheit
  end;
end;

procedure TBfaAddfieldComp.AddfieldDblClick(Sender: TObject);
begin
  try
    if Sender is TbfaSpinEdit then
      TbfaSpinEdit(Sender).SelectAll
    else if Sender is TcxCustomTextEdit then
    begin
      fLastDoubleClick := GetTickCount;
      fSelStart := TcxCustomTextEdit(Sender).SelStart;
      fSelLength := TcxCustomTextEdit(Sender).SelLength;
      if (Sender is TbfaComboBox) then
        fIgnoreClick := True;
    end;
  except
    // nur zur Sicherheit
  end;
end;

procedure TBfaAddfieldComp.AddfieldMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  try
    if (Sender is TcxCustomTextEdit) then
    begin
      if (Double(GetTickCount) < Double(fLastDoubleClick + GetDoubleClickTime * 1.5)) and
         (TcxCustomTextEdit(Sender).GetTextLen > TcxCustomTextEdit(Sender).SelLength) and
         (TcxCustomTextEdit(Sender).SelStart >= fSelStart) and
         (TcxCustomTextEdit(Sender).SelStart <= (fSelStart + fSelLength)) then
      begin
        if (fIgnoreClick = True) then
          fIgnoreClick := False
        else
        begin
          TcxCustomTextEdit(Sender).SelectAll;
          fLastDoubleClick := 0;
        end;
      end;
    end;
  except
    // nur zur Sicherheit
  end;
end;

procedure TBfaAddfieldComp.AddfieldStyleChanged(Sender: TObject);
begin
//  OutputDebugString('AddfieldStyleChanged');
  try
    if Assigned(btnGridAdd) then
      SetButtonColors(btnGridAdd, ctlGeneral);
    if Assigned(btnGridDelete) then
      SetButtonColors(btnGridDelete, ctlGeneral);
    if Assigned(btnGridStamp) then
      SetButtonColors(btnGridStamp, ctlGeneral);
    if Assigned(btnGridExport) then
      SetButtonColors(btnGridExport, ctlGeneral);
    if Assigned(btnGridConfig) then
      SetButtonColors(btnGridConfig, ctlGeneral);
  except
    //04.07.2023 CrashLog
  end;
end;

function TBfaAddfieldComp.AllowedCtrlKeys: Boolean;
begin
  Result := ((GetKeyState(Ord('A')) < 0) or (GetKeyState(Ord('C')) < 0) or (GetKeyState(Ord('V')) < 0) or (GetKeyState(Ord('X')) < 0) or (GetKeyState(Ord('Z')) < 0));
end;

function TBfaAddfieldComp.AltGrPressed: Boolean;
begin
  Result := (GetKeyState(VK_RMENU) < 0);
end;

function TBfaAddfieldComp.CtrlPressed: Boolean;
begin
  Result := (GetKeyState(VK_CONTROL) < 0);
end;

function TBfaAddfieldComp.IsDeleteOrBack: Boolean;
begin
  Result := ((GetKeyState(VK_DELETE) < 0) or (GetKeyState(VK_BACK) < 0));
end;

procedure TBfaAddfieldComp.LoadGridSearchField;
var
  i: Integer;
  Fieldname, Displayname: string;
begin
  cbxGridSearchField.Items.Clear;

  for i := 0 to compGrid.View.ColumnCount -1 do
  begin
    if compGrid.View.Columns[i].Properties = nil then
      Continue;
    if TcxCustomEditProperties(compGrid.View.Columns[i].Properties).ReadOnly then
      Continue;
    if compGrid.View.Columns[i].Name = 'RecID' then
      Continue;
    if LeftStr(compGrid.View.Columns[i].Name, 4) = 'del_' then
      Continue;

    Fieldname := compGrid.View.Columns[i].Name;
    Displayname := compGrid.View.Columns[i].Caption;
    cbxGridSearchField.Items.AddObject(Displayname, TbfaString.Create(Fieldname));
  end;

  for i := 0 to cbxGridSearchField.Items.Count -1 do
  begin
    if TbfaString(cbxGridSearchField.Items.Objects[i]).Value = compGrid.Config.SearchField then
    begin
      cbxGridSearchField.ItemIndex := i;
      Self.Searchfield := compGrid.Config.SearchField;
      Break;
    end;
  end;

  // wenn wir keine Voreinstellung haben, aber Felder in der Liste
  if (cbxGridSearchField.ItemIndex = -1) and (cbxGridSearchField.Items.Count > 0) then
  begin
    cbxGridSearchField.ItemIndex := 0;
    Self.Searchfield := TbfaString(cbxGridSearchField.Items.Objects[0]).Value;
  end;
end;

procedure TBfaAddfieldComp.btnIndexingClick(Sender: TObject);
var
  value : string;
begin
  fIsIndexing := not fIsIndexing;
  if fIsIndexing then
  begin
    cbxIndexing.Text := '';

    ShowIndexingCombo(fIsIndexing);

    if fOptions and cAddfieldOptStrict = cAddfieldOptStrict then
    begin
      if compComboBox <> nil then
        compComboBox.Properties.DropDownListStyle := lsFixedList;
      cbxIndexing.Properties.Items := fComboitems;
      cbxIndexing.ItemIndex := cbxIndexing.Properties.Items.IndexOf(fValue);
    end else
    begin
      value := ParseAddfields(Trim(value));
      if Assigned(OnFieldSearch) then
        OnFieldSearch(Self, value);
    end;

    if cbxIndexing.Text = '' then
    begin
      if comp is TbfaTextEdit then
      begin
        cbxIndexing.Text := TbfaTextEdit(comp).Text;
        cbxIndexing.Properties.OnChange(nil);
      end

      else if comp is TSpTBXComboBox then
      begin
        cbxIndexing.Text := TSpTBXComboBox(comp).Text;
        cbxIndexing.Properties.OnChange(nil);
      end else if comp is TbfaComboBox then
      begin
        cbxIndexing.Text := TbfaComboBox(comp).Text;
        cbxIndexing.Properties.OnChange(nil);
      end

      else if comp is TbfaCheckedCombo then
      begin
        cbxIndexing.Text := TbfaCheckedCombo(comp).Text;
        cbxIndexing.Properties.OnChange(nil);
      end

      else if comp is TcxDateEdit then
      begin
        cbxIndexing.Text := TcxDateEdit(comp).Text;
        cbxIndexing.Properties.OnChange(nil);
      end

      else if comp is TbfaSpinEdit then
      begin
        cbxIndexing.Text := TbfaSpinEdit(comp).Text;
        cbxIndexing.Properties.OnChange(nil);
      end;
    end;
  end
  else
  begin
    ShowIndexingCombo(fIsIndexing);
    Self.SetCompValue(cbxIndexing.Text);
    try
      SetColor(Self.comp, cbxIndexing.Color);
    except

    end;
  end;

end;

procedure TBfaAddfieldComp.CheckedComboClick(Sender: TObject; AButtonIndex: Integer);
var
  tp, tp3 : TPoint;
begin
// TODO SJ: Mehrfachauswahl noch prfen (OnButtonClick vs TdxCalloutPopup)
//  Self.compCalloutPopup.Popup(Self, Self.compCheckedCombo.Bounds);
//  Exit;

  if Assigned(OnComboClick) then
    OnComboClick(Self)
  else
  begin
    tp := TbfaCheckedCombo(Self.comp).ClientToScreen(Point(0,0));
    tp3 := TbfaCheckedListBox(Self.comp2).Parent.ScreenToClient(tp);
    TbfaCheckedListBox(Self.comp2).Width := TbfaCheckedCombo(Self.comp).Width;
    TbfaCheckedListBox(Self.comp2).Height := 160;
    TbfaCheckedListBox(Self.comp2).Left := tp3.X - MulDiv(2, Screen.PixelsPerInch, 96);
    TbfaCheckedListBox(Self.comp2).Top := tp3.Y + MulDiv(19, Screen.PixelsPerInch, 96);

    if Self.ListClose then
    begin
      TbfaCheckedListBox(Self.comp2).Visible := False;
      TbfaCheckedCombo(Self.comp).SelStart := Length(TbfaCheckedCombo(Self.comp).Text);
      Self.ListClose := False;
    end
    else
      TbfaCheckedListBox(Self.comp2).Visible := not TbfaCheckedListBox(Self.comp2).Visible;

    if TbfaCheckedListBox(Self.comp2).Visible then
    begin
      TbfaCheckedListBox(Self.comp2).BringToFront;
      TbfaCheckedListBox(Self.comp2).SetFocus;
      if Assigned(TbfaCheckedListBox(Self.comp2).OnDropDown) then
        TbfaCheckedListBox(Self.comp2).OnDropDown(Self.comp2);

    end else
      if Assigned(TbfaCheckedListBox(Self.comp2).OnCloseUp) then
        TbfaCheckedListBox(Self.comp2).OnCloseUp(Self.comp2);
  end;
end;

procedure TBfaAddfieldComp.CheckedComboOnExit(Sender: TObject);
begin
  if TbfaCheckedListBox(Self.comp2).Visible then
    TbfaCheckedListBox(Self.comp2).Visible := False;
end;

procedure TBfaAddfieldComp.CheckedComboOnKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (ssAlt in Shift) And (Key = VK_UP) then
  begin
    CheckedComboOnExit(Sender);
    TbfaCheckedCombo(Self.comp).Setfocus;
  end;

  if CtrlPressed and (Key = Ord('s')) then
    TbfaCheckedCombo(Sender).PostEditValue;
end;

function TBfaAddfieldComp.CheckValidChars(const Value: string): string;
var
  i: Integer;
  sTemp: string;
begin
  sTemp := '';
  for i := 1 to length(Value) do
    if (Pos(Value[i], fAllowedChar) <> 0) then
      sTemp := sTemp + Value[i];
  Result := sTemp;
end;

procedure TBfaAddfieldComp.ClearOldValue;
begin
  fOldValue := '';
  fOldValueSet := False;
end;

procedure TBfaAddfieldComp.ClearValue;
begin
  fIsSettingValue := True;
  try
    fValue := '';
    fOldValue := '';
    fOldValueSet := False;
    fModified := False;
    FModifiedHigh := False;

    if comp is TbfaTextEdit then
      TbfaTextEdit(comp).Text := fValue;

    if comp is TSpTBXComboBox then
    begin
      TSpTBXComboBox(comp).Text := fValue;
      if fValue = '' then
        TSpTBXComboBox(comp).ItemIndex := -1;
    end else if (comp is TbfaComboBox) then
    begin
      TbfaComboBox(comp).Text := fValue;
      if fValue = '' then
        TbfaComboBox(comp).ItemIndex := -1;
    end;

    if Assigned(comp2) and (comp2 is TSpTBXComboBox) then
    begin
      TSpTBXComboBox(comp2).Text := fValueHigh;
      if fValueHigh = '' then
        TSpTBXComboBox(comp2).ItemIndex := -1;
    end else if Assigned(comp2) and (comp2 is TbfaComboBox) then
    begin
      TbfaComboBox(comp2).Text := fValueHigh;
      if fValueHigh = '' then
        TbfaComboBox(comp2).ItemIndex := -1;
    end;

    if comp is TbfaSpinEdit then
      TbfaSpinEdit(comp).Text := fValue;

    if Assigned(comp2) and (comp2 is TbfaSpinEdit) then
      TbfaSpinEdit(comp2).Text := '';

    if comp is TSpTBXButtonEdit then
      TSpTBXButtonEdit(comp).Text := fValue
    else if (comp is TbfaCheckedCombo) then
      TbfaCheckedCombo(comp).Text := fValue;

    if comp is TcxDateEdit then
    begin
      TcxDateEdit(comp).Text := fValue;
      TcxDateEdit(comp2).Text := '';
    end;

    if (compGrid <> nil) and compGrid.DataSet.Active then
    begin
      try
        with compGrid.View.DataController do
        begin
          BeginUpdate;
          try
            while RecordCount > 0 do
              DeleteRecord(RecordCount-1);
          finally
            EndUpdate;
          end;
        end;
        while compGrid.DataSet.RecordCount > 0 do
          compGrid.DataSet.Delete;
        compGrid.RecID := 0;
        compGrid.NewHeight := 160;
      except

      end;
    end;

    // Feldrechte knnen auch aufaddiert als 3 (lesen und schreiben) oder 5 (lesen,schreiben ndern kommen)
    // daher wird hier nicht per and sondern direkt die 1 fr schreiben abgefragt
//    if (Rights and cAddfieldRead) = cAddfieldRead then
    if (Rights = 1) then
      SetColor(comp, cColorNoRightInit)
    else if (Options and cAddfieldOptMandatory) = cAddfieldOptMandatory then
        SetColor(comp, cColorRequiredField)
    else
      SetColor(comp, cColorWhite);

    if Assigned(comp2) then
    begin
      if (Rights = 1) then
        SetColor(comp2, cColorNoRightInit)
      else if (Options and cAddfieldOptMandatory) = cAddfieldOptMandatory then
          SetColor(comp2, cColorRequiredField)
      else
        SetColor(comp2, cColorWhite);
    end;

  finally
    fIsSettingValue := False;
  end;
end;

//04.10.2024 SD -->bfacrm:ticket__show__39992
function TBfaAddfieldComp.GridHeightCalculate(RowCount: Integer): Integer;
var j, amountGrid, hGrid, hOther: Integer;
begin;
  Result := 163;
  if Assigned(compGrid.Grid.Parent.Parent) then
  begin
    amountGrid := 0;
    hOther := 0;
    for j:= 0 to compGrid.Grid.Parent.Parent.ControlCount - 1 do
    begin
      if not compGrid.Grid.Parent.Parent.Controls[j].Visible then
        Continue;
      if TbfaAddfieldComp(compGrid.Grid.Parent.Parent.Controls[j]).AddfieldType = zfld13_Grid then
        inc(amountGrid)
      else
        hOther := hOther + compGrid.Grid.Parent.Parent.Controls[j].Height;
    end;
    hGrid := Round((compGrid.Grid.Parent.Parent.Height - hOther - 9) / amountGrid) - 44 + amountGrid;
    if hGrid < 163 then
      hGrid := 163;
    Result := hGrid;
  end;
end;
//04.10.2024 SD <--bfacrm:ticket__show__39992

procedure TbfaCheckedListBox.CMCANCELMODE(var Message: TCMCancelMode);
begin
//  inherited;
  if (Message.Sender <> Self) then
    Self.Visible := False;
  if Message.Sender is TSpTBXEditButton then      // wurde die Liste ber ein EditButton geschlossen?
    TBfaAddfieldComp(Self.Field).ListClose := True    // wenn ja, dann machen wir diese bei OnClick nicht sofort
  else                                            // wieder auf, wenn nein dann ffnet der Klick auf einen
    TBfaAddfieldComp(Self.Field).ListClose := False;  // EditButton wieder die Liste
end;

constructor TBfaAddfieldComp.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  fMin := 306;
  Self.Parent := TWinControl(AOwner); //16.09.2016
  Self.Top := 0;
  Self.Left := 0;
  Self.Height := MulDiv(45, Screen.PixelsPerInch, 96);
  Self.Visible := True;
  Self.Borders := False;
  {$IFDEF DEVEXSCROLLBOX} //22.09.2020
  if AOwner is TcxScrollbox then
    Self.Width := TcxScrollBox(AOwner).Width
  {$ELSE}
  if AOwner is TbfaScrollbox then
    Self.Width := TbfaScrollBox(AOwner).Width
  {$ENDIF}
  else
   Self.Width := fMin;

  Self.Anchors := [akLeft, akTop, akRight];
  Self.comp := nil;
  Self.comp2 := nil;

  Dispname := 'Zusatzfeld:';
  AddfieldType := zfld00_Universal;
  fIndexing := False;
  fIndexingOffset := 0;
  fAddbutton := False;
  fLeft := 10;
  fRight := 40;
  fFieldWidth := 100;

  if Self.Width - fRight  <= fMin then
    fSpare := (Self.Width - fLeft - fRight) - ((fFieldWidth) *2) +9
  else
    fSpare := 56;

  fEditmode := False;
  fState := 0;
  fOldValueSet := False;
  fDontSetColors := False;
  fComboitems := TStringList.Create;
  fComboitems.Options := fComboitems.Options - [soWriteBOM]; // bfacrm:ticket__show__33355
//  StyleController := TcxEditStyleController.Create(Self);

  CreateControl;
end;

constructor TBfaAddfieldComp.Create(AOwner: TComponent; Afieldtype: eAddFieldType;
  ADispname : string = 'Zusatzfeld'; AIndexing : Boolean = False;
  ImgList : TVirtualImageList = nil);
begin
  inherited Create(AOwner);
  fMin := 306;
  Self.Parent := TWinControl(AOwner); //16.09.2016
  Self.Top := 0;
  Self.Left := 0;
  if Afieldtype in [zfld03_MehrfachAuswahl] then
    Self.Height := MulDiv(48, Screen.PixelsPerInch, 96)
  else if Afieldtype in [zfld13_Grid] then
    Self.Height := MulDiv(200, Screen.PixelsPerInch, 96)
  else
    Self.Height := MulDiv(45, Screen.PixelsPerInch, 96);

  {$IFDEF DEVEXSCROLLBOX} //22.09.2020
  if AOwner is TcxScrollbox then
    Self.Width := TcxScrollBox(AOwner).Width
  {$ELSE}
  if AOwner is TbfaScrollbox then
    Self.Width := TbfaScrollBox(AOwner).Width
  {$ENDIF}
  else
    Self.Width := fMin;

  Self.Visible := True;
  Self.Borders := False;
  Self.Anchors := [akLeft, akTop, akRight];
  Self.comp := nil;
  Self.comp2 := nil;
  AddfieldType := Afieldtype;
  // kann nicht bei Gridfeldern gesetzt werden
  fIndexing := AIndexing and (AddfieldType <> zfld13_Grid);
  fIndexingOffset := 0;
  fLeft := 10;
  fRight := 40;
  fFieldWidth := 100;

  if Self.Width - fRight  <= fMin then
    fSpare := (Self.Width - fLeft - fRight) - ((fFieldWidth) *2) +9
  else
    fSpare := 56;

  FEditmode := False;
  fOldValueSet := False;
  Dispname  := ADispname;
  fDontSetColors := False;
  fComboitems := TStringList.Create;
  fComboitems.Options := fComboitems.Options - [soWriteBOM]; // bfacrm:ticket__show__33355
//  SearchItems := TStringList.Create;

  fBalloonHint := TBalloonHint.Create(Self);   //05.06.2019 Ticket 19429
  fBalloonHint.Delay := 0;   //05.06.2019 Ticket 19429
  fBalloonHint.HideAfter := 800;  //05.06.2019 Ticket 19429
//  StyleController := TcxEditStyleController.Create(Self);

  CreateControl(ImgList);
end;

procedure TBfaAddfieldComp.CreateControl(ImgList : TVirtualImageList = nil);
var
  iTop: integer;
  i: Integer;
  png: TPngImage;
  AImage: TdxSmartImage;
begin
  if fIndexing then
    fIndexingOffset := 20 // SJ 22.02.2021: changed from 20
  else
    fIndexingOffset := 0;

//  if stlFree = nil then
//  begin
//    stlFree := TcxStyle.Create(Self);
//    stlFree.Color := gclFree;
//    stlFree.TextColor := gclFreeText;
//  end;

  Label1 := TbfaLabel.Create(Self);
  Label1.Parent := Self;
  Label1.Caption := Dispname;
  Label1.Left := fLeft + fIndexingOffset;
  Label1.Visible := True;
  Label1.AutoSize := True;
  Label1.Top := 0;
  Label1.Height := 26; //22.09.2020
  Label1.Style.StyleController := ctlGeneral;
//  Label1.Style.TextColor := ctlGeneral.Style.TextColor;
  Label1.Style.BorderStyle := ebsNone;
  Label1.Style.TextColor := stlFree.TextColor; ///13.09.2022
  Label1.TabStop := False;

  iTop := 18;
  iTop := MulDiv(iTop, Screen.PixelsPerInch, 96);

  chkIndexing := TSpTBXCheckBox.Create(Self);
  chkIndexing.Parent := Self;
  chkIndexing.Left := fLeft;
  chkIndexing.Top := iTop - 2;
  chkIndexing.Visible := fIndexing;
  chkIndexing.Checked := False;
  chkIndexing.TabStop := True;

//***********************//
//**** Universalfeld ****//
//***********************//

  if fAddfieldType in [zfld00_Universal] then
  begin
    compEdit := TbfaTextEdit.Create(Self);
    compEdit.Parent := Self;
    compEdit.Left := fLeft + fIndexingOffset;
    compEdit.Top := iTop;
    compEdit.Width := Self.Width - fRight - fIndexingOffset;
    compEdit.Visible := True;

    if (Rights and cAddfieldRead) = cAddfieldRead then
      compEdit.Style.StyleController := ctlDisabled
    else if (Options and cAddfieldOptMandatory) = cAddfieldOptMandatory then
      compEdit.Style.StyleController := ctlRequired
    else
      compEdit.Style.StyleController := ctlGeneral;

    compEdit.Properties.OnChange := OnChange;
    compEdit.OnClick := AddfieldClick;
    compEdit.OnDblClick := AddfieldDblClick;
    compEdit.OnKeyPress := OnKeyPress;
    compEdit.OnKeyUp := OnKeyUp;

    comp := compEdit;
    NotAllowedChar := sNotAllowedChar;
  end;


//***************************//
//**** Datum + DLM-Datum ****//
//***************************//
  if fAddfieldType in [zfld01_Datum, zfld12_DLMDate] then
  begin
    compDateLow := TcxDateEdit.Create(Self);
    compDateLow.Parent := Self;
    compDateLow.Left := fLeft + fIndexingOffset;
    compDateLow.Top := iTop;
    compDateLow.Width := fFieldWidth;
    compDateLow.Visible := True;
    compDateLow.Properties.Kind := ckDate; //15.09.2017 Uhrzeit war auch dargestellt
    compDateLow.Properties.Showtime := False;  //15.09.2017 Uhrzeit war auch dargestellt

    if (Rights and cAddfieldRead) = cAddfieldRead then
      compDateLow.Style.StyleController := ctlDisabled
    else if (Options and cAddfieldOptMandatory) = cAddfieldOptMandatory then
      compDateLow.Style.StyleController := ctlRequired
    else
      compDateLow.Style.StyleController := ctlGeneral;

    // SJ 06.10.2021 - ikRegExpr deaktiviert SmartInput (Today, Yesterday, Sunday, ...)
    compDateLow.Properties.InputKind := ikRegExpr;

// SJ 27.09.2022 - bfacrm:ticket__show__32344
//    wegen dem EditFormat wird bei Eingabe von 20 + VK_RETURN nicht auf zB 20.09.2022 vervollstndigt,
//    sondern 1900 bzw 1899.
//
//    compDateLow.Properties.EditFormat := FormatSettings.ShortDateFormat;

    compDateLow.Properties.OnPopup := OnCalenderPopup;
    compDateLow.Properties.OnEditValueChanged := DateOnEditValueChanged;
    compDateLow.Properties.OnValidate := OnPropertiesValidate;
    compDateLow.Properties.OnChange := OnChange;
    compDateLow.Properties.OnCloseUp := OnDateCloseUp;
    compDateLow.Properties.BeepOnError := False;
    compDateLow.BeepOnEnter := False;
    compDateLow.OnKeyUp := OnKeyUp;
    compDateLow.OnKeyPress := OnKeyPress;
//    SJ 12.11.2020 - bei Datumsfeldern nicht ntig
//    compDateLow.OnClick := AddfieldClick;
//    compDateLow.OnDblClick := AddfieldDblClick;

    comp := compDateLow;

    Label2 := TbfaLabel.Create(Self);
    Label2.Parent := Self;
    Label2.Caption := _('bis:');
    Label2.Left := fLeft + fSpare + fFieldWidth;
    Label2.Visible := True;
    Label2.AutoSize := True;
    Label2.Top := 0;
    Label2.Style.StyleController := ctlGeneral; //13.09.2022
    Label2.Style.BorderStyle := ebsNone; //13.09.2022
    Label2.Style.TextColor := stlFree.TextColor; ///13.09.2022


    compDateHigh := TcxDateEdit.Create(Self);
    compDateHigh.Parent := Self;
    compDateHigh.Top := iTop;
    compDateHigh.Left := fLeft + fSpare + fFieldWidth;
    compDateHigh.Width := fFieldWidth;
    compDateHigh.Visible := True;
    compDateHigh.Properties.Kind := ckDate; //15.09.2017 Uhrzeit war auch dargestellt
    compDateHigh.Properties.Showtime := False;  //15.09.2017 Uhrzeit war auch dargestellt

    Label2.Style.StyleController := ctlGeneral;
    Label2.style.BorderStyle := ebsNone;

    if (Rights and cAddfieldRead) = cAddfieldRead then
      compDateHigh.Style.StyleController := ctlDisabled
    else if (Options and cAddfieldOptMandatory) = cAddfieldOptMandatory then
      compDateHigh.Style.StyleController := ctlRequired
    else
      compDateHigh.Style.StyleController := ctlGeneral;

    // SJ 06.10.2021 - ikRegExpr deaktiviert SmartInput (Today, Yesterday, Sunday, ...)
    compDateHigh.Properties.InputKind := ikRegExpr;

// SJ 27.09.2022 - bfacrm:ticket__show__32344
//    wegen dem EditFormat wird bei Eingabe von 20 + VK_RETURN nicht auf zB 20.09.2022 vervollstndigt,
//    sondern 1900 bzw 1899.
//
//    compDateHigh.Properties.EditFormat := FormatSettings.ShortDateFormat;

    compDateHigh.Properties.OnChange := OnChangeHigh;
    compDateHigh.Properties.OnPopup := OnCalenderPopup;
    compDateHigh.Properties.OnCloseUp := OnDateCloseUp;
    compDateHigh.Properties.OnEditValueChanged := DateOnEditValueChanged;
    compDateHigh.Properties.OnValidate := OnPropertiesValidate;
    compDateHigh.Properties.BeepOnError := False;
    compDateHigh.BeepOnEnter := False;
    compDateHigh.OnKeyUp := OnKeyUpHigh;
    compDateHigh.OnKeyPress := OnKeyPress;
//    SJ 12.11.2020 - bei Datumsfeldern nicht ntig
//    compDateHigh.OnClick := AddfieldClick;
//    compDateHigh.OnDblClick := AddfieldDblClick;

    fAllowedChar := '0123456789' + formatSettings.DateSeparator;

    comp2 := compDateHigh;
    Self.OnResize := OnResizeDate;
  end;

//***********************************//
//**** Auswahlfeld + DLM Auswahl ****//
//***********************************//

  if fAddfieldType in [zfld02_Auswahl, zfld11_DLM] then
  begin
    compComboBox := TbfaComboBox.Create(Self);
    compComboBox.Parent := Self;
    compComboBox.Left := fLeft + fIndexingOffset;
    compComboBox.Top := iTop;
    compComboBox.Width := Self.Width - fRight - fIndexingOffset;
    compComboBox.Visible := True;

    if (Rights and cAddfieldRead) = cAddfieldRead then
      compComboBox.Style.StyleController := ctlDisabled
    else if (Options and cAddfieldOptMandatory) = cAddfieldOptMandatory then
      compComboBox.Style.StyleController := ctlRequired
    else
      compComboBox.Style.StyleController := ctlGeneral;

    compComboBox.Properties.IncrementalSearch := True;
    compComboBox.Properties.IncrementalFiltering := True;
    compComboBox.Properties.DropDownRows := 30;
    compComboBox.Properties.OnChange := OnChange;
    compComboBox.Properties.DropDownListStyle := lsEditList;
//      compComboBox.OnClick := AddfieldClick; --> OnMouseUp fr Dreifachklick; OnClick reagiert bei Listenwert
    compComboBox.OnDblClick := AddfieldDblClick;
    compComboBox.OnMouseUp := AddfieldMouseUp;

    compComboBox.OnKeyUp := OnKeyUp;
    compComboBox.OnKeyPress := OnKeyPress;
    compComboBox.OnKeyDown := OnKeyDown;
    comp := compComboBox;
    NotAllowedChar := sNotAllowedChar;
  end;

//*************************//
//**** Mehrfachauswahl ****//
//*************************//

  if fAddfieldType in [zfld03_MehrfachAuswahl] then
  begin
    compCheckedCombo := TbfaCheckedCombo.Create(Self);
    compCheckedCombo.Parent := Self;
    compCheckedCombo.Left := fLeft + fIndexingOffset;
    compCheckedCombo.Top := iTop;
    compCheckedCombo.Width := Self.Width - fRight - fIndexingOffset;
    compCheckedCombo.Visible := True;
    compCheckedCombo.BevelInner := bvNone;
    compCheckedCombo.BevelOuter := bvNone;

    if (Rights and cAddfieldRead) = cAddfieldRead then
      compCheckedCombo.Style.StyleController := ctlDisabled
    else if (Options and cAddfieldOptMandatory) = cAddfieldOptMandatory then
      compCheckedCombo.Style.StyleController := ctlRequired
    else
      compCheckedCombo.Style.StyleController := ctlGeneral;

    compCheckedCombo.Properties.Buttons.Add;
// TODO SJ: Mehrfachauswahl noch prfen (OnButtonClick vs TdxCalloutPopup)
    compCheckedCombo.Properties.OnButtonClick := CheckedComboClick;
    compCheckedCombo.Properties.OnTokenAdd := TokenAdd;
    compCheckedCombo.Properties.OnTokenDelete := TokenDelete;
    compCheckedCombo.Properties.EditValueDelimiter := ';';
    compCheckedCombo.Properties.InputDelimiters := ';';
    compCheckedCombo.Properties.ImmediatePost := True;
    compCheckedCombo.Properties.PostEditValueOnFocusLeave := True;
    compCheckedCombo.Properties.OnChange := OnChange;
    compCheckedCombo.Properties.OnCustomDrawToken := TokenDraw;

    // SJ 16.07.2019 - OnChange einkommentiert - war nicht ganz klar warum auskommentiert
    // vermutlich Ticket 16692 - stand wohl Zusammenhang mit Strg+C, ist aber nach der Anpassung in der OnKeyUp
    // nicht mehr reproduzierbar. Daher OnChange wieder rein, damit der Copy&Paste abgefangen werden kann
    compCheckedCombo.OnKeyUp := OnKeyUp;
    compCheckedCombo.OnKeyPress := OnKeyPress;
    comp := compCheckedCombo;

    lstCheckedListbox := TbfaCheckedListBox.Create(Self);
    lstCheckedListbox.Parent := Self;
    lstCheckedListbox.Field := Self;
    lstCheckedListbox.Left := fLeft + fIndexingOffset;
    lstCheckedListbox.Top := iTop + MulDiv(compCheckedCombo.Height, Screen.PixelsPerInch, 96)-1;
    lstCheckedListbox.BevelInner := bvNone;
    lstCheckedListbox.BevelOuter := bvNone;
    lstCheckedListbox.Width := Self.Width - fRight - fIndexingOffset;
    lstCheckedListbox.OnExit := CheckedComboOnExit;
    lstCheckedListbox.OnKeyDown := CheckedComboOnKeyDown;  //26.02.2018 ALT + VK_UP sollte bei Mehrfachauswahlfeld auch funktionieren
    lstCheckedListbox.OnClickCheck := OnClickCheck;
    lstCheckedListbox.OnMouseDown := OnComboMouseDown; // SJ 20.01.2020
    lstCheckedListbox.Visible := False;
    lstCheckedListbox.OnCloseUp := CheckedComboOnExit;
    lstCheckedListbox.EditValueFormat := cvfIndices;
    comp2 := lstCheckedListbox;

//    compCalloutPopup := TdxCalloutPopup.Create(Self);
//    compCalloutPopup.PopupControl := lstCheckedListbox;
//    compCalloutPopup.HideCallout := True;
//    compCalloutPopup.AnimationOptions.FadeEffect := False;
//    compCalloutPopup.AnimationOptions.HidingAnimationTime := 0;
//    compCalloutPopup.AnimationOptions.MoveEffect := False;
//    compCalloutPopup.AnimationOptions.ShowingAnimationTime := 0;
//    compCalloutPopup.PopupControl.DoubleBuffered := True;
//    compCalloutPopup.BorderColor := clNone;

    NotAllowedChar := sNotAllowedChar;
  end;

//*******************//
//**** Intervall ****//
//*******************//

  if fAddfieldType in [zfld05_Intervall] then
  begin
    compSpinEditLow := TbfaSpinEdit.Create(Self);
    compSpinEditLow.Parent := Self;
    compSpinEditLow.Left := fLeft + fIndexingOffset;
    compSpinEditLow.Top := iTop;
    compSpinEditLow.Width := fFieldWidth;
    compSpinEditLow.Visible := True;
    compSpinEditLow.OnKeyPress := OnKeyPress;
    compSpinEditLow.OnKeyUp := OnKeyUp;
    fAllowedChar := '0123456789';

    if (Rights and cAddfieldRead) = cAddfieldRead then
      compSpinEditLow.Style.StyleController := ctlDisabled
    else if (Options and cAddfieldOptMandatory) = cAddfieldOptMandatory then
      compSpinEditLow.Style.StyleController := ctlRequired
    else
      compSpinEditLow.Style.StyleController := ctlGeneral;

    compSpinEditLow.Properties.OnChange := OnChange;
    compSpinEditLow.Properties.ValueType := vtInt;
    compSpinEditLow.Properties.MinValue := 0;
    compSpinEditLow.MaxDecimal := 0;
//    SJ 12.11.2020 - bei Intervallfeldern nicht ntig
//      compSpinEditLow.OnClick := AddfieldClick;
//      compSpinEditLow.OnDblClick := AddfieldDblClick;

    comp := compSpinEditLow;
    compComboBox := TbfaComboBox.Create(Self);
    compComboBox.Parent := Self;
    compComboBox.Top := iTop;
    compComboBox.Left := fLeft + fSpare + fFieldWidth;
    compComboBox.Width := fFieldWidth;
    compComboBox.Visible := True;

    if (Rights and cAddfieldRead) = cAddfieldRead then
      compComboBox.Style.StyleController := ctlDisabled
    else if (Options and cAddfieldOptMandatory) = cAddfieldOptMandatory then
      compComboBox.Style.StyleController := ctlRequired
    else
      compComboBox.Style.StyleController := ctlGeneral;

    compComboBox.Properties.Items.DelimitedText := 'T,W,M,J';
    compComboBox.Properties.DropDownRows := 30;
    compComboBox.Properties.OnChange := OnChangeHigh;
    compComboBox.Properties.DropDownListStyle := lsFixedList;
    compComboBox.Properties.ItemHeight := MulDiv(15, Screen.PixelsPerInch, 96);

    compComboBox.Height := MulDiv(17, Screen.PixelsPerInch, 96);
    compComboBox.ItemIndex := -1;  // SJ 23.07.2020  compComboBox.ItemIndex := 0;
    comp2 := compComboBox;
    Self.OnResize := OnResizeInterval;
  end;

//*******************************************//
//**** Ganzzahl + Fliesskomma + Waehrung ****//
//*******************************************//

  if (AddfieldType in [zfld06_GanzZahl, zfld07_FliessKomma, zfld08_Waehrung]) then
  begin
    compSpinEditLow := TbfaSpinEdit.Create(Self);
    compSpinEditLow.Parent := Self;
    compSpinEditLow.Left := fLeft + fIndexingOffset;
    compSpinEditLow.Top := iTop;
    compSpinEditLow.Width := fFieldWidth;
    compSpinEditLow.Visible := True;

    compSpinEditLow.Properties.OnChange := OnChange;
    compSpinEditLow.Properties.ImmediatePost := True;
//    SJ 12.11.2020 - bei Whrungsfeldern nicht ntig
//      compSpinEditLow.OnClick := AddfieldClick;
//      compSpinEditLow.OnDblClick := AddfieldDblClick;

    compSpinEditLow.OnKeyPress := OnKeyPress;
    compSpinEditLow.OnKeyUp := OnKeyUp;

    if (Rights and cAddfieldRead) = cAddfieldRead then
      compSpinEditLow.Style.StyleController := ctlDisabled
    else if (Options and cAddfieldOptMandatory) = cAddfieldOptMandatory then
      compSpinEditLow.Style.StyleController := ctlRequired
    else
      compSpinEditLow.Style.StyleController := ctlGeneral;

    if AddfieldType in [zfld06_GanzZahl] then
    begin
      fAllowedChar := '-0123456789';
      compSpinEditLow.Properties.ValueType := vtInt;
      compSpinEditLow.MaxDecimal := 0;
      compSpinEditLow.Properties.DisplayFormat := '0';
      compSpinEditLow.Properties.EditFormat := '0';
    end
    else if AddfieldType in [zfld07_FliessKomma] then
    begin
      fAllowedChar := '-0123456789.,';
      compSpinEditLow.Properties.ValueType := vtFloat;
      compSpinEditLow.MaxDecimal := 10;
      compSpinEditLow.Properties.DisplayFormat := '0.##########';
      compSpinEditLow.Properties.UseDisplayFormatWhenEditing := True;
//      compSpinEditLow.Properties.EditFormat := '0.##########';
    end
    else if AddfieldType in [zfld08_Waehrung] then
    begin
      fAllowedChar := '-0123456789.,';
      compSpinEditLow.Properties.ValueType := vtFloat;
      compSpinEditLow.MaxDecimal := 2;
      // wenn wir Tausendertrennzeichen nutzen, mssen wir die OnChange berarbeiten,
      // da dort modified gesetzt wird, auch wenn der Wert sich nicht gendert hat
      // wenn DisplayFormat und EditFormat sich ndern, wird OnChange jedoch ausgefhrt
//        compSpinEditLow.Properties.DisplayFormat := ',0.00';
      compSpinEditLow.Properties.DisplayFormat := '0.00';
      compSpinEditLow.Properties.EditFormat := '0.00';
    end;

    comp := compSpinEditLow;

    Label2 := TbfaLabel.Create(Self);
    Label2.Parent := Self;
    Label2.Caption := _('bis:');
    Label2.Left := fLeft + fSpare + fFieldWidth;
    Label2.Visible := True;
    Label2.AutoSize := True;
    Label2.Top := 0;

    compSpinEditHigh := TbfaSpinEdit.Create(Self);
    compSpinEditHigh.Parent := Self;
    compSpinEditHigh.Top := iTop;
    compSpinEditHigh.Left := fLeft + fSpare + fFieldWidth;
    compSpinEditHigh.Width := fFieldWidth;
    compSpinEditHigh.Visible := True;
    compSpinEditHigh.OnKeyPress := OnKeyPressHigh;
    compSpinEditHigh.OnKeyUp := OnKeyUpHigh;

    Label2.Style.StyleController := ctlGeneral;
    Label2.style.BorderStyle := ebsNone;
    compSpinEditHigh.Properties.OnChange := OnChangeHigh;
//    SJ 12.11.2020 - bei Whrungsfeldern nicht ntig
//      compSpinEditHigh.OnClick := AddfieldClick;
    compSpinEditLow.OnDblClick := AddfieldDblClick;
    compSpinEditHigh.OnDblClick := AddfieldDblClick;

    if (Rights and cAddfieldRead) = cAddfieldRead then
      compSpinEditHigh.Style.StyleController := ctlDisabled
    else if (Options and cAddfieldOptMandatory) = cAddfieldOptMandatory then
      compSpinEditHigh.Style.StyleController := ctlRequired
    else
      compSpinEditHigh.Style.StyleController := ctlGeneral;
    if AddfieldType in [zfld06_GanzZahl] then
    begin
      fAllowedChar := '-0123456789';
      compSpinEditHigh.Properties.ValueType := vtInt;
      compSpinEditHigh.MaxDecimal := 0;
      compSpinEditHigh.Properties.DisplayFormat := '0';
      compSpinEditHigh.Properties.EditFormat := '0';
    end;
    if AddfieldType in [zfld07_FliessKomma] then
    begin
      fAllowedChar := '-0123456789.,';
      compSpinEditHigh.Properties.ValueType := vtFloat;
      compSpinEditHigh.MaxDecimal := 10;
      compSpinEditHigh.Properties.DisplayFormat := '0.##########';
      compSpinEditHigh.Properties.UseDisplayFormatWhenEditing := True;
//      compSpinEditHigh.Properties.EditFormat := '0.##########';
    end;
    if AddfieldType in [zfld08_Waehrung] then
    begin
      fAllowedChar := '-0123456789.,';
      compSpinEditHigh.Properties.ValueType := vtFloat;
      compSpinEditHigh.MaxDecimal := 2;
      compSpinEditHigh.Properties.DisplayFormat := '0.00';
      compSpinEditHigh.Properties.EditFormat := '0.00';
    end;
//04.05.2017 <--

    comp2 := compSpinEditHigh;
    Self.OnResize := OnResizeNum;
  end;

//*******************************************//
//****            Grid-Feld              ****//
//*******************************************//

  if fAddfieldType in [zfld13_Grid] then
  begin
    cbxGridSearchField := TbfaComboBox.Create(Self);
    cbxGridSearchField.Parent := Self;
    cbxGridSearchField.Left := fLeft + fIndexingOffset;
    cbxGridSearchField.Top := iTop ;
    cbxGridSearchField.Width := 154;
    cbxGridSearchField.Visible := True;
    cbxGridSearchField.Anchors := [akTop, akLeft];
    cbxGridSearchField.Style.StyleController := ctlGeneral;
    cbxGridSearchField.Properties.OnChange := GridSearchFieldChange;
    cbxGridSearchField.Properties.DropDownListStyle := lsFixedList;

    edtGridSearch := TbfaTextEdit.Create(Self);
    edtGridSearch.Parent := Self;
    edtGridSearch.Left := fLeft + fIndexingOffset +151;
    edtGridSearch.Top := iTop ;
    edtGridSearch.Width := Self.Width - fRight - fIndexingOffset -4 -151;
    edtGridSearch.Visible := True;
    edtGridSearch.Anchors := [akTop, akRight];
    edtGridSearch.Enabled := True;
    edtGridSearch.Style.StyleController := ctlGeneral;
    edtGridSearch.Style.StyleController.OnStyleChanged := AddfieldStyleChanged;
    edtGridSearch.Properties.OnChange := OnChange;
    edtGridSearch.OnClick := AddfieldClick;
    edtGridSearch.OnDblClick := AddfieldDblClick;
    edtGridSearch.OnKeyPress := OnKeyPress;
    edtGridSearch.OnKeyUp := OnKeyUp;

    comp := edtGridSearch;

    compGrid := TbfaGridfield.Create(Self);
    compGrid.Grid.Parent := Self;
    compGrid.Grid.Left := fLeft + fIndexingOffset +1;
    compGrid.Grid.Top := iTop +22;
    compGrid.Grid.Width := Self.Width - fRight - fIndexingOffset -38;
    compGrid.Grid.Height := 160;

    compGrid.Grid.Visible := True;
    // <!-- SJ 21.04.2022: das bearbeiten wird in OnGridEditing geregelt
    compGrid.Grid.Enabled := True;
//    compGrid.Grid.Enabled := False;   // erst im EditMode enablen
//    compGrid.Grid.TabStop := False;
    // SJ 21.04.2022 -->
    compGrid.Grid.Anchors := [akTop, akBottom, akLeft, akRight];
    compGrid.View.OptionsView.Header := True;
    compGrid.View.OptionsView.GroupByBox := False;
    compGrid.View.OptionsView.Footer := False;
    compGrid.View.OptionsBehavior.FocusCellOnTab := True;
    compGrid.View.OptionsBehavior.FocusFirstCellOnNewRecord := True;
    compGrid.View.OptionsBehavior.GoToNextCellOnEnter := True;
    compGrid.View.OptionsCustomize.ColumnSorting := False;
    compGrid.View.OnEditing := OnGridEditing;
    compGrid.OnModified := SetModified;

    Self.Height := 204;

    imglGrid := TcxImageList.Create(Self);

    png := TPngImage.Create;
    AImage := TdxSmartImage.Create;

    AImage.Clear;
    png.LoadFromResourceName(hInstance, 'add1');
    AImage.Assign(png);
    AImage.Resize(24,24);
    imglGrid.Width := AImage.Width;
    imglGrid.Height := AImage.Height;
    imglGrid.Width := 24;
    imglGrid.Height := 24;
    imglGrid.Add(AImage);
    AImage.Clear;
    png.LoadFromResourceName(hInstance, 'remove1');
    AImage.Assign(png);
    AImage.Resize(24,24);
    imglGrid.Width := 24;
    imglGrid.Height := 24;
    imglGrid.Add(AImage);
    AImage.Clear;
    png.LoadFromResourceName(hInstance, 'stamp1');
    AImage.Assign(png);
    AImage.Resize(24,24);
    imglGrid.Width := 24;
    imglGrid.Height := 24;
    imglGrid.Add(AImage);
    AImage.Clear;
    png.LoadFromResourceName(hInstance, 'arrow1');
    AImage.Assign(png);
    AImage.Resize(24,24);
    imglGrid.Width := 24;
    imglGrid.Height := 24;
    imglGrid.Add(AImage);
    AImage.Clear;
    png.LoadFromResourceName(hInstance, 'settings1');
    AImage.Assign(png);
    AImage.Resize(24,24);
    imglGrid.Width := 24;
    imglGrid.Height := 24;
    imglGrid.Add(AImage);

    btnGridAdd := TcxButton.Create(Self);
    btnGridAdd.Parent := Self;
    btnGridAdd.Left := Self.Width - fIndexingOffset - 66;
    btnGridAdd.Top := iTop + 23;
    btnGridAdd.Width := 32;
    btnGridAdd.Height := 32;
    btnGridAdd.TabStop := False;
    btnGridAdd.OnClick := OnGridAddClick;
    btnGridAdd.Caption := '+';
    btnGridAdd.Hint := _('Zeile hinzufgen');
    btnGridAdd.ShowHint := True;
    SetButtonColors(btnGridAdd, ctlGeneral);

    btnGridDelete := TcxButton.Create(Self);
    btnGridDelete.Parent := Self;
    btnGridDelete.Left := Self.Width - fIndexingOffset - 66;
    btnGridDelete.Top := iTop + 23 + (1*32);
    btnGridDelete.Width := 32;
    btnGridDelete.Height := 32;
    btnGridDelete.TabStop := False;
    btnGridDelete.OnClick := OnGridDeleteClick;
    btnGridDelete.Caption := '-';
    btnGridDelete.Hint := _('Zeile entfernen');
    btnGridDelete.ShowHint := True;
    SetButtonColors(btnGridDelete, ctlGeneral);

    btnGridStamp := TcxButton.Create(Self);
    btnGridStamp.Parent := Self;
    btnGridStamp.Left := Self.Width - fIndexingOffset - 66;
    btnGridStamp.Top := iTop + 23 + (2*32);
    btnGridStamp.Width := 32;
    btnGridStamp.Height := 32;
    btnGridStamp.TabStop := False;
    btnGridStamp.Caption := 'S';
    btnGridStamp.OnClick := BtnGridStampClick;
    btnGridStamp.Hint := _('Tabelle als Stempel aufbringen');
    btnGridStamp.ShowHint := True;
    SetButtonColors(btnGridStamp, ctlGeneral);

    btnGridExport := TcxButton.Create(Self);
    btnGridExport.Parent := Self;
    btnGridExport.Left := Self.Width - fIndexingOffset - 66;
    btnGridExport.Top := iTop + 23 + (3*32);
    btnGridExport.Width := 32;
    btnGridExport.Height := 32;
    btnGridExport.TabStop := False;
    btnGridExport.Caption := '>';
    btnGridExport.OnClick := BtnGridExportClick;
    btnGridExport.Hint := _('Tabelle als CSV-Datei exportieren');
    btnGridExport.ShowHint := True;
    SetButtonColors(btnGridExport, ctlGeneral);
    //btnGridSearch := TSpTBXButton.Create(Self);
    //btnGridSearch.Parent := Self;
    //btnGridSearch.Left := Self.Width - fIndexingOffset - 25;
    //btnGridSearch.Top := iTop + 80 +22;
    //btnGridSearch.Width := 18;
    //btnGridSearch.Height := 18;
    //btnGridSearch.TabStop := False;
    btnGridConfig := TcxButton.Create(Self);
    btnGridConfig.Parent := Self;
    btnGridConfig.Left := Self.Width - fIndexingOffset - 66;
    btnGridConfig.Top := iTop + 23 + (4*32);
    btnGridConfig.Width := 32;
    btnGridConfig.Height := 32;
    btnGridConfig.TabStop := False;
    btnGridConfig.Caption := '*';
    btnGridConfig.Hint := _('Optionen anpassen (temporr)');
    btnGridConfig.ShowHint := True;
    SetButtonColors(btnGridConfig, ctlGeneral);

    btnGridAdd.OptionsImage.Images := imglGrid;
    btnGridAdd.OptionsImage.ImageIndex := 0;
    btnGridAdd.Caption := '';
    btnGridDelete.OptionsImage.Images := imglGrid;
    btnGridDelete.OptionsImage.ImageIndex := 1;
    btnGridDelete.Caption := '';
    btnGridStamp.OptionsImage.Images := imglGrid;
    btnGridStamp.OptionsImage.ImageIndex := 2;
    btnGridStamp.Caption := '';
    btnGridExport.OptionsImage.Images := imglGrid;
    btnGridExport.OptionsImage.ImageIndex := 3;
    btnGridExport.Caption := '';
    btnGridConfig.OptionsImage.Images := imglGrid;
    btnGridConfig.OptionsImage.ImageIndex := 4;
    btnGridConfig.Caption := '';

    compGridPopupConfig := TSpTBXPopupMenu.Create(btnGridConfig);
    compGridPopupConfig.Items.Add(TTBCustomItem.Create(nil));
//13.09.2024    compGridPopupConfig.Items[0].Caption := _('Titel');
    compGridPopupConfig.Items[0].OnClick := OnConfigItemClick;
    compGridPopupConfig.Items.Add(TTBCustomItem.Create(nil));
//13.09.2024    compGridPopupConfig.Items[1].Caption := _('Summen');
    compGridPopupConfig.Items[1].OnClick := OnConfigItemClick;
    compGridPopupConfig.Items.Add(TTBCustomItem.Create(nil));
//13.09.2024    compGridPopupConfig.Items[2].Caption := _('Gruppieren');
    compGridPopupConfig.Items[2].OnClick := OnConfigItemClick;
    compGridPopupConfig.Items.Add(TTBCustomItem.Create(nil));
//13.09.2024    compGridPopupConfig.Items[3].Caption := _('Indikator');
    compGridPopupConfig.Items[3].OnClick := OnConfigItemClick;
    compGridPopupConfig.Items.Add(TTBCustomItem.Create(nil));
//13.09.2024    compGridPopupConfig.Items[4].Caption := _('Standard-Popup');
    compGridPopupConfig.Items[4].OnClick := OnConfigItemClick;
    compGridPopupConfig.Items.Add(TTBCustomItem.Create(nil));
//13.09.2024    compGridPopupConfig.Items[5].Caption := _('Sortierbar');
    compGridPopupConfig.Items[5].OnClick := OnConfigItemClick;
    compGridPopupConfig.OnPopup := OnConfigPopup;
    compGridPopupConfig.Items.Add(TTBCustomItem.Create(nil));
//13.09.2024    compGridPopupConfig.Items[6].Caption := _('Multiselect');
    compGridPopupConfig.Items[6].OnClick := OnConfigItemClick;
    compGridPopupConfig.OnPopup := OnConfigPopup;
    compGridPopupConfig.Items.Add(TTBCustomItem.Create(nil));
//13.09.2024    compGridPopupConfig.Items[7].Caption := _('Export: verwende Spaltennamen');
    compGridPopupConfig.Items[7].OnClick := OnConfigItemClick;
    compGridPopupConfig.OnPopup := OnConfigPopup;
    compGridPopupConfig.Items.Add(TTBCustomItem.Create(nil));
//13.09.2024    compGridPopupConfig.Items[8].Caption := _('Export: weitere Zusatzfelder');
    compGridPopupConfig.Items[8].OnClick := OnConfigItemClick;
    compGridPopupConfig.OnPopup :=  OnConfigPopup;
    compGridPopupConfig.Items.Add(TTBCustomItem.Create(nil));
//13.09.2024    compGridPopupConfig.Items[9].Caption := _('Export: verwende Unicode');
    compGridPopupConfig.Items[9].OnClick := OnConfigItemClick;
    compGridPopupConfig.OnPopup :=  OnConfigPopup;

    btnGridConfig.PopupMenu := compGridPopupConfig;
    btnGridConfig.OnClick := OnConfigClick;

    btnGridAdd.Anchors := [akTop, akRight];
    btnGridDelete.Anchors := [akTop, akRight];
    btnGridStamp.Anchors := [akTop, akRight];
    btnGridExport.Anchors := [akTop, akRight];
    btnGridConfig.Anchors := [akTop, akRight];

    compGrid.View.OnEditKeyDown := OnGridEditKeyDown;
    compGrid.View.OnKeyDown := OnGridKeyDown;

    SetcompGridPopupConfigCaptions; //13.09.2024
  end;

  cbxIndexing := TbfaComboBox.Create(Self);
  cbxIndexing.Parent := Self;
  cbxIndexing.Left := fLeft + 20;
  cbxIndexing.Top := iTop;

  cbxIndexing.Width := Self.Width - fRight - 20;
  cbxIndexing.Properties.DropDownListStyle := lsEditList;

  // gibt es keine DropDownCount mehr?
  // cbxIndexing.DropDownCount := 30; //15.08.2017 Ticket 11486
  cbxIndexing.Visible := False;
  cbxIndexing.Properties.OnChange := OnIndexingChange;
  cbxIndexing.OnKeyPress := OnKeyPress;

  tlbNeg := TSpTBXToolbar.Create(Self);
  tlbNeg.Parent := Self;
  tlbNeg.Left := Self.Width - 25;
//bt 26.03.2021  tlbNeg.Top := iTop + 4 + MulDiv(1, Screen.PixelsPerInch, 96);
  tlbNeg.Top := iTop + 2 + MulDiv(1, Screen.PixelsPerInch, 96);
  tlbNeg.Width := 20;
  tlbNeg.Visible := False;
  //ctrlNeg := TTBControlItem.Create(tlbNeg);
  cliNeg := TSpTBXColorItem.Create(tlbNeg);
  cliNeg.OnClick := NegSelect;
//12.01.2023  cliNeg.Hint := _('bei Suche negieren');
  cliNeg.Hint := sCaptionNegatedSearch;
  tlbNeg.Items.Add(cliNeg);

  btnIndexing := TSpTBXButton.Create(Self);
  btnIndexing.Parent := Self;
//  SJ 22.02.2021
//  btnIndexing.Left := Self.Width - 32;
//  btnIndexing.Top := iTop + MulDiv(1, Screen.PixelsPerInch, 96);
//  SJ 22.02.2021
//bt 26.03.2021 -->
//  btnIndexing.Left := Self.Width - 25;
//  btnIndexing.Top := iTop + 4 + MulDiv(1, Screen.PixelsPerInch, 96);
//  btnIndexing.Width := 18;
//  btnIndexing.Height := 18;
//  btnIndexing.Left := Self.Width - 32;
//  btnIndexing.Caption := '>';
  btnIndexing.Left := Self.Width - 31;
  btnIndexing.Top := iTop - 1 + MulDiv(1, Screen.PixelsPerInch, 96);
  btnIndexing.Width := 25;
  btnIndexing.Height := 25;
  if ImgList <> nil then
  begin
    btnIndexing.Caption := '';
    btnIndexing.Images := ImgList;
    btnIndexing.ImageIndex := Integer(icoidx_tag_assistant_unselected);
  end
  else
    btnIndexing.Caption := '>';
//bt 26.03.2021 <--

  btnIndexing.Visible := True;
  btnIndexing.OnClick := btnIndexingClick;

  if comp is TbfaGridfield then
  begin
    chkIndexing.Visible := False;
    tlbNeg.Visible := False;
    cliNeg.Visible := False;
    btnIndexing.Visible := False;
    btnGridStamp.Visible := True;
    btnGridExport.Visible := True;
    btnGridSearch.Visible := True;
    btnGridConfig.Visible := True;
  end
  else
  begin
    if fIndexing then
    begin
      chkIndexing.Visible := True;
      tlbNeg.Visible := False;
      cliNeg.Visible := False;
      btnIndexing.Visible := True;
    end
    else begin
      chkIndexing.Visible := False;
      tlbNeg.Visible := True;
      cliNeg.Visible := True;
      btnIndexing.Visible := False;
    end;
  end;

  btnAdd := TSpTBXButton.Create(Self);
  btnAdd.Parent := Self;
  btnAdd.Left := Self.Width - 66;
  btnAdd.Top := -1;
  btnAdd.Width := 22;
  btnAdd.Height := 22;
  btnAdd.Visible := False;
  btnAdd.Caption := '+';

  chkIndexing.Anchors := [akLeft, akTop];
  Label1.Anchors := [akLeft, akTop];
  if Assigned(Label2) then
    Label2.Anchors := [akLeft, akTop];

  if (comp is TbfaTextEdit) then
    TbfaTextEdit(comp).Anchors := [akLeft, akTop, akRight];

  if (comp is TSpTBXComboBox) then
    TSpTBXComboBox(comp).Anchors := [akLeft, akTop, akRight]
  else if (comp is TbfaComboBox) then
    TbfaComboBox(comp).Anchors := [akLeft, akTop, akRight];

  if (comp is TSpTBXButtonEdit) then
    TSpTBXButtonEdit(comp).Anchors := [akLeft, akTop, akRight]
  else if (comp is TbfaCheckedCombo) then
    TbfaCheckedCombo(comp).Anchors := [akLeft, akTop, akRight]
  else if (comp is TbfaGridfield) then
    TbfaGridfield(comp).Grid.Anchors := [akLeft, akTop, akRight];

  if (comp is TcxDateEdit) then
    TcxDateEdit(comp).Anchors := [akLeft, akTop];
  if (comp2 is TcxDateEdit) then
    TcxDateEdit(comp2).Anchors := [akLeft, akTop];
  if (comp is TbfaSpinEdit) then
    TbfaSpinEdit(comp).Anchors := [akLeft, akTop];
  if (comp2 is TbfaSpinEdit) then
    TbfaSpinEdit(comp2).Anchors := [akLeft, akTop];
  if (comp2 is TSpTBXListBox) then
    TSpTBXListBox(comp2).Anchors := [akLeft, akTop, akRight];

  if (comp2 is TSpTBXComboBox) then
    TSpTBXComboBox(comp2).Anchors := [akLeft, akTop]
  else if (comp2 is TbfaComboBox) then
    TbfaComboBox(comp2).Anchors := [akLeft, akTop];

  cbxIndexing.Anchors := [akLeft, akTop, akRight];
  tlbNeg.Anchors := [akTop, akRight];
  btnIndexing.Anchors := [akTop, akRight];
  btnAdd.Anchors := [akTop, akRight];

  TranslateComponent(Self, 'Shared36'); //13.09.2024
end;

//13.09.2024 -->
procedure TBfaAddfieldComp.SetcompGridPopupConfigCaptions; //13.09.2024
begin
  compGridPopupConfig.Items[0].Caption := _('Titel');
  compGridPopupConfig.Items[1].Caption := _('Summen');
  compGridPopupConfig.Items[2].Caption := _('Gruppieren');
  compGridPopupConfig.Items[3].Caption := _('Indikator');
  compGridPopupConfig.Items[4].Caption := _('Standard-Popup');
  compGridPopupConfig.Items[5].Caption := _('Sortierbar');
  compGridPopupConfig.Items[6].Caption := _('Multiselect');
  compGridPopupConfig.Items[7].Caption := _('Export: verwende Spaltennamen');
  compGridPopupConfig.Items[8].Caption := _('Export: weitere Zusatzfelder');
  compGridPopupConfig.Items[9].Caption := _('Export: verwende Unicode');
end;
//13.09.2024 <--

procedure TBfaAddfieldComp.DateOnEditValueChanged(Sender: TObject);
var
  date, datehigh: TDateTime;
begin
   if fdteSearchDisable then
     Exit;
   if fIsSettingValue then
     Exit;
   try
     fdteSearchDisable := True;
     date := 0;
     datehigh := 0;
     if TcxDateEdit(Sender) = comp then
     begin
//23.03.2017 Ticket 10188        if (TcxDateEdit(comp).Date > TcxDateEdit(comp2).Date) and
//            (TcxDateEdit(Sender).Text <> '') and FModifiedHigh then
//        if TcxDateEdit(Sender).Text = '30.12.1899' then   // beim verlassen des feldes, wenn man mnauell gelscht hat, kommt der 30.12.1899
//        begin
//          TcxDateEdit(Sender).Text := '';
//        end;

        // Ticket 19145
        if IsEmptyDate(TcxDateEdit(Sender).Date) then
          TcxDateEdit(Sender).Date := -700000;

        if (TcxDateEdit(comp).Date > TcxDateEdit(comp2).Date) and
            (TcxDateEdit(Sender).Text <> '') and (TcxDateEdit(comp2).Text <> '') and
            FModifiedHigh then
           TcxDateEdit(comp2).Date := TcxDateEdit(Sender).Date;
        fValue := TcxDateEdit(comp).Text;
        date := TcxDateEdit(comp).Date;
        if ((fValue = '') {or (fValue = fOldValue)}) and not fEditMode then
        begin
          fModified := True;
          if not fDontSetColors then
          begin
            if (Rights = 1) or ((Rights and cAddfieldWrite = cAddfieldWrite) and (fValue <> '')) then
            begin
              SetColor(comp, cColorNoRightInit);
              fModified := False;
            end
            else if (Options and cAddfieldOptMandatory) = cAddfieldOptMandatory then
            begin
              SetColor(comp, cColorRequiredField);
              fModified := False;
            end
            else
              SetColor(comp, cColorLightGreen);
          end;
        end else
        begin
          if fEditMode then
          begin
            if (Rights  = 1) then
              fModified := False
            else if ((Rights and cAddfieldWrite = cAddfieldWrite) and (fOldValue = '')) then
              fModified := True
            else if (Rights and cAddfieldModify) = cAddfieldModify then   // wenn es kein ReadOnly Feld ist
            begin
              fModified := True;
            end;
            if not fDontSetColors and fModified then
              SetColor(comp, cColorLightGreen);
          end else
          begin
            fModified := True;
            if not fDontSetColors then
              SetColor(comp, cColorLightGreen);
          end;
        end;
     end;
     if TcxDateEdit(Sender) = comp2 then
     begin
//        if TcxDateEdit(Sender).Text = '30.12.1899' then   // beim verlassen des feldes, wenn man mnauell gelscht hat, kommt der 30.12.1899
//        begin
//          TcxDateEdit(Sender).Text := '';
//        end;

        // Ticket 19145
        if IsEmptyDate(TcxDateEdit(Sender).Date) then
          TcxDateEdit(Sender).Date := -700000;

        if (TcxDateEdit(Sender).Date < TcxDateEdit(comp).Date) and
            (TcxDateEdit(Sender).Text <> '') and FModified then
           TcxDateEdit(comp).Date := TcxDateEdit(Sender).Date;
        fValueHigh := TcxDateEdit(comp2).Text;
        datehigh := TcxDateEdit(comp2).Date;
        if (fValueHigh = '') then
        begin
          fModifiedHigh := False;
          if Assigned(comp2) then
          begin
            if not fDontSetColors then
            begin
              if (Rights = 1) then
                SetColor(comp2, cColorNoRightInit)
              else if (Options and cAddfieldOptMandatory) = cAddfieldOptMandatory then
                SetColor(comp2, cColorRequiredField)
              else
                SetColor(comp2, cColorWhite);
            end;
          end;
        end else
        begin
          if not fDontSetColors then
            SetColor(comp2, cColorLightGreen);
          fModifiedHigh := True;
        end;
     end;

      if Assigned(OnModified) then
        OnModified(Self);
      if Assigned(OnValueSet) then
        OnValueSet(fValue, fValueHigh, fModified, fModifiedHigh);
   finally
      fdteSearchDisable := False;
   end;
end;

function TBfaAddfieldComp.GetAddButton: Boolean;
begin
  Result := fAddbutton;
end;

function TBfaAddfieldComp.GetIndexing: Boolean;
begin
  Result := fIndexing;
end;

function TBfaAddfieldComp.GetIntervalByIndex(combo: TbfaComboBox): string;
begin
  case combo.ItemIndex of
    -1:
      Result := '';
    0:
      Result := 'T';
    1:
      Result := 'W';
    2:
      Result := 'M';
    3:
      Result := 'J';
    else
      Result := 'T';
  end;
end;

// TODO SJ - GetItems wohl nicht verwendet?
// auerdem knnte man ber die DelimitedText gehen ?
// TODO -o f.schneider wurde die Schleife statt DelimitedText verwendet, wegen ; in Werten?
// die wollen wir aber auch nicht mehr untersttzen
function TBfaAddfieldComp.GetItems: string;
var
  i : Integer;
begin
  Result := '';
  if AddfieldType in [zfld02_Auswahl, zfld11_DLM] then
  begin
    for i := 0 to TbfaComboBox(Self.comp).Properties.Items.count -1 do
    begin
      if (Result = '') then
        Result := TbfaComboBox(Self.comp).Properties.Items[i]
      else
        Result := Result + ';' + TbfaComboBox(Self.comp).Properties.Items[i];
    end;
  end
  else if AddfieldType in [zfld03_MehrfachAuswahl] then
  begin
    Result := TbfaCheckedListBox(Self.comp2).ItemsString.DelimitedText;
  end;
end;

function TBfaAddfieldComp.GetStrings: TStrings;
begin
  if AddfieldType in [zfld02_Auswahl, zfld11_DLM] then
    Result := TbfaComboBox(Self.comp).Properties.Items
  else if AddfieldType in [zfld03_MehrfachAuswahl] then
    Result := TbfaCheckedListBox(Self.comp2).ItemsString;
end;

function TBfaAddfieldComp.GetOldValue: Variant;
begin
  Result := Self.fOldValue;
end;

function TBfaAddfieldComp.GetValueHigh: Variant;
begin
  Result := Self.fValueHigh;
end;


procedure TBfaAddfieldComp.GridSearchFieldChange(Sender: TObject);
begin
  if cbxGridSearchField.ItemIndex > -1 then
    Self.SearchField :=  TbfaString(cbxGridSearchField.Items.Objects[cbxGridSearchField.ItemIndex]).Value;
end;

function TBfaAddfieldComp.GetCompValue: Variant;
begin
  Result := Self.fValue;
end;


procedure TBfaAddfieldComp.NegSelect(Sender: TObject);
begin
 if TSpTBXColorItem(Sender).Color = (*31.08.2020 Farben Anpassung clWhite*)cColorWhite then
   TSpTBXColorItem(Sender).Color := gclRed
 else
   TSpTBXColorItem(Sender).Color := (*31.08.2020 Farben Anpassung clWhite*)cColorWhite;

  Negate := TSpTBXColorItem(Sender).Color = gclRed;
end;

procedure TBfaAddfieldComp.OnChange(Sender: TObject; var CanEdit: Boolean);
begin
  OnChange(Sender);
end;

procedure TBfaAddfieldComp.OnChange(Sender: TObject);
var
  CursorPos: Integer;
  float: Double;
  lst: TStrings;
begin
  if fIsSettingValue then
    Exit;

  if (Sender is TbfaSpinEdit) then
    TbfaSpinEdit(Sender).SpinEditOnChange(Sender);

  // Universal = fMsgNotAllowedChar
  if AddfieldType in [zfld00_Universal] then
  begin
    // 15.07.2019 SJ: CopyPaste -> prfen auf ungltige Zeichen, entfernen, Meldung, Cursor korrekt setzen
    if (ParseNotAllowedChars(TbfaTextEdit(comp).Text) <> TbfaTextEdit(comp).Text) and
       (ParseNotAllowedChars(TbfaTextEdit(comp).Text) <> '') then
    begin
      fValue := ParseNotAllowedChars(TbfaTextEdit(comp).Text);
      CursorPos := TbfaTextEdit(comp).SelStart - (Length(TbfaTextEdit(comp).Text) - Length(fValue));
      TbfaTextEdit(comp).Text := fValue;
      ShowBalloonHint(Sender, fMsgNotAllowedChar);
      TbfaTextEdit(comp).SelStart := CursorPos;
    end
    // SJ: Paste von ungltigen Zeichen wrde zum leeren des Zusatzfeldes fhren
    // verhlt sich beim KeyPress so, dass der Key gelscht wird, deswegen hier auch
    else if (ParseNotAllowedChars(TbfaTextEdit(comp).Text) <> TbfaTextEdit(comp).Text) and
            (ParseNotAllowedChars(TbfaTextEdit(comp).Text) = '') then
    begin
      SendMessage(TWinControl(comp).Handle, WM_UNDO, 0, 0);
      ShowBalloonHint(Sender, fMsgNotAllowedChar);
    end
    else
      fValue := TbfaTextEdit(comp).Text
  end

  // Datum und DLMDate = fAllowedChar
  else if fAddfieldType in [zfld01_Datum, zfld12_DLMDate] then
  begin
    if (CheckValidChars(TcxDateEdit(comp).Text) <> TcxDateEdit(comp).Text) and
       (CheckValidChars(TcxDateEdit(comp).Text) <> '') then
    begin
      fValue := CheckValidChars(TcxDateEdit(comp).Text);
      CursorPos := TcxDateEdit(comp).SelStart - (Length(TcxDateEdit(comp).Text) - Length(fValue));
      TcxDateEdit(comp).Text := fValue;
      ShowBalloonHint(Sender, _(sMsgAllowedCharachters) + fAllowedChar);
      TcxDateEdit(comp).SelStart := CursorPos;
    end
    else if (ParseNotAllowedChars(TcxDateEdit(comp).Text) <> TcxDateEdit(comp).Text) and
            (ParseNotAllowedChars(TcxDateEdit(comp).Text) = '') then
    begin
      SendMessage(TWinControl(comp).Handle, WM_UNDO, 0, 0);
      ShowBalloonHint(Sender, fMsgNotAllowedChar);
    end
    else
    begin
      //02.09.2024 SD -->bfacrm:ticket__show__39649
      //Verbot der Anbringung des Datums 30.12.1899
      if TcxDateEdit(comp).Text = '' then
      begin
        fIsSettingValue := True;
        TcxDateEdit(comp).Date := -700000;
        fIsSettingValue := False;
      end;
      //02.09.2024 SD <--bfacrm:ticket__show__39649
      fValue := TcxDateEdit(comp).Text;
    end;

  end

  // Auswahl + DLM = fMsgNotAllowedChar
  else if AddfieldType in [zfld02_Auswahl, zfld11_DLM] then
  begin
    if TbfaComboBox(Self.comp).Text <> dgettext('Shared36', sCaptionOnlySearch) then
    begin
      if (ParseNotAllowedChars(TbfaComboBox(comp).Text) <> TbfaComboBox(comp).Text) and
         (ParseNotAllowedChars(TbfaComboBox(comp).Text) <> '') then
      begin
        fValue := ParseNotAllowedChars(TbfaComboBox(comp).Text);
        CursorPos := TbfaComboBox(comp).SelStart - (Length(TbfaComboBox(comp).Text) - Length(fValue));
        TbfaComboBox(comp).Text := fValue;
        ShowBalloonHint(Sender, fMsgNotAllowedChar);
        TbfaComboBox(comp).SelStart := CursorPos;
      end
      else if (ParseNotAllowedChars(TbfaComboBox(comp).Text) <> TbfaComboBox(comp).Text) and
              (ParseNotAllowedChars(TbfaComboBox(comp).Text) = '') then
      begin
        TbfaComboBox(comp).Text := fValue;
        ShowBalloonHint(Sender, fMsgNotAllowedChar);
        TbfaComboBox(comp).SelStart := Length(fValue);
      end
      else
        fValue := TbfaComboBox(Self.comp).Text;
    end else
      TbfaComboBox(Self.comp).ItemIndex := TbfaComboBox(Self.comp).ItemIndex -1;
  end

  // Mehrfachauswahl = fMsgNotAllowedChar
  else if AddfieldType in [zfld03_MehrfachAuswahl] then
  begin
    try
      if (ParseNotAllowedChars(TbfaCheckedCombo(comp).Text) <> TbfaCheckedCombo(comp).Text) and
         (ParseNotAllowedChars(TbfaCheckedCombo(comp).Text) <> '') then
      begin
        fValue := ParseNotAllowedChars(TbfaCheckedCombo(comp).Text);
        CursorPos := TbfaCheckedCombo(comp).SelStart - (Length(TbfaCheckedCombo(comp).Text) - Length(fValue));
        TbfaCheckedCombo(comp).Text := fValue;
        ShowBalloonHint(Sender, fMsgNotAllowedChar);
        TbfaCheckedCombo(comp).SelStart := CursorPos;
      end
      else if (ParseNotAllowedChars(TbfaCheckedCombo(comp).Text) <> TbfaCheckedCombo(comp).Text) and
              (ParseNotAllowedChars(TbfaCheckedCombo(comp).Text) = '') then
      begin
        SendMessage(TWinControl(comp).Handle, WM_UNDO, 0, 0);
        ShowBalloonHint(Sender, fMsgNotAllowedChar);
      end
      else
      begin
      // TODO SJ
        lst := TStringList.Create;
        lst.Delimiter := TbfaCheckedCombo(comp).Properties.EditValueDelimiter;
        lst.StrictDelimiter := True;
        lst.DelimitedText := TbfaCheckedCombo(comp).EditValue;


        if (Trim(TbfaCheckedCombo(comp).EditingValue) = '') and (Trim(TbfaCheckedCombo(comp).EditValue) <> '') then
          fValue := TbfaCheckedCombo(comp).EditValue
        else if (Trim(TbfaCheckedCombo(comp).EditValue) = '') and (Trim(TbfaCheckedCombo(comp).EditingValue) <> '') then
          fValue := TbfaCheckedCombo(comp).EditingValue
        else if (Trim(TbfaCheckedCombo(comp).EditingValue) <> '') and
                (TbfaCheckedCombo(comp).EditingValue <> TbfaCheckedCombo(comp).EditValue) and
                (lst.IndexOf(TbfaCheckedCombo(comp).EditingValue) = -1)
        then
          fValue := TbfaCheckedCombo(comp).EditValue + ';' + TbfaCheckedCombo(comp).EditingValue
        else
          fValue := TbfaCheckedCombo(comp).EditValue;

//        if (TbfaCheckedCombo(comp).EditingValue = '') or
//           (TbfaCheckedCombo(comp).EditingValue = TbfaCheckedCombo(comp).Text)
//        then
//          fValue := TbfaCheckedCombo(comp).Text
//        else
//          fValue := TbfaCheckedCombo(comp).Text + ';' + TbfaCheckedCombo(comp).EditingValue;
//        TbfaCheckedCombo(comp).PostEditValue;
      end;
    finally
      if (TbfaCheckedCombo(comp).EditingValue <> TbfaCheckedCombo(comp).EditValue) then
        TbfaCheckedCombo(comp).SkipSearchByReturn := True;
    end;
  end

  else if AddfieldType in [zfld05_Intervall] then
  begin
    if (compComboBox <> nil) then
    begin
  //16.11.2018 Ticket 16623 -->
      if (TbfaSpinEdit(comp).Text <> '') then
      begin
        if (compComboBox.ItemIndex = -1) then
          compComboBox.ItemIndex := 0;
        fModified := True;
        fModifiedHigh := True;
      end;
      if fModifiedHigh then
      begin
        SetColor(comp, cColorLightGreen);
        SetColor(comp2, cColorLightGreen);
      end;
  //16.11.2018 Ticket 16623 <--
      fValue := TbfaSpinEdit(comp).Text + GetIntervalByIndex(compComboBox);
    end;
  end

  else if AddfieldType in [zfld13_Grid] then
  begin
    if (TbfaSpinEdit(comp).EditingText = '') then
      fValue := ''
    else
      fValue := TbfaSpinEdit(comp).EditingValue;
  end
  // zfld06_GanzZahl, zfld07_FliessKomma, zfld08_Waehrung
  else
  begin
    if ((TbfaSpinEdit(comp).EditingText = '') and (TbfaSpinEdit(comp).EditingValue = 0)) then
      fValue := ''
    else
      fValue := TbfaSpinEdit(comp).EditingValue;
  end;

  //06.06.2024 SD -->bfacrm:ticket__show__38867
//  if (AddfieldType in [zfld00_Universal, zfld02_Auswahl, zfld03_MehrfachAuswahl, zfld11_DLM]) and
//     (fValue = '') and not fEditMode then
//  begin
//    fModified := False;
//
//    if not fDontSetColors then
//    begin
//      if (Rights = 1) or ((Rights and cAddfieldWrite = cAddfieldWrite) and (fValue <> '')) then
//        SetColor(comp, cColorNoRightInit)
//      else if (Options and cAddfieldOptMandatory) = cAddfieldOptMandatory then
//        SetColor(comp, cColorRequiredField)
//      else
//        SetColor(comp, cColorWhite);
//    end;
//  end
//  else if not fEditMode then
//  begin
//    fModified := True;
//
//    if not fDontSetColors then
//    begin
//      SetColor(comp, cColorLightGreen);
////      if (Rights= 1) or ((Rights and cAddfieldWrite = cAddfieldWrite) and (fValue <> '')) then
////        SetColor(comp, cColorNoRightInit)
//////      SJ 23.07.2019 - bei Suche sollen auch Nullen Pflichtfelder einfrben
//////                      Suche = fOldValue = ''
//////      else if (Options and cAddfieldOptMandatory) = cAddfieldOptMandatory then
////      else if ((Options and cAddfieldOptMandatory) = cAddfieldOptMandatory) and (fOldValue <> '') then
////        SetColor(comp, cColorRequiredField)
////      else
////        SetColor(comp, cColorLightGreen);
//    end;
//  end else
//  begin
  //06.06.2024 SD <--bfacrm:ticket__show__38867
    if fEditMode then
    begin
      if (Rights = 1) then
        fModified := False
      else if ((Options and cAddfieldOptStrict) = cAddfieldOptStrict) and (fValue <> '') and
              (AddfieldType in [zfld02_Auswahl, zfld11_DLM]) then
      begin
        fModified := fComboitems.IndexOf(fValue) > -1;          // strictmode und wert gehrt nicht zur auswahl? feld wird rot
        if not fDontSetColors then
        begin
          if not ListContains(fComboitems, fValue) then
            TbfaTextEdit(comp).Style.StyleController := ctlRedBackground
//            SetColor(comp, cColorRed)
          else
            SetColor(comp, cColorWhite);
        end;
      end{ else if (AddfieldType = zfld03_MehrfachAuswahl) and ((Options and cAddfieldOptStrict) = cAddfieldOptStrict) then
        fModified := (fValue <> fOldValue) or IsDeleteOrBack }
      else if ((Rights and cAddfieldWrite = cAddfieldWrite) and (fOldValue = '')) then
        fModified := True
      else if (Rights and cAddfieldModify) = cAddfieldModify then   // wenn es kein ReadOnly Feld ist
      begin
        fModified := True;
      end;
      if not fDontSetColors and fModified then
        SetColor(comp, cColorLightGreen);
    end else
    begin
      if ((Options and cAddfieldOptStrict) = cAddfieldOptStrict) and
         (AddfieldType in [zfld02_Auswahl, zfld11_DLM]) and
         (fComboitems.IndexOf(fValue) = -1) then
      begin
        fModified := True;      // wenn wir nicht im Editmode sind, dann drfen wir Suchen
        if not fDontSetColors and fModified then
          SetColor(comp, cColorLightGreen);
      end else if AddfieldType in [zfld05_Intervall, zfld06_GanzZahl, zfld07_FliessKomma, zfld08_Waehrung] then
      begin
        fModified := (fValue <> fOldValue) or IsDeleteOrBack;
        if not fDontSetColors and fModified then
          SetColor(comp, cColorLightGreen);
      end
      //02.09.2024 SD -->bfacrm:ticket__show__39649
      //korrektur des Verhaltens von Suchdatumsfeldern
      else if fAddfieldType in [zfld01_Datum, zfld12_DLMDate] then
        if TcxDateEdit(comp).Text = '' then
        begin
          fModified := False;
          if not fDontSetColors then
            SetColor(comp, cColorWhite);
        end
        else
        begin
          fModified := True;
          if not fDontSetColors then
            SetColor(comp, cColorLightGreen);
        end
      //02.09.2024 SD <--bfacrm:ticket__show__39649
      else begin
        fModified := True;
        if not fDontSetColors and fModified then
          SetColor(comp, cColorLightGreen);
      end;
    end;
//  end;

  fValue := ParseNonPrintable(fValue);

  if fIsSettingValue and (not fDontSetColors) and (not fModified) and (AddfieldType in [zfld08_Waehrung]) and
     TbfaSpinEdit(Self.comp).NegativeCurrencyRed and (TbfaSpinEdit(comp).Value < 0) then
  begin
//    SetColor(comp, cColorRed);
    // we could not set the color like in other cases.
    // if its a negative zfld08_Waehrung we take a look whats the current StyleController
    // and set it to its similar with red TextColor
    // !! in most cases we will do this in TBfaAddfieldComp.SetCompValue !!
    if TbfaSpinEdit(comp).Style.StyleController = ctlGeneral then
      TbfaSpinEdit(comp).Style.StyleController := ctlGeneralTextHighlighted
    else if TbfaSpinEdit(comp).Style.StyleController = ctlModified then
      TbfaSpinEdit(comp).Style.StyleController := ctlModifiedTextHighlighted
    else if TbfaSpinEdit(comp).Style.StyleController = ctlRequired then
      TbfaSpinEdit(comp).Style.StyleController := ctlRequiredTextHighlighted
    else if TbfaSpinEdit(comp).Style.StyleController = ctlDisabled then
      TbfaSpinEdit(comp).Style.StyleController := ctlDisabledTextHighlighted
    else
      TbfaSpinEdit(comp).Style.StyleController := ctlGeneral;
  end;

  if Assigned(OnModified) then
    OnModified(Self);
  if Assigned(OnValueSet) then
    OnValueSet(fValue, fValueHigh, fModified, fModifiedHigh);
end;

procedure TBfaAddfieldComp.OnChangeHigh(Sender: TObject; var CanEdit: Boolean);
begin
   OnChangeHigh(Sender);
end;

procedure TBfaAddfieldComp.OnChangeHigh(Sender: TObject);
var
  CursorPosHigh: Integer;

begin
  if fIsSettingValue then
    Exit;

  // Datum und DLMDate = fAllowedChar
  if fAddfieldType in [zfld01_Datum, zfld12_DLMDate] then
  begin
    if (CheckValidChars(TcxDateEdit(comp2).Text) <> TcxDateEdit(comp2).Text) and
        (CheckValidChars(TcxDateEdit(comp2).Text) <> '') then
    begin
      fValueHigh := CheckValidChars(TcxDateEdit(comp2).Text);
      CursorPosHigh := TcxDateEdit(comp2).SelStart - (Length(TcxDateEdit(comp2).Text) - Length(fValueHigh));
      TcxDateEdit(comp2).Text := fValueHigh;
      ShowBalloonHint(Sender, _(sMsgAllowedCharachters) + fAllowedChar);
      TcxDateEdit(comp2).SelStart := CursorPosHigh;
    end
    else if (ParseNotAllowedChars(TcxDateEdit(comp2).Text) <> TcxDateEdit(comp2).Text) and
        (ParseNotAllowedChars(TcxDateEdit(comp2).Text) = '') then
    begin
      SendMessage(TWinControl(comp2).Handle, WM_UNDO, 0, 0);
      ShowBalloonHint(Sender, fMsgNotAllowedChar);
    end
    else
    begin
      //02.09.2024 SD -->bfacrm:ticket__show__39649
      //Verbot der Anbringung des Datums 30.12.1899 und korrektur des Verhaltens von Suchdatumsfeldern
      if TcxDateEdit(comp2).Text = '' then
      begin
        fIsSettingValue := True;
        TcxDateEdit(comp2).Date := -700000;
        fIsSettingValue := False;
        fModifiedHigh := False;
        if not fDontSetColors then
          SetColor(comp2, cColorWhite);
      end
      else
      begin
        fModifiedHigh := True;
        if not fDontSetColors then
          SetColor(comp2, cColorLightGreen);
      end;
      //02.09.2024 SD <--bfacrm:ticket__show__39649
      fValueHigh := TcxDateEdit(comp2).Text
    end;
  end else if AddfieldType in [zfld05_Intervall] then
  begin
    if compComboBox <> nil then
    begin
  //16.11.2018 Ticket 16623 -->
      if compComboBox.ItemIndex = -1 then
      begin
        compComboBox.ItemIndex := 0;
        fModifiedHigh := True;
      end;
      SetColor(comp2, cColorLightGreen);
  //16.11.2018 Ticket 16623 <--
      fValue := TbfaSpinEdit(comp).Text + GetIntervalByIndex(compComboBox);
      fValueHigh := GetIntervalByIndex(compComboBox);
      if (GetIntervalByIndex(compComboBox) <> '') and (fValue <> fOldValue) then      // --> S.T. 9/28/2020 bfacrm:ticket__show__24502
        fModified := True;  // --> S.T. 9/28/2020 bfacrm:ticket__show__24502
      if fEditMode then	    // Ticket 15990: Suche geht nur, wenn zuletzt das Zahl-Feld verndert wurde.
        fModified := True;  // Ticket 15187: nderung im Invervallfeld (Einheit) soll Wert durchschreiben
      if Assigned(OnValueSet) then
        OnValueSet(fValue, fValueHigh, fModified, fModifiedHigh);
    end;

    // ndert sich comp2 muss auch comp modified gesetzt und eingefrbt werden
    if (GetIntervalByIndex(compComboBox) <> '') and (fValue <> fOldValue) then
    begin
      SetColor(comp, cColorLightGreen);
      SetColor(comp2, cColorLightGreen);
      fModified := True;
      fModifiedHigh := True;
    end;
  end else if (Sender is TbfaComboBox) then
    fValueHigh := TbfaComboBox(comp2).Text
  else
  // zfld05_Intervall, zfld06_GanzZahl, zfld07_FliessKomma, zfld08_Waehrung

  if ((TbfaSpinEdit(comp2).EditingText = '') and (TbfaSpinEdit(comp2).EditingValue = 0)) then
    fValueHigh := ''
  else
  begin
    fValueHigh := TbfaSpinEdit(comp2).EditingValue;
    fModifiedHigh := True;
    if (Rights and cAddfieldRead) <> cAddfieldRead then   // wenn es kein ReadOnly Feld ist
    begin
//        fModified := True;
    end;
    if not fDontSetColors then
      SetColor(comp2, cColorLightGreen);
  end;

  fValueHigh := ParseNonPrintable(fValueHigh);

  if Assigned(OnModified) then
    OnModified(Self);
  if Assigned(OnValueSet) and fEditmode then
    OnValueSet(fValue, fValueHigh, fModified, fModifiedHigh);
end;


procedure TBfaAddfieldComp.OnCalenderPopup(Sender: TObject);
var
  ACalendar: TcxPopupCalendar;
  I: Integer;
begin
  ACalendar := TcxPopupCalendar((Sender as TcxDateEdit).Properties.PopupControl);
  for I := 0 to ACalendar.ControlCount - 1 do
    if (ACalendar.Controls[I] is TcxButton) and (TCalendarButton(Integer(TcxButton(Sender).Tag))=btnClear) then
    begin
      TcxButton(ACalendar.Controls[I]).OnClick := ClickCalenderClear;
      TcxButton(ACalendar.Controls[I]).Caption := _('Lschen'); // SJ 13.09.2019 bei ndern der Sprache notwendig
      Break;
    end;

  if Assigned(Self.OnComboDropDown) then
    Self.OnComboDropDown(Sender);
end;

procedure TBfaAddfieldComp.ClickCalenderClear(Sender: TObject);
var
  DateEditInputField: TcxDateEdit;
begin
  try
    try
      if (Sender is TcxButton) and (TCalendarButton(Integer(TcxButton(Sender).Tag))=btnClear) then
      begin
        TcxPopupCalendar(Sender).Parent.Parent.Hide;  // PopupCalender ausblenden
        DateEditInputField := TcxDateEdit(TcxPopupCalendar(Sender).Parent.Owner);

        if DateEditInputField = compDateLow then
        begin
          fValue := '';

          if (fOldValue = '') then
          begin
            fModified := False;
            SetColor(DateEditInputField, cColorWhite);
          end;
          if (fOldValue <> '') then
          begin
          // SJ 10.05.2022
          // TODO SJ: 10.05.2022
          // Lschdatum lies sich nicht via Button lschen, weil dteSearch keine Rechte konfiguriert hat.
          // Das hier ist die schnelle Lsung, generell sollte ChangeArchive bei dteSearch aber
          // entsprechend die Rechte setzen.
          // Nachteil dieser Lsung: es kann zur Fehlermeldung bei Addfields ohne Rechte kommen.
          // "Fehler beim Speichern. Keine Berechtigung auf Zusatzfeld ..."
//            fModified := True;
//            fModified := (not fEditMode) or (Rights and cAddfieldModify = cAddfieldModify);

//06.06.2024 SD -->bfacrm:ticket__show__38867
//            if (Rights and cAddfieldModify = cAddfieldModify) and fEditMode then
//              fModified := True
//            else
//              fModified := False;
          // SJ 10.05.2022

            if fEditMode then
              if (Rights and cAddfieldModify = cAddfieldModify) then
                fModified := True
              else
                fModified := False
            else
//              fModified := True;  //02.09.2024 SD <--bfacrm:ticket__show__39649
              fModified := False; //02.09.2024 SD <--bfacrm:ticket__show__39649 In gereinigtem Zustand muss es unmodifiziert und grau sein
//06.06.2024 SD <--bfacrm:ticket__show__38867

            if not fDontSetColors then
              //02.09.2024 SD -->bfacrm:ticket__show__39649
              //Farbe je nach Zustand
//              SetColor(DateEditInputField, cColorLightgreen)
              if fModified then
                SetColor(DateEditInputField, cColorLightgreen)
              else
                SetColor(DateEditInputField, cColorWhite);
              //02.09.2024 SD <--bfacrm:ticket__show__39649
          end;
        end

        else if DateEditInputField = compDateHigh then
        begin
          fValueHigh := '';
          //02.09.2024 SD -->bfacrm:ticket__show__39649
          //wird nur fr die Suche verwendet. Daher sollte er immer unmodifiziert bleiben, wenn der Wert leer ist.
//          if (fOldValue = '') then
//          begin
//            fModifiedHigh := False;
//            if not fDontSetColors then
//              SetColor(DateEditInputField, cColorWhite);
//          end;
//          if (fOldValue <> '') then
//          begin
//          // SJ 10.05.2022 - siehe oben
//            fModified := True;
////            if (Rights and cAddfieldModify = cAddfieldModify) and fEditMode then
////              fModifiedHigh := True
////            else
////              fModifiedHigh := False;
//          // SJ 10.05.2022
//            if not fDontSetColors then
//              SetColor(DateEditInputField, cColorLightgreen);
//          end;
          fModifiedHigh := False;
          if not fDontSetColors then
            SetColor(DateEditInputField, cColorWhite);
          //02.09.2024 SD -->bfacrm:ticket__show__39649
        end;

//      <!-- SJ 06.01.2020 - bfacrm:ticket__show__21312
//      muss nach dem setzen von fValue passieren
        fIsSettingValue := True;
        DateEditInputField.Date := -700000;
        fIsSettingValue := False;
//      SJ 06.01.2020 - bfacrm:ticket__show__21312 -->

        if Assigned(OnModified) then
          OnModified(Self);
        if Assigned(OnValueSet) then
          OnValueSet(fValue, fValueHigh, fModified, fModifiedHigh);

        Exit;
      end;
    except

    end;
  finally
    //fIsSettingValue := False;
  end;
end;

procedure TBfaAddfieldComp.OnClick(Sender: TObject);
begin
  try
    if (Double(GetTickCount) < Double(fLastDoubleClick + GetDoubleClickTime * 1.5)) and
      (TEdit(Sender).SelStart >= fSelStart) and (TEdit(Sender).SelStart <= (fSelStart + fSelLength)) then
    begin
      TEdit(Sender).SelectAll;
      fLastDoubleClick := 0;
    end;
  except
    // nur zur Sicherheit
  end;
end;

procedure TBfaAddfieldComp.OnClickCheck(Sender: TObject; AIndex: Integer;
    APrevState, ANewState: TcxCheckBoxState);
var
  AText, AClickedText: String;
  AList: TStrings;
  idx: Integer;
begin
  try
    AClickedText := TbfaCheckedListBox(Sender).Items[AIndex].Text;

    try
//      AList := TStringList.Create('"', ';', [soStrictdelimiter]); // SJ 30.08.2022
//      AList := TStringList.Create(' ', ';', [soStrictdelimiter]); // SJ 03.11.2022 - bfacrm:ticket__show__32798
      AList := TStringList.Create(#39, ';', [soStrictdelimiter]);
      AList.DelimitedText := TbfaCheckedCombo(Self.comp).EditingText;

      if (ANewState = cbsChecked) and (AList.IndexOf(AClickedText) = -1) then
        AList.Add(AClickedText)
      else
      begin
        idx := AList.IndexOf(AClickedText);
        if idx > -1 then
          AList.Delete(idx);
      end;
      Self.Value := AList.DelimitedText;
      OnChange(Sender);
    finally
      FreeAndNil(AList);
    end;
  except
    //
  end;
end;

procedure TBfaAddfieldComp.OnComboMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  IndexItemClicked: Integer;
begin

end;

procedure TbfaAddfieldComp.OnConfigClick(Sender: TObject);
begin
  compGridPopupConfig.Popup(Mouse.CursorPos.X, Mouse.CursorPos.Y);
end;

procedure TbfaAddfieldComp.OnConfigClose(Sender: TObject);
var
  changedfooter, changedgroupby, changedheader: Boolean;
begin
  changedheader := compGrid.View.OptionsView.Header <> compGridPopupConfig.Items[0].Checked;  //04.10.2024 SD <--bfacrm:ticket__show__39992
  changedfooter := compGrid.View.OptionsView.Footer <> compGridPopupConfig.Items[1].Checked;
  changedgroupby := compGrid.View.OptionsView.GroupByBox <> compGridPopupConfig.Items[2].Checked;
  compGrid.View.OptionsView.Header := compGridPopupConfig.Items[0].Checked;
  compGrid.View.OptionsView.Footer := compGridPopupConfig.Items[1].Checked;
  compGrid.View.OptionsView.GroupByBox := compGridPopupConfig.Items[2].Checked;
  compGrid.View.OptionsView.Indicator := compGridPopupConfig.Items[3].Checked;
  compGrid.PopupStandard.UseBuiltInPopupMenus := compGridPopupConfig.Items[4].Checked;
  compGrid.View.OptionsCustomize.ColumnSorting := compGridPopupConfig.Items[5].Checked;
  // wenn wir sortieren wollen, dann mssen wir auch die mglichkeit haben, wieder in die ursprungsreihenfolge zu sortieren.
  // dafr brauchen wir die RecID Spalte
//  if compGridPopupConfig.Items[6].Checked then
//    compGrid.View.Styles.UseOddEvenStyles := bDefault
//  else
//    compGrid.View.Styles.UseOddEvenStyles := bFalse;
  compGrid.View.GetColumnByFieldName('RecID').Visible := compGrid.View.OptionsCustomize.ColumnSorting;
  compGrid.View.OptionsSelection.MultiSelect := compGridPopupConfig.Items[6].Checked;
  compGrid.FConfig.UseColumnNames := compGridPopupConfig.Items[7].Checked;
  compGrid.FConfig.ExportAddfields := compGridPopupConfig.Items[8].Checked;
  compGrid.FConfig.ExportUnicode := compGridPopupConfig.Items[9].Checked;

  var minrows: Integer := 6;
  if compGrid.View.OptionsView.Footer then
    minrows := minrows -1;
  if compGrid.View.OptionsView.GroupByBox then
    minrows := minrows -2;

  if changedfooter then
  begin
    if compGrid.View.OptionsView.Footer then
    begin
      compGrid.VisibleRows := compGrid.VisibleRows -1;
//04.10.2024 SD -->bfacrm:ticket__show__39992
//      if (compGrid.View.DataController.RecordCount >= compGrid.VisibleRows) then
//        compGrid.NewHeight := compGrid.NewHeight + 22;
//04.10.2024 SD <--bfacrm:ticket__show__39992
    end else
    begin
      compGrid.VisibleRows := compGrid.VisibleRows +1;
//04.10.2024 SD -->bfacrm:ticket__show__39992
////      if (compGrid.VisibleRows >= compGrid.View.DataController.RecordCount) then
//      if compGrid.View.DataController.RecordCount >= minrows then
//        compGrid.NewHeight := compGrid.NewHeight - 22;
//04.10.2024 SD <--bfacrm:ticket__show__39992
    end;
  end;
  if changedgroupby then
  begin
    if compGrid.View.OptionsView.GroupByBox then
    begin
      compGrid.VisibleRows := compGrid.VisibleRows -2;
//04.10.2024 SD -->bfacrm:ticket__show__39992
//      if compGrid.View.DataController.RowCount > minrows then
//         if (compGrid.View.DataController.RowCount - minrows) = 1 then
//           compGrid.NewHeight := compGrid.NewHeight + 22
//         else
//           compGrid.NewHeight := compGrid.NewHeight + 44;
//04.10.2024 SD <--bfacrm:ticket__show__39992
    end else
    begin
      compGrid.VisibleRows := compGrid.VisibleRows +2;
//04.10.2024 SD -->bfacrm:ticket__show__39992
//      if compGrid.View.DataController.RowCount > minrows then
//      begin
//         if (compGrid.View.DataController.RowCount - minrows) = 1 then
//           compGrid.NewHeight := compGrid.NewHeight - 22
//         else
//           compGrid.NewHeight := compGrid.NewHeight - 44;
//      end else
//      begin
//         compGrid.NewHeight := 160;
//         compGrid.VisibleRows := minrows;
//      end;
//04.10.2024 SD <--bfacrm:ticket__show__39992
    end;
  end;

  compGrid.NewHeight := GridHeightCalculate(compGrid.View.DataController.RowCount); //04.10.2024 SD <--bfacrm:ticket__show__39992

  if (changedfooter or changedgroupby or changedheader) and Assigned(compGrid.OnResizeHeight) and  //04.10.2024 SD <--bfacrm:ticket__show__39992
      (compGrid.View.DataController.RecordCount > 0) then
    compGrid.OnResizeHeight(compGrid, compGrid.Grid.Width, compGrid.NewHeight, fIsSettingValue);

  if changedgroupby and not compGrid.View.OptionsView.GroupByBox then
  begin
    for var i := 0 to compGrid.View.ColumnCount - 1 do
    begin
      if compGrid.View.Columns[i].GroupIndex <> -1 then
      begin
        compGrid.View.Columns[i].GroupIndex := -1;
        compGrid.View.Columns[i].Visible := True;
      end;
    end;
  end;
end;

procedure TbfaAddfieldComp.OnConfigItemClick(Sender: TObject);
begin
  TTBCustomItem(Sender).Checked := not TTBCustomItem(Sender).Checked;
  OnConfigClose(Sender);
end;

procedure TbfaAddfieldComp.OnConfigPopup(Sender: TObject);
begin
  compGridPopupConfig.Items[0].Checked := compGrid.View.OptionsView.Header;
  compGridPopupConfig.Items[1].Checked := compGrid.View.OptionsView.Footer;
  compGridPopupConfig.Items[2].Checked := compGrid.View.OptionsView.GroupByBox;
  compGridPopupConfig.Items[3].Checked := compGrid.View.OptionsView.Indicator;
  compGridPopupConfig.Items[4].Checked := compGrid.PopupStandard.UseBuiltInPopupMenus;
  compGridPopupConfig.Items[5].Checked := compGrid.View.OptionsCustomize.ColumnSorting;
//  compGridPopupConfig.Items[6].Checked := compGrid.View.Styles.UseOddEvenStyles = bDefault;
  compGridPopupConfig.Items[6].Checked := compGrid.View.OptionsSelection.MultiSelect;
  compGridPopupConfig.Items[7].Checked := compGrid.FConfig.UseColumnNames;
  compGridPopupConfig.Items[8].Checked := compGrid.FConfig.ExportAddfields;
  compGridPopupConfig.Items[9].Checked := compGrid.FConfig.ExportUnicode;
end;

procedure TBfaAddfieldComp.OnEditing(Sender: TObject; var CanEdit: Boolean);
begin
  if fIsSettingValue then
    Exit;
//15.09.2017 in Editmode nur wenn mit Maus auswhlt war
  fModified := True;

  SetColor(comp, cColorLightGreen);
end;

procedure TBfaAddfieldComp.OnExit(Sender: TObject);     //OnExit Function des Datefields
begin
  if TcxDateEdit(comp2).Text = '' then
     TcxDateEdit(comp2).Date := Date();
  if TcxDateEdit(comp).Date > TcxDateEdit(comp2).Date then
      TcxDateEdit(comp2).Date := TcxDateEdit(comp).Date;
  fValue := TcxDateEdit(comp).Text;
  if fValue = '' then
  begin
    fModified := False;
    SetColor(comp, (*31.08.2020 Farben Anpassung clWhite*)cColorWhite);
  end
  else
  begin
    fModified := True;
    if (Rights and cAddfieldRead) <> cAddfieldRead then   // wenn es kein ReadOnly Feld ist
    begin
      if Assigned(OnModified) then
        OnModified(Self);
    end;
    SetColor(comp, cColorLightGreen);
  end;
end;

procedure TBfaAddfieldComp.OnExitHigh(Sender: TObject);
begin
  if TcxDateEdit(comp).Text = '' then
     TcxDateEdit(comp).Date := Date();
  if TcxDateEdit(comp2).Date < TcxDateEdit(comp).Date then
    TcxDateEdit(comp).Date := TcxDateEdit(comp2).Date;
  SetColor(comp2, cColorLightGreen);
end;

procedure TbfaAddfieldComp.OnGridDeleteClick(Sender: TObject);
begin
  if not Editmode then
    Exit;
  if not compGrid.DataSet.Active then
    Exit;

  if compGrid.DataSet.RecordCount > 0 then
  begin
    compGrid.DataSet.Delete;
    fModified := True;
    var minrows: Integer := 6;
    if compGrid.View.OptionsView.Footer then
      minrows := minrows -1;
    if compGrid.View.OptionsView.GroupByBox then
      minrows := minrows -2;

//    if (compGrid.VisibleRows > minrows){ and (compGrid.VisibleRows > compGrid.View.DataController.RowCount)} then
    if (compGrid.View.DataController.RowCount > minrows){ and (compGrid.VisibleRows > compGrid.View.DataController.RowCount)} then
    begin
//      compGrid.NewHeight := compGrid.NewHeight -22; //04.10.2024 SD <--bfacrm:ticket__show__39992
      dec(compGrid.VisibleRows);
    end else
    begin
//      compGrid.NewHeight := 160;  //04.10.2024 SD <--bfacrm:ticket__show__39992
      compGrid.VisibleRows := minrows;
    end;

    compGrid.NewHeight := GridHeightCalculate(compGrid.View.DataController.RowCount); //04.10.2024 SD <--bfacrm:ticket__show__39992

    if Assigned(compGrid.OnResizeHeight) then
     compGrid.OnResizeHeight(compGrid, compGrid.Grid.Width, compGrid.NewHeight, fIsSettingValue);
  end;
end;

procedure TbfaAddfieldComp.OnGridEditKeyDown(Sender: TcxCustomGridTableView; AItem: TcxCustomGridTableItem;
  AEdit: TcxCustomEdit; var Key: Word; Shift: TShiftState);
var
  i, lastcolumn : Integer;
begin
//  OutputDebugString('OnGridEditKeyDown'); // das macht doch Probleme, oder???
  lastcolumn := 0;

  for i := 0 to compGrid.View.ColumnCount -1 do
  begin
    if compGrid.View.Columns[i].Properties is TcxCustomEditProperties then
      if not TcxCustomEditProperties(compGrid.View.Columns[i].Properties).ReadOnly then
        lastcolumn := i;
  end;

  if (AItem.Index = lastcolumn) and (Key = VK_RETURN) then
  begin
    try
      compGrid.View.DataController.Insert;
    except
      // hier kann es zb bei falsch konfigurierten Summenspalten zu Exception kommen
    end;
    Key := 0;
    if Assigned(compGrid.OnResizeHeight) then
      compGrid.OnResizeHeight(compGrid, compGrid.Grid.Width, compGrid.NewHeight, fIsSettingValue);
  end;

  if AEdit is TcxCurrencyEdit then
  begin
    if Key = 189 then
      TcxCurrencyEdit(AEdit).Value := TcxCurrencyEdit(AEdit).Value * -1;
  end;

end;

procedure TBfaAddfieldComp.BtnGridExportClick(Sender: TObject);
begin
  if Assigned(OnGridExportClick) then
    OnGridExportClick(Self);
end;

procedure TBfaAddfieldComp.BtnGridStampClick(Sender: TObject);
begin
  if not EditMode then
    Exit;
  if Assigned(OnGridStampClick) then
    OnGridStampClick(Self);
end;

procedure TbfaAddfieldComp.OnGridKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
//  OutputDebugString('OnGridKeyDown'); // das macht doch Probleme, oder???
  if compGrid.View.OptionsView.Footer and (compGrid.FooterSummaryItems.Count > 0) then
    compGrid.View.DataController.Summary.Recalculate;
end;

procedure TbfaAddfieldComp.OnGridAddClick(Sender: TObject);
var
  bool : Boolean;
begin
  if not Editmode then
    Exit;
  if not compGrid.DataSet.Active then
    Exit;

  compGrid.DataSet.Insert;
  compGrid.NewHeight := GridHeightCalculate(compGrid.View.DataController.RowCount); //04.10.2024 SD <--bfacrm:ticket__show__39992

  if Assigned(compGrid.OnResizeHeight) then
    compGrid.OnResizeHeight(compGrid, compGrid.Grid.Width, compGrid.NewHeight, fIsSettingValue);
  bool := True;
  compGrid.View.DataController.FocusControl(0, bool);
  if compGrid.Grid.Enabled then
    compGrid.Grid.SetFocus;
  fModified := True;
end;

procedure TBfaAddfieldComp.OnIndexingChange(Sender: TObject);
begin
  fValue := cbxIndexing.Text;
  if not EditMode then
  begin
    if (fValue = '') then
    begin
      fModified := False;
      if (Rights = 1) then
        SetColor(cbxIndexing, cColorNoRightInit)
      else
        SetColor(cbxIndexing, cColorWhite);
    end
    else
    begin
      if (Rights = 1) then
      begin
        SetColor(cbxIndexing, cColorNoRightInit);
        fModified := False;
      end else
      begin
        SetColor(cbxIndexing, cColorLightGreen);
        fModified := True;
      end;
    end;
  end else
  begin
    // EditMode aktiv
    if (fValue = fOldValue) then
    begin
      fModified := False;
      SetColor(cbxIndexing, cColorWhite);
    end
    else
    begin
      if (Rights = 1) or ((Rights and cAddfieldWrite = cAddfieldWrite) and (fOldValue <> '')) then
      begin
        SetColor(cbxIndexing, cColorNoRightInit);
        fModified := False;
      end else
      begin
        SetColor(cbxIndexing, cColorLightGreen);
        fModified := True;
      end;
    end;
  end;

  if Assigned(OnValueSet) then
    OnValueSet(fValue, fValueHigh, fModified, fModifiedHigh);

//  fValue := cbxIndexing.Text;
//  if (fValue = '') then
//  begin
//    fModified := False;
//    SetColor(cbxIndexing, (*31.08.2020 Farben Anpassung clWhite*)cColorWhite);
//  end
//  else
//  begin
//    fModified := True;
//    if (Rights = 1) (*or ((Rights and cAddfieldWrite = cAddfieldWrite) and (fValue <> ''))*) then
//      SetColor(cbxIndexing, cColorNoRightInit);
//    end
//      SetColor(cbxIndexing, cColorLightGreen);
//  end;
//
//  if Assigned(OnValueSet) then
//    OnValueSet(fValue, fValueHigh, fModified, fModifiedHigh);
end;

procedure TBfaAddfieldComp.OnKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  TSpTBXComboBox(comp).Perform(CB_SHOWDROPDOWN, Integer(True), 0);
end;

//04.06.2019 Ticket 19429 -->
procedure TBfaAddfieldComp.ShowBalloonHint(Sender: TObject; Message: string);
begin
  FBalloonHint.Description := Message;
  if (Sender is TControl) then
    FBalloonHint.ShowHint(TControl(Sender));

//  if (Sender is TbfaTextEdit) then
//    FBalloonHint.ShowHint(TbfaTextEdit(Sender))
//  else if (Sender is TSpTBXComboBox) then
//    FBalloonHint.ShowHint(TSpTBXComboBox(Sender))
//  else if (Sender is TbfaComboBox) then
//    FBalloonHint.ShowHint(TbfaComboBox(Sender))
//  else if (Sender is TbfaSpinEdit) then
//    FBalloonHint.ShowHint(TbfaSpinEdit(Sender))
//  else if (Sender is TSpTBXButtonEdit) then
//    FBalloonHint.ShowHint(TSpTBXButtonEdit(Sender))
//  else if (Sender is TbfaCheckedCombo) then
//    FBalloonHint.ShowHint(TbfaCheckedCombo(Sender));
end;
//04.06.2019 Ticket 19429 <--

procedure TBfaAddfieldComp.OnKeyPress(Sender: TObject; var Key: Char);
const
  VK_CUT = #24;
  VK_PASTE = #22;
  VK_UNDO = #26;
  VK_COPY = #3;
  VK_ESC = #27;
  VK_RETURN = #13;
  VK_CTRL_H = #8; // VCL-Standard Strg+H acts like Backspace
begin
  try
    if fIsSettingValue then
      Exit;

    if Sender is TbfaSpinEdit then
      TbfaSpinEdit(Sender).SpinEditOnKeyPress(Sender, Key, (Self.AddfieldType = zfld05_Intervall));

    // SJ 08.07.2020 - diverse if-Abfragen aufgerumt.
    //               - Key muss gelscht werden weil sonst Windows-Sound kommt
    case Key of
      VK_RETURN:
        Exit;
       {
         VK_RETURN darf nicht gelscht werden, weil er bei OnReturnSearch auch die Suche startet.
         Deswegen Exit, damit VK_RETURN nicht bei fNotAllowedChar + fAllowedChar geprft wird
       }

      VK_CUT, VK_ESC:
        begin
          Key := #0;
          Exit;
        end;

      VK_CTRL_H: // VCL-Standard Ctrl+H acts like Backspace
        begin
          if CtrlPressed then  // Key #8 is the same like backspace. Check Ctrl
          begin
            Key := #0;
            Exit;
          end;
        end;
    end;

  // #0 is an invalid key - exit
    if (Key = #0) then
      Exit;

  //12.09.2018 Ticket 15402 -->
  // all mgliche Strg-Kominationen haben zu Windows-Sounds gefhrt
  // deswegen die erlaubten Kominationen prfen und ansonsten den Key lschen
    if (CtrlPressed and not AllowedCtrlKeys and not AltGrPressed) then
    begin
      Key := #0;
      Perform(WM_NEXTDLGCTL, 0, 0);
      Exit;
    end;

    if (fAllowedChar <> '') and not (CtrlPressed or IsDeleteOrBack) then
    begin
      if (Pos(Key, fAllowedChar) = 0) then
      begin
        ShowBalloonHint(Sender, _(sMsgAllowedCharachters) + fAllowedChar);  //04.06.2019 Ticket 19429
        Beep;
        Key := #0;
        Abort;
      end;
    end;

    if (fNotAllowedChar <> '') then
    begin
      if Pos(Key, fNotAllowedChar) <> 0 then
      begin
        ShowBalloonHint(Sender, fMsgNotAllowedChar);  //04.06.2019 Ticket 19429
        Beep;
        Key := #0;
        Abort;
      end;
    end;
  finally
    if (Key = VK_RETURN) then // siehe Anmerkung bei case of
    begin
      if (Sender is TbfaCheckedCombo) and (TbfaCheckedCombo(Sender).SkipSearchByReturn = False) then
      begin
        if Assigned(OnReturnSearch) then
          OnReturnSearch;
      end;
      TbfaCheckedCombo(Sender).SkipSearchByReturn := False;
      Key := #0;
    end;
  end;
end;

procedure TBfaAddfieldComp.OnKeyPressHigh(Sender: TObject; var Key: Char);
const
  VK_CUT = #24;
  VK_PASTE = #22;
  VK_COPY = #3;
  VK_ESC = #27;
  VK_RETURN = #13;
  VK_CTRL_H = #8; // VCL-Standard Strg+H acts like Backspace
begin
  try
    if fIsSettingValue then
      Exit;

    if Sender is TbfaSpinEdit then
      TbfaSpinEdit(Sender).SpinEditOnKeyPress(Sender, Key, (Self.AddfieldType = zfld05_Intervall));

    // SJ 08.07.2020 - diverse if-Abfragen aufgerumt.
    //               - Key muss gelscht werden weil sonst Windows-Sound kommt
    case Key of
      VK_RETURN:
        Exit;
       { wird in finally gelscht }

      VK_CUT:
        begin
          SendMessage(TWinControl(Sender).Handle, WM_CUT, 0, 0);
          Key := #0;
          Exit;
        end;

      VK_COPY:
        begin
          SendMessage(TWinControl(Sender).Handle, WM_COPY, 0, 0);
          Key := #0;
          Exit;
        end;

      VK_PASTE:
        begin
          SendMessage(TWinControl(Sender).Handle, WM_PASTE, 0, 0);
          Key := #0;
          Exit;
        end;

      VK_ESC:
        begin
          Key := #0;
          Exit;
        end;

      VK_CTRL_H: // VCL-Standard Ctrl+H acts like Backspace
        begin
          if CtrlPressed then  // Key #8 is the same like backspace. Check Ctrl
          begin
            Key := #0;
            Exit;
          end;
        end;
    end;

  // #0 is an invalid key - exit
    if (Key = #0) then
      Exit;

  //12.09.2018 Ticket 15402 -->
  // all mgliche Strg-Kominationen haben zu Windows-Sounds gefhrt
  // deswegen die erlaubten Kominationen prfen und ansonsten den Key lschen
    if (CtrlPressed and not AllowedCtrlKeys and not AltGrPressed) then
    begin
      Key := #0;
      Perform(WM_NEXTDLGCTL, 0, 0);
      Exit;
    end;
  //12.09.2018 Ticket 15402 <--

    if (fAllowedChar <> '') and not (CtrlPressed or IsDeleteOrBack ) then
    begin
      if Pos(Key,fAllowedChar) = 0 then
      begin
        ShowBalloonHint(Sender, _(sMsgAllowedCharachters) + fAllowedChar);  //04.06.2019 Ticket 19429
        Key := #0;
        Abort;
      end;
    end;

    if (fNotAllowedChar <> '') then
    begin
      if Pos(Key, fNotAllowedChar) <> 0 then
      begin
        ShowBalloonHint(Sender, fMsgNotAllowedChar);  //04.06.2019 Ticket 19429
        Key := #0;
        Abort;
      end;
    end;
  finally
    if (Key = VK_RETURN) then
      Key := #0;
  end;
end;

procedure TBfaAddfieldComp.OnKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if fIsSettingValue then
    Exit;

 // SJ 19.11.21 - auch wenn fValue <> '', modified bei VK_DELETE an letzter Position
//  if ((Key = VK_BACK) or (Key = VK_DELETE)) and (fValue = '') and
//     ((Sender is TcxDateEdit) or (Sender is TbfaSpinEdit)) then

// SJ 25.04.2023 - bfacrm:ticket__show__34716
// -> Damit auch normale Felder fr die Suche modified gesetzt werden knnen
//  if ((Key in [VK_BACK, VK_DELETE])) and ((Sender is TcxDateEdit) or (Sender is TbfaSpinEdit)) then
  if ((Key in [VK_BACK, VK_DELETE])) then
  begin
    //02.09.2024 SD -->bfacrm:ticket__show__39649
    //korrektur des Verhaltens, wenn der Wert leer ist
    if (fAddfieldType in [zfld01_Datum, zfld12_DLMDate]) then
    begin
      if (TcxDateEdit(comp).Text = '') and not fEditMode then
      begin
        fModified := False;
        if not fDontSetColors then
          SetColor(comp, cColorWhite);
      end
      else
      begin
        fModified := True;
        if not fDontSetColors then
          SetColor(comp, cColorLightGreen);
      end
    end
    //02.09.2024 SD <--bfacrm:ticket__show__39649
    else
    begin
      fModified := True;
      if not fDontSetColors then
        SetColor(comp, cColorLightGreen);
    end;

    Exit;
  end;

  if (fAddfieldType in [zfld03_MehrfachAuswahl]) and ((Options and cAddfieldOptStrict) = cAddfieldOptStrict) then
  begin
    if TbfaComboBox(comp).Enabled then
      TbfaComboBox(comp).SetFocus;
//09.03.2018 Verbesserung von Auswahlfelder -->
    if (TbfaCheckedCombo(comp).Text = '') and ((Key = VK_BACK) or (Key = VK_DELETE)) then
      OnChange(Sender);
//09.03.2018 Verbesserung von Auswahlfelder <--
  end;

  // Anmerkung G.P.: Ticket 16692
  // Problem: Im Fall Ctrl+C ist Ctrl nicht im Schift enthalten, wenn zuerst Ctrl dann C losgelassen wird.
  // Grund ist unbekannt, da dies in einem einfachen Test-Projekt funktioniert.
  // Vermutung: Dynamisches Erzeugen und tiefe Verschachtelung provozieren den Effect.
  // Lsung, da keine bessere gefunden:
  // a) Oben werden die Kombinationen Ctrl+C, Ctrl+A, Ctrl+S abgefangen schon angefangen
  // b) Das OnChangeEvent wird schon bei einfachen Drckern von C, A, S vorher ausgelst, so dass an dieser Stelle OnChangeEvent berflssig ist.
  // c) Tasten A, C und S knnen immer blockiert werden.


// SJ 23.07.2020 moved to KeyPress <!--
//  if //    ((not (((Key = Ord('a')) or (Key = Ord('A'))) and (ssCtrl in shift)))
//     // and (not ((Shift = [ssCtrl]) and ((Key = Ord('S')) or (Key = Ord('s'))))) //15.08.2017 Ticket 10861
//(*08.01.2019 Kommentiere hier aus wegen Ticket 17406 und mache Ticket 16692 nochmal offen          not ((Key >= Ord('a')) and (Key <= Ord('z')))
//     and  not ((Key >= Ord('A')) and (Key <= Ord('Z')))
//     and  not ((Key >= Ord('0')) and (Key <= Ord('9')))
//     and (Key <> VK_TAB)
//     and (Key <> VK_F2)
//     and (Key <> VK_CONTROL)
//     and (Key <> VK_RETURN)
//     and (Key <> VK_ESCAPE)
//     and *) (not (Key in [VK_F1, VK_F2, VK_F3, VK_F4, VK_F5, VK_F6, VK_F7, VK_F8, VK_F9, VK_F10, VK_F11, VK_F12,
//                      VK_ESCAPE, VK_RETURN, VK_HOME,
//                      VK_END, VK_LEFT, VK_RIGHT, VK_UP, VK_DOWN, VK_CONTROL,
//                      VK_MENU, VK_SHIFT, VK_TAB])) //13.02.2018 bei dem Steuerungstasten modified darf auch nicht gesetzt
//     or
//     (
//      ((fAddfieldType = zfld02_Auswahl) or (AddfieldType = zfld11_DLM)) and (Key = VK_RETURN) //20.03.2018 Ticket 12525
//     )
//  then
//  begin
////    fIsDeleteOrBack := Key in [VK_DELETE, VK_BACK];
//    OnChange(Sender);
//  end;
// SJ 23.07.2020 moved to KeyPress -->
end;

procedure TBfaAddfieldComp.OnKeyUpHigh(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if fIsSettingValue then
    Exit;

// SJ 25.04.2023 - bfacrm:ticket__show__34716
// -> Damit auch normale Felder fr die Suche modified gesetzt werden knnen
//  if ((Key in [VK_BACK, VK_DELETE])) and (fValueHigh = '') and
//     ((Sender is TcxDateEdit) or (Sender is TbfaSpinEdit)) then
  if ((Key in [VK_BACK, VK_DELETE])) and (fValueHigh = '') then
  begin
    //02.09.2024 SD -->bfacrm:ticket__show__39649
    //korrektur des Verhaltens, wenn der Wert leer ist
    if fAddfieldType in [zfld01_Datum, zfld12_DLMDate] then
    begin
      if TcxDateEdit(comp2).Text = '' then
      begin
        fModified := False;
        if not fDontSetColors then
          SetColor(comp2, cColorWhite);
      end
      else
      begin
        fModified := True;
        if not fDontSetColors then
          SetColor(comp2, cColorLightGreen);
      end
    end
    //02.09.2024 SD <--bfacrm:ticket__show__39649
    else
    begin
      fModifiedHigh := True;
      if not fDontSetColors then
        SetColor(comp2, cColorLightGreen);
    end;

    Exit;
  end;

// SJ 23.07.2020 moved to KeyPress <!--
//  if (Key <> VK_TAB) and (Key <> VK_F2) and (Key <> VK_CONTROL) and (Key <> VK_RETURN) and (Key <> VK_ESCAPE)
//     and (not ((Shift = [ssCtrl]) and ((Key = Ord('S')) or (Key = Ord('s'))))) //15.08.2017 Ticket 10861
//     and (not (Key in [VK_ESCAPE, VK_RETURN, VK_BACK, VK_DELETE, VK_HOME,
//                      VK_END, VK_LEFT, VK_RIGHT, VK_UP, VK_DOWN, VK_CONTROL,
//                      VK_MENU, VK_SHIFT])) //13.02.2018 bei dem Steuerungstasten modified darf auch nicht gesetzt
//  then
//  begin
//    OnChangeHigh(Sender);
//  end;
// SJ 23.07.2020 moved to KeyPress -->
end;

procedure TBfaAddfieldComp.OnResizeDate(Sender: TObject);
begin
  try
//    fSpare := (Self.Width - fLeft - fRight) - ((fFieldWidth) *2) + 9;
//    Self.compDateHigh.Left := fLeft + fSpare + fFieldWidth;

    // SJ 17.01.2022 - Quickfix. Geht auch schner, aber so ist es am wenigsten Umbau
    if Self.Parent.Name = 'scrbxStandard' then
      fRight := 36
    else
      fRight := 40;

    Self.compDateHigh.Left := Self.Width - Self.compDateHigh.Width - fRight + fLeft;
    Self.Label2.Left := Self.compDateHigh.Left;
// SJ 10.11.2021 - HS: keine Obergrenze fr den Abstand von High Comp. High = rechtsbndig
//  if Self.Width > 350 then
//    Self.compDateHigh.Left := fLeft + fSpare + fFieldWidth + 9
//  else
//  begin
//    fSpare := (Self.Width - fLeft - fRight) - ((fFieldWidth) *2) + 9;
//    Self.compDateHigh.Left := fLeft + fSpare + fFieldWidth;
//  end;
  except
    //
  end;
end;

procedure TBfaAddfieldComp.OnResizeInterval(Sender: TObject);
begin
  try
    fSpare := (Self.Width - fLeft - fRight) - ((fFieldWidth) *2) + 9;
    Self.compComboBox.Left := fLeft + fSpare + fFieldWidth;
// SJ 10.11.2021 - HS: keine Obergrenze fr den Abstand von High Comp. High = rechtsbndig
//  if Self.Width > 350 then
//    Self.compComboBox.Left :=  fLeft + fSpare + fFieldWidth + 9
//  else
//  begin
//    fSpare := (Self.Width - fLeft - fRight) - ((fFieldWidth) *2) + 9;
//    Self.compComboBox.Left := fLeft + fSpare + fFieldWidth;
//  end;
  except
    //
  end;
end;

procedure TBfaAddfieldComp.OnResizeNum(Sender: TObject);
begin
  try
    fSpare := (Self.Width - fLeft - fRight) - ((fFieldWidth) *2) + 9;
    Self.compSpinEditHigh.Left := fLeft + fSpare + fFieldWidth;
    Self.Label2.Left := Self.compSpinEditHigh.Left;
// SJ 10.11.2021 - HS: keine Obergrenze fr den Abstand von High Comp. High = rechtsbndig
//  if Self.Width > 350 then
//    Self.compSpinEditHigh.Left :=  fLeft + fSpare + fFieldWidth + 9
//  else
//  begin
//    fSpare := (Self.Width - fLeft - fRight) - ((fFieldWidth) *2) + 9;
//    Self.compSpinEditHigh.Left := fLeft + fSpare + fFieldWidth;
//  end;
  except
    //
  end;
end;

procedure TBfaAddfieldComp.Reset;
var
  i: Integer;
  found: Boolean;
  MStream: TMemoryStream;
  uString: WideString;
begin
  fIsSettingValue := True;
  try
    fValue := fOldValue;
    fValueHigh := '';
    fModified := False;
    Negate := False;

    if comp <> nil then
    begin
      chkIndexing.Checked := False;
      if tlbNeg.Items <> nil then
        tlbNeg.Items[0].Checked := False;
      cliNeg.Color := (*31.08.2020 Farben Anpassung clWhite*)cColorWhite;
      if fIsIndexing then
        ShowIndexingCombo(False);
    end;

    if (comp is TbfaTextEdit) and (Self.AddfieldType <> zfld13_Grid) then
      TbfaTextEdit(comp).Text := fValue;

    if comp is TbfaComboBox then
    begin
      TbfaComboBox(comp).Properties.Items.BeginUpdate;

      try
        TbfaComboBox(comp).SkipChange := True;

//10.04.2018 weil Clear und Assign wahrscheinlich langsammer wird, deswegen vorher prfen ob notwendig -->
        found := (TbfaComboBox(comp).Properties.Items.Count <> TbfaComboBox(comp).StoredItems.Count)
           or (TbfaComboBox(comp).Properties.Items.Count <> (fComboitems.Count{ + SearchItems.Count}));

        // f.schneider 23.07.2020 bfacrm:ticket__show__22709
        // da wir keine unterschiedlichen Listen mehr ber die SearchItems bekommen (die wurden ausgebaut)
        // mssen wir auch nicht mehr die gesamte Liste nach Gleichheit durchlaufen.
        // das spart viel Zeit
//        if not found then
//        begin
//          if TbfaComboBox(comp).Items.Count = TbfaComboBox(comp).StoredItems.Count then
//          begin
//            for i := 0 to TbfaComboBox(comp).Items.Count - 1 do
//            begin
//              if TbfaComboBox(comp).Items[i] <> TbfaComboBox(comp).StoredItems[i] then
//              begin
//                found := True;
//                Break;
//              end;
//            end;
//          end
//          else
//            found := True;
//        end;
//10.04.2018 weil Clear und Assign wahrscheinlich langsammer wird, deswegen vorher prfen ob notwendig -->

        if found then
        begin
          if TbfaComboBox(comp).Properties.Items.Count > 0 then
            TbfaComboBox(comp).Properties.Items.Clear;
          if TbfaComboBox(comp).StoredItems.Count > 0 then
            TbfaComboBox(comp).StoredItems.Clear;
          if fComboitems.Count > 0 then
            TbfaComboBox(comp).StoredItems.AddStrings(fComboitems);
//          if SearchItems.Count > 0 then
//            TbfaComboBox(comp).StoredItems.AddStrings(SearchItems);

          if TbfaComboBox(comp).StoredItems.Count > 0 then
          begin
            MStream := TMemoryStream.Create;
            TbfaComboBox(comp).StoredItems.SaveToStream(MStream, TEncoding.UTF8);
            MStream.Position := 0;
            SendMessage(TbfaComboBox(comp).Handle, CB_INITSTORAGE, TbfaComboBox(comp).StoredItems.Count, MStream.Size);
            TbfaComboBox(comp).Properties.Items.LoadFromStream(MStream);
//            TbfaComboBox(comp).Items.Assign(TbfaComboBox(comp).StoredItems);

            FreeAndNil(MStream);
          end;
        end;

        TbfaComboBox(comp).Text := fValue;
        if fValue = '' then
          TbfaComboBox(comp).ItemIndex := -1;
      finally
        TbfaComboBox(comp).SkipChange := False;
        TbfaComboBox(comp).Properties.Items.EndUpdate;
      end;
    end;

    if comp2 is TSpTBXComboBox then
    begin
      TSpTBXComboBox(comp2).Text := fValueHigh;
      if fValueHigh = '' then
        TSpTBXComboBox(comp2).ItemIndex := -1;
    end else if (comp2 is TbfaComboBox) then
    begin
      TbfaComboBox(comp2).Text := fValueHigh;
      if fValueHigh = '' then
        TbfaComboBox(comp2).ItemIndex := -1;
    end;

    if (comp is TbfaSpinEdit) then
    begin
      TbfaSpinEdit(comp).Text := fValue;
      TbfaSpinEdit(comp).Value := Null;
    end;

    if (comp2 is TbfaSpinEdit) then
    begin
      TbfaSpinEdit(comp2).Text := fValueHigh;
      TbfaSpinEdit(comp).Value := Null;
    end;

    if (comp is TbfaCheckedCombo) then
    begin
      TbfaCheckedCombo(comp).Text := fValue;

//27.10.2017 Ticket 12418 -->
      try
        if comp = compCheckedCombo then
        begin
          if lstCheckedListbox <> nil then
          begin
            if lstCheckedListbox.Visible then  // Vorschlag: Wenn das Addfield resetted wird, kann die Listbox auch geschlossen werden.  // Ticket 14602
              lstCheckedListbox.Visible := False;

            lstCheckedListbox.Items.BeginUpdate;
            try
              if lstCheckedListbox.Items.Count > 0 then
                lstCheckedListbox.Items.Clear;

//20.03.2023              compCheckedCombo.Properties.Tokens.BeginUpdate;
              compCheckedCombo.Properties.BeginUpdate; //20.03.2023 bfacrm:ticket__show__34443
              try
                if compCheckedCombo.Properties.Tokens.Count > 0 then
                  compCheckedCombo.Properties.Tokens.Clear;

                if (fComboitems.Count > 0) then
                begin
                  for i := 0 to fComboItems.Count - 1 do
                  begin
                    uString := Trim(StringReplace(fComboItems[i], '"', '', [rfReplaceAll]));
                    with lstCheckedListbox.Items.Add do
                    begin
                      Text := fComboItems[i];
                      State := cbsUnchecked;
                      if (uString = '') and fEditMode then
                        Enabled := False;
                    end;
                    with compCheckedCombo.Properties.Tokens.Add do
                    begin
                      Text := fComboItems[i];
                    end;
                  end;
                end;
  //              if (SearchItems.Count > 0) then
  //              begin
  //                for i := 0 to fComboItems.Count - 1 do
  //                begin
  //                  with lstCheckedListbox.Items.Add do
  //                  begin
  //                    Text := SearchItems[i];
  //                    State := cbsUnchecked;
  //                    if (Text = sCaptionOnlySearch) then
  //                      Enabled := False;
  //                  end;
  //                end;
  //              end;
  //              if lstCheckedListbox.Items.IndexOf(sCaptionOnlySearch) > -1 then
  //              begin
  //                lstCheckedListbox.Items[lstCheckedListbox.Items.IndexOf(sCaptionOnlySearch)].Enabled := False;
  //              end;
              finally
                compCheckedCombo.Properties.EndUpdate; //20.03.2023 bfacrm:ticket__show__34443
//20.03.2023                compCheckedCombo.Properties.Tokens.EndUpdate;
              end;

            finally
              lstCheckedListbox.Items.EndUpdate;
            end;
          end;
        end;
      except
        //
      end;
//27.10.2017 Ticket 12418 <--
    end;

    if comp is TcxDateEdit then
    begin
      // bfacrm:ticket__show__29228
      // SJ 16.02.2022 - das Problem ist, dass beim setzen des Textes kein gltiges Datum
      // eingetragen wird. Dort wird zB '2020-12-30 00:00:00' gesetzt. Text ist dann nur '20.'
      // Wir gehen also ber den Setter. Dieser arbeitet mit DateOf / StrToDateDef.
      // Alternativ kann man natrlich auch den Code aus dem Setter wiederholen,
      // falls durch dort durchgefhrten Code irgendwelche anderen Probleme entstehen
      Value := fValue;
      ValueHigh := fValueHigh;
//      TcxDateEdit(comp).Text := fValue;
//      TcxDateEdit(comp2).Text := '';
    end;

    if (Rights = 1) then
      SetColor(comp, cColorNoRightInit)
    else if (Options and cAddfieldOptMandatory) = cAddfieldOptMandatory then
      SetColor(comp, cColorRequiredField)
    else
      SetColor(comp, cColorWhite);

    if Assigned(comp2) then
    begin
      if (Rights = 1) then
        SetColor(comp2, cColorNoRightInit)
      else if (Options and cAddfieldOptMandatory) = cAddfieldOptMandatory then
        SetColor(comp2, cColorRequiredField)
      else
        SetColor(comp2, cColorWhite);
    end;

  finally
    fIsSettingValue := False;
  end;
end;

procedure TBfaAddfieldComp.SetAddButton(const Value: Boolean);
begin
  fAddbutton := Value;
  if AddButton then
  begin
    btnAdd.Visible := True;
    compEdit.Width := compEdit.Width -24;
  end else
  begin
    btnAdd.Visible := False;
    compEdit.Width := Self.Width - fRight - fIndexingOffset + 5;
  end;
  btnAdd.TabStop := btnAdd.Visible;
end;

procedure TBfaAddfieldComp.SetAllowedChar(const Value: string);
begin
  fAllowedChar := Value;
  fMsgAllowedChar := 'Erlaubte Zeichen: ' + Value;
  if comp is TbfaTextEdit then
  begin
    TbfaTextEdit(comp).Hint := fMsgAllowedChar;
    TbfaTextEdit(comp).ShowHint := True;
  end;

  if comp is TSpTBXComboBox then
  begin
    TSpTBXComboBox(comp).Hint := fMsgAllowedChar;
    TSpTBXComboBox(comp).ShowHint := True;
  end else if (comp is TbfaComboBox) then
  begin
    TbfaComboBox(comp).Hint := fMsgAllowedChar;
    TbfaComboBox(comp).ShowHint := True;
  end;

  if comp is TbfaSpinEdit then
  begin
    TbfaSpinEdit(comp).Hint := fMsgAllowedChar;
    TbfaSpinEdit(comp).ShowHint := True;
  end;

  if comp2 is TbfaSpinEdit then
  begin
    TbfaSpinEdit(comp2).Hint := fMsgAllowedChar;
    TbfaSpinEdit(comp).ShowHint := True;
  end;

  if comp is TbfaCheckedCombo then
  begin
    TbfaCheckedCombo(comp).Hint := fMsgAllowedChar;
    TbfaCheckedCombo(comp).ShowHint := True;
  end;
end;

procedure TBfaAddfieldComp.SetButtonColors(AButton: TcxButton; AStyle: TcxEditStyleController);
begin
  try
    AButton.LookAndFeel.NativeStyle := False;
    AButton.Colors.Default := AStyle.Style.Color;
    AButton.Colors.DefaultText := AStyle.Style.TextColor;
    AButton.Colors.Normal := AStyle.Style.Color;
    AButton.Colors.NormalText := AStyle.Style.TextColor;
    AButton.Colors.Hot := AStyle.StyleHot.Color;
    AButton.Colors.HotText := AStyle.StyleHot.TextColor;
    AButton.Colors.Disabled := AStyle.StyleDisabled.Color;
    AButton.Colors.DisabledText := AStyle.StyleDisabled.TextColor;
    AButton.Colors.Assign(AButton.Colors);
  except
    //04.07.2023 CrashLog
  end;
end;

procedure TBfaAddfieldComp.SetCheckedComboValue;
var
  i : Integer;
begin
  Self.fValue := '';
  for i := 0 to Self.lstCheckedListbox.Items.Count -1 do
  begin
    if Self.lstCheckedListbox.Items[i].Checked then
    begin
      if fEditMode and (Self.lstCheckedListbox.Items[i].Text = '""') then
      begin
        Self.lstCheckedListbox.Items[i].Checked := False;
        Continue;   // der Eintrag mit "" ist nur zur Suche
      end;
      if Self.fValue = '' then
        Self.fValue := Self.lstCheckedListbox.Items[i].Text
      else
        Self.fValue := Self.fValue + ';' + Self.lstCheckedListbox.Items[i].Text;
    end;
  end;
  TbfaCheckedCombo(Self.comp).Text := Self.fValue;
  TbfaCheckedCombo(Self.comp).SelStart := Length(TbfaCheckedCombo(Self.comp).Text);
end;

procedure TBfaAddfieldComp.SetColor(Acomp: TComponent; Acolor: TColor);
begin
  try
    if Acomp = nil then
      Exit;

    // es gab ein Problem beim setzen des StyleControllers, welches auf ein inkompatibles
    // Casting zurck zu fhren war. Dewswegen casten wir jetzt jedes einzeln...
    if Acomp is TbfaCheckedCombo then
    begin
      if (AColor = cColorWhite) and (ctlGeneral <> nil) then
        TbfaCheckedCombo(Acomp).Style.StyleController := ctlGeneral
      else if (AColor = cColorRequiredField) and (ctlRequired <> nil) then
        TbfaCheckedCombo(Acomp).Style.StyleController := ctlRequired
      else if (AColor = cColorLightGreen) and (ctlModified <> nil) then
        TbfaCheckedCombo(Acomp).Style.StyleController := ctlModified
      else if (AColor = cColorNoRightInit) and (ctlDisabled <> nil) then
        TbfaCheckedCombo(Acomp).Style.StyleController := ctlDisabled
      else if (ctlGeneral <> nil) then
        TbfaCheckedCombo(Acomp).Style.StyleController := ctlGeneral
      else
        TbfaCheckedCombo(Acomp).Style.Color := AColor;
    end
    else if Acomp is TcxDateEdit then
    begin
      if (AColor = cColorWhite) and (ctlGeneral <> nil) then
        TcxDateEdit(Acomp).Style.StyleController := ctlGeneral
      else if (AColor = cColorRequiredField) and (ctlRequired <> nil) then
        TcxDateEdit(Acomp).Style.StyleController := ctlRequired
      else if (AColor = cColorLightGreen) and (ctlModified <> nil) then
        TcxDateEdit(Acomp).Style.StyleController := ctlModified
      else if (AColor = cColorNoRightInit) and (ctlDisabled <> nil) then
        TcxDateEdit(Acomp).Style.StyleController := ctlDisabled
      else if (ctlGeneral <> nil) then
        TcxDateEdit(Acomp).Style.StyleController := ctlGeneral
      else
        TcxDateEdit(Acomp).Style.Color := AColor;
    end
    else if Acomp is TbfaCheckedListBox then
    begin
      if (AColor = cColorWhite) and (ctlGeneral <> nil) then
        TbfaCheckedListBox(Acomp).Style.StyleController := ctlGeneral
      else if (AColor = cColorRequiredField) and (ctlRequired <> nil) then
        TbfaCheckedListBox(Acomp).Style.StyleController := ctlRequired
      else if (AColor = cColorLightGreen) and (ctlModified <> nil) then
        TbfaCheckedListBox(Acomp).Style.StyleController := ctlModified
      else if (AColor = cColorNoRightInit) and (ctlDisabled <> nil) then
        TbfaCheckedListBox(Acomp).Style.StyleController := ctlDisabled
      else if (ctlGeneral <> nil) then
        TbfaCheckedListBox(Acomp).Style.StyleController := ctlGeneral
      else
        TbfaCheckedListBox(Acomp).Style.Color := AColor;
    end
    else if Acomp is TbfaSpinEdit then
    begin
      if (AColor = cColorWhite) and (ctlGeneral <> nil) then
        TbfaSpinEdit(Acomp).Style.StyleController := ctlGeneral
      else if (AColor = cColorRequiredField) and (ctlRequired <> nil) then
        TbfaSpinEdit(Acomp).Style.StyleController := ctlRequired
      else if (AColor = cColorLightGreen) and (ctlModified <> nil) then
        TbfaSpinEdit(Acomp).Style.StyleController := ctlModified
      else if (AColor = cColorNoRightInit) and (ctlDisabled <> nil) then
        TbfaSpinEdit(Acomp).Style.StyleController := ctlDisabled
      else if (ctlGeneral <> nil) then
        TbfaSpinEdit(Acomp).Style.StyleController := ctlGeneral
      else
        TbfaSpinEdit(Acomp).Style.Color := AColor;
    end
    else if Acomp is TbfaComboBox then
    begin
      if (AColor = cColorWhite) and (ctlGeneral <> nil) then
        TbfaComboBox(Acomp).Style.StyleController := ctlGeneral
      else if (AColor = cColorRequiredField) and (ctlRequired <> nil) then
        TbfaComboBox(Acomp).Style.StyleController := ctlRequired
      else if (AColor = cColorLightGreen) and (ctlModified <> nil) then
        TbfaComboBox(Acomp).Style.StyleController := ctlModified
      else if (AColor = cColorNoRightInit) and (ctlDisabled <> nil) then
        TbfaComboBox(Acomp).Style.StyleController := ctlDisabled
      else if (ctlGeneral <> nil) then
        TbfaComboBox(Acomp).Style.StyleController := ctlGeneral
      else
        TbfaComboBox(Acomp).Style.Color := AColor;
    end
    else if Acomp is TbfaTextEdit then
    begin
      if (AColor = cColorWhite) and (ctlGeneral <> nil) then
        TbfaTextEdit(Acomp).Style.StyleController := ctlGeneral
      else if (AColor = cColorRequiredField) and (ctlRequired <> nil) then
        TbfaTextEdit(Acomp).Style.StyleController := ctlRequired
      else if (AColor = cColorLightGreen) and (ctlModified <> nil) then
        TbfaTextEdit(Acomp).Style.StyleController := ctlModified
      else if (AColor = cColorNoRightInit) and (ctlDisabled <> nil) then
        TbfaTextEdit(Acomp).Style.StyleController := ctlDisabled
      else if (ctlGeneral <> nil) then
        TbfaTextEdit(Acomp).Style.StyleController := ctlGeneral
      else
        TbfaTextEdit(Acomp).Style.Color := AColor;
    end;
  except
     //
  end;
end;

procedure TBfaAddfieldComp.SetColor(AColor: TColor);
begin
  try
    if ctlGeneral <> nil then
    begin
      if AColor = cColorWhite then
        TcxTextEdit(comp).Style.StyleController := ctlGeneral
      else if (AColor = cColorRequiredField) and (ctlRequired <> nil) then
        TcxTextEdit(comp).Style.StyleController := ctlRequired
      else if (AColor = cColorLightGreen) and (ctlModified <> nil) then
        TcxTextEdit(comp).Style.StyleController := ctlModified
      else if (AColor = cColorNoRightInit) and (ctlDisabled <> nil) then
        TcxTextEdit(comp).Style.StyleController := ctlDisabled
      else if (ctlGeneral <> nil) then
        TcxTextEdit(comp).Style.StyleController := ctlGeneral
      else
        TcxTextEdit(comp).Style.Color := AColor;
    end;
  except
    //
  end;
end;

procedure TBfaAddfieldComp.SetDispname(const Value: string);
begin
  fDispname := Value;
  if Pos('&', fDispname) > 0 then
    fDispname := StringReplace(fDispname,'&', '&&', [rfReplaceAll]);
  Label1.Caption := fDispname + ':';

  if (Options and cAddfieldOptMandatory) = cAddfieldOptMandatory then
    Label1.Style.Font.Style := [fsBold];
end;

procedure TbfaAddfieldComp.SetEditMode(const Value: Boolean);
var
  bool : Boolean;
  I: Integer;
  uString: WideString;
begin
  fEditmode := Value;
  if fAddfieldType in [zfld13_Grid] then
  begin
    edtGridSearch.Enabled := not fEditmode;
    edtGridSearch.TabStop := not fEditmode;
    cbxGridSearchField.TabStop := not fEditmode;
    // <!-- SJ 21.04.2022: das bearbeiten wird in OnGridEditing geregelt
//    compGrid.Grid.Enabled := fEditmode;
//    compGrid.Grid.TabStop := fEditmode;
    // SJ 21.04.2022 -->

    btnGridAdd.Enabled := fEditmode;
    btnGridDelete.Enabled := fEditmode;
    btnGridStamp.Enabled := fEditmode;

    if fEditmode and (compGrid.View.DataController.RecordCount = 0) and compGrid.DataSet.Active then
      compGrid.DataSet.Insert;

    if fEditMode and edtGridSearch.Focused then
    begin
      bool := True;
      compGrid.View.DataController.FocusControl(0, bool);
      compGrid.Grid.SetFocus;
    end
    else if compGrid.Grid.Focused then
      edtGridSearch.SetFocus;
  end

  else if (fAddfieldType in [zfld03_MehrfachAuswahl]) then
  begin
    for I := 0 to Self.lstCheckedListbox.Items.Count - 1 do
    begin
      // im Editmode nehmen wir "" raus
      uString := Trim(StringReplace(Self.lstCheckedListbox.Items[I].Text, '"', '', [rfReplaceAll]));
      if (uString = '') then
      begin
        if Self.lstCheckedListbox.Items[I].Checked and fEditMode then
          Self.lstCheckedListbox.Items[I].Checked := False;

        Self.lstCheckedListbox.Items[I].Enabled := not fEditMode;
      end;
    end;
  end

  {$IFDEF Version37}
  // bfacrm:ticket__show__29034 / bfacrm:ticket__show__33496
  // bis Felder beim EditMode deaktivieren
  else if fAddfieldType in [zfld01_Datum, zfld06_GanzZahl, zfld07_FliessKomma, zfld08_Waehrung, zfld12_DLMDate] then
  begin
    case fAddfieldType of
      zfld01_Datum, zfld12_DLMDate:
      begin
        if compDateHigh <> nil then
          compDateHigh.Enabled := (not Self.Editmode);
      end;

      zfld06_GanzZahl, zfld07_FliessKomma, zfld08_Waehrung:
      begin
        if compSpinEditHigh <> nil then
          compSpinEditHigh.Enabled := (not Self.Editmode);
      end;
    end;
  end;
  {$ENDIF}
end;

procedure TBfaAddfieldComp.SetIndexing(const Value: Boolean);
var
  diffwidth : Integer;
begin
  if fAddfieldType = zfld13_Grid then
    Exit;
  fIndexing := Value;
  if fIndexing then
  begin
    fIndexingOffset := 20;
    chkIndexing.Visible := True;
    cliNeg.Visible := False;
    tlbNeg.Visible := False;
    btnIndexing.Visible := True;
    diffwidth := 20;
  end
  else
  begin
    fIndexingOffset := 0;
    chkIndexing.Visible := False;
    cliNeg.Visible := True;
    tlbNeg.Visible := True;
    btnIndexing.Visible := False;
    diffwidth := -20;
  end;

  if (comp is TbfaTextEdit) and not (comp is TbfaSpinEdit) then
  begin
    TbfaTextEdit(comp).Left := fLeft + fIndexingOffset;
    TbfaTextEdit(comp).Width := TbfaTextEdit(comp).Width - diffwidth;
  end;

  if comp is TSpTBXComboBox then
  begin
    TSpTBXComboBox(comp).Left := fLeft + fIndexingOffset;
    TSpTBXComboBox(comp).Width := TSpTBXComboBox(comp).Width - diffwidth;
  end else if (comp is TbfaComboBox) then
  begin
    TbfaComboBox(comp).Left := fLeft + fIndexingOffset;
    TbfaComboBox(comp).Width := TbfaComboBox(comp).Width - diffwidth;
  end;

  if comp is TbfaSpinEdit then
  begin
    TbfaSpinEdit(comp).Left := fLeft + fIndexingOffset;
  end;

  if comp is TSpTBXButtonEdit then
  begin
    TSpTBXButtonEdit(comp).Left := fLeft + fIndexingOffset;
  end else if (comp is TbfaCheckedCombo) then
  begin
    TbfaCheckedCombo(comp).Left := fLeft + fIndexingOffset;
    TbfaCheckedCombo(comp).Width := TbfaCheckedCombo(comp).Width - diffwidth;
  end;

  if comp is TcxDateEdit then
  begin
    TcxDateEdit(comp).Left := fLeft + fIndexingOffset;
    TcxDateEdit(comp2).Left := TcxDateEdit(comp2).Left; // - diffwidth;
  end;

  Label1.Left := fLeft + fIndexingOffset;
  Self.Update;
end;

procedure TBfaAddfieldComp.SetItems(const Value: string);
begin
  fIsSettingValue := True;
  try
    if AddfieldType in [zfld02_Auswahl, zfld11_DLM] then
    begin
      TbfaComboBox(Self.comp).Properties.Items.Delimiter := ';';
      TbfaComboBox(Self.comp).Properties.Items.StrictDelimiter := True;
      TbfaComboBox(Self.comp).Properties.Items.DelimitedText := Value;
      fComboitems.AddStrings(TbfaComboBox(Self.comp).Properties.Items);
    end
    else if AddfieldType in [zfld05_Intervall] then
    begin
      TbfaComboBox(Self.comp2).Properties.Items.Delimiter := ';';
      TbfaComboBox(Self.comp2).Properties.Items.StrictDelimiter := True;
      TbfaComboBox(Self.comp2).Properties.Items.DelimitedText := Value;
    end;
  finally
    fIsSettingValue := False;
  end;
end;

procedure TBfaAddfieldComp.SetModified(const Value: Boolean);
begin
  if comp = nil then
    Exit;

  if not Value then
  begin
    fModified := False;
    if (Rights = 1) then
      SetColor(comp, cColorNoRightInit)
    else if (Options and cAddfieldOptMandatory) = cAddfieldOptMandatory then
      SetColor(comp, cColorRequiredField)
    else
      SetColor(comp, cColorWhite);
  end
  else
  begin
    fModified := True;
    SetColor(comp, cColorLightGreen);
  end;
//  if not Value then
//  begin
//    fModified := False;
//    if (Rights = 1) then
//    begin
//      SetColor(comp, cColorNoRightInit);
//      SetColor(comp2, cColorNoRightInit);
//    end
//    else if (Options and cAddfieldOptMandatory) = cAddfieldOptMandatory then
//    begin
//      SetColor(comp, cColorRequiredField);
//      SetColor(comp2, cColorRequiredField);
//    end else
//    begin
//      SetColor(comp, cColorWhite);
//      SetColor(comp2, cColorWhite);
//    end;
//  end
//  else
//  begin
//    fModified := True;
//    if (Rights and cAddfieldRead) <> cAddfieldRead then   // wenn es kein ReadOnly Feld ist
//    begin
////        fModified := True;
//    end;
//
//    SetColor(comp, cColorLightGreen);
//    SetColor(comp2, cColorLightGreen);
//  end;
end;

procedure TBfaAddfieldComp.SetModifiedHigh(const Value: Boolean);
begin
  if comp2 = nil then
    Exit;

  if not Value then
  begin
    fModifiedHigh := False;
    if (Rights = 1) then
      SetColor(comp2, cColorNoRightInit)
    else if (Options and cAddfieldOptMandatory) = cAddfieldOptMandatory then
      SetColor(comp2, cColorRequiredField)
    else
      SetColor(comp2, cColorWhite);
  end
  else
  begin
    fModifiedHigh := True;
    SetColor(comp2, cColorLightGreen);
  end;
end;

procedure TBfaAddfieldComp.SetNotAllowedChar(const Value: string);
begin
  fNotAllowedChar := Value;
  fMsgNotAllowedChar := _(sMsgNotAllowedCharachters) + Value;
end;

procedure TBfaAddfieldComp.SetOldValue(const Value: Variant);
begin
  if not fOldValueSet then  // kann nur zu beginn einmal gesetzt werden
  begin
    fOldValue := Value;
    fOldValueSet := True;
  end;
end;

procedure TBfaAddfieldComp.SetOptions(const Value: Integer);
begin
  fOptions := Value;
  if (fOptions and cAddfieldOptMandatory) = cAddfieldOptMandatory then
  begin
    if comp is TbfaTextEdit then
    begin
      TbfaTextEdit(comp).Color := cColorRequiredField;
    end;

    if comp is TbfaComboBox then
    begin
      TbfaComboBox(comp).Color := cColorRequiredField;
    end;

    if comp is TbfaSpinEdit then
    begin
      TbfaSpinEdit(comp).Color := cColorRequiredField;
    end;

    if comp2 is TbfaSpinEdit then
    begin
      TbfaSpinEdit(comp2).Color := cColorRequiredField;
    end;

    if comp is TSpTBXButtonEdit then
    begin
      TSpTBXButtonEdit(comp).Color := cColorRequiredField;
    end else if (comp is TbfaCheckedCombo) then
    begin
      TbfaCheckedCombo(comp).Color := cColorRequiredField;
    end;

    if comp is TcxDateEdit then
    begin
      Self.SetColor(comp, cColorRequiredField);
//      TcxDateEdit(comp).Style.Color := cColorRequiredField;
    end;

    if comp2 is TcxDateEdit then
    begin
      Self.SetColor(comp2, cColorRequiredField);
//      TcxDateEdit(comp2).Style.Color := cColorRequiredField;
    end;
  end;
  if (fOptions and cAddfieldOptStrict) = cAddfieldOptStrict then
  begin
    if comp is TSpTBXComboBox then
    begin
//      TSpTBXComboBox(comp).Style := csDropDownList;
    end;

  end;
end;

procedure TBfaAddfieldComp.SetRights(const Value: Integer);
begin
  fRights := Value;
  if ((fRights and cAddfieldRead) = cAddfieldRead) and
     ((fRights and cAddfieldWrite) = 0) and
     ((fRights and cAddfieldModify) = 0) then
  begin
    SetColor(comp, cColorNoRightInit);
    SetColor(comp2, cColorNoRightInit);
  end;
end;

procedure TBfaAddfieldComp.SetStrings(const Value: TStrings);
begin
  fIsSettingValue := True;
  try
    if (AddfieldType in [zfld02_Auswahl, zfld11_DLM]) then
    begin
      TbfaComboBox(Self.comp).Properties.Items.Assign(Value);
      fComboitems.AddStrings(TbfaComboBox(Self.comp).Properties.Items);
    end
    else if AddfieldType in [zfld03_MehrfachAuswahl] then
      TbfaCheckedListBox(comp2).Items.Assign(Value);
  finally
    fIsSettingValue := False;
  end;
end;

procedure TBfaAddfieldComp.SetValueHigh(const Value: Variant);
begin
  fValueHigh := Value;

  if (Self.AddfieldType in [zfld01_Datum, zfld12_DLMDate]) then
  begin
    if VarIsType(Value, varDate) then
      TcxDateEdit(Self.comp2).Date := DateOf(Value)
    else
//30.08.2021 Ticket 28297 Workaround, muss geprft werden! -->
//        TcxDateEdit(Self.comp).Text := Value;
    begin
      var str : string;
      if Pos(' ', Value) > 0 then
        str := Copy(Value, 1, Pos(' ', Value) - 1)
      else
        str := Value;
      if (str <> '') and (StrToDateDef(str, 0, formatSettings) <> 0) then
      begin
        TcxDateEdit(Self.comp2).Date := StrToDateDef(str, 0, formatSettings);
        fValueHigh := TcxDateEdit(Self.comp2).Text;
      end
      else
        TcxDateEdit(Self.comp2).Date := -700000;
    end;
//30.08.2021 Ticket 28297 <--
//      if (fRights = 1) or
//         ((fRights and cAddfieldWrite = cAddfieldWrite) and (TcxDateEdit(Self.comp).Text <> '')) then
//      begin
//        Self.SetColor(Self.comp, cColorNoRightInit);
//        Self.SetColor(Self.comp2, cColorNoRightInit);
//      end;
  end;
end;

procedure TBfaAddfieldComp.SetCompValue(const Value: Variant);
var
  lstValues, lstRows, lstKeys: TStringList;
  formatSettings: TFormatSettings;
  localformatSettings: TFormatSettings;
  columnconfig: TbfaColumnConfig;
  column : TcxGridDBColumn;
  ARow: TcxGridGroupRow;
  dvalue: Double;
  dtvalue: TDateTime;
  ivalue: Int64;
  reg : TRegEx;
  match: TMatch;
begin
  fIsSettingValue := True;
  try
    reg := TRegEx.Create('^\d{4}-\d{2}-\d{2}$');
    GetLocaleFormatSettings(GetUserDefaultLCID, localformatSettings);
    GetLocaleFormatSettings(GetUserDefaultLCID, formatSettings);
    formatSettings.DateSeparator := '-';           //24.04.2023 ???
    formatSettings.ShortDateFormat := 'yyyy-mm-dd';
    fValue := Value;
    if not fOldValueSet then  // kann nur zu beginn einmal gesetzt werden
    begin
      fOldValue := Value;
      fOldValueSet := True;
    end;

    //***************************//
    //****** Universalfeld ******//
    //***************************//
    if (Self.AddfieldType in [zfld00_Universal]) then
    begin
      TbfaTextEdit(Self.comp).Text := Value;
      if (fRights = 1) or ((fRights and cAddfieldWrite = cAddfieldWrite) and (TbfaTextEdit(Self.comp).Text <> '')) then
        TbfaTextEdit(Self.comp).Color := cColorNoRightInit;
    end

    //***************************//
    //**** Datum + DLM-Datum ****//
    //***************************//
    else if (Self.AddfieldType in [zfld01_Datum, zfld12_DLMDate]) then
    begin
      if VarIsType(Value, varDate) then
        TcxDateEdit(Self.comp).Date := DateOf(Value)
      else
//30.08.2021 Ticket 28297 Workaround, muss geprft werden! -->
//        TcxDateEdit(Self.comp).Text := Value;
      begin
        var str : string;
        if Pos(' ', Value) > 0 then
          str := Copy(Value, 1, Pos(' ', Value) - 1)
        else
          str := Value;
        if (str <> '') and (StrToDateDef(str, 0, formatSettings) <> 0) then
        begin
          TcxDateEdit(Self.comp).Date := StrToDateDef(str, 0, formatSettings);
          fValue := TcxDateEdit(Self.comp).Text;
        end
        else
          TcxDateEdit(Self.comp).Date := -700000;
      end;
//30.08.2021 Ticket 28297 <--
//      if (fRights = 1) or
//         ((fRights and cAddfieldWrite = cAddfieldWrite) and (TcxDateEdit(Self.comp).Text <> '')) then
//      begin
//        Self.SetColor(Self.comp, cColorNoRightInit);
//        Self.SetColor(Self.comp2, cColorNoRightInit);
//      end;
    end

    //***************************//
    //****** Auswahl + DLM ******//
    //***************************//
    else if (Self.AddfieldType in [zfld02_Auswahl, zfld11_DLM]) then
    begin
      if fOptions and cAddfieldOptStrict = cAddfieldOptStrict then
        TbfaComboBox(Self.comp).ItemIndex := TbfaComboBox(Self.comp).Properties.Items.IndexOf(Value)
      else
        TbfaComboBox(Self.comp).Text := Value;

      if (fRights = 1) or
        ((fRights and cAddfieldWrite = cAddfieldWrite) and (TbfaComboBox(Self.comp).Text <> '')) then
        TbfaComboBox(Self.comp).Color := cColorNoRightInit;
    end

    //***************************//
    //***** Mehrfachauswahl *****//
    //***************************//
    else if (Self.AddfieldType in [zfld03_MehrfachAuswahl]) then
    begin
      TbfaCheckedCombo(Self.comp).Text := Value;
      if (fRights = 1) or
        ((fRights and cAddfieldWrite = cAddfieldWrite) and (TbfaCheckedCombo(Self.comp).Text <> '')) then
        TbfaCheckedCombo(Self.comp).Color := cColorNoRightInit;
      try
        TbfaCheckedListBox(Self.comp2).Items.BeginUpdate;
        TbfaCheckedListBox(Self.comp2).CheckAll(cbUnchecked);

        lstValues := TStringList.Create(' ', ';', [soStrictDelimiter]);
        lstValues.DelimitedText := Value;

        for var i := 0 to TbfaCheckedListBox(Self.comp2).Items.Count -1 do
        begin
          if (lstValues.IndexOf(TbfaCheckedListBox(Self.comp2).Items[i].Text) > -1) {and
                       (TbfaCheckedListBox(Self.comp2).Items[i].Text <> '""')} then
            TbfaCheckedListBox(Self.comp2).Items[i].Checked := True;

          if (TbfaCheckedListBox(Self.comp2).Items[i].Text = dgettext('Shared36', sCaptionOnlySearch)) then
            TbfaCheckedListBox(Self.comp2).Items[i].Enabled := False;
        end;
      finally
        TbfaCheckedListBox(Self.comp2).Items.EndUpdate;
      end;
      FreeAndNil(lstValues);
    end

    //***************************//
    //******** Intervall ********//
    //***************************//
    else if (Self.AddfieldType in [zfld05_Intervall]) then
    begin
      TbfaComboBox(Self.comp2).ItemIndex := -1;

      if (Rightstr(Value, 1) = 'T') then
        TbfaComboBox(Self.comp2).ItemIndex := 0;
      if (Rightstr(Value, 1) = 'W') then
        TbfaComboBox(Self.comp2).ItemIndex := 1;
      if (Rightstr(Value, 1) = 'M') then
        TbfaComboBox(Self.comp2).ItemIndex := 2;
      if (Rightstr(Value, 1) = 'J') then
        TbfaComboBox(Self.comp2).ItemIndex := 3;
      TbfaSpinEdit(Self.comp).Text := LeftStr(Value, Length(Value)-1);

      if (fRights = 1) or
         ((fRights and cAddfieldWrite = cAddfieldWrite) and (TbfaSpinEdit(Self.comp).Text <> '')) then
      begin
        TbfaComboBox(Self.comp2).Color := cColorNoRightInit;
        TbfaSpinEdit(Self.comp).Color := cColorNoRightInit;
      end;
    end

    //*******************************************//
    //**** Ganzzahl + Fliesskomma + Waehrung ****//
    //*******************************************//
    else if (Self.AddfieldType in [zfld06_GanzZahl, zfld07_FliessKomma, zfld08_Waehrung]) then
    begin
      if (Trim(Value) = '') then
      begin
        TbfaSpinEdit(Self.comp).Text := Trim(Value); //27.03.2020 Ticket 21834
        TbfaSpinEdit(Self.comp).Value := Null;
      end else
      begin
        var Float: Extended;
      // SJ 29.11.2021 - FormatFloat, damit kein Modified bei EditValue <> DisplayValue entsteht
      // sonst wird die Farbe durch die OnChange gesetzt, wenn man in das Feld klickt
//        TbfaSpinEdit(Self.comp).Text := Value; //27.03.2020 Ticket 21834
        if TryStrToFloat(Value, Float, formatSettings) then
        begin
          TbfaSpinEdit(Self.comp).Text := FormatFloat(TbfaSpinEdit(Self.comp).Properties.DisplayFormat, Float);
          TbfaSpinEdit(Self.comp).Value := Float;
        end else
        begin
          try
            TbfaSpinEdit(Self.comp).Text := Value;
          except
            TbfaSpinEdit(Self.comp).Text := '';
          end;
          try
            TbfaSpinEdit(Self.comp).Value := Value;
          except
            TbfaSpinEdit(Self.comp).Value := Null;
          end;
        end;
      end;

      if (fRights = 1) or ((fRights and cAddfieldWrite = cAddfieldWrite) and (TbfaSpinEdit(Self.comp).Text <> '')) then
      begin
        TbfaSpinEdit(Self.comp2).Color := cColorNoRightInit;
        TbfaSpinEdit(Self.comp).Color := cColorNoRightInit;
      end
      else if (Self.AddfieldType in [zfld08_Waehrung]) and (TbfaSpinEdit(Self.comp).NegativeCurrencyRed) and
              (TbfaSpinEdit(Self.comp).Value < 0) then
      begin
      // we could not set the color like in other cases.
      // if its a negative zfld08_Waehrung we take a look whats the current StyleController
      // and set it to its similar with red TextColor
        if TbfaSpinEdit(comp).Style.StyleController = ctlGeneral then
          TbfaSpinEdit(comp).Style.StyleController := ctlGeneralTextHighlighted
        else if TbfaSpinEdit(comp).Style.StyleController = ctlModified then
          TbfaSpinEdit(comp).Style.StyleController := ctlModifiedTextHighlighted
        else if TbfaSpinEdit(comp).Style.StyleController = ctlRequired then
          TbfaSpinEdit(comp).Style.StyleController := ctlRequiredTextHighlighted
        else if TbfaSpinEdit(comp).Style.StyleController = ctlDisabled then
          TbfaSpinEdit(comp).Style.StyleController := ctlDisabledTextHighlighted
        else
          TbfaSpinEdit(comp).Style.StyleController := ctlGeneralTextHighlighted;
      end;
    end
    //*******************************************//
    //****           Gridfeld                ****//
    //*******************************************//
    //09.10.2024 SD -->bfacrm:ticket__show__39992
//    else if (Self.AddfieldType in [zfld13_Grid]) and (Value <> '') then
    else if (Self.AddfieldType in [zfld13_Grid]) then
    begin
      if (Value <> '') then
      begin
    //09.10.2024 SD <--bfacrm:ticket__show__39992
        lstKeys := TStringList.Create('"', ';', [soStrictDelimiter]);
        lstRows := TStringList.Create('"', '|', [soStrictDelimiter]);
        lstValues := TStringList.Create('"', ';', [soStrictDelimiter]);
        lstRows.DelimitedText := Value;

        if lstRows.Count > 0 then
        begin
          lstValues.DelimitedText := lstRows[0];
          for var i := 0 to lstValues.Count -1 do
            lstKeys.Add(lstValues[i]);

          var min_rows:= 0;
          if fEditmode then    //im Editmode bleibt wenigstens die erste Zeile stehen
            min_rows := 1;     // weil das die eingefgte Leerzeile ist, in der man eingeben kann

          if lstRows.Count > min_rows then       //erstmal leer setzen
          begin
            with compGrid.View.DataController do
            begin
              BeginUpdate;
              try
                while RecordCount > 0 do
                  DeleteRecord(RecordCount-1);
              finally
                EndUpdate;
              end;
            end;
            while compGrid.DataSet.RecordCount > 0 do   // dann lschen wir den Inhalt
               compGrid.DataSet.Delete;
            compGrid.RecID := 0;
            //04.10.2024 SD -->bfacrm:ticket__show__39992
  //            compGrid.NewHeight := 160;
              compGrid.NewHeight := GridHeightCalculate(lstRows.Count - 1);
            //04.10.2024 SD <--bfacrm:ticket__show__39992
          end;

          compGrid.View.DataController.BeginUpdate; //05.10.2022 Ticket 32469
          try
            var maxvalue := 0;
            for var i := 1 to lstRows.Count -1 do
            begin
              compGrid.DataSet.Insert;
              compGrid.DataSet.Edit;
              compGrid.DataSet.FieldByName('RecID').Value := i;
              compGrid.RecID := i;
              lstValues.Clear;
              lstValues.DelimitedText := lstRows[i];
              for var j := 0 to lstValues.Count -1 do
              begin
                try
                  columnconfig := compGrid.GetColumnConfigByFieldname(lstKeys[j]);
                  if (lstValues[j] <> '') then
                  begin
                    if columnconfig.ValueType in [vltFloat, vltCurrency] then
                    begin
                      // entweder Punkt oder Komma als Trennzeichen
                      if Pos(',', lstValues[j]) > 0 then
                        compGrid.DataSet.FieldByName(lstKeys[j]).Value := StringReplace(lstValues[j], ',', formatSettings.DecimalSeparator, [])
                      else if Pos('.', lstValues[j]) > 0 then
                        compGrid.DataSet.FieldByName(lstKeys[j]).Value := StringReplace(lstValues[j], '.', formatSettings.DecimalSeparator, [])
                      else
                        compGrid.DataSet.FieldByName(lstKeys[j]).Value := lstValues[j];
                    end
                    else if  columnconfig.ValueType = vltDate then
                    begin
                      if TRegEx.IsMatch(lstValues[j], '^\d{4}-\d{2}-\d{2}$') then   //mysql Datumsformat
                      begin
                        if TryStrToDate(lstValues[j], dtvalue, formatSettings) then
                          compGrid.DataSet.FieldByName(lstKeys[j]).Value  := dtvalue
                      end else if TryStrToDate(lstValues[j], dtvalue, localformatSettings) then
                        compGrid.DataSet.FieldByName(lstKeys[j]).Value  := dtvalue
                      else
                        compGrid.DataSet.FieldByName(lstKeys[j]).Value := lstValues[j];
                    end else
                      compGrid.DataSet.FieldByName(lstKeys[j]).Value := lstValues[j];

                    if columnconfig.ColumnType = cltAutoIncrement then
                      maxvalue :=  StrToInt(lstValues[j]);
                  end else
                  begin
                    if columnconfig.DefaultValue <> '' then
                    begin
                      case columnconfig.ValueType of
                        vltString:
                          compGrid.DataSet.FieldByName(lstKeys[j]).Value := columnconfig.DefaultValue;
                        vltInteger:
                           if TryStrToInt64(columnconfig.DefaultValue, ivalue) then
                            compGrid.DataSet.FieldByName(lstKeys[j]).Value := ivalue;
                        vltFloat:
                           if TryStrToFloat(columnconfig.DefaultValue, dvalue) then
                            compGrid.DataSet.FieldByName(lstKeys[j]).Value := dvalue;
                        vltDate:
                           if TryStrToDate(columnconfig.DefaultValue, dtvalue) then
                            compGrid.DataSet.FieldByName(lstKeys[j]).Value := dtvalue;
                        vltCurrency:
                          if TryStrToFloat(columnconfig.DefaultValue, dvalue) then
                            compGrid.DataSet.FieldByName(lstKeys[j]).Value := dvalue;
                      end;
                    end;

                    if columnconfig.ColumnType = cltAutoIncrement then
                    begin
                      inc(maxvalue);
                      compGrid.DataSet.FieldByName(lstKeys[j]).Value := maxvalue;

                    end;
                  end;
                except

                end;
              end;
              compGrid.DataSet.Post;
            end;
    //        compGrid.DataSet.EnableControls;
            if Assigned(compGrid.OnResizeHeight) then
             compGrid.OnResizeHeight(compGrid, compGrid.Grid.Width, compGrid.NewHeight, fIsSettingValue);
          finally
            compGrid.View.DataController.EndUpdate; //05.10.2022 Ticket 32469
            compGrid.View.Invalidate; //05.10.2022 Ticket 32469
          end;

          FreeAndNil(lstkeys);
          FreeAndNil(lstRows);
          FreeAndNil(lstValues);

        end;
      end
      //09.10.2024 SD -->bfacrm:ticket__show__39992
      else
      begin
        compGrid.NewHeight := GridHeightCalculate(0);
        if Assigned(compGrid.OnResizeHeight) then
         compGrid.OnResizeHeight(compGrid, compGrid.Grid.Width, compGrid.NewHeight, fIsSettingValue);
      end;
      //09.10.2024 SD <--bfacrm:ticket__show__39992
    end;
  finally
    fIsSettingValue := False;
  end;
end;

procedure TBfaAddfieldComp.ShowIndexingCombo(Indexing: Boolean);
begin
  fIsIndexing := Indexing;
  if Indexing then
  begin
    if comp is TbfaTextEdit then
    begin
      TbfaTextEdit(comp).Visible := False;
      value := TbfaTextEdit(comp).Text;
    end;

    if comp is TSpTBXComboBox then
    begin
      TSpTBXComboBox(comp).Visible := False;
      value := TSpTBXComboBox(comp).Text;
    end else if comp is TbfaComboBox then
    begin
      TbfaComboBox(comp).Visible := False;
      value := TbfaComboBox(comp).Text;
    end;

    if comp is TbfaCheckedCombo then
    begin
      TbfaCheckedCombo(comp).Visible := False;
      value := TbfaCheckedCombo(comp).Text;
    end;

    if comp is TcxDateEdit then
    begin
      TcxDateEdit(comp).Visible := False;
      value := TcxDateEdit(comp).Text;
    end;

    if comp is TbfaSpinEdit then
    begin
      TbfaSpinEdit(comp).Visible := False;
      value := TbfaSpinEdit(comp).Text;
    end;

    if Assigned(Label2) then
      Label2.Visible := False;

    cbxIndexing.Visible := True;
//bt 29.03.2021 -->
    if (btnIndexing.Images <> nil) then
      btnIndexing.ImageIndex := Integer(icoidx_tag_assistant_enabled)
    else
//bt 29.03.2021 <--
      btnIndexing.Caption := '<';
  end else
  begin
    if comp is TbfaTextEdit then
      TbfaTextEdit(comp).Visible := True;

    if comp is TSpTBXComboBox then
      TSpTBXComboBox(comp).Visible := True
    else if comp is TbfaComboBox then
      TbfaComboBox(comp).Visible := True;

    if comp is TbfaCheckedCombo then
      TbfaCheckedCombo(comp).Visible := True;

    if comp is TcxDateEdit then
      TcxDateEdit(comp).Visible := True;

    if comp is TbfaSpinEdit then
      TbfaSpinEdit(comp).Visible := True;

    cbxIndexing.Visible := False;
//bt 29.03.2021 -->
    if (btnIndexing.Images <> nil) then
      btnIndexing.ImageIndex := Integer(icoidx_tag_assistant_unselected)
    else
//bt 29.03.2021 <--
      btnIndexing.Caption := '>';
    if Assigned(Label2) then
      Label2.Visible := True;
  end;
end;

procedure TBfaAddfieldComp.TokenAdd(Sender: TObject;
  const ATokenText: string; AToken: TdxTokenEditToken; var AAllow: Boolean);
var
  lst: TStringList;
begin
  if (TbfaCheckedListBox(Self.comp2).Items.IndexOf(ATokenText) > -1) then
    TbfaCheckedListBox(Self.comp2).Items[TbfaCheckedListBox(Self.comp2).Items.IndexOf(ATokenText)].State := cbsChecked;
end;

procedure TBfaAddfieldComp.TokenDelete(Sender: TObject;
  const ATokenText: string; AToken: TdxTokenEditToken; var AAllow: Boolean);
var
  lst: TStringList;
begin
  if (TbfaCheckedListBox(Self.comp2).Items.IndexOf(ATokenText) > -1) then
    TbfaCheckedListBox(Self.comp2).Items[TbfaCheckedListBox(Self.comp2).Items.IndexOf(ATokenText)].State := cbsUnchecked;

  if Self.Value <> '' then
  begin
    lst := TStringList.Create(#0, ';', [soStrictDelimiter]);
    lst.DelimitedText := Trim(Self.Value);   // We have to trim, because we use space as a quoting char

    if lst.IndexOf(ATokenText) > -1 then
      lst.Delete(lst.IndexOf(ATokenText));
    Self.Value := lst.DelimitedText;
    FreeAndNil(lst);
  end;

  OnChange(Sender);
end;

procedure TBfaAddfieldComp.TokenDraw(Sender: TObject; ACanvas: TcxCanvas;
  AViewInfo: TdxTokenEditCustomTokenViewInfo; var AHandled: Boolean);
begin
  // SJ 03.03.2021 - we paint invalid Tokens in strict MultiSelect
  if fEditMode and (AddfieldType in [zfld03_MehrfachAuswahl]) and
     ((Options and cAddfieldOptStrict) = cAddfieldOptStrict) then
  begin
    if AViewInfo is TdxTokenEditTokenViewInfo then
    begin
      var ATokenViewInfo: TdxTokenEditTokenViewInfo;
      ATokenViewInfo := TdxTokenEditTokenViewInfo(AViewInfo);
      if ATokenViewInfo.Item = nil then
      begin
        FillRectByColor(ACanvas.Handle, AViewInfo.Bounds, cColorRed);
        FrameRectByColor(ACanvas.Handle, AViewInfo.Bounds, cColorBlack);
        ACanvas.Font.Color := gclFreeText;
      end;
    end;
  end;
end;

{ TBfaStatefieldComp }
constructor TBfaStatefieldComp.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Bitmask := 0;
  Name := '';
  fViewKind := tvkNone;
end;

{ TBfaStatefieldComp }

constructor TBfaStatefieldComp.Create(AOwner: TComponent; Bitmask: Int64; Name: string);
begin
  inherited Create(AOwner);
  Bitmask := Bitmask;
  if Pos('&', Name) > 0 then
    Name := StringReplace(Name,'&', '&&', [rfReplaceAll]);
  Self.Caption := Name;
  Self.Visible := True;
  Self.OnClick := OnClickState;
  fOldValueSet := False;
  fEditmode := False;
  Self.DisplayMode := nbdmImageAndText; //09.09.2020
  Self.FontSettings.Size := bfaFontSettingsSize; //09.09.2020
  Self.Options := [tboNoAutoHint];
end;

procedure TBfaStatefieldComp.OnClickState(Sender: TObject);
begin
  // bfacrm:ticket__show__26373
  // wir prfen explizit nach lesen Recht, deswegen ist (fRights and rFieldRead = rFieldRead) falsch
  if (Self.ViewKind = tvkImporter) and (fRights = rFieldRead) then
    Exit;

  if fValue = fsNone then
    fValue := fsSet
  else if fValue = fsSet then
    fValue := fsUnset
  else if fValue = fsUnset then
    fValue := fsNone;

  if fValue <> fOldValue then
    TSpTBXColorItem(Sender).FontSettings.Style := [fsBold]
  else
    TSpTBXColorItem(Sender).FontSettings.Style := [];

  if fEditMode then
  begin
    if (fRights and rFieldModify = rFieldModify) or ((fRights and rFieldWrite = rFieldWrite) and (fOldValue <> fsSet))  then
      fModified := fValue <> fOldValue;
  end else
    fModified := fValue <> fOldValue;

  case Ord(fValue) of
    0:
      TSpTBXItem(Self).ImageIndex := Integer(icoidx_status_field_unselected);
    1:
      TSpTBXItem(Self).ImageIndex := Integer(icoidx_status_field_enabled);
    2:
      TSpTBXItem(Self).ImageIndex := Integer(icoidx_status_field_disabled);
  end;

  if Assigned(OnValueSet) then
    OnValueSet(fValue, fModified);
end;

procedure TBfaStatefieldComp.Reset;
begin
  fValue := fOldValue;
  Self.FontSettings.Style := [];

  case Ord(fValue) of
    0:
      TSpTBXItem(Self).ImageIndex := Integer(icoidx_status_field_unselected);
    1:
      TSpTBXItem(Self).ImageIndex := Integer(icoidx_status_field_enabled);
    2:
      TSpTBXItem(Self).ImageIndex := Integer(icoidx_status_field_disabled);
  end;
end;

procedure TBfaStatefieldComp.SetModified(const Value: Boolean);
begin
  fModified := Value;
  if fModified then
    Self.FontSettings.Style := [fsBold]
  else
    Self.FontSettings.Style := [];
end;

procedure TBfaStatefieldComp.SetOldValue(const Value: TFieldState);
begin
  if not fOldValueSet then  // kann nur zu beginn einmal gesetzt werden
  begin
    fOldValue := Value;
    fOldValueSet := True;
  end;
end;

procedure TBfaStatefieldComp.SetRights(const Value: Integer);
begin
  fRights := Value;
  if (fRights  = 1) or
     ((fRights and cAddfieldWrite = cAddfieldWrite) and (fValue = fsSet)) then
    Self.FontSettings.Color := clGrayText
  else
    Self.FontSettings.Color := stlControl.TextColor; //31.08.2022 gclFreeText;
end;

procedure TBfaStatefieldComp.SetCompValue(state: TFieldState);
begin
  if not fOldValueSet then  // kann nur zu beginn einmal gesetzt werden
  begin
    fOldValue := state;
    fOldValueSet := True;
  end;
  fValue := state;

  if (fRights  = 1) or
     ((fRights and cAddfieldWrite = cAddfieldWrite) and (fValue = fsSet)) then
    Self.FontSettings.Color := clGrayText
  else
    Self.FontSettings.Color := stlControl.TextColor; //31.08.2022 gclFreeText;

  case Ord(fValue) of
    0:
      TSpTBXItem(Self).ImageIndex := Integer(icoidx_status_field_unselected);
    1:
      TSpTBXItem(Self).ImageIndex := Integer(icoidx_status_field_enabled);
    2:
      TSpTBXItem(Self).ImageIndex := Integer(icoidx_status_field_disabled);
  end;
end;

procedure TBfaStatefieldComp.ClearOldValue;
begin
  fOldValue := fsNone;
  fOldValueSet := False;
end;

procedure TBfaStatefieldComp.ClearValue;
begin
  fValue := fsNone;
  fOldValue := fsNone;
  fOldValueSet := False;
  Modified := False;
  if Self <> nil then
  begin
    Self.FontSettings.Style := [];
    TSpTBXItem(Self).ImageIndex := Integer(icoidx_status_field_unselected); //09.09.2020
  end;
end;

procedure TBfaStatefieldComp.SetValueAsClick(state: TFieldState);
begin
  if (fOldValue = fsNone) then
    fOldValue := fsSet;   // initial kann einmal fOldvalue gesetzt werden
  fValue := state;

  if (fRights and rFieldModify = rFieldModify) or ((fRights and rFieldWrite = rFieldWrite) and (fOldValue <> fsSet)) then
    fModified := fValue <> fOldValue;

  Self.FontSettings.Style := [fsBold];

  case Ord(fValue) of
    0:
      TSpTBXItem(Self).ImageIndex := Integer(icoidx_status_field_unselected);
    1:
      TSpTBXItem(Self).ImageIndex := Integer(icoidx_status_field_enabled);
    2:
      TSpTBXItem(Self).ImageIndex := Integer(icoidx_status_field_disabled);
  end;

  if Assigned(OnValueSet) then
    OnValueSet(fValue, fModified);
end;

procedure TBfaStatefieldComp.SetViewKind(const Value: TbfaViewKind);
begin
  // aktuell nur als Haltepunkt da...
  fViewKind := Value;
end;

procedure TBfaAddfieldComp.OnDateCloseUp(Sender: TObject);
var
  ADateTime: TDateTime;
  I: Integer;
begin
  try
    if fIsSettingValue then
      Exit;
//02.09.2024 SD -->bfacrm:ticket__show__39649
//kein Datum setzen, wenn der Benutzer es nicht ausgewhlt hat
//    if (Sender as TcxDateEdit).Text = '' then
//    begin
//      try
//        with TcxPopupCalendar((Sender as TcxDateEdit).Properties.PopupControl) do
//        begin
//          try
//            ADateTime := ADateTime + Trunc(SelectDate);
//          except
//            ADateTime := Today;
//          end;
//        end;
//        with (Sender as TcxDateEdit) do
//        begin
//          EditValue := ADateTime;
//          PostEditValue;
//        end;
//        fModified := True;
//      except
//        //
//      end;
//    end;
//02.09.2024 SD <--bfacrm:ticket__show__39649

    if Sender = compDateLow then
    begin
//02.09.2024 SD -->bfacrm:ticket__show__39649
//Zustand je nach Wert, Farbe je nach Zustand
//      fModified := True;
//      if not fDontSetColors then
//        SetColor(compDateLow, cColorLightGreen);
      fModified := fValue <> '';
      if not fDontSetColors then
      begin
        if fModified then
          SetColor(compDateLow, cColorLightGreen)
        else
          SetColor(compDateLow, cColorWhite);
      end;
//02.09.2024 SD <--bfacrm:ticket__show__39649
    end;

    if Sender = compDateHigh then
    begin
      fModifiedHigh := fValueHigh <> '';
      if not fDontSetColors then
      begin
        if fModifiedHigh then
          SetColor(compDateHigh, cColorLightGreen)
        else
          SetColor(compDateHigh, (*31.08.2020 Farben Anpassung clWhite*)cColorWhite);
      end;
    end;

    if Assigned(OnModified) then
        OnModified(Self);
    if Assigned(OnValueSet) {and fEditmode} then
      OnValueSet(fValue, fValueHigh, fModified, fModifiedHigh);
    if Assigned(Self.OnComboCloseUp) then
      Self.OnComboCloseUp(Sender);
  except
    //
  end;
end;

procedure TBfaAddfieldComp.OnPropertiesValidate(Sender: TObject;
  var DisplayValue: Variant; var ErrorText: TCaption; var Error: Boolean);
var
  dt: TDateTime;
begin
  if Error then
  begin
    Error := False;
    TcxDateEdit(Sender).Clear;
    TcxDateEdit(Sender).Text := '';
    DisplayValue := TcxDateEdit(Sender).Text;
    ShowBalloonHint(Sender, _(sMsgAllowedCharachters) + fAllowedChar);
  end;
end;

procedure TBfaAddfieldComp.OnDatePopup(Sender: TObject);
begin
  if Assigned(Self.OnComboDropDown) then
    Self.OnComboDropDown(Sender);
end;

procedure TBfaAddfieldComp.OnDoubleClick(Sender: TObject);
begin
  fLastDoubleClick := GetTickCount;
  fSelStart := TbfaTextEdit(Sender).SelStart;
  fSelLength := TbfaTextEdit(Sender).SelLength;
end;

procedure TBfaAddfieldComp.OnGridEditing(Sender: TcxCustomGridTableView; AItem: TcxCustomGridTableItem;
  var AAllow: Boolean);
begin
  AAllow := Self.Editmode;
end;

{ TbfaLabel }

constructor TbfaLabel.Create(AOwner: TComponent);
begin
  inherited;
  Self.ParentColor := False;
//  Self.Transparent := True;
end;

{ TbfaTextEdit }

function TbfaTextEdit.GetColor: TColor;
begin
  Result := Style.Color;
end;

procedure TbfaTextEdit.SetColor(const Value: TColor);
begin
  if ctlGeneral <> nil then
  begin
    if Value = cColorWhite then
      Self.Style.StyleController := ctlGeneral
    else if Value = cColorRequiredField then
      Self.Style.StyleController := ctlRequired
    else if Value = cColorLightGreen then
      Self.Style.StyleController := ctlModified
    else if Value = cColorNoRightInit then
      Self.Style.StyleController := ctlDisabled
    else
      Self.Style.StyleController := ctlGeneral;
  end
  else
    Style.Color := Value;
end;

{ TbfaSpinEdit }

constructor TbfaSpinEdit.Create(AOwner: TComponent);
begin
  inherited;

  Self.BeepOnEnter := False;
  Self.Properties.BeepOnError := False;
  Self.Properties.UseNullString := True;
  Self.Properties.Nullstring := '';
// bfacrm:ticket__show__33326
//  Self.Properties.MaxValue := 99999999999;
//  Self.Properties.MinValue := -99999999999;

  // nicht mehr als 15 Zeichen. Ab 16 gibt es Probleme (vermutlich wegen der wissenschaftlichen Schreibweise (zB 1.000E16))
  // dann fhrt ein klick in das Feld dazu, dass der Text auf 0 gesetzt wird.
  // siehe: https://supportcenter.devexpress.com/ticket/details/t1137610/tcxspinedit-scientific-notation-vs-editvalue
  Self.Properties.MaxValue := 999999999999999;
  Self.Properties.MinValue := -999999999999999;

  Self.EditingText := Self.Properties.Nullstring;
  Self.EditValue := Null;

  Self.Properties.OnValidate := OnValidate;
  Self.Properties.OnEditValueChanged := SpinEditOnValueChanged;
end;

function TbfaSpinEdit.IsValidChar(AChar: Char): Boolean;
begin
  if (ActiveProperties.ValueType in [vtFloat, vtInt]) and (Self.Owner <> nil) and
     (Self.Owner is TBfaAddfieldComp)
  then
    Result := inherited and (not (AChar in ['e', 'E', '+']))
  else
    Result := inherited;
end;

procedure TbfaSpinEdit.OnValidate(Sender: TObject; var DisplayValue: Variant;
  var ErrorText: TCaption; var Error: Boolean);
begin
  // allein stehenes Minus beim defocusieren soll gelscht werden
  if (DisplayValue = '-') then
  begin
    DisplayValue := '';
    Error := False;
  end;

  // Die Komponente setzt beim defocusieren automatisch eine 0 ein. Das wollen wir nicht, zwecks Wert lschen
  if (DisplayValue = '0') and (TbfaSpinEdit(Sender).DisplayValue = '') then
  begin
    DisplayValue := '';
    TbfaSpinEdit(Sender).EditValue := Null;
    Error := False;
  end;
end;

procedure TbfaSpinEdit.PasteFromClipboard;
var
  AChar: Char;
  AText: string;
  Result, SelectedAll: Boolean;
begin
  try
    Result := True;
    if not Clipboard.HasFormat(CF_TEXT) then
      Exit;

    AText := Self.Text;
    SelectedAll := (Length(Self.Text) = Self.SelLength);

    if (TComponent(Self).Owner is TbfaAddfieldComp) then
      if (ActiveProperties.ValueType in [vtFloat, vtInt]) then
        for AChar in Clipboard.AsText do
          if (Result = True) then
            Result := (Pos(AChar, TBfaAddfieldComp(Self.Owner).AllowedChar) > 0)
          else
            Exit;
  finally
    if (TComponent(Self).Owner is TbfaAddfieldComp) then
    begin
      if (Result = True) then
      begin
        inherited;

        // ab hier haben wir in Self.Text den von der DevEx genderten Text.
        // wenn der Text sich nicht unterscheidet, dann hat DevEx es nicht zugelassen
        // weil zB das Format nicht gestimmt hat

        if (AText = Self.Text) and (not SelectedAll)  then
          TbfaAddfieldComp(TComponent(Self).Owner).ShowBalloonHint(Self, _('Ungltiges Format'));
      end;

      if (Result = False) then
        TbfaAddfieldComp(TComponent(Self).Owner).ShowBalloonHint(Self, _(sMsgAllowedCharachters) + TBfaAddfieldComp(Self.Owner).AllowedChar);

    end else
      // kein Addfield? Standardverhalten
      inherited;
  end;
end;

function TbfaSpinEdit.GetColor: TColor;
begin
  Result := Style.Color;
end;

procedure TbfaSpinEdit.SetColor(const Value: TColor);
begin
  if ctlGeneral <> nil then
  begin
    if Value = cColorWhite then
      Self.Style.StyleController := ctlGeneral
    else if Value = cColorRequiredField then
      Self.Style.StyleController := ctlRequired
    else if Value = cColorLightGreen then
      Self.Style.StyleController := ctlModified
    else if Value = cColorNoRightInit then
      Self.Style.StyleController := ctlDisabled
    else
      Self.Style.StyleController := ctlGeneral;
  end
  else
    Style.Color := Value;
end;

procedure TbfaSpinEdit.SpinEditOnChange(Sender: TObject);
var
  ASelStart: Integer;
begin
  // OnChange-Erweiterung fr SpinEdits. Wird programmatisch in der TBfaAddfieldComp.OnChange aufgerufen
  // dient der verhinderung der Eingabe von ungltigen Werten
  ASelStart := TbfaSpinEdit(Sender).SelStart;

  // get the value of the SpinEdit and check against Min/Max
  try
    if (TbfaSpinEdit(Sender).EditingValue = Null) or
       (TbfaSpinEdit(Sender).EditingText = TbfaSpinEdit(Sender).Properties.Nullstring) then
      Exit;
  except on e: Exception do
    Exit;
  end;

  if (TbfaSpinEdit(Sender).EditingValue > TbfaSpinEdit(Sender).Properties.MaxValue) then
  begin
    TbfaSpinEdit(Sender).EditingText := TbfaSpinEdit(Sender).Properties.MaxValue.ToString;
    TbfaSpinEdit(Sender).EditValue := TbfaSpinEdit(Sender).Properties.MaxValue;
    TbfaSpinEdit(Sender).SelStart := ASelStart;
  end else if (TbfaSpinEdit(Sender).EditingValue < TbfaSpinEdit(Sender).Properties.MinValue) then
  begin
    TbfaSpinEdit(Sender).EditingText := TbfaSpinEdit(Sender).Properties.MinValue.ToString;
    TbfaSpinEdit(Sender).EditValue := TbfaSpinEdit(Sender).Properties.MinValue;
    TbfaSpinEdit(Sender).SelStart := ASelStart;
  end;
end;

procedure TbfaSpinEdit.SpinEditOnKeyPress(Sender: TObject; var Key: Char;
  isIntervall: Boolean);
var
  DecPos, ASelStart, ASelLength: Integer;
  Component: TBfaAddfieldComp;
begin
  ASelStart := TbfaSpinEdit(Sender).SelStart;
  ASelLength := TbfaSpinEdit(Sender).SelLength;
  DecPos := Pos(formatSettings.DecimalSeparator, TbfaSpinEdit(Sender).EditingText);

  // if the maximum of decimals is reached, discard additional
    if (Key <> #0) and (DecPos > 0) and (CursorPos >= DecPos) and (TbfaSpinEdit(Sender).SelLength = 0) and
       (( Length(TbfaSpinEdit(Sender).EditingText) - DecPos ) >= TBfaSpinEdit(Sender).MaxDecimal) then
    begin
      Case Char(Key) of
        '0'..'9':
          begin
            Key := #0;
              try
                if (TComponent(Sender).Owner is TBfaAddfieldComp) then
                begin
                  Component := TBfaAddfieldComp(TComponent(Sender).Owner);
                  Component.ShowBalloonHint(Sender, _('maximale Nachkommastellen erreicht'));
                end;
              Except
                //
              end;
            Exit;
          end;
      End;
    end;


  // switch negative/positive value
  // Intervall have no negative values, so avoid it if Intervall
  if (not isIntervall) and (Key = '-') then
  begin
    if ((TbfaSpinEdit(Sender).EditingText) = '') then
    begin
      TbfaSpinEdit(Sender).EditingText := '-0';
      TbfaSpinEdit(Sender).SelStart := 1;
      TbfaSpinEdit(Sender).SelLength := 1;
    end
    else if (Pos('-', TbfaSpinEdit(Sender).EditingText) > 0) then
    begin
      // if SelectAll (and minus at first pos), do not switch. only select from pos 1, for overwrite
      if (ASelLength = Length(TbfaSpinEdit(Sender).EditingText)) then
      begin
        TbfaSpinEdit(Sender).SelStart := 1;
        TbfaSpinEdit(Sender).SelLength := Length(TbfaSpinEdit(Sender).EditingText)-1;
      end else
      begin
        TbfaSpinEdit(Sender).EditingText := RightStr(TbfaSpinEdit(Sender).EditingText, Length(TbfaSpinEdit(Sender).EditingText)-1);
        TbfaSpinEdit(Sender).SelStart := ASelStart - 1;

        if (ASelStart = 0) then
          TbfaSpinEdit(Sender).SelLength := ASelLength - 1
        else
          TbfaSpinEdit(Sender).SelLength := ASelLength;
      end;
    end else
    begin
      TbfaSpinEdit(Sender).EditingText := '-' + TbfaSpinEdit(Sender).EditingText;
      TbfaSpinEdit(Sender).SelStart := ASelStart + 1;

      TbfaSpinEdit(Sender).SelLength := ASelLength;
    end;
    Key := #0;
    Exit;
  end;

  // only accept one DecimalSeparator
  if (DecPos > 0) and ((Key = formatSettings.DecimalSeparator) or (Key = formatSettings.ThousandSeparator)) then
  begin
    Key := #0;
    TbfaSpinEdit(Sender).SelStart := DecPos;
    TbfaSpinEdit(Sender).SelLength := 0;
    Exit;
  end;

  // DecimalSeparator stuff (if there is non)
  if ((Key = formatSettings.DecimalSeparator) or (Key = formatSettings.ThousandSeparator)) and (DecPos <= 0) then
  begin
    // no leading DecimalSeparator without zero
    if ((TbfaSpinEdit(Sender).EditingText = '') or (TbfaSpinEdit(Sender).EditingText = '0'))then
    begin
      TbfaSpinEdit(Sender).EditingText := '0' + formatSettings.DecimalSeparator;
      TbfaSpinEdit(Sender).SelStart := Length(TbfaSpinEdit(Sender).EditingText);
      Key := #0;
      Exit;
    end;

    // DecimalSeparator only at valid positions
    if ((Length(TbfaSpinEdit(Sender).EditingText) - ASelStart) > MaxDecimal) then
    begin
      TbfaSpinEdit(Sender).SelStart := (Length(TbfaSpinEdit(Sender).EditingText) - MaxDecimal);
      Key := #0;
      Exit;
    end;
  end;
end;

procedure TbfaSpinEdit.SpinEditOnValueChanged(Sender: TObject);
var
  DecPos, ASelStart: Integer;
  Left, Right: String;
begin
  ASelStart := TbfaSpinEdit(Sender).SelStart;

  // DecimalSeparator?
  DecPos := Pos(formatSettings.DecimalSeparator, TbfaSpinEdit(Sender).EditingText);
  if DecPos >= 1 then
    Left := LeftStr(TbfaSpinEdit(Sender).EditingText, DecPos - 1)
  else
    Left := TbfaSpinEdit(Sender).EditingText;

  ASelStart := ASelStart - (Length(Left) - Length(TrimLeadingZeros(Left)));
  Left := TrimLeadingZeros(Left);

  if (DecPos = 0) then
    Right := ''
  else
    Right := RightStr(TbfaSpinEdit(Sender).EditingText, Length(TbfaSpinEdit(Sender).EditingText) - DecPos);

  if (DecPos > 0) and (Left <> '') then
    Right := formatSettings.DecimalSeparator + Right;

  TbfaSpinEdit(Sender).EditingText := Left + Right;
  TbfaSpinEdit(Sender).SelStart := ASelStart;
end;

function TbfaSpinEdit.TrimLeadingZeros(const S: string): string;
var
  I, L: Integer;
  NegativeNumber: Boolean;
begin
  NegativeNumber := False;
  L := Length(S);

  // if we have a negative number, we will start trimming at position 2
  // and add the minus at the beginning
  if (L > 0) and (S[1] = '-') then
  begin
    I := 2;
    NegativeNumber := True;
  end
  else
    I := 1;

  while (I < L) and (S[I] = '0') do Inc(I);
  if not (NegativeNumber = True) then
    Result := Copy(S, I)
  else
    Result := '-' + Copy(S, I);
end;


{ TbfaComboBox }

procedure TbfaComboBox.SetAutoComplete(const Value: Boolean);
begin
  Properties.IncrementalSearch := Value;
end;

procedure TbfaComboBox.SetAutoDropDown(const Value: Boolean);
begin
  Properties.ImmediateDropDown := Value;
end;

procedure TbfaComboBox.SetColor(const Value: TColor);
begin
  if ctlGeneral <> nil then
  begin
    if Value = cColorWhite then
      Self.Style.StyleController := ctlGeneral
    else if Value = cColorRequiredField then
      Self.Style.StyleController := ctlRequired
    else if Value = cColorLightGreen then
      Self.Style.StyleController := ctlModified
    else if Value = cColorNoRightInit then
      Self.Style.StyleController := ctlDisabled
    else
      Self.Style.StyleController := ctlGeneral;
  end
  else
    Style.Color := Value;
end;

procedure TbfaComboBox.SetItems(const Value: TStrings);
begin
  Properties.Items := Value;
end;

procedure TbfaComboBox.SetStoredItems(const Value: TStringList);
begin
  if Assigned(FStoredItems) then
    FStoredItems.Assign(Value)
  else
    FStoredItems := Value;
end;

constructor TbfaComboBox.Create(AOwner: TComponent);
begin
  inherited;
  SkipChange := False;
//  AutoComplete := False;  //07.10.2024 SD <--bfacrm:ticket__show__40145
  FStoredItems := TStringList.Create;
end;

destructor TbfaComboBox.Destroy;
begin
  FStoredItems.Free;
  inherited;
end;

function TbfaComboBox.GetAutoComplete: Boolean;
begin
  Result := Properties.IncrementalSearch;
end;

function TbfaComboBox.GetAutoDropDown: Boolean;
begin
  Result := Properties.ImmediateDropDown;
end;

function TbfaComboBox.GetColor: TColor;
begin
  Result := Style.Color;
end;

function TbfaComboBox.GetItems: TStrings;
begin
  Result := Properties.Items;
end;

{ TbfaCheckedListBox }

procedure TbfaCheckedListBox.CheckAll(AState: TCheckBoxState);
var
  I: Integer;
  ANewState: TcxCheckBoxState;
begin
  case AState of
    cbUnchecked: ANewState := cbsUnchecked;
    cbChecked: ANewState := cbsChecked;
    cbGrayed: ANewState := cbsGrayed;
  end;

  if Items.Count > 0 then
    for I := 0 to Items.Count -1 do
      Items[I].State := ANewState;
end;

function TbfaCheckedListBox.GetItems: TStrings;
var
  I: Integer;
begin
  try
    Result := TStringList.Create;
    Result.Delimiter := ';';
    Result.StrictDelimiter := True;
    Result.BeginUpdate;
    for I := 0 to Items.Count - 1 do
      Result.Add(TcxCheckListBoxItem(Items[I]).Text);
    Result.EndUpdate;
  except
    //
  end;
end;

procedure TbfaCheckedListBox.WMLButtonDown(var Message: TWMLButtonDown);
var
  tp, tp2: TPoint;
  c, d: integer;
  test, test2: TRect;
begin
  c := Mouse.CursorPos.X;
  d := Mouse.CursorPos.Y;
  tp := TbfaCheckedListBox(Self).ClientToScreen(Point(0,0));
  test2 := TbfaCheckedListBox(Self).ClientToScreen(Rect(0,0,0,0));
  tp2.X := tp.X + Self.Height;
  tp2.Y := tp.Y + Self.Width;
  test.Create(tp, tp2);

  if not PtInRect( test, Mouse.CursorPos ) then
    Self.Visible := False;
  //
end;

procedure TbfaCheckedListBox.WndProc(var Message: TMessage);
begin
  if (InnerCheckListBox <> nil) and (Message.Msg = WM_CANCELMODE) then
    Self.Visible := False
  else
    inherited;
end;

{ TbfaCheckedCombo }

procedure TbfaCheckedCombo.CheckComboCanResize(Sender: TObject; var NewWidth,
  NewHeight: Integer; var Resize: Boolean);
begin
  try
    // neuberechnung der Addfield-Position. siehe Application
    if Assigned(OnResizeHeight) and (Sender is TbfaCheckedCombo) and
       (TbfaCheckedCombo(Sender).Parent <> nil) and
       (TbfaCheckedCombo(Sender).Parent is TBfaAddfieldComp) then
    begin
      if Resize then
        OnResizeHeight(Sender, NewWidth, NewHeight, TBfaAddfieldComp(TbfaCheckedCombo(Sender).Parent).IsSettingValue);
    end;

    // OnCanResize-Event for TokenEdit. Currently not in use
    if Assigned(OnBfaCanResize) then
      OnBfaCanResize(Sender, NewWidth, NewHeight, Resize);
  except
    //
  end;
end;

constructor TbfaCheckedCombo.Create(AOwner: TComponent);
begin
  inherited;
  Self.Properties.EditValueDelimiter := ';';
  Self.SkipSearchByReturn := False;
  Self.OnCanResize := Self.CheckComboCanResize;
end;

function TbfaCheckedCombo.GetColor: TColor;
begin
  Result := Style.Color;
end;

procedure TbfaCheckedCombo.SetColor(const Value: TColor);
begin
  if ctlGeneral <> nil then
  begin
    if Value = cColorWhite then
      Self.Style.StyleController := ctlGeneral
    else if Value = cColorRequiredField then
      Self.Style.StyleController := ctlRequired
    else if Value = cColorLightGreen then
      Self.Style.StyleController := ctlModified
    else if Value = cColorNoRightInit then
      Self.Style.StyleController := ctlDisabled
    else
      Self.Style.StyleController := ctlGeneral;
  end
  else
    Style.Color := Value;
end;


{ TbfaButtonEdit }

constructor TbfaButtonEdit.Create(AOwner: TComponent);
begin
  inherited;
  Self.OnClick := ButtonEditClick;
  Self.OnDblClick := ButtonEditDoubleClick;
end;

procedure TbfaButtonEdit.ButtonEditClick(Sender: TObject);
begin
  try
    if (Sender is TbfaButtonEdit) then
    begin
      if (Double(GetTickCount) < Double(fLastDoubleClick + GetDoubleClickTime * 1.5)) and
         (TbfaButtonEdit(Sender).GetTextLen > TbfaButtonEdit(Sender).SelLength) and
         (TbfaButtonEdit(Sender).SelStart >= fSelStart) and
         (TbfaButtonEdit(Sender).SelStart <= (fSelStart + fSelLength)) then
      begin
        TbfaButtonEdit(Sender).SelectAll;
        fLastDoubleClick := 0;
      end;
    end;
  except
    // nur zur Sicherheit
  end;
end;

procedure TbfaButtonEdit.ButtonEditDoubleClick(Sender: TObject);
begin
  Self.fLastDoubleClick := GetTickCount;
  Self.fSelStart := TbfaButtonEdit(Sender).SelStart;
  Self.fSelLength := TbfaButtonEdit(Sender).SelLength;
end;

{ TbfaGridfield }

procedure TbfaGridfield.bfaOnBeforeInsert(ADatacontroller: TcxCustomDataController);
var
  AColumn: TcxGridColumn;
begin
//  OutputDebugString('bfaOnBeforeInsert');
  //ADatacontroller.RecordCount := ADatacontroller.RecordCount +1;

end;

constructor TbfaGridfield.Create(AOwner: TComponent);
var
  gridcolumn: TcxGridDBColumn;
begin
  inherited;
  DataSource := TDataSource.Create(nil);
  DataSet := TClientDataSet.Create(nil);
  DataSource.DataSet := DataSet;
  DataSet.FieldDefs.Add('RecID', ftInteger, 0);
  RecID := 0;
  Grid := TcxGrid.Create(AOwner);
//  Grid.Parent := AOwner;
  // Creates a Level
  Level := Grid.Levels.Add;
  Level.Name := 'SomeLevelName';
  // Creates a View
  View := Grid.CreateView(TcxGridDBTableView) as TcxGridDBTableView;
  View.Name := 'SomeViewName';
  // ... and binds it to the Level
  Level.GridView := View;
  // Hooks up the View to the data
  View.DataController.DataSource := DataSource;
  View.DataController.Options := [dcoAssignGroupingValues, dcoAssignMasterDetailKeys,
    dcoSaveExpanding, dcoMultiSelectionSyncGroupWithChildren, dcoCaseInsensitive];
  View.RowLayout.DefaultColumnCount := 2;
  View.OptionsView.NoDataToDisplayInfoText := '';
  View.OnCustomDrawColumnHeader := GridCustomDrawColumnHeader;
  View.OnCustomDrawFooterCell := GridCustomDrawFooterCell;
  View.OnCustomDrawCell := GridCustomDrawCell;
  View.OnCustomDrawGroupCell := GridCustomDrawGroupCell;
  View.OnColumnPosChanged := GridColumnChanged;
  View.OnColumnSizeChanged := GridColumnChanged;
  Grid.LookAndFeel.ScrollbarMode := sbmClassic;
  Grid.LookAndFeel.NativeStyle := True;

  gridcolumn := View.CreateColumn;
  gridcolumn.DataBinding.ValueType := 'Integer';
  gridcolumn.DataBinding.FieldName := 'RecID';
  gridcolumn.Caption := 'RecID';
  gridcolumn.Name := 'RecID';
  gridcolumn.Visible := True;
  gridcolumn.SortOrder := soAscending;
  gridcolumn.PropertiesClass := TcxLabelProperties; // ;  TcxCustomEditProperties
  TcxTextEditProperties(gridcolumn.Properties).ReadOnly := True;
  gridcolumn.Options.Focusing := False;
  gridcolumn.Visible := False;
  gridcolumn.Styles.Header := stlColumn; //11.10.2022
  View.DataController.KeyFieldNames := 'RecID';
//  View.DataController.OnBeforeInsert := bfaOnBeforeInsert;
  View.DataController.OnAfterInsert := OnGridAfterInsert;

  PopupStandard := TcxGridPopupMenu.Create(Self);
  PopupStandard.Grid := Grid;
  PopupStandard.UseBuiltInPopupMenus := False;

  FConfig := TbfaGridConfig.Create;   // hier speichern wir die aktuelle Config des Grids

  StyleOdd := TcxStyle.Create(Self);
  StyleOdd.Color := clcxLightGray;
  View.Styles.ContentOdd := StyleOdd;
  View.Styles.UseOddEvenStyles := bFalse;
  View.OptionsSelection.MultiSelect := False;
  NegativeCurrencyRed := False;
  StyleBlack := TcxStyle.Create(Self);
  StyleRed := TcxStyle.Create(Self);
  StyleBlack.TextColor := cColorBlack;
  StyleRed.TextColor := cColorRed;
  StyleFree := TcxStyle.Create(Self);
  StyleFree.Color := gclFree;
  StyleFree.TextColor :=  gclFreeText;
  StyleSelected := TcxStyle.Create(Self);
  StyleSelected.Color := gclSelected;
  StyleSelected.TextColor :=  gclSelectedText;
  FBalloonHint := TBalloonHint.Create(Self);   //05.06.2019 Ticket 19429
  FBalloonHint.Delay := 0;   //05.06.2019 Ticket 19429
  FBalloonHint.HideAfter := 800;  //05.06.2019 Ticket 19429
  FNotAllowChar := ';' + sNotAllowedChar;
  FMsgNotAllowedChar := _(sMsgNotAllowedCharachters) + FNotAllowChar;
  ColumnModified := False;
  VisibleRows := 6;
  NewHeight := 160;
end;

destructor TbfaGridfield.Destroy;
begin
  DataSet.DisableControls;
  try
    DataSet.EmptyDataSet;
  finally
    DataSet.EnableControls
  end;
  DataSet.Close;
  FreeAndNil(DataSet);
  inherited;
end;

procedure TbfaGridfield.GridColumnChanged(Sender: TcxGridTableView; AColumn: TcxGridColumn);
begin
  ColumnModified := True;
end;

procedure TbfaGridfield.GridCustomDrawCell(Sender: TcxCustomGridTableView; ACanvas: TcxCanvas; AViewInfo: TcxGridTableDataCellViewInfo;
  var ADone: Boolean);
begin
  if AViewInfo.GridRecord.RecordIndex = -1 then
    Exit;

  try
    ACanvas.Font.Size := View.Styles.Content.Font.Size;
    ACanvas.Font.Color := StyleFree.TextColor;
    ACanvas.Brush.Color := StyleFree.Color;

    if (View.DataController.DataSource <> nil) and
       (View.DataController.DataSource.DataSet <> nil) and
       (View.Controller.SelectedRecordCount > 0) and
       AViewInfo.GridRecord.Selected
    then begin
      ACanvas.Font.Style := [fsBold];
      ACanvas.Font.Color := StyleSelected.TextColor;
      ACanvas.Brush.Color := StyleSelected.Color;
    end else
    begin
      ACanvas.Font.Style := [];
      ACanvas.Font.Color := StyleFree.TextColor;
      ACanvas.Brush.Color := StyleFree.Color;
    end;

    if (AViewInfo.Properties is TcxCurrencyEditProperties) and
      (AViewInfo.Value < 0)
    then
      ACanvas.Font.Color := StyleRed.TextColor; //15.04.2021 clRed;

    if FConfig.AlternateRows and
        Odd(AViewInfo.GridRecord.Index) then
    begin
      if (ACanvas.Brush.Color = gclUrgent) then
        ACanvas.Brush.Color := ColorAdjustLuma(colortorgb(ACanvas.Brush.Color), -30, false)
      else if (ACanvas.Brush.Color = gclWork) then
        ACanvas.Brush.Color := ColorAdjustLuma(colortorgb(ACanvas.Brush.Color), -55, false)
      else
        ACanvas.Brush.Color := ColorAdjustLuma(colortorgb(ACanvas.Brush.Color), -15, false);
    end;
  except
    //
  end;
end;

procedure TbfaGridfield.GridCustomDrawColumnHeader(Sender: TcxGridTableView; ACanvas: TcxCanvas; AViewInfo: TcxGridColumnHeaderViewInfo;
  var ADone: Boolean);
const
  States: array[TcxGridCellState] of TcxButtonState = (cxbsNormal, cxbsHot, cxbsPressed);
var
  AFilterRect, ASortRect, ASortOrderRect, ATextRect: TRect;
  I: Integer;
  AFilterButtonState: TcxButtonState;
  DC: HDC;
  lf: LOGFONT;
  hf: HFONT;
  AText: string;
  AHeaderPainter: GridColumnHeaderPainterAccess;
  AHeaderViewInfo: TcxGridColumnHeaderViewInfoAccess;
begin
  ADone := True;
  //18.10.2022 Ticket 32158 -->
  AHeaderViewInfo := TcxGridColumnHeaderViewInfoAccess(AviewInfo);
//  with TcxGridColumnHeaderViewInfoAccess(AviewInfo) do
//  begin
    ATextRect := AHeaderViewInfo.TextAreaBounds;
    for I := 0 to AHeaderViewInfo.AreaViewInfoCount - 1 do
    begin
      if AHeaderViewInfo.AreaViewInfos[I] is TcxGridColumnHeaderSortingMarkViewInfo then
      begin
        AText := IntToStr(AHeaderViewInfo.Column.SortIndex + 1);
        ASortRect := AHeaderViewInfo.AreaViewInfos[I].Bounds;
        ASortOrderRect := ASortRect;
        ASortOrderRect.Top := ASortOrderRect.Top + Ord(AHeaderViewInfo.IsPressed) - 3;
        ASortOrderRect.Right := ASortRect.Left - 1;
        ASortOrderRect.Left := ASortOrderRect.Right - ACanvas.TextWidth(AText);
        ASortRect.Bottom := ASortRect.Bottom + Ord(AHeaderViewInfo.IsPressed);
      end;
    end;
    if (Self.View.SortedItemCount > 1) and (AHeaderViewInfo.Column.Sortindex > -1) then   //25.10.2022 Ticket 32158
      ATextRect.Right := ASortOrderRect.Left - 2;
    ACanvas.Font.Color := View.Styles.Header.Font.Color;

    AHeaderViewInfo.Params.Color := View.Styles.Header.Color;
    ACanvas.Brush.Color := AHeaderViewInfo.Params.Color;

    cxLookAndFeelPaintersManager.GetPainter(lfsUltraFlat).DrawHeader(ACanvas, AHeaderViewInfo.Bounds, ATextRect,
        AHeaderViewInfo.Neighbors, [], States[AHeaderViewInfo.State], AHeaderViewInfo.Column.HeaderAlignmentHorz,
        AHeaderViewInfo.Column.HeaderGlyphAlignmentVert, False, True,
        AViewInfo.Text, ACanvas.Font, ACanvas.Font.Color, AHeaderViewInfo.Params.Color);

    ACanvas.DrawComplexFrame(AHeaderViewInfo.Bounds, ctlGeneral.Style.BorderColor, ctlGeneral.Style.BorderColor,
      [bRight], 1);
    ACanvas.DrawComplexFrame(AHeaderViewInfo.Bounds, ctlGeneral.Style.BorderColor, ctlGeneral.Style.BorderColor,
      [bBottom], 2);

    cxLookAndFeelPaintersManager.GetPainter(lfsUltraFlat).DrawSortingMark(ACanvas, ASortRect,
      AHeaderViewInfo.Column.SortOrder = soAscending);

    AHeaderPainter := GridColumnHeaderPainterAccess.Create(ACanvas, AViewInfo);
    try
      AHeaderPainter.DrawAreas;
    finally
      AHeaderPainter.Free;
    end;
    ADone := True; //30.09.2022
//  end;
//18.10.2022 <--
end;

procedure TbfaGridfield.GridCustomDrawFooterCell(Sender: TcxGridTableView;
    ACanvas: TcxCanvas; AViewInfo: TcxGridColumnHeaderViewInfo; var ADone:
    Boolean);
var
  ASummaryIndex, AIndex, ADataGroupIndex: Integer;
  AValue: Variant;
begin
  if AViewInfo.Container is TcxGridRowFooterViewInfo then
  begin
    AIndex := TcxGridRowFooterViewInfo(AViewInfo.Container).GridRecord.Index;
    ADataGroupIndex := Sender.DataController.Groups.DataGroupIndexByRowIndex[AIndex];
  end;
  ASummaryIndex := TcxGridFooterCellViewInfo(AViewInfo).SummaryItem.Index;
  if AViewInfo.Container is TcxGridRowFooterViewInfo then
    AValue := Sender.DataController.Summary.GroupSummaryValues[ADataGroupIndex, ASummaryIndex]
  else
    AValue := Sender.DataController.Summary.FooterSummaryValues[ASummaryIndex];
  ACanvas.Brush.Color := View.Styles.Footer.Color;

  ACanvas.FillRect(AViewInfo.Bounds);
  AViewInfo.LookAndFeelPainter.DrawFooterCellBorder(ACanvas, AViewInfo.Bounds);
  ACanvas.Font.Assign(AViewInfo.Params.Font);
  ACanvas.Font.Color := View.Styles.Footer.TextColor;

  if (AValue < 0) and NegativeCurrencyRed then
    ACanvas.Font.Color := StyleRed.Color
  else
    ACanvas.Font.Color := View.Styles.Footer.TextColor;
  ACanvas.Brush.Style := bsClear;
  ACanvas.DrawTexT(AViewInfo.TextForPainting, AViewInfo.TextAreaBounds, DT_RIGHT, True);
  ADone := True;
end;


procedure TbfaGridfield.GridCustomDrawGroupCell(Sender: TcxCustomGridTableView; ACanvas: TcxCanvas;
  AViewInfo: TcxGridTableCellViewInfo; var ADone: Boolean);
var
  AValue: Variant;
  IsSelected, GroupContainsSelected : Boolean;
begin
  if (AViewInfo is TcxGridGroupCellViewInfo) then
  begin
    IsSelected := AViewInfo.GridRecord.Selected;

    GroupContainsSelected :=
      View.DataController.GroupContainsSelectedRows(AViewInfo.GridRecord.Index) and not AViewInfo.GridRecord.Expanded;

    if IsSelected or GroupContainsSelected then
      ACanvas.Brush.Color := StyleSelected.Color
    else
      ACanvas.Brush.Color := StyleFree.Color;

    if ((AViewInfo.GridRecord as TcxGridGroupRow).GroupedColumn.Properties is TcxCurrencyEditProperties) then
    begin
      AValue := (AViewInfo.GridRecord as TcxGridGroupRow).Value;
      if (not VarIsNull(AValue)) and (AValue < 0) and NegativeCurrencyRed then
        ACanvas.Font.Color := StyleRed.Color
      else
        ACanvas.Font.Color := StyleFree.TextColor;
    end
    else
      ACanvas.Font.Color := StyleFree.TextColor;
  end;
end;

function TbfaGridfield.GetFooterSummaryItems: TcxDataFooterSummaryItems;
var
  ASummary: TcxDataSummary;
begin
  ASummary := View.DataController.Summary;
  Result := ASummary.FooterSummaryItems;
end;

function TbfaGridfield.GetGroupSummaryItems: TcxDataGroupSummaryItems;
var
  ASummary: TcxDataSummary;
begin
  ASummary := View.DataController.Summary;
  Result := ASummary.DefaultGroupSummaryItems;
end;

function TbfaGridfield.GetValue(UseColumnNames: Boolean = False; WithReadonly : Boolean = False; OnlySelection: Boolean = False): string;
var
  i, j : Integer;
  Fieldname : string;
  column : TcxGridDBColumn;
  value : string;
  dtvalue: TDate;
  lstValues : TStringList;
  lstRows : TStringList;
  lstKeys: TStringList;
  lstNames : TStringList;
  format : TFormatSettings;
begin
  Result := '';
  format := TFormatSettings.Create;
  format.ShortDateFormat := 'yyyy-MM-dd';

  lstKeys := TStringList.Create(#0, ';', [soStrictDelimiter]);
  lstNames := TStringList.Create(#0, ';', [soStrictDelimiter]);
  lstValues := TStringList.Create(#0, ';', [soStrictDelimiter]);
  lstRows := TStringList.Create(#0, '|', [soStrictDelimiter]);

  for i := 0 to View.ColumnCount -1 do
  begin
    if TcxCustomEditProperties(View.Columns[i].Properties).ReadOnly and not WithReadonly then
      Continue;
    if View.Columns[i].Name = 'RecID' then
      Continue;

    Fieldname := View.Columns[i].Name;
    lstKeys.Add(Fieldname);
    if UseColumnNames then
      lstNames.Add(View.Columns[i].Caption)
  end;

  if UseColumnNames then
    lstRows.Add(lstNames.DelimitedText)
  else
    lstRows.Add(lstKeys.DelimitedText);

  for j := View.DataController.RecordCount -1 downto 0 do
  begin
    lstValues.Clear;
    if OnlySelection and (View.DataController.GetSelectedCount > 0)
      and not View.DataController.IsRowSelected(j) then
        Continue;
    for i := 0 to lstKeys.Count -1 do
    begin
      column := View.GetColumnByFieldName(lstKeys[i]);
      if column = nil then
      begin
        lstValues.Add('');
        continue;
      end;

      if VarIsNull(View.DataController.Values[j, column.Index]) then
        value := ''
      else if VarIsType(View.DataController.Values[j, column.Index], varDate) then
      begin
        dtValue := View.DataController.Values[j, column.Index];
        value := DateToStr(View.DataController.Values[j, column.Index], format);
      end else
        value := View.DataController.Values[j, column.Index];
      lstValues.Add(value);
    end;
    if (lstValues.Count = 1) and (lstValues.DelimitedText = '""') then    // Sonderfall, einspaltiges Grid, kein Wert in der Reihe
      Continue;

    lstRows.Add(lstValues.DelimitedText)
  end;
  if (lstRows.Count > 1) or (OnlySelection and (View.DataController.GetSelectedCount > 0)) then
    Result := lstRows.DelimitedText;
  FreeAndNil(lstKeys);
  FreeAndNil(lstRows);
  FreeAndNil(lstValues);
  FreeAndNIl(lstNames);
end;

function TbfaGridfield.GetColumnConfigByFieldName(fieldname: string): TbfaColumnConfig;
begin
  Result := TbfaColumnConfig.Create;
  for var i := 0 to FConfig.Columns.Count -1 do
  begin
    if FConfig.Columns[i].Fieldname = fieldname then
    begin
      Result.Clone(FConfig.Columns[i]);
      Break;
    end;
  end;
end;

procedure TbfaGridfield.GetContentStyle(Sender: TcxCustomGridTableView; ARecord: TcxCustomGridRecord;
  AItem: TcxCustomGridTableItem; var AStyle: TcxStyle);
begin
  if (ARecord.Values[AItem.Index] < 0) and NegativeCurrencyRed then
    AStyle := StyleRed
  else
    AStyle := View.Styles.Content;
end;

procedure TbfaGridfield.GetExport(var header: string; var values: string; UseColumnNames: Boolean = False);
var
  i, j : Integer;
  Fieldname : string;
  column : TcxGridDBColumn;
  ARow: TcxGridGroupRow;
  value : string;
  val: Variant;
  lstValues : TStringList;
  lstRows : TStringList;
  lstKeys: TStringList;
  lstNames : TStringList;
  format : TFormatSettings;
begin
  format := TFormatSettings.Create;
  format.ShortDateFormat := 'yyyy-MM-dd';
  header := '';
  values := '';
  lstKeys := TStringList.Create(#0, ';', [soStrictDelimiter]);
  lstNames := TStringList.Create(#0, ';', [soStrictDelimiter]);
  lstValues := TStringList.Create(#0, ';', [soStrictDelimiter]);
  lstRows := TStringList.Create(#0, '|', [soStrictDelimiter]);

  // wenn wir nicht im MultiSelectMode etwas selektiert haben und auch nicht gruppiert haben
  // dann holen wir uns die Spaltentitel
  if not (View.OptionsSelection.MultiSelect and (View.DataController.GetSelectedCount > 0))
    and (View.DataController.Groups.ChildCount[-1] = 0) then
  begin
    for i := 0 to View.ColumnCount -1 do
    begin
      if View.Columns[i].Name = 'RecID' then
          Continue;
      if Self.FConfig.UseColumnNames or UseColumnNames then   // wir nehmen die Spaltennamen (Captions) per default
        lstNames.Add(View.Columns[i].Caption)
      else
        lstNames.Add(View.Columns[i].Name); // oder die add- Bezeichner
    end;
    header := lstNames.DelimitedText;
  end;

  for j := 0 to View.ViewData.RowCount -1 do
  begin
    lstValues.Clear;
    if View.OptionsSelection.MultiSelect and (View.DataController.GetSelectedCount > 0)
      and not View.ViewData.Rows[j].Selected then
        Continue;

    if (View.ViewData.Rows[j] is TcxGridGroupRow) then
    begin
      ARow := View.ViewData.Rows[j] as TcxGridGroupRow;
      value := ARow.DisplayText;
      lstValues.Add(value);
    end else
    begin
      for i := 0 to View.ColumnCount -1 do
      begin
        if View.Columns[i].Name = 'RecID' then
          Continue;

        val :=  View.ViewData.Rows[j].Values[i];
        if VarIsType(val, varDate) then
        begin
          value := DateToStr(val, format);
        end else
          value := View.ViewData.Rows[j].DisplayTexts[i];

        lstValues.Add(value);
      end;
    end;
    lstRows.Add(lstValues.DelimitedText);
  end;

  values := lstRows.DelimitedText;
  FreeAndNil(lstKeys);
  FreeAndNil(lstRows);
  FreeAndNil(lstValues);
  FreeAndNil(lstNames);
end;


procedure TbfaGridfield.LoadConfig(AConfig: TbfaGridConfig; SkipFormat: Boolean = False);
var
  i, j, k: Integer;
  gridcolumn: TcxGridDBColumn;
  sumitem: TcxGridTableSummaryItem;
  found : Boolean;
  disp_frmt : string;
begin
  if AConfig = nil then
    Exit;

  disp_frmt := '';
  Config.Reset;
  Config.Clone(AConfig);
  View.Controller.BeginUpdate; //05.10.2022 Ticket 32469
  try
    View.OptionsView.Header := FConfig.ShowHeader;
    View.OptionsView.Footer := FConfig.ShowSummary;
    if View.OptionsView.Footer then
      VisibleRows := VisibleRows -1;
    View.OptionsView.GroupByBox := FConfig.ShowGroupbybox;
    if View.OptionsView.GroupByBox then
      VisibleRows := VisibleRows -2;
    View.OptionsView.Indicator := FConfig.ShowIndicator;
    Self.PopupStandard.UseBuiltInPopupMenus := FConfig.UsePopups;

    View.OptionsCustomize.ColumnSorting := FConfig.ManualSorting;
    // wenn wir sortieren wollen, dann mssen wir auch die mglichkeit haben, wieder in die ursprungsreihenfolge zu sortieren.
    // dafr brauchen wir die RecID Spalte
    if FConfig.AlternateRows then
      View.Styles.UseOddEvenStyles := bDefault
    else
      View.Styles.UseOddEvenStyles := bFalse;
    View.GetColumnByFieldName('RecID').Visible := FConfig.ManualSorting;
    View.OptionsSelection.MultiSelect := FConfig.MultiSelect;

    for i := 0 to FConfig.Columns.Count -1 do
    begin
      found := False;
      disp_frmt := '';
      for j := 0 to View.ColumnCount -1 do      // prfen auf doppelten Namen oder gelschte Felder
      begin
        if (View.Columns[j].Name = TbfaColumnConfig(FConfig.Columns[i]).Name) or
           (LeftStr(TbfaColumnConfig(FConfig.Columns[i]).Fieldname, 4) = 'del_') then
        begin
          found := True;
          Break;
        end;
      end;
      if found then
        Continue;

      gridcolumn := View.CreateColumn;
      with TbfaColumnConfig(FConfig.Columns[i]) do
      begin
        if Name = 'RecID' then  // die Standardspalte RecID kann nicht konfiguriert werden :-)
          Continue;
        if LeftStr(FieldName, 4) = 'del_' then
          Continue;
        if Name = '' then
          Name := 'Spalte_' + IntToStr(View.ColumnCount -1);
        if Fieldname = '' then
          Fieldname := 'FIELD_' + IntToStr(View.ColumnCount -1);
        case ValueType of
          vltString:
          begin
            DataSet.FieldDefs.Add(Fieldname, ftWideString, 48);
            gridcolumn.DataBinding.ValueType := 'String';
          end;
          vltInteger:
          begin
            DataSet.FieldDefs.Add(Fieldname, ftInteger, 0);
            gridcolumn.DataBinding.ValueType := 'Integer';
          end;
          vltFloat:
          begin
            DataSet.FieldDefs.Add(Fieldname, ftFloat, 0);
            gridcolumn.DataBinding.ValueType := 'Float';
          end;
          vltDate:
          begin
            DataSet.FieldDefs.Add(Fieldname, ftDate, 0);
            gridcolumn.DataBinding.ValueType := 'Date';
          end;
          vltCurrency:
          begin
            DataSet.FieldDefs.Add(Fieldname, ftFloat, 0);
            gridcolumn.DataBinding.ValueType := 'Currency';
          end;
        end;

        gridcolumn.Name := Fieldname;
        gridcolumn.Caption := Name;
        gridcolumn.SortOrder := soNone;

        if ColWidth > 0 then
          gridcolumn.Width := ColWidth;

        if Expression <> '' then                          // Expression-Felder haben kein DataBinding, weil sie sonst nicht rechnen ;-)
        begin
          gridcolumn.DataBinding.Expression := Expression;
          ReadOnly := True;     // berechnete Felder sind immer ReadOnly
          gridcolumn.Options.Focusing := False;
//          gridcolumn.GridView.OptionsView.ColumnAutoWidth := True;
        end else
        begin
          gridcolumn.DataBinding.FieldName := Fieldname;  // alle anderen Felder schon...
        end;

        case ColumnType of
          cltEdit:
          begin
            gridcolumn.PropertiesClass := TcxTextEditProperties;
            TcxTextEditProperties(gridcolumn.Properties).ReadOnly := ReadOnly;
            if not TcxTextEditProperties(gridcolumn.Properties).ReadOnly then
            begin
              TcxTextEditProperties(gridcolumn.Properties).OnEditValueChanged := OnGridValueChanged;
              TcxTextEditProperties(gridcolumn.Properties).OnChange := OnGridChanged;
            end;
          end;
          cltDate:
          begin
            gridcolumn.PropertiesClass := TcxDateEditProperties;
            TcxDateEditProperties(gridcolumn.Properties).ReadOnly := ReadOnly;
            if not TcxDateEditProperties(gridcolumn.Properties).ReadOnly then
              TcxDateEditProperties(gridcolumn.Properties).OnEditValueChanged := OnGridValueChanged;
          end;
          cltNumber, cltFloat:
          begin
            gridcolumn.PropertiesClass := TcxSpinEditProperties;
            TcxSpinEditProperties(gridcolumn.Properties).ReadOnly := ReadOnly;
            if not TcxDateEditProperties(gridcolumn.Properties).ReadOnly then
              TcxDateEditProperties(gridcolumn.Properties).OnEditValueChanged := OnGridValueChanged;
            if (DisplayFormat <> '') and not SkipFormat then
            begin
              TcxSpinEditProperties(gridcolumn.Properties).DisplayFormat := DisplayFormat;
              disp_frmt := TcxSpinEditProperties(gridcolumn.Properties).DisplayFormat;
//              TcxSpinEditProperties(gridcolumn.Properties).EditFormat := DisplayFormat;
            end;
          end;
          cltCurrency:
          begin
            gridcolumn.PropertiesClass := TcxCurrencyEditProperties;

            if not SkipFormat  then
            begin
              if DisplayFormat = '' then
                TcxCurrencyEditProperties(gridcolumn.Properties).DisplayFormat := ',0.00 ;-,0.00 '
              else
                TcxCurrencyEditProperties(gridcolumn.Properties).DisplayFormat := DisplayFormat;
              disp_frmt := TcxCurrencyEditProperties(gridcolumn.Properties).DisplayFormat;
            end else
               TcxCurrencyEditProperties(gridcolumn.Properties).DisplayFormat := '';

//            TcxCurrencyEditProperties(gridcolumn.Properties).EditFormat := TcxCurrencyEditProperties(gridcolumn.Properties).DisplayFormat;
            TcxCurrencyEditProperties(gridcolumn.Properties).Alignment := TcxEditAlignment.Create(nil, taRightJustify, taTopJustify);
            gridcolumn.FooterAlignmentHorz := taRightJustify;
            TcxCurrencyEditProperties(gridcolumn.Properties).ReadOnly := ReadOnly;
            if not TcxCurrencyEditProperties(gridcolumn.Properties).ReadOnly then
              TcxCurrencyEditProperties(gridcolumn.Properties).OnEditValueChanged := OnGridValueChanged;
            gridcolumn.Styles.OnGetContentStyle := GetContentStyle;
            TcxCurrencyEditProperties(gridcolumn.Properties).OnChange := OnCurrencyChange;
          end;
          cltList:
          begin
            gridcolumn.PropertiesClass := TcxComboBoxProperties;
            TcxComboBoxProperties(gridcolumn.Properties).Items.AddStrings(ListItems);
            TcxComboBoxProperties(gridcolumn.Properties).DropDownListStyle := lsEditFixedList;
            TcxComboBoxProperties(gridcolumn.Properties).ImmediateDropDownWhenActivated := True;
            TcxComboBoxProperties(gridcolumn.Properties).ImmediateDropDownWhenKeyPressed := True;
            TcxComboBoxProperties(gridcolumn.Properties).PostPopupValueOnTab := True;
            TcxComboBoxProperties(gridcolumn.Properties).ImmediatePost := True;
            if not TcxComboBoxProperties(gridcolumn.Properties).ReadOnly then
              TcxComboBoxProperties(gridcolumn.Properties).OnEditValueChanged := OnGridValueChanged;
            TcxComboBoxProperties(gridcolumn.Properties).OnPopup := OnGridPopup;
            TcxComboBoxProperties(gridcolumn.Properties).OnCloseUp := OnGridCloseup;
          end;
          cltDBList:
          begin
            gridcolumn.PropertiesClass := TcxLookupComboBoxProperties;
            LookupDataSource := TDataSource.Create(nil);
            LookupDataSet := TClientDataSet.Create(nil);
            LookupDataSet.FieldDefs.Add('items', ftWideString, 48);
            // Entweder hier ein TClientDataset von der Anwendung bergeben lassen, oder die ListItems ins DataSet einlesen lassen
            LookupDataSet.CreateDataSet;
            LookupDataSource.DataSet := LookupDataSet;

            TcxLookupComboBoxProperties(gridcolumn.Properties).ListSource := LookupDataSource;
            TcxLookupComboBoxProperties(gridcolumn.Properties).KeyFieldNames := 'items';
            TcxLookupComboBoxProperties(gridcolumn.Properties).ListColumns.Add;
            TcxLookupComboBoxProperties(gridcolumn.Properties).ListColumns[0].FieldName := 'items';
            TcxLookupComboBoxProperties(gridcolumn.Properties).ListColumns[0].Caption := '';
            TcxLookupComboBoxProperties(gridcolumn.Properties).ListOptions.ShowHeader := False;
            TcxLookupComboBoxProperties(gridcolumn.Properties).DropDownListStyle := lsEditFixedList;
            TcxLookupComboBoxProperties(gridcolumn.Properties).ImmediateDropDownWhenActivated := True;
            TcxLookupComboBoxProperties(gridcolumn.Properties).ImmediateDropDownWhenKeyPressed := True;
            TcxLookupComboBoxProperties(gridcolumn.Properties).PostPopupValueOnTab := True;
            TcxLookupComboBoxProperties(gridcolumn.Properties).ImmediatePost := True;
            if not TcxLookupComboBoxProperties(gridcolumn.Properties).ReadOnly then
              TcxLookupComboBoxProperties(gridcolumn.Properties).OnEditValueChanged := OnGridValueChanged;
            TcxLookupComboBoxProperties(gridcolumn.Properties).OnPopup := OnGridPopup;
            TcxLookupComboBoxProperties(gridcolumn.Properties).OnCloseUp := OnGridCloseup;
          end;
          cltAutoIncrement:
          begin
            gridcolumn.PropertiesClass := TcxSpinEditProperties;
            TcxSpinEditProperties(gridcolumn.Properties).ReadOnly := ReadOnly;
            if not TcxDateEditProperties(gridcolumn.Properties).ReadOnly then
              TcxDateEditProperties(gridcolumn.Properties).OnEditValueChanged := OnGridValueChanged;
          end;
        end;

        if SummaryType <> smtNone then
        begin
          with TcxGridTableSummaryItem(FooterSummaryItems.Add) do
          try
            try
              BeginUpdate;
              Column := TcxGridDBColumn(gridcolumn);
              if disp_frmt <> '' then
                Format := disp_frmt;
              Kind := TcxSummaryKind(SummaryType);
            finally
              EndUpdate;
            end;
          except
            FooterSummaryItems.Items[FooterSummaryItems.Count-1].Free;
          end;
          with TcxGridTableSummaryItem(GroupSummaryItems.Add) do
          try
            try
              BeginUpdate;
              Column := TcxGridDBColumn(gridcolumn);
              if disp_frmt <> '' then
                Format := disp_frmt;
              Kind := TcxSummaryKind(SummaryType);
            finally
              EndUpdate;
            end;
          except
            GroupSummaryItems.Items[GroupSummaryItems.Count-1].Free;
          end;
        end;
      end;
      gridcolumn.Styles.Header := stlColumn; //11.10.2022
    end;
    DataSet.CreateDataSet;
    DataSource.DataSet := DataSet;
    DataSet.Insert;
  finally
    View.Controller.EndUpdate; //05.10.2022 Ticket 32469
  end;
end;

procedure TbfaGridfield.OnCurrencyChange(Sender: TObject);
begin
  OutputDebugString('OnCurrencyChange');
  if (TcxCurrencyEdit(Sender).Value < 0) and NegativeCurrencyRed then
    TcxCurrencyEdit(Sender).Style.TextColor := StyleRed.TextColor
  else
    TcxCurrencyEdit(Sender).Style.TextColor := View.Styles.Content.TextColor
end;

procedure TbfaGridfield.OnGridAfterInsert(ADatacontroller: TcxCustomDataController);
var
  column : TcxGridDBColumn;
  i, j, maxvalue : Integer;
  Fieldname: string;
  dvalue: Double;
  dtvalue: TDateTime;
  ivalue: Int64;
begin
  View.Controller.BeginUpdate; //05.10.2022 Ticket 32469
  try
    Inc(RecID);
    for i := 0 to FConfig.Columns.Count -1 do
    begin
      if LeftStr(FConfig.Columns[i].Fieldname, 4) = 'del_' then
          Continue;

      if FConfig.Columns[i].DefaultValue <> '' then
      begin
        DataSet.Edit;
        Fieldname := FConfig.Columns[i].Fieldname;
        case FConfig.Columns[i].ValueType of
          vltString:
            DataSet.FieldByName(Fieldname).Value := FConfig.Columns[i].DefaultValue;
          vltInteger:
             if TryStrToInt64(FConfig.Columns[i].DefaultValue, ivalue) then
              DataSet.FieldByName(Fieldname).Value := ivalue;
             //DataSet.FieldByName(Fieldname).Value := StrToInt64(FConfig.Columns[i].DefaultValue);
          vltFloat:
            if TryStrToFloat(FConfig.Columns[i].DefaultValue, dvalue) then
              DataSet.FieldByName(Fieldname).Value := dvalue;
             //DataSet.FieldByName(Fieldname).Value := StrToFloat(FConfig.Columns[i].DefaultValue);
          vltDate:
            if TryStrToDate(FConfig.Columns[i].DefaultValue, dtvalue) then
              DataSet.FieldByName(Fieldname).Value := dtvalue;
              //DataSet.FieldByName(Fieldname).Value := StrToDate(FConfig.Columns[i].DefaultValue);
          vltCurrency:
             if TryStrToFloat(FConfig.Columns[i].DefaultValue, dvalue) then
               DataSet.FieldByName(Fieldname).Value := dvalue;

               //DataSet.FieldByName(Fieldname).Value := StrToFloat(FConfig.Columns[i].DefaultValue);

        end;
        DataSet.Post;
      end;

      if FConfig.Columns[i].ColumnType = cltAutoIncrement then
      begin
        Fieldname := FConfig.Columns[i].Fieldname;
        maxvalue := 0;
        column := View.GetColumnByFieldName(FConfig.Columns[i].Fieldname);
        if column <> nil then
        begin
          for j := 0 to View.DataController.RecordCount -1 do
          begin
            if View.DataController.Values[j, column.Index] > maxvalue then
              maxvalue := View.DataController.Values[j, column.Index];
          end;
          inc(maxvalue);
          DataSet.Edit;
          DataSet.FieldByName(Fieldname).Value := maxvalue;
          DataSet.Post;
        end;
      end;
    end;

    DataSet.Edit;
    DataSet.FieldByName('RecID').Value := RecID;
    DataSet.Post;

    var minrows: Integer := 6;
    if View.OptionsView.Footer then
      minrows := minrows -1;
    if View.OptionsView.GroupByBox then
      minrows := minrows -2;

    if View.DataController.RowCount > minrows then
    begin
      NewHeight := NewHeight +22;
      inc(VisibleRows);
    end;

    // finde das nchste beschreibare Feld
    for i := 0 to View.ColumnCount -1 do
      if not TcxCustomEditProperties(View.Columns[i].Properties).ReadOnly then
        break;
    if i < View.ColumnCount then
      View.Columns[i].Focused := True;
    View.Controller.EditingController.ShowEdit;
  finally
    View.Controller.EndUpdate; //05.10.2022 Ticket 32469
  end;
end;

procedure TbfaGridfield.OnGridChanged(Sender: TObject);
var
  fValue : string;
  CursorPos : Integer;
begin
//  OutputDebugString('OnGridValueChanged');

  if Sender is TcxTextEdit then
  begin
    fValue := ParseNotAllowedChars(TcxTextEdit(Sender).Text, FNotAllowChar);
    if (fValue <> TcxTextEdit(Sender).Text) and
       (fValue <> '') then
    begin
      CursorPos := TcxTextEdit(Sender).SelStart - (Length(TcxTextEdit(Sender).Text) - Length(fValue));
      //21.02.2024 bfacrm:ticket__show__37555 TcxTextEdit(Sender).Text := fValue;
      FBalloonHint.Description := FMsgNotAllowedChar;
      FBalloonHint.ShowHint(TWinControl(Sender));
      TcxTextEdit(Sender).Text := fValue; //21.02.2024 bfacrm:ticket__show__37555
      TcxTextEdit(Sender).SelStart := CursorPos;
    end
    // SJ: Paste von ungltigen Zeichen wrde zum leeren des Zusatzfeldes fhren
    // verhlt sich beim KeyPress so, dass der Key gelscht wird, deswegen hier auch
    else if (fValue <> TcxTextEdit(Sender).Text) and
            (fValue = '') then
    begin
      SendMessage(TWinControl(Sender).Handle, WM_UNDO, 0, 0);
      //21.02.2024 bfacrm:ticket__show__37555 -->
      FBalloonHint.Description := FMsgNotAllowedChar;
      FBalloonHint.ShowHint(TWinControl(Sender));
      //21.02.2024 bfacrm:ticket__show__37555 <--
      TcxTextEdit(Sender).Text := '';
      //21.02.2024 bfacrm:ticket__show__37555 FBalloonHint.ShowHint(TWinControl(Sender));
    end
  end;
end;

procedure TbfaGridfield.OnGridCloseup(Sender: TObject);
begin
  if Assigned(OnGridComboCloseup) then
    OnGridComboCloseup(Sender);
end;

procedure TbfaGridfield.OnGridPopup(Sender: TObject);
begin
  if Assigned(OnGridComboDropDown) then
    OnGridComboDropDown(Sender);
end;

procedure TbfaGridfield.OnGridValueChanged(Sender: TObject);
var
  fValue : string;
  CursorPos : Integer;
begin
//  OutputDebugString('OnGridValueChanged');

  try
    TcxCustomEdit(Sender).PostEditValue;
  except

  end;
  if View.OptionsView.Footer and (FooterSummaryItems.Count > 0) then
    View.DataController.Summary.Recalculate;
  if Assigned(OnModified) then
    OnModified(True);

  if Assigned(OnValueSet) then
    OnValueSet(GetValue, null, True, False);

//  View.ApplyBestFit;
end;

procedure TbfaGridfield.ResetColumns;
var
  gridcolumn: TcxGridDBColumn;
begin
  View.Controller.BeginUpdate; //05.10.2022 Ticket 32469
  try
    if DataSet.Active then
    begin
       DataSet.DisableControls;
       try
         DataSet.EmptyDataSet;
       finally
         DataSet.EnableControls;
       end;
    end;

    DataSet.Fields.Clear;
    DataSet.FieldDefs.Clear;

    View.ClearItems;

    DataSet.FieldDefs.Add('RecID', ftInteger, 0);
    RecID := 0;

    gridcolumn := View.CreateColumn;
    gridcolumn.DataBinding.ValueType := 'Integer';
    gridcolumn.DataBinding.FieldName := 'RecID';
    gridcolumn.Caption := 'RecID';
    gridcolumn.Name := 'RecID';
    gridcolumn.Visible := False;
    gridcolumn.SortOrder := soAscending;
    gridcolumn.PropertiesClass := TcxLabelProperties; // ;  TcxCustomEditProperties
    TcxTextEditProperties(gridcolumn.Properties).ReadOnly := True;
    gridcolumn.Options.Focusing := False;
    gridcolumn.Visible := False;
    gridcolumn.Styles.Header := stlColumn; //11.10.2022
    View.DataController.KeyFieldNames := 'RecID';
    DataSet.Close;
  finally
    View.Controller.EndUpdate; //05.10.2022 Ticket 32469
  end;
end;

procedure TbfaGridfield.ResetGridHeight;
begin
  VisibleRows := 6;
  if View.OptionsView.Footer then
    VisibleRows := VisibleRows -1;
  if View.OptionsView.GroupByBox then
    VisibleRows := VisibleRows -2;


    //  TbfaAddfieldcomp(Self.Owner).Height := 204;
//  Grid.Height := 160;
  NewHeight := 160;
end;

end.
