#!/usr/bin/env python
# -*- coding: utf-8 -*-

from bfa_version import *

import argparse
import codecs
import datetime
import glob
import hashlib
import logging
import os
import re
import shutil
import socket
import sys
import time
import copy
import decimal
import uuid
import subprocess
from collections import OrderedDict
from logging.handlers import RotatingFileHandler
import base64

os.environ['SQLALCHEMY_SILENCE_UBER_WARNING'] = '1'

import sqlalchemy as sqla
import warnings
#from urllib import quote as urlquote

import bitfarm_configobj
import bitfarm_con
import bitfarm_xml
import bitfarm_doc
import bitfarm_proc
import bitfarm_oauth
import ctypes
import multiprocessing

import dpapi
import oauth2
import msal
import json

multiprocessing.freeze_support()

__version__ = version
__author__ = 'F.Schneider, T.Hassler'
__copyright = 'bitfarm Informationssystem GmbH, 57072 Siegen'
__license__ = 'GPL v2'

warnings.filterwarnings(action='ignore', module='sqlalchemy')

ipv4_pattern = "^(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\\.(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\\.(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\\.(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$"
ipv6_pattern = "^(([0-9a-fA-F]{1,4}:){7,7}[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,7}:|([0-9a-fA-F]{1,4}:){1,6}:[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,5}(:[0-9a-fA-F]{1,4}){1,2}|([0-9a-fA-F]{1,4}:){1,4}(:[0-9a-fA-F]{1,4}){1,3}|([0-9a-fA-F]{1,4}:){1,3}(:[0-9a-fA-F]{1,4}){1,4}|([0-9a-fA-F]{1,4}:){1,2}(:[0-9a-fA-F]{1,4}){1,5}|[0-9a-fA-F]{1,4}:((:[0-9a-fA-F]{1,4}){1,6})|:((:[0-9a-fA-F]{1,4}){1,7}|:)|fe80:(:[0-9a-fA-F]{0,4}){0,4}%[0-9a-zA-Z]{1,}|::(ffff(:0{1,4}){0,1}:){0,1}((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])|([0-9a-fA-F]{1,4}:){1,4}:((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9]))$"
date_pattern = "^\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}$"

RC_CODES = {0: "Alle Vorgänge durchgeführt.",
            1: "Unbekannter Fehler.",
            2: "Unbekanntes oder fehlendes Argument.",
            3: "Melden Sie sich an eine bestehende Datenbank an, oder erstellen Sie eine neue bitfarm Datenbank.",
            4: "Es konnte sich nicht am Datenbankserver angemeldet werden, User oder Password falsch",
            5: "Konnte Skript %s nicht finden/lesen",
            6: "Wenn eine Datenbank angelegt oder geupdated werden soll, muss mindestens ein Root-User und Passwort angegeben werden.",
            7: "Es konnte kein Connection String aufgebaut werden",
            8: "noch keine Unterstützung für Datenbank Version 40",
            9: "Es kann nicht auf eine niedrigere Version migriert werden",
            10: "Es ist keine Umgebungsvariable %s gesetzt",
            11: "Das Storageverzeichnis %s existiert schon und ist nicht leer",
            12: "Konnte Verzeichnis %s nicht anlegen, ggf besteht keine Berechtigung",
            13: "Es konnte sich nicht an die Datenbank angemeldet werden, Profil=%s",
            14: "Es konnte keine Datenbank-Engine erstellt werden, Profil=%s",
            15: "Wenn eine Datenbank geclont werden soll, muss ein Quellprofile mittels -profile angegeben werden.",
            16: "Eine Datenbank anlegen kann nicht mit einer der Funktionen only_... Funktionen kombiniert werden.",
            17: "Für eine neue Lizenzdatei muss ein Quellprofile mittels --profile angegeben werden.",
            18: "Melden Sie sich an einem Profil an, um diese Funktion auszuführen",
            19: "Es können nur Templates der Version 35 übersetzt werden",
            20: "Es können nur Vertragsmanagementdateien der Version 35 übersetzt werden",
            21: "Achtung, unterschiedliche Stammverzeichnisse für RS und NRS wird nicht unterstützt",
            22: "Achtung, Storage-Systemvariable %s (%s) verweist nicht auf Archivablage-Stammverzeichnis %s",
            23: "Wenn eine Migration durchgeführt werden soll, muss ein Quellprofile mittels -profile angegeben werden.",
            26: "Zum Synchronisieren von Dokumenten muss ein Quellprofile mittels --profile angegeben werden.",
            27: "Zum Synchronisieren von Dokumenten muss ein Zielprofile mittels --migrate_to angegeben werden.",
            28: "Es können nur Jobdateien der Version 35 übersetzt werden",
            29: "Achtung, es existiert schon eine Profildatei %s. Bitte vergeben Sie einen eindeutigen Namen für das Profil.",
            30: "Konnte Datenbank nicht anlegen",
            31: "Archivtabelle von %s konnte nicht abgefragt werden",
            32: "Konnte SA User %s nicht anlegen",
            33: "Konnte SU User %s nicht anlegen",
            34: "Felder zusammenfassen geht nur ab Version 3.6",
            35: "Fehler beim Erstellen einer doxidx Tabelle",
            36: "Fehler beim Erstellen einer add_%i Spalte in der %s Tabelle",
            37: "Fehler beim Erstellen eines Zusatzfeldes %s",
            38: "Für eine Update muss ein Profil mittels --profile angegeben werden.",
            41: "Konnte Datenbank %s nicht festlegen",
            42: "Es kann nicht in das Quellprofil migriert werden.",
            43: "import_file %s konnte nicht gefunden werden",
            44: "sync_file %s konnte nicht gefunden werden",
            45: "Achtung! Quelldatenbank ist nicht aktuell. Bitte führen Sie eine Datenbankupdate mit dem Parameter -up (--update) durch.",
            46: "Fehler im Format des Parameter -ia %s",
            48: "Nicht erlaubtes Datumsformat für --reload_sysmgr (entwerder tt.mm.yyyy oder yyyy-mm-tt)",
            50: "Konnte Charset utf8mb4 nicht festlegen",
            51: "Bei einer Migration ist der -da Schalter inaktiv. Dieser wird für Importe verwendet (-id, -if, -ia)",
            52: "Fehler bei der Abfrage der sys_arc",
            53: "Keine Archivtabellen in der sys_arc gefunden",
            54: "Zum Ändern des Rootpasswortes muss mindestens ein Root-User und Passwort angegeben werden.",
            55: "Zum Ändern des Passwortes müssen Sie an einem Profil angemeldet sein",
            56: "Zum Importieren von Dokumenten muss ein Zielprofile mittels --migrate_to angegeben werden.",
            57: "Die Passwörter stimmen nicht überein",
            58: "Es konnte sich nicht an die Datenbank angemeldet werden - SSL Fehler 2026 oder 3159  -- require_secure_transport=ON"}

def sys_exit(rc, *args):
    if rc in RC_CODES:
        if rc == 0:
            log.info(RC_CODES[rc])
        else:
            log.error(RC_CODES[rc] % args)
    sys.exit(rc)

def get_input(prompt = '', password=False):
    if prompt:
        sys.stdout.write(prompt)
        sys.stdout.flush()
    inp = ''
    while True:
        x = msvcrt.getwch()
        # x = x.decode('cp850')
        if x == '\r':   # return
            break
        if x == '\t':   # tab überspringen
            continue
        if ord(x) == 0 or ord(x) == 224:    # Funktionstatsen überspringen
            x = msvcrt.getwch()
            continue
        if ord(x) == 8:
            if len(inp) > 0:  # Backspace
                inp = inp[:-1]
                sys.stdout.write(x)
                sys.stdout.write(' ')
                sys.stdout.write(x)
                sys.stdout.flush()
                x = ''
            else:
                continue
        else:
            if password:
                sys.stdout.write('*')
            else:
                sys.stdout.write(x)
        sys.stdout.flush()
        inp += x

    sys.stdout.write('\n')
    sys.stdout.flush()
    return inp


import msvcrt
import getpass
import tracemalloc, json
import gc

#@streamlit.experimental_singleton
def init_tracking_object():
  tracemalloc.start(10)

  return {
    "runs": 0,
    "tracebacks": {}
  }

_TRACES = init_tracking_object()


def traceback_exclude_filter(patterns, tracebackList):
    """
    Returns False if any provided pattern exists in the filename of the traceback,
    Returns True otherwise.
    """
    for t in tracebackList:
        for p in patterns:
            if p in t.filename:
                return False
        return True


def traceback_include_filter(patterns, tracebackList):
    """
    Returns True if any provided pattern exists in the filename of the traceback,
    Returns False otherwise.
    """
    for t in tracebackList:
        for p in patterns:
            if p in t.filename:
                return True
    return False


def check_for_leaks(diff):
    """
    Checks if the same traceback appears consistently after multiple runs.

    diff - The object returned by tracemalloc#snapshot.compare_to
    """
    tracefn.write("Top Stats:")
    for d in diff[:10]:
        tracefn.write(str(d) + '\n')
    tracefn.write('\n')

    _TRACES["runs"] = _TRACES["runs"] + 1
    tracebacks = set()

    for sd in diff:
        for t in sd.traceback:
            tracebacks.add(t)

    if "tracebacks" not in _TRACES or len(_TRACES["tracebacks"]) == 0:
        for t in tracebacks:
            _TRACES["tracebacks"][t] = 1
    else:
        oldTracebacks = _TRACES["tracebacks"].keys()
        intersection = tracebacks.intersection(oldTracebacks)
        evictions = set()
        for t in _TRACES["tracebacks"]:
            if t not in intersection:
                evictions.add(t)
            else:
                _TRACES["tracebacks"][t] = _TRACES["tracebacks"][t] + 1

        for t in evictions:
            del _TRACES["tracebacks"][t]

    if _TRACES["runs"] > 1:
        tracefn.write(f'After {_TRACES["runs"]} runs the following traces were collected.\n')
        prettyPrint = {}
        # for t in _TRACES["tracebacks"]:
        #     prettyPrint[str(t)] = _TRACES["tracebacks"][t]
        # tracefn.write(json.dumps(prettyPrint, sort_keys=True, indent=4))
        for t in _TRACES["tracebacks"]:
            tracefn.write(str(t) + '\n')
        tracefn.write('\n')
        tracefn.flush()

def compare_snapshots():
    """
    Compares two consecutive snapshots and tracks if the same traceback can be found
    in the diff. If a traceback consistently appears during runs, it's a good indicator
    for a memory leak.
    """
    snapshot = tracemalloc.take_snapshot()
    if "snapshot" in _TRACES:
        diff = snapshot.compare_to(_TRACES["snapshot"], "lineno")
        diff = [d for d in diff if d.count_diff > 0 and 'tracemalloc' not in d.traceback._frames[0][0]]
        check_for_leaks(diff)

    _TRACES["snapshot"] = snapshot


class Password(argparse.Action):
    def __call__(self, parser, namespace, values, option_string):
        if values is None:
            values = getpass.getpass()
        setattr(namespace, self.dest, values)


def sha256sum(fname, blocksize=65536):
    hash256 = hashlib.sha256()
    with open(fname, "rb") as f:
        for block in iter(lambda: f.read(blocksize), ""):
            if block == b'':
                break
            hash256.update(block)
    return hash256.hexdigest()


def get_basepath(fname):
    ret = fname
    if fname[:2] == '\\\\':     # UNC
        if fname.find('\\', 2) > -1:
            ret = fname[:fname.find('\\', 2)]
    else:
        if fname.find('\\') > -1:
            ret = fname[:fname.find('\\')]
    return ret

def remove_braces(string):
    string = string.strip()
    prefix = ""
    if string[0] != '(' and string.find('(') > 0:   # wir haben als erstes Zeichen keine Klammer aber mindestens ab dem 2ten Zeichen?
        prefix = string[:string.find('(')]
        string = string[string.find('('):]
        string = string.strip()
    found_or = False
    while string[0] == '(' and string[-1] == ')' and not found_or:
        newstring = string[1:-1]  # neuen string erzeugen und auf plausibilität prüfen
        newstring = newstring.strip()
        braces = 0
        pos = 0
        for char in newstring:
            # if newstring[pos:pos +4].lower() == ' or ':
            #     found_or = True
            if char == "(":
                braces += 1
            if char == ")":
                braces -= 1
                if braces < 0:  # mehr geschlossen als geöffnet? unsymmetrisch...
                    break
            if braces == 0 and found_or:   # wir sind innerhalb des strings, es sind keine klammern offen, haben aber ein or gefunden?
                break                      # dann dürfen wir die ggf außenstehenden klammern nicht entfernen

            pos += 1
        if braces == 0 and not found_or:  # alles symmetrisch
            string = newstring.strip()
        else:
            break  # nein? raus hier

    return prefix + string


def split_sqlwhere(string):
    res = []

    word = ""
    braces = 0
    ind = -1
    while ind < len(string) - 1:
        ind += 1

        if string[ind:ind + 5].lower() == " and " and braces == 0:
            if word != "":
                res.append(word)
            res.append('and')
            word = ""
            ind += 4
            continue

        if string[ind:ind + 4].lower() == " or " and braces == 0:
            if word != "":
                res.append(word)
            res.append('or')
            word = ""
            ind += 3
            continue

        if string[ind:ind + 5].lower() == " not " and braces == 0:      # wir separieren erstmal die nots
            if word != "":
                res.append(word)
            res.append('not')
            word = ""
            ind += 4
            continue

        if (ind == 0 and string[ind:ind + 4].lower() == "not ") and braces == 0:        # wir separieren erstmal die nots
            if word != "":
                res.append(word)
            res.append('not')
            word = ""
            ind += 3
            continue

        if string[ind] == "(":
            if string[ind + 1] == ')':   # klammern ()
                word += string[ind]
                continue

            if ind > 1 and string[ind - 1] != ' ' and string[ind - 1] != '(':   # klammern innerhalb von strings nicht beachten
                word += string[ind]
                continue

            braces += 1
            if string[ind + 1] == ' ' or string[ind + 1] == '(':
                res.append('(')
            if braces == 1:     # erste Klammer geöffnet
                continue

        if string[ind] == ")":
            if ind > 1 and string[ind - 1] == '(':   # klammern innerhalb von strings nicht beachten, bzw geschlossene Klammern ()
                word += string[ind]
                continue

            if ind < len(string) -1 and string[ind + 1] == ' ' and braces == 0:
                word += string[ind]
                continue

            braces -= 1
            if ind > 1 and (string[ind -1] == ' ' or string[ind -1] == ')'):
                if word != "":
                    res.append(word)
                word = ''
                res.append(')')
            if braces == 0:     # erste klammer wieder geschlossen
                continue

        word += string[ind]

    if word != "":
        res.append(word)

    for i in range(len(res)):
        if res[i][0] == '(' and res[i][-1] == ')':  #geklammerten ausdruck gefunden, rekursion!
            res[i] = split_sqlwhere(res[i])

    # haben wir ein not vor einem einfachen ausdruck, dann fassen wir den zusammen
    for i in range(len(res)):
        if res[i] == 'not':
            if type(res[i+1]) is not list:
                res[i] = None
                res[i+1] = 'not %s' % res[i+1]
            elif type(res[i+1]) is list and len(res[i+1]) == 1:
                res[i] = None
                res[i+1] = 'not %s' % res[i+1][0]

    return res

def split_sqlwhere36(string):
    res = []
    items = re.split('(and)|(or)(not)|(\( )|( \))', string)
    res = list(filter(lambda x: x not in ['', ' ', None], items))

    # haben wir ein not vor einem einfachen ausdruck, dann fassen wir den zusammen
    for i in range(len(res)):
        res[i] = res[i].strip()
        if res[i] == 'not':
            if type(res[i+1]) is not list:
                res[i] = None
                res[i+1] = 'not %s' % res[i+1]
            elif type(res[i+1]) is list and len(res[i+1]) == 1:
                res[i] = None
                res[i+1] = 'not %s' % res[i+1][0]
    res = list(filter(lambda x: x is not None, res))
    return res

def translate_items(items, version36=False):
    i = 0
    while i < len(items):
        if items[i] is None:
            i += 1
            continue
        if type(items[i]) is list:
            items[i] = translate_items(items[i])
            i += 1
            continue

        if not version36:
            items[i] = remove_braces(items[i])
        log.debug('Verarbeite Element ' + str(items[i]))
        adds = re.findall(r'add_\d+\b', items[i])
        if len(adds) > 0:
            for addfield in adds:
                if type(fav.arcid) is list:
                    arcid = fav.selectedarcid
                elif type(fav.arcid) is str and ',' in fav.arcid:
                    fav.arcid = fav.arcid.split(',')
                    arcid = fav.selectedarcid
                elif fav.arcid is None:
                    arcid = fav.selectedarcid
                else:
                    arcid = fav.arcid

                log.debug('%s %s' % (addfield, arcid))
                if (addfield, arcid) in field_mapping:
                    # jetzt gesamten ausdruck
                    matches = re.findall(r'(%s)\s+(.+?)\s+(.+)' % addfield, items[i])
                    for match in matches:
                        for field in new_fields:
                            if field.add == field_mapping[(addfield, arcid)]:
                                if field_mapping[(addfield, arcid)] is not None:
                                    if (match[1] == '=' or match[1] == 'like') and match[2] == '""':
                                        if items[i][:3].lower() == 'not':
                                            items[i] = 'not %s is NULL' % field_mapping[(addfield, arcid)]
                                        else:
                                            items[i] = '%s is NULL' % field_mapping[(addfield, arcid)]
                                    else:
                                        items[i] = items[i].replace(addfield, field_mapping[(addfield, arcid)])  # und im sqlwhere lassen
                                    if not version36:
                                        if (items[i][:3].lower() == 'not' or (i > 0 and items[i-1].lower() == 'not')) and match[2] != '""' and match[2].lower() != 'null':   # bei einer not suche müssen wir mit or add_n is NULL ergänzen
                                            # items.insert(i + 1, 'or')
                                            # items.insert(i + 2, '%s is NULL' % field_mapping[(addfield, fav.arcid)])

                                            items[i] = [items[i], 'or', '%s is NULL' % field_mapping[(addfield, arcid)]]
                                break
                else:
                    log.warning("%s, %s nicht im field_mapping gefunden" % (addfield, arcid))

        # bei der migration  wird das feld 'auf wiedervorlage' als zusatzfeld angelegt, aber keinem archiv (nur der wurzel) zugeordnet
        # das feld hat daher die bitmask 1, aber keine position in irgendeinem archiv (außer der wurzel)
        # die doc_state aus den lesezeichen müssen hier den position des ursprünglichen archives zugeordnet werden.
        # (state.rights._rights[0].fieldpos)
        # in v3.5 entsprach das statusfeld der position 1 der bitmaske 2 usw., daher wird hier die bitmaske des lesezeichen
        # gegen 1 << (state.rights._rights[0].fieldpos geprüft.
        # später ist die position unabhängig, der status ergibt sich aus der jeweiligen bitmaske eines statusfeldes

        # doc_state anweisungen können geklammert auftreten, oder ohne klammern in einzelne chunks

        if items[i] is None:
            i += 1
            continue

        if type(items[i]) is list:      # bei einer not like suche kann items zu einer Liste werden (Suchausdruck or Field = NULL)
            i += 1                      # da kann dann auch kein statefield drinne sein
            continue

        stats = re.findall(r'doc_state & (\d+)[\)]? = (\d+)', items[i])
        if len(stats) > 0:
            found_state = False
            for stat0, stat1 in stats:
                # stat0 = int(stats[0][0])
                # stat1 = int(stats[0][1])
                stat0 = int(stat0)
                stat1 = int(stat1)
                new_value = 0

                for state in statefields:
                    if state.rights._rights[0].arcid == fav.arcid:
                        if stat0 & (1 << state.rights._rights[0].fieldpos) > 0:  # Treffer
                            for new_state in new_statefields:
                                if new_state.name.lower() == state.name.lower():  # Feld in den neuen Felder gefunden
                                    log.debug('Statusfeld gefunden %s %s %s %s' % (new_state.name, str(new_state.indexcolumnid), str(new_state.bitmask), str(new_value)))
                                    if stat1 > 0:
                                        new_value = new_state.bitmask
                                    if found_state:
                                        items.insert(i + 1, 'and')
                                        items.insert(i + 2, 'docstate_%i & %i = %i' % (new_state.indexcolumnid, new_state.bitmask, new_value))
                                        i += 2
                                    else:
                                        items[i] = 'docstate_%i & %i = %i' % (new_state.indexcolumnid, new_state.bitmask, new_value)
                                    found_state = True
                                    break
            if not found_state:
                items[i] = 'docstate_0 & %i = %i' % (stat0, stat1)

        i += 1  # while i < len(items)

    return items


def items_to_string(items):
    if type(items) is list:
        for i in range(len(items)):
            if type(items[i]) is list:
                items[i] = items_to_string(items[i])
            else:
                if items[i] != 'and' and items[i] != 'or':
                    items[i] = '%s' % items[i]
        if len(items) == 1:
            items = '%s' % items[0]
        else:
            items = '(%s)' % " ".join(items)
    else:
        items = '%s' % items
    return items


# def get_annotext(con, gdocid):
#     _ret = ""
#     rc = 0
#     if annofunc is not None:
#         try:
#             _ret = annofunc(ctypes.c_wchar_p(con.server), ctypes.c_int(con.port),
#                             ctypes.c_wchar_p(con.db_name), ctypes.c_wchar_p(con.user),
#                             ctypes.c_wchar_p(con.password), ctypes.c_int64(gdocid))
#         except:
#             log.error("Konnte keinen Annotationtext ermitteln für [%s]" % gdocid)
#             rc = annorc()
#
#         if not _ret:
#             rc = annorc()
#
#         if rc != 0:
#             log.error("Fehler in GetAnnoText rc = %d für [%s]" % (rc, gdocid))
#
#     return _ret

def get_annotext(con, gdocid):
    _ret = u""
    rc = 0
    if annofunc:
        try:
            _user = con.bitencrypt(con.user)
            _password = con.bitencrypt(con.password)
            _ret = annofunc(ctypes.c_wchar_p(con.server), ctypes.c_int(con.port), ctypes.c_wchar_p(con.db_name),  ctypes.c_wchar_p(_user), ctypes.c_wchar_p(_password), ctypes.c_int64(gdocid))
        except:
            log.exception('Aufruf fehlgeschlagen: Annotext64.dll')
            rc = annorc()

        if not _ret:
            rc = annorc()

        if rc != 0:
            log.error("Fehler in GetAnnoText rc = %d für [%s]" % (rc, gdocid))
        ctypes._reset_cache()
    return _ret

# def get_annotext(_profile, _gdocid):
#     _ret = ""
#     if annofunc is None:
#         return _ret
#     _args = [annofunc, _profile, str(_gdocid), '-q']
#     try:
#         _ret = subprocess.check_output(_args, universal_newlines=True)
#     except:
#         log.exception('Aufruf fehlgeschlagen: AnnotextCMD')
#     else:
#         return _ret


def create_key(con):
    named = 0
    concurrent = 0

    con.log = log
    ident = con.get_identifier()

    try:
        sql = 'select count(*) from sys_user where flags & 1 = 1 and flags & 194 = 0 and userid <> 1'  # enabled, nicht deleted, nicht dmsadmin, nicht concurrent

        result = con.db.execute(sql).fetchone()
        if result:
            named = result[0]
        sql = 'SELECT count(*) FROM sys_user where flags & 130 = 0 and flags & 65 = 65 and userid <> 1'  # enabled, concurrent, nicht deleted, nicht dmsadmin
        result = con.db.execute(sql).fetchone()
        if result:
            concurrent = result[0]
    except:
        log.exception("Fehler bei der Abfrage der User")
    else:
        ident.append(str(named))
        ident.append(str(concurrent))

        tstamp = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f')
        enc_tstamp = con.bitencrypt(tstamp)
        sql = 'insert into sys_config (name, value) values("created_%s", "%s") on duplicate key update value = "%s"' % (con.profile, enc_tstamp, enc_tstamp)
        try:
            con.db.execute(sql)
        except:
            log.exception("Fehler beim Setzen des Timestamps")
        else:
            ident.append(tstamp)

            # ident = [x.encode('utf8') if type(x) is str else x for x in ident]

            ident = ";".join(ident)
            ident = con.bitencrypt(ident)

            log.info("Serverkey erstellt für %s:" % con.profile)
            log.info("%s" % ident)

            with open('%s.key' % con.profile, 'w') as f:
                f.write(ident)


def smtp_oauth(con):
    refresh_token = None
    provider = None
    selection = None
    print("\n1. Microsoft\n2. Google\n")
    while selection is None:
        selection = input("choose provider: ")
        try:
            selection = int(selection.strip())
            if selection not in range(1, 3):
                print("invalid, retry")
                selection = None
            else:
                if selection == 1:
                    provider = "microsoft"
                else:
                    provider = "google"
        except:
            print("invalid, retry")
            selection = None

    print(provider)
    options = []
    if provider == "google":
        options = {"client_id": None, "client_secret": None}
        scope = ""
    elif provider == "microsoft":
        options = {"tenant_id": None, "client_id": None, "client_secret": None}
    for opt in options:
        options[opt] = input(f"{opt}: ")
    if provider == "google":
        print('\nuse a web browser to open the following url and follow the instructions:')
        url = oauth2.GeneratePermissionUrl(options["client_id"])
        print(url)
        bitfarm_oauth.paste_clipboard(url)
        authorization_code = input('after that enter the code shown on the website: ')

        response = oauth2.AuthorizeTokens(
            options["client_id"], options["client_secret"], authorization_code)

        refresh_token = response['refresh_token']
        # print('Refresh Token: %s' % response['refresh_token'])
        # print('Access Token: %s' % response['access_token'])
        # print('Access Token Expiration Seconds: %s' % response['expires_in'])

    elif provider == "microsoft":
        cfg = {
            "authority": f'https://login.microsoftonline.com/{options["tenant_id"]}',
            "client_id": options["client_id"],
            "scope": ["https://outlook.office.com/.default"],
            "secret": options["client_secret"],
            "tenant-id": options["tenant_id"]
        }

        app = msal.PublicClientApplication(
            cfg["client_id"], authority=cfg["authority"])

        flow = app.initiate_device_flow(scopes=cfg["scope"])
        if "user_code" not in flow:
            raise ValueError(
                "Fail to create device flow. Err: %s" % json.dumps(flow, indent=4))

        # print(flow["message"])
        print(
            f'\nuse a web browser to open the following url and enter the code {flow["user_code"]}')
        print(flow["verification_uri"])
        bitfarm_oauth.paste_clipboard(flow["verification_uri"])
        sys.stdout.flush()

        result = app.acquire_token_by_device_flow(flow)
        if "access_token" in result:
            if "refresh_token" in result:
                refresh_token = result["refresh_token"]
        else:
            print("something went wrong...")
    if refresh_token not in (None, ""):
        if "tenant_id" not in options:
            options["tenant_id"] = ""

        auth_dict = {"client_id": options["client_id"], "client_secret": options["client_secret"],
                     "refresh_token": refresh_token, "tenant_id": options["tenant_id"], "provider": provider}
        crypted_auth = dpapi.cryptData(bytes(json.dumps(auth_dict), "utf8"))

        cnt = 1
        fname = f"smtp_config_{con.profile}.oauth"
        while os.path.isfile(fname):
            fname = f"smtp_config_{con.profile}_{cnt:03d}.oauth"
            cnt += 1
        with open(fname, "wb") as fobj:
            fobj.write(crypted_auth)

        print(f"success! oauth-config {fname} saved")

        confile = os.path.join(bfapath, "%s.con" % con.profile)
        if not os.path.exists(bfapath):
            confile = os.path.join(os.path.dirname(sys.argv[0]), "%s.con" % con.profile)

        try:
            ini = bitfarm_configobj.ConfigObj(infile=confile, encoding="cp1252", write_empty_values=True)
        except:
            log.exception("Das Profil %s konnte nicht gelesen werden" % confile)
        else:
            smtp_port = 587
            smtpserver = ''
            sendername = ini['Scripts']['sendername'] if 'sendername' in ini['Scripts'] else ''
            if 'smtp_oauth' not in ini['Scripts']:
                if provider == "microsoft":
                    smtpserver = 'smtp.office365.com'
            else:
                if 'smtpserver' in ini['Scripts']:
                    smtpserver = ini['Scripts']['smtpserver']
                if 'smtpport' in ini['Scripts']:
                    smtp_port = ini['Scripts']['smtpport']
            input_sendername = input(f'sendername (default {sendername}): ')
            if input_sendername:
                sendername = input_sendername
            input_smtpserver = input(f'smtpserver (default {smtpserver}): ')
            if input_smtpserver:
                smtpserver = input_smtpserver
            input_smtp_port = input(f'smtpport (default {smtp_port}): ')
            if input_smtp_port:
                smtp_port = input_smtp_port

            try:
                ini['Scripts']['sendername'] = sendername
                ini['Scripts']['authuser'] = sendername
                ini['Scripts']['smtpserver'] = smtpserver
                ini['Scripts']['smtpport'] = smtp_port
                ini['Scripts']['usetls'] = 'True'
                ini['Scripts']['smtp_oauth'] = fname

                ini.write()
            except:
                log.exception("Fehler beim Aktualisieren der Con-Datei SMTP [Script]")
    else:
        log.error("Fehler beim Empfang des Access/Refresh Tokens")


def get_field_by_name(name):
    _ret = None
    for field in new_fields:
        if field.name.lower() == name.lower():
            _ret = field
            break
    return _ret


def commandline_arg(bytestring):
    # unicode_string = bytestring.decode(sys.getfilesystemencoding())
    return bytestring


def create_addfield(_con, field):
    # auf vorhandensein überprüfen
    found = False
    for idx in _con.main.sys_docidx.values():
        if field.add in idx.columns.keys():
            found = True
            break
    if not found:
        # Tabelle mit Platz finden
        _idx = None
        _ind = 0
        for idx in _con.main.sys_docidx.values():
            if bitfarm_proc.calc_row_length(idx.columns) + bitfarm_doc.FIELD_SIZES[field.type] < _con.main.max_row_size:
                _idx = idx
                _ind = int(idx.name[11:])
                break
            else:
                _ind += 1
        if _idx is None:  # weitere Tabelle erzeugen
            sql = "CREATE TABLE `sys_docidx_%i` ( \
                  `docidxid` bigint(20) NOT NULL AUTO_INCREMENT, \
                  `gdocid` bigint(20) NOT NULL, \
                  `modified` TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP, \
                  PRIMARY KEY (`docidxid`), \
                  UNIQUE INDEX uq_gdocid (gdocid) \
                ) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;" % _ind
            try:
                _con.main.db.execute(sql)
            except:
                log.exception("Fehler beim Erstellen einer doxidx Tabelle")
                _idx = None
                sys_exit(35)
            else:
                log.debug("Weitere Zusatzfeldtabelle angelegt sys_docidx_%i" % _ind)
                _idx = sqla.Table('sys_docidx_%i' % _ind, _con.main.meta, autoload=True)
                _con.main.sys_docidx[_ind] = _idx

        if _idx is not None:
            # höchsten add_x Index finden (in allen Tabellen)
            if field.add is None:
                add_ind = 0
                for idx in _con.main.sys_docidx.values():
                    for col in idx.columns.keys():
                        add_ind = int(col[4:]) if col[:4] == 'add_' and int(col[4:]) > add_ind else add_ind
                add_ind += 1
                # Spalte anlegen
                sql = "ALTER TABLE `%s` ADD COLUMN `add_%i` %s NULL" % (_idx.name, add_ind, bitfarm_doc.FIELD_TYPES[field.type])
            else:
                sql = "ALTER TABLE `%s` ADD COLUMN `%s` %s NULL" % (_idx.name, field.add, bitfarm_doc.FIELD_TYPES[field.type])
                add_ind = int(field.add[4:])
            # if field.type in [2, 3]:
            #     sql += ', ADD INDEX `idx_add_%i` (`add_%i`(255) ASC)' % (add_ind, add_ind)

            try:
                _con.main.db.execute(sql)
            except:
                log.exception("Fehler beim Erstellen einer add_%i Spalte in der %s Tabelle" % (add_ind, _idx.name))
                _idx = None
                sys_exit(36, add_ind, _idx.name)
            else:
                log.debug("Zusatzfeldspalte angelegt add_%i (%s)" % (add_ind, field.name))
                _idx = sqla.Table(_idx.name, sqla.MetaData(), autoload=True, autoload_with=_con.main.engine)
                _con.main.sys_docidx[_ind] = _idx
                # Zusatzfeld in die sys_addfields schreiben
                field.add = 'add_%i' % add_ind
                field.indextableid = _ind

                sql = "select * from sys_addfields where fieldname = '%s'" % field.add
                try:
                    result = _con.main.db.execute(sql).fetchone()
                except:
                    log.exception("Fehler beim Abfragen eines Zusatzfeldes %s" % field.add)
                    return
                values_query = field.values_query
                if type(values_query) is bitfarm_doc.BitfarmGridConfig:
                    values_query = bitfarm_xml.object_to_xml(values_query)
                vals = {_con.main.sys_addfields.c.displayname: field.name,
                        _con.main.sys_addfields.c.type: field.type,
                        _con.main.sys_addfields.c['items']: ";".join(field.items),
                        _con.main.sys_addfields.c.mode: field.mode,
                        _con.main.sys_addfields.c.options: field.options,
                        _con.main.sys_addfields.c.checkprogram: field.checkprogram,
                        _con.main.sys_addfields.c.values_query: values_query,
                        _con.main.sys_addfields.c.indextableid: field.indextableid,
                        _con.main.sys_addfields.c.flags: field.flags,
                        _con.main.sys_addfields.c.description: field.description}
                if result:
                    sql = sqla.update(_con.main.sys_addfields, values=vals).where(_con.main.sys_addfields.c.fieldname == field.add)
                else:
                    vals[_con.main.sys_addfields.c.fieldname] = field.add
                    sql = sqla.insert(_con.main.sys_addfields, values=vals)
                try:
                    _con.main.db.execute(sql)
                except:
                    log.exception("Fehler beim Erstellen eines Zusatzfeldes %s" % field.add)
                    sys_exit(37, field.add)
                else:
                    log.info("Zusatzfeld angelegt add_%i (%s)" % (add_ind, field.name))
                # Zuordnung und Berechtigung in die sys_addfieldxs schreiben
                # fieldid holen

                sql = "select addfieldid from sys_addfields where fieldname = '%s'" % field.add
                try:
                    result = _con.main.db.execute(sql).fetchone()
                except:
                    log.exception("Fehler beim Abfragen eines Zusatzfeldes %s (fieldid)" % field.add)
                if result:
                    field.id = result[0]

                    for right in field.rights:
                        if right.user is not None:
                            userid = right.user.id
                        elif right.userid is not None:
                            userid = right.userid
                        else:
                            userid = 0
                        if right.group is not None:
                            groupid = right.group.id
                        elif right.groupid is not None:
                            groupid = right.groupid
                        else:
                            groupid = 0

                        sql = 'insert into sys_addfieldxs (arcid, fieldid, userid, groupid, pos, rights) values (%i, %i, %i, %i, %i, %i)' \
                              % (right.arcid, field.id, userid, groupid, right.fieldpos, right.rights)
                        try:
                            _con.main.db.execute(sql)
                        except sqla.exc.IntegrityError:
                            log.warning("Zuordnung für dieses Feld schon vorhanden %i-%i-%i-%i" % (right.arcid, field.id, userid, groupid))
                        except:
                            log.exception("Fehler beim Zuordnen des Zusatzfeldes zu einem Archiv %s" % field.add)
                        else:
                            log.debug("Zusatzfeld %s zu Archiv %s zugeordnet, userid %s groupid %s" % (field.name, right.arcid, userid, groupid))

def deletegdocid35(con, gdocid):
    try:
        con.execute('delete from sys_gdocid where gdoc_id = %i' % gdocid)
    except:
        log.exception("Fehler beim Löschen der gdocid [%i]" % gdocid)

def escapeit(m):
    return '\\' + m.group(0)


if __name__ == "__main__":
    develop = False

    if develop:
        db_logger = logging.basicConfig()
        db_logger = logging.getLogger('sqlalchemy.engine')
        db_logger.setLevel(logging.INFO)

    log = logging.getLogger('Main')

    mypath = os.path.split(os.path.abspath(sys.argv[0]))[0]
    logdir = os.path.join(mypath, 'logs')
    if not os.path.exists(logdir):
        logdir = mypath

    # create console handler and set level to info
    level = 'info'
    levels = {'debug': logging.DEBUG,
              'info': logging.INFO,
              'warning': logging.WARNING,
              'error': logging.ERROR,
              'citical': logging.CRITICAL}
    formatter = logging.Formatter('%(asctime)s - %(name)s-%(levelname)s:   %(message)s')

    while log.root.handlers:
        log.root.removeHandler(log.root.handlers[0])

    #    log.root.removeHandler(log.root.handlers[-1])
    log.propagate = 0
    log.root.setLevel(logging.DEBUG)
    if not log.root.handlers:
        ch = logging.StreamHandler(sys.stdout)

        ch.setLevel(levels[level])
        ch.setFormatter(formatter)
        log.addHandler(ch)

    fh = RotatingFileHandler(os.path.join(logdir, "bfaDBAdmin_%s.log" % (datetime.datetime.now().strftime('%Y-%d-%m %H-%M-%S'))), maxBytes=1024 * 1024 * 100, backupCount=10000, encoding='utf8')
    fh.setFormatter(formatter)
    fh.setLevel(logging.DEBUG)
    log.addHandler(fh)

    if develop:
        db_logger.addHandler(fh)
        if ch is not None:
            db_logger.addHandler(ch)

    annolibpath = os.path.join(mypath, 'AnnoText64.dll')
    log.debug("load %s" % annolibpath)
    if not os.path.exists(annolibpath):
        log.exception('Konnte AnnoText64.dll nicht finden')
        annofunc = None
    else:
        try:
            annolib = ctypes.WinDLL(os.path.join(mypath,'AnnoText64.dll'))
            annofunc = annolib['GetAnnoText']
            annofunc.restype = ctypes.c_wchar_p
        except:
            log.exception('Konnte AnnoText64.dll nicht laden')
            annofunc = None
        else:
            # annofunc = annolib['GetAnnoText']
            # annofunc.restype = ctypes.c_wchar_p
            annorc = annolib['GetLastError']
            annorc.restype = ctypes.c_int
            log.debug("Annotext.dll geladen")

    try:
        bfapath = os.environ["bitfarm-archiv"]
    except:
        log.error('Konnte Systemvariable %bitfarm-archiv% nicht auflösen')
        bfapath = mypath
    #
    #     # os.chdir(self.mypath)
    # annolibpath = os.path.join(bfapath, 'AnnoTextCMD.exe')
    #
    # if not os.path.exists(annolibpath):
    #     log.exception('Konnte AnnoTextCMD.exe nicht laden')
    #     annofunc = None
    # else:
    #     annofunc = annolibpath

    my_con = None
    dp = None
    dc = None
    hostname = socket.getfqdn()
    field_exclude = []
    only_mode = False
    import_mode = False
    tracefn = None

    parser = argparse.ArgumentParser(description="bitfarm Archiv Datenbanktool, zur Erstellung, Konfiguration oder Migration von bitfarm Datenbanken")
    parser.add_argument("-p", "--profile", help="Name des Profils, an dem sich angemeldet werden soll", type=commandline_arg)
    parser.add_argument("-c", "--create_database", help="Name der Datenbank, die erstellt werden soll", type=commandline_arg)
    parser.add_argument("-n", "--name", help="Name des Profils bei -c, falls anders als Datenbankname", type=commandline_arg)
    parser.add_argument("-m", "--migrate_to", help="Name der Zieldatenbank für eine Migration oder Dokumentenimport, es muss sich an einem Profil angemeldet werden als Migrationsquelle", type=commandline_arg)
    parser.add_argument("-cl", "--clone_to", help="Name des Profils der Zieldatenbank, es wird nur die Struktur übertragen, keine Dokumente", type=commandline_arg)
    # parser.add_argument("-cp", "--copy_to", help="Name des Profils der Zieldatenbank, es wird eine Kopie der Datenbank angelegt")
    parser.add_argument("-v", "--version", help="Version der Datenbank bei create_database (default 36)", type=int, choices=[36], default=36)
    parser.add_argument("-srv", "--server", help="Name oder IP des Datenbankservers (default = localhost). Für root Verbindung zur Zieldatenbank", type=commandline_arg, default="localhost")
    parser.add_argument("-prt", "--port", help="Port des Datenbankservers (zB 3306 bei mysql). Für root Verbindung zur Zieldatenbank ", type=int, default=3306)
    parser.add_argument("-gpl", "--gpl", help="Für die Anmeldung an ein GPL Profile (-p)", action="store_true")
    parser.add_argument("-cgpl", "--create_gpl", help="Zum Erstellen oder Migirieren einer GPL Version (-c/-m)", action="store_true")
    parser.add_argument("-e", "--engine", help="Datenbank-Engine (default mysql)", default="mysql")
    parser.add_argument("-enc", "--encoding", help="Datenbank-Encoding (default utf8)", default="utf8")
    parser.add_argument("-su", "--sa_user", help="Name des schreibenden Datenbankusers (default sa)", type=commandline_arg)
    parser.add_argument("-sp", "--sa_pass", help="Passwort des schreibenden Datenbankusers", type=commandline_arg)
    parser.add_argument("-du", "--db_user", help="Name des lesenden Datenbankusers  (default su)", type=commandline_arg)
    parser.add_argument("-dp", "--db_pass", help="Passwort des lesenden Datenbankusers", type=commandline_arg)
    parser.add_argument("-csp", "--change_sa_pass", help="Passwort des schreibenden Datenbankusers ändern (Profil mit -p erforderlich)", action="store_true")
    parser.add_argument("-cdp", "--change_db_pass", help="Passwort des lesenden Datenbankusers ändern (Profil mit -p erforderlich)", action="store_true")
    parser.add_argument("-ru", "--root_user", help="Name des Datenbankusers mit Rootberechtigung (default root) für die Zieldatenbank (zB bei create_database)", type=commandline_arg, default="root")
    parser.add_argument("-rp", "--root_pass", help="Passwort des Datenbankusers mit Rootberechtigung für die Zieldatenbank (zB bei create_database)", type=commandline_arg)
    parser.add_argument("-crp", "--change_root_pass", help="Passwort des Datenbankusers mit Rootberechtigung für die Zieldatenbank ändern", action="store_true")
    parser.add_argument("-cap", "--change_admin_pass", help="Passwort des DMSAdmins ändern (Profil mit -p erforderlich)", action="store_true")
    parser.add_argument("-a", "--admin", help="Name eines oder mehrerer (semikolongetrennt) bitfarm-Benutzer, die Archiv-Admin werden sollen (nur Migration)", type=commandline_arg)
    # parser.add_argument("-ca", "--create_admin",  help="Name eines bitfarm-Benutzer, der Archiv-Admin werden sollen (nur Datenbank erstellen)", type=unicode)
    parser.add_argument("-ap", "--admin_pass",  help="Passwort des DMSAdmin Users (default bitfarmIT, nur Datenbank erstellen)", type=commandline_arg, default="bitfarmIT")
    parser.add_argument("-bsrv", "--bfaserver", help="Name oder IP des bitfarm Servers (default = localhost bzw. Name des DB-Servers)", type=commandline_arg, default="localhost")
    parser.add_argument("-bprt", "--bfaport", help="Port des bitfarm Servers (default = 50001)",  type=int, default=50001)
    parser.add_argument("-rs", "--path_rs", help="Pfad zum RS-Verzeichnis, default = archive_rs", type=commandline_arg, default='archive_rs')
    parser.add_argument("-nrs", "--path_nrs", help="Pfad zum NRS-Verzeichnis, default = archive_nrs", type=commandline_arg, default='archive_nrs')
    parser.add_argument("-stg", "--storage", help="Umgebungsvariable des Storage-Pfades, default bfastore0. Bei der Migration/Import ist damit das Zielverzeichnis gemeint.", default='%bfastore0%', type=commandline_arg)
    parser.add_argument("-od", "--only_docs", help="Es werden bei der Migration nur Dokumente übertragen", action="store_true")
    parser.add_argument("-of", "--only_fields", help="Es werden bei der Migration nur Zusatzfeldinhalte übertragen, vorhandene Werte werden aber nicht überschrieben", action="store_true")
    parser.add_argument("-ob", "--only_bookmarks", help="Es werden bei der Migration nur Lesezeichendateien übertragen", action="store_true")
    parser.add_argument("-or", "--only_resub",  help="Es wird bei der Migration nur die Wiedervorlage übertragen", action="store_true")
    parser.add_argument("-oa", "--only_annos", help="Annotation Texte übertragen (Migration) bzw. erzeugen (Profil)", action="store_true")
    parser.add_argument("-ot", "--only_templates", help="Es werden bei der Migration nur die Templates übersetzt", action="store_true")
    parser.add_argument("-om", "--only_mwl", help="Es werden bei der Migration nur die Vertragsmanagementdateien übersetzt", action="store_true")
    parser.add_argument("-otr", "--only_trigger", help="Es werden bei der Migration nur die Trigger in der Quell-DB erzeugt", action="store_true")
    parser.add_argument("-oj", "--only_jobfiles", help="Es werden vorhandene Job-Files auf das neue Template geändert", action="store_true")
    parser.add_argument("-oc", "--only_comments", help="Es werden bei der Migration nur Kommentare übertragen", action="store_true")
    parser.add_argument("-mf", "--move_files", help="Es werden bei der Migration auch die Dokumentdateien umgezogen", action="store_true")
    parser.add_argument("-iev", "--reload_iev", help="Zusätzlicher Schalter für -oa, es werden noch nicht importierte iev-Dateien nachverarbeitet", action="store_true")
    parser.add_argument("-fr", "--from_gdocid", help="Es werden Dokumente ab dieser Gdocid  migriert (einschliesslich)", type=int, default=0)
    parser.add_argument("-to", "--to_gdocid", help="Es werden Dokumente bis dieser Gdocid  migriert (ausschliesslich)", type=int)
    parser.add_argument("-ps", "--packsize", help="Anzahl der Dokumente die pro Commit geschrieben werden sollen", type=int, default=100)
    parser.add_argument("-gid", "--create_new_gdocid", help="Erstellt eine neue GDocid bei der Migration. Achtung, niemals  verwenden, wenn Dokumente extern verknüpft sind.", action="store_true")
    parser.add_argument("-sup", "--systemuserpass", help="Das Passwort des bitfarm Systemusers", type=commandline_arg, default="1E305D2915E0397E")
    parser.add_argument("-key", "--create_key", help="Erstellt eine neue Profil.key Datei für das angemeldete Profil", action="store_true")
    parser.add_argument("-ex", "--exclude_fields", help="Eine semikolongetrennte Liste von Zusatzfeldnamen, die bei der Migration nicht zusammengefasst werden sollen", type=commandline_arg)
    parser.add_argument("-ma", "--merge_fields", help="Eine semikolongetrennte Liste von Zusatzfeldnamen gleichen Typs, die zusammengefasst werden sollen (ab Version 36). Der Typ kann mit der Schreibweise Feldname:Typ-ID angegeben werden.", type=commandline_arg)
    parser.add_argument("-cf", "--check_fields", help="Es wird eine Übersicht der Zusatzfelder des angemeldeten Profils angezeigt", action="store_true")
    parser.add_argument("-cm", "--create_mapping", help="Es wird eine Mapping Tabelle erzeugt von Zusatzfeldern eines Quellsystems (-p) zu einem Zielsystem (-m)", action="store_true")
    parser.add_argument("-sy", "--sync", help="Synchronisiert Dokumente von der Quell in die Zieldatenbank (Migration). Entweder sys_mgr oder Liste an Gdocids (kommagetrennt) als Parameter", type=commandline_arg)
    parser.add_argument("-syf", "--sync_file", help="Synchronisiert Dokumente von der Quell in die Zieldatenbank (Migration), die Gdocids (kommagetrennt) werden in einer Datei übergeben", type=commandline_arg)
    parser.add_argument("-rsy", "--reload_sysmgr", help="Befüllt die sys_mgr Tabelle in einer 3.5er Datenbank mit Dokumenten, die seit dem angegebenen Datum geändert wurden. Das Datum wird als Parameter übergeben (tt.mm.yyyy oder yyyy-mm-tt)", type=commandline_arg)
    # parser.add_argument("-synosap", "--sync_no_sap", help="Bei der Synchronisation wird die SAP Tabelle übersprungen", action="store_true")
    parser.add_argument("-ch", "--check", help="Profil, gegen das der Datenbestand geprüft wird (wird am Ende einer Migration automatisch durchgeführt)", type=commandline_arg)
    parser.add_argument("-sql", "--create_sql", help="Name des SQL Scripts zur Anlage einer neuen bitfarm-Datenbank (default create_sys_36.sql/create_gpl_36.sql)", type=commandline_arg)
    parser.add_argument("-sc", "--skip_check", help="Überspringt die Datenbestandsüberprüfung am Ende der Migration", action="store_true")
    parser.add_argument("-up", "--update", help="Führt das Update Sript aus", action="store_true")
    parser.add_argument("-ups", "--update_searchengine", help="Führt ein Update der SearchEngine aus", action="store_true")
    parser.add_argument("-uf", "--update_file", help="Name des Updatescriptes, default Update_sys_36.sql (Update_gpl_36.sql in der GPL)", type=commandline_arg)
    parser.add_argument("-se", "--search_engine", help="Name des SearchEngine Scriptes, default SearchEngine_36.sql", default='SearchEngine_36.sql', type=commandline_arg)
    parser.add_argument("-id", "--import_documents", help="Importiert Dokumente, ein Kombination aus -od und -gid. Entweder eine Liste an Gdocids (kommagetrennt) als Parameter oder per Datei mit -if", type=commandline_arg)
    parser.add_argument("-ia", "--import_archive", help="Importiert Dokumente archivweise, es wird eine semikolongetrennte Liste von Archivzuweisungen (Quell-ArchivID:Ziel-ArchivID) übergeben", type=commandline_arg)
    parser.add_argument("-if", "--import_file", help="Importiert Dokumente von der Quell in die Zieldatenbank, die Gdocids (kommagetrennt) werden in einer Datei übergeben", type=commandline_arg)
    parser.add_argument("-sd", "--source_dir", help="Bei der Migration/Import mit move_files aus einem 3.6er System kann hiermit das Quell-Stammverzeichnis angegeben werden (überschreibt bfastore0 des Quellsystems)", type=commandline_arg)
    parser.add_argument("-sd35", "--source_dir35", help="Bei der Migration/Import mit move_files aus einem 3.5er System kann hiermit das Quell-Stammverzeichnis angegeben werden (Pfadersetzung)", type=commandline_arg)
    parser.add_argument("-da", "--dest_arcid", help="Beim Import von Dokumenten (-id oder -if) kann hiermit ein gesondertes Zielarchiv angegeben werden.", type=commandline_arg)
    parser.add_argument("-soa", "--smtp_oauth", help="Hinterlegt OAuth Anmeldedaten für SMTP im angemeldeten Profil", action="store_true")
    parser.add_argument("-m57", "--use_mysql57", help="Anpassung der Updateroutine für MySQL57 Datenbanken", action="store_true")
    parser.add_argument("-ru2", "--root_user_src", help="Name des Datenbankusers mit Rootberechtigung (default root) für die Zieldatenbank (zB bei --create_database)", type=commandline_arg, default="root")
    parser.add_argument("-rp2", "--root_pass_src", help="Passwort des Datenbankusers mit Rootberechtigung für die Zieldatenbank (zB bei --create_database)", type=commandline_arg)
    parser.add_argument("-ulh", "--use_localhost", help="Root-Verbindung zur Quelldatenbank (bei Migration) über localhost (statt Servername aus dem Profil)\nIn der Regel ist die Root-Anmeldung auf localhost beschränkt", action="store_true")
    parser.add_argument("-c2e", "--change_to_enterprise", help="Ändert eine Profildatei in ein Enterprise-Profil", action="store_true")
    parser.add_argument("-c2g", "--change_to_gpl", help="Ändert eine Profildatei in ein GPL-Profil", action="store_true")
    parser.add_argument("-lrc", "--list_returncodes", help="Eine Liste aller möglichen Returncodes und Bedeutung", action="store_true")

    parser.add_argument("-pass", type=str, dest='password', help=argparse.SUPPRESS)     # für automatisierte Aufrufe kann der SA User hier übergeben werden
    parser.add_argument("-user", type=str, dest='user', help=argparse.SUPPRESS)         # ansonsten muss auf der Console sich angemeldet werden
    parser.add_argument("-debug", dest='debug', action="store_true", help=argparse.SUPPRESS)

    args = parser.parse_args()

    log.info("bfaDBAdmin-Tool Version %s" % version)
    params = []
    i = 1
    while i < len(sys.argv):
        if sys.argv[i] in ('-rp', '-rp2', '-pass', '-user'):
            i += 2
            continue

        params.append(sys.argv[i])
        i += 1

    params = " ".join([x for x in params])
    log.info("Parameter %s" % params)

    if args.list_returncodes:
        for k,v in RC_CODES.items():
            sys.stdout.write(f'{k:3d}: {v}\n')
        sys.stdout.flush()
        sys_exit(0)

    # TODO: Das hier muss vor if args.profile and not args.update: wenn man ein update durchführten will gibt man das profil mit -p an, dann soll er auch
    # args.server und args.port aus dem Profil lesen.
    if ((args.create_database or args.update or args.update_searchengine or args.change_sa_pass or args.change_db_pass or args.change_root_pass or args.change_admin_pass)
            and args.root_user and not args.root_pass):
        password = get_input('Bitte geben Sie das Password des Rootusers (%s) for %s:%s ein: ' % (args.root_user, args.server, args.port), True)
        args.root_pass = password

    only_mode = args.only_docs or args.only_bookmarks or args.only_resub or args.only_annos or \
                args.only_templates or args.only_mwl or args.only_jobfiles or args.only_fields or \
                args.only_comments

    notstdout = args.user and args.password

    if args.sync_file:
        args.sync = True

    if args.update_searchengine:
        args.update = True

    if not args.update_file:
        args.update_file = 'update_sys_36.sql' if not args.gpl else 'update_gpl_36.sql'
    if not args.create_sql:
        args.create_sql = 'create_sys_36.sql' if not args.create_gpl else 'create_gpl_36.sql'

    if not (args.profile or args.create_database or args.change_root_pass):
        # log.error("Melden Sie sich an eine bestehende Datenbank an, oder erstellen Sie eine neue bitfarm Datenbank.")
        sys_exit(3)

    if (args.create_database or args.update) and not (args.root_pass and args.root_user):
        # log.error("Wenn eine Datenbank angelegt oder geupdated werden soll, muss mindestens ein Root-User und Passwort angegeben werden.")
        sys_exit(6)

    if args.change_root_pass and not (args.root_pass and args.root_user):
        #log.error("Zum Ändern des Rootpasswortes, muss mindestens ein Root-User und Passwort angegeben werden.")
        sys_exit(54)

    if (args.change_sa_pass or args.change_db_pass or args.change_admin_pass) and not args.profile:
       #log.error("Zum Ändern des Passowrtes müssen Sie an einem Profil angemeldet sein")
       sys_exit(55)

    # TODO: checken, das der Datenbankname überall mit ` Quotes verwendet wird, um Unicode (U+0001 - U+FFFF) verwenden zu können
    # fs: derzeit nicht, daher die eingeschränkte Variante
    if args.create_database:
        if re.search('[^a-zA-Z0-9$_]', args.create_database):
            args.name = args.create_database    # wir übernehmen den gewünschten Namen als Profilname
            pat = re.compile('[^a-zA-Z0-9$_]')
            args.create_database = pat.sub('_', args.create_database)   # und entfernen uas dem Datenbanknamen, die unerlaubten Zeichen
            log.warning("Es wurde nicht erlaubte Zeichen im Datenbanknamen gefunden.")
            log.warning("Der Datenbankname wurde auf %s gesetzt" % args.create_database)

    if args.migrate_to and not args.profile:
        # log.error("Wenn eine Migration durchgeführt werden soll, muss ein Quellprofile mittels -profile angegeben werden.")
        sys_exit(23)

    if args.migrate_to and args.profile == args.migrate_to:
        # log.error("Es kann nicht in das Quellprofil migriert werden.")
        sys_exit(42)

    if args.clone_to and not args.profile:
        # log.error("Wenn eine Datenbank geclont werden soll, muss ein Quellprofile mittels -profile angegeben werden.")
        sys_exit(15)

    if args.create_database and only_mode:
        # log.error("Eine Datenbank anlegen kann nicht mit einer der Funktionen only_... Funktionen kombiniert werden.")
        sys_exit(16)

    if args.sync and not args.profile:
        # log.error("Zum Synchronisieren von Dokumenten muss ein Quellprofile mittels --profile angegeben werden.")
        sys_exit(26)

    if args.sync and not args.migrate_to:
        # log.error("Zum Synchronisieren von Dokumenten muss ein Zielprofile mittels --migrate_to angegeben werden.")
        sys_exit(27)

    if (args.import_documents or args.import_archive) and not args.migrate_to:
        # log.error("Zum Importieren von Dokumenten muss ein Zielprofile mittels --migrate_to angegeben werden.")
        sys_exit(56)

    if args.create_key and not args.profile:
        # log.error("Für eine neue Lizenzdatei muss ein Quellprofile mittels --profile (oder -p) angegeben werden.")
        sys_exit(17)

    if args.update and not args.profile:
        # log.error("Für eine Update muss ein Profil mittels --profile (oder -p) angegeben werden.")
        sys_exit(38)

    if args.create_key and args.create_database:
        args.create_key = False

    if args.check_fields and not args.profile:
        # log.error("Melden Sie sich an einem Profil an, um diese Funktion auszuführen")
        sys_exit(18)

    if args.merge_fields and not args.profile:
        # log.error("Melden Sie sich an einem Profil an, um diese Funktion auszuführen")
        sys_exit(18)

    if args.only_trigger and not args.profile:
        # log.error("Melden Sie sich an einem Profil an, um diese Funktion auszuführen")
        sys_exit(18)

    if args.smtp_oauth and not args.profile:
        # log.error("Melden Sie sich an einem Profil an, um diese Funktion auszuführen")
        sys_exit(18)

    if args.only_docs and args.dest_arcid:
        # log.warning("Bei einer Migration ist der -da Schalter inaktiv. Dieser wird für Importe verwendet (-id, -if, -ia)")
        sys_exit(51)

    if args.reload_sysmgr:
        if not args.profile:
            # log.error("Melden Sie sich an einem Profil, um diese Funktion auszuführen")
            sys_exit(18)

        if not (re.match(r'\d{2}.\d{2}.\d{4}', args.reload_sysmgr) or re.match(r'\d{4}-\d{2}-\d{2}', args.reload_sysmgr)):
            # log.error("Nicht erlaubtes Datumsformat für --reload_sysmgr (entwerder tt.mm.yyyy oder yyyy-mm-tt)")
            sys_exit(48)

        if re.match(r'\d{2}.\d{2}.\d{4}', args.reload_sysmgr):
            args.reload_sysmgr = args.reload_sysmgr[6:10] + '-' + args.reload_sysmgr[3:5] + '-' + args.reload_sysmgr[0:2]

    if args.clone_to:  # wir setzen die Schalter entsprechend für Clonen
        args.migrate_to = args.clone_to
        args.only_docs = False
        args.move_files = False

    if args.admin and not args.migrate_to:
        log.warning("Einen Admin definieren ist nur bei der Migration möglich.")
        args.admin = None

    if args.change_to_enterprise and not args.profile:
        sys_exit(18)

    if args.change_to_gpl and not args.profile:
        sys_exit(18)

    # if args.create_admin and not args.create_database:
    #     log.warning("Einen Admin-User anlegen ist nur beim Erstellen einer Datenbank möglich.")
    #     args.create_admin = None


    # if args.copy_to:
    #     args.migrate_to = args.copy_to

    if args.import_documents:
        import_mode = True
        args.sync = None
        args.only_docs = True
        args.move_files = True
        args.create_new_gdocid = True
        #args.skip_check = True      # bei import von dokumenten bringt es nichts gegen archive zu prüfen
        args.import_documents = args.import_documents.split(",")

    if args.import_archive:
        import_mode = True
        args.sync = None
        args.only_docs = True
        args.move_files = True
        args.create_new_gdocid = True
        args.import_archive = args.import_archive.split(";")
        try:
            arcids = ",".join([x.split(':')[0] for x in args.import_archive])
        except:
            # log.error("Fehler im Format des Parameter -ia %s" % ";".join(args.import_archive))
            sys_exit(46, ";".join(args.import_archive))

    if args.import_file:
        import_mode = True
        args.sync = None
        args.only_docs = True
        args.move_files = True
        args.create_new_gdocid = True
        import_file = ""
        #args.skip_check = True      # bei import von dokumenten bringt es nichts gegen archive zu prüfen
        if os.path.exists(args.import_file):
            with open(args.import_file, 'r') as f:
                import_file = f.read()
            if import_file[-1] == ',':
                import_file = import_file[:-1]
        else:
            # log.error("import_file %s konnte nicht gefunden werden" % args.import_file)
            sys_exit(43, args.import_file)

        args.import_documents = import_file.split(",")

    if args.sync:       # wir setzen die schalter entsprechend fürs syncen
        args.only_docs = True

        if args.sync_file:
            if os.path.exists(args.sync_file):
                with open(args.sync_file, 'r') as f:
                    args.sync = f.read()
                if args.sync[-1] == ',':
                    args.sync = args.sync[:-1]
            else:
                # log.error("sync_file %s konnte nicht gefunden werden" % args.sync_file)
                sys_exit(44, args.sync_file)

        args.sync = args.sync.split(",")

    # Prüfung der bfastore0 Variable nur für Enterprise-Version
    if not args.gpl and not args.create_gpl:
        if args.storage[0] != '%' or args.storage[-1] != '%':
            if args.storage[0] != '%':
                args.storage = '%' + args.storage
            if args.storage[-1] != '%':
                args.storage = args.storage + '%'
            log.warning("Falsches Format der Systemvariable, wurde jetzt auf %s gesetzt" % args.storage)

        if not os.environ.get(args.storage[1:-1]):
            # log.error("Es ist keine Umgebungsvariable %s gesetzt" % args.storage)
            sys_exit(10, args.storage)

    if args.migrate_to:
        args.check = args.migrate_to

    if args.migrate_to and args.move_files:
        if os.path.exists(os.path.expandvars(args.storage)):
            if not args.only_docs:  # wenn nur dokumente übertragen werden sollen, brauchen wir keine prüfung auf leere ablageverzeichnisse
                if os.path.exists(os.path.join(os.path.expandvars(args.storage), args.path_rs)) and len(os.listdir(os.path.join(os.path.expandvars(args.storage), args.path_rs))) > 0:
                    # log.error("Das Storageverzeichnis %s existiert schon und ist nicht leer" % os.path.expandvars(args.storage))
                    sys_exit(11, os.path.join(os.path.expandvars(args.storage), args.path_rs))
                if os.path.exists(os.path.join(os.path.expandvars(args.storage), args.path_nrs)) and len(os.listdir(os.path.join(os.path.expandvars(args.storage), args.path_nrs))) > 0:
                    # log.error("Das Storageverzeichnis %s existiert schon und ist nicht leer" % os.path.expandvars(args.storage))
                    sys_exit(11, os.path.join(os.path.expandvars(args.storage), args.path_nrs))
        else:
            try:
                os.mkdir(os.path.expandvars(args.storage))
                log.info("Verzeichnis angelegt: %s" % os.path.expandvars(args.storage))
            except:
                log.exception("Konnte Verzeichnis %s nicht anlegen, ggf besteht keine Berechtigung" % os.path.expandvars(args.storage))
                sys_exit(12, os.path.expandvars(args.storage))

        if not args.only_docs:  # wenn nur dokumente übertragen werden sollen, müssen keine archivablageverzeichnisse erstellt werden (die sollten vorhanden sein)
            if not os.path.exists(os.path.join(os.path.expandvars(args.storage), args.path_rs)):
                try:
                    os.mkdir(os.path.join(os.path.expandvars(args.storage), args.path_rs))
                    log.info("Verzeichnis angelegt: %s" % os.path.join(os.path.expandvars(args.storage), args.path_rs))
                except:
                    log.exception("Konnte revisionssicheres Verzeichnis %s nicht anlegen, ggf besteht keine Berechtigung" % os.path.join(os.path.expandvars(args.storage), args.path_rs))
                    sys_exit(12, os.path.join(os.path.expandvars(args.storage), args.path_rs))
            if not os.path.exists(os.path.join(os.path.expandvars(args.storage), args.path_nrs)):
                try:
                    os.mkdir(os.path.join(os.path.expandvars(args.storage), args.path_nrs))
                    log.info("Verzeichnis angelegt: %s" % os.path.join(os.path.expandvars(args.storage), args.path_nrs))
                except:
                    log.exception("Konnte nichtrevisionssicheres Verzeichnis %s nicht anlegen, ggf besteht keine Berechtigung" % os.path.join(os.path.expandvars(args.storage), args.path_nrs))
                    sys_exit(12, os.path.join(os.path.expandvars(args.storage), args.path_nrs))

    # es wird nocheinmal der only_mode abgeglichen
    only_mode = args.only_docs or args.only_bookmarks or args.only_resub or args.only_annos or \
                args.only_templates or args.only_mwl or args.only_jobfiles or args.only_fields or \
                args.only_comments

    if args.profile and not args.update:    # beim update oder create brauchen wir eine root-anmeldung, ansonsten wird hier die Verbindung mit dem Profil aufgebaut
        my_con = bitfarm_proc.BitfarmConnections(args.profile, gpl=args.gpl)

        if not args.user:
            # user = get_input('Bitte geben Sie den Datenbankuser des Profils %s ein: ' % args.profile)
            # args.user = user
            args.user = my_con.main.user

        if not args.password:
            password = get_input('Bitte geben Sie das Password des Datenbankusers %s ein: ' % args.user, True)
            args.password = password

        my_con.create_db_engine(user=args.user, password=args.password, echo=False)

        if my_con.main.engine is None:
            # log.error("Es konnte keine Datenbank-Engine erstellt werden, Profil=%s" % args.profile)
            sys_exit(14, args.profile)
        else:
            log.info("Datenbank-Engine erstellt %s %s %s %s" % (args.profile, my_con.main.server, my_con.main.port, my_con.main.db_name))
            my_con.connect_db()

        if my_con.main.db is None:
            # log.error("Es konnte sich nicht an die Datenbank angemeldet werden, Profil=%s" % args.profile)
            sys_exit(13, args.profile)
        else:
            log.info("Datenbankverbindung mit %s hergestellt" % args.profile)

        my_proc = bitfarm_proc.BitfarmDocProcessing(my_con)

        # melden wir uns an einem 35 Profil an, wohlmöglich für eine Migration? Dann sollte die 35er DB auch aktuell sein
        if my_con.main.version == 35:
            if 'sys_cfgxs' not in my_con.main.tables or 'sys_stsxs' not in my_con.main.tables:
                # log.error("Achtung! Quelldatenbank ist nicht aktuell. Bitte führen Sie eine Datenbankupdate mit dem Parameter -up (--update) durch.")
                sys_exit(45)

        if args.create_key:
            create_key(my_con.main)

        if args.smtp_oauth:
            smtp_oauth(my_con.main)

        if args.change_to_enterprise:
            my_con.main.gpl = False
            confile = os.path.join(bfapath, "%s.con" % my_con.main.profile)
            if not os.path.exists(bfapath):
                confile = os.path.join(os.path.dirname(sys.argv[0]), "%s.con" % my_con.main.profile)

            try:
                ini = bitfarm_configobj.ConfigObj(infile=confile, encoding="cp1252", write_empty_values=True)
            except:
                log.exception("Das Profil %s konnte nicht gelesen werden" % confile)
            else:
                ini['Connection']['user'] = my_con.main.bitencrypt(my_con.main.user)
                ini['Connection']['pass'] = my_con.main.bitencrypt(my_con.main.password)
                ini['Connection']['suuser'] = my_con.main.bitencrypt(my_con.main.suuser)
                ini['Connection']['supass'] = my_con.main.bitencrypt(my_con.main.supassword)
                ini['Server']['edition'] = 'enterprise'
                ini.write()
            sys_exit(0)

        if args.change_to_gpl:
            my_con.main.gpl = True
            confile = os.path.join(bfapath, "%s.con" % my_con.main.profile)
            if not os.path.exists(bfapath):
                confile = os.path.join(os.path.dirname(sys.argv[0]), "%s.con" % my_con.main.profile)

            try:
                ini = bitfarm_configobj.ConfigObj(infile=confile, encoding="cp1252", write_empty_values=True)
            except:
                log.exception("Das Profil %s konnte nicht gelesen werden" % confile)
            else:
                ini['Connection']['user'] = my_con.main.bitencrypt(my_con.main.user)
                ini['Connection']['pass'] = my_con.main.bitencrypt(my_con.main.password)
                ini['Connection']['suuser'] = my_con.main.bitencrypt(my_con.main.suuser)
                ini['Connection']['supass'] = my_con.main.bitencrypt(my_con.main.supassword)
                ini['Server']['edition'] = 'gpl'
                ini.write()
            sys_exit(0)

    if args.migrate_to and not (args.move_files or args.create_mapping):  # wenn wir migrieren ohne die dateien umzuziehen dann setzen wir archive_rs und archive_nrs auf die werte des quellprofils
        if my_con.main.version == 35:
            split_path = my_con.main.archive_rs.lower().split(os.sep)
            if split_path[-1] == '':
                split_path.pop()
            rs_path = split_path.pop()
            rs_dir = os.sep.join(split_path)
            if args.path_rs.lower().find(my_con.main.archive_rs.lower()) == -1:
                if rs_path != args.path_rs:
                    log.warning("Der Parameter rs_path (%s) passt nicht zum Archivablageverzeichnis des Profils %s. Wird auf %s gesetzt." % (args.path_rs, my_con.main.profile, rs_path))
                    args.path_rs = rs_path

            split_path = my_con.main.archive_nrs.lower().split(os.sep)
            if split_path[-1] == '':
                split_path.pop()
            nrs_path = split_path.pop()
            nrs_dir = os.sep.join(split_path)
            if args.path_nrs.lower().find(my_con.main.archive_nrs.lower()) == -1:
                if nrs_path != args.path_nrs:
                    log.warning("Der Parameter nrs_path (%s) passt nicht zum Archivablageverzeichnis des Profils %s. Wird auf %s gesetzt." % (args.path_nrs, my_con.main.profile, nrs_path))
                    args.path_nrs = nrs_path

            if rs_dir != nrs_dir:
                # log.error("Achtung, unterschiedliche Stammverzeichnisse für RS und NRS wird nicht unterstützt")
                sys_exit(21)

            if not args.gpl and rs_dir.lower() != os.path.expandvars(args.storage).lower():     # Prüfung nur für Enterprise, bei GPL Migration werden Pfade nicht auf bfastore geändert
                # log.error("Achtung, Storage-Systemvariable %s (%s) verweist nicht auf Archivablage-Stammverzeichnis %s" % (args.storage, os.path.expandvars(args.storage), rs_dir))
                sys_exit(22, args.storage, os.path.expandvars(args.storage), rs_dir)

    if args.create_database or args.change_root_pass or args.change_sa_pass or args.change_db_pass or args.change_admin_pass:
        # Root Datenbank Connection
        if args.create_database:
            profile = args.name if args.name is not None else args.create_database

            if os.path.exists(os.path.join(bfapath, profile + '.con')):
                # log.error("Achtung, es existiert schon eine Profildatei %s. Bitte vergeben Sie einen eindeutigen Namen für das Profil." % os.path.join(bfapath, profile + '.con'))
                sys_exit(29, os.path.join(bfapath, profile + '.con'))
        else:
            profile = 'main'

        root_con = bitfarm_proc.BitfarmConnections()
        root_con.profiles.append(profile)
        root_con.main = bitfarm_proc.BitfarmCon()
        root_con.main.db_engine = 'mysql'
        root_con.main.db_encoding = 'utf8'
        root_con.main.mysql_encoding = 'utf8mb4'
        root_con.main.db_collation = 'utf8mb4_unicode_ci'
        root_con.main.version = 36
        root_con.main.gpl = args.create_gpl

        constr = root_con.main._makeconstr(args.root_user, args.root_pass, args.server, args.port, dbadmin=True)
        conargs = root_con.main._makeconargs()
        if not constr:
            # log.error("Es konnte kein Connection String aufgebaut werden")
            sys_exit(7)

        engine = sqla.create_engine(constr, echo=False, connect_args=conargs, pool_size=10)
        try:
            root_db = engine.connect()
        except sqla.exc.OperationalError as e:
            if hasattr(e, 'orig') and e.orig.args[0] == 1045:
                    # log.error("Es konnte sich nicht am Datenbankserver angemeldet werden, User oder Password falsch.")
                    log.error(e._message())
                    sys_exit(4)
            elif hasattr(e, 'orig') and e.orig.args[0] in [3159, 2026]:
                    # log.error("Es konnte sich nicht am Datenbankserver angemeldet werden, User oder Password falsch.")
                    log.error(e._message())
                    sys_exit(58)
            else:
                # log.exception("Es konnte sich nicht am Datenbankserver angemeldet werden")
                sys_exit(4)
        except sqla.exc.ProgrammingError as e:
            if hasattr(e, 'orig') and e.orig.args[0] == 1045:
                    # log.error("Es konnte sich nicht am Datenbankserver angemeldet werden, User oder Password falsch.")
                    log.error(e._message())
                    sys_exit(4)
            else:
                # log.exception("Es konnte sich nicht am Datenbankserver angemeldet werden")
                sys_exit(4)
        except:
            # log.exception("Es konnte sich nicht am Datenbankserver angemeldet werden")
            sys_exit(4)
        else:
            if root_db is None:
                # log.exception("Es konnte sich nicht am Datenbankserver angemeldet werden")
                sys_exit(4)

            log.info("Verbindung zu %s:%s hergestellt" % (args.server, args.port))

        conobj = bitfarm_proc.BitfarmCon()
        conobj.gpl = args.create_gpl

        if args.systemuserpass != "1E305D2915E0397E":
            args.systemuserpass = conobj.bitencrypt(args.systemuserpass)

        if args.change_root_pass:
            new_pass1 = get_input('Bitte geben Sie das neue Passwort des Root-Users ein: ', True)
            new_pass2 = get_input('Bitte wiederholen Sie das neue Passwort des Root-Users: ', True)

            if new_pass1 != new_pass2:
                sys_exit(57)

            sql = "ALTER USER root@localhost IDENTIFIED by '%s';" % new_pass1
            try:
                root_db.execute(sqla.text(sql))
            except:
                log.exception("Konnte Passwort nicht festlegen")
            else:
                sql = "FLUSH PRIVILEGES;"
                try:
                    root_db.execute(sqla.text(sql))
                except:
                    log.exception("Konnte Passwort nicht festlegen")
                else:
                    log.info("Neues Passwort für Root festgelegt")

        if args.change_sa_pass:
            new_pass1 = get_input('Bitte geben Sie das neue Passwort des %s-Users ein: ' % my_con.main.user, True)
            new_pass2 = get_input('Bitte wiederholen Sie das neue Passwort des %s-Users: ' % my_con.main.user, True)

            if new_pass1 != new_pass2:
                sys_exit(57)

            sql = "ALTER USER '%s'@'%%' IDENTIFIED by '%s';" % (my_con.main.user, new_pass1)
            try:
                root_db.execute(sqla.text(sql))
            except:
                log.exception("Konnte Passwort nicht festlegen")
            else:
                sql = "FLUSH PRIVILEGES;"
                try:
                    root_db.execute(sqla.text(sql))
                except:
                    log.exception("Konnte Passwort nicht festlegen")
                else:
                    log.info("Neues Passwort für %s festgelegt" % my_con.main.user)

                confile = os.path.join(bfapath, "%s.con" % my_con.main.profile)
                if not os.path.exists(bfapath):
                    confile = os.path.join(os.path.dirname(sys.argv[0]), "%s.con" % my_con.main.profile)

                try:
                    ini = bitfarm_configobj.ConfigObj(infile=confile, encoding="cp1252", write_empty_values=True)
                except:
                    log.exception("Das Profil %s konnte nicht gelesen werden" % confile)
                else:
                    ini['Connection']['pass'] = my_con.main.bitencrypt(new_pass1)
                    ini.write()

        if args.change_db_pass:
            new_pass1 = get_input('Bitte geben Sie das neue Passwort des %s-Users ein: ' % my_con.main.suuser, True)
            new_pass2 = get_input('Bitte wiederholen Sie das neue Passwort des %s-Users: ' % my_con.main.suuser, True)

            if new_pass1 != new_pass2:
                sys_exit(57)

            sql = "ALTER USER '%s'@'%%' IDENTIFIED by '%s';" % (my_con.main.suuser, new_pass1)
            try:
                root_db.execute(sqla.text(sql))
            except:
                log.exception("Konnte Passwort nicht festlegen")
            else:
                sql = "FLUSH PRIVILEGES;"
                try:
                    root_db.execute(sqla.text(sql))
                except:
                    log.exception("Konnte Passwort nicht festlegen")
                else:
                    log.info("Neues Passwort für %s festgelegt" % my_con.main.suuser)

                confile = os.path.join(bfapath, "%s.con" % my_con.main.profile)
                if not os.path.exists(bfapath):
                    confile = os.path.join(os.path.dirname(sys.argv[0]), "%s.con" % my_con.main.profile)

                try:
                    ini = bitfarm_configobj.ConfigObj(infile=confile, encoding="cp1252", write_empty_values=True)
                except:
                    log.exception("Das Profil %s konnte nicht gelesen werden" % confile)
                else:
                    ini['Connection']['supass'] = my_con.main.bitencrypt(new_pass1)
                    ini.write()

        if args.change_admin_pass:
            new_pass1 = get_input('Bitte geben Sie das neue Passwort des DMSAdmins ein: ', True)
            new_pass2 = get_input('Bitte wiederholen Sie das neue Passwort DMSAdmins: ' , True)

            if new_pass1 != new_pass2:
                sys_exit(57)

            try:
                root_db.execute('USE `%s`;' % my_con.main.db_name)
            except:
                # log.exception("Konnte Datenbank %s nicht festlegen" % my_con.main.db_name)
                sys_exit(41, my_con.main.db_name)
            else:
                log.info("Datenbank %s wurde festgelegt" % my_con.main.db_name)

            sql = "UPDATE sys_user SET userpass = '%s' WHERE flags & 129 = 129;" % my_con.main.bitencrypt(new_pass1)
            try:
                root_db.execute(sqla.text(sql))
            except:
                log.exception("Konnte Passwort nicht festlegen")
            else:
                log.info("Neues Passwort für DMSAdmin festgelegt")

            if my_con.main.bfauser is not None:
                if my_con.main.bfauser.lower() == b'dmsadmin':
                    my_con.main.bfapass = new_pass1.encode('utf8')     # bfauser und bfapass sind bytes
                    if os.path.exists(my_con.main.confile):
                        ini = bitfarm_configobj.ConfigObj(infile=my_con.main.confile, encoding="cp1252", write_empty_values=True)
                        if 'client' in ini:
                            ini['client']['bfapass'] = base64.b64encode(dpapi.cryptData(my_con.main.bfapass)).decode()
                            ini.write()

        if not args.create_database:
            sys_exit(0)

        log.info("Datenbank %s anlegen" % args.create_database)
        if not (args.sa_user and args.sa_pass):
            log.warning("Achtung, es wurde kein sa-User bzw Passwort angegeben, installiert wird der Standard sa-User")
            if args.sa_user is None:
                args.sa_user = 'sa'
            if args.sa_pass is None:
                args.sa_pass = 'bitfarmIT'
        if not (args.db_user and args.db_pass):
            log.warning("Achtung, es wurde kein su-User bzw Passwort angegeben, installiert wird der Standard su-User")
            if args.db_user is None:
                args.db_user = 'su'
            if args.db_pass is None:
                args.db_pass = 'bitfarmIT'

        try:
            root_db.execute('CREATE DATABASE IF NOT EXISTS `%s` CHARACTER SET %s COLLATE %s;' % (args.create_database, root_con.main.mysql_encoding, root_con.main.db_collation))
        except:
            log.exception("Konnte Datenbank nicht anlegen")
            sys_exit(30)
        else:
            log.info("Datenbank %s wurde angelegt" % args.create_database)
        try:
            root_db.execute('USE `%s`;' % args.create_database)
        except:
            log.exception("Konnte Datenbank nicht festlegen")
        else:
            log.info("Datenbank %s wurde festgelegt" % args.create_database)

        try:
            sql = "CREATE USER IF NOT EXISTS '%s'@'%%' IDENTIFIED BY '%s';" % (args.sa_user, args.sa_pass)
            root_db.execute(sqla.text(sql))
        except:
            log.exception("Konnte SA User %s nicht anlegen" % args.sa_user)
            sys_exit(32, args.sa_user)
        else:
            log.info("SA User %s wurde angelegt" % args.sa_user)

        try:
            sql = "CREATE USER IF NOT EXISTS '%s'@'%%' IDENTIFIED BY '%s';" % (args.db_user, args.db_pass)
            root_db.execute(sqla.text(sql))
        except:
            log.exception("Konnte SU User %s nicht anlegen" % args.db_user)
            sys_exit(33, args.db_user)
        else:
            log.info("SU User %s wurde angelegt" % args.db_user)

        if args.version == 36:
            log.info("Datenbank der Version 36 wird angelegt...")
            if args.create_sql[-4:].lower() != '.sql':
                args.create_sql += '.sql'
            if not os.path.exists(os.path.join(os.path.dirname(sys.argv[0]), args.create_sql)):
                # log.error("Konnte Skript %s nicht finden" % args.create_sql)
                sys_exit(5, args.create_sql)

            log.info("Lese das %s Skript" % args.create_sql)

            with codecs.open(os.path.join(os.path.dirname(sys.argv[0]), args.create_sql), "r", encoding="utf8") as f:
                sql = f.read()
                #        print [x.strip() for x in sql.split(';')]

            for exp in sql.split(';\r\n'):
                if exp.find('\ufeff') > -1:
                    exp = exp.replace('\ufeff', '')
                exp = exp.strip()
                exp = exp.replace('\r','')
                if len(exp) > 0:
                    log.debug("-----------------------------")
                    log.debug(exp)
                    try:
                        # root_db.execute(sqla.text(exp))
                        exp = re.sub(r':\w+\s', escapeit, exp)
                        root_db.execute(sqla.text(exp))
                    except:
                        log.exception("Fehler beim Ausführen einer SQL-Anweisung: %s" % sys.exc_info()[1])
            log.info("Datenbanktabellen angelegt")

            if not os.path.exists(os.path.join(os.path.dirname(sys.argv[0]), args.search_engine)):
                # log.error("Konnte SearchEngine_36.sql nicht finden")
                sys_exit(5, args.search_engine)

            log.info("Lese das %s Skript" % args.search_engine)

            with codecs.open(os.path.join(os.path.dirname(sys.argv[0]), args.search_engine), "r", encoding="utf8") as f:
                sql = f.read()

            for proc in sql.split('$$'):
                if proc.find('\ufeff') > -1:
                    proc = proc.replace('\ufeff', '')
                if proc.find('DELIMITER ;') > -1:
                    proc = proc.replace('DELIMITER ;', '')
                if proc.find('DELIMITER') > -1:
                    proc = proc.replace('DELIMITER', '')
                if proc.find("CREATE DEFINER = 'su'@'%'") > -1:
                    proc = proc.replace("CREATE DEFINER = 'su'@'%'", "CREATE DEFINER = '%s'@'%%'" % args.db_user)
                if proc.find("CREATE\r\nDEFINER = 'su'@'%'") > -1:
                    proc = proc.replace("CREATE\r\nDEFINER = 'su'@'%'", "CREATE DEFINER = '%s'@'%%'" % args.db_user)
                if proc.find("CREATE\nDEFINER = 'su'@'%'") > -1:
                    proc = proc.replace("CREATE\nDEFINER = 'su'@'%'", "CREATE DEFINER = '%s'@'%%'" % args.db_user)

                proc = proc.strip()
                proc = proc.replace('\r', '')
                if len(proc) > 0:
                    t = sqla.text(proc).execution_options(autocommit=True)
                    log.debug("-----------------------------")
                    log.debug(t.text)
                    try:
                        root_db.execute(t)
                    except:
                        log.exception("Fehler beim Ausführen einer SQL-Anweisung: %s" % proc.strip())

            log.info("SearchEngine installiert")
        else:
            # log.warning("noch keine Unterstützung für Datenbank Version 40")
            sys_exit(8)

        # wir müssen dem sa user berechtigung von any host geben, da in 36 es sein kann, das kommandozeilentools in die datenbank schreiben wollen (zb viewer-plugins)
        # in 37 legen wir den sa user dann auf localhost bzw. der maschine des bitfarm-servers fest.
        # sql = "GRANT ALL PRIVILEGES ON `%s`.* TO '%s'@'localhost' IDENTIFIED BY '%s'" % (args.create_database, args.sa_user, args.sa_pass)
        # try:
        #     root_con.execute(sqla.text(sql))
        # except:
        #     log.exception("Fehler beim Setzen der Berechtigung für den schreibenden User : %s" % args.sa_user)
        #
        # sql = "GRANT ALL PRIVILEGES ON `%s`.* TO '%s'@'%s' IDENTIFIED BY '%s'" %(args.create_database, args.sa_user, args.bfaserver, args.sa_pass)
        # try:
        #     root_con.execute(sqla.text(sql))
        # except:
        #     log.exception("Fehler beim Setzen der Berechtigung für den schreibenden User : %s" % args.sa_user)

        sql = "GRANT ALL PRIVILEGES ON `%s`.* TO '%s'@'%%'" % (args.create_database, args.sa_user)
        try:
            root_db.execute(sqla.text(sql))
        except:
            log.exception("Fehler beim Setzen der Berechtigung für den schreibenden User : %s" % args.sa_user)
        else:
            log.info("Berechtigung für den schreibenden User gesetzt")

        # sql = "GRANT CREATE TEMPORARY TABLES, SELECT, EXECUTE ON `%s`.* TO '%s'@'localhost' IDENTIFIED BY '%s'" % (args.create_database, args.db_user, args.db_pass)
        # try:
        #     root_con.execute(sqla.text(sql))
        # except:
        #     log.exception(
        #         "Fehler beim Setzen der Berechtigung für den lesenden User : %s" % args.db_user)

        sql = "GRANT CREATE TEMPORARY TABLES, SELECT, EXECUTE ON `%s`.* TO '%s'@'%%'" % (args.create_database, args.db_user)
        try:
            root_db.execute(sqla.text(sql))
        except:
            log.exception("Fehler beim Setzen der Berechtigung für den lesenden User : %s" % args.db_user)

        sql = "GRANT SELECT, INSERT ON `%s`.tmp_fullsql TO '%s'@'%%'" % (args.create_database, args.db_user)
        try:
            root_db.execute(sqla.text(sql))
        except:
            log.exception("Fehler beim Setzen der Berechtigung für den lesenden User auf dier tmp_fullsql Tabelle: %s" % args.db_user)

        # sql = "GRANT SELECT, INSERT ON `%s`.tmp_fullsql TO '%s'@'localhost' IDENTIFIED BY '%s'" % (args.create_database, args.db_user, args.db_pass)
        # try:
        #     root_con.execute(sqla.text(sql))
        # except:
        #     log.exception(
        #         "Fehler beim Setzen der Berechtigung für den lesenden User : %s" % args.db_user)

        sql = "Flush Privileges"
        try:
            root_db.execute(sql)
        except:
            log.exception("Fehler beim Setzen der Berechtigung für den lesenden User : %s" % args.db_user)
        else:
            log.info("Berechtigung für den lesenden User gesetzt")

        # Anlegen der Standardgruppe, User, Wiedervorlage Statusfeld, Wurzelverzeichnis
        sql = "select * from sys_arc where arcid = 1 and lgrid = 0"     # haben wir schon eine Wurzel?
        try:
            res = root_db.execute(sqla.text(sql)).fetchone()
        except:
            log.exception("Fehler Abfrage des Wurzelarchivs")
        else:
            if not res:
                sql = "INSERT INTO sys_arc (arcid, lgrid, arcname, flags, icon, type, options, storageid) VALUES (1, 0, '%s', 1, 0, 0, 0, 0)" % profile
                try:
                    root_db.execute(sqla.text(sql))
                except:
                    log.exception("Fehler beim Anlegen des Wurzelarchivs")
                else:
                    log.info("Wurzelarchiv angelegt")

        sql = "select * from sys_arc where type = 2"     # haben wir schon eine Wiedervorlagearchivs?
        try:
            res =root_db.execute(sqla.text(sql)).fetchone()
        except:
            log.exception("Fehler bei Abfrage des  Wiedervorlagearchivs")
        else:
            if not res:
                sql = "INSERT INTO sys_arc (arcid, lgrid, arcname, flags, icon, type, options, storageid) VALUES (2, 0, '%s', 1, 71, 2, 0, 0)" % "Wiedervorlage"
                try:
                    root_db.execute(sqla.text(sql))
                except:
                    log.exception("Fehler beim Anlegen des Wiedervorlagearchivs")
                else:
                    log.info("Wiedervorlagearchivs angelegt")

        sql = "select * from sys_user where userid = 1"     # haben wir schon eine System User?
        try:
            res =root_db.execute(sqla.text(sql)).fetchone()
        except:
            log.exception("Fehler bei Abfrage des Systemusers")
        else:
            if not res:
                sql = "INSERT INTO sys_user (userid, username, userpass, flags, displayname) VALUES (1, 'System', '%s', 2, 'System')" % args.systemuserpass
                try:
                    root_db.execute(sqla.text(sql))
                except:
                    log.exception("Fehler beim Anlegen des Systemusers")
                else:
                    log.info("Systemusers angelegt")

        sql = "SET SESSION sql_mode='NO_AUTO_VALUE_ON_ZERO'"
        try:
            root_db.execute(sqla.text(sql))
        except:
            log.exception("Fehler beim Setzen von NO_AUTO_VALUE_ON_ZERO")
        else:
            log.info("NO_AUTO_VALUE_ON_ZERO gesetzt")

        sql = "select * from sys_group where groupid = 0"     # haben wir schon eine System User?
        try:
            res = root_db.execute(sqla.text(sql)).fetchone()
        except:
            log.exception("Fehler bei Abfrage der Gruppe Alle(Builtin)")
        else:
            if not res:
                sql = "INSERT INTO sys_group (groupid, groupname, role, flags) VALUES (0, '%s', 28159, 1)" % 'Alle (Builtin)'
                try:
                    root_db.execute(sqla.text(sql))
                except:
                    log.exception("Fehler beim Anlegen des Default Gruppe")
                else:
                    log.info("Default Gruppe angelegt")

        sql = "select * from sys_groupuser where groupid = 0 and userid = 1"     # haben wir schon eine System User?
        try:
            res = root_db.execute(sqla.text(sql)).fetchone()
        except:
            log.exception("Fehler bei Abfrage der Gruppezugehörigkeit User System zu Alle(Builtin)")
        else:
            if not res:
                sql = "INSERT INTO sys_groupuser (memid, groupid, userid) VALUES (1, 0, 1)"
                try:
                    root_db.execute(sqla.text(sql))
                except:
                    log.exception("Fehler beim Zuordnen des Systemusers zur Default Gruppe")
                else:
                    log.info("Systemusers zur Default Gruppe zugeordnet")

        sql = "select * from sys_bookmarks where type = 0 and userid = 0 and groupid = 0"
        try:
            res = root_db.execute(sqla.text(sql)).fetchone()
        except:
            log.exception("Fehler bei Abfrage des Lesezeichens für Alle Benutzer")
        else:
            if not res:
                sql = "INSERT INTO sys_bookmarks (name, type, userid, groupid, bookmark) VALUES ('Lesezeichen für alle Benutzer', 0, 0, 0, '%s')" % '<?xml version="1.0"?><bookmark name="Lesezeichen für alle Benutzer"></bookmark>'
                try:
                    root_db.execute(sqla.text(sql))
                except:
                    log.exception("Fehler beim Anlegen eines Basiseintrages in den Lesezeichen")
                else:
                    log.info("Basiseintrages in den Lesezeichen angelegt")

        # sql = "INSERT INTO sys_statefields (stateid, displayname, bitmask, indexcolumnid) VALUES (1, '%s', 1, 0)" % ('auf Wiedervorlage')
        # try:
        #     root_con.execute(sqla.text(sql))
        # except:
        #     log.exception("Fehler beim Anlegen des Wiedervorlage Statusfeldes")
        # log.info(" Wiedervorlage Statusfeldes angelegt")

        sql = "select * from sys_doc where gdocid = 0"
        try:
            res = root_db.execute(sqla.text(sql)).fetchone()
        except:
            log.exception("Fehler bei Abfrage des Admin Dokuments in der sys_doc")
        else:
            if not res:
                sql = "INSERT INTO sys_doc (gdocid, userid, docdate, modified_userid, arcid) VALUES (0, 1, NOW(), 1, 0)"
                try:
                    root_db.execute(sqla.text(sql))
                except:
                    log.exception("Fehler beim Anlegen des Admin Dokuments")
                else:
                    log.info("Admin Dokument angelegt")

        sql = "SET SESSION sql_mode='NO_AUTO_VALUE_ON_ZERO'"
        try:
            root_db.execute(sqla.text(sql))
        except:
            log.exception("Fehler beim Setzen von NO_AUTO_VALUE_ON_ZERO")
        else:
            log.info("NO_AUTO_VALUE_ON_ZERO gesetzt")

        userid_admin = 2
        sql = "select userid from sys_user where flags & 129 = 129"
        try:
            res = root_db.execute(sqla.text(sql)).fetchone()
        except:
            log.exception("Fehler bei Abfrage des DMSAdmin in der sys_user")
        else:
            if not res:
                flags = '137'
                if args.create_gpl:
                    flags = '169'
                    sql = 'INSERT INTO sys_user (username, displayname, userpass, flags) VALUES ("%s", "%s", "%s", %s)' % ('DMSAdmin', 'DMSAdmin', conobj.bitencrypt('bitfarm'), flags)
                else:
                    sql = 'INSERT INTO sys_user (username, displayname, userpass, flags) VALUES ("%s", "%s", "%s", %s)' % ('DMSAdmin', 'DMSAdmin', conobj.bitencrypt(args.admin_pass), flags)
                try:
                    res = root_db.execute(sqla.text(sql))
                except:
                    log.exception("Fehler beim Anlegen des Admin-Users")
                else:
                    log.info("DMSAdmin User angelegt")
                    userid_admin = res.lastrowid
            else:
                userid_admin = res[0]

        sql = "select * from sys_groupuser where groupid = 0 and userid = %d" % userid_admin
        try:
            res = root_db.execute(sqla.text(sql)).fetchone()
        except:
            log.exception("Fehler bei Abfrage des DMSAdmin in der sys_user")
        else:
            if not res:
                sql = "INSERT INTO sys_groupuser (groupid, userid) VALUES (0, %d)" % userid_admin
                try:
                    root_db.execute(sqla.text(sql))
                except:
                    log.exception("Fehler beim Zuordnen des DMSAdmin zur Default Gruppe [Alle (Builtin)]")
                else:
                    log.info("DMSADmin zur Default Gruppe [Alle (Builtin)] zugeordnet")

        tstamp = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f')
        enc_tstamp = conobj.bitencrypt(tstamp)
        sql = 'insert into sys_config (name, value) values("created", "%s")' % enc_tstamp
        try:
            root_db.execute(sql)
        except:
            log.exception("Fehler beim Setzen des Timestamps")

        if args.bfaserver == 'localhost' and args.server != 'localhost':
            args.bfaserver = args.server
            log.info("Adresse des bitfarm Servers wurde auf %s gesetzt" % args.bfaserver)
        elif args.bfaserver == 'localhost' and args.server == 'localhost':
            args.bfaserver = hostname
            log.info("Adresse des bitfarm Servers wurde auf %s gesetzt" % args.bfaserver)

        confile = os.path.join(bfapath, "%s.con" % profile)
        if not os.path.exists(bfapath):
            confile = os.path.join(os.path.dirname(sys.argv[0]), "%s.con" % profile)
        ini = bitfarm_configobj.ConfigObj(infile=confile, encoding="cp1252", write_empty_values=True)

        ini["Connection"] = {}
        ini["Connection"]["server"] = hostname if args.server == 'localhost' else args.server
        ini["Connection"]["port"] = args.port
        ini["Connection"]["db"] = args.create_database
        ini["Connection"]["user"] = conobj.bitencrypt(args.sa_user)
        ini["Connection"]["pass"] = conobj.bitencrypt(args.sa_pass)
        ini["Connection"]["suuser"] = conobj.bitencrypt(args.db_user)
        ini["Connection"]["supass"] = conobj.bitencrypt(args.db_pass)
        ini["Connection"]["db_encoding"] = args.encoding
        ini["Server"] = {}
        ini["Server"]["version"] = args.version
        if args.create_gpl:
            ini["Server"]["edition"] = 'gpl'
        ini["Server"]["server"] = args.bfaserver
        ini["Server"]["port"] = args.bfaport

        ini["Scripts"] = {}
        if args.create_gpl:
            cname = socket.gethostname()
            ini["Scripts"]["storages"] = r'\\' + cname
        else:
            ini["Scripts"]["storages"] = '%bfastore0%'
        ini["Scripts"]["archive_rs"] = args.path_rs
        ini["Scripts"]["archive_nrs"] = args.path_nrs

        ini["Scripts"]['serverprogram'] = ""
        ini["Scripts"]['clientprogram'] = ""
        ini["Scripts"]['clientspooler'] = ""
        ini["Scripts"]['uncclientspooler'] = ""
        ini["Scripts"]['omnipage'] = ""
        ini["Scripts"]['ghostscript'] = ""
        ini["Scripts"]['gsview'] = ""
        ini["Scripts"]['smtpserver'] = ""
        ini["Scripts"]['smtpport'] = ""
        ini["Scripts"]['sendername'] = ""
        ini["Scripts"]['authuser'] = ""
        ini["Scripts"]['authpass'] = ""
        ini["Scripts"]['adminmail'] = ""
        tpath = os.path.join(bfapath, 'templates')
        if not os.path.exists(tpath):
            ini["Scripts"]['templates'] = ""
        else:
            ini["Scripts"]['templates'] = tpath + '\\'
        ini["Scripts"]['workflowfile'] = ""
        ini["Scripts"]['openofficepath'] = ""
        ini["Scripts"]['undistributed'] = ""
        ini["Scripts"]['unverteilt'] = ""
        tpath = os.path.join(bfapath, 'workflows')
        if not os.path.exists(tpath):
            ini["Scripts"]['workflows'] = ""
        else:
            ini["Scripts"]['workflows'] = tpath + '\\'
        ini["Scripts"]['tempdir'] = ""

        ini['Backup'] = {}
        ini['Backup']['backuppath'] = ""
        ini['Backup']['sqldatapath'] = ""
        ini['Backup']['backuparchiv'] = ""
        ini['Backup']['backupnrsarchiv'] = ""
        ini['Backup']['backupversion'] = ""
        ini['Backup']['backupworkflows'] = ""
        ini['Backup']['backupsvnrepository'] = ""
        ini['Backup']['backupmysqllog'] = ""
        ini['Backup']['filesize'] = ""
        ini['Backup']['backupzeitraum'] = ""
        ini['Backup']['mysqlservice'] = ""
        ini['Backup']['use7zip'] = ""

        ini["Options"] = {}
        ini['Options']['logdir'] = ""
        ini['Options']['savepass'] = ""
        ini['Options']['manuscanplugin'] = ""
        ini['Options']['importerplugin'] = ""
        ini['Options']['SvnInfo'] = ""
        ini['Options']['LinkChars'] = ""

        ini['DLM'] = {}
        ini['DLM']['expired_dir'] = ""
        ini['DLM']['check_time'] = ""
        ini['DLM']['dont_check_expired'] = ""

        if not args.create_gpl:
            ini['Updater'] = {}
            ini['Updater']['update_files'] = "%s.con;ViewerV3.exe;Updater.exe;Outlookimport.vbs;movetobitfarm.vbs;copytobitfarm.vbs;psprint.vbs;" \
                                             "clientspooler.vbs;druckimport.vbs;" \
                                             "Dokumentationen/Benutzerhandbuch V3.6.2.pdf;HotSearch.exe;bfaMessage.exe;scripts.ini" % profile
            ini['Updater']['update_dirs'] = "bfaDocLauncher;bfaDocLauncher/psutil;locale;locale/de/LC_Messages;locale/en/LC_Messages;Viewer-Files/Plugins"

        ini.write()
        log.info("Das Profil %s wurde geschrieben" % confile)

        new_con = bitfarm_proc.BitfarmCon(profile, gpl=args.create_gpl)
        new_con.create_db_engine()
        new_con.connect_db()
        if new_con.db is not None:
            log.info("Erfolgreich mit neuem Profil verbunden")
        new_con.db.close()

    if args.update:
        # root anmeldungen kommen normalerweise über localhost
        my_con = bitfarm_proc.BitfarmConnections(args.profile, gpl=args.gpl)
        # root_server = my_con.main.server        # für ein Update muss ein Profil angegeben sein, wir holen die Verbindungsdaten daher aus dem Profil
        root_server = 'localhost'        # für ein Update muss ein Profil angegeben sein, wir holen die Verbindungsdaten daher aus dem Profil
        root_port = my_con.main.port

        constr = my_con.main._makeconstr(args.root_user, args.root_pass, root_server, root_port, dbadmin=True)
        conargs = my_con.main._makeconargs()
        if not constr:
            # log.error("Es konnte kein Connection String aufgebaut werden")
            sys_exit(7)

        engine = sqla.create_engine(constr, echo=False, connect_args=conargs, pool_size=10)
        try:
            root_db = engine.connect()
        except sqla.exc.OperationalError as e:
            if hasattr(e, 'orig') and e.orig.args[0] == 1045:
                    # log.error("Es konnte sich nicht am Datenbankserver angemeldet werden, User oder Password falsch.")
                    log.error(e._message())
                    sys_exit(4)
            elif hasattr(e, 'orig') and e.orig.args[0] == 2003:
                    # log.error("Es konnte sich nicht am Datenbankserver angemeldet werden, Server nicht erreichbar.")
                    log.error(e._message())
                    sys_exit(4)
            else:
                # log.exception("Es konnte sich nicht am Datenbankserver angemeldet werden")
                sys_exit(4)
        except:
            # log.exception("Es konnte sich nicht am Datenbankserver angemeldet werden")
            sys_exit(4)
        else:
            if root_db is None:
                # log.exception("Es konnte sich nicht am Datenbankserver angemeldet werden")
                sys_exit(4)

            log.info("Verbindung zu %s:%s hergestellt" % (root_server, root_port))

            my_con.main.meta = sqla.MetaData()
            my_con.main.meta.bind = engine

            insp = sqla.inspect(engine)
            my_con.main.tables = insp.get_table_names(my_con.main.db_name)

        if my_con.main.version == 35:
            # sys_cfgxs und sys_stsxs anlegen
            try:
                root_db.execute('USE `%s`;' % my_con.main.db_name)
            except:
                # log.exception("Konnte Datenbank %s nicht festlegen" % my_con.main.db_name)
                sys_exit(41, my_con.main.db_name)
            else:
                log.info("Datenbank %s wurde festgelegt" % my_con.main.db_name)

            new_cfgxs = False
            try:
                res = root_db.execute("show tables like 'sys_cfgxs'").fetchall()
            except:
                log.exception("Fehler beim Prüfen auf sys_cfgxs")
            else:
                if not res:
                    new_cfgxs = True
                    sql = "CREATE TABLE `sys_cfgxs` ( " \
                          "`cfgxs_id` INTEGER UNSIGNED NOT NULL AUTO_INCREMENT," \
                          "`arc_id` INTEGER UNSIGNED NOT NULL, " \
                          "`cfg_id` INTEGER UNSIGNED NOT NULL," \
                          "`usr_id` INTEGER UNSIGNED NOT NULL," \
                          "`grp_id` INTEGER UNSIGNED NOT NULL," \
                          "PRIMARY KEY USING BTREE (`cfgxs_id`)," \
                          "INDEX `IX_arc_id`(`arc_id`)," \
                          "INDEX `IX_cfg_id`(`cfg_id`)," \
                          "INDEX `IX_usr_id`(`usr_id`)," \
                          "INDEX `IX_grp_id`(`grp_id`)) ENGINE = MyISAM " \
                          "COMMENT = 'Rechtetabelle der Zusatzfelder'"

                    t = sqla.text(sql).execution_options(autocommit=True)
                    try:
                        root_db.execute(t)
                    except:
                        log.exception("Fehler beim Ausführen einer SQL-Anweisung: %s" % sql)

            new_stsxs = False
            try:
                res = root_db.execute("show tables like 'sys_stsxs'").fetchall()
            except:
                log.exception("Fehler beim Prüfen auf sys_stsxs")
            else:
                if not res:
                    new_stsxs = True
                    sql = "CREATE TABLE `sys_stsxs` ( " \
                          "`stsxs_id` INTEGER UNSIGNED NOT NULL AUTO_INCREMENT," \
                          "`arc_id` INTEGER UNSIGNED NOT NULL, " \
                          "`sts_id` INTEGER UNSIGNED NOT NULL," \
                          "`usr_id` INTEGER UNSIGNED NOT NULL," \
                          "`grp_id` INTEGER UNSIGNED NOT NULL," \
                          "PRIMARY KEY USING BTREE (`stsxs_id`)," \
                          "INDEX `IX_arc_id`(`arc_id`)," \
                          "INDEX `IX_sts_id`(`sts_id`)," \
                          "INDEX `IX_usr_id`(`usr_id`)," \
                          "INDEX `IX_grp_id`(`grp_id`)) ENGINE = MyISAM " \
                          "COMMENT = 'Rechtetabelle der Statusfelder'"

                    t = sqla.text(sql).execution_options(autocommit=True)
                    try:
                        root_db.execute(t)
                    except:
                        log.exception("Fehler beim Ausführen einer SQL-Anweisung: %s" % sql)

            if new_cfgxs or new_stsxs:
                try:
                    rows = root_db.execute('select * from sys_arc where type = 1').fetchall()
                except:
                    log.exception("Fehler beim Abfrage der sys_arc")
                else:
                    for row in rows:
                        if new_cfgxs:
                            if 'cfgxs_' + row[3] in my_con.main.tables:
                                try:
                                    root_db.execute('insert into sys_cfgxs set arc_id = %s' % row[0])
                                    root_db.execute('insert into sys_cfgxs (arc_id, cfg_id, usr_id, grp_id) select %s, cfg_id, usr_id, grp_id from cfgxs_%s order by cfg_id' % (row[0], row[3]))
                                except:
                                    log.exception("Fehler beim Eintrga in die sys_cfgxs")

                        if new_stsxs:
                            if 'stsxs_' + row[3] in my_con.main.tables:
                                try:
                                    root_db.execute('insert into sys_stsxs set arc_id = %s' % row[0])
                                    root_db.execute('insert into sys_stsxs (arc_id, sts_id, usr_id, grp_id) select %s, sts_id, usr_id, grp_id from stsxs_%s order by sts_id' % (row[0], row[3]))
                                except:
                                    log.exception("Fehler beim Eintrga in die sys_stsxs")

            # gdocid repair
            arctblnames = {}
            try:
                rows = root_db.execute("select * from sys_arc where type = 1").fetchall()
            except:
                log.exception("Fehler beim Abfrage der sys_arc")
            else:
                for row in rows:
                    arctblnames[row[0]] = row[3]

            # Leereinträge löschen
            try:
                res = root_db.execute("select count(gdoc_id) from sys_gdocid where arc_id is NULL or doc_id is Null or arc_id = 0 or doc_id = 0").fetchone()
            except:
                log.exception("Fehler bei der Datenbankabfrage")
            else:
                if res and res[0] > 0:
                    try:
                        root_db.execute("delete from sys_gdocid where arc_id is NULL or doc_id is Null or arc_id = 0 or doc_id = 0")
                    except:
                        log.exception("Fehler beim Löschen von Leereinträgen in der sys_gdocid")
                    else:
                        log.info("Es wurden %i Leereinträge aus der sys_gdocid gelöscht" % res[0])

            # Alle gdocids in der sys_gdocid überprüfen
            try:
                res = root_db.execute("select * from sys_gdocid").fetchall()
            except:
                log.exception("Fehler bei der Datenbankabfrage")
            else:
                if res:
                    len_res = len(res)
                    cnt = 0
                    log.info("Überprüfe die vorhandenen Einträge (%i) in der sys_gdocid " % len_res)

                    for row in res:
                        cnt += 1
                        if not notstdout:
                            sys.stdout.write("Verarbeite Eintrag %i von %i\r" % (cnt, len_res))
                            sys.stdout.flush()

                        gdocid = row[0]
                        arcid = row[1]
                        docid = row[2]

                        if arcid not in arctblnames:
                            log.warning("gdocid [%s] verweist auf nicht vorhandene Tabelle" % gdocid)
                            deletegdocid35(root_db, gdocid)
                        else:
                            try:
                                res2 = root_db.execute("select * from arc_%s where doc_id = %s" % (arctblnames[arcid], docid)).fetchone()
                            except:
                                log.exception("Fehler beim Abfrage der sys_arc")
                            else:
                                if not res2:
                                    log.warning("Dokument mit der ID [%i] konnte nicht gefunden werden (%i.%i)" % (gdocid, arcid, docid))
                                    deletegdocid35(root_db, gdocid)
                                elif gdocid != res2['gdoc_id']:
                                    log.warning("gdocid [%i] verweist auf ein Dokument mit falscher gdocid (%i.%i)" % (gdocid, arcid, docid))
                                    try:
                                        root_db.execute("update arc_%s set gdoc_id = %i where doc_id = %i" % (arctblnames[arcid], gdocid, docid))
                                    except:
                                        log.exception("Fehler beim Aktualisieren der gdocid [%i] (%i.%i)" % (gdocid, arcid, docid))
                                    else:
                                        log.info("gdocid [%i] des Dokuments (%i.%i) aktualisiert" % (gdocid, arcid, docid))

            # alle Arc Tabellen überprüfen
            log.info("Überprüfe alle Dokumente...")
            for arcid in arctblnames.keys():
                try:
                    res = root_db.execute("select doc_id, gdoc_id from arc_%s order by doc_id" % arctblnames[arcid]).fetchall()
                except:
                    log.exception("Fehler beim Abfrage der arc_%s" % arctblnames[arcid])
                else:
                    if res:
                        log.info("Es wurden %i Dokumente in arc_%s gefunden" % (len(res), arctblnames[arcid]))
                        for row in res:
                            gdocid = row['gdoc_id']
                            docid = row['doc_id']

                            if not gdocid:
                                log.warning("Das Dokument (%i.%i) hat keine gdocid" % (arcid, docid))
                                try:
                                    res2 = root_db.execute("insert into sys_gdocid set arc_id = %i, doc_id = %i" % (arcid, docid))
                                    root_db.execute("update arc_%s set gdoc_id = %i where doc_id = %i" % (arctblnames[arcid], res2.lastrowid, docid))
                                except:
                                    log.exception("Fehler beim Erzeugen eine gdocid für (%i.%i)" % (arcid, docid))
                                else:
                                    log.warning("gdocid [%i] des Dokuments (%i.%i) aktualisiert" % (res2.lastrowid, arcid, docid))
                            else:
                                try:
                                    res2 = root_db.execute("select * from sys_gdocid where gdoc_id = %i" % gdocid).fetchone()
                                except:
                                    log.exception("Fehler bei Abfrage der gdocid")
                                else:
                                    if not res2:
                                        log.warning("Die gdocid [%i] des Dokuments (%i.%i) konnte nicht in der sys_gdocid gefunden werden" % (gdocid, arcid, docid))
                                        try:
                                            res2 = root_db.execute("insert into sys_gdocid set gdoc_id = %i, arc_id = %i, doc_id = %i" % (gdocid, arcid, docid))
                                        except:
                                            log.exception("Fehler beim Anlegen einer gdocid")
                                        else:
                                            log.info("gdocid [%i] angelegt" % gdocid)
                                    else:
                                        if arcid != res2['arc_id'] or docid != res2['doc_id']:
                                            log.warning("Die gdocid [%i] des Dokuments (%i.%i) verweist in der sys_gdocid auf ein anderes Dokument")
                                            try:
                                                res2 = root_db.execute("insert into sys_gdocid set arc_id = %i, doc_id = %i" % (arcid, docid))
                                                root_db.execute("update arc_%s set gdoc_id = %i where doc_id = %i" % (arctblnames[arcid], res2.lastrowid, docid))
                                            except:
                                                log.exception("Fehler beim Erzeugen eine gdocid für (%i.%i)" % (arcid, docid))
                                            else:
                                                log.warning("gdocid [%i] des Dokuments (%i.%i) aktualisiert" % (gdocid, arcid, docid))


            log.info("Update v35 durchgeführt")
            sys_exit(0)

        elif my_con.main.version == 36:
            # erst ermitteln wir die Datenbankversion
            try:
                mysql5 = root_db.connection.connection._server_version[0] == 5
            except:
                mysql5 = False
            try:
                root_db.execute('USE %s;' % my_con.main.db_name)
            except:
                # log.exception("Konnte Datenbank %s nicht festlegen" % my_con.main.db_name)
                sys_exit(41, my_con.main.db_name)
            else:
                log.info("Datenbank %s wurde festgelegt" % my_con.main.db_name)

            try:
                root_db.execute("SET NAMES utf8mb4;")
            except:
                # log.exception("Konnte Charset utf8mb4 nicht festlegen")
                sys_exit(50)
            else:
                log.info("Charset utf8mb4 wurde festgelegt")

            # Zwei Stored Procedures anlegen sp_add_column und sp_add_index
            log.info("Stored Procedures anlegen sp_add_column")
            proc = "DROP PROCEDURE IF EXISTS sp_add_column;"
            t = sqla.text(proc).execution_options(autocommit=True)
            try:
                root_db.execute(t)
            except:
                log.exception("Fehler beim Ausführen einer SQL-Anweisung: %s" % proc.strip())
                error = True

            if args.use_mysql57 or mysql5:
                proc = """CREATE DEFINER = '%s'@'%%'
                            PROCEDURE sp_add_column (IN prm_db varchar(255),
                            IN prm_table varchar(255),
                            IN prm_column varchar(255),
                            IN prm_definition text)
                            BEGIN
                            DECLARE s text;

                            IF NOT EXISTS( SELECT NULL
                            FROM INFORMATION_SCHEMA.COLUMNS
                            WHERE table_name COLLATE utf8_unicode_ci = prm_table COLLATE utf8_unicode_ci
                            AND table_schema COLLATE utf8_unicode_ci = prm_db COLLATE utf8_unicode_ci
                            AND column_name COLLATE utf8_unicode_ci = prm_column COLLATE utf8_unicode_ci) THEN

                            SET s = concat('ALTER TABLE ', prm_table, ' ADD COLUMN `', prm_column, '` ', prm_definition, ';');

                            Set @s = s;
                            PREPARE stmt FROM @s;
                            EXECUTE stmt;
                            DEALLOCATE PREPARE stmt;

                            END IF;
                            END;""" % my_con.main.user
            else:
                proc = """CREATE DEFINER = '%s'@'%%'
                            PROCEDURE sp_add_column (IN prm_db varchar(255),
                            IN prm_table varchar(255), 
                            IN prm_column varchar(255),
                            IN prm_definition text)
                            BEGIN
                            DECLARE s text;
                            DECLARE stmt text;
                            
                            IF NOT EXISTS( SELECT NULL
                            FROM INFORMATION_SCHEMA.COLUMNS
                            WHERE table_name = prm_table
                            AND table_schema = prm_db
                            AND column_name = prm_column) THEN
                            
                            SET s = concat('ALTER TABLE ', prm_table, ' ADD COLUMN `', prm_column, '` ', prm_definition, ';');
                            
                            Set @s = s;
                            PREPARE stmt FROM @s;
                            EXECUTE stmt;
                            DEALLOCATE PREPARE stmt;
                            
                            END IF;
                            END;""" % my_con.main.user

            t = sqla.text(proc).execution_options(autocommit=True)
            try:
                root_db.execute(t)
            except:
                log.exception("Fehler beim Ausführen einer SQL-Anweisung: %s" % proc.strip())
                error = True

            log.info("Stored Procedures anlegen sp_add_index")
            proc = "DROP PROCEDURE IF EXISTS sp_add_index;"
            t = sqla.text(proc).execution_options(autocommit=True)
            try:
                root_db.execute(t)
            except:
                log.exception("Fehler beim Ausführen einer SQL-Anweisung: %s" % proc.strip())
                error = True

            if args.use_mysql57 or mysql5:
                proc = """CREATE DEFINER = '%s'@'%%'
                        PROCEDURE sp_add_index (IN prm_db varchar(255), IN prm_table varchar(255), IN prm_index varchar(255), IN prm_type varchar(255), 
                        IN prm_definition varchar(255))
                        BEGIN
                            DECLARE s text;
                            
                            IF NOT EXISTS( SELECT NULL
                                    FROM INFORMATION_SCHEMA.STATISTICS
                                   WHERE table_name COLLATE utf8_unicode_ci = prm_table COLLATE utf8_unicode_ci
                                     AND table_schema COLLATE utf8_unicode_ci = prm_db COLLATE utf8_unicode_ci
                                     AND index_name COLLATE utf8_unicode_ci = prm_index COLLATE utf8_unicode_ci) THEN
                        
                                SET s = concat('ALTER TABLE ', prm_table, ' ADD ', prm_type, ' INDEX `', prm_index, '` ', prm_definition, ';');
                        
                                Set @s = s;
                                PREPARE stmt FROM @s;
                                EXECUTE stmt;
                                DEALLOCATE PREPARE stmt;
                        
                            END IF;
                        END;""" % my_con.main.user
            else:
                proc = """CREATE DEFINER = '%s'@'%%'
                        PROCEDURE sp_add_index (IN prm_db varchar(255), IN prm_table varchar(255), IN prm_index varchar(255), IN prm_type varchar(255), 
                        IN prm_definition varchar(255))
                        BEGIN
                            DECLARE s text;

                            IF NOT EXISTS( SELECT NULL
                                    FROM INFORMATION_SCHEMA.STATISTICS
                                   WHERE table_name = prm_table
                                     AND table_schema = prm_db
                                     AND index_name = prm_index) THEN

                                SET s = concat('ALTER TABLE ', prm_table, ' ADD ', prm_type, ' INDEX `', prm_index, '` ', prm_definition, ';');

                                Set @s = s;
                                PREPARE stmt FROM @s;
                                EXECUTE stmt;
                                DEALLOCATE PREPARE stmt;

                            END IF;
                        END;""" % my_con.main.user

            t = sqla.text(proc).execution_options(autocommit=True)
            try:
                root_db.execute(t)
            except:
                log.exception("Fehler beim Ausführen einer SQL-Anweisung: %s" % proc.strip())
                error = True

            # Update sys_doc und sys_arc mit DLM Spalten (falls nicht vorhanden)
            log.info("Spalte expired in sys_doc anlegen")
            proc = "CALL sp_add_column('%s', 'sys_doc', 'expired', 'DATETIME NULL DEFAULT NULL AFTER `filename`');" % my_con.main.db_name
            t = sqla.text(proc).execution_options(autocommit=True)
            try:
                root_db.execute(t)
            except:
                log.exception("Fehler beim Ausführen einer SQL-Anweisung: %s" % proc.strip())
                error = True

            log.info("Spalte expired_setby in sys_doc anlegen")
            proc = "CALL sp_add_column('%s', 'sys_doc', 'expired_setby', 'int(11) DEFAULT NULL AFTER `expired`');" % my_con.main.db_name
            t = sqla.text(proc).execution_options(autocommit=True)
            try:
                root_db.execute(t)
            except:
                log.exception("Fehler beim Ausführen einer SQL-Anweisung: %s" % proc.strip())
                error = True

            log.info("Spalte expired_userid in sys_doc anlegen")
            proc = "CALL sp_add_column('%s', 'sys_doc', 'expired_userid', ' int(11) DEFAULT NULL AFTER `expired_setby`');" % my_con.main.db_name
            t = sqla.text(proc).execution_options(autocommit=True)
            try:
                root_db.execute(t)
            except:
                log.exception("Fehler beim Ausführen einer SQL-Anweisung: %s" % proc.strip())
                error = True

            log.info("Spalte expired_flags in sys_doc anlegen")
            proc = "CALL sp_add_column('%s', 'sys_doc', 'expired_flags', ' smallint DEFAULT NULL AFTER `expired_userid`');" % my_con.main.db_name
            t = sqla.text(proc).execution_options(autocommit=True)
            try:
                root_db.execute(t)
            except:
                log.exception("Fehler beim Ausführen einer SQL-Anweisung: %s" % proc.strip())
                error = True

            log.info("Spalte duration in sys_arc anlegen")
            proc = "CALL sp_add_column('%s', 'sys_arc', 'duration', 'int(11) DEFAULT 0 AFTER `storageid`');" % my_con.main.db_name
            t = sqla.text(proc).execution_options(autocommit=True)
            try:
                root_db.execute(t)
            except:
                log.exception("Fehler beim Ausführen einer SQL-Anweisung: %s" % proc.strip())
                error = True

            log.info("Spalte interval in sys_arc anlegen")
            proc = "CALL sp_add_column('%s', 'sys_arc', 'interval', 'varchar(5) DEFAULT NULL AFTER `duration`');" % my_con.main.db_name
            t = sqla.text(proc).execution_options(autocommit=True)
            try:
                root_db.execute(t)
            except:
                log.exception("Fehler beim Ausführen einer SQL-Anweisung: %s" % proc.strip())
                error = True

            log.info("Index idx_arcid_docdate in sys_doc anlegen")
            proc = "CALL sp_add_index('%s', 'sys_doc', 'idx_arcid_docdate', '', '(arcid ASC, docdate ASC)');" % my_con.main.db_name
            t = sqla.text(proc).execution_options(autocommit=True)
            try:
                root_db.execute(t)
            except:
                log.exception("Fehler beim Ausführen einer SQL-Anweisung: %s" % proc.strip())
                error = True

            log.info("Index idx_docstate in sys_doc anlegen")
            proc = "CALL sp_add_index('%s', 'sys_doc', 'idx_docstate', '', '(docstate_0 ASC)');" % my_con.main.db_name
            t = sqla.text(proc).execution_options(autocommit=True)
            try:
                root_db.execute(t)
            except:
                log.exception("Fehler beim Ausführen einer SQL-Anweisung: %s" % proc.strip())
                error = True

            # Update der sys_doc mit den neuen volltextspalten
            error = False
            has_column = False
            sql = "select * from INFORMATION_SCHEMA.COLUMNS where table_schema = '%s' and table_name = 'sys_doc' and column_name = 'doctxt'" % my_con.main.db_name

            try:
                res = root_db.execute(sqla.text(sql)).fetchone()
            except:
                log.exception("Fehler beim Abfragen der sys_doc")
            else:
                if res:
                    has_column = True

            has_table = False
            sql = "select * from INFORMATION_SCHEMA.COLUMNS where table_schema = '%s' and table_name = 'sys_doctxt'" % my_con.main.db_name

            try:
                res = root_db.execute(sqla.text(sql)).fetchone()
            except:
                log.exception("Fehler beim Abfragen der sys_doc")
            else:
                if res:
                    has_table = True

            if not has_column:
                log.info("Spalte doctxt in sys_doc anlegen")
                proc = "CALL sp_add_column('%s', 'sys_doc' ,'doctxt', 'longtext  COLLATE utf8mb4_unicode_ci DEFAULT NULL AFTER `expired_flags`');" % my_con.main.db_name
                t = sqla.text(proc).execution_options(autocommit=True)
                try:
                    root_db.execute(t)
                except:
                    log.exception("Fehler beim Ausführen einer SQL-Anweisung: %s" % proc.strip())
                    error = True

                log.info("Spalte doctags in sys_doc anlegen")
                proc = "CALL sp_add_column('%s', 'sys_doc' ,'doctags', 'text  COLLATE utf8mb4_unicode_ci DEFAULT NULL AFTER `doctxt`');" % my_con.main.db_name
                t = sqla.text(proc).execution_options(autocommit=True)
                try:
                    root_db.execute(t)
                except:
                    log.exception("Fehler beim Ausführen einer SQL-Anweisung: %s" % proc.strip())
                    error = True

                log.info("Spalte fieldtxt in sys_doc anlegen")
                proc = "CALL sp_add_column('%s', 'sys_doc' ,'fieldtxt', 'text  COLLATE utf8mb4_unicode_ci DEFAULT NULL AFTER `doctags`')" % my_con.main.db_name
                t = sqla.text(proc).execution_options(autocommit=True)
                try:
                    root_db.execute(t)
                except:
                    log.exception("Fehler beim Ausführen einer SQL-Anweisung: %s" % proc.strip())
                    error = True

                log.info("Spalte annotxt in sys_doc anlegen")
                proc = "CALL sp_add_column('%s', 'sys_doc' ,'annotxt', 'text  COLLATE utf8mb4_unicode_ci DEFAULT NULL AFTER `fieldtxt`')" % my_con.main.db_name
                t = sqla.text(proc).execution_options(autocommit=True)

                try:
                    root_db.execute(t)
                except:
                    log.exception("Fehler beim Ausführen einer SQL-Anweisung: %s" % proc.strip())
                    error = True

            if has_table:
                root_db.execute("SET innodb_lock_wait_timeout = 50000;")
                root_db.execute("SET unique_checks=0;")
                root_db.execute("SET foreign_key_checks=0;")
                root_db.execute("SET NAMES utf8mb4;")
                result = True
                l1 = 0
                l2 = 1000
                while result:
                    sql = 'select gdocid from sys_doctxt where (doctxt is not NULL) or (doctags is not NULL) or (fieldtxt is not NULL) or (annotxt is not NULL) limit %s, %s' % (l1, l2)
                    docids = ''
                    try:
                        result = root_db.execute(sqla.text(sql)).fetchall()
                    except:
                        log.exception("Fehler beim Abfragen der sys_doctxt")
                    else:
                        if result:
                            r_sum = len(result)
                            r_first = result[0]
                            r_last = result[-1]
                            docids = ",".join([str(x[0]) for x in sorted(result)])

                            sql = "UPDATE sys_doc sd SET sd.doctxt = (Select doctxt from sys_doctxt as ftx WHERE sd.gdocid = ftx.gdocid), sd.doctags = (Select doctags from sys_doctxt as ftx WHERE sd.gdocid = ftx.gdocid), sd.fieldtxt = (Select fieldtxt from sys_doctxt as ftx WHERE sd.gdocid = ftx.gdocid), sd.annotxt = (Select annotxt from sys_doctxt as ftx WHERE sd.gdocid = ftx.gdocid) where sd.gdocid in (%s);" % docids
                            try:
                                root_db.execute(sqla.text(sql))
                            except:
                                log.exception("Fehler beim Aktualisieren der Volltextspalte")
                            else:
                                log.info("%s Volltexte übertragen von Dokument %s - %s" % (r_sum, r_first, r_last))
                    l1 += l2

                root_db.execute("SET unique_checks=1;")
                root_db.execute("SET foreign_key_checks=1;")

                log.info("Volltextindex anlegen doctxt")

                proc = "CALL sp_add_index('%s', 'sys_doc' ,'idx_doctxt', 'FULLTEXT', '(doctxt ASC)')" % my_con.main.db_name
                t = sqla.text(proc).execution_options(autocommit=True)
                try:
                    root_db.execute(t)
                except:
                    log.exception("Fehler beim Ausführen einer SQL-Anweisung: %s" % proc.strip())
                    error = True

                log.info("Volltextindex anlegen doctags")

                proc = "CALL sp_add_index('%s', 'sys_doc' ,'idx_doctags', 'FULLTEXT', '(doctags ASC)')" % my_con.main.db_name
                t = sqla.text(proc).execution_options(autocommit=True)
                try:
                    root_db.execute(t)
                except:
                    log.exception("Fehler beim Ausführen einer SQL-Anweisung: %s" % proc.strip())
                    error = True

                log.info("Volltextindex anlegen fieldtxt")

                proc = "CALL sp_add_index('%s', 'sys_doc' ,'idx_fieldtxt', 'FULLTEXT', '(fieldtxt ASC)')" % my_con.main.db_name
                t = sqla.text(proc).execution_options(autocommit=True)
                try:
                    root_db.execute(t)
                except:
                    log.exception("Fehler beim Ausführen einer SQL-Anweisung: %s" % proc.strip())
                    error = True

                log.info("Volltextindex anlegen annotxt")

                proc = "CALL sp_add_index('%s', 'sys_doc' ,'idx_annotxt', 'FULLTEXT', '(annotxt ASC)')" % my_con.main.db_name
                t = sqla.text(proc).execution_options(autocommit=True)
                try:
                    root_db.execute(t)
                except:
                    log.exception("Fehler beim Ausführen einer SQL-Anweisung: %s" % proc.strip())
                    error = True

                log.info("Übertragung des Volltextes abgeschlossen, Tabelle sys_doctxt wird in old_doctxt umbenannt")

                sql = 'rename table sys_doctxt to old_doctxt'
                try:
                    root_db.execute(sqla.text(sql))
                except:
                    log.exception("Fehler beim Umbennen der sys_doctxt in old_doctxt")


            log.info("Führe Datenbank Update durch...")
            if args.update_searchengine:
                update_files = [args.search_engine]
            else:
                update_files = [args.update_file, args.search_engine]

            for update_file in update_files:
                if update_file:
                    if not os.path.exists(os.path.join(os.path.dirname(sys.argv[0]), update_file)):
                        # log.error("Konnte Skript %s nicht finden" % args.update_file)
                        sys_exit(5, args.update_file)

            for update_file in update_files:
                if update_file:
                    log.info("Lese das %s Skript" % update_file)

                    with codecs.open(os.path.join(os.path.dirname(sys.argv[0]), update_file), "r", encoding="utf8") as f:
                        sql = f.read()

                    for proc in sql.split('$$'):
                        if proc.find('\ufeff') > -1:
                            proc = proc.replace('\ufeff', '')
                        if proc.find('DELIMITER ;') > -1:
                            proc = proc.replace('DELIMITER ;', '')
                        if proc.find('DELIMITER') > -1:
                            proc = proc.replace('DELIMITER', '')
                        if proc.find("CREATE DEFINER = 'sa'@'%'") > -1:
                            proc = proc.replace("CREATE DEFINER = 'sa'@'%'", "CREATE DEFINER = '%s'@'%%'" % my_con.main.user)
                        if proc.find("CREATE DEFINER = 'su'@'%'") > -1:
                            proc = proc.replace("CREATE DEFINER = 'su'@'%'", "CREATE DEFINER = '%s'@'%%'" % my_con.main.suuser)
                        if proc.find("CREATE\r\nDEFINER = 'sa'@'%'") > -1:
                            proc = proc.replace("CREATE\r\nDEFINER = 'sa'@'%'", "CREATE DEFINER = '%s'@'%%'" % my_con.main.user)
                        if proc.find("CREATE\r\nDEFINER = 'su'@'%'") > -1:
                            proc = proc.replace("CREATE\r\nDEFINER = 'su'@'%'", "CREATE DEFINER = '%s'@'%%'" % my_con.main.suuser)
                        if proc.find("_dbname_") > -1:
                            proc = proc.replace("_dbname_", "%s" % my_con.main.db_name)
                        if args.use_mysql57:
                            proc = proc.replace('utf8mb4_unicode_ci', 'utf8_unicode_ci')
                        proc = proc.strip()
                        if len(proc) > 0:
                            t = sqla.text(proc).execution_options(autocommit=True)
                            try:
                                root_db.execute(t)
                            except:
                                log.exception("Fehler beim Ausführen einer SQL-Anweisung: %s" % proc.strip())
                            else:
                                log.info("SQL-Statement verarbeitet")

                    log.info("Updatescript %s verarbeitet" % update_file)

            if args.update_searchengine:    # nach dem einfachen Update der SearchEngine ist hier Schluss
                log.info("Update der SearchEngine durchgeführt")
                sys_exit(0)

            # Änderung des UTF8 Codierung auf UTF8mb4  TODO: Routine wird aufwendiger, da sich die row_size der Tabellen ändert, müssen diese neu aufgebaut werden
            # sql = "SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE table_schema = '%s' AND table_name LIKE 'sys_docidx%%' AND CHARACTER_SET_NAME = 'utf8' AND (COLUMN_TYPE LIKE 'varchar%%' OR COLUMN_TYPE = 'mediumtext')" % my_con.main.db_name
            #
            # try:
            #     res = root_con.execute(sqla.text(sql)).fetchall()
            # except:
            #     log.exception("Fehler beim Abfragen der sys_doc")
            # else:
            #     if res:
            #         for col in res:
            #             table_name = col['TABLE_NAME']
            #             column_name = col['COLUMN_NAME']
            #             column_type = col['COLUMN_TYPE']
            #             if type(column_type) is bytes:
            #                 column_type = column_type.decode()
            #             alter = "ALTER TABLE `%s`.`%s` CHANGE COLUMN `%s` `%s` %s CHARACTER SET 'utf8mb4' COLLATE 'utf8mb4_unicode_ci' NULL DEFAULT NULL" % (my_con.main.db_name, table_name, column_name, column_name, column_type)
            #             try:
            #                 root_con.execute(sqla.text(alter))
            #             except:
            #                 log.exception("Fehler beim Ändern der Codierung auf utf8mb4")
            #                 log.error(alter)

            confile = os.path.join(bfapath, "%s.con" % my_con.main.profile)
            if not os.path.exists(bfapath):
                confile = os.path.join(os.path.dirname(sys.argv[0]), "%s.con" % my_con.main.profile)

            try:
                ini = bitfarm_configobj.ConfigObj(infile=confile, encoding="cp1252", write_empty_values=True)
            except:
                log.exception("Das Profil %s konnte nicht gelesen werden" % confile)
            else:
                if args.gpl or my_con.main.gpl:
                    ini['Server']['edition'] = 'gpl'
                    ini.write()
                else:
                    try:
                        if 'Updater' not in ini:
                            ini['Updater'] = {}
                            ini['Updater']['update_files'] = "%s.con;ViewerV3.exe;Updater.exe;Outlookimport.vbs;movetobitfarm.vbs;copytobitfarm.vbs;psprint.vbs;" \
                                                             "clientspooler.vbs;druckimport.vbs;" \
                                                             "Dokumentationen/Benutzerhandbuch V3.6.2.pdf;HotSearch.exe;bfaMessage.exe;scripts.ini" % my_con.main.profile
                            ini['Updater']['update_dirs'] = "bfaDocLauncher;bfaDocLauncher/psutil;locale;locale/de/LC_Messages;locale/en/LC_Messages;Viewer-Files/Plugins"
                            ini.write()
                    except:
                        log.exception("Fehler beim Aktualisieren der Con-Datei [Updater]")
                try:
                    if 'DLM' not in ini:
                        ini['DLM'] = {}
                        ini['DLM']['expired_dir'] = ""
                        ini['DLM']['check_time'] = ""
                        ini['DLM']['dont_check_expired'] = ""
                        ini.write()
                except:
                    log.exception("Fehler beim Aktualisieren der Con-Datei [DLM]")

            log.info("Update v36 durchgeführt")
            sys_exit(0)

    if args.check_fields or args.merge_fields or args.create_mapping:
        fields = []
        fields.extend(my_con.main.read_addfields())
        log.info("Anzahl Zusatzfelder %i" % len(fields))
        fields = sorted(fields, key=lambda fld: fld.name.lower())
        for field in fields:
            if field.flags is None:
                field.flags = []
            arcids = []
            if field.rights:
                for right in field.rights:
                    if right.arcid not in arcids:
                        arcids.append(right.arcid)
            log.info("Name: %s  Typ:%s [%d] Arcid:%s " % (field.name, bitfarm_doc.RESOLVE_FIELDTYPE[field.type], field.type, ",".join([str(x) for x in arcids]) if arcids else 'keine Zuordnung'))

        if my_con.main.version == 35:
            allfields = [(f.name.lower(), f.type) for f in fields if 'deleted' not in f.flags]
        else:
            allfields = [(f.name.lower(), f.type) for f in fields]  # wenn wir von einer 3.6er Datenbank kommen, dann menrken wir uns auch die deleted Felder,
                                                                    # damit sich die add_Bezeichner nicht ändern
        #allfields = [f for f in allfields if allfields.count(f) > 1]
        doubles = OrderedDict()

        for f in allfields:
            if allfields.count(f) > 1 and f not in doubles:
                doubles[f] = []

        for fname, ftype in doubles.keys():
            arcids = []
            for field in fields:
                if field.name.lower() == fname.lower() and field.type == ftype and field.rights and \
                    ((my_con.main.version == 35 and 'deleted' not in field.flags) or (my_con.main.version == 36)):
                    if field.rights:
                        for right in field.rights:
                            if right.arcid not in arcids:
                                arcids.append(right.arcid)
            doubles[(fname, ftype)] = arcids

        log.info("Doppelte Felder:")
        log.info("Anzahl doppelte Zusatzfelder %i" % len(doubles))
        for field, arcids in doubles.items():
            log.info("Name: %s  Typ:%s [%d]  Anzahl zugeordnete Archive: %s ArcID:%s" % (field[0], bitfarm_doc.RESOLVE_FIELDTYPE[field[1]], field[1], str(len(arcids)), ",".join([str(x) for x in arcids]) if arcids else 'keine Zuordnung'))

    if args.merge_fields:
        if my_con.main.version == 35:
            # log.error("Felder zusammenfassen geht nur ab Version 3.6")
            sys_exit(34)

        merge_fields = args.merge_fields.split(';')
        for field in merge_fields:
            if field[-1] != ':' and field.rfind(':') > -1:
                fname = field[:field.rfind(':')]
                ftype = int(field[field.rfind(':') + 1:])
            else:
                fname = field
                ftype = None

            found = False
            for dname, dtype in doubles:
                if fname.lower() == dname.lower() and ftype is None:
                    found = True
                    break
                elif fname.lower() == dname.lower() and ftype == dtype:
                    found = True
                    break

            if not found:
                if ftype is not None:
                    log.error("Das angegebene Feld %s vom Typ %s konnte nicht in den doppelten Feldern gefunden werden" % (fname, bitfarm_doc.RESOLVE_FIELDTYPE[ftype]))
                else:
                    log.error("Das angegebene Feld %s konnte nicht in den doppelten Feldern gefunden werden" % fname)
                continue
            else:
                if ftype is None:
                    ftype = dtype

                log.info("Doppeltes Feld %s Typ %s gefunden in folgenden Archiven %s" % (dname, bitfarm_doc.RESOLVE_FIELDTYPE[ftype], ",".join([str(x) for x in doubles[(dname, dtype)]]) if doubles[(dname, dtype)] else 'keine Zuordnung'))

                # jetzt prüfen ob es konflikte innerhalb der werte gibt
                dfields = []
                for addfield in fields:     # jetzt die felder sammeln und die query zusammenbauen
                    if addfield.name.lower() == fname.lower() and addfield.type == ftype and 'deleted' not in addfield.flags:
                        dfields.append(addfield)

                sql = 'select x0.gdocid, %s from sys_docidx_0 as x0' % ",".join([x.add for x in dfields])
                for addfield in dfields:
                    if addfield.indextableid > 0 and 'sys_docidx_%d' % addfield.indextableid not in sql:
                        sql += ' left join sys_docidx_%d as x%d on x0.gdocid = x%d.gdocid' % (addfield.indextableid, addfield.indextableid, addfield.indextableid)
                sql += ' where' + ' and'.join([' %s is not Null' % x.add for x in dfields])

                try:
                    result = my_con.main.db.execute(sql).fetchall()
                except:
                    log.exception("Fehler beim Suchen von Konflikten innerhalb der doppelten Felder")
                    continue
                else:
                    if result:
                        ids = []
                        for row in result:
                            if row[0] is None:
                                continue
                            values = []
                            for i in range(1, len(row)):
                                if row[i] not in values:
                                    values.append(row[i])
                            if len(values) > 1:     # sind die gefundenen werte alle gleich, so steht dieser genau einmal in values
                                ids.append(row[0])
                        if len(ids) > 0:
                            log.error("Achtung: bei folgenden Dokumenten gibt es Konflikte zwischen den doppelten Feldern.")
                            log.error("Nur eins der Felder sollte einen Wert tragen. Gdocid: %s" % ",".join([str(x) for x in ids]))
                            continue
                    else:
                        log.info("Keine Konflikte bei den Zusatzfeldwerten")

                        # die felder finden und zusammenführen, neues addfield erzeugen und speichern

                    newfield = bitfarm_doc.BitfarmAddfield()
                    newfield.name = fname
                    newfield.type = ftype
                    newfield.rights = bitfarm_doc.BitfarmFieldRights()
                    for add in fields:
                        if add.name.lower() == fname.lower() and add.type == ftype:
                            newfield.rights.add([x for x in add.rights])
                            newfield.items.extend(add.items)

                    create_addfield(my_con, newfield)

                    if newfield.id is None:
                        log.error("Fehler beim Anlegen des neuen, zusammengefassten Zusatzfeldes")
                        continue

                    # jetzt die Werte pro Feld holen und ins neue übertragen

                    for add in dfields:
                        sql = 'select gdocid, %s from sys_docidx_%d where %s is not Null' % (add.add, add.indextableid, add.add)
                        try:
                            result = my_con.main.db.execute(sql).fetchall()
                        except:
                            log.exception("Fehler beim Abfragen der alten Zusatzfeldwerte")
                        else:
                            if result:
                                for row in result:
                                    if row[0] is None:
                                        continue
                                    if type(row[1]) is str:
                                        sql = "insert into sys_docidx_%d (gdocid, %s) values (%i, '%s') on duplicate key update %s = '%s'"  % (newfield.indextableid, newfield.add, row[0], row[1], newfield.add, row[1])
                                    elif type(row[1]) is datetime.datetime:
                                        sql = "insert into sys_docidx_%d (gdocid, %s) values (%i, '%s') on duplicate key update %s = '%s'"  % (newfield.indextableid, newfield.add, row[0], row[1].strftime('%Y-%m-%d-'), newfield.add, row[1].strftime('%Y-%m-%d-'))
                                    else:
                                        sql = "insert into sys_docidx_%d (gdocid, %s) values (%i, %s) on duplicate key update %s = %s"  % (newfield.indextableid, newfield.add, row[0], row[1], newfield.add, row[1])
                                    try:
                                        my_con.main.db.execute(sql)
                                    except:
                                        log.exception("Fehler beim Übertragen der Zusatzfeldwerte")
                                    else:
                                        log.info("Zusatzfeldwerte gdocid %i von %s nach %s übertragen" % (row[0], add.add, newfield.add))

                    # jetzt alte felder auf deleted setzen

                    for add in dfields:
                        sql = "update sys_addfields set flags = 'deleted' where addfieldid = %d" % add.id
                        try:
                            my_con.main.db.execute(sql)
                        except:
                            log.exception("Fehler beim Setzen des Gelöscht-Flags der zusammengeführten Felder %s" % add.add)
                        else:   # deleted gesetzt? dann aus der sys_addfieldxs entfernen
                            sql = 'delete from sys_addfieldxs where fieldid = %d' % add.id
                            try:
                                my_con.main.db.execute(sql)
                            except:
                                log.exception("Fehler beim Entfernen der Zuordnung der zusammengeführten Felder %s" % add.add)

        log.info("merge_addfields durchgeführt...")
        sys_exit(0)

    if args.create_mapping:
        dest_con = bitfarm_proc.BitfarmConnections(args.check, gpl=args.create_gpl)
        dest_con.create_db_engine()
        if dest_con.main.engine is None:
            # log.error("Es konnte keine Datenbank-Engine erstellt werden. %s" % args.check)
            sys_exit(14, args.check)
        dest_con.connect_db()
        if dest_con.main.db is None:
            # log.error("Es konnte keine Verbindung zur Datenbank erstellt werden. %s" % args.check)
            sys_exit(13, args.check)

        dest_fields = dest_con.main.read_addfields()
        field_mapping = {}

        log.info("Aktualisiere das addfield mapping in der sys_config...")
        log.info("Felder im Quellarchiv %s" % len(fields))
        log.info("Felder im Zielarchiv %s" % len(dest_fields))

        for field in fields:
            if 'deleted' in field.flags and my_con.main.version == 35:
                continue
            for dest_field in dest_fields:
                # in 3.6 werden 'deleted' Felder jetzt mitgeführt
                # if 'deleted' in dest_field.flags:
                #     continue
                if field.name.lower() == dest_field.name.lower() and field.type == dest_field.type:
                    # wir nehmen jedes quellfeld + arcid ins mapping und weisen es einem zielfeld zu
                    # sollten wir schon eine zuordnung haben (d.h ein doppeltes, gleichnamiges zielfeld wurde gefunden)
                    # dann ersetzen wir die zuordnung, wenn das zielfeld im selben archiv gefunden wird
                    if field.rights:
                        for right in field.rights:
                            if (field.add, right.arcid) in field_mapping:  # doppelte zuordnung gefunden, wir suchen jetzt eine genaue archivzuordnung
                                if dest_field.rights:
                                    for drights in dest_field.rights:
                                        if drights.arcid == right.arcid:
                                            field_mapping[(field.add, right.arcid)] = dest_field.add
                            else:
                                field_mapping[(field.add, right.arcid)] = dest_field.add

        try:
            dest_con.main.db.execute('delete from sys_config where name like "add_%%"')
        except:
            log.exception("Fehler beim Löschen bestehender addfield mappings in der sys_config")

        for field in field_mapping.keys():
            sql = 'select * from sys_config where name = "%s"' % (field[0] + ',' + str(field[1]))

            result = dest_con.main.db.execute(sql).fetchall()

            if result:
                sql = 'update sys_config set value = "%s" where name = "%s"' % (field_mapping[field], field[0] + ',' + str(field[1]))
            else:
                sql = 'insert into sys_config (name, value) values ("%s", "%s")' % (field[0] + ',' + str(field[1]), field_mapping[field])

            try:
                dest_con.main.db.execute(sql)
            except sqla.exc.IntegrityError:
                log.warning("Eintrag in der sys_config schon vorhanden %s = %s" % (field, field_mapping[field]))
            except:
                log.exception("Fehler beim Schreiben in die sys_config %s = %s" % (field, field_mapping[field]))
            else:
                log.debug("Eintrag in die sys_config geschrieben %s = %s" % (field, field_mapping[field]))
        log.info("Zusatzfeldmapping in der sys_config aktualisiert")
        sys_exit(0)

    if (args.migrate_to or args.only_trigger) and my_con.main.version in [35, 36] and not args.sync and not only_mode and not args.gpl:
        # anstatt das SUPER Privileg vorrauszusetzen machen wir eine Root Verbindung zur Quelldatenbank (-p Profil)
        if my_con is None:
            my_con = bitfarm_proc.BitfarmConnections(args.profile, gpl=args.gpl)
        # für ein Update muss ein Profil angegeben sein, wir holen die Verbindungsdaten daher aus dem Profil
        root_server = my_con.main.server if not args.use_localhost else 'localhost'
        root_port = my_con.main.port

        if not args.root_user_src:
            sys.stdout.write('Bitte geben Sie den Rootuser des Profils %s ein: ' % args.profile)
            sys.stdout.flush()
            user = ''
            while True:
                x = msvcrt.getch()
                x = x.decode()
                if x == '\r':
                    break
                if ord(x) == 8:     #Backspace
                    user = user[:-1]
                    sys.stdout.write(x)
                    sys.stdout.write(' ')
                    sys.stdout.write(x)
                    sys.stdout.flush()
                    x = ''
                sys.stdout.write(x)
                sys.stdout.flush()
                user += x
            root_user2 = user
            sys.stdout.write('\n')
            sys.stdout.flush()
        else:
            root_user2 = args.root_user_src

        if not args.root_pass_src:
            sys.stdout.write('Bitte geben Sie das Passwort des Rootusers (%s) für das Profil %s ein: ' % (root_user2, args.profile))
            sys.stdout.flush()
            password = ''
            while True:
                x = msvcrt.getch()
                x = x.decode()
                if x == '\r':
                    break
                if ord(x) == 8:     #Backspace
                    password = password[:-1]
                    sys.stdout.write(x)
                    sys.stdout.write(' ')
                    sys.stdout.write(x)
                    sys.stdout.flush()
                    x = ''
                else:
                    sys.stdout.write('*')
                    sys.stdout.flush()
                password += x
            sys.stdout.write('\n')
            sys.stdout.flush()
            root_pass2 = password
        else:
            root_pass2 = args.root_pass_src

        if not args.use_localhost:
            m = re.match(ipv4_pattern, root_server)

            if m:
                sys.stdout.write('Soll die Verbindung über localhost oder IP %s hergestellt werden? (l/i): ' % root_server)
            else:
                sys.stdout.write('Soll die Verbindung über localhost oder Adresse %s hergestellt werden? (l/a): ' % root_server)
            sys.stdout.flush()
            inp = get_input()
            # while True:
            #     x = msvcrt.getch()
            #     x = x.decode()
            #     if x == '\r':
            #         break
            #     if ord(x) == 8:     #Backspace
            #         inp = inp[:-1]
            #         sys.stdout.write(x)
            #         sys.stdout.write(' ')
            #         sys.stdout.write(x)
            #         sys.stdout.flush()
            #         x = ''
            #     sys.stdout.write(x)
            #     sys.stdout.flush()
            #     inp += x
            #
            # sys.stdout.write('\n')
            # sys.stdout.flush()

            if inp == 'l' or inp == 'localhost':
                root_server = 'localhost'

        constr = my_con.main._makeconstr(root_user2, root_pass2, root_server, root_port, dbadmin=True)
        conargs = my_con.main._makeconargs()

        if not constr:
            # log.error("Es konnte kein Connection String aufgebaut werden")
            sys_exit(7)

        engine = sqla.create_engine(constr, echo=False, connect_args=conargs, pool_size=10)
        try:
            root_con2 = engine.connect()
        except sqla.exc.OperationalError as e:
            if hasattr(e, 'orig') and e.orig.args[0] == 1045:
                    # log.error("Es konnte sich nicht am Datenbankserver angemeldet werden, User oder Password falsch.")
                    log.error(e._message())
                    sys_exit(4)
            elif hasattr(e, 'orig') and e.orig.args[0] == 2003:
                    # log.error("Es konnte sich nicht am Datenbankserver angemeldet werden, Server nicht erreichbar.")
                    log.error(e._message())
                    sys_exit(4)
            else:
                log.exception("Es konnte sich nicht am Datenbankserver angemeldet werden")
                sys_exit(4)
        except:
            log.exception("Es konnte sich nicht am Datenbankserver angemeldet werden")
            sys_exit(4)
        else:
            if root_con2 is None:
                log.exception("Es konnte sich nicht am Datenbankserver angemeldet werden")
                sys_exit(4)

            log.info("Verbindung zu %s:%s hergestellt" % (root_server, root_port))

        try:
            root_con2.execute('USE %s;' % my_con.main.db_name)
        except:
            # log.exception("Konnte Datenbank %s nicht festlegen" % my_con.main.db_name)
            sys_exit(41, my_con.main.db_name)
        else:
            log.info("Datenbank %s wurde festgelegt" % my_con.main.db_name)

        log.info("Trigger in Quelldatenbank anlegen")

        sql = "CREATE TABLE IF NOT EXISTS sys_mgr " \
              "(mgrid bigint(20) UNSIGNED NOT NULL AUTO_INCREMENT, " \
              "gdocid bigint(20),  tstamp timestamp DEFAULT " \
              "CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP, " \
              "PRIMARY KEY (mgrid), UNIQUE INDEX UK_sys_mgr_gdocid (gdocid)) AUTO_INCREMENT = 1"

        try:
            root_con2.execute(sqla.text(sql))
        except:
            log.exception("Fehler beim Anlegen der sys_mgr")
        else:
            log.info("sys_mgr angelegt")

        if my_con.main.version == 35:
            sql = "select arc_tblname, arc_id from sys_arc where type = 1 and arc_tblname <> '' "
            try:
                result = root_con2.execute(sqla.text(sql)).fetchall()
            except:
                log.exception("Fehler beim Abfragen des sys_arc")
            else:
                if result:
                    for row in result:
                        if row[0] is None:
                            continue
                        sql = "DROP TRIGGER IF EXISTS trg_au_%s" % row[0]
                        try:
                            root_con2.execute(sqla.text(sql))
                        except:
                            log.exception("Fehler beim Anlegen der sys_mgr")

                        sql = "CREATE DEFINER = '%s'@'%%' TRIGGER trg_au_%s AFTER UPDATE " \
                              "ON arc_%s FOR EACH ROW BEGIN INSERT INTO sys_mgr (gdocid) VALUES (NEW.gdoc_id) " \
                              "ON DUPLICATE KEY UPDATE gdocid = NEW.gdoc_id; END" % (my_con.main.user, row[0], row[0])
                        try:
                            root_con2.execute(sqla.text(sql))
                        except:
                            log.exception("Fehler beim Anlegen der sys_mgr")
                        else:
                            log.info("TRIGGER trg_au_%s angelegt" % row[0])

                        sql = "DROP TRIGGER IF EXISTS trg_hu_%s" % row[0]
                        try:
                            root_con2.execute(sqla.text(sql))
                        except:
                            log.exception("Fehler beim Anlegen der sys_mgr")

                        sql = "CREATE DEFINER = '%s'@'%%' TRIGGER trg_hu_%s AFTER UPDATE " \
                              "ON hst_%s FOR EACH ROW BEGIN DECLARE var_gdocid bigint; " \
                              "set var_gdocid = (SELECT gdoc_id FROM sys_gdocid sg WHERE sg.arc_id = %d AND sg.doc_id = NEW.DOC_ID); " \
                              " INSERT INTO sys_mgr (gdocid) VALUES (var_gdocid) ON DUPLICATE KEY UPDATE gdocid = var_gdocid; END" % (
                              my_con.main.user, row[0], row[0], row[1])
                        try:
                            root_con2.execute(sqla.text(sql))
                        except:
                            log.exception("Fehler beim Anlegen der sys_mgr")
                        else:
                            log.info("TRIGGER trg_hu_%s angelegt" % row[0])

                        sql = "DROP TRIGGER IF EXISTS trg_hi_%s" % row[0]
                        try:
                            root_con2.execute(sqla.text(sql))
                        except:
                            log.exception("Fehler beim Anlegen der sys_mgr")

                        sql = "CREATE DEFINER = '%s'@'%%' TRIGGER trg_hi_%s AFTER INSERT " \
                              "ON hst_%s FOR EACH ROW BEGIN DECLARE var_gdocid bigint; " \
                              "set var_gdocid = (SELECT gdoc_id FROM sys_gdocid sg WHERE sg.arc_id = %d AND sg.doc_id = NEW.DOC_ID); " \
                              " INSERT INTO sys_mgr (gdocid) VALUES (var_gdocid) ON DUPLICATE KEY UPDATE gdocid = var_gdocid; END" % (
                              my_con.main.user, row[0], row[0], row[1])
                        try:
                            root_con2.execute(sqla.text(sql))
                        except:
                            log.exception("Fehler beim Anlegen der sys_mgr")
                        else:
                            log.info("TRIGGER trg_hi_%s angelegt" % row[0])

                    sql = "DROP TRIGGER IF EXISTS trg_u_gdocid"
                    try:
                        root_con2.execute(sqla.text(sql))
                    except:
                        log.exception("Fehler beim Droppen des Triggers trg_u_gdocid")

                    sql = "CREATE DEFINER = '%s'@'%%' TRIGGER trg_u_gdocid AFTER UPDATE " \
                          "ON sys_gdocid FOR EACH ROW BEGIN " \
                          " INSERT INTO sys_mgr (gdocid) VALUES (NEW.gdoc_id) ON DUPLICATE KEY UPDATE gdocid = NEW.gdoc_id; END" % my_con.main.user
                    try:
                        root_con2.execute(sqla.text(sql))
                    except:
                        log.exception("Fehler beim Anlegen der sys_mgr")
                    else:
                        log.info("TRIGGER trg_u_gdocids angelegt")

                    sql = "DROP TRIGGER IF EXISTS trg_i_gdocid"
                    try:
                        root_con2.execute(sqla.text(sql))
                    except:
                        log.exception("Fehler beim Droppen des Triggers trg_i_gdocid")

                    sql = "CREATE DEFINER = '%s'@'%%' TRIGGER trg_i_gdocid AFTER INSERT " \
                          "ON sys_gdocid FOR EACH ROW BEGIN " \
                          " INSERT INTO sys_mgr (gdocid) VALUES (NEW.gdoc_id) ON DUPLICATE KEY UPDATE gdocid = NEW.gdoc_id; END" % my_con.main.user
                    try:
                        root_con2.execute(sqla.text(sql))
                    except:
                        log.exception("Fehler beim Anlegen der sys_mgr")
                    else:
                        log.info("TRIGGER trg_i_gdocids angelegt")

        elif my_con.main.version == 36:
            sql = "DROP TRIGGER IF EXISTS trg_u_gdocid"
            try:
                root_con2.execute(sqla.text(sql))
            except:
                log.exception("Fehler beim Droppen des Triggers trg_u_gdocid")

            sql = "CREATE DEFINER = '%s'@'%%' TRIGGER trg_u_gdocid AFTER UPDATE " \
                  "ON sys_doc FOR EACH ROW BEGIN " \
                  " INSERT INTO sys_mgr (gdocid) VALUES (NEW.gdocid) ON DUPLICATE KEY UPDATE gdocid = NEW.gdocid; END" % my_con.main.user
            try:
                root_con2.execute(sqla.text(sql))
            except:
                log.exception("Fehler beim Anlegen der sys_mgr")
            else:
                log.info("TRIGGER trg_u_gdocids angelegt")

            sql = "DROP TRIGGER IF EXISTS trg_i_gdocid"
            try:
                root_con2.execute(sqla.text(sql))
            except:
                log.exception("Fehler beim Droppen des Triggers trg_i_gdocid")

            sql = "CREATE DEFINER = '%s'@'%%' TRIGGER trg_i_gdocid AFTER INSERT " \
                  "ON sys_doc FOR EACH ROW BEGIN " \
                  " INSERT INTO sys_mgr (gdocid) VALUES (NEW.gdocid) ON DUPLICATE KEY UPDATE gdocid = NEW.gdocid; END" % my_con.main.user
            try:
                root_con2.execute(sqla.text(sql))
            except:
                log.exception("Fehler beim Anlegen der sys_mgr")
            else:
                log.info("TRIGGER trg_i_gdocids angelegt")

            sql = "SELECT table_name FROM INFORMATION_SCHEMA.COLUMNS where table_schema = '%s' and table_name like 'sys_docidx_%%' group by table_name" % my_con.main.db_name

            try:
                result = root_con2.execute(sqla.text(sql)).fetchall()
            except:
                log.exception("Fehler beim Abfragen des sys_idx Tabellen")
            else:
                if result:
                    for row in result:
                        if row[0] is None:
                            continue
                        sql = "DROP TRIGGER IF EXISTS trg_idxu_%s" % row[0]
                        try:
                            root_con2.execute(sqla.text(sql))
                        except:
                            log.exception("Fehler beim Anlegen der sys_mgr")

                        sql = "CREATE DEFINER = '%s'@'%%' TRIGGER trg_idxu_%s AFTER UPDATE " \
                              "ON %s FOR EACH ROW BEGIN INSERT INTO sys_mgr (gdocid) VALUES (NEW.gdocid) " \
                              "ON DUPLICATE KEY UPDATE gdocid = NEW.gdocid; END" % (my_con.main.user, row[0], row[0])
                        try:
                            root_con2.execute(sqla.text(sql))
                        except:
                            log.exception("Fehler beim Anlegen der sys_mgr")
                        else:
                            log.info("TRIGGER trg_idxu_%s angelegt" % row[0])

                        sql = "DROP TRIGGER IF EXISTS trg_idxi_%s" % row[0]
                        try:
                            root_con2.execute(sqla.text(sql))
                        except:
                            log.exception("Fehler beim Anlegen der sys_mgr")

                        sql = "CREATE DEFINER = '%s'@'%%' TRIGGER trg_idxi_%s AFTER INSERT " \
                              "ON %s FOR EACH ROW BEGIN " \
                              " INSERT INTO sys_mgr (gdocid) VALUES (NEW.gdocid) ON DUPLICATE KEY UPDATE gdocid = NEW.gdocid; END" % (my_con.main.user, row[0], row[0])
                        try:
                            root_con2.execute(sqla.text(sql))
                        except:
                            log.exception("Fehler beim Anlegen der sys_mgr")
                        else:
                            log.info("TRIGGER trg_idxi_%s angelegt" % row[0])


        if args.only_trigger and not args.reload_sysmgr:
            log.info("Es sollten nur Trigger angelegt werden. Beende...")
            sys_exit(0)

    if args.reload_sysmgr:
        # if my_con.main.version != 35:
        #     log.error("die sys_mgr kann nur in einer v3.5 Datenbank aktualisiert werden.")
        #     sys.exit(48)

        log.info("Die sys_mgr Tabelle wird aktualisiert")

        sql = "truncate sys_mgr"
        try:
            my_con.main.db.execute(sqla.text(sql))
        except:
            log.exception("Fehler beim Leeren der sys_mgr")
        else:
            log.info("sys_mgr Tabelle geleert")

        if my_con.main.version == 35:
            sql = "select * from sys_arc where type = 1"
            syncids = []
            try:
                result = my_con.main.db.execute(sqla.text(sql)).fetchall()
            except:
                log.exception("Fehler bei der Abfrage der sys_arc")
            else:
                if result:
                    for row in result:
                        if row[0] is None:
                            continue
                        sql = "select gdoc.gdoc_id from arc_%s as arc left join sys_gdocid as gdoc on arc.doc_id = gdoc.doc_id where gdoc.arc_id = %s and tstamp >= '%s'" % (row['ARC_TBLName'],row['ARC_ID'], args.reload_sysmgr)
                        try:
                            docs = my_con.main.db.execute(sqla.text(sql)).fetchall()
                        except:
                            log.exception("Fehler bei der Abfrage der arc-Tabelle %s" % row['ARC_TBLName'])
                        else:
                            if docs:
                                for doc in docs:
                                    if type(doc[0]) is not int:
                                        log.error("Gdocid hat falsches Format %s" % doc[0])
                                    elif doc[0] not in syncids:
                                        syncids.append(doc[0])

                        sql = "select gdoc.gdoc_id from hst_%s as arc left join sys_gdocid as gdoc on arc.doc_id = gdoc.doc_id where gdoc.arc_id = %s and tstamp >= '%s'" % (row['ARC_TBLName'],row['ARC_ID'], args.reload_sysmgr)
                        try:
                            docs = my_con.main.db.execute(sqla.text(sql)).fetchall()
                        except:
                            log.exception("Fehler bei der Abfrage der hst-Tabelle %s" % row['ARC_TBLName'])
                        else:
                            if docs:
                                for doc in docs:
                                    if type(doc[0]) is not int:
                                        log.error("Gdocid hat falsches Format %s" % doc[0])
                                    elif doc[0] not in syncids:
                                        syncids.append(doc[0])

                if syncids:
                    ids = ",".join(["(%s)" % str(x) for x in syncids])
                    sql = "insert into sys_mgr (gdocid) values %s" % ids
                    try:
                        my_con.main.db.execute(sqla.text(sql))
                    except:
                        log.exception("Fehler beim Schreiben in die sys_mgr")
                    else:
                        log.info("sys_mgr Tabelle aktualisiert")

        elif my_con.main.version == 36:
            syncids = []
            sql = "select gdocid from sys_doc where (docdate >= '%s' or modified >= '%s') and gdocid <> 0" % (args.reload_sysmgr, args.reload_sysmgr)
            try:
                docs = my_con.main.db.execute(sqla.text(sql)).fetchall()
            except:
                log.exception("Fehler bei der Abfrage der sys_doc-Tabelle")
            else:
                if docs:
                    for doc in docs:
                        if doc[0] not in syncids:
                            syncids.append(doc[0])

            sql = "select gdocid from sys_history where tstamp >= '%s' and gdocid <> 0" % args.reload_sysmgr
            try:
                docs = my_con.main.db.execute(sqla.text(sql)).fetchall()
            except:
                log.exception("Fehler bei der Abfrage der sys_history-Tabelle")
            else:
                if docs:
                    for doc in docs:
                        if doc[0] not in syncids:
                            syncids.append(doc[0])

            if syncids:
                ids = ",".join(["(%s)" % str(x) for x in syncids])
                sql = "insert into sys_mgr (gdocid) values %s" % ids
                try:
                    my_con.main.db.execute(sqla.text(sql))
                except:
                    log.exception("Fehler beim Schreiben in die sys_mgr")
                else:
                    log.info("sys_mgr Tabelle aktualisiert")
        sys_exit(0)

    if args.check and not args.migrate_to:
        dest_con = bitfarm_proc.BitfarmConnections(args.check, gpl=args.create_gpl)
        dest_con.create_db_engine()
        if dest_con.main.engine is None:
            # log.error("Es konnte keine Datenbank-Engine erstellt werden. %s" % args.check)
            sys_exit(14, args.check)
        dest_con.connect_db()
        if dest_con.main.db is None:
            # log.error("Es konnte keine Verbindung zur Datenbank erstellt werden. %s" % args.check)
            sys_exit(13, args.check)

    if args.migrate_to:
        if args.exclude_fields:
            field_exclude = args.exclude_fields.split(";")
            field_exclude = map(lambda x: x.lower(), field_exclude)
            # field_exclude = map(lambda x: x.decode('cp1252').lower(), field_exclude)

        dest_con = bitfarm_proc.BitfarmConnections(args.migrate_to, gpl=args.create_gpl)
        dest_con.create_db_engine()
        if dest_con.main.engine is None:
            # log.error("Es konnte keine Datenbank-Engine erstellt werden (Zielprofil). %s" % args.migrate_to)
            sys_exit(14, args.migrate_to)
        dest_con.connect_db()
        if dest_con.main.db is None:
            # log.error("Es konnte keine Verbindung zur Datenbank erstellt werden (Zielprofil). %s" % args.migrate_to)
            sys_exit(13, args.migrate_to)
        dest_con.main.undistributed = my_con.main.undistributed
        dest_proc = bitfarm_proc.BitfarmDocProcessing(dest_con)
        if my_con.main.version > dest_con.main.version:
            # log.error("Es kann nicht auf eine niedrigere Version migriert werden")
            sys_exit(9)
        if args.clone_to:
            log.info("Clone %s (Version %i) nach %s (Version %i)" % (my_con.main.profile, my_con.main.version, dest_con.main.profile, dest_con.main.version))
        else:
            log.info("Migriere %s (Version %i) nach %s (Version %i)" % (my_con.main.profile, my_con.main.version, dest_con.main.profile, dest_con.main.version))
        if args.only_docs:
            log.info("Es werden nur Dokumente übertragen!")
        if args.only_bookmarks:
            log.info("Es werden nur die Lesezeichen übertragen")
        if args.only_resub:
            log.info("Es wird nur die Wiedervorlage übertragen")
        if args.only_annos:
            log.info("Es werden nur die Annotation Texte übertragen")
        if args.only_templates:
            log.info("Es werden nur Templates übersetzt")
            if my_con.main.version != 35:
                # log.warning("Es können nur Templates der Version 35 übersetzt werden")
                sys_exit(19)
        if args.only_mwl:
            log.info("Es werden nur Vertragsmanagementdateien übersetzt")
            if my_con.main.version != 35:
                # log.warning("Es können nur Vertragsmanagementdateien der Version 35 übersetzt werden")
                sys_exit(20)
        if args.only_jobfiles:
            log.info("Es werden nur Jobdateien übersetzt")
            if my_con.main.version != 35:
                # log.warning("Es können nur Jobdateien der Version 35 übersetzt werden")
                sys_exit(28)
        if args.only_fields:
            log.info("Es werden nur die Zusatzfeldinhalte übertragen")
        if args.only_comments:
            log.info("Es werden nur Kommentare übertragen")

        # wir setzen für alle Fälle den SQL-Mode für Einträge mit Primary Key 0 (zb sys_group)
        sql = "SET SESSION sql_mode='NO_AUTO_VALUE_ON_ZERO'"
        try:
            dest_con.main.db.execute(sqla.text(sql))
        except:
            log.exception("Fehler beim Setzen von NO_AUTO_VALUE_ON_ZERO")
        else:
            log.info("NO_AUTO_VALUE_ON_ZERO gesetzt")

        # only_mode = args.only_docs or args.only_bookmarks or args.only_resub or args.only_annos or args.only_templates or args.only_mwl

        # für die lesezeichenmigration benötigen wir eine SearchProcessing Instanz
        search_proc = bitfarm_proc.BitfarmSearchProcessing(dest_con)

        if not only_mode:
            mig_file = os.path.join(bfapath, my_con.main.profile + '.mgr')
            try:
                shutil.copyfile(dest_con.main.confile, mig_file)
            except:
                log.exception("Achtung: Migrationsprofil konnte nicht angelegt werden %s" % mig_file)
            else:
                log.info("Migrationsprofil anlegen %s")

                log.info("Profildateien zusammenführen %s > %s" % (mig_file, dest_con.main.confile))
                ini = bitfarm_configobj.ConfigObj(infile=mig_file, encoding="cp1252",
                                                  create_empty=True, write_empty_values=True)

                ini["Scripts"]["storages"] = '%bfastore0%'
                ini["Scripts"]["archive_rs"] = args.path_rs
                ini["Scripts"]["archive_nrs"] = args.path_nrs

                ini["Scripts"]['serverprogram'] = my_con.main.serverprogram
                ini["Scripts"]['clientprogram'] = my_con.main.clientprogram
                ini["Scripts"]['clientspooler'] = my_con.main.clientspooler
                ini["Scripts"]['uncclientspooler'] = my_con.main.uncclientspooler
                ini["Scripts"]['omnipage'] = my_con.main.omnipage
                ini["Scripts"]['ghostscript'] = my_con.main.ghostscript
                ini["Scripts"]['gsview'] = my_con.main.gsview
                ini["Scripts"]['smtpserver'] = my_con.main.smtpserver
                ini["Scripts"]['smtpport'] = my_con.main.smtpport
                ini["Scripts"]['sendername'] = my_con.main.sendername
                ini["Scripts"]['authuser'] = my_con.main.authuser
                ini["Scripts"]['authpass'] = my_con.main.authpass
                ini["Scripts"]['adminmail'] = my_con.main.adminmail
                tpath = os.path.join(bfapath, 'templates')
                if not os.path.exists(tpath):
                    ini["Scripts"]['templates'] = my_con.main.templates
                else:
                    ini["Scripts"]['templates'] = tpath + '\\'
                ini["Scripts"]['workflowfile'] = my_con.main.workflowfile
                ini["Scripts"]['openofficepath'] = my_con.main.openofficepath
                ini["Scripts"]['undistributed'] = my_con.main.undistributed
                ini["Scripts"]['unverteilt'] = my_con.main.undistributed
                ini["Scripts"]['workflows'] = my_con.main.workflows_dir
                ini["Scripts"]['tempdir'] = my_con.main.tempdir
                ini['Backup'] = {}
                ini['Backup']['backuppath'] = my_con.main.backuppath
                ini['Backup']['sqldatapath'] = my_con.main.sqldatapath
                ini['Backup']['backuparchiv'] = my_con.main.backuparchive
                ini['Backup']['backupnrsarchiv'] = my_con.main.backupnrsarchive
                ini['Backup']['backupversion'] = my_con.main.backupversion
                ini['Backup']['backupworkflows'] = my_con.main.backupworkflows
                ini['Backup']['backupsvnrepository'] = my_con.main.backupsvnrepository
                ini['Backup']['backupmysqllog'] = my_con.main.backupmysqllog
                ini['Backup']['filesize'] = my_con.main.filesize
                ini['Backup']['backupzeitraum'] = my_con.main.backupinterval
                ini['Backup']['mysqlservice'] = my_con.main.backupmysqlservice if my_con.main.backupmysqlservice is not None else 'MySQL57'
                ini['Backup']['use7zip'] = my_con.main.backupuse7zip

                ini["Options"] = {}
                if args.migrate_to and my_con.main.version == 35:
                    ini["Options"]["usesearch35"] = True
                    ini["Options"]["usestatus35"] = True

                ini['Options']['logdir'] = my_con.main.logdir
                ini['Options']['savepass'] = my_con.main.savepass
                ini['Options']['manuscanplugin'] = my_con.main.manuscanplugin
                ini['Options']['importerplugin'] = my_con.main.importerplugin
                ini['Options']['SvnInfo'] = my_con.main.svninfo
                ini['Options']['LinkChars'] = my_con.main.linkchars

                ini.write()

        log.info("Schritt 1: Archivstruktur ermitteln")

        archives = my_con.main.read_archives()
        # TODO: Wiedervorlageliste, delegierte WVL und ggf anderes wird zukünftig auch in der sys_arc gespeichert
        cnt = 1
        rowcount = len(archives)
        log.info("Anzahl der Archive %i" % rowcount)
        if not only_mode:
            log.info("Archivstruktur übertragen")
            if dest_con.main.version == 36:
                # if dest_con.main.gpl:   # wir suchen die userid des dmsadmin
                #     userid_admin = 2
                #     sql = "select userid from sys_user where flags & 129 = 129"
                #     try:
                #         res = dest_con.main.db.execute(sqla.text(sql)).fetchone()
                #     except:
                #         log.exception("Fehler bei Abfrage des DMSAdmin in der sys_user")
                #     else:
                #         if not res:
                #             flags = '137'
                #             if args.create_gpl:
                #                 flags = '169'
                #                 sql = 'INSERT INTO sys_user (username, displayname, userpass, flags) VALUES ("%s", "%s", "%s", %s)' % (
                #                 'DMSAdmin', 'DMSAdmin', conobj.bitencrypt('bitfarm'), flags)
                #             else:
                #                 sql = 'INSERT INTO sys_user (username, displayname, userpass, flags) VALUES ("%s", "%s", "%s", %s)' % (
                #                 'DMSAdmin', 'DMSAdmin', conobj.bitencrypt(args.admin_pass), flags)
                #             try:
                #                 res = dest_con.main.db.execute(sqla.text(sql))
                #             except:
                #                 log.exception("Fehler beim Anlegen des Admin-Users")
                #             else:
                #                 log.info("DMSAdmin User angelegt")
                #                 userid_admin = res.lastrowid
                #         else:
                #             userid_admin = res[0]

                # if my_con.main.version == 35:  # bei migration haben wir eine wurzel, dann muss das Standard-Wurzelarchiv weg
                sql = 'Delete from sys_arc where arcid = 1'     # TODO: etwas sicherere Abfrage einbauen
                try:
                    dest_con.main.db.execute(sqla.text(sql))
                except:
                    log.exception("Fehler beim Entfernen des Standard Wurzelarchives")

                # dann müssen wir das Wiedervorlage-Archiv auf eine andere arcid bringen
                found_wvl = False
                wvlid = 0
                for arc in archives:
                    if arc.id > wvlid:
                        wvlid = arc.id
                    if arc.type == 2:
                        found_wvl = True
                wvlid += 1

                #  haben wir in der Liste der Archive ein Wiedervorlagearchiv, dann entfernen wir den Basiseintrag aus der Grundinstallation
                if found_wvl:
                    sql = 'delete from sys_arc where type = 2'
                else:   #  ansonsten zählen wir den Basiseintrag auf die höchste Arcid hoch, damit wir mit keinem Archiv kollidieren
                    sql = 'update sys_arc set arcid = %s where type = 2' % wvlid
                try:
                    dest_con.main.db.execute(sqla.text(sql))
                except:
                    log.exception("Fehler beim Umziehen der Wiedervorlage")

                for arc in archives:
                    if not notstdout:
                        sys.stdout.write("Verarbeite Archiv %i von %i\r" % (cnt, rowcount))
                        sys.stdout.flush()
                    cnt += 1

                    if arc.lgrid == arc.id:  # gleichlautende arcid und lgrid darf nicht sein
                        log.warning("lgrid bei Archiv %s korrigiert (gleicher Wert)" % arc.arcname)
                        arc.lgrid = 0

                    if arc.lgrid is None:  # gleichlautende arcid und lgrid darf nicht sein
                        log.warning("lgrid bei Archiv %s korrigiert (Wert = NULL)" % arc.arcname)
                        arc.lgrid = 0

                    if my_con.main.version == 35:
                        arc.flags |= 1  # das flag enabled setzen

                    vals = {dest_con.main.sys_arc.c.arcid: arc.id,
                            dest_con.main.sys_arc.c.lgrid: arc.lgrid,
                            dest_con.main.sys_arc.c.arcname: arc.arcname,
                            dest_con.main.sys_arc.c.tblname: arc.tblname,
                            dest_con.main.sys_arc.c.flags: arc.flags,
                            dest_con.main.sys_arc.c.backpic: arc.backpic,
                            dest_con.main.sys_arc.c.icon: arc.icon,
                            dest_con.main.sys_arc.c.type: arc.type,
                            dest_con.main.sys_arc.c.options: arc.options,
                            dest_con.main.sys_arc.c.defdocname: arc.defdocname}
                    sql = sqla.insert(dest_con.main.sys_arc, values=vals)

                    try:
                        dest_con.main.db.execute(sql)
                    except:
                        log.exception("Fehler beim Erstellen eines Archiveintrages")
                    else:
                        log.debug("Archiv angelegt %s" % arc.arcname)

                    arc_rights = my_con.main.read_rights(arcid=arc.id, single_rights=True)
                    if arc_rights:
                        log.debug("Archivberechtigung %s" % str(arc_rights))
                        for right in arc_rights:
                            userid = right.user.id if right.user is not None else 0
                            groupid = right.group.id if right.group is not None else 0

                            if groupid != 0:
                                userid = 0

                            right.rights = right.rights & ~ 2121728     # die Berechtigung apread, apwrite und apmodify wird entfernt. In v36 gibt es keine expliziten Berechtigungen
                                                                        # auf private Annos
                            vals = {dest_con.main.sys_arcxs.c.arcid: arc.id,
                                    dest_con.main.sys_arcxs.c.userid: userid,
                                    dest_con.main.sys_arcxs.c.groupid: groupid,
                                    dest_con.main.sys_arcxs.c.rights: right.rights}

                            ins = sqla.insert(dest_con.main.sys_arcxs, values=vals)
                            try:
                                dest_con.main.db.execute(ins)
                            except:
                                log.exception("Fehler beim Erstellen der Archivberechtigung")
                            else:
                                log.debug("Berechtigung geschrieben arcid %s userid %s groupid %s rights %s" % (arc.id, userid, groupid, right.rights))

        log.info("Schritt 2: Usermanagment ermitteln")

        users = my_con.main.read_users()

        cnt = 1
        rowcount = len(users)
        log.info("Anzahl der User %i" % rowcount)

        if not only_mode:
            log.info("Usermanagment übertragen")
            if dest_con.main.version == 36:
                new_users = dest_con.main.read_users()
                maxid = 1
                if len(new_users) > 0:  # mehr als Systemuser
                    for user in users:
                        if user.id > maxid:
                            maxid = user.id

                    for user in new_users:
                        # try:    # sys_groupuser updaten
                        #     dest_con.main.db.execute('update sys_groupuser set userid = %d where userid = %d' % (user.id + maxid, user.id))
                        # except:
                        #     log.exception("Fehler beim Aktualisieren der GroupUser-Tabelle (%s)" % dest_con.main.profile)
                        user.id += maxid
                        users.append(user)

                # ziel-tabelle leeren
                try:
                    dest_con.main.db.execute('truncate sys_user')
                except:
                    log.exception("Fehler beim Leeren der User-Tabelle (%s)" % dest_con.main.profile)

                # falls irgendwer in der quelldatenbank den systemuser gelöscht haben sollte
                found = False
                for user in users:
                    if user.name == "System":
                        found = True
                        break

                if not found:
                    user = bitfarm_doc.BitfarmUser()
                    user.id = 1
                    user.name = "System"
                    user.displayname = "System"
                    user.flags = ['enabled']
                    user.password = args.systemuserpass
                    users.append(user)

                for user in users:
                    if not notstdout:
                        sys.stdout.write("Verarbeite User %i von %i\r" % (cnt, rowcount))
                        sys.stdout.flush()
                    cnt += 1
                    if user.name == "System":
                        for flag in list(user.flags):
                            if not flag:
                                user.flags.remove(flag)
                        if 'enabled' not in user.flags:
                            user.flags.append('enabled')

                    if args.admin:
                        if user.name in args.admin.split(';'):
                            user.flags.append('admin')

                    if dest_con.main.gpl:   # migrieren wir in eine 3.6er GPL sind auch dort zukünftig keine leeren Passwörter erlaubt
                        if not user.password:
                            user.flags.append('must_change_password')

                    vals = {dest_con.main.sys_user.c.userid: user.id,
                            dest_con.main.sys_user.c.username: user.name,
                            dest_con.main.sys_user.c.displayname: user.displayname,
                            dest_con.main.sys_user.c.userpass: user.password,
                            dest_con.main.sys_user.c.usermail: user.email,
                            dest_con.main.sys_user.c.flags: ",".join(user.flags)
                            }

                    sql = sqla.insert(dest_con.main.sys_user, values=vals)
                    try:
                        dest_con.main.db.execute(sql)
                    except sqla.exc.IntegrityError:
                        log.error("IntegrityError: %s" % sys.exc_info()[1])
                    except:
                        log.exception("Fehler beim Erstellen eines User-Eintrages %s" % user.name)
                    else:
                        log.debug("User %s angelegt" % user.name)

                    if 'dmsadmin' in user.flags:
                        userid_admin = user.id

                # dmsadmin auf alle Archive in der GPL berechtigen
                if dest_con.main.gpl:
                    for arc in archives:
                        vals = {dest_con.main.sys_arcxs.c.arcid: arc.id,
                                dest_con.main.sys_arcxs.c.userid: userid_admin,
                                dest_con.main.sys_arcxs.c.groupid: 0,
                                dest_con.main.sys_arcxs.c.rights: bitfarm_doc.ROLE_ADMIN}

                        ins = sqla.insert(dest_con.main.sys_arcxs, values=vals)
                        try:
                            dest_con.main.db.execute(ins)
                        except sqla.exc.IntegrityError as e:
                            if hasattr(e, 'orig') and e.orig.args[0] == 1062:
                                log.info("Berechtigung für DMSAdmin für arcid %s schon vorhanden" % arc.id)
                        except:
                            log.exception("Fehler beim Erstellen der Archivberechtigung für DMSADmin")
                        else:
                            log.debug("Berechtigung für DMSADmin geschrieben arcid %s userid %s groupid %s rights %s" % (
                            arc.id, userid, groupid, right.rights))

        groups = my_con.main.read_groups()

        cnt = 1
        rowcount = len(groups)
        log.info("Anzahl der Gruppen %i" % rowcount)

        if not only_mode:
            if dest_con.main.version == 36:
                for group in groups:
                    if not notstdout:
                        sys.stdout.write("Verarbeite Gruppe %i von %i\r" % (cnt, rowcount))
                        sys.stdout.flush()

                    # if my_con.main.version == 35:
                    #     if group.name.lower() == 'alle' or group.name.lower() == 'all':
                    #         group.name = 'Alle (%s)' % my_con.main.profile   # sollte es eine Gruppe Alle geben, benennen wir die um,
                    #                                                         # behalten sie aber bei, da sie ggf zur konfiguration benötigt wurde
                    cnt += 1

                    if my_con.main.version == 36:
                        if group.id == 0:
                            log.debug("Migration von 3.6er System, überspringe Gruppe 0 (schon vorhanden")
                            continue

                    vals = {dest_con.main.sys_group.c.groupid: group.id,
                            dest_con.main.sys_group.c.groupname: group.name}
                    sql = sqla.insert(dest_con.main.sys_group, values=vals)
                    try:
                        dest_con.main.db.execute(sql)
                    except sqla.exc.IntegrityError:
                        log.error("IntegrityError: %s" % sys.exc_info()[1])
                    except:
                        log.exception("Fehler beim Erstellen eines Group-Eintrages")
                    else:
                        log.debug("Gruppe %s angelegt" % group.name)

                    for mem in group.members:
                        pass
                    vals = [{'groupid': group.id, 'userid': x.id if type(x) is bitfarm_doc.BitfarmUser else x} for x in group.members]
                    sql = sqla.insert(dest_con.main.sys_groupuser)
                    try:
                        dest_con.main.db.execute(sql, vals)
                    except sqla.exc.IntegrityError:
                        log.error("IntegrityError: %s" % sys.exc_info()[1])
                    except:
                        log.exception("Fehler beim Erstellen eines GroupUser-Eintrages")
                    else:
                        log.debug("Gruppenmitgliedschaften angelegt")

                # wir fügen noch jeden user der group 0 hinzu
                vals = [{'groupid': 0, 'userid': x.id} for x in users]
                sql = sqla.insert(dest_con.main.sys_groupuser)
                try:
                    dest_con.main.db.execute(sql, vals)
                except sqla.exc.IntegrityError:
                    log.error("IntegrityError: %s" % sys.exc_info()[1])
                except:
                    log.exception("Fehler beim Erstellen der Gruppe 0 Einträge")
                else:
                    log.debug("Benutzer zur Gruppe 0 hinzugefügt")

            log.info("Vorgesetzte (sys_superior) übertragen")

            sql = sqla.select([my_con.main.sys_superior])

            result = my_con.main.db.execute(sql).fetchall()

            if result:
                cnt = 1
                rowcount = len(result)
                log.info("Anzahl der Einträge (sys_superior) %i" % len(result))
                if dest_con.main.version == 36:
                    for row in result:
                        if row[0] is None:
                            continue
                        if not notstdout:
                            sys.stdout.write("Verarbeite Eintrag %i von %i\r" % (cnt, rowcount))
                            sys.stdout.flush()
                        cnt += 1
                        log.debug(row)
                        if 'SUP_ID' in row:  # 3.5er Spaltennamen
                            supid = row.SUP_ID
                        else:
                            supid = row.supid
                        if 'sup_usr_id' in row:  # 3.5er Spaltennamen
                            sup_userid = row.sup_usr_id
                        else:
                            sup_userid = row.sup_userid
                        if 'inf_usr_id' in row:
                            inf_userid = row.inf_usr_id
                        else:
                            inf_userid = row.inf_userid
                        options = row.options
                        status = row.status
                        info = row.info

                        vals = {dest_con.main.sys_superior.c.supid: supid,
                                dest_con.main.sys_superior.c.sup_userid: sup_userid,
                                dest_con.main.sys_superior.c.inf_userid: inf_userid,
                                dest_con.main.sys_superior.c.options: options,
                                dest_con.main.sys_superior.c.status: status,
                                dest_con.main.sys_superior.c.info: info}

                        ins = sqla.insert(dest_con.main.sys_superior, values=vals)
                        try:
                            dest_con.main.db.execute(ins)
                        except:
                            log.exception("Fehler beim Anlegen einer Eintrags in der sys_superior supid= %i" % supid)
                        else:
                            log.debug("Eintrag in sys_superior angelegt supid= %i" % supid)

        log.info("Schritt 3: Alle Zusatzfelder ermitteln")

        # wir erstellen erstmal eine Liste an Zusatzfeldern
        src_fields = []
        src_fields.extend(my_con.main.read_addfields())
        log.info("Anzahl Quell-Zusatzfelder %i" % len(src_fields))

        if my_con.main.version == 35:   # bei Felder aus einer 35er Datenbank muss das Encoding angepasst werden
            for field in src_fields:
                    try:
                        field.name = field.name.encode('latin1').decode('cp1252')
                    except:
                        pass
                    for item in field.items:
                        try:
                            item = item.encode('latin1').decode('cp1252')
                        except:
                            pass

        new_fields = []
        if dest_con.main.version == 36:
            new_fields.extend(dest_con.main.read_addfields())
            for field in new_fields:
                field.oldfields = []

        # die beiden Listen werden abgeglichen, gleichnamige Felder gleichen Typs werden nicht noch mal angelegt
        for src in src_fields:
            found = False
            if my_con.main.version == 35:   # zusammenfassung aber nur, wenn die Quelldatenbank v3.5 ist
                for dest in new_fields:
                    if src.name.lower() == dest.name.lower() and src.type == dest.type and src.name.lower() not in field_exclude:
                        log.info("Doppeltes Zusatzfeld gefunden %s, wird zusammengefasst" % src.name)
                        dest.rights.add([x for x in src.rights])
                        for item in src.items:
                            if item not in dest.items:
                                dest.items.append(item)
                        for right in src.rights:
                            if (src.add, right.arcid) not in dest.oldfields:
                                dest.oldfields.append((src.add, right.arcid))
                        found = True
                        break  # wenn es das Feld gibt, muss nicht nochmal hinzugefügt werden, wir verlassen die innere Schleife
            if not found:
                add = bitfarm_doc.BitfarmAddfield()
                log.info("Zusatzfeld erzeugt %s" % src.name)
                add.name = src.name
                if my_con.main.version == 35:
                    add.add = None
                else:                   # wenn wir von 3.6 kommen, dann behalten wir die ID und den add_Bezeichner des Feldes
                    add.id = src.id
                    add.add = src.add
                add.type = src.type
                add.checkprogram = src.checkprogram
                add.values_query = src.values_query
                add.flags = src.flags
                if add.flags is None:
                    add.flags = ['enabled']
                if 'deleted' not in add.flags:  # wenn es nicht deleted ist
                    if 'enabled' not in add.flags:
                        add.flags.append('enabled')
                add.oldfields = []
                for right in src.rights:
                    if (src.add, right.arcid) not in add.oldfields:
                        add.oldfields.append((src.add, right.arcid))
                if src.name.lower() in field_exclude:
                    add.description = ""
                    for right in src.rights:
                        for arc in archives:
                            if right.arcid == arc.id:
                                if "Archiv:%s " % arc.arcname not in add.description:
                                    add.description += "Archiv:%s " % arc.arcname
                                    break
                else:
                    add.description = src.description
                add.mode = src.mode
                if add.type in [1,5,6,7,8,12]:  # Datums und Zahlenfelder auf Suchmethode 'exakt' setzen
                    add.mode = 0
                add.options = src.options
                if add.type == 2:           # auswahlfelder sind nach der migration per default strict
                    add.options |= 8
                if add.type not in [2, 3, 11]:   # kein auswahlfeld? kann auch nicht strict sein
                    add.options &= ~8
                add.rights = src.rights
                if my_con.main.version == 35 and args.gpl: # 14.10.2024 aus 35 kommen kommasepareted
                    if add.items is not None and isinstance(src.items, list) and len(src.items) > 0:
                        add.items = src.items[0].split(",")
                    else:
                        add.items = src.items
                else:
                    add.items = src.items
                # 14.10.2024    add.items = src.items
                add.pos = None
                add.indextableid = src.indextableid

                new_fields.append(add)

        log.info("Anzahl Ziel-Zusatzfelder %i" % len(new_fields))

        if not only_mode:
            log.info("Schritt 4: Alle Zusatzfelder übertragen")
            cnt = 1
            rowcount = len(new_fields)

            # die Felder werden in die Zieltabelle eingefügt
            if dest_con.main.version == 36:
                for field in new_fields:
                    if not notstdout:
                        sys.stdout.write("Verarbeite Zusatzfeld %i von %i\r" % (cnt, rowcount))
                        sys.stdout.flush()
                    cnt += 1

                    create_addfield(dest_con, field)

        field_mapping = {}
        if only_mode:   # im only_mode holen wir uns die zuordnung aus der sys_config
            sql = "select * from sys_config where name like 'add_%%'"
            try:
                result = dest_con.main.db.execute(sql).fetchall()
            except:
                log.exception("Fehler beim Lesen in die sys_config")
            else:
                if result:
                    for row in result:
                        if row[0] is None:
                            continue
                        old = row[1].split(",")
                        old = (old[0], int(old[1]))
                        field_mapping[old] = row[2]
        else:
            for field in new_fields:
                for old in field.oldfields:
                    field_mapping[old] = field.add
                    if args.migrate_to and not only_mode:  # bei der Migration Feldzurodnung in die sys_config schreiben
                        sql = 'insert into sys_config (name, value) values ("%s", "%s")' % (old[0] + ',' + str(old[1]), field.add)
                        try:
                            dest_con.main.db.execute(sql)
                        except sqla.exc.IntegrityError:
                            log.warning("Eintrag in der sys_config schon vorhanden %s = %s" % (old, field.add))
                        except:
                            log.exception("Fehler beim Schreiben in die sys_config %s = %s" % (old, field.add))
                        else:
                            log.debug("Eintrag in die sys_config geschrieben %s = %s" % (old, field.add))

        if args.only_fields:
            # my_proc = bitfarm_proc.BitfarmDocProcessing(my_con)
            i = 0
            result = True
            while result:
                if my_con.version == 35:
                    sql = 'select * from sys_gdocid limit %i, 1000' % (i * 1000)
                else:
                    sql = 'select * from sys_doc limit %i, 1000' % (i * 1000)
                try:
                    result = my_con.main.db.execute(sql).fetchall()
                except:
                    log.exception("Fehler bei Abfrage der sys_gdocid" if my_con.version == 35 else "Fehler bei Abfrage des sys_doc")

                if result:
                    log.info("Verarbeite %s Dokumente" % len(result))
                    for row in result:
                        if row[0] is None:
                            continue
                        doc = bitfarm_doc.BitfarmDocument()
                        if not my_proc.load(doc, row[0]):
                            log.error("Es konnte kein Dokument mit der ID %s im Quellarchiv gefunden werden" % row[0])
                        else:
                            dest_doc = bitfarm_doc.BitfarmDocument()
                            if not dest_proc.load(dest_doc, row[0], level=['base','fields']):
                                log.error("Es konnte das Dokument  %s im Zielarchiv nicht gefunden werden" % row[0])
                            else:
                                dest_modified = False
                                for src_field in doc.fields.values():
                                    if not src_field.value:
                                        continue
                                    if my_con.version == 35:
                                        if not (src_field.add, doc.arcid) in field_mapping:
                                            log.warning("Zuordnung nicht gefunden %s %s" % (src_field.add, doc.arcid))
                                        else:
                                            for dest_field in dest_doc.fields.values():
                                                if dest_field.add == field_mapping[(src_field.add, doc.arcid)]:
                                                    if dest_field.type != src_field.type:
                                                        if not ((src_field.type == 1 and dest_field.type == 12) or (src_field.type == 2 and dest_field.type == 11)):
                                                            log.warning("Das Zusatzfeld %s des Dokuments %s hat einen unterschiedlichen Typ" % (dest_field.name, doc.gdocid))
                                                            continue
                                                    if not dest_field.value:
                                                        dest_field.value = src_field.value
                                                        dest_modified = True
                                                        break
                                    elif my_con.main.version == 36:
                                        for dest_field in dest_doc.fields.values():
                                            if dest_field.add == src_field.add:
                                                if dest_field.type != src_field.type:
                                                    if not ((src_field.type == 1 and dest_field.type == 12) or (src_field.type == 2 and dest_field.type == 11)):
                                                        log.warning("Das Zusatzfeld %s des Dokuments %s hat einen unterschiedlichen Typ" % (dest_field.name, doc.gdocid))
                                                        continue
                                                if not dest_field.value:
                                                    dest_field.value = src_field.value
                                                    dest_modified = True
                                                    break

                                if dest_modified:
                                    for dest_field in dest_doc.fields.values():
                                        if dest_field.modified:
                                            if not dest_proc.save_addfield(dest_field, dest_doc.gdocid, dest_doc.profile):
                                                log.error("Fehler beim Speichern des Zusatzfelds %s bei Dokument %s" % (dest_field.name, dest_doc.gdocid))
                                            else:
                                                log.debug("Zusatzfeld %s des Dokument %s wurde aktualisiert" % (dest_field.name, dest_doc.gdocid))

                i += 1

            log.info("Abgleich der Felder durchgeführt")
            sys_exit(0)

        log.info("Schritt 5: Alle Statusfelder ermitteln")

        statefields = my_con.main.read_statefields()
        if statefields:
            log.debug("Anzahl der in der Quelle vorhandenen Statusfelder %i" % len(statefields))
        else:
            log.warning("Keine Statusfelder gefunden")

        if my_con.main.version == 35:  # wir müssen die Statusfelder überarbeiten, wenn diese eine Funktion hatten
            for state in statefields:
                try:
                    state.name = state.name.encode('latin1').decode('cp1252')
                except:
                    pass
                if state.func == 255:
                    state.func = 0
                else:
                    state.func += 1
                if state.addfield is not None and state.addfield != 'keinem Feld':  # wir haben einen Zusatzfeldnamen
                    for field in new_fields:  # den suchen wir in unseren aktuellen Zusatzfeldern
                        if field.name == state.addfield:  # Zusatzfeld gefunden
                            for right in field.rights._rights:
                                if right.arcid == state.rights._rights[0].arcid:  # wir haben ein passendes Zusatzfeld im passenden Archiv gefunden
                                    state.addfieldid = field.id
                                    break

        new_statefields = []
        # das erste Statusfeld in einer Datenbank ist IMMER 'auf Wiedervorlage' (entspricht 'in Bearbeitung' in v35)

        if dest_con.main.version == 36:
            new_statefields.extend(dest_con.main.read_statefields())
            log.debug("Anzahl der schon im Ziel vorhandene Statusfelder %i" % len(new_statefields))

            # if len(new_statefields) == 0 or new_statefields[0].name != 'auf Wiedervorlage':
            #     inresub = bitfarm_doc.BitfarmStatefield()
            #     inresub.bitmask = 1
            #     inresub.indexcolumnid = 0
            #     inresub.name = "auf Wiedervorlage"
            #     inresub.function = 0
            #     inresub.addfieldid = 0
            #     inresub.rights = bitfarm_doc.BitfarmFieldRights()
            #     new_statefields.insert(0, inresub)

            # bei der migration von 35 auf 36 wird der 'auf Wiedervorlage' Status allen Archiven zugeordnet
            # if my_con.main.version == 35 and new_statefields[0].name == 'auf Wiedervorlage':
            #     sql = 'select arc_id from sys_arc where type = 1'
            #     try:
            #         result = my_con.main.db.execute(sql)
            #     except:
            #         log.exception("Fehler bei der Abfrage der sys_arc v35")
            #     else:
            #         if result:
            #             for row in result:
            #                 inresubright = bitfarm_doc.BitfarmFieldRight()
            #                 inresubright.read = True
            #                 inresubright.arcid = row[0]
            #                 inresubright.fieldpos = 1
            #                 new_statefields[0].rights.add(inresubright)

        for state in statefields:
            found = False
            for field in new_statefields:  # mit der Liste der vorhandenen Statefields abgleichen
                if state.name.lower() == field.name.lower():  # wenn unser statefield schon vorhanden ist
                    log.debug("Doppeltes Statusfeld %s wird zusammengeführt" % state.name)
                    field.rights.add([x for x in state.rights])
                    if state.func != 0 and field.func != 0:
                        if field.func != state.func and field.addfieldid != state.addfieldid:
                            log.warning("Achtung: beim Zusammenführen des Statusfeldes %s hat es einen Konflikt mit der Funktionskonfiguration gegeben" % state.name)
                            ids = []
                            for x in [str(x.arcid) for x in state.rights]:
                                if x not in ids:
                                    ids.append(x)

                            log.warning("die betroffenen Archiv-IDs %s" % ",".join(ids))
                    elif state.func != 0:
                        field.func = state.func
                        field.addfieldid = state.addfieldid
                    found = True
                    break
            if not found:
                new_statefields.append(state)

        log.info("Anzahl der neuen Statusfelder %i" % len(new_statefields))

        for state in new_statefields:
            log.debug("Statusfeld: %s" % state)
            log.debug("Rechte: %s" % state.rights)

        if not only_mode:
            cnt = 1
            rowcount = len(new_statefields)

            if dest_con.main.version == 36:
                bitshift = 0
                indexcolumnid = 0
                for state in new_statefields:
                    if not notstdout:
                        sys.stdout.write("Verarbeite Statusfeld %i von %i\r" % (cnt, rowcount))
                        sys.stdout.flush()
                    cnt += 1
                    sql = sqla.select([dest_con.main.sys_statefields.c.displayname])
                    try:
                        result = dest_con.main.db.execute(sql).fetchall()
                    except:
                        log.exception("Fehler bei der Abfrage der vorhandenen Statusfelder")
                        continue
                    if state.name not in [x[0] for x in result]:
                        state.bitmask = (2 << bitshift)  # bitmasken beginnen bei 2, bit eins ist das 35er 'Auf Wiedervorlage'
                        vals = {dest_con.main.sys_statefields.c.displayname: state.name,
                                dest_con.main.sys_statefields.c.bitmask: state.bitmask,
                                dest_con.main.sys_statefields.c.addfieldid: state.addfieldid,
                                dest_con.main.sys_statefields.c.function: state.func,
                                dest_con.main.sys_statefields.c.indexcolumnid: indexcolumnid}
                        sql = sqla.insert(dest_con.main.sys_statefields, values=vals)

                        try:
                            result = dest_con.main.db.execute(sql)
                        except:
                            log.exception("Fehler beim Anlegen eines Statusfeldes %s" % state.name)
                        else:
                            stateid = result.inserted_primary_key[0]

                            log.info("Statusfeld [%i] angelegt %s" % (stateid, state.name))
                            bitshift += 1
                            if bitshift >= 62:  # wir können hier nur 63bits für die bitmaske benutzen, da delphi's int64 nicht unsigned ist
                                bitshift = 0
                                indexcolumnid += 1
                                if 'docstate_%i' % indexcolumnid not in dest_con.main.sys_doc.columns:
                                    sql = "ALTER TABLE `sys_doc` ADD COLUMN `docstate_%i` BIGINT(20) UNSIGNED DEFAULT 0" % indexcolumnid
                                    try:
                                        dest_con.main.db.execute(sql)
                                    except:
                                        log.exception("Fehler beim Anlegen einer weiteren Status-Spalte doc_state_%i in der sys_doc" % indexcolumnid)
                                        continue
                                    else:
                                        dest_con.main.sys_doc = sqla.Table('sys_doc', sqla.MetaData(), autoload=True, autoload_with=dest_con.main.engine)
                    else:
                        log.debug("Statusfeld schon in Zieldatenbank vorhanden %s" % state.name)
                        bitshift += 1
                        stateid = state.id

                    if state.rights is not None:
                        for right in state.rights:
                            if right.user is not None:
                                userid = right.user.id
                            else:
                                userid = 0
                            if right.group is not None:
                                groupid = right.group.id
                            else:
                                groupid = 0
                            vals = {dest_con.main.sys_statefieldxs.c.arcid: right.arcid,
                                    dest_con.main.sys_statefieldxs.c.fieldid: stateid,
                                    dest_con.main.sys_statefieldxs.c.userid: userid,
                                    dest_con.main.sys_statefieldxs.c.groupid: groupid,
                                    dest_con.main.sys_statefieldxs.c.pos: right.fieldpos,
                                    dest_con.main.sys_statefieldxs.c.rights: right.rights}
                            sql = sqla.insert(dest_con.main.sys_statefieldxs, values=vals)
                            try:
                                dest_con.main.db.execute(sql)
                            except sqla.exc.IntegrityError:
                                log.warning("Zurodnung für dieses Feld schon vorhanden %i-%i-%i-%i" % (right.arcid, stateid, userid, groupid))
                            except:
                                log.exception("Fehler beim Zuordnen eines Statusfeldes %s zu einem Archiv %s" % (state.name, right.arcid))
                            else:
                                log.debug("Statusfeld %s zu Archiv %s zugeordnet, userid %s groupid %s" % (state.name, right.arcid, userid, groupid))

        # weitere Tabellen, sys_timer, sys_doclink

        if not only_mode and not args.clone_to and not args.import_documents and not args.import_archive or args.only_resub:
            log.info("Schritt 6: Workflows (sys_timer) übertragen")

            wf_dir = my_con.main.workflows_dir if my_con.main.workflows_dir else os.path.join(bfapath, 'workflows')
            wf_dir36 = wf_dir + '36'

            if args.sync:
                if args.sync[0].lower() == 'sys_mgr':
                    sql = 'truncate sys_timer'
                    try:
                        dest_con.main.db.execute(sql)
                    except:
                        log.exception("Fehler beim Leeren des sys_timer")

                    if my_con.main.version == 35 and os.path.exists(wf_dir36):
                        shutil.rmtree(wf_dir36, ignore_errors=True)

                    if my_con.main.version == 35:
                        from_obj = sqla.join(my_con.main.sys_timer, my_con.main.sys_gdocid, onclause=(
                        sqla.and_(my_con.main.sys_timer.c.arc_id == my_con.main.sys_gdocid.c.arc_id,
                                  my_con.main.sys_timer.c.doc_id == my_con.main.sys_gdocid.c.doc_id)))
                        sql = sqla.select([my_con.main.sys_timer, my_con.main.sys_gdocid], from_obj=from_obj, use_labels=True)
                    elif my_con.main.version == 36:
                        sql = sqla.select([my_con.main.sys_timer], use_labels=True)
                else:
                    sql = None
            else:
                if my_con.main.version == 35:
                    from_obj = sqla.join(my_con.main.sys_timer, my_con.main.sys_gdocid, onclause=(
                    sqla.and_(my_con.main.sys_timer.c.arc_id == my_con.main.sys_gdocid.c.arc_id,
                              my_con.main.sys_timer.c.doc_id == my_con.main.sys_gdocid.c.doc_id)))
                    sql = sqla.select([my_con.main.sys_timer, my_con.main.sys_gdocid], from_obj=from_obj, use_labels=True)
                elif my_con.main.version == 36:
                    sql = sqla.select([my_con.main.sys_timer], use_labels=True)

            if my_con.main.version == 35 and not os.path.exists(wf_dir36):
                try:
                    os.mkdir(wf_dir36)
                except:
                    log.exception("Konnte workflows36-Verzeichnis nicht anlegen")
                    wf_dir = None

            if sql is not None:
                result = my_con.main.db.execute(sql).fetchall()

                if result:
                    cnt = 1
                    rowcount = len(result)
                    log.info("Anzahl der Wiedervorlagen %i" % len(result))
                    # if dest_con.main.version == 36:
                    for row in result:
                        if row[0] is None:
                            continue
                        if not notstdout:
                            sys.stdout.write("Verarbeite Wiedervorlagen %i von %i\r" % (cnt, rowcount))
                            sys.stdout.flush()
                        cnt += 1
                        log.debug(row)

                        if 'sys_timer_arc_id' in row:       # 3.5er Spaltennamen
                            wf_arcid = row.sys_timer_arc_id
                        else:
                            wf_arcid = None
                        if 'sys_timer_doc_id' in row:       # 3.5er Spaltennamen
                            wf_docid = row.sys_timer_doc_id
                        else:
                            wf_docid = None
                        if 'sys_timer_erinnerung' in row:   # 3.5er Spaltennamen
                            alarm = row.sys_timer_erinnerung
                        else:
                            alarm = row.sys_timer_alarm
                        if 'sys_timer_termin' in row:       # 3.5er Spaltennamen
                            date = row.sys_timer_termin
                        else:
                            date = row.sys_timer_date
                        if 'sys_timer_Optionen' in row:
                            options = row.sys_timer_Optionen
                        else:
                            options = row.sys_timer_options
                        if 'sys_timer_WKommentar' in row:       # nicht in GPL 3.5
                            comment = row.sys_timer_WKommentar
                        else:
                            if 'sys_timer_comment' in row:
                                comment = row.sys_timer_comment
                            else:
                                comment = None
                        if 'sys_gdocid_gdoc_id' in row:         # nicht in GPL 3.5
                            gdocid = row.sys_gdocid_gdoc_id
                        else:
                            if 'sys_timer_gdocid' in row:
                                gdocid = row.sys_timer_gdocid
                            else:
                                gdocid = None
                        if 'sys_timer_tim_id' in row:
                            timid = row.sys_timer_tim_id
                        else:
                            timid = row.sys_timer_timid
                        if 'sys_timer_tim_ref' in row:
                            timref = row.sys_timer_tim_ref
                        else:
                            timref = row.sys_timer_timref
                        if 'sys_timer_grp_id' in row:
                            groupid = row.sys_timer_grp_id
                        else:
                            groupid = row.sys_timer_groupid
                        if 'sys_timer_usr_id_work' in row:
                            userid_work = row.sys_timer_usr_id_work
                        else:
                            userid_work = row.sys_timer_userid_work
                        if 'sys_timer_usr_id_shed' in row:
                            userid_shed = row.sys_timer_usr_id_shed
                        else:
                            userid_shed = row.sys_timer_userid_shed

                        info = row.sys_timer_info
                        status = row.sys_timer_status

                        if my_con.main.version == 35:  # 35 Status auf 36 bringen (1 offen, 2 in arbeit, 4 fertig)
                            if status == 0:  # notwendig, um über searchobjekt genauer den status abfragen zu können
                                status = 1  # und eine brauchbare bitmaske zu haben, für zB rückfragen (status 8) die
                            elif status == 1:  # offen (8 | 1) oder in arbeit (8 | 2) sein können
                                status = 2
                            elif status == 2:
                                status = 4

                        vals = {dest_con.main.sys_timer.c.timid: timid,
                                dest_con.main.sys_timer.c.timref: timref,
                                dest_con.main.sys_timer.c.gdocid: gdocid,
                                dest_con.main.sys_timer.c.groupid: groupid,
                                dest_con.main.sys_timer.c.userid_work: userid_work,
                                dest_con.main.sys_timer.c.userid_shed: userid_shed,
                                dest_con.main.sys_timer.c.status: status,
                                dest_con.main.sys_timer.c.date: date,
                                dest_con.main.sys_timer.c.alarm: alarm,
                                dest_con.main.sys_timer.c.options: options,
                                dest_con.main.sys_timer.c.info: info,
                                dest_con.main.sys_timer.c.comment: comment}

                        ins = sqla.insert(dest_con.main.sys_timer, values=vals)
                        try:
                            dest_con.main.db.execute(ins)
                        except:
                            log.exception("Fehler beim Anlegen einer Wiedervorlage timid= %i" % timid)
                        else:
                            log.debug("Wiedervorlage timid= %i angelegt" % timid)

                            if wf_dir and wf_arcid and wf_docid:
                                if os.path.exists(os.path.join(wf_dir, 'workflow%s.%s-%s.txt' % (wf_arcid, wf_docid, timref))):
                                    with open(os.path.join(wf_dir36, 'workflow%s-%s.txt' % (gdocid, timref)), 'w') as f36:
                                        with open(os.path.join(wf_dir, 'workflow%s.%s-%s.txt' % (wf_arcid, wf_docid, timref)), 'r') as f:
                                            for line in f:
                                                if '%s.%s' % (wf_arcid, wf_docid) in line:
                                                    f36.write("%s\n" % gdocid)
                                                else:
                                                    f36.write(line)


        # if not only_mode and not args.clone_to or args.only_resub or args.sync:
            log.info("Schritt 7: Wiedervorlage Historie (sys_timerhist) übertragen")

            # if my_con.main.version == 35:
            if my_con.main.sys_timerhist is None:
                my_con.main.sys_timerhist = sqla.Table('sys_timerhist', my_con.main.meta, autoload=True)

            if args.sync:
                if args.sync[0].lower() == 'sys_mgr':
                    sql = 'truncate sys_timerhist'
                    try:
                        dest_con.main.db.execute(sql)
                    except:
                        log.exception("Fehler beim Leeren des sys_timerhist")
                    if my_con.main.version == 35:
                        sql = sqla.select([my_con.main.sys_timerhist], use_labels=True).group_by(my_con.main.sys_timerhist.c.tim_Ref, my_con.main.sys_timerhist.c.hist_id)
                    else:
                        sql = sqla.select([my_con.main.sys_timerhist], use_labels=True).group_by(my_con.main.sys_timerhist.c.timref, my_con.main.sys_timerhist.c.histid)
                    result = my_con.main.db.execute(sql).fetchall()
                else:
                    result = None
            else:
                if my_con.main.version == 35:
                    sql = sqla.select([my_con.main.sys_timerhist], use_labels=True).group_by(my_con.main.sys_timerhist.c.tim_Ref, my_con.main.sys_timerhist.c.hist_id)
                else:
                    sql = sqla.select([my_con.main.sys_timerhist], use_labels=True).group_by(my_con.main.sys_timerhist.c.timref, my_con.main.sys_timerhist.c.histid)
                result = my_con.main.db.execute(sql).fetchall()

            if result:
                for row in result:
                    if row[0] is None:
                        continue
                    # Einträge reihenweise durch und in sys_timer suchen
                    if my_con.main.version == 35:
                        sql = sqla.select([my_con.main.sys_timer], use_labels=True).where(
                            sqla.and_(my_con.main.sys_timer.c.tim_id == row[1],
                                      my_con.main.sys_timer.c.tim_ref == row[2]))
                    else:
                        sql = sqla.select([my_con.main.sys_timer], use_labels=True).where(
                            sqla.and_(my_con.main.sys_timer.c.timid == row[1],
                                      my_con.main.sys_timer.c.timref == row[2]))
                    res = my_con.main.db.execute(sql).fetchall()
                    if res:  # wir haben einen passenden Eintrag in der sys_timer? dann schreiben wir in die sys_timerhist
                        vals = {dest_con.main.sys_timerhist.c.timref: row[2],
                                dest_con.main.sys_timerhist.c.groupid: res[0][5],
                                dest_con.main.sys_timerhist.c.userid: res[0][7],
                                # in die userid kommt die usr_id_shed aus der sys_timer, die hatte weitergeleitet
                                dest_con.main.sys_timerhist.c.shedid: row[3],
                                dest_con.main.sys_timerhist.c.options: row[4],
                                dest_con.main.sys_timerhist.c.wfid: 0,
                                # die Einträge gibt es in v35 nicht, werden daher auf 0 gesetzt
                                dest_con.main.sys_timerhist.c.required: 0,
                                dest_con.main.sys_timerhist.c.prevhist: 0}
                        ins = sqla.insert(dest_con.main.sys_timerhist, values=vals)
                        try:
                            res2 = dest_con.main.db.execute(ins)
                        except:
                            log.exception("Fehler beim Anlegen einer Wiedervorlage-Historie timref = %i " % row[2])
                        else:
                            histid = res2.lastrowid

                            # eintrag erzeugt? jetzt eintrag in der sys_timer aktualisieren (histid eintragen)
                            vals = {dest_con.main.sys_timer.c.histid: histid}
                            sql = sqla.update(dest_con.main.sys_timer, values=vals).where(dest_con.main.sys_timer.c.timref == row[2])

                            try:
                                dest_con.main.db.execute(sql)
                            except:
                                log.exception("Fehler beim Aktualisieren der histid = %i " % histid)

            if args.only_resub:
                log.info("Es sollten nur die Wiedervorlage übertragen werden. Beende...")
                sys_exit(0)

        # SAP Einträge werden jetzt übers Dokument übertragen
        # if not only_mode and not args.clone_to and not args.import_documents and not args.import_archive:
        #     log.info("Schritt 8: SAP-Dokumente (sys_sap) übertragen")
        #
        #     if 'sys_sap' in my_con.main.tables and not args.sync_no_sap:
        #         if args.sync:
        #             if args.sync[0].lower() == 'sys_mgr':
        #                 sql = 'truncate sys_sap'
        #                 try:
        #                     dest_con.main.db.execute(sql)
        #                 except:
        #                     log.exception("Fehler beim Leeren des sys_sap")
        #
        #                 sys_sap = sqla.Table('sys_sap', my_con.main.meta, autoload=True)
        #                 sql = sqla.select([sys_sap])
        #             else:
        #                 sql = None
        #         else:
        #             sys_sap = sqla.Table('sys_sap', my_con.main.meta, autoload=True)
        #             sql = sqla.select([sys_sap])
        #
        #         if sql is not None:
        #             round = 0
        #             while True:
        #                 sql_limit = sql.offset(round * 1000).limit(1000)
        #                 result = my_con.main.db.execute(sql_limit).fetchall()
        #
        #                 if not result:
        #                     break
        #                 else:
        #                     cnt = 1
        #                     rowcount = len(result)
        #                     log.info("Runde %i, Anzahl der SAP-Dokumente %i" % (round + 1, len(result)))
        #                     if dest_con.main.version == 36:
        #                         for row in result:
        #                             if row[0] is None:
        #                                 continue
        #                             if not notstdout:
        #                                 sys.stdout.write("Verarbeite SAP-Dokumente %i von %i\r" % (cnt, rowcount))
        #                             cnt += 1
        #                             log.debug(row)
        #                             if 'sap_id' in row:  # 3.5er Spaltennamen
        #                                 sapid = row.sap_id
        #                             else:
        #                                 sapid = row.sapid
        #                             if 'gdoc_id' in row:  # 3.5er Spaltennamen
        #                                 gdocid = row.gdoc_id
        #                             else:
        #                                 gdocid = row.gdocid
        #                             if 'docID' in row:  # 3.5er Spaltennamen
        #                                 docid = row.docID
        #                             else:
        #                                 docid = row.docid
        #                             if 'contRep' in row:
        #                                 contrep = row.contRep
        #                             else:
        #                                 contrep = row.contrep
        #                             if 'compId' in row:
        #                                 compid = row.compId
        #                             else:
        #                                 compid = row.compid
        #                             if 'docProt' in row:
        #                                 docprot = row.docProt
        #                             else:
        #                                 docprot = row.docprot
        #                             if 'docStatus' in row:
        #                                 docstatus = row.docStatus
        #                             else:
        #                                 docstatus = row.docstatus
        #                             if 'TStampC' in row:
        #                                 tstampc = row.TStampC
        #                             else:
        #                                 tstampc = row.tstampc
        #                             if 'TStampM' in row:
        #                                 tstampm = row.TStampM
        #                             else:
        #                                 tstampm = row.tstampm
        #
        #                             content_path = row.content_path
        #                             content_type = row.content_type
        #                             charset = row.charset
        #                             version = row.version
        #
        #                             vals = {dest_con.main.sys_sap.c.sapid: sapid,
        #                                     dest_con.main.sys_sap.c.gdocid: gdocid,
        #                                     dest_con.main.sys_sap.c.contrep: contrep,
        #                                     dest_con.main.sys_sap.c.docid: docid,
        #                                     dest_con.main.sys_sap.c.compid: compid,
        #                                     dest_con.main.sys_sap.c.docprot: docprot,
        #                                     dest_con.main.sys_sap.c.docstatus: docstatus,
        #                                     dest_con.main.sys_sap.c.tstampc: tstampc,
        #                                     dest_con.main.sys_sap.c.tstampm: tstampm,
        #                                     dest_con.main.sys_sap.c.content_path: content_path,
        #                                     dest_con.main.sys_sap.c.content_type: content_type,
        #                                     dest_con.main.sys_sap.c.charset: charset,
        #                                     dest_con.main.sys_sap.c.version: version}
        #
        #                             ins = sqla.insert(dest_con.main.sys_sap, values=vals)
        #                             try:
        #                                 dest_con.main.db.execute(ins)
        #                             except:
        #                                 log.exception("Fehler beim Anlegen einer Eintrags in der sys_sap gdocid= %i" % gdocid)
        #                             else:
        #                                 log.debug("Eintrag in sys_sap gdocid= %i" % gdocid)
        #
        #                     round += 1

        if not only_mode and not args.clone_to and not args.import_documents and not args.import_archive:
            log.info("Schritt 9: Verknüpfungen (sys_doclink) übertragen")
            sql = None

            if args.sync:
                do_migrate = False
                sync_ids = []
                if args.sync[0].lower() == 'sys_mgr':
                    sys_mgr = sqla.Table('sys_mgr', my_con.main.meta, autoload=True)
                    sql = sqla.select([sys_mgr.c.gdocid]).order_by(sqla.desc(sys_mgr.c.gdocid))
                    try:
                        result = my_con.main.db.execute(sql).fetchall()
                    except:
                        log.exception("Fehler bei Synchronisation der sys_doclink")
                    else:
                        if result:
                            for row in result:
                                if row[0] is None:
                                    continue
                                sync_ids.append(int(row[0]))
                else:
                    sync_ids = [int(x) for x in args.sync]

                if sync_ids:
                    sql = 'delete from sys_doclink where gdocid in (%s)' % ",".join([str(x) for x in sync_ids])
                    try:
                        dest_con.main.db.execute(sql)
                    except:
                        log.exception("Fehler beim Löschen von Verknüpfung (Synchronisation)")
                    if my_con.main.version == 35:
                        from_obj = sqla.outerjoin(my_con.main.sys_doclink, my_con.main.sys_gdocid, onclause=(
                        sqla.and_(my_con.main.sys_doclink.c.ARC_ID == my_con.main.sys_gdocid.c.arc_id,
                                  my_con.main.sys_doclink.c.DOC_ID == my_con.main.sys_gdocid.c.doc_id)))

                        sql = sqla.select([my_con.main.sys_doclink, my_con.main.sys_gdocid], whereclause=(my_con.main.sys_gdocid.c.gdoc_id.in_(sync_ids)), from_obj=from_obj, use_labels=True)
                    else:
                        sql = sqla.select([my_con.main.sys_doclink], whereclause=(my_con.main.sys_doclink.c.gdocid.in_(sync_ids)), use_labels=True)
            else:
                if my_con.main.version == 35:
                    from_obj = sqla.outerjoin(my_con.main.sys_doclink, my_con.main.sys_gdocid, onclause=(
                    sqla.and_(my_con.main.sys_doclink.c.ARC_ID == my_con.main.sys_gdocid.c.arc_id,
                              my_con.main.sys_doclink.c.DOC_ID == my_con.main.sys_gdocid.c.doc_id)))
                    sql = sqla.select([my_con.main.sys_doclink, my_con.main.sys_gdocid], from_obj=from_obj, use_labels=True)

                elif my_con.main.version == 36:
                    sql = sqla.select([my_con.main.sys_doclink], use_labels=True)

            if sql is not None:
                round = 0
                while True:
                    sql_limit = sql.offset(round * 1000).limit(1000)
                    result = my_con.main.db.execute(sql_limit).fetchall()

                    if not result:
                        break
                    else:
                        valslist = []
                        cnt = 1
                        rowcount = len(result)
                        log.info("Runde %i, Anzahl der Verknüpfungen %i" % (round + 1, len(result)))
                        if dest_con.main.version == 36:
                            for row in result:
                                if row[0] is None:
                                    continue
                                if not notstdout:
                                    sys.stdout.write("Verarbeite Verknüpfungen %i von %i\r" % (cnt, rowcount))
                                    sys.stdout.flush()
                                cnt += 1
                                log.debug(row)
                                if 'sys_doclnk_REF_ID' in row:  # 3.5er Spaltennamen
                                    refid = row.sys_doclnk_REF_ID
                                else:
                                    refid = row.sys_doclink_refid
                                if 'sys_doclnk_REF_LNK' in row:  # 3.5er Spaltennamen
                                    reflink = 0  # wir setzen reflink bei der Migration von 35 -> 36 auf 0
                                else:
                                    reflink = row.sys_doclink_reflink
                                if 'sys_gdocid_gdoc_id' in row:
                                    gdocid = row.sys_gdocid_gdoc_id
                                else:
                                    gdocid = row.sys_doclink_gdocid
                                if 'sys_doclnk_USR_ID' in row:
                                    userid = row.sys_doclnk_USR_ID
                                else:
                                    userid = row.sys_doclink_userid
                                if 'sys_doclnk_REF_Type' in row:
                                    reftype = row.sys_doclnk_REF_Type
                                else:
                                    reftype = row.sys_doclink_reftype
                                if 'sys_doclnk_REF_NAME' in row:
                                    refname = row.sys_doclnk_REF_NAME
                                else:
                                    refname = row.sys_doclink_refname

                                if gdocid is None:
                                    log.warning("Die Verknüpfung mit der refid %s verweist auf kein Dokument" % refid)
                                    continue

                                # vals = {dest_con.main.sys_doclink.c.refid: refid,
                                #         dest_con.main.sys_doclink.c.reflink: reflink,
                                #         dest_con.main.sys_doclink.c.gdocid: gdocid,
                                #         dest_con.main.sys_doclink.c.userid: userid,
                                #         dest_con.main.sys_doclink.c.reftype: 0,         # wir setzen bei der migration den reftype (bisher unbenutzt)
                                #         dest_con.main.sys_doclink.c.refname: refname}  # auf Null, für den Fall, dass in Zukunft reftype benutzt werden soll

                                vals = {'refid': refid,
                                        'reflink': reflink,
                                        'gdocid': gdocid,
                                        'userid': userid,
                                        'reftype': 0,
                                        # wir setzen bei der migration den reftype (bisher unbenutzt)
                                        'refname': refname}  # auf Null, für den Fall, dass in Zukunft reftype benutzt werden soll

                                valslist.append(vals)

                                # TODO: das ist doch falsch eingerückt?
                                ins = sqla.insert(dest_con.main.sys_doclink, values=valslist)
                                try:
                                    dest_con.main.db.execute(ins)
                                except sqla.exc.IntegrityError:
                                    log.warning("Eintrag für refid %s schon vorhanden" % refid)
                                except:
                                    log.exception("Fehler beim Anlegen einer Verknüpfung refid= %i" % refid)
                                else:
                                    log.debug("Verknüpfung angelegt refid= %i" % refid)
                                valslist = []

                        round += 1

        if not only_mode and not args.clone_to and not args.import_documents and not args.import_archive and not args.gpl:
            log.info("Schritt 10: Urlaubsvertretung (sys_holiday) übertragen")

            if args.sync:
                if args.sync[0].lower() == 'sys_mgr':
                    sql = 'truncate sys_holiday'
                    try:
                        dest_con.main.db.execute(sql)
                    except:
                        log.exception("Fehler beim Leeren des sys_holiday")
                    sql = sqla.select([my_con.main.sys_holiday])
                else:
                    sql = None  # wenn wir einen sync mit einer Liste von IDs machen, brauchen wir die Tabelle nicht aktualisieren
            else:
                sql = sqla.select([my_con.main.sys_holiday])

            if sql is not None:
                result = my_con.main.db.execute(sql).fetchall()

                if result:
                    cnt = 1
                    rowcount = len(result)
                    log.info("Anzahl der Urlaubsvertretungen %i" % len(result))
                    if dest_con.main.version == 36:
                        for row in result:
                            if row[0] is None:
                                continue
                            if not notstdout:
                                sys.stdout.write("Verarbeite Urlaubsvertretungen %i von %i\r" % (cnt, rowcount))
                                sys.stdout.flush()
                            cnt += 1
                            log.debug(row)
                            if 'HOL_ID' in row:  # 3.5er Spaltennamen
                                holid = row.HOL_ID
                            else:
                                holid = row.holid
                            if 'USR_ID_SRC' in row:  # 3.5er Spaltennamen
                                userid_src = row.USR_ID_SRC
                            else:
                                userid_src = row.userid_src
                            if 'USR_ID_DST' in row:
                                userid_dst = row.USR_ID_DST
                            else:
                                userid_dst = row.userid_dst
                            if 'START' in row:
                                start = row.START
                            else:
                                start = row.start
                            if 'END' in row:
                                end = row.END
                            else:
                                end = row.end

                            if type(start) is datetime.datetime:
                                start = datetime.datetime.date(start)
                            if type(end) is datetime.datetime:
                                end = datetime.datetime.date(end)

                            vals = {dest_con.main.sys_holiday.c.holid: holid,
                                    dest_con.main.sys_holiday.c.userid_src: userid_src,
                                    dest_con.main.sys_holiday.c.userid_dst: userid_dst,
                                    dest_con.main.sys_holiday.c.start: start,
                                    dest_con.main.sys_holiday.c.end: end}

                            ins = sqla.insert(dest_con.main.sys_holiday, values=vals)
                            try:
                                dest_con.main.db.execute(ins)
                            except:
                                log.exception("Fehler beim Anlegen einer Eintrags in der sys_holiday holid= %i" % holid)
                            else:
                                log.debug("Eintrag in sys_holiday angelegt holid= %i" % holid)

        if not only_mode and not args.clone_to and not args.import_documents and not args.import_archive and not args.gpl:
            log.info("Schritt 12: Versionierung (sys_svn) übertragen")

            if args.sync:
                if args.sync[0].lower() == 'sys_mgr':
                    sql = 'truncate sys_svn'
                    try:
                        dest_con.main.db.execute(sql)
                    except:
                        log.exception("Fehler beim Leeren des sys_svn")
                    if my_con.main.version == 35:
                        from_obj = sqla.join(my_con.main.sys_svn, my_con.main.sys_gdocid, onclause=(
                        sqla.and_(my_con.main.sys_svn.c.ARC_ID == my_con.main.sys_gdocid.c.arc_id,
                                  my_con.main.sys_svn.c.DOC_ID == my_con.main.sys_gdocid.c.doc_id)))
                        sql = sqla.select([my_con.main.sys_svn, my_con.main.sys_gdocid], from_obj=from_obj, use_labels=True)
                    else:
                        sql = sqla.select([my_con.main.sys_svn], use_labels=True)
                else:
                    sql = None  # wenn wir einen sync mit einer Liste von IDs machen, brauchen wir die Tabelle nicht aktualisieren
            else:
                if my_con.main.version == 35:
                    from_obj = sqla.join(my_con.main.sys_svn, my_con.main.sys_gdocid, onclause=(
                    sqla.and_(my_con.main.sys_svn.c.ARC_ID == my_con.main.sys_gdocid.c.arc_id,
                              my_con.main.sys_svn.c.DOC_ID == my_con.main.sys_gdocid.c.doc_id)))
                    sql = sqla.select([my_con.main.sys_svn, my_con.main.sys_gdocid], from_obj=from_obj, use_labels=True)
                else:
                    sql = sqla.select([my_con.main.sys_svn], use_labels=True)

            if sql is not None:
                result = my_con.main.db.execute(sql).fetchall()

                if result:
                    cnt = 1
                    rowcount = len(result)
                    log.info("Anzahl der Einträge (sys_svn) %i" % len(result))
                    if dest_con.main.version == 36:
                        for row in result:
                            if row[0] is None:
                                continue
                            if not notstdout:
                                sys.stdout.write("Verarbeite Eintrag %i von %i\r" % (cnt, rowcount))
                                sys.stdout.flush()
                            cnt += 1
                            log.debug(row)
                            if 'sys_svn_SVN_ID' in row:
                                svnid = row.sys_svn_SVN_ID
                            else:
                                svnid = row.sys_svn_svnid
                            if 'sys_gdocid_gdoc_id' in row:
                                gdocid = row.sys_gdocid_gdoc_id
                            else:
                                gdocid = row.sys_svn_gdocid
                            if 'sys_svn_SVN_LINK' in row:
                                svn_link = row.sys_svn_SVN_LINK
                            else:
                                svn_link = row.sys_svn_svn_link
                            if 'sys_svn_SVN_VERSION' in row:
                                svn_version = row.sys_svn_SVN_VERSION
                            else:
                                svn_version = row.sys_svn_svn_version
                            if 'sys_svn_SVN_REVISION' in row:
                                svn_revision = row.sys_svn_SVN_REVISION
                            else:
                                svn_revision = row.sys_svn_svn_revision
                            if 'sys_svn_SVN_DATE' in row:
                                svn_date = row.sys_svn_SVN_DATE
                            else:
                                svn_date = row.sys_svn_svn_date
                            if 'sys_svn_SVN_INFO' in row:
                                svn_info = row.sys_svn_SVN_INFO
                            else:
                                svn_info = row.sys_svn_svn_info

                            if gdocid in [None, 0]:
                                if not notstdout:
                                    sys.stdout.write("Eintrag ohne Gdocid gefunden, überspringe...\n")
                                    sys.stdout.flush()
                                continue
                            if svn_date is None:
                                if not notstdout:
                                    sys.stdout.write("Eintrag ohne SVN_Date gefunden, überspringe...\n")
                                    sys.stdout.flush()
                                continue

                            vals = {dest_con.main.sys_svn.c.svnid: svnid,
                                    dest_con.main.sys_svn.c.gdocid: gdocid,
                                    dest_con.main.sys_svn.c.svn_link: svn_link,
                                    dest_con.main.sys_svn.c.svn_version: svn_version,
                                    dest_con.main.sys_svn.c.svn_revision: svn_revision,
                                    dest_con.main.sys_svn.c.svn_date: svn_date,
                                    dest_con.main.sys_svn.c.svn_info: svn_info}

                            ins = sqla.insert(dest_con.main.sys_svn, values=vals)
                            try:
                                dest_con.main.db.execute(ins)
                            except:
                                log.exception("Fehler beim Anlegen einer Versionierung svnid= %i" % svnid)
                            else:
                                log.debug("Eintrag in sys_svn angelegt svnid= %i" % svnid)

        if not only_mode and not args.clone_to and not args.import_documents and not args.import_archive and not args.gpl:
            log.info("Schritt 13: Abonnements (sys_abo) übertragen")

            if args.sync:
                if args.sync[0].lower() == 'sys_mgr':
                    sql = 'truncate sys_abo'
                    try:
                        dest_con.main.db.execute(sql)
                    except:
                        log.exception("Fehler beim Leere des sys_abo")
                    sql = sqla.select([my_con.main.sys_abo])
                else:
                    sql = None
            else:
                sql = sqla.select([my_con.main.sys_abo])

            if sql is not None:
                result = my_con.main.db.execute(sql).fetchall()

                if result:
                    log.info("Anzahl der Abonnements %i" % len(result))
                    if dest_con.main.version == 36:
                        cnt = 1
                        rowcount = len(result)
                        for row in result:
                            if row[0] is None:
                                continue
                            if not notstdout:
                                sys.stdout.write("Verarbeite Abonnements %i von %i\r" % (cnt, rowcount))
                                sys.stdout.flush()
                            cnt += 1
                            log.debug(row)
                            if 'ABO_ID' in row:  # 3.5er Spaltennamen
                                aboid = row.ABO_ID
                            else:
                                aboid = row.aboid
                            if 'Gdocid' in row:  # 3.5er Spaltennamen
                                gdocid = row.Gdocid
                            else:
                                gdocid = row.gdocid
                            if 'USR_ID' in row:
                                userid = row.USR_ID
                            else:
                                userid = row.userid
                            if 'TStamp' in row:
                                tstamp = row.TStamp
                            else:
                                tstamp = row.tstamp
                            if 'Info' in row:
                                info = row.Info
                            else:
                                info = row.info

                            vals = {dest_con.main.sys_abo.c.aboid: aboid,
                                    dest_con.main.sys_abo.c.gdocid: gdocid,
                                    dest_con.main.sys_abo.c.userid: userid,
                                    dest_con.main.sys_abo.c.tstamp: tstamp,
                                    dest_con.main.sys_abo.c.info: info}

                            ins = sqla.insert(dest_con.main.sys_abo, values=vals)
                            try:
                                dest_con.main.db.execute(ins)
                            except:
                                log.exception("Fehler beim Anlegen einer Eintrags in der sys_abo aboid= %i" % aboid)
                            else:
                                log.debug("Eintrag in sys_abo angelegt  aboid= %i" % aboid)

        if not only_mode or args.only_bookmarks:
            # Lesezeichen konvertieren
            log.info("Schritt 14: Lesezeichen übertragen")
            allfavs = {}

            if my_con.main.version == 35:
                favdir = os.path.join(bfapath, 'viewer-files\\globalfavorites\\lese*.xml')
                favfiles = glob.glob(favdir)

                if len(favfiles) == 0:
                    log.warning('Keine Lesezeichen unter %s gefunden' % favdir)

                elif len(favfiles) > 0:
                    log.info('Lesezeichen gefunden')
                    for favfile in favfiles:
                        log.info(favfile)

                        localfavuser = re.findall(r'\d+', os.path.basename(favfile))
                        if len(localfavuser) > 0:
                            localfavuser = int(localfavuser[0])
                        else:
                            localfavuser = None

                        favs = search_proc.read_xmlfile(favfile)
                        if favs is None:
                            continue
                        for fav in favs:
                            log.info('Verarbeite Lesezeichen %s' % fav.name)

                            if fav.sqlwhere is not None and fav.sqlwhere != '':
                                if 'tstamp' in fav.sqlwhere.lower():
                                    fav.sqlwhere = fav.sqlwhere.lower().replace('tstamp', 'docdate')
                                if 'ori_loc' in fav.sqlwhere.lower():
                                    fav.sqlwhere = fav.sqlwhere.lower().replace('ori_loc', 'oriloc')
                                if 'doc_loc' in fav.sqlwhere.lower():
                                    fav.sqlwhere = fav.sqlwhere.lower().replace('doc_loc', 'docloc')
                                if 'doc_txt' in fav.sqlwhere.lower():
                                    fav.sqlwhere = fav.sqlwhere.lower().replace('doc_txt', 'doctxt')
                                if 'usr_name' in fav.sqlwhere.lower():
                                    fav.sqlwhere = fav.sqlwhere.lower().replace('usr_name', 'su.username')
                                if 'gdocid' in fav.sqlwhere.lower():
                                    fav.sqlwhere = fav.sqlwhere.lower().replace('gdocid', 'sd.gdocid')
                                if 'or doc_id in' in fav.sqlwhere.lower():
                                    fav.sqlwhere = re.sub("or doc_id in \([\d,]+\)", '',  fav.sqlwhere.lower())

                                fav.sqlwhere = fav.sqlwhere.replace('\'', '"')

                                # neuer ansatz: sqlwhere nach klammerungen und konjunktionen zerschnneiden, dann liste auswerten
                                fav.sqlwhere = fav.sqlwhere.lower()
                                fav.sqlwhere = fav.sqlwhere.replace(')and', ') and').replace('and(', 'and (')
                                fav.sqlwhere = fav.sqlwhere.replace(')or', ') or').replace('or(', 'or (')
                                fav.sqlwhere = fav.sqlwhere.replace(')=', ') =').replace('=(', '= (')
                                # fav.sqlwhere = fav.sqlwhere.replace('))',') )').replace('((','( (')

                                fav.sqlwhere = remove_braces(fav.sqlwhere)
                                items = split_sqlwhere(fav.sqlwhere)
                            else:
                                items = []

                            if len(fav.addfields) > 0:
                                for add in fav.addfields:
                                    if add.add is None:
                                        log.warning("Zusatzfeld ohne add-Bezeichner gefunden (%s), wird übersprungen..." % add.name)
                                        continue

                                    # feld doppelt? (was bei lesezeichen die über eine Suche erzeugt wurden normal ist) -> überspringen
                                    if fav.sqlwhere is not None and add.add in fav.sqlwhere:
                                        continue
                                    items.append('and')
                                    for oldfield in src_fields:
                                        if oldfield.add == add.add and oldfield.rights[0].arcid == fav.arcid:
                                            add.type = oldfield.type
                                            break

                                    if add.value == '""':
                                        add.value = ''
                                        add.mode = 0
                                    if add.mode in [1, 3]:
                                        add.value = '%' + add.value
                                    if add.mode in [2, 3]:
                                        add.value = add.value + '%'

                                    if add.negated:
                                        if add.type in [0, 2, 3]:
                                            if add.value == '':
                                                items.append('not %s is NULL' % (add.add))
                                            else:
                                                items.append('not %s like "%s"' % (add.add, add.value))
                                        elif add.type in [1]:
                                            items.append('not %s = "%s"' % (add.add, add.value))
                                        else:
                                            items.append('not %s = %s' % (add.add, add.value))
                                    else:
                                        if add.type in [0, 2, 3]:
                                            if add.value == '':
                                                items.append('%s is NULL' % (add.add))
                                            else:
                                                items.append('%s like "%s"' % (add.add, add.value))
                                        elif add.type in [1]:
                                            items.append('%s = "%s"' % (add.add, add.value))
                                        else:
                                            items.append('%s = %s' % (add.add, add.value))

                                fav.addfields = []

                            if fav.state > 0:
                                items.append('and')
                                items.append('doc_state & %i) = %i' % (fav.state, fav.state))
                                fav.state = None

                            if fav.notstate > 0:
                                items.append('and')
                                items.append('doc_state & %i) = 0' % fav.notstate)
                                fav.notstate = None

                            if fav.searchstring:
                                items.append('and')
                                items.append('(doctxt like "%%%s%%" or doctags like "%%%s%%")' % (fav.searchstring, fav.searchstring))
                                fav.searchstring = None

                            # wir übertragen checkout True/False ins sqlwhere -Feld
                            if fav.checkout is not None:
                                items.append('and')
                                if fav.checkout:
                                    items.append('not checkout is NULL')
                                # else:
                                #     items.append('checkout is NUll')
                                fav.checkout = None

                            items = translate_items(items)

                            # überflüssige and/or entfernen
                            for ind in range(len(items)):
                                if items[ind] in ['and', 'or']:
                                    if ind == 0 or ind == len(items) - 1:  # ersten oder letzter Eintrag entfernen
                                        items[ind] = None
                                        continue
                                    elif items[ind + 1] in ['and', 'or', None]:
                                        items[ind] = None
                                        continue

                            # fertig, jetzt die None raussortieren
                            for item in list(items):
                                if item is None:
                                    items.remove(item)

                            for i in range(len(items)):
                                if items[i] != 'and' and items[i] != 'or':
                                    items[i] = items_to_string(items[i])
                                    # items[i] = '(%s)' % items[i]

                            if len(items) > 0:  # Achtung! Lesezeichen konnte nicht vollständig geparst werden
                                log.warning("Achtung! SLQWhere-Statement muss überprüft werden")
                                fav.sqlwhere = " ".join(items)
                                fav.verify = True
                            else:
                                log.warning("Achtung! Lesezeichen muss überprüft werden")
                                fav.verify = True
                                fav.sqlwhere = None

                        # jetzt zusammenfassen nach urights und grights (oder keines von beiden) und in die datenbank schreiben

                        for fav in favs:
                            # pro lesezeichendatei gibt es nur jeweils ein searchtype (searchtype = 1 lokales lesezeichen, 2 = globales lesezeichen, 3 = verlauf
                            typ = 0 if fav.searchtype == 2 else 1
                            fav.userid = 0
                            fav.groupid = 0
                            fav.id = None  # die unbrauchbaren ids aus den alten lesezeichen setzten wir zurück. id ist der aktuelle index innerhalb der lesezeichen xml
                            fav.guid = '{%s}' % str(uuid.uuid4()).upper()

                            if typ == 1 and localfavuser is not None:
                                fav.userid = localfavuser
                                fav.constructive = True  # eigene Lesezeichen immer aufbauend
                                if (typ, fav.userid, fav.groupid) in allfavs:
                                    allfavs[(typ, fav.userid, fav.groupid)].append(copy.deepcopy(fav))
                                else:
                                    allfavs[(typ, fav.userid, fav.groupid)] = [copy.deepcopy(fav)]
                            elif (fav.grights is None or len(fav.grights) == 0) and (fav.urights is None or len(fav.urights) == 0):
                                if (typ, fav.userid, fav.groupid) in allfavs:
                                    allfavs[(typ, fav.userid, fav.groupid)].append(copy.deepcopy(fav))
                                else:
                                    allfavs[(typ, fav.userid, fav.groupid)] = [copy.deepcopy(fav)]
                            else:
                                if fav.urights is not None and len(fav.urights) > 0:
                                    for uright in fav.urights:
                                        fav.userid = 0
                                        fav.groupid = 0
                                        for user in users:
                                            if uright.lower() == user.name.lower():
                                                fav.userid = user.id
                                                break
                                        if fav.userid == 0:     #nichts gefunden
                                            log.warning('Keine Zuordnung des Lesezeichens für Benutzer %s möglich' % uright.lower())
                                        if (typ, fav.userid, fav.groupid) in allfavs:
                                            allfavs[(typ, fav.userid, fav.groupid)].append(copy.deepcopy(fav))
                                        else:
                                            allfavs[(typ, fav.userid, fav.groupid)] = [copy.deepcopy(fav)]
                                if fav.grights is not None and len(fav.grights) > 0:
                                    for gright in fav.grights:
                                        fav.userid = 0
                                        fav.groupid = 0
                                        for group in groups:
                                            if gright.lower() == group.name.lower():
                                                fav.groupid = group.id
                                                break
                                        if fav.groupid == 0:     #nichts gefunden
                                            log.warning('Keine Zuordnung des Lesezeichens für Gruppe %s möglich' % gright.lower())
                                        if (typ, fav.userid, fav.groupid) in allfavs:
                                            allfavs[(typ, fav.userid, fav.groupid)].append(copy.deepcopy(fav))
                                        else:
                                            allfavs[(typ, fav.userid, fav.groupid)] = [copy.deepcopy(fav)]

            elif my_con.main.version == 36:
                # pass
                # TODO: v36 favoriten auslesen
                sql = sqla.select([my_con.main.sys_bookmarks], use_labels=False)
                result = my_con.main.db.execute(sql).fetchall()

                if result:
                    for row in result:
                        if row[0] is None:
                            continue
                        typ = row['type']
                        userid = row['userid']
                        groupid = row['groupid']
                        favxml= row['bookmark']
                        favs = search_proc.read_xml(favxml)

                        if favs is None:
                            continue

                        for fav in favs:
                            if fav.sqlwhere is not None and fav.sqlwhere != '':
                                origsqlwhere = fav.sqlwhere
                                # if 'tstamp' in fav.sqlwhere.lower():
                                #     fav.sqlwhere = fav.sqlwhere.lower().replace('tstamp', 'docdate')
                                # if 'ori_loc' in fav.sqlwhere.lower():
                                #     fav.sqlwhere = fav.sqlwhere.lower().replace('ori_loc', 'oriloc')
                                # if 'doc_loc' in fav.sqlwhere.lower():
                                #     fav.sqlwhere = fav.sqlwhere.lower().replace('doc_loc', 'docloc')
                                # if 'doc_txt' in fav.sqlwhere.lower():
                                #     fav.sqlwhere = fav.sqlwhere.lower().replace('doc_txt', 'doctxt')
                                # if 'usr_name' in fav.sqlwhere.lower():
                                #     fav.sqlwhere = fav.sqlwhere.lower().replace('usr_name', 'su.username')
                                # if 'gdocid' in fav.sqlwhere.lower():
                                #     fav.sqlwhere = fav.sqlwhere.lower().replace('gdocid', 'sd.gdocid')
                                # if 'or doc_id in' in fav.sqlwhere.lower():
                                #     fav.sqlwhere = re.sub("or doc_id in \([\d,]+\)", '',  fav.sqlwhere.lower())

                                fav.sqlwhere = fav.sqlwhere.replace('\'', '"')

                                # neuer ansatz: sqlwhere nach klammerungen und konjunktionen zerschnneiden, dann liste auswerten
                                #fav.sqlwhere = fav.sqlwhere.lower()
                                fav.sqlwhere = fav.sqlwhere.replace(')and', ') and').replace('and(', 'and (')
                                fav.sqlwhere = fav.sqlwhere.replace(')or', ') or').replace('or(', 'or (')
                                fav.sqlwhere = fav.sqlwhere.replace(')=', ') =').replace('=(', '= (')
                                # fav.sqlwhere = fav.sqlwhere.replace('))',') )').replace('((','( (')

                                #fav.sqlwhere = remove_braces(fav.sqlwhere)
                                items = split_sqlwhere36(fav.sqlwhere)
                                items = translate_items(items, True)

                                if len(items) > 0:
                                    flat_list = []
                                    for xs in items:
                                        if type(xs) is list:
                                            for x in xs:
                                                flat_list.append(x)
                                        else:
                                            flat_list.append(xs)
                                    fav.sqlwhere = " ".join(flat_list)

                            if fav.direction:
                                fav.order = fav.order + ' ' + fav.direction
                                fav.direction = None

                            if (typ, userid, groupid) in allfavs:
                                allfavs[(typ, userid, groupid)].append(copy.deepcopy(fav))
                            else:
                                allfavs[(typ, userid, groupid)] = [copy.deepcopy(fav)]

            if len(allfavs) > 0:
                for key, items in allfavs.items():
                    typ = key[0]
                    userid = key[1]
                    groupid = key[2]
                    name = 'Globales Lesezeichen' if typ == 0 else 'Lokales Lesezeichen'
                    log.debug('verarbeite %s %s %s' % (typ, userid, groupid))
                    if type(items) is list:
                        items = [x for x in items if isinstance(x, bitfarm_doc.BitfarmSearch)]
                        favxml = search_proc.create_xml(items)
                    elif isinstance(items, bitfarm_doc.BitfarmSearch):
                        favxml = search_proc.create_xml(items)
                    else:
                        favxml = items
                    with open('%i-%s-%s.xml' % (typ, str(userid), str(groupid)), 'w') as f:
                        f.write(favxml)
                    # vals = {
                    #         dest_con.main.sys_bookmarks.c.name: name,
                    #         dest_con.main.sys_bookmarks.c.type: typ,
                    #         dest_con.main.sys_bookmarks.c.userid: userid,
                    #         dest_con.main.sys_bookmarks.c.groupid:  groupid,
                    #         dest_con.main.sys_bookmarks.c.bookmark: favxml,
                    #         }
                    #
                    # ins = sqla.insert(dest_con.main.sys_bookmarks, values = vals)
                    favxml = favxml.replace('%', '%%')
                    sql = "insert into sys_bookmarks (type, userid, groupid, name, bookmark) values (%s, %s, %s, '%s' ,'%s')" \
                          " on duplicate key update bookmark = '%s', name = '%s'" % (typ, userid, groupid, name, favxml, favxml, name)

                    try:
                        dest_con.main.db.execute(sql)
                    except:
                        log.exception("Fehler beim Anlegen einer Eintrags in der sys_bookmarks %s" % name)
                    else:
                        log.debug("Eintrag in sys_bookmarks angelegt %s" % name)

            if args.only_bookmarks:
                log.info("Es sollten nur Lesezeichen übertragen werden. Beende...")
                sys_exit(0)

        if not only_mode or args.only_templates:
            log.info("Übersetze templates")
            if my_con.main.version != 35:
                log.warning("Es können nur Templates der Version 35 übersetzt werden")
            else:
                if dest_con.main.gpl:
                    tpath = os.path.join(bfapath, 'templates')
                    tpls = glob.glob(tpath + '\\*.tpl')
                    for tpl in tpls:
                        log.info("Übersetze Template %s" % tpl)
                        doc = bitfarm_doc.BitfarmDocument()
                        dest_proc.read_from_file(doc, tpl)
                        doc.profile = dest_con.main.profile  # die neuen Templates erhalten den neuen Profilnamen
                        doc.ocrtype = 'OMP'
                        found = False
                        for arc in archives:
                            if doc.tblname == arc.tblname:
                                found = True
                                arcid = arc.id
                                fname = os.path.join(tpath, os.path.basename(tpl))
                                dest_proc.create_new_template(arcid=arcid, doc=doc, filename=fname)
                        if not found:
                            log.warning("Für das Template %s konnte kein zugeodnetes Archiv gefunden werden" % tpl)
                            log.warning("Das Template bleibt unverändert")

                else:
                    tpath = os.path.join(bfapath, 'templates')
                    tpath36 = tpath + '36'
                    if not os.path.exists(tpath):
                        log.warning("Es konnte kein Templates-Verzeichnis gefunden werden %s" % tpath)
                        try:
                            os.mkdir(tpath)
                        except:
                            log.exception("Es konnte kein neues Templates-Verzeichnis angelegt werden")
                        else:
                            for arc in archives:
                                doc = bitfarm_doc.BitfarmDocument()
                                doc.profile = my_con.main.profile   # die neuen Templates erhalten den ursprünglichen Profilnamen
                                fname = os.path.join(tpath, arc.arcname + '.tpl')
                                if dest_proc.create_new_template(arcid=arc.id, doc=doc, filename=fname):
                                    log.debug("Template %s erstellt." % fname)
                                else:
                                    log.error("Fehler beim Erstellen des Template %s." % fname)

                            log.info("Neue Templates erstellt.")
                    else:
                        if not os.path.exists(tpath36):
                            try:
                                os.mkdir(tpath36)
                            except:
                                log.exception("Es konnte kein neues Templates-Verzeichnis angelegt werden")

                        if os.path.exists(tpath36):
                            tpls = glob.glob(tpath + '\\*.tpl')
                            for tpl in tpls:
                                log.info("Übersetze Template %s" % tpl)
                                doc = bitfarm_doc.BitfarmDocument()
                                dest_proc.read_from_file(doc, tpl)
                                doc.profile = my_con.main.profile   # die neuen Templates erhalten den ursprünglichen Profilnamen
                                found = False
                                for arc in archives:
                                    if doc.tblname == arc.tblname:
                                        found = True
                                        arcid = arc.id
                                        fname = os.path.join(tpath36, os.path.basename(tpl))
                                        dest_proc.create_new_template(arcid=arcid, doc=doc, filename=fname)
                                if not found:
                                    log.warning("Für das Template %s konnte kein zugeodnetes Archiv gefunden werden" % tpl)
                                    log.warning("Das Template wird unverändert kopiert")
                                    try:
                                        shutil.copyfile(tpl, os.path.join(tpath36, os.path.basename(tpl)))
                                    except:
                                        log.exception("Template %s konnte nicht umkopiert werden" % tpl)

            if args.only_templates:
                log.info("Es sollten nur Templates übersetzt werden. Beende...")
                sys_exit(0)

        if args.only_jobfiles:
            log.info("Übersetze only_jobfiles")
            if my_con.main.version != 35:
                log.warning("Es können nur only_jobfiles der Version 35 übersetzt werden")
            else:
                tpath = os.path.join(bfapath, 'templates')
                queue = os.path.join(bfapath, 'queue')
                if not os.path.exists(tpath):
                    log.warning("Es konnte kein Templates-Verzeichnis gefunden werden %s" % tpath)
                else:
                    if not os.path.exists(queue):
                        log.warning("Es konnte kein Queue-Verzeichnis gefunden werden %s" % tpath)
                    else:
                        tpls = glob.glob(queue + '\\*.job')
                        for tpl in tpls:
                            log.info("Übersetze Jobfile %s" % tpl)
                            doc = bitfarm_doc.BitfarmDocument()
                            dest_proc.read_from_file(doc, tpl)
                            doc.ftx = None      # wir haben kein FullText Object (sondern nur einen Pfad in doc.txtloc)
                            if doc.version == 36:
                                log.warning('Jobdatei %s der Version 36 gefunden, überspringe...' % os.path.basename(tpl))
                                continue
                            doc.profile = dest_con.main.profile   # die übersetzen Jobfiles erhalten den neuen Profilnamen
                            found = False
                            for arc in archives:
                                if doc.tblname == arc.tblname:
                                    found = True
                                    arcid = arc.id
                                    fname = os.path.join(queue, os.path.basename(tpl))
                                    dest_proc.create_new_template(arcid=arcid, doc=doc, filename=fname)
                            if not found:
                                log.warning("Für die Jobdatei %s konnte kein zugeodnetes Archiv gefunden werden" % tpl)
                                log.warning("Das Jobdatei bleibt unverändert")

            if args.only_jobfiles:
                log.info("Es sollten nur Jobfiles übersetzt werden. Beende...")
                sys_exit(0)

        if args.only_comments:
            log.info("Übertrage nur Kommentare")
            roundno = 0
            if my_con.main.version == 35:
                sql = "select arc_id, arc_tblname from sys_arc where arc_tblname <> '' "
                try:
                    result = my_con.main.db.execute(sql).fetchall()
                except:
                    log.exception("Exception:")
                    sys_exit(52)

                sys_arc = {}
                if result:
                    for row in result:
                        if row[0] is None:
                            continue
                        sys_arc[row[0]] = row[1]
                else:
                    # log.error("Keine Archivtabellen in der sys_arc gefunden")
                    sys_exit(53)

                while True:
                    sql = sqla.select([my_con.main.sys_gdocid], whereclause=(my_con.main.sys_gdocid.c.gdoc_id >= args.from_gdocid),
                                      limit=args.packsize, offset=roundno * args.packsize).order_by(sqla.asc(my_con.main.sys_gdocid.c.gdoc_id))
                    if args.to_gdocid:
                        sql.append_whereclause(my_con.main.sys_gdocid.c.gdoc_id < args.to_gdocid)

                    try:
                        result = my_con.main.db.execute(sql).fetchall()
                    except:
                        log.exception("Fehler bei der Abfrage der sys_gdocid")
                        break

                    if not result:
                        break
                    else:
                        log.info("Runde %i: Anzahl der geladenen Dokumente %i" % (roundno, len(result)))
                        for row in result:
                            if row[0] is None:
                                continue
                            if row[1] not in sys_arc:
                                log.error("Die arcid %s konnte nicht in der sys_arc gefunden werden" % row[1])
                            else:
                                sql = 'select * from com_%s where doc_id = %s order by tstamp desc' % (sys_arc[row[1]], row[2])

                                try:
                                    comments = my_con.main.db.execute(sql).fetchall()
                                except:
                                    log.exception("Fehler bei der Abfrage der com_%s" % sys_arc[row[1]])
                                else:
                                    for com in comments:
                                        vals = {dest_con.main.sys_com.c.gdocid: row[0],
                                                dest_con.main.sys_com.c.userid: com[4],
                                                dest_con.main.sys_com.c.tstamp: com[5],
                                                dest_con.main.sys_com.c.comment: com[6]}

                                        ins = sqla.insert(dest_con.main.sys_com, values=vals)

                                        try:
                                            dest_con.main.db.execute(ins)
                                        except:
                                            log.exception("Fehler beim Übertragen der Kommentare")
                                        else:
                                            log.debug("Kommentar %s aus Tabelle com_%s übertragen [%s]" % (com[0], sys_arc[row[1]], row[0]))

                        roundno += 1
            else:    # version 36
                while True:
                    sql = sqla.select([my_con.main.sys_com], use_labels=True, limit=args.packsize, offset=roundno * args.packsize)
                    if args.from_gdocid:
                        sql.append_whereclause(my_con.main.sys_gdocid.c.gdocid >= args.from_gdocid)
                    if args.to_gdocid:
                        sql.append_whereclause(my_con.main.sys_gdocid.c.gdocid < args.to_gdocid)

                    result = my_con.main.db.execute(sql).fetchall()

                    if not result:
                        break
                    else:
                        log.info("Runde %i: Anzahl der geladenen Dokumente %i" % (roundno, len(result)))
                        for row in result:
                            if row[0] is None:
                                continue
                            vals = {dest_con.main.sys_com.c.gdocid: row[1],
                                    dest_con.main.sys_com.c.userid: row[2],
                                    dest_con.main.sys_com.c.tstamp: row[3],
                                    dest_con.main.sys_com.c.comment: row[4]}

                            ins = sqla.insert(dest_con.main.sys_com, values=vals)

                            try:
                                dest_con.main.db.execute(ins)
                            except:
                                log.exception("Fehler beim Übertragen der Kommentare")
                            else:
                                log.debug("Kommentar %s sys_com übertragen [%s]" % (row[0], row[1]))
                        roundno += 1

            if args.only_comments:
                log.info("Es sollten nur Kommentare übertragen werden. Beende...")
                sys_exit(0)

        if not args.gpl and (not only_mode or args.only_mwl):
            log.info("Übersetze Vertragsmanagementdateien")
            if my_con.main.version != 35:
                log.warning("Es können nur Vertragsmanagementdateien der Version 35 übersetzt werden")
            else:
                mwlpath = os.path.join(bfapath, 'Vertragsmanagement')
                mwlpath36 = mwlpath + '36'
                if not os.path.exists(mwlpath):
                    log.warning("Es konnte kein Vertragsmanagement-Verzeichnis gefunden werden %s" % mwlpath)
                else:
                    if not os.path.exists(mwlpath36):
                        try:
                            os.mkdir(mwlpath36)
                        except:
                            log.exception("Es konnte kein neues Vertragsmanagement-Verzeichnis angelegt werden")
                        else:
                            try:
                                shutil.copyfile(os.path.join(mwlpath,'list.ini'), os.path.join(mwlpath36,'list.ini'))
                            except:
                                log.exception("Es konnte die List.ini-Datei nicht umkopiert werden.")

                    if os.path.exists(mwlpath36):
                        mwls = glob.glob(mwlpath + '\\*.mwl')
                        for mwl in mwls:
                            log.info("Übersetze Vertragsmanagementatei %s" % mwl)

                            ini = bitfarm_configobj.ConfigObj(infile=mwl, encoding='cp1252')
                            new_mwl = os.path.join(mwlpath36, os.path.basename(mwl))
                            new_ini = bitfarm_configobj.ConfigObj(infile=new_mwl, encoding='cp1252')
                            if len(ini.items()) > 0:
                                if 'archivID' not in ini['Info']:  # kein Eintrag archivID, entweder falsche mwl Datei oder schon neues Format (archiveid)
                                    log.warning("Eintrag archivID nicht gefunden. Falsches Format der mwl-Datei ")
                                    continue
                                try:
                                    new_ini['Info'] = {}
                                    new_ini['Info']['archiveid'] = ini['Info']['archivID']
                                    arcid = int(ini['Info']['archivID'])
                                    if (ini['Info']['startfeld'], arcid) not in field_mapping:
                                        field = get_field_by_name(ini['Info']['start'])
                                        if field is not None and field.add is not None:
                                            new_ini['Info']['startfield'] = field.add
                                        else:
                                            log.warning("Achtung: Das Startfeld konnte nicht ermittelt werden")
                                    else:
                                        if field_mapping[(ini['Info']['startfeld'],arcid)] is not None:
                                            new_ini['Info']['startfield'] = field_mapping[(ini['Info']['startfeld'],arcid)]
                                        else:
                                            if (ini['Info']['startfeld'], arcid) in dest_con.main.server_config:
                                                new_ini['Info']['startfield'] = \
                                                dest_con.main.server_config[(ini['Info']['startfeld'], arcid)]
                                            else:
                                                log.warning("Achtung: Das Startfeld konnte nicht ermittelt werden")
                                    if (ini['Info']['intervalfeld'], arcid) not in field_mapping:
                                        field = get_field_by_name(ini['Info']['interval'])
                                        if field is not None and field.add is not None:
                                            new_ini['Info']['intervalfield'] = field.add
                                        else:
                                            log.warning("Achtung: Das Intervalfeld konnte nicht ermittelt werden")
                                    else:
                                        if field_mapping[(ini['Info']['intervalfeld'], arcid)] is not None:
                                            new_ini['Info']['intervalfield'] = field_mapping[(ini['Info']['intervalfeld'], arcid)]
                                        else:
                                            if (ini['Info']['intervalfeld'], arcid) in dest_con.main.server_config:
                                                new_ini['Info']['intervalfield'] = \
                                                dest_con.main.server_config[(ini['Info']['intervalfeld'], arcid)]
                                            else:
                                                log.warning("Achtung: Das Intervalfeld konnte nicht ermittelt werden")
                                    if (ini['Info']['endfeld'], arcid) not in field_mapping:
                                        field = get_field_by_name(ini['Info']['end'])
                                        if field is not None and field.add is not None:
                                            new_ini['Info']['endfield'] = field.add
                                        else:
                                            log.warning("Achtung: Das Endfeld konnte nicht ermittelt werden")
                                    else:
                                        if field_mapping[(ini['Info']['endfeld'], arcid)] is not None:
                                            new_ini['Info']['endfield'] = field_mapping[(ini['Info']['endfeld'], arcid)]
                                        else:
                                            if (ini['Info']['endfeld'], arcid) in dest_con.main.server_config:
                                                new_ini['Info']['endfield'] = \
                                                dest_con.main.server_config[(ini['Info']['endfeld'], arcid)]
                                            else:
                                                log.warning("Achtung: Das Endfeld konnte nicht ermittelt werden")
                                    if (ini['Info']['lasttimeactivatedfeld'], arcid) not in field_mapping:
                                        field = get_field_by_name(ini['Info']['lasttimeactivated'])
                                        if field is not None and field.add is not None:
                                            new_ini['Info']['startfield'] = field.add
                                        else:
                                            log.warning("Achtung: Das lasttimeactivatedfeld konnte nicht ermittelt werden")
                                    else:
                                        if field_mapping[(ini['Info']['lasttimeactivatedfeld'], arcid)] is not None:
                                            new_ini['Info']['lastactivatedfield'] = field_mapping[(ini['Info']['lasttimeactivatedfeld'], arcid)]
                                        else:
                                            if (ini['Info']['lasttimeactivatedfeld'], arcid) in dest_con.main.server_config:
                                                new_ini['Info']['lastactivatedfield'] = \
                                                dest_con.main.server_config[(ini['Info']['lasttimeactivatedfeld'], arcid)]
                                            else:
                                                log.warning("Achtung: Das lasttimeactivatedfeld konnte nicht ermittelt werden")
                                    if (ini['Info']['kuendingungfeld'], arcid) not in field_mapping:
                                        field = get_field_by_name(ini['Info']['kuendingung'])
                                        if field is not None and field.add is not None:
                                            new_ini['Info']['noticefield'] = field.add
                                        else:
                                            log.warning("Achtung: Das Kündigungstfeld konnte nicht ermittelt werden")
                                    else:
                                        if field_mapping[(ini['Info']['kuendingungfeld'], arcid)] is not None:
                                            new_ini['Info']['noticefield'] = field_mapping[(ini['Info']['kuendingungfeld'], arcid)]
                                        else:
                                            if (ini['Info']['kuendingungfeld'], arcid) in dest_con.main.server_config:
                                                new_ini['Info']['noticefield'] = \
                                                dest_con.main.server_config[(ini['Info']['kuendingungfeld'], arcid)]
                                            else:
                                                log.warning("Achtung: Das Kündigungstfeld konnte nicht ermittelt werden")
                                    new_ini['Info']['offset'] = ini['Info']['versatz']
                                except:
                                    log.exception("Es hat einen Fehler beim Übersetzen der mwl Datei gegeben %s" % mwl)
                                else:
                                    new_ini.write()

            if args.only_mwl:
                log.info("Es sollten nur Vertragsmanagementateien übersetzt werden. Beende...")
                sys_exit(0)

        # Übertragung von Dokumenten
        if not args.clone_to and not args.only_annos:
            log.info("Verbinde mit %s" % args.migrate_to)
            clonestart = time.time()
            roundno = 0
            cnt = 1
            level = ['complete']
            if args.gpl:
                level = ['base', 'text', 'fields', 'links', 'paths', 'iev', 'history']
            # connection ohne autocommit erstellen
            dc = bitfarm_proc.BitfarmConnections(args.migrate_to, gpl=args.create_gpl)
            dc.create_db_engine(autocommit=False, echo=False)
            dc.connect_db()
            dp = bitfarm_proc.BitfarmDocProcessing(dc)
            dp.db["main"].execute("SET innodb_lock_wait_timeout = 50000;")
            dp.db["main"].execute("SET unique_checks=0;")
            dp.db["main"].execute("SET foreign_key_checks=0;")
            dp.db["main"].execute("SET NAMES utf8mb4;")
            if args.debug:
                dp.log = log

            log.info("Schritt 15: Dokumente (sys_doc) übertragen")
            all_imported = set()
            do_migrate = True

            if args.debug:
                tracemalloc.start()
                tracefn = open('tracemalloc_%s.log' % datetime.datetime.now().strftime('%Y-%m-%d %H.%M.%S'), 'w')
                gc.collect()
                compare_snapshots()

            while do_migrate:
                if args.sync:
                    sync_ids = []
                    if args.sync[0].lower() == 'sys_mgr':
                        sys_mgr = sqla.Table('sys_mgr', my_con.main.meta, autoload=True)
                        sql = sqla.select([sys_mgr.c.gdocid]).order_by(sqla.asc(sys_mgr.c.gdocid))
                        try:
                            result = my_con.main.db.execute(sql).fetchall()
                        except:
                            log.exception("Fehler bei Abfrage der sys_mgr")
                        else:
                            if result:
                                for row in result:
                                    if row[0] is None:
                                        continue
                                    sync_ids.append(int(row[0]))
                    else:
                        sync_ids = [int(x) for x in args.sync]

                    if sync_ids:
                        sql = 'delete from sys_history where gdocid in (%s)' % ",".join([str(x) for x in sync_ids])
                        try:
                            dest_con.main.db.execute(sql)
                        except:
                            log.exception("Fehler beim Löschen der alten Historyeinträge (Synchronisation)")

                    do_migrate = False
                    if args.sync[0].lower() == 'sys_mgr':
                        sys_mgr = sqla.Table('sys_mgr', my_con.main.meta, autoload=True)
                        sql = sqla.select([sys_mgr.c.gdocid]).order_by(sqla.asc(sys_mgr.c.gdocid))
                    else:
                        if my_con.main.version == 35:
                            sql = sqla.select([my_con.main.sys_gdocid.c.gdoc_id],
                                              whereclause=(my_con.main.sys_gdocid.c.gdoc_id.in_(sync_ids))).order_by(sqla.asc(my_con.main.sys_gdocid.c.gdoc_id))
                        else:
                            sql = sqla.select([my_con.main.sys_doc.c.gdocid],
                                              whereclause=(my_con.main.sys_doc.c.gdocid.in_(sync_ids))).order_by(sqla.asc(my_con.main.sys_doc.c.gdocid))
                    sync_ids = []
                elif args.import_documents:
                    if my_con.main.version == 35:
                        sql = sqla.select([my_con.main.sys_gdocid.c.gdoc_id], whereclause=(my_con.main.sys_gdocid.c.gdoc_id.in_(args.import_documents)),
                                          limit=args.packsize, offset=roundno * args.packsize).order_by(sqla.asc(my_con.main.sys_gdocid.c.gdoc_id))
                    else:
                        sql = sqla.select([my_con.main.sys_doc.c.gdocid], whereclause=(my_con.main.sys_doc.c.gdocid.in_(args.import_documents)),
                                          limit=args.packsize, offset=roundno * args.packsize).order_by(sqla.asc(my_con.main.sys_doc.c.gdocid))
                elif args.import_archive:
                    arcids = [int(x.split(':')[0]) for x in args.import_archive]
                    if my_con.main.version == 35:
                        sql = sqla.select([my_con.main.sys_gdocid.c.gdoc_id], whereclause=(my_con.main.sys_gdocid.c.arc_id.in_(arcids)),
                                          limit=args.packsize, offset=roundno * args.packsize).order_by(sqla.asc(my_con.main.sys_gdocid.c.gdoc_id))
                    else:
                        sql = sqla.select([my_con.main.sys_doc.c.gdocid], whereclause=(my_con.main.sys_doc.c.arcid.in_(arcids)),
                                          limit=args.packsize, offset=roundno * args.packsize).order_by(sqla.asc(my_con.main.sys_doc.c.gdocid))
                else:
                    if my_con.main.version == 35:
                        sql = sqla.select([my_con.main.sys_gdocid.c.gdoc_id], whereclause=(my_con.main.sys_gdocid.c.gdoc_id >= args.from_gdocid),
                                          limit=args.packsize, offset=roundno * args.packsize).order_by(sqla.asc(my_con.main.sys_gdocid.c.gdoc_id))
                        if args.to_gdocid:
                            sql.append_whereclause(my_con.main.sys_gdocid.c.gdoc_id < args.to_gdocid)
                    else:
                        sql = sqla.select([my_con.main.sys_doc.c.gdocid], whereclause=(my_con.main.sys_doc.c.gdocid >= args.from_gdocid),
                                          limit=args.packsize, offset=roundno * args.packsize).order_by(sqla.asc(my_con.main.sys_doc.c.gdocid))
                        if args.to_gdocid:
                            sql.append_whereclause(my_con.main.sys_doc.c.gdoc_id < args.to_gdocid)
                if args.debug:
                    log.info('vor fetchall %s' % str(tracemalloc.get_traced_memory()[0]))
                result = None
                try:
                    result = my_con.main.db.execute(sql).fetchall()
                except:
                    log.exception("Fehler bei der Abfrage der Dokumente")
                    break
                if args.debug:
                    compare_snapshots()
                    log.info('nach fetchall %s' % str(tracemalloc.get_traced_memory()[0]))
                if result:
                    result.reverse()
                    if not args.sync:
                        dp.db["main"].execute("START TRANSACTION;")
                    cnt = 1
                    rowcount = len(result)
                    log.info("Runde %i: Anzahl der geladenen Dokumente %i" % (roundno, rowcount))
                    starttime = time.time()
                    errordocs = []
                    importeddocs = []
                    srcnotloadeddocs = []

                    row = result.pop()
                    # noinspection PyUnboundLocalVariable
                    while row:
                        if row[0] is None:
                            log.warning("Keine Gdocid übergeben, überspringe Eintrag...")
                            try:
                                row = result.pop()
                            except:
                                row = None
                                break
                            else:
                                continue
                        if row[0] == 0:
                            log.warning("Admin Dokument [0] gefunden, überspringe Eintrag...")
                            try:
                                row = result.pop()
                            except:
                                row = None
                                break
                            else:
                                continue

                        if not notstdout:
                            sys.stdout.write("Verarbeite Dokument %i von %i\r" % (cnt, rowcount))
                            sys.stdout.flush()

                        if args.debug:
                            compare_snapshots()

                        cnt += 1
                        log.debug("Lade Dokument [%i]" % row[0])
                        doc = bitfarm_doc.BitfarmDocument()

                        if not my_proc.load(doc, row[0], level=level):
                            log.error("Das Dokument [%i] konnte nicht geladen werden" % row[0])
                            srcnotloadeddocs.append(row[0])
                        else:
                            doc.profile = dc.main.profile
                            if my_con.main.version == 35:
                                if doc.ftx.text:
                                    if doc.ftx.text.find('Anmerkungstexte eingefügt am:') > 0:
                                        _ftx = doc.ftx.text[: doc.ftx.text.find('Anmerkungstexte eingefügt am:')]
                                        doc.ftx.clear()
                                        doc.ftx.text = _ftx

                            if doc.userid is None or doc.userid == 0:
                                if doc.user:
                                    found = False
                                    for user in users:
                                        if doc.user.lower() == user.name.lower():
                                            found = True
                                            break
                                    if not found:
                                        user = bitfarm_doc.BitfarmUser()
                                        user.name = doc.user
                                        user.displayname = doc.user
                                        user.flags = ['deleted']
                                        users.append(user)
                                        sql = "insert into sys_user set username = '%s', displayname = '%s', flags = 2" % (user.name, user.displayname)

                                        try:
                                            ins = dest_con.main.db.execute(sql)
                                        except:
                                            log.exception("Fehler beim Eintragen eines Dummy-Users %s in die Usertabelle" % user.name)
                                            doc.userid = 1
                                        else:
                                            doc.userid = ins.lastrowid
                                else:
                                    doc.userid = 1

                            if args.create_new_gdocid:
                                log.warning("Für das Dokument [%i] wird eine neue ID erzeugt." % doc.gdocid)
                                del doc.gdocid

                            src_arcid = doc.arcid
                            if args.import_archive:
                                for elem in args.import_archive:
                                    if int(elem.split(':')[0]) == doc.arcid:
                                        dest_arcid = int(elem.split(':')[1])
                                        break
                            elif args.import_documents:
                                if args.dest_arcid:
                                    dest_arcid = int(args.dest_arcid)
                                else:
                                    dest_arcid = doc.arcid
                            else:
                                dest_arcid = doc.arcid

                            if not dp.create(doc, arcid=dest_arcid) and not args.sync:
                                log.error("Das Dokument [%s] konnte nicht erzeugt werden" % doc.gdocid)
                                errordocs.append(doc.gdocid)
                            else:
                                if args.create_new_gdocid:
                                    log.info("Eine neue ID [%i] wurde für das Dokument erzeugt." % doc.gdocid)

                                # Statusfelder übertragen
                                if my_con.main.version == 35:  # von statusdict nach statefields
                                    for items in doc.statusdict.items():
                                        for state in doc.statefields.items():
                                            if state[0].lower() == items[0].lower():
                                                state[1].value = items[1]
                                                break

                                # Filetype mit File Extension füllen
                                doc.filetype = os.path.splitext(doc.oriloc)[1] if doc.oriloc not in (None, "") else os.path.splitext(doc.docloc)[1]
                                if len(doc.filetype) > 15:
                                    doc.filetype = doc.filetype[:15]

                                if import_mode:
                                    doc.workflows = []          # wir übernehmen für importierte Dokumente keinen Workflows
                                                                # (wir kennen weder die user/gruppenids noch irgendwelche wfds)
                                    for com in doc.commentary:  # wir entfernen die com.id, damit für das importierte dokument neue
                                        del com.id              # einträge erzeugt werden (und nicht bestehende überschrieben werden)

                                    for sapfield in doc.sap:    # wir entfernen die sap_id, damit für das importierte dokument neue
                                        del sapfield.sap_id     # einträge erzeugt werden (und nicht bestehende überschrieben werden)

                                    doc.svn_history = []        # beim import wird SVN derzeit nicht unterstützt

                                    for anno in doc.annotations:
                                        anno.modified = True

                                    hist = bitfarm_doc.BitfarmHistory()
                                    hist.userid = 1
                                    hist.gdocid = doc.gdocid
                                    hist.taskid = 132
                                    hist.value1 = my_con.main.profile
                                    hist.value2 = row[0]
                                    doc.history.append(hist)

                                    dest_statefields = dc.main.read_statefields(arcid=dest_arcid)
                                    for field in doc.statefields.values():
                                        for dest_field in dest_statefields:
                                            if field.name.lower() == dest_field.name.lower():
                                                dest_field.value = field.value
                                                dest_field.modified = True
                                    doc.statefields = OrderedDict()
                                    doc.statefields = {x.name: x for x in dest_statefields}  # austauschen

                                # Zusatzfelder übertragen
                                for field in doc.fields.values():
                                    field.modified = True

                                dest_fields = []
                                if dc.main.version == 36:
                                    dest_fields.extend(dc.main.read_addfields(arcid=dest_arcid))

                                if field_mapping and dest_fields:       # wenn wir eine Feldzurodnung haben und auch Zielzusatzfelder, dann versuchen wir auch zuzuordnen
                                    for field in doc.fields.values():
                                        if (field.add, src_arcid) not in field_mapping:
                                            log.warning("Das Feld %s konnte nicht in der Zuordnungstabelle gefunden werden" % str((field.add, src_arcid)))
                                            continue
                                        dest_add = field_mapping[(field.add, src_arcid)]
                                        for dest_field in dest_fields:
                                            if dest_field.add == dest_add:  # Zuordnung des alten Feldes zum neuen Feld gefunden
                                                log.debug('Field: %s Dest: %s Value: %s Type: %s Modified: %s' % (field.add, dest_add, str(field.value), str(type(field.value)), str(dest_field.modified)))
                                                dest_field.modified = True

                                                if dest_con.main.gpl:
                                                    if dest_field.type in [1, 12]:
                                                        if type(field.value) is datetime.datetime:
                                                            dest_field.value = field.value
                                                        elif field.value is None:
                                                            dest_field.value = field.value
                                                        elif type(field.value) is str and re.match(date_pattern, field.value):
                                                            dest_field.value = field.value
                                                        else:
                                                            log.warning('Falscher Datentyp: Der Wert %s des Feldes %s konnte nicht nach %s [Typ %s] übernommen werden.' % (str(field.value), field.add, dest_field.add, bitfarm_doc.FIELD_TYPES[dest_field.type]))
                                                            dest_field.modified = False
                                                    elif dest_field.type == 6:
                                                        if type(field.value) is int:
                                                            dest_field.value = field.value
                                                        elif field.value is None:
                                                            dest_field.value = field.value
                                                        else:
                                                            try:
                                                                test = int(field.value)
                                                            except:
                                                                log.warning('Falscher Datentyp: Der Wert %s des Feldes %s konnte nicht nach %s [Typ %s] übernommen werden.' % (str(field.value), field.add, dest_field.add, bitfarm_doc.FIELD_TYPES[dest_field.type]))
                                                                dest_field.modified = False
                                                            else:
                                                                dest_field.value = field.value
                                                    elif dest_field.type in [7,8]:
                                                        if type(field.value) in [float, decimal.Decimal]:
                                                            dest_field.value = field.value
                                                        elif field.value is None:
                                                            dest_field.value = field.value
                                                        else:
                                                            if type(field.value) is str and field.value.count(',') == 1:
                                                                field.value = field.value.replace(',', '.')
                                                            try:
                                                                test = float(field.value)
                                                            except:
                                                                log.warning('Falscher Datentyp: Der Wert %s des Feldes %s konnte nicht nach %s [Typ %s] übernommen werden.' % (str(field.value), field.add, dest_field.add, bitfarm_doc.FIELD_TYPES[dest_field.type]))
                                                                dest_field.modified = False
                                                            else:
                                                                dest_field.value = field.value
                                                    else:
                                                        if type(field.value) is str:
                                                            try:
                                                                dest_field.value = field.value.encode('latin1').decode('cp1252')
                                                            except:
                                                                dest_field.value = field.value
                                                        else:
                                                            dest_field.value = field.value
                                                else:   # Enterprise: wenn hier falsche Datentypen in den Feldern stehen müssen wir in einen Fehler laufen,
                                                        # damit die Werte ggf. korrigiert werden können (oder entfernt)
                                                    if type(field.value) is str and my_con.main.version == 35:
                                                        try:
                                                            dest_field.value = field.value.encode('latin1').decode('cp1252')
                                                        except:
                                                            dest_field.value = field.value
                                                    else:
                                                        dest_field.value = field.value

                                                break

                                    doc.fields = None
                                    doc.fields = {x.add: x for x in dest_fields}  # austauschen
                                else:   # kein Mapping?
                                    if my_con.main.version == 35:   # wir kommen von einer 3.5, haben aber kein Mapping, können also auch keine Felder übernehmen?
                                        for field in doc.fields.values():
                                            field.modified = False  # dann müssen wir auch nichts speichern

                                if my_con.main.version == 35:  # annos vom filesystem (v35) einlesen und als annotations-objekte zum dokument hinzufügen
                                    annotxt = ""
                                    annopath = doc.extloc if doc.extloc else doc.docloc
                                    annopath = os.path.expandvars(annopath).lower()
                                    annopath = annopath.replace(my_con.main.archive_rs.lower(),
                                                                my_con.main.archive_nrs.lower())
                                    annopath = os.path.splitext(annopath)[0]

                                    ievlist = glob.iglob(annopath + "*.iev")
                                    # new_path could be rs-path, but iev should be placed in nrs-path
                                    #                            nrs_path = dp.create_arcpath(doc, is_revisionsave = False)
                                    for iev in ievlist:
                                        ievmeta = iev.replace(annopath, "")[:-4]
                                        if os.path.exists(os.path.expandvars(iev)):
                                            ext = iev.replace(annopath, '')
                                            # determine page + public/private via filename
                                            try:
                                                match = re.match("(\d*)[Pp]*(\d*)", ievmeta)
                                                if match:
                                                    page, userid = match.groups()
                                                    if len(userid) == 0:
                                                        userid = "0"
                                                    if len(page) == 0:
                                                        page = "0"
                                                    if len(userid) > 3: #größer drei stellen, kann nicht sein
                                                        userid = "0"
                                                    if len(page) > 3:   # mehr als 999 seiten und dann eine anno? wohl kaum
                                                        page = "0"
                                                else:
                                                    log.error("Layer und Seite konnte nicht ermittelt werden, verwende verwende Seite 1 Layer 0, iev = %s" % ievmeta)
                                                    userid = "0"
                                                    page = "0"
                                            except:
                                                log.error("Fehler beim Ermitteln von Layer und Seite: %s" % sys.exc_info()[1].message)
                                            else:
                                                anno = bitfarm_doc.BitfarmAnnotation()
                                                anno.iev = bitfarm_doc.BitfarmStream()
                                                anno.page = int(page)
                                                anno.layer = int(userid)
                                                anno.modified = True
                                                try:
                                                    with open(os.path.expandvars(iev), 'rb') as f:
                                                        anno.iev.stream.write(f.read())
                                                except:
                                                    log.exception("Fehler beim Lesen der Annotation-Datei %s" % iev)
                                                    continue

                                                doc.annotations.append(anno)
                                else:
                                    # sollten die Annos direkt aus der Datenbank kommen (>=3.6.0) dann müssen die auf modified gesetzt werden, damit sie gespeichert werden
                                    for anno in doc.annotations:
                                        anno.modified = True

                                if not dp.write(doc):
                                    log.error("Das Dokument [%i] konnte nicht gespeichert werden" % doc.gdocid)
                                    errordocs.append(doc.gdocid)
                                else:
                                    importeddocs.append(doc.gdocid)
                                    if import_mode:
                                        all_imported.add(doc.gdocid)
                                    if args.sync:
                                        sync_ids.append(doc.gdocid)
                                    log.debug("Das Dokument [%i] wurde gespeichert" % doc.gdocid)
                                    # an dieser Stelle ist das Dokument entweder in die Ziel-DB geschrieben worden oder es ist in der Liste errordocs

                                    try:
                                        # wenn das dokument gespeichert wurde entfernen wir schon gespeicherte elemente, die beim aktualisieren der pfade
                                        # nicht nochmal gespeichert werden müssen
                                        doc.ftx.clear()
                                        doc.annotations = []
                                        for field in doc.fields.values():
                                            del field
                                        for field in doc.statefields.values():
                                            del field
                                        doc.fields = OrderedDict()
                                        doc.statefields = OrderedDict()
                                        doc.history = []
                                        doc.svn_history = []
                                        doc.workflows = []
                                        doc.links = []
                                        doc.tags = []
                                        doc.commentary = []

                                        if args.move_files:  # Dateien mit umziehen
                                           # docloc
                                            if not dp.create_basename(doc):
                                                log.error("Konnte keinen neuen Basisnamen für [%i] erzeugen" % doc.gdocid)
                                                errordocs.append(doc.gdocid)
                                            else:
                                                # Dateien für doc, oriloc, srcloc umziehen
                                                # wir merken  uns ob die dokumente revisionssicher liegen
                                                docloc_revsafe = doc.docloc.lower().find(my_con.main.archive_rs.lower()) > -1
                                                oriloc_revsafe = doc.oriloc.lower().find(my_con.main.archive_rs.lower()) > -1
                                                srcloc_revsafe = doc.srcloc.lower().find(my_con.main.archive_rs.lower()) > -1

                                                log.debug("Docloc %s in %s revisionssicher: %s" % (doc.docloc.lower(), my_con.main.archive_rs.lower(), docloc_revsafe))
                                                log.debug("Oriloc %s in %s revisionssicher: %s" % (doc.oriloc.lower(), my_con.main.archive_rs.lower(), oriloc_revsafe))

                                                if args.source_dir:
                                                    if doc.extloc:
                                                        doc.extloc = doc.extloc.replace('%bfastore0%', args.source_dir)
                                                    if doc.docloc:
                                                        doc.docloc = doc.docloc.replace('%bfastore0%', args.source_dir)
                                                    if doc.oriloc:
                                                        doc.oriloc = doc.oriloc.replace('%bfastore0%', args.source_dir)
                                                    if doc.srcloc:
                                                        doc.srcloc = doc.srcloc.replace('%bfastore0%', args.source_dir)
                                                    for sapfield in doc.sap:
                                                        if sapfield.sap_content_path:
                                                            sap_content_path = sapfield.sap_content_path
                                                            del sapfield.sap_content_path
                                                            sapfield.sap_content_path = sap_content_path.replace('%bfastore0%', args.source_dir)

                                                if args.source_dir35:
                                                    if doc.extloc:
                                                        try:
                                                            fdir = get_basepath(doc.extloc)  # wir holen den basispfad (unabhängig ob Laufwerksbuchstabe oder UNC)
                                                            doc.extloc = doc.extloc.replace(fdir, args.source_dir35)
                                                        except:
                                                            log.error("Konnte keine Pfadersetzung durchführen für %s mit %s" % (doc.extloc, args.source_dir35))
                                                    if doc.docloc:
                                                        try:
                                                            fdir = get_basepath(doc.docloc)  # wir holen den basispfad (unabhängig ob Laufwerksbuchstabe oder UNC)
                                                            doc.docloc = doc.docloc.replace(fdir, args.source_dir35)
                                                        except:
                                                            log.error("Konnte keine Pfadersetzung durchführen für %s mit %s" % (doc.docloc, args.source_dir35))
                                                    if doc.oriloc:
                                                        try:
                                                            fdir = get_basepath(doc.oriloc)  # wir holen den basispfad (unabhängig ob Laufwerksbuchstabe oder UNC)
                                                            doc.oriloc = doc.oriloc.replace(fdir, args.source_dir35)
                                                        except:
                                                            log.error("Konnte keine Pfadersetzung durchführen für %s mit %s" % (doc.oriloc, args.source_dir35))
                                                    if doc.srcloc:
                                                        try:
                                                            fdir = get_basepath(doc.srcloc)  # wir holen den basispfad (unabhängig ob Laufwerksbuchstabe oder UNC)
                                                            doc.srcloc = doc.srcloc.replace(fdir, args.source_dir35)
                                                        except:
                                                            log.error("Konnte keine Pfadersetzung durchführen für %s mit %s" % (doc.srcloc, args.source_dir35))

                                                    for sapfield in doc.sap:
                                                        if sapfield.sap_content_path:
                                                            try:
                                                                fdir = get_basepath(sapfield.sap_content_path)  # wir holen den basispfad (unabhängig ob Laufwerksbuchstabe oder UNC)
                                                                sap_content_path = sapfield.sap_content_path
                                                                del sapfield.sap_content_path
                                                                sapfield.sap_content_path = sap_content_path.replace(fdir, args.source_dir35)
                                                            except:
                                                                log.error("Konnte keine Pfadersetzung (sap_content_path) durchführen für %s mit %s" % (sapfield.sap_content_path, args.source_dir35))

                                                if doc.extloc:  # erst extloc prüfen, wegen dem sha256 vergleich mit docloc
                                                    if not os.path.exists(os.path.expandvars(doc.extloc)):
                                                        log.warning("Datei existiert nicht %s [%s]" % (os.path.expandvars(doc.extloc), doc.gdocid))
                                                        doc.extloc = ''
                                                    else:
                                                        # mit docloc vergleichen und dubletten aussortieren
                                                        if not os.path.exists(os.path.expandvars(doc.docloc)):
                                                            doc.docloc = doc.extloc
                                                        if doc.docloc != doc.extloc and sha256sum(doc.extloc) == sha256sum(doc.docloc):
                                                            log.warning("Dublette in Extloc gefunden, wird nicht kopiert [%s]" % doc.gdocid)
                                                            doc.extloc = ''
                                                        else:
                                                            new_path = dp.create_arcpath(doc, is_revisionsave=False)
                                                            ext = os.path.splitext(doc.extloc)[1]

                                                            try:
                                                                shutil.copy(doc.extloc, os.path.join(os.path.expandvars(new_path), doc.basename + ext))
                                                            except:
                                                                log.exception("Die Datei konnte nicht nach %s kopiert werden [%s]" % (os.path.join(os.path.expandvars(new_path), doc.basename + ext), doc.gdocid))
                                                            else:
                                                                log.debug("Datei wurde nach %s kopiert (extloc)" % os.path.join(os.path.expandvars(new_path), doc.basename + ext))
                                                                doc.extloc = os.path.join(new_path, doc.basename + ext)

                                                if doc.docloc:
                                                    if not os.path.exists(os.path.expandvars(doc.docloc)):
                                                        log.warning("Datei existiert nicht %s [%s]" % (os.path.expandvars(doc.docloc), doc.gdocid))
                                                        doc.docloc = ''
                                                    else:
                                                        new_path = dp.create_arcpath(doc, is_revisionsave=docloc_revsafe)
                                                        ext = os.path.splitext(doc.docloc)[1]
                                                        ocr_xml = os.path.expandvars(doc.docloc)
                                                        ocr_xml = os.path.splitext(ocr_xml)[0] + '.ocr_xml'
                                                        try:
                                                            shutil.copy(doc.docloc, os.path.join(os.path.expandvars(new_path), doc.basename + ext))
                                                        except:
                                                            log.exception("Die Datei konnte nicht nach %s kopiert werden [%s]" % (os.path.join(os.path.expandvars(new_path), doc.basename + ext), doc.gdocid))
                                                        else:
                                                            log.debug("Datei wurde nach %s kopiert (docloc)" % os.path.join(os.path.expandvars(new_path), doc.basename + ext))
                                                            doc.docloc = os.path.join(new_path, doc.basename + ext)

                                                        if os.path.exists(ocr_xml):
                                                            try:
                                                                shutil.copy(ocr_xml, os.path.join( os.path.expandvars(new_path), doc.basename + '.ocr_xml'))
                                                            except:
                                                                log.exception("Die OCR_XML Datei konnte nicht nach %s kopiert werden [%s]" % (os.path.join(os.path.expandvars(new_path), doc.basename + '.ocr_xml'), doc.gdocid))
                                                            else:
                                                                log.debug("OCR_XML Datei wurde nach %s kopiert" % os.path.join(os.path.expandvars(new_path), doc.basename + '.ocr_xml'))
                                                if doc.oriloc:
                                                    if not os.path.exists(os.path.expandvars(doc.oriloc)):
                                                        log.warning("Datei existiert nicht %s [%s]" % (os.path.expandvars(doc.oriloc), doc.gdocid))
                                                        doc.oriloc = ''
                                                    else:
                                                        new_path = dp.create_arcpath(doc, is_revisionsave=oriloc_revsafe)
                                                        ext = os.path.splitext(doc.oriloc)[1]
                                                        ind = 0
                                                        for sapfield in doc.sap:
                                                            if sapfield.sap_content_path:
                                                                if sapfield.sap_content_path.lower() == doc.oriloc.lower():  # Oriloc wird sowieso umkopiert, wir müssen nur den Pfad anpassen
                                                                    del sapfield.sap_content_path
                                                                    sapfield.sap_content_path = os.path.join(new_path, doc.basename + ext)
                                                                else:                                       # kommt praktisch wohl nicht vor, wenn auch theoretisch möglich
                                                                    ind += 1
                                                                    sap_path = os.path.join(new_path, (doc.basename + '_%s' % ind + ext))
                                                                    try:
                                                                        shutil.copy(os.path.expandvars(sapfield.sap_content_path), os.path.expandvars(sap_path))
                                                                    except:
                                                                        log.exception("Die Datei (SAP) konnte nicht nach %s kopiert werden [%s]" % (os.path.expandvars(sap_path), doc.gdocid))
                                                                    else:
                                                                        log.debug("Datei (SAP) wurde nach %s kopiert (oriloc)" % os.path.expandvars(sap_path))
                                                                        del sapfield.sap_content_path
                                                                        sapfield.sap_content_path = sap_path

                                                        try:
                                                            shutil.copy(doc.oriloc, os.path.join(os.path.expandvars(new_path), doc.basename + ext))
                                                        except:
                                                            log.exception("Die Datei konnte nicht nach %s kopiert werden [%s]" % (os.path.join(os.path.expandvars(new_path), doc.basename + ext), doc.gdocid))
                                                        else:
                                                            log.debug("Datei wurde nach %s kopiert (oriloc)" % os.path.join(os.path.expandvars(new_path), doc.basename + ext))
                                                            doc.oriloc = os.path.join(new_path, doc.basename + ext)

                                                if doc.srcloc:
                                                    if not os.path.exists(os.path.expandvars(doc.srcloc)):
                                                        log.warning("Datei existiert nicht %s [%s]" % (os.path.expandvars(doc.srcloc), doc.gdocid))
                                                        doc.srcloc = ''
                                                    else:
                                                        new_path = dp.create_arcpath(doc, is_revisionsave=srcloc_revsafe)
                                                        ext = os.path.splitext(doc.srcloc)[1]

                                                        try:
                                                            shutil.copy(doc.srcloc, os.path.join(os.path.expandvars(new_path), doc.basename + ext))
                                                        except:
                                                            log.exception("Die Datei konnte nicht nach %s kopiert werden [%s]" % (os.path.join(os.path.expandvars(new_path), doc.basename + ext), doc.gdocid))
                                                        else:
                                                            log.debug("Datei wurde nach %s kopiert (srcloc)" % os.path.join(os.path.expandvars(new_path), doc.basename + ext))
                                                            doc.srcloc = os.path.join(new_path, doc.basename + ext)

                                                if not dp.write(doc):
                                                    log.error("Das Dokument [%i] konnte nicht aktualisiert werden (Pfade)" % doc.gdocid)
                                                    errordocs.append(doc.gdocid)
                                                else:
                                                    log.debug("Das Dokument [%i] wurde aktualisiert (Pfade)" % doc.gdocid)
                                                    # row = result.pop()
                                                    # continue
                                        elif not args.gpl and my_con.main.version == 35:
                                            if doc.extloc:  # erst extloc prüfen, wegen dem sha256 vergleich mit docloc
                                                new_loc = ''
                                                if not os.path.exists(os.path.expandvars(doc.extloc)):
                                                    log.warning("Datei existiert nicht %s [%s]" % (os.path.expandvars(doc.extloc), doc.gdocid))
                                                    doc.extloc = ''
                                                else:
                                                    if doc.extloc.lower().find(my_con.main.archive_rs.lower()) > -1:
                                                        new_loc = doc.extloc.lower().replace(my_con.main.archive_rs.lower(), os.path.join(dc.main.storages[doc.storageid], dc.main.archive_rs + '\\'))
                                                    elif doc.extloc.lower().find(my_con.main.archive_nrs.lower()) > -1:
                                                        new_loc = doc.extloc.lower().replace(my_con.main.archive_nrs.lower(), os.path.join(dc.main.storages[doc.storageid], dc.main.archive_nrs + '\\'))
                                                    else:
                                                        log.warning("Der Pfad %s konnte nicht ins neue Format konvertiert werden" % doc.extloc)
                                                    if os.path.exists(os.path.expandvars(new_loc)):
                                                        doc.extloc = new_loc
                                            if doc.docloc:
                                                new_loc = ''
                                                if not os.path.exists(os.path.expandvars(doc.docloc)):
                                                    log.warning("Datei existiert nicht %s [%s]" % (os.path.expandvars(doc.docloc), doc.gdocid))
                                                    doc.docloc = ''
                                                else:
                                                    if doc.docloc.lower().find(my_con.main.archive_rs.lower()) > -1:
                                                        new_loc = doc.docloc.lower().replace(my_con.main.archive_rs.lower(), os.path.join(dc.main.storages[doc.storageid], dc.main.archive_rs + '\\'))
                                                    elif doc.docloc.lower().find(my_con.main.archive_nrs.lower()) > -1:
                                                        new_loc = doc.docloc.lower().replace(my_con.main.archive_nrs.lower(), os.path.join(dc.main.storages[doc.storageid], dc.main.archive_nrs + '\\'))
                                                    else:
                                                        log.warning("Der Pfad %s konnte nicht ins neue Format konvertiert werden" % doc.docloc)
                                                    if os.path.exists(os.path.expandvars(new_loc)):
                                                        doc.docloc = new_loc
                                            if doc.oriloc:
                                                new_loc = ''
                                                if not os.path.exists(os.path.expandvars(doc.oriloc)):
                                                    log.warning("Datei existiert nicht %s [%s]" % (os.path.expandvars(doc.oriloc), doc.gdocid))
                                                    doc.oriloc = ''
                                                else:
                                                    if doc.oriloc.lower().find(my_con.main.archive_rs.lower()) > -1:
                                                        new_loc = doc.oriloc.lower().replace(my_con.main.archive_rs.lower(), os.path.join(dc.main.storages[doc.storageid], dc.main.archive_rs + '\\'))
                                                    elif doc.oriloc.lower().find(my_con.main.archive_nrs.lower()) > -1:
                                                        new_loc = doc.oriloc.lower().replace(my_con.main.archive_nrs.lower(), os.path.join(dc.main.storages[doc.storageid], dc.main.archive_nrs + '\\'))
                                                    else:
                                                        log.warning("Der Pfad %s konnte nicht ins neue Format konvertiert werden" % doc.oriloc)
                                                    if os.path.exists(os.path.expandvars(new_loc)):
                                                        for sapfield in doc.sap:
                                                            if sapfield.sap_content_path:
                                                                if sapfield.sap_content_path.lower() == doc.oriloc.lower():
                                                                    del sapfield.sap_content_path
                                                                    sapfield.sap_content_path = new_loc
                                                                else:
                                                                    sap_path = sapfield.sap_content_path
                                                                    if sapfield.sap_content_path.lower().find(my_con.main.archive_rs.lower()) > -1:
                                                                        sap_path = sapfield.sap_content_path.lower().replace(my_con.main.archive_rs.lower(), os.path.join(dc.main.storages[doc.storageid], dc.main.archive_rs + '\\'))
                                                                    elif sapfield.sap_content_path.lower().find(my_con.main.archive_nrs.lower()) > -1:
                                                                        sap_path = sapfield.sap_content_path.lower().replace(my_con.main.archive_nrs.lower(), os.path.join(dc.main.storages[doc.storageid], dc.main.archive_nrs + '\\'))
                                                                    del sapfield.sap_content_path
                                                                    sapfield.sap_content_path = sap_path

                                                        doc.oriloc = new_loc
                                            if doc.srcloc:
                                                new_loc = ''
                                                if not os.path.exists(os.path.expandvars(doc.srcloc)):
                                                    log.warning("Datei existiert nicht %s [%s]" % (os.path.expandvars(doc.srcloc), doc.gdocid))
                                                    doc.srcloc = ''
                                                else:
                                                    if doc.srcloc.lower().find(my_con.main.archive_rs.lower()) > -1:
                                                        new_loc = doc.srcloc.lower().replace(my_con.main.archive_rs.lower(), os.path.join(dc.main.storages[doc.storageid], dc.main.archive_rs + '\\'))
                                                    elif doc.srcloc.lower().find(my_con.main.archive_nrs.lower()) > -1:
                                                        new_loc = doc.srcloc.lower().replace(my_con.main.archive_nrs.lower(), os.path.join(dc.main.storages[doc.storageid], dc.main.archive_nrs + '\\'))
                                                    else:
                                                        log.warning("Der Pfad %s konnte nicht ins neue Format konvertiert werden" % doc.srcloc)
                                                    if os.path.exists(os.path.expandvars(new_loc)):
                                                        doc.srcloc = new_loc

                                            if not dp.write(doc):
                                                log.error("Das Dokument [%i] konnte nicht aktualisiert werden (Pfade)" % doc.gdocid)
                                                errordocs.append(doc.gdocid)
                                            else:
                                                log.debug("Das Dokument [%i] wurde aktualisiert (Pfade)" % doc.gdocid)
                                    except:
                                        log.exception("Allgemeiner Fehler. Dokument [%i] zu errordocs hinzugefügt" % doc.gdocid)
                                        errordocs.append(doc.gdocid)

                            if args.sync:
                                dp.db["main"].execute("COMMIT;")

                        # Dokument fertig verarbeitet
                        del doc
                        doc = None
                        try:
                            row = result.pop()
                        except:
                            row = None

                        if args.debug:
                            compare_snapshots()

                        if not row:
                            # result ist durch, jetzt prüfen, ob Fehler aufgetreten sind und neu importiert werden muss
                            if len(errordocs) > 0:
                                # transaction rollback, importeddocs-Liste als neues result laden
                                if len(errordocs) == 1:
                                    log.error('Es konnte ein Dokument nicht verarbeitet werden.')
                                else:
                                    log.error('Es konnten %d Dokumente nicht verarbeitet werden.' % len(errordocs))
                                if not args.sync:
                                    dp.db["main"].execute("ROLLBACK;")
                                result = [[gdocid] for gdocid in importeddocs if gdocid not in errordocs]
                                if len(result) > 0:
                                    result.reverse()
                                    row = result.pop()
                                rowcount = len(result)
                                cnt = 1
                                # errordocs, srcnotloadeddocs in Logfile schreiben
                                log.critical("%i Dokumente konnten nicht importiert werden: %s" % (len(errordocs), ", ".join([str(x) for x in errordocs])))
                                if len(srcnotloadeddocs) > 0:
                                    log.critical("%i Dokumente konnten aus der Quell-Datenbank nicht geladen werden: %s" % (len(srcnotloadeddocs), ", ".join([str(x) for x in srcnotloadeddocs])))
                                errordocs = []
                                importeddocs = []
                                srcnotloadeddocs = []
                                sync_ids = []
                                # neue transaction starten
                                if len(result) > 0:
                                    if not args.sync:
                                        dp.db["main"].execute("START TRANSACTION;")
                                else:
                                    break   # keine Dokumente die erneut reinlaufen sollen (also alle 1000 waren fehlerhaft) -> Schleife verlassen
                            else:
                                if len(srcnotloadeddocs) > 0:
                                    log.critical("%i Dokumente konnten aus der Quell-Datenbank nicht geladen werden: %s" % (len(srcnotloadeddocs), ", ".join([str(x) for x in srcnotloadeddocs])))
                                    srcnotloadeddocs = []
                                else:
                                    log.debug("Alle Dokumente in dieser Runde importiert, keine Fehler.")

                    if not args.sync:
                        dp.db["main"].execute("COMMIT;")
                    if rowcount > 0:
                        log.info("round %d, average time per doc %04f in this round" % (roundno, (time.time() - starttime) / float(rowcount)))

                    roundno += 1

                    if args.sync:
                        if args.sync[0] == 'sys_mgr' and len(sync_ids) > 0:
                            sql = 'delete from sys_mgr where gdocid in (%s)' % ",".join([str(x) for x in sync_ids])
                            try:
                                my_con.main.db.execute(sql)
                            except:
                                log.exception("Fehler beim Entfernen von Gdocids aus der sys_mgr")

                            # Nullwerte müssen sowieso gelöscht werden
                            sql = 'delete from sys_mgr where gdocid is NULL'
                            try:
                                my_con.main.db.execute(sql)
                            except:
                                log.exception("Fehler beim Entfernen von NULL Werten aus der sys_mgr")
                else:   # no result?
                    break

            result = None

            if args.debug:
                compare_snapshots()

    # wenn wir only_annos nur mit einer Anmeldung an profile durchführen wollen, muss hier eine eigene Datenbankverbindung aufgebaut werden
    if not args.clone_to and not args.migrate_to and args.only_annos:
        log.info("Verbinde mit %s" % args.profile)
        clonestart = time.time()
        roundno = 0
        # packsize = 1000
        cnt = 1
        # connection ohne autocommit erstellen
        dc = bitfarm_proc.BitfarmConnections(args.profile, gpl=args.create_gpl)
        dc.create_db_engine(autocommit=False, echo=False)
        dc.connect_db()
        dp = bitfarm_proc.BitfarmDocProcessing(dc)
        dp.db["main"].execute("SET innodb_lock_wait_timeout = 50000;")
        dp.db["main"].execute("SET unique_checks=0;")
        dp.db["main"].execute("SET foreign_key_checks=0;")
        dp.db["main"].execute("SET NAMES utf8mb4;")

        if args.debug:
            tracemalloc.start()
            if not tracefn:
                tracefn = open('tracemalloc_%s.log' % datetime.datetime.now().strftime('%Y-%m-%d %H.%M.%S'), 'w')
            gc.collect()
            compare_snapshots()

    if not args.clone_to and (args.migrate_to or args.only_annos):
        # wir prüfen, ob wir annotations haben und schreiben den annotext
        if args.reload_iev:
            log.info("reload_iev: es werden nicht importierte IEV Dateien nachverarbeitet.")
            sql = 'SELECT gdocid, docloc, extloc FROM sys_doc where gdocid not in (select gdocid from sys_annotations)'
            try:
                result = dp.db["main"].execute(sql).fetchall()
            except:
                log.exception("Fehler beim Abfragen der Dokumente ohne Annotations")

            if result:
                i = 0
                imported = 0
                log.info("Annotations für %s Dokumente werden nachverarbeitet" % len(result))
                dp.db["main"].execute("START TRANSACTION;")
                try:
                    for row in result:
                        if row[0] is None:
                            continue
                        annotxt = ""
                        if row[1] is None and row[2] is None:
                            i += 1
                            continue
                        annopath = row[2] if row[2] else row[1]
                        annopath = os.path.expandvars(annopath).lower()
                        annopath = annopath.replace(my_con.main.archive_rs.lower(),
                                                    my_con.main.archive_nrs.lower())
                        annopath = os.path.splitext(annopath)[0]

                        ievlist = glob.iglob(annopath + "*.iev")
                        # new_path could be rs-path, but iev should be placed in nrs-path
                        #                            nrs_path = dp.create_arcpath(doc, is_revisionsave = False)
                        for iev in ievlist:
                            ievmeta = iev.replace(annopath, "")[:-4]
                            if os.path.exists(os.path.expandvars(iev)):
                                ext = iev.replace(annopath, '')
                                # determine page + public/private via filename
                                try:
                                    match = re.match("(\d*)[Pp]*(\d*)", ievmeta)
                                    if match:
                                        page, userid = match.groups()
                                        if len(userid) == 0:
                                            userid = "0"
                                        if len(page) == 0:
                                            page = "0"
                                        if len(userid) > 3: #größer drei stellen, kann nicht sein
                                            userid = "0"
                                        if len(page) > 3:   # mehr als 999 seiten und dann eine anno? wohl kaum
                                            page = "0"
                                    else:
                                        log.error("Layer und Seite konnte nicht ermittelt werden, verwende verwende Seite 1 Layer 0, iev = %s" % ievmeta)
                                        userid = "0"
                                        page = "0"
                                except:
                                    log.error("Fehler beim Ermitteln von Layer und Seite: %s" % sys.exc_info()[1].message)
                                else:
                                    anno = bitfarm_doc.BitfarmAnnotation()
                                    anno.iev = bitfarm_doc.BitfarmStream()
                                    anno.page = int(page)
                                    anno.layer = int(userid)
                                    anno.modified = True
                                    try:
                                        with open(os.path.expandvars(iev), 'rb') as f:
                                            anno.iev.stream.write(f.read())
                                    except:
                                        log.exception("Fehler beim Lesen der Annotation-Datei %s" % iev)
                                        i += 1
                                        continue
                                    else:
                                        if dp.save_annotation(anno, row[0], dc.main.profile):
                                            imported += 1
                                            log.info("Annotation importiert für Dokument [%s]" % str(row[0]))
                                        else:
                                            log.error("Annotation für Dokument [%s] konnte nicht importiert werden" % str(row[0]))
                        i += 1

                        if i % args.packsize == 0:
                            log.info("%d Dokumente verarbeitet..." % args.packsize)
                            dp.db["main"].execute("COMMIT;")
                            dp.db["main"].execute("START TRANSACTION;")

                    log.info("%d Dokumente verarbeitet..." % (i % args.packsize))
                    log.info("%d Annotations importiert..." % imported)
                    dp.db["main"].execute("COMMIT;")
                except:
                    log.exception("Algemeiner Fehler beim Nachverarbeiten von Annotations")
                    dp.db["main"].execute("Rollback;")

        log.info("Schritt 16: Annotext erzeugen")
        sql = 'select distinct(gdocid) from sys_annotations'
        if args.sync:
            if len(sync_ids) > 0:
                sql += ' where gdocid in (%s)' % ",".join([str(x) for x in sync_ids])
            else:
                sql += ' where gdocid in (0)'   # diese abfrage findet nichts
        elif import_mode:
            sql += ' where gdocid in (%s)' % ",".join([str(x) for x in all_imported])
        else:
            sql += ' where gdocid >= %s' % args.from_gdocid
            if args.to_gdocid:
                sql += ' and gdocid < %s' % args.to_gdocid

        try:
            result = dp.db["main"].execute(sql).fetchall()
        except:
            log.exception("Fehler beim Abfragen der Annotations")

        if result:
            i = 0
            log.info("Annotexte für %s Dokumente werden erzeugt" % len(result))
            dp.db["main"].execute("START TRANSACTION;")
            try:
                for row in result:
                    if row[0] is None:
                        continue
                    if args.debug:
                        compare_snapshots()
                    annotxt = ""
                    time.sleep(0.01)
                    annotxt = get_annotext(dc.main, row[0])
                    log.debug("Annotext für [%s] gelesen" % row[0])
                    if not notstdout:
                        sys.stdout.write("Verarbeite Annotext für [%i]\r" % row[0])
                        sys.stdout.flush()
                    if annotxt:
                        vals = {dc.main.sys_doctxt.c.annotxt: annotxt}
                        sql = dc.main.sys_doctxt.update(whereclause=(dc.main.sys_doctxt.c.gdocid == row[0]), values=vals)
                        try:
                            dp.db["main"].execute(sql)
                        except:
                            log.exception("Fehler beim Speichern des Annotextes für Dokument %s" % row[0])
                        else:
                            i += 1

                        if i % args.packsize == 0:
                            log.info("übertrage %d Annotexte" % args.packsize)
                            dp.db["main"].execute("COMMIT;")
                            time.sleep(0.01)
                            dp.db["main"].execute("START TRANSACTION;")

                log.info("übertrage %d Annotexte" % (i % args.packsize))
                dp.db["main"].execute("COMMIT;")
            except:
                log.exception("Allgemeiner Fehler beim Speichern des Annotextes bei Dokument %s" % row[0])
                dp.db["main"].execute("Rollback;")
            if args.debug:
                compare_snapshots()
                tracefn.close()
                del _TRACES
        else:
            log.info("Es konnten keine Annotations gefunden werden.")

        if args.clone_to:
            log.info("Clonen durchgeführt")
        else:
            dp.db["main"].execute("SET unique_checks=1;")
            dp.db["main"].execute("SET foreign_key_checks=1;")

            log.info("Gesamtdauer %02f Sekunden" % (time.time() - clonestart))
            log.info("Migration durchgeführt" if not import_mode else "Import durchgeführt")

    if args.check and not args.skip_check and not args.clone_to:
        log.info("Abgleich alter Datenbestand (%s) gegen neuen Datenbestand (%s)" % (my_con.main.profile, dest_con.main.profile))

        cnt_docs_sum = 0
        cnt_docs_dest = 0
        cnt_missing = 0
        missing_list = []

        if import_mode:
            try:
                result = dest_con.main.db.execute('select gdocid from sys_doc where gdocid in (%s)' % ",".join([str(x) for x in all_imported])).fetchall()
            except:
                log.exception("Fehler bei Abfrage der importierten Dokumente")
            else:
                if len(result) != len(all_imported):
                    log.error("Es konnten nicht alle Dokumente in der Zieldatenbank gefunden werden")
                    set_dest = set()

                    for row in result:
                        if row[0] is not None:
                            set_dest.add(row[0])

                    missing = all_imported.difference(set_dest)

                    if len(missing) > 0:
                        missing_list.extend(missing)
                else:
                    log.info("Es wurden alle %s importierten Dokumente in der Zieldatenbank gefunden" % len(result))
        else:
            arcs = my_con.main.db.execute('select * from sys_arc where type = 1').fetchall()

            if not arcs:
                # log.error("Archivtabelle von %s konnte nicht abgefragt werden" % my_con.main.profile)
                sys_exit(31, my_con.main.profile)

            for arc in arcs:
                log.info("Prüfe Archiv %s" % arc[2])
                try:
                    if my_con.main.version == 35:
                        cnt_docs = my_con.main.db.execute("select count(*) from arc_%s" % arc[3]).fetchone()
                    elif my_con.main.version == 36:
                        cnt_docs = my_con.main.db.execute("select count(*) from sys_doc where arcid = %s" % arc[0]).fetchone()
                    else:
                        cnt_docs = None
                except:
                    log.exception("Fehler bei der Abfrage des Archivs")
                    cnt_docs = None
                if cnt_docs:
                    log.info("- Anzahl Dokumente %s" % cnt_docs[0])
                    cnt_docs_sum += int(cnt_docs[0])
                else:
                    log.info("- Keine Dokumente in dem Archiv vorhanden")

                result = True
                round = 0
                while result:
                    result = None
                    cnt_docids = 0
                    if my_con.main.version == 35:
                        # result = my_con.main.db.execute("select gdoc_id from arc_%s limit %s , 100" % (arc[3], round * 100)).fetchall()
                        try:
                            res = my_con.main.db.execute("select doc_id from arc_%s limit %s, 100" % (arc[3], round * 100)).fetchall()
                        except:
                            log.exception("Fehler beim Laden der Dokument IDs aus der Quelldatenbank (1)")
                        else:
                            docids = []
                            if res:
                                for row in res:
                                    docids.append(str(row[0]))
                                cnt_docids = len(docids)
                                docids = ",".join(docids)

                                try:
                                    result = my_con.main.db.execute("select g.gdoc_id from sys_gdocid as g " \
                                                                    "where g.arc_id = %s and g.doc_id in (%s)" % (arc[0], docids)).fetchall()
                                except:
                                    log.exception("Fehler beim Laden der Dokument IDs aus der Quelldatenbank (2)")

                    elif my_con.main.version == 36:
                        try:
                            result = my_con.main.db.execute("select gdocid from sys_doc where arcid = %s limit %s , 100" % (arc[0], round * 100)).fetchall()
                        except:
                            log.exception("Fehler beim Laden der Dokument IDs aus der Quelldatenbank")

                    if result:
                        if len(result) != cnt_docids:
                            if my_con.main.version == 35:
                                log.error("Achtung: Anzahl der Einträge in der arc_Tabelle differiert mit den gefundenen Dokumenten in der sys_gdocid")
                                log.error("arc_tabelle %s  sys_gdocid %s" % (str(cnt_docids), str(len(result))))
                            else:
                                log.error("Achtung: Anzahl der Einträge im Quellarchiv differiert mit den gefundenen Dokumenten im Zielarchiv")

                        ids = []
                        set_ids = set()
                        set_dest = set()
                        for row in result:
                            if row[0] is not None:
                                ids.append(str(row[0]))
                                set_ids.add(row[0])
                        if ids:
                            ids = ",".join(ids)
                            try:
                                result_dest = dest_con.main.db.execute("select gdocid from sys_doc where gdocid in (%s)" % ids).fetchall()
                            except:
                                log.exception("Fehler beim Laden der Dokument IDs aus der Zieldatenbank")
                            else:
                                if result_dest:
                                    for row in result_dest:
                                        if row[0] is not None:
                                            set_dest.add(row[0])

                        missing = set_ids.difference(set_dest)

                        if len(missing) > 0:
                            missing_list.extend(missing)
                            missing_ids = ",".join([str(x) for x in missing])
                            log.error('Runde %s: %s Dokumente fehlen in der Zieldatenbank %s' % (str(round + 1), len(missing), missing_ids))
                        else:
                            log.info("Runde %s: keine fehlenden Dokumente" % str(round + 1))
                    else:
                        if cnt_docids > 0:
                            log.error("Keine Einträge in der sys_gdocid gefunden. Gesucht wurde arcid %s doc_id %s" % (arc[0], docids))

                    round += 1

            log.info("Anzahl Dokument Quelldatenbank %s: %s" % (my_con.main.profile, cnt_docs_sum))
            result = dest_con.main.db.execute("select count(*) from sys_doc where gdocid <> 0").fetchone()
            if result:
                log.info("Anzahl Dokument Zieldatenbank %s: %s" % (dest_con.main.profile, result[0]))
            else:
                log.warning("In der Zieldatenbank sind keine Dokumente enthalten.")

        if len(missing_list) > 0:
            log.critical("Achtung! In der Zieldatenbank fehlen %i Dokument(e):" % len(missing_list))
            log.critical("%s" % ",".join([str(x) for x in missing_list]))

    log.info("Alle Vorgänge durchgeführt")
