﻿#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Diese Modul stellt alle Funktionen des bfaServers zur Verfügung. Sie sind in den Klassen
BitfarmLoginThread, BitfarmWorkerThread, BitfarmSocketConnector, BitfarmDLMCheck und BitfarmQueueCheck organisiert.
"""

__version__ = "v3.6.2.0"
# _ = lambda x: x

#import cProfile
import contextlib
#import pstats
#import trace
#import timeit
import traceback
import os
import sys
import re
import threading
import select
import shutil
import queue
import logging
#import logging.config
import datetime, time
import io
import warnings
# import configparser
import subprocess
import win32com
import win32com.client
# from win32com.client import Dispatch, GetActiveObject
import pythoncom
import socket
import ssl
import gettext
from collections import deque
from collections import OrderedDict
import sqlalchemy as sqla
# from pyad import *
import ldap
import hashlib
import base64
import json
import gzip
import uuid
import tempfile
import smtplib
import zipfile
import codecs
import mmap

from email.mime.multipart import MIMEMultipart
from email.mime.application import MIMEApplication
from email.mime.base import MIMEBase
from email.mime.text import MIMEText
from email.utils import formataddr
from email.header import Header

#import bitfarm_baseobj
import bitfarm_doc
import bitfarm_proc
import bitfarm_configobj
import bitfarm_oauth
# import bitfarm_files
import bitfarm_xml
#import bitfarm_workflow
import bitfarm_errors
from bitfarm_errors import *
import Renderer
from bfaRenderClient import BfaRenderClient
from bfa_tessocr import BfaTessOCR
from bfa_hocr import BfaHocr
#import bfaClassifier
import dpapi

try:
    import thtiflib

    thtiflibimported = True
except:
    thtiflibimported = False

# import glob
import atexit
# import ghostscript
# import pyPdf
import random
import glob
#import signal
import hashlib
#import bfaWorkflowCore
#import bfaProcessingCore
#import bfaCustomizing
from dateutil.relativedelta import *
# import numpy
# import cv2
# from keras.preprocessing.image import array_to_img, img_to_array

#_ = lambda x: x

# class BitfarmWorkerThread(bfaServerClasses.BitfarmWorkerThread):
#    pass
#
# class BitfarmSocketConnector(bfaServerClasses.BitfarmSocketConnector):
#    pass

# class bfaWorkflow(bfaWorkflowCore.bfaWorkflow):
#    pass
##TODO: klären ob bfaCustomizing in der GPL überhaupt geladen werden soll
# class bfaProcessing(bfaProcessingCore.bfaProcessing, bfaCustomizing.bfaCustomizing):
#    pass

repl_percent = lambda x: '%%' if x.group(0) == '%' else x.group(0)
repl_backslash = lambda x: '\\\\' if x.group(0) == '\\' else x.group(0)

BfaClassifier = None

EDITION = 'GPL'

def read_registry_file(bfaserver, profile, cipher):
    bfaserver.waitingperiod[profile] = None
    bfaserver.con[profile].named = -1
    bfaserver.con[profile].concurrent = -1
@contextlib.contextmanager
# def bfa_tempdir(args: object, kwargs: object) -> object:
def bfa_tempdir(*args, **kwargs):
    d = tempfile.mkdtemp(*args, **kwargs)
    try:
        yield d
    finally:
        shutil.rmtree(d)

class MyFile():
    def __init__(self, filename):
        self.file = open(filename, "rb")
        self.filename = filename

    def __del__(self):
        self.file.close()


class BitfarmLoginThread(threading.Thread):
    """Diese Klasse stellt den zentralen Thread zur Authentifizierung zur Verfügung. Über die Queue dieses Threads
    werden die Authentifizierungsrequests übergeben. Die Funktion login() sucht den entsprechenden User in der Datenbank
    und führt entweder ein Prüfung gegen das hinterlegte Passwort oder bei AD-User per LDAP gegen den Domaincontroller durch.
    Nach erfolgreicher Authentifizierung wird der User in die Liste der authentifizierten User bfaServer.authuser und die
    entsprechenden Berechtigungen in die bfaServer.userrights Liste übernommen.
    Die weiteren Funktionen dieser Klasse kümmern sich um den Aufbau bzw Bereinigen dieser Listen."""
    queue = queue.Queue(0)
    log = logging.getLogger("Main.BitfarmLoginThread")
    authlock = threading.Lock()

    def __init__(self, bfaserver):
        threading.Thread.__init__(self)
        self.stopsignal = False
        self.bfaserver = bfaserver
        self.timeout = 0
        self._job = None
        self.db = {}
        for con in list(self.bfaserver.con.connections.values()):
            self.db[con.profile] = con.engine.connect()
        self.log.debug(_('gestartet %s') % self.name)

    def reconnect(self, _profile):
        self.log.debug("%s reconnect" % self.name)
        _ret = False

        if self.bfaserver.con[_profile].engine is not None:
            try:
                self.db[_profile] = self.bfaserver.con[_profile].engine.connect().execution_options(autocommit=True)
            except:
                self.log.error(_("%s Reconnect gescheitert") % self.name)
            else:
                if not isinstance(self.db[_profile], sqla.engine.base.Connection):
                    self.log.error(_("%s Reconnect gescheitert") % self.name)
                else:
                    self.log.warning(_("%s Reconnect erfolgreich db = %s") % (self.name, self.db[_profile]))
                    _ret = True

        return _ret

    def read_userrights(self, userid, profile, db=None):
        self.log.debug("%s read_userrights" % self.name)
        if db is None:
            db = self.db[profile]

        userrights = {}
        try:
            con = self.bfaserver.con.connections[profile]
            grouplist = con.resolve_membership(db=db, userid=userid, logid=self.name)
            if len(grouplist) == 0:
                grouplist = [0]
            grouplist = ",".join([str(x) for x in grouplist])

            # sql = sqla.select([con.sys_arcxs]).where(con.sys_arcxs.c.userid == userid).order_by(con.sys_arcxs.c.arcid)
            sql = 'SELECT sys_arcxs.arcxsid, sys_arcxs.arcid, sys_arcxs.userid, sys_arcxs.groupid, sys_arcxs.rights +0 ' \
                  'FROM sys_arcxs WHERE sys_arcxs.userid = %s or (sys_arcxs.userid = 0 and sys_arcxs.groupid in (%s)) ' \
                  'ORDER BY sys_arcxs.arcid' % (userid, grouplist)
            #        sql = sql.where(self.bfaserver.con.main.sys_user.c.userpass ==userpass)

            result = db.execute(sqla.text(sql)).fetchall()

            if result:
                for element in result:
                    if element[4]:
                        if element[1] in userrights:
                            userrights[element[1]] |= int(element[4])
                        else:
                            userrights[element[1]] = int(element[4])

                BitfarmLoginThread.authlock.acquire()
                try:
                    self.bfaserver.userrights[profile][userid] = userrights
                finally:
                    BitfarmLoginThread.authlock.release()
        except:
            self.log.exception(_("%s Fehler bei Abfrage von sys_arcxs. Konnte Userrechte für User %s nicht ermitteln") % (self.name, userid))

    def login(self, userkey, username, userpass, sock, profile, lang):
        self.log.debug("%s login %s <%s>" % (self.name, username, hex(id(sock))))
        try:
            auth = False
            xml = ""
            msg = ""
            succeed = False

            con = self.bfaserver.con.connections[profile]

            if self.bfaserver.waitingperiod[profile] is not None:
                period = self.bfaserver.waitingperiod[profile].date() + datetime.timedelta(days=7) - datetime.datetime.now().date()
                if period.days < 1:
                    con.named = 1       # karenz abgelaufen, runterstufen auf 1 named (+dmsadmin)
                    con.concurrent = 0

            if con.sys_user is None:
                con.sys_user = sqla.Table('sys_user', con.meta, autoload=True)

            if username is None:
                sql = sqla.select([con.sys_user]).where(con.sys_user.c.userid == userkey[0])
            else:
                sql = sqla.select([con.sys_user]).where(sqla.or_(con.sys_user.c.username == username,
                                                                 con.sys_user.c.displayname == username))
            try:
                result = self.db[con.profile].execute(sql).fetchall()
            except sqla.exc.OperationalError:
                # versuche reconnect
                self.log.warning(_("Login: keine Verbindung zur Datenbank, versuche reconnect..."))
                time.sleep(.5)
                try:
                    self.db[con.profile] = con.engine.connect()
                except:
                    self.log.exception(_("Reconnect gescheitert"))
                    result = False
                    xml = bitfarm_xml.create_xml_request(reqtype="respAuthError", data=_("Keine Verbindung zur Datenbank."), datatype="text")
                else:
                    if username is None:
                        sql = sqla.select([con.sys_user]).where(con.sys_user.c.userid == userkey[0])
                    else:
                        sql = sqla.select([con.sys_user]).where(sqla.or_(con.sys_user.c.username == username,
                                                                         con.sys_user.c.displayname == username))
                    try:
                        result = self.db[con.profile].execute(sql).fetchall()
                    except:
                        self.log.exception(_("Zweite Abfrage gescheitert."))
                        result = False
                        xml = bitfarm_xml.create_xml_request(reqtype="respAuthError", data=_("Keine Verbindung zur Datenbank."), datatype="text")
                    else:   # wenn wir jetzt erfolgreich nach dem Reconnect abgefragt haben, muss der Server noch wissen, dass er seine Threads reconnecten muss
                        self.bfaserver.need_reconnect = True

                if not result:
                    try:
                        sock.send(xml)
                    except:
                        self.log.exception(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                    return None

            deleted = False
            not_enabled = False
            from_ad = False
            found = False
            ambiguous = False
            uname = ""
            upass = ""
            if result:
                self.log.debug("%s Eintrag gefunden %s <%s>" % (self.name, username, hex(id(sock))))
                for row in result:
                    uname = row[1]
                    upass = row[2]
                    from_ad = 'from_ad' in row[4]
                    deleted = 'deleted' in row[4]
                    not_enabled = 'enabled' not in row[4]
                    concurrent = 'concurrent' in row[4]
                    admin = 'dmsadmin' in row[4] and uname.lower() == 'dmsadmin'
                    if deleted:
                        continue
                    elif not_enabled:
                        continue
                    elif username is not None:  # wurde eine Name angegeben prüfen wir diesen, da er in username oder displayname stehen kann
                        if username.lower() == row[1].lower():
                            userkey = (row[0], userkey[1])
                            found = True
                            break
                        elif username.lower() == row[5].lower():  # displayname
                            if len(result) > 1:     # mehrere gleichnamige Displaynames? über eindeutigen Benutzernamen anmelden
                                ambiguous = True
                                break
                            else:
                                userkey = (row[0], userkey[1])
                                found = True
                                break
                    elif userkey[0] == row[0]:  # über die ID gefunden
                        userkey = (row[0], userkey[1])
                        found = True
                        break

                if deleted:
                    self.log.debug("%s Der Benutzer wurde gelöscht %s <%s>" % (self.name, username, hex(id(sock))))
                    xml = bitfarm_xml.create_xml_request(reqtype="respAuthError", data=_("Der Benutzer wurde gelöscht."), datatype="text")
                elif not_enabled:
                    self.log.debug("%s Der Benutzer wurde deaktiviert %s <%s>" % (self.name, username, hex(id(sock))))
                    xml = bitfarm_xml.create_xml_request(reqtype="respAuthError", data=_("Der Benutzer wurde deaktiviert."), datatype="text")
                if found:
                    if from_ad:
                        if userpass is None:
                            auth = False
                            self.log.error("%s Leere Passwörter sind bei AD-Benutzern nicht erlaubt. %s <%s>" % (self.name, username, hex(id(sock))))
                            xml = bitfarm_xml.create_xml_request(reqtype="respAuthError", data=_("Leere Passwörter sind bei AD-Benutzern nicht erlaubt."), datatype="text")
                        else:
                            self.log.debug("%s Anmeldung am AD %s <%s>" % (self.name, username, hex(id(sock))))

                            if self.bfaserver.ldapdebug or con.ldap_debug:
                                flog = open(os.path.join(self.bfaserver.mypath, 'logs', 'ldap_%s.log' % datetime.datetime.now().strftime('%Y%m%d%H%M%S%f')),'w')

                            username = uname
                            userpass = con.bitdecrypt(userpass)
                            ldapfilter = con.ldapfilter if con.ldapfilter is not None else username
                            self.log.debug("ldapfilter cn=%s" % ldapfilter)

                            for domain in con.domains:
                                if not domain:
                                    try:
                                        if domain in con.use_ldap_ssl and con.use_ldap_ssl[domain]:
                                            objRootDSE = win32com.client.GetObject('LDAPS://RootDSE')
                                        else:
                                            objRootDSE = win32com.client.GetObject('LDAP://RootDSE')
                                        objRootDSE.GetInfo()
                                        objRootDSE.Get('defaultNamingContext')
                                        base_dn = objRootDSE.Get('defaultNamingContext')
                                        domain = ".".join(x[3:] for x in base_dn.lower().split(','))
                                    except:
                                        self.log.exception("Fehler bei Domainabfrage über LDAP")
                                        if 'USERDNSDOMAIN' in os.environ:
                                            domain = os.environ['USERDNSDOMAIN'].lower()
                                            base_dn = ",".join(['DC=%s' % x for x in domain.split('.')])
                                        else:
                                            self.log.error("Konnte Systemvariable USERDNSDOMAIN nicht auflösen")
                                else:
                                    base_dn = ",".join(['DC=%s' % x for x in domain.split('.')])

                                if domain:      # wurde in der Con keine Domain eingetragen, dann steht der Schalter für SLL bzw TLS in con.use_ldap_ssl[None] / con.use_ldap_tls[None]
                                    if domain not in con.use_ldap_ssl:
                                        con.use_ldap_ssl[domain] = con.use_ldap_ssl[None]
                                    if domain not in con.use_ldap_tls:
                                        con.use_ldap_tls[domain] = con.use_ldap_tls[None]

                                    domaincontroller = con.domaincontrollers[domain] if domain in con.domaincontrollers and con.domaincontrollers[domain] else domain
                                    if domain in con.use_ldap_ssl and con.use_ldap_ssl[domain]:
                                        self.log.debug("Starte LDAPS Abfrage an Domain %s Controler %s" % (domain, domaincontroller))
                                    else:
                                        self.log.debug("Starte LDAP Abfrage an Domain %s Controler %s" % (domain, domaincontroller))
                                    if domain in con.use_ldap_tls and con.use_ldap_tls[domain]:
                                        self.log.debug("Verwende LDAP mit TLS")
                                    conn = None

                                    if con.use_ldap_ssl[domain]:
                                        try:
                                            ldap.set_option(ldap.OPT_X_TLS_REQUIRE_CERT, ldap.OPT_X_TLS_NEVER)
                                            if self.bfaserver.ldapdebug or con.ldap_debug:
                                                conn = ldap.initialize('LDAPS://%s:%d' % (domaincontroller, con.ldap_ssl_port), trace_level=9, trace_file=flog)
                                            else:
                                                conn = ldap.initialize('LDAPS://%s:%d' % (domaincontroller, con.ldap_ssl_port))
                                            conn.protocol_version = 3
                                            conn.set_option(ldap.OPT_REFERRALS, 0)
                                            conn.set_option(ldap.OPT_TIMEOUT, 10)
                                            conn.set_option(ldap.OPT_NETWORK_TIMEOUT, 10)
                                            conn.set_option(ldap.OPT_PROTOCOL_VERSION, 3)
                                            if con.ldap_ssl_cert:
                                                if os.path.exists(con.ldap_ssl_cert):
                                                    conn.set_option(ldap.OPT_X_TLS_CACERTFILE, con.ldap_ssl_cert)
                                                else:
                                                    self.log.error("Konnte SSL-Zertifikat nicht finden %s" % con.ldap_ssl_cert)
                                            conn.set_option(ldap.OPT_X_TLS, ldap.OPT_X_TLS_DEMAND)
                                            conn.set_option(ldap.OPT_X_TLS_DEMAND, True)
                                            # conn.set_option(ldap.OPT_DEBUG_LEVEL, 255)
                                        except:
                                            self.log.exception("Fehler bei LDAPS.Initialize")
                                            conn = None
                                            auth = False
                                    else:
                                        try:
                                            if con.use_ldap_tls[domain]:
                                                ldap.set_option(ldap.OPT_X_TLS_REQUIRE_CERT, ldap.OPT_X_TLS_ALLOW)
                                            if self.bfaserver.ldapdebug or con.ldap_debug:
                                                conn = ldap.initialize('LDAP://%s' % domaincontroller, trace_level=9, trace_file=flog)
                                            else:
                                                conn = ldap.initialize('LDAP://%s' % domaincontroller)
                                                # conn = ldap.initialize('ldaps://%s' % domain)
                                            conn.protocol_version = 3
                                            conn.set_option(ldap.OPT_REFERRALS, 0)
                                            conn.set_option(ldap.OPT_TIMEOUT, 10)
                                            conn.set_option(ldap.OPT_NETWORK_TIMEOUT, 10)
                                            conn.set_option(ldap.OPT_PROTOCOL_VERSION, 3)
                                            if con.use_ldap_tls[domain]:
                                                # conn.set_option(ldap.OPT_X_TLS_REQUIRE_CERT, ldap.OPT_X_TLS_DEMAND)
                                                # conn.set_option(ldap.OPT_X_TLS_NEWCTX, 0)
                                                try:
                                                    conn.start_tls_s()
                                                except:
                                                    self.log.exception(u"Fehler bei START_TLS")
                                                    conn = None
                                                    auth = False
                                            else:
                                                conn.set_option(ldap.OPT_X_TLS, ldap.OPT_X_TLS_NEVER)
                                                conn.set_option(ldap.OPT_X_TLS_REQUIRE_CERT, ldap.OPT_X_TLS_NEVER)
                                                conn.set_option(ldap.OPT_X_TLS_DEMAND, False)
                                        except:
                                            self.log.exception("Fehler bei LDAP.Initialize")
                                            conn = None
                                            auth = False

                                    if conn is None:
                                        auth = False
                                        self.log.error("%s Es konnte keine Verbindung zum Domain-Controller aufgebaut werden. %s %s <%s>" % (self.name, domain, username, hex(id(sock))))
                                        xml = bitfarm_xml.create_xml_request(reqtype="respAuthError", data=_("Es konnte keine Verbindung zum Domain-Controller aufgebaut werden.\n%s") % domain, datatype="text")
                                        break
                                    else:
                                        loop = 0
                                        while loop < 5 and not succeed:
                                            try:
                                                conn.simple_bind_s('%s@%s' % (username, domain), userpass)
                                                conn.search_s(base_dn, ldap.SCOPE_SUBTREE, 'cn=%s' % ldapfilter)
                                            except ldap.INVALID_CREDENTIALS:
                                                self.log.error("%s Passwort ist falsch, Versuch %d" % (self.name, loop + 1))
                                                auth = False
                                                xml = bitfarm_xml.create_xml_request(reqtype="respAuthError", data=_("Das Passwort ist falsch."), datatype="text")
                                                break
                                            except:
                                                self.log.error("%s Fehler bei simple_bind_s, Versuch %d" % (self.name, loop + 1))
                                                self.log.error("%s" % str(sys.exc_info()))
                                                auth = False
                                                time.sleep(.5)
                                            else:
                                                self.log.info("%s Anmeldung erfolgreich, Versuch %d" % (self.name, loop + 1))
                                                auth = True
                                                succeed = True

                                            loop += 1

                                        try:
                                            conn.unbind_s()
                                        except:
                                            self.log.exception(u"Fehler bei unbind_s() Versuch %d" % (loop + 1))

                                        if not succeed and loop == 5:
                                            self.log.error("Timeout: 5 Versuche bei der Anmeldung eines Users an den LDAP-Server erfolglos!")
                                            self.log.error("base_dn: %s" % base_dn)
                                            auth = False
                                            xml = bitfarm_xml.create_xml_request(reqtype="respAuthError", data=_("Der Benutzer konnte nicht authentifiziert werden (LDAP)."), datatype="text")
                                        elif succeed and auth:  # bei erfolgreicher Anmeldung können wir die Schleife direkt verlassen
                                            self.log.debug("Abfrage an Domain erfolgreich %s" % domain)
                                            break
                                else:
                                    auth = False
                                    self.log.error("%s Die Domain konnte nicht ermittelt werden. %s <%s>" % (self.name, username, hex(id(sock))))
                                    xml = bitfarm_xml.create_xml_request(reqtype="respAuthError", data=_("Die Domain konnte nicht ermittelt werden."), datatype="text")
                                    break

                            if self.bfaserver.ldapdebug or con.ldap_debug:
                                flog.close()

                    else:   # Authentifizierung gegen die sys_user Tabelle
                        if (userpass == upass) or (userpass is None and upass == ''):  # erfolgreich
                            auth = True
                        else:
                            auth = False
                            self.log.error("%s Das Passwort ist falsch. %s <%s>" % (self.name, username, hex(id(sock))))
                            xml = bitfarm_xml.create_xml_request(reqtype="respAuthError", data=_("Das Passwort ist falsch."), datatype="text")
                elif ambiguous:
                    self.log.error("%s Der Anzeigename ist nicht eindeutig. Bitte melden Sie sich über den eindeutigen Benutzernamen an. %s <%s>" % (self.name, username, hex(id(sock))))
                    xml = bitfarm_xml.create_xml_request(reqtype="respAuthError", data=_("Der Anzeigename ist nicht eindeutig. Bitte melden Sie sich über den eindeutigen Benutzernamen an."), datatype="text")
                else:
                    if not xml:
                        self.log.error("%s Der Benutzer ist nicht vorhanden. %s <%s>" % (self.name, username, hex(id(sock))))
                        xml = bitfarm_xml.create_xml_request(reqtype="respAuthError", data=_("Der Benutzer ist nicht vorhanden."), datatype="text")
            else:
                self.log.error("%s Der Benutzer ist nicht vorhanden. %s <%s>" % (self.name, username, hex(id(sock))))
                xml = bitfarm_xml.create_xml_request(reqtype="respAuthError", data=_("Der Benutzer ist nicht vorhanden."), datatype="text")

            if auth:    # jetzt prüfen wir die Anzahl der named oder concurrent User
                if concurrent:
                    if not admin and self.bfaserver.cnt_concurrent[con.profile] >= con.concurrent > -1:   # Anzahl Anmeldung größer als registriert größer als -1, (registriert = -1 heisst unlimited)
                        auth = False
                        xml = bitfarm_xml.create_xml_request(reqtype="respAuthError",
                                                data=_("Die Gesamtzahl der gleichzeitigen Benutzer wurde überschritten."), datatype="text")
                        self.log.warning(_("Die Gesamtzahl der gleichzeitigen Benutzer wurde überschritten."))
                else:
                    if not admin and self.bfaserver.cnt_named[con.profile] >= con.named > -1:
                        found_user = False
                        for key in self.bfaserver.authuser[profile]:    # userid suchen
                            if key[0] == userkey[0]:
                                found_user = True
                                break

                        if not found_user:                              # ist der user schon angemeldet, darf sich dieser beliebig oft nochmal anmelden
                            auth = False
                            xml = bitfarm_xml.create_xml_request(reqtype="respAuthError",
                                                    data=_("Die Gesamtzahl der angemeldeten Benutzer wurde überschritten."), datatype="text")
                            self.log.warning(_("Die Gesamtzahl der angemeldeten Benutzer wurde überschritten."))

            if auth:
                if self.bfaserver.waitingperiod[profile] is not None:
                    period = self.bfaserver.waitingperiod[profile].date() + datetime.timedelta(days=7) - datetime.datetime.now().date()
                    msg = _("Achtung! Der Server besitzt keine gültige Registrierungsdatei. Anmeldungen sind nur noch %d Tage möglich.\nWenden Sie sich an ihren Administrator bzw. den bitfarm Support.") % period.days
                    if period.days < 1 and not admin:
                        con.named = 1
                        con.concurrent = 0
                        msg = _("Achtung! Der Server besitzt keine gültige Registrierungsdatei und die Wartezeit ist abgelaufen.\nEs ist momentan nur eine Anmeldung möglich.\nWenden Sie sich an ihren Administrator bzw. den bitfarm Support.")

                obj = con.read_user(db=self.db[profile], userid=userkey[0], logid=self.name)
                BitfarmLoginThread.authlock.acquire()
                try:
                    found_user = False
                    found_userkey = False
                    for key in self.bfaserver.authuser[profile]:    # userid suchen
                        if key[0] == userkey[0]:
                            found_user = True
                            break
                    for key in self.bfaserver.authuser[profile]:    # userkey (userid,ip) suchen
                        if key == userkey:
                            found_userkey = True
                            break

                    if not found_user and not admin:       # wir zählen nur neu angemeldete User
                        if concurrent:
                            self.bfaserver.cnt_concurrent[profile] += 1
                        else:
                            self.bfaserver.cnt_named[profile] += 1

                    if found_userkey:   # wir haben schon diesen user angemeldet? dann anmeldung eines zweiten clients
                        self.log.debug(_('Neuanmeldung von %s') % str(userkey))
                        if sock not in self.bfaserver.authuser[profile][userkey]:
                            self.bfaserver.authuser[profile][userkey].append(sock)
                            self.log.debug(_('%s Verbindung hinzugefügt %s %s') % (self.name, str(userkey), str(self.bfaserver.authuser[profile][userkey])))
                    else:
                        # SessionID erst ab v3.6.2
                        sessionid = uuid.uuid4().hex
                        userkey = (userkey[0], sessionid)
                        self.bfaserver.authuser[con.profile][userkey] = [sock]

                    if obj:
                        self.bfaserver.user[con.profile][userkey[0]] = obj
                    else:
                        self.log.warning(_('Achtung, es konnte kein Userobjekt für ID %s hinterlegt werden') % userkey[0])

                    sock_ip = sock.getpeername()[0]
                    self.log.debug(_('Verbindung authentifiziert %s %s <%s>') % (str(userkey[0]), str(sock_ip), hex(id(sock))))
                    if self.bfaserver.develop:
                        self.log.info(str(userkey))
                    if con.auto_reconnect:      # wir hängen das Verhalten mit temporären Sockets an den auto_reconnect Schalter
                        self.bfaserver.active_user[profile][userkey] = datetime.datetime.now()
                    elif sock_ip != '127.0.0.1' and sock_ip != self.bfaserver.server_ip:            # nur bei Anmeldungen die nicht direkt über die eigene Maschine kommen, nehmen wir den Benutzer in active_user auf
                        self.bfaserver.active_user[profile][userkey] = datetime.datetime.now()      # so verhindern wir, dass User die über den bfaHttpServer kommen in einen Timeout laden
                finally:                                                                            # auf Dauer braucht der bfaHttpServer auch einen reqHeartBeat pro Benutzersession
                    BitfarmLoginThread.authlock.release()
                # SessionID erst ab v3.6.2
                xml = bitfarm_xml.create_xml_request(reqtype="respAuth", userid=userkey[0], data=msg, sessionid = con.bitencrypt(userkey[1]))
                self.log.info(_('Angemeldete Verbindungen: %i') % (len(self.bfaserver.authuser[profile])))
                self.log.debug(_("%s Profil: %s Anzahl named Lizenzen:%d/%d") % (self.name, profile, self.bfaserver.cnt_named[profile], con.named))
                if con.concurrent > 0:
                    self.log.debug(_("%s Profil: %s Anzahl concurrent Lizenzen:%d/%d") % (self.name, profile, self.bfaserver.cnt_concurrent[profile], con.concurrent))

                self.read_userrights(userkey[0], profile)

            try:
                sock.send(xml)
            except:
                self.log.exception(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
        except:
            self.log.exception(_("Allgemeiner Fehler bei der Anmeldung."))
            xml = bitfarm_xml.create_xml_request(reqtype="respAuthError",
                                                 data=_("Allgemeiner Fehler bei der Anmeldung."))
            try:
                sock.send(xml)
            except:
                self.log.exception(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)

    def clear_authlist(self, sock):
        self.log.debug(_('%s clear_authlist <%s>') % (self.name, hex(id(sock))))
        clearuser = True
        clearsock = None
        found = False

        for profile in self.bfaserver.authuser:
            userkey = None
            for userkey in self.bfaserver.authuser[profile]:
                if sock in self.bfaserver.authuser[profile][userkey]:
                    BitfarmLoginThread.authlock.acquire()
                    try:
                        self.bfaserver.authuser[profile][userkey].remove(sock)
                    except:
                        self.log.exception(_('%s Fehler beim entfernen des Sockets %s <%s>') % (self.name, str(userkey), hex(id(sock))))
                    finally:
                        BitfarmLoginThread.authlock.release()
                        found = True
                        break

            if userkey and found:
                self.remove_userkey(profile, userkey)

    def remove_userkey(self, profile, userkey):
        self.log.debug(_('%s remove_userkey %s') % (self.name, str(userkey[0])))
        if self.bfaserver.develop:
            self.log.info(str(userkey))
        clearuser = True
        con = self.bfaserver.con.connections[profile]

        # if userkey in self.bfaserver.authuser[profile] and len(self.bfaserver.authuser[profile][userkey]) == 0:     # keine Socketverbindung mehr für diesen User
        if userkey in self.bfaserver.authuser[profile]:
            if con.auto_reconnect and userkey in self.bfaserver.active_user[profile]:
                if self.bfaserver.active_user[profile][userkey] is not None and self.bfaserver.active_user[profile][userkey] + datetime.timedelta(seconds=con.user_timeout) > datetime.datetime.now(): # noch kein Timeout
                    self.log.debug('%s remove_userkey User %i noch innerhalb des Timeouts' % (self.name, userkey[0]))
                    return None     # remove_userkey verlassen
            else:   # auto_reconnct ist nicht aktiv oder wir führen keinen timestamp für diesen user (zb user über bfahttpserver, oder user anmeldung auf dem bitfarm-server)
                if len(self.bfaserver.authuser[profile][userkey]) > 0:  # wir haben noch Verbindungen für diesen User, dann nicht entfernen
                    self.log.debug('%s remove_userkey noch Verbindungen für User %i vorhanden' % (self.name, userkey[0]))
                    return None     # remove_userkey verlassen

            BitfarmLoginThread.authlock.acquire()
            try:
                del (self.bfaserver.authuser[profile][userkey])
                self.log.debug(_('%s Verbindung aus Authentifizierungsliste entfernt %s') % (self.name, str(userkey[0])))
            except:
                self.log.exception(_('%s Verbindung konnte nicht entfernt werde %s') % (self.name, str(userkey[0])))
            finally:
                BitfarmLoginThread.authlock.release()
            for key in self.bfaserver.authuser[profile]:
                if key[0] == userkey[0]:    # der user ist noch von einem anderen client aus angemeldet
                    clearuser = False
                    break

            if clearuser:
                BitfarmLoginThread.authlock.acquire()
                try:
                    if userkey[0] in self.bfaserver.userrights[profile]:
                        del (self.bfaserver.userrights[profile][userkey[0]])
                    if userkey[0] in self.bfaserver.user[profile]:
                        if 'concurrent' in self.bfaserver.user[profile][userkey[0]].flags:
                            if self.bfaserver.cnt_concurrent[profile] > 0:
                                self.bfaserver.cnt_concurrent[profile] -= 1
                        else:
                            if self.bfaserver.cnt_named[profile] > 0:
                                self.bfaserver.cnt_named[profile] -= 1
                        del (self.bfaserver.user[profile][userkey[0]])
                    if userkey[0] in list(self.bfaserver.in_progress[profile].values()):
                        for gdocid, userid in list(self.bfaserver.in_progress[profile].items()):
                            if userkey[0] == userid:  # gefunden
                                del self.bfaserver.in_progress[profile][gdocid]
                    # if userkey in self.bfaserver.active_user[profile]:
                    #     del self.bfaserver.active_user[profile][userkey]
                    for key in list(self.bfaserver.active_user[profile].keys()):
                        if userkey[0] == key[0]:
                            del self.bfaserver.active_user[profile][key]
                finally:
                    BitfarmLoginThread.authlock.release()
                self.log.debug(_('%s User %i aus Berechtigungsliste entfernt') % (self.name, userkey[0]))

                self.log.info(_('%s Angemeldete Verbindungen: %i') % (self.name, len(self.bfaserver.authuser[profile])))

    def replace_sock(self, userkey, sock, profile):
        self.log.debug(_('%s replace_sock %s <%s>') % (self.name, str(userkey), hex(id(sock))))
        BitfarmLoginThread.authlock.acquire()
        try:
            if sock in self.bfaserver.authuser[profile][userkey]:
                self.bfaserver.authuser[profile][userkey].remove(sock)
        finally:
            BitfarmLoginThread.authlock.release()
        try:
            sock.close()
        except:
            pass

    def clear_state(self, profile):
        self.log.debug("%s clear_state" % self.name)

        sql = 'update sys_doc set docstate_0 = docstate_0 & ~1 where docstate_0 > 0 and docstate_0 & 1 = 1'
        try:
            self.db[profile].execute(sql)
        except:
            self.log.exception(_('%s Fehler beim Zurücksetzen des Status 1') % self.name)
        else:
            self.log.debug('%s clear_state zurück gesetzt' % self.name)

            sql = 'select distinct(gdocid) from sys_timer where status & 4 = 0'
            try:
                result = self.db[profile].execute(sql).fetchall()
            except:
                self.log.exception(_('%s Fehler beim Zurücksetzen des Status 1') % self.name)
            else:
                if result:
                    gdocids = ",".join([str(x[0]) for x in result])
                    # gdocids = result[0] if result[0][-1] != ',' else result[0][:-1]
                    sql = 'update sys_doc set docstate_0 = docstate_0 | 1 where gdocid in (%s)' % gdocids
                    try:
                        self.db[profile].execute(sql)
                    except:
                        self.log.exception(_('%s Fehler beim Aktualisieren des Status 1') % self.name)
                    else:
                        self.log.debug(_("%s Status 1 aktualisiert.") % self.name)

    def run(self):
        pythoncom.CoInitialize()
        while not self.stopsignal:
            #        while self.is_alive():
            #            self.log.debug('running %s' %self.name)
            self._job = None
            try:
                try:
                    cmd, self._job = self.queue.get(True, 1)
                except queue.Empty:
                    #            self.log.debug(u"Nichts vom MP übergeben, dann weiter")
                    if self.stopsignal:  # wir haben alles abgearbeitet und sollen aufhören?
                        self.log.info(_("%s Stopsignal erhalten, Queue leer, beende Thread" % self.name))
                        break
                except:
                    self.log.exception(_('Fehler  bei get_next_job'))

                if self._job:  # login?
                    if cmd == 'login':
                        self.login(*self._job)
                    elif cmd == 'replace_sock':
                        self.replace_sock(*self._job)
                    elif cmd == 'clear_authlist':
                        self.clear_authlist(self._job)
                    elif cmd == 'remove_userkey':
                        self.remove_userkey(*self._job)
                    # elif cmd == 'clear_state':
                    #     self.clear_state(self._job)
            except:
                self.log.exception(_("Fehler in BitfarmLoginThread %s") % self.name)

        self.log.debug(_('%s stopped') % self.name)

    def shutdown(self):
        self.stopsignal = True


class BitfarmWorkerThread(threading.Thread):
    """Diese Klasse stellt alle Funktionen zur Verfügung die der Server verarbeiten kann.
    Die entsprechenden Requests (bfaReq Objekte) werden vom bfaServer in die Queue des WorkerThreads gestellt."""
    queue = queue.Queue(0)
    log = logging.getLogger("Main.BitfarmWorkerThread")
    cachelock = threading.Lock()
    in_progresslock = threading.Lock()
    auditprooflock = threading.Lock()
    annolock = threading.Lock()
    gridfieldlock = threading.Lock()
    refreshlock = threading.Lock()
    altertablelock = threading.Lock()
    classifierlock = threading.Lock()
    random.seed()

    def __init__(self, bfaserver, classifier=None):
        threading.Thread.__init__(self)
        self.classifier = classifier
        self.stopsignal = False
        self.bfaserver = bfaserver
        self.running = False
        self.starting = True
        self.bfaocr = BfaTessOCR(logid=self.name + "-ocr", logger=self.log)
        # self.bfaocr.log = self.log
        #self.classifier = bfaClassifier.BfaClassifier()
        if self.bfaserver.useremoterenderer is False:
            self.log.info(_("%s verwende lokalen Renderer") % self.name)
            configfile = os.path.join(self.bfaserver.mypath, "renderer.cfg")
            try:
                self.renderer = Renderer.Renderer(configfile)
            except:
                self.log.critical(_("%s Renderer konnte nicht gestartet werden. Server kann keine Voransichten rendern!") % self.name)
                self.log.critical(_("%s Configfile %s vorhanden ?") % (self.name, configfile))
        self.timeout = 0
        self._job = None
        #        self.bfaProc = bitfarm_doc.BitfarmDocProcessing(self.bfaserver.con)
        #        self.bfaWflow = bitfarm_workflow.BitfarmWorkflowProcessing(self.bfaserver.con)
        self.log.debug(_('%s gestartet...') % self.name)
        self.db = {}
        self.search_db = {}
        self.search_engine = {}

        for _profile in self.bfaserver.con.connections:
            _con = self.bfaserver.con[_profile]
            if _con.engine is not None:
                self.db[_profile] = _con.engine.connect().execution_options(autocommit=True)
                if not isinstance(self.db[_profile], sqla.engine.base.Connection):
                    raise DatabaseConError(_con.confile)
                else:
                    self.log.debug("%s db = %s" % (self.name, str(self.db[_profile])))

            connectionstr = _con._makeconstr(_con.suuser, _con.supassword)
            conargs = _con._makeconargs()

            self.search_engine[_profile] = sqla.create_engine(connectionstr, pool_pre_ping=True, pool_recycle=3600, pool_size=10,
                                                              max_overflow=10, connect_args=conargs, execution_options={"autocommit": True})
            try:
                self.search_db[_profile] = self.search_engine[_profile].connect()
            except:
                self.log.exception('Fehler bei Verbindung mit der Datenbank')
                raise DatabaseNotConnectedError(_profile)

        self.bfaProcessing = bitfarm_proc.BitfarmDocProcessing(self.bfaserver.con, _con_db=self.db, name=self.name,
                                                               _gridfieldlock=BitfarmWorkerThread.gridfieldlock, _gridfunc=self.bfaserver.gridfunc)
        self.bfaWorkflow = bitfarm_proc.BitfarmWorkflowProcessing(self.bfaserver.con, _con_db=self.db, name=self.name)
        self.SearchProc = bitfarm_proc.BitfarmSearchProcessing(self.bfaserver.con, _con_db=self.search_db, name=self.name)

    def reconnect(self, _profile):
        self.log.debug("%s reconnect" % self.name)
        _ret = False

        if self.bfaserver.con[_profile].engine is not None:
            try:
                self.db[_profile] = self.bfaserver.con[_profile].engine.connect().execution_options(autocommit=True)
            except:
                self.log.error(_("%s Reconnect gescheitert") % self.name)
            else:
                if not isinstance(self.db[_profile], sqla.engine.base.Connection):
                    self.log.error(_("%s Reconnect gescheitert") % self.name)
                else:
                    self.log.warning(_("%s Reconnect erfolgreich db = %s") % (self.name, self.db[_profile]))
                    _ret = True

                    if _profile == self.bfaserver.con.main.profile:     # die main connection auch neu zuweisen
                        self.bfaProcessing.db['main'] = self.db[_profile]
                        self.bfaWorkflow.db['main'] = self.db[_profile]
                        self.SearchProc.db['main'] = self.db[_profile]
        return _ret

    def run(self):
        self.running = True
        #        while not self.stopsignal:
        while self.is_alive():
            try:
                #                self.log.debug('running %s' %self.name)
                self._job = None
                self.timeout = 0
                try:
                    self._job = self.queue.get(True, 1)
                except queue.Empty:
                    #            self.log.debug(u"Nichts vom MP übergeben, dann weiter")
                    if self.stopsignal:  # wir haben alles abgearbeitet und sollen aufhören?
                        self.log.info(_("%s Stopsignal erhalten, Queue leer, beende Thread") % self.name)
                        break

                except:
                    self.log.exception(_('%s Fehler bei get_next_job') % self.name)

                if self._job:
                    self.log.debug(_('%s Job entgegen genommen <%s>') % (self.name, hex(id(self._job.sock))))
                    #                    self.timeout = time.time() + 2

                    if self.db[self._job.profile].invalidated:      # Datenbankverbindung weg
                        if self.bfaserver.con[self._job.profile].engine is not None:
                            if not self.reconnect(self._job.profile):
                                self.log.exception(_('%s Reconnect zu Datenbank fehlgeschlagen.') % self.name)
                                self._job.error = EC_NORECONNECT
                        else:
                            self.log.exception(_('%s Datenbank-Engine steht nicht zur Verfügung.') % self.name)
                            self._job.error = EC_NODATABASEENGINE

                        if self._job.error:
                            self.result = False
                            self._job.errortext = _("%s Keine Verbindung zu Datenbank, Funktion wird abgebrochen.") % self.name
                            self.log.error(self._job.errortext)

                    if not self._job.error:
                        if len(self._job.tasks) > 0:
                            if hasattr(self.bfaProcessing, self._job.tasks[0].task):
                                self.bfaProcessing._job = self._job
                                getattr(self.bfaProcessing, self._job.tasks[0].task)()
                            # elif hasattr(self.bfaWorkflow, self._job.tasks[0].task):   #TODO: hier wird jetzt direkt bitfarm_workflow.BitfarmWorkflowProcessing verbunden
                            #     self.bfaWorkflow._job = self._job
                            #     getattr(self.bfaWorkflow, self._job.tasks[0].task)()
                            elif hasattr(self, self._job.tasks[0].task):
                                getattr(self, self._job.tasks[0].task)()
                            else:
                                self.result = False
                                self._job.error = EC_FUNCTIONNOTFOUND
                                self._job.errortext = _("%s Die Funktion %s konnte nicht gefunden werden") % (self.name, self._job.tasks[0].task)
                                self.log.error(self._job.errortext)
                        else:
                            if self._job.reqtype and hasattr(self, self._job.reqtype):
                                getattr(self, self._job.reqtype)()
                            else:
                                self.result = False
                                self._job.error = EC_FUNCTIONNOTFOUND
                                self._job.errortext = _("%s Die Funktion %s konnte nicht gefunden werden") % (self.name, self._job.reqtype)
                                self.log.error(self._job.errortext)

            except:
                self.log.exception(_("Fehler in WorkerThread %s") % self.name)
                self._job.error = EC_WORKERTHREADERROR

            if self._job is not None:
                if self._job.error and self._job.sock is not None:
                    xml = None
                    if self._job.parameter:
                        if type(self._job.parameter) is str:
                            self._job.parameter = self._job.parameter.split(";")
                        if type(self._job.parameter) is list:
                            if "asjson" in self._job.parameter:
                                xml = bitfarm_xml.create_xml_request(reqtype="respError", data=self._job.errortext,
                                                                     parameter="json")
                    if xml is None:
                        self.log.error("%s Allgemeiner Serverfehler im Workerthread" % self.name)
                        xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=self._job.gdocid,
                                                             data=_("Allgemeiner Serverfehler"))
                    try:
                        self._job.sock.send(xml)
                    except:
                        self.log.warning(_("%s Client scheint weg zu sein, Empfangen wird abgebrochen") % self.name)

                if self._job.need_reconnect:
                    self.reconnect(self._job.profile)

                self.bfaserver.resultqueue.put(self._job)

        self.log.debug(_('%s stopped') % self.name)
        self.running = False

    def shutdown(self):
        self.stopsignal = True

    def _timeoutprocess(self, args, timeout=120):
        proc = subprocess.Popen(
            args, stdout=subprocess.PIPE, stderr=subprocess.PIPE, stdin=subprocess.PIPE)
        cnt = 0
        is_timeout = False
        # timeout wird in Sekunden angegeben, wir prüfen aber alle 0,1 sek
        timeout = 10 * timeout
        stdoutdata = b''
        stderrdata = b''
        while proc.returncode is None:
            time.sleep(0.1)
            cnt += 1
            if cnt > timeout:
                is_timeout = True
                break
            data = proc.communicate()
            stdoutdata += data[0]
            stderrdata += data[1]
            # self.log.debug("_timeoutprocess proc %s" %str(txt))
            proc.poll()
        if is_timeout:
            self.log.error(_("%s process timeout (%s)") % (self.name, " ".join(args)))
            proc.terminate()
            rc = None
            return (True, rc, (stdoutdata.decode('cp1252'), stderrdata.decode('cp1252')))
        else:
            rc = proc.returncode
            return (False, rc, (stdoutdata.decode('cp1252'), stderrdata.decode('cp1252')))

    def countpages(self, tifpath):
        self.log.debug(_("%s countpages called with path %s:") % (self.name, tifpath))
        if thtiflibimported:
            try:
                s = time.time()
                tif = thtiflib.Tiff(tifpath, withmeta=False)
                pagecount = tif.pagecount()
                self.log.debug(
                    _("%s thtiflib lade tif + Seitenanzahl ermitteln (%d Seiten) - %.09f Sek.") % (self.name, pagecount, time.time() - s))
                return pagecount
            except:
                self.log.warning(_("%s countpages: thtiflib konnte %s nicht parsen. Versuche nun countpages.exe") % (self.name, tifpath))
                pass
        self.log.debug(_("%s countpages: thtiflib nicht verfügbar, versuche nun countpages.exe") % (self.name))
        cpexe = os.path.join(self.bfaserver.bfapath, "countpages.exe")
        if not os.path.isfile(cpexe):
            self.log.warning(_("%s countpages.exe nicht gefunden") % (self.name))
            return None
        if not os.path.isfile(tifpath):
            self.log.warning(_("%s countpages: %s nicht gefunden") % (self.name, tifpath))
            return None
        args = [cpexe, tifpath]
        self.log.debug(_("%s countpages.exe Argumente: %s") % (self.name, args))
        timeout, rc, output = self._timeoutprocess(args)
        self.log.debug(_("%s countpages.exe Ergebnis: timeout: %s, returncode: %s, std-data: %s") % (self.name, str(timeout), str(rc), output))
        if timeout is False and rc == 0:
            pattern = "Anzahl Seiten:(\d*)"
            result = re.findall(pattern, output[0])
            if len(result) > 0:
                try:
                    pagecount = int(result[0])
                    self.log.debug(_("%s countpages.exe Ergebnis: %d Seiten") % (self.name,  pagecount))
                    return pagecount
                except:
                    self.log.error(_("%s Fehler beim Parsen der Ausgabe von countpages.exe") %(self.name))
                    return None
        return None

    def create_nrs_filepath(self, docloc):
        if docloc == '':
            return None
        filename = os.path.basename(docloc).lower()
        basepath = os.path.dirname(docloc).lower()
        if basepath[:2] == r'\\':
            parts = basepath.split(os.path.sep)
            parts[3] = self.bfaserver.con.connections[self._job.profile].archive_nrs.lower()
        # basepath = basepath.replace(self.bfaserver.con.connections[self._job.profile].archive_rs.lower(),self.bfaserver.con.connections[self._job.profile].archive_nrs.lower())
        else:
            parts = basepath.split(os.path.sep)
            parts[1] = self.bfaserver.con.connections[self._job.profile].archive_nrs.lower()
        basepath = os.path.sep.join(parts)
        #if not os.path.exists(basepath):
        #    try:
        #        os.makedirs(basepath)
        #    except:
        #        self.log.exception(u'Konnte NRS-Pfad nicht anlegen %s' % basepath)
        #        return None
        return os.path.join(basepath, filename)

    def get_filehash(self, filepath):
        if filepath is None:
            return None
        if os.path.exists(os.path.expandvars(filepath)):
            sha = hashlib.sha1()
            with open(os.path.expandvars(filepath), 'rb') as f:
                while True:
                    block = f.read(1024)  # Magic number: one-megabyte blocks.
                    if not block:
                        break
                    sha.update(block)
                return sha.hexdigest()
        else:
            return None

    def get_annotations(self, filepath):
        self.log.debug(_("%s get_annotations filepath: %s") % (self.name, filepath))
        filepath = self.create_nrs_filepath(filepath)
        _ret = []
        if not filepath:
            return _ret
        filename = os.path.basename(os.path.expandvars(filepath)).lower()
        basepath = os.path.dirname(os.path.expandvars(filepath)).lower()
        basename = os.path.splitext(filename)[0]

        ievlist = glob.iglob(os.path.join(basepath, basename) + "*.iev")
        for iev in ievlist:
            self.log.debug(_("%s get_annotations iev gefunden: %s") % (self.name, iev))
            iev = iev.lower()
            anno = bitfarm_doc.BitfarmAnnotation()
            anno.filepath = iev
            index = os.path.basename(iev).replace(basename + "_", "").replace(".iev", "")
            annoarr = index.split("_")
            try:
                anno.page = int(annoarr[0])
                anno.layer = int(annoarr[1])
            except:
                self.log.warning(_("%s Kann Annotation Dateiname %s nicht parsen") % (self.name, iev))
                continue
            self.log.debug(_("%s get_annotations layer %d, Seite %d") % (self.name, anno.layer, anno.page))
            if os.path.exists(iev):
                anno.iev = bitfarm_doc.BitfarmStream()
                with open(iev, 'rb') as f:
                    anno.iev.stream.write(f.read())
            _ret.append(anno)
        return _ret

    def load_annotations(self, doc):
        self.log.debug(_("%s load_annotations [%i]") % (self.name, doc.gdocid))

        return self.bfaProcessing.get_annotations(doc.gdocid, self._job.profile, logid=self.name)

    def check_logread(self, _rights = None):
        self.log.debug(_('%s check_logread') % self.name)

        if _rights is None:
            _rights = self.check_docrights(self._job.gdocid)
        if _rights.logread:
            sql = 'select count(*), h.userid, h.taskid, u.username from sys_user as u ' \
                  'left join  sys_history as h  on h.userid = u.userid ' \
                  'where gdocid = %s and h.userid = %s and h.taskid = 220 and day(tstamp) = day(now())' % (self._job.gdocid, self._job.userid)
            try:
                result = self.db[self._job.profile].execute(sqla.text(sql)).fetchone()
            except:
                self.log.exception(_("%s Fehler bei Abfrage der History.") % (self.name))
            else:
                if result[0] == 0:      # heute noch nicht gelesen? dann history schreiben
                    user = self.bfaserver.con.connections[self._job.profile].read_user(db=self.db[self._job.profile], userid=self._job.userid, logid=self.name)
                    hist = bitfarm_doc.BitfarmHistory()
                    hist.userid = self._job.userid
                    hist.gdocid = self._job.gdocid
                    hist.taskid = 220
                    if user:
                        hist.value1 = user.name
                    else:
                        hist.value1 = 'Userid: %s' % self._job.userid
                    self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

    def reqSqlMapper(self):
        self.log.debug(_('%s reqSqlMapper') % self.name)
        jobs = self._job.data
        if self._job.parameter:
            params = self._job.parameter
            if "section" in params:
                section = params["section"]
        else:
            section = "all"

        tempdir = tempfile.mkdtemp(prefix="bfa_sqlmapper_")
        jobfiles = []
        self.log.debug(_('%s saving jobfiles') % self.name)
        for (filename, content) in jobs:
            basename = os.path.basename(filename)
            newname = os.path.join(tempdir, basename)
            with open(newname, "w") as f:
                f.write(content)
            jobfiles.append((newname, filename))
        bfasqlmapper = os.path.join(os.path.expandvars(
            "%bitfarm-archiv%"), "bfa_sqlmapper_x64.exe")
        configfile = os.path.join(os.path.expandvars(
            "%bitfarm-archiv%"), "bfa_sqlmapper_x64.ini")
        rc = -1
        if os.path.isfile(bfasqlmapper) and os.path.isfile(configfile):
            self.log.debug(
                _('%s found bfa_sqlmapper_x64.exe and bfa_sqlmapper_x64.ini') % self.name)
            cmd = [bfasqlmapper, "-c", "servervp", "-s", section, "--jobdir",
                   tempdir, "--config", configfile]
            self.log.debug(_('%s running command: "%s"' %
                             (self.name, " ".join(cmd))))
            try:
                rc = subprocess.check_call(cmd)
            except Exception as e:
                self.log.error(_('%s error running sqlmapper') % self.name)
                self.log.error(e)
        else:
            self.log.error(
                _('%s bfa_sqlmapper_x64.exe and/or bfa_sqlmapper_x64.ini not found') % self.name)
        if rc != 0:
            xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=0,
                                                 data=_("Fehler beim Ausführen von bfa_sqlmapper"))
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(
                    _("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
        else:
            result = []
            self.log.debug(_('%s reading jobfiles') % self.name)
            for local_jobfile, remote_path in jobfiles:
                with open(local_jobfile, "r") as f:
                    content = f.read()
                result.append((remote_path, content))
            self.log.debug(_('%s returning jobfiles') % self.name)
            xml = bitfarm_xml.create_xml_request(
                reqtype="respSqlMapper", data=result)
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(
                    _("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)

        shutil.rmtree(tempdir)

    def reqAccessable(self):
        self.log.debug(_('%s reqAccessable') % self.name)

        gdocids = []
        if type(self._job.parameter) in [str, str]:
            gdocids = self._job.parameter.split(";")
            gdocids = [tuple(x.split(':')) for x in gdocids]

        _ret = []
        for gdocid, arcid in gdocids:
            doc_right = self.check_docrights(gdocid=int(gdocid), arcid=int(arcid))
            if self._job.userid in self.bfaserver.user[self._job.profile] and doc_right.read:
                _ret.append(gdocid + ':' + arcid)
        param = ";".join([str(x) for x in _ret])

        xml = bitfarm_xml.create_xml_request(reqtype="respAccessable", userid=0, parameter=param)
        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
        return None

    def reqReadArchives(self):
        self.log.debug(_('%s reqReadArchives') % self.name)
        asjson = False
        no_addfields = False
        no_fielditems = False
        rootid = 0
        if self._job.parameter:
            self.log.debug(_('%s reqReadArchives params: %s') % (self.name, self._job.parameter))
            if type(self._job.parameter) is not list:
                self._job.parameter = self._job.parameter.split(";")
            if type(self._job.parameter) == list:
                rootid = self._job.parameter[0]
            asjson = "asjson" in self._job.parameter
            no_addfields = "no_addfields" in self._job.parameter
            no_fielditems = "no_fielditems" in self._job.parameter
        self.log.debug(_("%s asjson: %s no_addfields: %s") % (self.name, asjson, no_addfields))
        archives = self.bfaserver.con.connections[self._job.profile].read_arctree(
            db=self.bfaProcessing.db[self._job.profile], user=self._job.userid, rootid=rootid,
            right=self._job.data, no_addfields=no_addfields, no_fielditems=no_fielditems, logid=self.name)
        resp = bitfarm_doc.bfaReq()
        resp.reqtype = 'respArchives'
        # asjson = False
        if asjson:
            resp.data = []
            for arc in archives:
                arcdata = bitfarm_xml.obj_to_dict(arc)
                resp.data.append(arcdata)
                resp.parameter = "json"
        else:
            resp.data = archives
            resp.parameter = "xml"

        resp.profile = self._job.profile
        xml = bitfarm_xml.object_to_xml(resp)
        try:
            self._job.sock.send(xml.encode())
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqReadAddfields(self):
        # TODO: über welchen User kommt der request, gibt es einen Server-User für den HTTP Dienst?
        self.log.debug(_('%s reqReadAddfields') % self.name)
        self.log.debug(_('%s reqReadAddfields job parameter: %s') % (self.name, self._job.parameter))
        no_fielditems = False
        all_fields = False
        asjson = False
        if self._job.parameter:
            no_fielditems = 'no_fielditems' in self._job.parameter
            all_fields = 'all_fields' in self._job.parameter
            asjson = "asjson" in self._job.parameter

        userid = self._job.userid if not all_fields else None
        addfields = self.bfaserver.con.connections[self._job.profile].read_addfields(
            db=self.bfaProcessing.db[self._job.profile], userid=userid, arcid=self._job.arcid, no_fielditems=no_fielditems, logid=self.name)
        resp = bitfarm_doc.bfaReq()
        resp.reqtype = 'respAddfields'
        # asjson = False
        if asjson:
            start = time.process_time()
            resp.data = bitfarm_xml.obj_to_dict(addfields)
            self.log.debug('process time object_to_dict %s' % str(time.process_time() - start))
            resp.parameter = "json"
        else:
            resp.data = addfields
            resp.parameter = "xml"

        resp.profile = self._job.profile
        # start = time.process_time()
        xml = bitfarm_xml.object_to_xml(resp)
        # self.log.debug('process time object_to_xml %s' % str(time.process_time() - start))
        try:
            self._job.sock.send(xml.encode())
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqReadUsers(self):
        self.log.debug(_('%s reqReadUsers') % self.name)
        users = self.bfaserver.con.connections[self._job.profile].read_users(db=self.db[self._job.profile], logid=self.name)
        resp = bitfarm_doc.bfaReq()
        resp.reqtype = 'respUsers'
        asjson = False
        if self._job.parameter:
            asjson = "asjson" in self._job.parameter
        # asjson = False
        if asjson:
            resp.data = bitfarm_xml.obj_to_dict(users)
            resp.parameter = "json"
        else:
            resp.data = users
            resp.parameter = "xml"

        resp.profile = self._job.profile
        xml = bitfarm_xml.object_to_xml(resp)
        try:
            self._job.sock.send(xml.encode())
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqReadGroups(self):
        self.log.debug(_('%s reqReadGroups') % self.name)
        groups = self.bfaserver.con.connections[self._job.profile].read_groups(db=self.bfaProcessing.db[self._job.profile])
        resp = bitfarm_doc.bfaReq()
        resp.reqtype = 'respGroups'
        asjson = False
        if self._job.parameter:
            asjson = "asjson" in self._job.parameter
        # asjson = False
        if asjson:
            resp.data = bitfarm_xml.obj_to_dict(groups)
            resp.parameter = "json"
        else:
            resp.data = groups
            resp.parameter = "xml"

        resp.profile = self._job.profile
        xml = bitfarm_xml.object_to_xml(resp, shortxml=False)
        try:
            self._job.sock.send(xml.encode())
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqSearch(self):
        self.log.debug(_('%s reqSearch') % self.name)
        params = None
        if not type(self._job.data) is bitfarm_doc.BitfarmSearch:
            return None

        sobj = self._job.data
        sobj.profile = self._job.profile
        if not sobj.user_context:
            sobj.user_context = self._job.userid
        if not sobj.userid:
            sobj.userid = self._job.userid

        result, cols = self.do_search(sobj)

        if self._job.parameter:
            if "asjson" in self._job.parameter:
                if result is None:
                    result = {}
                else:
                    result = bitfarm_xml.obj_to_dict(result)
                params = "json"
            else:
                params = "xml"
        else:
            params = "xml"
        if cols is None:
            cols = ''
        else:
            cols = ",".join(cols)

        in_editmode = ",".join([str(x) for x in list(self.bfaserver.in_progress[self._job.profile].keys())])
        xml = bitfarm_xml.create_xml_request(reqtype='respSearch', data=result, parameter=params, in_editmode=in_editmode, selected_cols=cols, shortxml=False)

        if self._job.parameter and "gzip" in self._job.parameter and "asjson" in self._job.parameter:  # der komplette response wird gzipt
            out = io.BytesIO()
            with gzip.GzipFile(fileobj=out, mode="w") as f:
                f.write(xml)
            xml = out.getvalue()

        if self.bfaserver.develop:
            with open('respSearch.xml', 'wb') as fn:
                fn.write(xml)

        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
        return None

    def do_search(self, sobj):
        result = None
        cols = ''
        guid = None
        if self.bfaserver.develop:
            sobj.debug = True

        if sobj.selected_cols is None:
            sobj.selected_cols = ['gdocid']
        if type(sobj.selected_cols) is str:
            sobj.selected_cols = sobj.selected_cols.split(',')
        if 'editmode' in sobj.selected_cols:
            sobj.selected_cols.remove('editmode')

        if sobj.bookmark is not None:
            typ, userid, groupid, index = sobj.bookmark.split(',')
            if self.bfaserver.con.main.is_guid(index):
                guid = index
            else:
                index = int(index)
            tmpobj = self.load_bookmark(self._job.profile, typ, userid, groupid, index, guid)
            if tmpobj is not None:
                if tmpobj.constructive:         # nur, wenn wir auch aufbauend auf diesem Lesezeichen suchen dürfen
                    sobj.sqlwhere = tmpobj.sqlwhere
                    sobj.arcid = tmpobj.arcid
                    sobj.selectedarcid = tmpobj.selectedarcid
                    if type(tmpobj.selected_cols) is str:
                        tmpobj.selected_cols = tmpobj.selected_cols.split(',')
                    for col in tmpobj.selected_cols:
                        if col not in sobj.selected_cols:
                            sobj.selected_cols.append(col)
                else:
                    return result, cols

        if sobj.sqlwhere is not None and sobj.sqlwhere != '':
            if sobj.sqlwhere.find('%username%') > -1:
                if self._job.userid in self.bfaserver.user[self._job.profile]:
                    username = self.bfaserver.user[self._job.profile][self._job.userid].name
                    if sobj.sqlwhere.find('like "%username%"') > -1:
                        sobj.sqlwhere = sobj.sqlwhere.replace('%username%', '%' + username + '%')
                    else:
                        sobj.sqlwhere = sobj.sqlwhere.replace('%username%', username)
            if sobj.sqlwhere:
                if 'doctxt' in sobj.sqlwhere and 'doctxt' not in sobj.selected_cols:
                    sobj.selected_cols.append('doctxt')
                if 'doctags' in sobj.sqlwhere and 'doctags' not in sobj.selected_cols:
                    sobj.selected_cols.append('doctags')
                if 'annotxt' in sobj.sqlwhere and 'annotxt' not in sobj.selected_cols:
                    sobj.selected_cols.append('annotxt')
                if 'fieldtxt' in sobj.sqlwhere and 'fieldtxt' not in sobj.selected_cols:
                    sobj.selected_cols.append('fieldtxt')
                if 'sg.groupid' in sobj.sqlwhere and 'sg.groupid' not in sobj.selected_cols:
                    sobj.selected_cols.append('sg.groupid')
                
                matches = re.findall(r'add_\d+', sobj.sqlwhere)
                if matches:
                    for match in matches:
                        if type(sobj.selected_cols) is list and match not in sobj.selected_cols:
                            sql = "select indextableid from sys_addfields where fieldname = '%s'" % match
                            try:
                                result = self.db[self._job.profile].execute(sqla.text(sql)).fetchone()
                            except:
                                self.log.exception("Fehler beim Abfragen der indextableid")
                            else:
                                if result:
                                    if match not in sobj.selected_cols and match + ':' + str(result[0]) not in sobj.selected_cols:
                                        sobj.selected_cols.append(match + ':' + str(result[0]))
        if sobj.order:
            searchorder = sobj.order.split(',')
            for order in searchorder:
                if order.find(' ') > -1:
                    order = order[:order.find(' ')]
                if order not in sobj.selected_cols:
                    sobj.selected_cols.append(order)

        if sobj.direction and sobj.direction not in sobj.order:
            sobj.order += ' ' + sobj.direction
            sobj.direction = None

        if not sobj.searchdatetype:
            sobj.searchdatetype = 8 if sobj.usemodifieddate else 4

        if type(sobj.selected_cols) is list:
            sobj.selected_cols = ",".join(sobj.selected_cols)
        for idx in range(len(sobj.statefields)):
            if isinstance(sobj.statefields[idx], bitfarm_doc.BitfarmStatefield) and not sobj.statefields[idx].bitmask:
                sql = 'select bitmask, indexcolumnid from sys_statefields where stateid = %s' % sobj.statefields[idx].id
                try:
                    result = self.db[self._job.profile].execute(sqla.text(sql)).fetchone()
                except:
                    self.log.exception(_("%s Fehler bei Abfrage der Bitmaske des Statusfeldes.") % self.name)
                else:
                    sobj.statefields[idx].bitmask = int(result[0])
                    sobj.statefields[idx].indexcolumnid = int(result[1])

        sobj.debug = True
        result, cols = self.SearchProc.search(sobj)

        if result is None:
            self.log.warning("Fehler bei der Suche")
        else:
            self.log.debug("Resultcount %s, selected_cols %s" % (str(len(result)), cols))

        tmpobj = None

        return result, cols

    def reqEdit2(self):
        self.log.debug(_('%s reqEdit2') % self.name)

        if self.bfaserver.develop:
            with open('reqEdit2_%i.xml' % self._job.gdocid, 'w') as fn:
                bitfarm_xml.object_to_xmlfile(self._job, fn)

        doc_right = self.check_docrights()

        if self._job.userid not in self.bfaserver.user[self._job.profile] or not doc_right.modify:
            xml = bitfarm_xml.create_xml_request(reqtype="respDenied", userid=0, gdocid=self._job.gdocid,
                                                 data=_("Keine Berechtigung auf das Dokument"))
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None
        doc = bitfarm_doc.BitfarmDocument()
        doc.profile = self._job.profile
        if self._job.fields_arcid is not None:
            doc.fields_arcid = self._job.fields_arcid
        if self._job.symbolic_arcid is not None:
            doc.symbolic_arcid = self._job.symbolic_arcid
        if not self.bfaProcessing.load(doc, self._job.gdocid, level=['base', 'paths', 'text', 'fields', 'iev', 'workflow'], logid=self.name):
            xml = bitfarm_xml.create_xml_request(reqtype="respDocError", userid=0, gdocid=self._job.gdocid)
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None
        else:
            self.add_private_annotations(doc, doc_right)

            doc.rights = doc_right.rights
            doc.in_progress = self._job.userid

            docids_in = []
            docids_out = []
            if type(self._job.parameter) in [str, str]:
                docids_in = self._job.parameter.split(";")
            elif type(self._job.parameter) is list:
                docids_in = self._job.parameter
            docids_in = [int(x) for x in docids_in]

            for docid in docids_in:
                doc_right = self.check_docrights(docid)
                if doc_right.modify and docid not in self.bfaserver.in_progress[self._job.profile]:
                    docids_out.append(docid)

            BitfarmWorkerThread.in_progresslock.acquire()
            try:
                self.bfaserver.in_progress[self._job.profile][self._job.gdocid] = self._job.userid
                for docid in docids_out:
                    self.bfaserver.in_progress[self._job.profile][docid] = self._job.userid

            finally:
                BitfarmWorkerThread.in_progresslock.release()
            if self.bfaProcessing.set_editmode(doc, self._job.userid, docids_out):
                doc.in_editmode = ",".join([str(x) for x in list(self.bfaserver.in_progress[self._job.profile].keys())])
                xml = bitfarm_xml.create_xml_request(reqtype="respAccess", gdocid=self._job.gdocid, data=doc,
                                                     datatype='BitfarmDocument',
                                                     parameter=';'.join([str(x) for x in docids_out]))
            else:
                userid = self.bfaserver.in_progress[self._job.profile][self._job.gdocid]
                xml = bitfarm_xml.create_xml_request(reqtype="respEditDenied", userid=userid, gdocid=self._job.gdocid)
            if self.bfaserver.develop:
                self.log.debug("%s %s" % (self.name, xml))
            try:
                self._job.sock.send(xml)
                if self.bfaserver.develop:
                    with open('respEdit_%i.xml' % self._job.gdocid, 'wb') as fn:
                        fn.write(xml)

            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None

    def reqEdit(self):
        self.log.debug(_('%s reqEdit') % self.name)

        if self.bfaserver.develop:
            with open('reqEdit_%i.xml' % self._job.gdocid, 'w') as fn:
                bitfarm_xml.object_to_xmlfile(self._job, fn)

        # diese Situation (Doc schon von einem anderen User im Editmode) wird schon im Server-Mainloop abgefangen,
        # aber wir wissen, wer die Mutter der Porzelankiste ist
        if self._job.gdocid in self.bfaserver.in_progress[self._job.profile] and self.bfaserver.in_progress[self._job.profile][self._job.gdocid] != self._job.userid:
            userid = self.bfaserver.in_progress[self._job.profile][self._job.gdocid]
            xml = bitfarm_xml.create_xml_request(reqtype="respEditDenied", userid=userid, gdocid=self._job.gdocid)

            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

        doc_right = self.check_docrights()
        if self._job.userid not in self.bfaserver.user[self._job.profile] or not doc_right.modify:
            xml = bitfarm_xml.create_xml_request(reqtype="respDenied", userid=0, gdocid=self._job.gdocid,
                                                 data=_("Keine Berechtigung auf das Dokument"))
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

        doc = bitfarm_doc.BitfarmDocument()
        doc.profile = self._job.profile
        if self._job.fields_arcid is not None:
            doc.fields_arcid = self._job.fields_arcid
        if self._job.symbolic_arcid is not None:
            doc.symbolic_arcid = self._job.symbolic_arcid
        if not self.bfaProcessing.load(doc, self._job.gdocid, level=['base', 'paths', 'text', 'fields', 'iev', 'workflow', 'no_items'], logid=self.name):
            xml = bitfarm_xml.create_xml_request(reqtype="respDocError", userid=0, gdocid=self._job.gdocid)
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None
        else:
            # wir prüfen zusätzlich, ob das Dokument ggf von jemand anderem ausgecheckt wurde
            if doc.checkout_userid is not None and doc.checkout_userid != self._job.userid:
                xml = bitfarm_xml.create_xml_request(reqtype="respEditDenied", userid=doc.checkout_userid, gdocid=self._job.gdocid,
                                                     data=_("Das Dokument wurde von einem anderen Benutzer ausgecheckt"))

                try:
                    self._job.sock.send(xml)
                except:
                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None

            self.add_private_annotations(doc, doc_right)

            doc.rights = doc_right.rights
            doc.in_progress = self._job.userid

            docids_in = []
            docids_out = []
            if type(self._job.parameter) in [str, str]:
                docids_in = self._job.parameter.split(";")
            elif type(self._job.parameter) is list:
                docids_in = self._job.parameter
            docids_in = [int(x) for x in docids_in]

            for docid in docids_in:
                doc_right = self.check_docrights(docid)
                if doc_right.modify:
                    if docid not in self.bfaserver.in_progress[self._job.profile]:
                        docids_out.append(docid)
                    elif self.bfaserver.in_progress[self._job.profile][docid] == self._job.userid:  # sollte ein doc schon im editmode sein
                        docids_out.append(docid)                                                    # von diesem user, dann wird es auch zurückgemeldet
                                                                                                    # (kann passieren beim ausführen von plugins mit resync)
            BitfarmWorkerThread.in_progresslock.acquire()
            try:
                self.bfaserver.in_progress[self._job.profile][self._job.gdocid] = self._job.userid
                for docid in docids_out:
                    self.bfaserver.in_progress[self._job.profile][docid] = self._job.userid

            finally:
                BitfarmWorkerThread.in_progresslock.release()

            doc.in_editmode = ",".join([str(x) for x in list(self.bfaserver.in_progress[self._job.profile].keys())])
            xml = bitfarm_xml.create_xml_request(reqtype="respAccess", gdocid=self._job.gdocid, data=doc,
                                                 datatype='BitfarmDocument',
                                                 parameter=';'.join([str(x) for x in docids_out]))
            if self.bfaserver.develop:
                self.log.debug("%s %s" % (self.name, xml))
            try:
                self._job.sock.send(xml)
                if self.bfaserver.develop:
                    with open('respEdit_%i.xml' % self._job.gdocid, 'wb') as fn:
                        fn.write(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None

    def reqCancel(self):
        self.log.debug(_('%s reqCancel') % self.name)

        if self.bfaserver.develop:
            with open('reqCancel_%i.xml' % self._job.gdocid, 'w') as fn:
                bitfarm_xml.object_to_xmlfile(self._job, fn)

        docids_in = []
        if type(self._job.parameter) in [str, str]:
            docids_in = self._job.parameter.split(";")
        elif type(self._job.parameter) is list:
            docids_in = self._job.parameter
        docids_in = [int(x) for x in docids_in]

        BitfarmWorkerThread.in_progresslock.acquire()
        try:
            if self._job.gdocid in self.bfaserver.in_progress[self._job.profile]:
                del self.bfaserver.in_progress[self._job.profile][self._job.gdocid]
            for docid in docids_in:
                if docid in self.bfaserver.in_progress[self._job.profile]:
                    del self.bfaserver.in_progress[self._job.profile][docid]
        finally:
            BitfarmWorkerThread.in_progresslock.release()
        # self.bfaProcessing.unset_editmode(self._job, self._job.userid, docids=docids_in)

        xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("reqCancel ausgeführt (2)"))
        if self.bfaserver.develop:
            self.log.debug(_("%s %s") % (self.name, xml))
        try:
            self._job.sock.send(xml)
            if self.bfaserver.develop:
                with open('respCancel_%i.xml' % self._job.gdocid, 'wb') as fn:
                    fn.write(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqSave(self):
        self.log.debug(_('%s reqSave') % self.name)
        con = self.bfaserver.con.connections[self._job.profile]
        docids_in = []
        if self.bfaserver.develop:
            with open('reqSave_%i.xml' % self._job.gdocid, 'w') as fn:
                bitfarm_xml.object_to_xmlfile(self._job, fn)
        if type(self._job.parameter) in [str, str]:
            docids_in = self._job.parameter.split(";")
        elif type(self._job.parameter) is list:
            docids_in = self._job.parameter
        docids_in = [int(x) for x in docids_in if int(x) != self._job.gdocid]

        gdocids, error, msg, addfields_saved, annos_saved = self.do_save(docids_in=docids_in)

        if error:
            xml = bitfarm_xml.create_xml_request(reqtype="respSaveError", data=error, datatype="text")
            self.log.error("%s %s" % (self.name, error))
        else:
            xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=msg, datatype="text")

        if self.bfaserver.develop:
            self.log.debug("%s %s" % (self.name, xml))
            with open('respSave_%i.xml' % self._job.gdocid, 'wb') as fn:
                fn.write(xml)
        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

        self.check_abo(gdocids)

        if addfields_saved:  # fieldtxt in sys_doctxt aktualisieren
            for gdocid in gdocids:
                if not self.bfaProcessing.refresh_fieldtxt(gdocid=gdocid, profile=self._job.profile):
                    self.log.error(_("%s fieldtxt konnte nicht aktualisiert werden [%i]") % (self.name, gdocid))

        if annos_saved:
            for gdocid in gdocids:
                annotext = ""
                BitfarmWorkerThread.annolock.acquire()
                try:
                    # annotext = self.bfaserver.get_annotext(con.profile, gdocid)
                    annotext = self.bfaserver.get_annotext(con.server, con.port, con.db_name, con.user, con.password, int(gdocid))
                except:
                    self.log.exception(_("%s Fehler beim Aktualisieren des Annotextes für %i") % (self.name, gdocid))
                else:
                    if annotext:
                        annotext = annotext.encode(con.db_encoding, "ignore")
                        annotext = annotext.decode(con.db_encoding, "ignore")
                    else:
                        annotext = None

                    vals = {con.sys_doctxt.c.annotxt: annotext}
                    sql = con.sys_doctxt.update(whereclause=(con.sys_doctxt.c.gdocid == gdocid), values=vals)

                    try:
                        self.db[self._job.profile].execute(sql)
                    except:
                        self.log.exception(_("%s Fehler beim Schreiben des Annotextes für %s") % (self.name,gdocid))
                finally:
                    BitfarmWorkerThread.annolock.release()

    def do_save(self, docids_in=[]):
        error = ""
        msg = ""
        addfields_saved = False
        annos_saved = False

        con = self.bfaserver.con.connections[self._job.profile]

        gdocids = [self._job.gdocid]
        gdocids.extend(docids_in)

        if type(self._job.data) is not list:
            self._job.data = [self._job.data]

        if len(self._job.data) == 0:
            error += _("Es wurden keine Objekte zum Speichern übergeben\n")
        else:
            for obj in self._job.data[:]:   # wir prüfen DocObjekte und packen Addfields, Statefields und Annos einzelnd in die Liste (für Historyeinträge)
                if isinstance(obj, bitfarm_doc.BitfarmDocument):
                    for field in list(obj.fields.values()):
                        if field.modified:
                            self._job.data.append(field)
                    for field in list(obj.statefields.values()):
                        if field.modified:
                            self._job.data.append(field)
                    for anno in obj.annotations:
                        if anno.modified:
                            self._job.data.append(anno)
                    obj.fields = OrderedDict()
                    obj.statefields = OrderedDict()
                    obj.anno = []

            for obj in self._job.data:
                try:
                    if isinstance(obj, bitfarm_doc.BitfarmAddfield):  # Zusatzfelder speichern
                        if obj.modified:
                            saveids = []

                            sql = 'select gdocid, %s from sys_docidx_%s where gdocid in (%s)' % (obj.add, obj.indextableid, ",".join([str(x) for x in gdocids]))
                            try:
                                result = self.db[self._job.profile].execute(sqla.text(sql)).fetchall()
                            except:
                                self.log.exception(_("%s Fehler bei Abfrage des Oldvalues des/der Zusatzfeldes/r.") % self.name)
                            else:
                                oldvalues = {}
                                if result:
                                    for row in result:
                                        oldvalues[row[0]] = row[1]

                            for gdocid in gdocids:
                                right = self.check_addfieldrights(obj.id, gdocid=gdocid)
                                if right.rights & 4 == 4:  # modify
                                    if gdocid not in saveids:
                                        saveids.append(gdocid)
                                if right.rights & 2 == 2:  # write, prüfen ob noch nicht beschrieben
                                    if gdocid not in oldvalues or oldvalues[gdocid] is None:
                                        if gdocid not in saveids:
                                            saveids.append(gdocid)

                                    # sql = 'select %s from sys_docidx_%s where gdocid = %s' % (obj.add, obj.indextableid, gdocid)
                                    # try:
                                    #     result = self.db[self._job.profile].execute(sqla.text(sql)).fetchone()
                                    # except:
                                    #     self.log.exception(_(u"%s Fehler bei Abfrage des Zusatzfeldes." ) % self.name)
                                    #     error += _(u"Fehler bei Abfrage des Zusatzfeldes.")
                                    # else:
                                    #     if result is None or result[0] is None:
                                    #         saveids.append(gdocid)

                            if len(saveids) == 0:
                                error += _("Keine Berechtigung auf Zusatzfeld %s.") % obj.name if obj.name is not None else obj.add
                            else:
                                saved = True

                                BitfarmWorkerThread.altertablelock.acquire()
                                try:
                                    if not self.bfaProcessing.save_addfield(obj, saveids, self._job.profile, self._job.userid):
                                        saved = False
                                        error += _("Das Zusatzfeld %s konnte nicht gespeichert werden.") % obj.name if obj.name is not None else obj.add
                                finally:
                                    BitfarmWorkerThread.altertablelock.release()

                                if saved:
                                    addfields_saved = True
                                    if len(saveids) != len(gdocids):
                                        msg += _("Es konnten nicht alle Zusatzfelder gespeichert werden.")

                                    for saveid in saveids:
                                        if obj.history:
                                            hist = bitfarm_xml.copy_bitfarmobj(obj.history)
                                        else:
                                            hist = bitfarm_doc.BitfarmHistory()
                                            hist.userid = self._job.userid

                                        oldvalue = oldvalues[saveid] if saveid in oldvalues else None

                                        if oldvalue == obj.value:   # auf den gleichen Wert gesetzt (zB wenn mehrere Dokumente verarbeitet werden) -> Historie überspringen
                                            continue

                                        if oldvalue is None:
                                            hist.taskid = 227
                                            hist.value1 = obj.name
                                            hist.value2 = obj.value
                                        elif obj.value is None:
                                            hist.taskid = 234
                                            hist.value1 = obj.name
                                        else:
                                            hist.taskid = 215
                                            hist.value1 = obj.name
                                            hist.value2 = oldvalue
                                            hist.value3 = obj.value

                                        self.bfaProcessing.write_history(history=hist, profile=self._job.profile,
                                                                                 gdocid=saveid, logid=self.name)

                                    # vals = {con.sys_doc.c.modified_userid: self._job.userid,
                                    #         con.sys_doc.c.modified: None}
                                    # sql = con.sys_doc.update(whereclause=(con.sys_doc.c.gdocid.in_(saveids)), values=vals)
                                    #
                                    # try:
                                    #     self.db[self._job.profile].execute(sql)
                                    # except:
                                    #     self.log.exception(_(u"%s Fehler beim Aktualisieren des Modified Date") % self.name)

                                    obj.history = []
                except:
                    self.log.exception(_("%s Es hat einen Fehler beim Speichern der Zusatzfelder gegeben.\n") %(self.name))
                    error += _("Es hat einen Fehler beim Speichern der Zusatzfelder gegeben.\n")

                try:
                    if isinstance(obj, bitfarm_doc.BitfarmStatefield):  # Statusfelder speichern
                        if obj.modified:
                            if not obj.bitmask:
                                sql = 'select bitmask, indexcolumnid from sys_statefields where stateid = %s' % obj.id
                                try:
                                    result = self.db[self._job.profile].execute(sqla.text(sql)).fetchone()
                                except:
                                    self.log.exception(_("%s Fehler bei Abfrage der Bitmaske des Statusfeldes.") % self.name)
                                    error += _("Fehler bei Abfrage der Bitmaske des Statusfeldes.")
                                else:
                                    obj.bitmask = int(result[0])
                                    obj.indexcolumnid = int(result[1])

                            if obj.bitmask:
                                saveids = []
                                oldvalues = {}
                                sql = 'select gdocid, docstate_%s from sys_doc where gdocid in (%s)' % (obj.indexcolumnid, ",".join([str(x) for x in gdocids]))
                                try:
                                    result = self.db[self._job.profile].execute(sqla.text(sql)).fetchall()
                                except:
                                    self.log.exception(_("%s Fehler bei Abfrage des Statusfeldes.") % self.name)
                                    error += _("Fehler bei Abfrage des Statusfeldes.")
                                else:
                                    if result:
                                        for row in result:
                                            oldvalues[row[0]] = row[1] & obj.bitmask == obj.bitmask

                                for gdocid in gdocids:
                                    right = self.check_statefieldrights(obj.id, gdocid=gdocid)
                                    if right.rights & 4 == 4:  # modify
                                        if gdocid not in saveids:
                                            saveids.append(gdocid)
                                    if right.rights & 2 == 2:  # write, prüfen ob noch nicht beschrieben
                                        # sql = 'select docstate_%s from sys_doc where gdocid = %s and docstate_%s & %s = %s' % (obj.indexcolumnid, gdocid, obj.indexcolumnid, obj.bitmask, obj.bitmask)
                                        # try:
                                        #     result = self.db[self._job.profile].execute(sqla.text(sql)).fetchone()
                                        # except:
                                        #     self.log.exception(_(u"%s Fehler bei Abfrage des Statusfeldes.") %(self.name))
                                        #     error += _(u"Fehler bei Abfrage des Statusfeldes.")
                                        # else:
                                        #     if result is None or result[0] is None:
                                        #         saveids.append(gdocid)
                                        if gdocid not in oldvalues or not oldvalues[gdocid]:
                                            if gdocid not in saveids:
                                                saveids.append(gdocid)

                                if len(saveids) == 0:
                                    error += _("Keine Berechtigung auf Statusfeld %s.") % obj.name
                                if not self.bfaProcessing.save_statefield(obj, saveids, self._job.profile, self._job.userid, oldvalues):
                                    error += _("Das Statusfeld %s konnte nicht gespeichert werden.") % obj.name
                                else:
                                    if len(saveids) != len(gdocids):
                                        msg += _("Es konnten nicht alle Statusfelder gespeichert werden.")

                                    for saveid in saveids:
                                        if obj.history:
                                            hist = bitfarm_xml.copy_bitfarmobj(obj.history)
                                        else:
                                            hist = bitfarm_doc.BitfarmHistory()
                                            hist.userid = self._job.userid

                                        oldvalue = oldvalues[saveid] if saveid in oldvalues else False

                                        if oldvalue == obj.value:   # auf den gleichen Wert gesetzt (zB wenn mehrere Dokumente verarbeitet werden) -> Historie überspringen
                                            continue

                                        hist.taskid = 216
                                        hist.value1 = obj.name
                                        hist.value2 = _("nicht gesetzt") if not oldvalue else _("gesetzt")
                                        hist.value3 = _("nicht gesetzt") if obj.value is None or not obj.value else _("gesetzt")

                                        self.bfaProcessing.write_history(history=hist, profile=self._job.profile,
                                                                                 gdocid=saveid, logid=self.name)

                                    # vals = {con.sys_doc.c.modified_userid: self._job.userid,
                                    #         con.sys_doc.c.modified: None}
                                    # sql = con.sys_doc.update(whereclause=(con.sys_doc.c.gdocid.in_(saveids)), values=vals)
                                    #
                                    # try:
                                    #     self.db[self._job.profile].execute(sql)
                                    # except:
                                    #     self.log.exception(_(u"%s Fehler beim Aktualisieren des Modified Date") % self.name)

                                    obj.history = []

                                    # if obj.func is not None and obj.func != 0 and obj.addfieldid is not None and obj.addfieldid != 0:
                                    #     sql = 'select * from sys_addfields where addfieldid = %s' % obj.addfieldid
                                    #     try:
                                    #         result = self.db[self._job.profile].execute(sqla.text(sql)).fetchone()
                                    #     except:
                                    #         self.log.exception(_(u"%s Fehler bei Abfrage der sys_addfields") % self.name)
                                    #         error += _(u"Fehler bei Abfrage der sys_addfields")
                                    #     else:
                                    #         indextable = int(result['indextableid'])
                                    #         fieldname = result['fieldname']
                                    #         displayname = result['displayname']
                                    #         typ = result['type']
                                    #         for saveid in saveids:
                                    #             hist = bitfarm_doc.BitfarmHistory()
                                    #             hist.userid = self._job.userid
                                    #
                                    #             oldvalue = oldvalues[saveid] if saveid in oldvalues else False
                                    #
                                    #             if oldvalue == obj.value:   # auf den gleichen Wert gesetzt (zB wenn mehrere Dokumente verarbeitet werden) -> Funktion überspringen
                                    #                 continue
                                    #
                                    #             if obj.value:
                                    #                 if obj.func == 1:
                                    #                     value = datetime.datetime.now().strftime('%H:%M:%S')
                                    #                 if obj.func == 2:
                                    #                     if typ == 1:
                                    #                         value = datetime.datetime.now().strftime('%Y-%m-%d')
                                    #                     else:
                                    #                         value = datetime.datetime.now().strftime('%d.%m.%Y')
                                    #                 if obj.func == 3:
                                    #                     value = datetime.datetime.now().strftime('%d.%m.%Y %H:%M:%S')
                                    #                 if obj.func == 4:
                                    #                     user = self.bfaserver.con.connections[self._job.profile].read_user(db=self.db[self._job.profile], userid=self._job.userid, logid=self.name)
                                    #                     value = u'%s' % user.name if user is not None else u""
                                    #                 if obj.func == 5:
                                    #                     user = self.bfaserver.con.connections[self._job.profile].read_user(db=self.db[self._job.profile], userid=self._job.userid, logid=self.name)
                                    #                     value = u'%s %s' % (user.name if user is not None else u"", datetime.datetime.now().strftime('%d.%m.%Y %H:%M:%S'))
                                    #                 sql = u"insert into sys_docidx_%s (gdocid, %s) values (%s, '%s') on duplicate key update %s = '%s'" % (indextable, fieldname, saveid, value, fieldname, value)
                                    #                 hist.taskid = 225
                                    #                 hist.value1 = displayname
                                    #                 hist.value2 = value
                                    #             else:
                                    #                 sql = u"insert into sys_docidx_%s (gdocid, %s) values (%s, NULL) on duplicate key update %s = NULL" % (indextable, fieldname, saveid, fieldname)
                                    #                 hist.taskid = 226
                                    #                 hist.value1 = displayname
                                    #
                                    #             try:
                                    #                 self.db[self._job.profile].execute(sqla.text(sql))
                                    #             except:
                                    #                 self.log.exception(_(u"%s Fehler bei Füllen des verknüpften Zusatzfeldes") % self.name)
                                    #                 error += _(u"Fehler bei Füllen des verknüpften Zusatzfeldes")
                                    #             else:
                                    #                 self.bfaProcessing.write_history(history=hist,
                                    #                                                          profile=self._job.profile,
                                    #                                                          gdocid=saveid, logid=self.name)
                except:
                    self.log.exception(_("%s Es hat einen Fehler beim Speichern der Statusfelder gegeben\n") % self.name)
                    error += _("Es hat einen Fehler beim Speichern der Statusfelder gegeben\n")

                try:
                    if isinstance(obj, bitfarm_doc.BitfarmAnnotation):  # Annos speichern, geht nur für ein Dokument nicht für eine Liste
                        if obj.modified:
                            if not self.bfaProcessing.save_annotation(obj, self._job.gdocid, self._job.profile, logid=self.name):
                                error += _("Die Annotation konnte nicht gespeichert werden")
                            else:
                                annos_saved = True
                                if obj.history and obj.layer == 0:      # Ticket 15477 wir schreiben nur die History für öffentliche Annos
                                    self.bfaProcessing.write_history(history=obj.history, profile=self._job.profile, logid=self.name)
                                obj.history = []

                                # vals = {con.sys_doc.c.modified_userid: self._job.userid,
                                #         con.sys_doc.c.modified: None}
                                # sql = con.sys_doc.update(whereclause=(con.sys_doc.c.gdocid == self._job.gdocid), values=vals)
                                #
                                # try:
                                #     self.db[self._job.profile].execute(sql)
                                # except:
                                #     self.log.exception(_(u"%s Fehler beim Aktualisieren des Modified Date") % self.name)
                except:
                    self.log.exception(_("%s Es hat einen Fehler beim Speichern der Annotations gegeben\n") % self.name)
                    error += _("Es hat einen Fehler beim Speichern der Annotations gegeben\n")

                try:
                    if isinstance(obj, bitfarm_doc.BitfarmDocument):  # Dokument speichern
                        if obj.modified:
                            if not obj.profile:
                                obj.profile = self._job.profile
                            obj.modified_userid = self._job.userid
                            if not self.bfaProcessing.write(obj, logid=self.name):
                                error += _("Das Dokument konnte nicht gespeichert werden")
                            else:
                                self.bfaProcessing.write_history(history=obj.history, profile=self._job.profile, logid=self.name)
                                obj.history = []
                except:
                    self.log.exception(_("%s Es hat einen Fehler beim Speichern des Dokuments gegeben\n") % self.name)
                    error += _("Es hat einen Fehler beim Speichern des Dokuments gegeben\n")

        if self._job.history is not None:
            self.bfaProcessing.write_history(history=self._job.history, profile=self._job.profile, logid=self.name)
            self._job.history = []

        # BitfarmWorkerThread.in_progresslock.acquire()
        # try:
        #     for gdocid in gdocids:
        #         if gdocid in self.bfaserver.in_progress[self._job.profile]:
        #             del self.bfaserver.in_progress[self._job.profile][gdocid]
        # finally:
        #     BitfarmWorkerThread.in_progresslock.release()
        # self.bfaProcessing.unset_editmode(self._job, self._job.userid, docids=gdocids)

        return gdocids, error, msg, addfields_saved, annos_saved

    def reqResetDLM(self):
        self.log.debug('%s reqResetDLM' % self.name)

        gdocids = self._job.parameter.split(";")

        for gdocid in gdocids:
            doc_right = self.check_docrights(gdocid=int(gdocid))
            error = ""
            if not ('admin' in self.bfaserver.user[self._job.profile][self._job.userid].flags or doc_right.admin):  # kein dmsadmin bzw kein adminrecht auf das dokument?
                error += "Keine Berechtigung, die Lebensdauer des Dokuments [%s] zurück zu setzen."
            else:
                doc = bitfarm_doc.BitfarmDocument()
                doc.profile = self._job.profile
                self.bfaProcessing.load(doc, gdocid, ['base'])
                success, expired = self.bfaProcessing.reset_expired_date(doc.arcid, doc.gdocid, self._job.profile)
                hist = bitfarm_doc.BitfarmHistory()
                hist.userid = self._job.userid

                if success:
                    hist.taskid = 119
                    hist.value1 = expired.strftime('%d.%m.%Y') if expired is not None else 'leer'
                # else:
                #     if expired is not None:
                #         hist.taskid = 117
                #         hist.value1 = expired.strftime('%d.%m.%Y') if expired is not None else 'leer'
                #     else:
                #         hist.taskid = 116

                    self.bfaProcessing.write_history(history=hist, profile=self._job.profile,
                                                     gdocid=doc.gdocid, logid=self.name)

                if error:
                    xml = bitfarm_xml.create_xml_request(reqtype="respResetDLMError", data=error, datatype="text")
                    self.log.error("%s %s" % (self.name, str(error)))
                else:
                    xml = bitfarm_xml.create_xml_request(reqtype="respPending")

                try:
                    self._job.sock.send(xml)
                except:
                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                    return None

        if error:
            xml = bitfarm_xml.create_xml_request(reqtype="respResetDLMError", data=error, datatype="text")
            self.log.error("%s %s" % (self.name, str(error)))
        else:
            xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("reqResetDLM ausgeführt"))

        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqSaveHistory(self):
        self.log.debug('%s reqSaveHistory' % self.name)

        if self._job.history is not None:
            if self._job.history:
                self.bfaProcessing.write_history(history=self._job.history, profile=self._job.profile, logid=self.name)
                self._job.history = []

        if self._job.data is not None:
            if self._job.data:
                self.bfaProcessing.write_history(history=self._job.data, profile=self._job.profile, logid=self.name)

        xml = bitfarm_xml.create_xml_request(reqtype="respOK", data='History geschrieben', datatype="text")

        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def check_abo(self, gdocids):
        self.log.debug('%s check_abo' % self.name)

        if type(gdocids) in [str, str, int, int]:
            gdocids = [int(gdocids)]

        ids = ",".join([str(x) for x in gdocids])
        sql = 'select * from sys_abo left join sys_user on sys_abo.userid = sys_user.userid ' \
              'where gdocid in (%s) and sys_abo.userid <> %s' % (ids, self._job.userid)
        try:
            rows = self.db[self._job.profile].execute(sqla.text(sql), use_labels=True).fetchall()
        except:
            self.log.exception(_("%s Fehler beim Abfragen der sys_abo [%s]\n") %(self.name,  ids))
        else:
            if rows:
                for row in rows:
                    if row['userid'] != self._job.userid:
                        wf = bitfarm_doc.BitfarmWorkflow()
                        wf.gdocid = row['gdocid']
                        wf.info = _("Dieses abonnierte Dokument wurde geändert")
                        wf.comment = row['info']
                        wf.profile = self._job.profile
                        recipients = [row['userid']]
                        # if not self.bfaWorkflow.bfaWflow.create_resub(wf, recipients, wf.info, wf.comment, 256):
                        if not self.bfaWorkflow.create_resub(wf, recipients, wf.info, wf.comment, 256):
                            self.log.error(_("%s Wiedervorlage konnte nicht angelegt werden [%s]\n" ) %(self.name,  wf.gdocid))
                        else:
                            hist = bitfarm_doc.BitfarmHistory()
                            hist.gdocid = wf.gdocid
                            hist.taskid = 14
                            hist.userid = self._job.userid
                            hist.value1 = row['displayname']
                            self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

    def reqSaveResub(self):
        self.log.debug('%s reqSaveResub' % self.name)
        error = ""

        if self.bfaserver.develop:
            with open('reqSaveResub_%i.xml' % self._job.gdocid, 'w') as fn:
                bitfarm_xml.object_to_xmlfile(self._job, fn)

        con = self.bfaserver.con.connections[self._job.profile]

        if type(self._job.data) is not list:
            self._job.data = [self._job.data]

        param = []
        if type(self._job.parameter) is str:
            param = self._job.parameter.split(";")
        elif type(self._job.parameter) is list:
            param = self._job.parameter

        if len(self._job.data) == 0:
            error += _("Es wurden keine Objekte zum Speichern übergeben\n")
        elif param[0] not in ['forward', 'create', 'change', 'set_done', 'cancel', 'delete']:
            error += _("Unbekannter Parameter %s\n" % param[0])
        else:
            for wf in self._job.data:
                if isinstance(wf, bitfarm_doc.BitfarmWorkflow):
                    recipients = []
                    user = None
                    group = None
                    # if wf.info:
                    #     wf.info = re.sub('%{1}', repl_percent, wf.info)
                    #     wf.info = re.sub('\\{1}', repl_backslash, wf.info)
                    # if wf.comment:
                    #     wf.comment = re.sub('%{1}', repl_percent, wf.comment)
                    if wf.options is None:
                        wf.options = 0

                    if wf.status is None:
                        wf.status = 0

                    if wf.userid_work is not None and type(wf.userid_work) in [int, int]:
                        user = con.read_user(db=self.db[con.profile], userid=wf.userid_work, logid=self.name)

                    if wf.groupid is not None and type(wf.groupid) in [int, int]:
                        group = con.read_group(db=self.db[con.profile], groupid=wf.groupid, logid=self.name)

                    if wf.users is not None and type(wf.users) in [str, str]:
                        for uid in wf.users.split(';'):
                            user = con.read_user(db=self.db[con.profile], userid=uid, logid=self.name)
                            if user is not None:
                                recipients.append(user)
                    if wf.groups is not None and type(wf.groups) in [str, str]:
                        for gid in wf.groups.split(';'):
                            group = con.read_group(db=self.db[con.profile], groupid=gid, logid=self.name)
                            if group is not None:
                                recipients.append(group)

                    if len(recipients) == 0 and ('forward' in param or 'create' in param):
                        error += _("Kein Empfänger für Wiedervorlage info=%s\n" % wf.info)
                        continue

                    curwf = bitfarm_doc.BitfarmWorkflow()
                    if wf.tim_id:
                        # Liste alle Userids für die wir die WVL bearbeiten dürfen (Urlaubsvertretung und Mitarbeiter)
                        auth_users = con.get_holidaylist(db=self.db[con.profile], user=self._job.userid, logid=self.name)
                        myuser = con.read_user(db=self.db[con.profile], userid=self._job.userid, logid=self.name)
                        if myuser.employees:
                            for employee in myuser.employees:
                                if employee not in auth_users:
                                    auth_users.append(employee)

                        sql = sqla.select([con.sys_timer], whereclause=(con.sys_timer.c.timid == wf.tim_id))

                        try:
                            row = self.db[self._job.profile].execute(sql).fetchone()
                        except:
                            error += _("Wiedervorlage konnte nicht gefunden werden timid=%s\n" % wf.tim_id)
                            continue
                        else:
                            if row:
                                curwf.tim_id = row[0]
                                curwf.tim_ref = row[1]
                                curwf.gdocid = row[2]
                                curwf.groupid = row[3]
                                curwf.userid_work = row[4]
                                curwf.userid_shed = row[5]
                                curwf.status = row[6]
                                curwf.date = row[7]
                                curwf.alarm_date = row[8]
                                curwf.options = row[9]
                                curwf.info = row[10]
                                curwf.comment = row[11]
                                curwf.hist_id = row[14]

                                forwarded = []
                                if curwf.hist_id != 0:
                                    sql = sqla.select([con.sys_timerhist.c.shedid], whereclause=(con.sys_timerhist.c.timref == curwf.tim_ref))

                                    try:
                                        result = self.db[self._job.profile].execute(sql).fetchall()
                                    except:
                                        error += _("Weiterleitung konnte nicht gefunden werden timid=%s\n" % wf.tim_id)
                                        continue
                                    else:
                                        if result:
                                            forwarded = [x[0] for x in result]

                                # if self._job.userid != curwf.userid_work and self._job.userid != curwf.userid_shed:
                                # TODO: hier muss ich noch die timerhist abfragen
                                if curwf.userid_work not in auth_users and curwf.userid_shed not in auth_users\
                                        and self._job.userid not in forwarded:
                                    error += _("Kein Zugriff auf den Workflow mit der timid=%s\n" % wf.tim_id)
                                    continue

                    if 'forward' in param:
                        # if not self.bfaWorkflow.bfaWflow.forward_resub(wf, recipients, wf.options):
                        if not self.bfaWorkflow.forward_resub(wf, recipients, wf.options):
                            if wf.error == EC_RESUBINCHECKBACK:
                                error += _("Die Wiedervorlage befindet sich in Rückfrage\n")
                                self.log.error("%s %s %s" % (self.name, error, wf.tim_id))
                            elif wf.error == EC_NEEDRECONNECT:
                                error += _("Die Wiedervorlage konnte nicht weitergeleitet werden\n")
                                self.log.error("%s %s %s" % (self.name, error, wf.tim_id))
                                self._job.need_reconnect = wf.error == EC_NEEDRECONNECT
                            else:
                                error += _("Die Wiedervorlage konnte nicht weitergeleitet werden\n")
                                self.log.error("%s %s %s" % (self.name, error, wf.tim_id))
                        else:
                            usernames = ",".join([x.name for x in recipients if isinstance(x, bitfarm_doc.BitfarmUser)])
                            groupnames = ",".join([x.name for x in recipients if isinstance(x, bitfarm_doc.BitfarmGroup)])

                            if wf.options & 64 == 64:
                                if not self.send_wfmail(wf, recipients, "Ihnen wurde eine Wiedervorlage weitergeleitet"):
                                    self.log.error("%s Die Benachrichtigung konnte nicht versendet werden tim_id=%s\n" % (self.name, wf.tim_id))

                            if usernames:
                                hist = bitfarm_doc.BitfarmHistory()
                                hist.gdocid = wf.gdocid
                                hist.userid = self._job.userid
                                hist.taskid = 403
                                hist.value1 = wf.info if wf.info is not None else ''
                                hist.value2 = usernames
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)
                            if groupnames:
                                hist = bitfarm_doc.BitfarmHistory()
                                hist.gdocid = wf.gdocid
                                hist.userid = self._job.userid
                                hist.taskid = 404
                                hist.value1 = wf.info if wf.info is not None else ''
                                hist.value2 = groupnames
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                    elif 'create' in param:
                        # if not self.bfaWorkflow.bfaWflow.create_resub(wf, recipients, wf.info, wf.comment, wf.options,
                        if not self.bfaWorkflow.create_resub(wf, recipients, wf.info, wf.comment, wf.options,
                                                                      None, wf.userid_shed, wf.date, wf.alarm_date):
                            if wf.error == EC_NOUSERORGROUPERROR or wf.error == EC_NEEDRECONNECT:
                                error += _("Die Wiedervorlage konnte nicht angelegt werden\n")
                                self.log.error("%s %s info = %s" % (self.name, error, wf.info))
                                self._job.need_reconnect = wf.error == EC_NEEDRECONNECT
                            else:
                                error += _("Die Wiedervorlage konnte nicht angelegt werden\n")
                                self.log.error("%s %s info = %s" % (self.name, error, wf.info))

                        else:
                            usernames = ",".join([x.name for x in recipients if isinstance(x, bitfarm_doc.BitfarmUser)])
                            groupnames = ",".join([x.name for x in recipients if isinstance(x, bitfarm_doc.BitfarmGroup)])

                            if wf.options & 64 == 64:
                                if not self.send_wfmail(wf, recipients, "Sie haben eine neue Wiedervorlage"):
                                    self.log.error("%s Die Benachrichtigung konnte nicht versendet werden tim_id=%s\n" % (self.name, wf.tim_id))

                            if usernames:
                                hist = bitfarm_doc.BitfarmHistory()
                                hist.gdocid = wf.gdocid
                                hist.userid = self._job.userid
                                hist.taskid = 401
                                hist.value1 = wf.info if wf.info is not None else ''
                                hist.value2 = usernames
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)
                            if groupnames:
                                hist = bitfarm_doc.BitfarmHistory()
                                hist.gdocid = wf.gdocid
                                hist.userid = self._job.userid
                                hist.taskid = 402
                                hist.value1 = wf.info if wf.info is not None else ''
                                hist.value2 = groupnames
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                    elif 'change' in param:
                        # if not self.bfaWorkflow.bfaWflow.change_resub(wf, wf.info, wf.comment, user, group,
                        if not self.bfaWorkflow.change_resub(wf, wf.info, wf.comment, user, group,
                                                                      wf.userid_shed, wf.date, wf.alarm_date,
                                                                      wf.status):
                            if wf.error == EC_NOUSERORGROUPERROR or wf.error == EC_NEEDRECONNECT:
                                error += _("Benutzer konnte nicht gefunden werden oder leere Gruppe gewählt\n")
                                self.log.error("%s %s timid = %s" % (self.name, error, wf.tim_id))
                                self._job.need_reconnect = wf.error == EC_NEEDRECONNECT
                            else:
                                error += _("Die Wiedervorlage konnte nicht geändert werden\n")
                                self.log.error("%s %s timid = %s" % (self.name, error, wf.tim_id))

                    elif 'delete' in param:
                        # if not self.bfaWorkflow.bfaWflow.delete_resub(wf):
                        if not self.bfaWorkflow.delete_resub(wf):
                            if wf.error == EC_DELETERESUBERROR or wf.error == EC_NEEDRECONNECT:
                                error += _("Die Wiedervorlage konnte nicht gelöscht werden\n")
                                self.log.error("%s %s timid = %s" % (self.name, error, wf.tim_id))
                                self._job.need_reconnect = wf.error == EC_NEEDRECONNECT
                            else:
                                error += _("Die Wiedervorlage konnte nicht gelöscht werden\n")
                                self.log.error("%s %s timid = %s" % (self.name, error, wf.tim_id))
                        else:
                            hist = bitfarm_doc.BitfarmHistory()
                            hist.gdocid = wf.gdocid
                            hist.userid = self._job.userid
                            hist.taskid = 407
                            hist.value1 = wf.info if wf.info is not None else ''
                            self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                    elif 'set_done' in param or 'cancel' in param:
                        # if not self.bfaWorkflow.bfaWflow.setstate_done(wf):
                        if not self.bfaWorkflow.setstate_done(wf):
                            if wf.error == EC_SETSTATEDONEERROR or wf.error == EC_NEEDRECONNECT:
                                error += _("Die Wiedervorlage konnte nicht fertig gesetzt werden\n")
                                self.log.error("%s %s timid = %s" % (self.name, error, wf.tim_id))
                                self._job.need_reconnect = wf.error == EC_NEEDRECONNECT
                            elif wf.error == EC_RESUBINCHECKBACK:
                                error += _("Die Wiedervorlage befindet sich in Rückfrage\n")
                                self.log.error("%s %s timid = %s" % (self.name, error, wf.tim_id))

                            # wenn wf.error == EC_WORKFLOWISDONE muss nichts getan werden, die WVL wurde schon fertig gesetzt
                        else:
                            if wf.error == EC_RETURNTOSENDERERROR or wf.error == EC_NEEDRECONNECT:
                                error += _("Die Wiedervorlage konnte nicht an den Absender zurückgegeben werden\n")
                                self.log.error("%s %s timid = %s" % (self.name, error, wf.tim_id))
                                self._job.need_reconnect = wf.error == EC_NEEDRECONNECT

                            hist = None
                            if wf.result:   # wvl komplett fertig? dann wfend datei
                                if not self.write_wfendfile(wf, 'cancel' in param):
                                    error += _("Fehler bei Übergabe an die Workflowprüfung\n")

                                hist = bitfarm_doc.BitfarmHistory()
                                hist.gdocid = wf.gdocid
                                hist.userid = self._job.userid
                                if 'cancel' in param:
                                    hist.taskid = 405
                                else:
                                    hist.taskid = 406
                                hist.value1 = wf.info if wf.info is not None else ''
                            else:
                                # JE WVL? dann wurde eine einzelne WVL fertig gesetzt
                                hist = bitfarm_doc.BitfarmHistory()
                                hist.gdocid = wf.gdocid
                                hist.userid = self._job.userid
                                if wf.options & 1 == 1:
                                    hist.taskid = 416
                                else:
                                    hist.taskid = 417
                                hist.value1 = wf.info if wf.info is not None else ''
                                wfoptions = []
                                if wf.options & 2 == 2:
                                    wfoptions.append(bitfarm_doc.RESUB_OPTIONS[2])
                                if wf.options & 4 == 4:
                                    wfoptions.append(bitfarm_doc.RESUB_OPTIONS[4])
                                if wf.options & 64 == 64:
                                    wfoptions.append(bitfarm_doc.RESUB_OPTIONS[64])
                                if wfoptions:
                                    hist.value2 = ", ".join(wfoptions)
                                else:
                                    hist.value2 = 'keine'
                            if hist:
                                self.log.debug("%s Historieeintrag für Wiedervorlage schreiben..." % self.name)
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                    if 'forward' in param or 'change' in param or 'create' in param:
                        if wf.info and wf.info != curwf.info and 'create' not in param:   # bei create wurde die info schon geschrieben
                            hist = bitfarm_doc.BitfarmHistory()
                            hist.gdocid = wf.gdocid
                            hist.userid = self._job.userid
                            hist.taskid = 408
                            hist.value1 = curwf.info
                            hist.value2 = wf.info if wf.info is not None else ''
                            self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)
                        if wf.comment and wf.comment != curwf.comment:
                            hist = bitfarm_doc.BitfarmHistory()
                            hist.gdocid = wf.gdocid
                            hist.userid = self._job.userid
                            if 'create' in param:
                                hist.taskid = 413
                            else:
                                hist.taskid = 409
                            hist.value1 = wf.info if wf.info is not None else ''
                            hist.value2 = wf.comment
                            self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)
                        if wf.status and wf.status != curwf.status:
                            hist = bitfarm_doc.BitfarmHistory()
                            hist.gdocid = wf.gdocid
                            hist.userid = self._job.userid
                            hist.taskid = 410
                            hist.value1 = wf.info if wf.info is not None else ''
                            hist.value2 = _("in Arbeit") if wf.status == 2 else _("offen")
                            self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)
                        if wf.date and wf.date != curwf.date:
                            hist = bitfarm_doc.BitfarmHistory()
                            hist.gdocid = wf.gdocid
                            hist.userid = self._job.userid
                            if 'create' in param:
                                hist.taskid = 414
                            else:
                                hist.taskid = 411
                            hist.value1 = wf.info if wf.info is not None else ''
                            hist.value2 = wf.date
                            hist.value3 = wf.alarm_date
                            self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)
                        if wf.options is not None and wf.options != curwf.options:
                            hist = bitfarm_doc.BitfarmHistory()
                            hist.gdocid = wf.gdocid
                            hist.userid = self._job.userid
                            hist.taskid = 415
                            hist.value1 = wf.info if wf.info is not None else ''
                            wfoptions = []
                            if wf.groups is not None or len(recipients) > 1:
                                if wf.options & 1 == 0:
                                    wfoptions.append(bitfarm_doc.RESUB_OPTIONS[0])
                                else:
                                    wfoptions.append(bitfarm_doc.RESUB_OPTIONS[1])
                            if wf.options & 2 == 2:
                                wfoptions.append(bitfarm_doc.RESUB_OPTIONS[2])
                            if wf.options & 4 == 4:
                                wfoptions.append(bitfarm_doc.RESUB_OPTIONS[4])
                            if wf.options & 64 == 64:
                                wfoptions.append(bitfarm_doc.RESUB_OPTIONS[64])
                            if wfoptions:
                                hist.value2 = ", ".join(wfoptions)
                            self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                    # prüfen, ob das Dokument noch auf Wiedervorlage liegt und entsprechend den Status 1 setzen
                    self.log.debug(_("%s Suche aktive Wiedervorlage...") % self.name)
                    sql = 'select count(*) from sys_timer where gdocid = %s and status & 4 = 0' % wf.gdocid
                    try:
                        result = self.db[self._job.profile].execute(sqla.text(sql)).fetchone()
                    except:
                        self.log.exception(_("%s Fehler beim Prüfen auf Vorhandensein in der Wiedervorlage [%s]") % (self.name, wf.gdocid))
                        error += _("Fehler beim Setzen des Status 'Auf Wiedervorlage' (1)\n")
                    else:
                        if result:
                            if int(result[0]) > 0:
                                sql = 'update sys_doc set docstate_0 = docstate_0 | 1 where gdocid = %s' % wf.gdocid
                            else:
                                sql = 'update sys_doc set docstate_0 = docstate_0 & ~1 where gdocid = %s' % wf.gdocid
                            try:
                                self.db[self._job.profile].execute(sqla.text(sql))
                            except:
                                self.log.exception(_("%s Fehler beim Setzen des Status 'auf Wiedervorlage' [%s]") % (self.name, wf.gdocid))
                                error += _("Fehler beim Setzen des Status 'auf Wiedervorlage' (2)\n")
                            else:
                                self.log.debug(_("%s Status 'auf Wiedervorlage' zurückgesetzt") % self.name)

        if "asjson" in param:
            asjsonparam = "json"
        else:
            asjsonparam = None
        if error == "":
            xml = bitfarm_xml.create_xml_request(reqtype="respOK", gdocid=self._job.gdocid, parameter=asjsonparam)
        else:
            xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=self._job.gdocid, data=error,
                                                 parameter=asjsonparam)
        try:
            self._job.sock.send(xml)
            if self.bfaserver.develop:
                self.log.debug("%s %s" % (self.name, xml))
                with open('respSaveResub_%i.xml' % self._job.gdocid, 'wb') as fn:
                    fn.write(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)

    def write_wfendfile(self, wf, cancel=False):
        self.log.debug('%s write_wfendfile cancel = %s' % (self.name, cancel))
        con = self.bfaserver.con.connections[wf.profile]

        doc = bitfarm_doc.BitfarmDocument()
        doc.profile = self._job.profile

        if not self.bfaProcessing.load(doc, wf.gdocid, ['base', 'paths', 'fields'], logid=self.name):
            self.log.error("Das Dokument %s konnte nicht geladen werden." % wf.gdocid)
            return False

        try:
            fname = datetime.datetime.now().strftime('%d%m%Y%H%M%S%f') + self.name[7:] + '.wfend'
            while os.path.exists(os.path.join(con.uncclientspooler, fname)):
                fname = datetime.datetime.now().strftime('%d%m%Y%H%M%S%f') + self.name[7:] + '.wfend'
        except:
            self.log.exception("Es konnte kein Name für die Wfend-Datei gefunden werden (UNCclientspoolerpfad erreichbar?)")
            return False

        wfend = []
        try:
            wfend.append("%s.%s" % (doc.arcid, doc.gdocid))
            wfend.append("%s" % wf.tim_ref)
            wfend.append("Archivname=%s" % doc.archive)
            wfend.append("Dokumenttitel:=%s" % doc.title)
            wfend.append("DocLoc:=%s" % doc.docloc)
            wfend.append("userid:=%s" % wf.userid)
            wfend.append("Zusatzfelder=%i" % len(doc.fields))
            for field in list(doc.fields.values()):
                if field.type in [0, 1, 2, 3]:
                    if field.value is not None:
                        wfend.append("%s:=%s" % (field.name, field.value))
                    else:
                        wfend.append("%s:=" % field.name)
            wfend.append("Zusatzstatus=%i" % (len(doc.statefields) + 2))
            wfend.append("Ausgecheckt=%i" % (1 if doc.is_checkout else 0))
            wfend.append("in Bearbeitung=%i" % (1 if doc.states[0] & 1 == 1 else 0))
            for field in list(doc.statefields.values()):
                wfend.append("%s=%i" % (field.name, (field.bitmask if field.value else 0)))

            if cancel:
                wfend.append("")
                wfend.append("Abbruch")
                wfend.append("")
        except:
            self.log.exception(_("%s Fehler beim Erzeugen der WFEnd-Datei %s") % (self.name, fname))
            return False

        try:
            wfend = "\n".join(wfend)
            # wfend = wfend.encode('cp1252')

            with open(os.path.join(con.uncclientspooler, fname), 'w') as f:
                f.write(wfend)
        except:
            self.log.exception(_("%s Fehler beim Schreiben der WFEnd-Datei %s") % (self.name, fname))
            return False
        else:
            self.log.debug('%s write_wfendfile done...' % (self.name))
            return True

    def reqDeleteResub(self):
        self.log.debug('%s reqDeleteresub' % self.name)

        if type(self._job.parameter) is str:
            timrefs = self._job.parameter.split(';')
        else:
            timrefs = self._job.parameter

        if 'admin' not in self.bfaserver.user[self._job.profile][self._job.userid].flags:
            xml = bitfarm_xml.create_xml_request(reqtype="respDenied")
        else:
            for timref in timrefs:
                userids = None
                groupids = None
                refid, gdocid = timref.split(',')

                userids = None
                sql = 'select group_concat(distinct(userid_work)) from sys_timer where timref = %s and groupid = 0' % refid
                try:
                    result = self.db[self._job.profile].execute(sqla.text(sql)).fetchone()
                except:
                    self.log.exception(_("%s Fehler beim Löschen einer Wiedervorlage (Admin)") % self.name)
                    xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=gdocid,
                                                         data=_("Fehler beim Löschen einer Wiedervorlage (Admin)"))
                else:
                    if result:
                        userids = result[0]

                groupids = None
                sql = 'select group_concat(distinct(groupid)) from sys_timer where timref = %s and groupid <> 0' % refid
                try:
                    result = self.db[self._job.profile].execute(sqla.text(sql)).fetchone()
                except:
                    self.log.exception(_("%s Fehler beim Löschen einer Wiedervorlage (Admin)") %(self.name))
                    xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=gdocid,
                                                         data=_("Fehler beim Löschen einer Wiedervorlage (Admin)"))
                else:
                    if result:
                        groupids = result[0]

                sql = 'delete from sys_timer where timref = %s' % refid
                try:
                    self.db[self._job.profile].execute(sqla.text(sql))
                except:
                    self.log.exception(_("%s Fehler beim Löschen einer Wiedervorlage (Admin)") %(self.name))
                    xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=gdocid,
                                                         data=_("Fehler beim Löschen einer Wiedervorlage (Admin)"))
                else:
                    sql = 'update sys_doc set docstate_0 = docstate_0 & ~1 where gdocid = %s' % gdocid
                    try:
                        self.db[self._job.profile].execute(sqla.text(sql))
                    except:
                        self.log.exception(_("%s Fehler beim Löschen einer Wiedervorlage (Admin)") % self.name)
                        xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=gdocid, data=_("Fehler beim Zurücksetzen des Status 'auf Wiedervorlage' (Admin)"))
                    else:
                        xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("reqDeleteResub ausgeführt"))

                        hist = bitfarm_doc.BitfarmHistory()
                        hist.userid = self._job.userid
                        hist.gdocid = 0     #Eintrag im Admin-Log
                        hist.taskid = 103
                        hist.value1 = refid
                        hist.value2 = userids if userids is not None else ''
                        hist.value3 = groupids if groupids is not None else ''
                        self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                        # noch ein Eintrag beim Dokument
                        hist = bitfarm_doc.BitfarmHistory()
                        hist.userid = self._job.userid
                        hist.gdocid = gdocid
                        hist.taskid = 103
                        hist.value1 = refid
                        hist.value2 = userids if userids is not None else ''
                        hist.value3 = groupids if groupids is not None else ''
                        self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
        return None

    def reqSavePreview(self):
        self.log.debug('%s reqSavePreview' % self.name)
        doc = bitfarm_doc.BitfarmDocument()
        doc.profile = self._job.profile
        con = self.bfaserver.con.connections[self._job.profile]
        xml = ""

        if not self.bfaProcessing.load(doc, self._job.gdocid, ['base', 'paths'], logid=self.name):
            xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=self._job.gdocid)
        else:
            fext = self._job.datatype if self._job.datatype else os.path.splitext(doc.docloc)[1]
            if doc.extloc:
                fpath = doc.extloc
            else:
                doc.tempname = doc.docloc.lower().replace(con.archive_rs.lower(), con.archive_nrs.lower())
                if os.path.exists(os.path.expandvars(doc.tempname + fext)): #sollte es den pfad schon geben, dann einen neuen erzeugen
                    self.bfaProcessing.create_tempname(doc)
                doc.extloc = os.path.splitext(doc.tempname)[0] + fext
                fpath = doc.extloc

            if not os.path.exists(os.path.dirname(os.path.expandvars(fpath))):
                try:
                    os.makedirs(os.path.dirname(os.path.expandvars(fpath)))
                except:
                    self.log.exception(_("%s Konnte neues Verzeichnis nicht anlegen") % (self.name,os.path.dirname(os.path.expandvars(fpath))))
                    xml = bitfarm_xml.create_xml_request(reqtype="respError",
                                                         data="Fehler beim Speichern des Dokuments.")

            if not xml:     # keine Fehler bis hierhin
                xml = bitfarm_xml.create_xml_request(reqtype="respSendFile", gdocid=self._job.gdocid)
                try:
                    self._job.sock.send(xml)
                except:
                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                    return None
                try:
                    self.log.debug(_("%s Schreibe Datei %s") % (self.name, fpath))
                    with open(os.path.expandvars(fpath), 'wb') as f:
                        received = 0
                        while received < self._job.datalen:
                            buff = self._job.sock.recv(1024)
                            received += len(buff)
                            if not buff:
                                break
                            f.write(buff)

                    doc.modified_userid = self._job.userid

                    if received != self._job.datalen:
                        self.log.error(_('%s Es wurden nicht alle Daten vom Client empfangen, Datei wird nicht gespeichert.') % self.name)
                        self.log.error(_('%s datalen: %d  received: %d') % (self.name, self._job.datalen, received))
                        xml = bitfarm_xml.create_xml_request(reqtype="respError", data=_('Fehler beim Speichern des Dokuments.'))
                        if os.path.exists(os.path.expandvars(fpath)):
                            try:
                                os.remove(os.path.expandvars(fpath))
                            except:
                                self.log.warning(_('%s Datei %s konnte nicht gelöscht werden') % (self.name, fpath))
                    elif not self.bfaProcessing.write(doc, logid=self.name):
                        xml = bitfarm_xml.create_xml_request(reqtype="respError", data=_('Fehler beim Speichern des Dokuments.'))
                    else:
                        hist = bitfarm_doc.BitfarmHistory()
                        hist.gdocid = self._job.gdocid
                        hist.taskid = 223
                        hist.userid = self._job.userid
                        self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                        # BitfarmWorkerThread.in_progresslock.acquire()
                        # try:
                        #     if self._job.gdocid in self.bfaserver.in_progress[self._job.profile]:
                        #         del self.bfaserver.in_progress[self._job.profile][self._job.gdocid]
                        # finally:
                        #     BitfarmWorkerThread.in_progresslock.release()
                        # self.bfaProcessing.unset_editmode(self._job, self._job.userid, docids=self._job.gdocid)

                        if (self._job.gdocid, fpath) in self.bfaserver.cached:
                            BitfarmWorkerThread.cachelock.acquire()
                            try:
                                self.bfaserver.cache[(self._job.gdocid, fpath)].close()
                                del self.bfaserver.cache[(self._job.gdocid, fpath)]
                                self.bfaserver.cached.remove((self._job.gdocid, fpath))
                            finally:
                                BitfarmWorkerThread.cachelock.release()

                        xml = bitfarm_xml.create_xml_request(reqtype="respOK", gdocid=self._job.gdocid, data=_('reqSavePreview ausgeführt'))
                except:
                    self.log.exception(_('%s Die Datei %s konnte nicht geschrieben werden') % (self.name, fpath))
                    xml = bitfarm_xml.create_xml_request(reqtype="respError", data=_('Fehler beim Speichern der Voransicht'))
        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqLastModified(self):
        self.log.debug('%s reqLastModified' % self.name)

        if self._job.gdocid:
            sql = 'select modified, modified_userid from sys_doc where gdocid = %s' % self._job.gdocid

            try:
                result = self.db[self._job.profile].execute(sqla.text(sql)).fetchone()
            except Exception as e:
                self.log.exception(_("%s Fehler bei der Abfrage der sys_doc: %s") %(self.name, e))
                xml = bitfarm_xml.create_xml_request(reqtype="respError", data="Fehler bei der Abfrage der sys_doc")
            else:
                if result:
                    req = bitfarm_doc.bfaReq()
                    req.reqtype = 'respLastModified'
                    req.userid = self._job.userid
                    req.gdocid = self._job.gdocid
                    req.modified_date = result[0]
                    req.modified_userid = result[1]
                    xml = bitfarm_xml.object_to_xml(req, shortxml=True)
                    xml = xml.encode()
                else:
                    xml = bitfarm_xml.create_xml_request(reqtype="respError",
                                                         data="Kein Dokument für die Gdocid %s" % self._job.gdocid)
        else:
            xml = bitfarm_xml.create_xml_request(reqtype="respError", data="Keine Gdocid übergeben")

        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
        return None

    def reqSaveAdmin(self):
        self.log.debug('%s reqSaveAdmin' % self.name)
        _con = self.bfaserver.con.connections[self._job.profile]
        guids = {}
        repos = {}
        error = []
        recalc_arcids = []
        changed_dlmfields = []
        if type(self._job.data) is not list:
            data = [self._job.data]
        else:
            data = self._job.data

        if self.bfaserver.develop:
            with open('reqSaveAdmin.xml', 'w') as fn:
                bitfarm_xml.object_to_xmlfile(self._job, fn)

        # xml = bitfarm_xml.create_xml_request(reqtype="respOK", gdocid=self._job.gdocid, data=_("reqSaveAdmin ausgeführt"))

        for elem in data:
            self.log.debug("%s element %s" % (self.name, elem))
            if elem.flags is None:
                elem.flags = 0
            if elem.flags & 2 == 2:     # ist das element deleted? dann kann es nicht gleichzeitig enabled sein
                elem.flags = elem.flags & ~1

            if isinstance(elem, bitfarm_doc.BitfarmUser):
                if not elem.modified:
                    continue
                self.log.debug(_('%s Speichere User %s') % (self.name, elem.name))

                sql = "select userid, username, userpass, usermail, flags +0, displayname, role +0 from sys_user where userid = %i" % elem.id
                try:
                    current = self.db[self._job.profile].execute(sql).fetchone()
                except:
                    msg = _("Fehler bei der Abfrage der Benutzertabelle")
                    self.log.exception("%s %s" % (self.name, msg))
                    error.append(msg)
                else:
                    if elem.displayname is None:
                        elem.displayname = elem.name

                    # wir prüfen, ob der username schon anderweitig vergeben wurde, aber nur wenn der Benutzer nicht sowieso gelöscht werden soll
                    if elem.flags & 6 == 0:         # bzw. der User kein AD Benutzer werden soll (reaktivierte User aus dem AD haben ja zwangsläufig den gleichen Namen)
                        sql = "select userid, flags +0 from sys_user where userid <> %i and username = '%s'" % (elem.id, elem.name)
                        try:
                            res = self.db[self._job.profile].execute(sql).fetchall()
                        except:
                            msg = _("Fehler bei der Abfrage der Benutzertabelle (3)")
                            self.log.exception("%s %s" % (self.name, msg))
                            error.append(msg)
                        else:
                            if res:     # noch ein user mit diesem namen, das darf ja wohl nicht wahr sein
                                msg = _("Benutzernamen müssen eindeutig sein. Benutzername %s ist schon vergeben") % elem.name
                                self.log.error("%s %s" % (self.name, msg))
                                error.append(msg)
                                continue

                    if not current and elem.flags & 4 == 4:     # haben wir einen neuen user aus dem ad prüfen wir, ob es den nicht schonmal einen gab
                        sql = "select userid, username, userpass, usermail, flags +0, displayname, role +0 from sys_user where username = '%s' and flags & 6 = 6 order by userid" % elem.name
                        try:
                            current = self.db[self._job.profile].execute(sql).fetchone()
                        except:
                            msg = _("Fehler bei der Abfrage der Benutzertabelle (2)")
                            self.log.exception("%s %s" % (self.name, msg))
                            error.append(msg)
                        else:
                            if current:
                                elem.id = current['userid']
                                if elem.tempid is not None:
                                    guids[elem.tempid] = elem.id

                    if current:
                        vals = {_con.sys_user.c.username: elem.name,
                                _con.sys_user.c.userpass: elem.password,
                                _con.sys_user.c.usermail: elem.email,
                                _con.sys_user.c.flags: elem.flags,
                                _con.sys_user.c.displayname: elem.displayname,
                                _con.sys_user.c.role: elem.role
                                }
                        sql = sqla.update(_con.sys_user, values=vals).where(_con.sys_user.c.userid == elem.id)
                    else:
                        sql = "select max(userid) from sys_user"
                        maxid = self.db[self._job.profile].execute(sql).fetchone()
                        if elem.id <= maxid[0]:
                            elem.id = int(maxid[0] + 1)
                        if elem.tempid is not None:
                            guids[elem.tempid] = elem.id

                        vals = {_con.sys_user.c.userid: elem.id,
                                _con.sys_user.c.username: elem.name,
                                _con.sys_user.c.userpass: elem.password,
                                _con.sys_user.c.usermail: elem.email,
                                _con.sys_user.c.flags: elem.flags,
                                _con.sys_user.c.displayname: elem.displayname,
                                _con.sys_user.c.role: elem.role
                                }
                        sql = sqla.insert(_con.sys_user, values=vals)

                    try:
                        self.db[self._job.profile].execute(sql)
                    except:
                        msg = _("Fehler beim Speichern des Benutzers %s") % elem.name
                        self.log.exception("%s %s" % (self.name, msg))
                        error.append(msg)
                    else:
                        hist = bitfarm_doc.BitfarmHistory()
                        hist.gdocid = 0  # 0 = Admin Einträge
                        hist.userid = self._job.userid
                        hist.value1 = str(elem.id)

                        if current:  # wir haben einen alten datensatz, dann abgleichen und history schreiben
                            current = [x if x is not None else 0 for x in
                                       [int(x) if type(x) is float else x for x in current]]
                            if elem.name is not None and current[1] != elem.name:
                                hist.taskid = 24
                                hist.value2 = current[1]
                                hist.value3 = elem.name
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            if elem.password is not None and current[2] != elem.password:
                                hist.taskid = 28
                                hist.value2 = None
                                hist.value3 = None
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                                if _con.bfauser is not None:
                                    if _con.bfauser.decode().lower() == elem.name.lower():
                                        password = _con.bitdecrypt(elem.password)
                                        _con.bfapass = password.encode('utf8')  # bfauser und bfapass sind bytes
                                        if os.path.exists(_con.confile):
                                            ini = bitfarm_configobj.ConfigObj(infile=_con.confile, encoding="cp1252", write_empty_values=True)
                                            if 'client' in ini:
                                                ini['client']['bfapass'] = base64.b64encode(dpapi.cryptData(_con.bfapass)).decode()
                                                ini.write()

                            if elem.email is not None and current[3] != elem.email:
                                hist.taskid = 26
                                hist.value2 = current[3]
                                hist.value3 = elem.email
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            if int(current[4]) != elem.flags:
                                flags_current = ",".join(
                                    [y for x, y in list(bitfarm_doc.FLAGS.items()) if int(current[4]) & x == x])
                                flags = ",".join([y for x, y in list(bitfarm_doc.FLAGS.items()) if elem.flags & x == x])
                                hist.taskid = 29
                                hist.value2 = flags_current
                                hist.value3 = flags
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            if elem.displayname is not None and current[5] != elem.displayname:
                                hist.taskid = 25
                                hist.value2 = current[5]
                                hist.value3 = elem.displayname
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            if int(current[6]) != elem.role:
                                hist.taskid = 27
                                if int(current[6]) in bitfarm_doc.RESOLVE_ROLE:
                                    hist.value2 = bitfarm_doc.RESOLVE_ROLE[current[6]]
                                else:
                                    hist.value2 = str(int(current[6]))

                                if elem.role in bitfarm_doc.RESOLVE_ROLE:
                                    hist.value3 = bitfarm_doc.RESOLVE_ROLE[elem.role]
                                else:
                                    hist.value3 = str(elem.role)
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)
                        else:
                            if elem.name is not None and elem.name != elem.displayname:
                                hist.taskid = 30
                                hist.value2 = elem.name
                                hist.value3 = elem.displayname
                            else:
                                hist.taskid = 22
                                hist.value2 = elem.name
                                hist.value3 = None
                            self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            if elem.email:
                                hist.taskid = 31
                                hist.value2 = elem.email
                                hist.value3 = None
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            if elem.password:
                                hist.taskid = 34
                                hist.value2 = None
                                hist.value3 = None
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            if elem.flags:
                                flags = ",".join([y for x, y in list(bitfarm_doc.FLAGS.items()) if elem.flags & x == x])
                                hist.taskid = 32
                                hist.value2 = flags
                                hist.value3 = None
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            if elem.role:
                                hist.taskid = 33
                                if elem.role in bitfarm_doc.RESOLVE_ROLE:
                                    hist.value2 = bitfarm_doc.RESOLVE_ROLE[elem.role]
                                else:
                                    hist.value2 = str(elem.role)
                                hist.value3 = None
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)
                        if not current:     # der benutzer ist neu angelegt worden. dann einen eintrag in die sys_groupuser für gruppe 0
                            sql = 'insert into sys_groupuser (groupid, userid) values (0,%s)' % elem.id
                            try:
                                self.db[self._job.profile].execute(sql)
                            except:
                                msg = _("%s Fehler beim Speichern des Users %s in die Gruppe 0") % (self.name, elem.name)
                                self.log.exception(msg)

                # Mitarbeiter speichern
                if elem.employees is not None and len(elem.employees) > 0:
                    if type(elem.employees) in [str, str]:
                        employees = elem.employees.split(';')
                    employees = set([int(x) for x in employees])

                    sql = 'select group_concat(inf_userid) from sys_superior where sup_userid = %i' % elem.id
                    try:
                        result = self.db[self._job.profile].execute(sql).fetchone()
                    except:
                        msg = _("Fehler bei der Abfrage der Vorgesetztenliste")
                        self.log.exception("%s %s" % (self.name, msg))
                        error.append(msg)
                    else:
                        if result and result[0] is not None:
                            current_inf = result[0].split(',')
                            current_inf = set([int(x) for x in current_inf])
                        else:
                            current_inf = set()

                        new_user = set()
                        new_user = employees - current_inf
                        del_user = set()
                        del_user = current_inf - employees

                        if len(new_user) > 0:
                            vals = ['(%s,%s)' % (elem.id, x) for x in new_user]
                            vals = ",".join(vals)
                            sql = 'insert into sys_superior (sup_userid, inf_userid) values %s' % vals

                            try:
                                self.db[self._job.profile].execute(sqla.text(sql))
                            except:
                                msg = _("Fehler beim Anlegen neuer Benutzer in der Vorgesetztenliste")
                                self.log.exception("%s %s" % (self.name, msg))
                                error.append(msg)
                            else:
                                hist = bitfarm_doc.BitfarmHistory()
                                hist.gdocid = 0  # 0 = Admin Einträge
                                hist.userid = self._job.userid
                                hist.taskid = 101
                                hist.value1 = elem.name
                                hist.value2 = ",".join([str(x) for x in new_user])
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                        if len(del_user) > 0:
                            vals = [str(x) for x in del_user]
                            vals = ",".join(vals)
                            sql = 'delete from sys_superior where sup_userid = %s and inf_userid in (%s)' % (elem.id, vals)

                            try:
                                self.db[self._job.profile].execute(sqla.text(sql))
                            except:
                                msg = _("Fehler beim Entfernen von Benutzern aus der Vorgesetztenliste")
                                self.log.exception("%s %s" % (self.name, msg))
                                error.append(msg)
                            else:
                                hist = bitfarm_doc.BitfarmHistory()
                                hist.gdocid = 0  # 0 = Admin Einträge
                                hist.userid = self._job.userid
                                hist.taskid = 102
                                hist.value1 = elem.name
                                hist.value2 = vals
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

            elif isinstance(elem, bitfarm_doc.BitfarmGroup):
                if not elem.modified:
                    continue
                self.log.debug(_('%s Speichere Gruppe %s') % (self.name, elem.name))
                if elem.id is None:  # kann mit None reinkommen, wenn die Builtin Gruppe ausgewählt wurde
                    elem.id = 0

                group_deleted = False

                sql = "select groupid, groupname, role +0, flags +0, adpath, email from sys_group where groupid = %i" % elem.id
                try:
                    current = self.db[self._job.profile].execute(sql).fetchone()
                except:
                    msg = _("Fehler bei der Abfrage der Gruppen")
                    self.log.exception("%s %s" % (self.name, msg))
                    error.append(msg)
                else:
                    if not current and elem.flags & 4 == 4:     # haben wir einen neuen user aus dem ad prüfen wir, ob es nicht schonmal einen gab
                        sql = "select groupid, groupname, role +0, flags +0, adpath, email from sys_group where groupname = '%s' and flags & 6 = 6 order by groupid" % elem.name
                        try:
                            current = self.db[self._job.profile].execute(sql).fetchone()
                        except:
                            msg = _("Fehler bei der Abfrage der Gruppen (2)")
                            self.log.exception("%s %s" % (self.name, msg))
                            error.append(msg)
                        else:
                            if current:
                                elem.id = current['groupid']
                                if elem.tempid is not None:
                                    guids[elem.tempid] = elem.id

                    if current:
                        vals = {_con.sys_group.c.groupname: elem.name,
                                _con.sys_group.c.role: elem.role,
                                _con.sys_group.c.flags: elem.flags,
                                _con.sys_group.c.adpath: elem.adpath,
                                _con.sys_group.c.email: elem.email
                                }
                        sql = sqla.update(_con.sys_group, values=vals).where(_con.sys_group.c.groupid == elem.id)
                    else:
                        sql = "select max(groupid) from sys_group"
                        maxid = self.db[self._job.profile].execute(sql).fetchone()
                        if elem.id <= maxid[0]:
                            elem.id = int(maxid[0] + 1)
                        if elem.tempid is not None:
                            guids[elem.tempid] = elem.id

                        vals = {_con.sys_group.c.groupid: elem.id,
                                _con.sys_group.c.groupname: elem.name,
                                _con.sys_group.c.role: elem.role,
                                _con.sys_group.c.flags: elem.flags,
                                _con.sys_group.c.adpath: elem.adpath,
                                _con.sys_group.c.email: elem.email
                                }
                        sql = sqla.insert(_con.sys_group, values=vals)

                    try:
                        self.db[self._job.profile].execute(sql)
                    except:
                        msg = _("Fehler bem Speichern der Gruppe %s" % elem.name)
                        self.log.exception("%s %s" % (self.name, msg))
                        error.append(msg)
                    else:
                        hist = bitfarm_doc.BitfarmHistory()
                        hist.gdocid = 0  # 0 = Admin Einträge
                        hist.userid = self._job.userid
                        hist.value1 = str(elem.id)

                        if current:  # wir haben einen alten datensatz, dann abgleichen und history schreiben
                            current = [x if x is not None else 0 for x in
                                       [int(x) if type(x) is float else x for x in current]]
                            if elem.name is not None and current[1] != elem.name:
                                hist.taskid = 39
                                hist.value2 = current[1]
                                hist.value3 = elem.name
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            if current[2] != elem.role:
                                hist.taskid = 40
                                if current[2] in bitfarm_doc.RESOLVE_ROLE:
                                    hist.value2 = bitfarm_doc.RESOLVE_ROLE[current[2]]
                                else:
                                    hist.value2 = str(current[2])
                                if elem.role in bitfarm_doc.RESOLVE_ROLE:
                                    hist.value3 = bitfarm_doc.RESOLVE_ROLE[elem.role]
                                else:
                                    hist.value3 = str(elem.role)

                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            if current[3] != elem.flags:
                                current_flags = ",".join([y for x, y in list(bitfarm_doc.FLAGS_GROUP.items()) if current[3] & x == x])
                                flags = ",".join([y for x, y in list(bitfarm_doc.FLAGS_GROUP.items()) if elem.flags & x == x])
                                hist.taskid = 41
                                hist.value2 = current_flags
                                hist.value3 = flags
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)
                                group_deleted = elem.flags & 2 == 2     # wurde die gruppe gelöscht?

                            if elem.adpath is not None and current[4] != elem.adpath:
                                hist.taskid = 42
                                hist.value2 = current[4]
                                hist.value3 = elem.adpath
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            if elem.email is not None and current[5] != elem.email:
                                hist.taskid = 105
                                hist.value2 = current[5] if current[5] != 0 else ''
                                hist.value3 = elem.email
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                        else:
                            if elem.name:
                                hist.taskid = 35
                                hist.value2 = elem.name
                                hist.value3 = None
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            if elem.role:
                                hist.taskid = 36
                                if elem.role in bitfarm_doc.RESOLVE_ROLE:
                                    hist.value2 = bitfarm_doc.RESOLVE_ROLE[elem.role]
                                else:
                                    hist.value2 = str(elem.role)
                                hist.value3 = None
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            if elem.flags:
                                flags = ",".join([y for x, y in list(bitfarm_doc.FLAGS_GROUP.items()) if elem.flags & x == x])
                                hist.taskid = 37
                                hist.value2 = flags
                                hist.value3 = None
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            if elem.adpath:
                                hist.taskid = 38
                                hist.value2 = elem.adpath
                                hist.value3 = None
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            if elem.email:
                                hist.taskid = 104
                                hist.value2 = elem.email
                                hist.value3 = None
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                        # wir lassen diese rechte erstmal stehen, sollte ein versehentlich gelöschter user wieder aktiviert werden haben wir alle seine berechtigungen
                        # if group_deleted:   # Gruppe gelöscht? aus sys_arcxs, sys_docxs, sys_addfieldxs und sys_statefieldxs löschen
                        #     for xs in ['sys_arcxs', 'sys_docxs', 'sys_addfieldxs', 'sys_statefieldxs']:
                        #         sql = u"delete from %s where userid = 0 and groupid = %s" % (xs, elem.id)
                        #         try:
                        #             self.db[self._job.profile].execute(sql)
                        #         except:
                        #             msg = _(u"Fehler bei der Löschen der Berechtigung einer Gruppe")
                        #             self.log.exception(u"%s %s in %s" % (self.name, msg, xs))
                        #             error.append(msg)

                # Gruppenzugehörigkeit speichern
                if elem.members is not None:
                    if len(elem.members) > 0:
                        if type(elem.members) in [str, str]:
                            members = elem.members.split(';')
                        for i in range(len(members)):
                            if members[i][0] == '{' and members[i][-1] == '}':  #temporäre guid?
                                if members[i] in guids:
                                    members[i] = guids[members[i]]
                            try:
                                members[i] = int(members[i])
                            except:
                                members[i] = None

                        members = set([int(x) for x in members if x is not None])

                        sql = 'select group_concat(userid) from sys_groupuser where groupid = %i' % elem.id
                        try:
                            result = self.db[self._job.profile].execute(sql).fetchone()
                        except:
                            msg = _("Fehler bei der Abfrage der Gruppen")
                            self.log.exception("%s %s" % (self.name, msg))
                            error.append(msg)
                        else:
                            new_user = set()
                            del_user = set()
                            if result and result[0] is not None:
                                current_members = result[0].split(',')
                                current_members = set([int(x) for x in current_members])

                                new_user = members - current_members
                                del_user = current_members - members
                            else:
                                new_user = members

                            if len(new_user) > 0:
                                vals = ['(%s,%s)' % (elem.id, x) for x in new_user]
                                vals = ",".join(vals)
                                sql = 'insert into sys_groupuser (groupid, userid) values %s' % vals

                                try:
                                    self.db[self._job.profile].execute(sqla.text(sql))
                                except:
                                    msg = _("Fehler beim Zuordnen eines Benutzers zu einer Gruppe")
                                    self.log.exception("%s %s" % (self.name, msg))
                                    error.append(msg)
                                else:
                                    hist = bitfarm_doc.BitfarmHistory()
                                    hist.gdocid = 0  # 0 = Admin Einträge
                                    hist.userid = self._job.userid
                                    hist.taskid = 99
                                    hist.value1 = elem.name
                                    hist.value2 = ",".join([str(x) for x in new_user])
                                    self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            if len(del_user) > 0:
                                vals = [str(x) for x in del_user]
                                vals = ",".join(vals)
                                sql = 'delete from sys_groupuser where groupid = %s and userid in (%s)' % (elem.id, vals)

                                try:
                                    self.db[self._job.profile].execute(sqla.text(sql))
                                except:
                                    msg = _("Fehler beim Entfernen eines Benutzers aus einer Gruppe")
                                    self.log.exception("%s %s" % (self.name, msg))
                                    error.append(msg)
                                else:
                                    hist = bitfarm_doc.BitfarmHistory()
                                    hist.gdocid = 0  # 0 = Admin Einträge
                                    hist.userid = self._job.userid
                                    hist.taskid = 100
                                    hist.value1 = elem.name
                                    hist.value2 = vals
                                    self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                    elif len(elem.members) == 0:
                        sql = 'delete from sys_groupuser where groupid = %s' % (elem.id)

                        try:
                            self.db[self._job.profile].execute(sqla.text(sql))
                        except:
                            msg = _("Fehler beim Entfernen aller Benutzers aus einer Gruppe")
                            self.log.exception("%s %s" % (self.name, msg))
                            error.append(msg)
                        else:
                            hist = bitfarm_doc.BitfarmHistory()
                            hist.gdocid = 0  # 0 = Admin Einträge
                            hist.userid = self._job.userid
                            hist.taskid = 133
                            hist.value1 = elem.name
                            self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

            elif isinstance(elem, bitfarm_doc.BitfarmArchive):
                if elem.modified:
                    self.log.debug(_('%s Speichere Archiv %s') % (self.name, elem.arcname))

                    if elem.id == elem.lgrid:   # ein Knoten kann nicht dieselbe ID im Parent haben
                        msg = _("Für das Archiv [%s] sind inkonsistente Daten geliefert worden. Speichern nicht möglich." % elem.id)
                        self.log.error("%s %s arcid = lgrid" % (self.name, msg))
                        error.append(msg)
                    else:
                        sql = "select arcid, lgrid, arcname, flags +0, backpic, options, defdocname, duration,`interval` from sys_arc where arcid = %i" % elem.id
                        try:
                            current = self.db[self._job.profile].execute(sql).fetchone()
                        except:
                            msg = _("Fehler bei der Abfrage der Archivtabelle")
                            self.log.exception("%s %s" % (self.name, msg))
                            error.append(msg)
                        else:
                            if current:
                                vals = {
                                    _con.sys_arc.c.lgrid: elem.lgrid,
                                    _con.sys_arc.c.arcname: elem.arcname,
                                    _con.sys_arc.c.flags: elem.flags,
                                    _con.sys_arc.c.backpic: elem.backpic,
                                    _con.sys_arc.c.options: elem.options,
                                    _con.sys_arc.c.defdocname: elem.defdocname
                                }
                                if elem.duration is not None:
                                    if elem.duration == 0:
                                        vals[_con.sys_arc.c.interval] = None
                                    vals[_con.sys_arc.c.duration] = elem.duration
                                if elem.interval is not None:
                                    vals[_con.sys_arc.c.interval] = elem.interval

                                sql = sqla.update(_con.sys_arc, values=vals).where(_con.sys_arc.c.arcid == elem.id)
                            else:
                                sql = "select max(arcid) from sys_arc"
                                maxid = self.db[self._job.profile].execute(sql).fetchone()
                                if elem.id <= maxid[0]:
                                    if elem.type == 0:  # haben wir ein neues Lager merken wir uns die alte und die neue ID
                                        repos[elem.id] = int(maxid[0] + 1)
                                    elem.id = int(maxid[0] + 1)
                                if elem.tempid is not None:
                                    guids[elem.tempid] = elem.id

                                vals = {_con.sys_arc.c.arcid: elem.id,
                                        _con.sys_arc.c.lgrid: elem.lgrid,
                                        _con.sys_arc.c.arcname: elem.arcname,
                                        _con.sys_arc.c.tblname: elem.tblname,
                                        _con.sys_arc.c.flags: elem.flags,
                                        _con.sys_arc.c.backpic: elem.backpic,
                                        _con.sys_arc.c['type']: elem.type,
                                        _con.sys_arc.c.options: elem.options,
                                        _con.sys_arc.c.defdocname: elem.defdocname
                                        }
                                if elem.duration is not None:
                                    if elem.duration == 0:
                                        vals[_con.sys_arc.c.interval] = None
                                    vals[_con.sys_arc.c.duration] = elem.duration
                                if elem.interval is not None:
                                    vals[_con.sys_arc.c.interval] = elem.interval

                                sql = sqla.insert(_con.sys_arc, values=vals)

                            try:
                                self.db[self._job.profile].execute(sql)
                            except:
                                msg = _("Fehler beim Ändern eines Archivs [%s]" % elem.id)
                                self.log.exception("%s %s" % (self.name, msg))
                                error.append(msg)
                            else:
                                hist = bitfarm_doc.BitfarmHistory()
                                hist.gdocid = 0  # 0 = Admin Einträge
                                hist.userid = self._job.userid
                                hist.value1 = str(elem.id)

                                if current:  # wir haben einen alten datensatz, dann abgleichen und history schreiben
                                    current = [x if x is not None else 0 for x in
                                               [int(x) if type(x) is float else x for x in current]]
                                    if current[1] != elem.lgrid:
                                        hist.taskid = 67 if elem.type == 1 else 68
                                        hist.value2 = str(current[1])
                                        hist.value3 = str(elem.lgrid)
                                        self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                                    if elem.arcname is not None and current[2] != elem.arcname:
                                        hist.taskid = 65 if elem.type == 1 else 66
                                        hist.value2 = current[2]
                                        hist.value3 = elem.arcname
                                        self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                                    if current[3] != elem.flags:
                                        current_flags = ",".join([y for x, y in list(bitfarm_doc.FLAGS.items()) if current[3] & x == x])
                                        flags = ",".join([y for x, y in list(bitfarm_doc.FLAGS.items()) if elem.flags & x == x])
                                        hist.taskid = 69 if elem.type == 1 else 70
                                        hist.value2 = current_flags
                                        hist.value3 = flags
                                        self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                                    if elem.backpic is not None and current[4] != elem.backpic:
                                        hist.taskid = 71 if elem.type == 1 else 72
                                        hist.value2 = current[4]
                                        hist.value3 = elem.backpic
                                        self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                                    if current[5] != elem.options:
                                        hist.taskid = 73 if elem.type == 1 else 74
                                        if current[5] in bitfarm_doc.RESOLVE_ARC_OPTIONS:
                                            hist.value2 = bitfarm_doc.RESOLVE_ARC_OPTIONS[current[5]]
                                        else:
                                            hist.value2 = str(current[5])
                                        if elem.options in bitfarm_doc.RESOLVE_ARC_OPTIONS:
                                            hist.value3 = bitfarm_doc.RESOLVE_ARC_OPTIONS[elem.options]
                                        else:
                                            hist.value3 = str(elem.options)
                                        hist.value2 = ",".join([y for x, y in list(bitfarm_doc.RESOLVE_ARC_OPTIONS.items()) if current[5] & x == x])
                                        hist.value3 = ",".join([y for x, y in list(bitfarm_doc.RESOLVE_ARC_OPTIONS.items()) if elem.options & x == x])

                                        self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                                    if elem.defdocname is not None and current[6] != elem.defdocname:
                                        hist.taskid = 75 if elem.type == 1 else 76
                                        hist.value2 = current[6] if current[6] != 0 else 'leer'
                                        hist.value3 = elem.defdocname
                                        self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                                    if elem.duration is not None and (current[7] != elem.duration or current[8] != elem.interval):
                                        if current[7]:
                                            hist.taskid = 114
                                            hist.value2 = "%s %s" % (current[7], current[8])
                                            hist.value3 = "%s %s" % (elem.duration, elem.interval if elem.interval is not None else "")
                                            self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)
                                        else:
                                            hist.taskid = 113
                                            hist.value2 = "%s %s" % (elem.duration, elem.interval if elem.interval is not None else "")
                                            self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                                        newjob = bitfarm_doc.bfaReq()
                                        newjob.reqtype = 'set_expired_for_archive'
                                        newjob.profile = self._job.profile
                                        newjob.arcid = elem.id
                                        newjob.parameter = [elem.duration, elem.interval]
                                        self.bfaserver.requestqueue.put(newjob)
                                else:
                                    if elem.arcname:
                                        hist.taskid = 77 if elem.type == 1 else 78
                                        hist.value2 = elem.arcname
                                        hist.value3 = str(elem.lgrid)
                                        self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                                    if elem.flags:
                                        flags = ",".join([y for x, y in list(bitfarm_doc.FLAGS.items()) if elem.flags & x == x])
                                        hist.taskid = 79 if elem.type == 1 else 80
                                        hist.value2 = flags
                                        hist.value3 = None
                                        self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                                    if elem.backpic:
                                        hist.taskid = 81 if elem.type == 1 else 82
                                        hist.value2 = elem.backpic
                                        hist.value3 = None
                                        self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                                    if elem.options:
                                        hist.taskid = 83 if elem.type == 1 else 84
                                        if elem.options in bitfarm_doc.RESOLVE_ROLE:
                                            hist.value2 = bitfarm_doc.RESOLVE_ROLE[elem.options]
                                        else:
                                            hist.value2 = str(elem.options)
                                        hist.value3 = None
                                        self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                                    if elem.defdocname:
                                        hist.taskid = 85 if elem.type == 1 else 86
                                        hist.value2 = elem.defdocname
                                        hist.value3 = None
                                        self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                                    if elem.duration is not None:
                                        hist.taskid = 113
                                        hist.value2 = "%s %s" % (elem.duration, elem.interval if elem.interval is not None else "")
                                        self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                                    if elem.duration:
                                        newjob = bitfarm_doc.bfaReq()
                                        newjob.reqtype = 'set_expired_for_archive'
                                        newjob.profile = self._job.profile
                                        newjob.arcid = elem.id
                                        newjob.parameter = [elem.duration, elem.interval]
                                        self.bfaserver.requestqueue.put(newjob)

                if elem.modified_user:
                    self.log.debug(_('%s Speichere Archiv, Zuordnung und Rechte %s') % (self.name, elem.arcname))
                    for right in elem.rights_list:
                        if right.modified:
                            if right.userid is None:
                                right.userid = 0
                            if right.groupid is None:
                                right.groupid = 0
                            right_arcid = elem.id if right.arcid < 0 else right.arcid

                            if right.deleted:
                                sql = 'delete from sys_arcxs where arcid = %i and userid = %i and groupid = %i' % (
                                right_arcid, right.userid, right.groupid)
                                try:
                                    self.db[self._job.profile].execute(sqla.text(sql))
                                except:
                                    msg = _("Fehler beim Löschen eines Archiv-Berechtigungsatzes")
                                    self.log.exception("%s %s arcid %i userid %i groupid %i" % (self.name, msg, right_arcid, right.userid, right.groupid))
                                    error.append(msg)
                                else:
                                    hist = bitfarm_doc.BitfarmHistory()
                                    hist.gdocid = 0  # 0 = Admin Einträge
                                    hist.userid = self._job.userid
                                    if right.groupid == 0 and right.userid == 0:
                                        hist.taskid = 94
                                        hist.value2 = right.groupid
                                    elif right.groupid == 0:
                                        hist.taskid = 93
                                        hist.value2 = right.userid
                                    else:
                                        hist.taskid = 94
                                        hist.value2 = right.groupid
                                    hist.value1 = right_arcid
                                    self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            else:
                                sql = 'insert into sys_arcxs (arcid, userid, groupid, rights) values (%i, %i, %i, %i) on duplicate key update rights = %i' \
                                      % (right_arcid, right.userid, right.groupid, right.rights, right.rights)

                                try:
                                    self.db[self._job.profile].execute(sqla.text(sql))
                                except:
                                    msg = _("%s Fehler beim Ändern eines Archiv-Berechtigungsatzes")
                                    self.log.exception("%s %s arcid %i userid %i groupid %i" % (self.name, msg, right_arcid, right.userid, right.groupid))
                                    error.append(msg)
                                else:
                                    hist = bitfarm_doc.BitfarmHistory()
                                    hist.gdocid = 0  # 0 = Admin Einträge
                                    hist.userid = self._job.userid
                                    if right.groupid == 0 and right.userid == 0:
                                        hist.taskid = 88
                                        hist.value2 = right.groupid
                                    elif right.groupid == 0:
                                        hist.taskid = 87
                                        hist.value2 = right.userid
                                    else:
                                        hist.taskid = 88
                                        hist.value2 = right.groupid
                                    hist.value1 = right_arcid
                                    hist.value3 = ",".join(
                                        [y for x, y in list(bitfarm_doc.RESOLVE_ARCRIGHTS.items()) if right.rights & x == x])
                                    self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                if elem.modified_addfields:
                    self.log.debug(_('%s Speichere Addfield, Zuordnung und Rechte %s') % (self.name, elem.arcname))
                    for field in elem.addfields:
                        for right in field.rights:
                            if right.modified:
                                if right.userid is None:
                                    right.userid = 0
                                if right.groupid is None:
                                    right.groupid = 0
                                if right.fieldpos is None:
                                    right.fieldpos = 0
                                right_arcid = elem.id if right.arcid < 0 else right.arcid

                                if right.deleted:
                                    sql = 'delete from sys_addfieldxs where arcid = %i and fieldid = %i and userid = %i and groupid = %i' % (
                                            right_arcid, right.fieldid, right.userid, right.groupid)
                                    try:
                                        self.db[self._job.profile].execute(sqla.text(sql))
                                    except:
                                        msg = _("Fehler beim Löschen eines Zusatzfeld-Berechtigungsatzes")
                                        self.log.exception("%s %s arcid %i fieldid %i userid %i groupid %i" % (self.name, msg, right_arcid, right.fieldid, right.userid, right.groupid))
                                        error.append(msg)
                                    else:
                                        hist = bitfarm_doc.BitfarmHistory()
                                        hist.gdocid = 0  # 0 = Admin Einträge
                                        hist.userid = self._job.userid
                                        if right.groupid == 0 and right.userid == 0:
                                            hist.taskid = 94
                                            hist.value2 = right.groupid
                                        elif right.groupid == 0:
                                            hist.taskid = 93
                                            hist.value2 = right.userid
                                        else:
                                            hist.taskid = 94
                                            hist.value2 = right.groupid
                                        hist.value1 = right_arcid
                                        self.bfaProcessing.write_history(history=hist,
                                                                                 profile=self._job.profile, logid=self.name)
                                else:
                                    sql = 'insert into sys_addfieldxs (arcid, fieldid, userid, groupid, pos, rights) values (%i, %i, %i, %i, %i, %i) on duplicate key update pos = %i, rights = %i' \
                                          % (right_arcid, right.fieldid, right.userid, right.groupid, right.fieldpos,
                                             right.rights, right.fieldpos, right.rights)

                                    try:
                                        self.db[self._job.profile].execute(sqla.text(sql))
                                    except:
                                        msg = _("Fehler beim Ändern eines Zusatzfeld-Berechtigungsatzes")
                                        self.log.exception("%s %s arcid %i fieldid %i userid %i groupid %i" % (self.name, msg, right_arcid, right.fieldid, right.userid, right.groupid))
                                        error.append(msg)
                                    else:
                                        hist = bitfarm_doc.BitfarmHistory()
                                        hist.gdocid = 0  # 0 = Admin Einträge
                                        hist.userid = self._job.userid
                                        if right.groupid == 0 and right.userid == 0:
                                            hist.taskid = 90
                                            hist.value2 = right.groupid
                                        elif right.groupid == 0:
                                            hist.taskid = 89
                                            hist.value2 = right.userid
                                        else:
                                            hist.taskid = 90
                                            hist.value2 = right.groupid

                                        hist.value1 = '%i.%i' % (right_arcid, right.fieldid)
                                        hist.value3 = ",".join([y for x, y in list(bitfarm_doc.RESOLVE_FIELDRIGHTS.items()) if
                                                                right.rights & x == x])
                                        self.bfaProcessing.write_history(history=hist,
                                                                                 profile=self._job.profile, logid=self.name)

                                sql = 'select type from sys_addfields where addfieldid = %s and type in (11,12)' % field.id
                                try:
                                    res = self.db[self._job.profile].execute(sqla.text(sql)).fetchall()
                                except:
                                    self.log.exception("%s Fehler bei Abfrage des Zusatzfeldtyps" % self.name)
                                else:
                                    if res:     # wenn die abfrage ein ergebnis liefert, dann ist es ein dlm feld
                                        if right_arcid not in recalc_arcids:
                                            recalc_arcids.append(right_arcid)

                if elem.modified_statefields:
                    self.log.debug(_('%s Speichere Statefield, Zuordnung und Rechte %s') %(self.name, elem.arcname))
                    for field in elem.statefields:
                        for right in field.rights:
                            if right.modified:
                                if right.userid is None:
                                    right.userid = 0
                                if right.groupid is None:
                                    right.groupid = 0
                                if right.fieldpos is None:
                                    right.fieldpos = 0
                                right_arcid = elem.id if right.arcid < 0 else right.arcid

                                if right.deleted:
                                    sql = 'delete from sys_statefieldxs where arcid = %i and fieldid = %i and userid = %i and groupid = %i' % (
                                            right_arcid, right.fieldid, right.userid, right.groupid)
                                    try:
                                        self.db[self._job.profile].execute(sqla.text(sql))
                                    except:
                                        msg = _("Fehler beim Löschen eines Statusfeld-Berechtigungsatzes")
                                        self.log.exception("%s %s arcid %i fieldid %i userid %i groupid %i" % (self.name, msg, right_arcid, right.fieldid, right.userid, right.groupid))
                                        error.append(msg)
                                    else:
                                        hist = bitfarm_doc.BitfarmHistory()
                                        hist.gdocid = 0  # 0 = Admin Einträge
                                        hist.userid = self._job.userid
                                        if right.groupid == 0 and right.userid == 0:
                                            hist.taskid = 94
                                            hist.value2 = right.groupid
                                        elif right.groupid == 0:
                                            hist.taskid = 93
                                            hist.value2 = right.userid
                                        else:
                                            hist.taskid = 94
                                            hist.value2 = right.groupid
                                        hist.value1 = right_arcid
                                        self.bfaProcessing.write_history(history=hist,
                                                                                 profile=self._job.profile, logid=self.name)
                                else:
                                    sql = 'insert into sys_statefieldxs (arcid, fieldid, userid, groupid, pos, rights) values (%i, %i, %i, %i, %i, %i) on duplicate key update pos = %i, rights = %i' \
                                          % (right_arcid, right.fieldid, right.userid, right.groupid, right.fieldpos,
                                             right.rights, right.fieldpos, right.rights)

                                    try:
                                        self.db[self._job.profile].execute(sqla.text(sql))
                                    except:
                                        msg = _("Fehler beim Ändern eines Statusfeld-Berechtigungsatzes")
                                        self.log.exception("%s %s arcid %i fieldid %i userid %i groupid %i" % (self.name, msg, right_arcid, right.fieldid, right.userid, right.groupid))
                                        error.append(msg)
                                    else:
                                        hist = bitfarm_doc.BitfarmHistory()
                                        hist.gdocid = 0  # 0 = Admin Einträge
                                        hist.userid = self._job.userid
                                        if right.groupid == 0 and right.userid == 0:
                                            hist.taskid = 92
                                            hist.value2 = right.groupid
                                        elif right.groupid == 0:
                                            hist.taskid = 91
                                            hist.value2 = right.userid
                                        else:
                                            hist.taskid = 92
                                            hist.value2 = right.groupid

                                        hist.value1 = '%i.%i' % (right_arcid, right.fieldid)
                                        hist.value3 = ",".join([y for x, y in list(bitfarm_doc.RESOLVE_FIELDRIGHTS.items()) if
                                                                right.rights & x == x])
                                        self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                                sql = 'select flags from sys_statefields where stateid = %s and flags & 4 = 4' % field.id
                                try:
                                    res = self.db[self._job.profile].execute(sqla.text(sql)).fetchall()
                                except:
                                    self.log.exception("%s Fehler bei Abfrage des Statusfeldtyps" % self.name)
                                else:
                                    if res:     # wenn die abfrage ein ergebnis liefert, dann ist es ein dlm feld
                                        if right_arcid not in recalc_arcids:
                                            recalc_arcids.append(right_arcid)

            elif isinstance(elem, bitfarm_doc.BitfarmAddfield):
                if not elem.modified:
                    continue
                self.log.debug(_('%s Speichere Addfield %s ID: %s') % (self.name, elem.add, elem.id))

                if elem.id is None or elem.id < 0:
                    # höchsten add_x Index finden (in allen Tabellen)
                    add_ind = self.get_max_addfieldid(_con, error)

                    elem.id = add_ind
                    elem.add = "add_%s" % add_ind
                    self.log.debug("Höchsten Index gefunden %s" % elem.id)

                sql = "select addfieldid, fieldname, displayname, type, items, mode, options, checkprogram, values_query, indextableid, flags +0, description" \
                      " from sys_addfields where addfieldid = %i" % elem.id
                try:
                    current = self.db[self._job.profile].execute(sql).fetchone()
                except:
                    msg = _("Fehler bei der Abfrage der Zusatzfelder")
                    self.log.exception("%s %s" % (self.name, msg))
                    error.append(msg)
                else:
                    if current:
                        self.log.debug("Feld %s schon vorhanden" % elem.id)
                        elem.add = current[1]
                        vals = {_con.sys_addfields.c.displayname: elem.name,
                                _con.sys_addfields.c.type: elem.type,
                                _con.sys_addfields.c.mode: elem.mode,
                                _con.sys_addfields.c.options: elem.options,
                                _con.sys_addfields.c.checkprogram: elem.checkprogram,
                                _con.sys_addfields.c.description: elem.description,
                                _con.sys_addfields.c.values_query: elem.values_query,
                                _con.sys_addfields.c.indextableid: elem.indextableid,
                                _con.sys_addfields.c.flags: elem.flags
                                }
                        if elem.items is not None and len(elem.items) > 0:
                            if type(elem.items) is list:
                                vals[_con.sys_addfields.c['items']] = ";".join(elem.items)
                            else:
                                vals[_con.sys_addfields.c['items']] = elem.items
                        else:
                            vals[_con.sys_addfields.c['items']] = None
                        sql = sqla.update(_con.sys_addfields, values=vals).where(_con.sys_addfields.c.addfieldid == elem.id)
                    else:
                        self.log.debug("Feld %s wird neu angelegt" % elem.id)
                        vals = {_con.sys_addfields.c.addfieldid: elem.id,
                                _con.sys_addfields.c.displayname: elem.name,
                                _con.sys_addfields.c.fieldname: elem.add,
                                _con.sys_addfields.c.type: elem.type,
                                _con.sys_addfields.c.mode: elem.mode,
                                _con.sys_addfields.c.options: elem.options,
                                _con.sys_addfields.c.checkprogram: elem.checkprogram,
                                _con.sys_addfields.c.description: elem.description,
                                _con.sys_addfields.c.values_query: elem.values_query,
                                _con.sys_addfields.c.indextableid: elem.indextableid,
                                _con.sys_addfields.c.flags: elem.flags
                                }
                        if elem.items is not None and len(elem.items) > 0:
                            if type(elem.items) is list:
                                vals[_con.sys_addfields.c['items']] = ";".join(elem.items)
                            else:
                                vals[_con.sys_addfields.c['items']] = elem.items
                        sql = sqla.insert(_con.sys_addfields, values=vals)

                    try:
                        self.db[self._job.profile].execute(sql)
                    except:
                        msg = _("Fehler beim Ändern eines Zusatzfeldes")
                        self.log.exception("%s %s" % (self.name, msg))
                        error.append(msg)
                    else:
                        # wenn als flag deleted ankommt, dann wurde das feld gelöscht. dann muss es auch aus der xs tabelle raus
                        if elem.flags & 2 == 2:
                            sql = 'delete from sys_addfieldxs where fieldid = %s' % elem.id
                            try:
                                self.db[self._job.profile].execute(sql)
                            except:
                                msg = _("Fehler beim Löschen der Berechtigung eines Zusatzfeldes")
                                self.log.exception("%s %s" % (self.name, msg))
                                error.append(msg)

                            # dlm zusatzfelder aus der sys_doclifecycle löschen
                            if elem.type in [11, 12]:
                                sql = 'delete from sys_doclifecycle where fieldid = %s and type in (1,2)' % elem.id
                                try:
                                    self.db[self._job.profile].execute(sql)
                                except:
                                    msg = _("Fehler beim Löschen eines Zusatzfeldes aus der DLM-Tabelle")
                                    self.log.exception("%s %s" % (self.name, msg))
                                    error.append(msg)
                            if elem.type == 13:     # wenn wir ein Gridfeld löschen, dann auch alle Spalten
                                gridconfig = bitfarm_xml.xml_to_object(elem.values_query)
                                for col in gridconfig.columns:
                                    vals = {_con.sys_addfields.c.flags: 'deleted'}
                                    sql = sqla.update(_con.sys_addfields, values=vals).where(_con.sys_addfields.c.fieldname == col.fieldname)

                                    try:
                                        self.db[self._job.profile].execute(sql)
                                    except:
                                        self.log.exception("%s Fehler beim Zurücksetzten des Feldes %s" % (self.name, col.fieldname))

                                vals = {_con.sys_addfields.c.values_query: None}
                                sql = sqla.update(_con.sys_addfields, values=vals).where(_con.sys_addfields.c.addfieldid == elem.id)

                                try:
                                    self.db[self._job.profile].execute(sql)
                                except:
                                    self.log.exception("%s Fehler beim Zurücksetzten des Feldes %s" % (self.name, elem.fieldname))

                        elif elem.type in [11, 12]:    # nicht gelöscht und dlm feld?
                            if elem.type == 11:
                                sql = 'select * from sys_doclifecycle where fieldid = %s and type = 1' % elem.id
                            elif elem.type == 12:
                                sql = 'select * from sys_doclifecycle where fieldid = %s and type = 2' % elem.id
                            try:
                                result = self.db[self._job.profile].execute(sql).fetchall()
                            except:
                                msg = _("Fehler beim Prüfen der DLM-Tabelle")
                                self.log.exception("%s %s" % (self.name, msg))
                                error.append(msg)
                            else:
                                if not result:  # feld noch nicht vorhanden
                                    sql = 'select max(prio) from sys_doclifecycle'
                                    try:
                                        result = self.db[self._job.profile].execute(sql).fetchone()
                                    except:
                                        msg = _("Fehler beim Ermitteln der höchsten Prio in der DLM-Tabelle")
                                        self.log.exception("%s %s" % (self.name, msg))
                                        error.append(msg)
                                    else:
                                        if result and result[0]:
                                            prio = result[0]
                                            prio += 1
                                        else:
                                            prio = 1

                                        if elem.items:
                                            if type(elem.items) is str:
                                                elem_items = elem.items.split(";")
                                            else:
                                                elem_items = elem.items
                                        else:
                                            elem_items = [elem.name]

                                        for item in elem_items:
                                            if elem.type == 11:
                                                sql = "insert into sys_doclifecycle (type, fieldid, item, prio) values (1, %s, '%s', %s)" % (elem.id, item, prio)
                                            elif elem.type == 12:
                                                sql = "insert into sys_doclifecycle (type, fieldid, prio) values (2, %s, %s)" % (elem.id, prio)

                                            try:
                                                self.db[self._job.profile].execute(sql)
                                            except:
                                                msg = _("Fehler beim Eintragen eines Zusatzfeldes in die DLM-Tabelle")
                                                self.log.exception("%s %s" % (self.name, msg))
                                                error.append(msg)
                                else:   # feld vorhanden? items abgleichen:
                                    prio = 1
                                    for row in result:
                                        if row['fieldid'] == elem.id and row['type'] == 1:  # feld gefunden? prio holen
                                            prio = row['prio']
                                            break
                                    
                                    if elem.items is not None:
                                        if type(elem.items) is str:
                                            elem_items = elem.items.split(";")
                                        else:
                                            elem_items = elem.items
                                    else:
                                        elem_items = [elem.name]

                                    current_items = []
                                    for row in result:
                                        current_items.append(row['item'])

                                    for item in elem_items[:]:
                                        if item in current_items:   # das übertragen item ist schon vorhanden? dann entfernen
                                            elem_items.remove(item)     # in elem_items bleibt übrig was noch nicht in der tabelle steht
                                            current_items.remove(item)  # in current_items bleibt übrig, was nicht schon vorhanden ist (kann also gelöscht werden)

                                    for item in elem_items:
                                        sql = "insert into sys_doclifecycle (type, fieldid, item, prio) values (1, %s, '%s', %s)" % (elem.id, item, prio)
                                        try:
                                            self.db[self._job.profile].execute(sql)
                                        except:
                                            msg = _("Fehler beim Eintragen eines Zusatzfeldeintrages in die DLM-Tabelle")
                                            self.log.exception("%s %s" % (self.name, msg))
                                            error.append(msg)

                                    for item in current_items:
                                        sql = "delete from sys_doclifecycle where type = 1 and fieldid = %s and item = '%s'" % (elem.id, item)
                                        try:
                                            self.db[self._job.profile].execute(sql)
                                        except:
                                            msg = _("Fehler beim Löschen eines Zusatzfeldes aus der DLM-Tabelle")
                                            self.log.exception("%s %s" % (self.name, msg))
                                            error.append(msg)

                        hist = bitfarm_doc.BitfarmHistory()
                        hist.gdocid = 0  # 0 = Admin Einträge
                        hist.userid = self._job.userid
                        hist.value1 = str(elem.id)

                        if current:  # wir haben einen alten datensatz, dann abgleichen und history schreiben
                            if elem.name is not None and current[2] != elem.name:
                                hist.taskid = 50
                                current = [x if x is not None else 0 for x in
                                           [int(x) if type(x) is float else x for x in current]]
                                hist.value2 = current[2]
                                hist.value3 = elem.name
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            if current[3] != elem.type:
                                hist.taskid = 51
                                if current[3] in bitfarm_doc.RESOLVE_FIELDTYPE:
                                    hist.value2 = bitfarm_doc.RESOLVE_FIELDTYPE[current[3]]
                                else:
                                    hist.value2 = str(current[3])
                                if elem.type in bitfarm_doc.RESOLVE_FIELDTYPE:
                                    hist.value3 = bitfarm_doc.RESOLVE_FIELDTYPE[elem.type]
                                else:
                                    hist.value3 = str(elem.type)

                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            if int(current[10]) != elem.flags:
                                current_flags = ",".join([y for x, y in list(bitfarm_doc.FLAGS.items()) if int(current[10]) & x == x])
                                flags = ",".join([y for x, y in list(bitfarm_doc.FLAGS.items()) if elem.flags & x == x])
                                hist.taskid = 52
                                hist.value2 = current_flags
                                hist.value3 = flags
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            if current[6] != elem.options:
                                current_options = ",".join(
                                    [y for x, y in list(bitfarm_doc.FIELD_OPTIONS.items()) if current[6] & x == x])
                                options = ",".join(
                                    [y for x, y in list(bitfarm_doc.FIELD_OPTIONS.items()) if elem.options & x == x])
                                hist.taskid = 53
                                hist.value2 = current_options
                                hist.value3 = options
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            if current[5] != elem.mode:
                                hist.taskid = 54
                                if current[5] in bitfarm_doc.RESOLVE_SEARCHMODE:
                                    hist.value2 = bitfarm_doc.RESOLVE_SEARCHMODE[current[5]]
                                else:
                                    hist.value2 = str(current[5])
                                if elem.mode in bitfarm_doc.RESOLVE_SEARCHMODE:
                                    hist.value3 = bitfarm_doc.RESOLVE_SEARCHMODE[elem.mode]
                                else:
                                    hist.value3 = str(elem.mode)
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            if elem.checkprogram is not None and current[7] != elem.checkprogram:
                                hist.taskid = 55
                                hist.value2 = current[7]
                                hist.value3 = elem.checkprogram
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            if elem.description is not None and current[11] != elem.description:
                                hist.taskid = 107
                                hist.value2 = current[11]
                                hist.value3 = elem.description
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)
                        else:
                            if elem.name:
                                hist.taskid = 43
                                hist.value2 = elem.name
                                hist.value3 = None
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            if elem.type:
                                hist.taskid = 44
                                if elem.type in bitfarm_doc.RESOLVE_FIELDTYPE:
                                    hist.value2 = bitfarm_doc.RESOLVE_FIELDTYPE[elem.type]
                                else:
                                    hist.value2 = str(elem.type)
                                hist.value3 = str(elem.type)

                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            if elem.flags:
                                flags = ",".join([y for x, y in list(bitfarm_doc.FLAGS.items()) if elem.flags & x == x])
                                hist.taskid = 45
                                hist.value2 = flags
                                hist.value3 = None
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            if elem.options:
                                options = ",".join([y for x, y in list(bitfarm_doc.FIELD_OPTIONS.items()) if elem.options & x == x])
                                hist.taskid = 46
                                hist.value2 = options
                                hist.value3 = None
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            if elem.mode:
                                hist.taskid = 47
                                if elem.mode in bitfarm_doc.RESOLVE_SEARCHMODE:
                                    hist.value2 = bitfarm_doc.RESOLVE_SEARCHMODE[elem.mode]
                                else:
                                    hist.value2 = str(elem.mode)
                                hist.value3 = None
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            if elem.checkprogram:
                                hist.taskid = 48
                                hist.value2 = elem.checkprogram
                                hist.value3 = None
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            if elem.description:
                                hist.taskid = 106
                                hist.value2 = elem.description
                                hist.value3 = None
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                        if not current:  # Spalte in sys_docidx anlegen
                            BitfarmWorkerThread.altertablelock.acquire()
                            try:
                                new_error = self.create_new_addfield(elem, _con, [])
                            finally:
                                BitfarmWorkerThread.altertablelock.release()
                            if new_error:
                                error.extend(new_error)
                                vals = {_con.sys_addfields.c.flags: 'deleted'}
                                sql = sqla.update(_con.sys_addfields, values=vals).where(_con.sys_addfields.c.fieldname == elem.add)

                                try:
                                    self.db[self._job.profile].execute(sql)
                                except:
                                    self.log.exception("%s Fehler beim Zurücksetzten des Feldes %s" % (self.name, elem.add))
                                continue

                        elif elem.type != current[3]:   #Feldtyp geändert?
                            if current[3] in [11, 12]:    # wenn das feld ein dlm feld war und jetzt nicht mehr, dann muss es raus aus der konfiguration
                                if current[3] == 11:
                                    sql = 'delete from sys_doclifecycle where fieldid = %s and type = 1' % elem.id
                                elif current[3] == 12:
                                    sql = 'delete from sys_doclifecycle where fieldid = %s and type = 2' % elem.id
                                try:
                                    self.db[self._job.profile].execute(sql)
                                except:
                                    msg = _("Fehler beim Löschen eines Zusatzfeldes aus der DLM-Tabelle")
                                    self.log.exception("%s %s" % (self.name, msg))
                                    error.append(msg)

                            # alle typen außer datum und auswahl, oder datum bzw auswahl, wenn nicht vorher schon dlm datum bzw dlm auswahl
                            if elem.type not in [1,12,2,11] or (elem.type == 1 and current[3] != 12) \
                                                            or (elem.type == 12 and current[3] != 1) \
                                                            or (elem.type == 2 and current[3] != 11) \
                                                            or (elem.type == 11 and current[3] != 2):
                                idxtblid = elem.indextableid
                                ftype = bitfarm_doc.FIELD_TYPES[current[3]]
                                BitfarmWorkerThread.altertablelock.acquire()
                                try:
                                    sql = "ALTER TABLE `sys_docidx_%s` LOCK = SHARED, CHANGE COLUMN `%s` `%s` %s NULL DEFAULT NULL" % (elem.indextableid, elem.add, elem.add, bitfarm_doc.FIELD_TYPES[elem.type])

                                    try:
                                        self.db[self._job.profile].execute(sql)
                                    except sqla.exc.OperationalError:
                                        self.log.exception(_("%s Fehler beim Ändern des Zusatzfeldtyps, Tabellengröße überschritten?") % self.name)
                                        error = self.create_new_addfield(elem, _con, error)
                                        # alte Spalte noch umbenennen, sonst macht die bei der Suche Probleme
                                        sql = "ALTER TABLE `sys_docidx_%s` LOCK = SHARED, CHANGE COLUMN `%s` `x%s` %s NULL DEFAULT NULL;" % (idxtblid, elem.add, elem.add, ftype)

                                        try:
                                            self.db[self._job.profile].execute(sql)
                                        except:
                                            msg = _("Fehler beim Ändern des Zusatzfeldtyps")
                                            self.log.exception("%s %s Spalte %s in x%s in der sys_docidx_%s Tabelle" % (self.name, msg, elem.add, elem.add, idxtblid))
                                            error.append(msg)
                                    except:
                                        msg = _("Fehler beim Ändern des Zusatzfeldtyps")
                                        self.log.exception("%s Fehler beim Ändern der %s Spalte in der sys_docidx_%s Tabelle") % (self.name, elem.add, elem.indextableid)
                                        error.append(msg)
                                    else:
                                        self.log.debug(_("%s Zusatzfeldspalte Typ geändert %s (%s)") % (self.name, elem.add, bitfarm_doc.FIELD_TYPES[elem.type]))
                                finally:
                                    BitfarmWorkerThread.altertablelock.release()

                        if elem.type in [2, 3, 11]:
                            sql = "SELECT COUNT(*) FROM information_schema.statistics WHERE TABLE_SCHEMA = DATABASE() AND TABLE_NAME = 'sys_docidx_%s'  AND INDEX_NAME = 'idx_%s'; " % (elem.indextableid, elem.add)

                            try:
                                result = self.db[self._job.profile].execute(sql).fetchone()
                            except:
                                self.log.exception("%s Fehler bei Abfrage des Index für Auswahlfelder" % self.name)
                            else:
                                if result:
                                    if result[0] == 0:      # keine Index vorhanden? Anlegen
                                        BitfarmWorkerThread.altertablelock.acquire()
                                        try:
                                            sql = "ALTER TABLE `sys_docidx_%s` LOCK = SHARED, ADD INDEX `idx_%s` (`%s` ASC);" % (elem.indextableid, elem.add, elem.add)

                                            try:
                                                self.db[self._job.profile].execute(sql)
                                            except:
                                                self.log.exception("%s Fehler bei Abfrage des Index für Auswahlfelder" % self.name)
                                        finally:
                                            BitfarmWorkerThread.altertablelock.release()

                            # if elem.options & 8 == 8:       #strict auswahl feld? wir suchen werte die nicht in items stehen
                            #     sql = 'select distinct(%s) from sys_docidx_%s where %s is not Null' % (elem.add, elem.indextableid, elem.add)
                            #     try:
                            #         result = self.db[self._job.profile].execute(sql).fetchall()
                            #     except:
                            #         self.log.exception(u"%s Fehler bei Abfrage des Items für Auswahlfelder" % self.name)
                            #     else:
                            #         if type(elem.items) is not list:
                            #             elem.items = elem.items.split(';')
                            #         added = False
                            #         for row in result:
                            #             items = row[0].split(';')
                            #             for item in items:
                            #                 if item not in elem.items:
                            #                     elem.items.append(item)
                            #                     added = True
                            #
                            #         if added:
                            #             vals[_con.sys_addfields.c['items']] = u";".join(elem.items)
                            #             sql = sqla.update(_con.sys_addfields, values=vals).where(
                            #                     _con.sys_addfields.c.addfieldid == elem.id)
                            #
                            #             try:
                            #                 self.db[self._job.profile].execute(sql)
                            #             except:
                            #                 msg = _(u"Fehler beim Ändern eines Zusatzfeldes")
                            #                 self.log.exception(u"%s %s" % (self.name, msg))
                            #                 error.append(msg)
                            #             else:
                            #                 self.log.info(u'Zusatzfeld Items von Feld %s (%s) aktualisiert' % (elem.name, elem.add))

                        if elem.type in [13] and elem.values_query:   # Grid-Feld? Prüfen, ob neue Spalten
                            gridconfig = bitfarm_xml.xml_to_object(elem.values_query)
                            found = False
                            for col in gridconfig.columns:
                                if col.fieldname[:4] == 'new_':
                                    found = True
                                    new_add = bitfarm_doc.BitfarmAddfield()
                                    add_ind = self.get_max_addfieldid(_con, error)
                                    new_add.id = add_ind
                                    new_add.add = "add_%s" % add_ind
                                    new_add.type = 14   # Grid-Spalte
                                    new_add.name = col.name
                                    new_add.flags = ['enabled']
                                    new_add.mode = 3

                                    BitfarmWorkerThread.altertablelock.acquire()
                                    try:
                                        new_error = self.create_new_addfield(new_add, _con, [])
                                    finally:
                                        BitfarmWorkerThread.altertablelock.release()
                                    if new_error:
                                        error.extend(new_error)
                                        vals = {_con.sys_addfields.c.flags: 'deleted'}
                                        sql = sqla.update(_con.sys_addfields, values=vals).where(_con.sys_addfields.c.fieldname == new_add.add)

                                        try:
                                            self.db[self._job.profile].execute(sql)
                                        except:
                                            self.log.exception("%s Fehler beim Zurücksetzten des Feldes %s" % (self.name, new_add.add))
                                        continue
                                    else:
                                        if gridconfig.searchfield == col.fieldname:
                                            gridconfig.searchfield = new_add.add
                                        col.fieldname = new_add.add

                                if col.fieldname[:4] == 'del_':     # ein Feld zum Löschen gefunden
                                    found = True
                                    delname = col.fieldname[4:]
                                    vals = {_con.sys_addfields.c.flags: 'deleted'}
                                    sql = sqla.update(_con.sys_addfields, values=vals).where(_con.sys_addfields.c.fieldname == delname)

                                    try:
                                        self.db[self._job.profile].execute(sql)
                                    except:
                                        self.log.exception("%s Fehler beim Zurücksetzten des Feldes %s" % (self.name, delname))

                            if found:
                                gridconfig.columns = [col for col in gridconfig.columns if not col.fieldname[:4] == 'del_']
                                values_query = bitfarm_xml.object_to_xml(gridconfig)
                                vals = {_con.sys_addfields.c.values_query: values_query}
                                sql = sqla.update(_con.sys_addfields, values=vals).where(_con.sys_addfields.c.fieldname == elem.add)

                                try:
                                    self.db[self._job.profile].execute(sql)
                                except:
                                    self.log.exception("%s Fehler beim Zurücksetzten des Feldes %s" % (self.name, elem.add))

                            if self.bfaserver.gridfunc:   # aktualisiert die Gridwerte mit den berechneten Feldern
                                self.gridfieldlock.acquire()
                                try:
                                    self.bfaserver.gridload()
                                except:
                                    self.log.exception("%s Fehler beim Aktualisieren der bfaGridField64.dll für das Feld %s" % (self.name, elem.add))
                                finally:
                                    self.gridfieldlock.release()

                        if elem.tempid is not None:
                            guids[elem.tempid] = str(elem.id) + ',' + str(elem.indextableid)

            elif isinstance(elem, bitfarm_doc.BitfarmStatefield):
                if not elem.modified:
                    continue
                self.log.debug(_('%s Speichere Statefield %s') % (self.name, elem.name))

                sql = "select stateid, displayname, bitmask, addfieldid, `function`, indexcolumnid, flags +0" \
                      " from sys_statefields where stateid = %i" % elem.id
                try:
                    current = self.db[self._job.profile].execute(sql).fetchone()
                except:
                    msg = _("Fehler bei der Abfrage der Statusfelder")
                    self.log.exception("%s %s" % (self.name, msg))
                    error.append(msg)
                else:
                    if current:
                        indexcolumnid = None
                        bitmask = None
                        vals = {_con.sys_statefields.c.displayname: elem.name,
                                _con.sys_statefields.c.addfieldid: elem.addfieldid,
                                _con.sys_statefields.c.function: elem.func,
                                _con.sys_statefields.c.flags: elem.flags
                                }
                        sql = sqla.update(_con.sys_statefields, values=vals).where(
                            _con.sys_statefields.c.stateid == elem.id)
                    else:
                        sql = "select max(stateid), max(indexcolumnid) from sys_statefields"
                        maxid = self.db[self._job.profile].execute(sql).fetchone()

                        if not maxid:
                            maxid = [0, 0]
                        else:
                            maxid = [x if x is not None else 0 for x in maxid]

                        if elem.id <= maxid[0]:
                            elem.id = int(maxid[0] + 1)

                        indexcolumnid = divmod(elem.id, 63)[0]
                        bitmask = 1 << (elem.id - (indexcolumnid * 63))
                        if elem.tempid is not None:
                            guids[elem.tempid] = str(elem.id) + ',' + str(indexcolumnid)

                        if indexcolumnid > maxid[1]:
                            try:
                                sql = 'ALTER TABLE sys_doc LOCK = SHARED, ADD COLUMN `docstate_%i` BIGINT(20) UNSIGNED NULL DEFAULT 0 AFTER `docstate_%i`' % (indexcolumnid, int(maxid[1]))
                                self.db[self._job.profile].execute(sql)
                            except sqla.exc.OperationalError:
                                self.log.error("%s Statusspalte docstate_%s schon vorhanden" % (self.name, indexcolumnid))
                            except:
                                msg = _("Fehler beim Anlegen einer neuen Statusspalte")
                                self.log.exception("%s %s" % (self.name, msg))
                                error.append(msg)
                                continue
                            else:
                                self.log.info(_("%s Neue Statusspalte angelegt") % self.name)

                                sql = 'update sys_doc set docstate_%i = 0 where docstate_%i is NULL' & (indexcolumnid, indexcolumnid)
                                try:
                                    self.db[self._job.profile].execute(sql)
                                except:
                                    msg = _("Fehler beim Setzen des Defaults (0) in der Statusspalte")
                                    self.log.exception("%s %s" % (self.name, msg))
                                    error.append(msg)

                        vals = {_con.sys_statefields.c.stateid: elem.id,
                                _con.sys_statefields.c.displayname: elem.name,
                                _con.sys_statefields.c.bitmask: bitmask,
                                _con.sys_statefields.c.addfieldid: elem.addfieldid,
                                _con.sys_statefields.c.function: elem.func,
                                _con.sys_statefields.c.indexcolumnid: indexcolumnid,
                                _con.sys_statefields.c.flags: elem.flags
                                }

                        sql = sqla.insert(_con.sys_statefields, values=vals)

                    try:
                        self.db[self._job.profile].execute(sql)
                    except:
                        msg = _("Fehler beim Ändern eines Statusfeldes")
                        self.log.exception("%s %s" % (self.name, msg))
                        error.append(msg)
                    else:
                        # wenn als flag deleted ankommt, dann wurde das feld gelöscht. dann muss es auch aus der xs tabelle raus
                        if elem.flags & 2 == 2:
                            sql = 'delete from sys_statefieldxs where fieldid = %s' % elem.id
                            try:
                                self.db[self._job.profile].execute(sql)
                            except:
                                msg = _("Fehler beim Löschen der Berechtigung eines Statusfeldes")
                                self.log.exception("%s %s" % (self.name, msg))
                                error.append(msg)

                            # dlm statusfelder aus der sys_doclifecycle löschen
                            if elem.flags & 4 == 4:
                                sql = 'delete from sys_doclifecycle where fieldid = %s and type = 0' % elem.id
                                try:
                                    self.db[self._job.profile].execute(sql)
                                except:
                                    msg = _("Fehler beim Löschen eines Statusfeldes aus der DLM-Tabelle")
                                    self.log.exception("%s %s" % (self.name, msg))
                                    error.append(msg)
                        elif elem.flags & 4 == 4:  # nicht gelöscht und dlm feld?
                            sql = 'select * from sys_doclifecycle where fieldid = %s and type = 0' % elem.id
                            try:
                                result = self.db[self._job.profile].execute(sql).fetchall()
                            except:
                                msg = _("Fehler beim Prüfen der DLM-Tabelle")
                                self.log.exception("%s %s" % (self.name, msg))
                                error.append(msg)
                            else:
                                if not result:  #feld noch nicht vorhanden
                                    sql = 'select max(prio) from sys_doclifecycle'
                                    try:
                                        result = self.db[self._job.profile].execute(sql).fetchone()
                                    except:
                                        msg = _("Fehler beim Ermitteln der höchsten Prio in der DLM-Tabelle")
                                        self.log.exception("%s %s" % (self.name, msg))
                                        error.append(msg)
                                    else:
                                        if result and result[0]:
                                            prio = result[0]
                                            prio += 1
                                        else:
                                            prio = 1

                                        sql = 'insert into sys_doclifecycle (type, fieldid, prio) values (0, %s, %s)' % (elem.id, prio)
                                        try:
                                            self.db[self._job.profile].execute(sql)
                                        except:
                                            msg = _("Fehler beim Eintragen eines Statusfeldes in der DLM-Tabelle")
                                            self.log.exception("%s %s" % (self.name, msg))
                                            error.append(msg)

                        hist = bitfarm_doc.BitfarmHistory()
                        hist.gdocid = 0  # 0 = Admin Einträge
                        hist.userid = self._job.userid
                        hist.value1 = str(elem.id)

                        if current:  # wir haben einen alten datensatz, dann abgleichen und history schreiben
                            current = [x if x is not None else 0 for x in
                                       [int(x) if type(x) is float else x for x in current]]
                            if elem.name is not None and current[1] != elem.name:
                                hist.taskid = 61
                                hist.value2 = current[1]
                                hist.value3 = elem.name
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            if elem.addfieldid is not None and current[3] != elem.addfieldid:
                                hist.taskid = 62
                                hist.value2 = str(current[3])
                                hist.value3 = str(elem.addfieldid)

                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            if elem.func is not None and current[4] != elem.func:
                                hist.taskid = 63
                                if current[4] in bitfarm_doc.RESOLVE_FUNCTION:
                                    hist.value2 = bitfarm_doc.RESOLVE_FUNCTION[current[4]]
                                else:
                                    hist.value2 = str(current[4])
                                if elem.func in bitfarm_doc.RESOLVE_FUNCTION:
                                    hist.value3 = bitfarm_doc.RESOLVE_FUNCTION[elem.func]
                                else:
                                    hist.value3 = str(elem.func)

                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            if current[6] != elem.flags:
                                current_flags = ",".join([y for x, y in list(bitfarm_doc.FLAGS_FIELD.items()) if current[6] & x == x])
                                flags = ",".join([y for x, y in list(bitfarm_doc.FLAGS_FIELD.items()) if elem.flags & x == x])
                                hist.taskid = 64
                                hist.value2 = current_flags
                                hist.value3 = flags
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                                if current[6] & 4 == 4:     # war das feld ein dlm-feld und jetzt nicht mehr? dann aus konfiguration entfernen
                                    sql = 'delete from sys_doclifecycle where fieldid = %s and type = 0' % elem.id
                                    try:
                                        self.db[self._job.profile].execute(sql)
                                    except:
                                        msg = _("Fehler beim Löschen eines Statusfeldes aus der DLM-Tabelle")
                                        self.log.exception("%s %s" % (self.name, msg))
                                        error.append(msg)
                        else:
                            if elem.name:
                                hist.taskid = 57
                                hist.value2 = elem.name
                                hist.value3 = None
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            if bitmask:
                                hist.taskid = 58
                                hist.value2 = str(bitmask)
                                hist.value3 = str(indexcolumnid)

                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            if elem.addfieldid:
                                hist.taskid = 59
                                hist.value2 = str(elem.addfieldid)
                                if elem.func in bitfarm_doc.RESOLVE_FUNCTION:
                                    hist.value3 = bitfarm_doc.RESOLVE_FUNCTION[elem.func]
                                else:
                                    hist.value3 = str(elem.func)

                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            if elem.flags:
                                flags = ",".join([y for x, y in list(bitfarm_doc.FLAGS_FIELD.items()) if elem.flags & x == x])
                                hist.taskid = 60
                                hist.value2 = flags
                                hist.value3 = None
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

            elif isinstance(elem, bitfarm_doc.BitfarmDLM):
                if not elem.modified:
                    continue
                self.log.debug(_('%s Speichere BitfarmDLM %s') % (self.name, elem.name))

                sql = "select id, `type`, fieldid, item, duration, `interval`, date, prio, mode" \
                      " from sys_doclifecycle where id = %i" % elem.id
                try:
                    current = self.db[self._job.profile].execute(sql).fetchone()
                except:
                    msg = _("Fehler bei der Abfrage DLM Konfiguration")
                    self.log.exception("%s %s" % (self.name, msg))
                    error.append(msg)
                else:
                    changed_dlmfields.append(elem)
                    if current:
                        vals = {_con.sys_doclifecycle.c.duration: elem.duration,
                                _con.sys_doclifecycle.c.interval: elem.interval,
                                _con.sys_doclifecycle.c.date: elem.date,
                                _con.sys_doclifecycle.c.prio: elem.prio
                                }
                        sql = sqla.update(_con.sys_doclifecycle, values=vals).where(
                            _con.sys_doclifecycle.c.id == elem.id)
                        try:
                            self.db[self._job.profile].execute(sql)
                        except:
                            msg = _("Fehler beim Ändern eines DLM Konfiguration (bestehendes DLM Objekt)")
                            self.log.exception("%s %s" % (self.name, msg))
                            error.append(msg)

                        if current['prio'] != elem.prio:    # hat sich die prio geändert? dann müssen ggf. alle einträge eines auswahlfeldes aktualisiert werden
                            vals = {_con.sys_doclifecycle.c.prio: elem.prio}
                            sql = sqla.update(_con.sys_doclifecycle, values=vals).where(
                                sqla.and_(_con.sys_doclifecycle.c.fieldid == elem.fieldid,
                                          _con.sys_doclifecycle.c.type == elem.type))
                            try:
                                self.db[self._job.profile].execute(sql)
                            except:
                                msg = _("Fehler beim Ändern der Priorität eines bestehenden DLM Objekts")
                                self.log.exception("%s %s" % (self.name, msg))
                                error.append(msg)
                    else:
                        vals = {_con.sys_doclifecycle.c.type: elem.type,
                                _con.sys_doclifecycle.c.fieldid: elem.fieldid,
                                _con.sys_doclifecycle.c.item: elem.item,
                                _con.sys_doclifecycle.c.duration: elem.duration,
                                _con.sys_doclifecycle.c.interval: elem.interval,
                                _con.sys_doclifecycle.c.date: elem.date,
                                _con.sys_doclifecycle.c.prio: elem.prio
                                }
                        sql = sqla.insert(_con.sys_doclifecycle, values=vals)
                        try:
                            self.db[self._job.profile].execute(sql)
                        except:
                            msg = _("Fehler beim Ändern eines DLM Konfiguration (neues DLM Objekt)")
                            self.log.exception("%s %s" % (self.name, msg))
                            error.append(msg)

                    hist = bitfarm_doc.BitfarmHistory()
                    hist.gdocid = 0  # 0 = Admin Einträge
                    hist.userid = self._job.userid
                    if elem.type == 0:
                        hist.taskid = 123
                        hist.value1 = elem.id
                        if elem.interval == 'c':
                            hist.value2 = "Nie (leeren)"
                        elif elem.duration is None:
                            hist.value2 = "unkonfiguriert"
                        else:
                            hist.value2 = "%s %s" % (elem.duration, bitfarm_doc.DLM_INTERVALS[elem.interval] if elem.interval is not None else "")
                    if elem.type == 1:
                        hist.taskid = 121
                        hist.value1 = elem.id
                        hist.value2 = elem.item
                        if elem.interval == 'c':
                            hist.value3 = "Nie (leeren)"
                        elif elem.duration is None:
                            hist.value3 = "unkonfiguriert"
                        else:
                            hist.value3 = "%s %s" % (elem.duration, bitfarm_doc.DLM_INTERVALS[elem.interval] if elem.interval is not None else "")
                    if elem.type == 2:
                        hist.taskid = 122
                        hist.value1 = elem.id
                        if elem.interval == 'c':
                            hist.value2 = "Nie (leeren)"
                        elif elem.duration is None:
                            hist.value2 = "unkonfiguriert"
                        else:
                            hist.value2 = "%s %s" % (elem.duration, bitfarm_doc.DLM_INTERVALS[elem.interval] if elem.interval is not None else "")

                    self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                    hist = bitfarm_doc.BitfarmHistory()
                    hist.gdocid = 0  # 0 = Admin Einträge
                    hist.userid = self._job.userid
                    if elem.type == 0:
                        hist.taskid = 126
                        hist.value1 = elem.id
                        hist.value2 = elem.prio
                    if elem.type == 1:
                        hist.taskid = 124
                        hist.value1 = elem.id
                        hist.value2 = elem.prio
                    if elem.type == 2:
                        hist.taskid = 125
                        hist.value1 = elem.id
                        hist.value2 = elem.prio

                    self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

        # wir aktualisieren die lgrid von neu angelegten archiven
        if len(repos) > 0:
            for oldid, newid in list(repos.items()):
                sql = "update sys_arc set lgrid = %i where lgrid = %i" % (newid, oldid)
                try:
                    self.db[self._job.profile].execute(sql)
                except:
                    msg = _("Fehler beim Aktualisieren der lgrid neu angelegter Archive")
                    self.log.exception("%s %s" % (self.name, msg))
                    error.append(msg)
        try:
            if len(guids) > 0:
                guids = ['%s,%s' % x for x in list(guids.items())]
                guids = ";".join(guids)
            if len(error) == 0:
                xml = bitfarm_xml.create_xml_request(reqtype="respSaveAdmin", gdocid=self._job.gdocid, parameter=guids, data=_("reqSaveAdmin ausgeführt"))
            else:
                error = "\n".join(error)
                xml = bitfarm_xml.create_xml_request(reqtype="respSaveAdminError", gdocid=self._job.gdocid, parameter=guids,
                                                     data=error)
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

        # jetzt noch die server-konfiguration aktualisieren
        self.log.info(_("lese Archivkonfiguration für %s") % self._job.profile)
        self.refreshlock.acquire()
        try:
            if self.bfaserver.develop:
                start = time.perf_counter()
            archives = self.bfaserver.con.connections[self._job.profile].read_arctree(self.db[self._job.profile])

            if self.bfaserver.develop:
                self.log.info('Process time: %s' % str(time.perf_counter()-start))
            for archive in archives:
                self.bfaserver.archives[self._job.profile][archive.id] = archive
        finally:
            self.refreshlock.release()

        self.log.info(_("Lese Benutzerberechtigungen für %s") % self._job.profile)
        for userid in list(self.bfaserver.userrights[self._job.profile].keys()):
            self.bfaserver.bfaLogin.read_userrights(userid, self._job.profile, db=self.db[self._job.profile])

        # sollten dlm felder umkonfiguriert worden sein, müssen wir alle archive ändern, wo diese felder vorhanden sind
        for elem in changed_dlmfields:
            if elem.type == 0:  # statusfelder
                sql = "select distinct(arcid) from sys_statefieldxs where fieldid = %s" % elem.fieldid
            else:
                sql = "select distinct(arcid) from sys_addfieldxs where fieldid = %s" % elem.fieldid
            try:
                result = self.db[self._job.profile].execute(sql).fetchall()
            except:
                self.log.exception("Fehler bei der Abfrage der Archive für ein DLM Feld")
            else:
                if result:
                    for row in result:
                        if row[0] not in recalc_arcids:
                            recalc_arcids.append(row[0])

        if recalc_arcids:
            newjob = bitfarm_doc.bfaReq()
            newjob.reqtype = 'recalculate_expired_for_archives'
            newjob.profile = self._job.profile
            newjob.parameter = recalc_arcids
            newjob.userid = self._job.userid
            self.bfaserver.requestqueue.put(newjob)

    def get_max_addfieldid(self, _con, error):
        add_ind = 0
        for idx in _con.sys_docidx.values():
            for col in list(idx.columns.keys()):
                add_ind = int(col[4:]) if col[:4] == 'add_' and int(col[4:]) > add_ind else add_ind
        add_ind += 1
        # höchste add_index in sys_addfields ermitteln
        sql = 'select max(addfieldid) from sys_addfields'
        try:
            res = self.db[self._job.profile].execute(sql).fetchone()
        except:
            msg = _("Fehler bei der Abfrage einer neuen Zusatzfeld ID")
            self.log.exception("%s %s" % (self.name, msg))
            error.append(msg)
        else:
            if res and res[0] is not None:
                if res[0] >= add_ind:  # höchste id größergleich unserer nächsten id? da muss wohl ein fehler passiert sein
                    add_ind = res[0] + 1
        return add_ind

    def reqResetAddfields(self):
        self.log.debug("%s reqResetAddfields" % self.name)
        _con = self.bfaserver.con.connections[self._job.profile]
        error = []
        if type(self._job.data) is not list:
            self._job.data = [self._job.data]

        for elem in self._job.data:
            if isinstance(elem, bitfarm_doc.BitfarmAddfield):
                if not elem.modified:
                    continue

                sql = "select gdocid from sys_docidx_%s where %s = '%s'" % (elem.indextableid, elem.add, elem.oldvalue)
                try:
                    result = self.db[self._job.profile].execute(sql).fetchall()
                except:
                    msg = _("Fehler beim Zurücksetzten des Zusatzfeldes %s" % elem.name)
                    self.log.exception("%s %s" % (self.name, msg))
                    error.append(msg)
                if result:
                    gdocids = [x[0] for x in result]
                    if elem.value:
                        sql = "update sys_docidx_%s set %s = '%s' where gdocid in (%s)" % (elem.indextableid, elem.add, elem.value,  ",".join([str(x) for x in gdocids]))
                    else:
                        sql = "update sys_docidx_%s set %s = NULL where gdocid in (%s)" % (elem.indextableid, elem.add, ",".join([str(x) for x in gdocids]))
                    try:
                        self.db[self._job.profile].execute(sql)
                    except:
                        msg = _("Fehler beim Zurücksetzten des Zusatzfeldes %s" % elem.name)
                        self.log.exception("%s %s" % (self.name, msg))
                        error.append(msg)
                    else:
                        hist = bitfarm_doc.BitfarmHistory()
                        hist.userid = self._job.userid
                        hist.taskid = 136
                        hist.value1 = elem.name
                        hist.value2 = elem.oldvalue
                        if elem.value:
                            hist.value3 = elem.value
                        else:
                            hist.value3 = 'leer'
                        gdocids.append(0)   # wir setzen eine Historymeldung auch ins Adminlog ab
                        self.bfaProcessing.write_history(history=hist, profile=self._job.profile,
                                                                 gdocid=gdocids, logid=self.name)
        try:
            if len(error) == 0:
                xml = bitfarm_xml.create_xml_request(reqtype="respOK", gdocid=self._job.gdocid, data=_("reqResetAddfields ausgeführt"))
            else:
                error = ";".join(error)
                xml = bitfarm_xml.create_xml_request(reqtype="respError", gdocid=self._job.gdocid, data=error)
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def recalculate_expired_for_archives(self):
        self.log.debug("%s recalculate_expired_for_archives" % self.name)

        hist = bitfarm_doc.BitfarmHistory()
        hist.userid = self._job.userid
        hist.taskid = 128
        hist.value1 = str(len(self._job.parameter))
        self.bfaProcessing.write_history(history=hist, profile=self._job.profile,
                                                 gdocid=0, logid=self.name)

        for arcid in self._job.parameter:
            self.log.info("%s Berechne das Löschdatum für Archiv %s" % (self.name, arcid))

            sql = "select gdocid from sys_doc where arcid = %s" % arcid
            try:
                result = self.db[self._job.profile].execute(sql).fetchall()
            except:
                self.log.exception("%s Fehler bei der Abfrage der Archive für ein DLM Feld" % self.name)
            else:
                if result:
                    sql = "update sys_doc set expired_flags = 1 where arcid = %s" % arcid
                    try:
                        self.db[self._job.profile].execute(sql)
                    except:
                        self.log.exception("%s Fehler beim Setzen des recalculate-Flags" % self.name)
                    else:
                        for row in result:
                            self.log.debug("%s Berechne Löschdatum für Dokument [%s] im Archiv %s" % (self.name, row[0], arcid))
                            success, expired = self.bfaProcessing.recalculate_expired_date(row[0], self._job.profile)
                            hist = bitfarm_doc.BitfarmHistory()
                            hist.userid = 1

                            if success:
                                hist.taskid = 115
                                hist.value1 = expired.strftime('%d.%m.%Y') if expired is not None else 'leer'
                            # else:
                            #     if expired is not None:
                            #         hist.taskid = 117
                            #         hist.value1 = expired.strftime('%d.%m.%Y') if expired is not None else 'leer'
                            #     else:
                            #         hist.taskid = 116

                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile,
                                                                 gdocid=row[0], logid=self.name)

                        hist = bitfarm_doc.BitfarmHistory()
                        hist.userid = 1
                        hist.taskid = 127
                        try:
                            if arcid in self.bfaserver.archives[self._job.profile]:
                                hist.value1 = self.bfaserver.archives[self._job.profile][arcid].arcname
                            else:
                                hist.value1 = 'unbekannt'
                        except:
                            self.log.exception("%s Fehler beim Abfragen des Archivnamens" % self.name)
                            hist.value1 = 'unbekannt'
                        hist.value2 = arcid
                        self.bfaProcessing.write_history(history=hist, profile=self._job.profile,
                                                                 gdocid=0, logid=self.name)
                else:
                    hist = bitfarm_doc.BitfarmHistory()
                    hist.userid = 1
                    hist.taskid = 130
                    hist.value1 = str(arcid)
                    self.bfaProcessing.write_history(history=hist, profile=self._job.profile,
                                                             gdocid=0, logid=self.name)
        time.sleep(1)
        hist = bitfarm_doc.BitfarmHistory()
        hist.userid = 1
        hist.taskid = 129
        try:
            if arcid in self.bfaserver.archives[self._job.profile]:
                hist.value1 = self.bfaserver.archives[self._job.profile][arcid].arcname
            else:
                hist.value1 = 'unbekannt'
        except:
            self.log.exception("%s Fehler beim Abfragen des Archivnamens" % self.name)
            hist.value1 = 'unbekannt'
        hist.value2 = arcid
        self.bfaProcessing.write_history(history=hist, profile=self._job.profile,
                                                 gdocid=0, logid=self.name)

        self.log.info("%s Neuberechnung durchgeführt" % self.name)

    def reqCalculateExpiredDate(self):
        self.log.debug("%s reqCalculateExpiredDate" % self.name)

        success, expired = self.bfaProcessing.recalculate_expired_date(self._job.gdocid, self._job.profile)
        hist = bitfarm_doc.BitfarmHistory()
        hist.userid = 1

        if success:
            hist.taskid = 115
            hist.value1 = expired.strftime('%d.%m.%Y') if expired is not None else 'leer'
        # else:
        #     if expired is not None:
        #         hist.taskid = 117
        #         hist.value1 = expired.strftime('%d.%m.%Y') if expired is not None else 'leer'
        #     else:
        #         hist.taskid = 116

            self.bfaProcessing.write_history(history=hist, profile=self._job.profile,
                                             gdocid=self._job.gdocid, logid=self.name)
        if success:
            xml = bitfarm_xml.create_xml_request(reqtype="respOK", gdocid=self._job.gdocid, data=_("reqCalculateExpiredDate ausgeführt"))
        else:
            xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=self._job.gdocid)

        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def set_expired_for_archive(self):
        self.log.debug("%s set_expired_for_archive" % self.name)

        duration = self._job.parameter[0]
        interval = self._job.parameter[1]

        hist = bitfarm_doc.BitfarmHistory()
        hist.userid = 1
        hist.taskid = 131
        try:
            if self._job.arcid in self.bfaserver.archives[self._job.profile]:
                hist.value1 = self.bfaserver.archives[self._job.profile][self._job.arcid ].arcname
            else:
                hist.value1 = 'unbekannt'
        except:
            self.log.exception("%s Fehler beim Abfragen des Archivnamens" % self.name)
            hist.value1 = 'unbekannt'
        hist.value2 = self._job.arcid
        self.bfaProcessing.write_history(history=hist, profile=self._job.profile,
                                                 gdocid=0, logid=self.name)

        sql = 'select gdocid, docdate from sys_doc where arcid = %s and expired_setby is Null' % self._job.arcid    # nur dokumente ohne händische eingabe

        try:
            result = self.db[self._job.profile].execute(sql).fetchall()
        except:
            self.log.exception("%s Es hat einen Fehler beim Setzen der Dokumentenlebensdauer gegeben")

        if result:
            sql = "insert into sys_config set name = 'recalculate_%s', value = '%s'" % (self.name, self._job.arcid)
            try:
                self.db[self._job.profile].execute(sql)
            except:
                self.log.exception("%s Fehler beim Setzen des Thread-Locks in der sys_config" % self.name)

            sql = "update sys_doc set expired_flags = 1 where arcid = %s and expired_setby is Null" % self._job.arcid
            try:
                self.db[self._job.profile].execute(sql)
            except:
                self.log.exception("%s Fehler beim Setzen des recalculate-Flags" % self.name)
            else:
                for row in result:
                    newdate = None
                    if interval == 'd':
                        newdate = row['docdate'] + relativedelta(days=+ duration)
                    elif interval == 'w':
                        newdate = row['docdate'] + relativedelta(weeks=+ duration)
                    elif interval == 'm':
                        newdate = row['docdate'] + relativedelta(months=+ duration)
                    elif interval == 'y':
                        newdate = row['docdate'] + relativedelta(years=+ duration)
                    elif interval == 'c':
                        newdate = None

                    if newdate is not None:
                        sql = "update sys_doc set expired = '%s', expired_userid = 1, expired_flags = NULL where gdocid = %s" % (newdate.strftime('%Y-%m-%d %H:%M:%S'), row['gdocid'])
                    else:
                        sql = "update sys_doc set expired = NULL, expired_userid = 1,  expired_flags = NULL where gdocid = %s" % (row['gdocid'])

                    try:
                        self.db[self._job.profile].execute(sql)
                    except:
                        self.log.exception("%s Es hat einen Fehler beim Setzen der Dokumentenlebensdauer gegeben")

                    hist = bitfarm_doc.BitfarmHistory()
                    hist.userid = 1     # Die Änderung am Dokument macht ja der Server
                    hist.taskid = 120
                    hist.value1 = newdate.strftime('%d.%m.%Y') if newdate is not None else 'leer'

                    self.bfaProcessing.write_history(history=hist, profile=self._job.profile,
                                                         gdocid=row['gdocid'], logid=self.name)

                sql = "delete from sys_config where name = 'recalculate_%s' and value = '%s'" % (self.name, self._job.arcid)
                try:
                    self.db[self._job.profile].execute(sql)
                except:
                    self.log.exception("%s Fehler beim Setzen des Thread-Locks in der sys_config" % self.name)
        else:
            hist = bitfarm_doc.BitfarmHistory()
            hist.userid = 1
            hist.taskid = 130
            hist.value1 = str(self._job.arcid)
            self.bfaProcessing.write_history(history=hist, profile=self._job.profile,
                                                     gdocid=0, logid=self.name)
        time.sleep(1)
        hist = bitfarm_doc.BitfarmHistory()
        hist.userid = 1
        hist.taskid = 129
        try:
            if self._job.arcid in self.bfaserver.archives[self._job.profile]:
                hist.value1 = self.bfaserver.archives[self._job.profile][self._job.arcid].arcname
            else:
                hist.value1 = 'unbekannt'
        except:
            self.log.exception("%s Fehler beim Abfragen des Archivnamens" % self.name)
            hist.value1 = 'unbekannt'
        hist.value2 = self._job.arcid
        self.bfaProcessing.write_history(history=hist, profile=self._job.profile,
                                                 gdocid=0, logid=self.name)

        self.log.debug("%s set_expired_for_archive done" % self.name)

    def create_new_addfield(self, elem, _con, error):
        _idx = None
        _ind = 0
        for idx in _con.sys_docidx.values():
            row_length = bitfarm_proc.calc_row_length(idx.columns) + bitfarm_doc.FIELD_SIZES[elem.type]
            if row_length < _con.max_row_size:
                self.log.debug("row_length: %s" % row_length)
                _idx = idx
                _ind = int(idx.name[11:])
                break
            else:
                _ind += 1
        self.log.debug("idx: %s ind: %s" % (str(_idx), _ind))
        if _idx is None:  # weitere Tabelle erzeugen
            self.log.debug("weitere Tabelle anlegen sys_docidx_%i" % _ind)
            sql = "CREATE TABLE `sys_docidx_%i` ( \
                  `docidxid` bigint(20) NOT NULL AUTO_INCREMENT, \
                  `gdocid` bigint(20) NOT NULL, \
                  `modified` TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP, \
                  PRIMARY KEY (`docidxid`), \
                  UNIQUE INDEX uq_gdocid (gdocid) \
                ) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;" % _ind
            try:
                self.db[self._job.profile].execute(sql)
            except:
                msg = _("Fehler beim Erstellen einer Metadaten-Tabelle") % (self.name, _ind)
                self.log.exception("%s %s sys_docidx_%i" % (self.name, msg, _ind))
                error.append(msg)
                _idx = None
            else:
                msg = _("%s Weitere Zusatzfeldtabelle angelegt sys_docidx_%i") % (self.name, _ind)
                self.log.debug(msg)
                _idx = sqla.Table('sys_docidx_%i' % _ind, _con.meta, autoload=True)
                _con.sys_docidx[_ind] = _idx

        if _idx is not None:
            # Spalte anlegen

            sql = "ALTER TABLE `%s` LOCK = SHARED, ADD COLUMN `%s` %s NULL;" % (_idx.name, elem.add, bitfarm_doc.FIELD_TYPES[elem.type])

            try:
                self.db[self._job.profile].execute(sql)
            except:
                msg = _("Fehler beim Erstellen einer Spalte in der Metadaten-Tabelle")
                self.log.exception("%s %s add %s idx %s" % (self.name, msg, elem.add, _idx.name))
                error.append(msg)
                _idx = sqla.Table(_idx.name, sqla.MetaData(), autoload=True, autoload_with=_con.engine)
                _con.sys_docidx[_ind] = _idx   # _con.sys_docidx aktualisieren
                _idx = None
            else:
                self.log.debug(_("%s Zusatzfeldspalte angelegt %s (%s)") % (self.name, elem.add, elem.name)) 
                _idx = sqla.Table(_idx.name, sqla.MetaData(), autoload=True, autoload_with=_con.engine)
                _con.sys_docidx[_ind] = _idx   # _con.sys_docidx aktualisieren
                elem.indextableid = _ind

                # Zusatzfeld, indextableid in die sys_addfields aktualisieren
                sql = sqla.select([_con.sys_addfields]).where(_con.sys_addfields.c.fieldname == elem.add)
                try:
                    res = self.db[self._job.profile].execute(sql)
                except:
                    msg = _("Fehler beim Abfragen des Zusatzfeldes in der sys_addfields")
                    self.log.exception("%s %s add %s" % (self.name, msg, elem.add))
                    error.append(msg)
                else:
                    if res.rowcount > 0:
                        vals = {_con.sys_addfields.c.indextableid: elem.indextableid}
                        sql = sqla.update(_con.sys_addfields, values=vals).where(_con.sys_addfields.c.fieldname == elem.add)
                    else:
                        self.log.debug("Feld %s wird neu angelegt" % elem.id)
                        vals = {_con.sys_addfields.c.addfieldid: elem.id,
                                _con.sys_addfields.c.displayname: elem.name,
                                _con.sys_addfields.c.fieldname: elem.add,
                                _con.sys_addfields.c.type: elem.type,
                                _con.sys_addfields.c.mode: elem.mode,
                                _con.sys_addfields.c.options: elem.options,
                                _con.sys_addfields.c.checkprogram: elem.checkprogram,
                                _con.sys_addfields.c.description: elem.description,
                                _con.sys_addfields.c.values_query: elem.values_query,
                                _con.sys_addfields.c.indextableid: elem.indextableid,
                                _con.sys_addfields.c.flags: elem.flags
                                }
                        if elem.items is not None and len(elem.items) > 0:
                            if type(elem.items) is list:
                                vals[_con.sys_addfields.c['items']] = ";".join(elem.items)
                            else:
                                vals[_con.sys_addfields.c['items']] = elem.items
                        sql = sqla.insert(_con.sys_addfields, values=vals)
                    try:
                        self.db[self._job.profile].execute(sql)
                    except:
                        msg = _("Fehler beim Erstellen eines Zusatzfeldes")
                        self.log.exception("%s %s add %s" % (self.name, msg, elem.add))
                        error.append(msg)
                    else:
                        self.log.info(_("%s Zusatzfeld angelegt %s (%s)") % (self.name, elem.add, elem.name))
        return error

    def reqCheckin(self):
        self.log.debug('%s reqCheckin' % self.name)
        _con = self.bfaserver.con.connections[self._job.profile]
        xml = None
        svninfo = ''
        username = ''

        params = self._job.parameter.split(';')
        if len(params) > 2:
            svninfo = params[1]
            username = params[2]

        doc = bitfarm_doc.BitfarmDocument()
        doc.profile = self._job.profile
        if not self.bfaProcessing.load(doc, self._job.gdocid, ['base', 'paths'], logid=self.name):
            xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=self._job.gdocid)
        else:
            if not doc.is_checkout:
                xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=self._job.gdocid,
                                                     data=_("Das Dokument war nicht ausgecheckt"))

            rights = _con.read_rights(db=self.db[self._job.profile], arcid=doc.arcid, user=self._job.userid)

            if doc.checkout_userid != self._job.userid and not rights[0].admin:
                xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=self._job.gdocid,
                                                     data=_("Das Dokument wurde von einem anderen Benutzer ausgecheckt"))

            if xml is None:
                if doc.checkout_userid != self._job.userid:  # wird das dokument von einem admin wieder eingecheckt steht in paramter die userid
                    userid = int(params[0])        # des auscheckenden users
                else:
                    userid = self._job.userid

                self.bfaProcessing.checkin(doc, userid)

                if not doc.is_checkout:
                    if not doc.is_revisionsave:  # wenn es wieder eingecheckt wurde und nicht revisionssicher liegt
                        folder = os.path.expandvars('%bitfarm-archiv%\checkout\\' + str(self._job.gdocid))  # versuchen wir die datei zu synchronisieren
                        fname = doc.filename if doc.filename else doc.title if doc.title else _('Dokument[%s]') % self._job.gdocid
                        fname += os.path.splitext(doc.oriloc)[1]
                        fname = bitfarm_doc.parse_filename(fname)
                        fpath = os.path.join(folder, fname)
                        if os.path.exists(fpath):
                            try:
                                shutil.copyfile(fpath, os.path.expandvars(doc.oriloc))
                            except:
                                self.log.exception(_("%s Die eingecheckte Datei konnte nicht im Archiv gespeichert werden" ) % self.name)
                                xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0,
                                                                     gdocid=self._job.gdocid, data=_( "Die eingecheckte Datei konnte nicht im Archiv gespeichert werden"))
                            else:
                                xml = bitfarm_xml.create_xml_request(reqtype="respOK", gdocid=self._job.gdocid, data=_("reqCheckin ausgeführt"))

                                success, xml = self.write_reocr_file(doc, username, svninfo, self._job.gdocid)
                                if success:
                                    if (self._job.gdocid, fpath) in self.bfaserver.cached:  # prüfen, ob wir die datei im cache haben
                                        BitfarmWorkerThread.cachelock.acquire()
                                        try:
                                            self.bfaserver.cache[(self._job.gdocid, fpath)].close()
                                            del self.bfaserver.cache[(self._job.gdocid, fpath)]
                                            self.bfaserver.cached.remove((self._job.gdocid, fpath))
                                        finally:
                                            BitfarmWorkerThread.cachelock.release()
                                    fpath_preview = doc.extloc if doc.extloc else doc.docloc    # prüfen, ob wir die voransicht im cache haben
                                    if (self._job.gdocid, fpath_preview) in self.bfaserver.cached:
                                        BitfarmWorkerThread.cachelock.acquire()
                                        try:
                                            self.bfaserver.cache[(self._job.gdocid, fpath_preview)].close()
                                            del self.bfaserver.cache[(self._job.gdocid, fpath_preview)]
                                            self.bfaserver.cached.remove((self._job.gdocid, fpath_preview))
                                        finally:
                                            BitfarmWorkerThread.cachelock.release()

                                    hist = bitfarm_doc.BitfarmHistory()
                                    hist.gdocid = self._job.gdocid
                                    hist.taskid = 229
                                    hist.userid = self._job.userid
                                    self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)
                                    self.check_abo(self._job.gdocid)

                                try:
                                    os.remove(fpath)
                                    os.rmdir(os.path.dirname(fpath))
                                except:
                                    self.log.exception(_("%s Die ausgecheckte Datei %s konnte nicht wieder entfernt werden") % (self.name, fpath))
                        else:
                            xml = bitfarm_xml.create_xml_request(reqtype="respOK", gdocid=self._job.gdocid, data=_("reqCheckin ausgeführt"))
                    else:
                        xml = bitfarm_xml.create_xml_request(reqtype="respOK", gdocid=self._job.gdocid, data=_("reqCheckin ausgeführt"))

                    hist = bitfarm_doc.BitfarmHistory()
                    hist.gdocid = self._job.gdocid
                    hist.taskid = 203
                    hist.userid = self._job.userid
                    self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)
                else:
                    xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=self._job.gdocid,
                                                         data=_("Dokument konnte nicht eingecheckt werden"))

        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqCheckout(self):
        self.log.debug('%s reqCheckout' % self.name)
        xml = None
        doc = bitfarm_doc.BitfarmDocument()
        doc.profile = self._job.profile

        if self._job.gdocid in self.bfaserver.in_progress[self._job.profile] and self.bfaserver.in_progress[self._job.profile][self._job.gdocid] != self._job.userid:
            xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=self._job.gdocid,
                                                     data=_("Das Dokument wird von einem anderen Benutzer bearbeitet."))

        elif not self.bfaProcessing.load(doc, self._job.gdocid, ['base', 'paths'], logid=self.name):
            xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=self._job.gdocid)
        else:
            if doc.is_checkout and doc.checkout_userid != self._job.userid:
                xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=self._job.gdocid,
                                                     data=_("Das Dokument wurde von einem anderen Benutzer ausgecheckt."))

            if xml is None:
                self.bfaProcessing.checkout(doc, self._job.userid)

                if not doc.is_checkout:
                    xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=self._job.gdocid,
                                                         data=_("Das Dokument konnte nicht ausgecheckt werden."))
                else:
                    xml = bitfarm_xml.create_xml_request(reqtype="respOK", gdocid=self._job.gdocid, data=_("reqCheckout ausgeführt"))
                    hist = bitfarm_doc.BitfarmHistory()
                    hist.gdocid = self._job.gdocid
                    hist.taskid = 202
                    hist.userid = self._job.userid
                    self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)
        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqGetCheckouts(self):
        self.log.debug('%s reqGetCheckouts' % self.name)
        _con = self.bfaserver.con.connections[self._job.profile]

        try:
            sql = 'select gdocid from sys_doc where checkout = %i' % self._job.userid

            result = self.db[self._job.profile].execute(sql).fetchall()
        except:
            self.log.exception(u'Fehler bei der Abfrage der ausgecheckten Dokumente')
            xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=self._job.gdocid,
                                                 data=_(u'Fehler bei der Abfrage der ausgecheckten Dokumente'))
        else:
            data = []
            if result:
                for row in result:
                    data.append(str(row[0]))
                data = ";".join(data)
            xml = bitfarm_xml.create_xml_request(reqtype="respGetCheckouts", userid=0, gdocid=self._job.gdocid,
                                                 data=data)
        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_(u"%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)

        return None

    def reqSyncDoc(self):
        self.log.debug('%s reqSyncDoc' % self.name)
        doc = bitfarm_doc.BitfarmDocument()
        doc.profile = self._job.profile
        if not self.bfaProcessing.load(doc, self._job.gdocid, ['base', 'paths'], logid=self.name):
            xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=self._job.gdocid)
        else:
            xml = None
            if doc.oriloc:
                fpath = doc.oriloc
            else:
                xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=self._job.gdocid,
                                                     data=_("Keine Originaldatei vorhanden"))

            if not doc.is_checkout:
                xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=self._job.gdocid,
                                                     data=_("Das Dokument war nicht ausgecheckt"))

            elif doc.checkout_userid != self._job.userid:
                xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=self._job.gdocid,
                                                     data=_("Das Dokument wurde von einem anderen Benutzer ausgecheckt"))

            if xml is None:
                try:
                    folder = os.path.expandvars('%bitfarm-archiv%\checkout\\' + str(self._job.gdocid))
                    fpath = os.path.join(folder, self._job.filename)
                    if not os.path.exists(folder):
                        try:
                            os.makedirs(folder)
                        except:
                            self.log.exception(_("%s Konnte Verzeichnis %s nicht anlegen") % (self.name, folder))
                            fpath = None

                    if self._job.filedate and fpath and os.path.exists(fpath):
                        fpathstats = os.stat(fpath)
                        if datetime.datetime.fromtimestamp(fpathstats.st_mtime).replace(microsecond=0) > self._job.filedate:  # wir haben auf der Platte eine aktuellere Version
                            xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=self._job.gdocid,
                                                                 data=_("Gesendetes Dokument veraltet. Bitte erneut auschecken"))
                            fpath = None

                    if fpath:
                        xml = bitfarm_xml.create_xml_request(reqtype="respSendFile", gdocid=self._job.gdocid)
                        try:
                            self._job.sock.send(xml)
                        except:
                            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                            return None

                        self.log.debug(_("%s Empfange Datei, Größe %d") % (self.name, self._job.datalen))
                        # with open(os.path.expandvars(fpath), 'wb') as f:
                        #     received = 0
                        #     while received < self._job.datalen:
                        #         buff = self._job.sock.recv(1024)
                        #         received += len(buff)
                        #         if not buff:
                        #             break
                        #         f.write(buff)

                        # stream = cStringIO.StringIO()
                        stream = mmap.mmap(-1, self._job.datalen)
                        received = 0
                        xml = None
                        try:
                            while received < self._job.datalen:
                                buff = self._job.sock.recv(8192)
                                received += len(buff)
                                if not buff:
                                    break
                                try:
                                    stream.write(buff)
                                except:     # es können tatsächlich auch mehr bytes über den Socket kommen. Das führt zu einem ValueError Out of bounds, weil der mmap Stream ja auf datalen festgelegt wurde
                                    self.log.exception("%s Fehler beim Empfangen der Daten" % self.name)
                                    break

                            if stream.tell() == 0:
                                self.log.error("Fehler: 0kb Datei erhalten")
                                xml = bitfarm_xml.create_xml_request(reqtype="respError",
                                                         data="Fehler: leere Datei erhalten.")
                            elif stream.tell() != self._job.datalen:
                                self.log.error("Fehler: Datei unvollständig erhalten %d von %d Bytes " % (stream.tell(), self._job.datalen))
                                xml = bitfarm_xml.create_xml_request(reqtype="respError",
                                                         data="Fehler beim Empfangen der Datei, Daten unvollständig.")
                            else:
                                self.log.debug(_("%s Schreibe Datei %s") % (self.name, fpath))
                                stream.seek(0)
                                bufferlen = 4096
                                bytes_to_write = self._job.datalen
                                with open(os.path.expandvars(fpath), 'wb') as f:
                                    while bytes_to_write > 0:
                                        f.write(stream.read(bufferlen))
                                        bytes_to_write -= bufferlen
                                        if bytes_to_write < bufferlen:
                                            bufferlen = bytes_to_write

                                stream.close()
                        except:
                            self.log.exception("%s Fehler beim Empfangen der Datei" % self.name)
                            xml = bitfarm_xml.create_xml_request(reqtype="respError",
                                                         data="Fehler beim Empfangen der Datei.")

                        if not xml:     # kein Fehler bis hierhin?
                            if not self.bfaProcessing.write(doc, logid=self.name):
                                xml = bitfarm_xml.create_xml_request(reqtype="respError",
                                                                     data="Fehler beim Speichern des Dokuments.")
                            else:
                                hist = bitfarm_doc.BitfarmHistory()
                                hist.gdocid = self._job.gdocid
                                hist.taskid = 224
                                hist.userid = self._job.userid
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                                BitfarmWorkerThread.in_progresslock.acquire()
                                try:
                                    if self._job.gdocid in self.bfaserver.in_progress[self._job.profile]:
                                        del self.bfaserver.in_progress[self._job.profile][self._job.gdocid]
                                finally:
                                    BitfarmWorkerThread.in_progresslock.release()
                                # self.bfaProcessing.unset_editmode(self._job, self._job.userid, docids=self._job.gdocid)

                                if (self._job.gdocid, fpath) in self.bfaserver.cached:
                                    BitfarmWorkerThread.cachelock.acquire()
                                    try:
                                        self.bfaserver.cache[(self._job.gdocid, fpath)].close()
                                        del self.bfaserver.cache[(self._job.gdocid, fpath)]
                                        self.bfaserver.cached.remove((self._job.gdocid, fpath))
                                    finally:
                                        BitfarmWorkerThread.cachelock.release()

                                fpath_preview = doc.extloc if doc.extloc else doc.docloc    # prüfen, ob wir die voransicht im cache haben
                                if (self._job.gdocid, fpath_preview) in self.bfaserver.cached:
                                    BitfarmWorkerThread.cachelock.acquire()
                                    try:
                                        self.bfaserver.cache[(self._job.gdocid, fpath_preview)].close()
                                        del self.bfaserver.cache[(self._job.gdocid, fpath_preview)]
                                        self.bfaserver.cached.remove((self._job.gdocid, fpath_preview))
                                    finally:
                                        BitfarmWorkerThread.cachelock.release()

                                if self._job.parameter is not None and 'checkin' in self._job.parameter:
                                    self.reqCheckin()
                                    return True
                                else:
                                    xml = bitfarm_xml.create_xml_request(reqtype="respOK", gdocid=self._job.gdocid, data=_("reqSyncDoc ausgeführt"))
                except:
                    self.log.exception(_("%s Die Datei %s konnte nicht geschrieben werden") % (self.name, fpath))
                    xml = bitfarm_xml.create_xml_request(reqtype="respError",
                                                         data="Fehler beim Speichern der Datei")
        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqArchive(self):
        self.log.debug('%s reqArchive' % self.name)
        doc = self._job.data
        _con = self.bfaserver.con.connections[self._job.profile]
        params = self._job.parameter.split(';')
        if not doc.profile:
            doc.profile = self._job.profile
        rights = _con.read_rights(db=self.db[self._job.profile], arcid=doc.arcid, user=self._job.userid)

        if self.bfaserver.develop:
            with open('reqArchive.xml', 'w') as fn:
                bitfarm_xml.object_to_xmlfile(self._job, fn)

        if len(rights) == 0 or not rights[0].docimport:
            xml = bitfarm_xml.create_xml_request(reqtype="respDenied",
                                                 data=_("Keine Berechtigung, das Dokument zu Archivieren"))
        elif _con.clientspooler is None:         # wir nehmen clientspooler statt unsclientspooler, damit shutil.move Dateien nur umbenennen braucht, statt zu kopieren
            self.log.error("Achtung: kein clientspooler-Pfad!")
            xml = bitfarm_xml.create_xml_request(reqtype="respError",
                                                 data=_("Fehler in der Konfiguration, Archivierung nicht möglich."))
        elif not os.path.exists(os.path.join(self.bfaserver.mypath, 'temp')):
            self.log.error("Achtung: kein temp-Verzeichnis!")
            xml = bitfarm_xml.create_xml_request(reqtype="respError",
                                                 data=_("Fehler in der Konfiguration, Archivierung nicht möglich."))
        elif self._job.datalen is None:
            self.log.error("Achtung: keine Datalen übergeben")
            xml = bitfarm_xml.create_xml_request(reqtype="respError",
                                                 data=_("Fehler: Defekte oder leere Datei übertragen."))
        else:
            doc.write_empty_values = True

            fbase = datetime.datetime.now().strftime('%d%m%Y%H%M%S%f') + self.name[7:]
            if 'fast' in params:
                fbase = 'fast' + fbase
            elif self._job.filename:
                fbase = self._job.filename + fbase
            fext = params[0]

            while os.path.exists(os.path.join(_con.clientspooler, fbase + '.job')):
                fbase = datetime.datetime.now().strftime('%d%m%Y%H%M%S%f') + self.name[7:]
                if 'fast' in params:
                    fbase = 'fast' + fbase
                elif self._job.filename:
                    fbase = self._job.filename + fbase
            doctags = []
            if doc.doctags is not None and doc.doctags != "":
                doc.doctags = doc.doctags.replace('\f', ';')
                doc.doctags = re.sub(r'[\x00-\x08\x0b-\x1f\x7f-\x9f]', '', doc.doctags)  # ascii steuerzeichen parsen, außer tab und lf
                for tag in doc.doctags.split(';'):
                    if tag and tag not in doc.tags:
                        doc.tags.append(tag)

            doctags = doc.tags[:]
            doc.tags = []

            if type(doc.links) is not list:
                doc.links = doc.links.split(";")

            # Pfade zurücksetzen
            doc.docloc = ""
            doc.extloc = ""
            doc.oriloc = ""

            if type(doc.statefields) is list:
                for idx in range(len(doc.statefields)):
                    if not doc.statefields[idx].bitmask:
                        sql = 'select bitmask, indexcolumnid from sys_statefields where stateid = %s' % doc.statefields[idx].id
                        try:
                            result = self.db[self._job.profile].execute(sqla.text(sql)).fetchone()
                        except:
                            self.log.exception(_("%s Fehler bei Abfrage der Bitmaske des Statusfeldes.") % self.name)
                        else:
                            doc.statefields[idx].bitmask = int(result[0])
                            doc.statefields[idx].indexcolumnid = int(result[1])
            elif type(doc.statefields) in [dict, OrderedDict]:
                for idx in list(doc.statefields.keys()):
                    if not doc.statefields[idx].bitmask:
                        sql = 'select bitmask, indexcolumnid from sys_statefields where stateid = %s' % doc.statefields[idx].id
                        try:
                            result = self.db[self._job.profile].execute(sqla.text(sql)).fetchone()
                        except:
                            self.log.exception(_("%s Fehler bei Abfrage der Bitmaske des Statusfeldes.") % self.name)
                        else:
                            doc.statefields[idx].bitmask = int(result[0])
                            doc.statefields[idx].indexcolumnid = int(result[1])

            if not self.bfaProcessing.write_to_file(doc, os.path.join(_con.clientspooler, fbase + '.job'), _con.usestatus35):
                self.log.exception(_("%s Die Datei %s konnte nicht geschrieben werden") % (self.name, os.path.join(_con.clientspooler, fbase + '.job')))
                xml = bitfarm_xml.create_xml_request(reqtype="respError", data="Fehler beim Speichern der Job-Datei.")
            else:
                try:
                    with codecs.open(os.path.join(_con.clientspooler, fbase + '.slw'), 'wb', encoding='cp1252') as f:
                        fname = self._job.filename if self._job.filename else ''
                        if fname:
                            f.write('Dateiname=%s\r\n' % fname)
                        for tag in doctags:
                            f.write('%s\r\n' % tag)
                        f.write('\r\n')
                except:
                    self.log.exception("Fehler beim Schreiben der .slw Datei")

                xml = bitfarm_xml.create_xml_request(reqtype="respSendFile")
                try:
                    self._job.sock.send(xml)
                except:
                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                    return None

                tpath = os.path.join(self.bfaserver.mypath, 'temp', fbase + fext)
                self.log.debug(_("%s Schreibe Datei %s") % (self.name, tpath))
                try:
                    with open(os.path.expandvars(tpath), 'wb') as f:
                        received = 0
                        while received < self._job.datalen:
                            buff = self._job.sock.recv(8192)
                            received += len(buff)
                            if not buff:
                                break
                            f.write(buff)
                except:
                    self.log.exception(_("%s Die Datei %s konnte nicht geschrieben werden") % (self.name, tpath))
                    xml = bitfarm_xml.create_xml_request(reqtype="respError", data="Fehler beim Speichern der Datei.")

                if received < self._job.datalen:
                    self.log.error(_("%s Die Datei %s wurde nicht vollständig empfangen, Archivierung abgebrochen...") % (self.name, tpath))

                    if os.path.exists(tpath):
                        try:
                            os.remove(tpath)
                        except:
                            self.log.exception(_("%s Fehler beim Löschen von %s") % (self.name, tpath))

                    if os.path.exists(os.path.join(_con.clientspooler, fbase + '.slw')):
                        try:
                            os.remove(os.path.join(_con.clientspooler, fbase + '.slw'))
                        except:
                            self.log.exception(_("%s Fehler beim Löschen von %s") % (self.name, os.path.join(_con.clientspooler, fbase + '.slw')))

                    if os.path.exists(os.path.join(_con.clientspooler, fbase + '.job')):
                        try:
                            os.remove(os.path.join(_con.clientspooler, fbase + '.job'))
                        except:
                            self.log.exception(_("%s Fehler beim Löschen von %s") % (self.name, os.path.join(_con.clientspooler, fbase + '.job')))

                    xml = bitfarm_xml.create_xml_request(reqtype="respError", data="Archivierung wurde abgebrochen.")
                else:
                    if len(doc.annotations) > 0:
                        for anno in doc.annotations:
                            if not anno.iev:
                                continue
                            suffix = str(anno.page) + ('P%s' % anno.layer if anno.layer > 0 else '')
                            annopath = os.path.join(_con.clientspooler, fbase + suffix + '.iev')
                            with open(os.path.expandvars(annopath), 'wb') as f:
                                f.write(anno.iev.stream.getvalue())

                    fpath = os.path.join(_con.clientspooler, fbase + fext)
                    try:
                        shutil.move(os.path.expandvars(tpath), os.path.expandvars(fpath))
                    except:
                        self.log.exception(_("%s Die Datei %s konnte nicht in die Übergabe kopiert werden") % (self.name, tpath))
                        xml = bitfarm_xml.create_xml_request(reqtype="respError",
                                                             data="Die Datei %s konnte nicht in die Übergabe kopiert werden" % tpath)
                    else:
                        xml = bitfarm_xml.create_xml_request(reqtype="respOK", gdocid=self._job.gdocid, data=_("reqArchive ausgeführt"))

        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqCreateDocument(self):
        """
        Erzeugt ein neues Dokument in der Datenbank und empfängt die Dokumentdateien über den Socket.\012
        Als Parameter wird eine Liste übergeben mit der arcid des Zielarchives an erster Stelle, das Quellprofil, die Ursprungs-ID,
        und dann jeweils ein Tupel für jede Dokumentdatei. Im Tupel steht der Ablageort ('docloc', 'extloc', 'oriloc', 'srcloc'),
        die Größe in Bytes, die Dateiendung und entweder 'rs' oder 'nrs', für den Ablageort (extloc wir immer nrs sein).\012
        Beispiel params = [123, ('docloc',65535,'.tif','rs'), ('extloc',‭32767‬,'.tif', 'nrs')].\012
        Im Zielarchiv benötigt man Adminberechtigung.
        Für jede Datei die gesendet werden soll, antwortet der Server mit 'respSendFile', nach der letzten gesendeten Datei
        antwortet der Server mit respOK und dem neu angelegten DocObject.
        @param bfaReq.parameter: list [arcid, 'quellprofile', quell-gdocic, ('docloc',65535,'.tif','rs')]
        @return:    bfaDocument-Objekt oder respError
        """

        self.log.debug('%s reqCreateDocument' % self.name)
        doc = bitfarm_doc.BitfarmDocument()
        doc.profile = self._job.profile
        con = self.bfaserver.con.connections[self._job.profile]
        xml = ""

        params = self._job.parameter
        if len(params) < 4:     # wenigstens arcid, quellprofil, quell id und ein tupel
            xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, data="Nicht genügend Parameter")

        try:
            arcid = params.pop(0)
        except:
            xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, data="Parameter arcid fehlt")

        try:
            src_profile = params.pop(0)
        except:
            xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, data="Parameter src_profile fehlt")

        try:
            src_id = params.pop(0)
        except:
            xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, data="Parameter src_id fehlt")

        if xml:     # Fehler?
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None
        else:
            rights = con.read_rights(db=self.db[self._job.profile], arcid=int(arcid), user=self._job.userid)

            if not rights[0].docimport:
                xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, data=_("Keine Berechtigung auf das Archiv %s") % arcid)
            elif not self.bfaProcessing.create(doc, int(arcid)):
                xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, data=_("Dokument konnte nicht erzeugt werden"))
            else:
                self.bfaProcessing.create_basename(doc)

                arcpath = self.bfaProcessing.create_arcpath(doc, True)
                if not os.path.exists(os.path.expandvars(arcpath)):
                    xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, data=_("Konnte Verzeichnis %s nicht erstellen") % arcpath)

            if xml:     # Fehler?
                try:
                    self._job.sock.send(xml)
                except:
                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                    return None
            else:
                for param in params:
                    xml = ""
                    try:
                        loc, fsize, fext, dest = param
                    except:
                        xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, data="Nicht genügend Parameter")
                    else:
                        if loc == 'extloc' or dest == 'nrs':
                            arcpath = arcpath.replace(con.archive_rs, con.archive_nrs)
                            if not os.path.exists(os.path.expandvars(arcpath)):
                                try:
                                    os.makedirs(os.path.expandvars(arcpath))
                                except:
                                    self.log.warning('%s Konnte Verzeichnis %s nicht erstellen' % (self.name, arcpath))
                                    xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, data="Konnte Verzeichnis %s nicht erstellen" % arcpath)

                        fpath = os.path.join(arcpath, doc.basename + fext)

                    if not xml:     # keine Fehler bis hierhin
                        self.log.debug('%s reqCreateDocument respSendFile' % self.name)
                        xml = bitfarm_xml.create_xml_request(reqtype="respSendFile", gdocid=self._job.gdocid)
                        try:
                            self._job.sock.send(xml)
                        except:
                            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                            return None
                        try:
                            self.log.debug(_("%s Schreibe Datei %s") % (self.name, fpath))
                            with open(os.path.expandvars(fpath), 'wb') as f:
                                received = 0
                                while received < fsize:
                                    buff = self._job.sock.recv(1024)
                                    received += len(buff)
                                    if not buff:
                                        break
                                    f.write(buff)

                            if loc == 'docloc':
                                doc.docloc = fpath
                            elif loc == 'extloc':
                                doc.extloc = fpath
                            elif loc == 'oriloc':
                                doc.oriloc = fpath
                            elif loc == 'srcloc':
                                doc.srcloc = fpath
                            doc.modified_userid = self._job.userid
                            if not self.bfaProcessing.write(doc, logid=self.name):
                                xml = bitfarm_xml.create_xml_request(reqtype="respError", data="Fehler beim Speichern des Dokuments.")
                            else:
                                xml = ""
                        except:
                            self.log.exception(_("%s Die Datei %s konnte nicht geschrieben werden") % (self.name, fpath))
                            xml = bitfarm_xml.create_xml_request(reqtype="respError", data="Fehler beim Speichern der Datei")

                    if xml:     # Fehler?
                        try:
                            self._job.sock.send(xml)
                        except:
                            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)

                        return None

                # wir sind durch, dann sollte es geklappt haben
                xml = bitfarm_xml.create_xml_request(reqtype="respOK", gdocid=doc.gdocid, data=doc,
                                                     datatype='BitfarmDocument')
                try:
                    self._job.sock.send(xml)
                except:
                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)

                hist = bitfarm_doc.BitfarmHistory()
                hist.gdocid = doc.gdocid
                hist.taskid = 134
                hist.userid = self._job.userid
                hist.value1 = src_profile
                hist.value2 = str(src_id)
                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                return True

    def reqAuditProof(self):
        self.log.debug('%s reqAuditProof' % self.name)
        _con = self.bfaserver.con.connections[self._job.profile]
        error = ""
        docids = self._job.parameter.split(';')

        for gdocid in docids:
            doc = bitfarm_doc.BitfarmDocument()
            doc.profile = self._job.profile
            if not self.bfaProcessing.load(doc, gdocid, ['base', 'paths'], logid=self.name):
                error += _("Das Dokument [%s] konnte nicht geladen werden.\n") % gdocid
            else:
                rights = _con.read_rights(db=self.db[self._job.profile], arcid=doc.arcid, user=self._job.userid)
                if len(rights) == 0 or not rights[0].revsave:
                    error += _("Keine Berechtigung, das Dokument [%s] revisionssicher abzulegen.\n") % gdocid
                else:
                    fname = doc.oriloc if doc.oriloc else doc.docloc
                    fname = fname.lower()
                    if _con.archive_rs.lower() in os.path.expandvars(fname):
                        error += _("Das Dokument [%s] liegt bereits revisionssicher.\n") % gdocid
                        continue

                    self.auditprooflock.acquire()
                    try:
                        fpath_rs = self.bfaProcessing.create_arcpath(doc, is_revisionsave=True)
                        if fpath_rs is None:
                            error += _("Es konnte für das Dokument [%s] kein Pfad angelegt werden.\n") % gdocid
                            continue
                        try:
                            if doc.oriloc:
                                fname = os.path.join(fpath_rs, os.path.basename(doc.oriloc))
                                while os.path.exists(os.path.expandvars(fname)):
                                    self.bfaProcessing.create_basename(doc)
                                    fname = doc.basename + os.path.splitext(doc.oriloc)[1]
                                    fname = os.path.join(fpath_rs, fname)
                                shutil.copy(os.path.expandvars(doc.oriloc), os.path.expandvars(fname))
                                doc.oriloc = fname
                            if doc.docloc:
                                fname = os.path.join(fpath_rs, os.path.basename(doc.docloc))
                                while os.path.exists(os.path.expandvars(fname)):
                                    self.bfaProcessing.create_basename(doc)
                                    fname = doc.basename + os.path.splitext(doc.docloc)[1]
                                    fname = os.path.join(fpath_rs, fname)
                                shutil.copy(os.path.expandvars(doc.docloc), os.path.expandvars(fname))
                                doc.docloc = fname

                        except:
                            self.log.exception(_("%s Fehler beim reqAuditProof %s") %(self.name, gdocid)) 
                            error += _("Fehler beim revisionssicheren Ablegen des Dokuments [%s].\n") % gdocid
                            continue
                        else:
                            if not self.bfaProcessing.write(doc, logid=self.name):
                                error += _("Die geänderten Pfade konnte nicht in der Datenbank gespeichert werden [%s]") % gdocid
                            else:
                                hist = bitfarm_doc.BitfarmHistory()
                                hist.gdocid = int(gdocid)
                                hist.taskid = 230
                                hist.userid = self._job.userid
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)
                    finally:
                        self.auditprooflock.release()

        if error:
            xml = bitfarm_xml.create_xml_request(reqtype="respError", data=error, datatype="text")
        else:
            xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("Dokument revisionssicher abgelegt."))
        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqShareLayer(self):
        self.log.debug('%s reqShareLayer' % self.name)
        if type(self._job.data) is list:
            users = self._job.data
        else:
            users = [self._job.data]

        for user in users:
            # erst die drei rechte entfernen
            bitmask = bitfarm_doc.RIGHT_APREAD | bitfarm_doc.RIGHT_APWRITE | bitfarm_doc.RIGHT_APMODIFY
            sql = 'insert into sys_docxs (gdocid, src_userid, userid, groupid, rights) values (%i, %i, %i, 0, 0) on duplicate key update rights = rights &~ (%i)' % (
            self._job.gdocid, self._job.userid, user.id, bitmask)

            try:
                self.db[self._job.profile].execute(sql)
                xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("reqShareLayer ausgeführt"))
            except:
                self.log.exception(_("%s Fehler beim Freigeben eines Layers") %(self.name))
                xml = bitfarm_xml.create_xml_request(reqtype="respError",
                                                     data=_("Fehler beim Freigeben eines Layers (1)"))
                break

            if user.rights > 0:
                sql = 'insert into sys_docxs (gdocid, src_userid, userid, groupid, rights) values (%i, %i, %i, 0, %i) on duplicate key update rights = (rights | %i)' % (
                self._job.gdocid, self._job.userid, user.id, user.rights, user.rights)

                try:
                    self.db[self._job.profile].execute(sql)
                    xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("reqShareLayer ausgeführt"))
                except:
                    self.log.exception(_("%s Fehler beim Freigeben eines Layers") % self.name)
                    xml = bitfarm_xml.create_xml_request(reqtype="respError",
                                                         data=_("Fehler beim Freigeben eines Layers (2)"))
                    break

        if self.bfaserver.develop:
            self.log.debug("%s %s" % (self.name, xml))

        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqAddfields(self):
        self.log.debug('%s reqAddfields' % self.name)

        _con = self.bfaserver.con.connections[self._job.profile]
        try:
            addfields = _con.read_addfields(db=self.db[self._job.profile], userid=self._job.userid, arcid=self._job.arcid)
        except:
            self.log.exception("Zusatzfelder konnten nicht geladen werden")
            xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=self._job.gdocid,
                                                 data=_("Zusatzfelder konnten nicht geladen werden"))
        else:
            _col = ['sys_doc.gdocid']
            _join = []
            for field in addfields:
                idx = _con.sys_docidx[field.indextableid]
                if idx.columns.get(field.add) is not None:
                    _col.append(field.add)
                    if idx not in _join:
                        _join.append(idx)

            sql = 'select %s from sys_doc ' % ", ".join(_col)
            for _j in _join:
                sql += 'left join %s on sys_doc.gdocid = %s.gdocid ' % (_j, _j)
            sql += 'where sys_doc.gdocid = %s' % self._job.gdocid

            # sql = sql.select_from(_from)
            try:
                result = self.db[self._job.profile].execute(sqla.text(sql)).fetchone()
            except:
                self.log.exception("Zusatzfelder konnten nicht geladen werden")
                xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=self._job.gdocid,
                                                     data=_("Zusatzfelder konnten nicht geladen werden"))
            else:
                if result:
                    for field in addfields:
                        if field.add in result:
                            if result[field.add]:
                                # if type(result[field.add]) is str:
                                #     field.oldvalue = result[field.add].decode(_con.db_encoding)
                                #     field.value = result[field.add].decode(_con.db_encoding)
                                # else:
                                field.oldvalue = result[field.add]
                                field.value = result[field.add]

                xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=addfields)
        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
        return None

    def reqAddTag(self):
        self.log.debug('%s reqAddTag' % self.name)
        gdocids = self._job.parameter.split(';')
        tag = gdocids.pop(0)
        _con = self.bfaserver.con.connections[self._job.profile]
        xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("reqAddTag ausgeführt"))

        for gdocid in [int(x) for x in gdocids]:
            doc = bitfarm_doc.BitfarmDocument()
            doc.profile = self._job.profile
            if not self.bfaProcessing.load(doc, gdocid, 'base;text', logid=self.name):
                xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=gdocid,
                                                     data=_("Dokument konnte nicht geladen werden"))
                break

            if tag not in doc.tags:
                doc.tags.append(tag)
            else:
                continue

            doc.userid = self._job.userid

            _tags = ''
            if len(doc.tags) > 0:
                _tags = "\n".join(doc.tags)
                _tags = _tags.encode(_con.db_encoding, "ignore")
                _tags = _tags.decode(_con.db_encoding, "ignore")

            vals = {_con.sys_doctxt.c.doctags: _tags}
            sql = _con.sys_doctxt.update(whereclause=(_con.sys_doctxt.c.gdocid == gdocid), values=vals)

            try:
                self.db[self._job.profile].execute(sql)
            except:
                self.log.exception(_("%s Fehler beim Speichern eines Schlagwortes") % self.name)
                xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=gdocid,
                                                     data=_("Fehler beim Speichern eines Schlagwortes"))
                break
            else:
                xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("reqAddTag ausgeführt"))

        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
        return None

    def reqDocAbo(self):
        self.log.debug('%s reqDocAbo' % self.name)
        gdocids = self._job.parameter.split(';')
        info = gdocids.pop(0)
        xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("reqDocAbo ausgeführt"))

        for gdocid in [int(x) for x in gdocids]:
            sql = "insert into sys_abo (gdocid, userid, info) values (%s, %s, '%s') on duplicate key update info = '%s'" % (
            gdocid, self._job.userid, info, info)

            #self.log.debug(sql)
            try:
                self.db[self._job.profile].execute(sqla.text(sql))
            except:
                self.log.exception(_("%s Fehler beim Speichern eines Abonnements [%s]") %(self.name, gdocid))
                xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=gdocid,
                                                     data=_("Fehler beim Speichern eines Abonnements [%s]") % gdocid)
                break
            else:
                xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("reqDocAbo ausgeführt"))
                hist = bitfarm_doc.BitfarmHistory()
                hist.gdocid = int(gdocid)
                hist.taskid = 204
                hist.userid = self._job.userid
                hist.value1 = info
                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
        return None

    def reqDeleteAbo(self):
        self.log.debug('%s reqDeleteAbo' % self.name)
        gdocids = self._job.parameter.split(';')
        xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("reqDeleteAbo ausgeführt"))

        for gdocid in [int(x) for x in gdocids]:
            sql = "delete from sys_abo where gdocid = %s and userid = %s" % (gdocid, self._job.userid)

            #self.log.debug(sql)
            try:
                self.db[self._job.profile].execute(sqla.text(sql))
            except:
                self.log.exception(_("%s Fehler beim Löschen eines Abonnements [%s]") %(self.name, gdocid))
                xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=gdocid,
                                                     data=_("Fehler beim Löschen eines Abonnements [%s]") % gdocid)
                break
            else:
                xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("reqDeleteAbo ausgeführt"))
                hist = bitfarm_doc.BitfarmHistory()
                hist.gdocid = int(gdocid)
                hist.taskid = 205
                hist.userid = self._job.userid
                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
        return None

    def reqAddNamingRule(self):
        if self._job.gdocid:
            doc = bitfarm_doc.BitfarmDocument()
            doc.profile = self._job.profile
            if not self.bfaProcessing.load(doc, self._job.gdocid, level=["fields", 'text'], logid=self.name):
                xml = bitfarm_xml.create_xml_request(reqtype="respDocError", userid=0, gdocid=self._job.gdocid)
                try:
                    self._job.sock.send(xml)
                except:
                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None
            else:
                txt = doc.fulltxt
        xml = bitfarm_xml.create_xml_request(reqtype="respAddNamingRule", data="response von reqAddNamingRule")
        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)

    def reqAutoname(self):
        if self._job.gdocid:
            doc = bitfarm_doc.BitfarmDocument()
            doc.profile = self._job.profile
            if not self.bfaProcessing.load(doc, self._job.gdocid, level=["fields", 'text'], logid=self.name):
                xml = bitfarm_xml.create_xml_request(reqtype="respDocError", userid=0, gdocid=self._job.gdocid)
                try:
                    self._job.sock.send(xml)
                except:
                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None
            else:
                txt = doc.fulltxt
        xml = bitfarm_xml.create_xml_request(reqtype="respAutoname", data="response von reqAutoname")
        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)

    def reqGetClasses(self):
        if self.classifier is None:
            xml = bitfarm_xml.create_xml_request(reqtype="respDocError", userid=0, data="classifier not loaded")
            self._job.sock.send(xml)
            return
        classes = self.classifier.get_classes()
        xml = bitfarm_xml.create_xml_request(reqtype="respGetClasses", data=classes, shortxml=False)
        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)

    def reqRemoveClassDoc(self):
        self.log.debug("reqRemoveClassDoc")
        if self.classifier is None:
            xml = bitfarm_xml.create_xml_request(reqtype="respDocError", userid=0, data="classifier not loaded")
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return

        if self._job.gdocid:
            self.classifierlock.acquire()
            try:
                self.classifier.remove_classdoc(self._job.gdocid)
            except Exception as e:
                self.log.error("Fehler bei reqRemoveClassDoc:")
                self.log.error(e)
            finally:
                self.classifierlock.release()
            xml = bitfarm_xml.create_xml_request(reqtype="respRemoveClassDoc")
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)

    def reqAddClassDoc(self):
        self.log.debug("reqAddClassDoc")
        if self.classifier is None:
            xml = bitfarm_xml.create_xml_request(reqtype="respDocError", userid=0, data="classifier not loaded")
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

        if self._job.gdocid:
            doc = bitfarm_doc.BitfarmDocument()
            doc.profile = self._job.profile
            if not self.bfaProcessing.load(doc, self._job.gdocid, level=['text'], logid=self.name):
                xml = bitfarm_xml.create_xml_request(reqtype="respDocError", userid=0, gdocid=self._job.gdocid)
                try:
                    self._job.sock.send(xml)
                except:
                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None
            else:
                txt = doc.fulltxt
                classname, rules, calc = self._job.data
        elif self._job.data:
            txt, classname, rules, calc = self._job.data
        self.classifierlock.acquire()
        try:
            self.classifier.add_class_doc(doc.gdocid, txt, classname, rules, calc, db=self.db[self._job.profile])
        finally:
            self.classifierlock.release()
        xml = bitfarm_xml.create_xml_request(reqtype="respAddClassDoc")
        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)

    def reqCreateNamingRule(self):
        self.log.debug("reqCreateNamingRule")
        if self._job.gdocid:

            doc = bitfarm_doc.BitfarmDocument()
            doc.profile = self._job.profile
            if not self.bfaProcessing.load(doc, self._job.gdocid, level=['base', 'text', "fields"], logid=self.name):
                xml = bitfarm_xml.create_xml_request(reqtype="respDocError", userid=0, gdocid=self._job.gdocid)
                try:
                    self._job.sock.send(xml)
                except:
                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None
            else:
                result = self.classifier.create_naming_rules(doc.fulltxt, doc.fields)
                xml = bitfarm_xml.create_xml_request(reqtype="respCreateNamingRule", data=json.dumps(result))
                self._job.sock.send(xml)
        else:
            xml = bitfarm_xml.create_xml_request(reqtype="respDocError", userid=0, gdocid=self._job.gdocid)
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None
            
    def reqClassify(self):
        self.log.debug("reqClassify")
        if self.classifier is None:
            xml = bitfarm_xml.create_xml_request(reqtype="respDocError", userid=0, data="classifier not loaded")
            self._job.sock.send(xml)
            return
        if self._job.gdocid:
            doc = bitfarm_doc.BitfarmDocument()
            doc.profile = self._job.profile
            #if not self.bfaProcessing.load(doc, self._job.gdocid, param, logid=self.name):
            if not self.bfaProcessing.load(doc, self._job.gdocid, level=['text'], logid=self.name):
                xml = bitfarm_xml.create_xml_request(reqtype="respDocError", userid=0, gdocid=self._job.gdocid)
                try:
                    self._job.sock.send(xml)
                except:
                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None
            else:
                txt = doc.fulltxt
        elif self._job.data:
            txt = self._job.data
        try:
            result = self.classifier.classify(txt)
        except:
            self.log.exception("Fehler bei reqClassify")
            xml = bitfarm_xml.create_xml_request(reqtype="respError", data=str(sys.exc_info()))
        else:
            xml = bitfarm_xml.create_xml_request(reqtype="respClassify", data=result)
        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)

    def reqSaveAccountingData(self):
        if self._job.gdocid:
            print(("reqSaveAccountingData", self._job.gdocid))

            sql = "select tblname,gridname from sys_buchungscols"

            # sql = 'SELECT holid, userid_src, userid_dst, start, end from sys_holiday where (userid_src=:userid or userid_dst=:userid) and end>=:enddate;'
            # try:
            #     result = self.db[self._job.profile].execute(sqla.text(sql).bindparams(userid=self._job.userid,
            #                                                                           enddate=datetime.datetime.today().strftime(
            #                                                                               "%Y-%m-%d"))).fetchall()
            result = self.db[self._job.profile].execute(sqla.text(sql)).fetchall()
            cols = dict(result)
            dcols = {v:k for k,v in list(cols.items())}

            # print self._job.userid
            # print self._job.data
            xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("reqSaveAccountingData ausgeführt"))
            self._job.sock.send(xml)

            for rowid, vals in self._job.data:
                # print rowid
                # print vals
                if rowid is not None:
                    keys = [dcols[x[0]] for x in vals]
                    values = [x[1] for x in vals]
                    sql = "UPDATE sys_buchungsdaten SET "
                    for k in keys:
                        sql += k + "=:" + k + ","
                    if sql.endswith(","):
                        sql = sql[:-1]
                    sql += " WHERE gdocid = %d;" % int(self._job.gdocid)

                    #for k,v in zip(keys, values):
                    params = dict(list(zip(keys, values)))
                    result = self.db[self._job.profile].execute(sqla.text(sql).bindparams(**params))

                else:
                    keys = [dcols[x[0]] for x in vals]
                    values = [x[1] for x in vals]
                    sql = "INSERT into sys_buchungsdaten (gdocid, " + ",".join(keys) + ") VALUES (" + str(int(self._job.gdocid))+","
                    for k in keys:
                        sql += ":" + k + ","
                    if sql.endswith(","):
                        sql = sql[:-1]
                    sql += ");"

                    #for k,v in zip(keys, values):
                    params = dict(list(zip(keys, values)))
                    result = self.db[self._job.profile].execute(sqla.text(sql).bindparams(**params))
                    # print result

    def reqLoadAccountingData(self):
            # print "reqLoadAccountingData", self._job.gdocid
            sql = "select tblname,gridname from sys_buchungscols"

            # result = self.db[self._job.profile].execute(sqla.text(sql).bindparams(userid=self._job.userid,
            #                                                                           enddate=datetime.datetime.today().strftime(
            #                                                                               "%Y-%m-%d"))).fetchall()

            result = self.db[self._job.profile].execute(sqla.text(sql)).fetchall()
            cols = dict(result)

            # print cols
            scols = list(cols.keys())
            svals = [cols[x] for x in scols]

            sql = "select idsys_buchungsdaten, " + ", ".join(scols) +  " from sys_buchungsdaten where gdocid=:gdocid"
            result = self.db[self._job.profile].execute(sqla.text(sql).bindparams(gdocid=self._job.gdocid)).fetchall()
            # print result
            data = {}
            for row in result:
                data[row[0]] = dict(list(zip(svals, row[1:])))
            # print data
            xml = bitfarm_xml.create_xml_request(reqtype="respLoadAccountingData", data=data, gdocid=self._job.gdocid)
            self._job.sock.send(xml)


    def reqSavePassword(self):
        self.log.debug('%s reqSavePassword' % self.name)
        _con = self.bfaserver.con.connections[self._job.profile]

        password = self._job.parameter.split(';')
        password = password[0]

        vals = {_con.sys_user.c.userpass: password}
        sql = sqla.update(_con.sys_user, values=vals).where(_con.sys_user.c.userid == self._job.userid)

        try:
            self.db[self._job.profile].execute(sql)
        except:
            msg = _("%s Fehler beim Speichern des Passworts") % self.name

            self.log.exception(msg)
            xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, data=msg)
        else:
            sql = 'update sys_user set flags = flags & ~ 32 where userid = %s' % self._job.userid
            try:
                self.db[self._job.profile].execute(sqla.text(sql))
            except:
                msg = _("%s Fehler beim Zurücksetzen der must_change_password Eigenschaft") % self.name
                self.log.exception(msg)

            hist = bitfarm_doc.BitfarmHistory()
            hist.gdocid = 0  # 0 = Admin Einträge
            hist.userid = self._job.userid
            hist.value1 = str(self._job.userid)
            hist.taskid = 28
            hist.value2 = None
            hist.value3 = None
            self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)
            xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("reqSavePassword ausgeführt"))

            try:
                #08.10.2024 self.log.debug(_con.bfauser.decode().lower())
                if _con.bfauser is not None:
                    self.log.debug(_con.bfauser.decode().lower()) # 08.10.2024
                    username = self.bfaserver.user[self._job.profile][self._job.userid].name
                    if _con.bfauser.decode().lower() == username.lower():
                        password = _con.bitdecrypt(password)
                        _con.bfapass = password.encode('utf8')     # bfauser und bfapass sind bytes
                        if os.path.exists(_con.confile):
                            ini = bitfarm_configobj.ConfigObj(infile=_con.confile, encoding="cp1252", write_empty_values=True)
                            if 'client' in ini:
                                ini['client']['bfapass'] = base64.b64encode(dpapi.cryptData(_con.bfapass)).decode()
                                ini.write()
            except:
                self.log.exception(_('%s Konnte client bfauser [%s] Eintrag im Profil nicht aktualisieren') % (self.name, ))
        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
        return None

    def reqSaveTags(self):
        self.log.debug('%s reqSaveTags' % self.name)
        con = self.bfaserver.con.connections[self._job.profile]
        gdocids = self._job.parameter.split(';')
        if self._job.data is not None:
            _tags = self._job.data.split(';')
            tags = "\n".join(_tags)
            tags = tags.encode(con.db_encoding, "ignore")
            tags = tags.decode(con.db_encoding, "ignore")
        else:
            _tags = []
            tags = ""
        # vals = {con.sys_doc.c.doctags: tags,
        #         con.sys_doc.c.modified_userid: self._job.userid,
        #         con.sys_doc.c.modified: None}
        vals = {con.sys_doc.c.doctags: tags}
        sql = con.sys_doc.update(whereclause=(con.sys_doc.c.gdocid.in_(gdocids)), values=vals)

        try:
            self.db[self._job.profile].execute(sql)
        except:
            self.log.exception(_("%s Fehler beim Speichern eines Schlagwortes") % self.name)
            xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, data=_("Fehler beim Speichern eines Schlagwortes"))
        else:
            xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("reqSaveTags ausgeführt"))
            hist = bitfarm_doc.BitfarmHistory()
            hist.userid = self._job.userid
            hist.taskid = 218
            hist.value1 = ",".join(_tags)
            self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name, gdocid=gdocids)

        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)

        return None

    def reqSaveHoliday(self):
        self.log.debug('%s reqSaveHoliday' % self.name)
        try:
            userid, replid, startdate, enddate = self._job.parameter.split(';')
        except:
            self.log.exception(_("%s Fehler bei der Parameterübergabe %s") % (self.name, self._job.parameters))
            xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0,
                                                 data=_("Fehler beim Anlegen einer Urlaubsvertretung"))
        else:
            if int(userid) != self._job.userid and 'admin' not in self.bfaserver.user[self._job.profile][self._job.userid].flags:
                xml = bitfarm_xml.create_xml_request(reqtype="respDenied")
            else:
                sql = 'insert into sys_holiday (userid_src, userid_dst, start, end) values (%s, %s, str_to_date("%s","%%Y-%%m-%%d"), str_to_date("%s","%%Y-%%m-%%d"))' % (
                userid, replid, startdate, enddate)
                try:
                    self.db[self._job.profile].execute(sqla.text(sql))
                except:
                    self.log.exception(_("%s Fehler beim Speichern einer Urlaubsvertretung") % self.name)
                    xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0,
                                                         data=_("Fehler beim Speichern einer Urlaubsvertretung"))
                else:
                    xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("reqSaveHoliday ausgeführt"))

                    hist = bitfarm_doc.BitfarmHistory()
                    hist.gdocid = 0  # 0 = Admin Einträge
                    hist.userid = self._job.userid
                    hist.taskid = 137
                    hist.value1 = userid
                    hist.value2 = replid
                    hist.value3 = "%s - %s" % (startdate, enddate)
                    self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqGetHoliday(self):
        self.log.debug('%s reqGetHoliday' % self.name)
        if not self._job.userid:
            self.log.exception(_("%s Fehler bei der Parameterübergabe, request erfordert userid" ) %(self.name))
            xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0,
                                                 data=_("Fehler beim Ermitteln einer Urlaubsvertretung"))
        else:
            sql = 'SELECT holid, userid_src, userid_dst, start, end from sys_holiday where (userid_src=:userid or userid_dst=:userid) and end>=:enddate;'
            try:
                result = self.db[self._job.profile].execute(sqla.text(sql).bindparams(userid=self._job.userid,
                                                                                      enddate=datetime.datetime.today().strftime(
                                                                                          "%Y-%m-%d"))).fetchall()
            except:
                self.log.exception(_("%s Fehler beim Ändern einer Urlaubsvertretung" ) %(self.name))
                xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0,
                                                     data=_("Fehler beim Ändern einer Urlaubsvertretung"))
            else:
                parameter = None
                if self._job.parameter is not None:
                    if "asjson" in self._job.parameter:
                        #self.log.debug("asjson is True")
                        # result = json.dumps(result)
                        convresult = []
                        for row in result:
                            convresult.append(
                                [row[0], row[1], row[2], row[3].strftime("%Y-%m-%d"), row[4].strftime("%Y-%m-%d")])
                        parameter = "json"
                        result = convresult
                        #self.log.debug('convresult %s' % str(convresult))
                xml = bitfarm_xml.create_xml_request(reqtype="respGetHoliday", data=result, parameter=parameter)
        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
        return None

    def reqModifyHoliday(self):
        self.log.debug('%s reqModifyHoliday' % self.name)
        try:
            holid, userid, replid, startdate, enddate = self._job.parameter.split(';')
        except:
            self.log.exception(_("%s Fehler bei der Parameterübergabe %s") %(self.name, self._job.parameters))
            xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0,
                                                 data=_("Fehler beim Ändern einer Urlaubsvertretung"))
        else:
            if int(userid) != self._job.userid and 'admin' not in self.bfaserver.user[self._job.profile][self._job.userid].flags:
                xml = bitfarm_xml.create_xml_request(reqtype="respDenied")
            else:
                sql = 'update sys_holiday  set userid_src = %s, userid_dst = %s, start = str_to_date("%s","%%Y-%%m-%%d"), end = str_to_date("%s","%%Y-%%m-%%d") where holid = %s' % (
                userid, replid, startdate, enddate, holid)
                try:
                    self.db[self._job.profile].execute(sqla.text(sql))
                except:
                    self.log.exception(_("%s Fehler beim Ändern einer Urlaubsvertretung") % self.name)
                    xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0,
                                                         data=_("Fehler beim Ändern einer Urlaubsvertretung"))
                else:
                    xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("reqModifyHoliday ausgeführt"))

                    hist = bitfarm_doc.BitfarmHistory()
                    hist.gdocid = 0  # 0 = Admin Einträge
                    hist.userid = self._job.userid
                    hist.taskid = 138
                    hist.value1 = userid
                    hist.value2 = replid
                    hist.value3 = "%s - %s" % (startdate, enddate)
                    self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqDeleteHoliday(self):
        self.log.debug('%s reqDeleteHoliday' % self.name)
        try:
            # holid, userid = self._job.parameter.split(";")
            params = self._job.parameter.split(";")
            holid = params[0]
            userid = params[1]
            startdate = ''
            enddate = ''
            if len(params) == 4:
                startdate = params[2]
                enddate = params[3]
        except:
            self.log.exception(_("%s Fehler bei der Parameterübergabe %s") % (self.name, self._job.parameters))
            xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0,
                                                 data=_("Fehler beim Löschen einer Urlaubsvertretung"))
        else:
            if int(userid) != self._job.userid and 'admin' not in self.bfaserver.user[self._job.profile][self._job.userid].flags:
                xml = bitfarm_xml.create_xml_request(reqtype="respDenied")
            else:
                sql = 'delete from sys_holiday where holid = %s' % holid
                try:
                    self.db[self._job.profile].execute(sqla.text(sql))
                except:
                    self.log.exception(_("%s Fehler beim Löschen einer Urlaubsvertretung") % self.name)
                    xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0,
                                                         data=_("Fehler beim Löschen einer Urlaubsvertretung"))
                else:
                    xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("reqDeleteHoliday ausgeführt"))

                    hist = bitfarm_doc.BitfarmHistory()
                    hist.gdocid = 0  # 0 = Admin Einträge
                    hist.userid = self._job.userid
                    hist.taskid = 139
                    hist.value1 = userid
                    hist.value2 = "%s - %s" % (startdate, enddate)
                    self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqWriteTemplate(self):
        self.log.debug('%s reqWriteTemplate' % self.name)
        ids = []
        if self._job.parameter is not None:
            ids = self._job.parameter.split(';')
        error = ""
        con = self.bfaserver.con.connections[self._job.profile]

        if not con.templates:
            if os.path.exists(os.path.join(self.bfaserver.bfapath, 'templates36')):
                tpath = os.path.join(self.bfaserver.bfapath, 'templates36')
            else:
                tpath = os.path.join(self.bfaserver.bfapath, 'templates')
        else:
            tpath = os.path.expandvars(con.templates)
        if len(ids) == 0:
            xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0,
                                                         data=_("Fehler beim Erstellen der Importvorlagen. Kein Archiv übergeben."))
        elif not os.path.exists(tpath):
            xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0,
                                                         data=_("Fehler beim Erstellen der Importvorlagen. Templatesverzeichnis nicht vorhanden."))
        else:
            for arcid in ids:
                if int(arcid) < 0:
                    self.log.warning("%s Template kann nur für bereits gespeicherte Archive erstellt werden Arcid %s" % (self.name, arcid))
                    error += "Template kann nur für bereits gespeicherte Archive erstellt werden. Arcid %s\n" % arcid
                    continue

                sql = 'select * from sys_arc where arcid = %s' % arcid
                try:
                    result = self.db[self._job.profile].execute(sqla.text(sql)).fetchone()
                except:
                    self.log.exception(_("%s Fehler bei der Datenbankabfrage für Arcid %s") % (self.name, arcid))
                    error += _("Fehler bei der Datenbankabfrage für Arcid %s\n") % arcid
                else:
                    if result:
                        arcname = result['arcname']

                        sql = "select * from sys_arc where arcname = '%s' and flags & 2 = 0" % arcname
                        try:
                            result = self.db[self._job.profile].execute(sqla.text(sql)).fetchall()
                        except:
                            self.log.exception(_("%s Fehler bei der Datenbankabfrage für Arcname %s") % (self.name, arcname))
                            error += _("Fehler bei der Datenbankabfrage für Arcid %s\n") % arcid
                        else:
                            if len(result) > 1:     # mehr als ein archiv mit diesem namen
                                arcname = '%s_%s' % (arcname, arcid)

                            arcname = bitfarm_doc.parse_filename(arcname)
                            fpath = os.path.join(tpath, arcname + '.tpl')

                            doc = bitfarm_doc.BitfarmDocument()
                            if os.path.exists(fpath):
                                self.bfaProcessing.read_from_file(doc, fpath)
                            doc.profile = self._job.profile
                            doc.fields = None
                            # if os.path.exists(os.path.join(self.bfaserver.bfapath, 'templates36')):
                            #     fpath = os.path.join(self.bfaserver.bfapath, 'templates36', os.path.basename(fpath))
                            # doc.fields = self.bfaProcessing.get_addfields(_con=self.bfaserver.con[self._job.profile],_db=self.db[self._job.profile],_doc=doc)
                            if not self.bfaProcessing.create_new_template(arcid=arcid, filename=fpath, doc=doc, profile=self._job.profile):
                                self.log.error(_("%s Datei konnte nicht erstellt werden %s") % (self.name, arcid))
                                error += _("Template konnte nicht erstellt werden für Arcid %s\n") % arcid
                    else:
                        self.log.exception(_("%s Die Arcid %s konnte nicht gefunden werden.") % (self.name, arcid))
                        error += _("Die Arcid %s konnte nicht gefunden werden.\n") % arcid
                # args = [os.path.join(self.bfaserver.bfapath, 'createnewtemplate.exe'), self._job.profile, arcid, os.path.join(self.bfaserver.bfapath, 'templates\\')]
                # _ret = self._timeoutprocess(args, 10)
                # #self.log.debug('%s' % str(_ret))
                # if _ret[0]:     # timeout
                #     self.log.warning(u"Es hat einen Timeout bei createnewtemplate gegeben.")
                #     error += 1
                #     break
                # else:
                #     if _ret[1] != 0:
                #         error += 1

            if error:
                xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, data=error)
            else:
                xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("reqWriteTemplate ausgeführt"))

        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
        return None

    def send_wfmail(self, wf, recipients, subject):
        self.log.debug('%s send_wfmail' % self.name)
        _ret = False
        _con = self.bfaserver.con.connections[self._job.profile]
        if type(self._job.parameter) is str:
            params = self._job.parameter.split(";")
        else:
            params = self._job.parameter
        recp = []
        if type(wf.gdocid) is list:
            gdocids = wf.gdocid
        else:
            gdocids = [wf.gdocid]
        mailbody = ''
        text = ''
        html = ''
        xml = ''
        error = False

        for obj in recipients:
            if obj.email:
                recp.append(obj.email)

        if len(recp) > 0:
            # Create message container - the correct MIME type is multipart/alternative.
            msg = MIMEMultipart('multipart')
            msg['Subject'] = "bitfarm Archiv: %s" % subject
            if _con.usetls:
                msg['From'] = _con.authuser
            else:
                msg['From'] = _con.sendername
            msg['To'] = ", ".join(recp)

            # Create the body of the message (a plain-text and an HTML version).
            for gdocid in gdocids:
                text += _("Das Dokument kann über gdocid:%s geöffnet werden\n") % gdocid
            if mailbody:
                text += _('Kurzmitteilung:\n') + mailbody.replace("\\n", "\n")
            html = '<META HTTP-EQUIV="Content-Type" CONTENT="text/html;charset=iso-8859-1">' + \
                   '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">' + \
                   '<HTML><HEAD>' + \
                   '<META http-equiv=Content-Type content="text/html; charset=iso-8859-1">' + \
                   '<META content="bitfarm Archiv V3" name=GENERATOR>' + \
                   '<STYLE></STYLE></HEAD><BODY bgColor=#ffffff>' + \
                   '<DIV><FONT face=Arial size=2>'
            if wf.info:
                html += 'Info: %s<BR><BR>' % wf.info
            html += _('Klicken Sie auf den nachstehenden Link, um das Dokument im bitfarm-Archiv Viewer zu öffnen:') + ' '
            for gdocid in gdocids:
                html += '<A href=gdocid:%s>Zum Dokument</A></FONT>&nbsp;<BR></DIV>' % gdocid
            if mailbody:
                mailbody = mailbody.replace("\\n", "<BR>")
                html += '<BR><DIV><STRONG><FONT face=Arial size=2>' + _(
                    'Kurzmitteilung:') + '</STRONG><BR>' + mailbody + '</FONT><BR><BR></DIV>'
            html += '<DIV><STRONG><FONT face=Arial><EM>bitfarm-<FONT color=#ff6600 >Archiv</FONT></STRONG> Dokumentenmanagement</EM></FONT><BR></BODY></HTML>'

            # Record the MIME types of both parts - text/plain and text/html.
            part1 = MIMEText(text, 'plain', 'utf-8')
            part2 = MIMEText(html, 'html', 'utf-8')

            # Attach parts into message container.
            # According to RFC 2046, the last part of a multipart message, in this case
            # the HTML message, is best and preferred.
            msg.attach(part1)
            msg.attach(part2)

            _err = self.send_smtpmail(_con, recp, msg)
            _ret = not _err

        return _ret

    def reqLinkMail(self):
        self.log.debug('%s reqLinkMail' % self.name)
        _con = self.bfaserver.con.connections[self._job.profile]
        user = _con.read_user(db=self.db[self._job.profile], userid=self._job.userid, logid=self.name)
        sendername = user.email
        if type(self._job.parameter) is str:
            params = self._job.parameter.split(";")
        else:
            params = self._job.parameter
        recp = []
        gdocids = []
        mailbody = ''
        text = ''
        html = ''
        for param in params:
            if param[:9] == 'mailbody:':
                mailbody = param[9:]
            elif param[:7] == 'gdocid:':
                gdocids = param[7:].split(",")
            else:
                recp.append(param)
        if sendername is None:
            sendername = _con.sendername
        # Create message container - the correct MIME type is multipart/alternative.
        msg = MIMEMultipart('multipart')
        if _con.usetls:
            author = _con.authuser
            msg['Subject'] = "bitfarm-Archiv Dokument von %s" % user.displayname
        else:
            author = formataddr((str(Header(user.displayname, 'utf-8')), str(Header(sendername, 'utf-8'))))
            msg['Subject'] = "bitfarm-Archiv Dokument"

        msg['From'] = author
        msg['To'] = ", ".join(recp)

        # Create the body of the message (a plain-text and an HTML version).
        for gdocid in gdocids:
            text += _("Das Dokument kann über gdocid:%s geöffnet werden\n") % gdocid
        if mailbody:
            text += _('Kurzmitteilung:\n') + mailbody.replace("\\n", "\n")
        html = '<META HTTP-EQUIV="Content-Type" CONTENT="text/html;charset=iso-8859-1">' + \
               '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">' + \
               '<HTML><HEAD>' + \
               '<META http-equiv=Content-Type content="text/html; charset=iso-8859-1">' + \
               '<META content="bitfarm Archiv V3" name=GENERATOR>' + \
               '<STYLE></STYLE></HEAD><BODY bgColor=#ffffff>' + \
               '<DIV><FONT face=Arial size=2>'
        html += _('Klicken Sie auf den nachstehenden Link, um das Dokument im bitfarm-Archiv Viewer zu öffnen:') + ' '
        for gdocid in gdocids:
            html += '<A href=gdocid:%s>Zum Dokument</A></FONT>&nbsp;<BR></DIV>' % gdocid
        if mailbody:
            mailbody = mailbody.replace("\\n", "<BR>")
            html += '<BR><DIV><STRONG><FONT face=Arial size=2>' + _('Kurzmitteilung:') + '</STRONG><BR>' + mailbody + '</FONT><BR><BR></DIV>'
        # html += '<DIV><STRONG><FONT face=Arial><EM>bitfarm-<FONT color=#ff6600 >Archiv</FONT></STRONG> Dokumentenmanagement</EM></FONT><BR></BODY></HTML>'
        html += '</BODY></HTML>'
        # Record the MIME types of both parts - text/plain and text/html.
        part1 = MIMEText(text, 'plain', 'utf-8')
        part2 = MIMEText(html, 'html', 'utf-8')

        # Attach parts into message container.
        # According to RFC 2046, the last part of a multipart message, in this case
        # the HTML message, is best and preferred.
        msg.attach(part1)
        msg.attach(part2)

        error = self.send_smtpmail(_con, recp, msg)

        if not error:
            xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("reqLinkMail ausgeführt"))
            hist = bitfarm_doc.BitfarmHistory()
            # hist.gdocid = gdocids
            hist.taskid = 233
            hist.userid = self._job.userid
            hist.value1 = ",".join(recp)
            self.bfaProcessing.write_history(history=hist, profile=self._job.profile, gdocid=gdocids, logid=self.name)
        else:
            xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=gdocid,
                                                         data=_("(Fehler bei %s)" % error))
        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
        return None

    def reqSMTPMail(self):
        self.log.debug('%s reqSMTPMail' % self.name)

        sendername = ''
        sendermail = ''
        recp = []
        gdocids = []
        attachments = []
        mailbody = ''
        text = ''
        html = ''
        subject = ''
        part1 = None
        part2 = None

        _con = self.bfaserver.con.connections[self._job.profile]
        #user = _con.read_user(db=self.db[self._job.profile], userid=self._job.userid, logid=self.name)

        if 'admin' not in self.bfaserver.user[self._job.profile][self._job.userid].flags:
            xml = bitfarm_xml.create_xml_request(reqtype="respAuthError", data=_("Keine Berechtigung"))
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)

        if type(self._job.parameter) is str:
            params = self._job.parameter.split(";")
        else:
            params = self._job.parameter

        if self._job.data is None:
            attachments = []
        elif type(self._job.data) is str:
            attachments = self._job.data.split(";")
        elif type(self._job.data) in [list, set, tuple]:
            if len(self._job.data) == 2 and type(self._job.data[0]) is str and type(self._job.data[1]) is str:
                attachments = [self._job.data]
            else:
                attachments = self._job.data
        elif type(self._job.data) in [dict, OrderedDict]:
            attachments = [self._job.data]

        if type(params) is list:
            for param in params:
                if param[:9] == 'mailbody:':
                    text = param[9:]
                elif param[:5] == 'text:':
                    text = param[5:]
                elif param[:5] == 'html:':
                    html = param[5:]
                elif param[:7] == 'sender:':
                    sendername = param[7:]
                elif param[:12] == 'displayname:':
                    sendername = param[12:]
                elif param[:11] == 'sendermail:':
                    sendermail = param[11:]
                elif param[:5] == 'from:':
                    sendermail = param[5:]
                elif param[:8] == 'subject:':
                    subject = param[8:]
                elif param[:3] == 'to:':
                    recp = param[3:].split(",")
                elif param[:5] == 'recp:':
                    recp = param[5:].split(",")

        elif type(params) is dict:
            if 'mailbody' in params:
                text = params['mailbody']
            if 'text' in params:
                text = params['text']
            if 'html' in params:
                html = params['html']
            if 'sender' in params:
                sendername = params['sender']
            if 'displayname' in params:
                sendername = params['displayname']
            if 'sendermail' in params:
                sendermail = params['sendermail']
            if 'from' in params:
                sendermail = params['from']
            if 'subject' in params:
                subject = params['subject']
            if 'to' in params:
                recp = params['to'].split(",")
            if 'recp' in params:
                recp = params['recp'].split(",")

        if not sendername:
            sendername = _con.authuser

        if not sendermail:
            sendermail = _con.sendername

        if not recp:
            xml = bitfarm_xml.create_xml_request(reqtype="respError", data=_("Keine Empfänger übergegeben"))
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)

        # Create message container - the correct MIME type is multipart/alternative.
        msg = MIMEMultipart('multipart')
        if sendername:
            author = formataddr((str(Header(sendername, 'utf-8')), str(Header(sendermail, 'utf-8'))))
        else:
            author = formataddr((str(Header(sendermail, 'utf-8')), str(Header(sendermail, 'utf-8'))))

        msg['Subject'] = subject
        msg['From'] = author
        msg['To'] = ", ".join(recp)

        # Record the MIME types of both parts - text/plain and text/html.
        # Attach parts into message container.
        # According to RFC 2046, the last part of a multipart message, in this case
        # the HTML message, is best and preferred.

        if text:
            part1 = MIMEText(text, 'plain', 'utf-8')
            msg.attach(part1)
        if html:
            part2 = MIMEText(html, 'html', 'utf-8')
            msg.attach(part2)

        for attachment in attachments:
            file = None
            filename = None
            if type(attachment) is str:
                items = attachment.split('|')
                for item in items:
                    if item[:7] == 'base64:':
                        file = item[7:]
                    if item[:9] == 'filename:':
                        filename = item[9:]
            elif type(attachment) in [list, set, tuple]:
                for item in attachment:
                    if item[:7] == 'base64:':
                        file = item[7:]
                    if item[:9] == 'filename:':
                        filename = item[9:]
            elif type(attachment) in [dict, OrderedDict]:
                if 'base64' in attachment:
                    file = attachment['base64']
                if 'filename' in attachment:
                    filename = attachment['filename']

            if filename and file:
                if len(file) > 78:
                    max_line_len = 78  # RFC822
                    file = "\n".join([file[i:i + max_line_len] for i in range(0, len(file), max_line_len)])
                part = MIMEBase('application', 'octet-stream')
                part.set_payload(file)
                part.add_header('Content-Transfer-Encoding', 'base64')
                part.add_header('content-disposition', 'attachment', filename=filename)
                msg.attach(part)

        error = self.send_smtpmail(_con, recp, msg)

        if not error:
            xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("reqSMTPMail ausgeführt"))
            hist = bitfarm_doc.BitfarmHistory()
            hist.taskid = 140
            hist.userid = self._job.userid
            hist.value1 = ",".join(recp)
            self.bfaProcessing.write_history(history=hist, profile=self._job.profile, gdocid=0, logid=self.name)
        else:
            xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0,  data=_("(Fehler bei %s)" % error))
        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
        return None

    def send_smtpmail(self, _con, recp, msg):
        self.log.debug('%s send_smtpmail' % self.name)
        # Send the message via local SMTP server.
        error = False
        try:
            s = smtplib.SMTP(_con.smtpserver, _con.smtpport, timeout=20)
        except:
            self.log.exception(_("%s Fehler beim Verbinden mit dem SMTP Server %s:%s") % (self.name, _con.smtpserver, _con.smtpport))
            error = 'connect'
        else:
            if _con.smtp_oauth:
                self.log.debug(f"{self.name} using oauth login via oauth_file '{_con.smtp_oauth}'")
                oauth_file = os.path.join(self.bfaserver.mypath, _con.smtp_oauth)
                if not os.path.exists(oauth_file):
                    oauth_file = os.path.join(self.bfaserver.bfapath, _con.smtp_oauth)

                if not os.path.exists(oauth_file):
                    self.log.error(f'{self.name} Konnte OAuth Konfiguration nicht finden {oauth_file}')
                    error = 'oauth_conf'
                else:
                    with open(oauth_file, "rb") as f:
                        crypted_auth = f.read()
                    self.log.debug(f"{self.name} decrypting oauth_file")
                    auth_dict = json.loads(dpapi.decryptData(crypted_auth))

                    oauth2_client_id = auth_dict["client_id"]
                    oauth2_client_secret = auth_dict["client_secret"]
                    oauth2_refresh_token = auth_dict["refresh_token"]
                    oauth2_tenant_id = auth_dict["tenant_id"]
                    oauth2_provider = auth_dict["provider"]
                    self.log.debug(f"{self.name} fetching tokens from {oauth2_provider}")
                    token = bitfarm_oauth.get_access_token(oauth2_provider, oauth2_client_id, oauth2_client_secret, oauth2_tenant_id, oauth2_refresh_token)
                    auth_string = bitfarm_oauth.generate_oauth2_string(_con.authuser, token[0], as_base64=True)

                    try:
                        s.ehlo_or_helo_if_needed()
                        s.starttls()
                        s.ehlo_or_helo_if_needed()
                    except:
                        self.log.exception('%s Fehler bei STARTTLS' % self.name)
                        error = 'starttls'
                    else:
                        ret = s.docmd('AUTH', 'XOAUTH2 ' + auth_string)

                        if ret[0] != 235:
                            self.log.error('%s Nicht authentifiziert: %s' % (self.name, ret[1]))
                            error = 'oauth_auth'
                        else:
                            self.log.debug("%s Anmeldung per OAuth2 erfolgreich..." % self.name)

            elif _con.authuser:
                try:
                    if _con.usetls:
                        try:
                            s.ehlo_or_helo_if_needed()
                            s.starttls()
                            s.ehlo_or_helo_if_needed()
                        except:
                            self.log.exception('%s Fehler bei STARTTLS' % self.name)
                            error = 'starttls'
                    s.login(_con.authuser, _con.authpass)
                except:
                    self.log.exception(_("%s Fehler beim Anmelden am SMTP Server %s:%s") % (self.name, _con.smtpserver, _con.smtpport))
                    error = 'login'
                else:
                    self.log.debug("%s Anmeldung erfolgreich..." % self.name)
        if not error:
            # sendmail function takes 3 arguments: sender's address, recipient's address
            # and message to send - here it is sent as one string.
            try:
                s.sendmail(_con.authuser, recp, msg.as_string())
            except:
                self.log.exception(_("%s Fehler beim Versenden einer LinkMail Absender: %s") % (self.name, _con.authuser))
                error = 'sendmail'
            finally:
                s.quit()

        return error

    def check_docrights(self, gdocid=None, arcid=None):
        if gdocid is None:
            gdocid = self._job.gdocid

        self.log.debug('%s check_docrights [%s] userid: %d' % (self.name, gdocid, self._job.userid))
        _ret = bitfarm_doc.BitfarmRights()

        if arcid is None:
            if self._job.symbolic_arcid:
                arcid = self._job.symbolic_arcid

        timid = None
        if self._job.tim_id is not None and int(self._job.tim_id) > 0:
            try:
                timid = int(self._job.tim_id)
            except:
                self.log.warning("%s Parameter timid ist keine Zahl %s" % (self.name, self._job.tim_id))
                return _ret

        try:
            userid = int(self._job.userid)
        except:
            self.log.warning("%s Parameter userid ist keine Zahl %s" % (self.name, self._job.userid))
            return _ret

        try:
            gdocid = int(gdocid)
        except:
            self.log.warning("%s Parameter gdocid ist keine Zahl %s" % (self.name, gdocid))
            return _ret

        if arcid:
            try:
                arcid = int(arcid)
            except:
                self.log.warning("%s Parameter arcid ist keine Zahl %s" % (self.name, arcid))
                return _ret

        doc_rights = self.bfaserver.con.connections[self._job.profile].read_rights(db=self.db[self._job.profile], arcid=arcid, gdocid=gdocid, user=userid, doc_rights=False)
        if doc_rights is not None and len(doc_rights) > 0:
            for right in doc_rights:
                if arcid:
                    if right.arcid == arcid:
                        _ret.rights |= right.rights
                else:
                    _ret.rights |= right.rights
                for x in right.userlist:
                    if x not in _ret.userlist:
                        _ret.userlist.append(x)
                for x in right.grouplist:
                    if x not in _ret.grouplist:
                        _ret.grouplist.append(x)

            #_ret = doc_rights[0]
        # if not _ret.read:   # keine Leseberechigung auf das Dokument/Archiv
        users = self.bfaserver.con.connections[self._job.profile].get_holidaylist(db=self.db[self._job.profile], user=userid)

        if timid:
            # sql = 'select timref from sys_timer where timid = %d and gdocid = %d and userid_work = %d and status & 4 = 0' % (timid, gdocid, userid)
            sql = 'select timref from sys_timer where timid = %d and gdocid = %d and userid_work in (%s) and status & 4 = 0' % (timid, gdocid, ','.join([str(x) for x in users]))
            try:
                result = self.db[self._job.profile].execute(sqla.text(sql)).fetchone()
            except:
                self.log.exception(_("%s Fehler bei der Überprüfung der Wiedervorlage %s") % (self.name, timid))
            else:       # wenn der user das Dokument auf Wiedervorlage hat, dann erhält er standardrechte
                if result:
                    _ret.rights |= bitfarm_doc.ROLE_STANDARD

        if not _ret.read or not _ret.modify:    # wenn ich keine lese oder bearbeiten recht habe prüfe ich, ob ich jemanden vertrete, der ggf. die rechte hat
            for user_id in users:
                if user_id == userid:
                    continue
                doc_rights = self.bfaserver.con.connections[self._job.profile].read_rights(db=self.db[self._job.profile], gdocid=gdocid, user=user_id, doc_rights=True)
                if doc_rights is not None and len(doc_rights) > 0:
                    _ret.rights |= doc_rights[0].rights
                    if user_id not in _ret.userlist:
                        _ret.userlist.append(user_id)

            # resub_right = self.bfaserver.con.connections[self._job.profile].read_resubright(
            #     db=self.bfaProcessing.db[self._job.profile], gdocid=gdocid, user=userid)
            # if resub_right is not None:
            #     _ret.rights |= resub_right.rights

        if len(_ret.grouplist) == 0:
            _ret.grouplist = [0]

        return _ret

    def check_docrights2(self, gdocid=None):
        if gdocid is None:
            gdocid = self._job.gdocid

        self.log.debug('%s check_docrights2 [%s]' % (self.name, gdocid))
        _ret = bitfarm_doc.BitfarmRights()
        self.log.debug('%s userid: %d' %(self.name, self._job.userid))
        users = self.bfaserver.con.connections[self._job.profile].get_holidaylist(db=self.db[self._job.profile], user=self._job.userid, logid=self.name)
        grouplist = self.bfaserver.con.connections[self._job.profile].resolve_membership(db=self.db[self._job.profile], userid=users, logid=self.name)
        if len(grouplist) == 0:
            grouplist = [0]
        grouplist = ','.join([str(x) for x in grouplist])
        userids = ','.join([str(x) for x in users])
        sql = "select bit_or(ax.rights) from sys_doc as d left join sys_arcxs as ax on d.arcid = ax.arcid where (ax.userid in (%s) or (ax.userid = 0 and ax.groupid in (%s))) and gdocid = %i" % (
        userids, grouplist, self._job.gdocid)
        try:
            res = self.db[self._job.profile].execute(sql).fetchall()
        except:
            pass

        if res:
            for row in res:
                _ret.rights |= row[0]

        sql = "select bit_or(rights) from sys_docxs where (userid in (%s) or (userid = 0 and groupid in (%s))) and gdocid = %i" % (
        userids, grouplist, self._job.gdocid)
        try:
            res = self.db[self._job.profile].execute(sql).fetchall()
        except:
            pass

        if res:
            for row in res:
                _ret.rights |= row[0]

        return _ret

    def check_addfieldrights(self, fieldid, arcid=None, gdocid=None):
        if arcid is None and gdocid is None:
            return None

        self.log.debug('%s check_addfieldrights arcid %s gdocid %s' % (self.name, arcid, gdocid))
        _ret = bitfarm_doc.BitfarmFieldRight()
        self.log.debug('%s userid: %d' % (self.name, self._job.userid))
        users = self.bfaserver.con.connections[self._job.profile].get_holidaylist(db=self.db[self._job.profile], user=self._job.userid, logid=self.name)
        grouplist = self.bfaserver.con.connections[self._job.profile].resolve_membership(db=self.db[self._job.profile], userid=users, logid=self.name)
        if len(grouplist) == 0:
            grouplist = [0]
        grouplist = ','.join([str(x) for x in grouplist])
        userids = ','.join([str(x) for x in users])

        if arcid is None:
            if self._job.arcid is not None:
                arcid = [self._job.arcid]
            else:
                try:
                    res = self.db[self._job.profile].execute('select arcid from sys_doc where gdocid = %s' % gdocid).fetchone()
                except:
                    self.log.exception("%s Fehler bei check_addfieldrights (2)" % self.name)

                if res:
                    arcid = [res[0]]
                else:
                    arcid = []
        else:
            arcid = [arcid]

        if self._job.fields_arcid is not None:
            arcid.append(self._job.fields_arcid)

        if self._job.symbolic_arcid is not None:
            arcid.append(self._job.symbolic_arcid)

        if gdocid is None:
            sql = "select bit_or(rights) from sys_addfieldxs where arcid in (%s) and fieldid = %s" \
                  " and (userid in (%s) or (userid = 0  and groupid in (%s)))" % (",".join([str(x) for x in arcid]), fieldid, userids, grouplist)
        else:
            if arcid:
                sql = "select bit_or(rights) from sys_addfieldxs where arcid in (%s) and fieldid = %s" \
                      " and (userid in (%s) or (userid = 0  and groupid in (%s)))" % (",".join([str(x) for x in arcid]), fieldid, userids, grouplist)
            else:
                sql = "select bit_or(rights) from sys_addfieldxs as ax" \
                      " left join sys_doc as d on ax.arcid = d.arcid" \
                      " where fieldid = %s and gdocid = %s" \
                      " and (ax.userid in (%s) or(ax.userid = 0 and ax.groupid in (%s)))" % (fieldid, gdocid, userids, grouplist)
        res = None
        try:
            res = self.db[self._job.profile].execute(sql).fetchall()
        except:
            self.log.exception("%s Fehler bei check_addfieldrights" % self.name)

        if res:
            for row in res:
                _ret.rights |= row[0]

        return _ret

    def check_statefieldrights(self, fieldid, arcid=None, gdocid=None):
        if arcid is None and gdocid is None:
            return None

        self.log.debug('%s check_statefieldrights [%s]' % (self.name, arcid))
        _ret = bitfarm_doc.BitfarmFieldRight()
        self.log.debug('%s userid: %d' % (self.name, self._job.userid))
        users = self.bfaserver.con.connections[self._job.profile].get_holidaylist(db=self.db[self._job.profile], user=self._job.userid)
        grouplist = self.bfaserver.con.connections[self._job.profile].resolve_membership(db=self.db[self._job.profile], userid=users, logid=self.name)
        if len(grouplist) == 0:
            grouplist = [0]
        grouplist = ','.join([str(x) for x in grouplist])
        userids = ','.join([str(x) for x in users])

        if arcid is None:
            if self._job.arcid is not None:
                arcid = [self._job.arcid]
            else:
                try:
                    res = self.db[self._job.profile].execute('select arcid from sys_doc where gdocid = %s' % gdocid).fetchone()
                except:
                    self.log.exception("%s Fehler bei check_statefieldrights (2)" % self.name)

                if res:
                    arcid = [res[0]]
                else:
                    arcid = []
        else:
            arcid = [arcid]

        if self._job.fields_arcid is not None:
            arcid.append(self._job.fields_arcid)

        if self._job.symbolic_arcid is not None:
            arcid.append(self._job.symbolic_arcid)

        if gdocid is None:
            sql = "select bit_or(rights) from sys_statefieldxs where arcid in (%s) and fieldid = %s" \
                  " and (userid in (%s) or (userid = 0 and groupid in (%s)))" % (",".join([str(x) for x in arcid]), fieldid, userids, grouplist)
        else:
            if arcid:
                sql = "select bit_or(rights) from sys_statefieldxs where arcid in (%s) and fieldid = %s" \
                      " and (userid in (%s) or (userid = 0 and groupid in (%s)))" % (",".join([str(x) for x in arcid]), fieldid, userids, grouplist)
            else:
                sql = "select bit_or(rights) from sys_statefieldxs as ax" \
                      " left join sys_doc as d on ax.arcid = d.arcid" \
                      " where fieldid = %s and gdocid = %s" \
                      " and (ax.userid in (%s) or(ax.userid = 0 and ax.groupid in (%s)))" % (fieldid, gdocid, userids, grouplist)

        try:
            res = self.db[self._job.profile].execute(sql).fetchall()
        except:
            self.log.exception("%s Fehler bei check_statefieldrights" % self.name)

        if res:
            for row in res:
                _ret.rights |= row[0]

        return _ret

    def check_arcright(self, arcid=None):
        if arcid is None:
            arcid = self._job.arcid
        self.log.debug('%s check_arcright %s' % (self.name, arcid))
        right = self.bfaserver.con.connections[self._job.profile].read_rights(db=self.db[self._job.profile], user=self._job.userid, arcid=arcid)
        return right

    def reqTif(self):
        self.log.debug('%s reqTif' % self.name)
        if self.bfaserver.useremoterenderer is True:
            return self.reqtifremote()
        else:
            return self.reqtiflocal()

    def reqtiflocal(self):
        self.log.debug('%s reqtiflocal' % self.name)

        # set default values for missing job-parameters
        self.log.debug('%s reqtiflocal params %s' % (self.name, str(self._job.parameter)))
        if self._job.parameter is None:
            params = {}
        else:
            params = self._job.parameter

        # check rights
        doc_right = self.check_docrights()
        if self._job.userid not in self.bfaserver.user[self._job.profile] or not doc_right.read:
            xml = bitfarm_xml.create_xml_request(reqtype="respAuthError", userid=0, gdocid=self._job.gdocid)
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Empfangen wird abgebrochen") % self.name)
            return None

            # load document
        doc = bitfarm_doc.BitfarmDocument()
        doc.profile = self._job.profile
        loadparams = ['base', "paths"]
        if "ievlayer" in params and params["ievlayer"] is not None:
            loadparams.append("iev")
        if not self.bfaProcessing.load(doc, self._job.gdocid, loadparams, logid=self.name):
            xml = bitfarm_xml.create_xml_request(reqtype="respDocError", userid=0, gdocid=self._job.gdocid)
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning("%s Client scheint weg zu sein, Senden wird abgebrochen" % self.name)
            return None

        if "ievlayer" in params:
            self.add_private_annotations(doc, doc_right)
        doc.rights = doc_right.rights

        renderannos = {}
        if "ievlayer" in params and params["ievlayer"] is not None:
            for anno in doc.annotations:
                if params["ievlayer"] == anno.layer:
                    renderannos[anno.page] = anno.iev.stream

        args = {}
        infile = os.path.expandvars(doc.extloc if doc.extloc else doc.docloc)

        #        if isolated is False:
        #            args["inputfile"] = infile
        #        else:
        #            args["myFile"] = open(infile, "rb")
        args["outtype"] = "tif"
        if len(renderannos) > 0:
            # args["ievdict"]=renderannos
            for pageno in renderannos:
                key = "iev_%d" % pageno
                args[key] = renderannos[pageno]
            args["myFile"] = MyFile(infile)
            args["outtypes"] = "tif"
            args["tifoptions"] = 'color=color'

            self.log.debug('%s reqimglocal convert args: %s' % (self.name, str(args)))

            uid = self.name
            tempdir = tempfile.mkdtemp(dir=self.renderer.tempdir)
            try:
                result = self.renderer.convert(uid, tempdir, **args)
                shutil.rmtree(tempdir)
            except:
                shutil.rmtree(tempdir)
                xml = bitfarm_xml.create_xml_request(reqtype="respDocError", userid=0, gdocid=self._job.gdocid)
                try:
                    self._job.sock.send(xml)
                except:
                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)

                return None

            if "tif" in result:
                stream = io.BytesIO()
                decodedresult = base64.b64decode(result["tif"])
                #                with open(previewfp, "wb") as pf:
                #                    pf.write(decodedresult)
                stream.write(decodedresult)
                datalen = stream.tell()
                stream.seek(0)
            else:
                xml = bitfarm_xml.create_xml_request(reqtype="respDocError", userid=0, gdocid=self._job.gdocid)
                try:
                    self._job.sock.send(xml)
                except:
                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None

        else:
            # keine Annos, tif aus Dateisystem lesen
            stream = io.BytesIO()
            with open(infile, "rb") as tiffile:
                stream.write(tiffile.read())
            datalen = stream.tell()
        stream.seek(0)
        xml = bitfarm_xml.create_xml_request(reqtype="respTif", userid=0, datalen=datalen, filename=doc.title + ".tif",
                                             gdocid=self._job.gdocid, datatype="tif")
        self.log.debug(_("%s reqtiflocal sende xml %s") % (self.name, xml))
        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None
        sent = 0
        while sent < datalen:
            buff = stream.read(self.bfaserver.split_size)
            try:
                self._job.sock.send(buff)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None
            sent += len(buff)
        return True

    def reqtifremote(self):
        self.log.debug('%s reqtifremote' %(self.name))

        # set default values for missing job-parameters
        self.log.debug('%s reqtifremote params %s' %(self.name, str(self._job.parameter)))
        if self._job.parameter is None:
            params = {}
        else:
            params = self._job.parameter

        # check rights
        doc_right = self.check_docrights()
        if self._job.userid not in self.bfaserver.user[self._job.profile] or not doc_right.read:
            xml = bitfarm_xml.create_xml_request(reqtype="respAuthError", userid=0, gdocid=self._job.gdocid)
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Empfangen wird abgebrochen") % self.name)
            return None

            # load document
        doc = bitfarm_doc.BitfarmDocument()
        doc.profile = self._job.profile
        loadparams = ['base', "paths"]
        if params["ievlayer"] is not None:
            loadparams.append("iev")
        if not self.bfaProcessing.load(doc, self._job.gdocid, loadparams, logid=self.name):
            xml = bitfarm_xml.create_xml_request(reqtype="respDocError", userid=0, gdocid=self._job.gdocid)
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

        if "ievlayer" in params:
            self.add_private_annotations(doc, doc_right)
        doc.rights = doc_right.rights
        # find the directory containing the preview-image
        # create the directory if missing
        # TODO: create subdirs for different profiles!
        # filepath = os.path.expandvars(doc.extloc if doc.extloc else doc.docloc)
        renderannos = {}
        if params["ievlayer"] is not None:
            for anno in doc.annotations:
                if params["ievlayer"] == anno.layer:
                    renderannos[anno.page] = anno.iev.stream

        args = {}
        infile = os.path.expandvars(doc.extloc if doc.extloc else doc.docloc)
        server, isolated = self.bfaserver.converterhosts.get()
        if isolated is False:
            args["inputfile"] = infile
        else:
            args["myFile"] = open(infile, "rb")
        args["outtype"] = "tif"
        if len(renderannos) > 0:
            args["ievdict"] = renderannos

            try:
                c = BfaRenderClient(server)
                try:
                    result = c.convert(**args)
                except Exception as e:
                    xml = bitfarm_xml.create_xml_request(reqtype="respDocError", userid=0, gdocid=self._job.gdocid,
                                                         data=_("Voransicht konnte nicht gerendert werden."))
                    self.log.warning(_("%s Voransicht konnte nicht gerendert werden. Fehler %s" ) %(self.name, e))
                    if xml is not None:
                        try:
                            self._job.sock.send(xml)
                        except:
                            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                        return None
                if "tif" in result:
                    stream = io.BytesIO()
                    decodedresult = base64.b64decode(result["tif"])
                    stream.write(decodedresult)
                    datalen = stream.tell()
                    stream.seek(0)
                else:
                    self.log.warning(_("%s Voransicht konnte nicht gerendert werden.") %(self.name))
                    xml = bitfarm_xml.create_xml_request(reqtype="respDocError", userid=0, gdocid=self._job.gdocid,
                                                         data=_("Voransicht konnte nicht gerendert werden."))
                    try:
                        self._job.sock.send(xml)
                    except:
                        self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                    return None
            finally:
                self.bfaserver.converterhosts.put((server, isolated))
        else:
            # keine Annos, tif aus Dateisystem lesen
            stream = io.BytesIO()
            with open(infile, "rb") as tiffile:
                stream.write(tiffile.read())
            datalen = stream.tell()
        stream.seek(0)
        xml = bitfarm_xml.create_xml_request(reqtype="respTif", userid=0, datalen=datalen, filename=doc.title + ".tif",
                                             gdocid=self._job.gdocid, datatype="tif")
        self.log.debug(_("%s reqtifremote sende xml %s") % (self.name, xml))
        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None
        sent = 0
        while sent < datalen:
            buff = stream.read(self.bfaserver.split_size)
            try:
                self._job.sock.send(buff)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None
            sent += len(buff)
        return True

    def reqImg(self):
        self.log.debug('%s reqImg' %(self.name))
        if self.bfaserver.useremoterenderer is True:
            return self.reqimgremote()
        else:
            return self.reqimglocal()

    def reqimglocal(self):
        self.log.debug('%s reqimglocal' %(self.name))
        self.log.debug('%s reqimglocal params %s' %(self.name, str(self._job.parameter)))
        tempdir = None
        params = self._job.parameter
        if "page" not in params:
            params["page"] = 0
        if "ievlayer" not in params:
            params["ievlayer"] = 0
        if "imgtype" not in params:
            params["imgtype"] = "png"
        if "forcerendering" not in params:
            forcerendering = False
        else:
            forcerendering = params["forcerendering"]

        # check rights
        doc_right = self.check_docrights()
        if self._job.userid not in self.bfaserver.user[self._job.profile] or not doc_right.read:
            xml = bitfarm_xml.create_xml_request(reqtype="respDenied", userid=0, gdocid=self._job.gdocid)
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Empfangen wird abgebrochen") % self.name)
            return None

            # load document
        doc = bitfarm_doc.BitfarmDocument()
        doc.profile = self._job.profile
        loadparams = ['base', "paths"]
        if params["ievlayer"] is not None:
            loadparams.append("iev")
        if not self.bfaProcessing.load(doc, self._job.gdocid, loadparams, logid=self.name):
            xml = bitfarm_xml.create_xml_request(reqtype="respDocError", userid=0, gdocid=self._job.gdocid)
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

        if "ievlayer" in params:
            self.add_private_annotations(doc, doc_right)
        doc.rights = doc_right.rights
        # find the directory containing the preview-image
        filepath = os.path.expandvars(doc.extloc if doc.extloc else doc.docloc)
        if not os.path.isfile(filepath):
            xml = bitfarm_xml.create_xml_request(reqtype="respDocError", userid=0, gdocid=self._job.gdocid)
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen" % self.name))
            return None
        subdir = os.path.dirname(filepath).split(os.path.sep)[-3:]
        previewdir = os.path.join(self.bfaserver.httppreviewdir, self._job.profile, *subdir)
        if not os.path.isdir(previewdir):
            os.makedirs(previewdir)
        if os.path.splitext(filepath)[1].lower() == ".tif":
            if doc.pages is None or doc.pages == 0:
                doc.pages = self.countpages(filepath)
            if doc.pages is None:
                self.log.error(_("%s Anzahl der Seiten konnte nicht ermittelt werden.") % self.name)
                xml = bitfarm_xml.create_xml_request(reqtype="respDocError", userid=0, gdocid=self._job.gdocid,
                                                    data="Anzahl der Seiten konnte nicht ermittelt werden.")
                try:
                    self._job.sock.send(xml)
                except:
                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None
            pagecount = doc.pages
        else:
            doc.pages = 1
            pagecount = 1
        if params["page"] + 1 > pagecount or params["page"] < 0:
            self.log.error(_("%s Nicht vorhandene Seite angefordert.") % self.name)
            xml = bitfarm_xml.create_xml_request(reqtype="respDocError", userid=0, gdocid=self._job.gdocid,
                                                 data="no such page")
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

        # find the iev for requested page/layer
        reqiev = None
        ievstream = None
        ievmodified_date = None
        if params["ievlayer"] is not None:
            for anno in doc.annotations:
                if anno.page == params["page"]:
                    if params["ievlayer"] == anno.layer:
                        ievstream = anno.iev
                        ievmodified_date = anno.modified_date

        # find the correct filename for the requested preview-image
        if ievstream is not None:
            previewfp = os.path.join(previewdir, str(self._job.gdocid) + "_" + str(params["page"]) + "_" + str(
                params["ievlayer"]) + "." + str(params["imgtype"]))
        else:
            previewfp = os.path.join(previewdir,
                                     str(self._job.gdocid) + "_" + str(params["page"]) + "_None" + "." + str(
                                         params["imgtype"]))

        # lets find out, if we have an uptodate preview-image
        dorendering = False
        if not os.path.isfile(previewfp):
            self.log.debug(_('%s reqimglocal Voransicht nicht vorhanden, erzeuge neue Voransicht.') %(self.name))
            dorendering = True
        else:
            # tif is newer than existing preview-image
            if os.stat(os.path.expandvars(doc.extloc if doc.extloc else doc.docloc)).st_mtime > os.stat(
                    previewfp).st_mtime:
                dorendering = True
                self.log.debug(_('%s reqimglocal Archiv-Tif ist neuer als Voransicht, erzeuge neue Voransicht.') %(self.name))
            # iev is newer 
            if ievstream is not None:
                if time.mktime(ievmodified_date.timetuple()) > os.stat(previewfp).st_mtime:
                    self.log.debug(_('%s reqimglocal Annotation ist neuer als Voransicht, erzeuge neue Voransicht.') %(self.name))
                    dorendering = True

        # use Renderer to create preview-image
        if dorendering or forcerendering is True:
            infile = os.path.expandvars(doc.extloc if doc.extloc else doc.docloc)
            args = {}
            args["myFile"] = MyFile(infile)
            args["outtypes"] = params["imgtype"]
            if params["imgtype"] == "png":
                args["pngoptions"] = 'color=color'
                args["pngoptions"] += ",page=" + str(params["page"])
            if ievstream is not None:
                ievstream.stream.seek(0)
                args["iev"] = ievstream.stream
            self.log.debug('%s reqimglocal convert args: %s' % (self.name, str(args)))

            uid = self.name
            tempdir = tempfile.mkdtemp(dir=self.renderer.tempdir)
            try:
                result = self.renderer.convert(uid, tempdir, **args)
                shutil.rmtree(tempdir)
            except Exception as e:
                self.log.error(_("%s Fehler beim Erstellen der Voransicht: %s, %s") % (self.name, str(e), traceback.format_exc()))
                xml = bitfarm_xml.create_xml_request(reqtype="respDocError", userid=0, gdocid=self._job.gdocid)
                try:
                    self._job.sock.send(xml)
                except:
                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)

                return None

            if params["imgtype"] in result:
                stream = io.BytesIO()
                decodedresult = base64.b64decode(result[params["imgtype"]])
                with open(previewfp, "wb") as pf:
                    pf.write(decodedresult)
                stream.write(decodedresult)
                datalen = stream.tell()
                stream.seek(0)
            else:
                xml = bitfarm_xml.create_xml_request(reqtype="respDocError", userid=0, gdocid=self._job.gdocid)
                try:
                    self._job.sock.send(xml)
                except:
                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None
        else:
            self.log.debug(_('%s reqImg Voransicht ist vorhanden und aktuell.') % self.name)
            stream = io.BytesIO()
            try:
                with open(previewfp, "rb") as pf:
                    stream.write(pf.read())
                datalen = stream.tell()
                stream.seek(0)
            except:
                xml = bitfarm_xml.create_xml_request(reqtype="respDocError", userid=0, gdocid=self._job.gdocid)
                try:
                    self._job.sock.send(xml)
                except:
                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None

        xml = bitfarm_xml.create_xml_request(reqtype="respImg", userid=0, datalen=datalen,
                                             filename=doc.title + "." + params["imgtype"], gdocid=self._job.gdocid,
                                             datatype=params["imgtype"])
        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None
        sent = 0
        while sent < datalen:
            buff = stream.read(self.bfaserver.split_size)
            try:
                self._job.sock.send(buff)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None
            sent += len(buff)
        self.check_logread(doc_right)

        return True

    def reqimgremote(self):
        self.log.debug('%s reqimgremote' %(self.name))

        # set default values for missing job-parameters
        self.log.debug('%s reqimgremote params %s' %(self.name, str(self._job.parameter)))
        params = self._job.parameter
        if "page" not in params:
            params["page"] = 0
        if "ievlayer" not in params:
            params["ievlayer"] = 0
        if "imgtype" not in params:
            params["imgtype"] = "png"
        if "forcerendering" not in params:
            forcerendering = False
        else:
            forcerendering = params["forcerendering"]        
        # check rights
        doc_right = self.check_docrights()
        if self._job.userid not in self.bfaserver.user[self._job.profile] or not doc_right.read:
            xml = bitfarm_xml.create_xml_request(reqtype="respDenied", userid=0, gdocid=self._job.gdocid)
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Empfangen wird abgebrochen") % self.name)
            return None

            # load document
        doc = bitfarm_doc.BitfarmDocument()
        doc.profile = self._job.profile
        loadparams = ['base', "paths"]
        if params["ievlayer"] is not None:
            loadparams.append("iev")
        if not self.bfaProcessing.load(doc, self._job.gdocid, loadparams, logid=self.name):
            xml = bitfarm_xml.create_xml_request(reqtype="respDocError", userid=0, gdocid=self._job.gdocid)
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

        if "ievlayer" in params:
            self.add_private_annotations(doc, doc_right)
        doc.rights = doc_right.rights
        # find the directory containing the preview-image
        # create the directory if missing
        filepath = os.path.expandvars(doc.extloc if doc.extloc else doc.docloc)
        subdir = os.path.dirname(filepath).split(os.path.sep)[-3:]
        previewdir = os.path.join(self.bfaserver.httppreviewdir, self._job.profile, *subdir)
        if not os.path.isdir(previewdir):
            os.makedirs(previewdir)

        # check if requested page exits
        # if not use to the documents last page
        # print doc.pages
        if doc.pages is None:
            doc.pages = self.countpages(filepath)
            # self.bfaProcessing.write(doc)
        pagecount = doc.pages
        if params["page"] + 1 > pagecount or params["page"] < 0:
            self.log.debug(_("%s Nicht vorhandene Seite angefordert.") %(self.name))
            xml = bitfarm_xml.create_xml_request(reqtype="respDocError", userid=0, gdocid=self._job.gdocid,
                                                 data="no such page")
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

        # find the iev for requested page/layer
        reqiev = None
        ievstream = None
        ievmodified_date = None
        if params["ievlayer"] is not None:
            for anno in doc.annotations:
                if anno.page == params["page"]:
                    if params["ievlayer"] == anno.layer:
                        ievstream = anno.iev
                        ievmodified_date = anno.modified_date
                        # ievlist = self.get_annotations(doc.extloc if doc.extloc else doc.docloc)
                        # for iev in ievlist:
                        # if iev.page == params["page"] and iev.layer == params["ievlayer"]:
                        # reqiev = iev
                        ## TODO: Rechtesystem IEV wird nochmal angepasst!
                        # if params["ievlayer"] == 0 and doc_right.aread is False:
                        # reqiev = None
                        # elif params["ievlayer"] != 0 and params["ievlayer"] != self._job.userid and doc_right.apread is False:
                        # reqiev = None
                        # break

        # find the correct filename for the requested preview-image
        if ievstream is not None:
            previewfp = os.path.join(previewdir, str(self._job.gdocid) + "_" + str(params["page"]) + "_" + str(
                params["ievlayer"]) + "." + str(params["imgtype"]))
        else:
            previewfp = os.path.join(previewdir,
                                     str(self._job.gdocid) + "_" + str(params["page"]) + "_None" + "." + str(
                                         params["imgtype"]))

        # lets find out, if we have an uptodate preview-image
        dorendering = False
        if not os.path.isfile(previewfp):
            self.log.debug(_('%s reqimgremote Voransicht nicht vorhanden, erzeuge neue Voransicht.') %(self.name))
            dorendering = True
        else:
            # tif is newer than existing preview-image
            if os.stat(os.path.expandvars(doc.extloc if doc.extloc else doc.docloc)).st_mtime > os.stat(
                    previewfp).st_mtime:
                dorendering = True
                self.log.debug(_('%s reqimgremote Archiv-Tif ist neuer als Voransicht, erzeuge neue Voransicht.') %(self.name))
            # iev is newer 
            if ievstream is not None:
                if time.mktime(ievmodified_date.timetuple()) > os.stat(previewfp).st_mtime:
                    self.log.debug(_('%s reqimgremote Annotation ist neuer als Voransicht, erzeuge neue Voransicht.') %(self.name))
                    dorendering = True

        # use converter-service to create preview-image
        if dorendering or forcerendering is True:
            server, isolated = self.bfaserver.converterhosts.get()
            self.log.debug(
                _('%s reqimgremote nutzt Server %s (isolated: %s) um neue Voransicht zu erstellen') % (self.name, server, str(isolated)))
            try:
                # TODO: cleanup code, andere img-typen einbauen, Quell-Datei kann auch jpg/bmp sein!
                c = BfaRenderClient(server)
                args = {}
                infile = os.path.expandvars(doc.extloc if doc.extloc else doc.docloc)
                if isolated is False:
                    args["inputfile"] = infile
                else:
                    args["myFile"] = open(infile, "rb")
                args["outtype"] = params["imgtype"]
                if params["imgtype"] == "png":
                    args["pngoptions"] = 'color=color'
                    args["pngoptions"] += ",page=" + str(params["page"])
                if ievstream is not None:
                    ievstream.stream.seek(0)
                    args["ievstream"] = ievstream.stream
                self.log.debug(_('%s reqimgremote convert args: %s') %(self.name, str(args)))
                xml = None
                try:
                    result = c.convert(**args)
                except:
                    xml = bitfarm_xml.create_xml_request(reqtype="respDocError", userid=0, gdocid=self._job.gdocid,
                                                         data=_("Voransicht konnte nicht gerendert werden."))
                    self.log.warning(_("%s Voransicht konnte nicht gerendert werden." ) %(self.name))
                    if xml is not None:
                        try:
                            self._job.sock.send(xml)
                        except:
                            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                        return None
                if params["imgtype"] in result:
                    stream = io.BytesIO()
                    decodedresult = base64.b64decode(result[params["imgtype"]])
                    with open(previewfp, "wb") as pf:
                        pf.write(decodedresult)
                    stream.write(decodedresult)
                    datalen = stream.tell()
                    stream.seek(0)
                else:
                    self.log.warning(_("%s Voransicht konnte nicht gerendert werden.") %(self.name))
                    xml = bitfarm_xml.create_xml_request(reqtype="respDocError", userid=0, gdocid=self._job.gdocid,
                                                         data=_("Voransicht konnte nicht gerendert werden."))
                    try:
                        self._job.sock.send(xml)
                    except:
                        self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                    return None
            finally:
                self.bfaserver.converterhosts.put((server, isolated))

        else:
            self.log.debug(_('%s reqimgremote Voransicht ist vorhanden und aktuell.') %(self.name))
            stream = io.BytesIO()
            with open(previewfp, "rb") as pf:
                stream.write(pf.read())
            datalen = stream.tell()
            stream.seek(0)

        xml = bitfarm_xml.create_xml_request(reqtype="respImg", userid=0, datalen=datalen,
                                             filename=doc.title + "." + params["imgtype"], gdocid=self._job.gdocid,
                                             datatype=params["imgtype"])
        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None
        sent = 0
        while sent < datalen:
            buff = stream.read(self.bfaserver.split_size)
            try:
                self._job.sock.send(buff)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None
            sent += len(buff)
        self.check_logread(doc_right)

        return True

    def reqOcrXml(self):
        self.log.debug('%s reqOcrXml' % self.name)

        doc_right = self.check_docrights()

        if self._job.userid not in self.bfaserver.user[self._job.profile] or not doc_right.read:
            xml = bitfarm_xml.create_xml_request(reqtype="respDenied", userid=0, gdocid=self._job.gdocid)
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None
        doc = bitfarm_doc.BitfarmDocument()
        doc.profile = self._job.profile
        if type(self._job.parameter) in [str, str]:
            param = self._job.parameter.split(";")
        elif type(self._job.parameter) is list:
            param = self._job.parameter
        elif self._job.parameter is None:
            param = ['base', 'paths']
        if not self.bfaProcessing.load(doc, self._job.gdocid, ['base', 'paths'], logid=self.name):
            xml = bitfarm_xml.create_xml_request(reqtype="respDocError", userid=0, gdocid=self._job.gdocid)
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None
        else:
            infile = os.path.expandvars(doc.extloc if doc.extloc else doc.docloc)
            
            try:
                self.log.debug("infile: %s" % str(infile))
                tif = thtiflib.Tiff(infile, withmeta=True)
                resolution = [(ifd["fields"]["ImageWidth"][0], ifd["fields"]["ImageLength"][0]) for ifd in tif.ifds]
                self.log.debug("resolution: %s" % str(resolution))
            except:
                resolution = None
                
            xmlfilepath = os.path.splitext(infile)[0] + ".ocr_xml"
            if not os.path.isfile(xmlfilepath):
                xmlfilepath = os.path.splitext(infile)[0] + ".ocr_xml"
            if os.path.isfile(xmlfilepath):
                with codecs.open(xmlfilepath, "r", 'utf-16') as fobj:
                    xmlstr = fobj.read()
                    xml = bitfarm_xml.create_xml_request(reqtype="respOcrXml", userid=0, gdocid=self._job.gdocid, data=xmlstr)
                    try:
                        self._job.sock.send(xml)
                    except:
                        self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                    return None
            else:
                xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=self._job.gdocid, data="ocr.xml not found")
                try:
                    self._job.sock.send(xml)
                except:
                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None

    def reqFtxPos(self):
        self.log.debug('%s reqFtxPos' % self.name)

        doc_right = self.check_docrights()

        if self._job.userid not in self.bfaserver.user[self._job.profile] or not doc_right.read:
            xml = bitfarm_xml.create_xml_request(reqtype="respDenied", userid=0, gdocid=self._job.gdocid)
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None
        doc = bitfarm_doc.BitfarmDocument()
        doc.profile = self._job.profile
        if type(self._job.parameter) in [str, str]:
            param = self._job.parameter.split(";")
        elif type(self._job.parameter) is list:
            param = self._job.parameter
        elif self._job.parameter is None:
            param = ['base', 'paths']
        if not self.bfaProcessing.load(doc, self._job.gdocid, ['base', 'paths'], logid=self.name):
            xml = bitfarm_xml.create_xml_request(reqtype="respDocError", userid=0, gdocid=self._job.gdocid)
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None
        else:
            infile = os.path.expandvars(doc.extloc if doc.extloc else doc.docloc)
            
            try:
                self.log.debug("infile: %s" % infile)
                tif = thtiflib.Tiff(infile, withmeta=True)
                resolution = [(ifd["fields"]["ImageWidth"][0], ifd["fields"]["ImageLength"][0]) for ifd in tif.ifds]
                self.log.debug("resolution: %s" % resolution)
            except:
                resolution = None
                
            xmlfilepath = os.path.splitext(infile)[0] + ".ocr_xml"
            hocrfilepath = os.path.splitext(infile)[0] + ".hocr"
            if not os.path.isfile(xmlfilepath): 
                
                xml_doc_loc = os.path.expandvars(os.path.splitext(doc.docloc)[0])+".ocr_xml"
                if os.path.isfile(xml_doc_loc):
                    xmlfilepath=xml_doc_loc
                elif doc.extloc:
                    xml_ext_loc = os.path.expandvars(os.path.splitext(doc.extloc)[0])+".ocr_xml"
                    if os.path.isfile(xml_ext_loc):
                        xmlfilepath=xml_ext_loc
            if not os.path.isfile(hocrfilepath): 
                
                hocr_doc_loc = os.path.expandvars(os.path.splitext(doc.docloc)[0])+".hocr"
                if os.path.isfile(hocr_doc_loc):
                    hocrfilepath = hocr_doc_loc
                elif doc.extloc:
                    hocr_ext_loc = os.path.expandvars(os.path.splitext(doc.extloc)[0])+".hocr"
                    if os.path.isfile(hocr_ext_loc):
                        hocrfilepath = hocr_ext_loc
            if os.path.isfile(xmlfilepath):      
                self.log.debug("ocr_xml-file: %s" % xmlfilepath)
                xmlparser = bitfarm_xml.OcrXmlParser()
                xmlparser.readfile(xmlfilepath, resolution=resolution)
                # self.log.debug("xmlparser getwordpos for %s" %self._job.data[0])
                posdata = {}
                if type(self._job.data) in (str, str):
                    for w in self._job.data.split(" "):
                        self.log.debug("%s 1 xml wordpos for %s" % (self.name, w))
                        posdata[w] = xmlparser.getWordPos(w.strip(), exact=False, ignorecase=True, returndict=True)
                elif type(self._job.data) is list:
                    for w in self._job.data:
                        self.log.debug("%s 2 xml wordpos for %s" % (self.name, w.strip()))
                        posdata[w] = xmlparser.getWordPos(w.strip(), exact=False, ignorecase=True, returndict=True)
                else:
                    self.log.debug("%s ftxpos request data type %s" %(self.name, type(self._job.data)))
                self.log.debug("%s positions (ocr_xml): %s" % (self.name, posdata))
                xml = bitfarm_xml.create_xml_request(reqtype="respFtxPos", gdocid=self._job.gdocid, data=posdata)
                try:
                    self._job.sock.send(xml)
                except:
                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None
            elif os.path.isfile(hocrfilepath):
                self.log.debug("hocr-file: %s" % hocrfilepath)
                hocr = BfaHocr(hocrfilepath, parse_word_pos=True)
                posdata = hocr.get_wordpos(self._job.data.strip())

                posdata = {}
                if type(self._job.data) in (str, str):
                    for w in self._job.data.split(" "):
                        self.log.debug("%s 1 hocr wordpos for %s" % (self.name, w))
                        thisw = hocr.get_wordpos(w.strip())
                        if len(thisw[w]) > 0:
                            posdata[w] = thisw[w]
                elif type(self._job.data) == type([]):
                    for w in self._job.data:
                        self.log.debug("%s 2 hocr wordpos for %s" % (self.name, w.strip()))
                        thisw = hocr.get_wordpos(w.strip())
                        if len(thisw[w]) > 0:
                            posdata[w] = thisw[w]

                self.log.debug("%s positions (hocr): %s" % (self.name, posdata))
                xml = bitfarm_xml.create_xml_request(reqtype="respFtxPos", gdocid=self._job.gdocid, data=posdata)
                try:
                    self._job.sock.send(xml)
                except:
                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)

            else:
                xml = bitfarm_xml.create_xml_request(reqtype="respFtxPosError", gdocid=self._job.gdocid,
                                                     data="ocr_xml/hocr not found")
                try:
                    self._job.sock.send(xml)
                except:
                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None

    def reqDocOcr(self):
        """
        serverseitige OCR für bereits archivierte Dokumente:
            input-Daten: gdocid und Koordinaten
                         self._job.gdocid int und self._job.data (page, (top_left_x, top_left_y), (bottom_right_x, bottom_right_y)) mit page, x/y -> int
                         page beginnt mit 0!
                -> Originaltif laden, gemäß Koordinaten beschneiden mit FilterV4
                -> Ausschnitt mit bfa_tessocr Text erkennen
            output: respOcr mit unicode-Text oder reqDocOcrError
        """
        self.log.debug('%s reqDocOcr' % self.name)

        doc_right = self.check_docrights()

        if self._job.userid not in self.bfaserver.user[self._job.profile] or not doc_right.read:
            xml = bitfarm_xml.create_xml_request(reqtype="respDenied", userid=0, gdocid=self._job.gdocid)
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None
        doc = bitfarm_doc.BitfarmDocument()
        doc.profile = self._job.profile
        if not self.bfaProcessing.load(doc, self._job.gdocid, ['base', 'paths'], logid=self.name):
            xml = bitfarm_xml.create_xml_request(reqtype="respDocOcrError", userid=0, gdocid=self._job.gdocid)
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None
        else:
            infile = os.path.expandvars(doc.extloc if doc.extloc else doc.docloc)
        
        # croppen mit FilterV4
        filterv4exe = os.path.join(os.path.expandvars("%bitfarm-archiv%"), "filterv4.exe")
        (page, (top_left_x, top_left_y), (bottom_right_x, bottom_right_y)) = self._job.data
        with bfa_tempdir(prefix="bfa_ocr_") as td:
            outfile = os.path.join(td, "temp.tif")
            cmd = [filterv4exe, "-in=" + infile, "-page=" + str(page), "-cutleft=" + str(top_left_x), "-cuttop=" + str(top_left_y), "-cutwidth=" + str(bottom_right_x - top_left_x), \
                    "-cutheight=" + str(bottom_right_y - top_left_y), "-out="+outfile, "-savepage="+str(page)]
            self.log.debug(" ".join(cmd))
            timeout, rc, output = self._timeoutprocess(cmd)
            if timeout is True or rc != 0:
                errortxt = "error on reqDocOcr FilterV4!, timeout/rc/output: %s/%d/%s" %(str(timeout), rc, output)
                self.log.error(errortxt)
                xml = bitfarm_xml.create_xml_request(reqtype="respDocOcrError", userid=0, gdocid=self._job.gdocid, data=errortxt)
                try:
                    self._job.sock.send(xml)
                except:
                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None
            if not os.path.isfile(outfile):
                errortxt = "error on reqDocOcr FilterV4, outputfile not found."
                self.log.error(errortxt)
                xml = bitfarm_xml.create_xml_request(reqtype="respDocOcrError", userid=0, gdocid=self._job.gdocid, data=errortxt)
                try:
                    self._job.sock.send(xml)
                except:
                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None
            try:
                result = self.bfaocr.ocr(outfile, outformat=["txt"])
            except Exception as e:
                self.log.error(_("%s ocr exception: %s" % str(e)))
                xml = bitfarm_xml.create_xml_request(reqtype="respDocOcrError", userid=0, gdocid=self._job.gdocid, data="ocr exception")
                try:
                    self._job.sock.send(xml)
                except:
                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None
            ocrdata = result["txt"]

        xml = bitfarm_xml.create_xml_request(reqtype="respOcr", userid=0, gdocid=self._job.gdocid, data=ocrdata)
        # self.log.debug("xml response: ")
        # self.log.debug(xml)
        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
        return None
    
    def reqImgOcr(self):
        """
        serverseitige OCR für Bilder/Bildausschnitte:
            input-Daten: self._job.data -> (image-Stream (base64-codiert), filetype (z.B. ".tif"))
                -> image-Stream temporär speichern
                -> mit bfa_tessocr Text erkennen
            output: respOcr mit unicode-Text oder reqImgOcrError
        """
        self.log.debug('%s reqImgOcr' % self.name)
        try:
            if type(self._job.data) is tuple:
                b64_image_stream, filetype = self._job.data
                image_stream = base64.b64decode(b64_image_stream)
            else:
                image_stream = base64.b64decode(self._job.data)
                filetype = self._job.filename
        except Exception as e:
            self.log.error("%s reqImgOcr exception parsing job.data: %s" % str(e))
            xml = bitfarm_xml.create_xml_request(reqtype="respDocOcrError", userid=0, gdocid=self._job.gdocid, data="ocr exception")
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

        with bfa_tempdir(prefix="bfa_img_ocr_") as td:
            inputfile = os.path.join(td, "temp-ocr" + filetype)
            with open(inputfile, "wb") as fobj:
                fobj.write(image_stream)
            try:
                ocrresult = self.bfaocr.ocr(inputfile, addorientation=True, numproc=1)
            except Exception as e:
                self.log.error("%s ocr exception: %s" % (self.name, str(e)))
                xml = bitfarm_xml.create_xml_request(reqtype="respImgOcrError", userid=0, gdocid=self._job.gdocid, data="ocr exception")
            else:
                xml = bitfarm_xml.create_xml_request(reqtype="respOcr", userid=0, gdocid=self._job.gdocid, data=ocrresult)

            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqGetOCRText(self):
        """
        Viewerfreundliche serverseitige OCR für Bilder/Bildausschnitte:
            input-Daten: self._job.data -> image-Stream (base64-codiert)
                         self._job.filename -> Filetyp (Endung, zb .tif)
                -> image-Stream temporär speichern
                -> mit bfa_tessocr Text erkennen
            output: respOcr mit unicode-Text in data oder reqImgOcrError
        """
        self.log.debug('%s reqGetOCRText' % self.name)
        try:
            image_stream = base64.b64decode(self._job.data)
            filetype = self._job.filename
        except Exception as e:
            self.log.error("%s reqImgOcr exception parsing job.data: %s" % str(e))
            xml = bitfarm_xml.create_xml_request(reqtype="respDocOcrError", userid=0, gdocid=self._job.gdocid, data="ocr exception")
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

        with bfa_tempdir(prefix="bfa_img_ocr_") as td:
            inputfile = os.path.join(td, "temp-ocr" + filetype)
            with open(inputfile, "wb") as fobj:
                fobj.write(image_stream)
            try:
                ocrresult = self.bfaocr.ocr(inputfile, outformat=["txt"], addorientation=False, numproc=1)
            except Exception as e:
                self.log.error("%s ocr exception: %s" % (self.name, str(e)))
                xml = bitfarm_xml.create_xml_request(reqtype="respImgOcrError", userid=0, gdocid=self._job.gdocid, data="Fehler bei Bereichs-OCR")
            else:
                if 'txt' in ocrresult:
                    xml = bitfarm_xml.create_xml_request(reqtype="respOcr", userid=0, gdocid=self._job.gdocid, data=ocrresult['txt'])
                else:
                    xml = bitfarm_xml.create_xml_request(reqtype="respImgOcrError", userid=0, gdocid=self._job.gdocid, data="Fehler bei Bereichs-OCR")

            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqHistory(self):
        self.log.debug('%s reqHistory' % self.name)

        if self._job.gdocid is None:
            if 'admin' not in self.bfaserver.user[self._job.profile][self._job.userid].flags:
                xml = bitfarm_xml.create_xml_request(reqtype="respDenied")
                try:
                    self._job.sock.send(xml)
                except:
                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None
        else:
            doc_right = self.check_docrights()

            if self._job.userid not in self.bfaserver.user[self._job.profile] or not doc_right.history:
                xml = bitfarm_xml.create_xml_request(reqtype="respDenied", userid=0, gdocid=self._job.gdocid)
                try:
                    self._job.sock.send(xml)
                except:
                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None

        if type(self._job.parameter) in [str, str]:
            param = self._job.parameter.split(";")
        elif type(self._job.parameter) is list:
            param = self._job.parameter
        else:
            param = []

        hist_params = {}
        asjson = False
        if "base" in param:
            param.remove("base")
        if "asjson" in param:
            asjson = True
            param.remove("asjson")

        for item in param:
            key, value = item.split(':')
            hist_params[key] = value

        history = self.bfaProcessing.read_history(gdocid=self._job.gdocid, profile=self._job.profile, params=hist_params)
        # Parsen und übersetzen
        for hist in history:
            if hist.taskid in self.bfaserver.con.connections[self._job.profile].history_map:
                hist._msg = _(self.bfaserver.con.connections[self._job.profile].history_map[hist.taskid], self._job.lang)
                if hist.value1 == 'leer':
                    hist.value1 = _('leer', self._job.lang)
                if hist.value2 == 'leer':
                    hist.value2 = _('leer', self._job.lang)
                if hist.value3 == 'leer':
                    hist.value3 = _('leer', self._job.lang)

                if hist._msg.count('%s') == 1:
                    hist._msg = hist.msg % hist.value1
                elif hist._msg.count('%s') == 2:
                    hist._msg = hist.msg % (hist.value1, hist.value2)
                elif hist._msg.count('%s') == 3:
                    hist._msg = hist.msg % (hist.value1, hist.value2, hist.value3)

        if asjson is False:
            xml = bitfarm_xml.create_xml_request(reqtype="respHistory", gdocid=self._job.gdocid, data=history)
        else:
            # docdata = json.dumps(bitfarm_xml.obj_to_dict(doc))
            xml = bitfarm_xml.create_xml_request(reqtype="respHistory", gdocid=self._job.gdocid,
                                                 data=bitfarm_xml.obj_to_dict(history), parameter="json")
        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqDoc(self):
        self.log.debug('%s reqDoc' % self.name)

        doc_right = self.check_docrights()

        if self._job.userid not in self.bfaserver.user[self._job.profile] or not doc_right.read:
            xml = bitfarm_xml.create_xml_request(reqtype="respDenied", userid=0, gdocid=self._job.gdocid)
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None
        doc = bitfarm_doc.BitfarmDocument()
        doc.profile = self._job.profile
        if self._job.fields_arcid is not None:
            doc.fields_arcid = self._job.fields_arcid
        # if self._job.fields_arcid is None and self._job.symbolic_arcid is not None:
        #     doc.fields_arcid = self._job.symbolic_arcid

        if type(self._job.parameter) in [str, str]:
            param = self._job.parameter.split(";")
        elif type(self._job.parameter) is list:
            param = self._job.parameter
        elif self._job.parameter is None:
            param = ['base', 'paths']
        self.log.debug("%s reqdoc params %s" % (self.name, str(param)))
        if not self.bfaProcessing.load(doc, self._job.gdocid, param, logid=self.name):
            xml = bitfarm_xml.create_xml_request(reqtype="respDocError", userid=0, gdocid=self._job.gdocid)
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None
        else:
            if "iev" in self._job.parameter:
                self.add_private_annotations(doc, doc_right)
            doc.rights = doc_right.rights

            doc.in_editmode = self.get_inedit()

            filepath = os.path.expandvars(doc.extloc if doc.extloc else doc.docloc)
            self.log.debug("%s reqdoc filepath %s" % (self.name, filepath))
            self.log.debug("%s reqdoc doc.extloc %s" % (self.name, doc.extloc))
            self.log.debug("%s reqdoc doc.docloc %s" % (self.name, doc.docloc))
            if doc.pages in (None, 0):
                if os.path.splitext(filepath)[1].lower() == ".tif":
                    doc.pages = self.countpages(filepath)
                else:
                    doc.pages = 1
                # self.bfaProcessing.write(doc)

                # if "iev" in self._job.parameter:
                # if doc.annotations is None or len(doc.annotations)==0:
                # doc.annotations = self.get_annotations(doc.extloc if doc.extloc else doc.docloc)
            self.log.debug("%s reqdoc iev lookup" % self.name)
            if "iev" in self._job.parameter and 'include_annostream' not in self._job.parameter:
                for anno in doc.annotations:
                    # self.log.debug("reqDoc, anno: %s" %str(anno))
                    # self.log.debug("reqDoc, anno props: %s" %str(dir(anno)))
                    try:
                        del anno.iev
                        del anno.right
                    except:
                        pass

            if self.bfaserver.develop and "iev" in self._job.parameter:
                fnames = glob.glob(self.bfaserver.mypath + r'\%d*.svg' % self._job.gdocid)
                for fname in fnames:
                    fbase = os.path.basename(fname)
                    fbase = os.path.splitext(fbase)[0]
                    data = fbase.split('_')
                    anno = bitfarm_doc.BitfarmAnnotation()
                    anno.layer = int(data[1])
                    anno.page = int(data[2])
                    anno.modified_date = datetime.datetime.now()
                    with open(fname, 'r') as f:
                        svg = f.read()
                    anno.svg = svg
                    doc.annotations.append(anno)

            if self._job.gdocid in self.bfaserver.in_progress[self._job.profile]:
                doc.in_progress = self.bfaserver.in_progress[self._job.profile][self._job.gdocid]

            # Hier könnte man noch Volumen sparen, derzeit verwendet die App/Webclient diese Daten aber noch um in
            # for field in doc.fields.values():     # der Detailansicht die Zusatzfelder aufzubauen.
            #     field.items = []

            if "asjson" not in param:
                xml = bitfarm_xml.create_xml_request(reqtype="respDoc", gdocid=self._job.gdocid, data=doc,
                                                     datatype='BitfarmDocument', parameter="xml")
            else:
                # docdata = json.dumps(bitfarm_xml.obj_to_dict(doc))
                xml = bitfarm_xml.create_xml_request(reqtype="respDoc", gdocid=self._job.gdocid,
                                                     data=bitfarm_xml.obj_to_dict(doc), datatype='BitfarmDocument',
                                                     parameter="json", shortxml=False)
            self.log.debug("%s reqdoc gzip" % self.name)
            if "gzip" in param:
                out = io.BytesIO()
                with gzip.GzipFile(fileobj=out, mode="w") as f:
                    f.write(xml)
                xml = out.getvalue()
            self.log.debug(_("%s reqdoc sende xml") % self.name)
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None

    def reqGetBookmarks(self):
        self.log.debug('%s reqGetBookmarks' % self.name)

        bookmarks = []
        types =[]

        params = self._job.parameter.split(';')
        if 'globale' in params:
            types.append('0')
        if 'locale' in params:
            types.append('1')
        if 'history' in params:
            types.append('2')

        user = self.bfaserver.con.connections[self._job.profile].read_user(db=self.db[self._job.profile], userid=self._job.userid, logid=self.name)
        groups = ",".join([str(x) for x in user.grouplist])
        # types = ",".join(types)

        for favtype in types:
            if favtype == '0':    # globale Lesezeichen
                users = self.bfaserver.con.connections[self._job.profile].get_holidaylist(db=self.db[self._job.profile], user=self._job.userid, logid=self.name)
                users = ",".join([str(x) for x in users])
                sql = 'select * from sys_bookmarks where type in (%s) and (userid in (%s) or (userid = 0 and groupid in (%s)))' % (favtype, users, groups)
            else:
                sql = 'select * from sys_bookmarks where type in (%s) and (userid = %s or (userid = 0 and groupid in (%s)))' % (favtype, user.id, groups)

            try:
                result = self.db[self._job.profile].execute(sql).fetchall()
            except:
                self.log.exception(_("%s Fehler beim Lesen der Lesezeichen") % self.name)
            else:
                if result:
                    for row in result:
                        sobj = row['bookmark'].encode('utf8')
                        try:
                            result2 = self.SearchProc.read_xml(sobj)
                        except:
                            self.log.exception("Fehler beim Lesen des lesezeichens %s" % sobj)
                        else:
                            if result2 is not None:
                                i = 0
                                for obj in result2:
                                    if row['userid'] != self._job.userid and obj.dont_inheritate:
                                        continue
                                    obj.id = "%s,%s,%s,%s;%s;%s" % (row['type'], row['userid'], row['groupid'], str(i), 'constructive' if obj.constructive else 'strict', str(obj.selectedarcid))
                                    bookmarks.append(obj)
                                    i += 1

        if "asjson" not in params:
            xml = bitfarm_xml.create_xml_request(reqtype="respGetBookmarks", data=bookmarks)
        else:
            bookmarklist = ["%s;%s" % (obj.name, obj.id) for obj in bookmarks]
            xml = bitfarm_xml.create_xml_request(reqtype="respGetBookmarks", data=bitfarm_xml.obj_to_dict(bookmarklist), parameter="json")

        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqExecuteBookmark(self):
        self.log.debug('%s reqExecuteBookmark' % self.name)
        xml = ""
        bookmarks = []
        sobj = None
        guid = None
        params = self._job.parameter.split(';')
        try:
            typ, userid, groupid, index = params[0].split(',')
            if self.bfaserver.con.main.is_guid(index):
                guid = index
            else:
                index = int(index)
        except:
            self.log.error("Keine gültige ID des Lesezeichens übermittelt.")
            xml = bitfarm_xml.create_xml_request(reqtype="respError", data=_("Keine gültige ID des Lesezeichens übermittelt."))
        else:
            sobj = self.load_bookmark(self._job.profile, typ, userid, groupid, index, guid)


            # sql = 'select * from sys_bookmarks where type = %s and userid = %s and groupid = %s' % (typ, userid, groupid)
            #
            # try:
            #     result = self.db[self._job.profile].execute(sql).fetchall()
            # except:
            #     self.log.exception(_("%s Fehler beim Lesen der Lesezeichen") % self.name)
            # else:
            #     if result:
            #         for row in result:
            #             value = row['bookmark'].encode('utf8')
            #             try:
            #                 bookmarks = self.SearchProc.read_xml(value)
            #             except:
            #                 self.log.exception("Fehler beim Lesen des Lesezeichens %s" % value)
            #             else:
            #                 if bookmarks is not None and len(bookmarks) > 0:
            #                     if guid:
            #                         for bookmark in bookmarks:
            #                             if bookmark.guid is not None and bookmark.guid.lower() == guid.lower():
            #                                 sobj = bookmark
            #                                 break
            #                     elif guid is None and index < len(bookmarks):
            #                         sobj = bookmarks[index]
            #
            #                     if sobj is not None:
            #                         sobj.user_context = self._job.userid
            #                         sobj.arcid = sobj.arcid if sobj.arcid is not None else sobj.selectedarcid
            #                         sobj.order = 'docdate'
            #                         sobj.direction = 'desc'
            #                         if len(params) > 1:
            #                             sobj.debug = params[1] == 'debug'
            #                         if sobj.selected_cols is None:
            #                             sobj.selected_cols = ['gdocid','docname','su.username','docdate']
            #                         if 'gdocid' not in sobj.selected_cols:
            #                             sobj.selected_cols.append('gdocid')
            #                         if 'docname' not in sobj.selected_cols:
            #                             sobj.selected_cols.append('docname')
            #                         if 'su.username' not in sobj.selected_cols and 'username' not in sobj.selected_cols:
            #                             sobj.selected_cols.append('su.username')
            #                         if 'docdate' not in sobj.selected_cols:
            #                             sobj.selected_cols.append('docdate')

            if sobj is None:
                self.log.warning("Kein Lesezeichen für diese ID gefunden %s,%s,%s,%s" % (typ, userid, groupid, index))
                xml = bitfarm_xml.create_xml_request(reqtype="respError", data=_("Lesezeichen konnte nicht gefunden werden."))
            else:
                if len(params) > 1:
                    sobj.debug = 'debug' in params
                    usercontext = None
                    usercontext = [int(x[12:]) for x in params if x[:11] == 'usercontext']
                    if usercontext:
                        sobj.user_context = usercontext[0]
                self._job.data = sobj
                self.reqSearch()

        if xml:
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None

    def load_bookmark(self, profile, typ, userid, groupid, index, guid):
        self.log.debug("load_bookmark %s %s %s %s %s" % (profile, typ, userid, groupid, index))
        sobj = None

        sql = 'select * from sys_bookmarks where type = %s and userid = %s and groupid = %s' % (typ, userid, groupid)

        try:
            result = self.db[profile].execute(sql).fetchall()
        except:
            self.log.exception(_("%s Fehler beim Lesen der Lesezeichen") % self.name)
        else:
            if result:
                for row in result:
                    value = row['bookmark'].encode('utf8')
                    try:
                        bookmarks = self.SearchProc.read_xml(value)
                    except:
                        self.log.exception("Fehler beim Lesen des Lesezeichens %s" % value)
                    else:
                        if bookmarks is not None and len(bookmarks) > 0:
                            if guid:
                                for bookmark in bookmarks:
                                    if bookmark.guid is not None and bookmark.guid.lower() == guid.lower():
                                        sobj = bookmark
                                        break
                            elif guid is None and index < len(bookmarks):
                                sobj = bookmarks[index]

                            if sobj is not None:
                                sobj.user_context = self._job.userid
                                sobj.arcid = sobj.arcid if sobj.arcid is not None else sobj.selectedarcid
                                sobj.order = 'docdate'
                                sobj.direction = 'desc'

                                if sobj.selected_cols is None:
                                    sobj.selected_cols = ['gdocid','docname','su.username','docdate']
                                if 'gdocid' not in sobj.selected_cols:
                                    sobj.selected_cols.append('gdocid')
                                if 'docname' not in sobj.selected_cols:
                                    sobj.selected_cols.append('docname')
                                if 'su.username' not in sobj.selected_cols and 'username' not in sobj.selected_cols:
                                    sobj.selected_cols.append('su.username')
                                if 'docdate' not in sobj.selected_cols:
                                    sobj.selected_cols.append('docdate')
                                break
        return sobj

    def reqSaveBookmarks(self):
        self.log.debug('%s reqSaveBookmarks' % self.name)

        if self._job.data is None or self._job.data == '':
            xml = bitfarm_xml.create_xml_request(reqtype="respError", data=_("Keine Lesezeichen übertragen"))
        else:
            self._job.data = self._job.data.replace(r'%', r'%%')
            self._job.data = self._job.data.replace(chr(39), chr(92) + chr(39))  # escape high comma

            typ, userid, groupid, name = self._job.parameter.split(';')

            sql = "insert into sys_bookmarks (type, userid, groupid, name, bookmark) values (%s, %s, %s, '%s' ,'%s') on duplicate key update bookmark = '%s', name = '%s'" % (
                    typ, userid, groupid, name, self._job.data, self._job.data, name)

            try:
                self.db[self._job.profile].execute(sql)
                xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("reqSaveBookmarks ausgeführt"))
            except:
                self.log.exception(_("%s Fehler beim Speichern der Lesezeichen") % self.name)
                xml = bitfarm_xml.create_xml_request(reqtype="respError", data=_("Fehler beim Speichern der Lesezeichen"))

        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqDeleteGlobalBookmark(self):
        self.log.debug('%s reqDeleteGlobalBookmark' % self.name)
        typ, userid, groupid = self._job.parameter.split(';')
        sql = "delete from sys_bookmarks where type = 0 and userid = %s and groupid = %s" % (userid, groupid)

        try:
            self.db[self._job.profile].execute(sql)
            xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("reqDeleteGlobalBookmark ausgeführt"))
        except:
            self.log.exception(_("%s Fehler beim Löschen des Lesezeichens") % self.name)
            xml = bitfarm_xml.create_xml_request(reqtype="respError", data=_("Fehler beim Speichern der Lesezeichen"))

        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqCreateCommentary(self):
        self.log.debug('%s reqCreateCommentary' % self.name)
        _con = self.bfaserver.con.connections[self._job.profile]
        doc_right = self.check_docrights()

        if self._job.userid not in self.bfaserver.user[self._job.profile] or (not doc_right.awrite and not doc_right.amodify):
            xml = bitfarm_xml.create_xml_request(reqtype="respDenied", userid=0, gdocid=self._job.gdocid)
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

        # self._job.data = self._job.data.replace(r'%', r'%%')
        self._job.data = self._job.data.replace(chr(39), chr(92) + chr(39))  # escape high comma

        # sql = "insert into sys_com (gdocid, userid, comment) values (%s, %s, '%s')" % (
        # self._job.gdocid, self._job.userid, self._job.data)

        vals = {_con.sys_com.c.gdocid: self._job.gdocid,
                _con.sys_com.c.userid: self._job.userid,
                _con.sys_com.c.comment: self._job.data}

        sql = sqla.insert(_con.sys_com, values=vals)

        try:
            self.db[self._job.profile].execute(sql)
            xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("reqCreateCommentary ausgeführt"))

            hist = bitfarm_doc.BitfarmHistory()
            hist.gdocid = self._job.gdocid
            hist.taskid = 309
            hist.userid = self._job.userid
            self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)
        except:
            self.log.exception(_("%s Fehler beim Speichern des Kommentars") % self.name)
            xml = bitfarm_xml.create_xml_request(reqtype="respError", data=_("Fehler beim Speichern des Kommentars"))

        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqModifyCommentary(self):
        self.log.debug('%s reqModifyCommentary' % self.name)
        _con = self.bfaserver.con.connections[self._job.profile]
        doc_right = self.check_docrights()

        if self._job.userid not in self.bfaserver.user[self._job.profile] or (not doc_right.awrite and not doc_right.amodify):
            xml = bitfarm_xml.create_xml_request(reqtype="respDenied", userid=0, gdocid=self._job.gdocid)
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

        # self._job.data = self._job.data.replace(r'%', r'%%')
        self._job.data = self._job.data.replace(chr(39), chr(92) + chr(39))  # escape high comma

        sql = "update sys_com set comment = '%s' where commentid = %s" % (self._job.data, self._job.parameter)

        vals = {_con.sys_com.c.comment: self._job.data}

        sql = sqla.update(_con.sys_com, values=vals).where(_con.sys_com.c.commentid == int(self._job.parameter))

        try:
            self.db[self._job.profile].execute(sql)
            xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("reqModifyCommentary ausgeführt"))

            hist = bitfarm_doc.BitfarmHistory()
            hist.gdocid = self._job.gdocid
            hist.taskid = 310
            hist.userid = self._job.userid
            self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)
        except:
            self.log.exception(_("%s Fehler beim Ändern des Kommentars") % self.name)
            xml = bitfarm_xml.create_xml_request(reqtype="respError", data=_("Fehler beim Ändern des Kommentars"))

        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqDeleteCommentary(self):
        self.log.debug('%s reqDeleteCommentary' % self.name)

        doc_right = self.check_docrights()

        if self._job.userid not in self.bfaserver.user[self._job.profile] or (not doc_right.awrite and not doc_right.amodify):
            xml = bitfarm_xml.create_xml_request(reqtype="respDenied", userid=0, gdocid=self._job.gdocid)
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

        sql = "delete from sys_com where commentid = %s and userid = %s" % (self._job.parameter, self._job.userid)

        try:
            self.db[self._job.profile].execute(sql)
            xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("reqDeleteCommentary ausgeführt"))

            hist = bitfarm_doc.BitfarmHistory()
            hist.gdocid = self._job.gdocid
            hist.taskid = 311
            hist.userid = self._job.userid
            self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)
        except:
            self.log.exception(_("%s Fehler beim Löschen des Kommentars") % self.name)
            xml = bitfarm_xml.create_xml_request(reqtype="respError", data=_("Fehler beim Löschen des Kommentars"))

        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqCreateSymbolicLink(self):
        self.log.debug('%s reqCreateSymbolicLink' % self.name)
        moves = self._job.parameter.split(';')
        msg = ""
        error = ""

        if self._job.data is not None:
            if type(self._job.data) is not list:
                self._job.data = [self._job.data]

        for move in moves:
            gdocid, src_arcid, dest_arcid = move.split(',')

            sql = "select gdocid, arcid from sys_symlink where gdocid = %s and arcid = %s" % (gdocid, src_arcid)

            try:
                res = self.db[self._job.profile].execute(sql).fetchall()
            except:
                self.log.exception(_("%s Fehler beim Prüfen der Symbolic Links [%s]") %(self.name, gdocid))
                error += _("Fehler beim Prüfen der Symbolic Links [%s]") % gdocid
            else:
                if len(res) == 0:    # haupteintrag erstellen (die arcid des ursprungsdokuments)
                    sql = "insert into sys_symlink (gdocid, arcid) values (%s, %s)" % (gdocid, src_arcid)

                    try:
                        self.db[self._job.profile].execute(sql)
                    except:
                        self.log.exception(_("%s Fehler beim Erstellen des Symbolic Links [%s] (Haupteintrag)") %(self.name, gdocid))
                        error += _("Fehler beim Erstellen des Symbolic Links [%s] (Haupteintrag)") % gdocid

                sql = "select gdocid, arcid from sys_symlink where gdocid = %s and arcid = %s" % (gdocid, dest_arcid)

                try:
                    res = self.db[self._job.profile].execute(sql).fetchall()
                except:
                    self.log.exception(_("%s Fehler beim Prüfen der Symbolic Links [%s]") %(self.name, gdocid))
                    error += _("Fehler beim Prüfen der Symbolic Links [%s]") % gdocid
                else:
                    if len(res) == 0:    # symbolic link erstellen
                        sql = "insert into sys_symlink (gdocid, arcid) values (%s, %s)" % (gdocid, dest_arcid)

                        try:
                            self.db[self._job.profile].execute(sql)
                        except:
                            self.log.exception(_("%s Fehler beim Erstellen des Symbolic Links [%s]") %(self.name, gdocid))
                            error += _("Fehler beim Erstellen des Symbolic Links [%s]") % gdocid
                        else:
                            hist = bitfarm_doc.BitfarmHistory()
                            hist.gdocid = gdocid
                            hist.taskid = 228
                            hist.userid = self._job.userid
                            hist.value1 = self.bfaserver.archives[self._job.profile][int(dest_arcid)].arcname
                            self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            error, msg = self.save_objects(dest_arcid, gdocid, error, msg)
                    else:
                        error += _("Der Symbolic Link ist in diesem Archiv schon vorhanden. [%s]") % gdocid
        if error:
            xml = bitfarm_xml.create_xml_request(reqtype="respError", data=error, datatype="text")
        else:
            xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("Symbolic Link erzeugt."))
        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqDeleteSymbolicLink(self):
        self.log.debug('%s reqDeleteSymbolicLink' % self.name)

        deletes = self._job.parameter.split(';')

        for delete in deletes:
            gdocid, symlink_arcid = delete.split(',')

            sql = "select gdocid, arcid from sys_symlink where gdocid = %s and arcid = %s" % (gdocid, symlink_arcid)

            try:
                res = self.db[self._job.profile].execute(sql).fetchall()
            except:
                self.log.exception(_("%s Fehler beim Prüfen der Symbolic Links") %(self.name))
                xml = bitfarm_xml.create_xml_request(reqtype="respError",
                                                     data=_("Fehler beim Prüfen der Symbolic Links [%s]") % gdocid)
                break
            else:
                if len(res) > 0:
                    sql = "delete from sys_symlink where gdocid = %s and arcid = %s" % (gdocid, symlink_arcid)

                    try:
                        self.db[self._job.profile].execute(sql)
                    except:
                        self.log.exception(_("%s Fehler beim Löschen des Symbolic Links") %(self.name))
                        xml = bitfarm_xml.create_xml_request(reqtype="respError", data=_("Fehler beim Löschen des Symbolic Links [%s]") % gdocid)
                        break
                    else:
                        xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("Symbolic Link gelöscht [%s]") % gdocid)

                        hist = bitfarm_doc.BitfarmHistory()
                        hist.gdocid = gdocid
                        hist.userid = self._job.userid
                        hist.taskid = 251
                        hist.value1 = self.bfaserver.archives[self._job.profile][int(symlink_arcid)].arcname
                        self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)
                else:
                    xml = bitfarm_xml.create_xml_request(reqtype="respError", data=_(
                        "Symbolic Link nicht in diesem Archiv vorhanden [%s]") % gdocid)
                    break

            # wir prüfen, ob noch genau ein eintrag mit dieser gdocid vorhanden ist. das ist der basiseintrag des hauptdokuments
            # der muss auch entfernt werden
            sql = "select gdocid from sys_symlink where gdocid = %s " % (gdocid)

            try:
                res = self.db[self._job.profile].execute(sql).fetchall()
            except:
                self.log.exception(_("%s Fehler beim Prüfen der Symbolic Links (Haupteintrag)") %(self.name))
                xml = bitfarm_xml.create_xml_request(reqtype="respError", data=_(
                    "Fehler beim Prüfen der Symbolic Links (Haupteintrag) [%s]") % gdocid)
                break
            else:
                if len(res) == 1:
                    sql = "delete from sys_symlink where gdocid = %s " % (gdocid)

                    try:
                        self.db[self._job.profile].execute(sql)
                    except:
                        self.log.exception(_("%s Fehler beim Löschen des Symbolic Links (Haupteintrag)") %(self.name))
                        xml = bitfarm_xml.create_xml_request(reqtype="respError", data=_(
                            "Fehler beim Löschen des Symbolic Links (Haupteintrag) [%s]") % gdocid)
                        break
                    else:
                        xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("Symbolic Link gelöscht (Haupteintrag) [%s]") % gdocid)

        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqDeleteDoc(self):
        self.log.debug('%s reqDeleteDoc' % self.name)

        con = self.bfaserver.con.connections[self._job.profile]
        params = self._job.parameter.split(";")

        msg = ""
        _ret = True

        for gdocid in params:
            doc = bitfarm_doc.BitfarmDocument()
            doc.profile = self._job.profile
            if not self.bfaProcessing.load(doc, int(gdocid), ['base', 'paths'], logid=self.name):
                self.log.error(_("%s Dokument [%s] konnte nicht geladen werden") % (self.name, gdocid))
                xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, data="Dokument [%s] konnte nicht geladen werden." % gdocid)
            else:
                if doc.is_revisionsave:
                    msg += _("Dokument [%s] liegt revisionssicher.\n" % gdocid)
                    continue

                if gdocid in self.bfaserver.in_progress[self._job.profile]:
                    msg += _("Dokument [%s] ist in Bearbeitung.\n" % gdocid)
                    continue

                if doc.is_checkout:
                    msg += _("Dokument [%s] ist ausgecheckt.\n" % gdocid)
                    continue

                sql = 'select count(*) from sys_timer where ((userid_work <> 0 and groupid = 0) or groupid = 0) and (status & 4 = 0) and gdocid = %s' % gdocid

                try:
                    result = self.db[self._job.profile].execute(sql).fetchone()
                except:
                    self.log.exception(_("%s Fehler beim Abfragen der sys_timer") % self.name)
                else:
                    if result[0] > 0:
                        msg += _("Dokument [%s] liegt auf Wiedervorlage." % gdocid)
                        continue

                if doc.oriloc:
                    # sanity check
                    if doc.oriloc.lower().find(con.archive_rs.lower()) > -1:
                        msg += _("Dokument [%s] liegt revisionssicher." % gdocid)
                        continue
                    else:
                        try:
                            if os.path.exists(os.path.expandvars(doc.oriloc)):
                                os.remove(os.path.expandvars(doc.oriloc))
                        except:
                            self.log.exception("%s Fehler beim Löschen einer NRS-Datei aus oriloc %s" % (self.name, doc.oriloc))
                            msg += _("Fehler beim Löschen der Datei (Originaldatei) [%s]\n" % gdocid)
                            _ret = _ret and False
                        else:
                            self.log.warning("%s NRS Dokument wurde gelöscht %s" % (self.name, doc.oriloc))

                if doc.docloc:
                    # sanity check
                    if doc.docloc.lower().find(con.archive_rs.lower()) > -1:
                        msg += _("Dokument [%s] liegt revisionssicher." % gdocid)
                        continue
                    else:
                        try:
                            if os.path.exists(os.path.expandvars(doc.docloc)):
                                os.remove(os.path.expandvars(doc.docloc))
                        except:
                            self.log.exception("%s Fehler beim Löschen einer NRS-Datei aus docloc %s" % (self.name, doc.oriloc))
                            msg += _("Fehler beim Löschen der Datei (Voransicht) [%s]\n" % gdocid)
                            _ret = _ret and False
                        else:
                            self.log.warning("%s NRS Dokument wurde gelöscht %s" % (self.name, doc.oriloc))

                if doc.extloc:
                    # sanity check
                    if doc.extloc.lower().find(con.archive_rs.lower()) > -1:                # wenn das passiert, dann ist vorher schon gewaltig was schiefgelaufen
                        msg += _("Dokument [%s] liegt revisionssicher." % gdocid)
                        continue
                    else:
                        try:
                            if os.path.exists(os.path.expandvars(doc.extloc)):
                                os.remove(os.path.expandvars(doc.extloc))
                        except:
                            self.log.exception("%s Fehler beim Löschen einer NRS-Datei aus extloc %s" % (self.name, doc.extloc))
                            msg += _("Fehler beim Löschen der Datei (Voransichtskopie) [%s]\n" % gdocid)
                            _ret = _ret and False
                        else:
                            self.log.warning("%s NRS Dokument wurde gelöscht %s" % (self.name, doc.extloc))

                sql = 'delete from sys_doc where gdocid = %s' % gdocid
                try:
                    self.db[self._job.profile].execute(sql)
                except:
                    self.log.exception(_("%s Fehler beim Löschen des Dokuments aus der sys_doc") % self.name)
                    _ret = _ret and False

                sql = 'delete from sys_history where gdocid = %s' % gdocid
                try:
                    self.db[self._job.profile].execute(sql)
                except:
                    self.log.exception(_("%s Fehler beim Löschen des Dokuments aus der sys_history") % self.name)
                    _ret = _ret and False

                sql = 'delete from sys_annotations where gdocid = %s' % gdocid
                try:
                    self.db[self._job.profile].execute(sql)
                except:
                    self.log.exception(_("%s Fehler beim Löschen des Dokuments aus der sys_annotations") % self.name)
                    _ret = _ret and False

                # sql = 'delete from sys_doctxt where gdocid = %s' % gdocid
                # try:
                #     self.db[self._job.profile].execute(sql)
                # except:
                #     self.log.exception(_(u"%s Fehler beim Löschen des Dokuments aus der sys_doctxt") % self.name)
                #     _ret = _ret and False

                sql = 'delete from sys_doclink where gdocid = %s' % gdocid
                try:
                    self.db[self._job.profile].execute(sql)
                except:
                    self.log.exception(_("%s Fehler beim Löschen des Dokuments aus der sys_doclink") % self.name)
                    _ret = _ret and False

                sql = 'delete from sys_symlink where gdocid = %s' % gdocid
                try:
                    self.db[self._job.profile].execute(sql)
                except:
                    self.log.exception(_("%s Fehler beim Löschen des Dokuments aus der sys_symlink") % self.name)
                    _ret = _ret and False

                for idx in con.sys_docidx.values():
                    sql = 'delete from %s where gdocid = %s' % (idx.name, gdocid)
                    try:
                        self.db[self._job.profile].execute(sql)
                    except:
                        self.log.exception(_("%s Fehler beim Löschen des Dokuments aus der %s") % (self.name, idx.name))
                        _ret = _ret and False

                hist = bitfarm_doc.BitfarmHistory()
                hist.gdocid = 0  # 0 = Admin Einträge
                hist.userid = self._job.userid
                hist.taskid = 231
                hist.value1 = gdocid
                hist.value2 = self._job.userid
                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

        if not _ret or msg != "":
            xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, data=msg)
        else:
            xml = bitfarm_xml.create_xml_request(reqtype="respOK", userid=0, data=_("reqDeleteDoc ausgeführt"))

        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqResetPreview(self):
        self.log.debug('%s reqResetPreview' % self.name)

        con = self.bfaserver.con.connections[self._job.profile]
        try:
            params = self._job.parameter.split(";")
        except:
            self.log.exception("%s Fehler bei der Parameterübergabe" % self.name)
            params = []
            _ret = False
        else:
            msg = ""
            _ret = True

        for gdocid in params:
            doc = bitfarm_doc.BitfarmDocument()
            doc.profile = self._job.profile
            if not self.bfaProcessing.load(doc, int(gdocid), ['base', 'paths'], logid=self.name):
                self.log.error(_("%s Dokument [%s] konnte nicht geladen werden") % (self.name, gdocid))
            else:
                if not doc.extloc:
                    continue

                doc_right = self.check_docrights(gdocid=int(gdocid))

                if self._job.userid not in self.bfaserver.user[self._job.profile] or not doc_right.modify:
                    msg += _("Keine Berechtigung auf das Dokument [%s]\n") % gdocid
                    _ret = _ret and False
                    continue

                if doc.extloc:
                    # sanity check
                    if doc.extloc.lower().find(con.archive_rs.lower()) > -1:     # wenn das passiert, dann ist vorher schon gewaltig was schiefgelaufen
                        msg += _("Dokument [%s] liegt revisionssicher." % gdocid)
                        continue
                    else:
                        try:
                            if os.path.expandvars(doc.extloc.lower()) not in [os.path.expandvars(doc.docloc.lower()), os.path.expandvars(doc.oriloc.lower()), os.path.expandvars(doc.srcloc.lower())]:
                                if os.path.exists(os.path.expandvars(doc.extloc)):
                                    os.remove(os.path.expandvars(doc.extloc))
                        except:
                            self.log.exception("%s Fehler beim Löschen der Datei (Voransichtskopie) %s" % (self.name, doc.extloc))
                            msg += _("Fehler beim Löschen der Datei (Voransichtskopie) [%s]\n" % gdocid)
                            _ret = _ret and False
                        else:
                            doc.extloc = ""

                            sql = 'update sys_doc set extloc = NULL where gdocid = %s' % gdocid
                            try:
                                self.db[self._job.profile].execute(sql)
                            except:
                                self.log.exception(_("%s Fehler beim Entfernen des Extloc-Pfades" % self.name))
                                msg += _("Fehler beim Entfernen des Extloc-Pfades [%s]\n" % gdocid)
                                _ret = _ret and False
                            else:
                                hist = bitfarm_doc.BitfarmHistory()
                                hist.gdocid = int(gdocid)
                                hist.userid = self._job.userid
                                hist.taskid = 232
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

        if not _ret or msg != "":
            xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, data=msg)
        else:
            xml = bitfarm_xml.create_xml_request(reqtype="respOK", userid=0, data=_("reqresetPreview ausgeführt"))

        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def write_reocr_file(self, doc, username, svninfo, gdocid):
        self.log.debug('%s write_reocr_file' % self.name)

        _ret = False
        xml = ""
        con = self.bfaserver.con.connections[self._job.profile]

        fname = datetime.datetime.now().strftime('%d%m%Y%H%M%S%f') + self.name[7:] + '.kom'
        while os.path.exists(os.path.join(con.uncclientspooler, fname)):
            fname = datetime.datetime.now().strftime('%d%m%Y%H%M%S%f') + self.name[7:] + '.kom'

        komfile = []
        try:
            komfile.append("[Version]")
            komfile.append("Version=3.0")
            komfile.append("[Command]")
            komfile.append("Command=ReOCRFile")
            komfile.append("doc_loc=%s" % os.path.expandvars(doc.docloc))
            komfile.append("ext_loc=%s" % os.path.expandvars(doc.extloc))
            komfile.append("ori_loc=%s" % os.path.expandvars(doc.oriloc))
            komfile.append("Profil=%s" % doc.profile)
            komfile.append("Benutzer=%s" % username)
            komfile.append("SvnInfo=%s" % svninfo)
            komfile.append("gdocid=%s" % gdocid)

        except:
            self.log.exception(_("%s Fehler beim Erzeugen der komfile-Datei %s.") %(self.name, fname))
            xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, data="Fehler beim Erzeugen der komfile-Datei.")
        else:
            try:
                komfile = "\n".join(komfile)
                # komfile = komfile.encode('cp1252')

                with open(os.path.join(con.uncclientspooler, fname), 'w', encoding='cp1252') as f:
                    f.write(komfile)
            except:
                self.log.exception(_("%s Fehler beim Schreiben der komfile-Datei %s.") % (self.name, fname))
                xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=int(gdocid))
            else:
                _ret = True
                xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("ReOCR durchgeführt."))

        return _ret, xml

    def reqReOCR(self):
        self.log.debug('%s reqReOCR' % self.name)

        con = self.bfaserver.con.connections[self._job.profile]
        params = self._job.parameter.split(";")
        svninfo = params.pop(0)
        username = params.pop(0)
        xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, data="Fehler bei Dokument aktualisieren.")

        for gdocid in params:
            doc = bitfarm_doc.BitfarmDocument()
            doc.profile = self._job.profile
            if not self.bfaProcessing.load(doc, int(gdocid), 'paths', logid=self.name):
                self.log.exception(_("%s Dokument [%s] konnte nicht geladen werden") % (self.name, gdocid))
                xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, data="Dokument [%s] konnte nicht geladen werden." % gdocid)
            else:
                # fname = datetime.datetime.now().strftime('%d%m%Y%H%M%S%f') + '.kom'
                # while os.path.exists(os.path.join(con.uncclientspooler, fname)):
                #     fname = datetime.datetime.now().strftime('%d%m%Y%H%M%S%f') + '.kom'
                #
                # komfile = []
                # try:
                #     komfile.append("[Version]")
                #     komfile.append("Version=3.0")
                #     komfile.append("[Command]")
                #     komfile.append("Command=ReOCRFile")
                #     komfile.append("doc_loc=%s" % os.path.expandvars(doc.docloc))
                #     komfile.append("ext_loc=%s" % os.path.expandvars(doc.extloc))
                #     komfile.append("ori_loc=%s" % os.path.expandvars(doc.oriloc))
                #     komfile.append("Profil=%s" % doc.profile)
                #     komfile.append("Benutzer=%s" % username)
                #     komfile.append("SvnInfo=%s" % svninfo)
                #     komfile.append("gdocid=%s" % gdocid)
                #
                # except:
                #     self.log.exception(_(u"%s Fehler beim Erzeugen der komfile-Datei %s.") %(self.name, fname))
                #     xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, data=u"Fehler beim Erzeugen der komfile-Datei.")
                # else:
                #     try:
                #         komfile = u"\n".join(komfile)
                #         komfile = komfile.encode('cp1252')
                #
                #         with open(os.path.join(con.uncclientspooler, fname), 'w') as f:
                #             f.write(komfile)
                #     except:
                #         self.log.exception(_(u"%s Fehler beim Schreiben der komfile-Datei %s.") % (self.name, fname))
                #         xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=int(gdocid))
                #     else:
                #         xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_(u"ReOCR durchgeführt."))
                #         hist = bitfarm_doc.BitfarmHistory()
                #         hist.gdocid = gdocid
                #         hist.taskid = 229
                #         hist.userid = self._job.userid
                #         self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)
                #         self.check_abo(gdocid)

                success, xml = self.write_reocr_file(doc, username, svninfo, gdocid)
                if success:
                    hist = bitfarm_doc.BitfarmHistory()
                    hist.gdocid = gdocid
                    hist.taskid = 229
                    hist.userid = self._job.userid
                    self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)
                    self.check_abo(gdocid)
        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqCreateNewLink(self):
        self.log.debug('%s reqCreateNewLink' % self.name)

        refname, gdocids = self._job.parameter.split(";")
        gdocids = gdocids.split(",")
        refname = self.bfaProcessing.parse_sqlstring(refname)
        sql = "select gdocid from sys_doclink where refname = '%s'" % refname

        try:
            res = self.db[self._job.profile].execute(sql).fetchall()
        except:
            self.log.exception(_("%s Fehler beim Erstellen der Verknüpfung") % self.name)
            xml = bitfarm_xml.create_xml_request(reqtype="respError", data=_("Fehler beim Erstellen der Verknüpfung"))
        else:
            current = []
            for row in res:
                current.append(row[0])

            gdocids = [x for x in gdocids if int(x) not in current]
            if gdocids:
                vals = ["(%s,'%s')" % (x, refname) for x in gdocids]
                vals = ",".join(vals)
                sql = "insert into sys_doclink (gdocid, refname) values %s on duplicate key update refname = values(refname)" % vals

                try:
                    self.db[self._job.profile].execute(sql)
                except:
                    self.log.exception(_("%s Fehler beim Erstellen der Verknüpfung") % self.name)
                    xml = bitfarm_xml.create_xml_request(reqtype="respError", data=_("Fehler beim Erstellen der Verknüpfung"))
                else:
                    xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("Verknüpfung erzeugt."))
                    hist = bitfarm_doc.BitfarmHistory()
                    hist.taskid = 213
                    hist.userid = self._job.userid
                    hist.value1 = refname
                    self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name, gdocid=gdocids)
            else:
                xml = bitfarm_xml.create_xml_request(reqtype="respError", data=_("Dokument(e) schon in Verknüpfung"))
        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqRemoveLink(self):
        self.log.debug('%s reqRemoveLink' % self.name)

        refname, gdocids = self._job.parameter.split(";")

        sql = "delete from sys_doclink where refname = '%s' and gdocid in (%s)" % (refname, gdocids)

        try:
            self.db[self._job.profile].execute(sql)
        except:
            self.log.exception(_("%s Fehler beim Entfernen eines Dokuments aus einer Verknüpfung") % self.name)
            xml = bitfarm_xml.create_xml_request(reqtype="respError",
                                                 data=_("Fehler beim Entfernen eines Dokuments aus einer Verknüpfung"))
        else:
            xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("Dokument aus Verknüpfung entfernt."))
            hist = bitfarm_doc.BitfarmHistory()
            hist.taskid = 214
            hist.userid = self._job.userid
            hist.value1 = refname
            self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name, gdocid=gdocids)

        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqDeleteLink(self):
        self.log.debug('%s reqDeleteLink' % self.name)

        # self._job.parameter = self._job.parameter.split(";")
        # refname = self._job.parameter[0]
        refname = self._job.parameter
        gdocids = []
        sql = "select gdocid from sys_doclink where refname = '%s' group by gdocid" % refname

        try:
            result = self.db[self._job.profile].execute(sql).fetchall()
        except:
            self.log.exception(_("%s Fehler beim Lesen einer Verknüpfung") % self.name)
        else:
            if result:
                for row in result:
                    gdocids.append(row[0])

        sql = "delete from sys_doclink where refname = '%s' " % refname
        try:
            self.db[self._job.profile].execute(sql)
        except:
            self.log.exception(_("%s Fehler beim Löschen einer Verknüpfung") % self.name)
            xml = bitfarm_xml.create_xml_request(reqtype="respError", data=_("Fehler beim Löschen einer Verknüpfung"))
        else:
            xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("Verknüpfung gelöscht."))
            if len(gdocids) == 0:
                gdocids = [0]
            hist = bitfarm_doc.BitfarmHistory()
            hist.taskid = 212
            hist.userid = self._job.userid
            hist.value1 = refname
            self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name, gdocid=gdocids)

        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def save_objects(self, arcid, gdocid, error, msg):
        self.log.debug('%s save_objects' % self.name)

        if self._job.data is not None:
            if type(self._job.data) is not list:
                self._job.data = [self._job.data]
            for obj in self._job.data:
                try:
                    if isinstance(obj, bitfarm_doc.BitfarmAddfield):  # Zusatzfelder speichern
                        if obj.modified:
                            saveids = []

                            right = self.check_addfieldrights(obj.id, arcid=arcid, gdocid=gdocid)
                            if right.rights & 4 == 4:  # modify
                                saveids.append(gdocid)
                            if right.rights & 2 == 2:  # write, prüfen ob noch nicht beschrieben
                                sql = 'select %s from sys_docidx_%s where gdocid = %s' % (obj.add, obj.indextableid, gdocid)
                                try:
                                    result = self.db[self._job.profile].execute(sqla.text(sql)).fetchone()
                                except:
                                    self.log.exception(_("%s Fehler bei Abfrage des Zusatzfeldes.") %(self.name))
                                    error += _("Fehler bei Abfrage des Zusatzfeldes.")
                                else:
                                    if result is None or result[0] is None:
                                        saveids.append(gdocid)

                        if len(saveids) == 0:
                            error += _("Keine Berechtigung auf Zusatzfeld %s.") % obj.name
                        elif not self.bfaProcessing.save_addfield(obj, saveids, self._job.profile, self._job.userid):
                            error += _("Das Zusatzfeld %s konnte nicht gespeichert werden.") % obj.name
                        else:
                            if len(saveids) != 1:
                                msg += _("Es konnten nicht alle Zusatzfelder gespeichert werden.")
                            if obj.history:
                                self.bfaProcessing.write_history(history=obj.history, profile=self._job.profile, gdocid=saveids, logid=self.name)
                                obj.history = []
                            else:
                                hist = bitfarm_doc.BitfarmHistory()
                                hist.userid = self._job.userid
                                if obj.oldvalue is None:
                                    hist.taskid = 227
                                    hist.value1 = obj.name
                                    hist.value2 = obj.value
                                else:
                                    hist.taskid = 215
                                    hist.value1 = obj.name
                                    hist.value2 = obj.oldvalue
                                    hist.value3 = obj.value
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile,
                                                                         gdocid=saveids, logid=self.name)
                except:
                    self.log.exception(_("%s Es hat einen Fehler beim Speichern der Zusatzfelder gegeben.\n") %(self.name))
                    error += _("Es hat einen Fehler beim Speichern der Zusatzfelder gegeben.\n")

        return error, msg

    def write_movefile(self, gdocid):
        self.log.debug('%s write_movefile' % self.name)

        doc = bitfarm_doc.BitfarmDocument()
        doc.profile = self._job.profile

        con = self.bfaserver.con.connections[self._job.profile]

        if not self.bfaProcessing.load(doc, gdocid, ['base', 'paths', 'fields', 'svn'], logid=self.name):
            self.log.error("Das Dokument %s konnte nicht geladen werden." % gdocid)
            return False

        try:
            movejob = bitfarm_configobj.ConfigObj(encoding='cp1252')
            movejob['Version'] = {}
            movejob['Version']['Version'] = '3.0'
            movejob['Archiv'] = {}
            movejob['Archiv']['Profil'] = self._job.profile
            try:
                movejob['Archiv']['Name'] = self.bfaserver.archives[self._job.profile][doc.arcid].arcname
            except:
                self.log.warning("Arcid nicht in der Archivliste vorhanden.")
            movejob['Archiv']['Tabelle'] = doc.arcid
            movejob['Document'] = {}
            movejob['Document']['DOCID'] = '%s.%s' %(doc.arcid,  gdocid)
            movejob['Document']['gdoc_id'] = str(gdocid)
            movejob['Document']['Orginal'] = doc.docloc
            movejob['Document']['Kopie'] = doc.extloc
            movejob['Document']['Quelle'] = doc.docloc

            movejob['SVN'] = {}
            movejob['SVN']['SVN_Link'] = doc.svn_history[-1].svn_link if len(doc.svn_history) > 0 else ""
            movejob['SVN']['SVN_Version'] = doc.svn_history[-1].svn_version if len(doc.svn_history) > 0 else ""
            movejob['SVN']['SVN_Revision'] = doc.svn_history[-1].svn_revision if len(doc.svn_history) > 0 else ""

            ind = 0
            movejob['Zusatzfelder'] = {}
            movejob['Zusatzfelder']['Felder'] = len(doc.fields)
            if len(doc.fields) > 0:
                for obj in list(doc.fields.values()):
                    if isinstance(obj, bitfarm_doc.BitfarmAddfield):  # Zusatzfelder speichern
                        ind += 1
                        movejob['Zusatzfelder']['Felder'] = ind
                        movejob['Zusatzfelder']['ZusTitel' + str(ind)] = obj.name
                        movejob['Zusatzfelder']['ZusFeld' + str(ind)] = obj.add
                        if obj.value is not None:
                            movejob['Zusatzfelder']['ZusWert' + str(ind)] = obj.value
                        else:
                            movejob['Zusatzfelder']['ZusWert' + str(ind)] = ""

            movefilename = os.path.basename(doc.docloc)
            movefilename = os.path.splitext(movefilename)[0]
            suffix = str(datetime.datetime.now().microsecond/1000)
            while os.path.exists(os.path.join(con.uncclientspooler, movefilename + suffix + '.move')):
                suffix = str(datetime.datetime.now().microsecond/1000)

            movejob.filename = os.path.join(con.uncclientspooler, movefilename + suffix + '.move')
            movejob.write_empty_values = True
            movejob.write()
            movejob = None
            return True
        except:     # wegen einem Fehler hier lassen wir nicht das Verschieben ungültig sein
            self.log.exception('Fehler beim Schreiben der Move-Datei [%s]' % gdocid)
            return False

    def reqMoveDoc(self):
        self.log.debug('%s reqMoveDoc' % self.name)
        con = self.bfaserver.con.connections[self._job.profile]
        moves = self._job.parameter.split(';')
        msg = ""
        error = ""

        if self._job.data is not None:
            if type(self._job.data) is not list:
                self._job.data = [self._job.data]

        for move in moves:
            set_defdocname = False
            get_docname = False
            gdocid, src_arcid, dest_arcid = move.split(',')

            rights = con.read_rights(db=self.db[self._job.profile], arcid=int(src_arcid), user=self._job.userid)

            if not rights[0].move:  # kein Verschieben Recht im Quellarchiv
                error = _("Sie haben kein Verschiebenrecht im Quellarchiv")
            else:
                rights = con.read_rights(db=self.db[self._job.profile], arcid=int(dest_arcid), user=self._job.userid)
                if not rights[0].docimport:
                    error = _("Sie haben kein Importrecht im Ziellarchiv")

            if error:
                xml = bitfarm_xml.create_xml_request(reqtype="respError", data=error, datatype="text")
                try:
                    self._job.sock.send(xml)
                except:
                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None

            sql = "select gdocid, arcid from sys_symlink where gdocid = %s and arcid = %s" % (gdocid, dest_arcid)

            try:
                res = self.db[self._job.profile].execute(sql).fetchall()
            except:
                self.log.exception(_("%s Fehler beim Prüfen der Symbolic Links [%s]") % (self.name, gdocid))
                error += _("Fehler beim Prüfen der Symbolic Links [%s]\n") % gdocid
            else:
                if len(res) == 0:
                    if int(dest_arcid) in self.bfaserver.archives[self._job.profile]:
                        if self.bfaserver.archives[self._job.profile][int(dest_arcid)].defdocname:
                            set_defdocname = self.bfaserver.archives[self._job.profile][int(dest_arcid)].defdocname
                            doc = bitfarm_doc.BitfarmDocument()
                            doc.profile = self._job.profile
                            self.bfaProcessing.load(doc, gdocid, 'base')
                            get_docname = doc.title
                            del doc
                            sql = "update sys_doc set arcid = %s, docname = '%s' where gdocid = %s and arcid = %s" % (dest_arcid, set_defdocname, gdocid, src_arcid)
                        else:
                            sql = "update sys_doc set arcid = %s where gdocid = %s and arcid = %s" % (dest_arcid, gdocid, src_arcid)

                        try:
                            self.db[self._job.profile].execute(sql)
                        except:
                            self.log.exception(_("%s Fehler beim  Verschieben des Dokuments [%s]\n") % (self.name, gdocid))
                            error += _("Fehler beim  Verschieben des Dokuments [%s]\n") % gdocid
                            continue
                        else:
                            hist = bitfarm_doc.BitfarmHistory()
                            hist.gdocid = gdocid
                            hist.taskid = 201
                            hist.userid = self._job.userid
                            hist.value1 = self.bfaserver.archives[self._job.profile][int(src_arcid)].arcname
                            hist.value2 = self.bfaserver.archives[self._job.profile][int(dest_arcid)].arcname
                            self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            if set_defdocname:
                                hist = bitfarm_doc.BitfarmHistory()
                                hist.gdocid = gdocid
                                hist.taskid = 217
                                hist.userid = self._job.userid
                                hist.value1 = get_docname
                                hist.value2 = set_defdocname
                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            success, expired = self.bfaProcessing.recalculate_expired_date(gdocid, self._job.profile)
                            hist = bitfarm_doc.BitfarmHistory()
                            hist.userid = self._job.userid

                            if success:
                                hist.taskid = 115
                                hist.value1 = expired.strftime('%d.%m.%Y') if expired is not None else 'leer'
                            # else:
                            #     if expired is not None:
                            #         hist.taskid = 117
                            #         hist.value1 = expired.strftime('%d.%m.%Y') if expired is not None else 'leer'
                            #     else:
                            #         hist.taskid = 116

                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile,
                                                                 gdocid=gdocid, logid=self.name)
                    else:
                        error += _("Zielarchiv [%s] nicht vorhanden. Dokument [%s] konnte nicht verschoben werden\n") % (dest_arcid, gdocid)
                        continue
                else:
                    error += _("Das Dokument ist als Symbolic Link in diesem Archiv schon vorhanden. [%s]\n") % gdocid
                    continue

            # den haupteintrag des dokumentes in der sys_symlink aktualisieren (falls vorhanden)
            sql = "select gdocid, arcid from sys_symlink where gdocid = %s and arcid = %s" % (gdocid, src_arcid)

            try:
                res = self.db[self._job.profile].execute(sql).fetchall()
            except:
                self.log.exception(_("%s Fehler beim Prüfen der Symbolic Links [%s]") % (self.name, gdocid))
                error += _("Fehler beim Prüfen der Symbolic Links [%s]\n") % gdocid
                continue
            else:
                if len(res) == 1:
                    sql = "update sys_symlink set arcid = %s where gdocid = %s and arcid = %s" % (dest_arcid, gdocid, src_arcid)

                    try:
                        self.db[self._job.profile].execute(sql)
                    except:
                        self.log.exception(_("%s Fehler beim  Verschieben des Dokuments(Symlink) [%s]") % (self.name, gdocid))
                        error += _("Fehler beim  Verschieben des Dokuments(Symlink) [%s]\n") % gdocid
                        continue
                    else:
                        msg += _("Dokument verschoben. [%s]\n") % gdocid

                        # else:
                        #     msg += _(u"Das Dokument ist als Symbolic Link in diesem Archiv schon vorhanden [%s]\n") % gdocid
                        #     xml = bitfarm_xml.create_xml_request(reqtype="respError", data=msg)

            error, msg = self.save_objects(dest_arcid, gdocid, error, msg)

            if not self.write_movefile(gdocid):
                self.log.error("Es konnte kein Move-File geschrieben werden.")

            if not self.bfaProcessing.refresh_fieldtxt(gdocid=int(gdocid), profile=self._job.profile):
                self.log.error(_("%s fieldtxt konnte nicht aktualisiert werden [%i]") % (self.name, int(gdocid)))

        if error:
            xml = bitfarm_xml.create_xml_request(reqtype="respError", data=error, datatype="text")
        else:
            xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("Dokument verschoben."))
        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqSaveDocName(self):
        self.log.debug('%s reqSaveDocName' % self.name)

        con = self.bfaserver.con.connections[self._job.profile]

        msg = ""
        docname = self._job.data
        oldname = docname
        gdocids = self._job.parameter.split(';')
        gdocids = [int(x) for x in gdocids]
        if len(gdocids) == 0:
            msg += _("%s Keine Gdocid(s) übergeben\n") % (self.name)
            self.log.error(msg)
        elif not docname:
            msg += _("%s Kein Dokumententitel übergeben %s\n") % (self.name, str(gdocids))
            self.log.error(msg)
        else:
            # newdocname = re.sub('%{1}', repl_percent, docname)
            for gdocid in gdocids:
                sql = "select docname from sys_doc where gdocid = %i" % gdocid

                try:
                    res = self.db[self._job.profile].execute(sql).fetchone()
                except:
                    msg += _("%s Fehler beim Abfragen des alten Dokumententitels [%s]\n") % (self.name, gdocid)
                    self.log.exception(msg)
                else:
                    oldname = res[0]

                if docname != oldname:
                    # vals = {con.sys_doc.c.docname: docname,
                    #         con.sys_doc.c.modified_userid: self._job.userid,
                    #         con.sys_doc.c.modified: None}
                    vals = {con.sys_doc.c.docname: docname}
                    sql = con.sys_doc.update(whereclause=(con.sys_doc.c.gdocid == gdocid), values=vals)

                    try:
                        self.db[self._job.profile].execute(sql)
                    except:
                        msg += _("%s Fehler beim Setzen des Dokumententitels [%s]\n") % (self.name, gdocid)
                        self.log.exception(msg)
                    else:
                        hist = bitfarm_doc.BitfarmHistory()
                        hist.gdocid = gdocid
                        hist.taskid = 217
                        hist.userid = self._job.userid
                        hist.value1 = oldname
                        hist.value2 = docname
                        self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                    # zu den Schlagworten hinzufügen
                    sql = "select doctags from sys_doctxt where gdocid = %i" % gdocid
                    sql = con.sys_doctxt.select().where(con.sys_doctxt.c.gdocid == gdocid)
                    try:
                        result = self.db[self._job.profile].execute(sql).fetchone()
                    except:
                        msg += _("%s Fehler beim Abfragen der Schlagworte [%s]\n") % (self.name, gdocid)
                        self.log.exception(msg)
                    else:
                        slw = result['doctags'] if result and result['doctags'] else ""
                            # slw = re.sub('%{1}', repl_percent, slw)
                        if slw:
                            slw += '\r\n%s' % docname
                        else:
                            slw = docname

                        # sql = u"update sys_doctxt set doctags = '%s' where gdocid = %i" % (slw, gdocid)
                        sql = con.sys_doctxt.update().values({'doctags': slw}).where(con.sys_doctxt.c.gdocid == gdocid)
                        try:
                            self.db[self._job.profile].execute(sql)
                        except:
                            msg += _("%s Fehler beim Hinzufügen des Dokumententitels zu den Schlagworten [%s]\n") % (self.name, gdocid)
                            self.log.exception(msg)

        if msg:
            xml = bitfarm_xml.create_xml_request(reqtype="respError", data=msg)
        else:
            xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=_("reqSaveDocName ausgeführt"))

        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqMoveSymlink(self):
        self.log.debug('%s reqMoveSymlink' % self.name)
        moves = self._job.parameter.split(';')
        error = ""
        msg = ""
        for move in moves:
            gdocid, src_arcid, dest_arcid = move.split(',')

            sql = "select gdocid, arcid from sys_symlink where gdocid = %s and arcid = %s" % (gdocid, dest_arcid)

            try:
                res = self.db[self._job.profile].execute(sql).fetchall()
            except:
                self.log.exception(_("%s Fehler beim Prüfen der Symbolic Links [%s]") %(self.name, gdocid))
                msg += _("Fehler beim Prüfen der Symbolic Links [%s]\n") % gdocid
                xml = bitfarm_xml.create_xml_request(reqtype="respError", data=msg)
            else:
                if len(res) == 0:
                    sql = "update sys_symlink set arcid = %s where gdocid = %s and arcid = %s" % (dest_arcid, gdocid, src_arcid)

                    try:
                        self.db[self._job.profile].execute(sql)
                    except:
                        self.log.exception(_("%s Fehler beim  Verschieben des Symbolic Link [%s]") %(self.name, gdocid))
                        msg += _("Fehler beim  Verschieben des Symbolic Link [%s]\n") % gdocid
                        xml = bitfarm_xml.create_xml_request(reqtype="respError", data=msg)
                    else:
                        msg += _("Symbolic Link verschoben [%s]\n") % gdocid
                        xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=msg)
                        hist = bitfarm_doc.BitfarmHistory()
                        hist.gdocid = gdocid
                        hist.taskid = 221
                        hist.userid = self._job.userid
                        hist.value1 = self.bfaserver.archives[self._job.profile][int(src_arcid)].arcname
                        hist.value2 = self.bfaserver.archives[self._job.profile][int(dest_arcid)].arcname
                        self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                        error, msg = self.save_objects(dest_arcid, gdocid, error, msg)

                else:
                    msg += _("Symbolic Link in diesem Archiv schon vorhanden [%s]\n") % gdocid
                    xml = bitfarm_xml.create_xml_request(reqtype="respError", data=msg)

        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqCreateEmptyDoc(self):
        self.log.debug('%s reqCreateEmptyDoc' % self.name)
        created = False
        error = False
        gdocids = []
        addfields_saved = False
        annos_saved = False
        msg = ''
        arcid, doctype = self._job.parameter.split(';')
        arcid = int(arcid)
        right = self.check_arcright(arcid=arcid)
        if len(right) == 0 or not right[0].docimport:
            xml = bitfarm_xml.create_xml_request(reqtype="respError", data=_("Keine Berechtigung, in diesem Archiv ein leeres Dokument zu erzeugen."))
        elif arcid not in self.bfaserver.archives[self._job.profile]:
            xml = bitfarm_xml.create_xml_request(reqtype="respError", data=_("Archiv-ID nicht vorhanden."))
        elif self.bfaserver.archives[self._job.profile][arcid].type == 0:
            xml = bitfarm_xml.create_xml_request(reqtype="respError", data=_("Dokumente können nicht in Lager erzeugt werden."))
        else:
            doc = bitfarm_doc.BitfarmDocument()
            doc.profile = self._job.profile
            doc.arcid = arcid
            if arcid in self.bfaserver.archives[self._job.profile]:
                doc.storageid = self.bfaserver.archives[self._job.profile][arcid].storageid
            doc.userid = self._job.userid
            #doc.title = _(u"leeres %s Dokument") % doctype
            doc.title = _("%s" % doctype)

            if not self.bfaProcessing.create_tempname(doc):
                xml = bitfarm_xml.create_xml_request(reqtype="respError", data=_("Es konnte kein Pfad für das leere Dokument erzeugt werden."))
            else:
                if doctype.lower() == 'text':
                    try:
                        with open(os.path.expandvars(doc.tempname + '.txt'), 'w') as f:
                            f.write('\n')
                    except:
                        self.log.exception(_("%s Die Textdatei konnte nicht erzeugt werden.") % self.name)
                        xml = bitfarm_xml.create_xml_request(reqtype="respError",
                                                             data=_("Die Textdatei konnte nicht erzeugt werden."))
                    else:
                        created = True
                        fext = '.txt'
                else:
                    if doctype.lower() in self.bfaserver.con.connections[self._job.profile].emptydocs:
                        fsrc = os.path.expandvars(self.bfaserver.con.connections[self._job.profile].emptydocs[doctype.lower()])
                        fext = os.path.splitext(fsrc)[1]
                        try:
                            shutil.copyfile(fsrc, os.path.expandvars(doc.tempname + fext))
                        except:
                            self.log.exception(_("%s Die Datei konnte nicht kopiert werden.") % self.name)
                            xml = bitfarm_xml.create_xml_request(reqtype="respError",
                                                                 data=_("Die Datei konnte nicht kopiert werden."))
                        else:
                            created = True
                    else:
                        xml = bitfarm_xml.create_xml_request(reqtype="respError",
                                                             data=_("Die Vorlage %s konnte nicht gefunden werden.") % doctype)
                        self.log.error("Angeforderte Vorlage %s nicht in Liste der Vorlagen %s" % (doctype, ";".join(list(self.bfaserver.con.connections[self._job.profile].emptydocs.keys()))))
                if created:
                    fsrc = os.path.expandvars(r'%bitfarm-archiv%\empty.tif')
                    try:
                        shutil.copyfile(fsrc, os.path.expandvars(doc.tempname + '.tif'))
                    except:
                        self.log.exception(_("%s Die leere Voransicht konnte nicht kopiert werden." % self.name))
                        xml = bitfarm_xml.create_xml_request(reqtype="respError", data=_("Die leere Voransicht konnte nicht kopiert werden."))
                        created = False
                    else:
                        doc.docloc = doc.tempname + '.tif'
                        doc.oriloc = doc.tempname + fext
                        doc.filetype = fext
                        doc.modified_userid = self._job.userid
                        if not self.bfaProcessing.create(doc, arcid):
                            xml = bitfarm_xml.create_xml_request(reqtype="respError", data=_("Es konnte kein leeres Dokument erzeugt werden."))
                        elif not self.bfaProcessing.write(doc, logid=self.name):
                            xml = bitfarm_xml.create_xml_request(reqtype="respError",
                                                                 data=_("Das Dokument konnte nicht aktualisiert werden."))
                        else:
                            hist = bitfarm_doc.BitfarmHistory()
                            hist.gdocid = doc.gdocid
                            hist.userid = self._job.userid
                            hist.taskid = 222
                            hist.value1 = doctype
                            hist.value2 = self.bfaserver.archives[self._job.profile][arcid].arcname
                            self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                            success, expired = self.bfaProcessing.set_standard_expired_date(arcid, doc.gdocid, self._job.profile)
                            hist = bitfarm_doc.BitfarmHistory()
                            hist.userid = self._job.userid

                            if success:
                                hist.taskid = 115
                                hist.value1 = expired.strftime('%d.%m.%Y') if expired is not None else 'leer'
                            # else:
                            #     if expired is not None:
                            #         hist.taskid = 117
                            #         hist.value1 = expired.strftime('%d.%m.%Y') if expired is not None else 'leer'
                            #     else:
                            #         hist.taskid = 116

                                self.bfaProcessing.write_history(history=hist, profile=self._job.profile,
                                                                     gdocid=doc.gdocid, logid=self.name)

                            if self._job.data is not None and len(self._job.data) > 0:
                                self._job.gdocid = doc.gdocid
                                gdocids, error, msg, addfields_saved, annos_saved = self.do_save()

                            if error:
                                xml = bitfarm_xml.create_xml_request(reqtype="respError", data=error, datatype="text")
                                self.log.error("%s %s" % (self.name, error))
                            else:
                                xml = bitfarm_xml.create_xml_request(reqtype="respOK", gdocid=doc.gdocid,
                                                                 data=_("Leeres %s Dokument angelegt.") % doctype)
        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

        if gdocids:
            self.check_abo(gdocids)

        if addfields_saved:  # fieldtxt in sys_doctxt aktualisieren
            for gdocid in gdocids:
                if not self.bfaProcessing.refresh_fieldtxt(gdocid=gdocid, profile=self._job.profile):
                    self.log.error(_("%s fieldtxt konnte nicht aktualisiert werden [%i]") % (self.name, gdocid))

    # def duplicate_files(self, _con, _doc, _error, _right_src, _gdocid_src):
    #     self.log.debug('%s duplicate_files' % self.name)
    #
    #     result = True
    #     revision_safe = False
    #     nonrevision_safe = False
    #     newpath = ""
    #     newpath_nrs = ""
    #     fpath = ""
    #     if _doc.docloc:
    #         if _doc.docloc.lower().find(_con.archive_nrs.lower()) > -1:
    #             nonrevision_safe = nonrevision_safe or True
    #         else:
    #             revision_safe = revision_safe or True
    #     if _doc.extloc:
    #         if _doc.extloc.lower().find(_con.archive_nrs.lower()) > -1:
    #             nonrevision_safe = nonrevision_safe or True
    #         else:
    #             revision_safe = revision_safe or True
    #     if _doc.oriloc and (_right_src[0].write or _right_src[0].modify):
    #         if _doc.oriloc.lower().find(_con.archive_nrs.lower()) > -1:
    #             nonrevision_safe = nonrevision_safe or True
    #         else:
    #             revision_safe = revision_safe or True
    #
    #     self.bfaProcessing.create_basename(_doc)
    #     if revision_safe:
    #         newpath = self.bfaProcessing.create_arcpath(_doc, is_revisionsave=True)
    #     if nonrevision_safe:
    #         newpath_nrs = self.bfaProcessing.create_arcpath(_doc, is_revisionsave=False)
    #
    #     if _doc.docloc:
    #         fext = os.path.splitext(_doc.docloc)[1]
    #         if _doc.docloc.lower().find(_con.archive_nrs.lower()) > -1:
    #             fpath = os.path.join(newpath_nrs, _doc.basename + fext)
    #         else:
    #             fpath = os.path.join(newpath, _doc.basename + fext)
    #         try:
    #             shutil.copyfile(os.path.expandvars(_doc.docloc), os.path.expandvars(fpath))
    #         except:
    #             self.log.exception("Die Datei konnte nicht kopiert werden %s nach %s" % (_doc.docloc, fpath))
    #             _error += "Die Voransicht des Dokuments [%s] konnte nicht kopiert werden\n" % _gdocid_src
    #             result = False
    #         else:
    #             _doc.docloc = fpath
    #
    #     if _doc.extloc:
    #         fext = os.path.splitext(_doc.extloc)[1]
    #         if _doc.extloc.lower().find(_con.archive_nrs.lower()) > -1:
    #             fpath = os.path.join(newpath_nrs, _doc.basename + fext)
    #         else:
    #             fpath = os.path.join(newpath, _doc.basename + fext)
    #         try:
    #             shutil.copyfile(os.path.expandvars(_doc.extloc), os.path.expandvars(fpath))
    #         except:
    #             self.log.exception("Die Datei konnte nicht kopiert werden %s nach %s" % (_doc.extloc, fpath))
    #             _error += "Die Voransichtskopie des Dokuments [%s] konnte nicht kopiert werden\n" % _gdocid_src
    #             result = False
    #         else:
    #             _doc.extloc = fpath
    #
    #     if _doc.oriloc and (_right_src[0].write or _right_src[0].modify):
    #         fext = os.path.splitext(_doc.oriloc)[1]
    #         if _doc.oriloc.lower().find(_con.archive_nrs.lower()) > -1:
    #             fpath = os.path.join(newpath_nrs, _doc.basename + fext)
    #         else:
    #             fpath = os.path.join(newpath, _doc.basename + fext)
    #         try:
    #             shutil.copyfile(os.path.expandvars(_doc.oriloc), os.path.expandvars(fpath))
    #         except:
    #             self.log.exception("Die Datei konnte nicht kopiert werden %s nach %s" % (_doc.oriloc, fpath))
    #             _error += "Die Originaldatei des Dokuments [%s] konnte nicht kopiert werden\n" % _gdocid_src
    #             result = False
    #         else:
    #             _doc.oriloc = fpath
    #     else:
    #         _doc.oriloc = None
    #
    #     return result, _error

    def reqDuplicateDoc(self):
        self.log.debug('%s reqDuplicateDoc' % self.name)
        con = self.bfaserver.con.connections[self._job.profile]
        error = ""
        moves = self._job.parameter.split(';')
        with_annos = False
        if 'public' in moves:
            with_annos = True
            moves.remove('public')
        newids = []

        for move in moves:
            gdocid, src_arcid, dest_arcid = move.split(',')

            doc = bitfarm_doc.BitfarmDocument()
            doc.profile = self._job.profile
            if not self.bfaProcessing.load(doc, gdocid, ['base', 'paths', 'text', 'fields', 'iev', 'links']):
                error += "Das Dokument [%s] konnte nicht geladen werden\n" % gdocid
                continue

            right_src = self.check_arcright(arcid=src_arcid)
            if src_arcid != dest_arcid:
                if not right_src[0].move:  # kein Verschieben Recht im Quellarchiv
                    error += _("Sie haben kein Verschiebenrecht im Quellarchiv\n")
                    continue

            right_dest = self.check_arcright(arcid=dest_arcid)
            if not right_dest[0].docimport:
                error += _("Sie haben kein Importrecht im Zielarchiv\n")
                continue

            newdoc = bitfarm_doc.BitfarmDocument()
            newdoc.profile = doc.profile
            if not self.bfaProcessing.create(newdoc, dest_arcid):
                error += _("Es konnte kein neues Dokument erzeugt werden\n")
            else:
                del doc.gdocid

                doc.gdocid = newdoc.gdocid
                doc.arcid = newdoc.arcid
                if doc.fields:
                    for field in list(doc.fields.values()):
                        field.modified = True
                if doc.statefields:
                    for field in list(doc.statefields.values()):
                        field.modified = True
                if with_annos and (right_src[0].aread or right_src[0].awrite or right_src[0].amodify):
                    if doc.annotations:
                        for anno in doc.annotations:
                            anno.modified = True

                if doc.fulltxt:
                    doc.ftx.text = doc.fulltxt

                success, error = self.bfaProcessing.duplicate_files(con, doc, error, right_src, gdocid)

                if not success:
                    error += "Das duplizierte Dokument [%s] konnte nicht gespeichert werden\n" % doc.gdocid
                elif not self.bfaProcessing.write(doc):
                    error += "Das duplizierte Dokument [%s] konnte nicht gespeichert werden\n" % doc.gdocid
                else:
                    newids.append(str(doc.gdocid))
                    hist = bitfarm_doc.BitfarmHistory()
                    hist.gdocid = doc.gdocid
                    hist.userid = self._job.userid
                    hist.taskid = 239
                    hist.value1 = gdocid
                    self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                    hist = bitfarm_doc.BitfarmHistory()
                    hist.gdocid = gdocid
                    hist.userid = self._job.userid
                    hist.taskid = 240
                    hist.value1 = doc.gdocid
                    self.bfaProcessing.write_history(history=hist, profile=self._job.profile, logid=self.name)

                error, msg = self.save_objects(dest_arcid, doc.gdocid, error, "")

        if error:
            xml = bitfarm_xml.create_xml_request(reqtype="respDuplicateError",  data=_("Fehler beim Duplizieren\n%s") % error)
        else:
            xml = bitfarm_xml.create_xml_request(reqtype="respOK",  data=";".join(newids))

        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqSendOfflineDB(self):
        self.log.debug('%s reqSendOfflineDB' % self.name)

        if self._job.userid not in self.bfaserver.user[self._job.profile]:
            xml = bitfarm_xml.create_xml_request(reqtype="respDenied", userid=0, gdocid=self._job.gdocid)
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

        if not os.path.exists(os.path.join(self.bfaserver.mypath, 'OfflineDB.zip')):
            msg = _("Die OfflineDB.zip konnte auf dem Server nicht gefunden werden.")
            xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=self._job.gdocid, data=msg)
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

        doc = bitfarm_doc.BitfarmDocument()
        doc.profile = self._job.profile

        self.send_file_to_client(doc, os.path.join(self.bfaserver.mypath, 'OfflineDB.zip'), dontcache=True)

    def reqSendDBSql(self):
        self.log.debug('%s reqSendDBSql' % self.name)

        if self._job.userid not in self.bfaserver.user[self._job.profile]:
            xml = bitfarm_xml.create_xml_request(reqtype="respDenied", userid=0, gdocid=self._job.gdocid)
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

        if not os.path.exists(os.path.join(self.bfaserver.mypath, 'SearchEngine_36.sql')) or not os.path.exists(os.path.join(self.bfaserver.mypath, 'create_sys_36.sql')):
            msg = _("Die Datenbank-Skripte konnten auf dem Server nicht gefunden werden.")
            xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=self._job.gdocid, data=msg)
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

        with zipfile.ZipFile(os.path.join(self.bfaserver.mypath, 'DBSql.zip'), 'w') as myzip:
            myzip.write(os.path.join(self.bfaserver.mypath, 'SearchEngine_36.sql'), 'SearchEngine_36.sql')
            myzip.write(os.path.join(self.bfaserver.mypath, 'create_sys_36.sql'), 'create_sys_36.sql')
            myzip.close()

        doc = bitfarm_doc.BitfarmDocument()
        doc.profile = self._job.profile

        self.send_file_to_client(doc, os.path.join(self.bfaserver.mypath, 'DBSql.zip'), dontcache=True)

    def reqSendViewer(self):
        self.log.debug('%s reqSendViewer' % self.name)

        if self._job.userid not in self.bfaserver.user[self._job.profile]:
            xml = bitfarm_xml.create_xml_request(reqtype="respDenied", userid=0, gdocid=self._job.gdocid)
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

        if not os.path.exists(os.path.join(self.bfaserver.bfapath, 'ViewerV3.exe')):
            msg = _("Der ViewerV3 konnte auf dem Server nicht gefunden werden.")
            xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=self._job.gdocid, data=msg)
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

        if not os.path.exists(os.path.join(self.bfaserver.bfapath, 'midas.dll')):
            msg = _("Die Midas.dll konnte auf dem Server nicht gefunden werden.")
            xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=self._job.gdocid, data=msg)
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

        with zipfile.ZipFile(os.path.join(self.bfaserver.mypath, 'ViewerV3.zip'), 'w') as myzip:
            myzip.write(os.path.join(self.bfaserver.bfapath, 'ViewerV3.exe'), 'ViewerV3.exe')
            myzip.write(os.path.join(self.bfaserver.bfapath, 'midas.dll'), 'midas.dll')
            myzip.write(os.path.join(self.bfaserver.bfapath, 'libeay32.dll'), 'libeay32.dll')
            myzip.write(os.path.join(self.bfaserver.bfapath, 'ssleay32.dll'), 'ssleay32.dll')
            myzip.write(os.path.join(self.bfaserver.bfapath, 'Viewer-Files','Skins','bitfarm-Archiv.skn'), 'Viewer-Files\\Skins\\bitfarm-Archiv.skn')
            myzip.close()

        doc = bitfarm_doc.BitfarmDocument()
        doc.profile = self._job.profile

        self.send_file_to_client(doc, os.path.join(self.bfaserver.mypath, 'ViewerV3.zip'), dontcache=True)

    def reqSendDocLauncher(self):
        self.log.debug('%s reqSendDocLauncher' % self.name)

        if self._job.userid not in self.bfaserver.user[self._job.profile]:
            xml = bitfarm_xml.create_xml_request(reqtype="respDenied", userid=0, gdocid=self._job.gdocid)
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

        if not os.path.exists(os.path.join(self.bfaserver.bfapath, 'bfaDocLauncher.zip')):
            # msg = _("Der bfaDocLauncher konnte auf dem Server nicht gefunden werden.")
            # xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=self._job.gdocid, data=msg)
            # try:
            #     self._job.sock.send(xml)
            # except:
            #     self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            # return None

            path = os.path.join(self.bfaserver.bfapath, 'bfaDocLauncher')
            try:
                zf = zipfile.ZipFile(path + '.zip', "w")

                for dirname, subdirs, files in os.walk(path):
                    for filename in files:
                        zf.write(os.path.join(dirname, filename),
                                 os.path.relpath(os.path.join(dirname, filename), os.path.join(path, '..')))
                zf.write(os.path.join(self.bfaserver.bfapath, self._job.profile + '.con'), 'bfaDoclauncher\\%s.con' % self._job.profile)
                zf.close()
            except:
                msg = _("Konnte bfaDocLauncher.zip nicht erzeugen")
                self.log.exception(msg)
                xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=self._job.gdocid, data=msg)
                try:
                    self._job.sock.send(xml)
                except:
                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None

        doc = bitfarm_doc.BitfarmDocument()
        doc.profile = self._job.profile

        self.send_file_to_client(doc, os.path.join(self.bfaserver.bfapath, 'bfaDocLauncher.zip'), dontcache=True)

    def reqCheckUpdates(self):
        self.log.debug('%s reqCheckUpdates' % self.name)
        result = []
        con = self.bfaserver.con.connections[self._job.profile]

        if not con.update_files and not con.update_dirs:
            xml = bitfarm_xml.create_xml_request(reqtype="respNoUpdateConfiguration", data=_("Keine Client Updates konfiguriert"))
        else:
            update_files = con.update_files[:]
            if 'Updater.exe' not in update_files:       # sollte der Updater selbst nicht in der Liste stehen, fügen wir ihn hier hinzu
                update_files.append('Updater.exe')
            if 'scripts.ini' not in update_files:       # sollte die scripts.ini nicht in der Liste stehen, fügen wir ihn hier hinzu
                update_files.append('scripts.ini')
            if '%s.con' % self._job.profile not in update_files:       # sollte das Profil nicht in der Liste stehen, fügen wir ihn hier hinzu
                update_files.append('%s.con' % self._job.profile)
            if con.update_dirs:
                for dname in con.update_dirs:
                    dname = os.path.join(self.bfaserver.bfapath, dname)
                    if os.path.isdir(dname) and os.path.exists(dname):
                        fnames = glob.glob(dname + r'\\*.*')
                        for fname in fnames:
                            fname = fname.replace(self.bfaserver.bfapath + '\\', '')
                            update_files.append(fname)

            for fname in update_files:
                fpath = os.path.join(self.bfaserver.bfapath, fname)
                if os.path.exists(fpath):
                    stats = os.stat(fpath)
                    # mtime = time.localtime(stats.st_mtime)
                    mtime = time.gmtime(stats.st_mtime)
                    # da in Zip Dateien keine Filedates mit ungeraden Sekunden möglich sind, runden wir ungerade Sekunden ab
                    # das macht das ZipFile Module genauso

                    # Behandlung des UTC Zeitstempels, siehe reqSendUpdates
                    tstamp = (fname, mtime.tm_year, mtime.tm_mon, mtime.tm_mday, mtime.tm_hour if mtime.tm_isdst == 0 else mtime.tm_hour -1, mtime.tm_min, mtime.tm_sec if mtime.tm_sec % 2 == 0 else mtime.tm_sec -1)
                    try:
                        result.append("%s|%04d-%02d-%02d %02d:%02d:%02d" % tstamp)
                        # result.append("%s|%s" % (fname, time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(stats.st_mtime))))
                    except:
                        self.log.exception("Konnte Datei %s nicht prüfen" % fpath)
                else:
                    self.log.warning("Konnte Datei %s nicht finden" % fpath)
            if result:
                xml = bitfarm_xml.create_xml_request(reqtype="respOK",  data=";".join(result))
            else:
                xml = bitfarm_xml.create_xml_request(reqtype="respError", data=_("Keine Dateien gefunden"))

        try:
            self._job.sock.send(xml)
        except:
            self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None

    def reqSendUpdates(self):
        self.log.debug('%s reqSendUpdates' % self.name)

        update_files = self._job.parameter.split(';')

        tempdir = tempfile.mkdtemp(dir=os.path.join(self.bfaserver.mypath, 'temp'))
        try:
            with zipfile.ZipFile(os.path.join(tempdir, 'Updates.zip'), 'w') as myzip:
                for fname in update_files:
                    fpath = os.path.join(self.bfaserver.bfapath, fname)
                    if os.path.exists(fpath):
                        stats = os.stat(fpath)
                        # mtime = time.gmtime(stats.st_mtime)
                        mtime = time.localtime(stats.st_mtime)
                        isdst = mtime.tm_isdst
                        # da in Zip Dateien keine Filedates mit ungeraden Sekunden möglich sind, runden wir ungerade Sekunden ab
                        # das macht das ZipFile Module genauso

                        tstamp = (mtime.tm_year, mtime.tm_mon, mtime.tm_mday, mtime.tm_hour, mtime.tm_min, mtime.tm_sec if mtime.tm_sec % 2 == 0 else mtime.tm_sec - 1)
                        lstamp = '%04d-%02d-%02d %02d:%02d:%02d' % tstamp
                        self.log.debug('%s Original: %s | %s %s' % (self.name, fname, lstamp, mtime.tm_isdst))
                        if time.localtime().tm_isdst == 0:   # haben wir Winterzeit
                            tstamp = (mtime.tm_year, mtime.tm_mon, mtime.tm_mday, mtime.tm_hour if mtime.tm_isdst == 0 else mtime.tm_hour - 1, mtime.tm_min, mtime.tm_sec if mtime.tm_sec % 2 == 0 else mtime.tm_sec - 1)
                        else:   # nein wir haben Sommerzeit
                            tstamp = (mtime.tm_year, mtime.tm_mon, mtime.tm_mday, mtime.tm_hour + 1 if mtime.tm_isdst == 0 else mtime.tm_hour, mtime.tm_min, mtime.tm_sec if mtime.tm_sec % 2 == 0 else mtime.tm_sec - 1)
                        # zur Unterscheidlichen Behandlung der Zeitstempel in reqCheckUpdates und reqSendUpdates:
                        # reqCheckUpdates prüft den UTC Zeitstempel, damit das mit dem per Delphi geprüften Zeitstempel passt, muss die Sommerzeit berücksichtigt werden
                        # in reqSendUpdates nehmen wir den localtime Zeitstempel und müssen abhängig von der aktuell geltenen Sommer oder Winterzeit
                        # die Zeitstempel anpassen.
                        lstamp = '%04d-%02d-%02d %02d:%02d:%02d' % tstamp
                        self.log.debug('%s Packe Update.zip: %s | %s %s' % (self.name, fname, lstamp, mtime.tm_isdst))
                        info = zipfile.ZipInfo(filename=fname, date_time=tstamp)
                        with open(fpath, 'rb') as f:
                            myzip.writestr(info, f.read())
                myzip.close()
        except:
            self.log.exception("Fehler beim Erstellen der Update.zip Datei")
            shutil.rmtree(tempdir)

            xml = bitfarm_xml.create_xml_request(reqtype="respError", data=_("Die Update.zip Datei konnte nicht erstellt werden"))
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None
        else:
            doc = bitfarm_doc.BitfarmDocument()
            doc.profile = self._job.profile

            self.send_file_to_client(doc, os.path.join(tempdir, 'Updates.zip'), dontcache=True)

            shutil.rmtree(tempdir)

    def create_adconnection(self, username, userpass, mydomain=None):
        self.log.debug("%s Anmeldung am AD" % self.name)
        con = self.bfaserver.con.connections[self._job.profile]
        conn = None

        if self.bfaserver.ldapdebug or con.ldap_debug:
            flog = open(os.path.join(self.bfaserver.mypath, 'logs', 'ldap_%s.log' % datetime.datetime.now().strftime('%Y%m%d%H%M%S%f')),'w')

        # wenn wir eine domain haben an der wir uns gezielt anmelden sollen (zB wegen einem referral)
        if mydomain is not None:
            domains = [mydomain.lower()]
        else:
            domains = con.domains

        for domain in domains:
            if not domain:
                try:
                    #TODO: funktioniert nicht im Threads, aber für AD-Abgleich sollte Domäne auch schon bekannt sein.
                    if con.use_ldap_ssl[domain]:
                        objRootDSE = win32com.client.GetObject('LDAPS://RootDSE')
                    else:
                        objRootDSE = win32com.client.GetObject('LDAP://RootDSE')

                    objRootDSE.GetInfo()
                    objRootDSE.Get('defaultNamingContext')
                    base_dn = objRootDSE.Get('defaultNamingContext')
                    domain = ".".join(x[3:] for x in base_dn.lower().split(','))
                except:
                    self.log.exception("%s Fehler bei Domainabfrage über LDAP" % self.name)
                    if 'USERDNSDOMAIN' in os.environ:
                        domain = os.environ['USERDNSDOMAIN'].lower()
                        base_dn = ",".join(['DC=%s' % x for x in domain.split('.')])
                    else:
                        self.log.error("%s Konnte Systemvariable USERDNSDOMAIN nicht auflösen" % self.name)
            else:
                base_dn = ",".join(['DC=%s' % x for x in domain.lower().split('.')])

            if domain:
                if domain not in con.use_ldap_ssl:
                    con.use_ldap_ssl[domain] = con.use_ldap_ssl[None]
                if domain not in con.use_ldap_tls:
                    con.use_ldap_tls[domain] = con.use_ldap_tls[None]

                domaincontroller = con.domaincontrollers[domain] if domain in con.domaincontrollers and con.domaincontrollers[domain] else domain
                if domain in con.use_ldap_ssl and con.use_ldap_ssl[domain]:
                    self.log.debug(u"%s Starte LDAPS Abfrage an Domain %s Controler %s" % (self.name, domain, domaincontroller))
                else:
                    self.log.debug(u"%s Starte LDAP Abfrage an Domain %s Controler %s" % (self.name, domain, domaincontroller))
                if domain in con.use_ldap_tls and con.use_ldap_tls[domain]:
                    self.log.debug(u"%s Verwende LDAP mit TLS" % self.name)
                conn = None

                if con.use_ldap_ssl[domain]:
                    try:
                        ldap.set_option(ldap.OPT_X_TLS_REQUIRE_CERT, ldap.OPT_X_TLS_NEVER)
                        if self.bfaserver.ldapdebug or con.ldap_debug:
                            conn = ldap.initialize('LDAPS://%s:%d' % (domaincontroller, con.ldap_ssl_port), trace_level=1, trace_file=flog)
                        else:
                            conn = ldap.initialize('LDAPS://%s:%d' % (domaincontroller, con.ldap_ssl_port))
                        conn.set_option(ldap.OPT_REFERRALS, 0)
                        conn.set_option(ldap.OPT_TIMEOUT, 10)
                        conn.set_option(ldap.OPT_NETWORK_TIMEOUT, 10)
                        conn.set_option(ldap.OPT_PROTOCOL_VERSION, 3)
                        if con.ldap_ssl_cert:
                            if os.path.exists(con.ldap_ssl_cert):
                                conn.set_option(ldap.OPT_X_TLS_CACERTFILE, con.ldap_ssl_cert)
                            else:
                                self.log.error("%s Konnte SSL-Zertifikat nicht finden %s" % (self.name, con.ldap_ssl_cert))
                        conn.set_option(ldap.OPT_X_TLS, ldap.OPT_X_TLS_DEMAND)
                        conn.set_option(ldap.OPT_X_TLS_DEMAND, True)
                        # conn.set_option(ldap.OPT_DEBUG_LEVEL, 255)
                    except:
                        self.log.exception("%s Fehler bei LDAPS.Initialize" % self.name)
                        conn = None
                else:
                    try:
                        if con.use_ldap_tls[domain]:
                            ldap.set_option(ldap.OPT_X_TLS_REQUIRE_CERT, ldap.OPT_X_TLS_ALLOW)
                        if self.bfaserver.ldapdebug or con.ldap_debug:
                            conn = ldap.initialize('LDAP://%s' % domaincontroller, trace_level=1, trace_file=flog)
                        else:
                            conn = ldap.initialize('LDAP://%s' % domaincontroller)
                            # conn = ldap.initialize('ldaps://%s' % domain)
                        conn.protocol_version = 3
                        conn.set_option(ldap.OPT_REFERRALS, 0)
                        conn.set_option(ldap.OPT_TIMEOUT, 10)
                        conn.set_option(ldap.OPT_NETWORK_TIMEOUT, 10)
                        conn.set_option(ldap.OPT_PROTOCOL_VERSION, 3)
                        if con.use_ldap_tls[domain]:
                            # conn.set_option(ldap.OPT_X_TLS_REQUIRE_CERT, ldap.OPT_X_TLS_DEMAND)
                            # conn.set_option(ldap.OPT_X_TLS_NEWCTX, 0)
                            try:
                                conn.start_tls_s()
                            except:
                                self.log.exception(u"%s Fehler bei START_TLS" % self.name)
                                conn = None
                                auth = False
                        else:
                            conn.set_option(ldap.OPT_X_TLS, ldap.OPT_X_TLS_NEVER)
                            conn.set_option(ldap.OPT_X_TLS_REQUIRE_CERT, ldap.OPT_X_TLS_NEVER)
                            conn.set_option(ldap.OPT_X_TLS_DEMAND, False)
                    except:
                        self.log.exception("%s Fehler bei LDAP.Initialize" % self.name)
                        conn = None
                        continue

                if conn:
                    succeed = False
                    loop = 0
                    while loop < 5 and not succeed:
                        try:
                            conn.simple_bind_s('%s@%s' % (username, domain), userpass)

                            conn.search_s(base_dn, ldap.SCOPE_SUBTREE, 'cn=%s' % username)
                        except ldap.INVALID_CREDENTIALS:
                            self.log.error("%s Passwort ist falsch" % self.name)
                            conn = None
                            break
                        except:
                            self.log.error(u"Fehler bei simple_bind_s, Versuch %d" % (loop + 1))
                            self.log.error(u"%s %s" % (self.name, str(sys.exc_info())))
                            time.sleep(0.5)
                        else:
                            succeed = True
                            break   # erfolgreich, dann raus aus der Schleife

                        loop += 1

                        if not succeed and loop == 5:
                            self.log.error(u"%s Timeout: 5 Versuche bei der Anmeldung eines Users an den LDAP-Server erfolglos!" % self.name)
                            self.log.error(u"%s base_dn: %s" % (self.name, base_dn))
                            conn = None

                    if succeed:
                        break

                    continue    # weiter mit nächster Domäne

            if conn is None or not succeed:
                self.log.error(u"%s Es konnte keine Verbindung zum Domain-Controller aufgebaut werden.\n%s" % (self.name, domain))
                return None
            else:
                self.log.debug("%s Anmeldung an %s erfolgreich" % (self.name, domain))
                return conn

        return conn

    def reqADTree(self):
        self.log.debug('%s reqADTree' % self.name)
        error = False
        error_cnt = 0
        referall = False
        error_no_object = False
        no_object = []
        con = self.bfaserver.con.connections[self._job.profile]
        ldappath, username, userpass = self._job.parameter.split(';')
        userpass = con.bitdecrypt(userpass)
        conn = self.create_adconnection(username, userpass)
        if conn is None:
            xml = bitfarm_xml.create_xml_request(reqtype="respError", data=_("Es konnte keine Verbindung zum Domain-Controller aufgebaut werden"))
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None
        else:
            data = []
            if ldappath:
                # ldappath = ldap.dn.escape_dn_chars(ldappath)
                self.log.debug('Ldap-Abfrage ldappath %s' % ldappath)
                res = None
                result = []

                page_control = ldap.controls.libldap.SimplePagedResultsControl(True, size=self.bfaserver.ldap_sizelimit, cookie='')
                response = conn.search_ext(ldappath, ldap.SCOPE_ONELEVEL,
                                           'objectClass=*', ['displayname','objectClass','name'], serverctrls=[page_control])

                pages = 0
                while True:
                    pages += 1
                    self.log.debug('Pages %i' % pages)
                    rdata = [None]

                    try:
                        rtype, rdata, rmsgid, serverctrls = conn.result3(response)
                            # res = conn.search_s(ldappath, ldap.SCOPE_ONELEVEL, '(objectClass=*)', ['displayname','objectClass','name'])
                    except ldap.REFERRAL:
                        self.log.error("%s Abfrage von %s ergab REFERRAL (1)" % (self.name, ldappath))
                        referall = True
                        exc_args = sys.exc_info()[1][0]
                        if 'info' in exc_args:
                            ref_domain = exc_args['info'].lower()
                            ref_domain = ref_domain[ref_domain.find('dc='):]
                            for domain in con.domains:
                                domain_dn = ",".join(['dc=%s' % x for x in domain.lower().split('.')])
                                self.log.debug("%s domain_dn %s -> referall %s" % (self.name, domain_dn, ref_domain))
                                if domain_dn == ref_domain:     # okay, wir haben in den uns bekannten domains, die auf die das Referall umleiten will
                                    conn2 = self.create_adconnection(username, userpass, domain)
                                    if conn2:
                                        try:
                                            response = conn2.search_ext(ldappath, ldap.SCOPE_ONELEVEL,
                                                                           'objectClass=*', ['displayname','objectClass','name'], serverctrls=[page_control])
                                        except:
                                            self.log.exception("%s Fehler beim Ummelden an andere Domain (Referall) %s" % (self.name, domain))
                                        else:
                                            referall = False
                                            break   # erfolgreich? dann raus aus der schleife
                        if not referall:
                            conn = conn2
                        else:
                            error = True
                            break       #kein Ummeldung möglich? dann raus
                        continue        # umgemeldet bei referral, dann nochmal die Abfrage und oben in der Schleife die Ergebnisse holen

                    except ldap.NO_SUCH_OBJECT:
                        self.log.error('%s Das Object %s konnte nicht gefunden werden' % (self.name, ldappath))
                        error_no_object = True
                        no_object.append(ldappath)
                        break   # wenn es nicht da ist, müssen wir auch nicht mehrmals Suchen
                    except:
                        self.log.exception("%s Unbekannter Fehler %s" % (self.name, ldappath))
                        error = True
                    else:
                        error = False
                        result.extend(rdata)
                        self.log.debug('Ldap-Abfrage Ergebnisse: %i' % len(result))
                        controls = [control for control in serverctrls
                                    if control.controlType == ldap.controls.libldap.SimplePagedResultsControl.controlType]
                        if not controls:
                            # print('The server ignores RFC 2696 control')
                            break
                        if not controls[0].cookie:
                            break
                        page_control.cookie = controls[0].cookie
                        response = conn.search_ext(ldappath, ldap.SCOPE_ONELEVEL,
                                                   'objectClass=*', ['displayname','objectClass','name'], serverctrls=[page_control])
                    if error:
                        error_cnt += 1
                        if error_cnt > 9:
                            self.log.error('Es hat zehn Fehler in der Abfrageschleife gegeben...')
                            break

                self.log.debug('Abfrageschleife beendet')
                if result and type(result) is list:
                    for x in result:
                        if x[0] is not None and x[1]:       # wenn wir einen LDAP DN und Attribute (x[1]) geliefert bekommen haben
                            self.log.debug('LDAP DN %s Attribute %s' % (x[0], x[1]))
                            if 'objectClass' in x[1] and x[1]['objectClass'][-1] != b'user':
                                ad = bitfarm_doc.BitfarmADEntry()
                                ad.displayname = x[0]
                                ad.adpath = x[0]
                                if 'objectClass' in x[1]:
                                    ad.adclass = x[1]['objectClass'][-1].decode()    # der Liste Eintrag aus dem Tupel
                                if 'name' in x[1]:
                                    ad.displayname = x[1]['name'][0].decode()
                                has_user = None
                                try:
                                    has_user = conn.search_s(ad.adpath, ldap.SCOPE_SUBTREE, '(objectClass=user)', ['displayname','objectClass'])
                                except ldap.REFERRAL:
                                    self.log.error("%s Abfrage von %s ergab REFERRAL (2)" % (self.name, ldappath))
                                    referall = True
                                    exc_args = sys.exc_info()[1][0]
                                    if 'info' in exc_args:
                                        ref_domain = exc_args['info'].lower()
                                        ref_domain = ref_domain[ref_domain.find('dc='):]
                                        for domain in con.domains:
                                            domain_dn = ",".join(['dc=%s' % x for x in domain.lower().split('.')])
                                            self.log.debug("%s domain_dn %s -> referall %s" % (self.name, domain_dn, ref_domain))
                                            if domain_dn == ref_domain:     # okay, wir haben in den uns bekannten domains die domain, auf die das Referall umleiten will
                                                conn2 = self.create_adconnection(username, userpass, domain)
                                                if conn2:
                                                    try:
                                                        has_user = conn2.search_s(ad.adpath, ldap.SCOPE_SUBTREE, '(objectClass=user)', ['displayname','objectClass'])
                                                    except:
                                                        self.log.exception("%s Fehler beim Ummelden an andere Domain (Referall) %s" % (self.name, domain))
                                                    else:
                                                        referall = False
                                                        break   # erfolgreich? dann raus aus der schleife
                                except ldap.NO_SUCH_OBJECT:
                                    self.log.error('%s Das Object %s konnte nicht gefunden werden' % (self.name, ad.adpath))
                                    error_no_object = True
                                    no_object.append(ad.adpath)
                                    break   # wenn es nicht da ist, müssen wir auch nicht mehrmals Suchen
                                except ldap.SIZELIMIT_EXCEEDED:     # wenn wir bei der Abfrage der User-Objekte ein Sizelimit_exceed haben, dann haben wir wohl mehr als genug user
                                    self.log.error(u"%s sizelimit_exceeded (1)" % self.name)
                                    has_user = [('item', {'objectClass': [b'user']})]   # wir bauen uns ein Ergebnis
                                except:
                                    self.log.exception("%s Unbekannter Fehler %s" % (self.name, ldappath))
                                    error = True

                                if type(has_user) is list and len(has_user) > 0:
                                    for item in has_user:
                                        if 'objectClass' in item[1] and item[1]['objectClass'][-1] == b'user':
                                            ad.has_user = True
                                            break

                                if not ad.has_user:
                                    has_user = None
                                    try:
                                        self.log.debug('Abfrage Gruppen Objekte adpath %s' % ad.adpath)
                                        has_user = conn.search_s(ad.adpath, ldap.SCOPE_SUBTREE, '(objectClass=group)', ['displayname','objectClass','member'])
                                    except ldap.REFERRAL:
                                        self.log.error("%s Abfrage von %s ergab REFERRAL (3)" % (self.name, ldappath))
                                        referall = True
                                        exc_args = sys.exc_info()[1][0]
                                        if 'info' in exc_args:
                                            ref_domain = exc_args['info'].lower()
                                            ref_domain = ref_domain[ref_domain.find('dc='):]
                                            for domain in con.domains:
                                                domain_dn = ",".join(['dc=%s' % x for x in domain.lower().split('.')])
                                                self.log.debug("%s domain_dn %s -> referall %s" % (self.name, domain_dn, ref_domain))
                                                if domain_dn == ref_domain:     # okay, wir haben in den uns bekannten domains die domain, auf die das Referall umleiten will
                                                    conn2 = self.create_adconnection(username, userpass, domain)
                                                    if conn2:
                                                        try:
                                                            has_user = conn2.search_s(ad.adpath, ldap.SCOPE_SUBTREE, '(objectClass=group)', ['displayname','objectClass','member'])
                                                        except:
                                                            self.log.exception("%s Fehler beim Ummelden an andere Domain (Referall) %s" % (self.name, domain))
                                                        else:
                                                            referall = False
                                                            break   # erfolgreich? dann raus aus der schleife

                                    except ldap.NO_SUCH_OBJECT:
                                        self.log.error('%s Das Object %s konnte nicht gefunden werden' % (self.name, ad.adpath))
                                        error_no_object = True
                                        no_object.append(ad.adpath)
                                        break   # wenn es nicht da ist, müssen wir auch nicht mehrmals Suchen
                                    except ldap.SIZELIMIT_EXCEEDED:     # wenn wir bei der Abfrage der User-Objekte ein Sizelimit_exceed haben, dann haben wir wohl mehr als genug user
                                        self.log.error(u"%s sizelimit_exceeded (2)" % self.name)
                                        has_user = [('item', {'objectClass': [b'group'], 'member': [b'some member']})]   # wir bauen uns ein Ergebnis
                                    except:
                                        self.log.exception("%s Unbekannter Fehler %s" % (self.name, ldappath))
                                        error = True

                                    if type(has_user) is list and len(has_user) > 0:
                                        for item in has_user:
                                            if 'objectClass' in item[1] and item[1]['objectClass'][-1] == b'group' and 'member' in item[1]:
                                                ad.has_user = True
                                                break

                                data.append(ad)
            if referall:
                self.log.error(u"%s Im abgefragten AD-Objekt befinden sich umgeleitete Objekte, die nicht aufgelöst werden können (REFERALLS)" % self.name)
                xml = bitfarm_xml.create_xml_request(reqtype="respError", data=u"Im abgefragten AD-Objekt befinden sich umgeleitete Objekte, die nicht aufgelöst werden können (REFERALLS)")
            elif error:
                self.log.error(u"%s Allgemeiner Fehler bei der AD-Abfrage" % self.name)
                xml = bitfarm_xml.create_xml_request(reqtype="respError", data=u"Allgemeiner Fehler bei der AD-Abfrage")
            elif error_no_object:
                self.log.error(u"%s AD Objekt konnte nicht gefunden werdenn" % self.name)
                if len(no_object) == 1:
                    msg = "Es konnte ein AD Objekt nicht gefunden werden:\n%s" % no_object[0]
                else:
                    msg = "Es konnten mehrere AD Objekte nicht gefunden werden:\n%s" % ",".join(no_object)
                xml = bitfarm_xml.create_xml_request(reqtype="respError", data=msg)
            else:
                self.log.debug('Sende Response OK, Ergebnisse in data %i' % len(data))
                xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=data)

            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None

    def reqADUser(self):
        self.log.debug('%s reqADUser' % self.name)
        error = False
        error_cnt = 0
        referall = False
        error_no_object = False
        no_object = []
        con = self.bfaserver.con.connections[self._job.profile]
        ldappath, username, userpass = self._job.parameter.split(';')
        userpass = con.bitdecrypt(userpass)
        conn = self.create_adconnection(username, userpass)
        if conn is None:
            xml = bitfarm_xml.create_xml_request(reqtype="respError", data=_("Es konnte keine Verbindung zum Domain-Controller aufgebaut werden"))
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None
        else:
            data = []
            if ldappath:
                # ldappath = ldap.dn.escape_dn_chars(ldappath)
                self.log.debug('Ldap-Abfrage ldappath %s' % ldappath)
                res = None
                result = []

                page_control = ldap.controls.libldap.SimplePagedResultsControl(True, size=self.bfaserver.ldap_sizelimit, cookie='')
                response = conn.search_ext(ldappath, ldap.SCOPE_SUBTREE,
                                           'objectClass=*', ['samaccountname','objectClass','member','displayName','mail','userAccountControl'], serverctrls=[page_control])

                pages = 0
                while True:
                    pages += 1
                    self.log.debug('Pages %i' % pages)
                    rdata = [None]

                    try:
                        rtype, rdata, rmsgid, serverctrls = conn.result3(response)
                    except ldap.REFERRAL:
                        self.log.error("%s Abfrage von %s ergab REFERRAL (1)" % (self.name, ldappath))
                        referall = True
                        exc_args = sys.exc_info()[1][0]
                        if 'info' in exc_args:
                            ref_domain = exc_args['info'].lower()
                            ref_domain = ref_domain[ref_domain.find('dc='):]
                            for domain in con.domains:
                                domain_dn = ",".join(['dc=%s' % x for x in domain.lower().split('.')])
                                self.log.debug("%s domain_dn %s -> referall %s" % (self.name, domain_dn, ref_domain))
                                if domain_dn == ref_domain:     # okay, wir haben in den uns bekannten domains die domain, auf die das Referall umleiten will
                                    conn2 = self.create_adconnection(username, userpass, domain)
                                    if conn2:
                                        try:
                                            response = conn2.search_ext(ldappath, ldap.SCOPE_SUBTREE,
                                                        '(objectClass=*)', ['sAMAccountName','displayName','mail','userAccountControl','objectClass','member'], serverctrls=[page_control])
                                        except:
                                            self.log.exception("%s Fehler beim Ummelden an andere Domain (Referall) %s" % (self.name, domain))
                                        else:
                                            referall = False
                                            break   # erfolgreich? dann raus aus der schleife
                            if not referall:
                                conn = conn2
                            else:
                                error = True
                                break       #kein Ummeldung möglich? dann raus
                            continue        # umgemeldet bei referral, dann nochmal die Abfrage und ob in der Schleife die Ergebnisse holen

                    except ldap.NO_SUCH_OBJECT:
                        self.log.error('%s Das Object %s konnte nicht gefunden werden' % (self.name, ldappath) )
                        error_no_object = True
                        no_object.append(ldappath)
                        break   # wenn es nicht da ist, müssen wir auch nicht mehrmals Suchen
                    except:
                        self.log.exception("%s Unbekannter Fehler %s" % (self.name, ldappath))
                        error = True
                    else:
                        error = False
                        result.extend(rdata)
                        self.log.debug('Ldap-Abfrage Ergebnisse: %i' % len(result))

                        controls = [control for control in serverctrls
                                    if control.controlType == ldap.controls.libldap.SimplePagedResultsControl.controlType]
                        if not controls:
                            # print('The server ignores RFC 2696 control')
                            break
                        if not controls[0].cookie:
                            break
                        page_control.cookie = controls[0].cookie
                        response = conn.search_ext(ldappath, ldap.SCOPE_SUBTREE,
                                                   'objectClass=*', ['samaccountname','objectClass','member','displayName','mail','userAccountControl'], serverctrls=[page_control])
                    if error:
                        error_cnt += 1
                        if error_cnt > 9:
                            self.log.error('Es hat zehn Fehler in der Abfrageschleife gegeben...')
                            break

                self.log.debug('Abfrageschleife beendet')

                if result and type(result) is list:
                    for x in result:
                        if x[0] is not None and x[1]:       # wenn wir einen LDAP DN und Attribute (x[1]) geliefert bekommen haben
                            self.log.debug('LDAP DN %s Attribute %s' % (x[0], x[1]))
                            if 'objectClass' in x[1]:
                                objclass = x[1]['objectClass'][-1].decode()
                            if objclass == 'user':
                                ad = bitfarm_doc.BitfarmADEntry()
                                ad.displayname = x[0]
                                ad.adpath = x[0]
                                # Results kommen als ByteStrings und müssen nach UTF-8 ecodiert werden
                                if 'objectClass' in x[1]:
                                    ad.adclass = x[1]['objectClass'][-1].decode()   # der Liste Eintrag aus dem Tupel
                                if 'displayName' in x[1]:
                                    ad.displayname = x[1]['displayName'][0].decode()
                                if 'sAMAccountName' in x[1]:
                                    ad.samaccountname = x[1]['sAMAccountName'][0].decode()
                                if 'mail' in x[1]:
                                    ad.mail = x[1]['mail'][0].decode()
                                if 'userAccountControl' in x[1]:
                                    ad.useraccountcontrol = x[1]['userAccountControl'][0].decode()

                                if ad.adpath not in [item.adpath for item in data]:
                                    data.append(ad)

                            if objclass == 'group':
                                if 'member' in x[1]:
                                    page_control2 = ldap.controls.libldap.SimplePagedResultsControl(True, size=self.bfaserver.ldap_sizelimit, cookie='')

                                    for member in x[1]['member']:
                                        member = member.decode()
                                        response2 = conn.search_ext(member, ldap.SCOPE_SUBTREE,
                                                    'objectClass=*', ['samaccountname','objectClass', 'member', 'displayName','mail','userAccountControl'], serverctrls=[page_control2])

                                        pages2 = 0
                                        while True:
                                            pages2 += 1
                                            mem = None

                                            try:
                                                # mem = conn.search_ext(member, ldap.SCOPE_SUBTREE, '(objectClass=user)', ['sAMAccountName','displayName','mail','userAccountControl','objectClass'])
                                                self.log.debug('Abfrage Member %s' % member)
                                                rtype, mem, rmsgid, serverctrls = conn.result3(response2)
                                            except ldap.REFERRAL:
                                                self.log.error("%s Abfrage von %s ergab REFERRAL (2)" % (self.name, member))
                                                referall = True
                                                exc_args = sys.exc_info()[1][0]
                                                if 'info' in exc_args:
                                                    ref_domain = exc_args['info'].lower()
                                                    ref_domain = ref_domain[ref_domain.find('dc='):]

                                                    for domain in con.domains:
                                                        domain_dn = ",".join(['dc=%s' % x for x in domain.lower().split('.')])
                                                        self.log.debug("%s domain_dn %s -> referall %s" % (self.name, domain_dn, ref_domain))
                                                        if domain_dn == ref_domain.lower():     # okay, wir haben in den uns bekannten domains die domain, auf die das Referall umleiten will
                                                            conn2 = self.create_adconnection(username, userpass, domain)
                                                            if conn2:
                                                                try:
                                                                    response2 = conn2.search_ext(member, ldap.SCOPE_SUBTREE, '(objectClass=user)', ['sAMAccountName','displayName','mail','userAccountControl','objectClass'], serverctrls=[page_control2])
                                                                except:
                                                                    self.log.exception(u"%s Fehler beim Ummelden an andere Domain (Referall) %s" % (self.name, domain))
                                                                else:
                                                                    referall = False
                                                                    break   # erfolgreich? dann raus aus der schleife

                                                    if not referall:
                                                        conn = conn2
                                                    else:
                                                        error = True
                                                        break       #kein Ummeldung möglich? dann raus
                                                    continue
                                            except ldap.NO_SUCH_OBJECT:
                                                self.log.error('%s Das Object %s konnte nicht gefunden werden' % (self.name, member))
                                                error_no_object = True
                                                no_object.append(member)
                                                break   # wenn es nicht da ist, müssen wir auch nicht mehrmals Suchen
                                            except:
                                                self.log.exception("%s Unbekannter Fehler %s" % (self.name, ldappath))
                                                error = True
                                            else:
                                                error = False
                                                if mem:
                                                    if type(mem) is list and len(mem) > 0:
                                                        mem = mem[0]
                                                    if mem is not None and 'objectClass' in mem[1] and mem[1]['objectClass'][-1] == b'user':
                                                        ad = bitfarm_doc.BitfarmADEntry()
                                                        ad.displayname = mem[0]
                                                        ad.adpath = mem[0]
                                                        # Results kommen als ByteStrings und müssen nach UTF-8 ecodiert werden
                                                        if 'objectClass' in mem[1]:
                                                            ad.adclass = mem[1]['objectClass'][-1].decode()
                                                        if 'displayName' in mem[1]:
                                                            ad.displayname = mem[1]['displayName'][0].decode()
                                                        if 'sAMAccountName' in mem[1]:
                                                            ad.samaccountname = mem[1]['sAMAccountName'][0].decode()
                                                        if 'mail' in mem[1]:
                                                            ad.mail = mem[1]['mail'][0].decode()
                                                        if 'userAccountControl' in mem[1]:
                                                            ad.useraccountcontrol = mem[1]['userAccountControl'][0].decode()
                                                        if ad.adpath not in [item.adpath for item in data]:
                                                            data.append(ad)

                                                controls = [control for control in serverctrls
                                                            if control.controlType == ldap.controls.libldap.SimplePagedResultsControl.controlType]
                                                if not controls:
                                                    # print('The server ignores RFC 2696 control')
                                                    break
                                                if not controls[0].cookie:
                                                    break
                                                page_control2.cookie = controls[0].cookie
                                                response2 = conn.search_ext(ldappath, ldap.SCOPE_SUBTREE,
                                                            'objectClass=*', ['samaccountname','objectClass','member', 'displayName','mail','userAccountControl'], serverctrls=[page_control2])

                                            if error:
                                                error_cnt += 1
                                                if error_cnt > 9:
                                                    self.log.error('Es hat zehn Fehler in der Abfrageschleife gegeben...')
                                                    break

            if referall:
                self.log.error(u"%s Im abgefragten AD-Objekt befinden sich umgeleitete Objekte, die nicht aufgelöst werden können (REFERALLS)" % self.name)
                xml = bitfarm_xml.create_xml_request(reqtype="respError", data="Im abgefragten AD-Objekt befinden sich umgeleitete Objekte, die nicht aufgelöst werden können (REFERALLS)")
            elif error:
                self.log.error(u"%s Allgemeiner Fehler bei der AD-Abfrage" % self.name)
                xml = bitfarm_xml.create_xml_request(reqtype="respError", data="Allgemeiner Fehler bei der AD-Abfrage")
            elif error_no_object:
                self.log.error(u"%s AD Objekt konnte nicht gefunden werdenn" % self.name)
                if len(no_object) == 1:
                    msg = "Es konnte ein AD Objekt nicht gefunden werden:\n%s" % no_object[0]
                else:
                    msg = "Es konnten mehrere AD Objekte nicht gefunden werden:\n%s" % ",".join(no_object)
                xml = bitfarm_xml.create_xml_request(reqtype="respError", data=msg)
            else:
                self.log.debug('Sende Response OK, Ergebnisse in data %i' % len(data))
                xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=data)

            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None

    def reqFile(self):
        self.log.debug('%s reqFile' % self.name)

        filepath = self._job.filename
        doc = bitfarm_doc.BitfarmDocument()
        doc.gdocid = -1
        self._job.gdocid = -1
        return self.send_file_to_client(doc, filepath, None)

    def reqPreview(self):
        self.log.debug('%s reqPreview' % self.name)
        self.send_file()

    def reqOri(self):
        self.log.debug('%s reqOri' % self.name)
        self.send_file()

    def reqIEV(self):
        self.log.debug('%s reqIEV' % self.name)
        self.send_file()

    def add_private_annotations(self, doc, doc_right):
        # den bereits geladenen Annotations (public) die Dokumentrechte mitgeben
        for anno in doc.annotations:
            anno.rights = doc_right.rights

        # der eigene Layer kommt immer hinzu
        annos = []
        annos = self.bfaProcessing.get_annotations(self._job.gdocid, profile=self._job.profile,
                                                           userids=[self._job.userid], logid=self.name)
        for anno in annos:
            anno.rights = bitfarm_doc.RIGHT_APREAD | bitfarm_doc.RIGHT_APWRITE | bitfarm_doc.RIGHT_APMODIFY
            doc_right.apread = True  # wenn wir private Annos haben, kommt zur Dokumentenberechtigung wenigsten apread hinzu
        doc.annotations.extend(annos)

        annos = []
        iev_rights = self.bfaserver.con.connections[self._job.profile].read_docrights(
            db=self.bfaProcessing.db[self._job.profile], gdocid=self._job.gdocid, user=self._job.userid)
        if iev_rights is not None and len(iev_rights) > 0:
            for iev_right in iev_rights:
                if iev_right.apread:  # wir haben mindestens leseberechtigung auf einen freigegebenen layer
                    if iev_right.user is not None:
                        annos = self.bfaProcessing.get_annotations(self._job.gdocid, profile=self._job.profile,
                                                                           userids=[iev_right.user.src_userid], logid=self.name)
                        for anno in annos:
                            anno.rights = iev_right.rights
                            if (anno.rights & bitfarm_doc.RIGHT_APREAD) == bitfarm_doc.RIGHT_APREAD:
                                doc_right.apread = True  # wenn wir private Annos haben, kommt zur Dokumentenberechtigung wenigsten apread hinzu

        doc.annotations.extend(annos)

    def get_inedit(self):
        ret = ""
        if self.bfaserver.con.connections[self._job.profile].resultlist_showinedit: #zeige Dokumente die aktuell in Bearbeitung sind
            ret = ",".join([str(x) for x in list(self.bfaserver.in_progress[self._job.profile].keys())])
        else:
            sql = "select distinct(gdocid) from sys_timer where status & 4 = 0"

            try:
                result = self.db[self._job.profile].execute(sql).fetchall()
            except:
                self.log.exception("Fehler bei Abfrage der Wiedervorlage")
            else:
                if result:
                    ret = ",".join([str(x[0]) for x in result])
        return ret

    def send_file(self):
        self.log.debug('%s send_file' % self.name)

        if self._job.reqtype in ["reqPreview", "reqFile", "reqIEV", "reqOri"]:
            page = None
            filepath = None
            filehash = None
            dstfile = None
            svnfile = None
            dontcache = False
            _con = self.bfaserver.con.connections[self._job.profile]

            doc_right = self.check_docrights()

            if self._job.userid not in self.bfaserver.user[self._job.profile] or not doc_right.read:
                xml = bitfarm_xml.create_xml_request(reqtype="respDenied", userid=0, gdocid=self._job.gdocid, data=_('Keine Berechtigung auf das Dokument.'))
                try:
                    self._job.sock.send(xml)
                except:
                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None

            doc = bitfarm_doc.BitfarmDocument()
            doc.profile = self._job.profile

            param = None
            if self._job.fields_arcid is not None:
                doc.fields_arcid = self._job.fields_arcid
            # if self._job.fields_arcid is None and self._job.symbolic_arcid is not None:
            #     doc.fields_arcid = self._job.symbolic_arcid

            if type(self._job.parameter) is str:
                param = self._job.parameter.split(";")
            elif type(self._job.parameter) is list:
                param = self._job.parameter

            if self._job.parameter is None or param is None:
                param = ['base', 'paths']

            if not self.bfaProcessing.load(doc, self._job.gdocid, param, logid=self.name):
                xml = bitfarm_xml.create_xml_request(reqtype="respDocError", userid=0, gdocid=self._job.gdocid)
                try:
                    self._job.sock.send(xml)
                except:
                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None

            self.log.debug(_("%s Dokument geladen [%s]") % (self.name, self._job.gdocid))
            if self._job.gdocid in self.bfaserver.in_progress[self._job.profile]:
                doc.in_progress = self.bfaserver.in_progress[self._job.profile][self._job.gdocid]

            doc.in_editmode = self.get_inedit()
            # if self.bfaserver.con.connections[self._job.profile].resultlist_showinedit: #zeige Dokumente die aktuell in Bearbeitung sind
            #     doc.in_editmode = ",".join([str(x) for x in self.bfaserver.in_progress[self._job.profile].keys()])
            # else:
            #     sql = u"select distinct(gdocid) from sys_timer where status & 4 = 0"
            #
            #     try:
            #         result = self.db[self._job.profile].execute(sql).fetchall()
            #     except:
            #         self.log.exception(u"Fehler bei Abfrage der Wiedervorlage")
            #     else:
            #         if result:
            #             doc.in_editmode = ",".join([str(x[0]) for x in result])

            if 'iev' in param:  # wir holen zu den öffentlich annotation noch die privaten, falls vorhanden
                self.add_private_annotations(doc, doc_right)

            doc.rights = doc_right.rights

            users = self.bfaserver.con.connections[self._job.profile].get_holidaylist(db=self.db[self._job.profile], user=self._job.userid, logid=self.name)
            if len(users) > 1:
                grouplist = self.bfaserver.con.connections[self._job.profile].resolve_membership(db=self.db[self._job.profile], userid=users, logid=self.name)
            else:
                grouplist = doc_right.grouplist[:]

            if self._job.resubuser is not None:
                users.append(self._job.resubuser)

            for field in list(doc.fields.values()):
                field.items = []      # Items beim Senden einezelner Dokumente rausnehmen
                access = False
                for right in field.rights:
                    if right.userid in users or right.groupid in grouplist:
                        access = True
                        break
                if not access:
                    del doc.fields[field.add]

            for field in list(doc.statefields.values()):
                access = False
                for right in field.rights:
                    if right.userid in users or right.groupid in grouplist:
                        access = True
                        break
                if not access:
                    del doc.statefields[field.name]

            if self._job.reqtype == "reqFile":
                filepath = (self._job.filename if self._job.filename else doc.docloc)
                annopath = doc.extloc
                if not annopath:
                    annopath = self.create_nrs_filepath(doc.docloc)
                annouser = self._job.annouser if self._job.annouser != 0 else self._job.userid
            #                    self._job.annodict = self.get_anno_dict(annopath, ievuser)
            elif self._job.reqtype == "reqPreview":
                filepath = doc.extloc if doc.extloc else doc.docloc
                annopath = doc.extloc
                if not annopath:
                    annopath = self.create_nrs_filepath(doc.docloc)
                annouser = self._job.annouser if self._job.annouser != 0 else self._job.userid
                #                    self._job.annodict = self.get_anno_dict(annoPath,  ievuser)
                page = self._job.page
            elif self._job.reqtype == "reqIEV":
                filepath = doc.extloc
                if not filepath:
                    filepath = self.create_nrs_filepath(doc.docloc)
                annouser = self._job.annouser if self._job.annouser != 0 else self._job.userid
                #filepath = self.get_iev_loc(filepath, self._job.page, self._job.ievtype, ievuser)
                filehash = self.get_filehash(filepath)
            elif self._job.reqtype == "reqOri":
                dontcache = True
                filepath = None

                if self._job.revision == 0:
                    filepath = doc.oriloc
                else:  # SVN
                    if 'svn' not in _con.ini or _con.ini["svn"].svnpath is None or not os.path.exists(_con.ini["svn"].svnpath):
                       self.log.error("%s Es wurde eine Dateiversion angefodert, aber die SVN-Konfiguration ist fehlerhaft." % self.name)
                       filepath = doc.oriloc
                    else:
                        do_svn = True
                        rc = 0
                        srcfile = os.path.expandvars(doc.oriloc).lower()
                        if _con.archive_rs.lower() in srcfile:   # das Dokument liegt mittlerweile revisionssicher
                            srcfile = srcfile.replace(_con.archive_rs.lower(), _con.archive_nrs.lower())    # pfad auf nrs umbiegen
                            if not os.path.exists(srcfile):  # nicht vorhanden (ggf einen neuen Pfad erhalten)
                                srcfile = doc.extloc if doc.extloc else doc.docloc
                                srcfile = os.path.expandvars(srcfile).lower()
                                srcfile = os.path.splitext(srcfile)[0] + os.path.splitext(doc.oriloc)[1]
                                if not os.path.exists(srcfile):  # nicht vorhanden, dann haben wir nichts was wir laden können
                                    self.log.error("%s Es konnte kein Ursprungspfad für die Version in NRS gefunden werden %s" % (self.name, srcfile))
                                    do_svn = False

                        if not os.path.isfile(srcfile) or not os.path.exists(srcfile):
                            self.log.error("%s Pfad auf Ursprungsdatei falsch oder nicht vorhanden %s" % (self.name, srcfile))
                            do_svn = False

                        if do_svn:
                            svnfile = os.path.join(self.bfaserver.tempdir, os.path.basename(srcfile))

                            args = 'chcp 1252\n"%s" cat -r %d "%s" > "%s"' % (_con.ini["svn"].svnpath, self._job.revision, srcfile, svnfile)
                            self.log.debug("%s svn-args %s" % (self.name, args))
                            batpath = os.path.join(self.bfaserver.tempdir, "svn_%d_%d.bat" % (self._job.gdocid, self._job.revision))
                            try:
                                batfile = open(batpath, "w")
                                batfile.write(args)
                                batfile.close()
                            except:
                                self.log.exception(_("%s Konnte svn.bat nicht schreiben %s") % (self.name, batpath))
                            # rc = subprocess.call([batpath], shell=False)
                            # try:
                            #     p = subprocess.Popen([batpath], shell=False, stdout=subprocess.PIPE, stdin=subprocess.PIPE, stderr=subprocess.PIPE)
                            #     rc = p.communicate()
                            # except:
                            #     self.log.exception(u"Fehler bei Aufruf der SVN.bat")
                            else:
                                timeout, rc, output = self._timeoutprocess([batpath])

                                if timeout:
                                    self.log.error("%s Timeout beim Aufruf von svn.bat" % self.name)

                                if rc == 0:
                                    if os.path.exists(svnfile):
                                        filepath = svnfile
                                else:
                                    self.log.error("%s Fehler beim Aufruf von svn.bat %s" % (self.name, output if output else "(keine Ausgabe erhalten"))
                                    filepath = None

                                self.log.debug(_("%s svn.exe rc = %s") % (self.name, rc))
                                try:
                                    os.remove(batpath)
                                except:
                                    self.log.exception(_("%s Konnte svn.bat nicht löschen %s") % (self.name, batpath))

                            if filepath is None:
                                xml = bitfarm_xml.create_xml_request(reqtype="respError", userid=0, gdocid=self._job.gdocid,
                                                                     datatype="text", data=_("Version %s des Dokuments konnte nicht geöffnet werden" % self._job.version))

                                self.log.warning('%s response: %s' % (self.name, xml))
                                try:
                                    self._job.sock.send(xml)
                                except:
                                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)

                if filepath:
                    if 'checkout' in param and self._job.revision == 0:
                        self.bfaProcessing.checkout(doc, self._job.userid)

                    if doc.is_checkout and doc.checkout_userid == self._job.userid and self._job.revision == 0:
                        # ausgecheckte Dokumente werden in %bitfarm-archiv%\checkout ausgelagert
                        folder = os.path.expandvars('%bitfarm-archiv%\checkout\\' + str(self._job.gdocid))
                        if not os.path.exists(folder):
                            try:
                                os.makedirs(folder)
                            except:
                                self.log.exception(_("%s Konnte Verzeichnis %s nicht anlegen") % (self.name, folder))
                                filepath = None

                        fname = doc.filename if doc.filename else doc.title if doc.title else _('Dokument[%s]' % self._job.gdocid)
                        fname += os.path.splitext(filepath)[1]
                        fname = bitfarm_doc.parse_filename(fname)
                        dstfile = os.path.join(folder, fname)
                        if os.path.exists(dstfile):  # die ausgecheckte Datei ist schon vorhanden?
                            filepathstats = os.stat(os.path.expandvars(filepath))
                            dstfilestats = os.stat(dstfile)
                            if filepathstats.st_mtime > dstfilestats.st_mtime:  # ist die Datei in der Archivablage aktueller?
                                try:
                                    shutil.copyfile(os.path.expandvars(filepath), dstfile)
                                except:
                                    self.log.exception(_("%s Konnte Datei %s  nicht ins Verzeichnis %s nicht kopieren") % (self.name, filepath, folder))
                                    filepath = None
                                else:
                                    filepath = dstfile
                            else:
                                filepath = dstfile
                        else:
                            try:
                                shutil.copyfile(os.path.expandvars(filepath), dstfile)
                            except:
                                self.log.exception(_("%s Konnte Datei %s  nicht ins Verzeichnis %s nicht kopieren") % (self.name, filepath, folder))
                                filepath = None
                            else:
                                filepath = dstfile

            if filepath:
                s = time.time()
                if not self.send_file_to_client(doc, filepath, filehash, dontcache=dontcache):
                    self.log.debug(_("%s Senden fehlgeschlagen oder abgebrochen") % self.name)
                else:
                    self.check_logread(doc_right)
                    self.log.debug(_("%s Daten wurden erfolgreich versendet in %f") % (self.name, time.time() - s))
                    if svnfile:
                        try:
                            os.remove(svnfile)
                        except:
                            self.log.exception(_("%s Konnte SVN Datei nicht löschen %s") % (self.name, svnfile))
            else:
                xml = bitfarm_xml.create_xml_request(reqtype="respError", data=doc, datatype='BitfarmDocument',
                                                 errortext=_("Keinen Dateipfad gefunden"))

                self.log.warning('%s response: %s' % (self.name, xml))
                try:
                    self._job.sock.send(xml)
                except:
                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)

            self.log.debug(_("%s Request fertig verarbeitet") % self.name)

    def send_file_to_client(self, doc, filepath, filehash=None, dontcache=False):
        self.log.debug(_("%s Starte Übertragung von Binärdaten (%s)") % (self.name, filepath))

        filedate = None
        datalen = None
        anno_datalen = 0
        annos = []

        if os.path.exists(os.path.expandvars(filepath)):
            self.log.debug(_("%s Sende Datei komplett") % self.name)
            starttime = time.time()
            try:
                f = open(os.path.expandvars(filepath), 'rb')
            except:
                self.log.exception(_("%s Fehler beim Lesen der Datei %s") % (self.name, filepath))
                xml = bitfarm_xml.create_xml_request(reqtype="respDocError", datatype="text",
                                                     data=_("Fehler beim Lesen der Datei"))
                try:
                    self._job.sock.send(xml)
                except:
                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                return None

            stats = os.fstat(f.fileno())
            datalen = stats.st_size
            filedate = datetime.datetime.fromtimestamp(stats.st_mtime).replace(microsecond=0)
            if datalen == 0:
                floc = doc.extloc if doc.extloc else doc.docloc
                if filepath == floc and os.path.splitext(filepath)[1].lower() == '.tif' and os.path.exists(os.path.join(self.bfaserver.bfapath, 'empty.tif')):
                    filepath = os.path.join(self.bfaserver.bfapath, 'empty.tif')
                    f.close()
                    f = open(os.path.expandvars(filepath), 'rb')
                    stats = os.fstat(f.fileno())
                    datalen = stats.st_size
                else:
                    self.log.warning(_("%s Datei ist leer (0Kb) %s") % (self.name, filepath))
                    xml = bitfarm_xml.create_xml_request(reqtype="respDocError", data=doc,
                                                     datatype='BitfarmDocument', errortext=_("Datei ist leer (0Kb) %s") % filepath)
                    try:
                        self._job.sock.send(xml)
                    except:
                        self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                    return None

            if self._job.filedate is not None:
                if filedate >= self._job.filedate:  # scheint aktuell zu sein
                    self.log.warning(_("%s Datei ist aktuell %s") % (self.name, filepath))
                    xml = bitfarm_xml.create_xml_request(reqtype="respIsLatest")
                    try:
                        self._job.sock.send(xml)
                    except:
                        self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                    return None

            if self._job.filehash is not None:
                filehash = self.get_filehash(filepath)
                if filehash == self._job.filehash:  # scheint gleich zu sein
                    self.log.warning(_("%s Datei ist aktuell %s") % (self.name, filepath))
                    xml = bitfarm_xml.create_xml_request(reqtype="respIsLatest")
                    try:
                        self._job.sock.send(xml)
                    except:
                        self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                    return None

            readcache = False
            streamed = 0
            response_send = False

            if (self._job.gdocid, filepath) in self.bfaserver.cached:
                readcache = True
            # readcache = False
            if readcache:
                self.log.debug(_("%s Datei im Cache, nun versende Daten") % self.name)
                try:
                    cachedstream = self.bfaserver.cache[(self._job.gdocid, filepath)]
                except:
                    self.log.exception(_("%s Datei nicht mehr im Cache") % self.name)
                    readcache = False

            if readcache:
                try:
                    cachedstream.seek(0, 2)
                    csize = cachedstream.tell()
                    cachedstream.seek(0)
                except:
                    self.log.exception(_("%s Konnte nicht die Größe des Streams lesen") % self.name)
                    readcache = False
                else:
                    if csize != datalen:    #gecachte Datei hat unterschiedliche größe
                        self.log.debug(_("Die Datei hat eine andere Größe als der Stream, lade neu %s <> %s") % (datalen, csize))
                        readcache = False
                        BitfarmWorkerThread.cachelock.acquire()
                        try:
                            self.bfaserver.cache[(self._job.gdocid, filepath)].close()
                            del self.bfaserver.cache[(self._job.gdocid, filepath)]
                            self.bfaserver.cached.remove((self._job.gdocid, filepath))
                        finally:
                            BitfarmWorkerThread.cachelock.release()
                    else:
                        datalen = csize

            if readcache:
                xml = bitfarm_xml.create_xml_request(reqtype="respDoc", gdocid=self._job.gdocid, datalen=datalen,
                                                     filehash=filehash, filename=os.path.basename(filepath), data=doc,
                                                     datatype='BitfarmDocument')
                if self.bfaserver.develop:
                    if self._job.gdocid is not None:
                        with open('%i.xml' % self._job.gdocid, 'wb') as fn:
                            fn.write(xml)

                try:
                    self._job.sock.send(xml)
                except:
                    self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                    return None
                finally:
                    response_send = True
                self.log.debug(_("%s respDoc gesendet, versende nun Daten (Cache)") % self.name)
                if self.bfaserver.develop:
                    self.log.debug("%s %s" % (self.name, xml))
                while streamed < datalen:
                    BitfarmWorkerThread.cachelock.acquire()  # seek und read sollte atomic sein
                    try:
                        cachedstream.seek(streamed)
                        buff = cachedstream.read(self.bfaserver.split_size)
                    except ValueError:  # wurde der Stream aus dem Cache entfernt?
                        self.log.warning(_("%s Konnte nicht vom Stream lesen") % self.name)
                        readcache = False
                        break
                    finally:
                        BitfarmWorkerThread.cachelock.release()
                    try:
                        self._job.sock.send(buff)
                    except socket.timeout:
                        self.log.error(_("%s Socket-timeout") % self.name)
                        cachedstream = None
                        return None
                    except:
                        self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                        cachedstream = None
                        return None
                    # streamed += stream.tell()
                    streamed += len(buff)
                cachedstream = None
                if not readcache:
                    self.log.warning(_("%s Lesen aus dem Cache wurde abgebrochen, lese weiter von der Disk, streamed = %i") % (self.name, streamed))
                else:
                    self.log.debug("%s streamed %i" % (self.name, streamed))

            if not readcache:
                if not response_send:
                    xml = bitfarm_xml.create_xml_request(reqtype="respDoc", gdocid=self._job.gdocid, datalen=datalen,
                                                         filehash=filehash, filedate=filedate,
                                                         filename=os.path.basename(filepath), data=doc,
                                                         datatype='BitfarmDocument')
                    if self.bfaserver.develop:
                        if self._job.gdocid is not None:
                            with open('%i.xml' % self._job.gdocid, 'wb') as fn:
                                fn.write(xml)

                    try:
                        self._job.sock.send(xml)
                    except:
                        self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                        return None
                    self.log.debug(_("%s respDoc gesendet, versende nun Daten (Disk)") % self.name)
                    if self.bfaserver.develop:
                        self.log.debug("%s %s" % (self.name, xml))
                while streamed < datalen:
                    # stream.write(f.read(split_size))
                    try:
                        f.seek(streamed)
                        buff = f.read(self.bfaserver.split_size)
                    except:  # wurde der Stream aus dem Cache entfernt?
                        self.log.exception(_("%s Konnte nicht von der Datei lesen lesen") % self.name)
                        readcache = False
                        break

                    try:
                        self._job.sock.send(buff)
                    except socket.timeout:
                        self.log.error(_("%s Socket-timeout") % self.name)
                        f.close()
                        return None
                    except:
                        self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                        f.close()
                        return None
                    # streamed += stream.tell()
                    streamed += len(buff)
                self.log.debug("%s streamed %i" % (self.name, streamed))
                # stream.seek(0, 2)
                # stream.flush()

                if datalen < self.bfaserver.cache_maxlen and not dontcache:  # dateien unter cache_maxlen größe kommen in den cache
                    if (self._job.gdocid, filepath) not in self.bfaserver.cached:
                        self.log.debug(_("%s Daten kleiner als cache_maxsize, speichere im Cache") % self.name)
                        stream = io.BytesIO()
                        stream.seek(0)
                        f.seek(0)
                        while stream.tell() < datalen:
                            try:
                                stream.write(f.read(self.bfaserver.split_size))
                                #                            self.log.debug('stream.tell %i' %stream.tell())
                            except:
                                self.log.exception(_("Fehler beim Schreiben in den Cache %s") % self.name)
                                stream.close()
                                return None

                        BitfarmWorkerThread.cachelock.acquire()
                        try:
                            if len(self.bfaserver.cached) >= self.bfaserver.cache_size:
                                elem = self.bfaserver.cached.popleft()
                                self.log.debug(_("%s Entferne Datei aus dem Cache %i") % (self.name, elem[0]))
                                self.bfaserver.cache[elem].close()
                                del self.bfaserver.cache[elem]

                            self.bfaserver.cache[(self._job.gdocid, filepath)] = stream
                            self.bfaserver.cached.append((self._job.gdocid, filepath))
                        finally:
                            BitfarmWorkerThread.cachelock.release()
                    else:
                        self.log.debug(_("%s Datei scheint schon im Cache zu sein!") % self.name)
            f.close()

            return True
        else:
            self.log.debug(
                _('%s Der Server kann die Datei nicht finden [%s] %s') % (self.name, self._job.gdocid, filepath))
            xml = bitfarm_xml.create_xml_request(reqtype="respError", data=doc, datatype='BitfarmDocument',
                                                 errortext=_("Der Server kann die Datei nicht finden!"))
            try:
                self._job.sock.send(xml)
            except:
                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
            return None


class BitfarmSocketConnector(threading.Thread):
    """Diese Klasse stellt dem Server einen (oder zukünftig auch mehrere) Socket zur Verfügung,
    über den der Server erreicht und die Request entgegennehmen kann"""
    queue = queue.Queue(0)
    log = logging.getLogger("Main.BitfarmSocket")
    authlock = threading.Lock()

    def __init__(self, bfaserver, port):
        threading.Thread.__init__(self)
        self.stopsignal = False
        self.bfaserver = bfaserver
        self.server = None
        if type(port) in [str, str]:
            self.port = int(port)
        else:
            self.port = port
        self.clients = []
        self.reqclients = []
        # self.ssl = False
        certfile = os.path.join(self.bfaserver.mypath, 'certs','bfaServer36.crt')
        keyfile = os.path.join(self.bfaserver.mypath, 'certs','bfaServer36.key')
        if not os.path.exists(certfile) or not os.path.exists(keyfile):
            self.log.warning('Zertifikate nicht gefunden, Server arbeitet ohne SSL-Verschlüsselung')
            self.bfaserver.nossl = True
        else:
            self.context = ssl.SSLContext(ssl.PROTOCOL_TLS_SERVER)
            self.context.load_cert_chain(certfile, keyfile)
            # self.context.load_cert_chain(r'C:\OpenSSL\bin\local.crt', r'C:\OpenSSL\bin\local.key')
            self.context.load_default_certs(purpose=ssl.Purpose.CLIENT_AUTH)
            # self.context = ssl.create_default_context(purpose=ssl.Purpose.CLIENT_AUTH)
            # self.context = ssl.create_default_context()
            # self.context.verify_mode = ssl.CERT_REQUIRED
        self.running = False
        self.starting = True
        self.timeout = 0
        #        self.db = {}
        #        for con in self.bfaserver.con.connections.values():
        #            self.db[con.profile] = con.engine.connect()
        self.log.debug(_('gestartet %s') % self.name)

    def open_socket(self):
        self.timeout = time.time() + 20  # 10 sec Timeout bei starten des Sockets
        try:
            self.server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            if self.bfaserver.use_servername:
                self.server.bind((self.bfaserver.server, self.port))
                self.log.info('Bitfarm-Server Socket erstellt %s:%s' % (self.bfaserver.server, self.port))
            else:
                # self.server.bind((socket.gethostname(), self.port))
                self.server.bind(('0.0.0.0', self.port))
                self.log.info('Bitfarm-Server Socket erstellt 0.0.0.0:%s' % self.port)

            self.server.listen(10)
            self.server.setblocking(1)
        except socket.error as message:
            if self.server:
                self.server.close()
            self.log.critical(_("Der Server konnte den Socket nicht öffnen: %s") % message.strerror)
            self.running = False
        except:
            if self.server:
                self.server.close()
            self.log.critical(_("Der Server konnte den Socket nicht öffnen: unbekannter Fehler"))
            self.running = False
        else:
            self.running = True
            self.log.debug(_("Server Socket geöffnet %s") % str(self.server.getsockname()))
        self.starting = False
        return self.running

    def run(self):
        pythoncom.CoInitialize()
        i = 0
        if not self.open_socket():
            return None
        #        while not self.stopsignal:
        self.timeout = 0
        readsockets = []
        writesockets = []
        errorsockets = []

        while self.is_alive():
            #            self.log.debug('running %s' %self.name)
            client = None
            sock = None
            try:
                try:
                    readsockets, writesockets, errorsockets = select.select([self.server] + self.clients,
                                                                            self.reqclients, [], 0.5)
                except ValueError:
                    self.log.error("%s Too many open file descriptors (>512)" % self.name)
                    readsockets = []
                except:
                    self.log.exception(_("%s !!! Fehler beim Lesen von einem Client-Socket") % self.name)
                    readsockets = []

                if not readsockets:     #gerade nichts zu tun
                    if self.stopsignal:
                        # wir haben alles abgearbeitet und sollen aufhören?
                        self.log.info(_("%s Stopsignal erhalten, keine eingehenden Sockets, beende Thread") % self.name)
                        break

                    for profile in self.bfaserver.authuser:
                        if self.bfaserver.con[profile].auto_reconnect:
                            BitfarmLoginThread.authlock.acquire()
                            try:
                                for userkey in self.bfaserver.authuser[profile]:

                                # if len(self.bfaserver.authuser[profile][userkey]) == 0:     # keine Socketverbindung mehr für diesen User
                                    if userkey in self.bfaserver.active_user[profile] and self.bfaserver.active_user[profile][userkey] is not None:
                                        if self.bfaserver.active_user[profile][userkey] + datetime.timedelta(seconds=self.bfaserver.con.connections[profile].user_timeout) < datetime.datetime.now(): # Timeout!
                                            self.log.debug('%s Timeout der Verbindung des User %i' % (self.name, userkey[0]))
                                            self.bfaserver.active_user[profile][userkey] = None
                                            self.bfaserver.bfaLogin.queue.put(('remove_userkey', (profile, userkey)))
                                            break
                            finally:
                                BitfarmLoginThread.authlock.release()

                for sock in readsockets:
                    if sock is self.server:
                        try:
                            if not self.bfaserver.nossl:
                                client_nossl, addr = self.server.accept()
                                # client = self.ssl.wrap_socket(client_nossl, certfile=self.certfile, keyfile=self.keyfile,
                                #                          server_side=True)
                                self.context.check_hostname = False
                                try:
                                    client = self.context.wrap_socket(client_nossl, server_side=True)
                                except:
                                    self.log.error('Fehler beim Herstellen einer SSL Verbindung (Client Addr %s) %s' % (addr[0], str(sys.exc_info()[1])))
                                    del client_nossl
                                else:
                                    if len(self.clients) > 500:
                                        self.log.warning(_("%s Zuviele Client-Socket Connections %s mit <%s>") % (self.name, addr[0], hex(id(client))))
                                        client.close()
                                    else:
                                        self.clients.append(client)
                                        self.log.info(_("%s +++ Client %s verbunden (SSL) mit <%s>") % (self.name, addr[0], hex(id(client))))
                            else:
                                client, addr = self.server.accept()
                                if len(self.clients) > 500:
                                    self.log.warning(_("%s Zuviele Client-Socket Connections %s mit <%s>") % (self.name, addr[0], hex(id(client))))
                                    client.close()
                                else:
                                    self.clients.append(client)
                                    self.log.info(_("%s +++ Client %s verbunden mit <%s>") % (self.name, addr[0], hex(id(client))))
                        except:
                            self.log.exception(_("%s !!! Fehler bei der Herstellung einer Client-Socket Verbindung <%s>") % (self.name, hex(id(sock))))
                            if sock:
                                # sock.close()
                                if client in self.clients:
                                    self.clients.remove(client)
                    else:
                        message = ''
                        try:
                            while message[-9:].lower() != '</bfareq>':
                                buff = sock.recv(1024).decode()
                                if buff == '':
                                    break
                                message += buff
                        except:
                            message = ''

                        ip = sock.getpeername()[0]
                        if message:
                            if (message[0:7].lower() == '<bfareq>' or
                                            message[0:49].lower() == '<?xml version="1.0" encoding="us-ascii"?><bfareq>' or
                                            message[0:49].lower() == '<?xml version=\'1.0\' encoding=\'us-ascii\'?><bfareq>') and \
                                            message[-9:].lower() == '</bfareq>':
                                self.log.info(_("%s Request von <%s> empfangen") % (self.name, hex(id(sock))))

                                req = bitfarm_xml.xml_to_object(message)
                                userkey = (req.userid, req.sessionid)
                                if self.bfaserver.develop:
                                    self.log.info(str(userkey))

                                del req.sessionid
                                if message.find('<sessionid') > -1:
                                    message = message[:message.find('<sessionid')] + message[message.find('/sessionid>') + 11:]

                                if not isinstance(req, bitfarm_doc.bfaReq):
                                    self.log.warning(_("%s Ankommende Nachricht kein bitfarm Request Object! %s") % (self.name, message))

                                if req.reqtype == "reqAuthenticate" and not self.bfaserver.develop:
                                    self.log.debug("%s reqAuthenticate %s %s" % (self.name, req.profile, req.parameter))
                                elif req.reqtype in ['reqADTree', 'reqADUser'] and not self.bfaserver.develop:
                                    self.log.debug("%s %s %s %s" % (self.name, req.reqtype, req.profile, req.parameter.split(';')[0]))
                                else:
                                    self.log.debug("%s <%s> %s" % (self.name, hex(id(sock)), message))

                                if req.profile is None:
                                    req.profile = self.bfaserver.con.main.profile

                                req.profile = req.profile.lower()
                                if req.profile not in self.bfaserver.con.connections:
                                    xml = bitfarm_xml.create_xml_request(reqtype="respAuthError", data=_("Anmeldung mit dem Profil %s an diesem Server nicht möglich") % req.profile)
                                    try:
                                        sock.send(xml)
                                    except:
                                        self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)

                                elif req.reqtype in ["reqCalculateExpiredDate", "reqCheckUpdates", "reqSendUpdates", "reqGetOCRText"]:  # Request von dbimp oder Updater. wird durchgereicht...
                                    req.sock = sock
                                    self.bfaserver.requestqueue.put(req)
                                elif req.reqtype == "reqDisconnectUser":
                                    if userkey in self.bfaserver.active_user[req.profile]:
                                        self.bfaserver.active_user[req.profile][userkey] = None
                                    self.bfaserver.bfaLogin.queue.put(('remove_userkey', (req.profile, userkey)))
                                    self.bfaserver.bfaLogin.queue.put(('clear_authlist', sock))

                                    if sock in self.clients:
                                        self.clients.remove(sock)
                                elif req.reqtype == "reqHeartBeat":
                                    if userkey in self.bfaserver.active_user[req.profile]:
                                        self.bfaserver.active_user[req.profile][userkey] = datetime.datetime.now()
                                elif req.reqtype == "reqQueueCount":
                                    if userkey in self.bfaserver.active_user[req.profile]:
                                        self.bfaserver.active_user[req.profile][userkey] = datetime.datetime.now()
                                    data = ["%s: %s" % (x[0], x[1]) for x in sorted(self.bfaserver.queue_counts[req.profile].items())]
                                    data = ";".join(data)
                                    xml = bitfarm_xml.create_xml_request(reqtype="respOK", data=data)
                                    try:
                                        sock.send(xml)
                                    except:
                                        self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                                elif req.reqtype == "reqAuthenticate":
                                    if userkey not in self.bfaserver.authuser[req.profile]:
                                        try:
                                            self.log.debug(_("%s Profil: %s Anzahl named Lizenzen:%d/%d") % (self.name, req.profile, self.bfaserver.cnt_named[req.profile],
                                                                                                    self.bfaserver.con.connections[req.profile].named))
                                            if self.bfaserver.con.connections[req.profile].concurrent > 0:
                                                self.log.debug(_("%s Profil: %s Anzahl concurrent Lizenzen:%d/%d") % (self.name, req.profile, self.bfaserver.cnt_concurrent[req.profile],
                                                                                                    self.bfaserver.con.connections[req.profile].concurrent))

                                            # if self.bfaserver.cnt_named[req.profile] > self.bfaserver.con.connections[req.profile].named > -1 and \
                                            #    self.bfaserver.cnt_concurrent[req.profile] > self.bfaserver.con.connections[req.profile].concurrent > -1:
                                            #     xml = bitfarm_xml.create_xml_request(reqtype="respAuthError", data=_(u"Die Anzahl der maximalen Benutzer ist erreicht"))
                                            #     try:
                                            #         sock.send(xml)
                                            #     except:
                                            #         self.log.warning(_(u"%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                                            # else:
                                            #     # login funktion sendet response selbst
                                            self.bfaserver.bfaLogin.queue.put(('login', (userkey, req.parameter, req.data, sock, req.profile, req.lang)))
                                        except:
                                            self.log.exception(_("%s Allgemeiner Fehler bei der Anmeldung.") % self.name)
                                            xml = bitfarm_xml.create_xml_request(reqtype="respAuthError", data=_("Allgemeiner Fehler bei der Anmeldung."))
                                            try:
                                                sock.send(xml)
                                            except:
                                                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)

                                    elif userkey in self.bfaserver.authuser[req.profile]:
                                        if userkey in self.bfaserver.active_user[req.profile]:
                                            self.bfaserver.active_user[req.profile][userkey] = datetime.datetime.now()

                                        self.log.debug(_('Neuanmeldung von %s') % str(userkey))
                                        if sock not in self.bfaserver.authuser[req.profile][userkey]:
                                            # BitfarmSocketConnector.authlock.acquire()
                                            # try:
                                            #     # if sock in self.bfaserver.authuser[req.profile][userkey]:
                                            #     #     self.bfaserver.authuser[req.profile][userkey].remove(sock)
                                            #     self.bfaserver.authuser[req.profile][userkey].append(sock)
                                            # finally:
                                            #     BitfarmSocketConnector.authlock.release()
                                            #
                                            #
                                            # self.log.debug(_(u'%s Verbindung hinzugefügt %s %s') % (self.name, str(userkey), str(self.bfaserver.authuser[req.profile][userkey])))
                                            # self.log.debug(_(u"%s %d client-sockets offen") % (self.name, len(self.clients)))
                                            self.bfaserver.bfaLogin.queue.put(('login', (userkey, req.parameter, req.data, sock, req.profile, req.lang)))
                                        else:
                                            xml = bitfarm_xml.create_xml_request(reqtype="respAuth", data=_("Neuanmeldung erfolgreich"), datatype="text", userid=userkey[0])
                                            try:
                                                sock.send(xml)
                                            except:
                                                self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)


                                # in diesem Zweig werden Request authentifizierter Clients verarbeitet
                                elif userkey in self.bfaserver.authuser[req.profile]:
                                    if userkey in self.bfaserver.active_user[req.profile]:
                                        self.bfaserver.active_user[req.profile][userkey] = datetime.datetime.now()
                                    self.log.debug(_("%s Request an Server-Mainloop übergeben") % self.name)
                                    req.sock = sock
                                    self.bfaserver.requestqueue.put(req)
                                    if req.reqtype in ['reqSavePreview', 'reqSyncDoc', 'reqCreateDocument',
                                                       'reqArchive', 'reqPredictClass']:   # bei Sendrequests müssen wir den Socket aus self.clients entfernen, sonst
                                        self.clients.remove(sock)       # blockiert die Kommunikation über diesen Socket den Multiplexer
                                        self.bfaserver.bfaLogin.queue.put(('clear_authlist', sock))  # der Socket wird auch aus der Userliste entfernt
                                else:
                                    # request eines nicht authentifizierten Viewers
                                    xml = bitfarm_xml.create_xml_request(reqtype="respAuthError", data=_(
                                        "Userid %s nicht an Profil %s authentifiziert" % (userkey[0], req.profile)),
                                                                         datatype="text")
                                    try:
                                        sock.send(xml)
                                    except:
                                        self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)
                            else:
                                self.log.warning(_("%s verwerfe unbekannten Request %s auf %s") % (self.name, message, sock))
                                self.log.debug(str(message[0:7].lower() == '<bfareq>'))
                                self.log.debug(str(message[0:49].lower() == '<?xml version="1.0" encoding="us-ascii"?><bfareq>'))
                                self.log.debug(str(message[0:49].lower() == '<?xml version=\'1.0\' encoding=\'us-ascii\'?><bfareq>'))
                                self.log.debug(str(message[-9:].lower() == '</bfareq>'))
                                with open('unknown_%s.dat' % datetime.datetime.now().strftime('%Y%m%d%H%M%S%f'), 'wb') as f:
                                    f.write(message.encode())

                        else:
                            self.log.info(_("%s +++ Verbindung zu %s auf <%s> beendet") % (self.name, ip, hex(id(sock))))
                            if self.bfaserver.nossl:
                                self.log.info(_("%s (Da der Server ohne SSL Verschlüsselung läuft, kann es sich auch um einen SSL-Client gehandelt haben <%s>)") % (self.name, hex(id(sock))))
                            self.bfaserver.bfaLogin.queue.put(('clear_authlist', sock))

                            if sock in self.clients:
                                self.clients.remove(sock)

            except Exception as e:
                self.log.exception(_("%s Allgemeiner Fehler im SocketConnector-Mainloop: %s") % (self.name, str(e)))
                self.log.debug(_("%s traceback: %s") % (self.name, traceback.format_exc()))
                if sock is not None:
                    xml = bitfarm_xml.create_xml_request(reqtype="respError", data=_("Allgemeiner Fehler im SocketConnector-Mainloop."))
                    try:
                        sock.send(xml)
                    except:
                        self.log.warning(_("%s Client scheint weg zu sein, Senden wird abgebrochen") % self.name)

        self.log.debug(_('%s stopped') % self.name)

    def shutdown(self):
        self.stopsignal = True
        # TODO: überlegen ob ein Nachricht an alle Clients gesendet werden soll


class BitfarmQueueCheck(threading.Thread):
    """Eine Hilfsthread der periodisch die Auslastung der Queue(s) überprüft und für der reqQueueCount den Clients zur Verfügung stellt."""
    log = logging.getLogger("Main.BitfarmQueueCheck")

    def __init__(self, bfaserver):
        threading.Thread.__init__(self)
        self.stopsignal = False
        self.bfaserver = bfaserver

    def run(self):
        round = 19  # erste Abfrage nach 2 sec jede weitere nach 20 sec ;-)
        while self.is_alive():
        # while self.is_alive():
            if self.stopsignal:
                break

            time.sleep(1)

            round += 1
            if round > 20:
                round = 0
                for profile in self.bfaserver.con.profiles:
                    queues = [('Hauptserver', os.path.join(self.bfaserver.bfapath, 'queue'))]
                    if 'multiqueue' in self.bfaserver.con[profile].ini:
                        for key in self.bfaserver.con[profile].ini['multiqueue']:
                            if 'multiqueueserverpath' in key:
                                if self.bfaserver.con[profile].ini['multiqueue'][key]:
                                    host = 'Queueserver ' + key[-2:]
                                    queues.append((host, os.path.join(self.bfaserver.con[profile].ini['multiqueue'][key], 'queue')))

                    for queue in queues:
                        # self.log.debug('Prüfe Queue %s %s' % (queue[0], queue[1]))
                        if os.path.exists(queue[1]):
                            try:
                                queue_job = 0
                                queue_cng = 0
                                queue_job = sum(1 for f in glob.iglob(os.path.join(queue[1], "*.job")))
                                queue_cng = sum(1 for f in glob.iglob(os.path.join(queue[1], "*.cng")))
                                queue_cnt = queue_job + queue_cng
                            except:
                                self.log.exception("Konnte Anzahl der Jobs in der Queue %s nicht ermitteln" % queue[1])
                            else:
                                self.bfaserver.queue_counts[profile][queue[0]] = queue_cnt
                                # if queue_cnt > 0:
                                #     self.log.debug('Anzahl Jobs: %d' % queue_cnt)
                        else:
                            self.log.error('Pfad nicht erreichbar %s' % queue[1])
        self.log.debug(_('%s stopped') % self.name)

    def shutdown(self):
        self.stopsignal = True


class BitfarmDLMCheck(threading.Thread):
    """Eine Hilfsthread der periodisch einmal am Tag die Konfiguration des DLM überprüft und
    fällige Dokumente löscht."""
    log = logging.getLogger("Main.BitfarmDLMCheck")

    def __init__(self, bfaserver):
        threading.Thread.__init__(self)
        self.queue = queue.Queue(0)
        self.stopsignal = False
        self.bfaserver = bfaserver
        self.checked = {}          # wurde heute schon gecheckt?
        self.maintained = {}          # wurde heute schon gemaintained?
        self.timer = None
        self.db = {}
        for con in list(self.bfaserver.con.connections.values()):
            self.db[con.profile] = con.engine.connect()
            self.checked[con.profile] = False
            self.maintained[con.profile] = False
        self.bfaProcessing = bitfarm_proc.BitfarmDocProcessing(self.bfaserver.con, _con_db=self.db)


    # def reconnect(self, _profile):
    #     self.log.debug(u"%s reconnect" % self.name)
    #     _ret = False
    #
    #     if self.bfaserver.con[_profile].engine is not None:
    #         try:
    #             self.db[_profile] = self.bfaserver.con[_profile].engine.connect().execution_options(autocommit=True)
    #         except:
    #             self.log.error(_(u"%s Reconnect gescheitert") % self.name)
    #         else:
    #             if not isinstance(self.db[_profile], sqla.engine.base.Connection):
    #                 self.log.error(_(u"%s Reconnect gescheitert") % self.name)
    #             else:
    #                 self.log.warning(_(u"%s Reconnect erfolgreich db = %s") % (self.name, self.db[_profile]))
    #                 _ret = True
    #
    #     return _ret

    def maintain_db(self, profile):
        self.maintained[profile] = True
        self.log.debug("%s maintain_db %s" % (self.name, profile))
        _con = self.bfaserver.con.connections[profile]

        # tmp_fullsql klein halten
        sql = 'delete from tmp_fullsql where tstamp < now() - interval 3 day'
        try:
            self.db[profile].execute(sql)
        except:
            self.log.exception("%s Fehler beim Bereinigen der tmp_fullsql" % self.name)
        else:
            self.log.info("%s tmp_fullsql bereinigt" % self.name)

        # DLM Konfiguration auf ungültige Datumsfelder überprüfen
        if _con.sys_doclifecycle is not None:
            sql = 'select * from sys_doclifecycle where date < "1900-01-01"'
            try:
                result = self.db[profile].execute(sqla.text(sql)).fetchall()
            except:
                self.log.exception("%s Fehler beim Prüfen der sys_doclifecycle" % self.name)
            else:
                if result:
                    self.log.warning("%s DB Maintain: Fehlerhafte DLM-Konfiguration gefunden (Datum < 1900)" % self.name)
                    ids = []
                    for row in result:
                        ids.append(str(row['id']))
                    if ids:
                        sql = 'update sys_doclifecycle set date = Null where id in (%s)' % ",".join(ids)
                        try:
                            self.db[profile].execute(sqla.text(sql))
                        except:
                            self.log.exception("%s Fehler beim Korrigieren der sys_doclifecycle" % self.name)
                        else:
                            self.log.info("%s Fehler der Datumsfelderin der sys_doclifecycle korrigiert. ID [%s]" % (self.name,",".join(ids)))

        # Addfield Konfiguration auf Zahlen, Datums und Intervalfelder prüfen, die nicht Suchmethode exakt haben
        sql = 'select * from sys_addfields where type in (1,5,6,7,8,12) and mode <> 0'
        try:
            result = self.db[profile].execute(sqla.text(sql)).fetchall()
        except:
            self.log.exception("%s Fehler beim Prüfen der sys_addfields" % self.name)
        else:
            if result:
                self.log.warning("%s DB Maintain: Fehlerhafte Addfield-Konfiguration gefunden (Suchmethode nicht 'exakt' bei Datums,Interval oder Zahlenfelder)" % self.name)
                ids = []
                for row in result:
                    ids.append(str(row['addfieldid']))
                if ids:
                    sql = 'update sys_addfields set mode = 0 where addfieldid in (%s)' % ",".join(ids)
                    try:
                        self.db[profile].execute(sqla.text(sql))
                    except:
                        self.log.exception("%s Fehler beim Korrigieren der sys_addfields" % self.name)
                    else:
                        self.log.info("%s Fehler der Datumsfelderin der sys_addfields korrigiert. ID [%s]" % (self.name, ",".join(ids)))

        # Fertig gesetzte Wiedervorlagen entfernen
        sql = 'select * from sys_timer where status & 4 = 4 and `date` < (now() - interval %d day)' % self.bfaserver.keep_resubs
        try:
            result = self.db[profile].execute(sqla.text(sql)).fetchall()
        except:
            self.log.exception("%s Fehler beim Prüfen der sys_timer" % self.name)
        else:
            if result:
                self.log.info('%s Es wurden %s Wiedervorlagen gefunden, die älter als %d Tage sind.' % (self.name, len(result), self.bfaserver.keep_resubs))

                sql = 'delete from sys_timer where status  & 4 = 4 and date < (now() - interval %d day)' % self.bfaserver.keep_resubs
                try:
                    self.db[profile].execute(sql)
                except:
                    self.log.exception("%s Fehler beim Bereinigen der sys_timer" % self.name)
                else:
                    self.log.info("%s Wiedervorlagenliste bereinigt." % self.name)

        # Rechte auf private Annos aus sys_arcxs entfernen falls vorhanden (apread, apwrite und apmodify wird nur bei Freigabe benötigt und über sys_docxs geregelt)
        sql = 'select * from sys_arcxs where rights & 2121728 > 0'
        try:
            result = self.db[profile].execute(sqla.text(sql)).fetchall()
        except:
            self.log.exception("%s Fehler beim Prüfen der sys_arcxs" % self.name)
        else:
            if result:
                self.log.info('%s Private Anno Berechtigung aus sys_arcxs entfernen.' % self.name)

                sql = 'update sys_arcxs set rights = (rights & ~ 2121728) where rights & 2121728 > 0'
                try:
                    self.db[profile].execute(sql)
                except:
                    self.log.exception("%s Fehler beim Bereinigen der sys_arcxs" % self.name)
                else:
                    self.log.info("%s sys_arcxs bereinigt." % self.name)

        sql = 'select * from sys_user where role & 2121728 > 0'
        try:
            result = self.db[profile].execute(sqla.text(sql)).fetchall()
        except:
            self.log.exception("%s Fehler beim Prüfen der sys_user" % self.name)
        else:
            if result:
                self.log.info('%s Private Anno Berechtigung aus sys_user.role entfernen.' % self.name)

                sql = 'update sys_user set role = (role & ~ 2121728) where role & 2121728 > 0'
                try:
                    self.db[profile].execute(sql)
                except:
                    self.log.exception("%s Fehler beim Bereinigen der sys_user.role" % self.name)
                else:
                    self.log.info("%s sys_user.role bereinigt." % self.name)

        # Duplizierte Dokumente, die den gleichen Pfad haben korrigieren
        sql = 'select * from sys_history where taskid = 239'
        try:
            result = self.db[profile].execute(sqla.text(sql)).fetchall()
        except:
            self.log.exception("%s Fehler beim Prüfen der sys_history" % self.name)
        else:
            if result:
                for row in result:
                    dst_gdocid = int(row['gdocid'])
                    src_gdocid = int(row['value1'])
                    # wir prüfen ob das Zieldokument immer noch dieselben Pfade hat
                    sql = 'select * from sys_doc where gdocid = %d' % dst_gdocid
                    try:
                        dst_doc = self.db[profile].execute(sqla.text(sql)).fetchone()
                    except:
                        self.log.exception("%s Fehler beim Prüfen der sys_history" % self.name)
                    else:
                        if dst_doc:
                            dst_docloc = dst_doc['docloc']
                            dst_extloc = dst_doc['extloc']
                            dst_oriloc = dst_doc['oriloc']

                            sql = 'select * from sys_doc where gdocid = %d' % src_gdocid
                            try:
                                src_doc = self.db[profile].execute(sqla.text(sql)).fetchone()
                            except:
                                self.log.exception("%s Fehler beim Prüfen der sys_history" % self.name)
                            else:
                                if src_doc:
                                    src_docloc = src_doc['docloc']
                                    src_extloc = src_doc['extloc']
                                    src_oriloc = src_doc['oriloc']

                                    if (dst_docloc is not None and dst_docloc == src_docloc) or (dst_extloc and dst_extloc == src_extloc) or (dst_oriloc is not None and dst_oriloc == src_oriloc):
                                        doc = bitfarm_doc.BitfarmDocument()
                                        doc.profile = profile
                                        doc.gdocid = dst_gdocid
                                        self.bfaProcessing.load(doc, level=['base', 'paths'])
                                        success, error = self.bfaProcessing.duplicate_files(_con, doc)
                                        if not success:
                                            self.log.error('%s Es hat einen Fehler bei der Korrektur eines duplizierten Dokuments [%s] gegeben' % (self.name, doc.gdocid))
                                            if error:
                                                self.log.error('%s %s' % (self.name, error))
                                        else:
                                            if not self.bfaProcessing.write(doc):
                                                self.log.error('%s Das duplizierte Dokument konnte nicht gespeichert werden [%s]' % (self.name, doc.gdocid))
                                            else:
                                                self.log.info('%s Die Pfade eines duplizierten Dokuments wurden korrigiert [%d]' % (self.name, doc.gdocid))

        # doppelte Prozentzeichen aus Lesezeichen entfernen
        result = True
        while result:
            sql = "select * from sys_bookmarks where bookmark rlike '%(%)+'"
            try:
                result = self.db[profile].execute(sqla.text(sql)).fetchall()
            except:
                self.log.exception("%s Fehler beim Prüfen der sys_bookmarks" % self.name)
                result = False
            else:
                if result:
                    self.log.info('%s Entferne doppelte Prozentzeichen aus Lesezeichen.' % self.name)

                    sql = "update sys_bookmarks set bookmark = Replace(bookmark, '%%', '%') where bookmark rlike '%(%)+'"
                    try:
                        self.db[profile].execute(sqla.text(sql))
                    except:
                        self.log.exception("%s Fehler beim Bereinigen der sys_bookmarks" % self.name)
                        result = False
                    else:
                        self.log.info("%s sys_bookmarks bereinigt." % self.name)

        # sql = "select * from sys_addfields where type in (2,3,11) and options & 8 = 8"
        # try:
        #     addfields = self.db[profile].execute(sql).fetchall()
        # except:
        #     self.log.exception(u"%s Fehler beim Suchen von strict Auswahlfeldern" % self.name)
        # else:
        #     for addfield in addfields:
        #         items = []
        #         if addfield['items']:
        #             items = addfield['items'].split(';')
        #             added = False
        #             sql = 'select distinct(%s) from sys_docidx_%s where %s is not Null' % (addfield['fieldname'], addfield['indextableid'], addfield['fieldname'])
        #
        #             try:
        #                 values = self.db[profile].execute(sql).fetchall()
        #             except:
        #                 self.log.exception(u"%s Fehler beim Abfragen der vorhandenen Werte" % self.name)
        #             else:
        #                 for value in values:
        #                     vals = value[0].split(';')
        #                     for val in vals:
        #                         if val not in items:
        #                             items.append(val)
        #                             added = True
        #
        #                 if added:
        #                     vals = {}
        #                     vals[_con.sys_addfields.c['items']] = u";".join(items)
        #                     sql = sqla.update(_con.sys_addfields, values=vals).where(
        #                             _con.sys_addfields.c.addfieldid == addfield['addfieldid'])
        #
        #                     try:
        #                         self.db[profile].execute(sql)
        #                     except:
        #                         msg = _(u"Fehler beim Ändern eines Zusatzfeldes")
        #                         self.log.exception(u"%s %s" % (self.name, msg))
        #                     else:
        #                         self.log.info(u'Zusatzfeld Items von Feld %s (%s) aktualisiert' % (addfield['displayname'], addfield['fieldname']))
    def check_dlm(self, profile):
        self.log.debug("%s check_dlm %s" % (self.name, profile))
        con = self.bfaserver.con.connections[profile]

        if con.dont_check_expired:
            return None

        if datetime.datetime.now().time() >= con.check_time and not self.checked[profile]:
            self.checked[profile] = True
            self.log.info("überprüfe Dokumentenlebensdauer für Profil %s" % profile)

            expired_dir = os.path.expandvars(con.expired_dir)
            if not os.path.exists(expired_dir):
                self.log.warning("%s Konnte das Expired Verzeichnis %s nicht finden, wird neu angelegt" % (self.name, expired_dir))
                try:
                    os.mkdir(expired_dir)
                except:
                    self.log.exception("%s Fehler beim Anlegen des Expired Verzeichnis %s" % (self.name, expired_dir))
                    return None

            try:
                self.db[profile].execute('select now()')
            except sqla.exc.OperationalError:
                self.log.error(_("%s Keine Datenbankverbindung zu %s, versuche reconnect...") % (self.name, profile))

                if not self.reconnect(profile):
                    self.log.error("%s Reconnect nicht erfolgreich für %s" % (self.name, profile))
                    self.checked[profile] = False
                    return None

            sql = "select * from sys_config where name like 'recalculate_%%'"
            try:
                recalc = self.db[profile].execute(sql).fetchall()
            except:
                self.log.exception("%s Fehler beim Suchen liegengebliebener Thread-Locks in der sys_config" % self.name)

            if recalc:
                for row in recalc:
                    sql = "update sys_doc set expired_flags = 1 where arcid = %s" % row['value']
                    try:
                        self.db[profile].execute(sql)
                    except:
                        self.log.exception("%s Fehler beim Setzen des recalculate-Flags" % self.name)

            if self.stopsignal:
                return None

            # erst prüfen ob noch welche berechnet werden müssen
            sql = "select gdocid from sys_doc where expired_flags = 1"
            try:
                result = self.db[profile].execute(sqla.text(sql)).fetchall()
            except:
                self.log.exception(_("%s Fehler bei Abfrage des recalculate-Flags.") % self.name)
            else:
                if result:
                    self.log.warning("%s DLM: Es müssen %s Dokument(e) neuberechnet werden" % (self.name, len(result)))
                    for row in result:
                        if self.stopsignal:
                            break
                        success = False
                        expired = None
                        try:
                            success, expired = self.bfaProcessing.recalculate_expired_date(row['gdocid'], profile)
                        except:
                            self.log.exception(
                                "%s Fehler bei recalculate_expired_date, mache mit nächsteem Eintrag weiter" % self.name)
                            continue

                        hist = bitfarm_doc.BitfarmHistory()
                        hist.userid = 1

                        try:
                            if success:
                                hist.taskid = 115
                                hist.value1 = expired.strftime('%d.%m.%Y') if expired is not None else 'leer'

                                self.bfaProcessing.write_history(history=hist, profile=profile,
                                                                 gdocid=row['gdocid'], logid=self.name)
                        except:
                            self.log.exception("%s Fehler beim Erzeugen eines Historieeintrags für %s. Mache mit nächstem Eintrag weiter" % (self.name, row['gdocid']))
                            continue

            if self.stopsignal:
                return None

            if recalc:
                for row in recalc:
                    sql = "delete from sys_config where conid = %s" % row['conid']
                    try:
                        self.db[profile].execute(sql)
                    except:
                        self.log.exception("%s Fehler beim Löschen des recalculate-Flags" % self.name)

            sql = "select gdocid, expired from sys_doc where date(expired) <= '%s' and expired_flags is NULL" % datetime.datetime.now().strftime('%Y-%m-%d')
            try:
                result = self.db[profile].execute(sqla.text(sql)).fetchall()
            except:
                self.log.exception(_("%s Fehler bei Abfrage der Dokumentenlebensdauer.") % self.name)
            else:
                if result:
                    self.log.warning("%s DLM: Es müssen %s Dokument(e) gelöscht werden" % (self.name, len(result)))
                    expired_dir = os.path.join(expired_dir, datetime.datetime.now().strftime('%Y_%m_%d'))
                    if not os.path.exists(expired_dir):
                        try:
                            os.mkdir(expired_dir)
                        except:
                            self.log.exception("%s Fehler beim Anlegen des Unterverzeichnis %s" % (self.name, expired_dir))
                            expired_dir = os.path.expandvars(con.expired_dir)
                        else:
                            self.log.debug("%s Unterverzeichnis angelegt %s" % (self.name, expired_dir))

                    for row in result:
                        if self.stopsignal:
                            return None

                        error = False
                        docloc = None
                        ocr_xml = None
                        hocr = None
                        dest_docloc = None
                        dest_oriloc = None
                        dest_srcloc = None
                        dest_ocrxml = None
                        dest_hocr = None
                        fpath = None
                        doc = None

                        self.log.info(
                            "%s Für das Dokument [%s] wurde die Lebensdauer erreicht (%s). Das Dokument wird nach %s verschoben" % (
                            self.name, row['gdocid'], bitfarm_doc.bfa_strftime(row['expired'], ('%d.%m.%Y')),
                            expired_dir))
                        doc = bitfarm_doc.BitfarmDocument()
                        doc.profile = profile
                        if not self.bfaProcessing.load(doc, int(row['gdocid'])):
                            self.log.error("%s Dokument [%s] konnte nicht geladen werden." % (self.name, row['gdocid']))
                            continue

                        docloc = doc.extloc if doc.extloc else doc.docloc
                        if docloc:
                            dest_docloc = os.path.join(expired_dir, os.path.basename(docloc))

                        # prüfen ob der pfad noch irgendwo verwendet wurde
                        sql = ""
                        if doc.docloc:
                            sql = "select gdocid from sys_doc where (docloc ='%s' or oriloc='%s') and gdocid <> %s" % (
                            doc.docloc, doc.docloc, doc.gdocid)
                            try:
                                result = self.db[profile].execute(sqla.text(sql)).fetchall()
                            except:
                                self.log.exception(_("%s Fehler bei Suche doppelter Pfad docloc.") % self.name)
                            else:
                                if result:
                                    doubles = [str(row[0]) for row in result]
                                    self.log.error(
                                        "Achtung, der docloc Pfad %s des zu löschenden Dokuments [%s] wird auch von [%s] verwendet, es wird deshalb nicht gelöscht." % (
                                        doc.docloc, doc.gdocid, ",".join(doubles)))
                                    continue

                        if doc.oriloc:
                            sql = "select gdocid from sys_doc where (docloc ='%s' or oriloc='%s') and gdocid <> %s" % (doc.oriloc, doc.oriloc, doc.gdocid)
                            try:
                                result = self.db[profile].execute(sqla.text(sql)).fetchall()
                            except:
                                self.log.exception(_("%s Fehler bei Suche doppelter Pfad oriloc.") % self.name)
                            else:
                                if result:
                                    doubles = [str(row[0]) for row in result]
                                    self.log.error(
                                        "Achtung, der oriloc Pfad %s des zu löschenden Dokuments [%s] wird auch von [%s] verwendet, es wird deshalb nicht gelöscht." % (
                                        doc.oriloc, doc.gdocid, ",".join(doubles)))
                                    continue

                        if docloc:
                            if not os.path.exists(os.path.expandvars(docloc)):
                                self.log.warning("%s Die Datei %s konnte nicht gefunden werden" % (self.name, docloc))
                                dest_docloc = None
                            else:
                                try:
                                    shutil.copy2(os.path.expandvars(docloc), dest_docloc)
                                except:
                                    self.log.exception("%s Die Datei %s konnte nicht nach %s kopiert werden" % (self.name, docloc, expired_dir))
                                    dest_docloc = None
                                    error = True

                        if doc.oriloc and doc.oriloc != docloc:
                            dest_oriloc = os.path.join(expired_dir, os.path.basename(doc.oriloc))
                            if not os.path.exists(os.path.expandvars(doc.oriloc)):
                                self.log.warning("%s Die Datei %s konnte nicht gefunden werden" % (self.name, doc.oriloc))
                                dest_oriloc = None
                            else:
                                try:
                                    shutil.copy2(os.path.expandvars(doc.oriloc), dest_oriloc)
                                except:
                                    self.log.exception("%s Die Datei %s konnte nicht nach %s kopiert werden" % (self.name, doc.oriloc, expired_dir))
                                    dest_oriloc = None
                                    error = True

                        if doc.srcloc and doc.srcloc != docloc and doc.srcloc != doc.oriloc:
                            dest_srcloc = os.path.join(expired_dir, os.path.basename(doc.srcloc))
                            if not os.path.exists(os.path.expandvars(doc.srcloc)):
                                self.log.warning("%s Die Datei %s konnte nicht gefunden werden" % (self.name, doc.srcloc))
                                dest_srcloc = None
                            else:
                                try:
                                    shutil.copy2(os.path.expandvars(doc.srcloc), dest_srcloc)
                                except:
                                    self.log.exception("%s Die Datei %s konnte nicht nach %s kopiert werden" % (self.name, doc.srcloc, expired_dir))
                                    dest_srcloc = None
                                    error = True

                        if os.path.exists(os.path.splitext(os.path.expandvars(doc.docloc))[0] + '.ocr_xml'):
                            ocr_xml = os.path.splitext(doc.docloc)[0] + '.ocr_xml'
                        elif os.path.exists(os.path.splitext(os.path.expandvars(doc.extloc))[0] + '.ocr_xml'):
                            ocr_xml = os.path.splitext(doc.extloc)[0] + '.ocr_xml'

                        if ocr_xml:
                            dest_ocrxml = os.path.join(expired_dir, os.path.basename(ocr_xml))
                            if not os.path.exists(os.path.expandvars(ocr_xml)):
                                self.log.warning("%s Die Datei %s konnte nicht gefunden werden" % (self.name, ocr_xml))
                                dest_ocrxml = None
                            else:
                                try:
                                    shutil.copy2(os.path.expandvars(ocr_xml), dest_ocrxml)
                                except:
                                    self.log.exception("%s Die Datei %s konnte nicht nach %s kopiert werden" % (self.name, ocr_xml, expired_dir))
                                    dest_ocrxml = None
                                    error = True

                        if os.path.exists(os.path.splitext(os.path.expandvars(doc.docloc))[0] + '.hocr'):
                            hocr = os.path.splitext(doc.docloc)[0] + '.hocr'
                        elif os.path.exists(os.path.splitext(os.path.expandvars(doc.extloc))[0] + '.hocr'):
                            hocr = os.path.splitext(doc.extloc)[0] + '.hocr'

                        if hocr:
                            dest_hocr = os.path.join(expired_dir, os.path.basename(hocr))
                            if not os.path.exists(os.path.expandvars(hocr)):
                                self.log.warning("%s Die Datei %s konnte nicht gefunden werden" % (self.name, hocr))
                                dest_hocr = None
                            else:
                                try:
                                    shutil.copy2(os.path.expandvars(hocr), dest_hocr)
                                except:
                                    self.log.exception("%s Die Datei %s konnte nicht nach %s kopiert werden" % (self.name, hocr, expired_dir))
                                    dest_hocr = None
                                    error = True

                        fpath = os.path.join(expired_dir, str(row['gdocid']) + '.xml')
                        try:
                            bitfarm_xml.object_to_xmlfile(doc, fpath, shortxml=False)
                        except:
                            self.log.exception("%s Fehler beim Schreiben der Dokumentendaten")
                            error = True
                        else:
                            try:
                                with zipfile.ZipFile(os.path.join(expired_dir, str(row['gdocid']) + '.zip'),'w') as myzip:
                                    if dest_docloc:
                                        myzip.write(dest_docloc, os.path.basename(dest_docloc))
                                    if dest_oriloc:
                                        myzip.write(dest_oriloc, os.path.basename(dest_oriloc))
                                    if dest_srcloc:
                                        myzip.write(dest_srcloc, os.path.basename(dest_srcloc))
                                    if dest_ocrxml:
                                        myzip.write(dest_ocrxml, os.path.basename(dest_ocrxml))
                                    if dest_hocr:
                                        myzip.write(dest_hocr, os.path.basename(dest_hocr))
                                    myzip.write(fpath, os.path.basename(fpath))
                                    myzip.close()
                            except:
                                self.log.exception("%s Fehler beim Schreiben der Zip-Datei")
                                error = True

                        if not error:
                            try:
                                if dest_docloc:
                                    os.remove(dest_docloc)
                                if dest_oriloc:
                                    os.remove(dest_oriloc)
                                if dest_srcloc:
                                    os.remove(dest_srcloc)
                                if dest_ocrxml:
                                    os.remove(dest_ocrxml)
                                if dest_hocr:
                                    os.remove(dest_hocr)
                                if fpath:
                                    os.remove(fpath)
                            except:
                                self.log.exception("%s Fehler beim Löschen der temporären Dateien" % self.name)  # dieser Fehler ist nicht schlimm, wir können weitermachen

                            # jetzt die originalen Dateien löschen
                            # liste aller möglichen pfade aufbauen
                            fnames = []
                            if doc.docloc:
                                fname = None
                                fnames.append(doc.docloc.lower())
                                fnames.append(os.path.splitext(doc.docloc.lower())[0] + '.job')
                                fnames.append(os.path.splitext(doc.docloc.lower())[0] + '.txt')
                                fnames.append(os.path.splitext(doc.docloc.lower())[0] + '.ocr_xml')
                                fnames.append(os.path.splitext(doc.docloc.lower())[0] + '.hocr')
                                if doc.docloc.lower().find(con.archive_rs.lower()) >= 0:
                                    fname = doc.docloc.lower().replace(con.archive_rs.lower(), con.archive_nrs.lower())
                                elif doc.docloc.lower().find(con.archive_nrs.lower()) >= 0:
                                    fname = doc.docloc.lower().replace(con.archive_nrs.lower(), con.archive_rs.lower())
                                if fname:
                                    fnames.append(fname)
                                    fnames.append(os.path.splitext(fname)[0] + '.job')
                                    fnames.append(os.path.splitext(fname)[0] + '.txt')
                                    fnames.append(os.path.splitext(fname)[0] + '.ocr_xml')
                                    fnames.append(os.path.splitext(fname)[0] + '.hocr')
                            if doc.extloc:
                                fnames.append(doc.extloc.lower())
                                if doc.extloc.lower().find(con.archive_rs.lower()) >= 0:
                                    fnames.append(
                                        doc.extloc.lower().replace(con.archive_rs.lower(), con.archive_nrs.lower()))
                                elif doc.extloc.lower().find(con.archive_nrs.lower()) >= 0:
                                    fnames.append(
                                        doc.extloc.lower().replace(con.archive_nrs.lower(), con.archive_rs.lower()))
                            if doc.oriloc:
                                fnames.append(doc.oriloc.lower())
                                if doc.oriloc.lower().find(con.archive_rs.lower()) >= 0:
                                    fnames.append(
                                        doc.oriloc.lower().replace(con.archive_rs.lower(), con.archive_nrs.lower()))
                                elif doc.oriloc.lower().find(con.archive_nrs.lower()) >= 0:
                                    fnames.append(
                                        doc.oriloc.lower().replace(con.archive_nrs.lower(), con.archive_rs.lower()))
                            if doc.srcloc:
                                fnames.append(doc.srcloc.lower())
                                if doc.srcloc.lower().find(con.archive_rs.lower()) >= 0:
                                    fnames.append(
                                        doc.srcloc.lower().replace(con.archive_rs.lower(), con.archive_nrs.lower()))
                                elif doc.srcloc.lower().find(con.archive_nrs.lower()) >= 0:
                                    fnames.append(
                                        doc.srcloc.lower().replace(con.archive_nrs.lower(), con.archive_rs.lower()))

                            # bei oriloc prüfen wir, ob das dokument im svn liegt
                            if doc.oriloc and 'svn' in con.ini and hasattr(con.ini['svn'], 'svnpath') and con.ini['svn'].svnpath:
                                if not os.path.exists(os.path.expandvars(con.ini['svn'].svnpath)):
                                    self.log.warning("%s SVN konfiguriert aber konnte %s nicht finden" % (self.name, con.ini['svn'].svnpath))
                                else:
                                    if doc.oriloc.find(con.archive_rs) >= 0:
                                        svn_oriloc = doc.oriloc.replace(con.archive_rs, con.archive_nrs)
                                    else:
                                        svn_oriloc = doc.oriloc
                                    svn_oriloc = os.path.expandvars(svn_oriloc)

                                    match = []
                                    cmd = [os.path.expandvars(con.ini['svn'].svnpath), 'info', svn_oriloc]
                                    self.log.debug("%s %s" % (self.name, " ".join(cmd)))
                                    try:
                                        output = subprocess.check_output(cmd, shell=True)
                                        match = re.findall("Revision\:\s(\d+)", output)
                                    except subprocess.CalledProcessError as e:
                                        self.log.exception("%s Fehler beim Suchen der Originaldatei im SVN, output = %s" % (self.name, e.output))
                                    except:
                                        self.log.exception("%s Fehler beim Suchen der Originaldatei im SVN" % self.name)
                                    else:
                                        if len(match) > 0:  # revision gefunden
                                            cmd = [con.ini['svn'].svnpath, 'delete', '--force', svn_oriloc]
                                            try:
                                                output = subprocess.check_output(cmd, shell=True)
                                            except subprocess.CalledProcessError as e:
                                                self.log.exception("%s Fehler beim Löschen der Originaldatei im SVN im SVN, output = %s" % (self.name, e.output))
                                            except:
                                                self.log.exception("%s Fehler beim Löschen der Originaldatei im SVN" % self.name)
                                            else:
                                                cmd = [con.ini['svn'].svnpath, 'commit', '-m', '--force-log', svn_oriloc]
                                                try:
                                                    output = subprocess.check_output(cmd, shell=True)
                                                except subprocess.CalledProcessError as e:
                                                    self.log.exception("%s Fehler beim Commit der gelöschten Originaldatei im SVN im SVN, output = %s" % (self.name, e.output))
                                                except:
                                                    self.log.exception("%s Fehler beim Commit der gelöschten Originaldatei im SVN" % self.name)
                                                else:
                                                    self.log.info("%s Die Originaldatei wurde im SVN entfernt")

                            for fname in fnames:
                                try:
                                    if os.path.exists(os.path.expandvars(fname)):
                                        os.remove(os.path.expandvars(fname))
                                except:
                                    self.log.exception("%s Fehler beim Löschen der Datei  %s " % (self.name, fname))
                                    error = True

                            fname = os.path.expandvars('%bitfarm-archiv%\checkout\\' + str(row['gdocid']))
                            if os.path.exists(fname):
                                try:
                                    shutil.rmtree(fname, ignore_errors=True)
                                except:
                                    self.log.exception("%s Fehler beim Löschen der Datei im checkout-Verzeichnis %s " % (self.name, fname))
                                    error = True

                            annopath = doc.extloc if doc.extloc else doc.docloc
                            annopath = os.path.expandvars(annopath).lower()
                            annopath = annopath.replace(con.archive_rs.lower(),
                                                        con.archive_nrs.lower())
                            annopath = os.path.splitext(annopath)[0]

                            ievlist = glob.iglob(annopath + "*.iev")
                            for iev in ievlist:
                                try:
                                    if os.path.exists(os.path.expandvars(iev)):
                                        os.remove(os.path.expandvars(iev))
                                except:
                                    self.log.exception("%s Fehler beim Löschen der Annotation Datei %s " % (self.name, iev))
                                    error = True

                        if not error:  # finale, wir löschen den eintrag in der datenbank
                            sql = 'delete from sys_doc where gdocid = %s' % row['gdocid']
                            try:
                                self.db[profile].execute(sql)
                            except:
                                self.log.exception("Fehler beim Löschen des Dokuments in der Datenbank")

                            sql = 'delete from sys_history where gdocid = %s' % row['gdocid']
                            try:
                                self.db[profile].execute(sql)
                            except:
                                self.log.exception("Fehler beim Löschen des Dokuments in der Datenbank")

                            sql = 'delete from sys_timer where gdocid = %s' % row['gdocid']
                            try:
                                self.db[profile].execute(sql)
                            except:
                                self.log.exception("Fehler beim Löschen des Dokuments in der Datenbank")

                            sql = 'delete from sys_annotations where gdocid = %s' % row['gdocid']
                            try:
                                self.db[profile].execute(sql)
                            except:
                                self.log.exception("Fehler beim Löschen des Dokuments in der Datenbank")

                            sql = 'delete from sys_com where gdocid = %s' % row['gdocid']
                            try:
                                self.db[profile].execute(sql)
                            except:
                                self.log.exception("Fehler beim Löschen des Dokuments in der Datenbank")

                            if con.sys_svn is not None:
                                sql = 'delete from sys_svn where gdocid = %s' % row['gdocid']
                                try:
                                    self.db[profile].execute(sql)
                                except:
                                    self.log.exception("Fehler beim Löschen des Dokuments in der Datenbank")

                            if con.sys_sap is not None:
                                sql = 'delete from sys_sap where gdocid = %s' % row['gdocid']
                                try:
                                    self.db[profile].execute(sql)
                                except:
                                    self.log.exception("Fehler beim Löschen des Dokuments in der Datenbank")

                            sql = 'delete from sys_doclink where gdocid = %s' % row['gdocid']
                            try:
                                self.db[profile].execute(sql)
                            except:
                                self.log.exception("Fehler beim Löschen des Dokuments in der Datenbank")

                            sql = 'delete from sys_symlink where gdocid = %s' % row['gdocid']
                            try:
                                self.db[profile].execute(sql)
                            except:
                                self.log.exception("Fehler beim Löschen des Dokuments in der Datenbank")

                            for idx in con.sys_docidx.values():
                                sql = 'delete from %s where gdocid = %s' % (idx.name, row['gdocid'])
                                try:
                                    self.db[profile].execute(sql)
                                except:
                                    self.log.exception("Fehler beim Löschen des Dokuments in der Datenbank")

                            self.log.info("%s Das Dokument %s wurde gelöscht." % (self.name, row['gdocid']))
                            hist = bitfarm_doc.BitfarmHistory()
                            hist.gdocid = 0
                            hist.taskid = 118
                            hist.userid = 1
                            hist.value1 = doc.gdocid
                            hist.value2 = doc.date
                            hist.value3 = doc.arcid
                            self.bfaProcessing.write_history(history=hist, profile=profile, logid=self.name)
                        else:
                            self.log.error("%s Fehler beim Löschen des Dokuments %s" % (self.name, row['gdocid']))
                            error = False
            self.log.info("Überprüfung der Dokumentenlebensdauer für Profil %s abgeschlossen" % profile)


    def run(self):
        while self.is_alive():
            if self.stopsignal:
                break

            for profile in self.bfaserver.con.profiles:
                if datetime.datetime.now().hour == 0 and self.checked[profile]:
                    self.checked[profile] = False
                    self.maintained[profile] = False

            for profile in self.bfaserver.con.profiles:
                con = self.bfaserver.con.connections[profile]

                if datetime.datetime.now().time() >= con.check_time and (not self.maintained[profile] or not self.checked[profile]):
                    try:
                        self.db[profile].execute('select now()')
                    except sqla.exc.OperationalError:
                        self.log.error(_("%s Keine Datenbankverbindung zu %s, versuche reconnect...") % (self.name, profile))

                        if not self.reconnect(profile):
                            self.log.error("%s Reconnect nicht erfolgreich für %s" % (self.name, profile))
                            self.maintained[profile] = False
                            self.checked[profile] = False
                            continue

                    self.maintain_db(profile)
                    self.check_dlm(profile)

            try:
                job = self.queue.get(True, 50)
            except queue.Empty:
                #            self.log.debug(u"Nichts vom MP übergeben, dann weiter")
                if self.stopsignal:  # wir haben alles abgearbeitet und sollen aufhören?
                    self.log.info(_("%s Stopsignal erhalten, Queue leer, beende Thread") % self.name)
                    break
            else:
                if job == 'stop':
                    self.log.info(_("%s Stopsignal erhalten, Queue leer, beende Thread (2)") % self.name)
                    break

        self.log.debug(_('%s stopped') % self.name)

    def shutdown(self):
        self.stopsignal = True
        self.queue.put('stop')

    def reconnect(self, _profile):
        self.log.debug("%s reconnect" % self.name)
        _ret = True
        con = self.bfaserver.con.connections[_profile]

        try:
            self.db[_profile] = con.engine.connect()
        except:
            self.log.exception(_("%s Reconnect gescheitert") % self.name)
            _ret = _ret and False
        else:
            if not isinstance(self.db[_profile], sqla.engine.base.Connection):
                self.log.error(_("%s Reconnect gescheitert") % self.name)
                _ret = _ret and False
            else:
                self.log.warning(_("%s Reconnect erfolgreich db = %s") % (self.name, self.db[_profile]))
                _ret = _ret and True

                if _profile == self.bfaserver.con.main.profile:     # die main connection auch neu zuweisen
                    self.bfaProcessing.db['main'] = self.db[_profile]

        return _ret
